package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaAccountTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaAccountTypeEnum.FieldWithMetaAccountTypeEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AccountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify an account as an account number alongside, optionally. an account name, an account type, an account beneficiary and a servicing party.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Account", source="FpML_5_10")
@RosettaSynonym(value="Account", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Account", source="CME_ClearedConfirm_1_17")
public class Account extends RosettaModelObject implements GlobalKey {
	private final ReferenceWithMetaParty accountBeneficiary;
	private final FieldWithMetaString accountName;
	private final FieldWithMetaString accountNumber;
	private final FieldWithMetaAccountTypeEnum accountType;
	private final MetaFields meta;
	private final ReferenceWithMetaParty servicingParty;
	private static AccountMeta metaData = new AccountMeta();
	
	Account(AccountBuilder builder) {
		this.accountBeneficiary = ofNullable(builder.getAccountBeneficiary()).map(f->f.build()).orElse(null);
		this.accountName = ofNullable(builder.getAccountName()).map(f->f.build()).orElse(null);
		this.accountNumber = ofNullable(builder.getAccountNumber()).map(f->f.build()).orElse(null);
		this.accountType = ofNullable(builder.getAccountType()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.servicingParty = ofNullable(builder.getServicingParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A reference to the party beneficiary of the account.
	 */
	@RosettaSynonym(value="accountBeneficiary", source="FpML_5_10")
	@RosettaSynonym(value="accountBeneficiary", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accountBeneficiary", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getAccountBeneficiary() {
		return accountBeneficiary;
	}
	
	/**
	 * The name by which the account is known.
	 */
	@RosettaSynonym(value="accountName", source="FpML_5_10")
	@RosettaSynonym(value="accountName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accountName", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="name", source="ISDA_Create_1_0")
	public final FieldWithMetaString getAccountName() {
		return accountName;
	}
	
	/**
	 * The account number.
	 */
	@RosettaSynonym(value="accountId", source="FpML_5_10")
	@RosettaSynonym(value="accountId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accountId", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="partyA_cash,partyA_securities", source="ISDA_Create_1_0")
	public final FieldWithMetaString getAccountNumber() {
		return accountNumber;
	}
	
	/**
	 * The type of account, e.g. client, house.
	 */
	@RosettaSynonym(value="accountType", source="FpML_5_10")
	@RosettaSynonym(value="accountType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accountType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaAccountTypeEnum getAccountType() {
		return accountType;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The reference to the legal entity that services the account, i.e. in the books of which the account is held.
	 */
	@RosettaSynonym(value="servicingParty", source="FpML_5_10")
	@RosettaSynonym(value="servicingParty", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="servicingParty", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getServicingParty() {
		return servicingParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends Account> metaData() {
		return metaData;
	} 
	
	public AccountBuilder toBuilder() {
		AccountBuilder builder = new AccountBuilder();
		ofNullable(getAccountBeneficiary()).ifPresent(builder::setAccountBeneficiary);
		ofNullable(getAccountName()).ifPresent(builder::setAccountName);
		ofNullable(getAccountNumber()).ifPresent(builder::setAccountNumber);
		ofNullable(getAccountType()).ifPresent(builder::setAccountType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getServicingParty()).ifPresent(builder::setServicingParty);
		return builder;
	}

	public static AccountBuilder builder() {
		return new AccountBuilder();
	}

	public static  class AccountBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AccountBuilder> {
	
		protected ReferenceWithMetaPartyBuilder accountBeneficiary;
		protected FieldWithMetaStringBuilder accountName;
		protected FieldWithMetaStringBuilder accountNumber;
		protected FieldWithMetaAccountTypeEnumBuilder accountType;
		protected MetaFields.MetaFieldsBuilder meta;
		protected ReferenceWithMetaPartyBuilder servicingParty;
	
		public AccountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Account> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaPartyBuilder getAccountBeneficiary() {
			return accountBeneficiary;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateAccountBeneficiary() {
			if (accountBeneficiary!=null) {
				return accountBeneficiary;
			}
			else return accountBeneficiary = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getAccountName() {
			return accountName;
		}
		
		public FieldWithMetaStringBuilder getOrCreateAccountName() {
			if (accountName!=null) {
				return accountName;
			}
			else return accountName = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getAccountNumber() {
			return accountNumber;
		}
		
		public FieldWithMetaStringBuilder getOrCreateAccountNumber() {
			if (accountNumber!=null) {
				return accountNumber;
			}
			else return accountNumber = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaAccountTypeEnumBuilder getAccountType() {
			return accountType;
		}
		
		public FieldWithMetaAccountTypeEnumBuilder getOrCreateAccountType() {
			if (accountType!=null) {
				return accountType;
			}
			else return accountType = new FieldWithMetaAccountTypeEnumBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getServicingParty() {
			return servicingParty;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateServicingParty() {
			if (servicingParty!=null) {
				return servicingParty;
			}
			else return servicingParty = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public AccountBuilder setAccountBeneficiary(ReferenceWithMetaParty accountBeneficiary) {
			this.accountBeneficiary = accountBeneficiary.toBuilder();
			return this;
		}
		public AccountBuilder setAccountBeneficiaryBuilder(ReferenceWithMetaPartyBuilder accountBeneficiary) {
			this.accountBeneficiary = accountBeneficiary;
			return this;
		}
		public AccountBuilder setAccountBeneficiaryRef(Party.PartyBuilder accountBeneficiary) {
			return setAccountBeneficiary(ReferenceWithMetaParty.builder().setValueBuilder(accountBeneficiary).build());
		}
		public AccountBuilder setAccountBeneficiaryRef(Party accountBeneficiary) {
			return setAccountBeneficiaryRef(Party.builder());
		}
		public AccountBuilder setAccountName(FieldWithMetaString accountName) {
			this.accountName = accountName.toBuilder();
			return this;
		}
		public AccountBuilder setAccountNameRef(String accountName) {
			return setAccountName(FieldWithMetaString.builder().setValue(accountName).build());
		}
		public AccountBuilder setAccountNumber(FieldWithMetaString accountNumber) {
			this.accountNumber = accountNumber.toBuilder();
			return this;
		}
		public AccountBuilder setAccountNumberRef(String accountNumber) {
			return setAccountNumber(FieldWithMetaString.builder().setValue(accountNumber).build());
		}
		public AccountBuilder setAccountType(FieldWithMetaAccountTypeEnum accountType) {
			this.accountType = accountType.toBuilder();
			return this;
		}
		public AccountBuilder setAccountTypeRef(AccountTypeEnum accountType) {
			return setAccountType(FieldWithMetaAccountTypeEnum.builder().setValue(accountType).build());
		}
		public AccountBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AccountBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AccountBuilder setServicingParty(ReferenceWithMetaParty servicingParty) {
			this.servicingParty = servicingParty.toBuilder();
			return this;
		}
		public AccountBuilder setServicingPartyBuilder(ReferenceWithMetaPartyBuilder servicingParty) {
			this.servicingParty = servicingParty;
			return this;
		}
		public AccountBuilder setServicingPartyRef(Party.PartyBuilder servicingParty) {
			return setServicingParty(ReferenceWithMetaParty.builder().setValueBuilder(servicingParty).build());
		}
		public AccountBuilder setServicingPartyRef(Party servicingParty) {
			return setServicingPartyRef(Party.builder());
		}
		
		public Account build() {
			return new Account(this);
		}
	
		@Override
		public AccountBuilder prune() {
			if (accountBeneficiary!=null && !accountBeneficiary.prune().hasData()) accountBeneficiary = null;
			if (accountName!=null && !accountName.prune().hasData()) accountName = null;
			if (accountNumber!=null && !accountNumber.prune().hasData()) accountNumber = null;
			if (accountType!=null && !accountType.prune().hasData()) accountType = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (servicingParty!=null && !servicingParty.prune().hasData()) servicingParty = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccountBeneficiary()!=null && getAccountBeneficiary().hasData()) return true;
			if (getAccountName()!=null) return true;
			if (getAccountNumber()!=null) return true;
			if (getAccountType()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getServicingParty()!=null && getServicingParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("accountBeneficiary"), processor, ReferenceWithMetaParty.class, accountBeneficiary);
			processRosetta(path.newSubPath("accountName"), processor, FieldWithMetaString.class, accountName);
			processRosetta(path.newSubPath("accountNumber"), processor, FieldWithMetaString.class, accountNumber);
			processRosetta(path.newSubPath("accountType"), processor, FieldWithMetaAccountTypeEnum.class, accountType);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("servicingParty"), processor, ReferenceWithMetaParty.class, servicingParty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AccountBuilder _that = (AccountBuilder) o;
		
			if (accountBeneficiary != null ? !accountBeneficiary.equals(_that.accountBeneficiary) : _that.accountBeneficiary != null) return false;
			if (accountName != null ? !accountName.equals(_that.accountName) : _that.accountName != null) return false;
			if (accountNumber != null ? !accountNumber.equals(_that.accountNumber) : _that.accountNumber != null) return false;
			if (accountType != null ? !accountType.equals(_that.accountType) : _that.accountType != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (servicingParty != null ? !servicingParty.equals(_that.servicingParty) : _that.servicingParty != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accountBeneficiary != null ? accountBeneficiary.hashCode() : 0);
			_result = 31 * _result + (accountName != null ? accountName.hashCode() : 0);
			_result = 31 * _result + (accountNumber != null ? accountNumber.hashCode() : 0);
			_result = 31 * _result + (accountType != null ? accountType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (servicingParty != null ? servicingParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AccountBuilder {" +
				"accountBeneficiary=" + this.accountBeneficiary + ", " +
				"accountName=" + this.accountName + ", " +
				"accountNumber=" + this.accountNumber + ", " +
				"accountType=" + this.accountType + ", " +
				"meta=" + this.meta + ", " +
				"servicingParty=" + this.servicingParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("accountBeneficiary"), processor, ReferenceWithMetaParty.class, accountBeneficiary);
		processRosetta(path.newSubPath("accountName"), processor, FieldWithMetaString.class, accountName);
		processRosetta(path.newSubPath("accountNumber"), processor, FieldWithMetaString.class, accountNumber);
		processRosetta(path.newSubPath("accountType"), processor, FieldWithMetaAccountTypeEnum.class, accountType);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("servicingParty"), processor, ReferenceWithMetaParty.class, servicingParty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Account _that = (Account) o;
	
		if (accountBeneficiary != null ? !accountBeneficiary.equals(_that.accountBeneficiary) : _that.accountBeneficiary != null) return false;
		if (accountName != null ? !accountName.equals(_that.accountName) : _that.accountName != null) return false;
		if (accountNumber != null ? !accountNumber.equals(_that.accountNumber) : _that.accountNumber != null) return false;
		if (accountType != null ? !accountType.equals(_that.accountType) : _that.accountType != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (servicingParty != null ? !servicingParty.equals(_that.servicingParty) : _that.servicingParty != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accountBeneficiary != null ? accountBeneficiary.hashCode() : 0);
		_result = 31 * _result + (accountName != null ? accountName.hashCode() : 0);
		_result = 31 * _result + (accountNumber != null ? accountNumber.hashCode() : 0);
		_result = 31 * _result + (accountType != null ? accountType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (servicingParty != null ? servicingParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Account {" +
			"accountBeneficiary=" + this.accountBeneficiary + ", " +
			"accountName=" + this.accountName + ", " +
			"accountNumber=" + this.accountNumber + ", " +
			"accountType=" + this.accountType + ", " +
			"meta=" + this.meta + ", " +
			"servicingParty=" + this.servicingParty +
		'}';
	}
}
