package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to qualify the type of account.
 * 
 * @version 2.5.4
 */
public enum AccountTypeEnum {
    

	/**
	 * Aggregate client account, as defined under ESMA MiFIR.
	 */
	@RosettaSynonym(value = "AggregateClient", source = "FpML_5_10")
	@RosettaSynonym(value = "AggregateClient", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AggregateClient", source = "DTCC_11_0")
	@RosettaSynonym(value = "AggregateClient", source = "DTCC_9_0")
	@RosettaSynonym(value = "AggregateClient", source = "CME_ClearedConfirm_1_17")
	AGGREGATE_CLIENT,
	    

	/**
	 * The account contains trading activity or positions that belong to a client of the firm that opened the account.
	 */
	@RosettaSynonym(value = "Client", source = "FpML_5_10")
	@RosettaSynonym(value = "Client", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Client", source = "DTCC_11_0")
	@RosettaSynonym(value = "Client", source = "DTCC_9_0")
	@RosettaSynonym(value = "Client", source = "CME_ClearedConfirm_1_17")
	CLIENT,
	    

	/**
	 * The account contains proprietary trading activity or positions, belonging to the firm that is the owner of the account.
	 */
	@RosettaSynonym(value = "House", source = "FpML_5_10")
	@RosettaSynonym(value = "House", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "House", source = "DTCC_11_0")
	@RosettaSynonym(value = "House", source = "DTCC_9_0")
	@RosettaSynonym(value = "House", source = "CME_ClearedConfirm_1_17")
	HOUSE    
	;

	private final String displayName;
	
	AccountTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
