package org.isda.cdm;

import org.isda.cdm.Id;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AcctOwnrMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class AcctOwnr extends RosettaModelObject {
	private final Id id;
	private static AcctOwnrMeta metaData = new AcctOwnrMeta();
	
	AcctOwnr(AcctOwnrBuilder builder) {
		this.id = ofNullable(builder.getId()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Id", source="ISO20022")
	public final Id getId() {
		return id;
	}
	
	
	@Override
	public RosettaMetaData<? extends AcctOwnr> metaData() {
		return metaData;
	} 
	
	public AcctOwnrBuilder toBuilder() {
		AcctOwnrBuilder builder = new AcctOwnrBuilder();
		ofNullable(getId()).ifPresent(builder::setId);
		return builder;
	}

	public static AcctOwnrBuilder builder() {
		return new AcctOwnrBuilder();
	}

	public static  class AcctOwnrBuilder extends RosettaModelObjectBuilder {
	
		protected Id.IdBuilder id;
	
		public AcctOwnrBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AcctOwnr> metaData() {
			return metaData;
		} 
	
		
		public Id.IdBuilder getId() {
			return id;
		}
		
		public Id.IdBuilder getOrCreateId() {
			if (id!=null) {
				return id;
			}
			else return id = new Id.IdBuilder();
		}
		
	
		public AcctOwnrBuilder setId(Id id) {
			this.id = id.toBuilder();
			return this;
		}
		public AcctOwnrBuilder setIdBuilder(Id.IdBuilder id) {
			this.id = id;
			return this;
		}
		
		public AcctOwnr build() {
			return new AcctOwnr(this);
		}
	
		@Override
		public AcctOwnrBuilder prune() {
			if (id!=null && !id.prune().hasData()) id = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getId()!=null && getId().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("id"), processor, Id.class, id);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AcctOwnrBuilder _that = (AcctOwnrBuilder) o;
		
			if (id != null ? !id.equals(_that.id) : _that.id != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (id != null ? id.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AcctOwnrBuilder {" +
				"id=" + this.id +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("id"), processor, Id.class, id);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AcctOwnr _that = (AcctOwnr) o;
	
		if (id != null ? !id.equals(_that.id) : _that.id != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (id != null ? id.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AcctOwnr {" +
			"id=" + this.id +
		'}';
	}
}
