package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify the actions associated with transactions.
 * 
 * @version 2.5.4
 */
public enum ActionEnum {
    

	/**
	 * A new instance of a transaction event, which is also characterized by the fact that the eventIdentifier has an associated version 1.
	 */
	@RosettaSynonym(value = "New", source = "DTCC_11_0")
	@RosettaSynonym(value = "New", source = "DTCC_9_0")
	@RosettaSynonym(value = "0", source = "CME_SubmissionIRS_1_0")
	NEW,
	    

	/**
	 * A correction of a prior instance of the transaction event. The eventIdentifier has an associated version greater than 1.
	 */
	@RosettaSynonym(value = "Modify", source = "DTCC_11_0")
	@RosettaSynonym(value = "Modify", source = "DTCC_9_0")
	@RosettaSynonym(value = "2", source = "CME_SubmissionIRS_1_0")
	CORRECT,
	    

	/**
	 * A cancellation of a prior instance of the transaction event. The eventIdentifier has an associated version greater than 1.
	 */
	@RosettaSynonym(value = "Disable", source = "DTCC_11_0")
	@RosettaSynonym(value = "Disable", source = "DTCC_9_0")
	@RosettaSynonym(value = "1", source = "CME_SubmissionIRS_1_0")
	CANCEL    
	;

	private final String displayName;
	
	ActionEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
