package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.PriceExpressionEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ActualPriceMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ActualPrice", source="FpML_5_10")
public class ActualPrice extends RosettaModelObject {
	private final BigDecimal amount;
	private final FieldWithMetaString currency;
	private final PriceExpressionEnum priceExpression;
	private static ActualPriceMeta metaData = new ActualPriceMeta();
	
	ActualPrice(ActualPriceBuilder builder) {
		this.amount = builder.getAmount();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.priceExpression = builder.getPriceExpression();
	}
	
	@RosettaSynonym(value="amount", source="FpML_5_10")
	public final BigDecimal getAmount() {
		return amount;
	}
	
	/**
	 * Specifies the currency associated with the net price. This element is not present if the price is expressed in percentage terms (as specified through the priceExpression element).
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * Specifies whether the price is expressed in absolute or relative terms.
	 */
	@RosettaSynonym(value="priceExpression", source="FpML_5_10")
	public final PriceExpressionEnum getPriceExpression() {
		return priceExpression;
	}
	
	
	@Override
	public RosettaMetaData<? extends ActualPrice> metaData() {
		return metaData;
	} 
	
	public ActualPriceBuilder toBuilder() {
		ActualPriceBuilder builder = new ActualPriceBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getPriceExpression()).ifPresent(builder::setPriceExpression);
		return builder;
	}

	public static ActualPriceBuilder builder() {
		return new ActualPriceBuilder();
	}

	public static  class ActualPriceBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal amount;
		protected FieldWithMetaStringBuilder currency;
		protected PriceExpressionEnum priceExpression;
	
		public ActualPriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ActualPrice> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmount() {
			return amount;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public PriceExpressionEnum getPriceExpression() {
			return priceExpression;
		}
		
	
		public ActualPriceBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		public ActualPriceBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public ActualPriceBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public ActualPriceBuilder setPriceExpression(PriceExpressionEnum priceExpression) {
			this.priceExpression = priceExpression;
			return this;
		}
		
		public ActualPrice build() {
			return new ActualPrice(this);
		}
	
		@Override
		public ActualPriceBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getPriceExpression()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
			processor.processBasic(path.newSubPath("priceExpression"), PriceExpressionEnum.class, priceExpression, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ActualPriceBuilder _that = (ActualPriceBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (priceExpression != null ? !priceExpression.equals(_that.priceExpression) : _that.priceExpression != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (priceExpression != null ? priceExpression.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ActualPriceBuilder {" +
				"amount=" + this.amount + ", " +
				"currency=" + this.currency + ", " +
				"priceExpression=" + this.priceExpression +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
		processor.processBasic(path.newSubPath("priceExpression"), PriceExpressionEnum.class, priceExpression, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ActualPrice _that = (ActualPrice) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (priceExpression != null ? !priceExpression.equals(_that.priceExpression) : _that.priceExpression != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (priceExpression != null ? priceExpression.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ActualPrice {" +
			"amount=" + this.amount + ", " +
			"currency=" + this.currency + ", " +
			"priceExpression=" + this.priceExpression +
		'}';
	}
}
