package org.isda.cdm;

import java.lang.Boolean;
import java.math.BigDecimal;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalDisruptionEventsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A type for defining the Additional Disruption Events.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdditionalDisruptionEvents", source="FpML_5_10")
public class AdditionalDisruptionEvents extends RosettaModelObject {
	private final Boolean changeInLaw;
	private final ReferenceWithMetaParty determiningPartyReference;
	private final Boolean failureToDeliver;
	private final Boolean foreignOwnershipEvent;
	private final Boolean hedgingDisruption;
	private final Boolean increasedCostOfHedging;
	private final Boolean increasedCostOfStockBorrow;
	private final BigDecimal initialStockLoanRate;
	private final Boolean insolvencyFiling;
	private final Boolean lossOfStockBorrow;
	private final BigDecimal maximumStockLoanRate;
	private static AdditionalDisruptionEventsMeta metaData = new AdditionalDisruptionEventsMeta();
	
	AdditionalDisruptionEvents(AdditionalDisruptionEventsBuilder builder) {
		this.changeInLaw = builder.getChangeInLaw();
		this.determiningPartyReference = ofNullable(builder.getDeterminingPartyReference()).map(f->f.build()).orElse(null);
		this.failureToDeliver = builder.getFailureToDeliver();
		this.foreignOwnershipEvent = builder.getForeignOwnershipEvent();
		this.hedgingDisruption = builder.getHedgingDisruption();
		this.increasedCostOfHedging = builder.getIncreasedCostOfHedging();
		this.increasedCostOfStockBorrow = builder.getIncreasedCostOfStockBorrow();
		this.initialStockLoanRate = builder.getInitialStockLoanRate();
		this.insolvencyFiling = builder.getInsolvencyFiling();
		this.lossOfStockBorrow = builder.getLossOfStockBorrow();
		this.maximumStockLoanRate = builder.getMaximumStockLoanRate();
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Change in Law | 2018 ISDA CDM Equity Confirmation for Security Equity Swap: Change In Law | If true, then change in law is applicable.
	 */
	@RosettaSynonym(value="changeInLaw", source="FpML_5_10")
	public final Boolean getChangeInLaw() {
		return changeInLaw;
	}
	
	/**
	 * A reference to the party which determines additional disruption events.
	 */
	@RosettaSynonym(value="determiningPartyReference", source="FpML_5_10")
	public final ReferenceWithMetaParty getDeterminingPartyReference() {
		return determiningPartyReference;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Failure to Deliver | Where the underlying is shares and the transaction is physically settled, then, if true, a failure to deliver the shares on the settlement date will not be an event of default for the purposes of the master agreement.
	 */
	@RosettaSynonym(value="failureToDeliver", source="FpML_5_10")
	public final Boolean getFailureToDeliver() {
		return failureToDeliver;
	}
	
	/**
	 * If true, then foreign ownership event is applicable.
	 */
	@RosettaSynonym(value="foreignOwnershipEvent", source="FpML_5_10")
	public final Boolean getForeignOwnershipEvent() {
		return foreignOwnershipEvent;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Hedging Disruption | If true, then hedging disruption is applicable.
	 */
	@RosettaSynonym(value="hedgingDisruption", source="FpML_5_10")
	public final Boolean getHedgingDisruption() {
		return hedgingDisruption;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Increased Cost of Hedging | If true, then increased cost of hedging is applicable.
	 */
	@RosettaSynonym(value="increasedCostOfHedging", source="FpML_5_10")
	public final Boolean getIncreasedCostOfHedging() {
		return increasedCostOfHedging;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Increased Cost of Stock Borrow | If true, then increased cost of stock borrow is applicable.
	 */
	@RosettaSynonym(value="increasedCostOfStockBorrow", source="FpML_5_10")
	public final Boolean getIncreasedCostOfStockBorrow() {
		return increasedCostOfStockBorrow;
	}
	
	/**
	 * Specifies the initial stock loan rate for Increased Cost of Stock Borrow. A percentage of 5% is represented as 0.05.
	 */
	@RosettaSynonym(value="initialStockLoanRate", source="FpML_5_10")
	public final BigDecimal getInitialStockLoanRate() {
		return initialStockLoanRate;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Insolvency Filing | If true, then insolvency filing is applicable.
	 */
	@RosettaSynonym(value="insolvencyFiling", source="FpML_5_10")
	public final Boolean getInsolvencyFiling() {
		return insolvencyFiling;
	}
	
	/**
	 * 2002 ISDA Equity Derivatives Definitions: Loss of Stock Borrow | If true, then loss of stock borrow is applicable.
	 */
	@RosettaSynonym(value="lossOfStockBorrow", source="FpML_5_10")
	public final Boolean getLossOfStockBorrow() {
		return lossOfStockBorrow;
	}
	
	/**
	 * Specifies the maximum stock loan rate for Loss of Stock Borrow. A percentage of 5% is represented as 0.05.
	 */
	@RosettaSynonym(value="maximumStockLoanRate", source="FpML_5_10")
	public final BigDecimal getMaximumStockLoanRate() {
		return maximumStockLoanRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalDisruptionEvents> metaData() {
		return metaData;
	} 
	
	public AdditionalDisruptionEventsBuilder toBuilder() {
		AdditionalDisruptionEventsBuilder builder = new AdditionalDisruptionEventsBuilder();
		ofNullable(getChangeInLaw()).ifPresent(builder::setChangeInLaw);
		ofNullable(getDeterminingPartyReference()).ifPresent(builder::setDeterminingPartyReference);
		ofNullable(getFailureToDeliver()).ifPresent(builder::setFailureToDeliver);
		ofNullable(getForeignOwnershipEvent()).ifPresent(builder::setForeignOwnershipEvent);
		ofNullable(getHedgingDisruption()).ifPresent(builder::setHedgingDisruption);
		ofNullable(getIncreasedCostOfHedging()).ifPresent(builder::setIncreasedCostOfHedging);
		ofNullable(getIncreasedCostOfStockBorrow()).ifPresent(builder::setIncreasedCostOfStockBorrow);
		ofNullable(getInitialStockLoanRate()).ifPresent(builder::setInitialStockLoanRate);
		ofNullable(getInsolvencyFiling()).ifPresent(builder::setInsolvencyFiling);
		ofNullable(getLossOfStockBorrow()).ifPresent(builder::setLossOfStockBorrow);
		ofNullable(getMaximumStockLoanRate()).ifPresent(builder::setMaximumStockLoanRate);
		return builder;
	}

	public static AdditionalDisruptionEventsBuilder builder() {
		return new AdditionalDisruptionEventsBuilder();
	}

	public static  class AdditionalDisruptionEventsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean changeInLaw;
		protected ReferenceWithMetaPartyBuilder determiningPartyReference;
		protected Boolean failureToDeliver;
		protected Boolean foreignOwnershipEvent;
		protected Boolean hedgingDisruption;
		protected Boolean increasedCostOfHedging;
		protected Boolean increasedCostOfStockBorrow;
		protected BigDecimal initialStockLoanRate;
		protected Boolean insolvencyFiling;
		protected Boolean lossOfStockBorrow;
		protected BigDecimal maximumStockLoanRate;
	
		public AdditionalDisruptionEventsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalDisruptionEvents> metaData() {
			return metaData;
		} 
	
		
		public Boolean getChangeInLaw() {
			return changeInLaw;
		}
		
		
		public ReferenceWithMetaPartyBuilder getDeterminingPartyReference() {
			return determiningPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateDeterminingPartyReference() {
			if (determiningPartyReference!=null) {
				return determiningPartyReference;
			}
			else return determiningPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public Boolean getFailureToDeliver() {
			return failureToDeliver;
		}
		
		
		public Boolean getForeignOwnershipEvent() {
			return foreignOwnershipEvent;
		}
		
		
		public Boolean getHedgingDisruption() {
			return hedgingDisruption;
		}
		
		
		public Boolean getIncreasedCostOfHedging() {
			return increasedCostOfHedging;
		}
		
		
		public Boolean getIncreasedCostOfStockBorrow() {
			return increasedCostOfStockBorrow;
		}
		
		
		public BigDecimal getInitialStockLoanRate() {
			return initialStockLoanRate;
		}
		
		
		public Boolean getInsolvencyFiling() {
			return insolvencyFiling;
		}
		
		
		public Boolean getLossOfStockBorrow() {
			return lossOfStockBorrow;
		}
		
		
		public BigDecimal getMaximumStockLoanRate() {
			return maximumStockLoanRate;
		}
		
	
		public AdditionalDisruptionEventsBuilder setChangeInLaw(Boolean changeInLaw) {
			this.changeInLaw = changeInLaw;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setDeterminingPartyReference(ReferenceWithMetaParty determiningPartyReference) {
			this.determiningPartyReference = determiningPartyReference.toBuilder();
			return this;
		}
		public AdditionalDisruptionEventsBuilder setDeterminingPartyReferenceBuilder(ReferenceWithMetaPartyBuilder determiningPartyReference) {
			this.determiningPartyReference = determiningPartyReference;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setDeterminingPartyReferenceRef(Party.PartyBuilder determiningPartyReference) {
			return setDeterminingPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(determiningPartyReference).build());
		}
		public AdditionalDisruptionEventsBuilder setDeterminingPartyReferenceRef(Party determiningPartyReference) {
			return setDeterminingPartyReferenceRef(Party.builder());
		}
		public AdditionalDisruptionEventsBuilder setFailureToDeliver(Boolean failureToDeliver) {
			this.failureToDeliver = failureToDeliver;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setForeignOwnershipEvent(Boolean foreignOwnershipEvent) {
			this.foreignOwnershipEvent = foreignOwnershipEvent;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setHedgingDisruption(Boolean hedgingDisruption) {
			this.hedgingDisruption = hedgingDisruption;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setIncreasedCostOfHedging(Boolean increasedCostOfHedging) {
			this.increasedCostOfHedging = increasedCostOfHedging;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setIncreasedCostOfStockBorrow(Boolean increasedCostOfStockBorrow) {
			this.increasedCostOfStockBorrow = increasedCostOfStockBorrow;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setInitialStockLoanRate(BigDecimal initialStockLoanRate) {
			this.initialStockLoanRate = initialStockLoanRate;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setInsolvencyFiling(Boolean insolvencyFiling) {
			this.insolvencyFiling = insolvencyFiling;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setLossOfStockBorrow(Boolean lossOfStockBorrow) {
			this.lossOfStockBorrow = lossOfStockBorrow;
			return this;
		}
		public AdditionalDisruptionEventsBuilder setMaximumStockLoanRate(BigDecimal maximumStockLoanRate) {
			this.maximumStockLoanRate = maximumStockLoanRate;
			return this;
		}
		
		public AdditionalDisruptionEvents build() {
			return new AdditionalDisruptionEvents(this);
		}
	
		@Override
		public AdditionalDisruptionEventsBuilder prune() {
			if (determiningPartyReference!=null && !determiningPartyReference.prune().hasData()) determiningPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getChangeInLaw()!=null) return true;
			if (getDeterminingPartyReference()!=null && getDeterminingPartyReference().hasData()) return true;
			if (getFailureToDeliver()!=null) return true;
			if (getForeignOwnershipEvent()!=null) return true;
			if (getHedgingDisruption()!=null) return true;
			if (getIncreasedCostOfHedging()!=null) return true;
			if (getIncreasedCostOfStockBorrow()!=null) return true;
			if (getInitialStockLoanRate()!=null) return true;
			if (getInsolvencyFiling()!=null) return true;
			if (getLossOfStockBorrow()!=null) return true;
			if (getMaximumStockLoanRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("changeInLaw"), Boolean.class, changeInLaw, this);
			processor.processBasic(path.newSubPath("failureToDeliver"), Boolean.class, failureToDeliver, this);
			processor.processBasic(path.newSubPath("foreignOwnershipEvent"), Boolean.class, foreignOwnershipEvent, this);
			processor.processBasic(path.newSubPath("hedgingDisruption"), Boolean.class, hedgingDisruption, this);
			processor.processBasic(path.newSubPath("increasedCostOfHedging"), Boolean.class, increasedCostOfHedging, this);
			processor.processBasic(path.newSubPath("increasedCostOfStockBorrow"), Boolean.class, increasedCostOfStockBorrow, this);
			processor.processBasic(path.newSubPath("initialStockLoanRate"), BigDecimal.class, initialStockLoanRate, this);
			processor.processBasic(path.newSubPath("insolvencyFiling"), Boolean.class, insolvencyFiling, this);
			processor.processBasic(path.newSubPath("lossOfStockBorrow"), Boolean.class, lossOfStockBorrow, this);
			processor.processBasic(path.newSubPath("maximumStockLoanRate"), BigDecimal.class, maximumStockLoanRate, this);
			
			processRosetta(path.newSubPath("determiningPartyReference"), processor, ReferenceWithMetaParty.class, determiningPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalDisruptionEventsBuilder _that = (AdditionalDisruptionEventsBuilder) o;
		
			if (changeInLaw != null ? !changeInLaw.equals(_that.changeInLaw) : _that.changeInLaw != null) return false;
			if (determiningPartyReference != null ? !determiningPartyReference.equals(_that.determiningPartyReference) : _that.determiningPartyReference != null) return false;
			if (failureToDeliver != null ? !failureToDeliver.equals(_that.failureToDeliver) : _that.failureToDeliver != null) return false;
			if (foreignOwnershipEvent != null ? !foreignOwnershipEvent.equals(_that.foreignOwnershipEvent) : _that.foreignOwnershipEvent != null) return false;
			if (hedgingDisruption != null ? !hedgingDisruption.equals(_that.hedgingDisruption) : _that.hedgingDisruption != null) return false;
			if (increasedCostOfHedging != null ? !increasedCostOfHedging.equals(_that.increasedCostOfHedging) : _that.increasedCostOfHedging != null) return false;
			if (increasedCostOfStockBorrow != null ? !increasedCostOfStockBorrow.equals(_that.increasedCostOfStockBorrow) : _that.increasedCostOfStockBorrow != null) return false;
			if (initialStockLoanRate != null ? !initialStockLoanRate.equals(_that.initialStockLoanRate) : _that.initialStockLoanRate != null) return false;
			if (insolvencyFiling != null ? !insolvencyFiling.equals(_that.insolvencyFiling) : _that.insolvencyFiling != null) return false;
			if (lossOfStockBorrow != null ? !lossOfStockBorrow.equals(_that.lossOfStockBorrow) : _that.lossOfStockBorrow != null) return false;
			if (maximumStockLoanRate != null ? !maximumStockLoanRate.equals(_that.maximumStockLoanRate) : _that.maximumStockLoanRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (changeInLaw != null ? changeInLaw.hashCode() : 0);
			_result = 31 * _result + (determiningPartyReference != null ? determiningPartyReference.hashCode() : 0);
			_result = 31 * _result + (failureToDeliver != null ? failureToDeliver.hashCode() : 0);
			_result = 31 * _result + (foreignOwnershipEvent != null ? foreignOwnershipEvent.hashCode() : 0);
			_result = 31 * _result + (hedgingDisruption != null ? hedgingDisruption.hashCode() : 0);
			_result = 31 * _result + (increasedCostOfHedging != null ? increasedCostOfHedging.hashCode() : 0);
			_result = 31 * _result + (increasedCostOfStockBorrow != null ? increasedCostOfStockBorrow.hashCode() : 0);
			_result = 31 * _result + (initialStockLoanRate != null ? initialStockLoanRate.hashCode() : 0);
			_result = 31 * _result + (insolvencyFiling != null ? insolvencyFiling.hashCode() : 0);
			_result = 31 * _result + (lossOfStockBorrow != null ? lossOfStockBorrow.hashCode() : 0);
			_result = 31 * _result + (maximumStockLoanRate != null ? maximumStockLoanRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalDisruptionEventsBuilder {" +
				"changeInLaw=" + this.changeInLaw + ", " +
				"determiningPartyReference=" + this.determiningPartyReference + ", " +
				"failureToDeliver=" + this.failureToDeliver + ", " +
				"foreignOwnershipEvent=" + this.foreignOwnershipEvent + ", " +
				"hedgingDisruption=" + this.hedgingDisruption + ", " +
				"increasedCostOfHedging=" + this.increasedCostOfHedging + ", " +
				"increasedCostOfStockBorrow=" + this.increasedCostOfStockBorrow + ", " +
				"initialStockLoanRate=" + this.initialStockLoanRate + ", " +
				"insolvencyFiling=" + this.insolvencyFiling + ", " +
				"lossOfStockBorrow=" + this.lossOfStockBorrow + ", " +
				"maximumStockLoanRate=" + this.maximumStockLoanRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("changeInLaw"), Boolean.class, changeInLaw, this);
		processor.processBasic(path.newSubPath("failureToDeliver"), Boolean.class, failureToDeliver, this);
		processor.processBasic(path.newSubPath("foreignOwnershipEvent"), Boolean.class, foreignOwnershipEvent, this);
		processor.processBasic(path.newSubPath("hedgingDisruption"), Boolean.class, hedgingDisruption, this);
		processor.processBasic(path.newSubPath("increasedCostOfHedging"), Boolean.class, increasedCostOfHedging, this);
		processor.processBasic(path.newSubPath("increasedCostOfStockBorrow"), Boolean.class, increasedCostOfStockBorrow, this);
		processor.processBasic(path.newSubPath("initialStockLoanRate"), BigDecimal.class, initialStockLoanRate, this);
		processor.processBasic(path.newSubPath("insolvencyFiling"), Boolean.class, insolvencyFiling, this);
		processor.processBasic(path.newSubPath("lossOfStockBorrow"), Boolean.class, lossOfStockBorrow, this);
		processor.processBasic(path.newSubPath("maximumStockLoanRate"), BigDecimal.class, maximumStockLoanRate, this);
		
		processRosetta(path.newSubPath("determiningPartyReference"), processor, ReferenceWithMetaParty.class, determiningPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalDisruptionEvents _that = (AdditionalDisruptionEvents) o;
	
		if (changeInLaw != null ? !changeInLaw.equals(_that.changeInLaw) : _that.changeInLaw != null) return false;
		if (determiningPartyReference != null ? !determiningPartyReference.equals(_that.determiningPartyReference) : _that.determiningPartyReference != null) return false;
		if (failureToDeliver != null ? !failureToDeliver.equals(_that.failureToDeliver) : _that.failureToDeliver != null) return false;
		if (foreignOwnershipEvent != null ? !foreignOwnershipEvent.equals(_that.foreignOwnershipEvent) : _that.foreignOwnershipEvent != null) return false;
		if (hedgingDisruption != null ? !hedgingDisruption.equals(_that.hedgingDisruption) : _that.hedgingDisruption != null) return false;
		if (increasedCostOfHedging != null ? !increasedCostOfHedging.equals(_that.increasedCostOfHedging) : _that.increasedCostOfHedging != null) return false;
		if (increasedCostOfStockBorrow != null ? !increasedCostOfStockBorrow.equals(_that.increasedCostOfStockBorrow) : _that.increasedCostOfStockBorrow != null) return false;
		if (initialStockLoanRate != null ? !initialStockLoanRate.equals(_that.initialStockLoanRate) : _that.initialStockLoanRate != null) return false;
		if (insolvencyFiling != null ? !insolvencyFiling.equals(_that.insolvencyFiling) : _that.insolvencyFiling != null) return false;
		if (lossOfStockBorrow != null ? !lossOfStockBorrow.equals(_that.lossOfStockBorrow) : _that.lossOfStockBorrow != null) return false;
		if (maximumStockLoanRate != null ? !maximumStockLoanRate.equals(_that.maximumStockLoanRate) : _that.maximumStockLoanRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (changeInLaw != null ? changeInLaw.hashCode() : 0);
		_result = 31 * _result + (determiningPartyReference != null ? determiningPartyReference.hashCode() : 0);
		_result = 31 * _result + (failureToDeliver != null ? failureToDeliver.hashCode() : 0);
		_result = 31 * _result + (foreignOwnershipEvent != null ? foreignOwnershipEvent.hashCode() : 0);
		_result = 31 * _result + (hedgingDisruption != null ? hedgingDisruption.hashCode() : 0);
		_result = 31 * _result + (increasedCostOfHedging != null ? increasedCostOfHedging.hashCode() : 0);
		_result = 31 * _result + (increasedCostOfStockBorrow != null ? increasedCostOfStockBorrow.hashCode() : 0);
		_result = 31 * _result + (initialStockLoanRate != null ? initialStockLoanRate.hashCode() : 0);
		_result = 31 * _result + (insolvencyFiling != null ? insolvencyFiling.hashCode() : 0);
		_result = 31 * _result + (lossOfStockBorrow != null ? lossOfStockBorrow.hashCode() : 0);
		_result = 31 * _result + (maximumStockLoanRate != null ? maximumStockLoanRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalDisruptionEvents {" +
			"changeInLaw=" + this.changeInLaw + ", " +
			"determiningPartyReference=" + this.determiningPartyReference + ", " +
			"failureToDeliver=" + this.failureToDeliver + ", " +
			"foreignOwnershipEvent=" + this.foreignOwnershipEvent + ", " +
			"hedgingDisruption=" + this.hedgingDisruption + ", " +
			"increasedCostOfHedging=" + this.increasedCostOfHedging + ", " +
			"increasedCostOfStockBorrow=" + this.increasedCostOfStockBorrow + ", " +
			"initialStockLoanRate=" + this.initialStockLoanRate + ", " +
			"insolvencyFiling=" + this.insolvencyFiling + ", " +
			"lossOfStockBorrow=" + this.lossOfStockBorrow + ", " +
			"maximumStockLoanRate=" + this.maximumStockLoanRate +
		'}';
	}
}
