package org.isda.cdm;

import java.lang.Boolean;
import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdditionalRepresentationElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the parties&#39; Additional Representation(s) election.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdditionalRepresentationElection extends RosettaModelObject {
	private final Boolean isApplicable;
	private final String party;
	private static AdditionalRepresentationElectionMeta metaData = new AdditionalRepresentationElectionMeta();
	
	AdditionalRepresentationElection(AdditionalRepresentationElectionBuilder builder) {
		this.isApplicable = builder.getIsApplicable();
		this.party = builder.getParty();
	}
	
	/**
	 * The Additional Representation is applicable when True, and not applicable when False.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final Boolean getIsApplicable() {
		return isApplicable;
	}
	
	/**
	 * The elective party.
	 */
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	@RosettaSynonym(value="", source="ISDA_Create_1_0")
	public final String getParty() {
		return party;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdditionalRepresentationElection> metaData() {
		return metaData;
	} 
	
	public AdditionalRepresentationElectionBuilder toBuilder() {
		AdditionalRepresentationElectionBuilder builder = new AdditionalRepresentationElectionBuilder();
		ofNullable(getIsApplicable()).ifPresent(builder::setIsApplicable);
		ofNullable(getParty()).ifPresent(builder::setParty);
		return builder;
	}

	public static AdditionalRepresentationElectionBuilder builder() {
		return new AdditionalRepresentationElectionBuilder();
	}

	public static  class AdditionalRepresentationElectionBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean isApplicable;
		protected String party;
	
		public AdditionalRepresentationElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdditionalRepresentationElection> metaData() {
			return metaData;
		} 
	
		
		public Boolean getIsApplicable() {
			return isApplicable;
		}
		
		
		public String getParty() {
			return party;
		}
		
	
		public AdditionalRepresentationElectionBuilder setIsApplicable(Boolean isApplicable) {
			this.isApplicable = isApplicable;
			return this;
		}
		public AdditionalRepresentationElectionBuilder setParty(String party) {
			this.party = party;
			return this;
		}
		
		public AdditionalRepresentationElection build() {
			return new AdditionalRepresentationElection(this);
		}
	
		@Override
		public AdditionalRepresentationElectionBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsApplicable()!=null) return true;
			if (getParty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
			processor.processBasic(path.newSubPath("party"), String.class, party, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdditionalRepresentationElectionBuilder _that = (AdditionalRepresentationElectionBuilder) o;
		
			if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
			if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdditionalRepresentationElectionBuilder {" +
				"isApplicable=" + this.isApplicable + ", " +
				"party=" + this.party +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isApplicable"), Boolean.class, isApplicable, this);
		processor.processBasic(path.newSubPath("party"), String.class, party, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdditionalRepresentationElection _that = (AdditionalRepresentationElection) o;
	
		if (isApplicable != null ? !isApplicable.equals(_that.isApplicable) : _that.isApplicable != null) return false;
		if (party != null ? !party.equals(_that.party) : _that.party != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (isApplicable != null ? isApplicable.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdditionalRepresentationElection {" +
			"isApplicable=" + this.isApplicable + ", " +
			"party=" + this.party +
		'}';
	}
}
