package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AddtlAttrbtsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class AddtlAttrbts extends RosettaModelObject {
	private final String rskRdcgTx;
	private final String sctiesFincgTxInd;
	private static AddtlAttrbtsMeta metaData = new AddtlAttrbtsMeta();
	
	AddtlAttrbts(AddtlAttrbtsBuilder builder) {
		this.rskRdcgTx = builder.getRskRdcgTx();
		this.sctiesFincgTxInd = builder.getSctiesFincgTxInd();
	}
	
	@RosettaSynonym(value="RskRdcgTx", source="ISO20022")
	public final String getRskRdcgTx() {
		return rskRdcgTx;
	}
	
	@RosettaSynonym(value="SctiesFincgTxInd", source="ISO20022")
	public final String getSctiesFincgTxInd() {
		return sctiesFincgTxInd;
	}
	
	
	@Override
	public RosettaMetaData<? extends AddtlAttrbts> metaData() {
		return metaData;
	} 
	
	public AddtlAttrbtsBuilder toBuilder() {
		AddtlAttrbtsBuilder builder = new AddtlAttrbtsBuilder();
		ofNullable(getRskRdcgTx()).ifPresent(builder::setRskRdcgTx);
		ofNullable(getSctiesFincgTxInd()).ifPresent(builder::setSctiesFincgTxInd);
		return builder;
	}

	public static AddtlAttrbtsBuilder builder() {
		return new AddtlAttrbtsBuilder();
	}

	public static  class AddtlAttrbtsBuilder extends RosettaModelObjectBuilder {
	
		protected String rskRdcgTx;
		protected String sctiesFincgTxInd;
	
		public AddtlAttrbtsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AddtlAttrbts> metaData() {
			return metaData;
		} 
	
		
		public String getRskRdcgTx() {
			return rskRdcgTx;
		}
		
		
		public String getSctiesFincgTxInd() {
			return sctiesFincgTxInd;
		}
		
	
		public AddtlAttrbtsBuilder setRskRdcgTx(String rskRdcgTx) {
			this.rskRdcgTx = rskRdcgTx;
			return this;
		}
		public AddtlAttrbtsBuilder setSctiesFincgTxInd(String sctiesFincgTxInd) {
			this.sctiesFincgTxInd = sctiesFincgTxInd;
			return this;
		}
		
		public AddtlAttrbts build() {
			return new AddtlAttrbts(this);
		}
	
		@Override
		public AddtlAttrbtsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getRskRdcgTx()!=null) return true;
			if (getSctiesFincgTxInd()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("rskRdcgTx"), String.class, rskRdcgTx, this);
			processor.processBasic(path.newSubPath("sctiesFincgTxInd"), String.class, sctiesFincgTxInd, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AddtlAttrbtsBuilder _that = (AddtlAttrbtsBuilder) o;
		
			if (rskRdcgTx != null ? !rskRdcgTx.equals(_that.rskRdcgTx) : _that.rskRdcgTx != null) return false;
			if (sctiesFincgTxInd != null ? !sctiesFincgTxInd.equals(_that.sctiesFincgTxInd) : _that.sctiesFincgTxInd != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (rskRdcgTx != null ? rskRdcgTx.hashCode() : 0);
			_result = 31 * _result + (sctiesFincgTxInd != null ? sctiesFincgTxInd.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AddtlAttrbtsBuilder {" +
				"rskRdcgTx=" + this.rskRdcgTx + ", " +
				"sctiesFincgTxInd=" + this.sctiesFincgTxInd +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("rskRdcgTx"), String.class, rskRdcgTx, this);
		processor.processBasic(path.newSubPath("sctiesFincgTxInd"), String.class, sctiesFincgTxInd, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AddtlAttrbts _that = (AddtlAttrbts) o;
	
		if (rskRdcgTx != null ? !rskRdcgTx.equals(_that.rskRdcgTx) : _that.rskRdcgTx != null) return false;
		if (sctiesFincgTxInd != null ? !sctiesFincgTxInd.equals(_that.sctiesFincgTxInd) : _that.sctiesFincgTxInd != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (rskRdcgTx != null ? rskRdcgTx.hashCode() : 0);
		_result = 31 * _result + (sctiesFincgTxInd != null ? sctiesFincgTxInd.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AddtlAttrbts {" +
			"rskRdcgTx=" + this.rskRdcgTx + ", " +
			"sctiesFincgTxInd=" + this.sctiesFincgTxInd +
		'}';
	}
}
