package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessDayAdjustments;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessDayAdjustments.ReferenceWithMetaBusinessDayAdjustmentsBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableDateMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A class for defining a date that shall be subject to adjustment if it would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the date.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdjustableDate", source="FpML_5_10")
@RosettaSynonym(value="AdjustableDate", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="AdjustableDate", source="DTCC_11_0")
@RosettaSynonym(value="AdjustableDate", source="DTCC_9_0")
@RosettaSynonym(value="AdjustableDate", source="CME_ClearedConfirm_1_17")

@RosettaSynonym(value="AdjustableDate2", source="FpML_5_10")
@RosettaSynonym(value="AdjustableDate2", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="AdjustableDate2", source="DTCC_11_0")
@RosettaSynonym(value="AdjustableDate2", source="DTCC_9_0")
@RosettaSynonym(value="AdjustableDate2", source="CME_ClearedConfirm_1_17")
public class AdjustableDate extends RosettaModelObject implements GlobalKey {
	private final FieldWithMetaDate adjustedDate;
	private final BusinessDayAdjustments dateAdjustments;
	private final ReferenceWithMetaBusinessDayAdjustments dateAdjustmentsReference;
	private final MetaFields meta;
	private final Date unadjustedDate;
	private static AdjustableDateMeta metaData = new AdjustableDateMeta();
	
	AdjustableDate(AdjustableDateBuilder builder) {
		this.adjustedDate = ofNullable(builder.getAdjustedDate()).map(f->f.build()).orElse(null);
		this.dateAdjustments = ofNullable(builder.getDateAdjustments()).map(f->f.build()).orElse(null);
		this.dateAdjustmentsReference = ofNullable(builder.getDateAdjustmentsReference()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.unadjustedDate = builder.getUnadjustedDate();
	}
	
	/**
	 * The date once the adjustment has been performed. (Note that this date may change if the business center holidays change).
	 */
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="adjustedTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedTerminationDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustedEffectiveDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedEffectiveDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedEffectiveDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedEffectiveDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedEffectiveDate", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaDate getAdjustedDate() {
		return adjustedDate;
	}
	
	/**
	 * The business day convention and financial business centers used for adjusting the date if it would otherwise fall on a day that is not a business date in the specified business centers.
	 */
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="calculationPeriodDates.terminationDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates.terminationDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="calculationPeriodDates.terminationDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="calculationPeriodDates.terminationDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates.terminationDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="calculationPeriodDates.effectiveDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="calculationPeriodDates.effectiveDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="calculationPeriodDates.effectiveDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="calculationPeriodDates.effectiveDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="calculationPeriodDates.effectiveDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="paymentDate")
	public final BusinessDayAdjustments getDateAdjustments() {
		return dateAdjustments;
	}
	
	/**
	 * A pointer style reference to date adjustments defined elsewhere in the document.
	 */
	@RosettaSynonym(value="dateAdjustmentsReference", source="FpML_5_10")
	@RosettaSynonym(value="dateAdjustmentsReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateAdjustmentsReference", source="DTCC_11_0")
	@RosettaSynonym(value="dateAdjustmentsReference", source="DTCC_9_0")
	@RosettaSynonym(value="dateAdjustmentsReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaBusinessDayAdjustments getDateAdjustmentsReference() {
		return dateAdjustmentsReference;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A date subject to adjustment. While in FpML this date is required, this cardinality constraint has been relaxed as part of the CDM in order to support the FRA representation, which effective and termination dates are specified in FpML as adjusted dates.
	 */
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", maps=2)
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="firstPeriodStartDate", source="FpML_5_10")
	@RosettaSynonym(value="firstPeriodStartDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="firstPeriodStartDate", source="DTCC_11_0")
	@RosettaSynonym(value="firstPeriodStartDate", source="DTCC_9_0")
	@RosettaSynonym(value="firstPeriodStartDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustablePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustablePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedDate() {
		return unadjustedDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableDate> metaData() {
		return metaData;
	} 
	
	public AdjustableDateBuilder toBuilder() {
		AdjustableDateBuilder builder = new AdjustableDateBuilder();
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getDateAdjustments()).ifPresent(builder::setDateAdjustments);
		ofNullable(getDateAdjustmentsReference()).ifPresent(builder::setDateAdjustmentsReference);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getUnadjustedDate()).ifPresent(builder::setUnadjustedDate);
		return builder;
	}

	public static AdjustableDateBuilder builder() {
		return new AdjustableDateBuilder();
	}

	public static  class AdjustableDateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AdjustableDateBuilder> {
	
		protected FieldWithMetaDateBuilder adjustedDate;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments;
		protected ReferenceWithMetaBusinessDayAdjustmentsBuilder dateAdjustmentsReference;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Date unadjustedDate;
	
		public AdjustableDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableDate> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaDateBuilder getAdjustedDate() {
			return adjustedDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateAdjustedDate() {
			if (adjustedDate!=null) {
				return adjustedDate;
			}
			else return adjustedDate = new FieldWithMetaDateBuilder();
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getDateAdjustments() {
			return dateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateDateAdjustments() {
			if (dateAdjustments!=null) {
				return dateAdjustments;
			}
			else return dateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public ReferenceWithMetaBusinessDayAdjustmentsBuilder getDateAdjustmentsReference() {
			return dateAdjustmentsReference;
		}
		
		public ReferenceWithMetaBusinessDayAdjustmentsBuilder getOrCreateDateAdjustmentsReference() {
			if (dateAdjustmentsReference!=null) {
				return dateAdjustmentsReference;
			}
			else return dateAdjustmentsReference = new ReferenceWithMetaBusinessDayAdjustmentsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Date getUnadjustedDate() {
			return unadjustedDate;
		}
		
	
		public AdjustableDateBuilder setAdjustedDate(FieldWithMetaDate adjustedDate) {
			this.adjustedDate = adjustedDate.toBuilder();
			return this;
		}
		public AdjustableDateBuilder setAdjustedDateRef(Date adjustedDate) {
			return setAdjustedDate(FieldWithMetaDate.builder().setValue(adjustedDate).build());
		}
		public AdjustableDateBuilder setDateAdjustments(BusinessDayAdjustments dateAdjustments) {
			this.dateAdjustments = dateAdjustments.toBuilder();
			return this;
		}
		public AdjustableDateBuilder setDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments) {
			this.dateAdjustments = dateAdjustments;
			return this;
		}
		public AdjustableDateBuilder setDateAdjustmentsReference(ReferenceWithMetaBusinessDayAdjustments dateAdjustmentsReference) {
			this.dateAdjustmentsReference = dateAdjustmentsReference.toBuilder();
			return this;
		}
		public AdjustableDateBuilder setDateAdjustmentsReferenceBuilder(ReferenceWithMetaBusinessDayAdjustmentsBuilder dateAdjustmentsReference) {
			this.dateAdjustmentsReference = dateAdjustmentsReference;
			return this;
		}
		public AdjustableDateBuilder setDateAdjustmentsReferenceRef(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustmentsReference) {
			return setDateAdjustmentsReference(ReferenceWithMetaBusinessDayAdjustments.builder().setValueBuilder(dateAdjustmentsReference).build());
		}
		public AdjustableDateBuilder setDateAdjustmentsReferenceRef(BusinessDayAdjustments dateAdjustmentsReference) {
			return setDateAdjustmentsReferenceRef(BusinessDayAdjustments.builder());
		}
		public AdjustableDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AdjustableDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AdjustableDateBuilder setUnadjustedDate(Date unadjustedDate) {
			this.unadjustedDate = unadjustedDate;
			return this;
		}
		
		public AdjustableDate build() {
			return new AdjustableDate(this);
		}
	
		@Override
		public AdjustableDateBuilder prune() {
			if (adjustedDate!=null && !adjustedDate.prune().hasData()) adjustedDate = null;
			if (dateAdjustments!=null && !dateAdjustments.prune().hasData()) dateAdjustments = null;
			if (dateAdjustmentsReference!=null && !dateAdjustmentsReference.prune().hasData()) dateAdjustmentsReference = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedDate()!=null) return true;
			if (getDateAdjustments()!=null && getDateAdjustments().hasData()) return true;
			if (getDateAdjustmentsReference()!=null && getDateAdjustmentsReference().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getUnadjustedDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
			
			processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
			processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
			processRosetta(path.newSubPath("dateAdjustmentsReference"), processor, ReferenceWithMetaBusinessDayAdjustments.class, dateAdjustmentsReference);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableDateBuilder _that = (AdjustableDateBuilder) o;
		
			if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
			if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
			if (dateAdjustmentsReference != null ? !dateAdjustmentsReference.equals(_that.dateAdjustmentsReference) : _that.dateAdjustmentsReference != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
			_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
			_result = 31 * _result + (dateAdjustmentsReference != null ? dateAdjustmentsReference.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableDateBuilder {" +
				"adjustedDate=" + this.adjustedDate + ", " +
				"dateAdjustments=" + this.dateAdjustments + ", " +
				"dateAdjustmentsReference=" + this.dateAdjustmentsReference + ", " +
				"meta=" + this.meta + ", " +
				"unadjustedDate=" + this.unadjustedDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
		
		processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
		processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
		processRosetta(path.newSubPath("dateAdjustmentsReference"), processor, ReferenceWithMetaBusinessDayAdjustments.class, dateAdjustmentsReference);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableDate _that = (AdjustableDate) o;
	
		if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
		if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
		if (dateAdjustmentsReference != null ? !dateAdjustmentsReference.equals(_that.dateAdjustmentsReference) : _that.dateAdjustmentsReference != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
		_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
		_result = 31 * _result + (dateAdjustmentsReference != null ? dateAdjustmentsReference.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableDate {" +
			"adjustedDate=" + this.adjustedDate + ", " +
			"dateAdjustments=" + this.dateAdjustments + ", " +
			"dateAdjustmentsReference=" + this.dateAdjustmentsReference + ", " +
			"meta=" + this.meta + ", " +
			"unadjustedDate=" + this.unadjustedDate +
		'}';
	}
}
