package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining a series of dates that shall be subject to adjustment if they would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the dates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdjustableDates", source="FpML_5_10")
@RosettaSynonym(value="AdjustableDates", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="AdjustableDates", source="DTCC_11_0")
@RosettaSynonym(value="AdjustableDates", source="DTCC_9_0")
@RosettaSynonym(value="AdjustableDates", source="CME_ClearedConfirm_1_17")
public class AdjustableDates extends RosettaModelObject implements GlobalKey {
	private final List<FieldWithMetaDate> adjustedDate;
	private final BusinessDayAdjustments dateAdjustments;
	private final MetaFields meta;
	private final List<Date> unadjustedDate;
	private static AdjustableDatesMeta metaData = new AdjustableDatesMeta();
	
	AdjustableDates(AdjustableDatesBuilder builder) {
		this.adjustedDate = ofNullable(builder.getAdjustedDate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.dateAdjustments = ofNullable(builder.getDateAdjustments()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.unadjustedDate = builder.getUnadjustedDate();
	}
	
	/**
	 * The date(s) once the adjustment has been performed. (Note that this date may change if the business center holidays change).
	 */
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaDate> getAdjustedDate() {
		return adjustedDate;
	}
	
	/**
	 * The business day convention and financial business centers used for adjusting the date if it would otherwise fall on a day that is not a business date in the specified business centers.
	 */
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17")
	public final BusinessDayAdjustments getDateAdjustments() {
		return dateAdjustments;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A date subject to adjustment.
	 */
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17")
	public final List<Date> getUnadjustedDate() {
		return unadjustedDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableDates> metaData() {
		return metaData;
	} 
	
	public AdjustableDatesBuilder toBuilder() {
		AdjustableDatesBuilder builder = new AdjustableDatesBuilder();
		ofNullable(getAdjustedDate()).ifPresent(adjustedDate -> adjustedDate.forEach(builder::addAdjustedDate));
		ofNullable(getDateAdjustments()).ifPresent(builder::setDateAdjustments);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getUnadjustedDate()).ifPresent(unadjustedDate -> unadjustedDate.forEach(builder::addUnadjustedDate));
		return builder;
	}

	public static AdjustableDatesBuilder builder() {
		return new AdjustableDatesBuilder();
	}

	public static  class AdjustableDatesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AdjustableDatesBuilder> {
	
		protected List<FieldWithMetaDateBuilder> adjustedDate;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<Date> unadjustedDate;
	
		public AdjustableDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableDates> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaDateBuilder> getAdjustedDate() {
			return adjustedDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateAdjustedDate(int index) {
			if (adjustedDate==null) {
				this.adjustedDate = new ArrayList<>();
			}
			return getIndex(adjustedDate, index, ()->new FieldWithMetaDateBuilder());
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getDateAdjustments() {
			return dateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateDateAdjustments() {
			if (dateAdjustments!=null) {
				return dateAdjustments;
			}
			else return dateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<Date> getUnadjustedDate() {
			return unadjustedDate;
		}
		
	
		public AdjustableDatesBuilder addAdjustedDate(FieldWithMetaDate adjustedDate) {
			if(this.adjustedDate == null){
				this.adjustedDate = new ArrayList<>();
			}
			this.adjustedDate.add(adjustedDate.toBuilder());
			return this;
		}
		public AdjustableDatesBuilder addAdjustedDate(FieldWithMetaDate adjustedDate, int _idx) {
			if(this.adjustedDate == null){
				this.adjustedDate = new ArrayList<>();
			}
			getIndex(this.adjustedDate, _idx, () -> adjustedDate.toBuilder());
			this.adjustedDate.set(_idx, adjustedDate.toBuilder());
			return this;
		}
		public AdjustableDatesBuilder addAdjustedDate(List<FieldWithMetaDate> adjustedDates) {
			if(this.adjustedDate == null){
				this.adjustedDate = new ArrayList<>();
			}
			for (FieldWithMetaDate toAdd : adjustedDates) {
				this.adjustedDate.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public AdjustableDatesBuilder clearAdjustedDate() {
			this.adjustedDate = null;
			return this;
		}
		public AdjustableDatesBuilder setDateAdjustments(BusinessDayAdjustments dateAdjustments) {
			this.dateAdjustments = dateAdjustments.toBuilder();
			return this;
		}
		public AdjustableDatesBuilder setDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments) {
			this.dateAdjustments = dateAdjustments;
			return this;
		}
		public AdjustableDatesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AdjustableDatesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AdjustableDatesBuilder addUnadjustedDate(Date unadjustedDate) {
			if(this.unadjustedDate == null){
				this.unadjustedDate = new ArrayList<>();
			}
			this.unadjustedDate.add(unadjustedDate);
			return this;
		}
		public AdjustableDatesBuilder addUnadjustedDate(Date unadjustedDate, int _idx) {
			if(this.unadjustedDate == null){
				this.unadjustedDate = new ArrayList<>();
			}
			getIndex(this.unadjustedDate, _idx, () -> unadjustedDate);
			this.unadjustedDate.set(_idx, unadjustedDate);
			return this;
		}
		public AdjustableDatesBuilder addUnadjustedDate(List<Date> unadjustedDates) {
			if(this.unadjustedDate == null){
				this.unadjustedDate = new ArrayList<>();
			}
			for (Date toAdd : unadjustedDates) {
				this.unadjustedDate.add(toAdd);
			}
			return this;
		}
		
		public AdjustableDatesBuilder clearUnadjustedDate() {
			this.unadjustedDate = null;
			return this;
		}
		
		public AdjustableDates build() {
			return new AdjustableDates(this);
		}
	
		@Override
		public AdjustableDatesBuilder prune() {
			if (adjustedDate!=null) adjustedDate = adjustedDate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (dateAdjustments!=null && !dateAdjustments.prune().hasData()) dateAdjustments = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedDate()!=null && !getAdjustedDate().isEmpty()) return true;
			if (getDateAdjustments()!=null && getDateAdjustments().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getUnadjustedDate()!=null && !getUnadjustedDate().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
			
			processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
			processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableDatesBuilder _that = (AdjustableDatesBuilder) o;
		
			if (!ListEquals.listEquals(adjustedDate, _that.adjustedDate)) return false;
			if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(unadjustedDate, _that.unadjustedDate)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
			_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableDatesBuilder {" +
				"adjustedDate=" + this.adjustedDate + ", " +
				"dateAdjustments=" + this.dateAdjustments + ", " +
				"meta=" + this.meta + ", " +
				"unadjustedDate=" + this.unadjustedDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		unadjustedDate.stream().forEach(a->processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, a, this));
		
		processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
		processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableDates _that = (AdjustableDates) o;
	
		if (!ListEquals.listEquals(adjustedDate, _that.adjustedDate)) return false;
		if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(unadjustedDate, _that.unadjustedDate)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
		_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableDates {" +
			"adjustedDate=" + this.adjustedDate + ", " +
			"dateAdjustments=" + this.dateAdjustments + ", " +
			"meta=" + this.meta + ", " +
			"unadjustedDate=" + this.unadjustedDate +
		'}';
	}
}
