package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableOrAdjustedDateMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining a date that shall be subject to adjustment if it would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the date.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdjustableOrAdjustedDate", source="FpML_5_10")
@RosettaSynonym(value="AdjustableOrAdjustedDate", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="AdjustableOrAdjustedDate", source="DTCC_11_0")
@RosettaSynonym(value="AdjustableOrAdjustedDate", source="DTCC_9_0")
@RosettaSynonym(value="AdjustableOrAdjustedDate", source="CME_ClearedConfirm_1_17")
public class AdjustableOrAdjustedDate extends RosettaModelObject implements GlobalKey {
	private final FieldWithMetaDate adjustedDate;
	private final BusinessDayAdjustments dateAdjustments;
	private final MetaFields meta;
	private final Date unadjustedDate;
	private static AdjustableOrAdjustedDateMeta metaData = new AdjustableOrAdjustedDateMeta();
	
	AdjustableOrAdjustedDate(AdjustableOrAdjustedDateBuilder builder) {
		this.adjustedDate = ofNullable(builder.getAdjustedDate()).map(f->f.build()).orElse(null);
		this.dateAdjustments = ofNullable(builder.getDateAdjustments()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.unadjustedDate = builder.getUnadjustedDate();
	}
	
	/**
	 * The date once the adjustment has been performed. (Note that this date may change if the business center holidays change).
	 */
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaDate getAdjustedDate() {
		return adjustedDate;
	}
	
	/**
	 * The business day convention and financial business centers used for adjusting the date if it would otherwise fall on a day that is not a business date in the specified business centers.
	 */
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17")
	public final BusinessDayAdjustments getDateAdjustments() {
		return dateAdjustments;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A date subject to adjustment.
	 */
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedDate() {
		return unadjustedDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableOrAdjustedDate> metaData() {
		return metaData;
	} 
	
	public AdjustableOrAdjustedDateBuilder toBuilder() {
		AdjustableOrAdjustedDateBuilder builder = new AdjustableOrAdjustedDateBuilder();
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getDateAdjustments()).ifPresent(builder::setDateAdjustments);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getUnadjustedDate()).ifPresent(builder::setUnadjustedDate);
		return builder;
	}

	public static AdjustableOrAdjustedDateBuilder builder() {
		return new AdjustableOrAdjustedDateBuilder();
	}

	public static  class AdjustableOrAdjustedDateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AdjustableOrAdjustedDateBuilder> {
	
		protected FieldWithMetaDateBuilder adjustedDate;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Date unadjustedDate;
	
		public AdjustableOrAdjustedDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableOrAdjustedDate> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaDateBuilder getAdjustedDate() {
			return adjustedDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateAdjustedDate() {
			if (adjustedDate!=null) {
				return adjustedDate;
			}
			else return adjustedDate = new FieldWithMetaDateBuilder();
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getDateAdjustments() {
			return dateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateDateAdjustments() {
			if (dateAdjustments!=null) {
				return dateAdjustments;
			}
			else return dateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Date getUnadjustedDate() {
			return unadjustedDate;
		}
		
	
		public AdjustableOrAdjustedDateBuilder setAdjustedDate(FieldWithMetaDate adjustedDate) {
			this.adjustedDate = adjustedDate.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedDateBuilder setAdjustedDateRef(Date adjustedDate) {
			return setAdjustedDate(FieldWithMetaDate.builder().setValue(adjustedDate).build());
		}
		public AdjustableOrAdjustedDateBuilder setDateAdjustments(BusinessDayAdjustments dateAdjustments) {
			this.dateAdjustments = dateAdjustments.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedDateBuilder setDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments) {
			this.dateAdjustments = dateAdjustments;
			return this;
		}
		public AdjustableOrAdjustedDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AdjustableOrAdjustedDateBuilder setUnadjustedDate(Date unadjustedDate) {
			this.unadjustedDate = unadjustedDate;
			return this;
		}
		
		public AdjustableOrAdjustedDate build() {
			return new AdjustableOrAdjustedDate(this);
		}
	
		@Override
		public AdjustableOrAdjustedDateBuilder prune() {
			if (adjustedDate!=null && !adjustedDate.prune().hasData()) adjustedDate = null;
			if (dateAdjustments!=null && !dateAdjustments.prune().hasData()) dateAdjustments = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedDate()!=null) return true;
			if (getDateAdjustments()!=null && getDateAdjustments().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getUnadjustedDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
			
			processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
			processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableOrAdjustedDateBuilder _that = (AdjustableOrAdjustedDateBuilder) o;
		
			if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
			if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
			_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableOrAdjustedDateBuilder {" +
				"adjustedDate=" + this.adjustedDate + ", " +
				"dateAdjustments=" + this.dateAdjustments + ", " +
				"meta=" + this.meta + ", " +
				"unadjustedDate=" + this.unadjustedDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
		
		processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
		processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableOrAdjustedDate _that = (AdjustableOrAdjustedDate) o;
	
		if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
		if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
		_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableOrAdjustedDate {" +
			"adjustedDate=" + this.adjustedDate + ", " +
			"dateAdjustments=" + this.dateAdjustments + ", " +
			"meta=" + this.meta + ", " +
			"unadjustedDate=" + this.unadjustedDate +
		'}';
	}
}
