package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.metafields.FieldWithMetaDate;
import org.isda.cdm.metafields.FieldWithMetaDate.FieldWithMetaDateBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableOrAdjustedOrRelativeDateMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * This Rosetta class specifies the date as either an unadjusted, adjusted or relative date. It supplements the features of the AdjustableOrAdjustedDate to support the credit default swap option premium, which uses the relative date construct.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdjustableOrAdjustedOrRelativeDate extends RosettaModelObject {
	private final FieldWithMetaDate adjustedDate;
	private final BusinessDayAdjustments dateAdjustments;
	private final RelativeDateOffset relativeDate;
	private final Date unadjustedDate;
	private static AdjustableOrAdjustedOrRelativeDateMeta metaData = new AdjustableOrAdjustedOrRelativeDateMeta();
	
	AdjustableOrAdjustedOrRelativeDate(AdjustableOrAdjustedOrRelativeDateBuilder builder) {
		this.adjustedDate = ofNullable(builder.getAdjustedDate()).map(f->f.build()).orElse(null);
		this.dateAdjustments = ofNullable(builder.getDateAdjustments()).map(f->f.build()).orElse(null);
		this.relativeDate = ofNullable(builder.getRelativeDate()).map(f->f.build()).orElse(null);
		this.unadjustedDate = builder.getUnadjustedDate();
	}
	
	/**
	 * The date once the adjustment has been performed. (Note that this date may change if the business center holidays change).
	 */
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="adjustedDate", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="adjustedDate", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	@RosettaSynonym(value="adjustedPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustedPaymentDate", source="FpML_5_10", path="FpML")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_11_0", path="FpML")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_9_0", path="FpML")
	public final FieldWithMetaDate getAdjustedDate() {
		return adjustedDate;
	}
	
	/**
	 * The business day convention and financial business centers used for adjusting the date if it would otherwise fall on a day that is not a business date in the specified business centers.
	 */
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="dateAdjustments", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="dateAdjustments", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	public final BusinessDayAdjustments getDateAdjustments() {
		return dateAdjustments;
	}
	
	/**
	 * A date specified as some offset to another date (the anchor date).
	 */
	@RosettaSynonym(value="relativeDate", source="FpML_5_10")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	public final RelativeDateOffset getRelativeDate() {
		return relativeDate;
	}
	
	/**
	 * A date subject to adjustment.
	 */
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="adjustablePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustablePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustablePaymentDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate.adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="unadjustedDate", source="FpML_5_10", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_SubmissionIRS_1_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_11_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="DTCC_9_0", path="adjustableDate")
	@RosettaSynonym(value="unadjustedDate", source="CME_ClearedConfirm_1_17", path="adjustableDate")
	public final Date getUnadjustedDate() {
		return unadjustedDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableOrAdjustedOrRelativeDate> metaData() {
		return metaData;
	} 
	
	public AdjustableOrAdjustedOrRelativeDateBuilder toBuilder() {
		AdjustableOrAdjustedOrRelativeDateBuilder builder = new AdjustableOrAdjustedOrRelativeDateBuilder();
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getDateAdjustments()).ifPresent(builder::setDateAdjustments);
		ofNullable(getRelativeDate()).ifPresent(builder::setRelativeDate);
		ofNullable(getUnadjustedDate()).ifPresent(builder::setUnadjustedDate);
		return builder;
	}

	public static AdjustableOrAdjustedOrRelativeDateBuilder builder() {
		return new AdjustableOrAdjustedOrRelativeDateBuilder();
	}

	public static  class AdjustableOrAdjustedOrRelativeDateBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaDateBuilder adjustedDate;
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments;
		protected RelativeDateOffset.RelativeDateOffsetBuilder relativeDate;
		protected Date unadjustedDate;
	
		public AdjustableOrAdjustedOrRelativeDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableOrAdjustedOrRelativeDate> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaDateBuilder getAdjustedDate() {
			return adjustedDate;
		}
		
		public FieldWithMetaDateBuilder getOrCreateAdjustedDate() {
			if (adjustedDate!=null) {
				return adjustedDate;
			}
			else return adjustedDate = new FieldWithMetaDateBuilder();
		}
		
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getDateAdjustments() {
			return dateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateDateAdjustments() {
			if (dateAdjustments!=null) {
				return dateAdjustments;
			}
			else return dateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getRelativeDate() {
			return relativeDate;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateRelativeDate() {
			if (relativeDate!=null) {
				return relativeDate;
			}
			else return relativeDate = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public Date getUnadjustedDate() {
			return unadjustedDate;
		}
		
	
		public AdjustableOrAdjustedOrRelativeDateBuilder setAdjustedDate(FieldWithMetaDate adjustedDate) {
			this.adjustedDate = adjustedDate.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setAdjustedDateRef(Date adjustedDate) {
			return setAdjustedDate(FieldWithMetaDate.builder().setValue(adjustedDate).build());
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setDateAdjustments(BusinessDayAdjustments dateAdjustments) {
			this.dateAdjustments = dateAdjustments.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder dateAdjustments) {
			this.dateAdjustments = dateAdjustments;
			return this;
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setRelativeDate(RelativeDateOffset relativeDate) {
			this.relativeDate = relativeDate.toBuilder();
			return this;
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setRelativeDateBuilder(RelativeDateOffset.RelativeDateOffsetBuilder relativeDate) {
			this.relativeDate = relativeDate;
			return this;
		}
		public AdjustableOrAdjustedOrRelativeDateBuilder setUnadjustedDate(Date unadjustedDate) {
			this.unadjustedDate = unadjustedDate;
			return this;
		}
		
		public AdjustableOrAdjustedOrRelativeDate build() {
			return new AdjustableOrAdjustedOrRelativeDate(this);
		}
	
		@Override
		public AdjustableOrAdjustedOrRelativeDateBuilder prune() {
			if (adjustedDate!=null && !adjustedDate.prune().hasData()) adjustedDate = null;
			if (dateAdjustments!=null && !dateAdjustments.prune().hasData()) dateAdjustments = null;
			if (relativeDate!=null && !relativeDate.prune().hasData()) relativeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedDate()!=null) return true;
			if (getDateAdjustments()!=null && getDateAdjustments().hasData()) return true;
			if (getRelativeDate()!=null && getRelativeDate().hasData()) return true;
			if (getUnadjustedDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
			
			processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
			processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
			processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableOrAdjustedOrRelativeDateBuilder _that = (AdjustableOrAdjustedOrRelativeDateBuilder) o;
		
			if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
			if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
			if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
			if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
			_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
			_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
			_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableOrAdjustedOrRelativeDateBuilder {" +
				"adjustedDate=" + this.adjustedDate + ", " +
				"dateAdjustments=" + this.dateAdjustments + ", " +
				"relativeDate=" + this.relativeDate + ", " +
				"unadjustedDate=" + this.unadjustedDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unadjustedDate"), Date.class, unadjustedDate, this);
		
		processRosetta(path.newSubPath("adjustedDate"), processor, FieldWithMetaDate.class, adjustedDate);
		processRosetta(path.newSubPath("dateAdjustments"), processor, BusinessDayAdjustments.class, dateAdjustments);
		processRosetta(path.newSubPath("relativeDate"), processor, RelativeDateOffset.class, relativeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableOrAdjustedOrRelativeDate _that = (AdjustableOrAdjustedOrRelativeDate) o;
	
		if (adjustedDate != null ? !adjustedDate.equals(_that.adjustedDate) : _that.adjustedDate != null) return false;
		if (dateAdjustments != null ? !dateAdjustments.equals(_that.dateAdjustments) : _that.dateAdjustments != null) return false;
		if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
		if (unadjustedDate != null ? !unadjustedDate.equals(_that.unadjustedDate) : _that.unadjustedDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedDate != null ? adjustedDate.hashCode() : 0);
		_result = 31 * _result + (dateAdjustments != null ? dateAdjustments.hashCode() : 0);
		_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
		_result = 31 * _result + (unadjustedDate != null ? unadjustedDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableOrAdjustedOrRelativeDate {" +
			"adjustedDate=" + this.adjustedDate + ", " +
			"dateAdjustments=" + this.dateAdjustments + ", " +
			"relativeDate=" + this.relativeDate + ", " +
			"unadjustedDate=" + this.unadjustedDate +
		'}';
	}
}
