package org.isda.cdm;

import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustedRelativeDateOffset;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustableOrRelativeDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class giving the choice between defining a date as an explicit date together with applicable adjustments or as relative to some other (anchor) date.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AdjustableOrRelativeDate extends RosettaModelObject implements GlobalKey {
	private final AdjustableDate adjustableDate;
	private final MetaFields meta;
	private final AdjustedRelativeDateOffset relativeDate;
	private static AdjustableOrRelativeDateMeta metaData = new AdjustableOrRelativeDateMeta();
	
	AdjustableOrRelativeDate(AdjustableOrRelativeDateBuilder builder) {
		this.adjustableDate = ofNullable(builder.getAdjustableDate()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.relativeDate = ofNullable(builder.getRelativeDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A date that shall be subject to adjustment if it would otherwise fall on a day that is not a business day in the specified business centers, together with the convention for adjusting the date.
	 */
	public final AdjustableDate getAdjustableDate() {
		return adjustableDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A date specified as some offset to another date (the anchor date).
	 */
	@RosettaSynonym(value="relativeDate", source="FpML_5_10")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDate", source="FpML_5_10", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="CME_SubmissionIRS_1_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="DTCC_11_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="DTCC_9_0", path="paymentDate")
	@RosettaSynonym(value="relativeDate", source="CME_ClearedConfirm_1_17", path="paymentDate")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="relativeDate", source="FpML_5_10", path="paymentDateFinal")
	public final AdjustedRelativeDateOffset getRelativeDate() {
		return relativeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustableOrRelativeDate> metaData() {
		return metaData;
	} 
	
	public AdjustableOrRelativeDateBuilder toBuilder() {
		AdjustableOrRelativeDateBuilder builder = new AdjustableOrRelativeDateBuilder();
		ofNullable(getAdjustableDate()).ifPresent(builder::setAdjustableDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRelativeDate()).ifPresent(builder::setRelativeDate);
		return builder;
	}

	public static AdjustableOrRelativeDateBuilder builder() {
		return new AdjustableOrRelativeDateBuilder();
	}

	public static  class AdjustableOrRelativeDateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<AdjustableOrRelativeDateBuilder> {
	
		protected AdjustableDate.AdjustableDateBuilder adjustableDate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected AdjustedRelativeDateOffset.AdjustedRelativeDateOffsetBuilder relativeDate;
	
		public AdjustableOrRelativeDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustableOrRelativeDate> metaData() {
			return metaData;
		} 
	
		
		public AdjustableDate.AdjustableDateBuilder getAdjustableDate() {
			return adjustableDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateAdjustableDate() {
			if (adjustableDate!=null) {
				return adjustableDate;
			}
			else return adjustableDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public AdjustedRelativeDateOffset.AdjustedRelativeDateOffsetBuilder getRelativeDate() {
			return relativeDate;
		}
		
		public AdjustedRelativeDateOffset.AdjustedRelativeDateOffsetBuilder getOrCreateRelativeDate() {
			if (relativeDate!=null) {
				return relativeDate;
			}
			else return relativeDate = new AdjustedRelativeDateOffset.AdjustedRelativeDateOffsetBuilder();
		}
		
	
		public AdjustableOrRelativeDateBuilder setAdjustableDate(AdjustableDate adjustableDate) {
			this.adjustableDate = adjustableDate.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDateBuilder setAdjustableDateBuilder(AdjustableDate.AdjustableDateBuilder adjustableDate) {
			this.adjustableDate = adjustableDate;
			return this;
		}
		public AdjustableOrRelativeDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public AdjustableOrRelativeDateBuilder setRelativeDate(AdjustedRelativeDateOffset relativeDate) {
			this.relativeDate = relativeDate.toBuilder();
			return this;
		}
		public AdjustableOrRelativeDateBuilder setRelativeDateBuilder(AdjustedRelativeDateOffset.AdjustedRelativeDateOffsetBuilder relativeDate) {
			this.relativeDate = relativeDate;
			return this;
		}
		
		public AdjustableOrRelativeDate build() {
			return new AdjustableOrRelativeDate(this);
		}
	
		@Override
		public AdjustableOrRelativeDateBuilder prune() {
			if (adjustableDate!=null && !adjustableDate.prune().hasData()) adjustableDate = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (relativeDate!=null && !relativeDate.prune().hasData()) relativeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustableDate()!=null && getAdjustableDate().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getRelativeDate()!=null && getRelativeDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("adjustableDate"), processor, AdjustableDate.class, adjustableDate);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("relativeDate"), processor, AdjustedRelativeDateOffset.class, relativeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AdjustableOrRelativeDateBuilder _that = (AdjustableOrRelativeDateBuilder) o;
		
			if (adjustableDate != null ? !adjustableDate.equals(_that.adjustableDate) : _that.adjustableDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustableDate != null ? adjustableDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustableOrRelativeDateBuilder {" +
				"adjustableDate=" + this.adjustableDate + ", " +
				"meta=" + this.meta + ", " +
				"relativeDate=" + this.relativeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("adjustableDate"), processor, AdjustableDate.class, adjustableDate);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("relativeDate"), processor, AdjustedRelativeDateOffset.class, relativeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AdjustableOrRelativeDate _that = (AdjustableOrRelativeDate) o;
	
		if (adjustableDate != null ? !adjustableDate.equals(_that.adjustableDate) : _that.adjustableDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (relativeDate != null ? !relativeDate.equals(_that.relativeDate) : _that.relativeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustableDate != null ? adjustableDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (relativeDate != null ? relativeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustableOrRelativeDate {" +
			"adjustableDate=" + this.adjustableDate + ", " +
			"meta=" + this.meta + ", " +
			"relativeDate=" + this.relativeDate +
		'}';
	}
}
