package org.isda.cdm;

import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.metafields.BasicReferenceWithMetaDate;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters.ReferenceWithMetaBusinessCentersBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AdjustedRelativeDateOffsetMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;
		import com.rosetta.model.lib.records.Date;

/**
 * A type defining a date (referred to as the derived date) as a relative offset from another date (referred to as the anchor date) plus optional date adjustments.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AdjustedRelativeDateOffset", source="FpML_5_10")
public class AdjustedRelativeDateOffset extends RelativeDateOffset {
	private final BusinessDayAdjustments relativeDateAdjustments;
	private static AdjustedRelativeDateOffsetMeta metaData = new AdjustedRelativeDateOffsetMeta();
	
	AdjustedRelativeDateOffset(AdjustedRelativeDateOffsetBuilder builder) {
		super(builder);
		this.relativeDateAdjustments = ofNullable(builder.getRelativeDateAdjustments()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The business day convention and financial business centers used for adjusting the relative date if it would otherwise fall on a day that is not a business date in the specified business centers.
	 */
	@RosettaSynonym(value="relativeDateAdjustments", source="FpML_5_10")
	@RosettaSynonym(value="relativeDateAdjustments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relativeDateAdjustments", source="DTCC_11_0")
	@RosettaSynonym(value="relativeDateAdjustments", source="DTCC_9_0")
	@RosettaSynonym(value="relativeDateAdjustments", source="CME_ClearedConfirm_1_17")
	public final BusinessDayAdjustments getRelativeDateAdjustments() {
		return relativeDateAdjustments;
	}
	
	
	@Override
	public RosettaMetaData<? extends AdjustedRelativeDateOffset> metaData() {
		return metaData;
	} 
	
	public AdjustedRelativeDateOffsetBuilder toBuilder() {
		AdjustedRelativeDateOffsetBuilder builder = new AdjustedRelativeDateOffsetBuilder();
		ofNullable(getRelativeDateAdjustments()).ifPresent(builder::setRelativeDateAdjustments);
		ofNullable(getAdjustedDate()).ifPresent(builder::setAdjustedDate);
		ofNullable(getBusinessCenters()).ifPresent(builder::setBusinessCenters);
		ofNullable(getBusinessCentersReference()).ifPresent(builder::setBusinessCentersReference);
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getDateRelativeTo()).ifPresent(builder::setDateRelativeTo);
		ofNullable(getDayType()).ifPresent(builder::setDayType);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static AdjustedRelativeDateOffsetBuilder builder() {
		return new AdjustedRelativeDateOffsetBuilder();
	}

	public static  class AdjustedRelativeDateOffsetBuilder extends RelativeDateOffsetBuilder {
	
		protected BusinessDayAdjustments.BusinessDayAdjustmentsBuilder relativeDateAdjustments;
	
		public AdjustedRelativeDateOffsetBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AdjustedRelativeDateOffset> metaData() {
			return metaData;
		} 
	
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getRelativeDateAdjustments() {
			return relativeDateAdjustments;
		}
		
		public BusinessDayAdjustments.BusinessDayAdjustmentsBuilder getOrCreateRelativeDateAdjustments() {
			if (relativeDateAdjustments!=null) {
				return relativeDateAdjustments;
			}
			else return relativeDateAdjustments = new BusinessDayAdjustments.BusinessDayAdjustmentsBuilder();
		}
		
	
		public AdjustedRelativeDateOffsetBuilder setRelativeDateAdjustments(BusinessDayAdjustments relativeDateAdjustments) {
			this.relativeDateAdjustments = relativeDateAdjustments.toBuilder();
			return this;
		}
		public AdjustedRelativeDateOffsetBuilder setRelativeDateAdjustmentsBuilder(BusinessDayAdjustments.BusinessDayAdjustmentsBuilder relativeDateAdjustments) {
			this.relativeDateAdjustments = relativeDateAdjustments;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setAdjustedDate(Date adjustedDate) {
			this.adjustedDate = adjustedDate;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCenters(BusinessCenters businessCenters) {
			this.businessCenters = businessCenters.toBuilder();
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCentersBuilder(BusinessCenters.BusinessCentersBuilder businessCenters) {
			this.businessCenters = businessCenters;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCentersReference(ReferenceWithMetaBusinessCenters businessCentersReference) {
			this.businessCentersReference = businessCentersReference.toBuilder();
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCentersReferenceBuilder(ReferenceWithMetaBusinessCentersBuilder businessCentersReference) {
			this.businessCentersReference = businessCentersReference;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCentersReferenceRef(BusinessCenters.BusinessCentersBuilder businessCentersReference) {
			return setBusinessCentersReference(ReferenceWithMetaBusinessCenters.builder().setValueBuilder(businessCentersReference).build());
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessCentersReferenceRef(BusinessCenters businessCentersReference) {
			return setBusinessCentersReferenceRef(BusinessCenters.builder());
		}
		@Override public AdjustedRelativeDateOffsetBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setDateRelativeTo(BasicReferenceWithMetaDate dateRelativeTo) {
			this.dateRelativeTo = dateRelativeTo.toBuilder();
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setDateRelativeToRef(Date dateRelativeTo) {
			return setDateRelativeTo(BasicReferenceWithMetaDate.builder().setValue(dateRelativeTo).build());
		}
		@Override public AdjustedRelativeDateOffsetBuilder setDayType(DayTypeEnum dayType) {
			this.dayType = dayType;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setPeriod(PeriodEnum period) {
			this.period = period;
			return this;
		}
		@Override public AdjustedRelativeDateOffsetBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public AdjustedRelativeDateOffset build() {
			return new AdjustedRelativeDateOffset(this);
		}
	
		@Override
		public AdjustedRelativeDateOffsetBuilder prune() {
			super.prune();
			if (relativeDateAdjustments!=null && !relativeDateAdjustments.prune().hasData()) relativeDateAdjustments = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getRelativeDateAdjustments()!=null && getRelativeDateAdjustments().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("relativeDateAdjustments"), processor, BusinessDayAdjustments.class, relativeDateAdjustments);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			AdjustedRelativeDateOffsetBuilder _that = (AdjustedRelativeDateOffsetBuilder) o;
		
			if (relativeDateAdjustments != null ? !relativeDateAdjustments.equals(_that.relativeDateAdjustments) : _that.relativeDateAdjustments != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (relativeDateAdjustments != null ? relativeDateAdjustments.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AdjustedRelativeDateOffsetBuilder {" +
				"relativeDateAdjustments=" + this.relativeDateAdjustments +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("relativeDateAdjustments"), processor, BusinessDayAdjustments.class, relativeDateAdjustments);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		AdjustedRelativeDateOffset _that = (AdjustedRelativeDateOffset) o;
	
		if (relativeDateAdjustments != null ? !relativeDateAdjustments.equals(_that.relativeDateAdjustments) : _that.relativeDateAdjustments != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (relativeDateAdjustments != null ? relativeDateAdjustments.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AdjustedRelativeDateOffset {" +
			"relativeDateAdjustments=" + this.relativeDateAdjustments +
		'}' + " " + super.toString();
	}
}
