package org.isda.cdm;

import org.isda.cdm.Quantity;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AllocationBreakdownMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class AllocationBreakdown extends RosettaModelObject {
	private final ReferenceWithMetaParty partyReference;
	private final Quantity quantity;
	private static AllocationBreakdownMeta metaData = new AllocationBreakdownMeta();
	
	AllocationBreakdown(AllocationBreakdownBuilder builder) {
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
		this.quantity = ofNullable(builder.getQuantity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Reference to the party to specify the account.
	 */
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	/**
	 * The quantity to be allocated to the party.
	 */
	public final Quantity getQuantity() {
		return quantity;
	}
	
	
	@Override
	public RosettaMetaData<? extends AllocationBreakdown> metaData() {
		return metaData;
	} 
	
	public AllocationBreakdownBuilder toBuilder() {
		AllocationBreakdownBuilder builder = new AllocationBreakdownBuilder();
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		return builder;
	}

	public static AllocationBreakdownBuilder builder() {
		return new AllocationBreakdownBuilder();
	}

	public static  class AllocationBreakdownBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaPartyBuilder partyReference;
		protected Quantity.QuantityBuilder quantity;
	
		public AllocationBreakdownBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AllocationBreakdown> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public Quantity.QuantityBuilder getQuantity() {
			return quantity;
		}
		
		public Quantity.QuantityBuilder getOrCreateQuantity() {
			if (quantity!=null) {
				return quantity;
			}
			else return quantity = new Quantity.QuantityBuilder();
		}
		
	
		public AllocationBreakdownBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public AllocationBreakdownBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public AllocationBreakdownBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public AllocationBreakdownBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		public AllocationBreakdownBuilder setQuantity(Quantity quantity) {
			this.quantity = quantity.toBuilder();
			return this;
		}
		public AllocationBreakdownBuilder setQuantityBuilder(Quantity.QuantityBuilder quantity) {
			this.quantity = quantity;
			return this;
		}
		
		public AllocationBreakdown build() {
			return new AllocationBreakdown(this);
		}
	
		@Override
		public AllocationBreakdownBuilder prune() {
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			if (quantity!=null && !quantity.prune().hasData()) quantity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			if (getQuantity()!=null && getQuantity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
			processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AllocationBreakdownBuilder _that = (AllocationBreakdownBuilder) o;
		
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AllocationBreakdownBuilder {" +
				"partyReference=" + this.partyReference + ", " +
				"quantity=" + this.quantity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		processRosetta(path.newSubPath("quantity"), processor, Quantity.class, quantity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AllocationBreakdown _that = (AllocationBreakdown) o;
	
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AllocationBreakdown {" +
			"partyReference=" + this.partyReference + ", " +
			"quantity=" + this.quantity +
		'}';
	}
}
