package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AllocationBreakdown;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AllocationInstructionsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class AllocationInstructions extends RosettaModelObject {
	private final List<AllocationBreakdown> breakdowns;
	private static AllocationInstructionsMeta metaData = new AllocationInstructionsMeta();
	
	AllocationInstructions(AllocationInstructionsBuilder builder) {
		this.breakdowns = ofNullable(builder.getBreakdowns()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The set of allocation breakdowns to be applied to a block trade
	 */
	public final List<AllocationBreakdown> getBreakdowns() {
		return breakdowns;
	}
	
	
	@Override
	public RosettaMetaData<? extends AllocationInstructions> metaData() {
		return metaData;
	} 
	
	public AllocationInstructionsBuilder toBuilder() {
		AllocationInstructionsBuilder builder = new AllocationInstructionsBuilder();
		ofNullable(getBreakdowns()).ifPresent(breakdowns -> breakdowns.forEach(builder::addBreakdowns));
		return builder;
	}

	public static AllocationInstructionsBuilder builder() {
		return new AllocationInstructionsBuilder();
	}

	public static  class AllocationInstructionsBuilder extends RosettaModelObjectBuilder {
	
		protected List<AllocationBreakdown.AllocationBreakdownBuilder> breakdowns;
	
		public AllocationInstructionsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AllocationInstructions> metaData() {
			return metaData;
		} 
	
		
		public List<AllocationBreakdown.AllocationBreakdownBuilder> getBreakdowns() {
			return breakdowns;
		}
		
		public AllocationBreakdown.AllocationBreakdownBuilder getOrCreateBreakdowns(int index) {
			if (breakdowns==null) {
				this.breakdowns = new ArrayList<>();
			}
			return getIndex(breakdowns, index, ()->new AllocationBreakdown.AllocationBreakdownBuilder());
		}
		
	
		public AllocationInstructionsBuilder addBreakdowns(AllocationBreakdown breakdowns) {
			if(this.breakdowns == null){
				this.breakdowns = new ArrayList<>();
			}
			this.breakdowns.add(breakdowns.toBuilder());
			return this;
		}
		public AllocationInstructionsBuilder addBreakdowns(AllocationBreakdown breakdowns, int _idx) {
			if(this.breakdowns == null){
				this.breakdowns = new ArrayList<>();
			}
			getIndex(this.breakdowns, _idx, () -> breakdowns.toBuilder());
			this.breakdowns.set(_idx, breakdowns.toBuilder());
			return this;
		}
		public AllocationInstructionsBuilder addBreakdowns(List<AllocationBreakdown> breakdownss) {
			if(this.breakdowns == null){
				this.breakdowns = new ArrayList<>();
			}
			for (AllocationBreakdown toAdd : breakdownss) {
				this.breakdowns.add(toAdd.toBuilder());
			}
			return this;
		}
		public AllocationInstructionsBuilder addBreakdownsBuilder(AllocationBreakdown.AllocationBreakdownBuilder breakdowns) {
			if(this.breakdowns == null){
				this.breakdowns = new ArrayList<>();
			}
			this.breakdowns.add(breakdowns);
			return this;
		}
		
		public AllocationInstructionsBuilder clearBreakdowns() {
			this.breakdowns = null;
			return this;
		}
		
		public AllocationInstructions build() {
			return new AllocationInstructions(this);
		}
	
		@Override
		public AllocationInstructionsBuilder prune() {
			if (breakdowns!=null) breakdowns = breakdowns.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBreakdowns()!=null && getBreakdowns().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("breakdowns"), processor, AllocationBreakdown.class, breakdowns);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AllocationInstructionsBuilder _that = (AllocationInstructionsBuilder) o;
		
			if (!ListEquals.listEquals(breakdowns, _that.breakdowns)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (breakdowns != null ? breakdowns.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AllocationInstructionsBuilder {" +
				"breakdowns=" + this.breakdowns +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("breakdowns"), processor, AllocationBreakdown.class, breakdowns);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AllocationInstructions _that = (AllocationInstructions) o;
	
		if (!ListEquals.listEquals(breakdowns, _that.breakdowns)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (breakdowns != null ? breakdowns.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AllocationInstructions {" +
			"breakdowns=" + this.breakdowns +
		'}';
	}
}
