package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the effective date of the Amendment to Termination Currency when specified as a specific date (e.g. the annex date). ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (t).
 * 
 * @version 2.5.4
 */
public enum AmendmentEffectiveDateEnum {
    

	/**
	 * The effective date corresponds to the Annex date.
	 */
	@RosettaSynonym(value = "annex", source = "ISDA_Create_1_0")
	ANNEX_DATE    
	;

	private final String displayName;
	
	AmendmentEffectiveDateEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
