package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AmountScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a currency amount or a currency amount schedule.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AmountSchedule", source="FpML_5_10")
public class AmountSchedule extends Schedule {
	private final List<FieldWithMetaString> currency;
	private static AmountScheduleMeta metaData = new AmountScheduleMeta();
	
	AmountSchedule(AmountScheduleBuilder builder) {
		super(builder);
		this.currency = ofNullable(builder.getCurrency()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The currency in which the amount schedule is denominated. The currency is specified outside of the actual schedule in order to be applied uniformly to it. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency", source="DTCC_11_0")
	@RosettaSynonym(value="currency", source="DTCC_9_0")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaString> getCurrency() {
		return currency;
	}
	
	
	@Override
	public RosettaMetaData<? extends AmountSchedule> metaData() {
		return metaData;
	} 
	
	public AmountScheduleBuilder toBuilder() {
		AmountScheduleBuilder builder = new AmountScheduleBuilder();
		ofNullable(getCurrency()).ifPresent(currency -> currency.forEach(builder::addCurrency));
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static AmountScheduleBuilder builder() {
		return new AmountScheduleBuilder();
	}

	public static  class AmountScheduleBuilder extends ScheduleBuilder {
	
		protected List<FieldWithMetaStringBuilder> currency;
	
		public AmountScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AmountSchedule> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency(int index) {
			if (currency==null) {
				this.currency = new ArrayList<>();
			}
			return getIndex(currency, index, ()->new FieldWithMetaStringBuilder());
		}
		
	
		public AmountScheduleBuilder addCurrency(FieldWithMetaString currency) {
			if(this.currency == null){
				this.currency = new ArrayList<>();
			}
			this.currency.add(currency.toBuilder());
			return this;
		}
		public AmountScheduleBuilder addCurrency(FieldWithMetaString currency, int _idx) {
			if(this.currency == null){
				this.currency = new ArrayList<>();
			}
			getIndex(this.currency, _idx, () -> currency.toBuilder());
			this.currency.set(_idx, currency.toBuilder());
			return this;
		}
		public AmountScheduleBuilder addCurrency(List<FieldWithMetaString> currencys) {
			if(this.currency == null){
				this.currency = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : currencys) {
				this.currency.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public AmountScheduleBuilder clearCurrency() {
			this.currency = null;
			return this;
		}
		@Override public AmountScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		@Override public AmountScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public AmountScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public AmountScheduleBuilder addStep(Step step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		@Override public AmountScheduleBuilder addStep(Step step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		@Override public AmountScheduleBuilder addStep(List<Step> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (Step toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public AmountScheduleBuilder addStepBuilder(Step.StepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		@Override public AmountScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public AmountSchedule build() {
			return new AmountSchedule(this);
		}
	
		@Override
		public AmountScheduleBuilder prune() {
			super.prune();
			if (currency!=null) currency = currency.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCurrency()!=null && !getCurrency().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			AmountScheduleBuilder _that = (AmountScheduleBuilder) o;
		
			if (!ListEquals.listEquals(currency, _that.currency)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AmountScheduleBuilder {" +
				"currency=" + this.currency +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		AmountSchedule _that = (AmountSchedule) o;
	
		if (!ListEquals.listEquals(currency, _that.currency)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AmountSchedule {" +
			"currency=" + this.currency +
		'}' + " " + super.toString();
	}
}
