package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.AveragingInOutEnum;
import org.isda.cdm.AveragingPeriod;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AsianMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * As per ISDA 2002 Definitions.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Asian", source="FpML_5_10")
public class Asian extends RosettaModelObject {
	private final AveragingInOutEnum averagingInOut;
	private final AveragingPeriod averagingPeriodIn;
	private final AveragingPeriod averagingPeriodOut;
	private final BigDecimal strikeFactor;
	private static AsianMeta metaData = new AsianMeta();
	
	Asian(AsianBuilder builder) {
		this.averagingInOut = builder.getAveragingInOut();
		this.averagingPeriodIn = ofNullable(builder.getAveragingPeriodIn()).map(f->f.build()).orElse(null);
		this.averagingPeriodOut = ofNullable(builder.getAveragingPeriodOut()).map(f->f.build()).orElse(null);
		this.strikeFactor = builder.getStrikeFactor();
	}
	
	@RosettaSynonym(value="averagingInOut", source="FpML_5_10")
	@RosettaSynonym(value="averagingInOut", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingInOut", source="DTCC_11_0")
	@RosettaSynonym(value="averagingInOut", source="DTCC_9_0")
	@RosettaSynonym(value="averagingInOut", source="CME_ClearedConfirm_1_17")
	public final AveragingInOutEnum getAveragingInOut() {
		return averagingInOut;
	}
	
	/**
	 * The averaging in period.
	 */
	@RosettaSynonym(value="averagingPeriodIn", source="FpML_5_10")
	@RosettaSynonym(value="averagingPeriodIn", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingPeriodIn", source="DTCC_11_0")
	@RosettaSynonym(value="averagingPeriodIn", source="DTCC_9_0")
	@RosettaSynonym(value="averagingPeriodIn", source="CME_ClearedConfirm_1_17")
	public final AveragingPeriod getAveragingPeriodIn() {
		return averagingPeriodIn;
	}
	
	/**
	 * The averaging out period.
	 */
	@RosettaSynonym(value="averagingPeriodOut", source="FpML_5_10")
	@RosettaSynonym(value="averagingPeriodOut", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingPeriodOut", source="DTCC_11_0")
	@RosettaSynonym(value="averagingPeriodOut", source="DTCC_9_0")
	@RosettaSynonym(value="averagingPeriodOut", source="CME_ClearedConfirm_1_17")
	public final AveragingPeriod getAveragingPeriodOut() {
		return averagingPeriodOut;
	}
	
	/**
	 * The factor of strike.
	 */
	@RosettaSynonym(value="strikeFactor", source="FpML_5_10")
	@RosettaSynonym(value="strikeFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="strikeFactor", source="DTCC_11_0")
	@RosettaSynonym(value="strikeFactor", source="DTCC_9_0")
	@RosettaSynonym(value="strikeFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getStrikeFactor() {
		return strikeFactor;
	}
	
	
	@Override
	public RosettaMetaData<? extends Asian> metaData() {
		return metaData;
	} 
	
	public AsianBuilder toBuilder() {
		AsianBuilder builder = new AsianBuilder();
		ofNullable(getAveragingInOut()).ifPresent(builder::setAveragingInOut);
		ofNullable(getAveragingPeriodIn()).ifPresent(builder::setAveragingPeriodIn);
		ofNullable(getAveragingPeriodOut()).ifPresent(builder::setAveragingPeriodOut);
		ofNullable(getStrikeFactor()).ifPresent(builder::setStrikeFactor);
		return builder;
	}

	public static AsianBuilder builder() {
		return new AsianBuilder();
	}

	public static  class AsianBuilder extends RosettaModelObjectBuilder {
	
		protected AveragingInOutEnum averagingInOut;
		protected AveragingPeriod.AveragingPeriodBuilder averagingPeriodIn;
		protected AveragingPeriod.AveragingPeriodBuilder averagingPeriodOut;
		protected BigDecimal strikeFactor;
	
		public AsianBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Asian> metaData() {
			return metaData;
		} 
	
		
		public AveragingInOutEnum getAveragingInOut() {
			return averagingInOut;
		}
		
		
		public AveragingPeriod.AveragingPeriodBuilder getAveragingPeriodIn() {
			return averagingPeriodIn;
		}
		
		public AveragingPeriod.AveragingPeriodBuilder getOrCreateAveragingPeriodIn() {
			if (averagingPeriodIn!=null) {
				return averagingPeriodIn;
			}
			else return averagingPeriodIn = new AveragingPeriod.AveragingPeriodBuilder();
		}
		
		
		public AveragingPeriod.AveragingPeriodBuilder getAveragingPeriodOut() {
			return averagingPeriodOut;
		}
		
		public AveragingPeriod.AveragingPeriodBuilder getOrCreateAveragingPeriodOut() {
			if (averagingPeriodOut!=null) {
				return averagingPeriodOut;
			}
			else return averagingPeriodOut = new AveragingPeriod.AveragingPeriodBuilder();
		}
		
		
		public BigDecimal getStrikeFactor() {
			return strikeFactor;
		}
		
	
		public AsianBuilder setAveragingInOut(AveragingInOutEnum averagingInOut) {
			this.averagingInOut = averagingInOut;
			return this;
		}
		public AsianBuilder setAveragingPeriodIn(AveragingPeriod averagingPeriodIn) {
			this.averagingPeriodIn = averagingPeriodIn.toBuilder();
			return this;
		}
		public AsianBuilder setAveragingPeriodInBuilder(AveragingPeriod.AveragingPeriodBuilder averagingPeriodIn) {
			this.averagingPeriodIn = averagingPeriodIn;
			return this;
		}
		public AsianBuilder setAveragingPeriodOut(AveragingPeriod averagingPeriodOut) {
			this.averagingPeriodOut = averagingPeriodOut.toBuilder();
			return this;
		}
		public AsianBuilder setAveragingPeriodOutBuilder(AveragingPeriod.AveragingPeriodBuilder averagingPeriodOut) {
			this.averagingPeriodOut = averagingPeriodOut;
			return this;
		}
		public AsianBuilder setStrikeFactor(BigDecimal strikeFactor) {
			this.strikeFactor = strikeFactor;
			return this;
		}
		
		public Asian build() {
			return new Asian(this);
		}
	
		@Override
		public AsianBuilder prune() {
			if (averagingPeriodIn!=null && !averagingPeriodIn.prune().hasData()) averagingPeriodIn = null;
			if (averagingPeriodOut!=null && !averagingPeriodOut.prune().hasData()) averagingPeriodOut = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAveragingInOut()!=null) return true;
			if (getAveragingPeriodIn()!=null && getAveragingPeriodIn().hasData()) return true;
			if (getAveragingPeriodOut()!=null && getAveragingPeriodOut().hasData()) return true;
			if (getStrikeFactor()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("averagingInOut"), AveragingInOutEnum.class, averagingInOut, this);
			processor.processBasic(path.newSubPath("strikeFactor"), BigDecimal.class, strikeFactor, this);
			
			processRosetta(path.newSubPath("averagingPeriodIn"), processor, AveragingPeriod.class, averagingPeriodIn);
			processRosetta(path.newSubPath("averagingPeriodOut"), processor, AveragingPeriod.class, averagingPeriodOut);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AsianBuilder _that = (AsianBuilder) o;
		
			if (averagingInOut != null ? !averagingInOut.equals(_that.averagingInOut) : _that.averagingInOut != null) return false;
			if (averagingPeriodIn != null ? !averagingPeriodIn.equals(_that.averagingPeriodIn) : _that.averagingPeriodIn != null) return false;
			if (averagingPeriodOut != null ? !averagingPeriodOut.equals(_that.averagingPeriodOut) : _that.averagingPeriodOut != null) return false;
			if (strikeFactor != null ? !strikeFactor.equals(_that.strikeFactor) : _that.strikeFactor != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (averagingInOut != null ? averagingInOut.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (averagingPeriodIn != null ? averagingPeriodIn.hashCode() : 0);
			_result = 31 * _result + (averagingPeriodOut != null ? averagingPeriodOut.hashCode() : 0);
			_result = 31 * _result + (strikeFactor != null ? strikeFactor.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AsianBuilder {" +
				"averagingInOut=" + this.averagingInOut + ", " +
				"averagingPeriodIn=" + this.averagingPeriodIn + ", " +
				"averagingPeriodOut=" + this.averagingPeriodOut + ", " +
				"strikeFactor=" + this.strikeFactor +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("averagingInOut"), AveragingInOutEnum.class, averagingInOut, this);
		processor.processBasic(path.newSubPath("strikeFactor"), BigDecimal.class, strikeFactor, this);
		
		processRosetta(path.newSubPath("averagingPeriodIn"), processor, AveragingPeriod.class, averagingPeriodIn);
		processRosetta(path.newSubPath("averagingPeriodOut"), processor, AveragingPeriod.class, averagingPeriodOut);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Asian _that = (Asian) o;
	
		if (averagingInOut != null ? !averagingInOut.equals(_that.averagingInOut) : _that.averagingInOut != null) return false;
		if (averagingPeriodIn != null ? !averagingPeriodIn.equals(_that.averagingPeriodIn) : _that.averagingPeriodIn != null) return false;
		if (averagingPeriodOut != null ? !averagingPeriodOut.equals(_that.averagingPeriodOut) : _that.averagingPeriodOut != null) return false;
		if (strikeFactor != null ? !strikeFactor.equals(_that.strikeFactor) : _that.strikeFactor != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (averagingInOut != null ? averagingInOut.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (averagingPeriodIn != null ? averagingPeriodIn.hashCode() : 0);
		_result = 31 * _result + (averagingPeriodOut != null ? averagingPeriodOut.hashCode() : 0);
		_result = 31 * _result + (strikeFactor != null ? strikeFactor.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Asian {" +
			"averagingInOut=" + this.averagingInOut + ", " +
			"averagingPeriodIn=" + this.averagingPeriodIn + ", " +
			"averagingPeriodOut=" + this.averagingPeriodOut + ", " +
			"strikeFactor=" + this.strikeFactor +
		'}';
	}
}
