package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AssignedIdentifierMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the identifier value and its associated version.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class AssignedIdentifier extends RosettaModelObject {
	private final FieldWithMetaString identifier;
	private final Integer version;
	private static AssignedIdentifierMeta metaData = new AssignedIdentifierMeta();
	
	AssignedIdentifier(AssignedIdentifierBuilder builder) {
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.version = builder.getVersion();
	}
	
	/**
	 * The identifier value.
	 */
	@RosettaSynonym(value="tradeId", source="FpML_5_10")
	@RosettaSynonym(value="tradeId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tradeId", source="FpML_5_10", path="versionedTradeId")
	@RosettaSynonym(value="identifier", source="Rosetta_Workbench")
	@RosettaSynonym(value="tradeId", source="DTCC_11_0", path="partyTradeIdentifier", maps=2)
	@RosettaSynonym(value="tradeId", source="DTCC_9_0", path="partyTradeIdentifier", maps=2)
	@RosettaSynonym(value="USITradeId", source="DTCC_11_0")
	@RosettaSynonym(value="USITradeId", source="DTCC_9_0")
	@RosettaSynonym(value="UTITradeId", source="DTCC_11_0")
	@RosettaSynonym(value="UTITradeId", source="DTCC_9_0")
	@RosettaSynonym(value="identifier", source="DTCC_11_0", path="payment")
	@RosettaSynonym(value="identifier", source="DTCC_9_0", path="payment")
	@RosettaSynonym(value="tradeId", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="usi", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="RptID", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt")
	@RosettaSynonym(value="businessUnitId", source="FpML_5_10")
	@RosettaSynonym(value="id", source="ISDA_Create_1_0")
	public final FieldWithMetaString getIdentifier() {
		return identifier;
	}
	
	/**
	 * The identifier version, which is specified as an integer and is meant to be incremented each time the transaction terms (whether contract or event) change. This version is made option to support the use case where the identifier is referenced without the version. The constraint that a contract and a lifecycle event need to have an associated version is enforced through data rules.
	 */
	@RosettaSynonym(value="version", source="FpML_5_10", path="versionedTradeId")
	@RosettaSynonym(value="version", source="FpML_5_10")
	public final Integer getVersion() {
		return version;
	}
	
	
	@Override
	public RosettaMetaData<? extends AssignedIdentifier> metaData() {
		return metaData;
	} 
	
	public AssignedIdentifierBuilder toBuilder() {
		AssignedIdentifierBuilder builder = new AssignedIdentifierBuilder();
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getVersion()).ifPresent(builder::setVersion);
		return builder;
	}

	public static AssignedIdentifierBuilder builder() {
		return new AssignedIdentifierBuilder();
	}

	public static  class AssignedIdentifierBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder identifier;
		protected Integer version;
	
		public AssignedIdentifierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AssignedIdentifier> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new FieldWithMetaStringBuilder();
		}
		
		
		public Integer getVersion() {
			return version;
		}
		
	
		public AssignedIdentifierBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public AssignedIdentifierBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		public AssignedIdentifierBuilder setVersion(Integer version) {
			this.version = version;
			return this;
		}
		
		public AssignedIdentifier build() {
			return new AssignedIdentifier(this);
		}
	
		@Override
		public AssignedIdentifierBuilder prune() {
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null) return true;
			if (getVersion()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("version"), Integer.class, version, this);
			
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AssignedIdentifierBuilder _that = (AssignedIdentifierBuilder) o;
		
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AssignedIdentifierBuilder {" +
				"identifier=" + this.identifier + ", " +
				"version=" + this.version +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("version"), Integer.class, version, this);
		
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AssignedIdentifier _that = (AssignedIdentifier) o;
	
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (version != null ? version.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AssignedIdentifier {" +
			"identifier=" + this.identifier + ", " +
			"version=" + this.version +
		'}';
	}
}
