package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of averaging used in an Asian option.
 * 
 * @version 2.5.4
 */
public enum AveragingInOutEnum {
    

	/**
	 * The average price is used to derive the strike price. Also known as &#39;Asian strike&#39; style option.
	 */
	@RosettaSynonym(value = "In", source = "FpML_5_10")
	@RosettaSynonym(value = "In", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "In", source = "DTCC_11_0")
	@RosettaSynonym(value = "In", source = "DTCC_9_0")
	@RosettaSynonym(value = "In", source = "CME_ClearedConfirm_1_17")
	IN,
	    

	/**
	 * The average price is used to derive the expiration price. Also known as &#39;Asian price&#39; style option.
	 */
	@RosettaSynonym(value = "Out", source = "FpML_5_10")
	@RosettaSynonym(value = "Out", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Out", source = "DTCC_11_0")
	@RosettaSynonym(value = "Out", source = "DTCC_9_0")
	@RosettaSynonym(value = "Out", source = "CME_ClearedConfirm_1_17")
	OUT,
	    

	/**
	 * The average price is used to derive both the strike and the expiration price.
	 */
	@RosettaSynonym(value = "Both", source = "FpML_5_10")
	@RosettaSynonym(value = "Both", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Both", source = "DTCC_11_0")
	@RosettaSynonym(value = "Both", source = "DTCC_9_0")
	@RosettaSynonym(value = "Both", source = "CME_ClearedConfirm_1_17")
	BOTH    
	;

	private final String displayName;
	
	AveragingInOutEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
