package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the method of calculation to be used when averaging rates. Per ISDA 2000 Definitions, Section 6.2. Certain Definitions Relating to Floating Amounts.
 * 
 * @version 2.5.4
 */
public enum AveragingMethodEnum {
    

	/**
	 * The arithmetic mean of the relevant rates for each reset date.
	 */
	@RosettaSynonym(value = "Unweighted", source = "FpML_5_10")
	@RosettaSynonym(value = "Unweighted", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Unweighted", source = "DTCC_11_0")
	@RosettaSynonym(value = "Unweighted", source = "DTCC_9_0")
	@RosettaSynonym(value = "Unweighted", source = "CME_ClearedConfirm_1_17")
	UNWEIGHTED,
	    

	/**
	 * The arithmetic mean of the relevant rates in effect for each day in a calculation period calculated by multiplying each relevant rate by the number of days such relevant rate is in effect, determining the sum of such products and dividing such sum by the number of days in the calculation period.
	 */
	@RosettaSynonym(value = "Weighted", source = "FpML_5_10")
	@RosettaSynonym(value = "Weighted", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Weighted", source = "DTCC_11_0")
	@RosettaSynonym(value = "Weighted", source = "DTCC_9_0")
	@RosettaSynonym(value = "Weighted", source = "CME_ClearedConfirm_1_17")
	WEIGHTED    
	;

	private final String displayName;
	
	AveragingMethodEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
