package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.WeightedAveragingObservation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AveragingObservationListMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * An unordered list of weighted averaging observations.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AveragingObservationList", source="FpML_5_10")
public class AveragingObservationList extends RosettaModelObject {
	private final List<WeightedAveragingObservation> averagingObservation;
	private static AveragingObservationListMeta metaData = new AveragingObservationListMeta();
	
	AveragingObservationList(AveragingObservationListBuilder builder) {
		this.averagingObservation = ofNullable(builder.getAveragingObservation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A single weighted averaging observation.
	 */
	@RosettaSynonym(value="averagingObservation", source="FpML_5_10")
	@RosettaSynonym(value="averagingObservation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingObservation", source="DTCC_11_0")
	@RosettaSynonym(value="averagingObservation", source="DTCC_9_0")
	@RosettaSynonym(value="averagingObservation", source="CME_ClearedConfirm_1_17")
	public final List<WeightedAveragingObservation> getAveragingObservation() {
		return averagingObservation;
	}
	
	
	@Override
	public RosettaMetaData<? extends AveragingObservationList> metaData() {
		return metaData;
	} 
	
	public AveragingObservationListBuilder toBuilder() {
		AveragingObservationListBuilder builder = new AveragingObservationListBuilder();
		ofNullable(getAveragingObservation()).ifPresent(averagingObservation -> averagingObservation.forEach(builder::addAveragingObservation));
		return builder;
	}

	public static AveragingObservationListBuilder builder() {
		return new AveragingObservationListBuilder();
	}

	public static  class AveragingObservationListBuilder extends RosettaModelObjectBuilder {
	
		protected List<WeightedAveragingObservation.WeightedAveragingObservationBuilder> averagingObservation;
	
		public AveragingObservationListBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AveragingObservationList> metaData() {
			return metaData;
		} 
	
		
		public List<WeightedAveragingObservation.WeightedAveragingObservationBuilder> getAveragingObservation() {
			return averagingObservation;
		}
		
		public WeightedAveragingObservation.WeightedAveragingObservationBuilder getOrCreateAveragingObservation(int index) {
			if (averagingObservation==null) {
				this.averagingObservation = new ArrayList<>();
			}
			return getIndex(averagingObservation, index, ()->new WeightedAveragingObservation.WeightedAveragingObservationBuilder());
		}
		
	
		public AveragingObservationListBuilder addAveragingObservation(WeightedAveragingObservation averagingObservation) {
			if(this.averagingObservation == null){
				this.averagingObservation = new ArrayList<>();
			}
			this.averagingObservation.add(averagingObservation.toBuilder());
			return this;
		}
		public AveragingObservationListBuilder addAveragingObservation(WeightedAveragingObservation averagingObservation, int _idx) {
			if(this.averagingObservation == null){
				this.averagingObservation = new ArrayList<>();
			}
			getIndex(this.averagingObservation, _idx, () -> averagingObservation.toBuilder());
			this.averagingObservation.set(_idx, averagingObservation.toBuilder());
			return this;
		}
		public AveragingObservationListBuilder addAveragingObservation(List<WeightedAveragingObservation> averagingObservations) {
			if(this.averagingObservation == null){
				this.averagingObservation = new ArrayList<>();
			}
			for (WeightedAveragingObservation toAdd : averagingObservations) {
				this.averagingObservation.add(toAdd.toBuilder());
			}
			return this;
		}
		public AveragingObservationListBuilder addAveragingObservationBuilder(WeightedAveragingObservation.WeightedAveragingObservationBuilder averagingObservation) {
			if(this.averagingObservation == null){
				this.averagingObservation = new ArrayList<>();
			}
			this.averagingObservation.add(averagingObservation);
			return this;
		}
		
		public AveragingObservationListBuilder clearAveragingObservation() {
			this.averagingObservation = null;
			return this;
		}
		
		public AveragingObservationList build() {
			return new AveragingObservationList(this);
		}
	
		@Override
		public AveragingObservationListBuilder prune() {
			if (averagingObservation!=null) averagingObservation = averagingObservation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAveragingObservation()!=null && getAveragingObservation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("averagingObservation"), processor, WeightedAveragingObservation.class, averagingObservation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AveragingObservationListBuilder _that = (AveragingObservationListBuilder) o;
		
			if (!ListEquals.listEquals(averagingObservation, _that.averagingObservation)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (averagingObservation != null ? averagingObservation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AveragingObservationListBuilder {" +
				"averagingObservation=" + this.averagingObservation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("averagingObservation"), processor, WeightedAveragingObservation.class, averagingObservation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AveragingObservationList _that = (AveragingObservationList) o;
	
		if (!ListEquals.listEquals(averagingObservation, _that.averagingObservation)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (averagingObservation != null ? averagingObservation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AveragingObservationList {" +
			"averagingObservation=" + this.averagingObservation +
		'}';
	}
}
