package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AveragingObservationList;
import org.isda.cdm.AveragingSchedule;
import org.isda.cdm.DateTimeList;
import org.isda.cdm.metafields.FieldWithMetaMarketDisruptionEnum;
import org.isda.cdm.metafields.FieldWithMetaMarketDisruptionEnum.FieldWithMetaMarketDisruptionEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AveragingPeriodMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Period over which an average value is taken.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AveragingPeriod", source="FpML_5_10")
public class AveragingPeriod extends RosettaModelObject {
	private final DateTimeList averagingDateTimes;
	private final AveragingObservationList averagingObservations;
	private final FieldWithMetaMarketDisruptionEnum marketDisruption;
	private final List<AveragingSchedule> schedule;
	private static AveragingPeriodMeta metaData = new AveragingPeriodMeta();
	
	AveragingPeriod(AveragingPeriodBuilder builder) {
		this.averagingDateTimes = ofNullable(builder.getAveragingDateTimes()).map(f->f.build()).orElse(null);
		this.averagingObservations = ofNullable(builder.getAveragingObservations()).map(f->f.build()).orElse(null);
		this.marketDisruption = ofNullable(builder.getMarketDisruption()).map(f->f.build()).orElse(null);
		this.schedule = ofNullable(builder.getSchedule()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * An unweighted list of averaging observation date and times.
	 */
	@RosettaSynonym(value="averagingDateTimes", source="FpML_5_10")
	@RosettaSynonym(value="averagingDateTimes", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingDateTimes", source="DTCC_11_0")
	@RosettaSynonym(value="averagingDateTimes", source="DTCC_9_0")
	@RosettaSynonym(value="averagingDateTimes", source="CME_ClearedConfirm_1_17")
	public final DateTimeList getAveragingDateTimes() {
		return averagingDateTimes;
	}
	
	/**
	 * A weighted list of averaging observation date and times.
	 */
	@RosettaSynonym(value="averagingObservations", source="FpML_5_10")
	@RosettaSynonym(value="averagingObservations", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingObservations", source="DTCC_11_0")
	@RosettaSynonym(value="averagingObservations", source="DTCC_9_0")
	@RosettaSynonym(value="averagingObservations", source="CME_ClearedConfirm_1_17")
	public final AveragingObservationList getAveragingObservations() {
		return averagingObservations;
	}
	
	/**
	 * The market disruption event as defined by ISDA 2002 Definitions.
	 */
	@RosettaSynonym(value="marketDisruption", source="FpML_5_10")
	@RosettaSynonym(value="marketDisruption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="marketDisruption", source="DTCC_11_0")
	@RosettaSynonym(value="marketDisruption", source="DTCC_9_0")
	@RosettaSynonym(value="marketDisruption", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMarketDisruptionEnum getMarketDisruption() {
		return marketDisruption;
	}
	
	/**
	 * A schedule for generating averaging observation dates.
	 */
	@RosettaSynonym(value="schedule", source="FpML_5_10")
	@RosettaSynonym(value="schedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="schedule", source="DTCC_11_0")
	@RosettaSynonym(value="schedule", source="DTCC_9_0")
	@RosettaSynonym(value="schedule", source="CME_ClearedConfirm_1_17")
	public final List<AveragingSchedule> getSchedule() {
		return schedule;
	}
	
	
	@Override
	public RosettaMetaData<? extends AveragingPeriod> metaData() {
		return metaData;
	} 
	
	public AveragingPeriodBuilder toBuilder() {
		AveragingPeriodBuilder builder = new AveragingPeriodBuilder();
		ofNullable(getAveragingDateTimes()).ifPresent(builder::setAveragingDateTimes);
		ofNullable(getAveragingObservations()).ifPresent(builder::setAveragingObservations);
		ofNullable(getMarketDisruption()).ifPresent(builder::setMarketDisruption);
		ofNullable(getSchedule()).ifPresent(schedule -> schedule.forEach(builder::addSchedule));
		return builder;
	}

	public static AveragingPeriodBuilder builder() {
		return new AveragingPeriodBuilder();
	}

	public static  class AveragingPeriodBuilder extends RosettaModelObjectBuilder {
	
		protected DateTimeList.DateTimeListBuilder averagingDateTimes;
		protected AveragingObservationList.AveragingObservationListBuilder averagingObservations;
		protected FieldWithMetaMarketDisruptionEnumBuilder marketDisruption;
		protected List<AveragingSchedule.AveragingScheduleBuilder> schedule;
	
		public AveragingPeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AveragingPeriod> metaData() {
			return metaData;
		} 
	
		
		public DateTimeList.DateTimeListBuilder getAveragingDateTimes() {
			return averagingDateTimes;
		}
		
		public DateTimeList.DateTimeListBuilder getOrCreateAveragingDateTimes() {
			if (averagingDateTimes!=null) {
				return averagingDateTimes;
			}
			else return averagingDateTimes = new DateTimeList.DateTimeListBuilder();
		}
		
		
		public AveragingObservationList.AveragingObservationListBuilder getAveragingObservations() {
			return averagingObservations;
		}
		
		public AveragingObservationList.AveragingObservationListBuilder getOrCreateAveragingObservations() {
			if (averagingObservations!=null) {
				return averagingObservations;
			}
			else return averagingObservations = new AveragingObservationList.AveragingObservationListBuilder();
		}
		
		
		public FieldWithMetaMarketDisruptionEnumBuilder getMarketDisruption() {
			return marketDisruption;
		}
		
		public FieldWithMetaMarketDisruptionEnumBuilder getOrCreateMarketDisruption() {
			if (marketDisruption!=null) {
				return marketDisruption;
			}
			else return marketDisruption = new FieldWithMetaMarketDisruptionEnumBuilder();
		}
		
		
		public List<AveragingSchedule.AveragingScheduleBuilder> getSchedule() {
			return schedule;
		}
		
		public AveragingSchedule.AveragingScheduleBuilder getOrCreateSchedule(int index) {
			if (schedule==null) {
				this.schedule = new ArrayList<>();
			}
			return getIndex(schedule, index, ()->new AveragingSchedule.AveragingScheduleBuilder());
		}
		
	
		public AveragingPeriodBuilder setAveragingDateTimes(DateTimeList averagingDateTimes) {
			this.averagingDateTimes = averagingDateTimes.toBuilder();
			return this;
		}
		public AveragingPeriodBuilder setAveragingDateTimesBuilder(DateTimeList.DateTimeListBuilder averagingDateTimes) {
			this.averagingDateTimes = averagingDateTimes;
			return this;
		}
		public AveragingPeriodBuilder setAveragingObservations(AveragingObservationList averagingObservations) {
			this.averagingObservations = averagingObservations.toBuilder();
			return this;
		}
		public AveragingPeriodBuilder setAveragingObservationsBuilder(AveragingObservationList.AveragingObservationListBuilder averagingObservations) {
			this.averagingObservations = averagingObservations;
			return this;
		}
		public AveragingPeriodBuilder setMarketDisruption(FieldWithMetaMarketDisruptionEnum marketDisruption) {
			this.marketDisruption = marketDisruption.toBuilder();
			return this;
		}
		public AveragingPeriodBuilder setMarketDisruptionRef(MarketDisruptionEnum marketDisruption) {
			return setMarketDisruption(FieldWithMetaMarketDisruptionEnum.builder().setValue(marketDisruption).build());
		}
		public AveragingPeriodBuilder addSchedule(AveragingSchedule schedule) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			this.schedule.add(schedule.toBuilder());
			return this;
		}
		public AveragingPeriodBuilder addSchedule(AveragingSchedule schedule, int _idx) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			getIndex(this.schedule, _idx, () -> schedule.toBuilder());
			this.schedule.set(_idx, schedule.toBuilder());
			return this;
		}
		public AveragingPeriodBuilder addSchedule(List<AveragingSchedule> schedules) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			for (AveragingSchedule toAdd : schedules) {
				this.schedule.add(toAdd.toBuilder());
			}
			return this;
		}
		public AveragingPeriodBuilder addScheduleBuilder(AveragingSchedule.AveragingScheduleBuilder schedule) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			this.schedule.add(schedule);
			return this;
		}
		
		public AveragingPeriodBuilder clearSchedule() {
			this.schedule = null;
			return this;
		}
		
		public AveragingPeriod build() {
			return new AveragingPeriod(this);
		}
	
		@Override
		public AveragingPeriodBuilder prune() {
			if (averagingDateTimes!=null && !averagingDateTimes.prune().hasData()) averagingDateTimes = null;
			if (averagingObservations!=null && !averagingObservations.prune().hasData()) averagingObservations = null;
			if (marketDisruption!=null && !marketDisruption.prune().hasData()) marketDisruption = null;
			if (schedule!=null) schedule = schedule.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAveragingDateTimes()!=null && getAveragingDateTimes().hasData()) return true;
			if (getAveragingObservations()!=null && getAveragingObservations().hasData()) return true;
			if (getMarketDisruption()!=null) return true;
			if (getSchedule()!=null && getSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("averagingDateTimes"), processor, DateTimeList.class, averagingDateTimes);
			processRosetta(path.newSubPath("averagingObservations"), processor, AveragingObservationList.class, averagingObservations);
			processRosetta(path.newSubPath("marketDisruption"), processor, FieldWithMetaMarketDisruptionEnum.class, marketDisruption);
			processRosetta(path.newSubPath("schedule"), processor, AveragingSchedule.class, schedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AveragingPeriodBuilder _that = (AveragingPeriodBuilder) o;
		
			if (averagingDateTimes != null ? !averagingDateTimes.equals(_that.averagingDateTimes) : _that.averagingDateTimes != null) return false;
			if (averagingObservations != null ? !averagingObservations.equals(_that.averagingObservations) : _that.averagingObservations != null) return false;
			if (marketDisruption != null ? !marketDisruption.equals(_that.marketDisruption) : _that.marketDisruption != null) return false;
			if (!ListEquals.listEquals(schedule, _that.schedule)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (averagingDateTimes != null ? averagingDateTimes.hashCode() : 0);
			_result = 31 * _result + (averagingObservations != null ? averagingObservations.hashCode() : 0);
			_result = 31 * _result + (marketDisruption != null ? marketDisruption.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (schedule != null ? schedule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AveragingPeriodBuilder {" +
				"averagingDateTimes=" + this.averagingDateTimes + ", " +
				"averagingObservations=" + this.averagingObservations + ", " +
				"marketDisruption=" + this.marketDisruption + ", " +
				"schedule=" + this.schedule +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("averagingDateTimes"), processor, DateTimeList.class, averagingDateTimes);
		processRosetta(path.newSubPath("averagingObservations"), processor, AveragingObservationList.class, averagingObservations);
		processRosetta(path.newSubPath("marketDisruption"), processor, FieldWithMetaMarketDisruptionEnum.class, marketDisruption);
		processRosetta(path.newSubPath("schedule"), processor, AveragingSchedule.class, schedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AveragingPeriod _that = (AveragingPeriod) o;
	
		if (averagingDateTimes != null ? !averagingDateTimes.equals(_that.averagingDateTimes) : _that.averagingDateTimes != null) return false;
		if (averagingObservations != null ? !averagingObservations.equals(_that.averagingObservations) : _that.averagingObservations != null) return false;
		if (marketDisruption != null ? !marketDisruption.equals(_that.marketDisruption) : _that.marketDisruption != null) return false;
		if (!ListEquals.listEquals(schedule, _that.schedule)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (averagingDateTimes != null ? averagingDateTimes.hashCode() : 0);
		_result = 31 * _result + (averagingObservations != null ? averagingObservations.hashCode() : 0);
		_result = 31 * _result + (marketDisruption != null ? marketDisruption.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (schedule != null ? schedule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AveragingPeriod {" +
			"averagingDateTimes=" + this.averagingDateTimes + ", " +
			"averagingObservations=" + this.averagingObservations + ", " +
			"marketDisruption=" + this.marketDisruption + ", " +
			"schedule=" + this.schedule +
		'}';
	}
}
