package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.CalculationPeriodFrequency;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.AveragingScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * Class to representing a method for generating a series of dates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="AveragingSchedule", source="FpML_5_10")
public class AveragingSchedule extends RosettaModelObject {
	private final CalculationPeriodFrequency averagingPeriodFrequency;
	private final Date endDate;
	private final Date startDate;
	private static AveragingScheduleMeta metaData = new AveragingScheduleMeta();
	
	AveragingSchedule(AveragingScheduleBuilder builder) {
		this.averagingPeriodFrequency = ofNullable(builder.getAveragingPeriodFrequency()).map(f->f.build()).orElse(null);
		this.endDate = builder.getEndDate();
		this.startDate = builder.getStartDate();
	}
	
	/**
	 * The frequency at which averaging period occurs with the regular part of the valuation schedule and their roll date convention.
	 */
	@RosettaSynonym(value="averagingPeriodFrequency", source="FpML_5_10")
	@RosettaSynonym(value="averagingPeriodFrequency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="averagingPeriodFrequency", source="DTCC_11_0")
	@RosettaSynonym(value="averagingPeriodFrequency", source="DTCC_9_0")
	@RosettaSynonym(value="averagingPeriodFrequency", source="CME_ClearedConfirm_1_17")
	public final CalculationPeriodFrequency getAveragingPeriodFrequency() {
		return averagingPeriodFrequency;
	}
	
	/**
	 * Date on which this period ends.
	 */
	@RosettaSynonym(value="endDate", source="FpML_5_10")
	@RosettaSynonym(value="endDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="endDate", source="DTCC_11_0")
	@RosettaSynonym(value="endDate", source="DTCC_9_0")
	@RosettaSynonym(value="endDate", source="CME_ClearedConfirm_1_17")
	public final Date getEndDate() {
		return endDate;
	}
	
	/**
	 * Date on which this period begins.
	 */
	@RosettaSynonym(value="startDate", source="FpML_5_10")
	@RosettaSynonym(value="startDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="startDate", source="DTCC_11_0")
	@RosettaSynonym(value="startDate", source="DTCC_9_0")
	@RosettaSynonym(value="startDate", source="CME_ClearedConfirm_1_17")
	public final Date getStartDate() {
		return startDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends AveragingSchedule> metaData() {
		return metaData;
	} 
	
	public AveragingScheduleBuilder toBuilder() {
		AveragingScheduleBuilder builder = new AveragingScheduleBuilder();
		ofNullable(getAveragingPeriodFrequency()).ifPresent(builder::setAveragingPeriodFrequency);
		ofNullable(getEndDate()).ifPresent(builder::setEndDate);
		ofNullable(getStartDate()).ifPresent(builder::setStartDate);
		return builder;
	}

	public static AveragingScheduleBuilder builder() {
		return new AveragingScheduleBuilder();
	}

	public static  class AveragingScheduleBuilder extends RosettaModelObjectBuilder {
	
		protected CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder averagingPeriodFrequency;
		protected Date endDate;
		protected Date startDate;
	
		public AveragingScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends AveragingSchedule> metaData() {
			return metaData;
		} 
	
		
		public CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder getAveragingPeriodFrequency() {
			return averagingPeriodFrequency;
		}
		
		public CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder getOrCreateAveragingPeriodFrequency() {
			if (averagingPeriodFrequency!=null) {
				return averagingPeriodFrequency;
			}
			else return averagingPeriodFrequency = new CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder();
		}
		
		
		public Date getEndDate() {
			return endDate;
		}
		
		
		public Date getStartDate() {
			return startDate;
		}
		
	
		public AveragingScheduleBuilder setAveragingPeriodFrequency(CalculationPeriodFrequency averagingPeriodFrequency) {
			this.averagingPeriodFrequency = averagingPeriodFrequency.toBuilder();
			return this;
		}
		public AveragingScheduleBuilder setAveragingPeriodFrequencyBuilder(CalculationPeriodFrequency.CalculationPeriodFrequencyBuilder averagingPeriodFrequency) {
			this.averagingPeriodFrequency = averagingPeriodFrequency;
			return this;
		}
		public AveragingScheduleBuilder setEndDate(Date endDate) {
			this.endDate = endDate;
			return this;
		}
		public AveragingScheduleBuilder setStartDate(Date startDate) {
			this.startDate = startDate;
			return this;
		}
		
		public AveragingSchedule build() {
			return new AveragingSchedule(this);
		}
	
		@Override
		public AveragingScheduleBuilder prune() {
			if (averagingPeriodFrequency!=null && !averagingPeriodFrequency.prune().hasData()) averagingPeriodFrequency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAveragingPeriodFrequency()!=null && getAveragingPeriodFrequency().hasData()) return true;
			if (getEndDate()!=null) return true;
			if (getStartDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("endDate"), Date.class, endDate, this);
			processor.processBasic(path.newSubPath("startDate"), Date.class, startDate, this);
			
			processRosetta(path.newSubPath("averagingPeriodFrequency"), processor, CalculationPeriodFrequency.class, averagingPeriodFrequency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			AveragingScheduleBuilder _that = (AveragingScheduleBuilder) o;
		
			if (averagingPeriodFrequency != null ? !averagingPeriodFrequency.equals(_that.averagingPeriodFrequency) : _that.averagingPeriodFrequency != null) return false;
			if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
			if (startDate != null ? !startDate.equals(_that.startDate) : _that.startDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (averagingPeriodFrequency != null ? averagingPeriodFrequency.hashCode() : 0);
			_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
			_result = 31 * _result + (startDate != null ? startDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AveragingScheduleBuilder {" +
				"averagingPeriodFrequency=" + this.averagingPeriodFrequency + ", " +
				"endDate=" + this.endDate + ", " +
				"startDate=" + this.startDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("endDate"), Date.class, endDate, this);
		processor.processBasic(path.newSubPath("startDate"), Date.class, startDate, this);
		
		processRosetta(path.newSubPath("averagingPeriodFrequency"), processor, CalculationPeriodFrequency.class, averagingPeriodFrequency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		AveragingSchedule _that = (AveragingSchedule) o;
	
		if (averagingPeriodFrequency != null ? !averagingPeriodFrequency.equals(_that.averagingPeriodFrequency) : _that.averagingPeriodFrequency != null) return false;
		if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
		if (startDate != null ? !startDate.equals(_that.startDate) : _that.startDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (averagingPeriodFrequency != null ? averagingPeriodFrequency.hashCode() : 0);
		_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
		_result = 31 * _result + (startDate != null ? startDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "AveragingSchedule {" +
			"averagingPeriodFrequency=" + this.averagingPeriodFrequency + ", " +
			"endDate=" + this.endDate + ", " +
			"startDate=" + this.startDate +
		'}';
	}
}
