package org.isda.cdm;

import java.lang.Integer;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ReferencePool;
import org.isda.cdm.Tranche;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BasketReferenceInformationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * CDS Basket Reference Information.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BasketReferenceInformation", source="FpML_5_10")
public class BasketReferenceInformation extends RosettaModelObject {
	private final List<FieldWithMetaString> basketId;
	private final FieldWithMetaString basketName;
	private final Integer mthToDefault;
	private final Integer nthToDefault;
	private final ReferencePool referencePool;
	private final Tranche tranche;
	private static BasketReferenceInformationMeta metaData = new BasketReferenceInformationMeta();
	
	BasketReferenceInformation(BasketReferenceInformationBuilder builder) {
		this.basketId = ofNullable(builder.getBasketId()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.basketName = ofNullable(builder.getBasketName()).map(f->f.build()).orElse(null);
		this.mthToDefault = builder.getMthToDefault();
		this.nthToDefault = builder.getNthToDefault();
		this.referencePool = ofNullable(builder.getReferencePool()).map(f->f.build()).orElse(null);
		this.tranche = ofNullable(builder.getTranche()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A CDS basket identifier.
	 */
	@RosettaSynonym(value="basketId", source="FpML_5_10")
	@RosettaSynonym(value="basketId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="basketId", source="DTCC_11_0")
	@RosettaSynonym(value="basketId", source="DTCC_9_0")
	@RosettaSynonym(value="basketId", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaString> getBasketId() {
		return basketId;
	}
	
	/**
	 * The name of the basket expressed as a free format string. FpML does not define usage rules for this element.
	 */
	@RosettaSynonym(value="basketName", source="FpML_5_10")
	@RosettaSynonym(value="basketName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="basketName", source="DTCC_11_0")
	@RosettaSynonym(value="basketName", source="DTCC_9_0")
	@RosettaSynonym(value="basketName", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getBasketName() {
		return basketName;
	}
	
	/**
	 * M th reference obligation to default to allow representation of N th to M th defaults.
	 */
	@RosettaSynonym(value="mthToDefault", source="FpML_5_10")
	@RosettaSynonym(value="mthToDefault", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mthToDefault", source="DTCC_11_0")
	@RosettaSynonym(value="mthToDefault", source="DTCC_9_0")
	@RosettaSynonym(value="mthToDefault", source="CME_ClearedConfirm_1_17")
	public final Integer getMthToDefault() {
		return mthToDefault;
	}
	
	/**
	 * N th reference obligation to default triggers payout.
	 */
	@RosettaSynonym(value="nthToDefault", source="FpML_5_10")
	@RosettaSynonym(value="nthToDefault", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="nthToDefault", source="DTCC_11_0")
	@RosettaSynonym(value="nthToDefault", source="DTCC_9_0")
	@RosettaSynonym(value="nthToDefault", source="CME_ClearedConfirm_1_17")
	public final Integer getNthToDefault() {
		return nthToDefault;
	}
	
	/**
	 * This element contains all the reference pool items to define the reference entity and reference obligation(s) in the basket.
	 */
	@RosettaSynonym(value="referencePool", source="FpML_5_10")
	@RosettaSynonym(value="referencePool", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referencePool", source="DTCC_11_0")
	@RosettaSynonym(value="referencePool", source="DTCC_9_0")
	@RosettaSynonym(value="referencePool", source="CME_ClearedConfirm_1_17")
	public final ReferencePool getReferencePool() {
		return referencePool;
	}
	
	/**
	 * This element contains CDS tranche terms.
	 */
	@RosettaSynonym(value="tranche", source="FpML_5_10")
	@RosettaSynonym(value="tranche", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tranche", source="DTCC_11_0")
	@RosettaSynonym(value="tranche", source="DTCC_9_0")
	@RosettaSynonym(value="tranche", source="CME_ClearedConfirm_1_17")
	public final Tranche getTranche() {
		return tranche;
	}
	
	
	@Override
	public RosettaMetaData<? extends BasketReferenceInformation> metaData() {
		return metaData;
	} 
	
	public BasketReferenceInformationBuilder toBuilder() {
		BasketReferenceInformationBuilder builder = new BasketReferenceInformationBuilder();
		ofNullable(getBasketId()).ifPresent(basketId -> basketId.forEach(builder::addBasketId));
		ofNullable(getBasketName()).ifPresent(builder::setBasketName);
		ofNullable(getMthToDefault()).ifPresent(builder::setMthToDefault);
		ofNullable(getNthToDefault()).ifPresent(builder::setNthToDefault);
		ofNullable(getReferencePool()).ifPresent(builder::setReferencePool);
		ofNullable(getTranche()).ifPresent(builder::setTranche);
		return builder;
	}

	public static BasketReferenceInformationBuilder builder() {
		return new BasketReferenceInformationBuilder();
	}

	public static  class BasketReferenceInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<FieldWithMetaStringBuilder> basketId;
		protected FieldWithMetaStringBuilder basketName;
		protected Integer mthToDefault;
		protected Integer nthToDefault;
		protected ReferencePool.ReferencePoolBuilder referencePool;
		protected Tranche.TrancheBuilder tranche;
	
		public BasketReferenceInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BasketReferenceInformation> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getBasketId() {
			return basketId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateBasketId(int index) {
			if (basketId==null) {
				this.basketId = new ArrayList<>();
			}
			return getIndex(basketId, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getBasketName() {
			return basketName;
		}
		
		public FieldWithMetaStringBuilder getOrCreateBasketName() {
			if (basketName!=null) {
				return basketName;
			}
			else return basketName = new FieldWithMetaStringBuilder();
		}
		
		
		public Integer getMthToDefault() {
			return mthToDefault;
		}
		
		
		public Integer getNthToDefault() {
			return nthToDefault;
		}
		
		
		public ReferencePool.ReferencePoolBuilder getReferencePool() {
			return referencePool;
		}
		
		public ReferencePool.ReferencePoolBuilder getOrCreateReferencePool() {
			if (referencePool!=null) {
				return referencePool;
			}
			else return referencePool = new ReferencePool.ReferencePoolBuilder();
		}
		
		
		public Tranche.TrancheBuilder getTranche() {
			return tranche;
		}
		
		public Tranche.TrancheBuilder getOrCreateTranche() {
			if (tranche!=null) {
				return tranche;
			}
			else return tranche = new Tranche.TrancheBuilder();
		}
		
	
		public BasketReferenceInformationBuilder addBasketId(FieldWithMetaString basketId) {
			if(this.basketId == null){
				this.basketId = new ArrayList<>();
			}
			this.basketId.add(basketId.toBuilder());
			return this;
		}
		public BasketReferenceInformationBuilder addBasketId(FieldWithMetaString basketId, int _idx) {
			if(this.basketId == null){
				this.basketId = new ArrayList<>();
			}
			getIndex(this.basketId, _idx, () -> basketId.toBuilder());
			this.basketId.set(_idx, basketId.toBuilder());
			return this;
		}
		public BasketReferenceInformationBuilder addBasketId(List<FieldWithMetaString> basketIds) {
			if(this.basketId == null){
				this.basketId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : basketIds) {
				this.basketId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public BasketReferenceInformationBuilder clearBasketId() {
			this.basketId = null;
			return this;
		}
		public BasketReferenceInformationBuilder setBasketName(FieldWithMetaString basketName) {
			this.basketName = basketName.toBuilder();
			return this;
		}
		public BasketReferenceInformationBuilder setBasketNameRef(String basketName) {
			return setBasketName(FieldWithMetaString.builder().setValue(basketName).build());
		}
		public BasketReferenceInformationBuilder setMthToDefault(Integer mthToDefault) {
			this.mthToDefault = mthToDefault;
			return this;
		}
		public BasketReferenceInformationBuilder setNthToDefault(Integer nthToDefault) {
			this.nthToDefault = nthToDefault;
			return this;
		}
		public BasketReferenceInformationBuilder setReferencePool(ReferencePool referencePool) {
			this.referencePool = referencePool.toBuilder();
			return this;
		}
		public BasketReferenceInformationBuilder setReferencePoolBuilder(ReferencePool.ReferencePoolBuilder referencePool) {
			this.referencePool = referencePool;
			return this;
		}
		public BasketReferenceInformationBuilder setTranche(Tranche tranche) {
			this.tranche = tranche.toBuilder();
			return this;
		}
		public BasketReferenceInformationBuilder setTrancheBuilder(Tranche.TrancheBuilder tranche) {
			this.tranche = tranche;
			return this;
		}
		
		public BasketReferenceInformation build() {
			return new BasketReferenceInformation(this);
		}
	
		@Override
		public BasketReferenceInformationBuilder prune() {
			if (basketId!=null) basketId = basketId.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (basketName!=null && !basketName.prune().hasData()) basketName = null;
			if (referencePool!=null && !referencePool.prune().hasData()) referencePool = null;
			if (tranche!=null && !tranche.prune().hasData()) tranche = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasketId()!=null && !getBasketId().isEmpty()) return true;
			if (getBasketName()!=null) return true;
			if (getMthToDefault()!=null) return true;
			if (getNthToDefault()!=null) return true;
			if (getReferencePool()!=null && getReferencePool().hasData()) return true;
			if (getTranche()!=null && getTranche().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("mthToDefault"), Integer.class, mthToDefault, this);
			processor.processBasic(path.newSubPath("nthToDefault"), Integer.class, nthToDefault, this);
			
			processRosetta(path.newSubPath("basketId"), processor, FieldWithMetaString.class, basketId);
			processRosetta(path.newSubPath("basketName"), processor, FieldWithMetaString.class, basketName);
			processRosetta(path.newSubPath("referencePool"), processor, ReferencePool.class, referencePool);
			processRosetta(path.newSubPath("tranche"), processor, Tranche.class, tranche);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BasketReferenceInformationBuilder _that = (BasketReferenceInformationBuilder) o;
		
			if (!ListEquals.listEquals(basketId, _that.basketId)) return false;
			if (basketName != null ? !basketName.equals(_that.basketName) : _that.basketName != null) return false;
			if (mthToDefault != null ? !mthToDefault.equals(_that.mthToDefault) : _that.mthToDefault != null) return false;
			if (nthToDefault != null ? !nthToDefault.equals(_that.nthToDefault) : _that.nthToDefault != null) return false;
			if (referencePool != null ? !referencePool.equals(_that.referencePool) : _that.referencePool != null) return false;
			if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basketId != null ? basketId.hashCode() : 0);
			_result = 31 * _result + (basketName != null ? basketName.hashCode() : 0);
			_result = 31 * _result + (mthToDefault != null ? mthToDefault.hashCode() : 0);
			_result = 31 * _result + (nthToDefault != null ? nthToDefault.hashCode() : 0);
			_result = 31 * _result + (referencePool != null ? referencePool.hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BasketReferenceInformationBuilder {" +
				"basketId=" + this.basketId + ", " +
				"basketName=" + this.basketName + ", " +
				"mthToDefault=" + this.mthToDefault + ", " +
				"nthToDefault=" + this.nthToDefault + ", " +
				"referencePool=" + this.referencePool + ", " +
				"tranche=" + this.tranche +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("mthToDefault"), Integer.class, mthToDefault, this);
		processor.processBasic(path.newSubPath("nthToDefault"), Integer.class, nthToDefault, this);
		
		processRosetta(path.newSubPath("basketId"), processor, FieldWithMetaString.class, basketId);
		processRosetta(path.newSubPath("basketName"), processor, FieldWithMetaString.class, basketName);
		processRosetta(path.newSubPath("referencePool"), processor, ReferencePool.class, referencePool);
		processRosetta(path.newSubPath("tranche"), processor, Tranche.class, tranche);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BasketReferenceInformation _that = (BasketReferenceInformation) o;
	
		if (!ListEquals.listEquals(basketId, _that.basketId)) return false;
		if (basketName != null ? !basketName.equals(_that.basketName) : _that.basketName != null) return false;
		if (mthToDefault != null ? !mthToDefault.equals(_that.mthToDefault) : _that.mthToDefault != null) return false;
		if (nthToDefault != null ? !nthToDefault.equals(_that.nthToDefault) : _that.nthToDefault != null) return false;
		if (referencePool != null ? !referencePool.equals(_that.referencePool) : _that.referencePool != null) return false;
		if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (basketId != null ? basketId.hashCode() : 0);
		_result = 31 * _result + (basketName != null ? basketName.hashCode() : 0);
		_result = 31 * _result + (mthToDefault != null ? mthToDefault.hashCode() : 0);
		_result = 31 * _result + (nthToDefault != null ? nthToDefault.hashCode() : 0);
		_result = 31 * _result + (referencePool != null ? referencePool.hashCode() : 0);
		_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BasketReferenceInformation {" +
			"basketId=" + this.basketId + ", " +
			"basketName=" + this.basketName + ", " +
			"mthToDefault=" + this.mthToDefault + ", " +
			"nthToDefault=" + this.nthToDefault + ", " +
			"referencePool=" + this.referencePool + ", " +
			"tranche=" + this.tranche +
		'}';
	}
}
