package org.isda.cdm;

import org.isda.cdm.BondChoiceModel;
import org.isda.cdm.Equity;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BondEquityModelMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Bond equity model to value convertible bonds and modelled onto BondEquity.model in FpML.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BondEquityModel extends RosettaModelObject {
	private final BondChoiceModel bondchoiceModel;
	private final Equity equity;
	private static BondEquityModelMeta metaData = new BondEquityModelMeta();
	
	BondEquityModel(BondEquityModelBuilder builder) {
		this.bondchoiceModel = ofNullable(builder.getBondchoiceModel()).map(f->f.build()).orElse(null);
		this.equity = ofNullable(builder.getEquity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Either the bond or convertible bond.
	 */
	public final BondChoiceModel getBondchoiceModel() {
		return bondchoiceModel;
	}
	
	/**
	 * The equity.
	 */
	public final Equity getEquity() {
		return equity;
	}
	
	
	@Override
	public RosettaMetaData<? extends BondEquityModel> metaData() {
		return metaData;
	} 
	
	public BondEquityModelBuilder toBuilder() {
		BondEquityModelBuilder builder = new BondEquityModelBuilder();
		ofNullable(getBondchoiceModel()).ifPresent(builder::setBondchoiceModel);
		ofNullable(getEquity()).ifPresent(builder::setEquity);
		return builder;
	}

	public static BondEquityModelBuilder builder() {
		return new BondEquityModelBuilder();
	}

	public static  class BondEquityModelBuilder extends RosettaModelObjectBuilder {
	
		protected BondChoiceModel.BondChoiceModelBuilder bondchoiceModel;
		protected Equity.EquityBuilder equity;
	
		public BondEquityModelBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BondEquityModel> metaData() {
			return metaData;
		} 
	
		
		public BondChoiceModel.BondChoiceModelBuilder getBondchoiceModel() {
			return bondchoiceModel;
		}
		
		public BondChoiceModel.BondChoiceModelBuilder getOrCreateBondchoiceModel() {
			if (bondchoiceModel!=null) {
				return bondchoiceModel;
			}
			else return bondchoiceModel = new BondChoiceModel.BondChoiceModelBuilder();
		}
		
		
		public Equity.EquityBuilder getEquity() {
			return equity;
		}
		
		public Equity.EquityBuilder getOrCreateEquity() {
			if (equity!=null) {
				return equity;
			}
			else return equity = new Equity.EquityBuilder();
		}
		
	
		public BondEquityModelBuilder setBondchoiceModel(BondChoiceModel bondchoiceModel) {
			this.bondchoiceModel = bondchoiceModel.toBuilder();
			return this;
		}
		public BondEquityModelBuilder setBondchoiceModelBuilder(BondChoiceModel.BondChoiceModelBuilder bondchoiceModel) {
			this.bondchoiceModel = bondchoiceModel;
			return this;
		}
		public BondEquityModelBuilder setEquity(Equity equity) {
			this.equity = equity.toBuilder();
			return this;
		}
		public BondEquityModelBuilder setEquityBuilder(Equity.EquityBuilder equity) {
			this.equity = equity;
			return this;
		}
		
		public BondEquityModel build() {
			return new BondEquityModel(this);
		}
	
		@Override
		public BondEquityModelBuilder prune() {
			if (bondchoiceModel!=null && !bondchoiceModel.prune().hasData()) bondchoiceModel = null;
			if (equity!=null && !equity.prune().hasData()) equity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBondchoiceModel()!=null && getBondchoiceModel().hasData()) return true;
			if (getEquity()!=null && getEquity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("bondchoiceModel"), processor, BondChoiceModel.class, bondchoiceModel);
			processRosetta(path.newSubPath("equity"), processor, Equity.class, equity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BondEquityModelBuilder _that = (BondEquityModelBuilder) o;
		
			if (bondchoiceModel != null ? !bondchoiceModel.equals(_that.bondchoiceModel) : _that.bondchoiceModel != null) return false;
			if (equity != null ? !equity.equals(_that.equity) : _that.equity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bondchoiceModel != null ? bondchoiceModel.hashCode() : 0);
			_result = 31 * _result + (equity != null ? equity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BondEquityModelBuilder {" +
				"bondchoiceModel=" + this.bondchoiceModel + ", " +
				"equity=" + this.equity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("bondchoiceModel"), processor, BondChoiceModel.class, bondchoiceModel);
		processRosetta(path.newSubPath("equity"), processor, Equity.class, equity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BondEquityModel _that = (BondEquityModel) o;
	
		if (bondchoiceModel != null ? !bondchoiceModel.equals(_that.bondchoiceModel) : _that.bondchoiceModel != null) return false;
		if (equity != null ? !equity.equals(_that.equity) : _that.equity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bondchoiceModel != null ? bondchoiceModel.hashCode() : 0);
		_result = 31 * _result + (equity != null ? equity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BondEquityModel {" +
			"bondchoiceModel=" + this.bondchoiceModel + ", " +
			"equity=" + this.equity +
		'}';
	}
}
