package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaBrokerConfirmationTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaBrokerConfirmationTypeEnum.FieldWithMetaBrokerConfirmationTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BrokerConfirmationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Identifies the market sector in which the trade has been arranged.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BrokerConfirmation", source="FpML_5_10")
public class BrokerConfirmation extends RosettaModelObject {
	private final FieldWithMetaBrokerConfirmationTypeEnum brokerConfirmationType;
	private static BrokerConfirmationMeta metaData = new BrokerConfirmationMeta();
	
	BrokerConfirmation(BrokerConfirmationBuilder builder) {
		this.brokerConfirmationType = ofNullable(builder.getBrokerConfirmationType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The type of broker confirmation executed between the parties.
	 */
	@RosettaSynonym(value="brokerConfirmationType", source="FpML_5_10")
	@RosettaSynonym(value="brokerConfirmationType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="brokerConfirmationType", source="DTCC_11_0")
	@RosettaSynonym(value="brokerConfirmationType", source="DTCC_9_0")
	@RosettaSynonym(value="brokerConfirmationType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaBrokerConfirmationTypeEnum getBrokerConfirmationType() {
		return brokerConfirmationType;
	}
	
	
	@Override
	public RosettaMetaData<? extends BrokerConfirmation> metaData() {
		return metaData;
	} 
	
	public BrokerConfirmationBuilder toBuilder() {
		BrokerConfirmationBuilder builder = new BrokerConfirmationBuilder();
		ofNullable(getBrokerConfirmationType()).ifPresent(builder::setBrokerConfirmationType);
		return builder;
	}

	public static BrokerConfirmationBuilder builder() {
		return new BrokerConfirmationBuilder();
	}

	public static  class BrokerConfirmationBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaBrokerConfirmationTypeEnumBuilder brokerConfirmationType;
	
		public BrokerConfirmationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BrokerConfirmation> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaBrokerConfirmationTypeEnumBuilder getBrokerConfirmationType() {
			return brokerConfirmationType;
		}
		
		public FieldWithMetaBrokerConfirmationTypeEnumBuilder getOrCreateBrokerConfirmationType() {
			if (brokerConfirmationType!=null) {
				return brokerConfirmationType;
			}
			else return brokerConfirmationType = new FieldWithMetaBrokerConfirmationTypeEnumBuilder();
		}
		
	
		public BrokerConfirmationBuilder setBrokerConfirmationType(FieldWithMetaBrokerConfirmationTypeEnum brokerConfirmationType) {
			this.brokerConfirmationType = brokerConfirmationType.toBuilder();
			return this;
		}
		public BrokerConfirmationBuilder setBrokerConfirmationTypeRef(BrokerConfirmationTypeEnum brokerConfirmationType) {
			return setBrokerConfirmationType(FieldWithMetaBrokerConfirmationTypeEnum.builder().setValue(brokerConfirmationType).build());
		}
		
		public BrokerConfirmation build() {
			return new BrokerConfirmation(this);
		}
	
		@Override
		public BrokerConfirmationBuilder prune() {
			if (brokerConfirmationType!=null && !brokerConfirmationType.prune().hasData()) brokerConfirmationType = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBrokerConfirmationType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("brokerConfirmationType"), processor, FieldWithMetaBrokerConfirmationTypeEnum.class, brokerConfirmationType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BrokerConfirmationBuilder _that = (BrokerConfirmationBuilder) o;
		
			if (brokerConfirmationType != null ? !brokerConfirmationType.equals(_that.brokerConfirmationType) : _that.brokerConfirmationType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (brokerConfirmationType != null ? brokerConfirmationType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BrokerConfirmationBuilder {" +
				"brokerConfirmationType=" + this.brokerConfirmationType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("brokerConfirmationType"), processor, FieldWithMetaBrokerConfirmationTypeEnum.class, brokerConfirmationType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BrokerConfirmation _that = (BrokerConfirmation) o;
	
		if (brokerConfirmationType != null ? !brokerConfirmationType.equals(_that.brokerConfirmationType) : _that.brokerConfirmationType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (brokerConfirmationType != null ? brokerConfirmationType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BrokerConfirmation {" +
			"brokerConfirmationType=" + this.brokerConfirmationType +
		'}';
	}
}
