package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of Broker Confirm that the FpML trade represents.
 * 
 * @version 2.5.4
 */
public enum BrokerConfirmationTypeEnum {
    

	/**
	 * Broker Confirmation Type representing ABX index trades.
	 */
	@RosettaSynonym(value = "ABX", source = "FpML_5_10")
	@RosettaSynonym(value = "ABX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ABX", source = "DTCC_11_0")
	@RosettaSynonym(value = "ABX", source = "DTCC_9_0")
	@RosettaSynonym(value = "ABX", source = "CME_ClearedConfirm_1_17")
	ABX,
	    

	/**
	 * Broker Confirmation Type of Asia Corporate.
	 */
	@RosettaSynonym(value = "AsiaCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "AsiaCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsiaCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsiaCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsiaCorporate", source = "CME_ClearedConfirm_1_17")
	ASIA_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of Asia Sovereign.
	 */
	@RosettaSynonym(value = "AsiaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "AsiaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsiaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsiaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsiaSovereign", source = "CME_ClearedConfirm_1_17")
	ASIA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of Australia Corporate.
	 */
	@RosettaSynonym(value = "AustraliaCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "AustraliaCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AustraliaCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "AustraliaCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "AustraliaCorporate", source = "CME_ClearedConfirm_1_17")
	AUSTRALIA_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of Australia Sovereign.
	 */
	@RosettaSynonym(value = "AustraliaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "AustraliaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AustraliaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "AustraliaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "AustraliaSovereign", source = "CME_ClearedConfirm_1_17")
	AUSTRALIA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for use with Credit Derivative Transactions on Leveraged Loans.
	 */
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "FpML_5_10")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "CME_ClearedConfirm_1_17")
	CD_SON_LEVERAGED_LOANS,
	    

	/**
	 * Broker Confirmation Type for use with Credit Derivative Transactions on Mortgage-backed Security with Pay-As-You-Go or Physical Settlement.
	 */
	@RosettaSynonym(value = "CDSonMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "CDSonMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDSonMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDSonMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDSonMBS", source = "CME_ClearedConfirm_1_17")
	CD_SON_MBS,
	    

	/**
	 * Broker Confirmation Type for CDX Emerging Markets Untranched Transactions.
	 */
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "CME_ClearedConfirm_1_17")
	CDX_EMERGING_MARKETS,
	    

	/**
	 * Broker Confirmation Type for CDX Emerging Markets Diversified Untranched Transactions.
	 */
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "CME_ClearedConfirm_1_17")
	CDX_EMERGING_MARKETS_DIVERSIFIED,
	    

	/**
	 * Broker Confirmation Type for CDX Swaption Transactions.
	 */
	@RosettaSynonym(value = "CDXSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXSwaption", source = "CME_ClearedConfirm_1_17")
	CDX_SWAPTION,
	    

	/**
	 * Broker Confirmation Type for Dow Jones CDX Tranche Transactions.
	 */
	@RosettaSynonym(value = "CDXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXTranche", source = "CME_ClearedConfirm_1_17")
	CDX_TRANCHE,
	    

	/**
	 * Broker Confirmation Type representing CMBX index trades.
	 */
	@RosettaSynonym(value = "CMBX", source = "FpML_5_10")
	@RosettaSynonym(value = "CMBX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CMBX", source = "DTCC_11_0")
	@RosettaSynonym(value = "CMBX", source = "DTCC_9_0")
	@RosettaSynonym(value = "CMBX", source = "CME_ClearedConfirm_1_17")
	CMBX,
	    

	/**
	 * Broker Confirmation Type for CDS Index trades relating to Dow Jones CDX.EM index series.
	 */
	@RosettaSynonym(value = "DJ.CDX.EM ", source = "FpML_5_10")
	@RosettaSynonym(value = "DJ.CDX.EM ", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DJ.CDX.EM ", source = "DTCC_11_0")
	@RosettaSynonym(value = "DJ.CDX.EM ", source = "DTCC_9_0")
	@RosettaSynonym(value = "DJ.CDX.EM ", source = "CME_ClearedConfirm_1_17")
	DJ_CDX_EM_("DJ.CDX.EM "),
	

	/**
	 * Broker Confirmation Type for CDS Index trades relating to Dow Jones CDX.NA.IG and Dow Jones CDX.NA.HY index series.
	 */
	@RosettaSynonym(value = "DJ.CDX.NA", source = "FpML_5_10")
	@RosettaSynonym(value = "DJ.CDX.NA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DJ.CDX.NA", source = "DTCC_11_0")
	@RosettaSynonym(value = "DJ.CDX.NA", source = "DTCC_9_0")
	@RosettaSynonym(value = "DJ.CDX.NA", source = "CME_ClearedConfirm_1_17")
	DJ_CDX_NA("DJ.CDX.NA"),
	

	/**
	 * Broker Confirmation Type of Emerging European and Middle Eastern Sovereign.
	 */
	@RosettaSynonym(value = "EmergingEuropeanAndMiddleEasternSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "EmergingEuropeanAndMiddleEasternSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EmergingEuropeanAndMiddleEasternSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "EmergingEuropeanAndMiddleEasternSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "EmergingEuropeanAndMiddleEasternSovereign", source = "CME_ClearedConfirm_1_17")
	EMERGING_EUROPEAN_AND_MIDDLE_EASTERN_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for EMERGING EUROPEAN CORPORATE.
	 */
	@RosettaSynonym(value = "EmergingEuropeanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "EmergingEuropeanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporate", source = "CME_ClearedConfirm_1_17")
	EMERGING_EUROPEAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type for EMERGING EUROPEAN CORPORATE LPN.
	 */
	@RosettaSynonym(value = "EmergingEuropeanCorporateLPN", source = "FpML_5_10")
	@RosettaSynonym(value = "EmergingEuropeanCorporateLPN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporateLPN", source = "DTCC_11_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporateLPN", source = "DTCC_9_0")
	@RosettaSynonym(value = "EmergingEuropeanCorporateLPN", source = "CME_ClearedConfirm_1_17")
	EMERGING_EUROPEAN_CORPORATE_LPN,
	    

	/**
	 * Broker Confirmation Type for Single Name European CMBS Transactions.
	 */
	@RosettaSynonym(value = "EuropeanCMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "EuropeanCMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "CME_ClearedConfirm_1_17")
	EUROPEAN_CMBS,
	    

	/**
	 * Broker Confirmation Type of European Corporate.
	 */
	@RosettaSynonym(value = "EuropeanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "EuropeanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EuropeanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "EuropeanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "EuropeanCorporate", source = "CME_ClearedConfirm_1_17")
	EUROPEAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type for Single Name European RMBS Transactions.
	 */
	@RosettaSynonym(value = "EuropeanRMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "EuropeanRMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "CME_ClearedConfirm_1_17")
	EUROPEAN_RMBS,
	    

	/**
	 * Broker Confirmation Type of Japan Corporate.
	 */
	@RosettaSynonym(value = "JapanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "JapanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JapanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "JapanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "JapanCorporate", source = "CME_ClearedConfirm_1_17")
	JAPAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of Japan Sovereign.
	 */
	@RosettaSynonym(value = "JapanSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "JapanSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JapanSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "JapanSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "JapanSovereign", source = "CME_ClearedConfirm_1_17")
	JAPAN_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of Latin America Corporate.
	 */
	@RosettaSynonym(value = "LatinAmericaCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "LatinAmericaCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LatinAmericaCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "LatinAmericaCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "LatinAmericaCorporate", source = "CME_ClearedConfirm_1_17")
	LATIN_AMERICA_CORPORATE,
	    

	/**
	 * Broker Confirmation Type for LATIN AMERICA CORPORATE B.
	 */
	@RosettaSynonym(value = "LatinAmericaCorporateBond", source = "FpML_5_10")
	@RosettaSynonym(value = "LatinAmericaCorporateBond", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBond", source = "DTCC_11_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBond", source = "DTCC_9_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBond", source = "CME_ClearedConfirm_1_17")
	LATIN_AMERICA_CORPORATE_BOND,
	    

	/**
	 * Broker Confirmation Type for LATIN AMERICA CORPORATE BL.
	 */
	@RosettaSynonym(value = "LatinAmericaCorporateBondOrLoan", source = "FpML_5_10")
	@RosettaSynonym(value = "LatinAmericaCorporateBondOrLoan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBondOrLoan", source = "DTCC_11_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBondOrLoan", source = "DTCC_9_0")
	@RosettaSynonym(value = "LatinAmericaCorporateBondOrLoan", source = "CME_ClearedConfirm_1_17")
	LATIN_AMERICA_CORPORATE_BOND_OR_LOAN,
	    

	/**
	 * Broker Confirmation Type of Latin America Sovereign.
	 */
	@RosettaSynonym(value = "LatinAmericaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "LatinAmericaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LatinAmericaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "LatinAmericaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "LatinAmericaSovereign", source = "CME_ClearedConfirm_1_17")
	LATIN_AMERICA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for MBX Transactions.
	 */
	@RosettaSynonym(value = "MBX", source = "FpML_5_10")
	@RosettaSynonym(value = "MBX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MBX", source = "DTCC_11_0")
	@RosettaSynonym(value = "MBX", source = "DTCC_9_0")
	@RosettaSynonym(value = "MBX", source = "CME_ClearedConfirm_1_17")
	MBX,
	    

	/**
	 * Broker Confirmation Type for Municipal CDX Untranched Transactions.
	 */
	@RosettaSynonym(value = "MCDX", source = "FpML_5_10")
	@RosettaSynonym(value = "MCDX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MCDX", source = "DTCC_11_0")
	@RosettaSynonym(value = "MCDX", source = "DTCC_9_0")
	@RosettaSynonym(value = "MCDX", source = "CME_ClearedConfirm_1_17")
	MCDX,
	    

	/**
	 * Broker Confirmation Type of New Zealand Corporate.
	 */
	@RosettaSynonym(value = "NewZealandCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "NewZealandCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NewZealandCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "NewZealandCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "NewZealandCorporate", source = "CME_ClearedConfirm_1_17")
	NEW_ZEALAND_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of New Zealand Sovereign.
	 */
	@RosettaSynonym(value = "NewZealandSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "NewZealandSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NewZealandSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "NewZealandSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "NewZealandSovereign", source = "CME_ClearedConfirm_1_17")
	NEW_ZEALAND_SOVEREIGN,
	    

	/**
	 * Broker ConfirmationType of North American Corporate.
	 */
	@RosettaSynonym(value = "NorthAmericanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "NorthAmericanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NorthAmericanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "NorthAmericanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "NorthAmericanCorporate", source = "CME_ClearedConfirm_1_17")
	NORTH_AMERICAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type for PO Index Transactions.
	 */
	@RosettaSynonym(value = "PO", source = "FpML_5_10")
	@RosettaSynonym(value = "PO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PO", source = "DTCC_11_0")
	@RosettaSynonym(value = "PO", source = "DTCC_9_0")
	@RosettaSynonym(value = "PO", source = "CME_ClearedConfirm_1_17")
	PO,
	    

	/**
	 * Broker Confirmation Type of Singapore Corporate.
	 */
	@RosettaSynonym(value = "SingaporeCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "SingaporeCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SingaporeCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SingaporeCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SingaporeCorporate", source = "CME_ClearedConfirm_1_17")
	SINGAPORE_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of Singapore Sovereign.
	 */
	@RosettaSynonym(value = "SingaporeSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "SingaporeSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SingaporeSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "SingaporeSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "SingaporeSovereign", source = "CME_ClearedConfirm_1_17")
	SINGAPORE_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of STANDARD ASIA CORPORATE.
	 */
	@RosettaSynonym(value = "StandardAsiaCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardAsiaCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardAsiaCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardAsiaCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardAsiaCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_ASIA_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD ASIA SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardAsiaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardAsiaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardAsiaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardAsiaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardAsiaSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_ASIA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of STANDARD AUSTRALIA CORPORATE.
	 */
	@RosettaSynonym(value = "StandardAustraliaCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardAustraliaCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardAustraliaCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardAustraliaCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardAustraliaCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_AUSTRALIA_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD AUSTRALIA SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardAustraliaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardAustraliaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardAustraliaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardAustraliaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardAustraliaSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_AUSTRALIA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for Standard CDX Tranche Transactions.
	 */
	@RosettaSynonym(value = "StandardCDXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardCDXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "CME_ClearedConfirm_1_17")
	STANDARD_CDX_TRANCHE,
	    

	/**
	 * Broker Confirmation Type of STANDARD EMERGING EUROPEAN AND MIDDLE EASTERN SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardEmergingEuropeanAndMiddleEasternSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardEmergingEuropeanAndMiddleEasternSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanAndMiddleEasternSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanAndMiddleEasternSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanAndMiddleEasternSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_EMERGING_EUROPEAN_AND_MIDDLE_EASTERN_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of STANDARD EMERGING EUROPEAN CORPORATE.
	 */
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_EMERGING_EUROPEAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD EMERGING EUROPEAN CORPORATE LPN.
	 */
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporateLPN", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporateLPN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporateLPN", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporateLPN", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardEmergingEuropeanCorporateLPN", source = "CME_ClearedConfirm_1_17")
	STANDARD_EMERGING_EUROPEAN_CORPORATE_LPN,
	    

	/**
	 * Broker Confirmation Type for STANDARD EUROPEAN CORPORATE.
	 */
	@RosettaSynonym(value = "StandardEuropeanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardEuropeanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardEuropeanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardEuropeanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardEuropeanCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_EUROPEAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD JAPAN CORPORATE.
	 */
	@RosettaSynonym(value = "StandardJapanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardJapanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardJapanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardJapanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardJapanCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_JAPAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD JAPAN SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardJapanSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardJapanSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardJapanSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardJapanSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardJapanSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_JAPAN_SOVEREIGN,
	    

	/**
	 * Standard Syndicated Secured Loan Credit Default Swap Broker Confirmation Type.
	 */
	@RosettaSynonym(value = "StandardLCDS", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDS", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDS", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDS", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDS,
	    

	/**
	 * Broker Confirmation Type for Standard Syndicated Secured Loan Credit Default Swap Bullet Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDSBullet", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDS_BULLET,
	    

	/**
	 * Broker Confirmation Type for Standard Syndicated Secured Loan Credit Default Swap Index Bullet Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDXBullet", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDX_BULLET,
	    

	/**
	 * Broker Confirmation Type for Standard Syndicated Secured Loan Credit Default Swap Index Bullet Tranche Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDX_BULLET_TRANCHE,
	    

	/**
	 * Broker Confirmation Type of STANDARD LATIN AMERICA CORPORATE B.
	 */
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBond", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBond", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBond", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBond", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBond", source = "CME_ClearedConfirm_1_17")
	STANDARD_LATIN_AMERICA_CORPORATE_BOND,
	    

	/**
	 * Broker Confirmation Type of STANDARD LATIN AMERICA CORPORATE BL.
	 */
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBondOrLoan", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBondOrLoan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBondOrLoan", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBondOrLoan", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLatinAmericaCorporateBondOrLoan", source = "CME_ClearedConfirm_1_17")
	STANDARD_LATIN_AMERICA_CORPORATE_BOND_OR_LOAN,
	    

	/**
	 * Broker Confirmation Type of STANDARD LATIN AMERICA SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardLatinAmericaSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLatinAmericaSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLatinAmericaSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLatinAmericaSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLatinAmericaSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_LATIN_AMERICA_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type of STANDARD NEW ZEALAND CORPORATE.
	 */
	@RosettaSynonym(value = "StandardNewZealandCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardNewZealandCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardNewZealandCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardNewZealandCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardNewZealandCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_NEW_ZEALAND_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD NEW ZEALAND SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardNewZealandSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardNewZealandSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardNewZealandSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardNewZealandSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardNewZealandSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_NEW_ZEALAND_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for STANDARD NORTH AMERICAN CORPORATE.
	 */
	@RosettaSynonym(value = "StandardNorthAmericanCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardNorthAmericanCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardNorthAmericanCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardNorthAmericanCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardNorthAmericanCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_NORTH_AMERICAN_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD SINGAPORE CORPORATE.
	 */
	@RosettaSynonym(value = "StandardSingaporeCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardSingaporeCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardSingaporeCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardSingaporeCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardSingaporeCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_SINGAPORE_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of STANDARD SINGAPORE SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardSingaporeSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardSingaporeSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardSingaporeSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardSingaporeSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardSingaporeSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_SINGAPORE_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for STANDARD SUBORDINATED EUROPEAN INSURANCE CORPORATE.
	 */
	@RosettaSynonym(value = "StandardSubordinatedEuropeanInsuranceCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardSubordinatedEuropeanInsuranceCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardSubordinatedEuropeanInsuranceCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardSubordinatedEuropeanInsuranceCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardSubordinatedEuropeanInsuranceCorporate", source = "CME_ClearedConfirm_1_17")
	STANDARD_SUBORDINATED_EUROPEAN_INSURANCE_CORPORATE,
	    

	/**
	 * Broker Confirmation Type for STANDARD WESTERN EUROPEAN SOVEREIGN.
	 */
	@RosettaSynonym(value = "StandardWesternEuropeanSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardWesternEuropeanSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardWesternEuropeanSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardWesternEuropeanSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardWesternEuropeanSovereign", source = "CME_ClearedConfirm_1_17")
	STANDARD_WESTERN_EUROPEAN_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for Standard iTraxx Europe Tranched Transactions.
	 */
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "CME_ClearedConfirm_1_17")
	STANDARDI_TRAXX_EUROPE_TRANCHE,
	    

	/**
	 * Broker Confirmation Type of Subordinated European Insurance Corporate.
	 */
	@RosettaSynonym(value = "SubordinatedEuropeanInsuranceCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "SubordinatedEuropeanInsuranceCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SubordinatedEuropeanInsuranceCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SubordinatedEuropeanInsuranceCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SubordinatedEuropeanInsuranceCorporate", source = "CME_ClearedConfirm_1_17")
	SUBORDINATED_EUROPEAN_INSURANCE_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of SUKUK CORPORATE.
	 */
	@RosettaSynonym(value = "SukukCorporate", source = "FpML_5_10")
	@RosettaSynonym(value = "SukukCorporate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SukukCorporate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SukukCorporate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SukukCorporate", source = "CME_ClearedConfirm_1_17")
	SUKUK_CORPORATE,
	    

	/**
	 * Broker Confirmation Type of SUKUK SOVEREIGN.
	 */
	@RosettaSynonym(value = "SukukSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "SukukSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SukukSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "SukukSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "SukukSovereign", source = "CME_ClearedConfirm_1_17")
	SUKUK_SOVEREIGN,
	    

	/**
	 * Syndicated Secured Loan Credit Default Swap Broker Confirmation Type.
	 */
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "FpML_5_10")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "DTCC_11_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "DTCC_9_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "CME_ClearedConfirm_1_17")
	SYNDICATED_SECURED_LOAN_CDS,
	    

	/**
	 * Broker Confirmation Type for TRX Transactions.
	 */
	@RosettaSynonym(value = "TRX", source = "FpML_5_10")
	@RosettaSynonym(value = "TRX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRX", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRX", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRX", source = "CME_ClearedConfirm_1_17")
	TRX,
	    

	/**
	 * Broker Confirmation Type for TRX.II Transactions.
	 */
	@RosettaSynonym(value = "TRX.II", source = "FpML_5_10")
	@RosettaSynonym(value = "TRX.II", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRX.II", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRX.II", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRX.II", source = "CME_ClearedConfirm_1_17")
	TRX_II("TRX.II"),
	

	/**
	 * Broker Confirmation Type for U.S. MUNICIPAL FULL FAITH AND CREDIT.
	 */
	@RosettaSynonym(value = "USMunicipalFullFaithAndCredit", source = "FpML_5_10")
	@RosettaSynonym(value = "USMunicipalFullFaithAndCredit", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USMunicipalFullFaithAndCredit", source = "DTCC_11_0")
	@RosettaSynonym(value = "USMunicipalFullFaithAndCredit", source = "DTCC_9_0")
	@RosettaSynonym(value = "USMunicipalFullFaithAndCredit", source = "CME_ClearedConfirm_1_17")
	US_MUNICIPAL_FULL_FAITH_AND_CREDIT,
	    

	/**
	 * Broker Confirmation Type for U.S. MUNICIPAL GENERAL FUND.
	 */
	@RosettaSynonym(value = "USMunicipalGeneralFund", source = "FpML_5_10")
	@RosettaSynonym(value = "USMunicipalGeneralFund", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USMunicipalGeneralFund", source = "DTCC_11_0")
	@RosettaSynonym(value = "USMunicipalGeneralFund", source = "DTCC_9_0")
	@RosettaSynonym(value = "USMunicipalGeneralFund", source = "CME_ClearedConfirm_1_17")
	US_MUNICIPAL_GENERAL_FUND,
	    

	/**
	 * Broker Confirmation Type for U.S. MUNICIPAL REVENUE.
	 */
	@RosettaSynonym(value = "USMunicipalRevenue", source = "FpML_5_10")
	@RosettaSynonym(value = "USMunicipalRevenue", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USMunicipalRevenue", source = "DTCC_11_0")
	@RosettaSynonym(value = "USMunicipalRevenue", source = "DTCC_9_0")
	@RosettaSynonym(value = "USMunicipalRevenue", source = "CME_ClearedConfirm_1_17")
	US_MUNICIPAL_REVENUE,
	    

	/**
	 * Broker Confirmation Type of Western European Sovereign.
	 */
	@RosettaSynonym(value = "WesternEuropeanSovereign", source = "FpML_5_10")
	@RosettaSynonym(value = "WesternEuropeanSovereign", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "WesternEuropeanSovereign", source = "DTCC_11_0")
	@RosettaSynonym(value = "WesternEuropeanSovereign", source = "DTCC_9_0")
	@RosettaSynonym(value = "WesternEuropeanSovereign", source = "CME_ClearedConfirm_1_17")
	WESTERN_EUROPEAN_SOVEREIGN,
	    

	/**
	 * Broker Confirmation Type for iTraxx Asia Excluding Japan.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN,
	    

	/**
	 * Broker Confirmation Type for iTraxx Asia Ex-Japan Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN_SWAPTION,
	    

	/**
	 * Broker Confirmation Type for iTraxx Asia Excluding Japan Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN_TRANCHE,
	    

	/**
	 * Broker Confirmation Type for iTraxx Australia.
	 */
	@RosettaSynonym(value = "iTraxxAustralia", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustralia", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA,
	    

	/**
	 * Broker Confirmation Type for iTraxx Australia Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA_SWAPTION,
	    

	/**
	 * Broker Confirmation Type for iTraxx Australia Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA_TRANCHE,
	    

	/**
	 * Broker Confirmation Type for iTraxx CJ.
	 */
	@RosettaSynonym(value = "iTraxxCJ", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxCJ", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_CJ,
	    

	/**
	 * Broker Confirmation Type for iTraxx CJ Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxCJTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_CJ_TRANCHE,
	    

	/**
	 * Broker Confirmation Type for iTraxx Europe Transactions
	 */
	@RosettaSynonym(value = "iTraxxEurope", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEurope", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE,
	    

	/**
	 * Broker Confirmation Type for iTraxx Europe Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_SWAPTION,
	    

	/**
	 * Broker Confirmation Type for iTraxx Europe Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_TRANCHE,
	    

	/**
	 * Broker Confirmation Type for iTraxx Japan.
	 */
	@RosettaSynonym(value = "iTraxxJapan", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN,
	    

	/**
	 * Broker Confirmation Type for iTraxx Japan Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN_SWAPTION,
	    

	/**
	 * Broker Confirmation Type for iTraxx Japan Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN_TRANCHE,
	    

	/**
	 * Broker Confirmation Type for iTraxx LevX.
	 */
	@RosettaSynonym(value = "iTraxxLevX", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxLevX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_LEV_X,
	    

	/**
	 * Broker Confirmation Type for iTraxx SDI 75 Transactions.
	 */
	@RosettaSynonym(value = "iTraxxSDI75", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxSDI75", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxSDI75", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxSDI75", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxSDI75", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_SDI_75,
	    

	/**
	 * Broker Confirmation Type for iTraxx SovX.
	 */
	@RosettaSynonym(value = "iTraxxSovX", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxSovX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_SOV_X    
	;

	private final String displayName;
	
	BrokerConfirmationTypeEnum() {
		this.displayName = null;
	}

	BrokerConfirmationTypeEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
