package org.isda.cdm;

import java.time.LocalTime;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum.FieldWithMetaBusinessCenterEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BusinessCenterTimeMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.LocalTime;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining a time with respect to a business day calendar location. For example, 11:00:00 GBLO.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="BusinessCenterTime", source="FpML_5_10")
public class BusinessCenterTime extends RosettaModelObject {
	private final FieldWithMetaBusinessCenterEnum businessCenter;
	private final LocalTime hourMinuteTime;
	private static BusinessCenterTimeMeta metaData = new BusinessCenterTimeMeta();
	
	BusinessCenterTime(BusinessCenterTimeBuilder builder) {
		this.businessCenter = ofNullable(builder.getBusinessCenter()).map(f->f.build()).orElse(null);
		this.hourMinuteTime = builder.getHourMinuteTime();
	}
	
	/**
	 * A code identifying a business day calendar location. A business day calendar location is drawn from the list identified by the business day calendar location enumeration.
	 */
	@RosettaSynonym(value="businessCenter", source="FpML_5_10")
	@RosettaSynonym(value="businessCenter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenter", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="partyA_location", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_location", source="ISDA_Create_1_0")
	@RosettaSynonym(value="location", source="ISDA_Create_1_0")
	public final FieldWithMetaBusinessCenterEnum getBusinessCenter() {
		return businessCenter;
	}
	
	/**
	 * A time specified in hh:mm:ss format where the second component must be &#39;00&#39;, e.g. 11am would be represented as 11:00:00.
	 */
	@RosettaSynonym(value="hourMinuteTime", source="FpML_5_10")
	@RosettaSynonym(value="hourMinuteTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="hourMinuteTime", source="DTCC_11_0")
	@RosettaSynonym(value="hourMinuteTime", source="DTCC_9_0")
	@RosettaSynonym(value="hourMinuteTime", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="partyA_time", source="ISDA_Create_1_0")
	@RosettaSynonym(value="partyB_time", source="ISDA_Create_1_0")
	@RosettaSynonym(value="time", source="ISDA_Create_1_0")
	public final LocalTime getHourMinuteTime() {
		return hourMinuteTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends BusinessCenterTime> metaData() {
		return metaData;
	} 
	
	public BusinessCenterTimeBuilder toBuilder() {
		BusinessCenterTimeBuilder builder = new BusinessCenterTimeBuilder();
		ofNullable(getBusinessCenter()).ifPresent(builder::setBusinessCenter);
		ofNullable(getHourMinuteTime()).ifPresent(builder::setHourMinuteTime);
		return builder;
	}

	public static BusinessCenterTimeBuilder builder() {
		return new BusinessCenterTimeBuilder();
	}

	public static  class BusinessCenterTimeBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaBusinessCenterEnumBuilder businessCenter;
		protected LocalTime hourMinuteTime;
	
		public BusinessCenterTimeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BusinessCenterTime> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaBusinessCenterEnumBuilder getBusinessCenter() {
			return businessCenter;
		}
		
		public FieldWithMetaBusinessCenterEnumBuilder getOrCreateBusinessCenter() {
			if (businessCenter!=null) {
				return businessCenter;
			}
			else return businessCenter = new FieldWithMetaBusinessCenterEnumBuilder();
		}
		
		
		public LocalTime getHourMinuteTime() {
			return hourMinuteTime;
		}
		
	
		public BusinessCenterTimeBuilder setBusinessCenter(FieldWithMetaBusinessCenterEnum businessCenter) {
			this.businessCenter = businessCenter.toBuilder();
			return this;
		}
		public BusinessCenterTimeBuilder setBusinessCenterRef(BusinessCenterEnum businessCenter) {
			return setBusinessCenter(FieldWithMetaBusinessCenterEnum.builder().setValue(businessCenter).build());
		}
		public BusinessCenterTimeBuilder setHourMinuteTime(LocalTime hourMinuteTime) {
			this.hourMinuteTime = hourMinuteTime;
			return this;
		}
		
		public BusinessCenterTime build() {
			return new BusinessCenterTime(this);
		}
	
		@Override
		public BusinessCenterTimeBuilder prune() {
			if (businessCenter!=null && !businessCenter.prune().hasData()) businessCenter = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenter()!=null) return true;
			if (getHourMinuteTime()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("hourMinuteTime"), LocalTime.class, hourMinuteTime, this);
			
			processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BusinessCenterTimeBuilder _that = (BusinessCenterTimeBuilder) o;
		
			if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
			if (hourMinuteTime != null ? !hourMinuteTime.equals(_that.hourMinuteTime) : _that.hourMinuteTime != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (hourMinuteTime != null ? hourMinuteTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BusinessCenterTimeBuilder {" +
				"businessCenter=" + this.businessCenter + ", " +
				"hourMinuteTime=" + this.hourMinuteTime +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("hourMinuteTime"), LocalTime.class, hourMinuteTime, this);
		
		processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BusinessCenterTime _that = (BusinessCenterTime) o;
	
		if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
		if (hourMinuteTime != null ? !hourMinuteTime.equals(_that.hourMinuteTime) : _that.hourMinuteTime != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (hourMinuteTime != null ? hourMinuteTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BusinessCenterTime {" +
			"businessCenter=" + this.businessCenter + ", " +
			"hourMinuteTime=" + this.hourMinuteTime +
		'}';
	}
}
