package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum.FieldWithMetaBusinessCenterEnumBuilder;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters;
import org.isda.cdm.metafields.ReferenceWithMetaBusinessCenters.ReferenceWithMetaBusinessCentersBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BusinessCentersMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class for specifying the business day calendar location used in determining whether a day is a business day or not, either by specifying this business center by reference to an enumerated list that is maintained by the FpML standard, or by reference to such specification when it exists elsewhere as part of the instance document. This class corresponds to the FpML BusinessCentersOrReference.model.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BusinessCenters extends RosettaModelObject implements GlobalKey {
	private final List<FieldWithMetaBusinessCenterEnum> businessCenter;
	private final ReferenceWithMetaBusinessCenters businessCentersReference;
	private final MetaFields meta;
	private static BusinessCentersMeta metaData = new BusinessCentersMeta();
	
	BusinessCenters(BusinessCentersBuilder builder) {
		this.businessCenter = ofNullable(builder.getBusinessCenter()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.businessCentersReference = ofNullable(builder.getBusinessCentersReference()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A code identifying one or several business day calendar location(s). The set of business day calendar locations are specified by the business day calendar location enumeration which is maintained by the FpML standard.
	 */
	@RosettaSynonym(value="businessCenter", source="FpML_5_10")
	@RosettaSynonym(value="businessCenter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenter", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="businessCenter", source="FpML_5_10", path="businessCenters")
	@RosettaSynonym(value="businessCenter", source="CME_SubmissionIRS_1_0", path="businessCenters")
	@RosettaSynonym(value="businessCenter", source="DTCC_11_0", path="businessCenters")
	@RosettaSynonym(value="businessCenter", source="DTCC_9_0", path="businessCenters")
	@RosettaSynonym(value="businessCenter", source="CME_ClearedConfirm_1_17", path="businessCenters")
	public final List<FieldWithMetaBusinessCenterEnum> getBusinessCenter() {
		return businessCenter;
	}
	
	/**
	 * A reference to a financial business center location specified elsewhere in the instance document.
	 */
	@RosettaSynonym(value="businessCentersReference", source="FpML_5_10")
	@RosettaSynonym(value="businessCentersReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_11_0")
	@RosettaSynonym(value="businessCentersReference", source="DTCC_9_0")
	@RosettaSynonym(value="businessCentersReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaBusinessCenters getBusinessCentersReference() {
		return businessCentersReference;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends BusinessCenters> metaData() {
		return metaData;
	} 
	
	public BusinessCentersBuilder toBuilder() {
		BusinessCentersBuilder builder = new BusinessCentersBuilder();
		ofNullable(getBusinessCenter()).ifPresent(businessCenter -> businessCenter.forEach(builder::addBusinessCenter));
		ofNullable(getBusinessCentersReference()).ifPresent(builder::setBusinessCentersReference);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static BusinessCentersBuilder builder() {
		return new BusinessCentersBuilder();
	}

	public static  class BusinessCentersBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<BusinessCentersBuilder> {
	
		protected List<FieldWithMetaBusinessCenterEnumBuilder> businessCenter;
		protected ReferenceWithMetaBusinessCentersBuilder businessCentersReference;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public BusinessCentersBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BusinessCenters> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaBusinessCenterEnumBuilder> getBusinessCenter() {
			return businessCenter;
		}
		
		public FieldWithMetaBusinessCenterEnumBuilder getOrCreateBusinessCenter(int index) {
			if (businessCenter==null) {
				this.businessCenter = new ArrayList<>();
			}
			return getIndex(businessCenter, index, ()->new FieldWithMetaBusinessCenterEnumBuilder());
		}
		
		
		public ReferenceWithMetaBusinessCentersBuilder getBusinessCentersReference() {
			return businessCentersReference;
		}
		
		public ReferenceWithMetaBusinessCentersBuilder getOrCreateBusinessCentersReference() {
			if (businessCentersReference!=null) {
				return businessCentersReference;
			}
			else return businessCentersReference = new ReferenceWithMetaBusinessCentersBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public BusinessCentersBuilder addBusinessCenter(FieldWithMetaBusinessCenterEnum businessCenter) {
			if(this.businessCenter == null){
				this.businessCenter = new ArrayList<>();
			}
			this.businessCenter.add(businessCenter.toBuilder());
			return this;
		}
		public BusinessCentersBuilder addBusinessCenter(FieldWithMetaBusinessCenterEnum businessCenter, int _idx) {
			if(this.businessCenter == null){
				this.businessCenter = new ArrayList<>();
			}
			getIndex(this.businessCenter, _idx, () -> businessCenter.toBuilder());
			this.businessCenter.set(_idx, businessCenter.toBuilder());
			return this;
		}
		public BusinessCentersBuilder addBusinessCenter(List<FieldWithMetaBusinessCenterEnum> businessCenters) {
			if(this.businessCenter == null){
				this.businessCenter = new ArrayList<>();
			}
			for (FieldWithMetaBusinessCenterEnum toAdd : businessCenters) {
				this.businessCenter.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public BusinessCentersBuilder clearBusinessCenter() {
			this.businessCenter = null;
			return this;
		}
		public BusinessCentersBuilder setBusinessCentersReference(ReferenceWithMetaBusinessCenters businessCentersReference) {
			this.businessCentersReference = businessCentersReference.toBuilder();
			return this;
		}
		public BusinessCentersBuilder setBusinessCentersReferenceBuilder(ReferenceWithMetaBusinessCentersBuilder businessCentersReference) {
			this.businessCentersReference = businessCentersReference;
			return this;
		}
		public BusinessCentersBuilder setBusinessCentersReferenceRef(BusinessCenters.BusinessCentersBuilder businessCentersReference) {
			return setBusinessCentersReference(ReferenceWithMetaBusinessCenters.builder().setValueBuilder(businessCentersReference).build());
		}
		public BusinessCentersBuilder setBusinessCentersReferenceRef(BusinessCenters businessCentersReference) {
			return setBusinessCentersReferenceRef(BusinessCenters.builder());
		}
		public BusinessCentersBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public BusinessCentersBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public BusinessCenters build() {
			return new BusinessCenters(this);
		}
	
		@Override
		public BusinessCentersBuilder prune() {
			if (businessCenter!=null) businessCenter = businessCenter.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (businessCentersReference!=null && !businessCentersReference.prune().hasData()) businessCentersReference = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenter()!=null && !getBusinessCenter().isEmpty()) return true;
			if (getBusinessCentersReference()!=null && getBusinessCentersReference().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
			processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BusinessCentersBuilder _that = (BusinessCentersBuilder) o;
		
			if (!ListEquals.listEquals(businessCenter, _that.businessCenter)) return false;
			if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenter != null ? businessCenter.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BusinessCentersBuilder {" +
				"businessCenter=" + this.businessCenter + ", " +
				"businessCentersReference=" + this.businessCentersReference + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
		processRosetta(path.newSubPath("businessCentersReference"), processor, ReferenceWithMetaBusinessCenters.class, businessCentersReference);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BusinessCenters _that = (BusinessCenters) o;
	
		if (!ListEquals.listEquals(businessCenter, _that.businessCenter)) return false;
		if (businessCentersReference != null ? !businessCentersReference.equals(_that.businessCentersReference) : _that.businessCentersReference != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenter != null ? businessCenter.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		_result = 31 * _result + (businessCentersReference != null ? businessCentersReference.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BusinessCenters {" +
			"businessCenter=" + this.businessCenter + ", " +
			"businessCentersReference=" + this.businessCentersReference + ", " +
			"meta=" + this.meta +
		'}';
	}
}
