package org.isda.cdm;

import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.BuyerSellerMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * This class corresponds to the FpML BuyerSeller.model construct.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class BuyerSeller extends RosettaModelObject {
	private final ReferenceWithMetaAccount buyerAccountReference;
	private final ReferenceWithMetaParty buyerPartyReference;
	private final ReferenceWithMetaAccount sellerAccountReference;
	private final ReferenceWithMetaParty sellerPartyReference;
	private static BuyerSellerMeta metaData = new BuyerSellerMeta();
	
	BuyerSeller(BuyerSellerBuilder builder) {
		this.buyerAccountReference = ofNullable(builder.getBuyerAccountReference()).map(f->f.build()).orElse(null);
		this.buyerPartyReference = ofNullable(builder.getBuyerPartyReference()).map(f->f.build()).orElse(null);
		this.sellerAccountReference = ofNullable(builder.getSellerAccountReference()).map(f->f.build()).orElse(null);
		this.sellerPartyReference = ofNullable(builder.getSellerPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A reference to the account that buys this instrument.
	 */
	@RosettaSynonym(value="buyerAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="buyerAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyerAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="buyerAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="buyerAccountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getBuyerAccountReference() {
		return buyerAccountReference;
	}
	
	/**
	 * A reference to the party that buys this instrument, i.e. pays for this instrument and receives the rights defined by it. ISDA 2002 Equity Definitions section 1.18: `Buyer` means the party specified as such in the related Confirmation. | ISDA 2006 Definitions article 12.1 (b)(i) relating to a Swaption: &#39;Buyer&#39; means the party that will, on each Premium Payment Date, pay to Seller the Premium | ISDA 2006 Definitions article 12.1 (b)(ii) relating to Swap Transactions with applicable Early Termination: the party specified as such in the related Confirmation, or the Exercising Party if neither party is specified | ISDA 2006 Definitions article 12.1 (b)(iii) relating to any other Option Transaction: the party specified as such in the related Confirmation. | ISDA 2014 Credit Definition article 1.4: `Buyer` means the Fixed Rate Payer.
	 */
	@RosettaSynonym(value="buyerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="buyerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="buyerPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getBuyerPartyReference() {
		return buyerPartyReference;
	}
	
	/**
	 * A reference to the account that sells this instrument.
	 */
	@RosettaSynonym(value="sellerAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="sellerAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sellerAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="sellerAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="sellerAccountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getSellerAccountReference() {
		return sellerAccountReference;
	}
	
	/**
	 * A reference to the party that sells (&#39;writes&#39;) this instrument, i.e. that grants the rights defined by this instrument and in return receives a payment for it. ISDA 2002 Equity Definitions section 1.19: `Seller` means the party specified as such in the related Confirmation. | ISDA 2006 Definitions article 12.1 (a)(i) relating to a Swaption: &#39;Seller&#39; means the party the party specified as such or as writer in the related Confirmation | ISDA 2006 Definitions article 12.1 (a)(ii) relating to Swap Transactions with applicable Early Termination: the party specified as such or as writer in the related Confirmation or, if neither party is specified as such, the Non-exercising Party | ISDA 2006 Definitions article 12.1 (a)(iii) relating to any other Option Transaction: the party specified as such in the related Confirmation. | ISDA 2014 Credit Definition article 1.4: `Seller` means the Floating Rate Payer.
	 */
	@RosettaSynonym(value="sellerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="sellerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="sellerPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getSellerPartyReference() {
		return sellerPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends BuyerSeller> metaData() {
		return metaData;
	} 
	
	public BuyerSellerBuilder toBuilder() {
		BuyerSellerBuilder builder = new BuyerSellerBuilder();
		ofNullable(getBuyerAccountReference()).ifPresent(builder::setBuyerAccountReference);
		ofNullable(getBuyerPartyReference()).ifPresent(builder::setBuyerPartyReference);
		ofNullable(getSellerAccountReference()).ifPresent(builder::setSellerAccountReference);
		ofNullable(getSellerPartyReference()).ifPresent(builder::setSellerPartyReference);
		return builder;
	}

	public static BuyerSellerBuilder builder() {
		return new BuyerSellerBuilder();
	}

	public static  class BuyerSellerBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaAccountBuilder buyerAccountReference;
		protected ReferenceWithMetaPartyBuilder buyerPartyReference;
		protected ReferenceWithMetaAccountBuilder sellerAccountReference;
		protected ReferenceWithMetaPartyBuilder sellerPartyReference;
	
		public BuyerSellerBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends BuyerSeller> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaAccountBuilder getBuyerAccountReference() {
			return buyerAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateBuyerAccountReference() {
			if (buyerAccountReference!=null) {
				return buyerAccountReference;
			}
			else return buyerAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getBuyerPartyReference() {
			return buyerPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateBuyerPartyReference() {
			if (buyerPartyReference!=null) {
				return buyerPartyReference;
			}
			else return buyerPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaAccountBuilder getSellerAccountReference() {
			return sellerAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateSellerAccountReference() {
			if (sellerAccountReference!=null) {
				return sellerAccountReference;
			}
			else return sellerAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getSellerPartyReference() {
			return sellerPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateSellerPartyReference() {
			if (sellerPartyReference!=null) {
				return sellerPartyReference;
			}
			else return sellerPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public BuyerSellerBuilder setBuyerAccountReference(ReferenceWithMetaAccount buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference.toBuilder();
			return this;
		}
		public BuyerSellerBuilder setBuyerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference;
			return this;
		}
		public BuyerSellerBuilder setBuyerAccountReferenceRef(Account.AccountBuilder buyerAccountReference) {
			return setBuyerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(buyerAccountReference).build());
		}
		public BuyerSellerBuilder setBuyerAccountReferenceRef(Account buyerAccountReference) {
			return setBuyerAccountReferenceRef(Account.builder());
		}
		public BuyerSellerBuilder setBuyerPartyReference(ReferenceWithMetaParty buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference.toBuilder();
			return this;
		}
		public BuyerSellerBuilder setBuyerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference;
			return this;
		}
		public BuyerSellerBuilder setBuyerPartyReferenceRef(Party.PartyBuilder buyerPartyReference) {
			return setBuyerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(buyerPartyReference).build());
		}
		public BuyerSellerBuilder setBuyerPartyReferenceRef(Party buyerPartyReference) {
			return setBuyerPartyReferenceRef(Party.builder());
		}
		public BuyerSellerBuilder setSellerAccountReference(ReferenceWithMetaAccount sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference.toBuilder();
			return this;
		}
		public BuyerSellerBuilder setSellerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference;
			return this;
		}
		public BuyerSellerBuilder setSellerAccountReferenceRef(Account.AccountBuilder sellerAccountReference) {
			return setSellerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(sellerAccountReference).build());
		}
		public BuyerSellerBuilder setSellerAccountReferenceRef(Account sellerAccountReference) {
			return setSellerAccountReferenceRef(Account.builder());
		}
		public BuyerSellerBuilder setSellerPartyReference(ReferenceWithMetaParty sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference.toBuilder();
			return this;
		}
		public BuyerSellerBuilder setSellerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference;
			return this;
		}
		public BuyerSellerBuilder setSellerPartyReferenceRef(Party.PartyBuilder sellerPartyReference) {
			return setSellerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(sellerPartyReference).build());
		}
		public BuyerSellerBuilder setSellerPartyReferenceRef(Party sellerPartyReference) {
			return setSellerPartyReferenceRef(Party.builder());
		}
		
		public BuyerSeller build() {
			return new BuyerSeller(this);
		}
	
		@Override
		public BuyerSellerBuilder prune() {
			if (buyerAccountReference!=null && !buyerAccountReference.prune().hasData()) buyerAccountReference = null;
			if (buyerPartyReference!=null && !buyerPartyReference.prune().hasData()) buyerPartyReference = null;
			if (sellerAccountReference!=null && !sellerAccountReference.prune().hasData()) sellerAccountReference = null;
			if (sellerPartyReference!=null && !sellerPartyReference.prune().hasData()) sellerPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBuyerAccountReference()!=null && getBuyerAccountReference().hasData()) return true;
			if (getBuyerPartyReference()!=null && getBuyerPartyReference().hasData()) return true;
			if (getSellerAccountReference()!=null && getSellerAccountReference().hasData()) return true;
			if (getSellerPartyReference()!=null && getSellerPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("buyerAccountReference"), processor, ReferenceWithMetaAccount.class, buyerAccountReference);
			processRosetta(path.newSubPath("buyerPartyReference"), processor, ReferenceWithMetaParty.class, buyerPartyReference);
			processRosetta(path.newSubPath("sellerAccountReference"), processor, ReferenceWithMetaAccount.class, sellerAccountReference);
			processRosetta(path.newSubPath("sellerPartyReference"), processor, ReferenceWithMetaParty.class, sellerPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			BuyerSellerBuilder _that = (BuyerSellerBuilder) o;
		
			if (buyerAccountReference != null ? !buyerAccountReference.equals(_that.buyerAccountReference) : _that.buyerAccountReference != null) return false;
			if (buyerPartyReference != null ? !buyerPartyReference.equals(_that.buyerPartyReference) : _that.buyerPartyReference != null) return false;
			if (sellerAccountReference != null ? !sellerAccountReference.equals(_that.sellerAccountReference) : _that.sellerAccountReference != null) return false;
			if (sellerPartyReference != null ? !sellerPartyReference.equals(_that.sellerPartyReference) : _that.sellerPartyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (buyerAccountReference != null ? buyerAccountReference.hashCode() : 0);
			_result = 31 * _result + (buyerPartyReference != null ? buyerPartyReference.hashCode() : 0);
			_result = 31 * _result + (sellerAccountReference != null ? sellerAccountReference.hashCode() : 0);
			_result = 31 * _result + (sellerPartyReference != null ? sellerPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BuyerSellerBuilder {" +
				"buyerAccountReference=" + this.buyerAccountReference + ", " +
				"buyerPartyReference=" + this.buyerPartyReference + ", " +
				"sellerAccountReference=" + this.sellerAccountReference + ", " +
				"sellerPartyReference=" + this.sellerPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("buyerAccountReference"), processor, ReferenceWithMetaAccount.class, buyerAccountReference);
		processRosetta(path.newSubPath("buyerPartyReference"), processor, ReferenceWithMetaParty.class, buyerPartyReference);
		processRosetta(path.newSubPath("sellerAccountReference"), processor, ReferenceWithMetaAccount.class, sellerAccountReference);
		processRosetta(path.newSubPath("sellerPartyReference"), processor, ReferenceWithMetaParty.class, sellerPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		BuyerSeller _that = (BuyerSeller) o;
	
		if (buyerAccountReference != null ? !buyerAccountReference.equals(_that.buyerAccountReference) : _that.buyerAccountReference != null) return false;
		if (buyerPartyReference != null ? !buyerPartyReference.equals(_that.buyerPartyReference) : _that.buyerPartyReference != null) return false;
		if (sellerAccountReference != null ? !sellerAccountReference.equals(_that.sellerAccountReference) : _that.sellerAccountReference != null) return false;
		if (sellerPartyReference != null ? !sellerPartyReference.equals(_that.sellerPartyReference) : _that.sellerPartyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (buyerAccountReference != null ? buyerAccountReference.hashCode() : 0);
		_result = 31 * _result + (buyerPartyReference != null ? buyerPartyReference.hashCode() : 0);
		_result = 31 * _result + (sellerAccountReference != null ? sellerAccountReference.hashCode() : 0);
		_result = 31 * _result + (sellerPartyReference != null ? sellerPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "BuyerSeller {" +
			"buyerAccountReference=" + this.buyerAccountReference + ", " +
			"buyerPartyReference=" + this.buyerPartyReference + ", " +
			"sellerAccountReference=" + this.sellerAccountReference + ", " +
			"sellerPartyReference=" + this.sellerPartyReference +
		'}';
	}
}
