package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CalculationAgentPartyEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum.FieldWithMetaBusinessCenterEnumBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationAgentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the ISDA calculation agent responsible for performing duties as defined in the applicable product definitions.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CalculationAgent", source="FpML_5_10")
public class CalculationAgent extends RosettaModelObject {
	private final FieldWithMetaBusinessCenterEnum calculationAgentBusinessCenter;
	private final CalculationAgentPartyEnum calculationAgentParty;
	private final List<ReferenceWithMetaParty> calculationAgentPartyReference;
	private static CalculationAgentMeta metaData = new CalculationAgentMeta();
	
	CalculationAgent(CalculationAgentBuilder builder) {
		this.calculationAgentBusinessCenter = ofNullable(builder.getCalculationAgentBusinessCenter()).map(f->f.build()).orElse(null);
		this.calculationAgentParty = builder.getCalculationAgentParty();
		this.calculationAgentPartyReference = ofNullable(builder.getCalculationAgentPartyReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The city in which the office through which ISDA Calculation Agent is acting for purposes of the transaction is located The short-form confirm for a trade that is executed under a Sovereign or Asia Pacific Master Confirmation Agreement ( MCA ), does not need to specify the Calculation Agent. However, the confirm does need to specify the Calculation Agent City. This is due to the fact that the MCA sets the value for Calculation Agent but does not set the value for Calculation Agent City.
	 */
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="FpML_5_10")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_11_0")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="DTCC_9_0")
	@RosettaSynonym(value="calculationAgentBusinessCenter", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaBusinessCenterEnum getCalculationAgentBusinessCenter() {
		return calculationAgentBusinessCenter;
	}
	
	/**
	 * The ISDA calculation agent responsible for performing duties as defined in the applicable product definitions. For example, the Calculation Agent may be defined as being the Non-exercising Party.
	 */
	@RosettaSynonym(value="calculationAgentParty", source="FpML_5_10", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_SubmissionIRS_1_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_11_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_9_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_ClearedConfirm_1_17", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="FpML_5_10", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_SubmissionIRS_1_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_11_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_9_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_ClearedConfirm_1_17", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="FpML_5_10", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_SubmissionIRS_1_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_11_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_9_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_ClearedConfirm_1_17", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="FpML_5_10", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_SubmissionIRS_1_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_11_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="DTCC_9_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentParty", source="CME_ClearedConfirm_1_17", path="swaption.calculationAgent")
	public final CalculationAgentPartyEnum getCalculationAgentParty() {
		return calculationAgentParty;
	}
	
	/**
	 * In FpML, a pointer style reference to a party identifier defined elsewhere in the document. The party referenced is the ISDA Calculation Agent for the trade. If more than one party is referenced then the parties are assumed to be co-calculation agents, i.e. they have joint responsibility.
	 */
	@RosettaSynonym(value="calculationAgentPartyReference", source="FpML_5_10", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_SubmissionIRS_1_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_11_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_9_0", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_ClearedConfirm_1_17", path="trade.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="FpML_5_10", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_SubmissionIRS_1_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_11_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_9_0", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_ClearedConfirm_1_17", path="calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="FpML_5_10", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_SubmissionIRS_1_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_11_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_9_0", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_ClearedConfirm_1_17", path="trade.swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="FpML_5_10", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_SubmissionIRS_1_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_11_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="DTCC_9_0", path="swaption.calculationAgent")
	@RosettaSynonym(value="calculationAgentPartyReference", source="CME_ClearedConfirm_1_17", path="swaption.calculationAgent")
	public final List<ReferenceWithMetaParty> getCalculationAgentPartyReference() {
		return calculationAgentPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationAgent> metaData() {
		return metaData;
	} 
	
	public CalculationAgentBuilder toBuilder() {
		CalculationAgentBuilder builder = new CalculationAgentBuilder();
		ofNullable(getCalculationAgentBusinessCenter()).ifPresent(builder::setCalculationAgentBusinessCenter);
		ofNullable(getCalculationAgentParty()).ifPresent(builder::setCalculationAgentParty);
		ofNullable(getCalculationAgentPartyReference()).ifPresent(calculationAgentPartyReference -> calculationAgentPartyReference.forEach(builder::addCalculationAgentPartyReference));
		return builder;
	}

	public static CalculationAgentBuilder builder() {
		return new CalculationAgentBuilder();
	}

	public static  class CalculationAgentBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaBusinessCenterEnumBuilder calculationAgentBusinessCenter;
		protected CalculationAgentPartyEnum calculationAgentParty;
		protected List<ReferenceWithMetaPartyBuilder> calculationAgentPartyReference;
	
		public CalculationAgentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationAgent> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaBusinessCenterEnumBuilder getCalculationAgentBusinessCenter() {
			return calculationAgentBusinessCenter;
		}
		
		public FieldWithMetaBusinessCenterEnumBuilder getOrCreateCalculationAgentBusinessCenter() {
			if (calculationAgentBusinessCenter!=null) {
				return calculationAgentBusinessCenter;
			}
			else return calculationAgentBusinessCenter = new FieldWithMetaBusinessCenterEnumBuilder();
		}
		
		
		public CalculationAgentPartyEnum getCalculationAgentParty() {
			return calculationAgentParty;
		}
		
		
		public List<ReferenceWithMetaPartyBuilder> getCalculationAgentPartyReference() {
			return calculationAgentPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateCalculationAgentPartyReference(int index) {
			if (calculationAgentPartyReference==null) {
				this.calculationAgentPartyReference = new ArrayList<>();
			}
			return getIndex(calculationAgentPartyReference, index, ()->new ReferenceWithMetaPartyBuilder());
		}
		
	
		public CalculationAgentBuilder setCalculationAgentBusinessCenter(FieldWithMetaBusinessCenterEnum calculationAgentBusinessCenter) {
			this.calculationAgentBusinessCenter = calculationAgentBusinessCenter.toBuilder();
			return this;
		}
		public CalculationAgentBuilder setCalculationAgentBusinessCenterRef(BusinessCenterEnum calculationAgentBusinessCenter) {
			return setCalculationAgentBusinessCenter(FieldWithMetaBusinessCenterEnum.builder().setValue(calculationAgentBusinessCenter).build());
		}
		public CalculationAgentBuilder setCalculationAgentParty(CalculationAgentPartyEnum calculationAgentParty) {
			this.calculationAgentParty = calculationAgentParty;
			return this;
		}
		public CalculationAgentBuilder addCalculationAgentPartyReference(ReferenceWithMetaParty calculationAgentPartyReference) {
			if(this.calculationAgentPartyReference == null){
				this.calculationAgentPartyReference = new ArrayList<>();
			}
			this.calculationAgentPartyReference.add(calculationAgentPartyReference.toBuilder());
			return this;
		}
		public CalculationAgentBuilder addCalculationAgentPartyReference(ReferenceWithMetaParty calculationAgentPartyReference, int _idx) {
			if(this.calculationAgentPartyReference == null){
				this.calculationAgentPartyReference = new ArrayList<>();
			}
			getIndex(this.calculationAgentPartyReference, _idx, () -> calculationAgentPartyReference.toBuilder());
			this.calculationAgentPartyReference.set(_idx, calculationAgentPartyReference.toBuilder());
			return this;
		}
		public CalculationAgentBuilder addCalculationAgentPartyReference(List<ReferenceWithMetaParty> calculationAgentPartyReferences) {
			if(this.calculationAgentPartyReference == null){
				this.calculationAgentPartyReference = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : calculationAgentPartyReferences) {
				this.calculationAgentPartyReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public CalculationAgentBuilder addCalculationAgentPartyReferenceBuilder(ReferenceWithMetaPartyBuilder calculationAgentPartyReference) {
			if(this.calculationAgentPartyReference == null){
				this.calculationAgentPartyReference = new ArrayList<>();
			}
			this.calculationAgentPartyReference.add(calculationAgentPartyReference);
			return this;
		}
		
		public CalculationAgentBuilder addCalculationAgentPartyReferenceRef(Party.PartyBuilder calculationAgentPartyReference) {
			return addCalculationAgentPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(calculationAgentPartyReference).build());
		}
		
		public CalculationAgentBuilder addCalculationAgentPartyReferenceRef(Party.PartyBuilder calculationAgentPartyReference, int _idx) {
			return addCalculationAgentPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(calculationAgentPartyReference).build(), _idx);
		}
		
		public CalculationAgentBuilder addCalculationAgentPartyReferenceRef(List<Party> calculationAgentPartyReferences) {
			for (Party toAdd : calculationAgentPartyReferences) {
				addCalculationAgentPartyReferenceRef(toAdd);
			}
			return this;
		}
		
		public CalculationAgentBuilder addCalculationAgentPartyReferenceRef(Party calculationAgentPartyReference) {
			return addCalculationAgentPartyReferenceRef(calculationAgentPartyReference.toBuilder());
		}
		
		public CalculationAgentBuilder clearCalculationAgentPartyReference() {
			this.calculationAgentPartyReference = null;
			return this;
		}
		
		public CalculationAgent build() {
			return new CalculationAgent(this);
		}
	
		@Override
		public CalculationAgentBuilder prune() {
			if (calculationAgentBusinessCenter!=null && !calculationAgentBusinessCenter.prune().hasData()) calculationAgentBusinessCenter = null;
			if (calculationAgentPartyReference!=null) calculationAgentPartyReference = calculationAgentPartyReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationAgentBusinessCenter()!=null) return true;
			if (getCalculationAgentParty()!=null) return true;
			if (getCalculationAgentPartyReference()!=null && getCalculationAgentPartyReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("calculationAgentParty"), CalculationAgentPartyEnum.class, calculationAgentParty, this);
			
			processRosetta(path.newSubPath("calculationAgentBusinessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, calculationAgentBusinessCenter);
			processRosetta(path.newSubPath("calculationAgentPartyReference"), processor, ReferenceWithMetaParty.class, calculationAgentPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationAgentBuilder _that = (CalculationAgentBuilder) o;
		
			if (calculationAgentBusinessCenter != null ? !calculationAgentBusinessCenter.equals(_that.calculationAgentBusinessCenter) : _that.calculationAgentBusinessCenter != null) return false;
			if (calculationAgentParty != null ? !calculationAgentParty.equals(_that.calculationAgentParty) : _that.calculationAgentParty != null) return false;
			if (!ListEquals.listEquals(calculationAgentPartyReference, _that.calculationAgentPartyReference)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationAgentBusinessCenter != null ? calculationAgentBusinessCenter.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (calculationAgentParty != null ? calculationAgentParty.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (calculationAgentPartyReference != null ? calculationAgentPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationAgentBuilder {" +
				"calculationAgentBusinessCenter=" + this.calculationAgentBusinessCenter + ", " +
				"calculationAgentParty=" + this.calculationAgentParty + ", " +
				"calculationAgentPartyReference=" + this.calculationAgentPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("calculationAgentParty"), CalculationAgentPartyEnum.class, calculationAgentParty, this);
		
		processRosetta(path.newSubPath("calculationAgentBusinessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, calculationAgentBusinessCenter);
		processRosetta(path.newSubPath("calculationAgentPartyReference"), processor, ReferenceWithMetaParty.class, calculationAgentPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationAgent _that = (CalculationAgent) o;
	
		if (calculationAgentBusinessCenter != null ? !calculationAgentBusinessCenter.equals(_that.calculationAgentBusinessCenter) : _that.calculationAgentBusinessCenter != null) return false;
		if (calculationAgentParty != null ? !calculationAgentParty.equals(_that.calculationAgentParty) : _that.calculationAgentParty != null) return false;
		if (!ListEquals.listEquals(calculationAgentPartyReference, _that.calculationAgentPartyReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationAgentBusinessCenter != null ? calculationAgentBusinessCenter.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (calculationAgentParty != null ? calculationAgentParty.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (calculationAgentPartyReference != null ? calculationAgentPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationAgent {" +
			"calculationAgentBusinessCenter=" + this.calculationAgentBusinessCenter + ", " +
			"calculationAgentParty=" + this.calculationAgentParty + ", " +
			"calculationAgentPartyReference=" + this.calculationAgentPartyReference +
		'}';
	}
}
