package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify how a calculation agent will be determined.
 * 
 * @version 2.5.4
 */
public enum CalculationAgentPartyEnum {
    

	/**
	 * The party that gives notice of exercise. Per 2000 ISDA Definitions, Section 11.1. Parties, paragraph (d).
	 */
	@RosettaSynonym(value = "ExercisingParty", source = "FpML_5_10")
	@RosettaSynonym(value = "ExercisingParty", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExercisingParty", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExercisingParty", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExercisingParty", source = "CME_ClearedConfirm_1_17")
	EXERCISING_PARTY,
	    

	/**
	 * The party that is given notice of exercise. Per 2000 ISDA Definitions, Section 11.1. Parties, paragraph (e).
	 */
	@RosettaSynonym(value = "NonExercisingParty", source = "FpML_5_10")
	@RosettaSynonym(value = "NonExercisingParty", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NonExercisingParty", source = "DTCC_11_0")
	@RosettaSynonym(value = "NonExercisingParty", source = "DTCC_9_0")
	@RosettaSynonym(value = "NonExercisingParty", source = "CME_ClearedConfirm_1_17")
	NON_EXERCISING_PARTY,
	    

	/**
	 * The Calculation Agent is determined by reference to the relevant master agreement.
	 */
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "FpML_5_10")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "CME_ClearedConfirm_1_17")
	AS_SPECIFIED_IN_MASTER_AGREEMENT,
	    

	/**
	 * The Calculation Agent is determined by reference to the relevant standard terms supplement.
	 */
	@RosettaSynonym(value = "AsSpecifiedInStandardTermsSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "AsSpecifiedInStandardTermsSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsSpecifiedInStandardTermsSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsSpecifiedInStandardTermsSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsSpecifiedInStandardTermsSupplement", source = "CME_ClearedConfirm_1_17")
	AS_SPECIFIED_IN_STANDARD_TERMS_SUPPLEMENT,
	    

	/**
	 * Both parties with joined rights to be a calculation agent.
	 */
	@RosettaSynonym(value = "Both", source = "FpML_5_10")
	@RosettaSynonym(value = "Both", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Both", source = "DTCC_11_0")
	@RosettaSynonym(value = "Both", source = "DTCC_9_0")
	@RosettaSynonym(value = "Both", source = "CME_ClearedConfirm_1_17")
	BOTH    
	;

	private final String displayName;
	
	CalculationAgentPartyEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
