package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Step;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CalculationAmount", source="FpML_5_10")
public class CalculationAmount extends Money {
	private final List<Step> step;
	private static CalculationAmountMeta metaData = new CalculationAmountMeta();
	
	CalculationAmount(CalculationAmountBuilder builder) {
		super(builder);
		this.step = ofNullable(builder.getStep()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A schedule of step date and value pairs. On each step date the associated step value becomes effective. A list of steps may be ordered in the document by ascending step date. FpML specifies that an FpML document containing an unordered list of steps is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="step", source="FpML_5_10")
	@RosettaSynonym(value="step", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="step", source="DTCC_11_0")
	@RosettaSynonym(value="step", source="DTCC_9_0")
	@RosettaSynonym(value="step", source="CME_ClearedConfirm_1_17")
	public final List<Step> getStep() {
		return step;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationAmount> metaData() {
		return metaData;
	} 
	
	public CalculationAmountBuilder toBuilder() {
		CalculationAmountBuilder builder = new CalculationAmountBuilder();
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static CalculationAmountBuilder builder() {
		return new CalculationAmountBuilder();
	}

	public static  class CalculationAmountBuilder extends MoneyBuilder {
	
		protected List<Step.StepBuilder> step;
	
		public CalculationAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationAmount> metaData() {
			return metaData;
		} 
	
		
		public List<Step.StepBuilder> getStep() {
			return step;
		}
		
		public Step.StepBuilder getOrCreateStep(int index) {
			if (step==null) {
				this.step = new ArrayList<>();
			}
			return getIndex(step, index, ()->new Step.StepBuilder());
		}
		
	
		public CalculationAmountBuilder addStep(Step step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		public CalculationAmountBuilder addStep(Step step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		public CalculationAmountBuilder addStep(List<Step> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (Step toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		public CalculationAmountBuilder addStepBuilder(Step.StepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		public CalculationAmountBuilder clearStep() {
			this.step = null;
			return this;
		}
		@Override public CalculationAmountBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		@Override public CalculationAmountBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		@Override public CalculationAmountBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public CalculationAmountBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public CalculationAmountBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public CalculationAmount build() {
			return new CalculationAmount(this);
		}
	
		@Override
		public CalculationAmountBuilder prune() {
			super.prune();
			if (step!=null) step = step.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getStep()!=null && getStep().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("step"), processor, Step.class, step);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CalculationAmountBuilder _that = (CalculationAmountBuilder) o;
		
			if (!ListEquals.listEquals(step, _that.step)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (step != null ? step.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationAmountBuilder {" +
				"step=" + this.step +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("step"), processor, Step.class, step);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CalculationAmount _that = (CalculationAmount) o;
	
		if (!ListEquals.listEquals(step, _that.step)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (step != null ? step.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationAmount {" +
			"step=" + this.step +
		'}' + " " + super.toString();
	}
}
