package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.SimmCalculationCurrency;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationCurrencyElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the SIMM Calculation Currency elections by each party to the agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationCurrencyElection extends RosettaModelObject {
	private final List<SimmCalculationCurrency> partyElection;
	private static CalculationCurrencyElectionMeta metaData = new CalculationCurrencyElectionMeta();
	
	CalculationCurrencyElection(CalculationCurrencyElectionBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; SIMM Calculation Currency election.
	 */
	public final List<SimmCalculationCurrency> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationCurrencyElection> metaData() {
		return metaData;
	} 
	
	public CalculationCurrencyElectionBuilder toBuilder() {
		CalculationCurrencyElectionBuilder builder = new CalculationCurrencyElectionBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static CalculationCurrencyElectionBuilder builder() {
		return new CalculationCurrencyElectionBuilder();
	}

	public static  class CalculationCurrencyElectionBuilder extends RosettaModelObjectBuilder {
	
		protected List<SimmCalculationCurrency.SimmCalculationCurrencyBuilder> partyElection;
	
		public CalculationCurrencyElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationCurrencyElection> metaData() {
			return metaData;
		} 
	
		
		public List<SimmCalculationCurrency.SimmCalculationCurrencyBuilder> getPartyElection() {
			return partyElection;
		}
		
		public SimmCalculationCurrency.SimmCalculationCurrencyBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new SimmCalculationCurrency.SimmCalculationCurrencyBuilder());
		}
		
	
		public CalculationCurrencyElectionBuilder addPartyElection(SimmCalculationCurrency partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public CalculationCurrencyElectionBuilder addPartyElection(SimmCalculationCurrency partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public CalculationCurrencyElectionBuilder addPartyElection(List<SimmCalculationCurrency> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (SimmCalculationCurrency toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public CalculationCurrencyElectionBuilder addPartyElectionBuilder(SimmCalculationCurrency.SimmCalculationCurrencyBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public CalculationCurrencyElectionBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public CalculationCurrencyElection build() {
			return new CalculationCurrencyElection(this);
		}
	
		@Override
		public CalculationCurrencyElectionBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, SimmCalculationCurrency.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationCurrencyElectionBuilder _that = (CalculationCurrencyElectionBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationCurrencyElectionBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, SimmCalculationCurrency.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationCurrencyElection _that = (CalculationCurrencyElection) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationCurrencyElection {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
