package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CalculationDateLocationElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationDateLocationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Calculation Date Location election for the respective parties to the legal agreement. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(i): Calculation Date Location. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(i): Calculation Date Location.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationDateLocation extends RosettaModelObject {
	private final List<CalculationDateLocationElection> partyElection;
	private static CalculationDateLocationMeta metaData = new CalculationDateLocationMeta();
	
	CalculationDateLocation(CalculationDateLocationBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; calculation date location election.
	 */
	public final List<CalculationDateLocationElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationDateLocation> metaData() {
		return metaData;
	} 
	
	public CalculationDateLocationBuilder toBuilder() {
		CalculationDateLocationBuilder builder = new CalculationDateLocationBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static CalculationDateLocationBuilder builder() {
		return new CalculationDateLocationBuilder();
	}

	public static  class CalculationDateLocationBuilder extends RosettaModelObjectBuilder {
	
		protected List<CalculationDateLocationElection.CalculationDateLocationElectionBuilder> partyElection;
	
		public CalculationDateLocationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationDateLocation> metaData() {
			return metaData;
		} 
	
		
		public List<CalculationDateLocationElection.CalculationDateLocationElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public CalculationDateLocationElection.CalculationDateLocationElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new CalculationDateLocationElection.CalculationDateLocationElectionBuilder());
		}
		
	
		public CalculationDateLocationBuilder addPartyElection(CalculationDateLocationElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public CalculationDateLocationBuilder addPartyElection(CalculationDateLocationElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public CalculationDateLocationBuilder addPartyElection(List<CalculationDateLocationElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (CalculationDateLocationElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public CalculationDateLocationBuilder addPartyElectionBuilder(CalculationDateLocationElection.CalculationDateLocationElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public CalculationDateLocationBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public CalculationDateLocation build() {
			return new CalculationDateLocation(this);
		}
	
		@Override
		public CalculationDateLocationBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, CalculationDateLocationElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationDateLocationBuilder _that = (CalculationDateLocationBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationDateLocationBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, CalculationDateLocationElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationDateLocation _that = (CalculationDateLocation) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationDateLocation {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
