package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationPeriodBaseMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * The calculation period adjusted start and end dates, which are the baseline arguments needed to compute an interest accrual calculation.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CalculationPeriodBase extends RosettaModelObject implements GlobalKey {
	private final Date adjustedEndDate;
	private final Date adjustedStartDate;
	private final MetaFields meta;
	private static CalculationPeriodBaseMeta metaData = new CalculationPeriodBaseMeta();
	
	CalculationPeriodBase(CalculationPeriodBaseBuilder builder) {
		this.adjustedEndDate = builder.getAdjustedEndDate();
		this.adjustedStartDate = builder.getAdjustedStartDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The calculation period end date, adjusted according to any relevant business day convention.
	 */
	@RosettaSynonym(value="adjustedEndDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedEndDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedEndDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedEndDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedEndDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedEndDate() {
		return adjustedEndDate;
	}
	
	/**
	 * The calculation period start date, adjusted according to any relevant business day convention.
	 */
	@RosettaSynonym(value="adjustedStartDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedStartDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedStartDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedStartDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedStartDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedStartDate() {
		return adjustedStartDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationPeriodBase> metaData() {
		return metaData;
	} 
	
	public CalculationPeriodBaseBuilder toBuilder() {
		CalculationPeriodBaseBuilder builder = new CalculationPeriodBaseBuilder();
		ofNullable(getAdjustedEndDate()).ifPresent(builder::setAdjustedEndDate);
		ofNullable(getAdjustedStartDate()).ifPresent(builder::setAdjustedStartDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static CalculationPeriodBaseBuilder builder() {
		return new CalculationPeriodBaseBuilder();
	}

	public static  class CalculationPeriodBaseBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<CalculationPeriodBaseBuilder> {
	
		protected Date adjustedEndDate;
		protected Date adjustedStartDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public CalculationPeriodBaseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationPeriodBase> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedEndDate() {
			return adjustedEndDate;
		}
		
		
		public Date getAdjustedStartDate() {
			return adjustedStartDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public CalculationPeriodBaseBuilder setAdjustedEndDate(Date adjustedEndDate) {
			this.adjustedEndDate = adjustedEndDate;
			return this;
		}
		public CalculationPeriodBaseBuilder setAdjustedStartDate(Date adjustedStartDate) {
			this.adjustedStartDate = adjustedStartDate;
			return this;
		}
		public CalculationPeriodBaseBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public CalculationPeriodBaseBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public CalculationPeriodBase build() {
			return new CalculationPeriodBase(this);
		}
	
		@Override
		public CalculationPeriodBaseBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedEndDate()!=null) return true;
			if (getAdjustedStartDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedEndDate"), Date.class, adjustedEndDate, this);
			processor.processBasic(path.newSubPath("adjustedStartDate"), Date.class, adjustedStartDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationPeriodBaseBuilder _that = (CalculationPeriodBaseBuilder) o;
		
			if (adjustedEndDate != null ? !adjustedEndDate.equals(_that.adjustedEndDate) : _that.adjustedEndDate != null) return false;
			if (adjustedStartDate != null ? !adjustedStartDate.equals(_that.adjustedStartDate) : _that.adjustedStartDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedEndDate != null ? adjustedEndDate.hashCode() : 0);
			_result = 31 * _result + (adjustedStartDate != null ? adjustedStartDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationPeriodBaseBuilder {" +
				"adjustedEndDate=" + this.adjustedEndDate + ", " +
				"adjustedStartDate=" + this.adjustedStartDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedEndDate"), Date.class, adjustedEndDate, this);
		processor.processBasic(path.newSubPath("adjustedStartDate"), Date.class, adjustedStartDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationPeriodBase _that = (CalculationPeriodBase) o;
	
		if (adjustedEndDate != null ? !adjustedEndDate.equals(_that.adjustedEndDate) : _that.adjustedEndDate != null) return false;
		if (adjustedStartDate != null ? !adjustedStartDate.equals(_that.adjustedStartDate) : _that.adjustedStartDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedEndDate != null ? adjustedEndDate.hashCode() : 0);
		_result = 31 * _result + (adjustedStartDate != null ? adjustedStartDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationPeriodBase {" +
			"adjustedEndDate=" + this.adjustedEndDate + ", " +
			"adjustedStartDate=" + this.adjustedStartDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
