package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.Boolean;
import java.lang.Integer;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalculationPeriodDataMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CalculationPeriodData extends RosettaModelObject {
	private final Integer daysInLeapYearPeriod;
	private final Integer daysInPeriod;
	private final Date endDate;
	private final Boolean isFirstPeriod;
	private final Boolean isLastPeriod;
	private final Date startDate;
	private static CalculationPeriodDataMeta metaData = new CalculationPeriodDataMeta();
	
	CalculationPeriodData(CalculationPeriodDataBuilder builder) {
		this.daysInLeapYearPeriod = builder.getDaysInLeapYearPeriod();
		this.daysInPeriod = builder.getDaysInPeriod();
		this.endDate = builder.getEndDate();
		this.isFirstPeriod = builder.getIsFirstPeriod();
		this.isLastPeriod = builder.getIsLastPeriod();
		this.startDate = builder.getStartDate();
	}
	
	public final Integer getDaysInLeapYearPeriod() {
		return daysInLeapYearPeriod;
	}
	
	public final Integer getDaysInPeriod() {
		return daysInPeriod;
	}
	
	public final Date getEndDate() {
		return endDate;
	}
	
	public final Boolean getIsFirstPeriod() {
		return isFirstPeriod;
	}
	
	public final Boolean getIsLastPeriod() {
		return isLastPeriod;
	}
	
	public final Date getStartDate() {
		return startDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalculationPeriodData> metaData() {
		return metaData;
	} 
	
	public CalculationPeriodDataBuilder toBuilder() {
		CalculationPeriodDataBuilder builder = new CalculationPeriodDataBuilder();
		ofNullable(getDaysInLeapYearPeriod()).ifPresent(builder::setDaysInLeapYearPeriod);
		ofNullable(getDaysInPeriod()).ifPresent(builder::setDaysInPeriod);
		ofNullable(getEndDate()).ifPresent(builder::setEndDate);
		ofNullable(getIsFirstPeriod()).ifPresent(builder::setIsFirstPeriod);
		ofNullable(getIsLastPeriod()).ifPresent(builder::setIsLastPeriod);
		ofNullable(getStartDate()).ifPresent(builder::setStartDate);
		return builder;
	}

	public static CalculationPeriodDataBuilder builder() {
		return new CalculationPeriodDataBuilder();
	}

	public static  class CalculationPeriodDataBuilder extends RosettaModelObjectBuilder {
	
		protected Integer daysInLeapYearPeriod;
		protected Integer daysInPeriod;
		protected Date endDate;
		protected Boolean isFirstPeriod;
		protected Boolean isLastPeriod;
		protected Date startDate;
	
		public CalculationPeriodDataBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalculationPeriodData> metaData() {
			return metaData;
		} 
	
		
		public Integer getDaysInLeapYearPeriod() {
			return daysInLeapYearPeriod;
		}
		
		
		public Integer getDaysInPeriod() {
			return daysInPeriod;
		}
		
		
		public Date getEndDate() {
			return endDate;
		}
		
		
		public Boolean getIsFirstPeriod() {
			return isFirstPeriod;
		}
		
		
		public Boolean getIsLastPeriod() {
			return isLastPeriod;
		}
		
		
		public Date getStartDate() {
			return startDate;
		}
		
	
		public CalculationPeriodDataBuilder setDaysInLeapYearPeriod(Integer daysInLeapYearPeriod) {
			this.daysInLeapYearPeriod = daysInLeapYearPeriod;
			return this;
		}
		public CalculationPeriodDataBuilder setDaysInPeriod(Integer daysInPeriod) {
			this.daysInPeriod = daysInPeriod;
			return this;
		}
		public CalculationPeriodDataBuilder setEndDate(Date endDate) {
			this.endDate = endDate;
			return this;
		}
		public CalculationPeriodDataBuilder setIsFirstPeriod(Boolean isFirstPeriod) {
			this.isFirstPeriod = isFirstPeriod;
			return this;
		}
		public CalculationPeriodDataBuilder setIsLastPeriod(Boolean isLastPeriod) {
			this.isLastPeriod = isLastPeriod;
			return this;
		}
		public CalculationPeriodDataBuilder setStartDate(Date startDate) {
			this.startDate = startDate;
			return this;
		}
		
		public CalculationPeriodData build() {
			return new CalculationPeriodData(this);
		}
	
		@Override
		public CalculationPeriodDataBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDaysInLeapYearPeriod()!=null) return true;
			if (getDaysInPeriod()!=null) return true;
			if (getEndDate()!=null) return true;
			if (getIsFirstPeriod()!=null) return true;
			if (getIsLastPeriod()!=null) return true;
			if (getStartDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("daysInLeapYearPeriod"), Integer.class, daysInLeapYearPeriod, this);
			processor.processBasic(path.newSubPath("daysInPeriod"), Integer.class, daysInPeriod, this);
			processor.processBasic(path.newSubPath("endDate"), Date.class, endDate, this);
			processor.processBasic(path.newSubPath("isFirstPeriod"), Boolean.class, isFirstPeriod, this);
			processor.processBasic(path.newSubPath("isLastPeriod"), Boolean.class, isLastPeriod, this);
			processor.processBasic(path.newSubPath("startDate"), Date.class, startDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalculationPeriodDataBuilder _that = (CalculationPeriodDataBuilder) o;
		
			if (daysInLeapYearPeriod != null ? !daysInLeapYearPeriod.equals(_that.daysInLeapYearPeriod) : _that.daysInLeapYearPeriod != null) return false;
			if (daysInPeriod != null ? !daysInPeriod.equals(_that.daysInPeriod) : _that.daysInPeriod != null) return false;
			if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
			if (isFirstPeriod != null ? !isFirstPeriod.equals(_that.isFirstPeriod) : _that.isFirstPeriod != null) return false;
			if (isLastPeriod != null ? !isLastPeriod.equals(_that.isLastPeriod) : _that.isLastPeriod != null) return false;
			if (startDate != null ? !startDate.equals(_that.startDate) : _that.startDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (daysInLeapYearPeriod != null ? daysInLeapYearPeriod.hashCode() : 0);
			_result = 31 * _result + (daysInPeriod != null ? daysInPeriod.hashCode() : 0);
			_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
			_result = 31 * _result + (isFirstPeriod != null ? isFirstPeriod.hashCode() : 0);
			_result = 31 * _result + (isLastPeriod != null ? isLastPeriod.hashCode() : 0);
			_result = 31 * _result + (startDate != null ? startDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalculationPeriodDataBuilder {" +
				"daysInLeapYearPeriod=" + this.daysInLeapYearPeriod + ", " +
				"daysInPeriod=" + this.daysInPeriod + ", " +
				"endDate=" + this.endDate + ", " +
				"isFirstPeriod=" + this.isFirstPeriod + ", " +
				"isLastPeriod=" + this.isLastPeriod + ", " +
				"startDate=" + this.startDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("daysInLeapYearPeriod"), Integer.class, daysInLeapYearPeriod, this);
		processor.processBasic(path.newSubPath("daysInPeriod"), Integer.class, daysInPeriod, this);
		processor.processBasic(path.newSubPath("endDate"), Date.class, endDate, this);
		processor.processBasic(path.newSubPath("isFirstPeriod"), Boolean.class, isFirstPeriod, this);
		processor.processBasic(path.newSubPath("isLastPeriod"), Boolean.class, isLastPeriod, this);
		processor.processBasic(path.newSubPath("startDate"), Date.class, startDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalculationPeriodData _that = (CalculationPeriodData) o;
	
		if (daysInLeapYearPeriod != null ? !daysInLeapYearPeriod.equals(_that.daysInLeapYearPeriod) : _that.daysInLeapYearPeriod != null) return false;
		if (daysInPeriod != null ? !daysInPeriod.equals(_that.daysInPeriod) : _that.daysInPeriod != null) return false;
		if (endDate != null ? !endDate.equals(_that.endDate) : _that.endDate != null) return false;
		if (isFirstPeriod != null ? !isFirstPeriod.equals(_that.isFirstPeriod) : _that.isFirstPeriod != null) return false;
		if (isLastPeriod != null ? !isLastPeriod.equals(_that.isLastPeriod) : _that.isLastPeriod != null) return false;
		if (startDate != null ? !startDate.equals(_that.startDate) : _that.startDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (daysInLeapYearPeriod != null ? daysInLeapYearPeriod.hashCode() : 0);
		_result = 31 * _result + (daysInPeriod != null ? daysInPeriod.hashCode() : 0);
		_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
		_result = 31 * _result + (isFirstPeriod != null ? isFirstPeriod.hashCode() : 0);
		_result = 31 * _result + (isLastPeriod != null ? isLastPeriod.hashCode() : 0);
		_result = 31 * _result + (startDate != null ? startDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalculationPeriodData {" +
			"daysInLeapYearPeriod=" + this.daysInLeapYearPeriod + ", " +
			"daysInPeriod=" + this.daysInPeriod + ", " +
			"endDate=" + this.endDate + ", " +
			"isFirstPeriod=" + this.isFirstPeriod + ", " +
			"isLastPeriod=" + this.isLastPeriod + ", " +
			"startDate=" + this.startDate +
		'}';
	}
}
