package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CalendarSpreadMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A type for defining a calendar spread feature.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CalendarSpread", source="FpML_5_10")
public class CalendarSpread extends RosettaModelObject {
	private final AdjustableOrRelativeDate expirationDateTwo;
	private static CalendarSpreadMeta metaData = new CalendarSpreadMeta();
	
	CalendarSpread(CalendarSpreadBuilder builder) {
		this.expirationDateTwo = ofNullable(builder.getExpirationDateTwo()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="expirationDateTwo", source="FpML_5_10")
	@RosettaSynonym(value="expirationDateTwo", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="expirationDateTwo", source="DTCC_11_0")
	@RosettaSynonym(value="expirationDateTwo", source="DTCC_9_0")
	@RosettaSynonym(value="expirationDateTwo", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getExpirationDateTwo() {
		return expirationDateTwo;
	}
	
	
	@Override
	public RosettaMetaData<? extends CalendarSpread> metaData() {
		return metaData;
	} 
	
	public CalendarSpreadBuilder toBuilder() {
		CalendarSpreadBuilder builder = new CalendarSpreadBuilder();
		ofNullable(getExpirationDateTwo()).ifPresent(builder::setExpirationDateTwo);
		return builder;
	}

	public static CalendarSpreadBuilder builder() {
		return new CalendarSpreadBuilder();
	}

	public static  class CalendarSpreadBuilder extends RosettaModelObjectBuilder {
	
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDateTwo;
	
		public CalendarSpreadBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CalendarSpread> metaData() {
			return metaData;
		} 
	
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getExpirationDateTwo() {
			return expirationDateTwo;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreateExpirationDateTwo() {
			if (expirationDateTwo!=null) {
				return expirationDateTwo;
			}
			else return expirationDateTwo = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
	
		public CalendarSpreadBuilder setExpirationDateTwo(AdjustableOrRelativeDate expirationDateTwo) {
			this.expirationDateTwo = expirationDateTwo.toBuilder();
			return this;
		}
		public CalendarSpreadBuilder setExpirationDateTwoBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder expirationDateTwo) {
			this.expirationDateTwo = expirationDateTwo;
			return this;
		}
		
		public CalendarSpread build() {
			return new CalendarSpread(this);
		}
	
		@Override
		public CalendarSpreadBuilder prune() {
			if (expirationDateTwo!=null && !expirationDateTwo.prune().hasData()) expirationDateTwo = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExpirationDateTwo()!=null && getExpirationDateTwo().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("expirationDateTwo"), processor, AdjustableOrRelativeDate.class, expirationDateTwo);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CalendarSpreadBuilder _that = (CalendarSpreadBuilder) o;
		
			if (expirationDateTwo != null ? !expirationDateTwo.equals(_that.expirationDateTwo) : _that.expirationDateTwo != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (expirationDateTwo != null ? expirationDateTwo.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CalendarSpreadBuilder {" +
				"expirationDateTwo=" + this.expirationDateTwo +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("expirationDateTwo"), processor, AdjustableOrRelativeDate.class, expirationDateTwo);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CalendarSpread _that = (CalendarSpread) o;
	
		if (expirationDateTwo != null ? !expirationDateTwo.equals(_that.expirationDateTwo) : _that.expirationDateTwo != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (expirationDateTwo != null ? expirationDateTwo.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CalendarSpread {" +
			"expirationDateTwo=" + this.expirationDateTwo +
		'}';
	}
}
