package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CancellationEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CancelableProvisionAdjustedDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to define the adjusted dates for a cancelable provision on a swap transaction.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CancelableProvisionAdjustedDates", source="FpML_5_10")
public class CancelableProvisionAdjustedDates extends RosettaModelObject {
	private final List<CancellationEvent> cancellationEvent;
	private static CancelableProvisionAdjustedDatesMeta metaData = new CancelableProvisionAdjustedDatesMeta();
	
	CancelableProvisionAdjustedDates(CancelableProvisionAdjustedDatesBuilder builder) {
		this.cancellationEvent = ofNullable(builder.getCancellationEvent()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The adjusted dates for an individual cancellation date.
	 */
	@RosettaSynonym(value="cancellationEvent", source="FpML_5_10")
	@RosettaSynonym(value="cancellationEvent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="cancellationEvent", source="DTCC_11_0")
	@RosettaSynonym(value="cancellationEvent", source="DTCC_9_0")
	@RosettaSynonym(value="cancellationEvent", source="CME_ClearedConfirm_1_17")
	public final List<CancellationEvent> getCancellationEvent() {
		return cancellationEvent;
	}
	
	
	@Override
	public RosettaMetaData<? extends CancelableProvisionAdjustedDates> metaData() {
		return metaData;
	} 
	
	public CancelableProvisionAdjustedDatesBuilder toBuilder() {
		CancelableProvisionAdjustedDatesBuilder builder = new CancelableProvisionAdjustedDatesBuilder();
		ofNullable(getCancellationEvent()).ifPresent(cancellationEvent -> cancellationEvent.forEach(builder::addCancellationEvent));
		return builder;
	}

	public static CancelableProvisionAdjustedDatesBuilder builder() {
		return new CancelableProvisionAdjustedDatesBuilder();
	}

	public static  class CancelableProvisionAdjustedDatesBuilder extends RosettaModelObjectBuilder {
	
		protected List<CancellationEvent.CancellationEventBuilder> cancellationEvent;
	
		public CancelableProvisionAdjustedDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CancelableProvisionAdjustedDates> metaData() {
			return metaData;
		} 
	
		
		public List<CancellationEvent.CancellationEventBuilder> getCancellationEvent() {
			return cancellationEvent;
		}
		
		public CancellationEvent.CancellationEventBuilder getOrCreateCancellationEvent(int index) {
			if (cancellationEvent==null) {
				this.cancellationEvent = new ArrayList<>();
			}
			return getIndex(cancellationEvent, index, ()->new CancellationEvent.CancellationEventBuilder());
		}
		
	
		public CancelableProvisionAdjustedDatesBuilder addCancellationEvent(CancellationEvent cancellationEvent) {
			if(this.cancellationEvent == null){
				this.cancellationEvent = new ArrayList<>();
			}
			this.cancellationEvent.add(cancellationEvent.toBuilder());
			return this;
		}
		public CancelableProvisionAdjustedDatesBuilder addCancellationEvent(CancellationEvent cancellationEvent, int _idx) {
			if(this.cancellationEvent == null){
				this.cancellationEvent = new ArrayList<>();
			}
			getIndex(this.cancellationEvent, _idx, () -> cancellationEvent.toBuilder());
			this.cancellationEvent.set(_idx, cancellationEvent.toBuilder());
			return this;
		}
		public CancelableProvisionAdjustedDatesBuilder addCancellationEvent(List<CancellationEvent> cancellationEvents) {
			if(this.cancellationEvent == null){
				this.cancellationEvent = new ArrayList<>();
			}
			for (CancellationEvent toAdd : cancellationEvents) {
				this.cancellationEvent.add(toAdd.toBuilder());
			}
			return this;
		}
		public CancelableProvisionAdjustedDatesBuilder addCancellationEventBuilder(CancellationEvent.CancellationEventBuilder cancellationEvent) {
			if(this.cancellationEvent == null){
				this.cancellationEvent = new ArrayList<>();
			}
			this.cancellationEvent.add(cancellationEvent);
			return this;
		}
		
		public CancelableProvisionAdjustedDatesBuilder clearCancellationEvent() {
			this.cancellationEvent = null;
			return this;
		}
		
		public CancelableProvisionAdjustedDates build() {
			return new CancelableProvisionAdjustedDates(this);
		}
	
		@Override
		public CancelableProvisionAdjustedDatesBuilder prune() {
			if (cancellationEvent!=null) cancellationEvent = cancellationEvent.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCancellationEvent()!=null && getCancellationEvent().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cancellationEvent"), processor, CancellationEvent.class, cancellationEvent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CancelableProvisionAdjustedDatesBuilder _that = (CancelableProvisionAdjustedDatesBuilder) o;
		
			if (!ListEquals.listEquals(cancellationEvent, _that.cancellationEvent)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cancellationEvent != null ? cancellationEvent.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CancelableProvisionAdjustedDatesBuilder {" +
				"cancellationEvent=" + this.cancellationEvent +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cancellationEvent"), processor, CancellationEvent.class, cancellationEvent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CancelableProvisionAdjustedDates _that = (CancelableProvisionAdjustedDates) o;
	
		if (!ListEquals.listEquals(cancellationEvent, _that.cancellationEvent)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cancellationEvent != null ? cancellationEvent.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CancelableProvisionAdjustedDates {" +
			"cancellationEvent=" + this.cancellationEvent +
		'}';
	}
}
