package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ReferenceBank;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CashSettlementReferenceBanksMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the list of reference institutions polled for relevant rates or prices when determining the cash settlement amount for a product where cash settlement is applicable.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CashSettlementReferenceBanks", source="FpML_5_10")
public class CashSettlementReferenceBanks extends RosettaModelObject {
	private final List<ReferenceBank> referenceBank;
	private static CashSettlementReferenceBanksMeta metaData = new CashSettlementReferenceBanksMeta();
	
	CashSettlementReferenceBanks(CashSettlementReferenceBanksBuilder builder) {
		this.referenceBank = ofNullable(builder.getReferenceBank()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * An institution (party) identified by means of a coding scheme and an optional name.
	 */
	@RosettaSynonym(value="referenceBank", source="FpML_5_10")
	@RosettaSynonym(value="referenceBank", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceBank", source="DTCC_11_0")
	@RosettaSynonym(value="referenceBank", source="DTCC_9_0")
	@RosettaSynonym(value="referenceBank", source="CME_ClearedConfirm_1_17")
	public final List<ReferenceBank> getReferenceBank() {
		return referenceBank;
	}
	
	
	@Override
	public RosettaMetaData<? extends CashSettlementReferenceBanks> metaData() {
		return metaData;
	} 
	
	public CashSettlementReferenceBanksBuilder toBuilder() {
		CashSettlementReferenceBanksBuilder builder = new CashSettlementReferenceBanksBuilder();
		ofNullable(getReferenceBank()).ifPresent(referenceBank -> referenceBank.forEach(builder::addReferenceBank));
		return builder;
	}

	public static CashSettlementReferenceBanksBuilder builder() {
		return new CashSettlementReferenceBanksBuilder();
	}

	public static  class CashSettlementReferenceBanksBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceBank.ReferenceBankBuilder> referenceBank;
	
		public CashSettlementReferenceBanksBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CashSettlementReferenceBanks> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceBank.ReferenceBankBuilder> getReferenceBank() {
			return referenceBank;
		}
		
		public ReferenceBank.ReferenceBankBuilder getOrCreateReferenceBank(int index) {
			if (referenceBank==null) {
				this.referenceBank = new ArrayList<>();
			}
			return getIndex(referenceBank, index, ()->new ReferenceBank.ReferenceBankBuilder());
		}
		
	
		public CashSettlementReferenceBanksBuilder addReferenceBank(ReferenceBank referenceBank) {
			if(this.referenceBank == null){
				this.referenceBank = new ArrayList<>();
			}
			this.referenceBank.add(referenceBank.toBuilder());
			return this;
		}
		public CashSettlementReferenceBanksBuilder addReferenceBank(ReferenceBank referenceBank, int _idx) {
			if(this.referenceBank == null){
				this.referenceBank = new ArrayList<>();
			}
			getIndex(this.referenceBank, _idx, () -> referenceBank.toBuilder());
			this.referenceBank.set(_idx, referenceBank.toBuilder());
			return this;
		}
		public CashSettlementReferenceBanksBuilder addReferenceBank(List<ReferenceBank> referenceBanks) {
			if(this.referenceBank == null){
				this.referenceBank = new ArrayList<>();
			}
			for (ReferenceBank toAdd : referenceBanks) {
				this.referenceBank.add(toAdd.toBuilder());
			}
			return this;
		}
		public CashSettlementReferenceBanksBuilder addReferenceBankBuilder(ReferenceBank.ReferenceBankBuilder referenceBank) {
			if(this.referenceBank == null){
				this.referenceBank = new ArrayList<>();
			}
			this.referenceBank.add(referenceBank);
			return this;
		}
		
		public CashSettlementReferenceBanksBuilder clearReferenceBank() {
			this.referenceBank = null;
			return this;
		}
		
		public CashSettlementReferenceBanks build() {
			return new CashSettlementReferenceBanks(this);
		}
	
		@Override
		public CashSettlementReferenceBanksBuilder prune() {
			if (referenceBank!=null) referenceBank = referenceBank.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getReferenceBank()!=null && getReferenceBank().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("referenceBank"), processor, ReferenceBank.class, referenceBank);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CashSettlementReferenceBanksBuilder _that = (CashSettlementReferenceBanksBuilder) o;
		
			if (!ListEquals.listEquals(referenceBank, _that.referenceBank)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (referenceBank != null ? referenceBank.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CashSettlementReferenceBanksBuilder {" +
				"referenceBank=" + this.referenceBank +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("referenceBank"), processor, ReferenceBank.class, referenceBank);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CashSettlementReferenceBanks _that = (CashSettlementReferenceBanks) o;
	
		if (!ListEquals.listEquals(referenceBank, _that.referenceBank)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (referenceBank != null ? referenceBank.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CashSettlementReferenceBanks {" +
			"referenceBank=" + this.referenceBank +
		'}';
	}
}
