package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of organisation involved in the transaction.
 * 
 * @version 2.5.4
 */
public enum CategoryEnum {
    

	/**
	 * The trade or trade report represents the information from the perspective of the sender of the report, typically a clearing member firm or dealer (acting as an agent).
	 */
	@RosettaSynonym(value = "Agent", source = "FpML_5_10")
	@RosettaSynonym(value = "Agent", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Agent", source = "DTCC_11_0")
	@RosettaSynonym(value = "Agent", source = "DTCC_9_0")
	AGENT,
	    

	/**
	 * The trade or trade report represents the information from the perspective of the counterparty of the sender of the report, which is typically a clearing member firm or dealer.
	 */
	@RosettaSynonym(value = "Counterparty", source = "FpML_5_10")
	@RosettaSynonym(value = "Counterparty", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Counterparty", source = "DTCC_11_0")
	@RosettaSynonym(value = "Counterparty", source = "DTCC_9_0")
	COUNTERPARTY,
	    

	/**
	 * The trade or trade report represents the information from the perspective of a client opposite the sender of the report, which is typically a clearing member firm or dealer.
	 */
	@RosettaSynonym(value = "Customer", source = "FpML_5_10")
	@RosettaSynonym(value = "Customer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Customer", source = "DTCC_11_0")
	@RosettaSynonym(value = "Customer", source = "DTCC_9_0")
	@RosettaSynonym(value = "CUST", source = "CME_ClearedConfirm_1_17")
	CUSTOMER,
	    

	/**
	 * The trade or trade report represents the information from the perspective of the sender of the report, typically a clearing member firm or dealer (acting as a principal).
	 */
	@RosettaSynonym(value = "Principal", source = "FpML_5_10")
	@RosettaSynonym(value = "Principal", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Principal", source = "DTCC_11_0")
	@RosettaSynonym(value = "Principal", source = "DTCC_9_0")
	@RosettaSynonym(value = "HOUS", source = "CME_ClearedConfirm_1_17")
	PRINCIPAL    
	;

	private final String displayName;
	
	CategoryEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
