package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify what led to the contract or execution closure.
 * 
 * @version 2.5.4
 */
public enum ClosedStateEnum {
    

	/**
	 * The execution or contract has been allocated.
	 */
	@RosettaSynonym(value = "allocated", source = "Rosetta_Workbench")
	ALLOCATED,
	    

	/**
	 * The execution or contract has been cancelled.
	 */
	@RosettaSynonym(value = "cancelled", source = "Rosetta_Workbench")
	CANCELLED,
	    

	/**
	 * The (option) contract has been exercised.
	 */
	@RosettaSynonym(value = "exercised", source = "Rosetta_Workbench")
	EXERCISED,
	    

	/**
	 * The (option) contract has expired without being exercised.
	 */
	EXPIRED,
	    

	/**
	 * The contract has reached its contractual termination date.
	 */
	MATURED,
	    

	/**
	 * The contract has been novated. This state applies to the stepped-out contract component of the novation event.
	 */
	@RosettaSynonym(value = "novated", source = "Rosetta_Workbench")
	NOVATED,
	    

	/**
	 * The contract has been subject of an early termination event.
	 */
	@RosettaSynonym(value = "terminated", source = "Rosetta_Workbench")
	TERMINATED    
	;

	private final String displayName;
	
	ClosedStateEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
