package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The ISDA Collateral Assets Definitions as published by ISDA in the 2003 ISDA Collateral Asset Definitions.
 * 
 * @version 2.5.4
 */
public enum CollateralAssetDefinitionsEnum {
    

	/**
	 * Australian Dollar (AUD) Cash.
	 */
	@RosettaSynonym(value = "AU-CASH", source = "FpML_5_10")
	AU_CASH,
	    

	/**
	 * Australian Government Securities Capital-Indexed Bonds.
	 */
	@RosettaSynonym(value = "AU-CIB", source = "FpML_5_10")
	AU_CIB("AU-CIB"),
	

	/**
	 * Australian Semi-Government Securities Fixed Interest Bonds.
	 */
	@RosettaSynonym(value = "AU-FIB", source = "FpML_5_10")
	AU_FIB("AU-FIB"),
	

	/**
	 * Australian Government Securities Fixed Rate Bonds.
	 */
	@RosettaSynonym(value = "AU-FRB", source = "FpML_5_10")
	AU_FRB("AU-FRB"),
	

	/**
	 * Australian Semi-Government Securities Index Linked Bonds.
	 */
	@RosettaSynonym(value = "AU-ILB", source = "FpML_5_10")
	AU_ILB("AU-ILB"),
	

	/**
	 * Australian Government Securities Treasury Notes.
	 */
	@RosettaSynonym(value = "AU-NOTE", source = "FpML_5_10")
	AU_NOTE("AU-NOTE"),
	

	/**
	 * Australian Semi-Government Securities Treasury Notes.
	 */
	@RosettaSynonym(value = "AU-STATENOTE", source = "FpML_5_10")
	AU_STATENOTE("AU-STATENOTE"),
	

	/**
	 * Australian Government Securities Treasury Adjustable Rate Bonds.
	 */
	@RosettaSynonym(value = "AU-TAB", source = "FpML_5_10")
	AU_TAB("AU-TAB"),
	

	/**
	 * BEL20 Equity Securities.
	 */
	@RosettaSynonym(value = "BE-BEL20", source = "FpML_5_10")
	BE_BEL20("BE-BEL20"),
	

	/**
	 * Belgian Treasury Certificates.
	 */
	@RosettaSynonym(value = "BE-CERT", source = "FpML_5_10")
	BE_CERT("BE-CERT"),
	

	/**
	 * Belgian Linear Obligations.
	 */
	@RosettaSynonym(value = "BE-LINEAR", source = "FpML_5_10")
	BE_LINEAR("BE-LINEAR"),
	

	/**
	 * Belgian Treasury notes.
	 */
	@RosettaSynonym(value = "BE-NOTE", source = "FpML_5_10")
	BE_NOTE("BE-NOTE"),
	

	/**
	 * Public sector issues guaranteed by Regional Authorities.
	 */
	@RosettaSynonym(value = "BE-REGIONGT", source = "FpML_5_10")
	BE_REGIONGT("BE-REGIONGT"),
	

	/**
	 * Public sector issues guaranteed by the Belgian State.
	 */
	@RosettaSynonym(value = "BE-STATEGT", source = "FpML_5_10")
	BE_STATEGT("BE-STATEGT"),
	

	/**
	 * Belgian State Loans.
	 */
	@RosettaSynonym(value = "BE-STATELOAN", source = "FpML_5_10")
	BE_STATELOAN("BE-STATELOAN"),
	

	/**
	 * Canada Bonds.
	 */
	@RosettaSynonym(value = "CA-BOND", source = "FpML_5_10")
	CA_BOND("CA-BOND"),
	

	/**
	 * Canadian Dollar (CAD) Cash.
	 */
	@RosettaSynonym(value = "CA-CASH", source = "FpML_5_10")
	CA_CASH("CA-CASH"),
	

	/**
	 * Government of Canada Real Return Bonds.
	 */
	@RosettaSynonym(value = "CA-RRB", source = "FpML_5_10")
	CA_RRB("CA-RRB"),
	

	/**
	 * Government of Canada Treasury Bills.
	 */
	@RosettaSynonym(value = "CA-TBILL", source = "FpML_5_10")
	CA_TBILL("CA-TBILL"),
	

	/**
	 * Public Authority Bond.
	 */
	@RosettaSynonym(value = "CH-CANTON", source = "FpML_5_10")
	CH_CANTON("CH-CANTON"),
	

	/**
	 * Swiss Franc (CHF) Cash.
	 */
	@RosettaSynonym(value = "CH-CASH", source = "FpML_5_10")
	CH_CASH("CH-CASH"),
	

	/**
	 * Federal Bond.
	 */
	@RosettaSynonym(value = "CH-FEDBOND", source = "FpML_5_10")
	CH_FEDBOND("CH-FEDBOND"),
	

	/**
	 * Unverzinsliche Schatzanweisungen (Bills).
	 */
	@RosettaSynonym(value = "DE-BILL", source = "FpML_5_10")
	DE_BILL("DE-BILL"),
	

	/**
	 * Bundesanleihen (Bonds).
	 */
	@RosettaSynonym(value = "DE-BOND", source = "FpML_5_10")
	DE_BOND("DE-BOND"),
	

	/**
	 * Negotiable Debt Obligations issued by or taken over and since serviced and managed by the Erblasttilgungsfond (Redemption Fund for Inherited Liabilities) backed by Federal Republic of Germany, including but not limited to former issues of the Treuhandanstalt, the Bundesbahn, the Bundespost, the Economic Recovery Program (ERP), the privatised Federal Railway (Bahn AG), the telecommunications element of the Federal Post Office (Telekom) and the German Unity Fund.
	 */
	@RosettaSynonym(value = "DE-ERBLAST", source = "FpML_5_10")
	DE_ERBLAST("DE-ERBLAST"),
	

	/**
	 * Kommunalschuldverschreib	ungen (Municipal Bonds).
	 */
	@RosettaSynonym(value = "DE-MUNI", source = "FpML_5_10")
	DE_MUNI("DE-MUNI"),
	

	/**
	 * Bundesschatzanweisungen (Notes).
	 */
	@RosettaSynonym(value = "DE-NOTE2", source = "FpML_5_10")
	DE_NOTE2("DE-NOTE2"),
	

	/**
	 * Bundesobligationen (Notes).
	 */
	@RosettaSynonym(value = "DE-NOTE5.5", source = "FpML_5_10")
	DE_NOTE5_5("DE-NOTE5.5"),
	

	/**
	 * Hypothekenpfandbriefe (Mortgage Bonds).
	 */
	@RosettaSynonym(value = "DE-PFAND", source = "FpML_5_10")
	DE_PFAND("DE-PFAND"),
	

	/**
	 * Skatkammerbeviser (Treasury Bills).
	 */
	@RosettaSynonym(value = "DK-BILL", source = "FpML_5_10")
	DK_BILL("DK-BILL"),
	

	/**
	 * BoligX obligationer.
	 */
	@RosettaSynonym(value = "DK-BOLIGX", source = "FpML_5_10")
	DK_BOLIGX("DK-BOLIGX"),
	

	/**
	 * Statsobligationer (Government Bonds).
	 */
	@RosettaSynonym(value = "DK-BOND", source = "FpML_5_10")
	DK_BOND("DK-BOND"),
	

	/**
	 * Callable Mortgage Bonds.
	 */
	@RosettaSynonym(value = "DK-CALLMORT", source = "FpML_5_10")
	DK_CALLMORT("DK-CALLMORT"),
	

	/**
	 * Danish Krone (DKK) Cash.
	 */
	@RosettaSynonym(value = "DK-CASH", source = "FpML_5_10")
	DK_CASH("DK-CASH"),
	

	/**
	 * KFX Equity Securities.
	 */
	@RosettaSynonym(value = "DK-KFX", source = "FpML_5_10")
	DK_KFX("DK-KFX"),
	

	/**
	 * Non-callable Mortgage Bonds.
	 */
	@RosettaSynonym(value = "DK-MORT", source = "FpML_5_10")
	DK_MORT("DK-MORT"),
	

	/**
	 * Statsgaeldsbeviser (Treasury Notes).
	 */
	@RosettaSynonym(value = "DK-NOTE", source = "FpML_5_10")
	DK_NOTE("DK-NOTE"),
	

	/**
	 * Treasury Bills - Letras del Tesoro.
	 */
	@RosettaSynonym(value = "ES-BILL", source = "FpML_5_10")
	ES_BILL("ES-BILL"),
	

	/**
	 * Public Government Debt.
	 */
	@RosettaSynonym(value = "ES-BOND", source = "FpML_5_10")
	ES_BOND("ES-BOND"),
	

	/**
	 * Cedulas.
	 */
	@RosettaSynonym(value = "ES-CEDULAS", source = "FpML_5_10")
	ES_CEDULAS("ES-CEDULAS"),
	

	/**
	 * Corporate Bonds.
	 */
	@RosettaSynonym(value = "ES-CORP", source = "FpML_5_10")
	ES_CORP("ES-CORP"),
	

	/**
	 * Equity securities issued by a Spanish company, and listed as an IBEX 35 constituent company as reported by the Sociedad de Bolsas, each share representing the minimum unit of participation of a shareholder in the stock capital of the company.
	 */
	@RosettaSynonym(value = "ES-EQUITY", source = "FpML_5_10")
	ES_EQUITY("ES-EQUITY"),
	

	/**
	 * Euro (EUR) Cash.
	 */
	@RosettaSynonym(value = "EU-CASH", source = "FpML_5_10")
	EU_CASH("EU-CASH"),
	

	/**
	 * FTSE Euro 100 Index Equity Securities.
	 */
	@RosettaSynonym(value = "EU-EURO100", source = "FpML_5_10")
	EU_EURO100("EU-EURO100"),
	

	/**
	 * FTSE Eurotop 300 Index Equity Securities.
	 */
	@RosettaSynonym(value = "EU-EUROTOP300", source = "FpML_5_10")
	EU_EUROTOP300("EU-EUROTOP300"),
	

	/**
	 * EuroSTOXX 50 Index Equity Securities.
	 */
	@RosettaSynonym(value = "EU-STOXX50", source = "FpML_5_10")
	EU_STOXX50("EU-STOXX50"),
	

	/**
	 * STOXX 600 Index Equity Securities.
	 */
	@RosettaSynonym(value = "EU-STOXX600", source = "FpML_5_10")
	EU_STOXX600("EU-STOXX600"),
	

	/**
	 * Treasury bills.
	 */
	@RosettaSynonym(value = "FI-BILL", source = "FpML_5_10")
	FI_BILL("FI-BILL"),
	

	/**
	 * Serial bonds (Finnish Government Bond).
	 */
	@RosettaSynonym(value = "FI-BOND", source = "FpML_5_10")
	FI_BOND("FI-BOND"),
	

	/**
	 * HEX Equity Securities.
	 */
	@RosettaSynonym(value = "FI-HEX", source = "FpML_5_10")
	FI_HEX("FI-HEX"),
	

	/**
	 * Commercial Paper: (Billet de Trésorerie).
	 */
	@RosettaSynonym(value = "FR-BDT", source = "FpML_5_10")
	FR_BDT("FR-BDT"),
	

	/**
	 * Treasury Notes: Bons du Trésor à Taux Annuel (BTAN).
	 */
	@RosettaSynonym(value = "FR-BTAN", source = "FpML_5_10")
	FR_BTAN("FR-BTAN"),
	

	/**
	 * Treasury Bills: Bons du Trésor à Taux Fixe (BTF).
	 */
	@RosettaSynonym(value = "FR-BTF", source = "FpML_5_10")
	FR_BTF("FR-BTF"),
	

	/**
	 * Government bonds: Obligations Assimilables du Trésor (OAT).
	 */
	@RosettaSynonym(value = "FR-OAT", source = "FpML_5_10")
	FR_OAT("FR-OAT"),
	

	/**
	 * STRIPS.
	 */
	@RosettaSynonym(value = "FR-STRIP", source = "FpML_5_10")
	FR_STRIP("FR-STRIP"),
	

	/**
	 * Generally Accepted Australian Government Obligations.
	 */
	@RosettaSynonym(value = "GA-AU-GOV", source = "FpML_5_10")
	GA_AU_GOV("GA-AU-GOV"),
	

	/**
	 * Generally Accepted Belgian Government Obligations.
	 */
	@RosettaSynonym(value = "GA-BE-GOV", source = "FpML_5_10")
	GA_BE_GOV("GA-BE-GOV"),
	

	/**
	 * Generally Accepted Canadian Government Obligations.
	 */
	@RosettaSynonym(value = "GA-CA-GOV", source = "FpML_5_10")
	GA_CA_GOV("GA-CA-GOV"),
	

	/**
	 * Generally Accepted Swiss Government Obligations.
	 */
	@RosettaSynonym(value = "GA-CH-GOV", source = "FpML_5_10")
	GA_CH_GOV("GA-CH-GOV"),
	

	/**
	 * Generally Accepted German Government Obligations.
	 */
	@RosettaSynonym(value = "GA-DE-GOV", source = "FpML_5_10")
	GA_DE_GOV("GA-DE-GOV"),
	

	/**
	 * Generally Accepted Danish Government Obligations.
	 */
	@RosettaSynonym(value = "GA-DK-GOV", source = "FpML_5_10")
	GA_DK_GOV("GA-DK-GOV"),
	

	/**
	 * Generally Accepted Spanish Government Obligations.
	 */
	@RosettaSynonym(value = "GA-ES-GOV", source = "FpML_5_10")
	GA_ES_GOV("GA-ES-GOV"),
	

	/**
	 * Generally Accepted Euro Zone Government Securities.
	 */
	@RosettaSynonym(value = "GA-EUROZONE-GOV", source = "FpML_5_10")
	GA_EUROZONE_GOV("GA-EUROZONE-GOV"),
	

	/**
	 * Generally Accepted EU Member State Government Securities.
	 */
	@RosettaSynonym(value = "GA-EU-GOV", source = "FpML_5_10")
	GA_EU_GOV("GA-EU-GOV"),
	

	/**
	 * Generally Accepted Finnish Government Obligations.
	 */
	@RosettaSynonym(value = "GA-FI-GOV", source = "FpML_5_10")
	GA_FI_GOV("GA-FI-GOV"),
	

	/**
	 * Generally Accepted French Government Obligations.
	 */
	@RosettaSynonym(value = "GA-FR-GOV", source = "FpML_5_10")
	GA_FR_GOV("GA-FR-GOV"),
	

	/**
	 * Generally Accepted G5 Government Obligations.
	 */
	@RosettaSynonym(value = "GA-G5-GOV", source = "FpML_5_10")
	GA_G5_GOV("GA-G5-GOV"),
	

	/**
	 * Generally Accepted British Government Obligations.
	 */
	@RosettaSynonym(value = "GA-GB-GOV", source = "FpML_5_10")
	GA_GB_GOV("GA-GB-GOV"),
	

	/**
	 * Generally Accepted Hong Kong Government Obligations.
	 */
	@RosettaSynonym(value = "GA-HK-GOV", source = "FpML_5_10")
	GA_HK_GOV("GA-HK-GOV"),
	

	/**
	 * Generally Accepted Italian Government Obligations.
	 */
	@RosettaSynonym(value = "GA-IT-GOV", source = "FpML_5_10")
	GA_IT_GOV("GA-IT-GOV"),
	

	/**
	 * Generally Accepted Japanese Government Obligations.
	 */
	@RosettaSynonym(value = "GA-JP-GOV", source = "FpML_5_10")
	GA_JP_GOV("GA-JP-GOV"),
	

	/**
	 * Generally Accepted Korean Government Obligations.
	 */
	@RosettaSynonym(value = "GA-KR-GOV", source = "FpML_5_10")
	GA_KR_GOV("GA-KR-GOV"),
	

	/**
	 * Generally Accepted Netherlands Government Obligations.
	 */
	@RosettaSynonym(value = "GA-NL-GOV", source = "FpML_5_10")
	GA_NL_GOV("GA-NL-GOV"),
	

	/**
	 * Generally Accepted Norwegian Government Obligations.
	 */
	@RosettaSynonym(value = "GA-NO-GOV", source = "FpML_5_10")
	GA_NO_GOV("GA-NO-GOV"),
	

	/**
	 * Generally Accepted New Zealand Government Obligations.
	 */
	@RosettaSynonym(value = "GA-NZ-GOV", source = "FpML_5_10")
	GA_NZ_GOV("GA-NZ-GOV"),
	

	/**
	 * Generally Accepted Swedish Government Obligations.
	 */
	@RosettaSynonym(value = "GA-SE-GOV", source = "FpML_5_10")
	GA_SE_GOV("GA-SE-GOV"),
	

	/**
	 * Generally Accepted Singaporean Government Obligations.
	 */
	@RosettaSynonym(value = "GA-SG-GOV", source = "FpML_5_10")
	GA_SG_GOV("GA-SG-GOV"),
	

	/**
	 * Generally Accepted US Agency Obligations.
	 */
	@RosettaSynonym(value = "GA-US-AGENCY", source = "FpML_5_10")
	GA_US_AGENCY("GA-US-AGENCY"),
	

	/**
	 * Generally Accepted US Government Obligations.
	 */
	@RosettaSynonym(value = "GA-US-GOV", source = "FpML_5_10")
	GA_US_GOV("GA-US-GOV"),
	

	/**
	 * Generally Accepted US Mortgage-Backed Obligations.
	 */
	@RosettaSynonym(value = "GA-US-MORTGAGES", source = "FpML_5_10")
	GA_US_MORTGAGES("GA-US-MORTGAGES"),
	

	/**
	 * British Pound Sterling (GBP) Cash.
	 */
	@RosettaSynonym(value = "GB-CASH", source = "FpML_5_10")
	GB_CASH("GB-CASH"),
	

	/**
	 * Double-dated Gilts.
	 */
	@RosettaSynonym(value = "GB-DDGILT", source = "FpML_5_10")
	GB_DDGILT("GB-DDGILT"),
	

	/**
	 * FTSE 100 Equity Securities.
	 */
	@RosettaSynonym(value = "GB-FT100", source = "FpML_5_10")
	GB_FT100("GB-FT100"),
	

	/**
	 * FTSE 250 Equity Securities.
	 */
	@RosettaSynonym(value = "GB-FT250", source = "FpML_5_10")
	GB_FT250("GB-FT250"),
	

	/**
	 * FTSE 350 Equity Securities.
	 */
	@RosettaSynonym(value = "GB-FT350", source = "FpML_5_10")
	GB_FT350("GB-FT350"),
	

	/**
	 * Conventional Gilts.
	 */
	@RosettaSynonym(value = "GB-GILT", source = "FpML_5_10")
	@RosettaSynonym(value = "UK Gilts", source = "ISDA_Create_1_0")
	GB_GILT("GB-GILT"),
	

	/**
	 * Index-Linked Gilts.
	 */
	@RosettaSynonym(value = "GB-INDEXGILT", source = "FpML_5_10")
	GB_INDEXGILT("GB-INDEXGILT"),
	

	/**
	 * Undated or Perpetual Gilts.
	 */
	@RosettaSynonym(value = "GB-PERPGILT", source = "FpML_5_10")
	GB_PERPGILT("GB-PERPGILT"),
	

	/**
	 * Rump Stock.
	 */
	@RosettaSynonym(value = "GB-RUMPGILT", source = "FpML_5_10")
	GB_RUMPGILT("GB-RUMPGILT"),
	

	/**
	 * Bank of England Euro Bills.
	 */
	@RosettaSynonym(value = "GB-SUPR1", source = "FpML_5_10")
	GB_SUPR1("GB-SUPR1"),
	

	/**
	 * Bank of England Euro Notes.
	 */
	@RosettaSynonym(value = "GB-SUPR2", source = "FpML_5_10")
	GB_SUPR2("GB-SUPR2"),
	

	/**
	 * UK Treasury Bills.
	 */
	@RosettaSynonym(value = "GB-TBILL", source = "FpML_5_10")
	GB_TBILL("GB-TBILL"),
	

	/**
	 * Gilt Strips or Zero Coupon Gilts.
	 */
	@RosettaSynonym(value = "GB-ZEROGILT", source = "FpML_5_10")
	GB_ZEROGILT("GB-ZEROGILT"),
	

	/**
	 * Hong Kong Government Exchange Fund Bills.
	 */
	@RosettaSynonym(value = "HK-BILL", source = "FpML_5_10")
	HK_BILL("HK-BILL"),
	

	/**
	 * Hong Kong Dollar (HKD) Cash.
	 */
	@RosettaSynonym(value = "HK-CASH", source = "FpML_5_10")
	HK_CASH("HK-CASH"),
	

	/**
	 * Hong Kong Government Exchange Fund Notes.
	 */
	@RosettaSynonym(value = "HK-NOTE", source = "FpML_5_10")
	HK_NOTE("HK-NOTE"),
	

	/**
	 * Botbuoni Ordinari del Tesoro (BOT) zero coupon debt securities issued by the Italian Treasury with maturities up to 365 days.
	 */
	@RosettaSynonym(value = "IT-BOT", source = "FpML_5_10")
	IT_BOT("IT-BOT"),
	

	/**
	 * Buoni del Tesoro Poliennali fixed interest semi-annual debt securities issued by the Italian Treasury with original maturities between 3 and 30 years.
	 */
	@RosettaSynonym(value = "IT-BTP", source = "FpML_5_10")
	IT_BTP("IT-BTP"),
	

	/**
	 * Certificati di Credito del Tesoro a Cedola Variable (CCT) or floating rate interest bearing debt securities issued by the Italian Treasury.
	 */
	@RosettaSynonym(value = "IT-CCT", source = "FpML_5_10")
	IT_CCT("IT-CCT"),
	

	/**
	 * Corporate bonds.
	 */
	@RosettaSynonym(value = "IT-CORP", source = "FpML_5_10")
	IT_CORP("IT-CORP"),
	

	/**
	 * Certificati del Tesoro zero coupon debt securities issued by the Italian Treasury with maturities between 18 and 24 months.
	 */
	@RosettaSynonym(value = "IT-CTZ", source = "FpML_5_10")
	IT_CTZ("IT-CTZ"),
	

	/**
	 * MIB30 Equity Securities.
	 */
	@RosettaSynonym(value = "IT-MIB30", source = "FpML_5_10")
	IT_MIB30("IT-MIB30"),
	

	/**
	 * Debt securities issued and marketed by the Republic of Italy outside the Italian market, traded as Eurobonds.
	 */
	@RosettaSynonym(value = "IT-REP", source = "FpML_5_10")
	IT_REP("IT-REP"),
	

	/**
	 * Japanese Yen (JPY) Cash.
	 */
	@RosettaSynonym(value = "JP-CASH", source = "FpML_5_10")
	JP_CASH("JP-CASH"),
	

	/**
	 * Corporate bonds including straight bonds.
	 */
	@RosettaSynonym(value = "JP-CORPORATE", source = "FpML_5_10")
	JP_CORPORATE("JP-CORPORATE"),
	

	/**
	 * Commercial Paper.
	 */
	@RosettaSynonym(value = "JP-CP", source = "FpML_5_10")
	JP_CP("JP-CP"),
	

	/**
	 * Equity securities issued by a Japanese company, each share representing the minimum unit of participation of a partner in the stock capital of the company.
	 */
	@RosettaSynonym(value = "JP-EQUITY", source = "FpML_5_10")
	JP_EQUITY("JP-EQUITY"),
	

	/**
	 * Yen-denominated foreign bonds.
	 */
	@RosettaSynonym(value = "JP-EUROBOND", source = "FpML_5_10")
	JP_EUROBOND("JP-EUROBOND"),
	

	/**
	 * Japanese Government Bonds.
	 */
	@RosettaSynonym(value = "JP-JGB", source = "FpML_5_10")
	JP_JGB("JP-JGB"),
	

	/**
	 * Korean Treasury Bonds.
	 */
	@RosettaSynonym(value = "KR-BOND", source = "FpML_5_10")
	KR_BOND("KR-BOND"),
	

	/**
	 * Korean Won (KRW) Cash.
	 */
	@RosettaSynonym(value = "KR-CASH", source = "FpML_5_10")
	KR_CASH("KR-CASH"),
	

	/**
	 * Non Korean Won denominated Export-Import Bank of Korea bonds.
	 */
	@RosettaSynonym(value = "KR-EXIM", source = "FpML_5_10")
	KR_EXIM("KR-EXIM"),
	

	/**
	 * Korean Development Insurance Corporation Bonds (Korean Won denominated).
	 */
	@RosettaSynonym(value = "KR-KDICKRW", source = "FpML_5_10")
	KR_KDICKRW("KR-KDICKRW"),
	

	/**
	 * Non-Korean Won denominated Korea Development Bank bonds (KDBs).
	 */
	@RosettaSynonym(value = "KR-KDR", source = "FpML_5_10")
	KR_KDR("KR-KDR"),
	

	/**
	 * KEPCO bonds.
	 */
	@RosettaSynonym(value = "KR-KEPCO", source = "FpML_5_10")
	KR_KEPCO("KR-KEPCO"),
	

	/**
	 * Monetary Stabilisation Bonds.
	 */
	@RosettaSynonym(value = "KR-MSB", source = "FpML_5_10")
	KR_MSB("KR-MSB"),
	

	/**
	 * Non Korean Won denominated Korea National Housing Corporation bonds (KNHCs).
	 */
	@RosettaSynonym(value = "KR-NHC", source = "FpML_5_10")
	KR_NHC("KR-NHC"),
	

	/**
	 * Non-Korean Won denominated Republic of Korea bonds (ROKs).
	 */
	@RosettaSynonym(value = "KR-ROK", source = "FpML_5_10")
	KR_ROK("KR-ROK"),
	

	/**
	 * AEX Equity Securities.
	 */
	@RosettaSynonym(value = "NL-AEX", source = "FpML_5_10")
	NL_AEX("NL-AEX"),
	

	/**
	 * Dutch Treasury Certificates.
	 */
	@RosettaSynonym(value = "NL-BILL", source = "FpML_5_10")
	NL_BILL("NL-BILL"),
	

	/**
	 * Dutch State Loans.
	 */
	@RosettaSynonym(value = "NL-BOND", source = "FpML_5_10")
	NL_BOND("NL-BOND"),
	

	/**
	 * Norwegian Government Bonds.
	 */
	@RosettaSynonym(value = "NO-BOND", source = "FpML_5_10")
	NO_BOND("NO-BOND"),
	

	/**
	 * Norwegian Krone (NOK) Cash.
	 */
	@RosettaSynonym(value = "NO-CASH", source = "FpML_5_10")
	NO_CASH("NO-CASH"),
	

	/**
	 * OBX Equity Securities.
	 */
	@RosettaSynonym(value = "NO-OBX", source = "FpML_5_10")
	NO_OBX("NO-OBX"),
	

	/**
	 * Norwegian T-Bills.
	 */
	@RosettaSynonym(value = "NO-TBILL", source = "FpML_5_10")
	NO_TBILL("NO-TBILL"),
	

	/**
	 * New Zealand Government Bonds.
	 */
	@RosettaSynonym(value = "NZ-BOND", source = "FpML_5_10")
	NZ_BOND("NZ-BOND"),
	

	/**
	 * New Zealand Dollar (NZD) Cash.
	 */
	@RosettaSynonym(value = "NZ-CASH", source = "FpML_5_10")
	NZ_CASH("NZ-CASH"),
	

	/**
	 * New Zealand Government Treasury Bills.
	 */
	@RosettaSynonym(value = "NZ-TBILL", source = "FpML_5_10")
	NZ_TBILL("NZ-TBILL"),
	

	/**
	 * Swedish Krona (SEK) Cash.
	 */
	@RosettaSynonym(value = "SE-CASH", source = "FpML_5_10")
	SE_CASH("SE-CASH"),
	

	/**
	 * Swedish Government Bonds (SGB).
	 */
	@RosettaSynonym(value = "SE-GOVT", source = "FpML_5_10")
	SE_GOVT("SE-GOVT"),
	

	/**
	 * Swedish Index Linked Government bonds.
	 */
	@RosettaSynonym(value = "SE-ILGOVT", source = "FpML_5_10")
	SE_ILGOVT("SE-ILGOVT"),
	

	/**
	 * Swedish Mortgage Bonds.
	 */
	@RosettaSynonym(value = "SE-MORT", source = "FpML_5_10")
	SE_MORT("SE-MORT"),
	

	/**
	 * OMX Equity Securities.
	 */
	@RosettaSynonym(value = "SE-OMX", source = "FpML_5_10")
	SE_OMX("SE-OMX"),
	

	/**
	 * Swedish Treasury Bills (STB).
	 */
	@RosettaSynonym(value = "SE-TBILL", source = "FpML_5_10")
	SE_TBILL("SE-TBILL"),
	

	/**
	 * Singapore Government (SGS) Bonds.
	 */
	@RosettaSynonym(value = "SG-BOND", source = "FpML_5_10")
	SG_BOND("SG-BOND"),
	

	/**
	 * Singapore Dollar (SGD) Cash.
	 */
	@RosettaSynonym(value = "SG-CASH", source = "FpML_5_10")
	SG_CASH("SG-CASH"),
	

	/**
	 * Singapore Government T-Bills (T-Bills).
	 */
	@RosettaSynonym(value = "SG-TBILL", source = "FpML_5_10")
	SG_TBILL("SG-TBILL"),
	

	/**
	 * Inter-American Development Bank Bonds.
	 */
	@RosettaSynonym(value = "SU-IADB", source = "FpML_5_10")
	SU_IADB("SU-IADB"),
	

	/**
	 * International Bank for Reconstruction and Development (World Bank) Discount Notes.
	 */
	@RosettaSynonym(value = "SU-IBRDDN", source = "FpML_5_10")
	SU_IBRDDN("SU-IBRDDN"),
	

	/**
	 * International Bank for Reconstruction and Development (World Bank or IBRD) Global Benchmark Bonds.
	 */
	@RosettaSynonym(value = "SU-IBRDGB", source = "FpML_5_10")
	SU_IBRDGB("SU-IBRDGB"),
	

	/**
	 * Adjustable Rate Mortgage (ARM) Bonds.
	 */
	@RosettaSynonym(value = "US-ARM", source = "FpML_5_10")
	US_ARM("US-ARM"),
	

	/**
	 * United States of America Dollar (USD) Cash.
	 */
	@RosettaSynonym(value = "US-CASH", source = "FpML_5_10")
	US_CASH("US-CASH"),
	

	/**
	 * REMICs, CMOs and other derivative structures.
	 */
	@RosettaSynonym(value = "US-DERIV", source = "FpML_5_10")
	US_DERIV("US-DERIV"),
	

	/**
	 * Dow Jones Industrial Average Equity Securities.
	 */
	@RosettaSynonym(value = "US-DOW", source = "FpML_5_10")
	US_DOW("US-DOW"),
	

	/**
	 * Dow Jones Composite Average Equity Securities.
	 */
	@RosettaSynonym(value = "US-DOW-COMP", source = "FpML_5_10")
	US_DOW_COMP("US-DOW-COMP"),
	

	/**
	 * Dow Jones Transportation	Average Equity Securities.
	 */
	@RosettaSynonym(value = "US-DOW-TRAN", source = "FpML_5_10")
	US_DOW_TRAN("US-DOW-TRAN"),
	

	/**
	 * Dow Jones Utilities Average Equity Securities.
	 */
	@RosettaSynonym(value = "US-DOW-UTIL", source = "FpML_5_10")
	US_DOW_UTIL("US-DOW-UTIL"),
	

	/**
	 * Federal Agricultural Mortgage Corp (Farmer Mac) Bonds.
	 */
	@RosettaSynonym(value = "US-FAMC", source = "FpML_5_10")
	US_FAMC("US-FAMC"),
	

	/**
	 * Farm Credit System (FCS) Bonds.
	 */
	@RosettaSynonym(value = "US-FCS", source = "FpML_5_10")
	US_FCS("US-FCS"),
	

	/**
	 * Farm Credit System Financial Assistance Corporation (FCSFAC) Bonds.
	 */
	@RosettaSynonym(value = "US-FCSFAC", source = "FpML_5_10")
	US_FCSFAC("US-FCSFAC"),
	

	/**
	 * Callable Agency Debt – Federal Home Loan Bank (FHLB).
	 */
	@RosettaSynonym(value = "US-FHLB", source = "FpML_5_10")
	US_FHLB("US-FHLB"),
	

	/**
	 * Non-Callable Federal Home Loan Bank Debt.
	 */
	@RosettaSynonym(value = "US-FHLBNC", source = "FpML_5_10")
	US_FHLBNC("US-FHLBNC"),
	

	/**
	 * Non-Callable Federal Home Loan Bank Discount Notes.
	 */
	@RosettaSynonym(value = "US-FHLBNCDN", source = "FpML_5_10")
	US_FHLBNCDN("US-FHLBNCDN"),
	

	/**
	 * Callable Agency Debt – the Federal Home Loan Mortgage Corporation (FHLMC or Freddie Mac).
	 */
	@RosettaSynonym(value = "US-FHLMC", source = "FpML_5_10")
	US_FHLMC("US-FHLMC"),
	

	/**
	 * Federal Home Loan Mortgage Corporation Certificates – Mortgage Backed Securities.
	 */
	@RosettaSynonym(value = "US-FHLMCMBS", source = "FpML_5_10")
	US_FHLMCMBS("US-FHLMCMBS"),
	

	/**
	 * Financing Corp (FICO) Bonds.
	 */
	@RosettaSynonym(value = "US-FICO", source = "FpML_5_10")
	US_FICO("US-FICO"),
	

	/**
	 * Callable Agency Debt – Federal National Mortgage Association (FNMA or Fannie Mae).
	 */
	@RosettaSynonym(value = "US-FNMA", source = "FpML_5_10")
	US_FNMA("US-FNMA"),
	

	/**
	 * Federal National Mortgage Association Certificates – Mortgage Backed Securities.
	 */
	@RosettaSynonym(value = "US-FNMAMBS", source = "FpML_5_10")
	US_FNMAMBS("US-FNMAMBS"),
	

	/**
	 * Callable Agency Debt – Government National Mortgage Association (GNMA).
	 */
	@RosettaSynonym(value = "US-GNMA", source = "FpML_5_10")
	US_GNMA("US-GNMA"),
	

	/**
	 * Government National Mortgage Association Certificates – Mortgage Backed Securities (GNMA or Ginnie Mae)
	 */
	@RosettaSynonym(value = "US-GNMAMBS", source = "FpML_5_10")
	US_GNMAMBS("US-GNMAMBS"),
	

	/**
	 * Lehman Brothers Credit Bond Index Debt Securities.
	 */
	@RosettaSynonym(value = "US-LEHM-BOND", source = "FpML_5_10")
	US_LEHM_BOND("US-LEHM-BOND"),
	

	/**
	 * NASDAQ-100 Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-NAS-100", source = "FpML_5_10")
	US_NAS_100("US-NAS-100"),
	

	/**
	 * NASDAQ Composite Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-NAS-COMP", source = "FpML_5_10")
	US_NAS_COMP("US-NAS-COMP"),
	

	/**
	 * Non-Callable Agency Debt – Various Issuers.
	 */
	@RosettaSynonym(value = "US-NCAD", source = "FpML_5_10")
	US_NCAD("US-NCAD"),
	

	/**
	 * Non-Callable Agency Discount Notes – Various Issuers.
	 */
	@RosettaSynonym(value = "US-NCADN", source = "FpML_5_10")
	US_NCADN("US-NCADN"),
	

	/**
	 * NYSE Composite Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-NYSE-COMP", source = "FpML_5_10")
	US_NYSE_COMP("US-NYSE-COMP"),
	

	/**
	 * Resolution Funding Corp (REFCorp) Bonds.
	 */
	@RosettaSynonym(value = "US-REFCORP", source = "FpML_5_10")
	US_REFCORP("US-REFCORP"),
	

	/**
	 * Student Loan Marketing Association (Sallie Mae) Bonds.
	 */
	@RosettaSynonym(value = "US-SLMA", source = "FpML_5_10")
	US_SLMA("US-SLMA"),
	

	/**
	 * US Treasury Strips.
	 */
	@RosettaSynonym(value = "US-STRIP", source = "FpML_5_10")
	US_STRIP("US-STRIP"),
	

	/**
	 * Standard &amp; Poor’s 100 Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-S&P100", source = "FpML_5_10")
	US_S_P100("US-S&P100"),
	

	/**
	 * Standard &amp; Poor’s Midcap 400 Equity Securities. corporations that are included within the Standard And Poor&#39;s Midcap 400 Index published by Standard And Poor&#39;s, a division of The McGraw-Hill Companies, Inc.
	 */
	@RosettaSynonym(value = "US-S&P400", source = "FpML_5_10")
	US_S_P400("US-S&P400"),
	

	/**
	 * Standard &amp; Poor’s 500 Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-S&P500", source = "FpML_5_10")
	US_S_P500("US-S&P500"),
	

	/**
	 * Standard &amp; Poor’s Smallcap 600 Index Equity Securities.
	 */
	@RosettaSynonym(value = "US-S&P600", source = "FpML_5_10")
	US_S_P600("US-S&P600"),
	

	/**
	 * US Treasury Bills.
	 */
	@RosettaSynonym(value = "US-TBILL", source = "FpML_5_10")
	US_TBILL("US-TBILL"),
	

	/**
	 * US Treasury Bonds.
	 */
	@RosettaSynonym(value = "US-TBOND", source = "FpML_5_10")
	US_TBOND("US-TBOND"),
	

	/**
	 * US Treasury Inflation Protected Issues (TIPS).
	 */
	@RosettaSynonym(value = "US-TIPS", source = "FpML_5_10")
	US_TIPS("US-TIPS"),
	

	/**
	 * US Treasury Notes.
	 */
	@RosettaSynonym(value = "US-TNOTE", source = "FpML_5_10")
	US_TNOTE("US-TNOTE"),
	

	/**
	 * Tennessee Valley Authority (TVA) Bonds.
	 */
	@RosettaSynonym(value = "US-TVA", source = "FpML_5_10")
	US_TVA("US-TVA")
	;

	private final String displayName;
	
	CollateralAssetDefinitionsEnum() {
		this.displayName = null;
	}

	CollateralAssetDefinitionsEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
