package org.isda.cdm;

import org.isda.cdm.CollateralManagementAgreementElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagementAgreementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Collateral Management Agreement election by the respective parties to a Japanese Law ISDA CSA. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(i): Collateral Management Agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManagementAgreement extends RosettaModelObject {
	private final CollateralManagementAgreementElection partyElection;
	private static CollateralManagementAgreementMeta metaData = new CollateralManagementAgreementMeta();
	
	CollateralManagementAgreement(CollateralManagementAgreementBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The parties&#39; Collateral Management Agreement election.
	 */
	public final CollateralManagementAgreementElection getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManagementAgreement> metaData() {
		return metaData;
	} 
	
	public CollateralManagementAgreementBuilder toBuilder() {
		CollateralManagementAgreementBuilder builder = new CollateralManagementAgreementBuilder();
		ofNullable(getPartyElection()).ifPresent(builder::setPartyElection);
		return builder;
	}

	public static CollateralManagementAgreementBuilder builder() {
		return new CollateralManagementAgreementBuilder();
	}

	public static  class CollateralManagementAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected CollateralManagementAgreementElection.CollateralManagementAgreementElectionBuilder partyElection;
	
		public CollateralManagementAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManagementAgreement> metaData() {
			return metaData;
		} 
	
		
		public CollateralManagementAgreementElection.CollateralManagementAgreementElectionBuilder getPartyElection() {
			return partyElection;
		}
		
		public CollateralManagementAgreementElection.CollateralManagementAgreementElectionBuilder getOrCreatePartyElection() {
			if (partyElection!=null) {
				return partyElection;
			}
			else return partyElection = new CollateralManagementAgreementElection.CollateralManagementAgreementElectionBuilder();
		}
		
	
		public CollateralManagementAgreementBuilder setPartyElection(CollateralManagementAgreementElection partyElection) {
			this.partyElection = partyElection.toBuilder();
			return this;
		}
		public CollateralManagementAgreementBuilder setPartyElectionBuilder(CollateralManagementAgreementElection.CollateralManagementAgreementElectionBuilder partyElection) {
			this.partyElection = partyElection;
			return this;
		}
		
		public CollateralManagementAgreement build() {
			return new CollateralManagementAgreement(this);
		}
	
		@Override
		public CollateralManagementAgreementBuilder prune() {
			if (partyElection!=null && !partyElection.prune().hasData()) partyElection = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, CollateralManagementAgreementElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagementAgreementBuilder _that = (CollateralManagementAgreementBuilder) o;
		
			if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagementAgreementBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, CollateralManagementAgreementElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManagementAgreement _that = (CollateralManagementAgreement) o;
	
		if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManagementAgreement {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
