package org.isda.cdm;

import org.isda.cdm.CollateralManagerElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CollateralManagerMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the Collateral Manager election by the respective parties to a Japanese Law ISDA CSA. ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (b)(ii): Collateral Manager.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CollateralManager extends RosettaModelObject {
	private final CollateralManagerElection partyElection;
	private static CollateralManagerMeta metaData = new CollateralManagerMeta();
	
	CollateralManager(CollateralManagerBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The parties&#39; Collateral Manager election.
	 */
	public final CollateralManagerElection getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends CollateralManager> metaData() {
		return metaData;
	} 
	
	public CollateralManagerBuilder toBuilder() {
		CollateralManagerBuilder builder = new CollateralManagerBuilder();
		ofNullable(getPartyElection()).ifPresent(builder::setPartyElection);
		return builder;
	}

	public static CollateralManagerBuilder builder() {
		return new CollateralManagerBuilder();
	}

	public static  class CollateralManagerBuilder extends RosettaModelObjectBuilder {
	
		protected CollateralManagerElection.CollateralManagerElectionBuilder partyElection;
	
		public CollateralManagerBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CollateralManager> metaData() {
			return metaData;
		} 
	
		
		public CollateralManagerElection.CollateralManagerElectionBuilder getPartyElection() {
			return partyElection;
		}
		
		public CollateralManagerElection.CollateralManagerElectionBuilder getOrCreatePartyElection() {
			if (partyElection!=null) {
				return partyElection;
			}
			else return partyElection = new CollateralManagerElection.CollateralManagerElectionBuilder();
		}
		
	
		public CollateralManagerBuilder setPartyElection(CollateralManagerElection partyElection) {
			this.partyElection = partyElection.toBuilder();
			return this;
		}
		public CollateralManagerBuilder setPartyElectionBuilder(CollateralManagerElection.CollateralManagerElectionBuilder partyElection) {
			this.partyElection = partyElection;
			return this;
		}
		
		public CollateralManager build() {
			return new CollateralManager(this);
		}
	
		@Override
		public CollateralManagerBuilder prune() {
			if (partyElection!=null && !partyElection.prune().hasData()) partyElection = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, CollateralManagerElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CollateralManagerBuilder _that = (CollateralManagerBuilder) o;
		
			if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CollateralManagerBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, CollateralManagerElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CollateralManager _that = (CollateralManager) o;
	
		if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CollateralManager {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
