package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify the Commodity Reference Prices specified in the Annex to the 2005 ISDA Commodity Definitions.
 * 
 * @version 2.5.4
 */
public enum CommodityReferencePriceEnum {
    

	/**
	 * Per 2005 ISDA Commodity Definitions, Sub-Annex A, Section 7.1 Commodity Reference Prices, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ALUMINIUM ALLOY-LME 15 MONTH", source = "FpML_5_10")
	@RosettaSynonym(value = "ALUMINIUM ALLOY-LME 15 MONTH", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ALUMINIUM ALLOY-LME 15 MONTH", source = "DTCC_11_0")
	@RosettaSynonym(value = "ALUMINIUM ALLOY-LME 15 MONTH", source = "DTCC_9_0")
	@RosettaSynonym(value = "ALUMINIUM ALLOY-LME 15 MONTH", source = "CME_ClearedConfirm_1_17")
	ALUMINIUM_ALLOY_LME_15_MONTH("ALUMINIUM ALLOY-LME 15 MONTH"),
	

	/**
	 * A code for the NYMEX Central Appalachian Coal commodity
	 */
	@RosettaSynonym(value = "COAL-CENTRAL APPALACHIAN-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "COAL-CENTRAL APPALACHIAN-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COAL-CENTRAL APPALACHIAN-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "COAL-CENTRAL APPALACHIAN-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "COAL-CENTRAL APPALACHIAN-NYMEX", source = "CME_ClearedConfirm_1_17")
	COAL_CENTRAL_APPALACHIAN_NYMEX("COAL-CENTRAL APPALACHIAN-NYMEX"),
	

	/**
	 * A code for the ICE Futures U.S. (‘ICUS’) Cocoa commodity
	 */
	@RosettaSynonym(value = "COCOA-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "COCOA-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COCOA-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "COCOA-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "COCOA-ICE", source = "CME_ClearedConfirm_1_17")
	COCOA_ICE("COCOA-ICE"),
	

	/**
	 * A code for the ICUS Coffee C commodity
	 */
	@RosettaSynonym(value = "COFFEE ARABICA-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "COFFEE ARABICA-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COFFEE ARABICA-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "COFFEE ARABICA-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "COFFEE ARABICA-ICE", source = "CME_ClearedConfirm_1_17")
	COFFEE_ARABICA_ICE("COFFEE ARABICA-ICE"),
	

	/**
	 * A code for the ICUS Coffee C commodity
	 */
	@RosettaSynonym(value = "COFFEE ROBUSTA-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "COFFEE ROBUSTA-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COFFEE ROBUSTA-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "COFFEE ROBUSTA-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "COFFEE ROBUSTA-ICE", source = "CME_ClearedConfirm_1_17")
	COFFEE_ROBUSTA_ICE("COFFEE ROBUSTA-ICE"),
	

	/**
	 * A code for the COMEX (‘CMX’) Copper Grade #1 commodity
	 */
	@RosettaSynonym(value = "COPPER-COMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "COPPER-COMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COPPER-COMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "COPPER-COMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "COPPER-COMEX", source = "CME_ClearedConfirm_1_17")
	COPPER_COMEX("COPPER-COMEX"),
	

	/**
	 * A code for the Chicago Board of Trade (‘CBOT’) Corn commodity
	 */
	@RosettaSynonym(value = "CORN-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "CORN-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CORN-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "CORN-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "CORN-CBOT", source = "CME_ClearedConfirm_1_17")
	CORN_CBOT("CORN-CBOT"),
	

	/**
	 * A code for the ICUS Cotton No. 2 commodity
	 */
	@RosettaSynonym(value = "COTTON NO. 2-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "COTTON NO. 2-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COTTON NO. 2-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "COTTON NO. 2-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "COTTON NO. 2-ICE", source = "CME_ClearedConfirm_1_17")
	COTTON_NO__2_ICE("COTTON NO. 2-ICE"),
	

	/**
	 * A code for the CBOT Ethanol commodity
	 */
	@RosettaSynonym(value = "ETHANOL-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "ETHANOL-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ETHANOL-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "ETHANOL-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "ETHANOL-CBOT", source = "CME_ClearedConfirm_1_17")
	ETHANOL_CBOT("ETHANOL-CBOT"),
	

	/**
	 * A code for the CME Feeder Cattle commodity
	 */
	@RosettaSynonym(value = "FEEDER CATTLE-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "FEEDER CATTLE-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FEEDER CATTLE-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "FEEDER CATTLE-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "FEEDER CATTLE-CME", source = "CME_ClearedConfirm_1_17")
	FEEDER_CATTLE_CME("FEEDER CATTLE-CME"),
	

	/**
	 * A code for the ICUS Frozen Concentrated Orange Juice commodity
	 */
	@RosettaSynonym(value = "FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE", source = "CME_ClearedConfirm_1_17")
	FROZEN_CONCENTRATED_ORANGE_JUICE_NO__1_ICE("FROZEN CONCENTRATED ORANGE JUICE NO. 1-ICE"),
	

	/**
	 * A code for the NYMEX Gasoline Blendstock (RBOB) commodity
	 */
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-ICE", source = "CME_ClearedConfirm_1_17")
	GASOLINE_RBOB_NEW_YORK_ICE("GASOLINE-RBOB-NEW YORK-ICE"),
	

	/**
	 * A code for the NYMEX Gasoline Blendstock (RBOB) commodity
	 */
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "GASOLINE-RBOB-NEW YORK-NYMEX", source = "CME_ClearedConfirm_1_17")
	GASOLINE_RBOB_NEW_YORK_NYMEX("GASOLINE-RBOB-NEW YORK-NYMEX"),
	

	/**
	 * A code for the CMX Gold commodity
	 */
	@RosettaSynonym(value = "GOLD-COMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "GOLD-COMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GOLD-COMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "GOLD-COMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "GOLD-COMEX", source = "CME_ClearedConfirm_1_17")
	GOLD_COMEX("GOLD-COMEX"),
	

	/**
	 * A code for the NYMEX No. 2 Heating Oil, New York Harbor commodity
	 */
	@RosettaSynonym(value = "HEATING OIL-NEW YORK-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "HEATING OIL-NEW YORK-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HEATING OIL-NEW YORK-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "HEATING OIL-NEW YORK-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "HEATING OIL-NEW YORK-NYMEX", source = "CME_ClearedConfirm_1_17")
	HEATING_OIL_NEW_YORK_NYMEX("HEATING OIL-NEW YORK-NYMEX"),
	

	/**
	 * A code for the CME Lean Hogs commodity
	 */
	@RosettaSynonym(value = "LEAN HOGS-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "LEAN HOGS-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LEAN HOGS-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "LEAN HOGS-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "LEAN HOGS-CME", source = "CME_ClearedConfirm_1_17")
	LEAN_HOGS_CME("LEAN HOGS-CME"),
	

	/**
	 * A code for the CME Live Cattle commodity
	 */
	@RosettaSynonym(value = "LIVE CATTLE-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "LIVE CATTLE-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LIVE CATTLE-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "LIVE CATTLE-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "LIVE CATTLE-CME", source = "CME_ClearedConfirm_1_17")
	LIVE_CATTLE_CME("LIVE CATTLE-CME"),
	

	/**
	 * A code for the CME Random Length Lumber commodity
	 */
	@RosettaSynonym(value = "LUMBER-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "LUMBER-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LUMBER-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "LUMBER-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "LUMBER-CME", source = "CME_ClearedConfirm_1_17")
	LUMBER_CME("LUMBER-CME"),
	

	/**
	 * A code for the CME Milk Class III commodity
	 */
	@RosettaSynonym(value = "MILK-CLASS III-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "MILK-CLASS III-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MILK-CLASS III-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "MILK-CLASS III-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "MILK-CLASS III-CME", source = "CME_ClearedConfirm_1_17")
	MILK_CLASS_III_CME("MILK-CLASS III-CME"),
	

	/**
	 * A code for the CME Non Fat Dry Milk commodity
	 */
	@RosettaSynonym(value = "MILK-NONFAT-DRY-CME", source = "FpML_5_10")
	@RosettaSynonym(value = "MILK-NONFAT-DRY-CME", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MILK-NONFAT-DRY-CME", source = "DTCC_11_0")
	@RosettaSynonym(value = "MILK-NONFAT-DRY-CME", source = "DTCC_9_0")
	@RosettaSynonym(value = "MILK-NONFAT-DRY-CME", source = "CME_ClearedConfirm_1_17")
	MILK_NONFAT_DRY_CME("MILK-NONFAT-DRY-CME"),
	

	/**
	 * A code for the NYMEX Natural Gas commodity
	 */
	@RosettaSynonym(value = "NATURAL GAS-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "NATURAL GAS-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NATURAL GAS-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "NATURAL GAS-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "NATURAL GAS-NYMEX", source = "CME_ClearedConfirm_1_17")
	NATURAL_GAS_NYMEX("NATURAL GAS-NYMEX"),
	

	/**
	 * A code for the NYMEX Panhandle Basis Swap commodity
	 */
	@RosettaSynonym(value = "NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC", source = "FpML_5_10")
	@RosettaSynonym(value = "NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC", source = "DTCC_11_0")
	@RosettaSynonym(value = "NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC", source = "DTCC_9_0")
	@RosettaSynonym(value = "NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC", source = "CME_ClearedConfirm_1_17")
	NATURAL_GAS_PEPL__TEXOK_MAINLINE__INSIDE_FERC("NATURAL GAS-PEPL (TEXOK MAINLINE)-INSIDE FERC"),
	

	/**
	 * A code for the NYMEX Waha Basis Swap commodity
	 */
	@RosettaSynonym(value = "NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC", source = "FpML_5_10")
	@RosettaSynonym(value = "NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC", source = "DTCC_11_0")
	@RosettaSynonym(value = "NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC", source = "DTCC_9_0")
	@RosettaSynonym(value = "NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC", source = "CME_ClearedConfirm_1_17")
	NATURAL_GAS_W__TEXAS__WAHA__INSIDE_FERC("NATURAL GAS-W. TEXAS (WAHA)-INSIDE FERC"),
	

	/**
	 * A code for the CBOT Oats commodity
	 */
	@RosettaSynonym(value = "OATS-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "OATS-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OATS-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "OATS-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "OATS-CBOT", source = "CME_ClearedConfirm_1_17")
	OATS_CBOT("OATS-CBOT"),
	

	/**
	 * A code for the NYMEX Crude Oil, Light Sweet commodity
	 */
	@RosettaSynonym(value = "OIL-WTI-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "OIL-WTI-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OIL-WTI-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "OIL-WTI-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "OIL-WTI-NYMEX", source = "CME_ClearedConfirm_1_17")
	OIL_WTI_NYMEX("OIL-WTI-NYMEX"),
	

	/**
	 * A code for the NYMEX Palladium commodity
	 */
	@RosettaSynonym(value = "PALLADIUM-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "PALLADIUM-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PALLADIUM-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "PALLADIUM-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "PALLADIUM-NYMEX", source = "CME_ClearedConfirm_1_17")
	PALLADIUM_NYMEX("PALLADIUM-NYMEX"),
	

	/**
	 * A code for the NYMEX Platinum commodity
	 */
	@RosettaSynonym(value = "PLATINUM-NYMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "PLATINUM-NYMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLATINUM-NYMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLATINUM-NYMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLATINUM-NYMEX", source = "CME_ClearedConfirm_1_17")
	PLATINUM_NYMEX("PLATINUM-NYMEX"),
	

	/**
	 * A code for the CBOT Rough Rice commodity
	 */
	@RosettaSynonym(value = "RICE-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "RICE-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RICE-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "RICE-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "RICE-CBOT", source = "CME_ClearedConfirm_1_17")
	RICE_CBOT("RICE-CBOT"),
	

	/**
	 * A code for the CMX Silver commodity
	 */
	@RosettaSynonym(value = "SILVER-COMEX", source = "FpML_5_10")
	@RosettaSynonym(value = "SILVER-COMEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SILVER-COMEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "SILVER-COMEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "SILVER-COMEX", source = "CME_ClearedConfirm_1_17")
	SILVER_COMEX("SILVER-COMEX"),
	

	/**
	 * A code for the CBOT Soybeans commodity
	 */
	@RosettaSynonym(value = "SOYBEANS-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "SOYBEANS-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SOYBEANS-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "SOYBEANS-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "SOYBEANS-CBOT", source = "CME_ClearedConfirm_1_17")
	SOYBEANS_CBOT("SOYBEANS-CBOT"),
	

	/**
	 * A code for the CBOT Soybean Meal commodity
	 */
	@RosettaSynonym(value = "SOYBEAN MEAL-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "SOYBEAN MEAL-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SOYBEAN MEAL-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "SOYBEAN MEAL-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "SOYBEAN MEAL-CBOT", source = "CME_ClearedConfirm_1_17")
	SOYBEAN_MEAL_CBOT("SOYBEAN MEAL-CBOT"),
	

	/**
	 * A code for the CBOT Soybean Oil commodity
	 */
	@RosettaSynonym(value = "SOYBEAN OIL-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "SOYBEAN OIL-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SOYBEAN OIL-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "SOYBEAN OIL-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "SOYBEAN OIL-CBOT", source = "CME_ClearedConfirm_1_17")
	SOYBEAN_OIL_CBOT("SOYBEAN OIL-CBOT"),
	

	/**
	 * A code for the ICUS Sugar No. 11 commodity
	 */
	@RosettaSynonym(value = "SUGAR # 11 (WORLD)-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "SUGAR # 11 (WORLD)-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SUGAR # 11 (WORLD)-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "SUGAR # 11 (WORLD)-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "SUGAR # 11 (WORLD)-ICE", source = "CME_ClearedConfirm_1_17")
	SUGAR___11__WORLD__ICE("SUGAR # 11 (WORLD)-ICE"),
	

	/**
	 * A code for the ICUS Sugar No. 16 commodity
	 */
	@RosettaSynonym(value = "SUGAR # 16 (US)-ICE", source = "FpML_5_10")
	@RosettaSynonym(value = "SUGAR # 16 (US)-ICE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SUGAR # 16 (US)-ICE", source = "DTCC_11_0")
	@RosettaSynonym(value = "SUGAR # 16 (US)-ICE", source = "DTCC_9_0")
	@RosettaSynonym(value = "SUGAR # 16 (US)-ICE", source = "CME_ClearedConfirm_1_17")
	SUGAR___16__US__ICE("SUGAR # 16 (US)-ICE"),
	

	/**
	 * A code for the CBOT Wheat commodity
	 */
	@RosettaSynonym(value = "WHEAT-CBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "WHEAT-CBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "WHEAT-CBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "WHEAT-CBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "WHEAT-CBOT", source = "CME_ClearedConfirm_1_17")
	WHEAT_CBOT("WHEAT-CBOT"),
	

	/**
	 * A code for the Kansas City Board of Trade (‘KCBT’)Wheat commodity
	 */
	@RosettaSynonym(value = "WHEAT HRW-KCBOT", source = "FpML_5_10")
	@RosettaSynonym(value = "WHEAT HRW-KCBOT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "WHEAT HRW-KCBOT", source = "DTCC_11_0")
	@RosettaSynonym(value = "WHEAT HRW-KCBOT", source = "DTCC_9_0")
	@RosettaSynonym(value = "WHEAT HRW-KCBOT", source = "CME_ClearedConfirm_1_17")
	WHEAT_HRW_KCBOT("WHEAT HRW-KCBOT"),
	

	/**
	 * A code for the Wheat commodity
	 */
	@RosettaSynonym(value = "WHEAT RED SPRING-MGE", source = "FpML_5_10")
	@RosettaSynonym(value = "WHEAT RED SPRING-MGE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "WHEAT RED SPRING-MGE", source = "DTCC_11_0")
	@RosettaSynonym(value = "WHEAT RED SPRING-MGE", source = "DTCC_9_0")
	@RosettaSynonym(value = "WHEAT RED SPRING-MGE", source = "CME_ClearedConfirm_1_17")
	WHEAT_RED_SPRING_MGE("WHEAT RED SPRING-MGE")
	;

	private final String displayName;
	
	CommodityReferencePriceEnum() {
		this.displayName = null;
	}

	CommodityReferencePriceEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
