package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.Commodity;
import org.isda.cdm.TimeUnitEnum;
import org.isda.cdm.TransferorTransferee;
import org.isda.cdm.UnitEnum;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CommodityTransferBreakdownMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CommodityTransferBreakdown extends TransferBreakdown {
	private final Commodity commodity;
	private final BigDecimal quantity;
	private final TimeUnitEnum timeUnit;
	private final TransferorTransferee transferorTransferee;
	private final UnitEnum unit;
	private static CommodityTransferBreakdownMeta metaData = new CommodityTransferBreakdownMeta();
	
	CommodityTransferBreakdown(CommodityTransferBreakdownBuilder builder) {
		super(builder);
		this.commodity = ofNullable(builder.getCommodity()).map(f->f.build()).orElse(null);
		this.quantity = builder.getQuantity();
		this.timeUnit = builder.getTimeUnit();
		this.transferorTransferee = ofNullable(builder.getTransferorTransferee()).map(f->f.build()).orElse(null);
		this.unit = builder.getUnit();
	}
	
	public final Commodity getCommodity() {
		return commodity;
	}
	
	@RosettaSynonym(value="quantity", source="Rosetta_Workbench")
	public final BigDecimal getQuantity() {
		return quantity;
	}
	
	public final TimeUnitEnum getTimeUnit() {
		return timeUnit;
	}
	
	/**
	 * The transferee and transferor party information.
	 */
	public final TransferorTransferee getTransferorTransferee() {
		return transferorTransferee;
	}
	
	/**
	 * The unit of measure, applicable to physical assets.
	 */
	public final UnitEnum getUnit() {
		return unit;
	}
	
	
	@Override
	public RosettaMetaData<? extends CommodityTransferBreakdown> metaData() {
		return metaData;
	} 
	
	public CommodityTransferBreakdownBuilder toBuilder() {
		CommodityTransferBreakdownBuilder builder = new CommodityTransferBreakdownBuilder();
		ofNullable(getCommodity()).ifPresent(builder::setCommodity);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getTimeUnit()).ifPresent(builder::setTimeUnit);
		ofNullable(getTransferorTransferee()).ifPresent(builder::setTransferorTransferee);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		ofNullable(getLineage()).ifPresent(lineage -> lineage.forEach(builder::addLineage));
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static CommodityTransferBreakdownBuilder builder() {
		return new CommodityTransferBreakdownBuilder();
	}

	public static  class CommodityTransferBreakdownBuilder extends TransferBreakdownBuilder {
	
		protected Commodity.CommodityBuilder commodity;
		protected BigDecimal quantity;
		protected TimeUnitEnum timeUnit;
		protected TransferorTransferee.TransferorTransfereeBuilder transferorTransferee;
		protected UnitEnum unit;
	
		public CommodityTransferBreakdownBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CommodityTransferBreakdown> metaData() {
			return metaData;
		} 
	
		
		public Commodity.CommodityBuilder getCommodity() {
			return commodity;
		}
		
		public Commodity.CommodityBuilder getOrCreateCommodity() {
			if (commodity!=null) {
				return commodity;
			}
			else return commodity = new Commodity.CommodityBuilder();
		}
		
		
		public BigDecimal getQuantity() {
			return quantity;
		}
		
		
		public TimeUnitEnum getTimeUnit() {
			return timeUnit;
		}
		
		
		public TransferorTransferee.TransferorTransfereeBuilder getTransferorTransferee() {
			return transferorTransferee;
		}
		
		public TransferorTransferee.TransferorTransfereeBuilder getOrCreateTransferorTransferee() {
			if (transferorTransferee!=null) {
				return transferorTransferee;
			}
			else return transferorTransferee = new TransferorTransferee.TransferorTransfereeBuilder();
		}
		
		
		public UnitEnum getUnit() {
			return unit;
		}
		
	
		public CommodityTransferBreakdownBuilder setCommodity(Commodity commodity) {
			this.commodity = commodity.toBuilder();
			return this;
		}
		public CommodityTransferBreakdownBuilder setCommodityBuilder(Commodity.CommodityBuilder commodity) {
			this.commodity = commodity;
			return this;
		}
		public CommodityTransferBreakdownBuilder setQuantity(BigDecimal quantity) {
			this.quantity = quantity;
			return this;
		}
		public CommodityTransferBreakdownBuilder setTimeUnit(TimeUnitEnum timeUnit) {
			this.timeUnit = timeUnit;
			return this;
		}
		public CommodityTransferBreakdownBuilder setTransferorTransferee(TransferorTransferee transferorTransferee) {
			this.transferorTransferee = transferorTransferee.toBuilder();
			return this;
		}
		public CommodityTransferBreakdownBuilder setTransferorTransfereeBuilder(TransferorTransferee.TransferorTransfereeBuilder transferorTransferee) {
			this.transferorTransferee = transferorTransferee;
			return this;
		}
		public CommodityTransferBreakdownBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder addLineage(Lineage lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage.toBuilder());
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder addLineage(Lineage lineage, int _idx) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			getIndex(this.lineage, _idx, () -> lineage.toBuilder());
			this.lineage.set(_idx, lineage.toBuilder());
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder addLineage(List<Lineage> lineages) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			for (Lineage toAdd : lineages) {
				this.lineage.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder addLineageBuilder(Lineage.LineageBuilder lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage);
			return this;
		}
		
		@Override public CommodityTransferBreakdownBuilder clearLineage() {
			this.lineage = null;
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public CommodityTransferBreakdownBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public CommodityTransferBreakdownBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public CommodityTransferBreakdown build() {
			return new CommodityTransferBreakdown(this);
		}
	
		@Override
		public CommodityTransferBreakdownBuilder prune() {
			super.prune();
			if (commodity!=null && !commodity.prune().hasData()) commodity = null;
			if (transferorTransferee!=null && !transferorTransferee.prune().hasData()) transferorTransferee = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCommodity()!=null && getCommodity().hasData()) return true;
			if (getQuantity()!=null) return true;
			if (getTimeUnit()!=null) return true;
			if (getTransferorTransferee()!=null && getTransferorTransferee().hasData()) return true;
			if (getUnit()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
			processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
			processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
			
			processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
			processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CommodityTransferBreakdownBuilder _that = (CommodityTransferBreakdownBuilder) o;
		
			if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
			if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
			_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CommodityTransferBreakdownBuilder {" +
				"commodity=" + this.commodity + ", " +
				"quantity=" + this.quantity + ", " +
				"timeUnit=" + this.timeUnit + ", " +
				"transferorTransferee=" + this.transferorTransferee + ", " +
				"unit=" + this.unit +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
		processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
		processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
		
		processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
		processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CommodityTransferBreakdown _that = (CommodityTransferBreakdown) o;
	
		if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
		if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
		_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CommodityTransferBreakdown {" +
			"commodity=" + this.commodity + ", " +
			"quantity=" + this.quantity + ", " +
			"timeUnit=" + this.timeUnit + ", " +
			"transferorTransferee=" + this.transferorTransferee + ", " +
			"unit=" + this.unit +
		'}' + " " + super.toString();
	}
}
