package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AssetTransferTypeEnum;
import org.isda.cdm.Commodity;
import org.isda.cdm.CommodityTransferBreakdown;
import org.isda.cdm.TimeUnitEnum;
import org.isda.cdm.TransferorTransferee;
import org.isda.cdm.UnitEnum;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CommodityTransferComponentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CommodityTransferComponent extends TransferBase {
	private final AssetTransferTypeEnum assetTransferType;
	private final List<CommodityTransferBreakdown> breakdown;
	private final Commodity commodity;
	private final BigDecimal quantity;
	private final TimeUnitEnum timeUnit;
	private final TransferorTransferee transferorTransferee;
	private final UnitEnum unit;
	private static CommodityTransferComponentMeta metaData = new CommodityTransferComponentMeta();
	
	CommodityTransferComponent(CommodityTransferComponentBuilder builder) {
		super(builder);
		this.assetTransferType = builder.getAssetTransferType();
		this.breakdown = ofNullable(builder.getBreakdown()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.commodity = ofNullable(builder.getCommodity()).map(f->f.build()).orElse(null);
		this.quantity = builder.getQuantity();
		this.timeUnit = builder.getTimeUnit();
		this.transferorTransferee = ofNullable(builder.getTransferorTransferee()).map(f->f.build()).orElse(null);
		this.unit = builder.getUnit();
	}
	
	/**
	 * The type of transfer, e.g. Free of Payment.
	 */
	public final AssetTransferTypeEnum getAssetTransferType() {
		return assetTransferType;
	}
	
	/**
	 * The security transfer breakdown, when the transfer corresponds to a net transfer across several components which breakdown is deemed relevant (e.g. the net security transfer related to several contracts).
	 */
	@RosettaSynonym(value="breakdown", source="Rosetta_Workbench")
	public final List<CommodityTransferBreakdown> getBreakdown() {
		return breakdown;
	}
	
	public final Commodity getCommodity() {
		return commodity;
	}
	
	@RosettaSynonym(value="quantity", source="Rosetta_Workbench")
	public final BigDecimal getQuantity() {
		return quantity;
	}
	
	public final TimeUnitEnum getTimeUnit() {
		return timeUnit;
	}
	
	/**
	 * The transferee and transferor party information.
	 */
	public final TransferorTransferee getTransferorTransferee() {
		return transferorTransferee;
	}
	
	/**
	 * The unit of measure, applicable to physical assets.
	 */
	public final UnitEnum getUnit() {
		return unit;
	}
	
	
	@Override
	public RosettaMetaData<? extends CommodityTransferComponent> metaData() {
		return metaData;
	} 
	
	public CommodityTransferComponentBuilder toBuilder() {
		CommodityTransferComponentBuilder builder = new CommodityTransferComponentBuilder();
		ofNullable(getAssetTransferType()).ifPresent(builder::setAssetTransferType);
		ofNullable(getBreakdown()).ifPresent(breakdown -> breakdown.forEach(builder::addBreakdown));
		ofNullable(getCommodity()).ifPresent(builder::setCommodity);
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getTimeUnit()).ifPresent(builder::setTimeUnit);
		ofNullable(getTransferorTransferee()).ifPresent(builder::setTransferorTransferee);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static CommodityTransferComponentBuilder builder() {
		return new CommodityTransferComponentBuilder();
	}

	public static  class CommodityTransferComponentBuilder extends TransferBaseBuilder {
	
		protected AssetTransferTypeEnum assetTransferType;
		protected List<CommodityTransferBreakdown.CommodityTransferBreakdownBuilder> breakdown;
		protected Commodity.CommodityBuilder commodity;
		protected BigDecimal quantity;
		protected TimeUnitEnum timeUnit;
		protected TransferorTransferee.TransferorTransfereeBuilder transferorTransferee;
		protected UnitEnum unit;
	
		public CommodityTransferComponentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CommodityTransferComponent> metaData() {
			return metaData;
		} 
	
		
		public AssetTransferTypeEnum getAssetTransferType() {
			return assetTransferType;
		}
		
		
		public List<CommodityTransferBreakdown.CommodityTransferBreakdownBuilder> getBreakdown() {
			return breakdown;
		}
		
		public CommodityTransferBreakdown.CommodityTransferBreakdownBuilder getOrCreateBreakdown(int index) {
			if (breakdown==null) {
				this.breakdown = new ArrayList<>();
			}
			return getIndex(breakdown, index, ()->new CommodityTransferBreakdown.CommodityTransferBreakdownBuilder());
		}
		
		
		public Commodity.CommodityBuilder getCommodity() {
			return commodity;
		}
		
		public Commodity.CommodityBuilder getOrCreateCommodity() {
			if (commodity!=null) {
				return commodity;
			}
			else return commodity = new Commodity.CommodityBuilder();
		}
		
		
		public BigDecimal getQuantity() {
			return quantity;
		}
		
		
		public TimeUnitEnum getTimeUnit() {
			return timeUnit;
		}
		
		
		public TransferorTransferee.TransferorTransfereeBuilder getTransferorTransferee() {
			return transferorTransferee;
		}
		
		public TransferorTransferee.TransferorTransfereeBuilder getOrCreateTransferorTransferee() {
			if (transferorTransferee!=null) {
				return transferorTransferee;
			}
			else return transferorTransferee = new TransferorTransferee.TransferorTransfereeBuilder();
		}
		
		
		public UnitEnum getUnit() {
			return unit;
		}
		
	
		public CommodityTransferComponentBuilder setAssetTransferType(AssetTransferTypeEnum assetTransferType) {
			this.assetTransferType = assetTransferType;
			return this;
		}
		public CommodityTransferComponentBuilder addBreakdown(CommodityTransferBreakdown breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown.toBuilder());
			return this;
		}
		public CommodityTransferComponentBuilder addBreakdown(CommodityTransferBreakdown breakdown, int _idx) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			getIndex(this.breakdown, _idx, () -> breakdown.toBuilder());
			this.breakdown.set(_idx, breakdown.toBuilder());
			return this;
		}
		public CommodityTransferComponentBuilder addBreakdown(List<CommodityTransferBreakdown> breakdowns) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			for (CommodityTransferBreakdown toAdd : breakdowns) {
				this.breakdown.add(toAdd.toBuilder());
			}
			return this;
		}
		public CommodityTransferComponentBuilder addBreakdownBuilder(CommodityTransferBreakdown.CommodityTransferBreakdownBuilder breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown);
			return this;
		}
		
		public CommodityTransferComponentBuilder clearBreakdown() {
			this.breakdown = null;
			return this;
		}
		public CommodityTransferComponentBuilder setCommodity(Commodity commodity) {
			this.commodity = commodity.toBuilder();
			return this;
		}
		public CommodityTransferComponentBuilder setCommodityBuilder(Commodity.CommodityBuilder commodity) {
			this.commodity = commodity;
			return this;
		}
		public CommodityTransferComponentBuilder setQuantity(BigDecimal quantity) {
			this.quantity = quantity;
			return this;
		}
		public CommodityTransferComponentBuilder setTimeUnit(TimeUnitEnum timeUnit) {
			this.timeUnit = timeUnit;
			return this;
		}
		public CommodityTransferComponentBuilder setTransferorTransferee(TransferorTransferee transferorTransferee) {
			this.transferorTransferee = transferorTransferee.toBuilder();
			return this;
		}
		public CommodityTransferComponentBuilder setTransferorTransfereeBuilder(TransferorTransferee.TransferorTransfereeBuilder transferorTransferee) {
			this.transferorTransferee = transferorTransferee;
			return this;
		}
		public CommodityTransferComponentBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		@Override public CommodityTransferComponentBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public CommodityTransferComponentBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public CommodityTransferComponentBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public CommodityTransferComponentBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public CommodityTransferComponent build() {
			return new CommodityTransferComponent(this);
		}
	
		@Override
		public CommodityTransferComponentBuilder prune() {
			super.prune();
			if (breakdown!=null) breakdown = breakdown.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (commodity!=null && !commodity.prune().hasData()) commodity = null;
			if (transferorTransferee!=null && !transferorTransferee.prune().hasData()) transferorTransferee = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAssetTransferType()!=null) return true;
			if (getBreakdown()!=null && getBreakdown().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCommodity()!=null && getCommodity().hasData()) return true;
			if (getQuantity()!=null) return true;
			if (getTimeUnit()!=null) return true;
			if (getTransferorTransferee()!=null && getTransferorTransferee().hasData()) return true;
			if (getUnit()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("assetTransferType"), AssetTransferTypeEnum.class, assetTransferType, this);
			processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
			processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
			processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
			
			processRosetta(path.newSubPath("breakdown"), processor, CommodityTransferBreakdown.class, breakdown);
			processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
			processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			CommodityTransferComponentBuilder _that = (CommodityTransferComponentBuilder) o;
		
			if (assetTransferType != null ? !assetTransferType.equals(_that.assetTransferType) : _that.assetTransferType != null) return false;
			if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
			if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
			if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (assetTransferType != null ? assetTransferType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
			_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
			_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CommodityTransferComponentBuilder {" +
				"assetTransferType=" + this.assetTransferType + ", " +
				"breakdown=" + this.breakdown + ", " +
				"commodity=" + this.commodity + ", " +
				"quantity=" + this.quantity + ", " +
				"timeUnit=" + this.timeUnit + ", " +
				"transferorTransferee=" + this.transferorTransferee + ", " +
				"unit=" + this.unit +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("assetTransferType"), AssetTransferTypeEnum.class, assetTransferType, this);
		processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
		processor.processBasic(path.newSubPath("timeUnit"), TimeUnitEnum.class, timeUnit, this);
		processor.processBasic(path.newSubPath("unit"), UnitEnum.class, unit, this);
		
		processRosetta(path.newSubPath("breakdown"), processor, CommodityTransferBreakdown.class, breakdown);
		processRosetta(path.newSubPath("commodity"), processor, Commodity.class, commodity);
		processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		CommodityTransferComponent _that = (CommodityTransferComponent) o;
	
		if (assetTransferType != null ? !assetTransferType.equals(_that.assetTransferType) : _that.assetTransferType != null) return false;
		if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
		if (commodity != null ? !commodity.equals(_that.commodity) : _that.commodity != null) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (timeUnit != null ? !timeUnit.equals(_that.timeUnit) : _that.timeUnit != null) return false;
		if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (assetTransferType != null ? assetTransferType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
		_result = 31 * _result + (commodity != null ? commodity.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (timeUnit != null ? timeUnit.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
		_result = 31 * _result + (unit != null ? unit.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CommodityTransferComponent {" +
			"assetTransferType=" + this.assetTransferType + ", " +
			"breakdown=" + this.breakdown + ", " +
			"commodity=" + this.commodity + ", " +
			"quantity=" + this.quantity + ", " +
			"timeUnit=" + this.timeUnit + ", " +
			"transferorTransferee=" + this.transferorTransferee + ", " +
			"unit=" + this.unit +
		'}' + " " + super.toString();
	}
}
