package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of compounding, e.g. flat, straight.
 * 
 * @version 2.5.4
 */
public enum CompoundingMethodEnum {
    

	/**
	 * Flat compounding. Compounding excludes the spread. Note that the first compounding period has it&#39;s interest calculated including any spread then subsequent periods compound this at a rate excluding the spread.
	 */
	@RosettaSynonym(value = "Flat", source = "FpML_5_10")
	@RosettaSynonym(value = "Flat", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Flat", source = "DTCC_11_0")
	@RosettaSynonym(value = "Flat", source = "DTCC_9_0")
	@RosettaSynonym(value = "Flat", source = "CME_ClearedConfirm_1_17")
	FLAT,
	    

	/**
	 * No compounding is to be applied.
	 */
	@RosettaSynonym(value = "None", source = "FpML_5_10")
	@RosettaSynonym(value = "None", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "None", source = "DTCC_11_0")
	@RosettaSynonym(value = "None", source = "DTCC_9_0")
	@RosettaSynonym(value = "None", source = "CME_ClearedConfirm_1_17")
	NONE,
	    

	/**
	 * Straight compounding. Compounding includes the spread.
	 */
	@RosettaSynonym(value = "Straight", source = "FpML_5_10")
	@RosettaSynonym(value = "Straight", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Straight", source = "DTCC_11_0")
	@RosettaSynonym(value = "Straight", source = "DTCC_9_0")
	@RosettaSynonym(value = "Straight", source = "CME_ClearedConfirm_1_17")
	STRAIGHT,
	    

	/**
	 * Spread Exclusive compounding.
	 */
	@RosettaSynonym(value = "SpreadExclusive", source = "FpML_5_10")
	@RosettaSynonym(value = "SpreadExclusive", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SpreadExclusive", source = "DTCC_11_0")
	@RosettaSynonym(value = "SpreadExclusive", source = "DTCC_9_0")
	@RosettaSynonym(value = "SpreadExclusive", source = "CME_ClearedConfirm_1_17")
	SPREAD_EXCLUSIVE    
	;

	private final String displayName;
	
	CompoundingMethodEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
