package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ConditionsElections;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ConditionsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify each party&#39;s election with respect to the Termination Events that will be deemed an Access Condition (Initial Margin CSA) or a Specified Condition (Variation Margin CSA). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (f)(ii). | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(ii). | ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (e): Conditions Precedent and Secured Party’s Rights and Remedies.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Conditions extends RosettaModelObject {
	private final List<ConditionsElections> partyElection;
	private static ConditionsMeta metaData = new ConditionsMeta();
	
	Conditions(ConditionsBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; Access Condition (Initial Margin CSA) or a Specified Condition (Variation Margin CSA) election.
	 */
	public final List<ConditionsElections> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends Conditions> metaData() {
		return metaData;
	} 
	
	public ConditionsBuilder toBuilder() {
		ConditionsBuilder builder = new ConditionsBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static ConditionsBuilder builder() {
		return new ConditionsBuilder();
	}

	public static  class ConditionsBuilder extends RosettaModelObjectBuilder {
	
		protected List<ConditionsElections.ConditionsElectionsBuilder> partyElection;
	
		public ConditionsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Conditions> metaData() {
			return metaData;
		} 
	
		
		public List<ConditionsElections.ConditionsElectionsBuilder> getPartyElection() {
			return partyElection;
		}
		
		public ConditionsElections.ConditionsElectionsBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new ConditionsElections.ConditionsElectionsBuilder());
		}
		
	
		public ConditionsBuilder addPartyElection(ConditionsElections partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public ConditionsBuilder addPartyElection(ConditionsElections partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public ConditionsBuilder addPartyElection(List<ConditionsElections> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (ConditionsElections toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public ConditionsBuilder addPartyElectionBuilder(ConditionsElections.ConditionsElectionsBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public ConditionsBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public Conditions build() {
			return new Conditions(this);
		}
	
		@Override
		public ConditionsBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, ConditionsElections.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ConditionsBuilder _that = (ConditionsBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ConditionsBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, ConditionsElections.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Conditions _that = (Conditions) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Conditions {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
