package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ConfirmationStatusEnum;
import org.isda.cdm.Identifier;
import org.isda.cdm.Lineage;
import org.isda.cdm.Party;
import org.isda.cdm.PartyRole;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ConfirmationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a trade confirmation.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Confirmation extends RosettaModelObject {
	private final List<Identifier> identifier;
	private final Lineage lineage;
	private final List<Party> party;
	private final List<PartyRole> partyRole;
	private final ConfirmationStatusEnum status;
	private static ConfirmationMeta metaData = new ConfirmationMeta();
	
	Confirmation(ConfirmationBuilder builder) {
		this.identifier = ofNullable(builder.getIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.lineage = ofNullable(builder.getLineage()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyRole = ofNullable(builder.getPartyRole()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.status = builder.getStatus();
	}
	
	/**
	 * The identifier(s) associated with the trade and resulting confirmation.
	 */
	public final List<Identifier> getIdentifier() {
		return identifier;
	}
	
	/**
	 * The lineage attribute provides a linkage to previous lifecycle events and associated data.
	 */
	public final Lineage getLineage() {
		return lineage;
	}
	
	/**
	 * The parties associated with the trade.
	 */
	public final List<Party> getParty() {
		return party;
	}
	
	/**
	 * The role(s) that party(ies) may have in relation to the trade
	 */
	public final List<PartyRole> getPartyRole() {
		return partyRole;
	}
	
	public final ConfirmationStatusEnum getStatus() {
		return status;
	}
	
	
	@Override
	public RosettaMetaData<? extends Confirmation> metaData() {
		return metaData;
	} 
	
	public ConfirmationBuilder toBuilder() {
		ConfirmationBuilder builder = new ConfirmationBuilder();
		ofNullable(getIdentifier()).ifPresent(identifier -> identifier.forEach(builder::addIdentifier));
		ofNullable(getLineage()).ifPresent(builder::setLineage);
		ofNullable(getParty()).ifPresent(party -> party.forEach(builder::addParty));
		ofNullable(getPartyRole()).ifPresent(partyRole -> partyRole.forEach(builder::addPartyRole));
		ofNullable(getStatus()).ifPresent(builder::setStatus);
		return builder;
	}

	public static ConfirmationBuilder builder() {
		return new ConfirmationBuilder();
	}

	public static  class ConfirmationBuilder extends RosettaModelObjectBuilder {
	
		protected List<Identifier.IdentifierBuilder> identifier;
		protected Lineage.LineageBuilder lineage;
		protected List<Party.PartyBuilder> party;
		protected List<PartyRole.PartyRoleBuilder> partyRole;
		protected ConfirmationStatusEnum status;
	
		public ConfirmationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Confirmation> metaData() {
			return metaData;
		} 
	
		
		public List<Identifier.IdentifierBuilder> getIdentifier() {
			return identifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateIdentifier(int index) {
			if (identifier==null) {
				this.identifier = new ArrayList<>();
			}
			return getIndex(identifier, index, ()->new Identifier.IdentifierBuilder());
		}
		
		
		public Lineage.LineageBuilder getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage() {
			if (lineage!=null) {
				return lineage;
			}
			else return lineage = new Lineage.LineageBuilder();
		}
		
		
		public List<Party.PartyBuilder> getParty() {
			return party;
		}
		
		public Party.PartyBuilder getOrCreateParty(int index) {
			if (party==null) {
				this.party = new ArrayList<>();
			}
			return getIndex(party, index, ()->new Party.PartyBuilder());
		}
		
		
		public List<PartyRole.PartyRoleBuilder> getPartyRole() {
			return partyRole;
		}
		
		public PartyRole.PartyRoleBuilder getOrCreatePartyRole(int index) {
			if (partyRole==null) {
				this.partyRole = new ArrayList<>();
			}
			return getIndex(partyRole, index, ()->new PartyRole.PartyRoleBuilder());
		}
		
		
		public ConfirmationStatusEnum getStatus() {
			return status;
		}
		
	
		public ConfirmationBuilder addIdentifier(Identifier identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		public ConfirmationBuilder addIdentifier(Identifier identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		public ConfirmationBuilder addIdentifier(List<Identifier> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (Identifier toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public ConfirmationBuilder addIdentifierBuilder(Identifier.IdentifierBuilder identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier);
			return this;
		}
		
		public ConfirmationBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		public ConfirmationBuilder setLineage(Lineage lineage) {
			this.lineage = lineage.toBuilder();
			return this;
		}
		public ConfirmationBuilder setLineageBuilder(Lineage.LineageBuilder lineage) {
			this.lineage = lineage;
			return this;
		}
		public ConfirmationBuilder addParty(Party party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party.toBuilder());
			return this;
		}
		public ConfirmationBuilder addParty(Party party, int _idx) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			getIndex(this.party, _idx, () -> party.toBuilder());
			this.party.set(_idx, party.toBuilder());
			return this;
		}
		public ConfirmationBuilder addParty(List<Party> partys) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			for (Party toAdd : partys) {
				this.party.add(toAdd.toBuilder());
			}
			return this;
		}
		public ConfirmationBuilder addPartyBuilder(Party.PartyBuilder party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party);
			return this;
		}
		
		public ConfirmationBuilder clearParty() {
			this.party = null;
			return this;
		}
		public ConfirmationBuilder addPartyRole(PartyRole partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole.toBuilder());
			return this;
		}
		public ConfirmationBuilder addPartyRole(PartyRole partyRole, int _idx) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			getIndex(this.partyRole, _idx, () -> partyRole.toBuilder());
			this.partyRole.set(_idx, partyRole.toBuilder());
			return this;
		}
		public ConfirmationBuilder addPartyRole(List<PartyRole> partyRoles) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			for (PartyRole toAdd : partyRoles) {
				this.partyRole.add(toAdd.toBuilder());
			}
			return this;
		}
		public ConfirmationBuilder addPartyRoleBuilder(PartyRole.PartyRoleBuilder partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole);
			return this;
		}
		
		public ConfirmationBuilder clearPartyRole() {
			this.partyRole = null;
			return this;
		}
		public ConfirmationBuilder setStatus(ConfirmationStatusEnum status) {
			this.status = status;
			return this;
		}
		
		public Confirmation build() {
			return new Confirmation(this);
		}
	
		@Override
		public ConfirmationBuilder prune() {
			if (identifier!=null) identifier = identifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (lineage!=null && !lineage.prune().hasData()) lineage = null;
			if (party!=null) party = party.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyRole!=null) partyRole = partyRole.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null && getIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getLineage()!=null && getLineage().hasData()) return true;
			if (getParty()!=null && getParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyRole()!=null && getPartyRole().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getStatus()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("status"), ConfirmationStatusEnum.class, status, this);
			
			processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
			processRosetta(path.newSubPath("party"), processor, Party.class, party);
			processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ConfirmationBuilder _that = (ConfirmationBuilder) o;
		
			if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
			if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
			if (!ListEquals.listEquals(party, _that.party)) return false;
			if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
			if (status != null ? !status.equals(_that.status) : _that.status != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
			_result = 31 * _result + (status != null ? status.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ConfirmationBuilder {" +
				"identifier=" + this.identifier + ", " +
				"lineage=" + this.lineage + ", " +
				"party=" + this.party + ", " +
				"partyRole=" + this.partyRole + ", " +
				"status=" + this.status +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("status"), ConfirmationStatusEnum.class, status, this);
		
		processRosetta(path.newSubPath("identifier"), processor, Identifier.class, identifier);
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		processRosetta(path.newSubPath("party"), processor, Party.class, party);
		processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Confirmation _that = (Confirmation) o;
	
		if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
		if (lineage != null ? !lineage.equals(_that.lineage) : _that.lineage != null) return false;
		if (!ListEquals.listEquals(party, _that.party)) return false;
		if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
		if (status != null ? !status.equals(_that.status) : _that.status != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
		_result = 31 * _result + (status != null ? status.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Confirmation {" +
			"identifier=" + this.identifier + ", " +
			"lineage=" + this.lineage + ", " +
			"party=" + this.party + ", " +
			"partyRole=" + this.partyRole + ", " +
			"status=" + this.status +
		'}';
	}
}
