package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PartyContactInformation;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContactElectionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the parties&#39; election to specify contact information, in relation to elections such as the Addresses for Transfer or the Demand and Notices as specified in the ISDA Credit Support Annex agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ContactElection extends RosettaModelObject {
	private final List<PartyContactInformation> partyElection;
	private static ContactElectionMeta metaData = new ContactElectionMeta();
	
	ContactElection(ContactElectionBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; contact information election.
	 */
	public final List<PartyContactInformation> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContactElection> metaData() {
		return metaData;
	} 
	
	public ContactElectionBuilder toBuilder() {
		ContactElectionBuilder builder = new ContactElectionBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static ContactElectionBuilder builder() {
		return new ContactElectionBuilder();
	}

	public static  class ContactElectionBuilder extends RosettaModelObjectBuilder {
	
		protected List<PartyContactInformation.PartyContactInformationBuilder> partyElection;
	
		public ContactElectionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContactElection> metaData() {
			return metaData;
		} 
	
		
		public List<PartyContactInformation.PartyContactInformationBuilder> getPartyElection() {
			return partyElection;
		}
		
		public PartyContactInformation.PartyContactInformationBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new PartyContactInformation.PartyContactInformationBuilder());
		}
		
	
		public ContactElectionBuilder addPartyElection(PartyContactInformation partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public ContactElectionBuilder addPartyElection(PartyContactInformation partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public ContactElectionBuilder addPartyElection(List<PartyContactInformation> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (PartyContactInformation toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContactElectionBuilder addPartyElectionBuilder(PartyContactInformation.PartyContactInformationBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public ContactElectionBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public ContactElection build() {
			return new ContactElection(this);
		}
	
		@Override
		public ContactElectionBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, PartyContactInformation.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContactElectionBuilder _that = (ContactElectionBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContactElectionBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, PartyContactInformation.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContactElection _that = (ContactElection) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContactElection {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
