package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Address;
import org.isda.cdm.TelephoneNumber;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContactInformationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify contact information associated with a party: telephone, postal/street address, email and web page.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ContactInformation", source="FpML_5_10")
public class ContactInformation extends RosettaModelObject {
	private final List<Address> address;
	private final List<String> email;
	private final List<TelephoneNumber> telephone;
	private final List<String> webPage;
	private static ContactInformationMeta metaData = new ContactInformationMeta();
	
	ContactInformation(ContactInformationBuilder builder) {
		this.address = ofNullable(builder.getAddress()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.email = builder.getEmail();
		this.telephone = ofNullable(builder.getTelephone()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.webPage = builder.getWebPage();
	}
	
	/**
	 * The street/postal address.
	 */
	@RosettaSynonym(value="address", source="FpML_5_10")
	public final List<Address> getAddress() {
		return address;
	}
	
	/**
	 * The email address.
	 */
	@RosettaSynonym(value="email", source="FpML_5_10")
	public final List<String> getEmail() {
		return email;
	}
	
	/**
	 * The telephone number.
	 */
	@RosettaSynonym(value="telephone", source="FpML_5_10")
	public final List<TelephoneNumber> getTelephone() {
		return telephone;
	}
	
	/**
	 * The web page. This attribute is not specified as part of the FpML ContactInformation complex type.
	 */
	public final List<String> getWebPage() {
		return webPage;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContactInformation> metaData() {
		return metaData;
	} 
	
	public ContactInformationBuilder toBuilder() {
		ContactInformationBuilder builder = new ContactInformationBuilder();
		ofNullable(getAddress()).ifPresent(address -> address.forEach(builder::addAddress));
		ofNullable(getEmail()).ifPresent(email -> email.forEach(builder::addEmail));
		ofNullable(getTelephone()).ifPresent(telephone -> telephone.forEach(builder::addTelephone));
		ofNullable(getWebPage()).ifPresent(webPage -> webPage.forEach(builder::addWebPage));
		return builder;
	}

	public static ContactInformationBuilder builder() {
		return new ContactInformationBuilder();
	}

	public static  class ContactInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<Address.AddressBuilder> address;
		protected List<String> email;
		protected List<TelephoneNumber.TelephoneNumberBuilder> telephone;
		protected List<String> webPage;
	
		public ContactInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContactInformation> metaData() {
			return metaData;
		} 
	
		
		public List<Address.AddressBuilder> getAddress() {
			return address;
		}
		
		public Address.AddressBuilder getOrCreateAddress(int index) {
			if (address==null) {
				this.address = new ArrayList<>();
			}
			return getIndex(address, index, ()->new Address.AddressBuilder());
		}
		
		
		public List<String> getEmail() {
			return email;
		}
		
		
		public List<TelephoneNumber.TelephoneNumberBuilder> getTelephone() {
			return telephone;
		}
		
		public TelephoneNumber.TelephoneNumberBuilder getOrCreateTelephone(int index) {
			if (telephone==null) {
				this.telephone = new ArrayList<>();
			}
			return getIndex(telephone, index, ()->new TelephoneNumber.TelephoneNumberBuilder());
		}
		
		
		public List<String> getWebPage() {
			return webPage;
		}
		
	
		public ContactInformationBuilder addAddress(Address address) {
			if(this.address == null){
				this.address = new ArrayList<>();
			}
			this.address.add(address.toBuilder());
			return this;
		}
		public ContactInformationBuilder addAddress(Address address, int _idx) {
			if(this.address == null){
				this.address = new ArrayList<>();
			}
			getIndex(this.address, _idx, () -> address.toBuilder());
			this.address.set(_idx, address.toBuilder());
			return this;
		}
		public ContactInformationBuilder addAddress(List<Address> addresss) {
			if(this.address == null){
				this.address = new ArrayList<>();
			}
			for (Address toAdd : addresss) {
				this.address.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContactInformationBuilder addAddressBuilder(Address.AddressBuilder address) {
			if(this.address == null){
				this.address = new ArrayList<>();
			}
			this.address.add(address);
			return this;
		}
		
		public ContactInformationBuilder clearAddress() {
			this.address = null;
			return this;
		}
		public ContactInformationBuilder addEmail(String email) {
			if(this.email == null){
				this.email = new ArrayList<>();
			}
			this.email.add(email);
			return this;
		}
		public ContactInformationBuilder addEmail(String email, int _idx) {
			if(this.email == null){
				this.email = new ArrayList<>();
			}
			getIndex(this.email, _idx, () -> email);
			this.email.set(_idx, email);
			return this;
		}
		public ContactInformationBuilder addEmail(List<String> emails) {
			if(this.email == null){
				this.email = new ArrayList<>();
			}
			for (String toAdd : emails) {
				this.email.add(toAdd);
			}
			return this;
		}
		
		public ContactInformationBuilder clearEmail() {
			this.email = null;
			return this;
		}
		public ContactInformationBuilder addTelephone(TelephoneNumber telephone) {
			if(this.telephone == null){
				this.telephone = new ArrayList<>();
			}
			this.telephone.add(telephone.toBuilder());
			return this;
		}
		public ContactInformationBuilder addTelephone(TelephoneNumber telephone, int _idx) {
			if(this.telephone == null){
				this.telephone = new ArrayList<>();
			}
			getIndex(this.telephone, _idx, () -> telephone.toBuilder());
			this.telephone.set(_idx, telephone.toBuilder());
			return this;
		}
		public ContactInformationBuilder addTelephone(List<TelephoneNumber> telephones) {
			if(this.telephone == null){
				this.telephone = new ArrayList<>();
			}
			for (TelephoneNumber toAdd : telephones) {
				this.telephone.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContactInformationBuilder addTelephoneBuilder(TelephoneNumber.TelephoneNumberBuilder telephone) {
			if(this.telephone == null){
				this.telephone = new ArrayList<>();
			}
			this.telephone.add(telephone);
			return this;
		}
		
		public ContactInformationBuilder clearTelephone() {
			this.telephone = null;
			return this;
		}
		public ContactInformationBuilder addWebPage(String webPage) {
			if(this.webPage == null){
				this.webPage = new ArrayList<>();
			}
			this.webPage.add(webPage);
			return this;
		}
		public ContactInformationBuilder addWebPage(String webPage, int _idx) {
			if(this.webPage == null){
				this.webPage = new ArrayList<>();
			}
			getIndex(this.webPage, _idx, () -> webPage);
			this.webPage.set(_idx, webPage);
			return this;
		}
		public ContactInformationBuilder addWebPage(List<String> webPages) {
			if(this.webPage == null){
				this.webPage = new ArrayList<>();
			}
			for (String toAdd : webPages) {
				this.webPage.add(toAdd);
			}
			return this;
		}
		
		public ContactInformationBuilder clearWebPage() {
			this.webPage = null;
			return this;
		}
		
		public ContactInformation build() {
			return new ContactInformation(this);
		}
	
		@Override
		public ContactInformationBuilder prune() {
			if (address!=null) address = address.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (telephone!=null) telephone = telephone.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAddress()!=null && getAddress().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEmail()!=null && !getEmail().isEmpty()) return true;
			if (getTelephone()!=null && getTelephone().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getWebPage()!=null && !getWebPage().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("email"), String.class, email, this);
			processor.processBasic(path.newSubPath("webPage"), String.class, webPage, this);
			
			processRosetta(path.newSubPath("address"), processor, Address.class, address);
			processRosetta(path.newSubPath("telephone"), processor, TelephoneNumber.class, telephone);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContactInformationBuilder _that = (ContactInformationBuilder) o;
		
			if (!ListEquals.listEquals(address, _that.address)) return false;
			if (!ListEquals.listEquals(email, _that.email)) return false;
			if (!ListEquals.listEquals(telephone, _that.telephone)) return false;
			if (!ListEquals.listEquals(webPage, _that.webPage)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (address != null ? address.hashCode() : 0);
			_result = 31 * _result + (email != null ? email.hashCode() : 0);
			_result = 31 * _result + (telephone != null ? telephone.hashCode() : 0);
			_result = 31 * _result + (webPage != null ? webPage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContactInformationBuilder {" +
				"address=" + this.address + ", " +
				"email=" + this.email + ", " +
				"telephone=" + this.telephone + ", " +
				"webPage=" + this.webPage +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		email.stream().forEach(a->processor.processBasic(path.newSubPath("email"), String.class, a, this));
		webPage.stream().forEach(a->processor.processBasic(path.newSubPath("webPage"), String.class, a, this));
		
		processRosetta(path.newSubPath("address"), processor, Address.class, address);
		processRosetta(path.newSubPath("telephone"), processor, TelephoneNumber.class, telephone);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContactInformation _that = (ContactInformation) o;
	
		if (!ListEquals.listEquals(address, _that.address)) return false;
		if (!ListEquals.listEquals(email, _that.email)) return false;
		if (!ListEquals.listEquals(telephone, _that.telephone)) return false;
		if (!ListEquals.listEquals(webPage, _that.webPage)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (address != null ? address.hashCode() : 0);
		_result = 31 * _result + (email != null ? email.hashCode() : 0);
		_result = 31 * _result + (telephone != null ? telephone.hashCode() : 0);
		_result = 31 * _result + (webPage != null ? webPage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContactInformation {" +
			"address=" + this.address + ", " +
			"email=" + this.email + ", " +
			"telephone=" + this.telephone + ", " +
			"webPage=" + this.webPage +
		'}';
	}
}
