package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Account;
import org.isda.cdm.CalculationAgent;
import org.isda.cdm.ClosedState;
import org.isda.cdm.Collateral;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.Documentation;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.Identifier;
import org.isda.cdm.Party;
import org.isda.cdm.PartyContractInformation;
import org.isda.cdm.PartyRole;
import org.isda.cdm.TradeDate;
import org.isda.cdm.metafields.FieldWithMetaGoverningLawEnum;
import org.isda.cdm.metafields.FieldWithMetaGoverningLawEnum.FieldWithMetaGoverningLawEnumBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * A class to specify a financial contract object, which can be invoked either within the context of an event, or independently from it. It corresponds to the FpML Trade in the Confirmation view (while the CDM Execution class corresponds to the FpML trade in the Pre-Trade view). The associated rosettaKey denotes the ability to associate a hash value to the Contract instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Contract extends RosettaModelObject implements GlobalKey {
	private final List<Account> account;
	private final CalculationAgent calculationAgent;
	private final Date clearedDate;
	private final ClosedState closedState;
	private final Collateral collateral;
	private final List<Identifier> contractIdentifier;
	private final ContractualProduct contractualProduct;
	private final ExecutionQuantity contractualQuantity;
	private final Documentation documentation;
	private final FieldWithMetaGoverningLawEnum governingLaw;
	private final MetaFields meta;
	private final List<Party> party;
	private final List<PartyContractInformation> partyContractInformation;
	private final List<PartyRole> partyRole;
	private final TradeDate tradeDate;
	private static ContractMeta metaData = new ContractMeta();
	
	Contract(ContractBuilder builder) {
		this.account = ofNullable(builder.getAccount()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.calculationAgent = ofNullable(builder.getCalculationAgent()).map(f->f.build()).orElse(null);
		this.clearedDate = builder.getClearedDate();
		this.closedState = ofNullable(builder.getClosedState()).map(f->f.build()).orElse(null);
		this.collateral = ofNullable(builder.getCollateral()).map(f->f.build()).orElse(null);
		this.contractIdentifier = ofNullable(builder.getContractIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contractualProduct = ofNullable(builder.getContractualProduct()).map(f->f.build()).orElse(null);
		this.contractualQuantity = ofNullable(builder.getContractualQuantity()).map(f->f.build()).orElse(null);
		this.documentation = ofNullable(builder.getDocumentation()).map(f->f.build()).orElse(null);
		this.governingLaw = ofNullable(builder.getGoverningLaw()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.party = ofNullable(builder.getParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyContractInformation = ofNullable(builder.getPartyContractInformation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyRole = ofNullable(builder.getPartyRole()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.tradeDate = ofNullable(builder.getTradeDate()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Optional account information.
	 */
	@RosettaSynonym(value="account", source="FpML_5_10")
	@RosettaSynonym(value="account", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="account", source="DTCC_11_0")
	@RosettaSynonym(value="account", source="DTCC_9_0")
	public final List<Account> getAccount() {
		return account;
	}
	
	/**
	 * The ISDA calculation agent responsible for performing duties as defined in the applicable product definitions.
	 */
	public final CalculationAgent getCalculationAgent() {
		return calculationAgent;
	}
	
	/**
	 * If the trade was cleared (novated) through a central counterparty clearing service, this represents the date the trade was cleared (transferred to the central counterparty).
	 */
	@RosettaSynonym(value="clearedDate", source="FpML_5_10", path="trade.tradeHeader")
	@RosettaSynonym(value="clearedDate", source="DTCC_11_0", path="trade.tradeHeader")
	@RosettaSynonym(value="clearedDate", source="DTCC_9_0", path="trade.tradeHeader")
	@RosettaSynonym(value="clearedDate", source="CME_SubmissionIRS_1_0", path="trade.tradeHeader")
	@RosettaSynonym(value="clearedDate", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	@RosettaSynonym(value="clearedDate", source="FpML_5_10", path="tradeHeader")
	@RosettaSynonym(value="clearedDate", source="CME_ClearedConfirm_1_17", path="tradeHeader")
	public final Date getClearedDate() {
		return clearedDate;
	}
	
	/**
	 * The qualification of what led to the contract closure alongside with the dates on which this closure took effect.
	 */
	@RosettaSynonym(value="closedState", source="Rosetta_Workbench")
	public final ClosedState getClosedState() {
		return closedState;
	}
	
	/**
	 * Defines the collateral obligations of a party.
	 */
	@RosettaSynonym(value="collateral", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="collateral", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="collateral", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="collateral", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="collateral", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="collateral", source="FpML_5_10")
	@RosettaSynonym(value="collateral", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="collateral", source="CME_ClearedConfirm_1_17")
	public final Collateral getCollateral() {
		return collateral;
	}
	
	/**
	 * The identifier(s) that uniquely identify a contract, and which can be qualified by referencing or defining the issuer(s) of such identifier(s). There can be several contract identifier, an example of such being a contract is reportable to both the CFTC and ESMA and then has an associated USI (Unique Swap Identifier) UTI (Unique Trade Identifier).
	 */
	@RosettaSynonym(value="partyTradeIdentifier", source="FpML_5_10", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="CME_SubmissionIRS_1_0", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="FpML_5_10", path="tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="CME_SubmissionIRS_1_0", path="tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	@RosettaSynonym(value="universalSwapIdentifier", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeIdentifier", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade.tradeHeader")
	@RosettaSynonym(value="Submitter", source="DTCC_11_0")
	@RosettaSynonym(value="Submitter", source="DTCC_9_0")
	@RosettaSynonym(value="ContraTradeId", source="DTCC_11_0")
	@RosettaSynonym(value="ContraTradeId", source="DTCC_9_0")
	@RosettaSynonym(value="YourTradeId", source="DTCC_11_0")
	@RosettaSynonym(value="YourTradeId", source="DTCC_9_0")
	@RosettaSynonym(value="tradeIdentifyingItems", source="DTCC_11_0")
	@RosettaSynonym(value="tradeIdentifyingItems", source="DTCC_9_0")
	@RosettaSynonym(value="USI", source="DTCC_11_0")
	@RosettaSynonym(value="USI", source="DTCC_9_0")
	@RosettaSynonym(value="OriginatingUSI", source="DTCC_11_0")
	@RosettaSynonym(value="OriginatingUSI", source="DTCC_9_0")
	@RosettaSynonym(value="UTI", source="DTCC_11_0")
	@RosettaSynonym(value="UTI", source="DTCC_9_0")
	public final List<Identifier> getContractIdentifier() {
		return contractIdentifier;
	}
	
	/**
	 * The contractual product information that is associated with the contract, which combines the economic terms, the product identification and the product taxonomy. The corresponding FpML construct is the product abstract element and the associated substitution group.
	 */
	@RosettaSynonym(value="trade", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML")
	public final ContractualProduct getContractualProduct() {
		return contractualProduct;
	}
	
	/**
	 * Complex attribute to specify the quantity of the contractual product that the contract is written on. This temporarily makes uses of the &#39;ExecutionQuantity&#39; class, until such time when the latter can be renamed as, and supersede, the existing &#39;ContractualQuantity&#39; class.
	 */
	@RosettaSynonym(value="returnLeg", source="FpML_5_10", path="trade.returnSwap")
	@RosettaSynonym(value="returnLeg", source="FpML_5_10", path="trade.equitySwapTransactionSupplement")
	@RosettaSynonym(value="swap", source="FpML_5_10", path="")
	@RosettaSynonym(value="swap", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="swap", source="DTCC_11_0", path="")
	@RosettaSynonym(value="swap", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="swap", source="DTCC_9_0", path="")
	@RosettaSynonym(value="swap", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="swap", source="CME_ClearedConfirm_1_17", path="")
	@RosettaSynonym(value="swap", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade")
	@RosettaSynonym(value="swap", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="swap", source="DTCC_11_0", path="trade.swaption")
	@RosettaSynonym(value="swap", source="DTCC_9_0", path="trade.swaption")
	@RosettaSynonym(value="swap", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="swap", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="swap", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="swap", source="DTCC_11_0", path="swaption")
	@RosettaSynonym(value="swap", source="DTCC_9_0", path="swaption")
	@RosettaSynonym(value="swap", source="CME_ClearedConfirm_1_17", path="swaption")
	@RosettaSynonym(value="capFloor", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="capFloor", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="capFloor", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="capFloor", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="capFloor", source="CME_ClearedConfirm_1_17", path="trade")
	public final ExecutionQuantity getContractualQuantity() {
		return contractualQuantity;
	}
	
	/**
	 * Specifies the document(s) that govern the document, either as a reference to such documents when specified as part of the CDM, or through identification of some of the key terms of those documents, such as the type of document, the document identifier, the publisher, the document vintage and the agreement date.
	 */
	public final Documentation getDocumentation() {
		return documentation;
	}
	
	/**
	 * Identification of the law governing the contract.
	 */
	@RosettaSynonym(value="governingLaw", source="FpML_5_10", path="trade")
	@RosettaSynonym(value="governingLaw", source="DTCC_11_0", path="trade")
	@RosettaSynonym(value="governingLaw", source="DTCC_9_0", path="trade")
	@RosettaSynonym(value="governingLaw", source="CME_SubmissionIRS_1_0", path="trade")
	@RosettaSynonym(value="governingLaw", source="CME_ClearedConfirm_1_17", path="trade")
	@RosettaSynonym(value="governingLaw", source="FpML_5_10")
	@RosettaSynonym(value="governingLaw", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="governingLaw", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaGoverningLawEnum getGoverningLaw() {
		return governingLaw;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The parties to the contract. The cardinality is optional to address the case where the contract object is part of an event record, in which case the party is specified as part of that object. In that respect, the CDM approach is distinct from FpML, where party information is specified as part of a wrapper alongside the trade object.
	 */
	@RosettaSynonym(value="party", source="FpML_5_10")
	@RosettaSynonym(value="party", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="party", source="DTCC_11_0")
	@RosettaSynonym(value="party", source="DTCC_9_0")
	@RosettaSynonym(value="party", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="party", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML")
	public final List<Party> getParty() {
		return party;
	}
	
	/**
	 * Additional contract information that may be provided by each involved party.
	 */
	@RosettaSynonym(value="partyTradeInformation", source="FpML_5_10", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeInformation", source="CME_SubmissionIRS_1_0", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeInformation", source="DTCC_11_0", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeInformation", source="DTCC_9_0", path="trade.tradeHeader")
	@RosettaSynonym(value="partyTradeInformation", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	public final List<PartyContractInformation> getPartyContractInformation() {
		return partyContractInformation;
	}
	
	/**
	 * The role(s) that party(ies) may have in relation to the contract, further to the principal parties (i.e payer/receive or buyer/seller) to it.
	 */
	public final List<PartyRole> getPartyRole() {
		return partyRole;
	}
	
	/**
	 * The date on which the contract has been executed.
	 */
	public final TradeDate getTradeDate() {
		return tradeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends Contract> metaData() {
		return metaData;
	} 
	
	public ContractBuilder toBuilder() {
		ContractBuilder builder = new ContractBuilder();
		ofNullable(getAccount()).ifPresent(account -> account.forEach(builder::addAccount));
		ofNullable(getCalculationAgent()).ifPresent(builder::setCalculationAgent);
		ofNullable(getClearedDate()).ifPresent(builder::setClearedDate);
		ofNullable(getClosedState()).ifPresent(builder::setClosedState);
		ofNullable(getCollateral()).ifPresent(builder::setCollateral);
		ofNullable(getContractIdentifier()).ifPresent(contractIdentifier -> contractIdentifier.forEach(builder::addContractIdentifier));
		ofNullable(getContractualProduct()).ifPresent(builder::setContractualProduct);
		ofNullable(getContractualQuantity()).ifPresent(builder::setContractualQuantity);
		ofNullable(getDocumentation()).ifPresent(builder::setDocumentation);
		ofNullable(getGoverningLaw()).ifPresent(builder::setGoverningLaw);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getParty()).ifPresent(party -> party.forEach(builder::addParty));
		ofNullable(getPartyContractInformation()).ifPresent(partyContractInformation -> partyContractInformation.forEach(builder::addPartyContractInformation));
		ofNullable(getPartyRole()).ifPresent(partyRole -> partyRole.forEach(builder::addPartyRole));
		ofNullable(getTradeDate()).ifPresent(builder::setTradeDate);
		return builder;
	}

	public static ContractBuilder builder() {
		return new ContractBuilder();
	}

	public static  class ContractBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ContractBuilder> {
	
		protected List<Account.AccountBuilder> account;
		protected CalculationAgent.CalculationAgentBuilder calculationAgent;
		protected Date clearedDate;
		protected ClosedState.ClosedStateBuilder closedState;
		protected Collateral.CollateralBuilder collateral;
		protected List<Identifier.IdentifierBuilder> contractIdentifier;
		protected ContractualProduct.ContractualProductBuilder contractualProduct;
		protected ExecutionQuantity.ExecutionQuantityBuilder contractualQuantity;
		protected Documentation.DocumentationBuilder documentation;
		protected FieldWithMetaGoverningLawEnumBuilder governingLaw;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<Party.PartyBuilder> party;
		protected List<PartyContractInformation.PartyContractInformationBuilder> partyContractInformation;
		protected List<PartyRole.PartyRoleBuilder> partyRole;
		protected TradeDate.TradeDateBuilder tradeDate;
	
		public ContractBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Contract> metaData() {
			return metaData;
		} 
	
		
		public List<Account.AccountBuilder> getAccount() {
			return account;
		}
		
		public Account.AccountBuilder getOrCreateAccount(int index) {
			if (account==null) {
				this.account = new ArrayList<>();
			}
			return getIndex(account, index, ()->new Account.AccountBuilder());
		}
		
		
		public CalculationAgent.CalculationAgentBuilder getCalculationAgent() {
			return calculationAgent;
		}
		
		public CalculationAgent.CalculationAgentBuilder getOrCreateCalculationAgent() {
			if (calculationAgent!=null) {
				return calculationAgent;
			}
			else return calculationAgent = new CalculationAgent.CalculationAgentBuilder();
		}
		
		
		public Date getClearedDate() {
			return clearedDate;
		}
		
		
		public ClosedState.ClosedStateBuilder getClosedState() {
			return closedState;
		}
		
		public ClosedState.ClosedStateBuilder getOrCreateClosedState() {
			if (closedState!=null) {
				return closedState;
			}
			else return closedState = new ClosedState.ClosedStateBuilder();
		}
		
		
		public Collateral.CollateralBuilder getCollateral() {
			return collateral;
		}
		
		public Collateral.CollateralBuilder getOrCreateCollateral() {
			if (collateral!=null) {
				return collateral;
			}
			else return collateral = new Collateral.CollateralBuilder();
		}
		
		
		public List<Identifier.IdentifierBuilder> getContractIdentifier() {
			return contractIdentifier;
		}
		
		public Identifier.IdentifierBuilder getOrCreateContractIdentifier(int index) {
			if (contractIdentifier==null) {
				this.contractIdentifier = new ArrayList<>();
			}
			return getIndex(contractIdentifier, index, ()->new Identifier.IdentifierBuilder());
		}
		
		
		public ContractualProduct.ContractualProductBuilder getContractualProduct() {
			return contractualProduct;
		}
		
		public ContractualProduct.ContractualProductBuilder getOrCreateContractualProduct() {
			if (contractualProduct!=null) {
				return contractualProduct;
			}
			else return contractualProduct = new ContractualProduct.ContractualProductBuilder();
		}
		
		
		public ExecutionQuantity.ExecutionQuantityBuilder getContractualQuantity() {
			return contractualQuantity;
		}
		
		public ExecutionQuantity.ExecutionQuantityBuilder getOrCreateContractualQuantity() {
			if (contractualQuantity!=null) {
				return contractualQuantity;
			}
			else return contractualQuantity = new ExecutionQuantity.ExecutionQuantityBuilder();
		}
		
		
		public Documentation.DocumentationBuilder getDocumentation() {
			return documentation;
		}
		
		public Documentation.DocumentationBuilder getOrCreateDocumentation() {
			if (documentation!=null) {
				return documentation;
			}
			else return documentation = new Documentation.DocumentationBuilder();
		}
		
		
		public FieldWithMetaGoverningLawEnumBuilder getGoverningLaw() {
			return governingLaw;
		}
		
		public FieldWithMetaGoverningLawEnumBuilder getOrCreateGoverningLaw() {
			if (governingLaw!=null) {
				return governingLaw;
			}
			else return governingLaw = new FieldWithMetaGoverningLawEnumBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<Party.PartyBuilder> getParty() {
			return party;
		}
		
		public Party.PartyBuilder getOrCreateParty(int index) {
			if (party==null) {
				this.party = new ArrayList<>();
			}
			return getIndex(party, index, ()->new Party.PartyBuilder());
		}
		
		
		public List<PartyContractInformation.PartyContractInformationBuilder> getPartyContractInformation() {
			return partyContractInformation;
		}
		
		public PartyContractInformation.PartyContractInformationBuilder getOrCreatePartyContractInformation(int index) {
			if (partyContractInformation==null) {
				this.partyContractInformation = new ArrayList<>();
			}
			return getIndex(partyContractInformation, index, ()->new PartyContractInformation.PartyContractInformationBuilder());
		}
		
		
		public List<PartyRole.PartyRoleBuilder> getPartyRole() {
			return partyRole;
		}
		
		public PartyRole.PartyRoleBuilder getOrCreatePartyRole(int index) {
			if (partyRole==null) {
				this.partyRole = new ArrayList<>();
			}
			return getIndex(partyRole, index, ()->new PartyRole.PartyRoleBuilder());
		}
		
		
		public TradeDate.TradeDateBuilder getTradeDate() {
			return tradeDate;
		}
		
		public TradeDate.TradeDateBuilder getOrCreateTradeDate() {
			if (tradeDate!=null) {
				return tradeDate;
			}
			else return tradeDate = new TradeDate.TradeDateBuilder();
		}
		
	
		public ContractBuilder addAccount(Account account) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			this.account.add(account.toBuilder());
			return this;
		}
		public ContractBuilder addAccount(Account account, int _idx) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			getIndex(this.account, _idx, () -> account.toBuilder());
			this.account.set(_idx, account.toBuilder());
			return this;
		}
		public ContractBuilder addAccount(List<Account> accounts) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			for (Account toAdd : accounts) {
				this.account.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractBuilder addAccountBuilder(Account.AccountBuilder account) {
			if(this.account == null){
				this.account = new ArrayList<>();
			}
			this.account.add(account);
			return this;
		}
		
		public ContractBuilder clearAccount() {
			this.account = null;
			return this;
		}
		public ContractBuilder setCalculationAgent(CalculationAgent calculationAgent) {
			this.calculationAgent = calculationAgent.toBuilder();
			return this;
		}
		public ContractBuilder setCalculationAgentBuilder(CalculationAgent.CalculationAgentBuilder calculationAgent) {
			this.calculationAgent = calculationAgent;
			return this;
		}
		public ContractBuilder setClearedDate(Date clearedDate) {
			this.clearedDate = clearedDate;
			return this;
		}
		public ContractBuilder setClosedState(ClosedState closedState) {
			this.closedState = closedState.toBuilder();
			return this;
		}
		public ContractBuilder setClosedStateBuilder(ClosedState.ClosedStateBuilder closedState) {
			this.closedState = closedState;
			return this;
		}
		public ContractBuilder setCollateral(Collateral collateral) {
			this.collateral = collateral.toBuilder();
			return this;
		}
		public ContractBuilder setCollateralBuilder(Collateral.CollateralBuilder collateral) {
			this.collateral = collateral;
			return this;
		}
		public ContractBuilder addContractIdentifier(Identifier contractIdentifier) {
			if(this.contractIdentifier == null){
				this.contractIdentifier = new ArrayList<>();
			}
			this.contractIdentifier.add(contractIdentifier.toBuilder());
			return this;
		}
		public ContractBuilder addContractIdentifier(Identifier contractIdentifier, int _idx) {
			if(this.contractIdentifier == null){
				this.contractIdentifier = new ArrayList<>();
			}
			getIndex(this.contractIdentifier, _idx, () -> contractIdentifier.toBuilder());
			this.contractIdentifier.set(_idx, contractIdentifier.toBuilder());
			return this;
		}
		public ContractBuilder addContractIdentifier(List<Identifier> contractIdentifiers) {
			if(this.contractIdentifier == null){
				this.contractIdentifier = new ArrayList<>();
			}
			for (Identifier toAdd : contractIdentifiers) {
				this.contractIdentifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractBuilder addContractIdentifierBuilder(Identifier.IdentifierBuilder contractIdentifier) {
			if(this.contractIdentifier == null){
				this.contractIdentifier = new ArrayList<>();
			}
			this.contractIdentifier.add(contractIdentifier);
			return this;
		}
		
		public ContractBuilder clearContractIdentifier() {
			this.contractIdentifier = null;
			return this;
		}
		public ContractBuilder setContractualProduct(ContractualProduct contractualProduct) {
			this.contractualProduct = contractualProduct.toBuilder();
			return this;
		}
		public ContractBuilder setContractualProductBuilder(ContractualProduct.ContractualProductBuilder contractualProduct) {
			this.contractualProduct = contractualProduct;
			return this;
		}
		public ContractBuilder setContractualQuantity(ExecutionQuantity contractualQuantity) {
			this.contractualQuantity = contractualQuantity.toBuilder();
			return this;
		}
		public ContractBuilder setContractualQuantityBuilder(ExecutionQuantity.ExecutionQuantityBuilder contractualQuantity) {
			this.contractualQuantity = contractualQuantity;
			return this;
		}
		public ContractBuilder setDocumentation(Documentation documentation) {
			this.documentation = documentation.toBuilder();
			return this;
		}
		public ContractBuilder setDocumentationBuilder(Documentation.DocumentationBuilder documentation) {
			this.documentation = documentation;
			return this;
		}
		public ContractBuilder setGoverningLaw(FieldWithMetaGoverningLawEnum governingLaw) {
			this.governingLaw = governingLaw.toBuilder();
			return this;
		}
		public ContractBuilder setGoverningLawRef(GoverningLawEnum governingLaw) {
			return setGoverningLaw(FieldWithMetaGoverningLawEnum.builder().setValue(governingLaw).build());
		}
		public ContractBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ContractBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ContractBuilder addParty(Party party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party.toBuilder());
			return this;
		}
		public ContractBuilder addParty(Party party, int _idx) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			getIndex(this.party, _idx, () -> party.toBuilder());
			this.party.set(_idx, party.toBuilder());
			return this;
		}
		public ContractBuilder addParty(List<Party> partys) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			for (Party toAdd : partys) {
				this.party.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractBuilder addPartyBuilder(Party.PartyBuilder party) {
			if(this.party == null){
				this.party = new ArrayList<>();
			}
			this.party.add(party);
			return this;
		}
		
		public ContractBuilder clearParty() {
			this.party = null;
			return this;
		}
		public ContractBuilder addPartyContractInformation(PartyContractInformation partyContractInformation) {
			if(this.partyContractInformation == null){
				this.partyContractInformation = new ArrayList<>();
			}
			this.partyContractInformation.add(partyContractInformation.toBuilder());
			return this;
		}
		public ContractBuilder addPartyContractInformation(PartyContractInformation partyContractInformation, int _idx) {
			if(this.partyContractInformation == null){
				this.partyContractInformation = new ArrayList<>();
			}
			getIndex(this.partyContractInformation, _idx, () -> partyContractInformation.toBuilder());
			this.partyContractInformation.set(_idx, partyContractInformation.toBuilder());
			return this;
		}
		public ContractBuilder addPartyContractInformation(List<PartyContractInformation> partyContractInformations) {
			if(this.partyContractInformation == null){
				this.partyContractInformation = new ArrayList<>();
			}
			for (PartyContractInformation toAdd : partyContractInformations) {
				this.partyContractInformation.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractBuilder addPartyContractInformationBuilder(PartyContractInformation.PartyContractInformationBuilder partyContractInformation) {
			if(this.partyContractInformation == null){
				this.partyContractInformation = new ArrayList<>();
			}
			this.partyContractInformation.add(partyContractInformation);
			return this;
		}
		
		public ContractBuilder clearPartyContractInformation() {
			this.partyContractInformation = null;
			return this;
		}
		public ContractBuilder addPartyRole(PartyRole partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole.toBuilder());
			return this;
		}
		public ContractBuilder addPartyRole(PartyRole partyRole, int _idx) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			getIndex(this.partyRole, _idx, () -> partyRole.toBuilder());
			this.partyRole.set(_idx, partyRole.toBuilder());
			return this;
		}
		public ContractBuilder addPartyRole(List<PartyRole> partyRoles) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			for (PartyRole toAdd : partyRoles) {
				this.partyRole.add(toAdd.toBuilder());
			}
			return this;
		}
		public ContractBuilder addPartyRoleBuilder(PartyRole.PartyRoleBuilder partyRole) {
			if(this.partyRole == null){
				this.partyRole = new ArrayList<>();
			}
			this.partyRole.add(partyRole);
			return this;
		}
		
		public ContractBuilder clearPartyRole() {
			this.partyRole = null;
			return this;
		}
		public ContractBuilder setTradeDate(TradeDate tradeDate) {
			this.tradeDate = tradeDate.toBuilder();
			return this;
		}
		public ContractBuilder setTradeDateBuilder(TradeDate.TradeDateBuilder tradeDate) {
			this.tradeDate = tradeDate;
			return this;
		}
		
		public Contract build() {
			return new Contract(this);
		}
	
		@Override
		public ContractBuilder prune() {
			if (account!=null) account = account.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (calculationAgent!=null && !calculationAgent.prune().hasData()) calculationAgent = null;
			if (closedState!=null && !closedState.prune().hasData()) closedState = null;
			if (collateral!=null && !collateral.prune().hasData()) collateral = null;
			if (contractIdentifier!=null) contractIdentifier = contractIdentifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contractualProduct!=null && !contractualProduct.prune().hasData()) contractualProduct = null;
			if (contractualQuantity!=null && !contractualQuantity.prune().hasData()) contractualQuantity = null;
			if (documentation!=null && !documentation.prune().hasData()) documentation = null;
			if (governingLaw!=null && !governingLaw.prune().hasData()) governingLaw = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (party!=null) party = party.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyContractInformation!=null) partyContractInformation = partyContractInformation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyRole!=null) partyRole = partyRole.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (tradeDate!=null && !tradeDate.prune().hasData()) tradeDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccount()!=null && getAccount().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCalculationAgent()!=null && getCalculationAgent().hasData()) return true;
			if (getClearedDate()!=null) return true;
			if (getClosedState()!=null && getClosedState().hasData()) return true;
			if (getCollateral()!=null && getCollateral().hasData()) return true;
			if (getContractIdentifier()!=null && getContractIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getContractualProduct()!=null && getContractualProduct().hasData()) return true;
			if (getContractualQuantity()!=null && getContractualQuantity().hasData()) return true;
			if (getDocumentation()!=null && getDocumentation().hasData()) return true;
			if (getGoverningLaw()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getParty()!=null && getParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyContractInformation()!=null && getPartyContractInformation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyRole()!=null && getPartyRole().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTradeDate()!=null && getTradeDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("clearedDate"), Date.class, clearedDate, this);
			
			processRosetta(path.newSubPath("account"), processor, Account.class, account);
			processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
			processRosetta(path.newSubPath("closedState"), processor, ClosedState.class, closedState);
			processRosetta(path.newSubPath("collateral"), processor, Collateral.class, collateral);
			processRosetta(path.newSubPath("contractIdentifier"), processor, Identifier.class, contractIdentifier);
			processRosetta(path.newSubPath("contractualProduct"), processor, ContractualProduct.class, contractualProduct);
			processRosetta(path.newSubPath("contractualQuantity"), processor, ExecutionQuantity.class, contractualQuantity);
			processRosetta(path.newSubPath("documentation"), processor, Documentation.class, documentation);
			processRosetta(path.newSubPath("governingLaw"), processor, FieldWithMetaGoverningLawEnum.class, governingLaw);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("party"), processor, Party.class, party);
			processRosetta(path.newSubPath("partyContractInformation"), processor, PartyContractInformation.class, partyContractInformation);
			processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
			processRosetta(path.newSubPath("tradeDate"), processor, TradeDate.class, tradeDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractBuilder _that = (ContractBuilder) o;
		
			if (!ListEquals.listEquals(account, _that.account)) return false;
			if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
			if (clearedDate != null ? !clearedDate.equals(_that.clearedDate) : _that.clearedDate != null) return false;
			if (closedState != null ? !closedState.equals(_that.closedState) : _that.closedState != null) return false;
			if (collateral != null ? !collateral.equals(_that.collateral) : _that.collateral != null) return false;
			if (!ListEquals.listEquals(contractIdentifier, _that.contractIdentifier)) return false;
			if (contractualProduct != null ? !contractualProduct.equals(_that.contractualProduct) : _that.contractualProduct != null) return false;
			if (contractualQuantity != null ? !contractualQuantity.equals(_that.contractualQuantity) : _that.contractualQuantity != null) return false;
			if (documentation != null ? !documentation.equals(_that.documentation) : _that.documentation != null) return false;
			if (governingLaw != null ? !governingLaw.equals(_that.governingLaw) : _that.governingLaw != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(party, _that.party)) return false;
			if (!ListEquals.listEquals(partyContractInformation, _that.partyContractInformation)) return false;
			if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
			if (tradeDate != null ? !tradeDate.equals(_that.tradeDate) : _that.tradeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (account != null ? account.hashCode() : 0);
			_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
			_result = 31 * _result + (clearedDate != null ? clearedDate.hashCode() : 0);
			_result = 31 * _result + (closedState != null ? closedState.hashCode() : 0);
			_result = 31 * _result + (collateral != null ? collateral.hashCode() : 0);
			_result = 31 * _result + (contractIdentifier != null ? contractIdentifier.hashCode() : 0);
			_result = 31 * _result + (contractualProduct != null ? contractualProduct.hashCode() : 0);
			_result = 31 * _result + (contractualQuantity != null ? contractualQuantity.hashCode() : 0);
			_result = 31 * _result + (documentation != null ? documentation.hashCode() : 0);
			_result = 31 * _result + (governingLaw != null ? governingLaw.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (party != null ? party.hashCode() : 0);
			_result = 31 * _result + (partyContractInformation != null ? partyContractInformation.hashCode() : 0);
			_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
			_result = 31 * _result + (tradeDate != null ? tradeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractBuilder {" +
				"account=" + this.account + ", " +
				"calculationAgent=" + this.calculationAgent + ", " +
				"clearedDate=" + this.clearedDate + ", " +
				"closedState=" + this.closedState + ", " +
				"collateral=" + this.collateral + ", " +
				"contractIdentifier=" + this.contractIdentifier + ", " +
				"contractualProduct=" + this.contractualProduct + ", " +
				"contractualQuantity=" + this.contractualQuantity + ", " +
				"documentation=" + this.documentation + ", " +
				"governingLaw=" + this.governingLaw + ", " +
				"meta=" + this.meta + ", " +
				"party=" + this.party + ", " +
				"partyContractInformation=" + this.partyContractInformation + ", " +
				"partyRole=" + this.partyRole + ", " +
				"tradeDate=" + this.tradeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("clearedDate"), Date.class, clearedDate, this);
		
		processRosetta(path.newSubPath("account"), processor, Account.class, account);
		processRosetta(path.newSubPath("calculationAgent"), processor, CalculationAgent.class, calculationAgent);
		processRosetta(path.newSubPath("closedState"), processor, ClosedState.class, closedState);
		processRosetta(path.newSubPath("collateral"), processor, Collateral.class, collateral);
		processRosetta(path.newSubPath("contractIdentifier"), processor, Identifier.class, contractIdentifier);
		processRosetta(path.newSubPath("contractualProduct"), processor, ContractualProduct.class, contractualProduct);
		processRosetta(path.newSubPath("contractualQuantity"), processor, ExecutionQuantity.class, contractualQuantity);
		processRosetta(path.newSubPath("documentation"), processor, Documentation.class, documentation);
		processRosetta(path.newSubPath("governingLaw"), processor, FieldWithMetaGoverningLawEnum.class, governingLaw);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("party"), processor, Party.class, party);
		processRosetta(path.newSubPath("partyContractInformation"), processor, PartyContractInformation.class, partyContractInformation);
		processRosetta(path.newSubPath("partyRole"), processor, PartyRole.class, partyRole);
		processRosetta(path.newSubPath("tradeDate"), processor, TradeDate.class, tradeDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Contract _that = (Contract) o;
	
		if (!ListEquals.listEquals(account, _that.account)) return false;
		if (calculationAgent != null ? !calculationAgent.equals(_that.calculationAgent) : _that.calculationAgent != null) return false;
		if (clearedDate != null ? !clearedDate.equals(_that.clearedDate) : _that.clearedDate != null) return false;
		if (closedState != null ? !closedState.equals(_that.closedState) : _that.closedState != null) return false;
		if (collateral != null ? !collateral.equals(_that.collateral) : _that.collateral != null) return false;
		if (!ListEquals.listEquals(contractIdentifier, _that.contractIdentifier)) return false;
		if (contractualProduct != null ? !contractualProduct.equals(_that.contractualProduct) : _that.contractualProduct != null) return false;
		if (contractualQuantity != null ? !contractualQuantity.equals(_that.contractualQuantity) : _that.contractualQuantity != null) return false;
		if (documentation != null ? !documentation.equals(_that.documentation) : _that.documentation != null) return false;
		if (governingLaw != null ? !governingLaw.equals(_that.governingLaw) : _that.governingLaw != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(party, _that.party)) return false;
		if (!ListEquals.listEquals(partyContractInformation, _that.partyContractInformation)) return false;
		if (!ListEquals.listEquals(partyRole, _that.partyRole)) return false;
		if (tradeDate != null ? !tradeDate.equals(_that.tradeDate) : _that.tradeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (account != null ? account.hashCode() : 0);
		_result = 31 * _result + (calculationAgent != null ? calculationAgent.hashCode() : 0);
		_result = 31 * _result + (clearedDate != null ? clearedDate.hashCode() : 0);
		_result = 31 * _result + (closedState != null ? closedState.hashCode() : 0);
		_result = 31 * _result + (collateral != null ? collateral.hashCode() : 0);
		_result = 31 * _result + (contractIdentifier != null ? contractIdentifier.hashCode() : 0);
		_result = 31 * _result + (contractualProduct != null ? contractualProduct.hashCode() : 0);
		_result = 31 * _result + (contractualQuantity != null ? contractualQuantity.hashCode() : 0);
		_result = 31 * _result + (documentation != null ? documentation.hashCode() : 0);
		_result = 31 * _result + (governingLaw != null ? governingLaw.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (party != null ? party.hashCode() : 0);
		_result = 31 * _result + (partyContractInformation != null ? partyContractInformation.hashCode() : 0);
		_result = 31 * _result + (partyRole != null ? partyRole.hashCode() : 0);
		_result = 31 * _result + (tradeDate != null ? tradeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Contract {" +
			"account=" + this.account + ", " +
			"calculationAgent=" + this.calculationAgent + ", " +
			"clearedDate=" + this.clearedDate + ", " +
			"closedState=" + this.closedState + ", " +
			"collateral=" + this.collateral + ", " +
			"contractIdentifier=" + this.contractIdentifier + ", " +
			"contractualProduct=" + this.contractualProduct + ", " +
			"contractualQuantity=" + this.contractualQuantity + ", " +
			"documentation=" + this.documentation + ", " +
			"governingLaw=" + this.governingLaw + ", " +
			"meta=" + this.meta + ", " +
			"party=" + this.party + ", " +
			"partyContractInformation=" + this.partyContractInformation + ", " +
			"partyRole=" + this.partyRole + ", " +
			"tradeDate=" + this.tradeDate +
		'}';
	}
}
