package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify a set of standard contract definitions relevant to the transaction.
 * 
 * @version 2.5.4
 */
public enum ContractualDefinitionsEnum {
    

	/**
	 * ISDA 1991 Definitions
	 */
	@RosettaSynonym(value = "ISDA1991", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1991", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1991", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1991", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1991", source = "CME_ClearedConfirm_1_17")
	ISDA1991,
	    

	/**
	 * ISDA 1993 Commodity Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA1993Commodity", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1993Commodity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1993Commodity", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1993Commodity", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1993Commodity", source = "CME_ClearedConfirm_1_17")
	ISDA_1993_COMMODITY,
	    

	/**
	 * ISDA 1996 Equity Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA1996Equity", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1996Equity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1996Equity", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1996Equity", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1996Equity", source = "CME_ClearedConfirm_1_17")
	ISDA_1996_EQUITY,
	    

	/**
	 * ISDA 1997 Bullion Definitions
	 */
	@RosettaSynonym(value = "ISDA1997Bullion", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1997Bullion", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1997Bullion", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1997Bullion", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1997Bullion", source = "CME_ClearedConfirm_1_17")
	ISDA_1997_BULLION,
	    

	/**
	 * ISDA 1997 Government Bond Option Definitions
	 */
	@RosettaSynonym(value = "ISDA1997GovernmentBond", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1997GovernmentBond", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1997GovernmentBond", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1997GovernmentBond", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1997GovernmentBond", source = "CME_ClearedConfirm_1_17")
	ISDA_1997_GOVERNMENT_BOND,
	    

	/**
	 * ISDA 1998 FX and Currency Option Definitions
	 */
	@RosettaSynonym(value = "ISDA1998FX", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1998FX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1998FX", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1998FX", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1998FX", source = "CME_ClearedConfirm_1_17")
	ISDA1998FX,
	    

	/**
	 * ISDA 1999 Credit Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA1999Credit", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1999Credit", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1999Credit", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1999Credit", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1999Credit", source = "CME_ClearedConfirm_1_17")
	ISDA_1999_CREDIT,
	    

	/**
	 * ISDA 2000 Definitions
	 */
	@RosettaSynonym(value = "ISDA2000", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2000", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2000", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2000", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2000", source = "CME_ClearedConfirm_1_17")
	ISDA2000,
	    

	/**
	 * ISDA 2002 Equity Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2002Equity", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2002Equity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2002Equity", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2002Equity", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2002Equity", source = "CME_ClearedConfirm_1_17")
	ISDA_2002_EQUITY,
	    

	/**
	 * ISDA 2003 Credit Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2003Credit", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003Credit", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003Credit", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003Credit", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003Credit", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT,
	    

	/**
	 * ISDA 2004 Novation Definitions
	 */
	@RosettaSynonym(value = "ISDA2004Novation", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2004Novation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2004Novation", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2004Novation", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2004Novation", source = "CME_ClearedConfirm_1_17")
	ISDA_2004_NOVATION,
	    

	/**
	 * ISDA 2005 Commodity Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2005Commodity", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2005Commodity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2005Commodity", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2005Commodity", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2005Commodity", source = "CME_ClearedConfirm_1_17")
	ISDA_2005_COMMODITY,
	    

	/**
	 * ISDA 2006 Definitions
	 */
	@RosettaSynonym(value = "ISDA2006", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2006", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2006", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2006", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2006", source = "CME_ClearedConfirm_1_17")
	ISDA2006,
	    

	/**
	 * ISDA 2006 Inflation Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2006Inflation", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2006Inflation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2006Inflation", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2006Inflation", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2006Inflation", source = "CME_ClearedConfirm_1_17")
	ISDA_2006_INFLATION,
	    

	/**
	 * ISDA 2008 Inflation Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2008Inflation", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2008Inflation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2008Inflation", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2008Inflation", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2008Inflation", source = "CME_ClearedConfirm_1_17")
	ISDA_2008_INFLATION,
	    

	/**
	 * ISDA 2011 Equity Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2011Equity", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2011Equity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2011Equity", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2011Equity", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2011Equity", source = "CME_ClearedConfirm_1_17")
	ISDA_2011_EQUITY,
	    

	/**
	 * ISDA 2014 Credit Derivatives Definitions
	 */
	@RosettaSynonym(value = "ISDA2014Credit", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2014Credit", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2014Credit", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2014Credit", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2014Credit", source = "CME_ClearedConfirm_1_17")
	ISDA_2014_CREDIT    
	;

	private final String displayName;
	
	ContractualDefinitionsEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
