package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaMatrixTermEnum;
import org.isda.cdm.metafields.FieldWithMetaMatrixTermEnum.FieldWithMetaMatrixTermEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaMatrixTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaMatrixTypeEnum.FieldWithMetaMatrixTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractualMatrixMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ContractualMatrix", source="FpML_5_10")
public class ContractualMatrix extends RosettaModelObject {
	private final FieldWithMetaMatrixTermEnum matrixTerm;
	private final FieldWithMetaMatrixTypeEnum matrixType;
	private final Date publicationDate;
	private static ContractualMatrixMeta metaData = new ContractualMatrixMeta();
	
	ContractualMatrix(ContractualMatrixBuilder builder) {
		this.matrixTerm = ofNullable(builder.getMatrixTerm()).map(f->f.build()).orElse(null);
		this.matrixType = ofNullable(builder.getMatrixType()).map(f->f.build()).orElse(null);
		this.publicationDate = builder.getPublicationDate();
	}
	
	/**
	 * Defines any applicable key into the relevant matrix. For example, the Transaction Type would be the single term required for the Credit Derivatives Physical Settlement Matrix. This element should be omitted in the case of the 2000 ISDA Definitions Settlement Matrix for Early Termination and Swaptions.
	 */
	@RosettaSynonym(value="matrixTerm", source="FpML_5_10")
	@RosettaSynonym(value="matrixTerm", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="matrixTerm", source="DTCC_11_0")
	@RosettaSynonym(value="matrixTerm", source="DTCC_9_0")
	@RosettaSynonym(value="matrixTerm", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMatrixTermEnum getMatrixTerm() {
		return matrixTerm;
	}
	
	/**
	 * Identifies the form of applicable matrix.
	 */
	@RosettaSynonym(value="matrixType", source="FpML_5_10")
	@RosettaSynonym(value="matrixType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="matrixType", source="DTCC_11_0")
	@RosettaSynonym(value="matrixType", source="DTCC_9_0")
	@RosettaSynonym(value="matrixType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMatrixTypeEnum getMatrixType() {
		return matrixType;
	}
	
	/**
	 * Specifies the publication date of the applicable version of the matrix. When this element is omitted, the ISDA supplemental language for incorporation of the relevant matrix will generally define rules for which version of the matrix is applicable.
	 */
	@RosettaSynonym(value="publicationDate", source="FpML_5_10")
	@RosettaSynonym(value="publicationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_11_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_9_0")
	@RosettaSynonym(value="publicationDate", source="CME_ClearedConfirm_1_17")
	public final Date getPublicationDate() {
		return publicationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractualMatrix> metaData() {
		return metaData;
	} 
	
	public ContractualMatrixBuilder toBuilder() {
		ContractualMatrixBuilder builder = new ContractualMatrixBuilder();
		ofNullable(getMatrixTerm()).ifPresent(builder::setMatrixTerm);
		ofNullable(getMatrixType()).ifPresent(builder::setMatrixType);
		ofNullable(getPublicationDate()).ifPresent(builder::setPublicationDate);
		return builder;
	}

	public static ContractualMatrixBuilder builder() {
		return new ContractualMatrixBuilder();
	}

	public static  class ContractualMatrixBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaMatrixTermEnumBuilder matrixTerm;
		protected FieldWithMetaMatrixTypeEnumBuilder matrixType;
		protected Date publicationDate;
	
		public ContractualMatrixBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractualMatrix> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaMatrixTermEnumBuilder getMatrixTerm() {
			return matrixTerm;
		}
		
		public FieldWithMetaMatrixTermEnumBuilder getOrCreateMatrixTerm() {
			if (matrixTerm!=null) {
				return matrixTerm;
			}
			else return matrixTerm = new FieldWithMetaMatrixTermEnumBuilder();
		}
		
		
		public FieldWithMetaMatrixTypeEnumBuilder getMatrixType() {
			return matrixType;
		}
		
		public FieldWithMetaMatrixTypeEnumBuilder getOrCreateMatrixType() {
			if (matrixType!=null) {
				return matrixType;
			}
			else return matrixType = new FieldWithMetaMatrixTypeEnumBuilder();
		}
		
		
		public Date getPublicationDate() {
			return publicationDate;
		}
		
	
		public ContractualMatrixBuilder setMatrixTerm(FieldWithMetaMatrixTermEnum matrixTerm) {
			this.matrixTerm = matrixTerm.toBuilder();
			return this;
		}
		public ContractualMatrixBuilder setMatrixTermRef(MatrixTermEnum matrixTerm) {
			return setMatrixTerm(FieldWithMetaMatrixTermEnum.builder().setValue(matrixTerm).build());
		}
		public ContractualMatrixBuilder setMatrixType(FieldWithMetaMatrixTypeEnum matrixType) {
			this.matrixType = matrixType.toBuilder();
			return this;
		}
		public ContractualMatrixBuilder setMatrixTypeRef(MatrixTypeEnum matrixType) {
			return setMatrixType(FieldWithMetaMatrixTypeEnum.builder().setValue(matrixType).build());
		}
		public ContractualMatrixBuilder setPublicationDate(Date publicationDate) {
			this.publicationDate = publicationDate;
			return this;
		}
		
		public ContractualMatrix build() {
			return new ContractualMatrix(this);
		}
	
		@Override
		public ContractualMatrixBuilder prune() {
			if (matrixTerm!=null && !matrixTerm.prune().hasData()) matrixTerm = null;
			if (matrixType!=null && !matrixType.prune().hasData()) matrixType = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMatrixTerm()!=null) return true;
			if (getMatrixType()!=null) return true;
			if (getPublicationDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
			
			processRosetta(path.newSubPath("matrixTerm"), processor, FieldWithMetaMatrixTermEnum.class, matrixTerm);
			processRosetta(path.newSubPath("matrixType"), processor, FieldWithMetaMatrixTypeEnum.class, matrixType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractualMatrixBuilder _that = (ContractualMatrixBuilder) o;
		
			if (matrixTerm != null ? !matrixTerm.equals(_that.matrixTerm) : _that.matrixTerm != null) return false;
			if (matrixType != null ? !matrixType.equals(_that.matrixType) : _that.matrixType != null) return false;
			if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (matrixTerm != null ? matrixTerm.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (matrixType != null ? matrixType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractualMatrixBuilder {" +
				"matrixTerm=" + this.matrixTerm + ", " +
				"matrixType=" + this.matrixType + ", " +
				"publicationDate=" + this.publicationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
		
		processRosetta(path.newSubPath("matrixTerm"), processor, FieldWithMetaMatrixTermEnum.class, matrixTerm);
		processRosetta(path.newSubPath("matrixType"), processor, FieldWithMetaMatrixTypeEnum.class, matrixType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractualMatrix _that = (ContractualMatrix) o;
	
		if (matrixTerm != null ? !matrixTerm.equals(_that.matrixTerm) : _that.matrixTerm != null) return false;
		if (matrixType != null ? !matrixType.equals(_that.matrixType) : _that.matrixType != null) return false;
		if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (matrixTerm != null ? matrixTerm.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (matrixType != null ? matrixType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractualMatrix {" +
			"matrixTerm=" + this.matrixTerm + ", " +
			"matrixType=" + this.matrixType + ", " +
			"publicationDate=" + this.publicationDate +
		'}';
	}
}
