package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to define the supplements to a base set of ISDA Definitions that are applicable to the transaction.
 * 
 * @version 2.5.4
 */
public enum ContractualSupplementEnum {
    

	/**
	 * Standard Terms Supplement for ABX Transactions.
	 */
	@RosettaSynonym(value = "ABX", source = "FpML_5_10")
	@RosettaSynonym(value = "ABX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ABX", source = "DTCC_11_0")
	@RosettaSynonym(value = "ABX", source = "DTCC_9_0")
	@RosettaSynonym(value = "ABX", source = "CME_ClearedConfirm_1_17")
	ABX,
	    

	/**
	 * Standard Terms Supplement for Asset-Backed Tranche Transactions.
	 */
	@RosettaSynonym(value = "ABXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "ABXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ABXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "ABXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "ABXTranche", source = "CME_ClearedConfirm_1_17")
	ABX_TRANCHE,
	    

	/**
	 * ISDA Standard Terms Supplement for use with Credit Derivative Transactions on Leveraged Loans.
	 */
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "FpML_5_10")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDSonLeveragedLoans", source = "CME_ClearedConfirm_1_17")
	CD_SON_LEVERAGED_LOANS,
	    

	/**
	 * ISDA Standard Terms Supplement for use with Credit Derivative Transactions on Mortgage-backed Security with Pay-As-You-Go or Physical Settlement.
	 */
	@RosettaSynonym(value = "CDSonMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "CDSonMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDSonMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDSonMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDSonMBS", source = "CME_ClearedConfirm_1_17")
	CD_SON_MBS,
	    

	/**
	 * Standard Terms Supplement for CDX Untranched Transactions.
	 */
	@RosettaSynonym(value = "CDX", source = "FpML_5_10")
	@RosettaSynonym(value = "CDX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDX", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDX", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDX", source = "CME_ClearedConfirm_1_17")
	CDX,
	    

	/**
	 * Standard Terms Supplement for CDX Emerging Markets Untranched Transactions.
	 */
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXEmergingMarkets", source = "CME_ClearedConfirm_1_17")
	CDX_EMERGING_MARKETS,
	    

	/**
	 * Standard Terms Supplement for CDX Emerging Markets Diversified Untranched Transactions..
	 */
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXEmergingMarketsDiversified", source = "CME_ClearedConfirm_1_17")
	CDX_EMERGING_MARKETS_DIVERSIFIED,
	    

	/**
	 * Standard Terms Supplement for CDX Swaption Transactions.
	 */
	@RosettaSynonym(value = "CDXSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXSwaption", source = "CME_ClearedConfirm_1_17")
	CDX_SWAPTION,
	    

	/**
	 * Standard Terms Supplement for Dow Jones CDX Tranche Transactions.
	 */
	@RosettaSynonym(value = "CDXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "CDXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CDXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "CDXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "CDXTranche", source = "CME_ClearedConfirm_1_17")
	CDX_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for CMBX Transactions.
	 */
	@RosettaSynonym(value = "CMBX", source = "FpML_5_10")
	@RosettaSynonym(value = "CMBX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CMBX", source = "DTCC_11_0")
	@RosettaSynonym(value = "CMBX", source = "DTCC_9_0")
	@RosettaSynonym(value = "CMBX", source = "CME_ClearedConfirm_1_17")
	CMBX,
	    

	/**
	 * Standard Terms Supplement for Single Name European CMBS Transactions.
	 */
	@RosettaSynonym(value = "EuropeanCMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "EuropeanCMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "EuropeanCMBS", source = "CME_ClearedConfirm_1_17")
	EUROPEAN_CMBS,
	    

	/**
	 * Standard Terms Supplement for Single Name European RMBS Transactions.
	 */
	@RosettaSynonym(value = "EuropeanRMBS", source = "FpML_5_10")
	@RosettaSynonym(value = "EuropeanRMBS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "DTCC_11_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "DTCC_9_0")
	@RosettaSynonym(value = "EuropeanRMBS", source = "CME_ClearedConfirm_1_17")
	EUROPEAN_RMBS,
	    

	/**
	 * Standard Terms Supplement for IOS Transactions.
	 */
	@RosettaSynonym(value = "IOS", source = "FpML_5_10")
	@RosettaSynonym(value = "IOS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IOS", source = "DTCC_11_0")
	@RosettaSynonym(value = "IOS", source = "DTCC_9_0")
	@RosettaSynonym(value = "IOS", source = "CME_ClearedConfirm_1_17")
	IOS,
	    

	/**
	 * Supplement to the 1999 ISDA Credit Derivatives Definitions Relating to Convertible, Exchangeable or Accreting Obligations dated November 9, 2001.
	 */
	@RosettaSynonym(value = "ISDA1999CreditConvertibleExchangeableAccretingObligations", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1999CreditConvertibleExchangeableAccretingObligations", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1999CreditConvertibleExchangeableAccretingObligations", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1999CreditConvertibleExchangeableAccretingObligations", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1999CreditConvertibleExchangeableAccretingObligations", source = "CME_ClearedConfirm_1_17")
	ISDA_1999_CREDIT_CONVERTIBLE_EXCHANGEABLE_ACCRETING_OBLIGATIONS,
	    

	/**
	 * Restructuring Supplement to the 1999 ISDA Credit Derivatives Definitions dated May 11, 2001.
	 */
	@RosettaSynonym(value = "ISDA1999CreditRestructuring", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1999CreditRestructuring", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1999CreditRestructuring", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1999CreditRestructuring", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1999CreditRestructuring", source = "CME_ClearedConfirm_1_17")
	ISDA_1999_CREDIT_RESTRUCTURING,
	    

	/**
	 * Supplement Relating to Successor and Credit Events to the 1999 ISDA Credit Derivatives Definitions dated November 28, 2001.
	 */
	@RosettaSynonym(value = "ISDA1999CreditSuccessorAndCreditEvents", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1999CreditSuccessorAndCreditEvents", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1999CreditSuccessorAndCreditEvents", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1999CreditSuccessorAndCreditEvents", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1999CreditSuccessorAndCreditEvents", source = "CME_ClearedConfirm_1_17")
	ISDA_1999_CREDIT_SUCCESSOR_AND_CREDIT_EVENTS,
	    

	/**
	 * Additional Provisions for LPN dated December 6, 2007.
	 */
	@RosettaSynonym(value = "ISDA2003AdditionalProvisionsLPN", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003AdditionalProvisionsLPN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003AdditionalProvisionsLPN", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003AdditionalProvisionsLPN", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003AdditionalProvisionsLPN", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_ADDITIONAL_PROVISIONS_LPN,
	    

	/**
	 * Additional Provisions for Contingent Credit Spread Transactions dated August 15, 2008.
	 */
	@RosettaSynonym(value = "ISDA2003ContingentCreditSpreadTransaction", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003ContingentCreditSpreadTransaction", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003ContingentCreditSpreadTransaction", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003ContingentCreditSpreadTransaction", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003ContingentCreditSpreadTransaction", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CONTINGENT_CREDIT_SPREAD_TRANSACTION,
	    

	/**
	 * 2005 Matrix Supplement to the 2003 ISDA Credit Derivatives.
	 */
	@RosettaSynonym(value = "ISDA2003Credit2005MatrixSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003Credit2005MatrixSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003Credit2005MatrixSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003Credit2005MatrixSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003Credit2005MatrixSupplement", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_2005_MATRIX_SUPPLEMENT,
	    

	/**
	 * Additional Provisions for the Argentine Republic: Excluded Obligations and Excluded Deliverable Obligations dated December 21, 2005.
	 */
	@RosettaSynonym(value = "ISDA2003CreditArgentineRepublic", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditArgentineRepublic", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditArgentineRepublic", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditArgentineRepublic", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditArgentineRepublic", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_ARGENTINE_REPUBLIC,
	    

	/**
	 * ISDA Credit Derivatives Determinations Committees and Auction Settlement Supplement to the 2003 ISDA Credit Derivatives Definitions (published on [TBD]).
	 */
	@RosettaSynonym(value = "ISDA2003CreditAuctionSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditAuctionSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditAuctionSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditAuctionSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditAuctionSupplement", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_AUCTION_SUPPLEMENT,
	    

	/**
	 * May 2003 Supplement to the 2003 ISDA Credit Derivatives Definitions.
	 */
	@RosettaSynonym(value = "ISDA2003CreditMay2003", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditMay2003", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditMay2003", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditMay2003", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditMay2003", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_MAY_2003,
	    

	/**
	 * Additional Provisions for Physically Settled Default Swaps Monoline Insurer as Reference Entity dated May 9, 2003.
	 */
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_MONOLINE_INSURERS,
	    

	/**
	 * Additional Provisions for Physically Settled Default Swaps Monoline Insurer as Reference Entity dated January 21, 2005.
	 */
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers2005", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers2005", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers2005", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers2005", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditMonolineInsurers2005", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_MONOLINE_INSURERS_2005,
	    

	/**
	 * Additional Provisions for the Republic of Hungary: Obligation Characteristics and Deliverable Obligation Characteristics dated August 13, 2004.
	 */
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_REPUBLIC_OF_HUNGARY,
	    

	/**
	 * Additional Provisions for the Republic of Hungary: Obligation Characteristics and Deliverable Obligation Characteristics dated February 14, 2005. 
	 */
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary2005", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary2005", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary2005", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary2005", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditRepublicOfHungary2005", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_REPUBLIC_OF_HUNGARY_2005,
	    

	/**
	 * Additional Provisions for the Russian Federation: Obligation Characteristics and Deliverable Obligation Characteristics dated August 13, 2004.
	 */
	@RosettaSynonym(value = "ISDA2003CreditRussianFederation", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditRussianFederation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditRussianFederation", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditRussianFederation", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditRussianFederation", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_RUSSIAN_FEDERATION,
	    

	/**
	 * Additional Provisions for Credit Derivative Transactions - U.S. Municipal Entity as Reference Entity dated September 17, 2004.
	 */
	@RosettaSynonym(value = "ISDA2003CreditUSMunicipals", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003CreditUSMunicipals", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003CreditUSMunicipals", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003CreditUSMunicipals", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003CreditUSMunicipals", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_CREDIT_US_MUNICIPALS,
	    

	/**
	 * Additional Provisions for STMicroelectronics NV dated December 6, 2007.
	 */
	@RosettaSynonym(value = "ISDA2003STMicroelectronicsNV", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2003STMicroelectronicsNV", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2003STMicroelectronicsNV", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2003STMicroelectronicsNV", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2003STMicroelectronicsNV", source = "CME_ClearedConfirm_1_17")
	ISDA_2003_ST_MICROELECTRONICS_NV,
	    

	/**
	 * 2007 Full Lookthrough Depository Receipt Supplement to the 2002 Equity Derivatives Definitions.
	 */
	@RosettaSynonym(value = "ISDA2007FullLookthroughDepositoryReceiptSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2007FullLookthroughDepositoryReceiptSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2007FullLookthroughDepositoryReceiptSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2007FullLookthroughDepositoryReceiptSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2007FullLookthroughDepositoryReceiptSupplement", source = "CME_ClearedConfirm_1_17")
	ISDA_2007_FULL_LOOKTHROUGH_DEPOSITORY_RECEIPT_SUPPLEMENT,
	    

	/**
	 * 2007 Partial Lookthrough Depository Receipt Supplement to the 2002 ISDA Equity Derivatives Definitions.
	 */
	@RosettaSynonym(value = "ISDA2007PartialLookthroughDepositoryReceiptSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2007PartialLookthroughDepositoryReceiptSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2007PartialLookthroughDepositoryReceiptSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2007PartialLookthroughDepositoryReceiptSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2007PartialLookthroughDepositoryReceiptSupplement", source = "CME_ClearedConfirm_1_17")
	ISDA_2007_PARTIAL_LOOKTHROUGH_DEPOSITORY_RECEIPT_SUPPLEMENT,
	    

	/**
	 * Additional Provisions for Physically Settled Default Swaps Monoline Insurer.
	 */
	@RosettaSynonym(value = "ISDACreditMonolineInsurers", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDACreditMonolineInsurers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDACreditMonolineInsurers", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDACreditMonolineInsurers", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDACreditMonolineInsurers", source = "CME_ClearedConfirm_1_17")
	ISDA_CREDIT_MONOLINE_INSURERS,
	    

	/**
	 * Additional Provisions for Fixed Recovery Credit Default Swap Transactions
	 */
	@RosettaSynonym(value = "ISDADeliveryRestrictions", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDADeliveryRestrictions", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDADeliveryRestrictions", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDADeliveryRestrictions", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDADeliveryRestrictions", source = "CME_ClearedConfirm_1_17")
	ISDA_DELIVERY_RESTRICTIONS,
	    

	/**
	 * Additional Provisions for Fixed Recovery Credit Default Swap Transactions.
	 */
	@RosettaSynonym(value = "ISDAFixedRecovery", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDAFixedRecovery", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDAFixedRecovery", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDAFixedRecovery", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDAFixedRecovery", source = "CME_ClearedConfirm_1_17")
	ISDA_FIXED_RECOVERY,
	    

	/**
	 * Additional Provisions for LPN Reference Entities.
	 */
	@RosettaSynonym(value = "ISDALPNReferenceEntities", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDALPNReferenceEntities", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDALPNReferenceEntities", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDALPNReferenceEntities", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDALPNReferenceEntities", source = "CME_ClearedConfirm_1_17")
	ISDALPN_REFERENCE_ENTITIES,
	    

	/**
	 * Canadian Supplement to the 2004 Americas Interdealer Master Equity Derivatives Confirmation Agreement dated March 29, 2004.
	 */
	@RosettaSynonym(value = "ISDAMarch2004EquityCanadianSupplement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDAMarch2004EquityCanadianSupplement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDAMarch2004EquityCanadianSupplement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDAMarch2004EquityCanadianSupplement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDAMarch2004EquityCanadianSupplement", source = "CME_ClearedConfirm_1_17")
	ISDA_MARCH_2004_EQUITY_CANADIAN_SUPPLEMENT,
	    

	/**
	 * Additional Provisions for Recovery Lock Credit Default Swap Transactions.
	 */
	@RosettaSynonym(value = "ISDARecoveryLock", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDARecoveryLock", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDARecoveryLock", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDARecoveryLock", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDARecoveryLock", source = "CME_ClearedConfirm_1_17")
	ISDA_RECOVERY_LOCK,
	    

	/**
	 * Additional Provisions for Secured Deliverable Obligation Characteristic.
	 */
	@RosettaSynonym(value = "ISDASecuredDeliverableObligationCharacteristic", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDASecuredDeliverableObligationCharacteristic", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDASecuredDeliverableObligationCharacteristic", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDASecuredDeliverableObligationCharacteristic", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDASecuredDeliverableObligationCharacteristic", source = "CME_ClearedConfirm_1_17")
	ISDA_SECURED_DELIVERABLE_OBLIGATION_CHARACTERISTIC,
	    

	/**
	 * Standard Terms Supplement for Syndicated Secured Loan Credit Default Swap Index Transactions.
	 */
	@RosettaSynonym(value = "LCDX", source = "FpML_5_10")
	@RosettaSynonym(value = "LCDX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LCDX", source = "DTCC_11_0")
	@RosettaSynonym(value = "LCDX", source = "DTCC_9_0")
	@RosettaSynonym(value = "LCDX", source = "CME_ClearedConfirm_1_17")
	LCDX,
	    

	/**
	 * Standard Terms Supplement for Syndicated Secured Loan Credit Default Swap Index Tranche Transactions.
	 */
	@RosettaSynonym(value = "LCDXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "LCDXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LCDXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "LCDXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "LCDXTranche", source = "CME_ClearedConfirm_1_17")
	LCDX_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for MBX Transactions.
	 */
	@RosettaSynonym(value = "MBX", source = "FpML_5_10")
	@RosettaSynonym(value = "MBX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MBX", source = "DTCC_11_0")
	@RosettaSynonym(value = "MBX", source = "DTCC_9_0")
	@RosettaSynonym(value = "MBX", source = "CME_ClearedConfirm_1_17")
	MBX,
	    

	/**
	 * Standard Terms Supplement for Municipal CDX Untranched Transactions.
	 */
	@RosettaSynonym(value = "MCDX", source = "FpML_5_10")
	@RosettaSynonym(value = "MCDX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MCDX", source = "DTCC_11_0")
	@RosettaSynonym(value = "MCDX", source = "DTCC_9_0")
	@RosettaSynonym(value = "MCDX", source = "CME_ClearedConfirm_1_17")
	MCDX,
	    

	/**
	 * Standard Terms Supplement for PO Index Transactions.
	 */
	@RosettaSynonym(value = "PO", source = "FpML_5_10")
	@RosettaSynonym(value = "PO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PO", source = "DTCC_11_0")
	@RosettaSynonym(value = "PO", source = "DTCC_9_0")
	@RosettaSynonym(value = "PO", source = "CME_ClearedConfirm_1_17")
	PO,
	    

	/**
	 * Standard Terms Supplement for PrimeX Transactions.
	 */
	@RosettaSynonym(value = "PrimeX", source = "FpML_5_10")
	@RosettaSynonym(value = "PrimeX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PrimeX", source = "DTCC_11_0")
	@RosettaSynonym(value = "PrimeX", source = "DTCC_9_0")
	@RosettaSynonym(value = "PrimeX", source = "CME_ClearedConfirm_1_17")
	PRIME_X,
	    

	/**
	 * Standard Terms Supplement for Standard CDX Tranche Transactions.
	 */
	@RosettaSynonym(value = "StandardCDXTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardCDXTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardCDXTranche", source = "CME_ClearedConfirm_1_17")
	STANDARD_CDX_TRANCHE,
	    

	/**
	 * Standard Syndicated Secured Loan Credit Default Swap Standard Terms Supplement.
	 */
	@RosettaSynonym(value = "StandardLCDS", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDS", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDS", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDS", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDS,
	    

	/**
	 * Standard Terms Supplement for Standard Syndicated Secured Loan Credit Default Swap Bullet Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDSBullet", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDSBullet", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDS_BULLET,
	    

	/**
	 * Standard Terms Supplement for Standard Syndicated Secured Loan Credit Default Swap Index Bullet Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDXBullet", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDXBullet", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDX_BULLET,
	    

	/**
	 * Standard Terms Supplement for Standard Syndicated Secured Loan Credit Default Swap Index Bullet Tranche Transactions.
	 */
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardLCDXBulletTranche", source = "CME_ClearedConfirm_1_17")
	STANDARD_LCDX_BULLET_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for Standard iTraxx Europe Tranched Transactions.
	 */
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardiTraxxEuropeTranche", source = "CME_ClearedConfirm_1_17")
	STANDARDI_TRAXX_EUROPE_TRANCHE,
	    

	/**
	 * Syndicated Secured Loan Credit Default Swap Standard Terms Supplement.
	 */
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "FpML_5_10")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "DTCC_11_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "DTCC_9_0")
	@RosettaSynonym(value = "SyndicatedSecuredLoanCDS", source = "CME_ClearedConfirm_1_17")
	SYNDICATED_SECURED_LOAN_CDS,
	    

	/**
	 * Standard Terms Supplement for TRX Transactions.
	 */
	@RosettaSynonym(value = "TRX", source = "FpML_5_10")
	@RosettaSynonym(value = "TRX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRX", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRX", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRX", source = "CME_ClearedConfirm_1_17")
	TRX,
	    

	/**
	 * Standard Terms Supplement for TRX.II Transactions.
	 */
	@RosettaSynonym(value = "TRX.II", source = "FpML_5_10")
	@RosettaSynonym(value = "TRX.II", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRX.II", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRX.II", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRX.II", source = "CME_ClearedConfirm_1_17")
	TRX_II("TRX.II"),
	

	/**
	 * Standard Terms Supplement for iTraxx Asia Excluding Japan.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapan", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN,
	    

	/**
	 * Standard Terms Supplement for iTraxx Asia Ex-Japan Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN_SWAPTION,
	    

	/**
	 * Standard Terms Supplement for iTraxx Asia Excluding Japan Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAsiaExJapanTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_ASIA_EX_JAPAN_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for iTraxx Australia.
	 */
	@RosettaSynonym(value = "iTraxxAustralia", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustralia", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustralia", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA,
	    

	/**
	 * Standard Terms Supplement for iTraxx Australia Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustraliaSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA_SWAPTION,
	    

	/**
	 * Standard Terms Supplement for iTraxx Australia Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxAustraliaTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_AUSTRALIA_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for iTraxx CJ.
	 */
	@RosettaSynonym(value = "iTraxxCJ", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxCJ", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxCJ", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_CJ,
	    

	/**
	 * Standard Terms Supplement for iTraxx CJ Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxCJTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxCJTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_CJ_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for iTraxx Europe Transactions.
	 */
	@RosettaSynonym(value = "iTraxxEurope", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEurope", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEurope", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE,
	    

	/**
	 * Standard Terms Supplement for iTraxx Europe Dealer Form.
	 */
	@RosettaSynonym(value = "iTraxxEuropeDealer", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeDealer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeDealer", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeDealer", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeDealer", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_DEALER,
	    

	/**
	 * Standard Terms Supplement for iTraxx Europe Non-Dealer Form.
	 */
	@RosettaSynonym(value = "iTraxxEuropeNonDealer", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeNonDealer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeNonDealer", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeNonDealer", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeNonDealer", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_NON_DEALER,
	    

	/**
	 * Standard Terms Supplement for iTraxx Europe Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_SWAPTION,
	    

	/**
	 * Standard Terms Supplement for iTraxx Europe Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxEuropeTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_EUROPE_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for iTraxx Japan.
	 */
	@RosettaSynonym(value = "iTraxxJapan", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapan", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN,
	    

	/**
	 * Standard Terms Supplement for iTraxx Japan Swaption Transactions.
	 */
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapanSwaption", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN_SWAPTION,
	    

	/**
	 * Standard Terms Supplement for iTraxx Japan Tranched Transactions.
	 */
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxJapanTranche", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_JAPAN_TRANCHE,
	    

	/**
	 * Standard Terms Supplement for iTraxx LevX.
	 */
	@RosettaSynonym(value = "iTraxxLevX", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxLevX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxLevX", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_LEV_X,
	    

	/**
	 * Standard Terms Supplement for iTraxx SDI 75 Dealer Transactions.
	 */
	@RosettaSynonym(value = "iTraxxSDI75Dealer", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxSDI75Dealer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxSDI75Dealer", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxSDI75Dealer", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxSDI75Dealer", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_SDI_75_DEALER,
	    

	/**
	 * Standard Terms Supplement for iTraxx SDI 75 Non-Dealer Transactions.
	 */
	@RosettaSynonym(value = "iTraxxSDI75NonDealer", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxSDI75NonDealer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxSDI75NonDealer", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxSDI75NonDealer", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxSDI75NonDealer", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_SDI_75_NON_DEALER,
	    

	/**
	 * Standard Terms Supplement for iTraxx SovX.
	 */
	@RosettaSynonym(value = "iTraxxSovX", source = "FpML_5_10")
	@RosettaSynonym(value = "iTraxxSovX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "DTCC_11_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "DTCC_9_0")
	@RosettaSynonym(value = "iTraxxSovX", source = "CME_ClearedConfirm_1_17")
	I_TRAXX_SOV_X    
	;

	private final String displayName;
	
	ContractualSupplementEnum() {
		this.displayName = null;
	}

	ContractualSupplementEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
