package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaContractualSupplementEnum;
import org.isda.cdm.metafields.FieldWithMetaContractualSupplementEnum.FieldWithMetaContractualSupplementEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ContractualTermsSupplementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A contractual supplement (such as those published by ISDA) and its publication date that will apply to the trade.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ContractualTermsSupplement", source="FpML_5_10")
public class ContractualTermsSupplement extends RosettaModelObject {
	private final Date publicationDate;
	private final FieldWithMetaContractualSupplementEnum type;
	private static ContractualTermsSupplementMeta metaData = new ContractualTermsSupplementMeta();
	
	ContractualTermsSupplement(ContractualTermsSupplementBuilder builder) {
		this.publicationDate = builder.getPublicationDate();
		this.type = ofNullable(builder.getType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the publication date of the applicable version of the contractual supplement.
	 */
	@RosettaSynonym(value="publicationDate", source="FpML_5_10")
	@RosettaSynonym(value="publicationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_11_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_9_0")
	@RosettaSynonym(value="publicationDate", source="CME_ClearedConfirm_1_17")
	public final Date getPublicationDate() {
		return publicationDate;
	}
	
	/**
	 * Identifies the form of applicable contractual supplement.
	 */
	@RosettaSynonym(value="type", source="FpML_5_10")
	@RosettaSynonym(value="type", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="type", source="DTCC_11_0")
	@RosettaSynonym(value="type", source="DTCC_9_0")
	@RosettaSynonym(value="type", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaContractualSupplementEnum getType() {
		return type;
	}
	
	
	@Override
	public RosettaMetaData<? extends ContractualTermsSupplement> metaData() {
		return metaData;
	} 
	
	public ContractualTermsSupplementBuilder toBuilder() {
		ContractualTermsSupplementBuilder builder = new ContractualTermsSupplementBuilder();
		ofNullable(getPublicationDate()).ifPresent(builder::setPublicationDate);
		ofNullable(getType()).ifPresent(builder::setType);
		return builder;
	}

	public static ContractualTermsSupplementBuilder builder() {
		return new ContractualTermsSupplementBuilder();
	}

	public static  class ContractualTermsSupplementBuilder extends RosettaModelObjectBuilder {
	
		protected Date publicationDate;
		protected FieldWithMetaContractualSupplementEnumBuilder type;
	
		public ContractualTermsSupplementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ContractualTermsSupplement> metaData() {
			return metaData;
		} 
	
		
		public Date getPublicationDate() {
			return publicationDate;
		}
		
		
		public FieldWithMetaContractualSupplementEnumBuilder getType() {
			return type;
		}
		
		public FieldWithMetaContractualSupplementEnumBuilder getOrCreateType() {
			if (type!=null) {
				return type;
			}
			else return type = new FieldWithMetaContractualSupplementEnumBuilder();
		}
		
	
		public ContractualTermsSupplementBuilder setPublicationDate(Date publicationDate) {
			this.publicationDate = publicationDate;
			return this;
		}
		public ContractualTermsSupplementBuilder setType(FieldWithMetaContractualSupplementEnum type) {
			this.type = type.toBuilder();
			return this;
		}
		public ContractualTermsSupplementBuilder setTypeRef(ContractualSupplementEnum type) {
			return setType(FieldWithMetaContractualSupplementEnum.builder().setValue(type).build());
		}
		
		public ContractualTermsSupplement build() {
			return new ContractualTermsSupplement(this);
		}
	
		@Override
		public ContractualTermsSupplementBuilder prune() {
			if (type!=null && !type.prune().hasData()) type = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPublicationDate()!=null) return true;
			if (getType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
			
			processRosetta(path.newSubPath("type"), processor, FieldWithMetaContractualSupplementEnum.class, type);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ContractualTermsSupplementBuilder _that = (ContractualTermsSupplementBuilder) o;
		
			if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
			if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
			_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ContractualTermsSupplementBuilder {" +
				"publicationDate=" + this.publicationDate + ", " +
				"type=" + this.type +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
		
		processRosetta(path.newSubPath("type"), processor, FieldWithMetaContractualSupplementEnum.class, type);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ContractualTermsSupplement _that = (ContractualTermsSupplement) o;
	
		if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
		if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
		_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ContractualTermsSupplement {" +
			"publicationDate=" + this.publicationDate + ", " +
			"type=" + this.type +
		'}';
	}
}
