package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.LimitApplicableExtended;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditLimitInformationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the credit limit utilisation information.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditLimitInformation", source="CME_ClearedConfirm_1_17")
public class CreditLimitInformation extends RosettaModelObject {
	private final List<LimitApplicableExtended> limitApplicable;
	private static CreditLimitInformationMeta metaData = new CreditLimitInformationMeta();
	
	CreditLimitInformation(CreditLimitInformationBuilder builder) {
		this.limitApplicable = ofNullable(builder.getLimitApplicable()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	@RosettaSynonym(value="limitApplicable", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="limitApplicable", source="Rosetta_Workbench")
	public final List<LimitApplicableExtended> getLimitApplicable() {
		return limitApplicable;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditLimitInformation> metaData() {
		return metaData;
	} 
	
	public CreditLimitInformationBuilder toBuilder() {
		CreditLimitInformationBuilder builder = new CreditLimitInformationBuilder();
		ofNullable(getLimitApplicable()).ifPresent(limitApplicable -> limitApplicable.forEach(builder::addLimitApplicable));
		return builder;
	}

	public static CreditLimitInformationBuilder builder() {
		return new CreditLimitInformationBuilder();
	}

	public static  class CreditLimitInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<LimitApplicableExtended.LimitApplicableExtendedBuilder> limitApplicable;
	
		public CreditLimitInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditLimitInformation> metaData() {
			return metaData;
		} 
	
		
		public List<LimitApplicableExtended.LimitApplicableExtendedBuilder> getLimitApplicable() {
			return limitApplicable;
		}
		
		public LimitApplicableExtended.LimitApplicableExtendedBuilder getOrCreateLimitApplicable(int index) {
			if (limitApplicable==null) {
				this.limitApplicable = new ArrayList<>();
			}
			return getIndex(limitApplicable, index, ()->new LimitApplicableExtended.LimitApplicableExtendedBuilder());
		}
		
	
		public CreditLimitInformationBuilder addLimitApplicable(LimitApplicableExtended limitApplicable) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			this.limitApplicable.add(limitApplicable.toBuilder());
			return this;
		}
		public CreditLimitInformationBuilder addLimitApplicable(LimitApplicableExtended limitApplicable, int _idx) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			getIndex(this.limitApplicable, _idx, () -> limitApplicable.toBuilder());
			this.limitApplicable.set(_idx, limitApplicable.toBuilder());
			return this;
		}
		public CreditLimitInformationBuilder addLimitApplicable(List<LimitApplicableExtended> limitApplicables) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			for (LimitApplicableExtended toAdd : limitApplicables) {
				this.limitApplicable.add(toAdd.toBuilder());
			}
			return this;
		}
		public CreditLimitInformationBuilder addLimitApplicableBuilder(LimitApplicableExtended.LimitApplicableExtendedBuilder limitApplicable) {
			if(this.limitApplicable == null){
				this.limitApplicable = new ArrayList<>();
			}
			this.limitApplicable.add(limitApplicable);
			return this;
		}
		
		public CreditLimitInformationBuilder clearLimitApplicable() {
			this.limitApplicable = null;
			return this;
		}
		
		public CreditLimitInformation build() {
			return new CreditLimitInformation(this);
		}
	
		@Override
		public CreditLimitInformationBuilder prune() {
			if (limitApplicable!=null) limitApplicable = limitApplicable.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLimitApplicable()!=null && getLimitApplicable().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("limitApplicable"), processor, LimitApplicableExtended.class, limitApplicable);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditLimitInformationBuilder _that = (CreditLimitInformationBuilder) o;
		
			if (!ListEquals.listEquals(limitApplicable, _that.limitApplicable)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (limitApplicable != null ? limitApplicable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditLimitInformationBuilder {" +
				"limitApplicable=" + this.limitApplicable +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("limitApplicable"), processor, LimitApplicableExtended.class, limitApplicable);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditLimitInformation _that = (CreditLimitInformation) o;
	
		if (!ListEquals.listEquals(limitApplicable, _that.limitApplicable)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (limitApplicable != null ? limitApplicable.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditLimitInformation {" +
			"limitApplicable=" + this.limitApplicable +
		'}';
	}
}
