package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to qualify the type of credit limits.
 * 
 * @version 2.5.4
 */
public enum CreditLimitTypeEnum {
    

	/**
	 * The type of credit line expressed in CS01. The sensitivity with respect to changes in the CDS spread.
	 */
	@RosettaSynonym(value = "CS01", source = "FpML_5_10")
	@RosettaSynonym(value = "CS01", source = "CME_ClearedConfirm_1_17")
	CS01,
	    

	/**
	 * The type of credit line expressed in DV01. The dollar value of a one basis point decrease in interest rates. It shows the change in a bond&#39;s price compared to a decrease in the bond&#39;s yield.
	 */
	@RosettaSynonym(value = "DV01", source = "FpML_5_10")
	@RosettaSynonym(value = "DV01", source = "CME_ClearedConfirm_1_17")
	DV01,
	    

	/**
	 * The type of credit line expressed in Initial Margin value.
	 */
	@RosettaSynonym(value = "IM", source = "FpML_5_10")
	@RosettaSynonym(value = "IM", source = "CME_ClearedConfirm_1_17")
	IM,
	    

	/**
	 * The type of credit line expressed in Notional amount.
	 */
	@RosettaSynonym(value = "Notional", source = "FpML_5_10")
	@RosettaSynonym(value = "Notional", source = "CME_ClearedConfirm_1_17")
	NOTIONAL,
	    

	/**
	 * The type of credit line expressed as a Net Present Value.
	 */
	@RosettaSynonym(value = "NPV", source = "FpML_5_10")
	@RosettaSynonym(value = "NPV", source = "CME_ClearedConfirm_1_17")
	NPV,
	    

	/**
	 * The type of credit line expressed in PV01. The value of a one dollar or one basis point annuity.
	 */
	@RosettaSynonym(value = "PV01", source = "FpML_5_10")
	@RosettaSynonym(value = "PV01", source = "CME_ClearedConfirm_1_17")
	PV01    
	;

	private final String displayName;
	
	CreditLimitTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
