package org.isda.cdm;

import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditLimitUtilisationPositionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditLimitUtilizationPosition", source="FpML_5_10")
public class CreditLimitUtilisationPosition extends RosettaModelObject {
	private final BigDecimal global;
	private final BigDecimal longPosition;
	private final BigDecimal shortPosition;
	private static CreditLimitUtilisationPositionMeta metaData = new CreditLimitUtilisationPositionMeta();
	
	CreditLimitUtilisationPosition(CreditLimitUtilisationPositionBuilder builder) {
		this.global = builder.getGlobal();
		this.longPosition = builder.getLongPosition();
		this.shortPosition = builder.getShortPosition();
	}
	
	/**
	 * Global credit limit utilisation amount, agnostic of long/short position direction.
	 */
	@RosettaSynonym(value="global", source="FpML_5_10")
	public final BigDecimal getGlobal() {
		return global;
	}
	
	/**
	 * Credit limit utilisation attributable to long positions.
	 */
	@RosettaSynonym(value="long", source="FpML_5_10")
	public final BigDecimal getLongPosition() {
		return longPosition;
	}
	
	/**
	 * Credit limit utilisation attributable to short positions.
	 */
	@RosettaSynonym(value="short", source="FpML_5_10")
	public final BigDecimal getShortPosition() {
		return shortPosition;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditLimitUtilisationPosition> metaData() {
		return metaData;
	} 
	
	public CreditLimitUtilisationPositionBuilder toBuilder() {
		CreditLimitUtilisationPositionBuilder builder = new CreditLimitUtilisationPositionBuilder();
		ofNullable(getGlobal()).ifPresent(builder::setGlobal);
		ofNullable(getLongPosition()).ifPresent(builder::setLongPosition);
		ofNullable(getShortPosition()).ifPresent(builder::setShortPosition);
		return builder;
	}

	public static CreditLimitUtilisationPositionBuilder builder() {
		return new CreditLimitUtilisationPositionBuilder();
	}

	public static  class CreditLimitUtilisationPositionBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal global;
		protected BigDecimal longPosition;
		protected BigDecimal shortPosition;
	
		public CreditLimitUtilisationPositionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditLimitUtilisationPosition> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getGlobal() {
			return global;
		}
		
		
		public BigDecimal getLongPosition() {
			return longPosition;
		}
		
		
		public BigDecimal getShortPosition() {
			return shortPosition;
		}
		
	
		public CreditLimitUtilisationPositionBuilder setGlobal(BigDecimal global) {
			this.global = global;
			return this;
		}
		public CreditLimitUtilisationPositionBuilder setLongPosition(BigDecimal longPosition) {
			this.longPosition = longPosition;
			return this;
		}
		public CreditLimitUtilisationPositionBuilder setShortPosition(BigDecimal shortPosition) {
			this.shortPosition = shortPosition;
			return this;
		}
		
		public CreditLimitUtilisationPosition build() {
			return new CreditLimitUtilisationPosition(this);
		}
	
		@Override
		public CreditLimitUtilisationPositionBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getGlobal()!=null) return true;
			if (getLongPosition()!=null) return true;
			if (getShortPosition()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("global"), BigDecimal.class, global, this);
			processor.processBasic(path.newSubPath("longPosition"), BigDecimal.class, longPosition, this);
			processor.processBasic(path.newSubPath("shortPosition"), BigDecimal.class, shortPosition, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditLimitUtilisationPositionBuilder _that = (CreditLimitUtilisationPositionBuilder) o;
		
			if (global != null ? !global.equals(_that.global) : _that.global != null) return false;
			if (longPosition != null ? !longPosition.equals(_that.longPosition) : _that.longPosition != null) return false;
			if (shortPosition != null ? !shortPosition.equals(_that.shortPosition) : _that.shortPosition != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (global != null ? global.hashCode() : 0);
			_result = 31 * _result + (longPosition != null ? longPosition.hashCode() : 0);
			_result = 31 * _result + (shortPosition != null ? shortPosition.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditLimitUtilisationPositionBuilder {" +
				"global=" + this.global + ", " +
				"longPosition=" + this.longPosition + ", " +
				"shortPosition=" + this.shortPosition +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("global"), BigDecimal.class, global, this);
		processor.processBasic(path.newSubPath("longPosition"), BigDecimal.class, longPosition, this);
		processor.processBasic(path.newSubPath("shortPosition"), BigDecimal.class, shortPosition, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditLimitUtilisationPosition _that = (CreditLimitUtilisationPosition) o;
	
		if (global != null ? !global.equals(_that.global) : _that.global != null) return false;
		if (longPosition != null ? !longPosition.equals(_that.longPosition) : _that.longPosition != null) return false;
		if (shortPosition != null ? !shortPosition.equals(_that.shortPosition) : _that.shortPosition != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (global != null ? global.hashCode() : 0);
		_result = 31 * _result + (longPosition != null ? longPosition.hashCode() : 0);
		_result = 31 * _result + (shortPosition != null ? shortPosition.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditLimitUtilisationPosition {" +
			"global=" + this.global + ", " +
			"longPosition=" + this.longPosition + ", " +
			"shortPosition=" + this.shortPosition +
		'}';
	}
}
