package org.isda.cdm;

import org.isda.cdm.MultipleDebtTypes;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditRatingDebtMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The credit rating debt type(s) associated with the credit rating notation and scale. When several debt types are specified, they must be qualified through an &#39;any&#39; or &#39;all&#39;.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditRatingDebt", source="FpML_5_10")
public class CreditRatingDebt extends RosettaModelObject {
	private final FieldWithMetaString debtType;
	private final MultipleDebtTypes debtTypes;
	private static CreditRatingDebtMeta metaData = new CreditRatingDebtMeta();
	
	CreditRatingDebt(CreditRatingDebtBuilder builder) {
		this.debtType = ofNullable(builder.getDebtType()).map(f->f.build()).orElse(null);
		this.debtTypes = ofNullable(builder.getDebtTypes()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * This attribute is to be specified when only one debt type is specified. FpML doesn&#39;t specify values in relation to the associated scheme, which is hence not specified as an enumeration as part of the CDM.
	 */
	@RosettaSynonym(value="debtType", source="FpML_5_10")
	public final FieldWithMetaString getDebtType() {
		return debtType;
	}
	
	/**
	 * This attribute provides the ability to specify several debt types, alongside an &#39;any&#39; or &#39;all&#39; or all condition. As an example, Baa1 rating is required for any long term debt and deposit.
	 */
	public final MultipleDebtTypes getDebtTypes() {
		return debtTypes;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditRatingDebt> metaData() {
		return metaData;
	} 
	
	public CreditRatingDebtBuilder toBuilder() {
		CreditRatingDebtBuilder builder = new CreditRatingDebtBuilder();
		ofNullable(getDebtType()).ifPresent(builder::setDebtType);
		ofNullable(getDebtTypes()).ifPresent(builder::setDebtTypes);
		return builder;
	}

	public static CreditRatingDebtBuilder builder() {
		return new CreditRatingDebtBuilder();
	}

	public static  class CreditRatingDebtBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder debtType;
		protected MultipleDebtTypes.MultipleDebtTypesBuilder debtTypes;
	
		public CreditRatingDebtBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditRatingDebt> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getDebtType() {
			return debtType;
		}
		
		public FieldWithMetaStringBuilder getOrCreateDebtType() {
			if (debtType!=null) {
				return debtType;
			}
			else return debtType = new FieldWithMetaStringBuilder();
		}
		
		
		public MultipleDebtTypes.MultipleDebtTypesBuilder getDebtTypes() {
			return debtTypes;
		}
		
		public MultipleDebtTypes.MultipleDebtTypesBuilder getOrCreateDebtTypes() {
			if (debtTypes!=null) {
				return debtTypes;
			}
			else return debtTypes = new MultipleDebtTypes.MultipleDebtTypesBuilder();
		}
		
	
		public CreditRatingDebtBuilder setDebtType(FieldWithMetaString debtType) {
			this.debtType = debtType.toBuilder();
			return this;
		}
		public CreditRatingDebtBuilder setDebtTypeRef(String debtType) {
			return setDebtType(FieldWithMetaString.builder().setValue(debtType).build());
		}
		public CreditRatingDebtBuilder setDebtTypes(MultipleDebtTypes debtTypes) {
			this.debtTypes = debtTypes.toBuilder();
			return this;
		}
		public CreditRatingDebtBuilder setDebtTypesBuilder(MultipleDebtTypes.MultipleDebtTypesBuilder debtTypes) {
			this.debtTypes = debtTypes;
			return this;
		}
		
		public CreditRatingDebt build() {
			return new CreditRatingDebt(this);
		}
	
		@Override
		public CreditRatingDebtBuilder prune() {
			if (debtType!=null && !debtType.prune().hasData()) debtType = null;
			if (debtTypes!=null && !debtTypes.prune().hasData()) debtTypes = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDebtType()!=null) return true;
			if (getDebtTypes()!=null && getDebtTypes().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("debtType"), processor, FieldWithMetaString.class, debtType);
			processRosetta(path.newSubPath("debtTypes"), processor, MultipleDebtTypes.class, debtTypes);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditRatingDebtBuilder _that = (CreditRatingDebtBuilder) o;
		
			if (debtType != null ? !debtType.equals(_that.debtType) : _that.debtType != null) return false;
			if (debtTypes != null ? !debtTypes.equals(_that.debtTypes) : _that.debtTypes != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (debtType != null ? debtType.hashCode() : 0);
			_result = 31 * _result + (debtTypes != null ? debtTypes.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditRatingDebtBuilder {" +
				"debtType=" + this.debtType + ", " +
				"debtTypes=" + this.debtTypes +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("debtType"), processor, FieldWithMetaString.class, debtType);
		processRosetta(path.newSubPath("debtTypes"), processor, MultipleDebtTypes.class, debtTypes);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditRatingDebt _that = (CreditRatingDebt) o;
	
		if (debtType != null ? !debtType.equals(_that.debtType) : _that.debtType != null) return false;
		if (debtTypes != null ? !debtTypes.equals(_that.debtTypes) : _that.debtTypes != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (debtType != null ? debtType.hashCode() : 0);
		_result = 31 * _result + (debtTypes != null ? debtTypes.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditRatingDebt {" +
			"debtType=" + this.debtType + ", " +
			"debtTypes=" + this.debtTypes +
		'}';
	}
}
