package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaCreditSupportAgreementTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaCreditSupportAgreementTypeEnum.FieldWithMetaCreditSupportAgreementTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CreditSupportAgreementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * The agreement executed between the parties and intended to govern collateral arrangement for all OTC derivatives transactions between those parties.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="CreditSupportAgreement", source="FpML_5_10")
public class CreditSupportAgreement extends RosettaModelObject {
	private final Date date;
	private final String identifierValue;
	private final FieldWithMetaCreditSupportAgreementTypeEnum type;
	private static CreditSupportAgreementMeta metaData = new CreditSupportAgreementMeta();
	
	CreditSupportAgreement(CreditSupportAgreementBuilder builder) {
		this.date = builder.getDate();
		this.identifierValue = builder.getIdentifierValue();
		this.type = ofNullable(builder.getType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date of the agreement executed between the parties and intended to govern collateral arrangements for all OTC derivatives transactions between those parties.
	 */
	@RosettaSynonym(value="date", source="FpML_5_10")
	@RosettaSynonym(value="date", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="date", source="DTCC_11_0")
	@RosettaSynonym(value="date", source="DTCC_9_0")
	@RosettaSynonym(value="date", source="CME_ClearedConfirm_1_17")
	public final Date getDate() {
		return date;
	}
	
	/**
	 * An identifier used to uniquely identify the CSA. FpML specifies the type as creditSupportAgreementIdScheme, but without proposing any value.  As far as e understand, no scheme has yet been developed at this point.
	 */
	@RosettaSynonym(value="identifier", source="FpML_5_10")
	@RosettaSynonym(value="identifier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="identifier", source="DTCC_11_0")
	@RosettaSynonym(value="identifier", source="DTCC_9_0")
	@RosettaSynonym(value="identifier", source="CME_ClearedConfirm_1_17")
	public final String getIdentifierValue() {
		return identifierValue;
	}
	
	/**
	 * The type of ISDA Credit Support Agreement.
	 */
	@RosettaSynonym(value="type", source="FpML_5_10")
	@RosettaSynonym(value="type", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="type", source="DTCC_11_0")
	@RosettaSynonym(value="type", source="DTCC_9_0")
	@RosettaSynonym(value="type", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaCreditSupportAgreementTypeEnum getType() {
		return type;
	}
	
	
	@Override
	public RosettaMetaData<? extends CreditSupportAgreement> metaData() {
		return metaData;
	} 
	
	public CreditSupportAgreementBuilder toBuilder() {
		CreditSupportAgreementBuilder builder = new CreditSupportAgreementBuilder();
		ofNullable(getDate()).ifPresent(builder::setDate);
		ofNullable(getIdentifierValue()).ifPresent(builder::setIdentifierValue);
		ofNullable(getType()).ifPresent(builder::setType);
		return builder;
	}

	public static CreditSupportAgreementBuilder builder() {
		return new CreditSupportAgreementBuilder();
	}

	public static  class CreditSupportAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected Date date;
		protected String identifierValue;
		protected FieldWithMetaCreditSupportAgreementTypeEnumBuilder type;
	
		public CreditSupportAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CreditSupportAgreement> metaData() {
			return metaData;
		} 
	
		
		public Date getDate() {
			return date;
		}
		
		
		public String getIdentifierValue() {
			return identifierValue;
		}
		
		
		public FieldWithMetaCreditSupportAgreementTypeEnumBuilder getType() {
			return type;
		}
		
		public FieldWithMetaCreditSupportAgreementTypeEnumBuilder getOrCreateType() {
			if (type!=null) {
				return type;
			}
			else return type = new FieldWithMetaCreditSupportAgreementTypeEnumBuilder();
		}
		
	
		public CreditSupportAgreementBuilder setDate(Date date) {
			this.date = date;
			return this;
		}
		public CreditSupportAgreementBuilder setIdentifierValue(String identifierValue) {
			this.identifierValue = identifierValue;
			return this;
		}
		public CreditSupportAgreementBuilder setType(FieldWithMetaCreditSupportAgreementTypeEnum type) {
			this.type = type.toBuilder();
			return this;
		}
		public CreditSupportAgreementBuilder setTypeRef(CreditSupportAgreementTypeEnum type) {
			return setType(FieldWithMetaCreditSupportAgreementTypeEnum.builder().setValue(type).build());
		}
		
		public CreditSupportAgreement build() {
			return new CreditSupportAgreement(this);
		}
	
		@Override
		public CreditSupportAgreementBuilder prune() {
			if (type!=null && !type.prune().hasData()) type = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDate()!=null) return true;
			if (getIdentifierValue()!=null) return true;
			if (getType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			processor.processBasic(path.newSubPath("identifierValue"), String.class, identifierValue, this);
			
			processRosetta(path.newSubPath("type"), processor, FieldWithMetaCreditSupportAgreementTypeEnum.class, type);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CreditSupportAgreementBuilder _that = (CreditSupportAgreementBuilder) o;
		
			if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
			if (identifierValue != null ? !identifierValue.equals(_that.identifierValue) : _that.identifierValue != null) return false;
			if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			_result = 31 * _result + (identifierValue != null ? identifierValue.hashCode() : 0);
			_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CreditSupportAgreementBuilder {" +
				"date=" + this.date + ", " +
				"identifierValue=" + this.identifierValue + ", " +
				"type=" + this.type +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("date"), Date.class, date, this);
		processor.processBasic(path.newSubPath("identifierValue"), String.class, identifierValue, this);
		
		processRosetta(path.newSubPath("type"), processor, FieldWithMetaCreditSupportAgreementTypeEnum.class, type);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CreditSupportAgreement _that = (CreditSupportAgreement) o;
	
		if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
		if (identifierValue != null ? !identifierValue.equals(_that.identifierValue) : _that.identifierValue != null) return false;
		if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		_result = 31 * _result + (identifierValue != null ? identifierValue.hashCode() : 0);
		_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CreditSupportAgreement {" +
			"date=" + this.date + ", " +
			"identifierValue=" + this.identifierValue + ", " +
			"type=" + this.type +
		'}';
	}
}
