package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of ISDA Credit Support Agreement governing the transaction.
 * 
 * @version 2.5.4
 */
public enum CreditSupportAgreementTypeEnum {
    

	/**
	 * The ISDA 1994 Credit Support Annex New York Law (pledge) applies.
	 */
	@RosettaSynonym(value = "ISDA1994CreditSupportAnnexNewYorkLaw", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1994CreditSupportAnnexNewYorkLaw", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1994CreditSupportAnnexNewYorkLaw", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1994CreditSupportAnnexNewYorkLaw", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1994CreditSupportAnnexNewYorkLaw", source = "CME_ClearedConfirm_1_17")
	ISDA_1994_CREDIT_SUPPORT_ANNEX_NEW_YORK_LAW,
	    

	/**
	 * The ISDA 1995 Credit Support Annex English Law (title transfer) applies.
	 */
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexEnglishLaw", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexEnglishLaw", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexEnglishLaw", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexEnglishLaw", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexEnglishLaw", source = "CME_ClearedConfirm_1_17")
	ISDA_1995_CREDIT_SUPPORT_ANNEX_ENGLISH_LAW,
	    

	/**
	 * The ISDA 1995 Credit Support Annex Japanese Law applies.
	 */
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexJapaneseLaw", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexJapaneseLaw", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexJapaneseLaw", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexJapaneseLaw", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportAnnexJapaneseLaw", source = "CME_ClearedConfirm_1_17")
	ISDA_1995_CREDIT_SUPPORT_ANNEX_JAPANESE_LAW,
	    

	/**
	 * The ISDA 1995 Credit Support Deed English Law (charge) applies.
	 */
	@RosettaSynonym(value = "ISDA1995CreditSupportDeedEnglishLaw", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA1995CreditSupportDeedEnglishLaw", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportDeedEnglishLaw", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportDeedEnglishLaw", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA1995CreditSupportDeedEnglishLaw", source = "CME_ClearedConfirm_1_17")
	ISDA_1995_CREDIT_SUPPORT_DEED_ENGLISH_LAW,
	    

	/**
	 * The ISDA 2001 Margin Provisions applies.
	 */
	@RosettaSynonym(value = "ISDA2001MarginProvisions", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2001MarginProvisions", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2001MarginProvisions", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2001MarginProvisions", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2001MarginProvisions", source = "CME_ClearedConfirm_1_17")
	ISDA_2001_MARGIN_PROVISIONS,
	    

	/**
	 * The ISDA 2013 Standard Credit Support Agreement.
	 */
	@RosettaSynonym(value = "ISDA2013StandardCreditSupportAgreement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2013StandardCreditSupportAgreement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2013StandardCreditSupportAgreement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2013StandardCreditSupportAgreement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2013StandardCreditSupportAgreement", source = "CME_ClearedConfirm_1_17")
	ISDA_2013_STANDARD_CREDIT_SUPPORT_AGREEMENT,
	    

	/**
	 * The ISDA 2014 Standard Credit Support Agreement.
	 */
	@RosettaSynonym(value = "ISDA2014StandardCreditSupportAgreement", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA2014StandardCreditSupportAgreement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA2014StandardCreditSupportAgreement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA2014StandardCreditSupportAgreement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA2014StandardCreditSupportAgreement", source = "CME_ClearedConfirm_1_17")
	ISDA_2014_STANDARD_CREDIT_SUPPORT_AGREEMENT    
	;

	private final String displayName;
	
	CreditSupportAgreementTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
