package org.isda.cdm;

import org.isda.cdm.PrincipalExchanges;
import org.isda.cdm.SettlementProvision;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CrossCurrencyTermsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class CrossCurrencyTerms extends RosettaModelObject {
	private final PrincipalExchanges principalExchanges;
	private final SettlementProvision settlementProvision;
	private static CrossCurrencyTermsMeta metaData = new CrossCurrencyTermsMeta();
	
	CrossCurrencyTerms(CrossCurrencyTermsBuilder builder) {
		this.principalExchanges = ofNullable(builder.getPrincipalExchanges()).map(f->f.build()).orElse(null);
		this.settlementProvision = ofNullable(builder.getSettlementProvision()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="principalExchanges", source="FpML_5_10")
	@RosettaSynonym(value="principalExchanges", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="principalExchanges", source="DTCC_11_0")
	@RosettaSynonym(value="principalExchanges", source="DTCC_9_0")
	@RosettaSynonym(value="principalExchanges", source="CME_ClearedConfirm_1_17")
	public final PrincipalExchanges getPrincipalExchanges() {
		return principalExchanges;
	}
	
	@RosettaSynonym(value="settlementProvision", source="FpML_5_10")
	@RosettaSynonym(value="settlementProvision", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settlementProvision", source="DTCC_11_0")
	@RosettaSynonym(value="settlementProvision", source="DTCC_9_0")
	@RosettaSynonym(value="settlementProvision", source="CME_ClearedConfirm_1_17")
	public final SettlementProvision getSettlementProvision() {
		return settlementProvision;
	}
	
	
	@Override
	public RosettaMetaData<? extends CrossCurrencyTerms> metaData() {
		return metaData;
	} 
	
	public CrossCurrencyTermsBuilder toBuilder() {
		CrossCurrencyTermsBuilder builder = new CrossCurrencyTermsBuilder();
		ofNullable(getPrincipalExchanges()).ifPresent(builder::setPrincipalExchanges);
		ofNullable(getSettlementProvision()).ifPresent(builder::setSettlementProvision);
		return builder;
	}

	public static CrossCurrencyTermsBuilder builder() {
		return new CrossCurrencyTermsBuilder();
	}

	public static  class CrossCurrencyTermsBuilder extends RosettaModelObjectBuilder {
	
		protected PrincipalExchanges.PrincipalExchangesBuilder principalExchanges;
		protected SettlementProvision.SettlementProvisionBuilder settlementProvision;
	
		public CrossCurrencyTermsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CrossCurrencyTerms> metaData() {
			return metaData;
		} 
	
		
		public PrincipalExchanges.PrincipalExchangesBuilder getPrincipalExchanges() {
			return principalExchanges;
		}
		
		public PrincipalExchanges.PrincipalExchangesBuilder getOrCreatePrincipalExchanges() {
			if (principalExchanges!=null) {
				return principalExchanges;
			}
			else return principalExchanges = new PrincipalExchanges.PrincipalExchangesBuilder();
		}
		
		
		public SettlementProvision.SettlementProvisionBuilder getSettlementProvision() {
			return settlementProvision;
		}
		
		public SettlementProvision.SettlementProvisionBuilder getOrCreateSettlementProvision() {
			if (settlementProvision!=null) {
				return settlementProvision;
			}
			else return settlementProvision = new SettlementProvision.SettlementProvisionBuilder();
		}
		
	
		public CrossCurrencyTermsBuilder setPrincipalExchanges(PrincipalExchanges principalExchanges) {
			this.principalExchanges = principalExchanges.toBuilder();
			return this;
		}
		public CrossCurrencyTermsBuilder setPrincipalExchangesBuilder(PrincipalExchanges.PrincipalExchangesBuilder principalExchanges) {
			this.principalExchanges = principalExchanges;
			return this;
		}
		public CrossCurrencyTermsBuilder setSettlementProvision(SettlementProvision settlementProvision) {
			this.settlementProvision = settlementProvision.toBuilder();
			return this;
		}
		public CrossCurrencyTermsBuilder setSettlementProvisionBuilder(SettlementProvision.SettlementProvisionBuilder settlementProvision) {
			this.settlementProvision = settlementProvision;
			return this;
		}
		
		public CrossCurrencyTerms build() {
			return new CrossCurrencyTerms(this);
		}
	
		@Override
		public CrossCurrencyTermsBuilder prune() {
			if (principalExchanges!=null && !principalExchanges.prune().hasData()) principalExchanges = null;
			if (settlementProvision!=null && !settlementProvision.prune().hasData()) settlementProvision = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrincipalExchanges()!=null && getPrincipalExchanges().hasData()) return true;
			if (getSettlementProvision()!=null && getSettlementProvision().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("principalExchanges"), processor, PrincipalExchanges.class, principalExchanges);
			processRosetta(path.newSubPath("settlementProvision"), processor, SettlementProvision.class, settlementProvision);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CrossCurrencyTermsBuilder _that = (CrossCurrencyTermsBuilder) o;
		
			if (principalExchanges != null ? !principalExchanges.equals(_that.principalExchanges) : _that.principalExchanges != null) return false;
			if (settlementProvision != null ? !settlementProvision.equals(_that.settlementProvision) : _that.settlementProvision != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (principalExchanges != null ? principalExchanges.hashCode() : 0);
			_result = 31 * _result + (settlementProvision != null ? settlementProvision.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CrossCurrencyTermsBuilder {" +
				"principalExchanges=" + this.principalExchanges + ", " +
				"settlementProvision=" + this.settlementProvision +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("principalExchanges"), processor, PrincipalExchanges.class, principalExchanges);
		processRosetta(path.newSubPath("settlementProvision"), processor, SettlementProvision.class, settlementProvision);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CrossCurrencyTerms _that = (CrossCurrencyTerms) o;
	
		if (principalExchanges != null ? !principalExchanges.equals(_that.principalExchanges) : _that.principalExchanges != null) return false;
		if (settlementProvision != null ? !settlementProvision.equals(_that.settlementProvision) : _that.settlementProvision != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (principalExchanges != null ? principalExchanges.hashCode() : 0);
		_result = 31 * _result + (settlementProvision != null ? settlementProvision.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CrossCurrencyTerms {" +
			"principalExchanges=" + this.principalExchanges + ", " +
			"settlementProvision=" + this.settlementProvision +
		'}';
	}
}
