package org.isda.cdm;

import org.isda.cdm.InterestRateCurve;
import org.isda.cdm.metafields.FieldWithMetaCommodityReferencePriceEnum;
import org.isda.cdm.metafields.FieldWithMetaCommodityReferencePriceEnum.FieldWithMetaCommodityReferencePriceEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CurveMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Curve extends RosettaModelObject {
	private final FieldWithMetaCommodityReferencePriceEnum commodityCurve;
	private final InterestRateCurve interestRateCurve;
	private static CurveMeta metaData = new CurveMeta();
	
	Curve(CurveBuilder builder) {
		this.commodityCurve = ofNullable(builder.getCommodityCurve()).map(f->f.build()).orElse(null);
		this.interestRateCurve = ofNullable(builder.getInterestRateCurve()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="commodityCurve", source="Rosetta_Workbench")
	public final FieldWithMetaCommodityReferencePriceEnum getCommodityCurve() {
		return commodityCurve;
	}
	
	@RosettaSynonym(value="interestRateCurve", source="Rosetta_Workbench")
	public final InterestRateCurve getInterestRateCurve() {
		return interestRateCurve;
	}
	
	
	@Override
	public RosettaMetaData<? extends Curve> metaData() {
		return metaData;
	} 
	
	public CurveBuilder toBuilder() {
		CurveBuilder builder = new CurveBuilder();
		ofNullable(getCommodityCurve()).ifPresent(builder::setCommodityCurve);
		ofNullable(getInterestRateCurve()).ifPresent(builder::setInterestRateCurve);
		return builder;
	}

	public static CurveBuilder builder() {
		return new CurveBuilder();
	}

	public static  class CurveBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaCommodityReferencePriceEnumBuilder commodityCurve;
		protected InterestRateCurve.InterestRateCurveBuilder interestRateCurve;
	
		public CurveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Curve> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaCommodityReferencePriceEnumBuilder getCommodityCurve() {
			return commodityCurve;
		}
		
		public FieldWithMetaCommodityReferencePriceEnumBuilder getOrCreateCommodityCurve() {
			if (commodityCurve!=null) {
				return commodityCurve;
			}
			else return commodityCurve = new FieldWithMetaCommodityReferencePriceEnumBuilder();
		}
		
		
		public InterestRateCurve.InterestRateCurveBuilder getInterestRateCurve() {
			return interestRateCurve;
		}
		
		public InterestRateCurve.InterestRateCurveBuilder getOrCreateInterestRateCurve() {
			if (interestRateCurve!=null) {
				return interestRateCurve;
			}
			else return interestRateCurve = new InterestRateCurve.InterestRateCurveBuilder();
		}
		
	
		public CurveBuilder setCommodityCurve(FieldWithMetaCommodityReferencePriceEnum commodityCurve) {
			this.commodityCurve = commodityCurve.toBuilder();
			return this;
		}
		public CurveBuilder setCommodityCurveRef(CommodityReferencePriceEnum commodityCurve) {
			return setCommodityCurve(FieldWithMetaCommodityReferencePriceEnum.builder().setValue(commodityCurve).build());
		}
		public CurveBuilder setInterestRateCurve(InterestRateCurve interestRateCurve) {
			this.interestRateCurve = interestRateCurve.toBuilder();
			return this;
		}
		public CurveBuilder setInterestRateCurveBuilder(InterestRateCurve.InterestRateCurveBuilder interestRateCurve) {
			this.interestRateCurve = interestRateCurve;
			return this;
		}
		
		public Curve build() {
			return new Curve(this);
		}
	
		@Override
		public CurveBuilder prune() {
			if (commodityCurve!=null && !commodityCurve.prune().hasData()) commodityCurve = null;
			if (interestRateCurve!=null && !interestRateCurve.prune().hasData()) interestRateCurve = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCommodityCurve()!=null) return true;
			if (getInterestRateCurve()!=null && getInterestRateCurve().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("commodityCurve"), processor, FieldWithMetaCommodityReferencePriceEnum.class, commodityCurve);
			processRosetta(path.newSubPath("interestRateCurve"), processor, InterestRateCurve.class, interestRateCurve);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CurveBuilder _that = (CurveBuilder) o;
		
			if (commodityCurve != null ? !commodityCurve.equals(_that.commodityCurve) : _that.commodityCurve != null) return false;
			if (interestRateCurve != null ? !interestRateCurve.equals(_that.interestRateCurve) : _that.interestRateCurve != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (commodityCurve != null ? commodityCurve.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (interestRateCurve != null ? interestRateCurve.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CurveBuilder {" +
				"commodityCurve=" + this.commodityCurve + ", " +
				"interestRateCurve=" + this.interestRateCurve +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("commodityCurve"), processor, FieldWithMetaCommodityReferencePriceEnum.class, commodityCurve);
		processRosetta(path.newSubPath("interestRateCurve"), processor, InterestRateCurve.class, interestRateCurve);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Curve _that = (Curve) o;
	
		if (commodityCurve != null ? !commodityCurve.equals(_that.commodityCurve) : _that.commodityCurve != null) return false;
		if (interestRateCurve != null ? !interestRateCurve.equals(_that.interestRateCurve) : _that.interestRateCurve != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (commodityCurve != null ? commodityCurve.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (interestRateCurve != null ? interestRateCurve.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Curve {" +
			"commodityCurve=" + this.commodityCurve + ", " +
			"interestRateCurve=" + this.interestRateCurve +
		'}';
	}
}
