package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.Offset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustomisableOffsetMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify an offset either as a normalized [multiplier, period, dayType] or as a custom provision of type string.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustomisableOffset extends RosettaModelObject {
	private final String customProvision;
	private final Offset offset;
	private static CustomisableOffsetMeta metaData = new CustomisableOffsetMeta();
	
	CustomisableOffset(CustomisableOffsetBuilder builder) {
		this.customProvision = builder.getCustomProvision();
		this.offset = ofNullable(builder.getOffset()).map(f->f.build()).orElse(null);
	}
	
	public final String getCustomProvision() {
		return customProvision;
	}
	
	public final Offset getOffset() {
		return offset;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustomisableOffset> metaData() {
		return metaData;
	} 
	
	public CustomisableOffsetBuilder toBuilder() {
		CustomisableOffsetBuilder builder = new CustomisableOffsetBuilder();
		ofNullable(getCustomProvision()).ifPresent(builder::setCustomProvision);
		ofNullable(getOffset()).ifPresent(builder::setOffset);
		return builder;
	}

	public static CustomisableOffsetBuilder builder() {
		return new CustomisableOffsetBuilder();
	}

	public static  class CustomisableOffsetBuilder extends RosettaModelObjectBuilder {
	
		protected String customProvision;
		protected Offset.OffsetBuilder offset;
	
		public CustomisableOffsetBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustomisableOffset> metaData() {
			return metaData;
		} 
	
		
		public String getCustomProvision() {
			return customProvision;
		}
		
		
		public Offset.OffsetBuilder getOffset() {
			return offset;
		}
		
		public Offset.OffsetBuilder getOrCreateOffset() {
			if (offset!=null) {
				return offset;
			}
			else return offset = new Offset.OffsetBuilder();
		}
		
	
		public CustomisableOffsetBuilder setCustomProvision(String customProvision) {
			this.customProvision = customProvision;
			return this;
		}
		public CustomisableOffsetBuilder setOffset(Offset offset) {
			this.offset = offset.toBuilder();
			return this;
		}
		public CustomisableOffsetBuilder setOffsetBuilder(Offset.OffsetBuilder offset) {
			this.offset = offset;
			return this;
		}
		
		public CustomisableOffset build() {
			return new CustomisableOffset(this);
		}
	
		@Override
		public CustomisableOffsetBuilder prune() {
			if (offset!=null && !offset.prune().hasData()) offset = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomProvision()!=null) return true;
			if (getOffset()!=null && getOffset().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("customProvision"), String.class, customProvision, this);
			
			processRosetta(path.newSubPath("offset"), processor, Offset.class, offset);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustomisableOffsetBuilder _that = (CustomisableOffsetBuilder) o;
		
			if (customProvision != null ? !customProvision.equals(_that.customProvision) : _that.customProvision != null) return false;
			if (offset != null ? !offset.equals(_that.offset) : _that.offset != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customProvision != null ? customProvision.hashCode() : 0);
			_result = 31 * _result + (offset != null ? offset.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustomisableOffsetBuilder {" +
				"customProvision=" + this.customProvision + ", " +
				"offset=" + this.offset +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("customProvision"), String.class, customProvision, this);
		
		processRosetta(path.newSubPath("offset"), processor, Offset.class, offset);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustomisableOffset _that = (CustomisableOffset) o;
	
		if (customProvision != null ? !customProvision.equals(_that.customProvision) : _that.customProvision != null) return false;
		if (offset != null ? !offset.equals(_that.offset) : _that.offset != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customProvision != null ? customProvision.hashCode() : 0);
		_result = 31 * _result + (offset != null ? offset.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustomisableOffset {" +
			"customProvision=" + this.customProvision + ", " +
			"offset=" + this.offset +
		'}';
	}
}
