package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.CustomisedWorkflowMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * In its initial iteration, this class is meant to support the DTCC TIW workflow information.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class CustomisedWorkflow extends RosettaModelObject {
	private final String itemName;
	private final String itemValue;
	private static CustomisedWorkflowMeta metaData = new CustomisedWorkflowMeta();
	
	CustomisedWorkflow(CustomisedWorkflowBuilder builder) {
		this.itemName = builder.getItemName();
		this.itemValue = builder.getItemValue();
	}
	
	/**
	 * In this initial iteration, this corresponds to the DTCC TIW element name.
	 */
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	public final String getItemName() {
		return itemName;
	}
	
	/**
	 * In this initial iteration, this corresponds to the DTCC value.
	 */
	@RosettaSynonym(value="comment", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="comment", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="superId", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="superId", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="deskId", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="deskId", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="eTradeId", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="eTradeId", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="designatedParty", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="designatedParty", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="brokerName", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="brokerName", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="branchLocation", source="DTCC_11_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="branchLocation", source="DTCC_9_0", path="PartyWorkflowFields")
	@RosettaSynonym(value="midMarketPriceType", source="DTCC_11_0", path="PartyWorkflowFields.midMarketPrice")
	@RosettaSynonym(value="midMarketPriceType", source="DTCC_9_0", path="PartyWorkflowFields.midMarketPrice")
	@RosettaSynonym(value="amount", source="DTCC_11_0", path="PartyWorkflowFields.midMarketPrice")
	@RosettaSynonym(value="amount", source="DTCC_9_0", path="PartyWorkflowFields.midMarketPrice")
	@RosettaSynonym(value="ClOrdID", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="InptSrc", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="OrigTrdID", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="TS", source="CME_SubmissionIRS_1_0", path="TrdRegTS")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	public final String getItemValue() {
		return itemValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends CustomisedWorkflow> metaData() {
		return metaData;
	} 
	
	public CustomisedWorkflowBuilder toBuilder() {
		CustomisedWorkflowBuilder builder = new CustomisedWorkflowBuilder();
		ofNullable(getItemName()).ifPresent(builder::setItemName);
		ofNullable(getItemValue()).ifPresent(builder::setItemValue);
		return builder;
	}

	public static CustomisedWorkflowBuilder builder() {
		return new CustomisedWorkflowBuilder();
	}

	public static  class CustomisedWorkflowBuilder extends RosettaModelObjectBuilder {
	
		protected String itemName;
		protected String itemValue;
	
		public CustomisedWorkflowBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends CustomisedWorkflow> metaData() {
			return metaData;
		} 
	
		
		public String getItemName() {
			return itemName;
		}
		
		
		public String getItemValue() {
			return itemValue;
		}
		
	
		public CustomisedWorkflowBuilder setItemName(String itemName) {
			this.itemName = itemName;
			return this;
		}
		public CustomisedWorkflowBuilder setItemValue(String itemValue) {
			this.itemValue = itemValue;
			return this;
		}
		
		public CustomisedWorkflow build() {
			return new CustomisedWorkflow(this);
		}
	
		@Override
		public CustomisedWorkflowBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getItemName()!=null) return true;
			if (getItemValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("itemName"), String.class, itemName, this);
			processor.processBasic(path.newSubPath("itemValue"), String.class, itemValue, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			CustomisedWorkflowBuilder _that = (CustomisedWorkflowBuilder) o;
		
			if (itemName != null ? !itemName.equals(_that.itemName) : _that.itemName != null) return false;
			if (itemValue != null ? !itemValue.equals(_that.itemValue) : _that.itemValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (itemName != null ? itemName.hashCode() : 0);
			_result = 31 * _result + (itemValue != null ? itemValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CustomisedWorkflowBuilder {" +
				"itemName=" + this.itemName + ", " +
				"itemValue=" + this.itemValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("itemName"), String.class, itemName, this);
		processor.processBasic(path.newSubPath("itemValue"), String.class, itemValue, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		CustomisedWorkflow _that = (CustomisedWorkflow) o;
	
		if (itemName != null ? !itemName.equals(_that.itemName) : _that.itemName != null) return false;
		if (itemValue != null ? !itemValue.equals(_that.itemValue) : _that.itemValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (itemName != null ? itemName.hashCode() : 0);
		_result = 31 * _result + (itemValue != null ? itemValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "CustomisedWorkflow {" +
			"itemName=" + this.itemName + ", " +
			"itemValue=" + this.itemValue +
		'}';
	}
}
