package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.util.List;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DateListMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * List of dates.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DateList", source="FpML_5_10")
public class DateList extends RosettaModelObject {
	private final List<Date> date;
	private static DateListMeta metaData = new DateListMeta();
	
	DateList(DateListBuilder builder) {
		this.date = builder.getDate();
	}
	
	@RosettaSynonym(value="date", source="FpML_5_10")
	@RosettaSynonym(value="date", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="date", source="DTCC_11_0")
	@RosettaSynonym(value="date", source="DTCC_9_0")
	@RosettaSynonym(value="date", source="CME_ClearedConfirm_1_17")
	public final List<Date> getDate() {
		return date;
	}
	
	
	@Override
	public RosettaMetaData<? extends DateList> metaData() {
		return metaData;
	} 
	
	public DateListBuilder toBuilder() {
		DateListBuilder builder = new DateListBuilder();
		ofNullable(getDate()).ifPresent(date -> date.forEach(builder::addDate));
		return builder;
	}

	public static DateListBuilder builder() {
		return new DateListBuilder();
	}

	public static  class DateListBuilder extends RosettaModelObjectBuilder {
	
		protected List<Date> date;
	
		public DateListBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DateList> metaData() {
			return metaData;
		} 
	
		
		public List<Date> getDate() {
			return date;
		}
		
	
		public DateListBuilder addDate(Date date) {
			if(this.date == null){
				this.date = new ArrayList<>();
			}
			this.date.add(date);
			return this;
		}
		public DateListBuilder addDate(Date date, int _idx) {
			if(this.date == null){
				this.date = new ArrayList<>();
			}
			getIndex(this.date, _idx, () -> date);
			this.date.set(_idx, date);
			return this;
		}
		public DateListBuilder addDate(List<Date> dates) {
			if(this.date == null){
				this.date = new ArrayList<>();
			}
			for (Date toAdd : dates) {
				this.date.add(toAdd);
			}
			return this;
		}
		
		public DateListBuilder clearDate() {
			this.date = null;
			return this;
		}
		
		public DateList build() {
			return new DateList(this);
		}
	
		@Override
		public DateListBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDate()!=null && !getDate().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DateListBuilder _that = (DateListBuilder) o;
		
			if (!ListEquals.listEquals(date, _that.date)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DateListBuilder {" +
				"date=" + this.date +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		date.stream().forEach(a->processor.processBasic(path.newSubPath("date"), Date.class, a, this));
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DateList _that = (DateList) o;
	
		if (!ListEquals.listEquals(date, _that.date)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DateList {" +
			"date=" + this.date +
		'}';
	}
}
