package org.isda.cdm;

import com.rosetta.model.lib.records.Date;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DateRangeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a contiguous series of calendar dates. The date range is defined as all the dates between and including the first and the last date. The first date must fall before the last date.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DateRange", source="FpML_5_10")
public class DateRange extends RosettaModelObject {
	private final Date unadjustedFirstDate;
	private final Date unadjustedLastDate;
	private static DateRangeMeta metaData = new DateRangeMeta();
	
	DateRange(DateRangeBuilder builder) {
		this.unadjustedFirstDate = builder.getUnadjustedFirstDate();
		this.unadjustedLastDate = builder.getUnadjustedLastDate();
	}
	
	/**
	 * The first date of a date range.
	 */
	@RosettaSynonym(value="unadjustedFirstDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedFirstDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedFirstDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedFirstDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedFirstDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedFirstDate() {
		return unadjustedFirstDate;
	}
	
	/**
	 * The last date of a date range.
	 */
	@RosettaSynonym(value="unadjustedLastDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedLastDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedLastDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedLastDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedLastDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedLastDate() {
		return unadjustedLastDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends DateRange> metaData() {
		return metaData;
	} 
	
	public DateRangeBuilder toBuilder() {
		DateRangeBuilder builder = new DateRangeBuilder();
		ofNullable(getUnadjustedFirstDate()).ifPresent(builder::setUnadjustedFirstDate);
		ofNullable(getUnadjustedLastDate()).ifPresent(builder::setUnadjustedLastDate);
		return builder;
	}

	public static DateRangeBuilder builder() {
		return new DateRangeBuilder();
	}

	public static  class DateRangeBuilder extends RosettaModelObjectBuilder {
	
		protected Date unadjustedFirstDate;
		protected Date unadjustedLastDate;
	
		public DateRangeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DateRange> metaData() {
			return metaData;
		} 
	
		
		public Date getUnadjustedFirstDate() {
			return unadjustedFirstDate;
		}
		
		
		public Date getUnadjustedLastDate() {
			return unadjustedLastDate;
		}
		
	
		public DateRangeBuilder setUnadjustedFirstDate(Date unadjustedFirstDate) {
			this.unadjustedFirstDate = unadjustedFirstDate;
			return this;
		}
		public DateRangeBuilder setUnadjustedLastDate(Date unadjustedLastDate) {
			this.unadjustedLastDate = unadjustedLastDate;
			return this;
		}
		
		public DateRange build() {
			return new DateRange(this);
		}
	
		@Override
		public DateRangeBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUnadjustedFirstDate()!=null) return true;
			if (getUnadjustedLastDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unadjustedFirstDate"), Date.class, unadjustedFirstDate, this);
			processor.processBasic(path.newSubPath("unadjustedLastDate"), Date.class, unadjustedLastDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DateRangeBuilder _that = (DateRangeBuilder) o;
		
			if (unadjustedFirstDate != null ? !unadjustedFirstDate.equals(_that.unadjustedFirstDate) : _that.unadjustedFirstDate != null) return false;
			if (unadjustedLastDate != null ? !unadjustedLastDate.equals(_that.unadjustedLastDate) : _that.unadjustedLastDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (unadjustedFirstDate != null ? unadjustedFirstDate.hashCode() : 0);
			_result = 31 * _result + (unadjustedLastDate != null ? unadjustedLastDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DateRangeBuilder {" +
				"unadjustedFirstDate=" + this.unadjustedFirstDate + ", " +
				"unadjustedLastDate=" + this.unadjustedLastDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unadjustedFirstDate"), Date.class, unadjustedFirstDate, this);
		processor.processBasic(path.newSubPath("unadjustedLastDate"), Date.class, unadjustedLastDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DateRange _that = (DateRange) o;
	
		if (unadjustedFirstDate != null ? !unadjustedFirstDate.equals(_that.unadjustedFirstDate) : _that.unadjustedFirstDate != null) return false;
		if (unadjustedLastDate != null ? !unadjustedLastDate.equals(_that.unadjustedLastDate) : _that.unadjustedLastDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (unadjustedFirstDate != null ? unadjustedFirstDate.hashCode() : 0);
		_result = 31 * _result + (unadjustedLastDate != null ? unadjustedLastDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DateRange {" +
			"unadjustedFirstDate=" + this.unadjustedFirstDate + ", " +
			"unadjustedLastDate=" + this.unadjustedLastDate +
		'}';
	}
}
