package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates;
import org.isda.cdm.metafields.ReferenceWithMetaCalculationPeriodDates.ReferenceWithMetaCalculationPeriodDatesBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DateRelativeToCalculationPeriodDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to provide the ability to point to multiple payment nodes in the document through the unbounded paymentDatesReference.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DateRelativeToCalculationPeriodDates", source="FpML_5_10")
public class DateRelativeToCalculationPeriodDates extends RosettaModelObject {
	private final List<ReferenceWithMetaCalculationPeriodDates> calculationPeriodDatesReference;
	private static DateRelativeToCalculationPeriodDatesMeta metaData = new DateRelativeToCalculationPeriodDatesMeta();
	
	DateRelativeToCalculationPeriodDates(DateRelativeToCalculationPeriodDatesBuilder builder) {
		this.calculationPeriodDatesReference = ofNullable(builder.getCalculationPeriodDatesReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A set of href pointers to calculation period dates defined somewhere else in the document.
	 */
	@RosettaSynonym(value="calculationPeriodDatesReference", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriodDatesReference", source="CME_ClearedConfirm_1_17")
	public final List<ReferenceWithMetaCalculationPeriodDates> getCalculationPeriodDatesReference() {
		return calculationPeriodDatesReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends DateRelativeToCalculationPeriodDates> metaData() {
		return metaData;
	} 
	
	public DateRelativeToCalculationPeriodDatesBuilder toBuilder() {
		DateRelativeToCalculationPeriodDatesBuilder builder = new DateRelativeToCalculationPeriodDatesBuilder();
		ofNullable(getCalculationPeriodDatesReference()).ifPresent(calculationPeriodDatesReference -> calculationPeriodDatesReference.forEach(builder::addCalculationPeriodDatesReference));
		return builder;
	}

	public static DateRelativeToCalculationPeriodDatesBuilder builder() {
		return new DateRelativeToCalculationPeriodDatesBuilder();
	}

	public static  class DateRelativeToCalculationPeriodDatesBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaCalculationPeriodDatesBuilder> calculationPeriodDatesReference;
	
		public DateRelativeToCalculationPeriodDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DateRelativeToCalculationPeriodDates> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaCalculationPeriodDatesBuilder> getCalculationPeriodDatesReference() {
			return calculationPeriodDatesReference;
		}
		
		public ReferenceWithMetaCalculationPeriodDatesBuilder getOrCreateCalculationPeriodDatesReference(int index) {
			if (calculationPeriodDatesReference==null) {
				this.calculationPeriodDatesReference = new ArrayList<>();
			}
			return getIndex(calculationPeriodDatesReference, index, ()->new ReferenceWithMetaCalculationPeriodDatesBuilder());
		}
		
	
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference) {
			if(this.calculationPeriodDatesReference == null){
				this.calculationPeriodDatesReference = new ArrayList<>();
			}
			this.calculationPeriodDatesReference.add(calculationPeriodDatesReference.toBuilder());
			return this;
		}
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates calculationPeriodDatesReference, int _idx) {
			if(this.calculationPeriodDatesReference == null){
				this.calculationPeriodDatesReference = new ArrayList<>();
			}
			getIndex(this.calculationPeriodDatesReference, _idx, () -> calculationPeriodDatesReference.toBuilder());
			this.calculationPeriodDatesReference.set(_idx, calculationPeriodDatesReference.toBuilder());
			return this;
		}
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReference(List<ReferenceWithMetaCalculationPeriodDates> calculationPeriodDatesReferences) {
			if(this.calculationPeriodDatesReference == null){
				this.calculationPeriodDatesReference = new ArrayList<>();
			}
			for (ReferenceWithMetaCalculationPeriodDates toAdd : calculationPeriodDatesReferences) {
				this.calculationPeriodDatesReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReferenceBuilder(ReferenceWithMetaCalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			if(this.calculationPeriodDatesReference == null){
				this.calculationPeriodDatesReference = new ArrayList<>();
			}
			this.calculationPeriodDatesReference.add(calculationPeriodDatesReference);
			return this;
		}
		
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReferenceRef(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDatesReference) {
			return addCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates.builder().setValueBuilder(calculationPeriodDatesReference).build());
		}
		
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReferenceRef(CalculationPeriodDates.CalculationPeriodDatesBuilder calculationPeriodDatesReference, int _idx) {
			return addCalculationPeriodDatesReference(ReferenceWithMetaCalculationPeriodDates.builder().setValueBuilder(calculationPeriodDatesReference).build(), _idx);
		}
		
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReferenceRef(List<CalculationPeriodDates> calculationPeriodDatesReferences) {
			for (CalculationPeriodDates toAdd : calculationPeriodDatesReferences) {
				addCalculationPeriodDatesReferenceRef(toAdd);
			}
			return this;
		}
		
		public DateRelativeToCalculationPeriodDatesBuilder addCalculationPeriodDatesReferenceRef(CalculationPeriodDates calculationPeriodDatesReference) {
			return addCalculationPeriodDatesReferenceRef(calculationPeriodDatesReference.toBuilder());
		}
		
		public DateRelativeToCalculationPeriodDatesBuilder clearCalculationPeriodDatesReference() {
			this.calculationPeriodDatesReference = null;
			return this;
		}
		
		public DateRelativeToCalculationPeriodDates build() {
			return new DateRelativeToCalculationPeriodDates(this);
		}
	
		@Override
		public DateRelativeToCalculationPeriodDatesBuilder prune() {
			if (calculationPeriodDatesReference!=null) calculationPeriodDatesReference = calculationPeriodDatesReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationPeriodDatesReference()!=null && getCalculationPeriodDatesReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DateRelativeToCalculationPeriodDatesBuilder _that = (DateRelativeToCalculationPeriodDatesBuilder) o;
		
			if (!ListEquals.listEquals(calculationPeriodDatesReference, _that.calculationPeriodDatesReference)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DateRelativeToCalculationPeriodDatesBuilder {" +
				"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("calculationPeriodDatesReference"), processor, ReferenceWithMetaCalculationPeriodDates.class, calculationPeriodDatesReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DateRelativeToCalculationPeriodDates _that = (DateRelativeToCalculationPeriodDates) o;
	
		if (!ListEquals.listEquals(calculationPeriodDatesReference, _that.calculationPeriodDatesReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationPeriodDatesReference != null ? calculationPeriodDatesReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DateRelativeToCalculationPeriodDates {" +
			"calculationPeriodDatesReference=" + this.calculationPeriodDatesReference +
		'}';
	}
}
