package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaPaymentDates;
import org.isda.cdm.metafields.ReferenceWithMetaPaymentDates.ReferenceWithMetaPaymentDatesBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DateRelativeToPaymentDatesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to provide the ability to point to multiple payment nodes in the document through the unbounded paymentDatesReference.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DateRelativeToPaymentDates", source="FpML_5_10")
public class DateRelativeToPaymentDates extends RosettaModelObject {
	private final List<ReferenceWithMetaPaymentDates> paymentDatesReference;
	private static DateRelativeToPaymentDatesMeta metaData = new DateRelativeToPaymentDatesMeta();
	
	DateRelativeToPaymentDates(DateRelativeToPaymentDatesBuilder builder) {
		this.paymentDatesReference = ofNullable(builder.getPaymentDatesReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A set of href pointers to payment dates defined somewhere else in the document.
	 */
	@RosettaSynonym(value="paymentDatesReference", source="FpML_5_10")
	@RosettaSynonym(value="paymentDatesReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDatesReference", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDatesReference", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDatesReference", source="CME_ClearedConfirm_1_17")
	public final List<ReferenceWithMetaPaymentDates> getPaymentDatesReference() {
		return paymentDatesReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends DateRelativeToPaymentDates> metaData() {
		return metaData;
	} 
	
	public DateRelativeToPaymentDatesBuilder toBuilder() {
		DateRelativeToPaymentDatesBuilder builder = new DateRelativeToPaymentDatesBuilder();
		ofNullable(getPaymentDatesReference()).ifPresent(paymentDatesReference -> paymentDatesReference.forEach(builder::addPaymentDatesReference));
		return builder;
	}

	public static DateRelativeToPaymentDatesBuilder builder() {
		return new DateRelativeToPaymentDatesBuilder();
	}

	public static  class DateRelativeToPaymentDatesBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaPaymentDatesBuilder> paymentDatesReference;
	
		public DateRelativeToPaymentDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DateRelativeToPaymentDates> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaPaymentDatesBuilder> getPaymentDatesReference() {
			return paymentDatesReference;
		}
		
		public ReferenceWithMetaPaymentDatesBuilder getOrCreatePaymentDatesReference(int index) {
			if (paymentDatesReference==null) {
				this.paymentDatesReference = new ArrayList<>();
			}
			return getIndex(paymentDatesReference, index, ()->new ReferenceWithMetaPaymentDatesBuilder());
		}
		
	
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReference(ReferenceWithMetaPaymentDates paymentDatesReference) {
			if(this.paymentDatesReference == null){
				this.paymentDatesReference = new ArrayList<>();
			}
			this.paymentDatesReference.add(paymentDatesReference.toBuilder());
			return this;
		}
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReference(ReferenceWithMetaPaymentDates paymentDatesReference, int _idx) {
			if(this.paymentDatesReference == null){
				this.paymentDatesReference = new ArrayList<>();
			}
			getIndex(this.paymentDatesReference, _idx, () -> paymentDatesReference.toBuilder());
			this.paymentDatesReference.set(_idx, paymentDatesReference.toBuilder());
			return this;
		}
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReference(List<ReferenceWithMetaPaymentDates> paymentDatesReferences) {
			if(this.paymentDatesReference == null){
				this.paymentDatesReference = new ArrayList<>();
			}
			for (ReferenceWithMetaPaymentDates toAdd : paymentDatesReferences) {
				this.paymentDatesReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReferenceBuilder(ReferenceWithMetaPaymentDatesBuilder paymentDatesReference) {
			if(this.paymentDatesReference == null){
				this.paymentDatesReference = new ArrayList<>();
			}
			this.paymentDatesReference.add(paymentDatesReference);
			return this;
		}
		
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReferenceRef(PaymentDates.PaymentDatesBuilder paymentDatesReference) {
			return addPaymentDatesReference(ReferenceWithMetaPaymentDates.builder().setValueBuilder(paymentDatesReference).build());
		}
		
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReferenceRef(PaymentDates.PaymentDatesBuilder paymentDatesReference, int _idx) {
			return addPaymentDatesReference(ReferenceWithMetaPaymentDates.builder().setValueBuilder(paymentDatesReference).build(), _idx);
		}
		
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReferenceRef(List<PaymentDates> paymentDatesReferences) {
			for (PaymentDates toAdd : paymentDatesReferences) {
				addPaymentDatesReferenceRef(toAdd);
			}
			return this;
		}
		
		public DateRelativeToPaymentDatesBuilder addPaymentDatesReferenceRef(PaymentDates paymentDatesReference) {
			return addPaymentDatesReferenceRef(paymentDatesReference.toBuilder());
		}
		
		public DateRelativeToPaymentDatesBuilder clearPaymentDatesReference() {
			this.paymentDatesReference = null;
			return this;
		}
		
		public DateRelativeToPaymentDates build() {
			return new DateRelativeToPaymentDates(this);
		}
	
		@Override
		public DateRelativeToPaymentDatesBuilder prune() {
			if (paymentDatesReference!=null) paymentDatesReference = paymentDatesReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPaymentDatesReference()!=null && getPaymentDatesReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("paymentDatesReference"), processor, ReferenceWithMetaPaymentDates.class, paymentDatesReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DateRelativeToPaymentDatesBuilder _that = (DateRelativeToPaymentDatesBuilder) o;
		
			if (!ListEquals.listEquals(paymentDatesReference, _that.paymentDatesReference)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (paymentDatesReference != null ? paymentDatesReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DateRelativeToPaymentDatesBuilder {" +
				"paymentDatesReference=" + this.paymentDatesReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("paymentDatesReference"), processor, ReferenceWithMetaPaymentDates.class, paymentDatesReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DateRelativeToPaymentDates _that = (DateRelativeToPaymentDates) o;
	
		if (!ListEquals.listEquals(paymentDatesReference, _that.paymentDatesReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (paymentDatesReference != null ? paymentDatesReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DateRelativeToPaymentDates {" +
			"paymentDatesReference=" + this.paymentDatesReference +
		'}';
	}
}
