package org.isda.cdm;

import java.time.ZonedDateTime;
import java.util.List;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DateTimeListMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.ZonedDateTime;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * List of dateTimes.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DateTimeList", source="FpML_5_10")
public class DateTimeList extends RosettaModelObject {
	private final List<ZonedDateTime> dateTime;
	private static DateTimeListMeta metaData = new DateTimeListMeta();
	
	DateTimeList(DateTimeListBuilder builder) {
		this.dateTime = builder.getDateTime();
	}
	
	/**
	 * The CDM specifies that the zoned date time is to be expressed in accordance with ISO 8601, either as UTC as an offset to UTC.
	 */
	@RosettaSynonym(value="dateTime", source="FpML_5_10")
	@RosettaSynonym(value="dateTime", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateTime", source="DTCC_11_0")
	@RosettaSynonym(value="dateTime", source="DTCC_9_0")
	@RosettaSynonym(value="dateTime", source="CME_ClearedConfirm_1_17")
	public final List<ZonedDateTime> getDateTime() {
		return dateTime;
	}
	
	
	@Override
	public RosettaMetaData<? extends DateTimeList> metaData() {
		return metaData;
	} 
	
	public DateTimeListBuilder toBuilder() {
		DateTimeListBuilder builder = new DateTimeListBuilder();
		ofNullable(getDateTime()).ifPresent(dateTime -> dateTime.forEach(builder::addDateTime));
		return builder;
	}

	public static DateTimeListBuilder builder() {
		return new DateTimeListBuilder();
	}

	public static  class DateTimeListBuilder extends RosettaModelObjectBuilder {
	
		protected List<ZonedDateTime> dateTime;
	
		public DateTimeListBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DateTimeList> metaData() {
			return metaData;
		} 
	
		
		public List<ZonedDateTime> getDateTime() {
			return dateTime;
		}
		
	
		public DateTimeListBuilder addDateTime(ZonedDateTime dateTime) {
			if(this.dateTime == null){
				this.dateTime = new ArrayList<>();
			}
			this.dateTime.add(dateTime);
			return this;
		}
		public DateTimeListBuilder addDateTime(ZonedDateTime dateTime, int _idx) {
			if(this.dateTime == null){
				this.dateTime = new ArrayList<>();
			}
			getIndex(this.dateTime, _idx, () -> dateTime);
			this.dateTime.set(_idx, dateTime);
			return this;
		}
		public DateTimeListBuilder addDateTime(List<ZonedDateTime> dateTimes) {
			if(this.dateTime == null){
				this.dateTime = new ArrayList<>();
			}
			for (ZonedDateTime toAdd : dateTimes) {
				this.dateTime.add(toAdd);
			}
			return this;
		}
		
		public DateTimeListBuilder clearDateTime() {
			this.dateTime = null;
			return this;
		}
		
		public DateTimeList build() {
			return new DateTimeList(this);
		}
	
		@Override
		public DateTimeListBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateTime()!=null && !getDateTime().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, dateTime, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DateTimeListBuilder _that = (DateTimeListBuilder) o;
		
			if (!ListEquals.listEquals(dateTime, _that.dateTime)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DateTimeListBuilder {" +
				"dateTime=" + this.dateTime +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		dateTime.stream().forEach(a->processor.processBasic(path.newSubPath("dateTime"), ZonedDateTime.class, a, this));
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DateTimeList _that = (DateTimeList) o;
	
		if (!ListEquals.listEquals(dateTime, _that.dateTime)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateTime != null ? dateTime.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DateTimeList {" +
			"dateTime=" + this.dateTime +
		'}';
	}
}
