package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the day type classification used in counting the number of days between two dates.
 * 
 * @version 2.5.4
 */
public enum DayTypeEnum {
    

	/**
	 * When calculating the number of days between two dates the count includes only business days.
	 */
	@RosettaSynonym(value = "Business", source = "FpML_5_10")
	@RosettaSynonym(value = "Business", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Business", source = "DTCC_11_0")
	@RosettaSynonym(value = "Business", source = "DTCC_9_0")
	@RosettaSynonym(value = "Business", source = "CME_ClearedConfirm_1_17")
	BUSINESS,
	    

	/**
	 * When calculating the number of days between two dates the count includes all calendar days.
	 */
	@RosettaSynonym(value = "Calendar", source = "FpML_5_10")
	@RosettaSynonym(value = "Calendar", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Calendar", source = "DTCC_11_0")
	@RosettaSynonym(value = "Calendar", source = "DTCC_9_0")
	@RosettaSynonym(value = "Calendar", source = "CME_ClearedConfirm_1_17")
	CALENDAR,
	    

	/**
	 * When calculating the number of days between two dates the count includes only commodity business days.
	 */
	@RosettaSynonym(value = "CommodityBusiness", source = "FpML_5_10")
	@RosettaSynonym(value = "CommodityBusiness", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CommodityBusiness", source = "DTCC_11_0")
	@RosettaSynonym(value = "CommodityBusiness", source = "DTCC_9_0")
	@RosettaSynonym(value = "CommodityBusiness", source = "CME_ClearedConfirm_1_17")
	COMMODITY_BUSINESS,
	    

	/**
	 * When calculating the number of days between two dates the count includes only currency business days.
	 */
	@RosettaSynonym(value = "CurrencyBusiness", source = "FpML_5_10")
	@RosettaSynonym(value = "CurrencyBusiness", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CurrencyBusiness", source = "DTCC_11_0")
	@RosettaSynonym(value = "CurrencyBusiness", source = "DTCC_9_0")
	@RosettaSynonym(value = "CurrencyBusiness", source = "CME_ClearedConfirm_1_17")
	CURRENCY_BUSINESS,
	    

	/**
	 * When calculating the number of days between two dates the count includes only stock exchange business days.
	 */
	@RosettaSynonym(value = "ExchangeBusiness", source = "FpML_5_10")
	@RosettaSynonym(value = "ExchangeBusiness", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExchangeBusiness", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExchangeBusiness", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExchangeBusiness", source = "CME_ClearedConfirm_1_17")
	EXCHANGE_BUSINESS,
	    

	/**
	 * When calculating the number of days between two dates the count includes only scheduled trading days.
	 */
	@RosettaSynonym(value = "ScheduledTradingDay", source = "FpML_5_10")
	@RosettaSynonym(value = "ScheduledTradingDay", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ScheduledTradingDay", source = "DTCC_11_0")
	@RosettaSynonym(value = "ScheduledTradingDay", source = "DTCC_9_0")
	@RosettaSynonym(value = "ScheduledTradingDay", source = "CME_ClearedConfirm_1_17")
	SCHEDULED_TRADING_DAY    
	;

	private final String displayName;
	
	DayTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
