package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.UndrlygInstrm;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DerivInstrmAttrbtsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class DerivInstrmAttrbts extends RosettaModelObject {
	private final String dlvryTp;
	private final String pricMltplr;
	private final UndrlygInstrm undrlygInstrm;
	private final String xpryDt;
	private static DerivInstrmAttrbtsMeta metaData = new DerivInstrmAttrbtsMeta();
	
	DerivInstrmAttrbts(DerivInstrmAttrbtsBuilder builder) {
		this.dlvryTp = builder.getDlvryTp();
		this.pricMltplr = builder.getPricMltplr();
		this.undrlygInstrm = ofNullable(builder.getUndrlygInstrm()).map(f->f.build()).orElse(null);
		this.xpryDt = builder.getXpryDt();
	}
	
	@RosettaSynonym(value="DlvryTp", source="ISO20022")
	public final String getDlvryTp() {
		return dlvryTp;
	}
	
	@RosettaSynonym(value="PricMltplr", source="ISO20022")
	public final String getPricMltplr() {
		return pricMltplr;
	}
	
	@RosettaSynonym(value="UndrlygInstrm", source="ISO20022")
	public final UndrlygInstrm getUndrlygInstrm() {
		return undrlygInstrm;
	}
	
	@RosettaSynonym(value="XpryDt", source="ISO20022")
	public final String getXpryDt() {
		return xpryDt;
	}
	
	
	@Override
	public RosettaMetaData<? extends DerivInstrmAttrbts> metaData() {
		return metaData;
	} 
	
	public DerivInstrmAttrbtsBuilder toBuilder() {
		DerivInstrmAttrbtsBuilder builder = new DerivInstrmAttrbtsBuilder();
		ofNullable(getDlvryTp()).ifPresent(builder::setDlvryTp);
		ofNullable(getPricMltplr()).ifPresent(builder::setPricMltplr);
		ofNullable(getUndrlygInstrm()).ifPresent(builder::setUndrlygInstrm);
		ofNullable(getXpryDt()).ifPresent(builder::setXpryDt);
		return builder;
	}

	public static DerivInstrmAttrbtsBuilder builder() {
		return new DerivInstrmAttrbtsBuilder();
	}

	public static  class DerivInstrmAttrbtsBuilder extends RosettaModelObjectBuilder {
	
		protected String dlvryTp;
		protected String pricMltplr;
		protected UndrlygInstrm.UndrlygInstrmBuilder undrlygInstrm;
		protected String xpryDt;
	
		public DerivInstrmAttrbtsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DerivInstrmAttrbts> metaData() {
			return metaData;
		} 
	
		
		public String getDlvryTp() {
			return dlvryTp;
		}
		
		
		public String getPricMltplr() {
			return pricMltplr;
		}
		
		
		public UndrlygInstrm.UndrlygInstrmBuilder getUndrlygInstrm() {
			return undrlygInstrm;
		}
		
		public UndrlygInstrm.UndrlygInstrmBuilder getOrCreateUndrlygInstrm() {
			if (undrlygInstrm!=null) {
				return undrlygInstrm;
			}
			else return undrlygInstrm = new UndrlygInstrm.UndrlygInstrmBuilder();
		}
		
		
		public String getXpryDt() {
			return xpryDt;
		}
		
	
		public DerivInstrmAttrbtsBuilder setDlvryTp(String dlvryTp) {
			this.dlvryTp = dlvryTp;
			return this;
		}
		public DerivInstrmAttrbtsBuilder setPricMltplr(String pricMltplr) {
			this.pricMltplr = pricMltplr;
			return this;
		}
		public DerivInstrmAttrbtsBuilder setUndrlygInstrm(UndrlygInstrm undrlygInstrm) {
			this.undrlygInstrm = undrlygInstrm.toBuilder();
			return this;
		}
		public DerivInstrmAttrbtsBuilder setUndrlygInstrmBuilder(UndrlygInstrm.UndrlygInstrmBuilder undrlygInstrm) {
			this.undrlygInstrm = undrlygInstrm;
			return this;
		}
		public DerivInstrmAttrbtsBuilder setXpryDt(String xpryDt) {
			this.xpryDt = xpryDt;
			return this;
		}
		
		public DerivInstrmAttrbts build() {
			return new DerivInstrmAttrbts(this);
		}
	
		@Override
		public DerivInstrmAttrbtsBuilder prune() {
			if (undrlygInstrm!=null && !undrlygInstrm.prune().hasData()) undrlygInstrm = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDlvryTp()!=null) return true;
			if (getPricMltplr()!=null) return true;
			if (getUndrlygInstrm()!=null && getUndrlygInstrm().hasData()) return true;
			if (getXpryDt()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dlvryTp"), String.class, dlvryTp, this);
			processor.processBasic(path.newSubPath("pricMltplr"), String.class, pricMltplr, this);
			processor.processBasic(path.newSubPath("xpryDt"), String.class, xpryDt, this);
			
			processRosetta(path.newSubPath("undrlygInstrm"), processor, UndrlygInstrm.class, undrlygInstrm);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DerivInstrmAttrbtsBuilder _that = (DerivInstrmAttrbtsBuilder) o;
		
			if (dlvryTp != null ? !dlvryTp.equals(_that.dlvryTp) : _that.dlvryTp != null) return false;
			if (pricMltplr != null ? !pricMltplr.equals(_that.pricMltplr) : _that.pricMltplr != null) return false;
			if (undrlygInstrm != null ? !undrlygInstrm.equals(_that.undrlygInstrm) : _that.undrlygInstrm != null) return false;
			if (xpryDt != null ? !xpryDt.equals(_that.xpryDt) : _that.xpryDt != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dlvryTp != null ? dlvryTp.hashCode() : 0);
			_result = 31 * _result + (pricMltplr != null ? pricMltplr.hashCode() : 0);
			_result = 31 * _result + (undrlygInstrm != null ? undrlygInstrm.hashCode() : 0);
			_result = 31 * _result + (xpryDt != null ? xpryDt.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DerivInstrmAttrbtsBuilder {" +
				"dlvryTp=" + this.dlvryTp + ", " +
				"pricMltplr=" + this.pricMltplr + ", " +
				"undrlygInstrm=" + this.undrlygInstrm + ", " +
				"xpryDt=" + this.xpryDt +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dlvryTp"), String.class, dlvryTp, this);
		processor.processBasic(path.newSubPath("pricMltplr"), String.class, pricMltplr, this);
		processor.processBasic(path.newSubPath("xpryDt"), String.class, xpryDt, this);
		
		processRosetta(path.newSubPath("undrlygInstrm"), processor, UndrlygInstrm.class, undrlygInstrm);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DerivInstrmAttrbts _that = (DerivInstrmAttrbts) o;
	
		if (dlvryTp != null ? !dlvryTp.equals(_that.dlvryTp) : _that.dlvryTp != null) return false;
		if (pricMltplr != null ? !pricMltplr.equals(_that.pricMltplr) : _that.pricMltplr != null) return false;
		if (undrlygInstrm != null ? !undrlygInstrm.equals(_that.undrlygInstrm) : _that.undrlygInstrm != null) return false;
		if (xpryDt != null ? !xpryDt.equals(_that.xpryDt) : _that.xpryDt != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dlvryTp != null ? dlvryTp.hashCode() : 0);
		_result = 31 * _result + (pricMltplr != null ? pricMltplr.hashCode() : 0);
		_result = 31 * _result + (undrlygInstrm != null ? undrlygInstrm.hashCode() : 0);
		_result = 31 * _result + (xpryDt != null ? xpryDt.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DerivInstrmAttrbts {" +
			"dlvryTp=" + this.dlvryTp + ", " +
			"pricMltplr=" + this.pricMltplr + ", " +
			"undrlygInstrm=" + this.undrlygInstrm + ", " +
			"xpryDt=" + this.xpryDt +
		'}';
	}
}
