package org.isda.cdm;

import org.isda.cdm.DeterminationMethodEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DeterminationMethodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The purpose of this class is to be extended by the Price and EquityValuation classes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class DeterminationMethod extends RosettaModelObject {
	private final DeterminationMethodEnum determinationMethod;
	private static DeterminationMethodMeta metaData = new DeterminationMethodMeta();
	
	DeterminationMethod(DeterminationMethodBuilder builder) {
		this.determinationMethod = builder.getDeterminationMethod();
	}
	
	/**
	 * Specifies the method according to which an amount or a date is determined.
	 */
	@RosettaSynonym(value="determinationMethod", source="FpML_5_10")
	public final DeterminationMethodEnum getDeterminationMethod() {
		return determinationMethod;
	}
	
	
	@Override
	public RosettaMetaData<? extends DeterminationMethod> metaData() {
		return metaData;
	} 
	
	public DeterminationMethodBuilder toBuilder() {
		DeterminationMethodBuilder builder = new DeterminationMethodBuilder();
		ofNullable(getDeterminationMethod()).ifPresent(builder::setDeterminationMethod);
		return builder;
	}

	public static DeterminationMethodBuilder builder() {
		return new DeterminationMethodBuilder();
	}

	public static  class DeterminationMethodBuilder extends RosettaModelObjectBuilder {
	
		protected DeterminationMethodEnum determinationMethod;
	
		public DeterminationMethodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DeterminationMethod> metaData() {
			return metaData;
		} 
	
		
		public DeterminationMethodEnum getDeterminationMethod() {
			return determinationMethod;
		}
		
	
		public DeterminationMethodBuilder setDeterminationMethod(DeterminationMethodEnum determinationMethod) {
			this.determinationMethod = determinationMethod;
			return this;
		}
		
		public DeterminationMethod build() {
			return new DeterminationMethod(this);
		}
	
		@Override
		public DeterminationMethodBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDeterminationMethod()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DeterminationMethodBuilder _that = (DeterminationMethodBuilder) o;
		
			if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DeterminationMethodBuilder {" +
				"determinationMethod=" + this.determinationMethod +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("determinationMethod"), DeterminationMethodEnum.class, determinationMethod, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DeterminationMethod _that = (DeterminationMethod) o;
	
		if (determinationMethod != null ? !determinationMethod.equals(_that.determinationMethod) : _that.determinationMethod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (determinationMethod != null ? determinationMethod.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DeterminationMethod {" +
			"determinationMethod=" + this.determinationMethod +
		'}';
	}
}
