package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the method according to which an amount or a date is determined.
 * 
 * @version 2.5.4
 */
public enum DeterminationMethodEnum {
    

	/**
	 * Agreed separately between the parties.
	 */
	@RosettaSynonym(value = "AgreedInitialPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "AgreedInitialPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AgreedInitialPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "AgreedInitialPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "AgreedInitialPrice", source = "CME_ClearedConfirm_1_17")
	AGREED_INITIAL_PRICE,
	    

	/**
	 * As specified in Master Confirmation.
	 */
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "FpML_5_10")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "CME_ClearedConfirm_1_17")
	AS_SPECIFIED_IN_MASTER_CONFIRMATION,
	    

	/**
	 * Determined by the Calculation Agent.
	 */
	@RosettaSynonym(value = "CalculationAgent", source = "FpML_5_10")
	@RosettaSynonym(value = "CalculationAgent", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalculationAgent", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalculationAgent", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalculationAgent", source = "CME_ClearedConfirm_1_17")
	CALCULATION_AGENT,
	    

	/**
	 * Official Closing Price.
	 */
	@RosettaSynonym(value = "ClosingPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "ClosingPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ClosingPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "ClosingPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "ClosingPrice", source = "CME_ClearedConfirm_1_17")
	CLOSING_PRICE,
	    

	/**
	 * Determined by the Currency of Equity Dividends.
	 */
	@RosettaSynonym(value = "DividendCurrency", source = "FpML_5_10")
	@RosettaSynonym(value = "DividendCurrency", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DividendCurrency", source = "DTCC_11_0")
	@RosettaSynonym(value = "DividendCurrency", source = "DTCC_9_0")
	@RosettaSynonym(value = "DividendCurrency", source = "CME_ClearedConfirm_1_17")
	DIVIDEND_CURRENCY,
	    

	/**
	 * The initial Index Level is the level of the Expiring Contract as provided in the Master Confirmation.
	 */
	@RosettaSynonym(value = "ExpiringContractLevel", source = "FpML_5_10")
	@RosettaSynonym(value = "ExpiringContractLevel", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExpiringContractLevel", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExpiringContractLevel", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExpiringContractLevel", source = "CME_ClearedConfirm_1_17")
	EXPIRING_CONTRACT_LEVEL,
	    

	/**
	 * Determined by the Hedging Party.
	 */
	@RosettaSynonym(value = "HedgeExecution", source = "FpML_5_10")
	@RosettaSynonym(value = "HedgeExecution", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HedgeExecution", source = "DTCC_11_0")
	@RosettaSynonym(value = "HedgeExecution", source = "DTCC_9_0")
	@RosettaSynonym(value = "HedgeExecution", source = "CME_ClearedConfirm_1_17")
	HEDGE_EXECUTION,
	    

	/**
	 * Issuer Payment Currency.
	 */
	@RosettaSynonym(value = "IssuerPaymentCurrency", source = "FpML_5_10")
	@RosettaSynonym(value = "IssuerPaymentCurrency", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IssuerPaymentCurrency", source = "DTCC_11_0")
	@RosettaSynonym(value = "IssuerPaymentCurrency", source = "DTCC_9_0")
	@RosettaSynonym(value = "IssuerPaymentCurrency", source = "CME_ClearedConfirm_1_17")
	ISSUER_PAYMENT_CURRENCY,
	    

	/**
	 * Net Asset Value.
	 */
	@RosettaSynonym(value = "NAV", source = "FpML_5_10")
	@RosettaSynonym(value = "NAV", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NAV", source = "DTCC_11_0")
	@RosettaSynonym(value = "NAV", source = "DTCC_9_0")
	@RosettaSynonym(value = "NAV", source = "CME_ClearedConfirm_1_17")
	NAV,
	    

	/**
	 * Opening Price of the Market.
	 */
	@RosettaSynonym(value = "OpenPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "OpenPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OpenPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "OpenPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "OpenPrice", source = "CME_ClearedConfirm_1_17")
	OPEN_PRICE,
	    

	/**
	 * Official Settlement Price.
	 */
	@RosettaSynonym(value = "OSPPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "OSPPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OSPPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "OSPPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "OSPPrice", source = "CME_ClearedConfirm_1_17")
	OSP_PRICE,
	    

	/**
	 * Settlement Currency.
	 */
	@RosettaSynonym(value = "SettlementCurrency", source = "FpML_5_10")
	@RosettaSynonym(value = "SettlementCurrency", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SettlementCurrency", source = "DTCC_11_0")
	@RosettaSynonym(value = "SettlementCurrency", source = "DTCC_9_0")
	@RosettaSynonym(value = "SettlementCurrency", source = "CME_ClearedConfirm_1_17")
	SETTLEMENT_CURRENCY,
	    

	/**
	 * Date on which the strike is determined in respect of a forward starting swap.
	 */
	@RosettaSynonym(value = "StrikeDateDetermination", source = "FpML_5_10")
	@RosettaSynonym(value = "StrikeDateDetermination", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StrikeDateDetermination", source = "DTCC_11_0")
	@RosettaSynonym(value = "StrikeDateDetermination", source = "DTCC_9_0")
	@RosettaSynonym(value = "StrikeDateDetermination", source = "CME_ClearedConfirm_1_17")
	STRIKE_DATE_DETERMINATION,
	    

	/**
	 * Official Trade-Weighted Average Price.
	 */
	@RosettaSynonym(value = "TWAPPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "TWAPPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWAPPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWAPPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWAPPrice", source = "CME_ClearedConfirm_1_17")
	TWAP_PRICE,
	    

	/**
	 * Price determined at valuation time.
	 */
	@RosettaSynonym(value = "ValuationTime", source = "FpML_5_10")
	@RosettaSynonym(value = "ValuationTime", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ValuationTime", source = "DTCC_11_0")
	@RosettaSynonym(value = "ValuationTime", source = "DTCC_9_0")
	@RosettaSynonym(value = "ValuationTime", source = "CME_ClearedConfirm_1_17")
	VALUATION_TIME,
	    

	/**
	 * Official Volume-Weighted Average Price.
	 */
	@RosettaSynonym(value = "VWAPPrice", source = "FpML_5_10")
	@RosettaSynonym(value = "VWAPPrice", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "VWAPPrice", source = "DTCC_11_0")
	@RosettaSynonym(value = "VWAPPrice", source = "DTCC_9_0")
	@RosettaSynonym(value = "VWAPPrice", source = "CME_ClearedConfirm_1_17")
	VWAP_PRICE    
	;

	private final String displayName;
	
	DeterminationMethodEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
