package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the method of calculating discounted payment amounts. This enumerations combines the FpML DiscountingTypeEnum and FraDiscountingEnum enumerations.
 * 
 * @version 2.5.4
 */
public enum DiscountingTypeEnum {
    

	/**
	 * As specified by the 2006 ISDA Definitions, Section 8.4. Discounting, paragraph (a).
	 */
	@RosettaSynonym(value = "Standard", source = "FpML_5_10")
	@RosettaSynonym(value = "Standard", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Standard", source = "DTCC_11_0")
	@RosettaSynonym(value = "Standard", source = "DTCC_9_0")
	@RosettaSynonym(value = "Standard", source = "CME_ClearedConfirm_1_17")
	STANDARD,
	    

	/**
	 * As specified by the 2006 ISDA Definitions, Section 8.4. Discounting, paragraph (b).
	 */
	@RosettaSynonym(value = "FRA", source = "FpML_5_10")
	@RosettaSynonym(value = "FRA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FRA", source = "DTCC_11_0")
	@RosettaSynonym(value = "FRA", source = "DTCC_9_0")
	@RosettaSynonym(value = "FRA", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA", source = "CME_ClearedConfirm_1_17")
	FRA,
	    

	/**
	 * As specified by the 2006 ISDA Definitions, Section 8.4. Discounting, paragraph (e).
	 */
	@RosettaSynonym(value = "FRAYield", source = "FpML_5_10")
	@RosettaSynonym(value = "FRAYield", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FRAYield", source = "DTCC_11_0")
	@RosettaSynonym(value = "FRAYield", source = "DTCC_9_0")
	@RosettaSynonym(value = "FRAYield", source = "CME_ClearedConfirm_1_17")
	FRA_YIELD,
	    

	/**
	 * As specified by the the Australian Financial Markets Association (AFMA) OTC Financial Product Conventions. This discounting method should not be used for a trade documented under a legal framework where the 2006 ISDA Definitions have been incorporated.
	 */
	@RosettaSynonym(value = "AFMA", source = "FpML_5_10")
	@RosettaSynonym(value = "AFMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AFMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "AFMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "AFMA", source = "CME_ClearedConfirm_1_17")
	AFMA    
	;

	private final String displayName;
	
	DiscountingTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
