package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether the dividend is paid with respect to the Dividend Period.
 * 
 * @version 2.5.4
 */
public enum DividendAmountTypeEnum {
    

	/**
	 * The record date for a dividend occurs during a dividend period.
	 */
	@RosettaSynonym(value = "RecordAmount", source = "FpML_5_10")
	RECORD_AMOUNT,
	    

	/**
	 * The ex-date for a dividend occurs during a dividend period.
	 */
	@RosettaSynonym(value = "ExAmount", source = "FpML_5_10")
	EX_AMOUNT,
	    

	/**
	 * The payment date for a dividend occurs during a dividend period.
	 */
	@RosettaSynonym(value = "PaidAmount", source = "FpML_5_10")
	PAID_AMOUNT,
	    

	/**
	 * The Amount is determined as provided in the relevant Master Confirmation.
	 */
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "FpML_5_10")
	AS_SPECIFIED_IN_MASTER_CONFIRMATION    
	;

	private final String displayName;
	
	DividendAmountTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
