package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the date by reference to which the dividend will be paid.
 * 
 * @version 2.5.4
 */
public enum DividendDateReferenceEnum {
    

	/**
	 * The dividend date will be specified ad-hoc by the parties, typically on the dividend ex-date.
	 */
	@RosettaSynonym(value = "AdHocDate", source = "FpML_5_10")
	AD_HOC_DATE,
	    

	/**
	 * If &#39;Dividend Payment Date(s)&#39; is specified in the Transaction Supplement as &#39;Cash Settlement Payment Date&#39;, then the Dividend Payment Date in respect of a Dividend Amount shall be the Cash Settlement Payment Date relating to the end of the Dividend Period during which the Shares commenced trading &#39;ex&#39; the relevant dividend on the Exchange.
	 */
	@RosettaSynonym(value = "CashSettlementPaymentDate", source = "FpML_5_10")
	CASH_SETTLEMENT_PAYMENT_DATE,
	    

	/**
	 * If &#39;Dividend Payment Date(s)&#39; is specified in the Transaction Supplement as &#39;Cash Settlement Payment Date – Ex Dividend&#39;&#39;, then the Dividend Payment Date in respect of a Dividend Amount shall be the Cash Settlement Payment Date relating to the end of the Dividend Period during which the Shares commenced trading &#39;ex&#39; the relevant dividend on the Exchange.
	 */
	@RosettaSynonym(value = "CashSettlePaymentDateExDiv", source = "FpML_5_10")
	CASH_SETTLE_PAYMENT_DATE_EX_DIV,
	    

	/**
	 * If &#39;Dividend Payment Date(s)&#39; is specified in the Transaction Supplement as &#39;Cash Settlement Payment Date – Issuer Payment&#39;, then the Dividend Payment Date in respect of a Dividend Amount shall be the Cash Settlement Payment Date relating to the end of the Dividend Period during which the issuer pays the relevant dividend to a holder of record provided that in the case where the Equity Amount Payer is the party specified to be the sole Hedging Party and the Hedging Party has not received the Dividend Amount by such date, then the date falling a number of Currency Business Days as specified in the Cash Settlement Payment Date after actual receipt by the Hedging Party of the Received Ex Amount or Paid Ex Amount (as applicable).
	 */
	@RosettaSynonym(value = "CashSettlePaymentDateIssuerPayment", source = "FpML_5_10")
	CASH_SETTLE_PAYMENT_DATE_ISSUER_PAYMENT,
	    

	/**
	 * Total of dividends which go ex, paid on next following Cash Settlement Payment Date, which is immediately following the Dividend Period during which the Shares commence trading ex-dividend on the Exchange.
	 */
	@RosettaSynonym(value = "CumulativeEquityExDiv", source = "FpML_5_10")
	CUMULATIVE_EQUITY_EX_DIV,
	    

	/**
	 * Total of paid dividends, paid on next following Cash Settlement Payment Date, which is immediately following the Dividend Period during which the dividend is paid by the Issuer to the holders of record of a Share.
	 */
	@RosettaSynonym(value = "CumulativeEquityPaid", source = "FpML_5_10")
	CUMULATIVE_EQUITY_PAID,
	    

	/**
	 * Total of dividends which go ex, paid on next following Payment Date, which is immediately following the Dividend Period during which the Shares commence trading ex-dividend on the Exchange, or where the date on which the Shares commence trading ex-dividend is a Payment Date, such Payment Date.
	 */
	@RosettaSynonym(value = "CumulativeLiborExDiv", source = "FpML_5_10")
	CUMULATIVE_LIBOR_EX_DIV,
	    

	/**
	 * Total of paid dividends, paid on next following Payment Date, which is immediately following the Dividend Period during which the dividend is paid by the Issuer to the holders of record of a Share.
	 */
	@RosettaSynonym(value = "CumulativeLiborPaid", source = "FpML_5_10")
	CUMULATIVE_LIBOR_PAID,
	    

	/**
	 * Date on which the dividend will be paid by the issuer.
	 */
	@RosettaSynonym(value = "DividendPaymentDate", source = "FpML_5_10")
	DIVIDEND_PAYMENT_DATE,
	    

	/**
	 * In respect of each Dividend Period, the relevant Dividend Valuation Date.
	 */
	@RosettaSynonym(value = "DividendValuationDate", source = "FpML_5_10")
	DIVIDEND_VALUATION_DATE,
	    

	/**
	 * Equity payment date of the swap.
	 */
	@RosettaSynonym(value = "EquityPaymentDate", source = "FpML_5_10")
	EQUITY_PAYMENT_DATE,
	    

	/**
	 * Date on which a holder of the security is entitled to the dividend.
	 */
	@RosettaSynonym(value = "ExDate", source = "FpML_5_10")
	EX_DATE,
	    

	/**
	 * If &#39;Dividend Payment Date(s)&#39; is specified in the Transaction Supplement as &#39;Floating Amount Payment Date&#39;, then the Dividend Payment Date in respect of a Dividend Amount shall be the first Payment Date falling at least one Settlement Cycle after the date that the Shares have commenced trading &#39;ex&#39; the relevant dividend on the Exchange.
	 */
	@RosettaSynonym(value = "FloatingAmountPaymentDate", source = "FpML_5_10")
	FLOATING_AMOUNT_PAYMENT_DATE,
	    

	/**
	 * The next payment date of the swap.
	 */
	@RosettaSynonym(value = "FollowingPaymentDate", source = "FpML_5_10")
	FOLLOWING_PAYMENT_DATE,
	    

	/**
	 * Date on which the dividend will be recorded in the books of the paying agent.
	 */
	@RosettaSynonym(value = "RecordDate", source = "FpML_5_10")
	RECORD_DATE,
	    

	/**
	 * If &#39;Dividend Payment Date(s)&#39; is specified in the Transaction Supplement as &#39;Share Payment&#39;, then the Dividend Payment Date in respect of a Dividend Amount shall fall on a date on or before the date that is two (or any other number that is specified in the Transaction Supplement) Currency Business Days following the day on which the Issuer of the Shares pays the relevant dividend to holders of record of the Shares.
	 */
	@RosettaSynonym(value = "SharePayment", source = "FpML_5_10")
	SHARE_PAYMENT,
	    

	/**
	 * Termination date of the swap.
	 */
	@RosettaSynonym(value = "TerminationDate", source = "FpML_5_10")
	TERMINATION_DATE    
	;

	private final String displayName;
	
	DividendDateReferenceEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
