package org.isda.cdm;

import org.isda.cdm.AdjustableDate;
import org.isda.cdm.DividendDateReference;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DividendPaymentDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class describing the date on which the dividend will be paid/received. This class is also used to specify the date on which the FX rate will be determined, when applicable.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="DividendPaymentDate", source="FpML_5_10")
public class DividendPaymentDate extends RosettaModelObject {
	private final AdjustableDate adjustableDate;
	private final DividendDateReference dividendDateReference;
	private static DividendPaymentDateMeta metaData = new DividendPaymentDateMeta();
	
	DividendPaymentDate(DividendPaymentDateBuilder builder) {
		this.adjustableDate = ofNullable(builder.getAdjustableDate()).map(f->f.build()).orElse(null);
		this.dividendDateReference = ofNullable(builder.getDividendDateReference()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="adjustableDate", source="FpML_5_10")
	public final AdjustableDate getAdjustableDate() {
		return adjustableDate;
	}
	
	@RosettaSynonym(value="dividendDateReference", source="FpML_5_10")
	public final DividendDateReference getDividendDateReference() {
		return dividendDateReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends DividendPaymentDate> metaData() {
		return metaData;
	} 
	
	public DividendPaymentDateBuilder toBuilder() {
		DividendPaymentDateBuilder builder = new DividendPaymentDateBuilder();
		ofNullable(getAdjustableDate()).ifPresent(builder::setAdjustableDate);
		ofNullable(getDividendDateReference()).ifPresent(builder::setDividendDateReference);
		return builder;
	}

	public static DividendPaymentDateBuilder builder() {
		return new DividendPaymentDateBuilder();
	}

	public static  class DividendPaymentDateBuilder extends RosettaModelObjectBuilder {
	
		protected AdjustableDate.AdjustableDateBuilder adjustableDate;
		protected DividendDateReference.DividendDateReferenceBuilder dividendDateReference;
	
		public DividendPaymentDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DividendPaymentDate> metaData() {
			return metaData;
		} 
	
		
		public AdjustableDate.AdjustableDateBuilder getAdjustableDate() {
			return adjustableDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateAdjustableDate() {
			if (adjustableDate!=null) {
				return adjustableDate;
			}
			else return adjustableDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public DividendDateReference.DividendDateReferenceBuilder getDividendDateReference() {
			return dividendDateReference;
		}
		
		public DividendDateReference.DividendDateReferenceBuilder getOrCreateDividendDateReference() {
			if (dividendDateReference!=null) {
				return dividendDateReference;
			}
			else return dividendDateReference = new DividendDateReference.DividendDateReferenceBuilder();
		}
		
	
		public DividendPaymentDateBuilder setAdjustableDate(AdjustableDate adjustableDate) {
			this.adjustableDate = adjustableDate.toBuilder();
			return this;
		}
		public DividendPaymentDateBuilder setAdjustableDateBuilder(AdjustableDate.AdjustableDateBuilder adjustableDate) {
			this.adjustableDate = adjustableDate;
			return this;
		}
		public DividendPaymentDateBuilder setDividendDateReference(DividendDateReference dividendDateReference) {
			this.dividendDateReference = dividendDateReference.toBuilder();
			return this;
		}
		public DividendPaymentDateBuilder setDividendDateReferenceBuilder(DividendDateReference.DividendDateReferenceBuilder dividendDateReference) {
			this.dividendDateReference = dividendDateReference;
			return this;
		}
		
		public DividendPaymentDate build() {
			return new DividendPaymentDate(this);
		}
	
		@Override
		public DividendPaymentDateBuilder prune() {
			if (adjustableDate!=null && !adjustableDate.prune().hasData()) adjustableDate = null;
			if (dividendDateReference!=null && !dividendDateReference.prune().hasData()) dividendDateReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustableDate()!=null && getAdjustableDate().hasData()) return true;
			if (getDividendDateReference()!=null && getDividendDateReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("adjustableDate"), processor, AdjustableDate.class, adjustableDate);
			processRosetta(path.newSubPath("dividendDateReference"), processor, DividendDateReference.class, dividendDateReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DividendPaymentDateBuilder _that = (DividendPaymentDateBuilder) o;
		
			if (adjustableDate != null ? !adjustableDate.equals(_that.adjustableDate) : _that.adjustableDate != null) return false;
			if (dividendDateReference != null ? !dividendDateReference.equals(_that.dividendDateReference) : _that.dividendDateReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustableDate != null ? adjustableDate.hashCode() : 0);
			_result = 31 * _result + (dividendDateReference != null ? dividendDateReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DividendPaymentDateBuilder {" +
				"adjustableDate=" + this.adjustableDate + ", " +
				"dividendDateReference=" + this.dividendDateReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("adjustableDate"), processor, AdjustableDate.class, adjustableDate);
		processRosetta(path.newSubPath("dividendDateReference"), processor, DividendDateReference.class, dividendDateReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DividendPaymentDate _that = (DividendPaymentDate) o;
	
		if (adjustableDate != null ? !adjustableDate.equals(_that.adjustableDate) : _that.adjustableDate != null) return false;
		if (dividendDateReference != null ? !dividendDateReference.equals(_that.dividendDateReference) : _that.dividendDateReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustableDate != null ? adjustableDate.hashCode() : 0);
		_result = 31 * _result + (dividendDateReference != null ? dividendDateReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DividendPaymentDate {" +
			"adjustableDate=" + this.adjustableDate + ", " +
			"dividendDateReference=" + this.dividendDateReference +
		'}';
	}
}
