package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.BrokerConfirmation;
import org.isda.cdm.ContractualMatrix;
import org.isda.cdm.ContractualTermsSupplement;
import org.isda.cdm.CreditSupportAgreement;
import org.isda.cdm.MasterAgreement;
import org.isda.cdm.MasterConfirmation;
import org.isda.cdm.OtherAgreement;
import org.isda.cdm.Resource;
import org.isda.cdm.metafields.FieldWithMetaContractualDefinitionsEnum;
import org.isda.cdm.metafields.FieldWithMetaContractualDefinitionsEnum.FieldWithMetaContractualDefinitionsEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.DocumentationIdentificationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class for defining the definitions that govern the document and should include the year and type of definitions referenced, along with any relevant documentation (such as master agreement) and the date it was signed.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Documentation", source="FpML_5_10")
public class DocumentationIdentification extends RosettaModelObject {
	private final List<Resource> attachment;
	private final BrokerConfirmation brokerConfirmation;
	private final List<FieldWithMetaContractualDefinitionsEnum> contractualDefinitions;
	private final List<ContractualMatrix> contractualMatrix;
	private final List<ContractualTermsSupplement> contractualTermsSupplement;
	private final CreditSupportAgreement creditSupportAgreement;
	private final MasterAgreement masterAgreement;
	private final MasterConfirmation masterConfirmation;
	private final List<OtherAgreement> otherAgreement;
	private static DocumentationIdentificationMeta metaData = new DocumentationIdentificationMeta();
	
	DocumentationIdentification(DocumentationIdentificationBuilder builder) {
		this.attachment = ofNullable(builder.getAttachment()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.brokerConfirmation = ofNullable(builder.getBrokerConfirmation()).map(f->f.build()).orElse(null);
		this.contractualDefinitions = ofNullable(builder.getContractualDefinitions()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contractualMatrix = ofNullable(builder.getContractualMatrix()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contractualTermsSupplement = ofNullable(builder.getContractualTermsSupplement()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.creditSupportAgreement = ofNullable(builder.getCreditSupportAgreement()).map(f->f.build()).orElse(null);
		this.masterAgreement = ofNullable(builder.getMasterAgreement()).map(f->f.build()).orElse(null);
		this.masterConfirmation = ofNullable(builder.getMasterConfirmation()).map(f->f.build()).orElse(null);
		this.otherAgreement = ofNullable(builder.getOtherAgreement()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A human readable document related to this transaction, for example a confirmation.
	 */
	@RosettaSynonym(value="attachment", source="FpML_5_10")
	@RosettaSynonym(value="attachment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="attachment", source="DTCC_11_0")
	@RosettaSynonym(value="attachment", source="DTCC_9_0")
	@RosettaSynonym(value="attachment", source="CME_ClearedConfirm_1_17")
	public final List<Resource> getAttachment() {
		return attachment;
	}
	
	/**
	 * Specifies the details for a broker confirm.
	 */
	@RosettaSynonym(value="brokerConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="brokerConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="brokerConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="brokerConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="brokerConfirmation", source="CME_ClearedConfirm_1_17")
	public final BrokerConfirmation getBrokerConfirmation() {
		return brokerConfirmation;
	}
	
	/**
	 * The definitions such as those published by ISDA that will define the terms of the trade.
	 */
	@RosettaSynonym(value="contractualDefinitions", source="FpML_5_10")
	@RosettaSynonym(value="contractualDefinitions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="contractualDefinitions", source="DTCC_11_0")
	@RosettaSynonym(value="contractualDefinitions", source="DTCC_9_0")
	@RosettaSynonym(value="contractualDefinitions", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaContractualDefinitionsEnum> getContractualDefinitions() {
		return contractualDefinitions;
	}
	
	/**
	 * A reference to a contractual matrix of elected terms/values (such as those published by ISDA) that shall be deemed to apply to the trade. The applicable matrix is identified by reference to a name and optionally a publication date. Depending on the structure of the matrix, an additional term (specified in the matrixTerm element) may be required to further identify a subset of applicable terms/values within the matrix.
	 */
	@RosettaSynonym(value="contractualMatrix", source="FpML_5_10")
	@RosettaSynonym(value="contractualMatrix", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="contractualMatrix", source="DTCC_11_0")
	@RosettaSynonym(value="contractualMatrix", source="DTCC_9_0")
	@RosettaSynonym(value="contractualMatrix", source="CME_ClearedConfirm_1_17")
	public final List<ContractualMatrix> getContractualMatrix() {
		return contractualMatrix;
	}
	
	/**
	 * A contractual supplement (such as those published by ISDA) that will apply to the trade.
	 */
	@RosettaSynonym(value="contractualTermsSupplement", source="FpML_5_10")
	@RosettaSynonym(value="contractualTermsSupplement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="contractualTermsSupplement", source="DTCC_11_0")
	@RosettaSynonym(value="contractualTermsSupplement", source="DTCC_9_0")
	@RosettaSynonym(value="contractualTermsSupplement", source="CME_ClearedConfirm_1_17")
	public final List<ContractualTermsSupplement> getContractualTermsSupplement() {
		return contractualTermsSupplement;
	}
	
	/**
	 * The agreement executed between the parties and intended to govern collateral arrangement for all OTC derivatives transactions between those parties.
	 */
	@RosettaSynonym(value="creditSupportAgreement", source="FpML_5_10")
	@RosettaSynonym(value="creditSupportAgreement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditSupportAgreement", source="DTCC_11_0")
	@RosettaSynonym(value="creditSupportAgreement", source="DTCC_9_0")
	@RosettaSynonym(value="creditSupportAgreement", source="CME_ClearedConfirm_1_17")
	public final CreditSupportAgreement getCreditSupportAgreement() {
		return creditSupportAgreement;
	}
	
	/**
	 * The agreement executed between the parties and intended to govern all OTC derivatives transactions between those parties.
	 */
	@RosettaSynonym(value="masterAgreement", source="FpML_5_10")
	@RosettaSynonym(value="masterAgreement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterAgreement", source="DTCC_11_0")
	@RosettaSynonym(value="masterAgreement", source="DTCC_9_0")
	@RosettaSynonym(value="masterAgreement", source="CME_ClearedConfirm_1_17")
	public final MasterAgreement getMasterAgreement() {
		return masterAgreement;
	}
	
	@RosettaSynonym(value="masterConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="masterConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="masterConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="masterConfirmation", source="CME_ClearedConfirm_1_17")
	public final MasterConfirmation getMasterConfirmation() {
		return masterConfirmation;
	}
	
	/**
	 * Any other agreement executed between the parties.
	 */
	@RosettaSynonym(value="otherAgreement", source="FpML_5_10")
	@RosettaSynonym(value="otherAgreement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="otherAgreement", source="DTCC_11_0")
	@RosettaSynonym(value="otherAgreement", source="DTCC_9_0")
	@RosettaSynonym(value="otherAgreement", source="CME_ClearedConfirm_1_17")
	public final List<OtherAgreement> getOtherAgreement() {
		return otherAgreement;
	}
	
	
	@Override
	public RosettaMetaData<? extends DocumentationIdentification> metaData() {
		return metaData;
	} 
	
	public DocumentationIdentificationBuilder toBuilder() {
		DocumentationIdentificationBuilder builder = new DocumentationIdentificationBuilder();
		ofNullable(getAttachment()).ifPresent(attachment -> attachment.forEach(builder::addAttachment));
		ofNullable(getBrokerConfirmation()).ifPresent(builder::setBrokerConfirmation);
		ofNullable(getContractualDefinitions()).ifPresent(contractualDefinitions -> contractualDefinitions.forEach(builder::addContractualDefinitions));
		ofNullable(getContractualMatrix()).ifPresent(contractualMatrix -> contractualMatrix.forEach(builder::addContractualMatrix));
		ofNullable(getContractualTermsSupplement()).ifPresent(contractualTermsSupplement -> contractualTermsSupplement.forEach(builder::addContractualTermsSupplement));
		ofNullable(getCreditSupportAgreement()).ifPresent(builder::setCreditSupportAgreement);
		ofNullable(getMasterAgreement()).ifPresent(builder::setMasterAgreement);
		ofNullable(getMasterConfirmation()).ifPresent(builder::setMasterConfirmation);
		ofNullable(getOtherAgreement()).ifPresent(otherAgreement -> otherAgreement.forEach(builder::addOtherAgreement));
		return builder;
	}

	public static DocumentationIdentificationBuilder builder() {
		return new DocumentationIdentificationBuilder();
	}

	public static  class DocumentationIdentificationBuilder extends RosettaModelObjectBuilder {
	
		protected List<Resource.ResourceBuilder> attachment;
		protected BrokerConfirmation.BrokerConfirmationBuilder brokerConfirmation;
		protected List<FieldWithMetaContractualDefinitionsEnumBuilder> contractualDefinitions;
		protected List<ContractualMatrix.ContractualMatrixBuilder> contractualMatrix;
		protected List<ContractualTermsSupplement.ContractualTermsSupplementBuilder> contractualTermsSupplement;
		protected CreditSupportAgreement.CreditSupportAgreementBuilder creditSupportAgreement;
		protected MasterAgreement.MasterAgreementBuilder masterAgreement;
		protected MasterConfirmation.MasterConfirmationBuilder masterConfirmation;
		protected List<OtherAgreement.OtherAgreementBuilder> otherAgreement;
	
		public DocumentationIdentificationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends DocumentationIdentification> metaData() {
			return metaData;
		} 
	
		
		public List<Resource.ResourceBuilder> getAttachment() {
			return attachment;
		}
		
		public Resource.ResourceBuilder getOrCreateAttachment(int index) {
			if (attachment==null) {
				this.attachment = new ArrayList<>();
			}
			return getIndex(attachment, index, ()->new Resource.ResourceBuilder());
		}
		
		
		public BrokerConfirmation.BrokerConfirmationBuilder getBrokerConfirmation() {
			return brokerConfirmation;
		}
		
		public BrokerConfirmation.BrokerConfirmationBuilder getOrCreateBrokerConfirmation() {
			if (brokerConfirmation!=null) {
				return brokerConfirmation;
			}
			else return brokerConfirmation = new BrokerConfirmation.BrokerConfirmationBuilder();
		}
		
		
		public List<FieldWithMetaContractualDefinitionsEnumBuilder> getContractualDefinitions() {
			return contractualDefinitions;
		}
		
		public FieldWithMetaContractualDefinitionsEnumBuilder getOrCreateContractualDefinitions(int index) {
			if (contractualDefinitions==null) {
				this.contractualDefinitions = new ArrayList<>();
			}
			return getIndex(contractualDefinitions, index, ()->new FieldWithMetaContractualDefinitionsEnumBuilder());
		}
		
		
		public List<ContractualMatrix.ContractualMatrixBuilder> getContractualMatrix() {
			return contractualMatrix;
		}
		
		public ContractualMatrix.ContractualMatrixBuilder getOrCreateContractualMatrix(int index) {
			if (contractualMatrix==null) {
				this.contractualMatrix = new ArrayList<>();
			}
			return getIndex(contractualMatrix, index, ()->new ContractualMatrix.ContractualMatrixBuilder());
		}
		
		
		public List<ContractualTermsSupplement.ContractualTermsSupplementBuilder> getContractualTermsSupplement() {
			return contractualTermsSupplement;
		}
		
		public ContractualTermsSupplement.ContractualTermsSupplementBuilder getOrCreateContractualTermsSupplement(int index) {
			if (contractualTermsSupplement==null) {
				this.contractualTermsSupplement = new ArrayList<>();
			}
			return getIndex(contractualTermsSupplement, index, ()->new ContractualTermsSupplement.ContractualTermsSupplementBuilder());
		}
		
		
		public CreditSupportAgreement.CreditSupportAgreementBuilder getCreditSupportAgreement() {
			return creditSupportAgreement;
		}
		
		public CreditSupportAgreement.CreditSupportAgreementBuilder getOrCreateCreditSupportAgreement() {
			if (creditSupportAgreement!=null) {
				return creditSupportAgreement;
			}
			else return creditSupportAgreement = new CreditSupportAgreement.CreditSupportAgreementBuilder();
		}
		
		
		public MasterAgreement.MasterAgreementBuilder getMasterAgreement() {
			return masterAgreement;
		}
		
		public MasterAgreement.MasterAgreementBuilder getOrCreateMasterAgreement() {
			if (masterAgreement!=null) {
				return masterAgreement;
			}
			else return masterAgreement = new MasterAgreement.MasterAgreementBuilder();
		}
		
		
		public MasterConfirmation.MasterConfirmationBuilder getMasterConfirmation() {
			return masterConfirmation;
		}
		
		public MasterConfirmation.MasterConfirmationBuilder getOrCreateMasterConfirmation() {
			if (masterConfirmation!=null) {
				return masterConfirmation;
			}
			else return masterConfirmation = new MasterConfirmation.MasterConfirmationBuilder();
		}
		
		
		public List<OtherAgreement.OtherAgreementBuilder> getOtherAgreement() {
			return otherAgreement;
		}
		
		public OtherAgreement.OtherAgreementBuilder getOrCreateOtherAgreement(int index) {
			if (otherAgreement==null) {
				this.otherAgreement = new ArrayList<>();
			}
			return getIndex(otherAgreement, index, ()->new OtherAgreement.OtherAgreementBuilder());
		}
		
	
		public DocumentationIdentificationBuilder addAttachment(Resource attachment) {
			if(this.attachment == null){
				this.attachment = new ArrayList<>();
			}
			this.attachment.add(attachment.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addAttachment(Resource attachment, int _idx) {
			if(this.attachment == null){
				this.attachment = new ArrayList<>();
			}
			getIndex(this.attachment, _idx, () -> attachment.toBuilder());
			this.attachment.set(_idx, attachment.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addAttachment(List<Resource> attachments) {
			if(this.attachment == null){
				this.attachment = new ArrayList<>();
			}
			for (Resource toAdd : attachments) {
				this.attachment.add(toAdd.toBuilder());
			}
			return this;
		}
		public DocumentationIdentificationBuilder addAttachmentBuilder(Resource.ResourceBuilder attachment) {
			if(this.attachment == null){
				this.attachment = new ArrayList<>();
			}
			this.attachment.add(attachment);
			return this;
		}
		
		public DocumentationIdentificationBuilder clearAttachment() {
			this.attachment = null;
			return this;
		}
		public DocumentationIdentificationBuilder setBrokerConfirmation(BrokerConfirmation brokerConfirmation) {
			this.brokerConfirmation = brokerConfirmation.toBuilder();
			return this;
		}
		public DocumentationIdentificationBuilder setBrokerConfirmationBuilder(BrokerConfirmation.BrokerConfirmationBuilder brokerConfirmation) {
			this.brokerConfirmation = brokerConfirmation;
			return this;
		}
		public DocumentationIdentificationBuilder addContractualDefinitions(FieldWithMetaContractualDefinitionsEnum contractualDefinitions) {
			if(this.contractualDefinitions == null){
				this.contractualDefinitions = new ArrayList<>();
			}
			this.contractualDefinitions.add(contractualDefinitions.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualDefinitions(FieldWithMetaContractualDefinitionsEnum contractualDefinitions, int _idx) {
			if(this.contractualDefinitions == null){
				this.contractualDefinitions = new ArrayList<>();
			}
			getIndex(this.contractualDefinitions, _idx, () -> contractualDefinitions.toBuilder());
			this.contractualDefinitions.set(_idx, contractualDefinitions.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualDefinitions(List<FieldWithMetaContractualDefinitionsEnum> contractualDefinitionss) {
			if(this.contractualDefinitions == null){
				this.contractualDefinitions = new ArrayList<>();
			}
			for (FieldWithMetaContractualDefinitionsEnum toAdd : contractualDefinitionss) {
				this.contractualDefinitions.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public DocumentationIdentificationBuilder clearContractualDefinitions() {
			this.contractualDefinitions = null;
			return this;
		}
		public DocumentationIdentificationBuilder addContractualMatrix(ContractualMatrix contractualMatrix) {
			if(this.contractualMatrix == null){
				this.contractualMatrix = new ArrayList<>();
			}
			this.contractualMatrix.add(contractualMatrix.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualMatrix(ContractualMatrix contractualMatrix, int _idx) {
			if(this.contractualMatrix == null){
				this.contractualMatrix = new ArrayList<>();
			}
			getIndex(this.contractualMatrix, _idx, () -> contractualMatrix.toBuilder());
			this.contractualMatrix.set(_idx, contractualMatrix.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualMatrix(List<ContractualMatrix> contractualMatrixs) {
			if(this.contractualMatrix == null){
				this.contractualMatrix = new ArrayList<>();
			}
			for (ContractualMatrix toAdd : contractualMatrixs) {
				this.contractualMatrix.add(toAdd.toBuilder());
			}
			return this;
		}
		public DocumentationIdentificationBuilder addContractualMatrixBuilder(ContractualMatrix.ContractualMatrixBuilder contractualMatrix) {
			if(this.contractualMatrix == null){
				this.contractualMatrix = new ArrayList<>();
			}
			this.contractualMatrix.add(contractualMatrix);
			return this;
		}
		
		public DocumentationIdentificationBuilder clearContractualMatrix() {
			this.contractualMatrix = null;
			return this;
		}
		public DocumentationIdentificationBuilder addContractualTermsSupplement(ContractualTermsSupplement contractualTermsSupplement) {
			if(this.contractualTermsSupplement == null){
				this.contractualTermsSupplement = new ArrayList<>();
			}
			this.contractualTermsSupplement.add(contractualTermsSupplement.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualTermsSupplement(ContractualTermsSupplement contractualTermsSupplement, int _idx) {
			if(this.contractualTermsSupplement == null){
				this.contractualTermsSupplement = new ArrayList<>();
			}
			getIndex(this.contractualTermsSupplement, _idx, () -> contractualTermsSupplement.toBuilder());
			this.contractualTermsSupplement.set(_idx, contractualTermsSupplement.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addContractualTermsSupplement(List<ContractualTermsSupplement> contractualTermsSupplements) {
			if(this.contractualTermsSupplement == null){
				this.contractualTermsSupplement = new ArrayList<>();
			}
			for (ContractualTermsSupplement toAdd : contractualTermsSupplements) {
				this.contractualTermsSupplement.add(toAdd.toBuilder());
			}
			return this;
		}
		public DocumentationIdentificationBuilder addContractualTermsSupplementBuilder(ContractualTermsSupplement.ContractualTermsSupplementBuilder contractualTermsSupplement) {
			if(this.contractualTermsSupplement == null){
				this.contractualTermsSupplement = new ArrayList<>();
			}
			this.contractualTermsSupplement.add(contractualTermsSupplement);
			return this;
		}
		
		public DocumentationIdentificationBuilder clearContractualTermsSupplement() {
			this.contractualTermsSupplement = null;
			return this;
		}
		public DocumentationIdentificationBuilder setCreditSupportAgreement(CreditSupportAgreement creditSupportAgreement) {
			this.creditSupportAgreement = creditSupportAgreement.toBuilder();
			return this;
		}
		public DocumentationIdentificationBuilder setCreditSupportAgreementBuilder(CreditSupportAgreement.CreditSupportAgreementBuilder creditSupportAgreement) {
			this.creditSupportAgreement = creditSupportAgreement;
			return this;
		}
		public DocumentationIdentificationBuilder setMasterAgreement(MasterAgreement masterAgreement) {
			this.masterAgreement = masterAgreement.toBuilder();
			return this;
		}
		public DocumentationIdentificationBuilder setMasterAgreementBuilder(MasterAgreement.MasterAgreementBuilder masterAgreement) {
			this.masterAgreement = masterAgreement;
			return this;
		}
		public DocumentationIdentificationBuilder setMasterConfirmation(MasterConfirmation masterConfirmation) {
			this.masterConfirmation = masterConfirmation.toBuilder();
			return this;
		}
		public DocumentationIdentificationBuilder setMasterConfirmationBuilder(MasterConfirmation.MasterConfirmationBuilder masterConfirmation) {
			this.masterConfirmation = masterConfirmation;
			return this;
		}
		public DocumentationIdentificationBuilder addOtherAgreement(OtherAgreement otherAgreement) {
			if(this.otherAgreement == null){
				this.otherAgreement = new ArrayList<>();
			}
			this.otherAgreement.add(otherAgreement.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addOtherAgreement(OtherAgreement otherAgreement, int _idx) {
			if(this.otherAgreement == null){
				this.otherAgreement = new ArrayList<>();
			}
			getIndex(this.otherAgreement, _idx, () -> otherAgreement.toBuilder());
			this.otherAgreement.set(_idx, otherAgreement.toBuilder());
			return this;
		}
		public DocumentationIdentificationBuilder addOtherAgreement(List<OtherAgreement> otherAgreements) {
			if(this.otherAgreement == null){
				this.otherAgreement = new ArrayList<>();
			}
			for (OtherAgreement toAdd : otherAgreements) {
				this.otherAgreement.add(toAdd.toBuilder());
			}
			return this;
		}
		public DocumentationIdentificationBuilder addOtherAgreementBuilder(OtherAgreement.OtherAgreementBuilder otherAgreement) {
			if(this.otherAgreement == null){
				this.otherAgreement = new ArrayList<>();
			}
			this.otherAgreement.add(otherAgreement);
			return this;
		}
		
		public DocumentationIdentificationBuilder clearOtherAgreement() {
			this.otherAgreement = null;
			return this;
		}
		
		public DocumentationIdentification build() {
			return new DocumentationIdentification(this);
		}
	
		@Override
		public DocumentationIdentificationBuilder prune() {
			if (attachment!=null) attachment = attachment.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (brokerConfirmation!=null && !brokerConfirmation.prune().hasData()) brokerConfirmation = null;
			if (contractualDefinitions!=null) contractualDefinitions = contractualDefinitions.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contractualMatrix!=null) contractualMatrix = contractualMatrix.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contractualTermsSupplement!=null) contractualTermsSupplement = contractualTermsSupplement.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (creditSupportAgreement!=null && !creditSupportAgreement.prune().hasData()) creditSupportAgreement = null;
			if (masterAgreement!=null && !masterAgreement.prune().hasData()) masterAgreement = null;
			if (masterConfirmation!=null && !masterConfirmation.prune().hasData()) masterConfirmation = null;
			if (otherAgreement!=null) otherAgreement = otherAgreement.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAttachment()!=null && getAttachment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getBrokerConfirmation()!=null && getBrokerConfirmation().hasData()) return true;
			if (getContractualDefinitions()!=null && !getContractualDefinitions().isEmpty()) return true;
			if (getContractualMatrix()!=null && getContractualMatrix().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getContractualTermsSupplement()!=null && getContractualTermsSupplement().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCreditSupportAgreement()!=null && getCreditSupportAgreement().hasData()) return true;
			if (getMasterAgreement()!=null && getMasterAgreement().hasData()) return true;
			if (getMasterConfirmation()!=null && getMasterConfirmation().hasData()) return true;
			if (getOtherAgreement()!=null && getOtherAgreement().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("attachment"), processor, Resource.class, attachment);
			processRosetta(path.newSubPath("brokerConfirmation"), processor, BrokerConfirmation.class, brokerConfirmation);
			processRosetta(path.newSubPath("contractualDefinitions"), processor, FieldWithMetaContractualDefinitionsEnum.class, contractualDefinitions);
			processRosetta(path.newSubPath("contractualMatrix"), processor, ContractualMatrix.class, contractualMatrix);
			processRosetta(path.newSubPath("contractualTermsSupplement"), processor, ContractualTermsSupplement.class, contractualTermsSupplement);
			processRosetta(path.newSubPath("creditSupportAgreement"), processor, CreditSupportAgreement.class, creditSupportAgreement);
			processRosetta(path.newSubPath("masterAgreement"), processor, MasterAgreement.class, masterAgreement);
			processRosetta(path.newSubPath("masterConfirmation"), processor, MasterConfirmation.class, masterConfirmation);
			processRosetta(path.newSubPath("otherAgreement"), processor, OtherAgreement.class, otherAgreement);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			DocumentationIdentificationBuilder _that = (DocumentationIdentificationBuilder) o;
		
			if (!ListEquals.listEquals(attachment, _that.attachment)) return false;
			if (brokerConfirmation != null ? !brokerConfirmation.equals(_that.brokerConfirmation) : _that.brokerConfirmation != null) return false;
			if (!ListEquals.listEquals(contractualDefinitions, _that.contractualDefinitions)) return false;
			if (!ListEquals.listEquals(contractualMatrix, _that.contractualMatrix)) return false;
			if (!ListEquals.listEquals(contractualTermsSupplement, _that.contractualTermsSupplement)) return false;
			if (creditSupportAgreement != null ? !creditSupportAgreement.equals(_that.creditSupportAgreement) : _that.creditSupportAgreement != null) return false;
			if (masterAgreement != null ? !masterAgreement.equals(_that.masterAgreement) : _that.masterAgreement != null) return false;
			if (masterConfirmation != null ? !masterConfirmation.equals(_that.masterConfirmation) : _that.masterConfirmation != null) return false;
			if (!ListEquals.listEquals(otherAgreement, _that.otherAgreement)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (attachment != null ? attachment.hashCode() : 0);
			_result = 31 * _result + (brokerConfirmation != null ? brokerConfirmation.hashCode() : 0);
			_result = 31 * _result + (contractualDefinitions != null ? contractualDefinitions.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (contractualMatrix != null ? contractualMatrix.hashCode() : 0);
			_result = 31 * _result + (contractualTermsSupplement != null ? contractualTermsSupplement.hashCode() : 0);
			_result = 31 * _result + (creditSupportAgreement != null ? creditSupportAgreement.hashCode() : 0);
			_result = 31 * _result + (masterAgreement != null ? masterAgreement.hashCode() : 0);
			_result = 31 * _result + (masterConfirmation != null ? masterConfirmation.hashCode() : 0);
			_result = 31 * _result + (otherAgreement != null ? otherAgreement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DocumentationIdentificationBuilder {" +
				"attachment=" + this.attachment + ", " +
				"brokerConfirmation=" + this.brokerConfirmation + ", " +
				"contractualDefinitions=" + this.contractualDefinitions + ", " +
				"contractualMatrix=" + this.contractualMatrix + ", " +
				"contractualTermsSupplement=" + this.contractualTermsSupplement + ", " +
				"creditSupportAgreement=" + this.creditSupportAgreement + ", " +
				"masterAgreement=" + this.masterAgreement + ", " +
				"masterConfirmation=" + this.masterConfirmation + ", " +
				"otherAgreement=" + this.otherAgreement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("attachment"), processor, Resource.class, attachment);
		processRosetta(path.newSubPath("brokerConfirmation"), processor, BrokerConfirmation.class, brokerConfirmation);
		processRosetta(path.newSubPath("contractualDefinitions"), processor, FieldWithMetaContractualDefinitionsEnum.class, contractualDefinitions);
		processRosetta(path.newSubPath("contractualMatrix"), processor, ContractualMatrix.class, contractualMatrix);
		processRosetta(path.newSubPath("contractualTermsSupplement"), processor, ContractualTermsSupplement.class, contractualTermsSupplement);
		processRosetta(path.newSubPath("creditSupportAgreement"), processor, CreditSupportAgreement.class, creditSupportAgreement);
		processRosetta(path.newSubPath("masterAgreement"), processor, MasterAgreement.class, masterAgreement);
		processRosetta(path.newSubPath("masterConfirmation"), processor, MasterConfirmation.class, masterConfirmation);
		processRosetta(path.newSubPath("otherAgreement"), processor, OtherAgreement.class, otherAgreement);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		DocumentationIdentification _that = (DocumentationIdentification) o;
	
		if (!ListEquals.listEquals(attachment, _that.attachment)) return false;
		if (brokerConfirmation != null ? !brokerConfirmation.equals(_that.brokerConfirmation) : _that.brokerConfirmation != null) return false;
		if (!ListEquals.listEquals(contractualDefinitions, _that.contractualDefinitions)) return false;
		if (!ListEquals.listEquals(contractualMatrix, _that.contractualMatrix)) return false;
		if (!ListEquals.listEquals(contractualTermsSupplement, _that.contractualTermsSupplement)) return false;
		if (creditSupportAgreement != null ? !creditSupportAgreement.equals(_that.creditSupportAgreement) : _that.creditSupportAgreement != null) return false;
		if (masterAgreement != null ? !masterAgreement.equals(_that.masterAgreement) : _that.masterAgreement != null) return false;
		if (masterConfirmation != null ? !masterConfirmation.equals(_that.masterConfirmation) : _that.masterConfirmation != null) return false;
		if (!ListEquals.listEquals(otherAgreement, _that.otherAgreement)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (attachment != null ? attachment.hashCode() : 0);
		_result = 31 * _result + (brokerConfirmation != null ? brokerConfirmation.hashCode() : 0);
		_result = 31 * _result + (contractualDefinitions != null ? contractualDefinitions.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		_result = 31 * _result + (contractualMatrix != null ? contractualMatrix.hashCode() : 0);
		_result = 31 * _result + (contractualTermsSupplement != null ? contractualTermsSupplement.hashCode() : 0);
		_result = 31 * _result + (creditSupportAgreement != null ? creditSupportAgreement.hashCode() : 0);
		_result = 31 * _result + (masterAgreement != null ? masterAgreement.hashCode() : 0);
		_result = 31 * _result + (masterConfirmation != null ? masterConfirmation.hashCode() : 0);
		_result = 31 * _result + (otherAgreement != null ? otherAgreement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "DocumentationIdentification {" +
			"attachment=" + this.attachment + ", " +
			"brokerConfirmation=" + this.brokerConfirmation + ", " +
			"contractualDefinitions=" + this.contractualDefinitions + ", " +
			"contractualMatrix=" + this.contractualMatrix + ", " +
			"contractualTermsSupplement=" + this.contractualTermsSupplement + ", " +
			"creditSupportAgreement=" + this.creditSupportAgreement + ", " +
			"masterAgreement=" + this.masterAgreement + ", " +
			"masterConfirmation=" + this.masterConfirmation + ", " +
			"otherAgreement=" + this.otherAgreement +
		'}';
	}
}
