package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EarlyTerminationEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to define the adjusted dates associated with an early termination provision.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="EarlyTerminationEvent", source="FpML_5_10")
public class EarlyTerminationEvent extends RosettaModelObject implements GlobalKey {
	private final Date adjustedCashSettlementPaymentDate;
	private final Date adjustedCashSettlementValuationDate;
	private final Date adjustedEarlyTerminationDate;
	private final Date adjustedExerciseDate;
	private final Date adjustedExerciseFeePaymentDate;
	private final MetaFields meta;
	private static EarlyTerminationEventMeta metaData = new EarlyTerminationEventMeta();
	
	EarlyTerminationEvent(EarlyTerminationEventBuilder builder) {
		this.adjustedCashSettlementPaymentDate = builder.getAdjustedCashSettlementPaymentDate();
		this.adjustedCashSettlementValuationDate = builder.getAdjustedCashSettlementValuationDate();
		this.adjustedEarlyTerminationDate = builder.getAdjustedEarlyTerminationDate();
		this.adjustedExerciseDate = builder.getAdjustedExerciseDate();
		this.adjustedExerciseFeePaymentDate = builder.getAdjustedExerciseFeePaymentDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the cash settlement amount is paid. This date should already be adjusted for any applicable business date convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementPaymentDate() {
		return adjustedCashSettlementPaymentDate;
	}
	
	/**
	 * The date by which the cash settlement amount must be agreed. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementValuationDate() {
		return adjustedCashSettlementValuationDate;
	}
	
	/**
	 * The early termination date that is applicable if an early termination provision is exercised. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedEarlyTerminationDate() {
		return adjustedEarlyTerminationDate;
	}
	
	/**
	 * The date on which option exercise takes place. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseDate() {
		return adjustedExerciseDate;
	}
	
	/**
	 * The date on which the exercise fee amount is paid. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseFeePaymentDate() {
		return adjustedExerciseFeePaymentDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends EarlyTerminationEvent> metaData() {
		return metaData;
	} 
	
	public EarlyTerminationEventBuilder toBuilder() {
		EarlyTerminationEventBuilder builder = new EarlyTerminationEventBuilder();
		ofNullable(getAdjustedCashSettlementPaymentDate()).ifPresent(builder::setAdjustedCashSettlementPaymentDate);
		ofNullable(getAdjustedCashSettlementValuationDate()).ifPresent(builder::setAdjustedCashSettlementValuationDate);
		ofNullable(getAdjustedEarlyTerminationDate()).ifPresent(builder::setAdjustedEarlyTerminationDate);
		ofNullable(getAdjustedExerciseDate()).ifPresent(builder::setAdjustedExerciseDate);
		ofNullable(getAdjustedExerciseFeePaymentDate()).ifPresent(builder::setAdjustedExerciseFeePaymentDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static EarlyTerminationEventBuilder builder() {
		return new EarlyTerminationEventBuilder();
	}

	public static  class EarlyTerminationEventBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<EarlyTerminationEventBuilder> {
	
		protected Date adjustedCashSettlementPaymentDate;
		protected Date adjustedCashSettlementValuationDate;
		protected Date adjustedEarlyTerminationDate;
		protected Date adjustedExerciseDate;
		protected Date adjustedExerciseFeePaymentDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public EarlyTerminationEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EarlyTerminationEvent> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedCashSettlementPaymentDate() {
			return adjustedCashSettlementPaymentDate;
		}
		
		
		public Date getAdjustedCashSettlementValuationDate() {
			return adjustedCashSettlementValuationDate;
		}
		
		
		public Date getAdjustedEarlyTerminationDate() {
			return adjustedEarlyTerminationDate;
		}
		
		
		public Date getAdjustedExerciseDate() {
			return adjustedExerciseDate;
		}
		
		
		public Date getAdjustedExerciseFeePaymentDate() {
			return adjustedExerciseFeePaymentDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public EarlyTerminationEventBuilder setAdjustedCashSettlementPaymentDate(Date adjustedCashSettlementPaymentDate) {
			this.adjustedCashSettlementPaymentDate = adjustedCashSettlementPaymentDate;
			return this;
		}
		public EarlyTerminationEventBuilder setAdjustedCashSettlementValuationDate(Date adjustedCashSettlementValuationDate) {
			this.adjustedCashSettlementValuationDate = adjustedCashSettlementValuationDate;
			return this;
		}
		public EarlyTerminationEventBuilder setAdjustedEarlyTerminationDate(Date adjustedEarlyTerminationDate) {
			this.adjustedEarlyTerminationDate = adjustedEarlyTerminationDate;
			return this;
		}
		public EarlyTerminationEventBuilder setAdjustedExerciseDate(Date adjustedExerciseDate) {
			this.adjustedExerciseDate = adjustedExerciseDate;
			return this;
		}
		public EarlyTerminationEventBuilder setAdjustedExerciseFeePaymentDate(Date adjustedExerciseFeePaymentDate) {
			this.adjustedExerciseFeePaymentDate = adjustedExerciseFeePaymentDate;
			return this;
		}
		public EarlyTerminationEventBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public EarlyTerminationEventBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public EarlyTerminationEvent build() {
			return new EarlyTerminationEvent(this);
		}
	
		@Override
		public EarlyTerminationEventBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedCashSettlementPaymentDate()!=null) return true;
			if (getAdjustedCashSettlementValuationDate()!=null) return true;
			if (getAdjustedEarlyTerminationDate()!=null) return true;
			if (getAdjustedExerciseDate()!=null) return true;
			if (getAdjustedExerciseFeePaymentDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
			processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
			processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
			processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
			processor.processBasic(path.newSubPath("adjustedExerciseFeePaymentDate"), Date.class, adjustedExerciseFeePaymentDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EarlyTerminationEventBuilder _that = (EarlyTerminationEventBuilder) o;
		
			if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
			if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
			if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
			if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
			if (adjustedExerciseFeePaymentDate != null ? !adjustedExerciseFeePaymentDate.equals(_that.adjustedExerciseFeePaymentDate) : _that.adjustedExerciseFeePaymentDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
			_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
			_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExerciseFeePaymentDate != null ? adjustedExerciseFeePaymentDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EarlyTerminationEventBuilder {" +
				"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
				"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
				"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate + ", " +
				"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
				"adjustedExerciseFeePaymentDate=" + this.adjustedExerciseFeePaymentDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
		processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
		processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
		processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
		processor.processBasic(path.newSubPath("adjustedExerciseFeePaymentDate"), Date.class, adjustedExerciseFeePaymentDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EarlyTerminationEvent _that = (EarlyTerminationEvent) o;
	
		if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
		if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
		if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
		if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
		if (adjustedExerciseFeePaymentDate != null ? !adjustedExerciseFeePaymentDate.equals(_that.adjustedExerciseFeePaymentDate) : _that.adjustedExerciseFeePaymentDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
		_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
		_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExerciseFeePaymentDate != null ? adjustedExerciseFeePaymentDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EarlyTerminationEvent {" +
			"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
			"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
			"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate + ", " +
			"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
			"adjustedExerciseFeePaymentDate=" + this.adjustedExerciseFeePaymentDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
