package org.isda.cdm;

import org.isda.cdm.EligibleCollateralVariationMarginElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EligibleCollateralVariationMarginMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the eligible collateral elections by the respective parties to the Credit Support Annex for Variation Margin. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(ii): Eligible Collateral (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EligibleCollateralVariationMargin extends RosettaModelObject {
	private final EligibleCollateralVariationMarginElection partyElection;
	private static EligibleCollateralVariationMarginMeta metaData = new EligibleCollateralVariationMarginMeta();
	
	EligibleCollateralVariationMargin(EligibleCollateralVariationMarginBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The parties&#39; eligible collateral elections.
	 */
	public final EligibleCollateralVariationMarginElection getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends EligibleCollateralVariationMargin> metaData() {
		return metaData;
	} 
	
	public EligibleCollateralVariationMarginBuilder toBuilder() {
		EligibleCollateralVariationMarginBuilder builder = new EligibleCollateralVariationMarginBuilder();
		ofNullable(getPartyElection()).ifPresent(builder::setPartyElection);
		return builder;
	}

	public static EligibleCollateralVariationMarginBuilder builder() {
		return new EligibleCollateralVariationMarginBuilder();
	}

	public static  class EligibleCollateralVariationMarginBuilder extends RosettaModelObjectBuilder {
	
		protected EligibleCollateralVariationMarginElection.EligibleCollateralVariationMarginElectionBuilder partyElection;
	
		public EligibleCollateralVariationMarginBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EligibleCollateralVariationMargin> metaData() {
			return metaData;
		} 
	
		
		public EligibleCollateralVariationMarginElection.EligibleCollateralVariationMarginElectionBuilder getPartyElection() {
			return partyElection;
		}
		
		public EligibleCollateralVariationMarginElection.EligibleCollateralVariationMarginElectionBuilder getOrCreatePartyElection() {
			if (partyElection!=null) {
				return partyElection;
			}
			else return partyElection = new EligibleCollateralVariationMarginElection.EligibleCollateralVariationMarginElectionBuilder();
		}
		
	
		public EligibleCollateralVariationMarginBuilder setPartyElection(EligibleCollateralVariationMarginElection partyElection) {
			this.partyElection = partyElection.toBuilder();
			return this;
		}
		public EligibleCollateralVariationMarginBuilder setPartyElectionBuilder(EligibleCollateralVariationMarginElection.EligibleCollateralVariationMarginElectionBuilder partyElection) {
			this.partyElection = partyElection;
			return this;
		}
		
		public EligibleCollateralVariationMargin build() {
			return new EligibleCollateralVariationMargin(this);
		}
	
		@Override
		public EligibleCollateralVariationMarginBuilder prune() {
			if (partyElection!=null && !partyElection.prune().hasData()) partyElection = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, EligibleCollateralVariationMarginElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EligibleCollateralVariationMarginBuilder _that = (EligibleCollateralVariationMarginBuilder) o;
		
			if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EligibleCollateralVariationMarginBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, EligibleCollateralVariationMarginElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EligibleCollateralVariationMargin _that = (EligibleCollateralVariationMargin) o;
	
		if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EligibleCollateralVariationMargin {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
