package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ComputedAmount;
import org.isda.cdm.Event;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.EventTestBundleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class which combines several events for testing purposes.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class EventTestBundle extends RosettaModelObject {
	private final List<ComputedAmount> computedAmount;
	private final List<Event> event;
	private static EventTestBundleMeta metaData = new EventTestBundleMeta();
	
	EventTestBundle(EventTestBundleBuilder builder) {
		this.computedAmount = ofNullable(builder.getComputedAmount()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.event = ofNullable(builder.getEvent()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	@RosettaSynonym(value="computedAmount", source="Rosetta_Workbench")
	public final List<ComputedAmount> getComputedAmount() {
		return computedAmount;
	}
	
	@RosettaSynonym(value="event", source="Rosetta_Workbench")
	public final List<Event> getEvent() {
		return event;
	}
	
	
	@Override
	public RosettaMetaData<? extends EventTestBundle> metaData() {
		return metaData;
	} 
	
	public EventTestBundleBuilder toBuilder() {
		EventTestBundleBuilder builder = new EventTestBundleBuilder();
		ofNullable(getComputedAmount()).ifPresent(computedAmount -> computedAmount.forEach(builder::addComputedAmount));
		ofNullable(getEvent()).ifPresent(event -> event.forEach(builder::addEvent));
		return builder;
	}

	public static EventTestBundleBuilder builder() {
		return new EventTestBundleBuilder();
	}

	public static  class EventTestBundleBuilder extends RosettaModelObjectBuilder {
	
		protected List<ComputedAmount.ComputedAmountBuilder> computedAmount;
		protected List<Event.EventBuilder> event;
	
		public EventTestBundleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends EventTestBundle> metaData() {
			return metaData;
		} 
	
		
		public List<ComputedAmount.ComputedAmountBuilder> getComputedAmount() {
			return computedAmount;
		}
		
		public ComputedAmount.ComputedAmountBuilder getOrCreateComputedAmount(int index) {
			if (computedAmount==null) {
				this.computedAmount = new ArrayList<>();
			}
			return getIndex(computedAmount, index, ()->new ComputedAmount.ComputedAmountBuilder());
		}
		
		
		public List<Event.EventBuilder> getEvent() {
			return event;
		}
		
		public Event.EventBuilder getOrCreateEvent(int index) {
			if (event==null) {
				this.event = new ArrayList<>();
			}
			return getIndex(event, index, ()->new Event.EventBuilder());
		}
		
	
		public EventTestBundleBuilder addComputedAmount(ComputedAmount computedAmount) {
			if(this.computedAmount == null){
				this.computedAmount = new ArrayList<>();
			}
			this.computedAmount.add(computedAmount.toBuilder());
			return this;
		}
		public EventTestBundleBuilder addComputedAmount(ComputedAmount computedAmount, int _idx) {
			if(this.computedAmount == null){
				this.computedAmount = new ArrayList<>();
			}
			getIndex(this.computedAmount, _idx, () -> computedAmount.toBuilder());
			this.computedAmount.set(_idx, computedAmount.toBuilder());
			return this;
		}
		public EventTestBundleBuilder addComputedAmount(List<ComputedAmount> computedAmounts) {
			if(this.computedAmount == null){
				this.computedAmount = new ArrayList<>();
			}
			for (ComputedAmount toAdd : computedAmounts) {
				this.computedAmount.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventTestBundleBuilder addComputedAmountBuilder(ComputedAmount.ComputedAmountBuilder computedAmount) {
			if(this.computedAmount == null){
				this.computedAmount = new ArrayList<>();
			}
			this.computedAmount.add(computedAmount);
			return this;
		}
		
		public EventTestBundleBuilder clearComputedAmount() {
			this.computedAmount = null;
			return this;
		}
		public EventTestBundleBuilder addEvent(Event event) {
			if(this.event == null){
				this.event = new ArrayList<>();
			}
			this.event.add(event.toBuilder());
			return this;
		}
		public EventTestBundleBuilder addEvent(Event event, int _idx) {
			if(this.event == null){
				this.event = new ArrayList<>();
			}
			getIndex(this.event, _idx, () -> event.toBuilder());
			this.event.set(_idx, event.toBuilder());
			return this;
		}
		public EventTestBundleBuilder addEvent(List<Event> events) {
			if(this.event == null){
				this.event = new ArrayList<>();
			}
			for (Event toAdd : events) {
				this.event.add(toAdd.toBuilder());
			}
			return this;
		}
		public EventTestBundleBuilder addEventBuilder(Event.EventBuilder event) {
			if(this.event == null){
				this.event = new ArrayList<>();
			}
			this.event.add(event);
			return this;
		}
		
		public EventTestBundleBuilder clearEvent() {
			this.event = null;
			return this;
		}
		
		public EventTestBundle build() {
			return new EventTestBundle(this);
		}
	
		@Override
		public EventTestBundleBuilder prune() {
			if (computedAmount!=null) computedAmount = computedAmount.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (event!=null) event = event.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getComputedAmount()!=null && getComputedAmount().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEvent()!=null && getEvent().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("computedAmount"), processor, ComputedAmount.class, computedAmount);
			processRosetta(path.newSubPath("event"), processor, Event.class, event);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			EventTestBundleBuilder _that = (EventTestBundleBuilder) o;
		
			if (!ListEquals.listEquals(computedAmount, _that.computedAmount)) return false;
			if (!ListEquals.listEquals(event, _that.event)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (computedAmount != null ? computedAmount.hashCode() : 0);
			_result = 31 * _result + (event != null ? event.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "EventTestBundleBuilder {" +
				"computedAmount=" + this.computedAmount + ", " +
				"event=" + this.event +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("computedAmount"), processor, ComputedAmount.class, computedAmount);
		processRosetta(path.newSubPath("event"), processor, Event.class, event);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		EventTestBundle _that = (EventTestBundle) o;
	
		if (!ListEquals.listEquals(computedAmount, _that.computedAmount)) return false;
		if (!ListEquals.listEquals(event, _that.event)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (computedAmount != null ? computedAmount.hashCode() : 0);
		_result = 31 * _result + (event != null ? event.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "EventTestBundle {" +
			"computedAmount=" + this.computedAmount + ", " +
			"event=" + this.event +
		'}';
	}
}
