package org.isda.cdm;

import org.isda.cdm.Prsn;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExctgPrsnMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class ExctgPrsn extends RosettaModelObject {
	private final Prsn prsn;
	private static ExctgPrsnMeta metaData = new ExctgPrsnMeta();
	
	ExctgPrsn(ExctgPrsnBuilder builder) {
		this.prsn = ofNullable(builder.getPrsn()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Prsn", source="ISO20022")
	public final Prsn getPrsn() {
		return prsn;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExctgPrsn> metaData() {
		return metaData;
	} 
	
	public ExctgPrsnBuilder toBuilder() {
		ExctgPrsnBuilder builder = new ExctgPrsnBuilder();
		ofNullable(getPrsn()).ifPresent(builder::setPrsn);
		return builder;
	}

	public static ExctgPrsnBuilder builder() {
		return new ExctgPrsnBuilder();
	}

	public static  class ExctgPrsnBuilder extends RosettaModelObjectBuilder {
	
		protected Prsn.PrsnBuilder prsn;
	
		public ExctgPrsnBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExctgPrsn> metaData() {
			return metaData;
		} 
	
		
		public Prsn.PrsnBuilder getPrsn() {
			return prsn;
		}
		
		public Prsn.PrsnBuilder getOrCreatePrsn() {
			if (prsn!=null) {
				return prsn;
			}
			else return prsn = new Prsn.PrsnBuilder();
		}
		
	
		public ExctgPrsnBuilder setPrsn(Prsn prsn) {
			this.prsn = prsn.toBuilder();
			return this;
		}
		public ExctgPrsnBuilder setPrsnBuilder(Prsn.PrsnBuilder prsn) {
			this.prsn = prsn;
			return this;
		}
		
		public ExctgPrsn build() {
			return new ExctgPrsn(this);
		}
	
		@Override
		public ExctgPrsnBuilder prune() {
			if (prsn!=null && !prsn.prune().hasData()) prsn = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrsn()!=null && getPrsn().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("prsn"), processor, Prsn.class, prsn);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExctgPrsnBuilder _that = (ExctgPrsnBuilder) o;
		
			if (prsn != null ? !prsn.equals(_that.prsn) : _that.prsn != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (prsn != null ? prsn.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExctgPrsnBuilder {" +
				"prsn=" + this.prsn +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("prsn"), processor, Prsn.class, prsn);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExctgPrsn _that = (ExctgPrsn) o;
	
		if (prsn != null ? !prsn.equals(_that.prsn) : _that.prsn != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (prsn != null ? prsn.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExctgPrsn {" +
			"prsn=" + this.prsn +
		'}';
	}
}
