package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.Address;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExecutingEntityMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class ExecutingEntity extends RosettaModelObject {
	private final Address addressOfBranch;
	private final Address addressOfIncorporation;
	private final Boolean isInvestmentFirm;
	private static ExecutingEntityMeta metaData = new ExecutingEntityMeta();
	
	ExecutingEntity(ExecutingEntityBuilder builder) {
		this.addressOfBranch = ofNullable(builder.getAddressOfBranch()).map(f->f.build()).orElse(null);
		this.addressOfIncorporation = ofNullable(builder.getAddressOfIncorporation()).map(f->f.build()).orElse(null);
		this.isInvestmentFirm = builder.getIsInvestmentFirm();
	}
	
	public final Address getAddressOfBranch() {
		return addressOfBranch;
	}
	
	public final Address getAddressOfIncorporation() {
		return addressOfIncorporation;
	}
	
	public final Boolean getIsInvestmentFirm() {
		return isInvestmentFirm;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExecutingEntity> metaData() {
		return metaData;
	} 
	
	public ExecutingEntityBuilder toBuilder() {
		ExecutingEntityBuilder builder = new ExecutingEntityBuilder();
		ofNullable(getAddressOfBranch()).ifPresent(builder::setAddressOfBranch);
		ofNullable(getAddressOfIncorporation()).ifPresent(builder::setAddressOfIncorporation);
		ofNullable(getIsInvestmentFirm()).ifPresent(builder::setIsInvestmentFirm);
		return builder;
	}

	public static ExecutingEntityBuilder builder() {
		return new ExecutingEntityBuilder();
	}

	public static  class ExecutingEntityBuilder extends RosettaModelObjectBuilder {
	
		protected Address.AddressBuilder addressOfBranch;
		protected Address.AddressBuilder addressOfIncorporation;
		protected Boolean isInvestmentFirm;
	
		public ExecutingEntityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExecutingEntity> metaData() {
			return metaData;
		} 
	
		
		public Address.AddressBuilder getAddressOfBranch() {
			return addressOfBranch;
		}
		
		public Address.AddressBuilder getOrCreateAddressOfBranch() {
			if (addressOfBranch!=null) {
				return addressOfBranch;
			}
			else return addressOfBranch = new Address.AddressBuilder();
		}
		
		
		public Address.AddressBuilder getAddressOfIncorporation() {
			return addressOfIncorporation;
		}
		
		public Address.AddressBuilder getOrCreateAddressOfIncorporation() {
			if (addressOfIncorporation!=null) {
				return addressOfIncorporation;
			}
			else return addressOfIncorporation = new Address.AddressBuilder();
		}
		
		
		public Boolean getIsInvestmentFirm() {
			return isInvestmentFirm;
		}
		
	
		public ExecutingEntityBuilder setAddressOfBranch(Address addressOfBranch) {
			this.addressOfBranch = addressOfBranch.toBuilder();
			return this;
		}
		public ExecutingEntityBuilder setAddressOfBranchBuilder(Address.AddressBuilder addressOfBranch) {
			this.addressOfBranch = addressOfBranch;
			return this;
		}
		public ExecutingEntityBuilder setAddressOfIncorporation(Address addressOfIncorporation) {
			this.addressOfIncorporation = addressOfIncorporation.toBuilder();
			return this;
		}
		public ExecutingEntityBuilder setAddressOfIncorporationBuilder(Address.AddressBuilder addressOfIncorporation) {
			this.addressOfIncorporation = addressOfIncorporation;
			return this;
		}
		public ExecutingEntityBuilder setIsInvestmentFirm(Boolean isInvestmentFirm) {
			this.isInvestmentFirm = isInvestmentFirm;
			return this;
		}
		
		public ExecutingEntity build() {
			return new ExecutingEntity(this);
		}
	
		@Override
		public ExecutingEntityBuilder prune() {
			if (addressOfBranch!=null && !addressOfBranch.prune().hasData()) addressOfBranch = null;
			if (addressOfIncorporation!=null && !addressOfIncorporation.prune().hasData()) addressOfIncorporation = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAddressOfBranch()!=null && getAddressOfBranch().hasData()) return true;
			if (getAddressOfIncorporation()!=null && getAddressOfIncorporation().hasData()) return true;
			if (getIsInvestmentFirm()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isInvestmentFirm"), Boolean.class, isInvestmentFirm, this);
			
			processRosetta(path.newSubPath("addressOfBranch"), processor, Address.class, addressOfBranch);
			processRosetta(path.newSubPath("addressOfIncorporation"), processor, Address.class, addressOfIncorporation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExecutingEntityBuilder _that = (ExecutingEntityBuilder) o;
		
			if (addressOfBranch != null ? !addressOfBranch.equals(_that.addressOfBranch) : _that.addressOfBranch != null) return false;
			if (addressOfIncorporation != null ? !addressOfIncorporation.equals(_that.addressOfIncorporation) : _that.addressOfIncorporation != null) return false;
			if (isInvestmentFirm != null ? !isInvestmentFirm.equals(_that.isInvestmentFirm) : _that.isInvestmentFirm != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (addressOfBranch != null ? addressOfBranch.hashCode() : 0);
			_result = 31 * _result + (addressOfIncorporation != null ? addressOfIncorporation.hashCode() : 0);
			_result = 31 * _result + (isInvestmentFirm != null ? isInvestmentFirm.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExecutingEntityBuilder {" +
				"addressOfBranch=" + this.addressOfBranch + ", " +
				"addressOfIncorporation=" + this.addressOfIncorporation + ", " +
				"isInvestmentFirm=" + this.isInvestmentFirm +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isInvestmentFirm"), Boolean.class, isInvestmentFirm, this);
		
		processRosetta(path.newSubPath("addressOfBranch"), processor, Address.class, addressOfBranch);
		processRosetta(path.newSubPath("addressOfIncorporation"), processor, Address.class, addressOfIncorporation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExecutingEntity _that = (ExecutingEntity) o;
	
		if (addressOfBranch != null ? !addressOfBranch.equals(_that.addressOfBranch) : _that.addressOfBranch != null) return false;
		if (addressOfIncorporation != null ? !addressOfIncorporation.equals(_that.addressOfIncorporation) : _that.addressOfIncorporation != null) return false;
		if (isInvestmentFirm != null ? !isInvestmentFirm.equals(_that.isInvestmentFirm) : _that.isInvestmentFirm != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (addressOfBranch != null ? addressOfBranch.hashCode() : 0);
		_result = 31 * _result + (addressOfIncorporation != null ? addressOfIncorporation.hashCode() : 0);
		_result = 31 * _result + (isInvestmentFirm != null ? isInvestmentFirm.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExecutingEntity {" +
			"addressOfBranch=" + this.addressOfBranch + ", " +
			"addressOfIncorporation=" + this.addressOfIncorporation + ", " +
			"isInvestmentFirm=" + this.isInvestmentFirm +
		'}';
	}
}
