package org.isda.cdm;

import org.isda.cdm.Execution;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExecutionStateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify an execution instantiation with respect to the before and/or after state of lifecycle events.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExecutionState extends RosettaModelObject {
	private final Execution execution;
	private static ExecutionStateMeta metaData = new ExecutionStateMeta();
	
	ExecutionState(ExecutionStateBuilder builder) {
		this.execution = ofNullable(builder.getExecution()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="execution", source="Rosetta_Workbench")
	public final Execution getExecution() {
		return execution;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExecutionState> metaData() {
		return metaData;
	} 
	
	public ExecutionStateBuilder toBuilder() {
		ExecutionStateBuilder builder = new ExecutionStateBuilder();
		ofNullable(getExecution()).ifPresent(builder::setExecution);
		return builder;
	}

	public static ExecutionStateBuilder builder() {
		return new ExecutionStateBuilder();
	}

	public static  class ExecutionStateBuilder extends RosettaModelObjectBuilder {
	
		protected Execution.ExecutionBuilder execution;
	
		public ExecutionStateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExecutionState> metaData() {
			return metaData;
		} 
	
		
		public Execution.ExecutionBuilder getExecution() {
			return execution;
		}
		
		public Execution.ExecutionBuilder getOrCreateExecution() {
			if (execution!=null) {
				return execution;
			}
			else return execution = new Execution.ExecutionBuilder();
		}
		
	
		public ExecutionStateBuilder setExecution(Execution execution) {
			this.execution = execution.toBuilder();
			return this;
		}
		public ExecutionStateBuilder setExecutionBuilder(Execution.ExecutionBuilder execution) {
			this.execution = execution;
			return this;
		}
		
		public ExecutionState build() {
			return new ExecutionState(this);
		}
	
		@Override
		public ExecutionStateBuilder prune() {
			if (execution!=null && !execution.prune().hasData()) execution = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExecution()!=null && getExecution().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("execution"), processor, Execution.class, execution);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExecutionStateBuilder _that = (ExecutionStateBuilder) o;
		
			if (execution != null ? !execution.equals(_that.execution) : _that.execution != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExecutionStateBuilder {" +
				"execution=" + this.execution +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("execution"), processor, Execution.class, execution);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExecutionState _that = (ExecutionState) o;
	
		if (execution != null ? !execution.equals(_that.execution) : _that.execution != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExecutionState {" +
			"execution=" + this.execution +
		'}';
	}
}
