package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify how a contract has been executed, e.g. electronically, verbally, ...
 * 
 * @version 2.5.4
 */
public enum ExecutionTypeEnum {
    

	/**
	 * Execution via electronic execution facility, derivatives contract market, or other electronic message such as an instant message.
	 */
	@RosettaSynonym(value = "Electronic", source = "FpML_5_10")
	@RosettaSynonym(value = "Electronic", source = "DTCC_11_0")
	@RosettaSynonym(value = "Electronic", source = "DTCC_9_0")
	@RosettaSynonym(value = "Electronic", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "Electronic", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "E", source = "CME_SubmissionIRS_1_0")
	ELECTRONIC,
	    

	/**
	 * Bilateral execution between counterparties not pursuant to the rules of a SEF or DCM.
	 */
	@RosettaSynonym(value = "OffFacility", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "O", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Voice", source = "FpML_5_10")
	@RosettaSynonym(value = "Voice", source = "DTCC_11_0")
	@RosettaSynonym(value = "Voice", source = "DTCC_9_0")
	@RosettaSynonym(value = "Voice", source = "CME_ClearedConfirm_1_17")
	OFF_FACILITY    
	;

	private final String displayName;
	
	ExecutionTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
