package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the adjusted dates associated with a particular exercise event.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExerciseEvent extends RosettaModelObject implements GlobalKey {
	private final Date adjustedCashSettlementPaymentDate;
	private final Date adjustedCashSettlementValuationDate;
	private final Date adjustedExerciseDate;
	private final Date adjustedExerciseFeePaymentDate;
	private final Date adjustedRelevantSwapEffectiveDate;
	private final MetaFields meta;
	private static ExerciseEventMeta metaData = new ExerciseEventMeta();
	
	ExerciseEvent(ExerciseEventBuilder builder) {
		this.adjustedCashSettlementPaymentDate = builder.getAdjustedCashSettlementPaymentDate();
		this.adjustedCashSettlementValuationDate = builder.getAdjustedCashSettlementValuationDate();
		this.adjustedExerciseDate = builder.getAdjustedExerciseDate();
		this.adjustedExerciseFeePaymentDate = builder.getAdjustedExerciseFeePaymentDate();
		this.adjustedRelevantSwapEffectiveDate = builder.getAdjustedRelevantSwapEffectiveDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the cash settlement amount is paid. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementPaymentDate() {
		return adjustedCashSettlementPaymentDate;
	}
	
	/**
	 * The date by which the cash settlement amount must be agreed. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementValuationDate() {
		return adjustedCashSettlementValuationDate;
	}
	
	/**
	 * The date on which the option exercise takes place. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseDate() {
		return adjustedExerciseDate;
	}
	
	/**
	 * The date on which the exercise fee amount is paid. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseFeePaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseFeePaymentDate() {
		return adjustedExerciseFeePaymentDate;
	}
	
	/**
	 * The effective date of the underlying swap associated with a given exercise date. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedRelevantSwapEffectiveDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedRelevantSwapEffectiveDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedRelevantSwapEffectiveDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedRelevantSwapEffectiveDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedRelevantSwapEffectiveDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedRelevantSwapEffectiveDate() {
		return adjustedRelevantSwapEffectiveDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseEvent> metaData() {
		return metaData;
	} 
	
	public ExerciseEventBuilder toBuilder() {
		ExerciseEventBuilder builder = new ExerciseEventBuilder();
		ofNullable(getAdjustedCashSettlementPaymentDate()).ifPresent(builder::setAdjustedCashSettlementPaymentDate);
		ofNullable(getAdjustedCashSettlementValuationDate()).ifPresent(builder::setAdjustedCashSettlementValuationDate);
		ofNullable(getAdjustedExerciseDate()).ifPresent(builder::setAdjustedExerciseDate);
		ofNullable(getAdjustedExerciseFeePaymentDate()).ifPresent(builder::setAdjustedExerciseFeePaymentDate);
		ofNullable(getAdjustedRelevantSwapEffectiveDate()).ifPresent(builder::setAdjustedRelevantSwapEffectiveDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static ExerciseEventBuilder builder() {
		return new ExerciseEventBuilder();
	}

	public static  class ExerciseEventBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ExerciseEventBuilder> {
	
		protected Date adjustedCashSettlementPaymentDate;
		protected Date adjustedCashSettlementValuationDate;
		protected Date adjustedExerciseDate;
		protected Date adjustedExerciseFeePaymentDate;
		protected Date adjustedRelevantSwapEffectiveDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public ExerciseEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseEvent> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedCashSettlementPaymentDate() {
			return adjustedCashSettlementPaymentDate;
		}
		
		
		public Date getAdjustedCashSettlementValuationDate() {
			return adjustedCashSettlementValuationDate;
		}
		
		
		public Date getAdjustedExerciseDate() {
			return adjustedExerciseDate;
		}
		
		
		public Date getAdjustedExerciseFeePaymentDate() {
			return adjustedExerciseFeePaymentDate;
		}
		
		
		public Date getAdjustedRelevantSwapEffectiveDate() {
			return adjustedRelevantSwapEffectiveDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public ExerciseEventBuilder setAdjustedCashSettlementPaymentDate(Date adjustedCashSettlementPaymentDate) {
			this.adjustedCashSettlementPaymentDate = adjustedCashSettlementPaymentDate;
			return this;
		}
		public ExerciseEventBuilder setAdjustedCashSettlementValuationDate(Date adjustedCashSettlementValuationDate) {
			this.adjustedCashSettlementValuationDate = adjustedCashSettlementValuationDate;
			return this;
		}
		public ExerciseEventBuilder setAdjustedExerciseDate(Date adjustedExerciseDate) {
			this.adjustedExerciseDate = adjustedExerciseDate;
			return this;
		}
		public ExerciseEventBuilder setAdjustedExerciseFeePaymentDate(Date adjustedExerciseFeePaymentDate) {
			this.adjustedExerciseFeePaymentDate = adjustedExerciseFeePaymentDate;
			return this;
		}
		public ExerciseEventBuilder setAdjustedRelevantSwapEffectiveDate(Date adjustedRelevantSwapEffectiveDate) {
			this.adjustedRelevantSwapEffectiveDate = adjustedRelevantSwapEffectiveDate;
			return this;
		}
		public ExerciseEventBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ExerciseEventBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public ExerciseEvent build() {
			return new ExerciseEvent(this);
		}
	
		@Override
		public ExerciseEventBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedCashSettlementPaymentDate()!=null) return true;
			if (getAdjustedCashSettlementValuationDate()!=null) return true;
			if (getAdjustedExerciseDate()!=null) return true;
			if (getAdjustedExerciseFeePaymentDate()!=null) return true;
			if (getAdjustedRelevantSwapEffectiveDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
			processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
			processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
			processor.processBasic(path.newSubPath("adjustedExerciseFeePaymentDate"), Date.class, adjustedExerciseFeePaymentDate, this);
			processor.processBasic(path.newSubPath("adjustedRelevantSwapEffectiveDate"), Date.class, adjustedRelevantSwapEffectiveDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExerciseEventBuilder _that = (ExerciseEventBuilder) o;
		
			if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
			if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
			if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
			if (adjustedExerciseFeePaymentDate != null ? !adjustedExerciseFeePaymentDate.equals(_that.adjustedExerciseFeePaymentDate) : _that.adjustedExerciseFeePaymentDate != null) return false;
			if (adjustedRelevantSwapEffectiveDate != null ? !adjustedRelevantSwapEffectiveDate.equals(_that.adjustedRelevantSwapEffectiveDate) : _that.adjustedRelevantSwapEffectiveDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
			_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExerciseFeePaymentDate != null ? adjustedExerciseFeePaymentDate.hashCode() : 0);
			_result = 31 * _result + (adjustedRelevantSwapEffectiveDate != null ? adjustedRelevantSwapEffectiveDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseEventBuilder {" +
				"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
				"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
				"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
				"adjustedExerciseFeePaymentDate=" + this.adjustedExerciseFeePaymentDate + ", " +
				"adjustedRelevantSwapEffectiveDate=" + this.adjustedRelevantSwapEffectiveDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
		processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
		processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
		processor.processBasic(path.newSubPath("adjustedExerciseFeePaymentDate"), Date.class, adjustedExerciseFeePaymentDate, this);
		processor.processBasic(path.newSubPath("adjustedRelevantSwapEffectiveDate"), Date.class, adjustedRelevantSwapEffectiveDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExerciseEvent _that = (ExerciseEvent) o;
	
		if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
		if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
		if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
		if (adjustedExerciseFeePaymentDate != null ? !adjustedExerciseFeePaymentDate.equals(_that.adjustedExerciseFeePaymentDate) : _that.adjustedExerciseFeePaymentDate != null) return false;
		if (adjustedRelevantSwapEffectiveDate != null ? !adjustedRelevantSwapEffectiveDate.equals(_that.adjustedRelevantSwapEffectiveDate) : _that.adjustedRelevantSwapEffectiveDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
		_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExerciseFeePaymentDate != null ? adjustedExerciseFeePaymentDate.hashCode() : 0);
		_result = 31 * _result + (adjustedRelevantSwapEffectiveDate != null ? adjustedRelevantSwapEffectiveDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseEvent {" +
			"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
			"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
			"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
			"adjustedExerciseFeePaymentDate=" + this.adjustedExerciseFeePaymentDate + ", " +
			"adjustedRelevantSwapEffectiveDate=" + this.adjustedRelevantSwapEffectiveDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
