package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum;
import org.isda.cdm.metafields.FieldWithMetaBusinessCenterEnum.FieldWithMetaBusinessCenterEnumBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseNoticeMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining to whom and where notice of execution should be given. The partyReference refers to one of the principal parties of the trade. If present the exerciseNoticePartyReference refers to a party, other than the principal party, to whom notice should be given.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExerciseNotice", source="FpML_5_10")
public class ExerciseNotice extends RosettaModelObject {
	private final FieldWithMetaBusinessCenterEnum businessCenter;
	private final ReferenceWithMetaParty exerciseNoticePartyReference;
	private final ReferenceWithMetaParty partyReference;
	private static ExerciseNoticeMeta metaData = new ExerciseNoticeMeta();
	
	ExerciseNotice(ExerciseNoticeBuilder builder) {
		this.businessCenter = ofNullable(builder.getBusinessCenter()).map(f->f.build()).orElse(null);
		this.exerciseNoticePartyReference = ofNullable(builder.getExerciseNoticePartyReference()).map(f->f.build()).orElse(null);
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The business center.
	 */
	@RosettaSynonym(value="businessCenter", source="FpML_5_10")
	@RosettaSynonym(value="businessCenter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_11_0")
	@RosettaSynonym(value="businessCenter", source="DTCC_9_0")
	@RosettaSynonym(value="businessCenter", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaBusinessCenterEnum getBusinessCenter() {
		return businessCenter;
	}
	
	/**
	 * The party referenced is the party to which notice of exercise should be given by the buyer.
	 */
	@RosettaSynonym(value="exerciseNoticePartyReference", source="FpML_5_10")
	@RosettaSynonym(value="exerciseNoticePartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseNoticePartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseNoticePartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseNoticePartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getExerciseNoticePartyReference() {
		return exerciseNoticePartyReference;
	}
	
	/**
	 * The party referenced has allocated the trade identifier.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0")
	@RosettaSynonym(value="partyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseNotice> metaData() {
		return metaData;
	} 
	
	public ExerciseNoticeBuilder toBuilder() {
		ExerciseNoticeBuilder builder = new ExerciseNoticeBuilder();
		ofNullable(getBusinessCenter()).ifPresent(builder::setBusinessCenter);
		ofNullable(getExerciseNoticePartyReference()).ifPresent(builder::setExerciseNoticePartyReference);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		return builder;
	}

	public static ExerciseNoticeBuilder builder() {
		return new ExerciseNoticeBuilder();
	}

	public static  class ExerciseNoticeBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaBusinessCenterEnumBuilder businessCenter;
		protected ReferenceWithMetaPartyBuilder exerciseNoticePartyReference;
		protected ReferenceWithMetaPartyBuilder partyReference;
	
		public ExerciseNoticeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseNotice> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaBusinessCenterEnumBuilder getBusinessCenter() {
			return businessCenter;
		}
		
		public FieldWithMetaBusinessCenterEnumBuilder getOrCreateBusinessCenter() {
			if (businessCenter!=null) {
				return businessCenter;
			}
			else return businessCenter = new FieldWithMetaBusinessCenterEnumBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getExerciseNoticePartyReference() {
			return exerciseNoticePartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateExerciseNoticePartyReference() {
			if (exerciseNoticePartyReference!=null) {
				return exerciseNoticePartyReference;
			}
			else return exerciseNoticePartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public ExerciseNoticeBuilder setBusinessCenter(FieldWithMetaBusinessCenterEnum businessCenter) {
			this.businessCenter = businessCenter.toBuilder();
			return this;
		}
		public ExerciseNoticeBuilder setBusinessCenterRef(BusinessCenterEnum businessCenter) {
			return setBusinessCenter(FieldWithMetaBusinessCenterEnum.builder().setValue(businessCenter).build());
		}
		public ExerciseNoticeBuilder setExerciseNoticePartyReference(ReferenceWithMetaParty exerciseNoticePartyReference) {
			this.exerciseNoticePartyReference = exerciseNoticePartyReference.toBuilder();
			return this;
		}
		public ExerciseNoticeBuilder setExerciseNoticePartyReferenceBuilder(ReferenceWithMetaPartyBuilder exerciseNoticePartyReference) {
			this.exerciseNoticePartyReference = exerciseNoticePartyReference;
			return this;
		}
		public ExerciseNoticeBuilder setExerciseNoticePartyReferenceRef(Party.PartyBuilder exerciseNoticePartyReference) {
			return setExerciseNoticePartyReference(ReferenceWithMetaParty.builder().setValueBuilder(exerciseNoticePartyReference).build());
		}
		public ExerciseNoticeBuilder setExerciseNoticePartyReferenceRef(Party exerciseNoticePartyReference) {
			return setExerciseNoticePartyReferenceRef(Party.builder());
		}
		public ExerciseNoticeBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public ExerciseNoticeBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public ExerciseNoticeBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public ExerciseNoticeBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		
		public ExerciseNotice build() {
			return new ExerciseNotice(this);
		}
	
		@Override
		public ExerciseNoticeBuilder prune() {
			if (businessCenter!=null && !businessCenter.prune().hasData()) businessCenter = null;
			if (exerciseNoticePartyReference!=null && !exerciseNoticePartyReference.prune().hasData()) exerciseNoticePartyReference = null;
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessCenter()!=null) return true;
			if (getExerciseNoticePartyReference()!=null && getExerciseNoticePartyReference().hasData()) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
			processRosetta(path.newSubPath("exerciseNoticePartyReference"), processor, ReferenceWithMetaParty.class, exerciseNoticePartyReference);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExerciseNoticeBuilder _that = (ExerciseNoticeBuilder) o;
		
			if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
			if (exerciseNoticePartyReference != null ? !exerciseNoticePartyReference.equals(_that.exerciseNoticePartyReference) : _that.exerciseNoticePartyReference != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (exerciseNoticePartyReference != null ? exerciseNoticePartyReference.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseNoticeBuilder {" +
				"businessCenter=" + this.businessCenter + ", " +
				"exerciseNoticePartyReference=" + this.exerciseNoticePartyReference + ", " +
				"partyReference=" + this.partyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("businessCenter"), processor, FieldWithMetaBusinessCenterEnum.class, businessCenter);
		processRosetta(path.newSubPath("exerciseNoticePartyReference"), processor, ReferenceWithMetaParty.class, exerciseNoticePartyReference);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExerciseNotice _that = (ExerciseNotice) o;
	
		if (businessCenter != null ? !businessCenter.equals(_that.businessCenter) : _that.businessCenter != null) return false;
		if (exerciseNoticePartyReference != null ? !exerciseNoticePartyReference.equals(_that.exerciseNoticePartyReference) : _that.exerciseNoticePartyReference != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessCenter != null ? businessCenter.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (exerciseNoticePartyReference != null ? exerciseNoticePartyReference.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseNotice {" +
			"businessCenter=" + this.businessCenter + ", " +
			"exerciseNoticePartyReference=" + this.exerciseNoticePartyReference + ", " +
			"partyReference=" + this.partyReference +
		'}';
	}
}
