package org.isda.cdm;

import org.isda.cdm.Cashflow;
import org.isda.cdm.Contract;
import org.isda.cdm.PhysicalExercise;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExerciseOutcomeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The exercise outcome combines the option contract (which states would be &#39;Exercised&#39; in case of a full exercise and which would have a reduced notional in case of partial exercise) and either a physical or partial exercise.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExerciseOutcome extends RosettaModelObject {
	private final Cashflow cashExercise;
	private final Contract contract;
	private final PhysicalExercise physicalExercise;
	private static ExerciseOutcomeMeta metaData = new ExerciseOutcomeMeta();
	
	ExerciseOutcome(ExerciseOutcomeBuilder builder) {
		this.cashExercise = ofNullable(builder.getCashExercise()).map(f->f.build()).orElse(null);
		this.contract = ofNullable(builder.getContract()).map(f->f.build()).orElse(null);
		this.physicalExercise = ofNullable(builder.getPhysicalExercise()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="cashExercise", source="Rosetta_Workbench")
	public final Cashflow getCashExercise() {
		return cashExercise;
	}
	
	@RosettaSynonym(value="contract", source="Rosetta_Workbench")
	public final Contract getContract() {
		return contract;
	}
	
	@RosettaSynonym(value="physicalExercise", source="Rosetta_Workbench")
	public final PhysicalExercise getPhysicalExercise() {
		return physicalExercise;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExerciseOutcome> metaData() {
		return metaData;
	} 
	
	public ExerciseOutcomeBuilder toBuilder() {
		ExerciseOutcomeBuilder builder = new ExerciseOutcomeBuilder();
		ofNullable(getCashExercise()).ifPresent(builder::setCashExercise);
		ofNullable(getContract()).ifPresent(builder::setContract);
		ofNullable(getPhysicalExercise()).ifPresent(builder::setPhysicalExercise);
		return builder;
	}

	public static ExerciseOutcomeBuilder builder() {
		return new ExerciseOutcomeBuilder();
	}

	public static  class ExerciseOutcomeBuilder extends RosettaModelObjectBuilder {
	
		protected Cashflow.CashflowBuilder cashExercise;
		protected Contract.ContractBuilder contract;
		protected PhysicalExercise.PhysicalExerciseBuilder physicalExercise;
	
		public ExerciseOutcomeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExerciseOutcome> metaData() {
			return metaData;
		} 
	
		
		public Cashflow.CashflowBuilder getCashExercise() {
			return cashExercise;
		}
		
		public Cashflow.CashflowBuilder getOrCreateCashExercise() {
			if (cashExercise!=null) {
				return cashExercise;
			}
			else return cashExercise = new Cashflow.CashflowBuilder();
		}
		
		
		public Contract.ContractBuilder getContract() {
			return contract;
		}
		
		public Contract.ContractBuilder getOrCreateContract() {
			if (contract!=null) {
				return contract;
			}
			else return contract = new Contract.ContractBuilder();
		}
		
		
		public PhysicalExercise.PhysicalExerciseBuilder getPhysicalExercise() {
			return physicalExercise;
		}
		
		public PhysicalExercise.PhysicalExerciseBuilder getOrCreatePhysicalExercise() {
			if (physicalExercise!=null) {
				return physicalExercise;
			}
			else return physicalExercise = new PhysicalExercise.PhysicalExerciseBuilder();
		}
		
	
		public ExerciseOutcomeBuilder setCashExercise(Cashflow cashExercise) {
			this.cashExercise = cashExercise.toBuilder();
			return this;
		}
		public ExerciseOutcomeBuilder setCashExerciseBuilder(Cashflow.CashflowBuilder cashExercise) {
			this.cashExercise = cashExercise;
			return this;
		}
		public ExerciseOutcomeBuilder setContract(Contract contract) {
			this.contract = contract.toBuilder();
			return this;
		}
		public ExerciseOutcomeBuilder setContractBuilder(Contract.ContractBuilder contract) {
			this.contract = contract;
			return this;
		}
		public ExerciseOutcomeBuilder setPhysicalExercise(PhysicalExercise physicalExercise) {
			this.physicalExercise = physicalExercise.toBuilder();
			return this;
		}
		public ExerciseOutcomeBuilder setPhysicalExerciseBuilder(PhysicalExercise.PhysicalExerciseBuilder physicalExercise) {
			this.physicalExercise = physicalExercise;
			return this;
		}
		
		public ExerciseOutcome build() {
			return new ExerciseOutcome(this);
		}
	
		@Override
		public ExerciseOutcomeBuilder prune() {
			if (cashExercise!=null && !cashExercise.prune().hasData()) cashExercise = null;
			if (contract!=null && !contract.prune().hasData()) contract = null;
			if (physicalExercise!=null && !physicalExercise.prune().hasData()) physicalExercise = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashExercise()!=null && getCashExercise().hasData()) return true;
			if (getContract()!=null && getContract().hasData()) return true;
			if (getPhysicalExercise()!=null && getPhysicalExercise().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashExercise"), processor, Cashflow.class, cashExercise);
			processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
			processRosetta(path.newSubPath("physicalExercise"), processor, PhysicalExercise.class, physicalExercise);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExerciseOutcomeBuilder _that = (ExerciseOutcomeBuilder) o;
		
			if (cashExercise != null ? !cashExercise.equals(_that.cashExercise) : _that.cashExercise != null) return false;
			if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
			if (physicalExercise != null ? !physicalExercise.equals(_that.physicalExercise) : _that.physicalExercise != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashExercise != null ? cashExercise.hashCode() : 0);
			_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
			_result = 31 * _result + (physicalExercise != null ? physicalExercise.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExerciseOutcomeBuilder {" +
				"cashExercise=" + this.cashExercise + ", " +
				"contract=" + this.contract + ", " +
				"physicalExercise=" + this.physicalExercise +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashExercise"), processor, Cashflow.class, cashExercise);
		processRosetta(path.newSubPath("contract"), processor, Contract.class, contract);
		processRosetta(path.newSubPath("physicalExercise"), processor, PhysicalExercise.class, physicalExercise);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExerciseOutcome _that = (ExerciseOutcome) o;
	
		if (cashExercise != null ? !cashExercise.equals(_that.cashExercise) : _that.cashExercise != null) return false;
		if (contract != null ? !contract.equals(_that.contract) : _that.contract != null) return false;
		if (physicalExercise != null ? !physicalExercise.equals(_that.physicalExercise) : _that.physicalExercise != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashExercise != null ? cashExercise.hashCode() : 0);
		_result = 31 * _result + (contract != null ? contract.hashCode() : 0);
		_result = 31 * _result + (physicalExercise != null ? physicalExercise.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExerciseOutcome {" +
			"cashExercise=" + this.cashExercise + ", " +
			"contract=" + this.contract + ", " +
			"physicalExercise=" + this.physicalExercise +
		'}';
	}
}
