package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AdjustableOrAdjustedDate;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.ContractState;
import org.isda.cdm.ExerciseOutcome;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExercisePrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * This primitive leverages the FpML OptionExercise construct, except for the exerciseTiming which is deemed as associated to a request for exercise that is meant to take place, as opposed to the actual exercise event.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ExercisePrimitive extends RosettaModelObject {
	private final ExerciseOutcome after;
	private final ContractState before;
	private final AdjustableOrAdjustedDate exerciseDate;
	private final BusinessCenterTime exerciseTime;
	private final Boolean fullExercise;
	private static ExercisePrimitiveMeta metaData = new ExercisePrimitiveMeta();
	
	ExercisePrimitive(ExercisePrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
		this.exerciseDate = ofNullable(builder.getExerciseDate()).map(f->f.build()).orElse(null);
		this.exerciseTime = ofNullable(builder.getExerciseTime()).map(f->f.build()).orElse(null);
		this.fullExercise = builder.getFullExercise();
	}
	
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final ExerciseOutcome getAfter() {
		return after;
	}
	
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	public final ContractState getBefore() {
		return before;
	}
	
	@RosettaSynonym(value="exerciseDate", source="Rosetta_Workbench")
	public final AdjustableOrAdjustedDate getExerciseDate() {
		return exerciseDate;
	}
	
	@RosettaSynonym(value="exerciseTime", source="Rosetta_Workbench")
	public final BusinessCenterTime getExerciseTime() {
		return exerciseTime;
	}
	
	@RosettaSynonym(value="fullExercise", source="Rosetta_Workbench")
	public final Boolean getFullExercise() {
		return fullExercise;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExercisePrimitive> metaData() {
		return metaData;
	} 
	
	public ExercisePrimitiveBuilder toBuilder() {
		ExercisePrimitiveBuilder builder = new ExercisePrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		ofNullable(getExerciseDate()).ifPresent(builder::setExerciseDate);
		ofNullable(getExerciseTime()).ifPresent(builder::setExerciseTime);
		ofNullable(getFullExercise()).ifPresent(builder::setFullExercise);
		return builder;
	}

	public static ExercisePrimitiveBuilder builder() {
		return new ExercisePrimitiveBuilder();
	}

	public static  class ExercisePrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected ExerciseOutcome.ExerciseOutcomeBuilder after;
		protected ContractState.ContractStateBuilder before;
		protected AdjustableOrAdjustedDate.AdjustableOrAdjustedDateBuilder exerciseDate;
		protected BusinessCenterTime.BusinessCenterTimeBuilder exerciseTime;
		protected Boolean fullExercise;
	
		public ExercisePrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExercisePrimitive> metaData() {
			return metaData;
		} 
	
		
		public ExerciseOutcome.ExerciseOutcomeBuilder getAfter() {
			return after;
		}
		
		public ExerciseOutcome.ExerciseOutcomeBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new ExerciseOutcome.ExerciseOutcomeBuilder();
		}
		
		
		public ContractState.ContractStateBuilder getBefore() {
			return before;
		}
		
		public ContractState.ContractStateBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new ContractState.ContractStateBuilder();
		}
		
		
		public AdjustableOrAdjustedDate.AdjustableOrAdjustedDateBuilder getExerciseDate() {
			return exerciseDate;
		}
		
		public AdjustableOrAdjustedDate.AdjustableOrAdjustedDateBuilder getOrCreateExerciseDate() {
			if (exerciseDate!=null) {
				return exerciseDate;
			}
			else return exerciseDate = new AdjustableOrAdjustedDate.AdjustableOrAdjustedDateBuilder();
		}
		
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getExerciseTime() {
			return exerciseTime;
		}
		
		public BusinessCenterTime.BusinessCenterTimeBuilder getOrCreateExerciseTime() {
			if (exerciseTime!=null) {
				return exerciseTime;
			}
			else return exerciseTime = new BusinessCenterTime.BusinessCenterTimeBuilder();
		}
		
		
		public Boolean getFullExercise() {
			return fullExercise;
		}
		
	
		public ExercisePrimitiveBuilder setAfter(ExerciseOutcome after) {
			this.after = after.toBuilder();
			return this;
		}
		public ExercisePrimitiveBuilder setAfterBuilder(ExerciseOutcome.ExerciseOutcomeBuilder after) {
			this.after = after;
			return this;
		}
		public ExercisePrimitiveBuilder setBefore(ContractState before) {
			this.before = before.toBuilder();
			return this;
		}
		public ExercisePrimitiveBuilder setBeforeBuilder(ContractState.ContractStateBuilder before) {
			this.before = before;
			return this;
		}
		public ExercisePrimitiveBuilder setExerciseDate(AdjustableOrAdjustedDate exerciseDate) {
			this.exerciseDate = exerciseDate.toBuilder();
			return this;
		}
		public ExercisePrimitiveBuilder setExerciseDateBuilder(AdjustableOrAdjustedDate.AdjustableOrAdjustedDateBuilder exerciseDate) {
			this.exerciseDate = exerciseDate;
			return this;
		}
		public ExercisePrimitiveBuilder setExerciseTime(BusinessCenterTime exerciseTime) {
			this.exerciseTime = exerciseTime.toBuilder();
			return this;
		}
		public ExercisePrimitiveBuilder setExerciseTimeBuilder(BusinessCenterTime.BusinessCenterTimeBuilder exerciseTime) {
			this.exerciseTime = exerciseTime;
			return this;
		}
		public ExercisePrimitiveBuilder setFullExercise(Boolean fullExercise) {
			this.fullExercise = fullExercise;
			return this;
		}
		
		public ExercisePrimitive build() {
			return new ExercisePrimitive(this);
		}
	
		@Override
		public ExercisePrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			if (exerciseDate!=null && !exerciseDate.prune().hasData()) exerciseDate = null;
			if (exerciseTime!=null && !exerciseTime.prune().hasData()) exerciseTime = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			if (getExerciseDate()!=null && getExerciseDate().hasData()) return true;
			if (getExerciseTime()!=null && getExerciseTime().hasData()) return true;
			if (getFullExercise()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("fullExercise"), Boolean.class, fullExercise, this);
			
			processRosetta(path.newSubPath("after"), processor, ExerciseOutcome.class, after);
			processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
			processRosetta(path.newSubPath("exerciseDate"), processor, AdjustableOrAdjustedDate.class, exerciseDate);
			processRosetta(path.newSubPath("exerciseTime"), processor, BusinessCenterTime.class, exerciseTime);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExercisePrimitiveBuilder _that = (ExercisePrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			if (exerciseDate != null ? !exerciseDate.equals(_that.exerciseDate) : _that.exerciseDate != null) return false;
			if (exerciseTime != null ? !exerciseTime.equals(_that.exerciseTime) : _that.exerciseTime != null) return false;
			if (fullExercise != null ? !fullExercise.equals(_that.fullExercise) : _that.fullExercise != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			_result = 31 * _result + (exerciseDate != null ? exerciseDate.hashCode() : 0);
			_result = 31 * _result + (exerciseTime != null ? exerciseTime.hashCode() : 0);
			_result = 31 * _result + (fullExercise != null ? fullExercise.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExercisePrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before + ", " +
				"exerciseDate=" + this.exerciseDate + ", " +
				"exerciseTime=" + this.exerciseTime + ", " +
				"fullExercise=" + this.fullExercise +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("fullExercise"), Boolean.class, fullExercise, this);
		
		processRosetta(path.newSubPath("after"), processor, ExerciseOutcome.class, after);
		processRosetta(path.newSubPath("before"), processor, ContractState.class, before);
		processRosetta(path.newSubPath("exerciseDate"), processor, AdjustableOrAdjustedDate.class, exerciseDate);
		processRosetta(path.newSubPath("exerciseTime"), processor, BusinessCenterTime.class, exerciseTime);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExercisePrimitive _that = (ExercisePrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		if (exerciseDate != null ? !exerciseDate.equals(_that.exerciseDate) : _that.exerciseDate != null) return false;
		if (exerciseTime != null ? !exerciseTime.equals(_that.exerciseTime) : _that.exerciseTime != null) return false;
		if (fullExercise != null ? !fullExercise.equals(_that.fullExercise) : _that.fullExercise != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		_result = 31 * _result + (exerciseDate != null ? exerciseDate.hashCode() : 0);
		_result = 31 * _result + (exerciseTime != null ? exerciseTime.hashCode() : 0);
		_result = 31 * _result + (fullExercise != null ? fullExercise.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExercisePrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before + ", " +
			"exerciseDate=" + this.exerciseDate + ", " +
			"exerciseTime=" + this.exerciseTime + ", " +
			"fullExercise=" + this.fullExercise +
		'}';
	}
}
