package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.AmericanExercise;
import org.isda.cdm.BermudaExercise;
import org.isda.cdm.EuropeanExercise;
import org.isda.cdm.ExerciseNotice;
import org.isda.cdm.ExtendibleProvisionAdjustedDates;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExtendibleProvisionMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A class defining an option to extend an existing swap transaction on the specified exercise dates for a term ending on the specified new termination date. As a difference from FpML, it extends the BuyerSeller class, which represents the BuyerSeller.model.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExtendibleProvision", source="FpML_5_10")
public class ExtendibleProvision extends BuyerSeller {
	private final AmericanExercise americanExercise;
	private final BermudaExercise bermudaExercise;
	private final EuropeanExercise europeanExercise;
	private final ExerciseNotice exerciseNotice;
	private final ExtendibleProvisionAdjustedDates extendibleProvisionAdjustedDates;
	private final Boolean followUpConfirmation;
	private static ExtendibleProvisionMeta metaData = new ExtendibleProvisionMeta();
	
	ExtendibleProvision(ExtendibleProvisionBuilder builder) {
		super(builder);
		this.americanExercise = ofNullable(builder.getAmericanExercise()).map(f->f.build()).orElse(null);
		this.bermudaExercise = ofNullable(builder.getBermudaExercise()).map(f->f.build()).orElse(null);
		this.europeanExercise = ofNullable(builder.getEuropeanExercise()).map(f->f.build()).orElse(null);
		this.exerciseNotice = ofNullable(builder.getExerciseNotice()).map(f->f.build()).orElse(null);
		this.extendibleProvisionAdjustedDates = ofNullable(builder.getExtendibleProvisionAdjustedDates()).map(f->f.build()).orElse(null);
		this.followUpConfirmation = builder.getFollowUpConfirmation();
	}
	
	/**
	 * American exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="americanExercise", source="FpML_5_10")
	@RosettaSynonym(value="americanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="americanExercise", source="CME_ClearedConfirm_1_17")
	public final AmericanExercise getAmericanExercise() {
		return americanExercise;
	}
	
	/**
	 * Bermuda exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="bermudaExercise", source="FpML_5_10")
	@RosettaSynonym(value="bermudaExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_11_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_9_0")
	@RosettaSynonym(value="bermudaExercise", source="CME_ClearedConfirm_1_17")
	public final BermudaExercise getBermudaExercise() {
		return bermudaExercise;
	}
	
	/**
	 * European exercise. FpML implementations consists in an exercise substitution group.
	 */
	@RosettaSynonym(value="europeanExercise", source="FpML_5_10")
	@RosettaSynonym(value="europeanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="europeanExercise", source="CME_ClearedConfirm_1_17")
	public final EuropeanExercise getEuropeanExercise() {
		return europeanExercise;
	}
	
	/**
	 * Definition of the party to whom notice of exercise should be given.
	 */
	@RosettaSynonym(value="exerciseNotice", source="FpML_5_10")
	@RosettaSynonym(value="exerciseNotice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_11_0")
	@RosettaSynonym(value="exerciseNotice", source="DTCC_9_0")
	@RosettaSynonym(value="exerciseNotice", source="CME_ClearedConfirm_1_17")
	public final ExerciseNotice getExerciseNotice() {
		return exerciseNotice;
	}
	
	/**
	 * The adjusted dates associated with an extendible provision. These dates have been adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="extendibleProvisionAdjustedDates", source="FpML_5_10")
	@RosettaSynonym(value="extendibleProvisionAdjustedDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="extendibleProvisionAdjustedDates", source="DTCC_11_0")
	@RosettaSynonym(value="extendibleProvisionAdjustedDates", source="DTCC_9_0")
	@RosettaSynonym(value="extendibleProvisionAdjustedDates", source="CME_ClearedConfirm_1_17")
	public final ExtendibleProvisionAdjustedDates getExtendibleProvisionAdjustedDates() {
		return extendibleProvisionAdjustedDates;
	}
	
	/**
	 * A flag to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller&#39;s agent.
	 */
	@RosettaSynonym(value="followUpConfirmation", source="FpML_5_10")
	@RosettaSynonym(value="followUpConfirmation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_11_0")
	@RosettaSynonym(value="followUpConfirmation", source="DTCC_9_0")
	@RosettaSynonym(value="followUpConfirmation", source="CME_ClearedConfirm_1_17")
	public final Boolean getFollowUpConfirmation() {
		return followUpConfirmation;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExtendibleProvision> metaData() {
		return metaData;
	} 
	
	public ExtendibleProvisionBuilder toBuilder() {
		ExtendibleProvisionBuilder builder = new ExtendibleProvisionBuilder();
		ofNullable(getAmericanExercise()).ifPresent(builder::setAmericanExercise);
		ofNullable(getBermudaExercise()).ifPresent(builder::setBermudaExercise);
		ofNullable(getEuropeanExercise()).ifPresent(builder::setEuropeanExercise);
		ofNullable(getExerciseNotice()).ifPresent(builder::setExerciseNotice);
		ofNullable(getExtendibleProvisionAdjustedDates()).ifPresent(builder::setExtendibleProvisionAdjustedDates);
		ofNullable(getFollowUpConfirmation()).ifPresent(builder::setFollowUpConfirmation);
		ofNullable(getBuyerAccountReference()).ifPresent(builder::setBuyerAccountReference);
		ofNullable(getBuyerPartyReference()).ifPresent(builder::setBuyerPartyReference);
		ofNullable(getSellerAccountReference()).ifPresent(builder::setSellerAccountReference);
		ofNullable(getSellerPartyReference()).ifPresent(builder::setSellerPartyReference);
		return builder;
	}

	public static ExtendibleProvisionBuilder builder() {
		return new ExtendibleProvisionBuilder();
	}

	public static  class ExtendibleProvisionBuilder extends BuyerSellerBuilder {
	
		protected AmericanExercise.AmericanExerciseBuilder americanExercise;
		protected BermudaExercise.BermudaExerciseBuilder bermudaExercise;
		protected EuropeanExercise.EuropeanExerciseBuilder europeanExercise;
		protected ExerciseNotice.ExerciseNoticeBuilder exerciseNotice;
		protected ExtendibleProvisionAdjustedDates.ExtendibleProvisionAdjustedDatesBuilder extendibleProvisionAdjustedDates;
		protected Boolean followUpConfirmation;
	
		public ExtendibleProvisionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExtendibleProvision> metaData() {
			return metaData;
		} 
	
		
		public AmericanExercise.AmericanExerciseBuilder getAmericanExercise() {
			return americanExercise;
		}
		
		public AmericanExercise.AmericanExerciseBuilder getOrCreateAmericanExercise() {
			if (americanExercise!=null) {
				return americanExercise;
			}
			else return americanExercise = new AmericanExercise.AmericanExerciseBuilder();
		}
		
		
		public BermudaExercise.BermudaExerciseBuilder getBermudaExercise() {
			return bermudaExercise;
		}
		
		public BermudaExercise.BermudaExerciseBuilder getOrCreateBermudaExercise() {
			if (bermudaExercise!=null) {
				return bermudaExercise;
			}
			else return bermudaExercise = new BermudaExercise.BermudaExerciseBuilder();
		}
		
		
		public EuropeanExercise.EuropeanExerciseBuilder getEuropeanExercise() {
			return europeanExercise;
		}
		
		public EuropeanExercise.EuropeanExerciseBuilder getOrCreateEuropeanExercise() {
			if (europeanExercise!=null) {
				return europeanExercise;
			}
			else return europeanExercise = new EuropeanExercise.EuropeanExerciseBuilder();
		}
		
		
		public ExerciseNotice.ExerciseNoticeBuilder getExerciseNotice() {
			return exerciseNotice;
		}
		
		public ExerciseNotice.ExerciseNoticeBuilder getOrCreateExerciseNotice() {
			if (exerciseNotice!=null) {
				return exerciseNotice;
			}
			else return exerciseNotice = new ExerciseNotice.ExerciseNoticeBuilder();
		}
		
		
		public ExtendibleProvisionAdjustedDates.ExtendibleProvisionAdjustedDatesBuilder getExtendibleProvisionAdjustedDates() {
			return extendibleProvisionAdjustedDates;
		}
		
		public ExtendibleProvisionAdjustedDates.ExtendibleProvisionAdjustedDatesBuilder getOrCreateExtendibleProvisionAdjustedDates() {
			if (extendibleProvisionAdjustedDates!=null) {
				return extendibleProvisionAdjustedDates;
			}
			else return extendibleProvisionAdjustedDates = new ExtendibleProvisionAdjustedDates.ExtendibleProvisionAdjustedDatesBuilder();
		}
		
		
		public Boolean getFollowUpConfirmation() {
			return followUpConfirmation;
		}
		
	
		public ExtendibleProvisionBuilder setAmericanExercise(AmericanExercise americanExercise) {
			this.americanExercise = americanExercise.toBuilder();
			return this;
		}
		public ExtendibleProvisionBuilder setAmericanExerciseBuilder(AmericanExercise.AmericanExerciseBuilder americanExercise) {
			this.americanExercise = americanExercise;
			return this;
		}
		public ExtendibleProvisionBuilder setBermudaExercise(BermudaExercise bermudaExercise) {
			this.bermudaExercise = bermudaExercise.toBuilder();
			return this;
		}
		public ExtendibleProvisionBuilder setBermudaExerciseBuilder(BermudaExercise.BermudaExerciseBuilder bermudaExercise) {
			this.bermudaExercise = bermudaExercise;
			return this;
		}
		public ExtendibleProvisionBuilder setEuropeanExercise(EuropeanExercise europeanExercise) {
			this.europeanExercise = europeanExercise.toBuilder();
			return this;
		}
		public ExtendibleProvisionBuilder setEuropeanExerciseBuilder(EuropeanExercise.EuropeanExerciseBuilder europeanExercise) {
			this.europeanExercise = europeanExercise;
			return this;
		}
		public ExtendibleProvisionBuilder setExerciseNotice(ExerciseNotice exerciseNotice) {
			this.exerciseNotice = exerciseNotice.toBuilder();
			return this;
		}
		public ExtendibleProvisionBuilder setExerciseNoticeBuilder(ExerciseNotice.ExerciseNoticeBuilder exerciseNotice) {
			this.exerciseNotice = exerciseNotice;
			return this;
		}
		public ExtendibleProvisionBuilder setExtendibleProvisionAdjustedDates(ExtendibleProvisionAdjustedDates extendibleProvisionAdjustedDates) {
			this.extendibleProvisionAdjustedDates = extendibleProvisionAdjustedDates.toBuilder();
			return this;
		}
		public ExtendibleProvisionBuilder setExtendibleProvisionAdjustedDatesBuilder(ExtendibleProvisionAdjustedDates.ExtendibleProvisionAdjustedDatesBuilder extendibleProvisionAdjustedDates) {
			this.extendibleProvisionAdjustedDates = extendibleProvisionAdjustedDates;
			return this;
		}
		public ExtendibleProvisionBuilder setFollowUpConfirmation(Boolean followUpConfirmation) {
			this.followUpConfirmation = followUpConfirmation;
			return this;
		}
		@Override public ExtendibleProvisionBuilder setBuyerAccountReference(ReferenceWithMetaAccount buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference.toBuilder();
			return this;
		}
		@Override public ExtendibleProvisionBuilder setBuyerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder buyerAccountReference) {
			this.buyerAccountReference = buyerAccountReference;
			return this;
		}
		@Override public ExtendibleProvisionBuilder setBuyerAccountReferenceRef(Account.AccountBuilder buyerAccountReference) {
			return setBuyerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(buyerAccountReference).build());
		}
		@Override public ExtendibleProvisionBuilder setBuyerAccountReferenceRef(Account buyerAccountReference) {
			return setBuyerAccountReferenceRef(Account.builder());
		}
		@Override public ExtendibleProvisionBuilder setBuyerPartyReference(ReferenceWithMetaParty buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference.toBuilder();
			return this;
		}
		@Override public ExtendibleProvisionBuilder setBuyerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference;
			return this;
		}
		@Override public ExtendibleProvisionBuilder setBuyerPartyReferenceRef(Party.PartyBuilder buyerPartyReference) {
			return setBuyerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(buyerPartyReference).build());
		}
		@Override public ExtendibleProvisionBuilder setBuyerPartyReferenceRef(Party buyerPartyReference) {
			return setBuyerPartyReferenceRef(Party.builder());
		}
		@Override public ExtendibleProvisionBuilder setSellerAccountReference(ReferenceWithMetaAccount sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference.toBuilder();
			return this;
		}
		@Override public ExtendibleProvisionBuilder setSellerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder sellerAccountReference) {
			this.sellerAccountReference = sellerAccountReference;
			return this;
		}
		@Override public ExtendibleProvisionBuilder setSellerAccountReferenceRef(Account.AccountBuilder sellerAccountReference) {
			return setSellerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(sellerAccountReference).build());
		}
		@Override public ExtendibleProvisionBuilder setSellerAccountReferenceRef(Account sellerAccountReference) {
			return setSellerAccountReferenceRef(Account.builder());
		}
		@Override public ExtendibleProvisionBuilder setSellerPartyReference(ReferenceWithMetaParty sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference.toBuilder();
			return this;
		}
		@Override public ExtendibleProvisionBuilder setSellerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference;
			return this;
		}
		@Override public ExtendibleProvisionBuilder setSellerPartyReferenceRef(Party.PartyBuilder sellerPartyReference) {
			return setSellerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(sellerPartyReference).build());
		}
		@Override public ExtendibleProvisionBuilder setSellerPartyReferenceRef(Party sellerPartyReference) {
			return setSellerPartyReferenceRef(Party.builder());
		}
		
		public ExtendibleProvision build() {
			return new ExtendibleProvision(this);
		}
	
		@Override
		public ExtendibleProvisionBuilder prune() {
			super.prune();
			if (americanExercise!=null && !americanExercise.prune().hasData()) americanExercise = null;
			if (bermudaExercise!=null && !bermudaExercise.prune().hasData()) bermudaExercise = null;
			if (europeanExercise!=null && !europeanExercise.prune().hasData()) europeanExercise = null;
			if (exerciseNotice!=null && !exerciseNotice.prune().hasData()) exerciseNotice = null;
			if (extendibleProvisionAdjustedDates!=null && !extendibleProvisionAdjustedDates.prune().hasData()) extendibleProvisionAdjustedDates = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAmericanExercise()!=null && getAmericanExercise().hasData()) return true;
			if (getBermudaExercise()!=null && getBermudaExercise().hasData()) return true;
			if (getEuropeanExercise()!=null && getEuropeanExercise().hasData()) return true;
			if (getExerciseNotice()!=null && getExerciseNotice().hasData()) return true;
			if (getExtendibleProvisionAdjustedDates()!=null && getExtendibleProvisionAdjustedDates().hasData()) return true;
			if (getFollowUpConfirmation()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
			
			processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
			processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
			processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
			processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
			processRosetta(path.newSubPath("extendibleProvisionAdjustedDates"), processor, ExtendibleProvisionAdjustedDates.class, extendibleProvisionAdjustedDates);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			ExtendibleProvisionBuilder _that = (ExtendibleProvisionBuilder) o;
		
			if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
			if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
			if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
			if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
			if (extendibleProvisionAdjustedDates != null ? !extendibleProvisionAdjustedDates.equals(_that.extendibleProvisionAdjustedDates) : _that.extendibleProvisionAdjustedDates != null) return false;
			if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
			_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
			_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
			_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
			_result = 31 * _result + (extendibleProvisionAdjustedDates != null ? extendibleProvisionAdjustedDates.hashCode() : 0);
			_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExtendibleProvisionBuilder {" +
				"americanExercise=" + this.americanExercise + ", " +
				"bermudaExercise=" + this.bermudaExercise + ", " +
				"europeanExercise=" + this.europeanExercise + ", " +
				"exerciseNotice=" + this.exerciseNotice + ", " +
				"extendibleProvisionAdjustedDates=" + this.extendibleProvisionAdjustedDates + ", " +
				"followUpConfirmation=" + this.followUpConfirmation +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("followUpConfirmation"), Boolean.class, followUpConfirmation, this);
		
		processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
		processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
		processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
		processRosetta(path.newSubPath("exerciseNotice"), processor, ExerciseNotice.class, exerciseNotice);
		processRosetta(path.newSubPath("extendibleProvisionAdjustedDates"), processor, ExtendibleProvisionAdjustedDates.class, extendibleProvisionAdjustedDates);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		ExtendibleProvision _that = (ExtendibleProvision) o;
	
		if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
		if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
		if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
		if (exerciseNotice != null ? !exerciseNotice.equals(_that.exerciseNotice) : _that.exerciseNotice != null) return false;
		if (extendibleProvisionAdjustedDates != null ? !extendibleProvisionAdjustedDates.equals(_that.extendibleProvisionAdjustedDates) : _that.extendibleProvisionAdjustedDates != null) return false;
		if (followUpConfirmation != null ? !followUpConfirmation.equals(_that.followUpConfirmation) : _that.followUpConfirmation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
		_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
		_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
		_result = 31 * _result + (exerciseNotice != null ? exerciseNotice.hashCode() : 0);
		_result = 31 * _result + (extendibleProvisionAdjustedDates != null ? extendibleProvisionAdjustedDates.hashCode() : 0);
		_result = 31 * _result + (followUpConfirmation != null ? followUpConfirmation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExtendibleProvision {" +
			"americanExercise=" + this.americanExercise + ", " +
			"bermudaExercise=" + this.bermudaExercise + ", " +
			"europeanExercise=" + this.europeanExercise + ", " +
			"exerciseNotice=" + this.exerciseNotice + ", " +
			"extendibleProvisionAdjustedDates=" + this.extendibleProvisionAdjustedDates + ", " +
			"followUpConfirmation=" + this.followUpConfirmation +
		'}' + " " + super.toString();
	}
}
