package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ExtensionEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExtendibleProvisionAdjustedDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the adjusted dates associated with a provision to extend a swap.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExtendibleProvisionAdjustedDates", source="FpML_5_10")
public class ExtendibleProvisionAdjustedDates extends RosettaModelObject {
	private final List<ExtensionEvent> extensionEvent;
	private static ExtendibleProvisionAdjustedDatesMeta metaData = new ExtendibleProvisionAdjustedDatesMeta();
	
	ExtendibleProvisionAdjustedDates(ExtendibleProvisionAdjustedDatesBuilder builder) {
		this.extensionEvent = ofNullable(builder.getExtensionEvent()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The adjusted dates associated with a single extendible exercise date.
	 */
	@RosettaSynonym(value="extensionEvent", source="FpML_5_10")
	@RosettaSynonym(value="extensionEvent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="extensionEvent", source="DTCC_11_0")
	@RosettaSynonym(value="extensionEvent", source="DTCC_9_0")
	@RosettaSynonym(value="extensionEvent", source="CME_ClearedConfirm_1_17")
	public final List<ExtensionEvent> getExtensionEvent() {
		return extensionEvent;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExtendibleProvisionAdjustedDates> metaData() {
		return metaData;
	} 
	
	public ExtendibleProvisionAdjustedDatesBuilder toBuilder() {
		ExtendibleProvisionAdjustedDatesBuilder builder = new ExtendibleProvisionAdjustedDatesBuilder();
		ofNullable(getExtensionEvent()).ifPresent(extensionEvent -> extensionEvent.forEach(builder::addExtensionEvent));
		return builder;
	}

	public static ExtendibleProvisionAdjustedDatesBuilder builder() {
		return new ExtendibleProvisionAdjustedDatesBuilder();
	}

	public static  class ExtendibleProvisionAdjustedDatesBuilder extends RosettaModelObjectBuilder {
	
		protected List<ExtensionEvent.ExtensionEventBuilder> extensionEvent;
	
		public ExtendibleProvisionAdjustedDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExtendibleProvisionAdjustedDates> metaData() {
			return metaData;
		} 
	
		
		public List<ExtensionEvent.ExtensionEventBuilder> getExtensionEvent() {
			return extensionEvent;
		}
		
		public ExtensionEvent.ExtensionEventBuilder getOrCreateExtensionEvent(int index) {
			if (extensionEvent==null) {
				this.extensionEvent = new ArrayList<>();
			}
			return getIndex(extensionEvent, index, ()->new ExtensionEvent.ExtensionEventBuilder());
		}
		
	
		public ExtendibleProvisionAdjustedDatesBuilder addExtensionEvent(ExtensionEvent extensionEvent) {
			if(this.extensionEvent == null){
				this.extensionEvent = new ArrayList<>();
			}
			this.extensionEvent.add(extensionEvent.toBuilder());
			return this;
		}
		public ExtendibleProvisionAdjustedDatesBuilder addExtensionEvent(ExtensionEvent extensionEvent, int _idx) {
			if(this.extensionEvent == null){
				this.extensionEvent = new ArrayList<>();
			}
			getIndex(this.extensionEvent, _idx, () -> extensionEvent.toBuilder());
			this.extensionEvent.set(_idx, extensionEvent.toBuilder());
			return this;
		}
		public ExtendibleProvisionAdjustedDatesBuilder addExtensionEvent(List<ExtensionEvent> extensionEvents) {
			if(this.extensionEvent == null){
				this.extensionEvent = new ArrayList<>();
			}
			for (ExtensionEvent toAdd : extensionEvents) {
				this.extensionEvent.add(toAdd.toBuilder());
			}
			return this;
		}
		public ExtendibleProvisionAdjustedDatesBuilder addExtensionEventBuilder(ExtensionEvent.ExtensionEventBuilder extensionEvent) {
			if(this.extensionEvent == null){
				this.extensionEvent = new ArrayList<>();
			}
			this.extensionEvent.add(extensionEvent);
			return this;
		}
		
		public ExtendibleProvisionAdjustedDatesBuilder clearExtensionEvent() {
			this.extensionEvent = null;
			return this;
		}
		
		public ExtendibleProvisionAdjustedDates build() {
			return new ExtendibleProvisionAdjustedDates(this);
		}
	
		@Override
		public ExtendibleProvisionAdjustedDatesBuilder prune() {
			if (extensionEvent!=null) extensionEvent = extensionEvent.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExtensionEvent()!=null && getExtensionEvent().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("extensionEvent"), processor, ExtensionEvent.class, extensionEvent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExtendibleProvisionAdjustedDatesBuilder _that = (ExtendibleProvisionAdjustedDatesBuilder) o;
		
			if (!ListEquals.listEquals(extensionEvent, _that.extensionEvent)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (extensionEvent != null ? extensionEvent.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExtendibleProvisionAdjustedDatesBuilder {" +
				"extensionEvent=" + this.extensionEvent +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("extensionEvent"), processor, ExtensionEvent.class, extensionEvent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExtendibleProvisionAdjustedDates _that = (ExtendibleProvisionAdjustedDates) o;
	
		if (!ListEquals.listEquals(extensionEvent, _that.extensionEvent)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (extensionEvent != null ? extensionEvent.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExtendibleProvisionAdjustedDates {" +
			"extensionEvent=" + this.extensionEvent +
		'}';
	}
}
