package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ExtensionEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to define the adjusted dates associated with an individual extension event.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ExtensionEvent", source="FpML_5_10")
@RosettaSynonym(value="ExtensionEvent", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="ExtensionEvent", source="DTCC_11_0")
@RosettaSynonym(value="ExtensionEvent", source="DTCC_9_0")
@RosettaSynonym(value="ExtensionEvent", source="CME_ClearedConfirm_1_17")
public class ExtensionEvent extends RosettaModelObject implements GlobalKey {
	private final Date adjustedExerciseDate;
	private final Date adjustedExtendedTerminationDate;
	private final MetaFields meta;
	private static ExtensionEventMeta metaData = new ExtensionEventMeta();
	
	ExtensionEvent(ExtensionEventBuilder builder) {
		this.adjustedExerciseDate = builder.getAdjustedExerciseDate();
		this.adjustedExtendedTerminationDate = builder.getAdjustedExtendedTerminationDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which option exercise takes place. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExerciseDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExerciseDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExerciseDate() {
		return adjustedExerciseDate;
	}
	
	/**
	 * The termination date if an extendible provision is exercised. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedExtendedTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedExtendedTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedExtendedTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedExtendedTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedExtendedTerminationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedExtendedTerminationDate() {
		return adjustedExtendedTerminationDate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends ExtensionEvent> metaData() {
		return metaData;
	} 
	
	public ExtensionEventBuilder toBuilder() {
		ExtensionEventBuilder builder = new ExtensionEventBuilder();
		ofNullable(getAdjustedExerciseDate()).ifPresent(builder::setAdjustedExerciseDate);
		ofNullable(getAdjustedExtendedTerminationDate()).ifPresent(builder::setAdjustedExtendedTerminationDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static ExtensionEventBuilder builder() {
		return new ExtensionEventBuilder();
	}

	public static  class ExtensionEventBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ExtensionEventBuilder> {
	
		protected Date adjustedExerciseDate;
		protected Date adjustedExtendedTerminationDate;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public ExtensionEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ExtensionEvent> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedExerciseDate() {
			return adjustedExerciseDate;
		}
		
		
		public Date getAdjustedExtendedTerminationDate() {
			return adjustedExtendedTerminationDate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public ExtensionEventBuilder setAdjustedExerciseDate(Date adjustedExerciseDate) {
			this.adjustedExerciseDate = adjustedExerciseDate;
			return this;
		}
		public ExtensionEventBuilder setAdjustedExtendedTerminationDate(Date adjustedExtendedTerminationDate) {
			this.adjustedExtendedTerminationDate = adjustedExtendedTerminationDate;
			return this;
		}
		public ExtensionEventBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ExtensionEventBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public ExtensionEvent build() {
			return new ExtensionEvent(this);
		}
	
		@Override
		public ExtensionEventBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedExerciseDate()!=null) return true;
			if (getAdjustedExtendedTerminationDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
			processor.processBasic(path.newSubPath("adjustedExtendedTerminationDate"), Date.class, adjustedExtendedTerminationDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ExtensionEventBuilder _that = (ExtensionEventBuilder) o;
		
			if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
			if (adjustedExtendedTerminationDate != null ? !adjustedExtendedTerminationDate.equals(_that.adjustedExtendedTerminationDate) : _that.adjustedExtendedTerminationDate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
			_result = 31 * _result + (adjustedExtendedTerminationDate != null ? adjustedExtendedTerminationDate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ExtensionEventBuilder {" +
				"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
				"adjustedExtendedTerminationDate=" + this.adjustedExtendedTerminationDate + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedExerciseDate"), Date.class, adjustedExerciseDate, this);
		processor.processBasic(path.newSubPath("adjustedExtendedTerminationDate"), Date.class, adjustedExtendedTerminationDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ExtensionEvent _that = (ExtensionEvent) o;
	
		if (adjustedExerciseDate != null ? !adjustedExerciseDate.equals(_that.adjustedExerciseDate) : _that.adjustedExerciseDate != null) return false;
		if (adjustedExtendedTerminationDate != null ? !adjustedExtendedTerminationDate.equals(_that.adjustedExtendedTerminationDate) : _that.adjustedExtendedTerminationDate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedExerciseDate != null ? adjustedExerciseDate.hashCode() : 0);
		_result = 31 * _result + (adjustedExtendedTerminationDate != null ? adjustedExtendedTerminationDate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ExtensionEvent {" +
			"adjustedExerciseDate=" + this.adjustedExerciseDate + ", " +
			"adjustedExtendedTerminationDate=" + this.adjustedExtendedTerminationDate + ", " +
			"meta=" + this.meta +
		'}';
	}
}
