package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.GracePeriodExtension;
import org.isda.cdm.Money;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FailureToPayMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="notifyingParty", source="FpML_5_10")
public class FailureToPay extends RosettaModelObject {
	private final Boolean applicable;
	private final GracePeriodExtension gracePeriodExtension;
	private final Money paymentRequirement;
	private static FailureToPayMeta metaData = new FailureToPayMeta();
	
	FailureToPay(FailureToPayBuilder builder) {
		this.applicable = builder.getApplicable();
		this.gracePeriodExtension = ofNullable(builder.getGracePeriodExtension()).map(f->f.build()).orElse(null);
		this.paymentRequirement = ofNullable(builder.getPaymentRequirement()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates whether the failure to pay provision is applicable.
	 */
	@RosettaSynonym(value="applicable", source="FpML_5_10")
	@RosettaSynonym(value="applicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="applicable", source="DTCC_11_0")
	@RosettaSynonym(value="applicable", source="DTCC_9_0")
	@RosettaSynonym(value="applicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * If this element is specified, indicates whether or not a grace period extension is applicable. ISDA 2003 Term: Grace Period Extension Applicable.
	 */
	@RosettaSynonym(value="gracePeriodExtension", source="FpML_5_10")
	@RosettaSynonym(value="gracePeriodExtension", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="gracePeriodExtension", source="DTCC_11_0")
	@RosettaSynonym(value="gracePeriodExtension", source="DTCC_9_0")
	@RosettaSynonym(value="gracePeriodExtension", source="CME_ClearedConfirm_1_17")
	public final GracePeriodExtension getGracePeriodExtension() {
		return gracePeriodExtension;
	}
	
	/**
	 * Specifies a threshold for the failure to pay credit event. Market standard is USD 1,000,000 (JPY 100,000,000 for Japanese Yen trades) or its equivalent in the relevant obligation currency. This is applied on an aggregate basis across all Obligations of the Reference Entity. Intended to prevent technical/operational errors from triggering credit events. ISDA 2003 Term: Payment Requirement
	 */
	@RosettaSynonym(value="paymentRequirement", source="FpML_5_10")
	@RosettaSynonym(value="paymentRequirement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentRequirement", source="DTCC_11_0")
	@RosettaSynonym(value="paymentRequirement", source="DTCC_9_0")
	@RosettaSynonym(value="paymentRequirement", source="CME_ClearedConfirm_1_17")
	public final Money getPaymentRequirement() {
		return paymentRequirement;
	}
	
	
	@Override
	public RosettaMetaData<? extends FailureToPay> metaData() {
		return metaData;
	} 
	
	public FailureToPayBuilder toBuilder() {
		FailureToPayBuilder builder = new FailureToPayBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getGracePeriodExtension()).ifPresent(builder::setGracePeriodExtension);
		ofNullable(getPaymentRequirement()).ifPresent(builder::setPaymentRequirement);
		return builder;
	}

	public static FailureToPayBuilder builder() {
		return new FailureToPayBuilder();
	}

	public static  class FailureToPayBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected GracePeriodExtension.GracePeriodExtensionBuilder gracePeriodExtension;
		protected Money.MoneyBuilder paymentRequirement;
	
		public FailureToPayBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FailureToPay> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public GracePeriodExtension.GracePeriodExtensionBuilder getGracePeriodExtension() {
			return gracePeriodExtension;
		}
		
		public GracePeriodExtension.GracePeriodExtensionBuilder getOrCreateGracePeriodExtension() {
			if (gracePeriodExtension!=null) {
				return gracePeriodExtension;
			}
			else return gracePeriodExtension = new GracePeriodExtension.GracePeriodExtensionBuilder();
		}
		
		
		public Money.MoneyBuilder getPaymentRequirement() {
			return paymentRequirement;
		}
		
		public Money.MoneyBuilder getOrCreatePaymentRequirement() {
			if (paymentRequirement!=null) {
				return paymentRequirement;
			}
			else return paymentRequirement = new Money.MoneyBuilder();
		}
		
	
		public FailureToPayBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public FailureToPayBuilder setGracePeriodExtension(GracePeriodExtension gracePeriodExtension) {
			this.gracePeriodExtension = gracePeriodExtension.toBuilder();
			return this;
		}
		public FailureToPayBuilder setGracePeriodExtensionBuilder(GracePeriodExtension.GracePeriodExtensionBuilder gracePeriodExtension) {
			this.gracePeriodExtension = gracePeriodExtension;
			return this;
		}
		public FailureToPayBuilder setPaymentRequirement(Money paymentRequirement) {
			this.paymentRequirement = paymentRequirement.toBuilder();
			return this;
		}
		public FailureToPayBuilder setPaymentRequirementBuilder(Money.MoneyBuilder paymentRequirement) {
			this.paymentRequirement = paymentRequirement;
			return this;
		}
		
		public FailureToPay build() {
			return new FailureToPay(this);
		}
	
		@Override
		public FailureToPayBuilder prune() {
			if (gracePeriodExtension!=null && !gracePeriodExtension.prune().hasData()) gracePeriodExtension = null;
			if (paymentRequirement!=null && !paymentRequirement.prune().hasData()) paymentRequirement = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getGracePeriodExtension()!=null && getGracePeriodExtension().hasData()) return true;
			if (getPaymentRequirement()!=null && getPaymentRequirement().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			
			processRosetta(path.newSubPath("gracePeriodExtension"), processor, GracePeriodExtension.class, gracePeriodExtension);
			processRosetta(path.newSubPath("paymentRequirement"), processor, Money.class, paymentRequirement);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FailureToPayBuilder _that = (FailureToPayBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (gracePeriodExtension != null ? !gracePeriodExtension.equals(_that.gracePeriodExtension) : _that.gracePeriodExtension != null) return false;
			if (paymentRequirement != null ? !paymentRequirement.equals(_that.paymentRequirement) : _that.paymentRequirement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (gracePeriodExtension != null ? gracePeriodExtension.hashCode() : 0);
			_result = 31 * _result + (paymentRequirement != null ? paymentRequirement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FailureToPayBuilder {" +
				"applicable=" + this.applicable + ", " +
				"gracePeriodExtension=" + this.gracePeriodExtension + ", " +
				"paymentRequirement=" + this.paymentRequirement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		
		processRosetta(path.newSubPath("gracePeriodExtension"), processor, GracePeriodExtension.class, gracePeriodExtension);
		processRosetta(path.newSubPath("paymentRequirement"), processor, Money.class, paymentRequirement);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FailureToPay _that = (FailureToPay) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (gracePeriodExtension != null ? !gracePeriodExtension.equals(_that.gracePeriodExtension) : _that.gracePeriodExtension != null) return false;
		if (paymentRequirement != null ? !paymentRequirement.equals(_that.paymentRequirement) : _that.paymentRequirement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (gracePeriodExtension != null ? gracePeriodExtension.hashCode() : 0);
		_result = 31 * _result + (paymentRequirement != null ? paymentRequirement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FailureToPay {" +
			"applicable=" + this.applicable + ", " +
			"gracePeriodExtension=" + this.gracePeriodExtension + ", " +
			"paymentRequirement=" + this.paymentRequirement +
		'}';
	}
}
