package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CalculationAgent;
import org.isda.cdm.ValuationPostponement;
import org.isda.cdm.metafields.FieldWithMetaSettlementRateOptionEnum;
import org.isda.cdm.metafields.FieldWithMetaSettlementRateOptionEnum.FieldWithMetaSettlementRateOptionEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FallbackReferencePriceMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * The method, prioritised by the order it is listed in this element, to get a replacement rate for the disrupted settlement rate option.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FallbackReferencePrice", source="FpML_5_10")
public class FallbackReferencePrice extends RosettaModelObject {
	private final CalculationAgent calculationAgentDetermination;
	private final List<FieldWithMetaSettlementRateOptionEnum> fallBackSettlementRateOption;
	private final Boolean fallbackSurveyValuationPostponement;
	private final ValuationPostponement valuationPostponement;
	private static FallbackReferencePriceMeta metaData = new FallbackReferencePriceMeta();
	
	FallbackReferencePrice(FallbackReferencePriceBuilder builder) {
		this.calculationAgentDetermination = ofNullable(builder.getCalculationAgentDetermination()).map(f->f.build()).orElse(null);
		this.fallBackSettlementRateOption = ofNullable(builder.getFallBackSettlementRateOption()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.fallbackSurveyValuationPostponement = builder.getFallbackSurveyValuationPostponement();
		this.valuationPostponement = ofNullable(builder.getValuationPostponement()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The calculation agent will decide the rate.
	 */
	@RosettaSynonym(value="calculationAgentDetermination", source="FpML_5_10")
	@RosettaSynonym(value="calculationAgentDetermination", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationAgentDetermination", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="calculationAgentDetermination", source="DTCC_11_0")
	@RosettaSynonym(value="calculationAgentDetermination", source="DTCC_9_0")
	public final CalculationAgent getCalculationAgentDetermination() {
		return calculationAgentDetermination;
	}
	
	/**
	 * This settlement rate option will be used in its place.
	 */
	@RosettaSynonym(value="fallBackSettlementRateOption", source="FpML_5_10")
	@RosettaSynonym(value="fallBackSettlementRateOption", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fallBackSettlementRateOption", source="DTCC_11_0")
	@RosettaSynonym(value="fallBackSettlementRateOption", source="DTCC_9_0")
	@RosettaSynonym(value="fallBackSettlementRateOption", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaSettlementRateOptionEnum> getFallBackSettlementRateOption() {
		return fallBackSettlementRateOption;
	}
	
	/**
	 * Request rate quotes from the market. This element is set as type Empty in FpML. When present, the FpML synonym is mapped to a value True in the CDM.
	 */
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_11_0")
	@RosettaSynonym(value="", source="DTCC_9_0")
	public final Boolean getFallbackSurveyValuationPostponement() {
		return fallbackSurveyValuationPostponement;
	}
	
	/**
	 * Specifies how long to wait to get a quote from a settlement rate option upon a price source disruption.
	 */
	@RosettaSynonym(value="valuationPostponement", source="FpML_5_10")
	@RosettaSynonym(value="valuationPostponement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="valuationPostponement", source="DTCC_11_0")
	@RosettaSynonym(value="valuationPostponement", source="DTCC_9_0")
	@RosettaSynonym(value="valuationPostponement", source="CME_ClearedConfirm_1_17")
	public final ValuationPostponement getValuationPostponement() {
		return valuationPostponement;
	}
	
	
	@Override
	public RosettaMetaData<? extends FallbackReferencePrice> metaData() {
		return metaData;
	} 
	
	public FallbackReferencePriceBuilder toBuilder() {
		FallbackReferencePriceBuilder builder = new FallbackReferencePriceBuilder();
		ofNullable(getCalculationAgentDetermination()).ifPresent(builder::setCalculationAgentDetermination);
		ofNullable(getFallBackSettlementRateOption()).ifPresent(fallBackSettlementRateOption -> fallBackSettlementRateOption.forEach(builder::addFallBackSettlementRateOption));
		ofNullable(getFallbackSurveyValuationPostponement()).ifPresent(builder::setFallbackSurveyValuationPostponement);
		ofNullable(getValuationPostponement()).ifPresent(builder::setValuationPostponement);
		return builder;
	}

	public static FallbackReferencePriceBuilder builder() {
		return new FallbackReferencePriceBuilder();
	}

	public static  class FallbackReferencePriceBuilder extends RosettaModelObjectBuilder {
	
		protected CalculationAgent.CalculationAgentBuilder calculationAgentDetermination;
		protected List<FieldWithMetaSettlementRateOptionEnumBuilder> fallBackSettlementRateOption;
		protected Boolean fallbackSurveyValuationPostponement;
		protected ValuationPostponement.ValuationPostponementBuilder valuationPostponement;
	
		public FallbackReferencePriceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FallbackReferencePrice> metaData() {
			return metaData;
		} 
	
		
		public CalculationAgent.CalculationAgentBuilder getCalculationAgentDetermination() {
			return calculationAgentDetermination;
		}
		
		public CalculationAgent.CalculationAgentBuilder getOrCreateCalculationAgentDetermination() {
			if (calculationAgentDetermination!=null) {
				return calculationAgentDetermination;
			}
			else return calculationAgentDetermination = new CalculationAgent.CalculationAgentBuilder();
		}
		
		
		public List<FieldWithMetaSettlementRateOptionEnumBuilder> getFallBackSettlementRateOption() {
			return fallBackSettlementRateOption;
		}
		
		public FieldWithMetaSettlementRateOptionEnumBuilder getOrCreateFallBackSettlementRateOption(int index) {
			if (fallBackSettlementRateOption==null) {
				this.fallBackSettlementRateOption = new ArrayList<>();
			}
			return getIndex(fallBackSettlementRateOption, index, ()->new FieldWithMetaSettlementRateOptionEnumBuilder());
		}
		
		
		public Boolean getFallbackSurveyValuationPostponement() {
			return fallbackSurveyValuationPostponement;
		}
		
		
		public ValuationPostponement.ValuationPostponementBuilder getValuationPostponement() {
			return valuationPostponement;
		}
		
		public ValuationPostponement.ValuationPostponementBuilder getOrCreateValuationPostponement() {
			if (valuationPostponement!=null) {
				return valuationPostponement;
			}
			else return valuationPostponement = new ValuationPostponement.ValuationPostponementBuilder();
		}
		
	
		public FallbackReferencePriceBuilder setCalculationAgentDetermination(CalculationAgent calculationAgentDetermination) {
			this.calculationAgentDetermination = calculationAgentDetermination.toBuilder();
			return this;
		}
		public FallbackReferencePriceBuilder setCalculationAgentDeterminationBuilder(CalculationAgent.CalculationAgentBuilder calculationAgentDetermination) {
			this.calculationAgentDetermination = calculationAgentDetermination;
			return this;
		}
		public FallbackReferencePriceBuilder addFallBackSettlementRateOption(FieldWithMetaSettlementRateOptionEnum fallBackSettlementRateOption) {
			if(this.fallBackSettlementRateOption == null){
				this.fallBackSettlementRateOption = new ArrayList<>();
			}
			this.fallBackSettlementRateOption.add(fallBackSettlementRateOption.toBuilder());
			return this;
		}
		public FallbackReferencePriceBuilder addFallBackSettlementRateOption(FieldWithMetaSettlementRateOptionEnum fallBackSettlementRateOption, int _idx) {
			if(this.fallBackSettlementRateOption == null){
				this.fallBackSettlementRateOption = new ArrayList<>();
			}
			getIndex(this.fallBackSettlementRateOption, _idx, () -> fallBackSettlementRateOption.toBuilder());
			this.fallBackSettlementRateOption.set(_idx, fallBackSettlementRateOption.toBuilder());
			return this;
		}
		public FallbackReferencePriceBuilder addFallBackSettlementRateOption(List<FieldWithMetaSettlementRateOptionEnum> fallBackSettlementRateOptions) {
			if(this.fallBackSettlementRateOption == null){
				this.fallBackSettlementRateOption = new ArrayList<>();
			}
			for (FieldWithMetaSettlementRateOptionEnum toAdd : fallBackSettlementRateOptions) {
				this.fallBackSettlementRateOption.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public FallbackReferencePriceBuilder clearFallBackSettlementRateOption() {
			this.fallBackSettlementRateOption = null;
			return this;
		}
		public FallbackReferencePriceBuilder setFallbackSurveyValuationPostponement(Boolean fallbackSurveyValuationPostponement) {
			this.fallbackSurveyValuationPostponement = fallbackSurveyValuationPostponement;
			return this;
		}
		public FallbackReferencePriceBuilder setValuationPostponement(ValuationPostponement valuationPostponement) {
			this.valuationPostponement = valuationPostponement.toBuilder();
			return this;
		}
		public FallbackReferencePriceBuilder setValuationPostponementBuilder(ValuationPostponement.ValuationPostponementBuilder valuationPostponement) {
			this.valuationPostponement = valuationPostponement;
			return this;
		}
		
		public FallbackReferencePrice build() {
			return new FallbackReferencePrice(this);
		}
	
		@Override
		public FallbackReferencePriceBuilder prune() {
			if (calculationAgentDetermination!=null && !calculationAgentDetermination.prune().hasData()) calculationAgentDetermination = null;
			if (fallBackSettlementRateOption!=null) fallBackSettlementRateOption = fallBackSettlementRateOption.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (valuationPostponement!=null && !valuationPostponement.prune().hasData()) valuationPostponement = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCalculationAgentDetermination()!=null && getCalculationAgentDetermination().hasData()) return true;
			if (getFallBackSettlementRateOption()!=null && !getFallBackSettlementRateOption().isEmpty()) return true;
			if (getFallbackSurveyValuationPostponement()!=null) return true;
			if (getValuationPostponement()!=null && getValuationPostponement().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("fallbackSurveyValuationPostponement"), Boolean.class, fallbackSurveyValuationPostponement, this);
			
			processRosetta(path.newSubPath("calculationAgentDetermination"), processor, CalculationAgent.class, calculationAgentDetermination);
			processRosetta(path.newSubPath("fallBackSettlementRateOption"), processor, FieldWithMetaSettlementRateOptionEnum.class, fallBackSettlementRateOption);
			processRosetta(path.newSubPath("valuationPostponement"), processor, ValuationPostponement.class, valuationPostponement);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FallbackReferencePriceBuilder _that = (FallbackReferencePriceBuilder) o;
		
			if (calculationAgentDetermination != null ? !calculationAgentDetermination.equals(_that.calculationAgentDetermination) : _that.calculationAgentDetermination != null) return false;
			if (!ListEquals.listEquals(fallBackSettlementRateOption, _that.fallBackSettlementRateOption)) return false;
			if (fallbackSurveyValuationPostponement != null ? !fallbackSurveyValuationPostponement.equals(_that.fallbackSurveyValuationPostponement) : _that.fallbackSurveyValuationPostponement != null) return false;
			if (valuationPostponement != null ? !valuationPostponement.equals(_that.valuationPostponement) : _that.valuationPostponement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (calculationAgentDetermination != null ? calculationAgentDetermination.hashCode() : 0);
			_result = 31 * _result + (fallBackSettlementRateOption != null ? fallBackSettlementRateOption.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (fallbackSurveyValuationPostponement != null ? fallbackSurveyValuationPostponement.hashCode() : 0);
			_result = 31 * _result + (valuationPostponement != null ? valuationPostponement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FallbackReferencePriceBuilder {" +
				"calculationAgentDetermination=" + this.calculationAgentDetermination + ", " +
				"fallBackSettlementRateOption=" + this.fallBackSettlementRateOption + ", " +
				"fallbackSurveyValuationPostponement=" + this.fallbackSurveyValuationPostponement + ", " +
				"valuationPostponement=" + this.valuationPostponement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("fallbackSurveyValuationPostponement"), Boolean.class, fallbackSurveyValuationPostponement, this);
		
		processRosetta(path.newSubPath("calculationAgentDetermination"), processor, CalculationAgent.class, calculationAgentDetermination);
		processRosetta(path.newSubPath("fallBackSettlementRateOption"), processor, FieldWithMetaSettlementRateOptionEnum.class, fallBackSettlementRateOption);
		processRosetta(path.newSubPath("valuationPostponement"), processor, ValuationPostponement.class, valuationPostponement);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FallbackReferencePrice _that = (FallbackReferencePrice) o;
	
		if (calculationAgentDetermination != null ? !calculationAgentDetermination.equals(_that.calculationAgentDetermination) : _that.calculationAgentDetermination != null) return false;
		if (!ListEquals.listEquals(fallBackSettlementRateOption, _that.fallBackSettlementRateOption)) return false;
		if (fallbackSurveyValuationPostponement != null ? !fallbackSurveyValuationPostponement.equals(_that.fallbackSurveyValuationPostponement) : _that.fallbackSurveyValuationPostponement != null) return false;
		if (valuationPostponement != null ? !valuationPostponement.equals(_that.valuationPostponement) : _that.valuationPostponement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (calculationAgentDetermination != null ? calculationAgentDetermination.hashCode() : 0);
		_result = 31 * _result + (fallBackSettlementRateOption != null ? fallBackSettlementRateOption.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		_result = 31 * _result + (fallbackSurveyValuationPostponement != null ? fallbackSurveyValuationPostponement.hashCode() : 0);
		_result = 31 * _result + (valuationPostponement != null ? valuationPostponement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FallbackReferencePrice {" +
			"calculationAgentDetermination=" + this.calculationAgentDetermination + ", " +
			"fallBackSettlementRateOption=" + this.fallBackSettlementRateOption + ", " +
			"fallbackSurveyValuationPostponement=" + this.fallbackSurveyValuationPostponement + ", " +
			"valuationPostponement=" + this.valuationPostponement +
		'}';
	}
}
