package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.TimeTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FeaturePaymentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * Payment made following trigger occurrence.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FeaturePayment", source="FpML_5_10")
@RosettaSynonym(value="FeaturePayment", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="FeaturePayment", source="DTCC_11_0")
@RosettaSynonym(value="FeaturePayment", source="DTCC_9_0")
@RosettaSynonym(value="FeaturePayment", source="CME_ClearedConfirm_1_17")
public class FeaturePayment extends RosettaModelObject implements GlobalKey {
	private final BigDecimal amount;
	private final FieldWithMetaString currency;
	private final BigDecimal levelPercentage;
	private final MetaFields meta;
	private final PayerReceiver payerReceiver;
	private final AdjustableOrRelativeDate paymentDate;
	private final TimeTypeEnum time;
	private static FeaturePaymentMeta metaData = new FeaturePaymentMeta();
	
	FeaturePayment(FeaturePaymentBuilder builder) {
		this.amount = builder.getAmount();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.levelPercentage = builder.getLevelPercentage();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.payerReceiver = ofNullable(builder.getPayerReceiver()).map(f->f.build()).orElse(null);
		this.paymentDate = ofNullable(builder.getPaymentDate()).map(f->f.build()).orElse(null);
		this.time = builder.getTime();
	}
	
	/**
	 * The monetary quantity in currency units.
	 */
	@RosettaSynonym(value="amount", source="FpML_5_10")
	@RosettaSynonym(value="amount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="amount", source="DTCC_11_0")
	@RosettaSynonym(value="amount", source="DTCC_9_0")
	@RosettaSynonym(value="amount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getAmount() {
		return amount;
	}
	
	/**
	 * The currency in which an amount is denominated.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency", source="DTCC_11_0")
	@RosettaSynonym(value="currency", source="DTCC_9_0")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * The trigger level percentage.
	 */
	@RosettaSynonym(value="levelPercentage", source="FpML_5_10")
	@RosettaSynonym(value="levelPercentage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="levelPercentage", source="DTCC_11_0")
	@RosettaSynonym(value="levelPercentage", source="DTCC_9_0")
	@RosettaSynonym(value="levelPercentage", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getLevelPercentage() {
		return levelPercentage;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * This attribute doesn&#39;t exist as part of the FpML construct, which makes use of the PayerReceiver.model group.
	 */
	public final PayerReceiver getPayerReceiver() {
		return payerReceiver;
	}
	
	/**
	 * The feature payment date.
	 */
	@RosettaSynonym(value="featurePaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="featurePaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="featurePaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="featurePaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="featurePaymentDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getPaymentDate() {
		return paymentDate;
	}
	
	/**
	 * The feature payment time.
	 */
	@RosettaSynonym(value="time", source="FpML_5_10")
	@RosettaSynonym(value="time", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="time", source="DTCC_11_0")
	@RosettaSynonym(value="time", source="DTCC_9_0")
	@RosettaSynonym(value="time", source="CME_ClearedConfirm_1_17")
	public final TimeTypeEnum getTime() {
		return time;
	}
	
	
	@Override
	public RosettaMetaData<? extends FeaturePayment> metaData() {
		return metaData;
	} 
	
	public FeaturePaymentBuilder toBuilder() {
		FeaturePaymentBuilder builder = new FeaturePaymentBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getLevelPercentage()).ifPresent(builder::setLevelPercentage);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPayerReceiver()).ifPresent(builder::setPayerReceiver);
		ofNullable(getPaymentDate()).ifPresent(builder::setPaymentDate);
		ofNullable(getTime()).ifPresent(builder::setTime);
		return builder;
	}

	public static FeaturePaymentBuilder builder() {
		return new FeaturePaymentBuilder();
	}

	public static  class FeaturePaymentBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<FeaturePaymentBuilder> {
	
		protected BigDecimal amount;
		protected FieldWithMetaStringBuilder currency;
		protected BigDecimal levelPercentage;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayerReceiver.PayerReceiverBuilder payerReceiver;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate;
		protected TimeTypeEnum time;
	
		public FeaturePaymentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FeaturePayment> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmount() {
			return amount;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public BigDecimal getLevelPercentage() {
			return levelPercentage;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayerReceiver.PayerReceiverBuilder getPayerReceiver() {
			return payerReceiver;
		}
		
		public PayerReceiver.PayerReceiverBuilder getOrCreatePayerReceiver() {
			if (payerReceiver!=null) {
				return payerReceiver;
			}
			else return payerReceiver = new PayerReceiver.PayerReceiverBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getPaymentDate() {
			return paymentDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreatePaymentDate() {
			if (paymentDate!=null) {
				return paymentDate;
			}
			else return paymentDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public TimeTypeEnum getTime() {
			return time;
		}
		
	
		public FeaturePaymentBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		public FeaturePaymentBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public FeaturePaymentBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public FeaturePaymentBuilder setLevelPercentage(BigDecimal levelPercentage) {
			this.levelPercentage = levelPercentage;
			return this;
		}
		public FeaturePaymentBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public FeaturePaymentBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public FeaturePaymentBuilder setPayerReceiver(PayerReceiver payerReceiver) {
			this.payerReceiver = payerReceiver.toBuilder();
			return this;
		}
		public FeaturePaymentBuilder setPayerReceiverBuilder(PayerReceiver.PayerReceiverBuilder payerReceiver) {
			this.payerReceiver = payerReceiver;
			return this;
		}
		public FeaturePaymentBuilder setPaymentDate(AdjustableOrRelativeDate paymentDate) {
			this.paymentDate = paymentDate.toBuilder();
			return this;
		}
		public FeaturePaymentBuilder setPaymentDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate) {
			this.paymentDate = paymentDate;
			return this;
		}
		public FeaturePaymentBuilder setTime(TimeTypeEnum time) {
			this.time = time;
			return this;
		}
		
		public FeaturePayment build() {
			return new FeaturePayment(this);
		}
	
		@Override
		public FeaturePaymentBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (payerReceiver!=null && !payerReceiver.prune().hasData()) payerReceiver = null;
			if (paymentDate!=null && !paymentDate.prune().hasData()) paymentDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmount()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getLevelPercentage()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPayerReceiver()!=null && getPayerReceiver().hasData()) return true;
			if (getPaymentDate()!=null && getPaymentDate().hasData()) return true;
			if (getTime()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
			processor.processBasic(path.newSubPath("levelPercentage"), BigDecimal.class, levelPercentage, this);
			processor.processBasic(path.newSubPath("time"), TimeTypeEnum.class, time, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
			processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FeaturePaymentBuilder _that = (FeaturePaymentBuilder) o;
		
			if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (levelPercentage != null ? !levelPercentage.equals(_that.levelPercentage) : _that.levelPercentage != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
			if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
			if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (levelPercentage != null ? levelPercentage.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
			_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
			_result = 31 * _result + (time != null ? time.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FeaturePaymentBuilder {" +
				"amount=" + this.amount + ", " +
				"currency=" + this.currency + ", " +
				"levelPercentage=" + this.levelPercentage + ", " +
				"meta=" + this.meta + ", " +
				"payerReceiver=" + this.payerReceiver + ", " +
				"paymentDate=" + this.paymentDate + ", " +
				"time=" + this.time +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amount"), BigDecimal.class, amount, this);
		processor.processBasic(path.newSubPath("levelPercentage"), BigDecimal.class, levelPercentage, this);
		processor.processBasic(path.newSubPath("time"), TimeTypeEnum.class, time, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("payerReceiver"), processor, PayerReceiver.class, payerReceiver);
		processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FeaturePayment _that = (FeaturePayment) o;
	
		if (amount != null ? !amount.equals(_that.amount) : _that.amount != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (levelPercentage != null ? !levelPercentage.equals(_that.levelPercentage) : _that.levelPercentage != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (payerReceiver != null ? !payerReceiver.equals(_that.payerReceiver) : _that.payerReceiver != null) return false;
		if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
		if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amount != null ? amount.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (levelPercentage != null ? levelPercentage.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (payerReceiver != null ? payerReceiver.hashCode() : 0);
		_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
		_result = 31 * _result + (time != null ? time.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FeaturePayment {" +
			"amount=" + this.amount + ", " +
			"currency=" + this.currency + ", " +
			"levelPercentage=" + this.levelPercentage + ", " +
			"meta=" + this.meta + ", " +
			"payerReceiver=" + this.payerReceiver + ", " +
			"paymentDate=" + this.paymentDate + ", " +
			"time=" + this.time +
		'}';
	}
}
