package org.isda.cdm;

import org.isda.cdm.Othr;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FinInstrmMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class FinInstrm extends RosettaModelObject {
	private final Othr othr;
	private static FinInstrmMeta metaData = new FinInstrmMeta();
	
	FinInstrm(FinInstrmBuilder builder) {
		this.othr = ofNullable(builder.getOthr()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Othr", source="ISO20022")
	public final Othr getOthr() {
		return othr;
	}
	
	
	@Override
	public RosettaMetaData<? extends FinInstrm> metaData() {
		return metaData;
	} 
	
	public FinInstrmBuilder toBuilder() {
		FinInstrmBuilder builder = new FinInstrmBuilder();
		ofNullable(getOthr()).ifPresent(builder::setOthr);
		return builder;
	}

	public static FinInstrmBuilder builder() {
		return new FinInstrmBuilder();
	}

	public static  class FinInstrmBuilder extends RosettaModelObjectBuilder {
	
		protected Othr.OthrBuilder othr;
	
		public FinInstrmBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FinInstrm> metaData() {
			return metaData;
		} 
	
		
		public Othr.OthrBuilder getOthr() {
			return othr;
		}
		
		public Othr.OthrBuilder getOrCreateOthr() {
			if (othr!=null) {
				return othr;
			}
			else return othr = new Othr.OthrBuilder();
		}
		
	
		public FinInstrmBuilder setOthr(Othr othr) {
			this.othr = othr.toBuilder();
			return this;
		}
		public FinInstrmBuilder setOthrBuilder(Othr.OthrBuilder othr) {
			this.othr = othr;
			return this;
		}
		
		public FinInstrm build() {
			return new FinInstrm(this);
		}
	
		@Override
		public FinInstrmBuilder prune() {
			if (othr!=null && !othr.prune().hasData()) othr = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getOthr()!=null && getOthr().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("othr"), processor, Othr.class, othr);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FinInstrmBuilder _that = (FinInstrmBuilder) o;
		
			if (othr != null ? !othr.equals(_that.othr) : _that.othr != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (othr != null ? othr.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FinInstrmBuilder {" +
				"othr=" + this.othr +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("othr"), processor, Othr.class, othr);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FinInstrm _that = (FinInstrm) o;
	
		if (othr != null ? !othr.equals(_that.othr) : _that.othr != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (othr != null ? othr.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FinInstrm {" +
			"othr=" + this.othr +
		'}';
	}
}
