package org.isda.cdm;

import org.isda.cdm.BusinessDayConventionEnum;
import org.isda.cdm.metafields.ReferenceWithMetaAdjustableOrRelativeDates;
import org.isda.cdm.metafields.ReferenceWithMetaAdjustableOrRelativeDates.ReferenceWithMetaAdjustableOrRelativeDatesBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaInterestRatePayout;
import org.isda.cdm.metafields.ReferenceWithMetaInterestRatePayout.ReferenceWithMetaInterestRatePayoutBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FinalCalculationPeriodDateAdjustmentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to define business date convention adjustment to final payment period per leg.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FinalCalculationPeriodDateAdjustment", source="FpML_5_10")
public class FinalCalculationPeriodDateAdjustment extends RosettaModelObject {
	private final BusinessDayConventionEnum businessDayConvention;
	private final ReferenceWithMetaAdjustableOrRelativeDates relevantUnderlyingDateReference;
	private final ReferenceWithMetaInterestRatePayout swapStreamReference;
	private static FinalCalculationPeriodDateAdjustmentMeta metaData = new FinalCalculationPeriodDateAdjustmentMeta();
	
	FinalCalculationPeriodDateAdjustment(FinalCalculationPeriodDateAdjustmentBuilder builder) {
		this.businessDayConvention = builder.getBusinessDayConvention();
		this.relevantUnderlyingDateReference = ofNullable(builder.getRelevantUnderlyingDateReference()).map(f->f.build()).orElse(null);
		this.swapStreamReference = ofNullable(builder.getSwapStreamReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Override business date convention. This takes precedence over leg level information.
	 */
	@RosettaSynonym(value="businessDayConvention", source="FpML_5_10")
	@RosettaSynonym(value="businessDayConvention", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_11_0")
	@RosettaSynonym(value="businessDayConvention", source="DTCC_9_0")
	@RosettaSynonym(value="businessDayConvention", source="CME_ClearedConfirm_1_17")
	public final BusinessDayConventionEnum getBusinessDayConvention() {
		return businessDayConvention;
	}
	
	/**
	 * Reference to the unadjusted cancellation effective dates.
	 */
	@RosettaSynonym(value="relevantUnderlyingDateReference", source="FpML_5_10")
	@RosettaSynonym(value="relevantUnderlyingDateReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relevantUnderlyingDateReference", source="DTCC_11_0")
	@RosettaSynonym(value="relevantUnderlyingDateReference", source="DTCC_9_0")
	@RosettaSynonym(value="relevantUnderlyingDateReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAdjustableOrRelativeDates getRelevantUnderlyingDateReference() {
		return relevantUnderlyingDateReference;
	}
	
	/**
	 * Reference to the leg, where date adjustments may apply.
	 */
	@RosettaSynonym(value="swapStreamReference", source="FpML_5_10")
	@RosettaSynonym(value="swapStreamReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="swapStreamReference", source="DTCC_11_0")
	@RosettaSynonym(value="swapStreamReference", source="DTCC_9_0")
	@RosettaSynonym(value="swapStreamReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaInterestRatePayout getSwapStreamReference() {
		return swapStreamReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends FinalCalculationPeriodDateAdjustment> metaData() {
		return metaData;
	} 
	
	public FinalCalculationPeriodDateAdjustmentBuilder toBuilder() {
		FinalCalculationPeriodDateAdjustmentBuilder builder = new FinalCalculationPeriodDateAdjustmentBuilder();
		ofNullable(getBusinessDayConvention()).ifPresent(builder::setBusinessDayConvention);
		ofNullable(getRelevantUnderlyingDateReference()).ifPresent(builder::setRelevantUnderlyingDateReference);
		ofNullable(getSwapStreamReference()).ifPresent(builder::setSwapStreamReference);
		return builder;
	}

	public static FinalCalculationPeriodDateAdjustmentBuilder builder() {
		return new FinalCalculationPeriodDateAdjustmentBuilder();
	}

	public static  class FinalCalculationPeriodDateAdjustmentBuilder extends RosettaModelObjectBuilder {
	
		protected BusinessDayConventionEnum businessDayConvention;
		protected ReferenceWithMetaAdjustableOrRelativeDatesBuilder relevantUnderlyingDateReference;
		protected ReferenceWithMetaInterestRatePayoutBuilder swapStreamReference;
	
		public FinalCalculationPeriodDateAdjustmentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FinalCalculationPeriodDateAdjustment> metaData() {
			return metaData;
		} 
	
		
		public BusinessDayConventionEnum getBusinessDayConvention() {
			return businessDayConvention;
		}
		
		
		public ReferenceWithMetaAdjustableOrRelativeDatesBuilder getRelevantUnderlyingDateReference() {
			return relevantUnderlyingDateReference;
		}
		
		public ReferenceWithMetaAdjustableOrRelativeDatesBuilder getOrCreateRelevantUnderlyingDateReference() {
			if (relevantUnderlyingDateReference!=null) {
				return relevantUnderlyingDateReference;
			}
			else return relevantUnderlyingDateReference = new ReferenceWithMetaAdjustableOrRelativeDatesBuilder();
		}
		
		
		public ReferenceWithMetaInterestRatePayoutBuilder getSwapStreamReference() {
			return swapStreamReference;
		}
		
		public ReferenceWithMetaInterestRatePayoutBuilder getOrCreateSwapStreamReference() {
			if (swapStreamReference!=null) {
				return swapStreamReference;
			}
			else return swapStreamReference = new ReferenceWithMetaInterestRatePayoutBuilder();
		}
		
	
		public FinalCalculationPeriodDateAdjustmentBuilder setBusinessDayConvention(BusinessDayConventionEnum businessDayConvention) {
			this.businessDayConvention = businessDayConvention;
			return this;
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setRelevantUnderlyingDateReference(ReferenceWithMetaAdjustableOrRelativeDates relevantUnderlyingDateReference) {
			this.relevantUnderlyingDateReference = relevantUnderlyingDateReference.toBuilder();
			return this;
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setRelevantUnderlyingDateReferenceBuilder(ReferenceWithMetaAdjustableOrRelativeDatesBuilder relevantUnderlyingDateReference) {
			this.relevantUnderlyingDateReference = relevantUnderlyingDateReference;
			return this;
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setRelevantUnderlyingDateReferenceRef(AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder relevantUnderlyingDateReference) {
			return setRelevantUnderlyingDateReference(ReferenceWithMetaAdjustableOrRelativeDates.builder().setValueBuilder(relevantUnderlyingDateReference).build());
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setRelevantUnderlyingDateReferenceRef(AdjustableOrRelativeDates relevantUnderlyingDateReference) {
			return setRelevantUnderlyingDateReferenceRef(AdjustableOrRelativeDates.builder());
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setSwapStreamReference(ReferenceWithMetaInterestRatePayout swapStreamReference) {
			this.swapStreamReference = swapStreamReference.toBuilder();
			return this;
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setSwapStreamReferenceBuilder(ReferenceWithMetaInterestRatePayoutBuilder swapStreamReference) {
			this.swapStreamReference = swapStreamReference;
			return this;
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setSwapStreamReferenceRef(InterestRatePayout.InterestRatePayoutBuilder swapStreamReference) {
			return setSwapStreamReference(ReferenceWithMetaInterestRatePayout.builder().setValueBuilder(swapStreamReference).build());
		}
		public FinalCalculationPeriodDateAdjustmentBuilder setSwapStreamReferenceRef(InterestRatePayout swapStreamReference) {
			return setSwapStreamReferenceRef(InterestRatePayout.builder());
		}
		
		public FinalCalculationPeriodDateAdjustment build() {
			return new FinalCalculationPeriodDateAdjustment(this);
		}
	
		@Override
		public FinalCalculationPeriodDateAdjustmentBuilder prune() {
			if (relevantUnderlyingDateReference!=null && !relevantUnderlyingDateReference.prune().hasData()) relevantUnderlyingDateReference = null;
			if (swapStreamReference!=null && !swapStreamReference.prune().hasData()) swapStreamReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessDayConvention()!=null) return true;
			if (getRelevantUnderlyingDateReference()!=null && getRelevantUnderlyingDateReference().hasData()) return true;
			if (getSwapStreamReference()!=null && getSwapStreamReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
			
			processRosetta(path.newSubPath("relevantUnderlyingDateReference"), processor, ReferenceWithMetaAdjustableOrRelativeDates.class, relevantUnderlyingDateReference);
			processRosetta(path.newSubPath("swapStreamReference"), processor, ReferenceWithMetaInterestRatePayout.class, swapStreamReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FinalCalculationPeriodDateAdjustmentBuilder _that = (FinalCalculationPeriodDateAdjustmentBuilder) o;
		
			if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
			if (relevantUnderlyingDateReference != null ? !relevantUnderlyingDateReference.equals(_that.relevantUnderlyingDateReference) : _that.relevantUnderlyingDateReference != null) return false;
			if (swapStreamReference != null ? !swapStreamReference.equals(_that.swapStreamReference) : _that.swapStreamReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (relevantUnderlyingDateReference != null ? relevantUnderlyingDateReference.hashCode() : 0);
			_result = 31 * _result + (swapStreamReference != null ? swapStreamReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FinalCalculationPeriodDateAdjustmentBuilder {" +
				"businessDayConvention=" + this.businessDayConvention + ", " +
				"relevantUnderlyingDateReference=" + this.relevantUnderlyingDateReference + ", " +
				"swapStreamReference=" + this.swapStreamReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("businessDayConvention"), BusinessDayConventionEnum.class, businessDayConvention, this);
		
		processRosetta(path.newSubPath("relevantUnderlyingDateReference"), processor, ReferenceWithMetaAdjustableOrRelativeDates.class, relevantUnderlyingDateReference);
		processRosetta(path.newSubPath("swapStreamReference"), processor, ReferenceWithMetaInterestRatePayout.class, swapStreamReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FinalCalculationPeriodDateAdjustment _that = (FinalCalculationPeriodDateAdjustment) o;
	
		if (businessDayConvention != null ? !businessDayConvention.equals(_that.businessDayConvention) : _that.businessDayConvention != null) return false;
		if (relevantUnderlyingDateReference != null ? !relevantUnderlyingDateReference.equals(_that.relevantUnderlyingDateReference) : _that.relevantUnderlyingDateReference != null) return false;
		if (swapStreamReference != null ? !swapStreamReference.equals(_that.swapStreamReference) : _that.swapStreamReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessDayConvention != null ? businessDayConvention.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (relevantUnderlyingDateReference != null ? relevantUnderlyingDateReference.hashCode() : 0);
		_result = 31 * _result + (swapStreamReference != null ? swapStreamReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FinalCalculationPeriodDateAdjustment {" +
			"businessDayConvention=" + this.businessDayConvention + ", " +
			"relevantUnderlyingDateReference=" + this.relevantUnderlyingDateReference + ", " +
			"swapStreamReference=" + this.swapStreamReference +
		'}';
	}
}
