package org.isda.cdm;

import java.lang.Boolean;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FloatingAmountProvisionsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FloatingAmountProvisions", source="FpML_5_10")
public class FloatingAmountProvisions extends RosettaModelObject {
	private final Boolean stepUpProvision;
	private final Boolean wacCapInterestProvision;
	private static FloatingAmountProvisionsMeta metaData = new FloatingAmountProvisionsMeta();
	
	FloatingAmountProvisions(FloatingAmountProvisionsBuilder builder) {
		this.stepUpProvision = builder.getStepUpProvision();
		this.wacCapInterestProvision = builder.getWacCapInterestProvision();
	}
	
	/**
	 * As specified by the ISDA Standard Terms Supplement for use with trades on mortgage-backed securities. The presence of the element with value set to &#39;true&#39; signifies that the provision is applicable. If applicable, the applicable step-up terms are specified as part of that ISDA Standard Terms Supplement. From a usage standpoint, this provision is typically applicable in the case of RMBS and not applicable in case of CMBS trades.
	 */
	@RosettaSynonym(value="stepUpProvision", source="FpML_5_10")
	@RosettaSynonym(value="stepUpProvision", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stepUpProvision", source="DTCC_11_0")
	@RosettaSynonym(value="stepUpProvision", source="DTCC_9_0")
	@RosettaSynonym(value="stepUpProvision", source="CME_ClearedConfirm_1_17")
	public final Boolean getStepUpProvision() {
		return stepUpProvision;
	}
	
	/**
	 * As specified by the ISDA Supplement for use with trades on mortgage-backed securities, &#39;WAC Cap&#39; means a weighted average coupon or weighted average rate cap provision (however defined in the Underlying Instruments) of the Underlying Instruments that limits, increases or decreases the interest rate or interest entitlement, as set out in the Underlying Instruments on the Effective Date without regard to any subsequent amendment The presence of the element with value set to &#39;true&#39; signifies that the provision is applicable. From a usage standpoint, this provision is typically applicable in the case of CMBS and not applicable in case of RMBS trades.
	 */
	@RosettaSynonym(value="WACCapInterestProvision", source="FpML_5_10")
	@RosettaSynonym(value="WACCapInterestProvision", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="WACCapInterestProvision", source="DTCC_11_0")
	@RosettaSynonym(value="WACCapInterestProvision", source="DTCC_9_0")
	@RosettaSynonym(value="WACCapInterestProvision", source="CME_ClearedConfirm_1_17")
	public final Boolean getWacCapInterestProvision() {
		return wacCapInterestProvision;
	}
	
	
	@Override
	public RosettaMetaData<? extends FloatingAmountProvisions> metaData() {
		return metaData;
	} 
	
	public FloatingAmountProvisionsBuilder toBuilder() {
		FloatingAmountProvisionsBuilder builder = new FloatingAmountProvisionsBuilder();
		ofNullable(getStepUpProvision()).ifPresent(builder::setStepUpProvision);
		ofNullable(getWacCapInterestProvision()).ifPresent(builder::setWacCapInterestProvision);
		return builder;
	}

	public static FloatingAmountProvisionsBuilder builder() {
		return new FloatingAmountProvisionsBuilder();
	}

	public static  class FloatingAmountProvisionsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean stepUpProvision;
		protected Boolean wacCapInterestProvision;
	
		public FloatingAmountProvisionsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FloatingAmountProvisions> metaData() {
			return metaData;
		} 
	
		
		public Boolean getStepUpProvision() {
			return stepUpProvision;
		}
		
		
		public Boolean getWacCapInterestProvision() {
			return wacCapInterestProvision;
		}
		
	
		public FloatingAmountProvisionsBuilder setStepUpProvision(Boolean stepUpProvision) {
			this.stepUpProvision = stepUpProvision;
			return this;
		}
		public FloatingAmountProvisionsBuilder setWacCapInterestProvision(Boolean wacCapInterestProvision) {
			this.wacCapInterestProvision = wacCapInterestProvision;
			return this;
		}
		
		public FloatingAmountProvisions build() {
			return new FloatingAmountProvisions(this);
		}
	
		@Override
		public FloatingAmountProvisionsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getStepUpProvision()!=null) return true;
			if (getWacCapInterestProvision()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("stepUpProvision"), Boolean.class, stepUpProvision, this);
			processor.processBasic(path.newSubPath("wacCapInterestProvision"), Boolean.class, wacCapInterestProvision, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FloatingAmountProvisionsBuilder _that = (FloatingAmountProvisionsBuilder) o;
		
			if (stepUpProvision != null ? !stepUpProvision.equals(_that.stepUpProvision) : _that.stepUpProvision != null) return false;
			if (wacCapInterestProvision != null ? !wacCapInterestProvision.equals(_that.wacCapInterestProvision) : _that.wacCapInterestProvision != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (stepUpProvision != null ? stepUpProvision.hashCode() : 0);
			_result = 31 * _result + (wacCapInterestProvision != null ? wacCapInterestProvision.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FloatingAmountProvisionsBuilder {" +
				"stepUpProvision=" + this.stepUpProvision + ", " +
				"wacCapInterestProvision=" + this.wacCapInterestProvision +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("stepUpProvision"), Boolean.class, stepUpProvision, this);
		processor.processBasic(path.newSubPath("wacCapInterestProvision"), Boolean.class, wacCapInterestProvision, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FloatingAmountProvisions _that = (FloatingAmountProvisions) o;
	
		if (stepUpProvision != null ? !stepUpProvision.equals(_that.stepUpProvision) : _that.stepUpProvision != null) return false;
		if (wacCapInterestProvision != null ? !wacCapInterestProvision.equals(_that.wacCapInterestProvision) : _that.wacCapInterestProvision != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (stepUpProvision != null ? stepUpProvision.hashCode() : 0);
		_result = 31 * _result + (wacCapInterestProvision != null ? wacCapInterestProvision.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FloatingAmountProvisions {" +
			"stepUpProvision=" + this.stepUpProvision + ", " +
			"wacCapInterestProvision=" + this.wacCapInterestProvision +
		'}';
	}
}
