package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.Integer;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FutureValueAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a currency amount as at a future value date.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FutureValueAmount", source="FpML_5_10")
public class FutureValueAmount extends Money {
	private final Integer calculationPeriodNumberOfDays;
	private final Date valueDate;
	private static FutureValueAmountMeta metaData = new FutureValueAmountMeta();
	
	FutureValueAmount(FutureValueAmountBuilder builder) {
		super(builder);
		this.calculationPeriodNumberOfDays = builder.getCalculationPeriodNumberOfDays();
		this.valueDate = builder.getValueDate();
	}
	
	/**
	 * The number of days from the adjusted calculation period start date to the adjusted value date, calculated in accordance with the applicable day count fraction.
	 */
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriodNumberOfDays", source="CME_ClearedConfirm_1_17")
	public final Integer getCalculationPeriodNumberOfDays() {
		return calculationPeriodNumberOfDays;
	}
	
	/**
	 * Adjusted value date of the future value amount.
	 */
	@RosettaSynonym(value="valueDate", source="FpML_5_10")
	@RosettaSynonym(value="valueDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="valueDate", source="DTCC_11_0")
	@RosettaSynonym(value="valueDate", source="DTCC_9_0")
	@RosettaSynonym(value="valueDate", source="CME_ClearedConfirm_1_17")
	public final Date getValueDate() {
		return valueDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends FutureValueAmount> metaData() {
		return metaData;
	} 
	
	public FutureValueAmountBuilder toBuilder() {
		FutureValueAmountBuilder builder = new FutureValueAmountBuilder();
		ofNullable(getCalculationPeriodNumberOfDays()).ifPresent(builder::setCalculationPeriodNumberOfDays);
		ofNullable(getValueDate()).ifPresent(builder::setValueDate);
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static FutureValueAmountBuilder builder() {
		return new FutureValueAmountBuilder();
	}

	public static  class FutureValueAmountBuilder extends MoneyBuilder {
	
		protected Integer calculationPeriodNumberOfDays;
		protected Date valueDate;
	
		public FutureValueAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FutureValueAmount> metaData() {
			return metaData;
		} 
	
		
		public Integer getCalculationPeriodNumberOfDays() {
			return calculationPeriodNumberOfDays;
		}
		
		
		public Date getValueDate() {
			return valueDate;
		}
		
	
		public FutureValueAmountBuilder setCalculationPeriodNumberOfDays(Integer calculationPeriodNumberOfDays) {
			this.calculationPeriodNumberOfDays = calculationPeriodNumberOfDays;
			return this;
		}
		public FutureValueAmountBuilder setValueDate(Date valueDate) {
			this.valueDate = valueDate;
			return this;
		}
		@Override public FutureValueAmountBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		@Override public FutureValueAmountBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		@Override public FutureValueAmountBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public FutureValueAmountBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public FutureValueAmountBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public FutureValueAmount build() {
			return new FutureValueAmount(this);
		}
	
		@Override
		public FutureValueAmountBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCalculationPeriodNumberOfDays()!=null) return true;
			if (getValueDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("calculationPeriodNumberOfDays"), Integer.class, calculationPeriodNumberOfDays, this);
			processor.processBasic(path.newSubPath("valueDate"), Date.class, valueDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			FutureValueAmountBuilder _that = (FutureValueAmountBuilder) o;
		
			if (calculationPeriodNumberOfDays != null ? !calculationPeriodNumberOfDays.equals(_that.calculationPeriodNumberOfDays) : _that.calculationPeriodNumberOfDays != null) return false;
			if (valueDate != null ? !valueDate.equals(_that.valueDate) : _that.valueDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (calculationPeriodNumberOfDays != null ? calculationPeriodNumberOfDays.hashCode() : 0);
			_result = 31 * _result + (valueDate != null ? valueDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FutureValueAmountBuilder {" +
				"calculationPeriodNumberOfDays=" + this.calculationPeriodNumberOfDays + ", " +
				"valueDate=" + this.valueDate +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("calculationPeriodNumberOfDays"), Integer.class, calculationPeriodNumberOfDays, this);
		processor.processBasic(path.newSubPath("valueDate"), Date.class, valueDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		FutureValueAmount _that = (FutureValueAmount) o;
	
		if (calculationPeriodNumberOfDays != null ? !calculationPeriodNumberOfDays.equals(_that.calculationPeriodNumberOfDays) : _that.calculationPeriodNumberOfDays != null) return false;
		if (valueDate != null ? !valueDate.equals(_that.valueDate) : _that.valueDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (calculationPeriodNumberOfDays != null ? calculationPeriodNumberOfDays.hashCode() : 0);
		_result = 31 * _result + (valueDate != null ? valueDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FutureValueAmount {" +
			"calculationPeriodNumberOfDays=" + this.calculationPeriodNumberOfDays + ", " +
			"valueDate=" + this.valueDate +
		'}' + " " + super.toString();
	}
}
