package org.isda.cdm;

import org.isda.cdm.Composite;
import org.isda.cdm.Quanto;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxFeatureMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A type for defining FX Features.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxFeature", source="FpML_5_10")
public class FxFeature extends RosettaModelObject {
	private final Composite composite;
	private final Composite crossCurrency;
	private final Quanto quanto;
	private final FieldWithMetaString referenceCurrency;
	private static FxFeatureMeta metaData = new FxFeatureMeta();
	
	FxFeature(FxFeatureBuilder builder) {
		this.composite = ofNullable(builder.getComposite()).map(f->f.build()).orElse(null);
		this.crossCurrency = ofNullable(builder.getCrossCurrency()).map(f->f.build()).orElse(null);
		this.quanto = ofNullable(builder.getQuanto()).map(f->f.build()).orElse(null);
		this.referenceCurrency = ofNullable(builder.getReferenceCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * If &#39;Composite&#39; is specified as the Settlement Type in the relevant Transaction Supplement, an amount in the Settlement Currency, determined by the Calculation Agent as being equal to the number of Options exercised or deemed exercised, multiplied by: (Settlement Price – Strike Price) / (Strike Price – Settlement Price) x Multiplier provided that if the above is equal to a negative amount the Option Cash Settlement Amount shall be deemed to be zero.
	 */
	@RosettaSynonym(value="composite", source="FpML_5_10")
	@RosettaSynonym(value="composite", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="composite", source="DTCC_11_0")
	@RosettaSynonym(value="composite", source="DTCC_9_0")
	@RosettaSynonym(value="composite", source="CME_ClearedConfirm_1_17")
	public final Composite getComposite() {
		return composite;
	}
	
	/**
	 * If &#39;Cross-Currency&#39; is specified as the Settlement Type in the relevant Transaction Supplement, an amount in the Settlement Currency, determined by the Calculation Agent as being equal to the number of Options exercised or deemed exercised, multiplied by: (Settlement Price – Strike Price) / (Strike Price – Settlement Price) x Multiplier x one unit of the Reference Currency converted into an amount in the Settlement Currency using the rate of exchange of the Settlement Currency as quoted on the Reference Price Source on the Valuation Date, provided that if the above is equal to a negative amount the Option Cash Settlement Amount shall be deemed to be zero.
	 */
	@RosettaSynonym(value="crossCurrency", source="FpML_5_10")
	@RosettaSynonym(value="crossCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="crossCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="crossCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="crossCurrency", source="CME_ClearedConfirm_1_17")
	public final Composite getCrossCurrency() {
		return crossCurrency;
	}
	
	/**
	 * If &#39;Quanto&#39; is specified as the Settlement Type in the relevant Transaction Supplement, an amount, as determined by the Calculation Agent in accordance with the Section 8.2 of the Equity Definitions.
	 */
	@RosettaSynonym(value="quanto", source="FpML_5_10")
	@RosettaSynonym(value="quanto", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="quanto", source="DTCC_11_0")
	@RosettaSynonym(value="quanto", source="DTCC_9_0")
	@RosettaSynonym(value="quanto", source="CME_ClearedConfirm_1_17")
	public final Quanto getQuanto() {
		return quanto;
	}
	
	/**
	 * Specifies the reference currency of the trade.
	 */
	@RosettaSynonym(value="referenceCurrency", source="FpML_5_10")
	@RosettaSynonym(value="referenceCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="referenceCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="referenceCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getReferenceCurrency() {
		return referenceCurrency;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxFeature> metaData() {
		return metaData;
	} 
	
	public FxFeatureBuilder toBuilder() {
		FxFeatureBuilder builder = new FxFeatureBuilder();
		ofNullable(getComposite()).ifPresent(builder::setComposite);
		ofNullable(getCrossCurrency()).ifPresent(builder::setCrossCurrency);
		ofNullable(getQuanto()).ifPresent(builder::setQuanto);
		ofNullable(getReferenceCurrency()).ifPresent(builder::setReferenceCurrency);
		return builder;
	}

	public static FxFeatureBuilder builder() {
		return new FxFeatureBuilder();
	}

	public static  class FxFeatureBuilder extends RosettaModelObjectBuilder {
	
		protected Composite.CompositeBuilder composite;
		protected Composite.CompositeBuilder crossCurrency;
		protected Quanto.QuantoBuilder quanto;
		protected FieldWithMetaStringBuilder referenceCurrency;
	
		public FxFeatureBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxFeature> metaData() {
			return metaData;
		} 
	
		
		public Composite.CompositeBuilder getComposite() {
			return composite;
		}
		
		public Composite.CompositeBuilder getOrCreateComposite() {
			if (composite!=null) {
				return composite;
			}
			else return composite = new Composite.CompositeBuilder();
		}
		
		
		public Composite.CompositeBuilder getCrossCurrency() {
			return crossCurrency;
		}
		
		public Composite.CompositeBuilder getOrCreateCrossCurrency() {
			if (crossCurrency!=null) {
				return crossCurrency;
			}
			else return crossCurrency = new Composite.CompositeBuilder();
		}
		
		
		public Quanto.QuantoBuilder getQuanto() {
			return quanto;
		}
		
		public Quanto.QuantoBuilder getOrCreateQuanto() {
			if (quanto!=null) {
				return quanto;
			}
			else return quanto = new Quanto.QuantoBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getReferenceCurrency() {
			return referenceCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateReferenceCurrency() {
			if (referenceCurrency!=null) {
				return referenceCurrency;
			}
			else return referenceCurrency = new FieldWithMetaStringBuilder();
		}
		
	
		public FxFeatureBuilder setComposite(Composite composite) {
			this.composite = composite.toBuilder();
			return this;
		}
		public FxFeatureBuilder setCompositeBuilder(Composite.CompositeBuilder composite) {
			this.composite = composite;
			return this;
		}
		public FxFeatureBuilder setCrossCurrency(Composite crossCurrency) {
			this.crossCurrency = crossCurrency.toBuilder();
			return this;
		}
		public FxFeatureBuilder setCrossCurrencyBuilder(Composite.CompositeBuilder crossCurrency) {
			this.crossCurrency = crossCurrency;
			return this;
		}
		public FxFeatureBuilder setQuanto(Quanto quanto) {
			this.quanto = quanto.toBuilder();
			return this;
		}
		public FxFeatureBuilder setQuantoBuilder(Quanto.QuantoBuilder quanto) {
			this.quanto = quanto;
			return this;
		}
		public FxFeatureBuilder setReferenceCurrency(FieldWithMetaString referenceCurrency) {
			this.referenceCurrency = referenceCurrency.toBuilder();
			return this;
		}
		public FxFeatureBuilder setReferenceCurrencyRef(String referenceCurrency) {
			return setReferenceCurrency(FieldWithMetaString.builder().setValue(referenceCurrency).build());
		}
		
		public FxFeature build() {
			return new FxFeature(this);
		}
	
		@Override
		public FxFeatureBuilder prune() {
			if (composite!=null && !composite.prune().hasData()) composite = null;
			if (crossCurrency!=null && !crossCurrency.prune().hasData()) crossCurrency = null;
			if (quanto!=null && !quanto.prune().hasData()) quanto = null;
			if (referenceCurrency!=null && !referenceCurrency.prune().hasData()) referenceCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getComposite()!=null && getComposite().hasData()) return true;
			if (getCrossCurrency()!=null && getCrossCurrency().hasData()) return true;
			if (getQuanto()!=null && getQuanto().hasData()) return true;
			if (getReferenceCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("composite"), processor, Composite.class, composite);
			processRosetta(path.newSubPath("crossCurrency"), processor, Composite.class, crossCurrency);
			processRosetta(path.newSubPath("quanto"), processor, Quanto.class, quanto);
			processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxFeatureBuilder _that = (FxFeatureBuilder) o;
		
			if (composite != null ? !composite.equals(_that.composite) : _that.composite != null) return false;
			if (crossCurrency != null ? !crossCurrency.equals(_that.crossCurrency) : _that.crossCurrency != null) return false;
			if (quanto != null ? !quanto.equals(_that.quanto) : _that.quanto != null) return false;
			if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (composite != null ? composite.hashCode() : 0);
			_result = 31 * _result + (crossCurrency != null ? crossCurrency.hashCode() : 0);
			_result = 31 * _result + (quanto != null ? quanto.hashCode() : 0);
			_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxFeatureBuilder {" +
				"composite=" + this.composite + ", " +
				"crossCurrency=" + this.crossCurrency + ", " +
				"quanto=" + this.quanto + ", " +
				"referenceCurrency=" + this.referenceCurrency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("composite"), processor, Composite.class, composite);
		processRosetta(path.newSubPath("crossCurrency"), processor, Composite.class, crossCurrency);
		processRosetta(path.newSubPath("quanto"), processor, Quanto.class, quanto);
		processRosetta(path.newSubPath("referenceCurrency"), processor, FieldWithMetaString.class, referenceCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxFeature _that = (FxFeature) o;
	
		if (composite != null ? !composite.equals(_that.composite) : _that.composite != null) return false;
		if (crossCurrency != null ? !crossCurrency.equals(_that.crossCurrency) : _that.crossCurrency != null) return false;
		if (quanto != null ? !quanto.equals(_that.quanto) : _that.quanto != null) return false;
		if (referenceCurrency != null ? !referenceCurrency.equals(_that.referenceCurrency) : _that.referenceCurrency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (composite != null ? composite.hashCode() : 0);
		_result = 31 * _result + (crossCurrency != null ? crossCurrency.hashCode() : 0);
		_result = 31 * _result + (quanto != null ? quanto.hashCode() : 0);
		_result = 31 * _result + (referenceCurrency != null ? referenceCurrency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxFeature {" +
			"composite=" + this.composite + ", " +
			"crossCurrency=" + this.crossCurrency + ", " +
			"quanto=" + this.quanto + ", " +
			"referenceCurrency=" + this.referenceCurrency +
		'}';
	}
}
