package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxLinkedNotionalAmountMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to describe the cashflow representation for FX linked notionals.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxLinkedNotionalAmount", source="FpML_5_10")
public class FxLinkedNotionalAmount extends RosettaModelObject {
	private final Date adjustedFxSpotFixingDate;
	private final BigDecimal notionalAmount;
	private final BigDecimal observedFxSpotRate;
	private final Date resetDate;
	private static FxLinkedNotionalAmountMeta metaData = new FxLinkedNotionalAmountMeta();
	
	FxLinkedNotionalAmount(FxLinkedNotionalAmountBuilder builder) {
		this.adjustedFxSpotFixingDate = builder.getAdjustedFxSpotFixingDate();
		this.notionalAmount = builder.getNotionalAmount();
		this.observedFxSpotRate = builder.getObservedFxSpotRate();
		this.resetDate = builder.getResetDate();
	}
	
	/**
	 * The date on which the FX spot rate is observed. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedFxSpotFixingDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedFxSpotFixingDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedFxSpotFixingDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedFxSpotFixingDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedFxSpotFixingDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedFxSpotFixingDate() {
		return adjustedFxSpotFixingDate;
	}
	
	/**
	 * The calculation period notional amount.
	 */
	@RosettaSynonym(value="notionalAmount", source="FpML_5_10")
	@RosettaSynonym(value="notionalAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_11_0")
	@RosettaSynonym(value="notionalAmount", source="DTCC_9_0")
	@RosettaSynonym(value="notionalAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getNotionalAmount() {
		return notionalAmount;
	}
	
	/**
	 * The actual observed FX spot rate.
	 */
	@RosettaSynonym(value="observedFxSpotRate", source="FpML_5_10")
	@RosettaSynonym(value="observedFxSpotRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="observedFxSpotRate", source="DTCC_11_0")
	@RosettaSynonym(value="observedFxSpotRate", source="DTCC_9_0")
	@RosettaSynonym(value="observedFxSpotRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getObservedFxSpotRate() {
		return observedFxSpotRate;
	}
	
	/**
	 * The reset date.
	 */
	@RosettaSynonym(value="resetDate", source="FpML_5_10")
	@RosettaSynonym(value="resetDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="resetDate", source="DTCC_11_0")
	@RosettaSynonym(value="resetDate", source="DTCC_9_0")
	@RosettaSynonym(value="resetDate", source="CME_ClearedConfirm_1_17")
	public final Date getResetDate() {
		return resetDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxLinkedNotionalAmount> metaData() {
		return metaData;
	} 
	
	public FxLinkedNotionalAmountBuilder toBuilder() {
		FxLinkedNotionalAmountBuilder builder = new FxLinkedNotionalAmountBuilder();
		ofNullable(getAdjustedFxSpotFixingDate()).ifPresent(builder::setAdjustedFxSpotFixingDate);
		ofNullable(getNotionalAmount()).ifPresent(builder::setNotionalAmount);
		ofNullable(getObservedFxSpotRate()).ifPresent(builder::setObservedFxSpotRate);
		ofNullable(getResetDate()).ifPresent(builder::setResetDate);
		return builder;
	}

	public static FxLinkedNotionalAmountBuilder builder() {
		return new FxLinkedNotionalAmountBuilder();
	}

	public static  class FxLinkedNotionalAmountBuilder extends RosettaModelObjectBuilder {
	
		protected Date adjustedFxSpotFixingDate;
		protected BigDecimal notionalAmount;
		protected BigDecimal observedFxSpotRate;
		protected Date resetDate;
	
		public FxLinkedNotionalAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxLinkedNotionalAmount> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedFxSpotFixingDate() {
			return adjustedFxSpotFixingDate;
		}
		
		
		public BigDecimal getNotionalAmount() {
			return notionalAmount;
		}
		
		
		public BigDecimal getObservedFxSpotRate() {
			return observedFxSpotRate;
		}
		
		
		public Date getResetDate() {
			return resetDate;
		}
		
	
		public FxLinkedNotionalAmountBuilder setAdjustedFxSpotFixingDate(Date adjustedFxSpotFixingDate) {
			this.adjustedFxSpotFixingDate = adjustedFxSpotFixingDate;
			return this;
		}
		public FxLinkedNotionalAmountBuilder setNotionalAmount(BigDecimal notionalAmount) {
			this.notionalAmount = notionalAmount;
			return this;
		}
		public FxLinkedNotionalAmountBuilder setObservedFxSpotRate(BigDecimal observedFxSpotRate) {
			this.observedFxSpotRate = observedFxSpotRate;
			return this;
		}
		public FxLinkedNotionalAmountBuilder setResetDate(Date resetDate) {
			this.resetDate = resetDate;
			return this;
		}
		
		public FxLinkedNotionalAmount build() {
			return new FxLinkedNotionalAmount(this);
		}
	
		@Override
		public FxLinkedNotionalAmountBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedFxSpotFixingDate()!=null) return true;
			if (getNotionalAmount()!=null) return true;
			if (getObservedFxSpotRate()!=null) return true;
			if (getResetDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedFxSpotFixingDate"), Date.class, adjustedFxSpotFixingDate, this);
			processor.processBasic(path.newSubPath("notionalAmount"), BigDecimal.class, notionalAmount, this);
			processor.processBasic(path.newSubPath("observedFxSpotRate"), BigDecimal.class, observedFxSpotRate, this);
			processor.processBasic(path.newSubPath("resetDate"), Date.class, resetDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxLinkedNotionalAmountBuilder _that = (FxLinkedNotionalAmountBuilder) o;
		
			if (adjustedFxSpotFixingDate != null ? !adjustedFxSpotFixingDate.equals(_that.adjustedFxSpotFixingDate) : _that.adjustedFxSpotFixingDate != null) return false;
			if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
			if (observedFxSpotRate != null ? !observedFxSpotRate.equals(_that.observedFxSpotRate) : _that.observedFxSpotRate != null) return false;
			if (resetDate != null ? !resetDate.equals(_that.resetDate) : _that.resetDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedFxSpotFixingDate != null ? adjustedFxSpotFixingDate.hashCode() : 0);
			_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
			_result = 31 * _result + (observedFxSpotRate != null ? observedFxSpotRate.hashCode() : 0);
			_result = 31 * _result + (resetDate != null ? resetDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxLinkedNotionalAmountBuilder {" +
				"adjustedFxSpotFixingDate=" + this.adjustedFxSpotFixingDate + ", " +
				"notionalAmount=" + this.notionalAmount + ", " +
				"observedFxSpotRate=" + this.observedFxSpotRate + ", " +
				"resetDate=" + this.resetDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedFxSpotFixingDate"), Date.class, adjustedFxSpotFixingDate, this);
		processor.processBasic(path.newSubPath("notionalAmount"), BigDecimal.class, notionalAmount, this);
		processor.processBasic(path.newSubPath("observedFxSpotRate"), BigDecimal.class, observedFxSpotRate, this);
		processor.processBasic(path.newSubPath("resetDate"), Date.class, resetDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxLinkedNotionalAmount _that = (FxLinkedNotionalAmount) o;
	
		if (adjustedFxSpotFixingDate != null ? !adjustedFxSpotFixingDate.equals(_that.adjustedFxSpotFixingDate) : _that.adjustedFxSpotFixingDate != null) return false;
		if (notionalAmount != null ? !notionalAmount.equals(_that.notionalAmount) : _that.notionalAmount != null) return false;
		if (observedFxSpotRate != null ? !observedFxSpotRate.equals(_that.observedFxSpotRate) : _that.observedFxSpotRate != null) return false;
		if (resetDate != null ? !resetDate.equals(_that.resetDate) : _that.resetDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedFxSpotFixingDate != null ? adjustedFxSpotFixingDate.hashCode() : 0);
		_result = 31 * _result + (notionalAmount != null ? notionalAmount.hashCode() : 0);
		_result = 31 * _result + (observedFxSpotRate != null ? observedFxSpotRate.hashCode() : 0);
		_result = 31 * _result + (resetDate != null ? resetDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxLinkedNotionalAmount {" +
			"adjustedFxSpotFixingDate=" + this.adjustedFxSpotFixingDate + ", " +
			"notionalAmount=" + this.notionalAmount + ", " +
			"observedFxSpotRate=" + this.observedFxSpotRate + ", " +
			"resetDate=" + this.resetDate +
		'}';
	}
}
