package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.FxSpotRateSource;
import org.isda.cdm.RelativeDateOffset;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaNotionalSchedule;
import org.isda.cdm.metafields.ReferenceWithMetaNotionalSchedule.ReferenceWithMetaNotionalScheduleBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.FxLinkedNotionalScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class to describe a notional schedule where each notional that applies to a calculation period is calculated with reference to a notional amount or notional amount schedule in a different currency by means of a spot currency exchange rate which is normally observed at the beginning of each period.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxLinkedNotionalSchedule", source="FpML_5_10", path="fxLinkedNotionalSchedule" )
public class FxLinkedNotionalSchedule extends RosettaModelObject {
	private final ReferenceWithMetaNotionalSchedule constantNotionalScheduleReference;
	private final FxSpotRateSource fxSpotRateSource;
	private final BigDecimal initialValue;
	private final FieldWithMetaString varyingNotionalCurrency;
	private final RelativeDateOffset varyingNotionalFixingDates;
	private final RelativeDateOffset varyingNotionalInterimExchangePaymentDates;
	private static FxLinkedNotionalScheduleMeta metaData = new FxLinkedNotionalScheduleMeta();
	
	FxLinkedNotionalSchedule(FxLinkedNotionalScheduleBuilder builder) {
		this.constantNotionalScheduleReference = ofNullable(builder.getConstantNotionalScheduleReference()).map(f->f.build()).orElse(null);
		this.fxSpotRateSource = ofNullable(builder.getFxSpotRateSource()).map(f->f.build()).orElse(null);
		this.initialValue = builder.getInitialValue();
		this.varyingNotionalCurrency = ofNullable(builder.getVaryingNotionalCurrency()).map(f->f.build()).orElse(null);
		this.varyingNotionalFixingDates = ofNullable(builder.getVaryingNotionalFixingDates()).map(f->f.build()).orElse(null);
		this.varyingNotionalInterimExchangePaymentDates = ofNullable(builder.getVaryingNotionalInterimExchangePaymentDates()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A pointer style reference to the associated constant notional schedule defined elsewhere in the document which contains the currency amounts which will be converted into the varying notional currency amounts using the spot currency exchange rate.
	 */
	@RosettaSynonym(value="constantNotionalScheduleReference", source="FpML_5_10")
	@RosettaSynonym(value="constantNotionalScheduleReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="constantNotionalScheduleReference", source="DTCC_11_0")
	@RosettaSynonym(value="constantNotionalScheduleReference", source="DTCC_9_0")
	@RosettaSynonym(value="constantNotionalScheduleReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaNotionalSchedule getConstantNotionalScheduleReference() {
		return constantNotionalScheduleReference;
	}
	
	/**
	 * The information source and time at which the spot currency exchange rate will be observed.
	 */
	@RosettaSynonym(value="fxSpotRateSource", source="FpML_5_10")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_ClearedConfirm_1_17")
	public final FxSpotRateSource getFxSpotRateSource() {
		return fxSpotRateSource;
	}
	
	/**
	 * The initial currency amount for the varying notional. This may be omitted for a forward starting swap if the FX-linked notional value is not known at deal inception.
	 */
	@RosettaSynonym(value="initialValue", source="FpML_5_10")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getInitialValue() {
		return initialValue;
	}
	
	/**
	 * The currency of the varying notional amount, i.e. the notional amount being determined periodically based on observation of a spot currency exchange rate. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="varyingNotionalCurrency", source="FpML_5_10")
	@RosettaSynonym(value="varyingNotionalCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="varyingNotionalCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="varyingNotionalCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="varyingNotionalCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getVaryingNotionalCurrency() {
		return varyingNotionalCurrency;
	}
	
	/**
	 * The dates on which spot currency exchange rates are observed for purposes of determining the varying notional currency amount that will apply to a calculation period.
	 */
	@RosettaSynonym(value="varyingNotionalFixingDates", source="FpML_5_10")
	@RosettaSynonym(value="varyingNotionalFixingDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="varyingNotionalFixingDates", source="DTCC_11_0")
	@RosettaSynonym(value="varyingNotionalFixingDates", source="DTCC_9_0")
	@RosettaSynonym(value="varyingNotionalFixingDates", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getVaryingNotionalFixingDates() {
		return varyingNotionalFixingDates;
	}
	
	/**
	 * The dates on which interim exchanges of notional are paid. Interim exchanges will arise as a result of changes in the spot currency exchange amount or changes in the constant notional schedule (e.g. amortisation).
	 */
	@RosettaSynonym(value="varyingNotionalInterimExchangePaymentDates", source="FpML_5_10")
	@RosettaSynonym(value="varyingNotionalInterimExchangePaymentDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="varyingNotionalInterimExchangePaymentDates", source="DTCC_11_0")
	@RosettaSynonym(value="varyingNotionalInterimExchangePaymentDates", source="DTCC_9_0")
	@RosettaSynonym(value="varyingNotionalInterimExchangePaymentDates", source="CME_ClearedConfirm_1_17")
	public final RelativeDateOffset getVaryingNotionalInterimExchangePaymentDates() {
		return varyingNotionalInterimExchangePaymentDates;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxLinkedNotionalSchedule> metaData() {
		return metaData;
	} 
	
	public FxLinkedNotionalScheduleBuilder toBuilder() {
		FxLinkedNotionalScheduleBuilder builder = new FxLinkedNotionalScheduleBuilder();
		ofNullable(getConstantNotionalScheduleReference()).ifPresent(builder::setConstantNotionalScheduleReference);
		ofNullable(getFxSpotRateSource()).ifPresent(builder::setFxSpotRateSource);
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getVaryingNotionalCurrency()).ifPresent(builder::setVaryingNotionalCurrency);
		ofNullable(getVaryingNotionalFixingDates()).ifPresent(builder::setVaryingNotionalFixingDates);
		ofNullable(getVaryingNotionalInterimExchangePaymentDates()).ifPresent(builder::setVaryingNotionalInterimExchangePaymentDates);
		return builder;
	}

	public static FxLinkedNotionalScheduleBuilder builder() {
		return new FxLinkedNotionalScheduleBuilder();
	}

	public static  class FxLinkedNotionalScheduleBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaNotionalScheduleBuilder constantNotionalScheduleReference;
		protected FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource;
		protected BigDecimal initialValue;
		protected FieldWithMetaStringBuilder varyingNotionalCurrency;
		protected RelativeDateOffset.RelativeDateOffsetBuilder varyingNotionalFixingDates;
		protected RelativeDateOffset.RelativeDateOffsetBuilder varyingNotionalInterimExchangePaymentDates;
	
		public FxLinkedNotionalScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxLinkedNotionalSchedule> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaNotionalScheduleBuilder getConstantNotionalScheduleReference() {
			return constantNotionalScheduleReference;
		}
		
		public ReferenceWithMetaNotionalScheduleBuilder getOrCreateConstantNotionalScheduleReference() {
			if (constantNotionalScheduleReference!=null) {
				return constantNotionalScheduleReference;
			}
			else return constantNotionalScheduleReference = new ReferenceWithMetaNotionalScheduleBuilder();
		}
		
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getFxSpotRateSource() {
			return fxSpotRateSource;
		}
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getOrCreateFxSpotRateSource() {
			if (fxSpotRateSource!=null) {
				return fxSpotRateSource;
			}
			else return fxSpotRateSource = new FxSpotRateSource.FxSpotRateSourceBuilder();
		}
		
		
		public BigDecimal getInitialValue() {
			return initialValue;
		}
		
		
		public FieldWithMetaStringBuilder getVaryingNotionalCurrency() {
			return varyingNotionalCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateVaryingNotionalCurrency() {
			if (varyingNotionalCurrency!=null) {
				return varyingNotionalCurrency;
			}
			else return varyingNotionalCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getVaryingNotionalFixingDates() {
			return varyingNotionalFixingDates;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateVaryingNotionalFixingDates() {
			if (varyingNotionalFixingDates!=null) {
				return varyingNotionalFixingDates;
			}
			else return varyingNotionalFixingDates = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getVaryingNotionalInterimExchangePaymentDates() {
			return varyingNotionalInterimExchangePaymentDates;
		}
		
		public RelativeDateOffset.RelativeDateOffsetBuilder getOrCreateVaryingNotionalInterimExchangePaymentDates() {
			if (varyingNotionalInterimExchangePaymentDates!=null) {
				return varyingNotionalInterimExchangePaymentDates;
			}
			else return varyingNotionalInterimExchangePaymentDates = new RelativeDateOffset.RelativeDateOffsetBuilder();
		}
		
	
		public FxLinkedNotionalScheduleBuilder setConstantNotionalScheduleReference(ReferenceWithMetaNotionalSchedule constantNotionalScheduleReference) {
			this.constantNotionalScheduleReference = constantNotionalScheduleReference.toBuilder();
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setConstantNotionalScheduleReferenceBuilder(ReferenceWithMetaNotionalScheduleBuilder constantNotionalScheduleReference) {
			this.constantNotionalScheduleReference = constantNotionalScheduleReference;
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setConstantNotionalScheduleReferenceRef(NotionalSchedule.NotionalScheduleBuilder constantNotionalScheduleReference) {
			return setConstantNotionalScheduleReference(ReferenceWithMetaNotionalSchedule.builder().setValueBuilder(constantNotionalScheduleReference).build());
		}
		public FxLinkedNotionalScheduleBuilder setConstantNotionalScheduleReferenceRef(NotionalSchedule constantNotionalScheduleReference) {
			return setConstantNotionalScheduleReferenceRef(NotionalSchedule.builder());
		}
		public FxLinkedNotionalScheduleBuilder setFxSpotRateSource(FxSpotRateSource fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource.toBuilder();
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setFxSpotRateSourceBuilder(FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource;
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalCurrency(FieldWithMetaString varyingNotionalCurrency) {
			this.varyingNotionalCurrency = varyingNotionalCurrency.toBuilder();
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalCurrencyRef(String varyingNotionalCurrency) {
			return setVaryingNotionalCurrency(FieldWithMetaString.builder().setValue(varyingNotionalCurrency).build());
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalFixingDates(RelativeDateOffset varyingNotionalFixingDates) {
			this.varyingNotionalFixingDates = varyingNotionalFixingDates.toBuilder();
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalFixingDatesBuilder(RelativeDateOffset.RelativeDateOffsetBuilder varyingNotionalFixingDates) {
			this.varyingNotionalFixingDates = varyingNotionalFixingDates;
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalInterimExchangePaymentDates(RelativeDateOffset varyingNotionalInterimExchangePaymentDates) {
			this.varyingNotionalInterimExchangePaymentDates = varyingNotionalInterimExchangePaymentDates.toBuilder();
			return this;
		}
		public FxLinkedNotionalScheduleBuilder setVaryingNotionalInterimExchangePaymentDatesBuilder(RelativeDateOffset.RelativeDateOffsetBuilder varyingNotionalInterimExchangePaymentDates) {
			this.varyingNotionalInterimExchangePaymentDates = varyingNotionalInterimExchangePaymentDates;
			return this;
		}
		
		public FxLinkedNotionalSchedule build() {
			return new FxLinkedNotionalSchedule(this);
		}
	
		@Override
		public FxLinkedNotionalScheduleBuilder prune() {
			if (constantNotionalScheduleReference!=null && !constantNotionalScheduleReference.prune().hasData()) constantNotionalScheduleReference = null;
			if (fxSpotRateSource!=null && !fxSpotRateSource.prune().hasData()) fxSpotRateSource = null;
			if (varyingNotionalCurrency!=null && !varyingNotionalCurrency.prune().hasData()) varyingNotionalCurrency = null;
			if (varyingNotionalFixingDates!=null && !varyingNotionalFixingDates.prune().hasData()) varyingNotionalFixingDates = null;
			if (varyingNotionalInterimExchangePaymentDates!=null && !varyingNotionalInterimExchangePaymentDates.prune().hasData()) varyingNotionalInterimExchangePaymentDates = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getConstantNotionalScheduleReference()!=null && getConstantNotionalScheduleReference().hasData()) return true;
			if (getFxSpotRateSource()!=null && getFxSpotRateSource().hasData()) return true;
			if (getInitialValue()!=null) return true;
			if (getVaryingNotionalCurrency()!=null) return true;
			if (getVaryingNotionalFixingDates()!=null && getVaryingNotionalFixingDates().hasData()) return true;
			if (getVaryingNotionalInterimExchangePaymentDates()!=null && getVaryingNotionalInterimExchangePaymentDates().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
			
			processRosetta(path.newSubPath("constantNotionalScheduleReference"), processor, ReferenceWithMetaNotionalSchedule.class, constantNotionalScheduleReference);
			processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
			processRosetta(path.newSubPath("varyingNotionalCurrency"), processor, FieldWithMetaString.class, varyingNotionalCurrency);
			processRosetta(path.newSubPath("varyingNotionalFixingDates"), processor, RelativeDateOffset.class, varyingNotionalFixingDates);
			processRosetta(path.newSubPath("varyingNotionalInterimExchangePaymentDates"), processor, RelativeDateOffset.class, varyingNotionalInterimExchangePaymentDates);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxLinkedNotionalScheduleBuilder _that = (FxLinkedNotionalScheduleBuilder) o;
		
			if (constantNotionalScheduleReference != null ? !constantNotionalScheduleReference.equals(_that.constantNotionalScheduleReference) : _that.constantNotionalScheduleReference != null) return false;
			if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
			if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
			if (varyingNotionalCurrency != null ? !varyingNotionalCurrency.equals(_that.varyingNotionalCurrency) : _that.varyingNotionalCurrency != null) return false;
			if (varyingNotionalFixingDates != null ? !varyingNotionalFixingDates.equals(_that.varyingNotionalFixingDates) : _that.varyingNotionalFixingDates != null) return false;
			if (varyingNotionalInterimExchangePaymentDates != null ? !varyingNotionalInterimExchangePaymentDates.equals(_that.varyingNotionalInterimExchangePaymentDates) : _that.varyingNotionalInterimExchangePaymentDates != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (constantNotionalScheduleReference != null ? constantNotionalScheduleReference.hashCode() : 0);
			_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
			_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
			_result = 31 * _result + (varyingNotionalCurrency != null ? varyingNotionalCurrency.hashCode() : 0);
			_result = 31 * _result + (varyingNotionalFixingDates != null ? varyingNotionalFixingDates.hashCode() : 0);
			_result = 31 * _result + (varyingNotionalInterimExchangePaymentDates != null ? varyingNotionalInterimExchangePaymentDates.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxLinkedNotionalScheduleBuilder {" +
				"constantNotionalScheduleReference=" + this.constantNotionalScheduleReference + ", " +
				"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
				"initialValue=" + this.initialValue + ", " +
				"varyingNotionalCurrency=" + this.varyingNotionalCurrency + ", " +
				"varyingNotionalFixingDates=" + this.varyingNotionalFixingDates + ", " +
				"varyingNotionalInterimExchangePaymentDates=" + this.varyingNotionalInterimExchangePaymentDates +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
		
		processRosetta(path.newSubPath("constantNotionalScheduleReference"), processor, ReferenceWithMetaNotionalSchedule.class, constantNotionalScheduleReference);
		processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
		processRosetta(path.newSubPath("varyingNotionalCurrency"), processor, FieldWithMetaString.class, varyingNotionalCurrency);
		processRosetta(path.newSubPath("varyingNotionalFixingDates"), processor, RelativeDateOffset.class, varyingNotionalFixingDates);
		processRosetta(path.newSubPath("varyingNotionalInterimExchangePaymentDates"), processor, RelativeDateOffset.class, varyingNotionalInterimExchangePaymentDates);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxLinkedNotionalSchedule _that = (FxLinkedNotionalSchedule) o;
	
		if (constantNotionalScheduleReference != null ? !constantNotionalScheduleReference.equals(_that.constantNotionalScheduleReference) : _that.constantNotionalScheduleReference != null) return false;
		if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
		if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
		if (varyingNotionalCurrency != null ? !varyingNotionalCurrency.equals(_that.varyingNotionalCurrency) : _that.varyingNotionalCurrency != null) return false;
		if (varyingNotionalFixingDates != null ? !varyingNotionalFixingDates.equals(_that.varyingNotionalFixingDates) : _that.varyingNotionalFixingDates != null) return false;
		if (varyingNotionalInterimExchangePaymentDates != null ? !varyingNotionalInterimExchangePaymentDates.equals(_that.varyingNotionalInterimExchangePaymentDates) : _that.varyingNotionalInterimExchangePaymentDates != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (constantNotionalScheduleReference != null ? constantNotionalScheduleReference.hashCode() : 0);
		_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
		_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
		_result = 31 * _result + (varyingNotionalCurrency != null ? varyingNotionalCurrency.hashCode() : 0);
		_result = 31 * _result + (varyingNotionalFixingDates != null ? varyingNotionalFixingDates.hashCode() : 0);
		_result = 31 * _result + (varyingNotionalInterimExchangePaymentDates != null ? varyingNotionalInterimExchangePaymentDates.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxLinkedNotionalSchedule {" +
			"constantNotionalScheduleReference=" + this.constantNotionalScheduleReference + ", " +
			"fxSpotRateSource=" + this.fxSpotRateSource + ", " +
			"initialValue=" + this.initialValue + ", " +
			"varyingNotionalCurrency=" + this.varyingNotionalCurrency + ", " +
			"varyingNotionalFixingDates=" + this.varyingNotionalFixingDates + ", " +
			"varyingNotionalInterimExchangePaymentDates=" + this.varyingNotionalInterimExchangePaymentDates +
		'}';
	}
}
