package org.isda.cdm;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.FxSettlementRateSource;
import org.isda.cdm.meta.FxRateSourceFixingMeta;

import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;

import static java.util.Optional.ofNullable;

/**
 * Describes a rate source to be fixed and the date the fixing occurs
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="FxRateSourceFixing", source="FpML_5_10")
public class FxRateSourceFixing extends RosettaModelObject {
	private final AdjustableDate fixingDate;
	private final FxSettlementRateSource settlementRateSource;
	private static FxRateSourceFixingMeta metaData = new FxRateSourceFixingMeta();
	
	FxRateSourceFixing(FxRateSourceFixingBuilder builder) {
		this.fixingDate = ofNullable(builder.getFixingDate()).map(f->f.build()).orElse(null);
		this.settlementRateSource = ofNullable(builder.getSettlementRateSource()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the fixing is scheduled to occur.
	 */
	@RosettaSynonym(value="fixingDate", source="FpML_5_10")
	public final AdjustableDate getFixingDate() {
		return fixingDate;
	}
	
	@RosettaSynonym(value="settlementRateSource", source="FpML_5_10")
	public final FxSettlementRateSource getSettlementRateSource() {
		return settlementRateSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends FxRateSourceFixing> metaData() {
		return metaData;
	} 
	
	public FxRateSourceFixingBuilder toBuilder() {
		FxRateSourceFixingBuilder builder = new FxRateSourceFixingBuilder();
		ofNullable(getFixingDate()).ifPresent(builder::setFixingDate);
		ofNullable(getSettlementRateSource()).ifPresent(builder::setSettlementRateSource);
		return builder;
	}

	public static FxRateSourceFixingBuilder builder() {
		return new FxRateSourceFixingBuilder();
	}

	public static class FxRateSourceFixingBuilder extends RosettaModelObjectBuilder{
	
		protected AdjustableDate.AdjustableDateBuilder fixingDate;
		protected FxSettlementRateSource.FxSettlementRateSourceBuilder settlementRateSource;
	
		public FxRateSourceFixingBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends FxRateSourceFixing> metaData() {
			return metaData;
		} 
	
		
		public AdjustableDate.AdjustableDateBuilder getFixingDate() {
			return fixingDate;
		}
		
		public AdjustableDate.AdjustableDateBuilder getOrCreateFixingDate() {
			if (fixingDate!=null) {
				return fixingDate;
			}
			else return fixingDate = new AdjustableDate.AdjustableDateBuilder();
		}
		
		
		public FxSettlementRateSource.FxSettlementRateSourceBuilder getSettlementRateSource() {
			return settlementRateSource;
		}
		
		public FxSettlementRateSource.FxSettlementRateSourceBuilder getOrCreateSettlementRateSource() {
			if (settlementRateSource!=null) {
				return settlementRateSource;
			}
			else return settlementRateSource = new FxSettlementRateSource.FxSettlementRateSourceBuilder();
		}
		
	
		public FxRateSourceFixingBuilder setFixingDate(AdjustableDate fixingDate) {
			this.fixingDate = fixingDate.toBuilder();
			return this;
		}
		public FxRateSourceFixingBuilder setFixingDateBuilder(AdjustableDate.AdjustableDateBuilder fixingDate) {
			this.fixingDate = fixingDate;
			return this;
		}
		public FxRateSourceFixingBuilder setSettlementRateSource(FxSettlementRateSource settlementRateSource) {
			this.settlementRateSource = settlementRateSource.toBuilder();
			return this;
		}
		public FxRateSourceFixingBuilder setSettlementRateSourceBuilder(FxSettlementRateSource.FxSettlementRateSourceBuilder settlementRateSource) {
			this.settlementRateSource = settlementRateSource;
			return this;
		}
	
		public FxRateSourceFixing build() {
			return new FxRateSourceFixing(this);
		}
	
		@Override
		public FxRateSourceFixingBuilder prune() {
			if (fixingDate!=null && !fixingDate.prune().hasData()) fixingDate = null;
			if (settlementRateSource!=null && !settlementRateSource.prune().hasData()) settlementRateSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFixingDate()!=null && getFixingDate().hasData()) return true;
			if (getSettlementRateSource()!=null && getSettlementRateSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fixingDate"), processor, AdjustableDate.class, fixingDate);
			processRosetta(path.newSubPath("settlementRateSource"), processor, FxSettlementRateSource.class, settlementRateSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			FxRateSourceFixingBuilder _that = (FxRateSourceFixingBuilder) o;
		
			if (fixingDate != null ? !fixingDate.equals(_that.fixingDate) : _that.fixingDate != null) return false;
			if (settlementRateSource != null ? !settlementRateSource.equals(_that.settlementRateSource) : _that.settlementRateSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fixingDate != null ? fixingDate.hashCode() : 0);
			_result = 31 * _result + (settlementRateSource != null ? settlementRateSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FxRateSourceFixingBuilder {" +
				"fixingDate=" + this.fixingDate + ", " +
				"settlementRateSource=" + this.settlementRateSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fixingDate"), processor, AdjustableDate.class, fixingDate);
		processRosetta(path.newSubPath("settlementRateSource"), processor, FxSettlementRateSource.class, settlementRateSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		FxRateSourceFixing _that = (FxRateSourceFixing) o;
	
		if (fixingDate != null ? !fixingDate.equals(_that.fixingDate) : _that.fixingDate != null) return false;
		if (settlementRateSource != null ? !settlementRateSource.equals(_that.settlementRateSource) : _that.settlementRateSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fixingDate != null ? fixingDate.hashCode() : 0);
		_result = 31 * _result + (settlementRateSource != null ? settlementRateSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "FxRateSourceFixing {" +
			"fixingDate=" + this.fixingDate + ", " +
			"settlementRateSource=" + this.settlementRateSource +
		'}';
	}
}
