package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the law governing the contract or legal document.
 * 
 * @version 2.5.4
 */
public enum GoverningLawEnum {
    

	/**
	 * The Governing Law is determined by reference to the relevant master agreement.
	 */
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "FpML_5_10")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterAgreement", source = "CME_ClearedConfirm_1_17")
	AS_SPECIFIED_IN_MASTER_AGREEMENT,
	    

	/**
	 * Alberta law
	 */
	@RosettaSynonym(value = "CAAB", source = "FpML_5_10")
	@RosettaSynonym(value = "CAAB", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAAB", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAAB", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAAB", source = "CME_ClearedConfirm_1_17")
	CAAB,
	    

	/**
	 * British Columbia Law
	 */
	@RosettaSynonym(value = "CABC", source = "FpML_5_10")
	@RosettaSynonym(value = "CABC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CABC", source = "DTCC_11_0")
	@RosettaSynonym(value = "CABC", source = "DTCC_9_0")
	@RosettaSynonym(value = "CABC", source = "CME_ClearedConfirm_1_17")
	CABC,
	    

	/**
	 * Manitoba law
	 */
	@RosettaSynonym(value = "CAMN", source = "FpML_5_10")
	@RosettaSynonym(value = "CAMN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAMN", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAMN", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAMN", source = "CME_ClearedConfirm_1_17")
	CAMN,
	    

	/**
	 * Ontario law
	 */
	@RosettaSynonym(value = "CAON", source = "FpML_5_10")
	@RosettaSynonym(value = "CAON", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAON", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAON", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAON", source = "CME_ClearedConfirm_1_17")
	CAON,
	    

	/**
	 * Quebec law
	 */
	@RosettaSynonym(value = "CAQC", source = "FpML_5_10")
	@RosettaSynonym(value = "CAQC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAQC", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAQC", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAQC", source = "CME_ClearedConfirm_1_17")
	CAQC,
	    

	/**
	 * German law
	 */
	@RosettaSynonym(value = "DE", source = "FpML_5_10")
	@RosettaSynonym(value = "DE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DE", source = "DTCC_11_0")
	@RosettaSynonym(value = "DE", source = "DTCC_9_0")
	@RosettaSynonym(value = "DE", source = "CME_ClearedConfirm_1_17")
	DE,
	    

	/**
	 * French law
	 */
	@RosettaSynonym(value = "FR", source = "FpML_5_10")
	@RosettaSynonym(value = "FR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FR", source = "DTCC_11_0")
	@RosettaSynonym(value = "FR", source = "DTCC_9_0")
	@RosettaSynonym(value = "FR", source = "CME_ClearedConfirm_1_17")
	FR,
	    

	/**
	 * English law
	 */
	@RosettaSynonym(value = "GBEN", source = "FpML_5_10")
	@RosettaSynonym(value = "GBEN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBEN", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBEN", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBEN", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "ENGLISH", source = "AcadiaSoft_AM_1_0")
	@RosettaSynonym(value = "English", source = "ISDA_Create_1_0")
	GBEN,
	    

	/**
	 * The law of the island of Guernsey
	 */
	@RosettaSynonym(value = "GBGY", source = "FpML_5_10")
	@RosettaSynonym(value = "GBGY", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBGY", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBGY", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBGY", source = "CME_ClearedConfirm_1_17")
	GBGY,
	    

	/**
	 * The law of the Isle of Man
	 */
	@RosettaSynonym(value = "GBIM", source = "FpML_5_10")
	@RosettaSynonym(value = "GBIM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBIM", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBIM", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBIM", source = "CME_ClearedConfirm_1_17")
	GBIM,
	    

	/**
	 * The law of the island of Jersey
	 */
	@RosettaSynonym(value = "GBJY", source = "FpML_5_10")
	@RosettaSynonym(value = "GBJY", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBJY", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBJY", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBJY", source = "CME_ClearedConfirm_1_17")
	GBJY,
	    

	/**
	 * Scottish law
	 */
	@RosettaSynonym(value = "GBSC", source = "FpML_5_10")
	@RosettaSynonym(value = "GBSC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBSC", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBSC", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBSC", source = "CME_ClearedConfirm_1_17")
	GBSC,
	    

	/**
	 * Japanese law
	 */
	@RosettaSynonym(value = "JP", source = "FpML_5_10")
	@RosettaSynonym(value = "JP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JP", source = "DTCC_11_0")
	@RosettaSynonym(value = "JP", source = "DTCC_9_0")
	@RosettaSynonym(value = "JP", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "JAPAN", source = "AcadiaSoft_AM_1_0")
	@RosettaSynonym(value = "Japanese", source = "ISDA_Create_1_0")
	JP,
	    

	/**
	 * Californian law
	 */
	@RosettaSynonym(value = "USCA", source = "FpML_5_10")
	@RosettaSynonym(value = "USCA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USCA", source = "DTCC_11_0")
	@RosettaSynonym(value = "USCA", source = "DTCC_9_0")
	@RosettaSynonym(value = "USCA", source = "CME_ClearedConfirm_1_17")
	USCA,
	    

	/**
	 * Delaware law
	 */
	@RosettaSynonym(value = "USDE", source = "FpML_5_10")
	@RosettaSynonym(value = "USDE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USDE", source = "DTCC_11_0")
	@RosettaSynonym(value = "USDE", source = "DTCC_9_0")
	@RosettaSynonym(value = "USDE", source = "CME_ClearedConfirm_1_17")
	USDE,
	    

	/**
	 * Illinois law
	 */
	@RosettaSynonym(value = "USIL", source = "FpML_5_10")
	@RosettaSynonym(value = "USIL", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USIL", source = "DTCC_11_0")
	@RosettaSynonym(value = "USIL", source = "DTCC_9_0")
	@RosettaSynonym(value = "USIL", source = "CME_ClearedConfirm_1_17")
	USIL,
	    

	/**
	 * New York law
	 */
	@RosettaSynonym(value = "USNY", source = "FpML_5_10")
	@RosettaSynonym(value = "USNY", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USNY", source = "DTCC_11_0")
	@RosettaSynonym(value = "USNY", source = "DTCC_9_0")
	@RosettaSynonym(value = "USNY", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "NY", source = "AcadiaSoft_AM_1_0")
	@RosettaSynonym(value = "New York", source = "ISDA_Create_1_0")
	USNY    
	;

	private final String displayName;
	
	GoverningLawEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
