package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.Offset;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.GracePeriodExtensionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="GracePeriodExtension", source="FpML_5_10")
public class GracePeriodExtension extends RosettaModelObject {
	private final Boolean applicable;
	private final Offset gracePeriod;
	private static GracePeriodExtensionMeta metaData = new GracePeriodExtensionMeta();
	
	GracePeriodExtension(GracePeriodExtensionBuilder builder) {
		this.applicable = builder.getApplicable();
		this.gracePeriod = ofNullable(builder.getGracePeriod()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates whether the grace period extension provision is applicable.
	 */
	@RosettaSynonym(value="applicable", source="FpML_5_10")
	@RosettaSynonym(value="applicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="applicable", source="DTCC_11_0")
	@RosettaSynonym(value="applicable", source="DTCC_9_0")
	@RosettaSynonym(value="applicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * The number of calendar or business days after any due date that the reference entity has to fulfil its obligations before a failure to pay credit event is deemed to have occurred. ISDA 2003 Term: Grace Period.
	 */
	@RosettaSynonym(value="gracePeriod", source="FpML_5_10")
	@RosettaSynonym(value="gracePeriod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="gracePeriod", source="DTCC_11_0")
	@RosettaSynonym(value="gracePeriod", source="DTCC_9_0")
	@RosettaSynonym(value="gracePeriod", source="CME_ClearedConfirm_1_17")
	public final Offset getGracePeriod() {
		return gracePeriod;
	}
	
	
	@Override
	public RosettaMetaData<? extends GracePeriodExtension> metaData() {
		return metaData;
	} 
	
	public GracePeriodExtensionBuilder toBuilder() {
		GracePeriodExtensionBuilder builder = new GracePeriodExtensionBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getGracePeriod()).ifPresent(builder::setGracePeriod);
		return builder;
	}

	public static GracePeriodExtensionBuilder builder() {
		return new GracePeriodExtensionBuilder();
	}

	public static  class GracePeriodExtensionBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected Offset.OffsetBuilder gracePeriod;
	
		public GracePeriodExtensionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends GracePeriodExtension> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public Offset.OffsetBuilder getGracePeriod() {
			return gracePeriod;
		}
		
		public Offset.OffsetBuilder getOrCreateGracePeriod() {
			if (gracePeriod!=null) {
				return gracePeriod;
			}
			else return gracePeriod = new Offset.OffsetBuilder();
		}
		
	
		public GracePeriodExtensionBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public GracePeriodExtensionBuilder setGracePeriod(Offset gracePeriod) {
			this.gracePeriod = gracePeriod.toBuilder();
			return this;
		}
		public GracePeriodExtensionBuilder setGracePeriodBuilder(Offset.OffsetBuilder gracePeriod) {
			this.gracePeriod = gracePeriod;
			return this;
		}
		
		public GracePeriodExtension build() {
			return new GracePeriodExtension(this);
		}
	
		@Override
		public GracePeriodExtensionBuilder prune() {
			if (gracePeriod!=null && !gracePeriod.prune().hasData()) gracePeriod = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getGracePeriod()!=null && getGracePeriod().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			
			processRosetta(path.newSubPath("gracePeriod"), processor, Offset.class, gracePeriod);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			GracePeriodExtensionBuilder _that = (GracePeriodExtensionBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (gracePeriod != null ? !gracePeriod.equals(_that.gracePeriod) : _that.gracePeriod != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (gracePeriod != null ? gracePeriod.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "GracePeriodExtensionBuilder {" +
				"applicable=" + this.applicable + ", " +
				"gracePeriod=" + this.gracePeriod +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		
		processRosetta(path.newSubPath("gracePeriod"), processor, Offset.class, gracePeriod);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		GracePeriodExtension _that = (GracePeriodExtension) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (gracePeriod != null ? !gracePeriod.equals(_that.gracePeriod) : _that.gracePeriod != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (gracePeriod != null ? gracePeriod.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "GracePeriodExtension {" +
			"applicable=" + this.applicable + ", " +
			"gracePeriod=" + this.gracePeriod +
		'}';
	}
}
