package org.isda.cdm;

import org.isda.cdm.HoldingAndUsingPostedCollateralElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.HoldingAndUsingPostedCollateralMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the elections for the holding and using of posted collateral by the respective parties to the Credit Support Annex for Variation Margin. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (h): Holding and Using Posted Collateral (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class HoldingAndUsingPostedCollateral extends RosettaModelObject {
	private final HoldingAndUsingPostedCollateralElection partyElection;
	private static HoldingAndUsingPostedCollateralMeta metaData = new HoldingAndUsingPostedCollateralMeta();
	
	HoldingAndUsingPostedCollateral(HoldingAndUsingPostedCollateralBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The parties&#39; elections for the holding and using of posted collateral.
	 */
	public final HoldingAndUsingPostedCollateralElection getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends HoldingAndUsingPostedCollateral> metaData() {
		return metaData;
	} 
	
	public HoldingAndUsingPostedCollateralBuilder toBuilder() {
		HoldingAndUsingPostedCollateralBuilder builder = new HoldingAndUsingPostedCollateralBuilder();
		ofNullable(getPartyElection()).ifPresent(builder::setPartyElection);
		return builder;
	}

	public static HoldingAndUsingPostedCollateralBuilder builder() {
		return new HoldingAndUsingPostedCollateralBuilder();
	}

	public static  class HoldingAndUsingPostedCollateralBuilder extends RosettaModelObjectBuilder {
	
		protected HoldingAndUsingPostedCollateralElection.HoldingAndUsingPostedCollateralElectionBuilder partyElection;
	
		public HoldingAndUsingPostedCollateralBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends HoldingAndUsingPostedCollateral> metaData() {
			return metaData;
		} 
	
		
		public HoldingAndUsingPostedCollateralElection.HoldingAndUsingPostedCollateralElectionBuilder getPartyElection() {
			return partyElection;
		}
		
		public HoldingAndUsingPostedCollateralElection.HoldingAndUsingPostedCollateralElectionBuilder getOrCreatePartyElection() {
			if (partyElection!=null) {
				return partyElection;
			}
			else return partyElection = new HoldingAndUsingPostedCollateralElection.HoldingAndUsingPostedCollateralElectionBuilder();
		}
		
	
		public HoldingAndUsingPostedCollateralBuilder setPartyElection(HoldingAndUsingPostedCollateralElection partyElection) {
			this.partyElection = partyElection.toBuilder();
			return this;
		}
		public HoldingAndUsingPostedCollateralBuilder setPartyElectionBuilder(HoldingAndUsingPostedCollateralElection.HoldingAndUsingPostedCollateralElectionBuilder partyElection) {
			this.partyElection = partyElection;
			return this;
		}
		
		public HoldingAndUsingPostedCollateral build() {
			return new HoldingAndUsingPostedCollateral(this);
		}
	
		@Override
		public HoldingAndUsingPostedCollateralBuilder prune() {
			if (partyElection!=null && !partyElection.prune().hasData()) partyElection = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, HoldingAndUsingPostedCollateralElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			HoldingAndUsingPostedCollateralBuilder _that = (HoldingAndUsingPostedCollateralBuilder) o;
		
			if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "HoldingAndUsingPostedCollateralBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, HoldingAndUsingPostedCollateralElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		HoldingAndUsingPostedCollateral _that = (HoldingAndUsingPostedCollateral) o;
	
		if (partyElection != null ? !partyElection.equals(_that.partyElection) : _that.partyElection != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "HoldingAndUsingPostedCollateral {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
