package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IdMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Id extends RosettaModelObject {
	private final String lei;
	private static IdMeta metaData = new IdMeta();
	
	Id(IdBuilder builder) {
		this.lei = builder.getLei();
	}
	
	@RosettaSynonym(value="LEI", source="ISO20022")
	public final String getLei() {
		return lei;
	}
	
	
	@Override
	public RosettaMetaData<? extends Id> metaData() {
		return metaData;
	} 
	
	public IdBuilder toBuilder() {
		IdBuilder builder = new IdBuilder();
		ofNullable(getLei()).ifPresent(builder::setLei);
		return builder;
	}

	public static IdBuilder builder() {
		return new IdBuilder();
	}

	public static  class IdBuilder extends RosettaModelObjectBuilder {
	
		protected String lei;
	
		public IdBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Id> metaData() {
			return metaData;
		} 
	
		
		public String getLei() {
			return lei;
		}
		
	
		public IdBuilder setLei(String lei) {
			this.lei = lei;
			return this;
		}
		
		public Id build() {
			return new Id(this);
		}
	
		@Override
		public IdBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLei()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("lei"), String.class, lei, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IdBuilder _that = (IdBuilder) o;
		
			if (lei != null ? !lei.equals(_that.lei) : _that.lei != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (lei != null ? lei.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IdBuilder {" +
				"lei=" + this.lei +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("lei"), String.class, lei, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Id _that = (Id) o;
	
		if (lei != null ? !lei.equals(_that.lei) : _that.lei != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (lei != null ? lei.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Id {" +
			"lei=" + this.lei +
		'}';
	}
}
