package org.isda.cdm;

import org.isda.cdm.ProductIdentifier;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IdentifiedProductMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * An abstract class to specify a product which terms are abstracted through reference data.
 * 
 * @version 2.5.4
 */
@RosettaClass
public abstract class IdentifiedProduct extends RosettaModelObject {
	private final ProductIdentifier productIdentifier;
	private static IdentifiedProductMeta metaData = new IdentifiedProductMeta();
	
	IdentifiedProduct(IdentifiedProductBuilder builder) {
		this.productIdentifier = ofNullable(builder.getProductIdentifier()).map(f->f.build()).orElse(null);
	}
	
	public final ProductIdentifier getProductIdentifier() {
		return productIdentifier;
	}
	
	
	@Override
	public RosettaMetaData<? extends IdentifiedProduct> metaData() {
		return metaData;
	} 
	
	public abstract IdentifiedProductBuilder toBuilder();


	public static abstract
	 class IdentifiedProductBuilder extends RosettaModelObjectBuilder {
	
		protected ProductIdentifier.ProductIdentifierBuilder productIdentifier;
	
		public IdentifiedProductBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IdentifiedProduct> metaData() {
			return metaData;
		} 
	
		
		public ProductIdentifier.ProductIdentifierBuilder getProductIdentifier() {
			return productIdentifier;
		}
		
		public ProductIdentifier.ProductIdentifierBuilder getOrCreateProductIdentifier() {
			if (productIdentifier!=null) {
				return productIdentifier;
			}
			else return productIdentifier = new ProductIdentifier.ProductIdentifierBuilder();
		}
		
	
		public IdentifiedProductBuilder setProductIdentifier(ProductIdentifier productIdentifier) {
			this.productIdentifier = productIdentifier.toBuilder();
			return this;
		}
		public IdentifiedProductBuilder setProductIdentifierBuilder(ProductIdentifier.ProductIdentifierBuilder productIdentifier) {
			this.productIdentifier = productIdentifier;
			return this;
		}
		
		public abstract IdentifiedProduct build();
	
		@Override
		public IdentifiedProductBuilder prune() {
			if (productIdentifier!=null && !productIdentifier.prune().hasData()) productIdentifier = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getProductIdentifier()!=null && getProductIdentifier().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("productIdentifier"), processor, ProductIdentifier.class, productIdentifier);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IdentifiedProductBuilder _that = (IdentifiedProductBuilder) o;
		
			if (productIdentifier != null ? !productIdentifier.equals(_that.productIdentifier) : _that.productIdentifier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (productIdentifier != null ? productIdentifier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IdentifiedProductBuilder {" +
				"productIdentifier=" + this.productIdentifier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("productIdentifier"), processor, ProductIdentifier.class, productIdentifier);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		IdentifiedProduct _that = (IdentifiedProduct) o;
	
		if (productIdentifier != null ? !productIdentifier.equals(_that.productIdentifier) : _that.productIdentifier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (productIdentifier != null ? productIdentifier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IdentifiedProduct {" +
			"productIdentifier=" + this.productIdentifier +
		'}';
	}
}
