package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PaymentDetail;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IndependentAmountMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class specifying the Independent Amount as the combination of a payer/receiver, a payment amount, a payment date and an associated payment calculation rule.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="IndependentAmount", source="FpML_5_10")
public class IndependentAmount extends PayerReceiver {
	private final List<PaymentDetail> paymentDetail;
	private static IndependentAmountMeta metaData = new IndependentAmountMeta();
	
	IndependentAmount(IndependentAmountBuilder builder) {
		super(builder);
		this.paymentDetail = ofNullable(builder.getPaymentDetail()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * An attribute that specifies a payment as the combination of a payment amount, a payment date and an associated payment calculation rule.
	 */
	@RosettaSynonym(value="paymentDetail", source="FpML_5_10")
	@RosettaSynonym(value="paymentDetail", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDetail", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDetail", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDetail", source="CME_ClearedConfirm_1_17")
	public final List<PaymentDetail> getPaymentDetail() {
		return paymentDetail;
	}
	
	
	@Override
	public RosettaMetaData<? extends IndependentAmount> metaData() {
		return metaData;
	} 
	
	public IndependentAmountBuilder toBuilder() {
		IndependentAmountBuilder builder = new IndependentAmountBuilder();
		ofNullable(getPaymentDetail()).ifPresent(paymentDetail -> paymentDetail.forEach(builder::addPaymentDetail));
		ofNullable(getPayerAccountReference()).ifPresent(builder::setPayerAccountReference);
		ofNullable(getPayerPartyReference()).ifPresent(builder::setPayerPartyReference);
		ofNullable(getReceiverAccountReference()).ifPresent(builder::setReceiverAccountReference);
		ofNullable(getReceiverPartyReference()).ifPresent(builder::setReceiverPartyReference);
		return builder;
	}

	public static IndependentAmountBuilder builder() {
		return new IndependentAmountBuilder();
	}

	public static  class IndependentAmountBuilder extends PayerReceiverBuilder {
	
		protected List<PaymentDetail.PaymentDetailBuilder> paymentDetail;
	
		public IndependentAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IndependentAmount> metaData() {
			return metaData;
		} 
	
		
		public List<PaymentDetail.PaymentDetailBuilder> getPaymentDetail() {
			return paymentDetail;
		}
		
		public PaymentDetail.PaymentDetailBuilder getOrCreatePaymentDetail(int index) {
			if (paymentDetail==null) {
				this.paymentDetail = new ArrayList<>();
			}
			return getIndex(paymentDetail, index, ()->new PaymentDetail.PaymentDetailBuilder());
		}
		
	
		public IndependentAmountBuilder addPaymentDetail(PaymentDetail paymentDetail) {
			if(this.paymentDetail == null){
				this.paymentDetail = new ArrayList<>();
			}
			this.paymentDetail.add(paymentDetail.toBuilder());
			return this;
		}
		public IndependentAmountBuilder addPaymentDetail(PaymentDetail paymentDetail, int _idx) {
			if(this.paymentDetail == null){
				this.paymentDetail = new ArrayList<>();
			}
			getIndex(this.paymentDetail, _idx, () -> paymentDetail.toBuilder());
			this.paymentDetail.set(_idx, paymentDetail.toBuilder());
			return this;
		}
		public IndependentAmountBuilder addPaymentDetail(List<PaymentDetail> paymentDetails) {
			if(this.paymentDetail == null){
				this.paymentDetail = new ArrayList<>();
			}
			for (PaymentDetail toAdd : paymentDetails) {
				this.paymentDetail.add(toAdd.toBuilder());
			}
			return this;
		}
		public IndependentAmountBuilder addPaymentDetailBuilder(PaymentDetail.PaymentDetailBuilder paymentDetail) {
			if(this.paymentDetail == null){
				this.paymentDetail = new ArrayList<>();
			}
			this.paymentDetail.add(paymentDetail);
			return this;
		}
		
		public IndependentAmountBuilder clearPaymentDetail() {
			this.paymentDetail = null;
			return this;
		}
		@Override public IndependentAmountBuilder setPayerAccountReference(ReferenceWithMetaAccount payerAccountReference) {
			this.payerAccountReference = payerAccountReference.toBuilder();
			return this;
		}
		@Override public IndependentAmountBuilder setPayerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder payerAccountReference) {
			this.payerAccountReference = payerAccountReference;
			return this;
		}
		@Override public IndependentAmountBuilder setPayerAccountReferenceRef(Account.AccountBuilder payerAccountReference) {
			return setPayerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(payerAccountReference).build());
		}
		@Override public IndependentAmountBuilder setPayerAccountReferenceRef(Account payerAccountReference) {
			return setPayerAccountReferenceRef(Account.builder());
		}
		@Override public IndependentAmountBuilder setPayerPartyReference(ReferenceWithMetaParty payerPartyReference) {
			this.payerPartyReference = payerPartyReference.toBuilder();
			return this;
		}
		@Override public IndependentAmountBuilder setPayerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder payerPartyReference) {
			this.payerPartyReference = payerPartyReference;
			return this;
		}
		@Override public IndependentAmountBuilder setPayerPartyReferenceRef(Party.PartyBuilder payerPartyReference) {
			return setPayerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(payerPartyReference).build());
		}
		@Override public IndependentAmountBuilder setPayerPartyReferenceRef(Party payerPartyReference) {
			return setPayerPartyReferenceRef(Party.builder());
		}
		@Override public IndependentAmountBuilder setReceiverAccountReference(ReferenceWithMetaAccount receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference.toBuilder();
			return this;
		}
		@Override public IndependentAmountBuilder setReceiverAccountReferenceBuilder(ReferenceWithMetaAccountBuilder receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference;
			return this;
		}
		@Override public IndependentAmountBuilder setReceiverAccountReferenceRef(Account.AccountBuilder receiverAccountReference) {
			return setReceiverAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(receiverAccountReference).build());
		}
		@Override public IndependentAmountBuilder setReceiverAccountReferenceRef(Account receiverAccountReference) {
			return setReceiverAccountReferenceRef(Account.builder());
		}
		@Override public IndependentAmountBuilder setReceiverPartyReference(ReferenceWithMetaParty receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference.toBuilder();
			return this;
		}
		@Override public IndependentAmountBuilder setReceiverPartyReferenceBuilder(ReferenceWithMetaPartyBuilder receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference;
			return this;
		}
		@Override public IndependentAmountBuilder setReceiverPartyReferenceRef(Party.PartyBuilder receiverPartyReference) {
			return setReceiverPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(receiverPartyReference).build());
		}
		@Override public IndependentAmountBuilder setReceiverPartyReferenceRef(Party receiverPartyReference) {
			return setReceiverPartyReferenceRef(Party.builder());
		}
		
		public IndependentAmount build() {
			return new IndependentAmount(this);
		}
	
		@Override
		public IndependentAmountBuilder prune() {
			super.prune();
			if (paymentDetail!=null) paymentDetail = paymentDetail.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getPaymentDetail()!=null && getPaymentDetail().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("paymentDetail"), processor, PaymentDetail.class, paymentDetail);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			IndependentAmountBuilder _that = (IndependentAmountBuilder) o;
		
			if (!ListEquals.listEquals(paymentDetail, _that.paymentDetail)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (paymentDetail != null ? paymentDetail.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IndependentAmountBuilder {" +
				"paymentDetail=" + this.paymentDetail +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("paymentDetail"), processor, PaymentDetail.class, paymentDetail);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		IndependentAmount _that = (IndependentAmount) o;
	
		if (!ListEquals.listEquals(paymentDetail, _that.paymentDetail)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (paymentDetail != null ? paymentDetail.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IndependentAmount {" +
			"paymentDetail=" + this.paymentDetail +
		'}' + " " + super.toString();
	}
}
