package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.Integer;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.SettledEntityMatrix;
import org.isda.cdm.Tranche;
import org.isda.cdm.metafields.FieldWithMetaIndexAnnexSourceEnum;
import org.isda.cdm.metafields.FieldWithMetaIndexAnnexSourceEnum.FieldWithMetaIndexAnnexSourceEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IndexReferenceInformationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a Credit Default Swap Index.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="IndexReferenceInformation", source="FpML_5_10")
@RosettaSynonym(value="IndexReferenceInformation", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="IndexReferenceInformation", source="DTCC_11_0")
@RosettaSynonym(value="IndexReferenceInformation", source="DTCC_9_0")
@RosettaSynonym(value="IndexReferenceInformation", source="CME_ClearedConfirm_1_17")
public class IndexReferenceInformation extends RosettaModelObject implements GlobalKey {
	private final List<LegalEntity> excludedReferenceEntity;
	private final Date indexAnnexDate;
	private final FieldWithMetaIndexAnnexSourceEnum indexAnnexSource;
	private final Integer indexAnnexVersion;
	private final List<FieldWithMetaString> indexId;
	private final FieldWithMetaString indexName;
	private final Integer indexSeries;
	private final MetaFields meta;
	private final SettledEntityMatrix settledEntityMatrix;
	private final Tranche tranche;
	private static IndexReferenceInformationMeta metaData = new IndexReferenceInformationMeta();
	
	IndexReferenceInformation(IndexReferenceInformationBuilder builder) {
		this.excludedReferenceEntity = ofNullable(builder.getExcludedReferenceEntity()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.indexAnnexDate = builder.getIndexAnnexDate();
		this.indexAnnexSource = ofNullable(builder.getIndexAnnexSource()).map(f->f.build()).orElse(null);
		this.indexAnnexVersion = builder.getIndexAnnexVersion();
		this.indexId = ofNullable(builder.getIndexId()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.indexName = ofNullable(builder.getIndexName()).map(f->f.build()).orElse(null);
		this.indexSeries = builder.getIndexSeries();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.settledEntityMatrix = ofNullable(builder.getSettledEntityMatrix()).map(f->f.build()).orElse(null);
		this.tranche = ofNullable(builder.getTranche()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Excluded reference entity.
	 */
	@RosettaSynonym(value="excludedReferenceEntity", source="FpML_5_10")
	@RosettaSynonym(value="excludedReferenceEntity", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="excludedReferenceEntity", source="DTCC_11_0")
	@RosettaSynonym(value="excludedReferenceEntity", source="DTCC_9_0")
	@RosettaSynonym(value="excludedReferenceEntity", source="CME_ClearedConfirm_1_17")
	public final List<LegalEntity> getExcludedReferenceEntity() {
		return excludedReferenceEntity;
	}
	
	/**
	 * A CDS index series annex date.
	 */
	@RosettaSynonym(value="indexAnnexDate", source="FpML_5_10")
	@RosettaSynonym(value="indexAnnexDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexAnnexDate", source="DTCC_11_0")
	@RosettaSynonym(value="indexAnnexDate", source="DTCC_9_0")
	@RosettaSynonym(value="indexAnnexDate", source="CME_ClearedConfirm_1_17")
	public final Date getIndexAnnexDate() {
		return indexAnnexDate;
	}
	
	/**
	 * A CDS index series annex source.
	 */
	@RosettaSynonym(value="indexAnnexSource", source="FpML_5_10")
	@RosettaSynonym(value="indexAnnexSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexAnnexSource", source="DTCC_11_0")
	@RosettaSynonym(value="indexAnnexSource", source="DTCC_9_0")
	@RosettaSynonym(value="indexAnnexSource", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaIndexAnnexSourceEnum getIndexAnnexSource() {
		return indexAnnexSource;
	}
	
	/**
	 * A CDS index series version identifier, e.g. 1, 2, 3 etc.
	 */
	@RosettaSynonym(value="indexAnnexVersion", source="FpML_5_10")
	@RosettaSynonym(value="indexAnnexVersion", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexAnnexVersion", source="DTCC_11_0")
	@RosettaSynonym(value="indexAnnexVersion", source="DTCC_9_0")
	@RosettaSynonym(value="indexAnnexVersion", source="CME_ClearedConfirm_1_17")
	public final Integer getIndexAnnexVersion() {
		return indexAnnexVersion;
	}
	
	/**
	 * A CDS index identifier (e.g. RED pair code).
	 */
	@RosettaSynonym(value="indexId", source="FpML_5_10")
	@RosettaSynonym(value="indexId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexId", source="DTCC_11_0")
	@RosettaSynonym(value="indexId", source="DTCC_9_0")
	@RosettaSynonym(value="indexId", source="CME_ClearedConfirm_1_17")
	public final List<FieldWithMetaString> getIndexId() {
		return indexId;
	}
	
	/**
	 * The name of the index expressed as a free format string with an associated scheme.
	 */
	@RosettaSynonym(value="indexName", source="FpML_5_10")
	@RosettaSynonym(value="indexName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexName", source="DTCC_11_0")
	@RosettaSynonym(value="indexName", source="DTCC_9_0")
	@RosettaSynonym(value="indexName", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIndexName() {
		return indexName;
	}
	
	/**
	 * A CDS index series identifier, e.g. 1, 2, 3 etc.
	 */
	@RosettaSynonym(value="indexSeries", source="FpML_5_10")
	@RosettaSynonym(value="indexSeries", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexSeries", source="DTCC_11_0")
	@RosettaSynonym(value="indexSeries", source="DTCC_9_0")
	@RosettaSynonym(value="indexSeries", source="CME_ClearedConfirm_1_17")
	public final Integer getIndexSeries() {
		return indexSeries;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Used to specify the Relevant Settled Entity Matrix when there are settled entities at the time of the trade.
	 */
	@RosettaSynonym(value="settledEntityMatrix", source="FpML_5_10")
	@RosettaSynonym(value="settledEntityMatrix", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="settledEntityMatrix", source="DTCC_11_0")
	@RosettaSynonym(value="settledEntityMatrix", source="DTCC_9_0")
	@RosettaSynonym(value="settledEntityMatrix", source="CME_ClearedConfirm_1_17")
	public final SettledEntityMatrix getSettledEntityMatrix() {
		return settledEntityMatrix;
	}
	
	/**
	 * This element contains CDS tranche terms.
	 */
	@RosettaSynonym(value="tranche", source="FpML_5_10")
	@RosettaSynonym(value="tranche", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tranche", source="DTCC_11_0")
	@RosettaSynonym(value="tranche", source="DTCC_9_0")
	@RosettaSynonym(value="tranche", source="CME_ClearedConfirm_1_17")
	public final Tranche getTranche() {
		return tranche;
	}
	
	
	@Override
	public RosettaMetaData<? extends IndexReferenceInformation> metaData() {
		return metaData;
	} 
	
	public IndexReferenceInformationBuilder toBuilder() {
		IndexReferenceInformationBuilder builder = new IndexReferenceInformationBuilder();
		ofNullable(getExcludedReferenceEntity()).ifPresent(excludedReferenceEntity -> excludedReferenceEntity.forEach(builder::addExcludedReferenceEntity));
		ofNullable(getIndexAnnexDate()).ifPresent(builder::setIndexAnnexDate);
		ofNullable(getIndexAnnexSource()).ifPresent(builder::setIndexAnnexSource);
		ofNullable(getIndexAnnexVersion()).ifPresent(builder::setIndexAnnexVersion);
		ofNullable(getIndexId()).ifPresent(indexId -> indexId.forEach(builder::addIndexId));
		ofNullable(getIndexName()).ifPresent(builder::setIndexName);
		ofNullable(getIndexSeries()).ifPresent(builder::setIndexSeries);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSettledEntityMatrix()).ifPresent(builder::setSettledEntityMatrix);
		ofNullable(getTranche()).ifPresent(builder::setTranche);
		return builder;
	}

	public static IndexReferenceInformationBuilder builder() {
		return new IndexReferenceInformationBuilder();
	}

	public static  class IndexReferenceInformationBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<IndexReferenceInformationBuilder> {
	
		protected List<LegalEntity.LegalEntityBuilder> excludedReferenceEntity;
		protected Date indexAnnexDate;
		protected FieldWithMetaIndexAnnexSourceEnumBuilder indexAnnexSource;
		protected Integer indexAnnexVersion;
		protected List<FieldWithMetaStringBuilder> indexId;
		protected FieldWithMetaStringBuilder indexName;
		protected Integer indexSeries;
		protected MetaFields.MetaFieldsBuilder meta;
		protected SettledEntityMatrix.SettledEntityMatrixBuilder settledEntityMatrix;
		protected Tranche.TrancheBuilder tranche;
	
		public IndexReferenceInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IndexReferenceInformation> metaData() {
			return metaData;
		} 
	
		
		public List<LegalEntity.LegalEntityBuilder> getExcludedReferenceEntity() {
			return excludedReferenceEntity;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateExcludedReferenceEntity(int index) {
			if (excludedReferenceEntity==null) {
				this.excludedReferenceEntity = new ArrayList<>();
			}
			return getIndex(excludedReferenceEntity, index, ()->new LegalEntity.LegalEntityBuilder());
		}
		
		
		public Date getIndexAnnexDate() {
			return indexAnnexDate;
		}
		
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder getIndexAnnexSource() {
			return indexAnnexSource;
		}
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder getOrCreateIndexAnnexSource() {
			if (indexAnnexSource!=null) {
				return indexAnnexSource;
			}
			else return indexAnnexSource = new FieldWithMetaIndexAnnexSourceEnumBuilder();
		}
		
		
		public Integer getIndexAnnexVersion() {
			return indexAnnexVersion;
		}
		
		
		public List<FieldWithMetaStringBuilder> getIndexId() {
			return indexId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIndexId(int index) {
			if (indexId==null) {
				this.indexId = new ArrayList<>();
			}
			return getIndex(indexId, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getIndexName() {
			return indexName;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIndexName() {
			if (indexName!=null) {
				return indexName;
			}
			else return indexName = new FieldWithMetaStringBuilder();
		}
		
		
		public Integer getIndexSeries() {
			return indexSeries;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public SettledEntityMatrix.SettledEntityMatrixBuilder getSettledEntityMatrix() {
			return settledEntityMatrix;
		}
		
		public SettledEntityMatrix.SettledEntityMatrixBuilder getOrCreateSettledEntityMatrix() {
			if (settledEntityMatrix!=null) {
				return settledEntityMatrix;
			}
			else return settledEntityMatrix = new SettledEntityMatrix.SettledEntityMatrixBuilder();
		}
		
		
		public Tranche.TrancheBuilder getTranche() {
			return tranche;
		}
		
		public Tranche.TrancheBuilder getOrCreateTranche() {
			if (tranche!=null) {
				return tranche;
			}
			else return tranche = new Tranche.TrancheBuilder();
		}
		
	
		public IndexReferenceInformationBuilder addExcludedReferenceEntity(LegalEntity excludedReferenceEntity) {
			if(this.excludedReferenceEntity == null){
				this.excludedReferenceEntity = new ArrayList<>();
			}
			this.excludedReferenceEntity.add(excludedReferenceEntity.toBuilder());
			return this;
		}
		public IndexReferenceInformationBuilder addExcludedReferenceEntity(LegalEntity excludedReferenceEntity, int _idx) {
			if(this.excludedReferenceEntity == null){
				this.excludedReferenceEntity = new ArrayList<>();
			}
			getIndex(this.excludedReferenceEntity, _idx, () -> excludedReferenceEntity.toBuilder());
			this.excludedReferenceEntity.set(_idx, excludedReferenceEntity.toBuilder());
			return this;
		}
		public IndexReferenceInformationBuilder addExcludedReferenceEntity(List<LegalEntity> excludedReferenceEntitys) {
			if(this.excludedReferenceEntity == null){
				this.excludedReferenceEntity = new ArrayList<>();
			}
			for (LegalEntity toAdd : excludedReferenceEntitys) {
				this.excludedReferenceEntity.add(toAdd.toBuilder());
			}
			return this;
		}
		public IndexReferenceInformationBuilder addExcludedReferenceEntityBuilder(LegalEntity.LegalEntityBuilder excludedReferenceEntity) {
			if(this.excludedReferenceEntity == null){
				this.excludedReferenceEntity = new ArrayList<>();
			}
			this.excludedReferenceEntity.add(excludedReferenceEntity);
			return this;
		}
		
		public IndexReferenceInformationBuilder clearExcludedReferenceEntity() {
			this.excludedReferenceEntity = null;
			return this;
		}
		public IndexReferenceInformationBuilder setIndexAnnexDate(Date indexAnnexDate) {
			this.indexAnnexDate = indexAnnexDate;
			return this;
		}
		public IndexReferenceInformationBuilder setIndexAnnexSource(FieldWithMetaIndexAnnexSourceEnum indexAnnexSource) {
			this.indexAnnexSource = indexAnnexSource.toBuilder();
			return this;
		}
		public IndexReferenceInformationBuilder setIndexAnnexSourceRef(IndexAnnexSourceEnum indexAnnexSource) {
			return setIndexAnnexSource(FieldWithMetaIndexAnnexSourceEnum.builder().setValue(indexAnnexSource).build());
		}
		public IndexReferenceInformationBuilder setIndexAnnexVersion(Integer indexAnnexVersion) {
			this.indexAnnexVersion = indexAnnexVersion;
			return this;
		}
		public IndexReferenceInformationBuilder addIndexId(FieldWithMetaString indexId) {
			if(this.indexId == null){
				this.indexId = new ArrayList<>();
			}
			this.indexId.add(indexId.toBuilder());
			return this;
		}
		public IndexReferenceInformationBuilder addIndexId(FieldWithMetaString indexId, int _idx) {
			if(this.indexId == null){
				this.indexId = new ArrayList<>();
			}
			getIndex(this.indexId, _idx, () -> indexId.toBuilder());
			this.indexId.set(_idx, indexId.toBuilder());
			return this;
		}
		public IndexReferenceInformationBuilder addIndexId(List<FieldWithMetaString> indexIds) {
			if(this.indexId == null){
				this.indexId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : indexIds) {
				this.indexId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public IndexReferenceInformationBuilder clearIndexId() {
			this.indexId = null;
			return this;
		}
		public IndexReferenceInformationBuilder setIndexName(FieldWithMetaString indexName) {
			this.indexName = indexName.toBuilder();
			return this;
		}
		public IndexReferenceInformationBuilder setIndexNameRef(String indexName) {
			return setIndexName(FieldWithMetaString.builder().setValue(indexName).build());
		}
		public IndexReferenceInformationBuilder setIndexSeries(Integer indexSeries) {
			this.indexSeries = indexSeries;
			return this;
		}
		public IndexReferenceInformationBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public IndexReferenceInformationBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public IndexReferenceInformationBuilder setSettledEntityMatrix(SettledEntityMatrix settledEntityMatrix) {
			this.settledEntityMatrix = settledEntityMatrix.toBuilder();
			return this;
		}
		public IndexReferenceInformationBuilder setSettledEntityMatrixBuilder(SettledEntityMatrix.SettledEntityMatrixBuilder settledEntityMatrix) {
			this.settledEntityMatrix = settledEntityMatrix;
			return this;
		}
		public IndexReferenceInformationBuilder setTranche(Tranche tranche) {
			this.tranche = tranche.toBuilder();
			return this;
		}
		public IndexReferenceInformationBuilder setTrancheBuilder(Tranche.TrancheBuilder tranche) {
			this.tranche = tranche;
			return this;
		}
		
		public IndexReferenceInformation build() {
			return new IndexReferenceInformation(this);
		}
	
		@Override
		public IndexReferenceInformationBuilder prune() {
			if (excludedReferenceEntity!=null) excludedReferenceEntity = excludedReferenceEntity.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (indexAnnexSource!=null && !indexAnnexSource.prune().hasData()) indexAnnexSource = null;
			if (indexId!=null) indexId = indexId.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (indexName!=null && !indexName.prune().hasData()) indexName = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (settledEntityMatrix!=null && !settledEntityMatrix.prune().hasData()) settledEntityMatrix = null;
			if (tranche!=null && !tranche.prune().hasData()) tranche = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getExcludedReferenceEntity()!=null && getExcludedReferenceEntity().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getIndexAnnexDate()!=null) return true;
			if (getIndexAnnexSource()!=null) return true;
			if (getIndexAnnexVersion()!=null) return true;
			if (getIndexId()!=null && !getIndexId().isEmpty()) return true;
			if (getIndexName()!=null) return true;
			if (getIndexSeries()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSettledEntityMatrix()!=null && getSettledEntityMatrix().hasData()) return true;
			if (getTranche()!=null && getTranche().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("indexAnnexDate"), Date.class, indexAnnexDate, this);
			processor.processBasic(path.newSubPath("indexAnnexVersion"), Integer.class, indexAnnexVersion, this);
			processor.processBasic(path.newSubPath("indexSeries"), Integer.class, indexSeries, this);
			
			processRosetta(path.newSubPath("excludedReferenceEntity"), processor, LegalEntity.class, excludedReferenceEntity);
			processRosetta(path.newSubPath("indexAnnexSource"), processor, FieldWithMetaIndexAnnexSourceEnum.class, indexAnnexSource);
			processRosetta(path.newSubPath("indexId"), processor, FieldWithMetaString.class, indexId);
			processRosetta(path.newSubPath("indexName"), processor, FieldWithMetaString.class, indexName);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("settledEntityMatrix"), processor, SettledEntityMatrix.class, settledEntityMatrix);
			processRosetta(path.newSubPath("tranche"), processor, Tranche.class, tranche);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IndexReferenceInformationBuilder _that = (IndexReferenceInformationBuilder) o;
		
			if (!ListEquals.listEquals(excludedReferenceEntity, _that.excludedReferenceEntity)) return false;
			if (indexAnnexDate != null ? !indexAnnexDate.equals(_that.indexAnnexDate) : _that.indexAnnexDate != null) return false;
			if (indexAnnexSource != null ? !indexAnnexSource.equals(_that.indexAnnexSource) : _that.indexAnnexSource != null) return false;
			if (indexAnnexVersion != null ? !indexAnnexVersion.equals(_that.indexAnnexVersion) : _that.indexAnnexVersion != null) return false;
			if (!ListEquals.listEquals(indexId, _that.indexId)) return false;
			if (indexName != null ? !indexName.equals(_that.indexName) : _that.indexName != null) return false;
			if (indexSeries != null ? !indexSeries.equals(_that.indexSeries) : _that.indexSeries != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (settledEntityMatrix != null ? !settledEntityMatrix.equals(_that.settledEntityMatrix) : _that.settledEntityMatrix != null) return false;
			if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (excludedReferenceEntity != null ? excludedReferenceEntity.hashCode() : 0);
			_result = 31 * _result + (indexAnnexDate != null ? indexAnnexDate.hashCode() : 0);
			_result = 31 * _result + (indexAnnexSource != null ? indexAnnexSource.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (indexAnnexVersion != null ? indexAnnexVersion.hashCode() : 0);
			_result = 31 * _result + (indexId != null ? indexId.hashCode() : 0);
			_result = 31 * _result + (indexName != null ? indexName.hashCode() : 0);
			_result = 31 * _result + (indexSeries != null ? indexSeries.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (settledEntityMatrix != null ? settledEntityMatrix.hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IndexReferenceInformationBuilder {" +
				"excludedReferenceEntity=" + this.excludedReferenceEntity + ", " +
				"indexAnnexDate=" + this.indexAnnexDate + ", " +
				"indexAnnexSource=" + this.indexAnnexSource + ", " +
				"indexAnnexVersion=" + this.indexAnnexVersion + ", " +
				"indexId=" + this.indexId + ", " +
				"indexName=" + this.indexName + ", " +
				"indexSeries=" + this.indexSeries + ", " +
				"meta=" + this.meta + ", " +
				"settledEntityMatrix=" + this.settledEntityMatrix + ", " +
				"tranche=" + this.tranche +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("indexAnnexDate"), Date.class, indexAnnexDate, this);
		processor.processBasic(path.newSubPath("indexAnnexVersion"), Integer.class, indexAnnexVersion, this);
		processor.processBasic(path.newSubPath("indexSeries"), Integer.class, indexSeries, this);
		
		processRosetta(path.newSubPath("excludedReferenceEntity"), processor, LegalEntity.class, excludedReferenceEntity);
		processRosetta(path.newSubPath("indexAnnexSource"), processor, FieldWithMetaIndexAnnexSourceEnum.class, indexAnnexSource);
		processRosetta(path.newSubPath("indexId"), processor, FieldWithMetaString.class, indexId);
		processRosetta(path.newSubPath("indexName"), processor, FieldWithMetaString.class, indexName);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("settledEntityMatrix"), processor, SettledEntityMatrix.class, settledEntityMatrix);
		processRosetta(path.newSubPath("tranche"), processor, Tranche.class, tranche);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		IndexReferenceInformation _that = (IndexReferenceInformation) o;
	
		if (!ListEquals.listEquals(excludedReferenceEntity, _that.excludedReferenceEntity)) return false;
		if (indexAnnexDate != null ? !indexAnnexDate.equals(_that.indexAnnexDate) : _that.indexAnnexDate != null) return false;
		if (indexAnnexSource != null ? !indexAnnexSource.equals(_that.indexAnnexSource) : _that.indexAnnexSource != null) return false;
		if (indexAnnexVersion != null ? !indexAnnexVersion.equals(_that.indexAnnexVersion) : _that.indexAnnexVersion != null) return false;
		if (!ListEquals.listEquals(indexId, _that.indexId)) return false;
		if (indexName != null ? !indexName.equals(_that.indexName) : _that.indexName != null) return false;
		if (indexSeries != null ? !indexSeries.equals(_that.indexSeries) : _that.indexSeries != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (settledEntityMatrix != null ? !settledEntityMatrix.equals(_that.settledEntityMatrix) : _that.settledEntityMatrix != null) return false;
		if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (excludedReferenceEntity != null ? excludedReferenceEntity.hashCode() : 0);
		_result = 31 * _result + (indexAnnexDate != null ? indexAnnexDate.hashCode() : 0);
		_result = 31 * _result + (indexAnnexSource != null ? indexAnnexSource.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (indexAnnexVersion != null ? indexAnnexVersion.hashCode() : 0);
		_result = 31 * _result + (indexId != null ? indexId.hashCode() : 0);
		_result = 31 * _result + (indexName != null ? indexName.hashCode() : 0);
		_result = 31 * _result + (indexSeries != null ? indexSeries.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (settledEntityMatrix != null ? settledEntityMatrix.hashCode() : 0);
		_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IndexReferenceInformation {" +
			"excludedReferenceEntity=" + this.excludedReferenceEntity + ", " +
			"indexAnnexDate=" + this.indexAnnexDate + ", " +
			"indexAnnexSource=" + this.indexAnnexSource + ", " +
			"indexAnnexVersion=" + this.indexAnnexVersion + ", " +
			"indexId=" + this.indexId + ", " +
			"indexName=" + this.indexName + ", " +
			"indexSeries=" + this.indexSeries + ", " +
			"meta=" + this.meta + ", " +
			"settledEntityMatrix=" + this.settledEntityMatrix + ", " +
			"tranche=" + this.tranche +
		'}';
	}
}
