package org.isda.cdm;

import org.isda.cdm.Nm;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IndxMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Indx extends RosettaModelObject {
	private final Nm nm;
	private static IndxMeta metaData = new IndxMeta();
	
	Indx(IndxBuilder builder) {
		this.nm = ofNullable(builder.getNm()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Nm", source="ISO20022")
	public final Nm getNm() {
		return nm;
	}
	
	
	@Override
	public RosettaMetaData<? extends Indx> metaData() {
		return metaData;
	} 
	
	public IndxBuilder toBuilder() {
		IndxBuilder builder = new IndxBuilder();
		ofNullable(getNm()).ifPresent(builder::setNm);
		return builder;
	}

	public static IndxBuilder builder() {
		return new IndxBuilder();
	}

	public static  class IndxBuilder extends RosettaModelObjectBuilder {
	
		protected Nm.NmBuilder nm;
	
		public IndxBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Indx> metaData() {
			return metaData;
		} 
	
		
		public Nm.NmBuilder getNm() {
			return nm;
		}
		
		public Nm.NmBuilder getOrCreateNm() {
			if (nm!=null) {
				return nm;
			}
			else return nm = new Nm.NmBuilder();
		}
		
	
		public IndxBuilder setNm(Nm nm) {
			this.nm = nm.toBuilder();
			return this;
		}
		public IndxBuilder setNmBuilder(Nm.NmBuilder nm) {
			this.nm = nm;
			return this;
		}
		
		public Indx build() {
			return new Indx(this);
		}
	
		@Override
		public IndxBuilder prune() {
			if (nm!=null && !nm.prune().hasData()) nm = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNm()!=null && getNm().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("nm"), processor, Nm.class, nm);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IndxBuilder _that = (IndxBuilder) o;
		
			if (nm != null ? !nm.equals(_that.nm) : _that.nm != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (nm != null ? nm.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IndxBuilder {" +
				"nm=" + this.nm +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("nm"), processor, Nm.class, nm);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Indx _that = (Indx) o;
	
		if (nm != null ? !nm.equals(_that.nm) : _that.nm != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (nm != null ? nm.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Indx {" +
			"nm=" + this.nm +
		'}';
	}
}
