package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IneligibleCreditSupportMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the parties to which the provisions of Paragraph 11(g) of the ISDA 2016 Credit Support Annex for Variation Margin will apply to. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(iii): Legally Ineligible Credit Support (VM).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class IneligibleCreditSupport extends RosettaModelObject {
	private final List<ReferenceWithMetaParty> specifiedParty;
	private static IneligibleCreditSupportMeta metaData = new IneligibleCreditSupportMeta();
	
	IneligibleCreditSupport(IneligibleCreditSupportBuilder builder) {
		this.specifiedParty = ofNullable(builder.getSpecifiedParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties to which the provisions of Paragraph 11(g) of the ISDA 2016 Credit Support Annex for Variation Margin will apply to, as the Secured Party. ISDA 2016 Credit Support Annex for Variation Margin, paragraph 13, (c)(iii): Legally Ineligible Credit Support (VM).
	 */
	public final List<ReferenceWithMetaParty> getSpecifiedParty() {
		return specifiedParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends IneligibleCreditSupport> metaData() {
		return metaData;
	} 
	
	public IneligibleCreditSupportBuilder toBuilder() {
		IneligibleCreditSupportBuilder builder = new IneligibleCreditSupportBuilder();
		ofNullable(getSpecifiedParty()).ifPresent(specifiedParty -> specifiedParty.forEach(builder::addSpecifiedParty));
		return builder;
	}

	public static IneligibleCreditSupportBuilder builder() {
		return new IneligibleCreditSupportBuilder();
	}

	public static  class IneligibleCreditSupportBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaPartyBuilder> specifiedParty;
	
		public IneligibleCreditSupportBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IneligibleCreditSupport> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaPartyBuilder> getSpecifiedParty() {
			return specifiedParty;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateSpecifiedParty(int index) {
			if (specifiedParty==null) {
				this.specifiedParty = new ArrayList<>();
			}
			return getIndex(specifiedParty, index, ()->new ReferenceWithMetaPartyBuilder());
		}
		
	
		public IneligibleCreditSupportBuilder addSpecifiedParty(ReferenceWithMetaParty specifiedParty) {
			if(this.specifiedParty == null){
				this.specifiedParty = new ArrayList<>();
			}
			this.specifiedParty.add(specifiedParty.toBuilder());
			return this;
		}
		public IneligibleCreditSupportBuilder addSpecifiedParty(ReferenceWithMetaParty specifiedParty, int _idx) {
			if(this.specifiedParty == null){
				this.specifiedParty = new ArrayList<>();
			}
			getIndex(this.specifiedParty, _idx, () -> specifiedParty.toBuilder());
			this.specifiedParty.set(_idx, specifiedParty.toBuilder());
			return this;
		}
		public IneligibleCreditSupportBuilder addSpecifiedParty(List<ReferenceWithMetaParty> specifiedPartys) {
			if(this.specifiedParty == null){
				this.specifiedParty = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : specifiedPartys) {
				this.specifiedParty.add(toAdd.toBuilder());
			}
			return this;
		}
		public IneligibleCreditSupportBuilder addSpecifiedPartyBuilder(ReferenceWithMetaPartyBuilder specifiedParty) {
			if(this.specifiedParty == null){
				this.specifiedParty = new ArrayList<>();
			}
			this.specifiedParty.add(specifiedParty);
			return this;
		}
		
		public IneligibleCreditSupportBuilder addSpecifiedPartyRef(Party.PartyBuilder specifiedParty) {
			return addSpecifiedParty(ReferenceWithMetaParty.builder().setValueBuilder(specifiedParty).build());
		}
		
		public IneligibleCreditSupportBuilder addSpecifiedPartyRef(Party.PartyBuilder specifiedParty, int _idx) {
			return addSpecifiedParty(ReferenceWithMetaParty.builder().setValueBuilder(specifiedParty).build(), _idx);
		}
		
		public IneligibleCreditSupportBuilder addSpecifiedPartyRef(List<Party> specifiedPartys) {
			for (Party toAdd : specifiedPartys) {
				addSpecifiedPartyRef(toAdd);
			}
			return this;
		}
		
		public IneligibleCreditSupportBuilder addSpecifiedPartyRef(Party specifiedParty) {
			return addSpecifiedPartyRef(specifiedParty.toBuilder());
		}
		
		public IneligibleCreditSupportBuilder clearSpecifiedParty() {
			this.specifiedParty = null;
			return this;
		}
		
		public IneligibleCreditSupport build() {
			return new IneligibleCreditSupport(this);
		}
	
		@Override
		public IneligibleCreditSupportBuilder prune() {
			if (specifiedParty!=null) specifiedParty = specifiedParty.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSpecifiedParty()!=null && getSpecifiedParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("specifiedParty"), processor, ReferenceWithMetaParty.class, specifiedParty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IneligibleCreditSupportBuilder _that = (IneligibleCreditSupportBuilder) o;
		
			if (!ListEquals.listEquals(specifiedParty, _that.specifiedParty)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (specifiedParty != null ? specifiedParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IneligibleCreditSupportBuilder {" +
				"specifiedParty=" + this.specifiedParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("specifiedParty"), processor, ReferenceWithMetaParty.class, specifiedParty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		IneligibleCreditSupport _that = (IneligibleCreditSupport) o;
	
		if (!ListEquals.listEquals(specifiedParty, _that.specifiedParty)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (specifiedParty != null ? specifiedParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IneligibleCreditSupport {" +
			"specifiedParty=" + this.specifiedParty +
		'}';
	}
}
