package org.isda.cdm;

import java.lang.Boolean;
import java.math.BigDecimal;
import org.isda.cdm.Offset;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum;
import org.isda.cdm.metafields.FieldWithMetaInterpolationMethodEnum;
import org.isda.cdm.metafields.FieldWithMetaInterpolationMethodEnum.FieldWithMetaInterpolationMethodEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InflationRateSpecificationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the inflation rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="InflationRateCalculation", source="FpML_5_10")

@RosettaSynonym(value="InfltnIndx", source="ISO20022")
public class InflationRateSpecification extends FloatingRateSpecification {
	private final Boolean fallbackBondApplicable;
	private final FieldWithMetaString indexSource;
	private final Offset inflationLag;
	private final BigDecimal initialIndexLevel;
	private final FieldWithMetaInterpolationMethodEnum interpolationMethod;
	private final FieldWithMetaString mainPublication;
	private static InflationRateSpecificationMeta metaData = new InflationRateSpecificationMeta();
	
	InflationRateSpecification(InflationRateSpecificationBuilder builder) {
		super(builder);
		this.fallbackBondApplicable = builder.getFallbackBondApplicable();
		this.indexSource = ofNullable(builder.getIndexSource()).map(f->f.build()).orElse(null);
		this.inflationLag = ofNullable(builder.getInflationLag()).map(f->f.build()).orElse(null);
		this.initialIndexLevel = builder.getInitialIndexLevel();
		this.interpolationMethod = ofNullable(builder.getInterpolationMethod()).map(f->f.build()).orElse(null);
		this.mainPublication = ofNullable(builder.getMainPublication()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The applicability of a fallback bond as defined in the 2006 ISDA Inflation Derivatives Definitions, sections 1.3 and 1.8.
	 */
	@RosettaSynonym(value="fallbackBondApplicable", source="FpML_5_10")
	@RosettaSynonym(value="fallbackBondApplicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fallbackBondApplicable", source="DTCC_11_0")
	@RosettaSynonym(value="fallbackBondApplicable", source="DTCC_9_0")
	@RosettaSynonym(value="fallbackBondApplicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getFallbackBondApplicable() {
		return fallbackBondApplicable;
	}
	
	/**
	 * The reference source such as Reuters or Bloomberg. FpML specifies indexSource to be of type rateSourcePageScheme, but without specifying actual values.
	 */
	@RosettaSynonym(value="indexSource", source="FpML_5_10")
	@RosettaSynonym(value="indexSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexSource", source="DTCC_11_0")
	@RosettaSynonym(value="indexSource", source="DTCC_9_0")
	@RosettaSynonym(value="indexSource", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIndexSource() {
		return indexSource;
	}
	
	/**
	 * An off-setting period from the payment date which determines the reference period for which the inflation index is observed.
	 */
	@RosettaSynonym(value="inflationLag", source="FpML_5_10")
	@RosettaSynonym(value="inflationLag", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="inflationLag", source="DTCC_11_0")
	@RosettaSynonym(value="inflationLag", source="DTCC_9_0")
	@RosettaSynonym(value="inflationLag", source="CME_ClearedConfirm_1_17")
	public final Offset getInflationLag() {
		return inflationLag;
	}
	
	/**
	 * Initial known index level for the first calculation period.
	 */
	@RosettaSynonym(value="initialIndexLevel", source="FpML_5_10")
	@RosettaSynonym(value="initialIndexLevel", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialIndexLevel", source="DTCC_11_0")
	@RosettaSynonym(value="initialIndexLevel", source="DTCC_9_0")
	@RosettaSynonym(value="initialIndexLevel", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getInitialIndexLevel() {
		return initialIndexLevel;
	}
	
	/**
	 * The method used when calculating the Inflation Index Level from multiple points. The most common is Linear.
	 */
	@RosettaSynonym(value="interpolationMethod", source="FpML_5_10")
	@RosettaSynonym(value="interpolationMethod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interpolationMethod", source="DTCC_11_0")
	@RosettaSynonym(value="interpolationMethod", source="DTCC_9_0")
	@RosettaSynonym(value="interpolationMethod", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaInterpolationMethodEnum getInterpolationMethod() {
		return interpolationMethod;
	}
	
	/**
	 * The current main publication source such as relevant web site or a government body. FpML specifies mainPublication to be of type mainPublicationSource, but without specifying actual values.
	 */
	@RosettaSynonym(value="mainPublication", source="FpML_5_10")
	@RosettaSynonym(value="mainPublication", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mainPublication", source="DTCC_11_0")
	@RosettaSynonym(value="mainPublication", source="DTCC_9_0")
	@RosettaSynonym(value="mainPublication", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getMainPublication() {
		return mainPublication;
	}
	
	
	@Override
	public RosettaMetaData<? extends InflationRateSpecification> metaData() {
		return metaData;
	} 
	
	public InflationRateSpecificationBuilder toBuilder() {
		InflationRateSpecificationBuilder builder = new InflationRateSpecificationBuilder();
		ofNullable(getFallbackBondApplicable()).ifPresent(builder::setFallbackBondApplicable);
		ofNullable(getIndexSource()).ifPresent(builder::setIndexSource);
		ofNullable(getInflationLag()).ifPresent(builder::setInflationLag);
		ofNullable(getInitialIndexLevel()).ifPresent(builder::setInitialIndexLevel);
		ofNullable(getInterpolationMethod()).ifPresent(builder::setInterpolationMethod);
		ofNullable(getMainPublication()).ifPresent(builder::setMainPublication);
		ofNullable(getAveragingMethod()).ifPresent(builder::setAveragingMethod);
		ofNullable(getFinalRateRounding()).ifPresent(builder::setFinalRateRounding);
		ofNullable(getInitialRate()).ifPresent(builder::setInitialRate);
		ofNullable(getNegativeInterestRateTreatment()).ifPresent(builder::setNegativeInterestRateTreatment);
		ofNullable(getCapRateSchedule()).ifPresent(capRateSchedule -> capRateSchedule.forEach(builder::addCapRateSchedule));
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getFloatingRateMultiplierSchedule()).ifPresent(builder::setFloatingRateMultiplierSchedule);
		ofNullable(getFloorRateSchedule()).ifPresent(floorRateSchedule -> floorRateSchedule.forEach(builder::addFloorRateSchedule));
		ofNullable(getIndexTenor()).ifPresent(builder::setIndexTenor);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getRateTreatment()).ifPresent(builder::setRateTreatment);
		ofNullable(getSpreadSchedule()).ifPresent(spreadSchedule -> spreadSchedule.forEach(builder::addSpreadSchedule));
		return builder;
	}

	public static InflationRateSpecificationBuilder builder() {
		return new InflationRateSpecificationBuilder();
	}

	public static  class InflationRateSpecificationBuilder extends FloatingRateSpecificationBuilder {
	
		protected Boolean fallbackBondApplicable;
		protected FieldWithMetaStringBuilder indexSource;
		protected Offset.OffsetBuilder inflationLag;
		protected BigDecimal initialIndexLevel;
		protected FieldWithMetaInterpolationMethodEnumBuilder interpolationMethod;
		protected FieldWithMetaStringBuilder mainPublication;
	
		public InflationRateSpecificationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InflationRateSpecification> metaData() {
			return metaData;
		} 
	
		
		public Boolean getFallbackBondApplicable() {
			return fallbackBondApplicable;
		}
		
		
		public FieldWithMetaStringBuilder getIndexSource() {
			return indexSource;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIndexSource() {
			if (indexSource!=null) {
				return indexSource;
			}
			else return indexSource = new FieldWithMetaStringBuilder();
		}
		
		
		public Offset.OffsetBuilder getInflationLag() {
			return inflationLag;
		}
		
		public Offset.OffsetBuilder getOrCreateInflationLag() {
			if (inflationLag!=null) {
				return inflationLag;
			}
			else return inflationLag = new Offset.OffsetBuilder();
		}
		
		
		public BigDecimal getInitialIndexLevel() {
			return initialIndexLevel;
		}
		
		
		public FieldWithMetaInterpolationMethodEnumBuilder getInterpolationMethod() {
			return interpolationMethod;
		}
		
		public FieldWithMetaInterpolationMethodEnumBuilder getOrCreateInterpolationMethod() {
			if (interpolationMethod!=null) {
				return interpolationMethod;
			}
			else return interpolationMethod = new FieldWithMetaInterpolationMethodEnumBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getMainPublication() {
			return mainPublication;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMainPublication() {
			if (mainPublication!=null) {
				return mainPublication;
			}
			else return mainPublication = new FieldWithMetaStringBuilder();
		}
		
	
		public InflationRateSpecificationBuilder setFallbackBondApplicable(Boolean fallbackBondApplicable) {
			this.fallbackBondApplicable = fallbackBondApplicable;
			return this;
		}
		public InflationRateSpecificationBuilder setIndexSource(FieldWithMetaString indexSource) {
			this.indexSource = indexSource.toBuilder();
			return this;
		}
		public InflationRateSpecificationBuilder setIndexSourceRef(String indexSource) {
			return setIndexSource(FieldWithMetaString.builder().setValue(indexSource).build());
		}
		public InflationRateSpecificationBuilder setInflationLag(Offset inflationLag) {
			this.inflationLag = inflationLag.toBuilder();
			return this;
		}
		public InflationRateSpecificationBuilder setInflationLagBuilder(Offset.OffsetBuilder inflationLag) {
			this.inflationLag = inflationLag;
			return this;
		}
		public InflationRateSpecificationBuilder setInitialIndexLevel(BigDecimal initialIndexLevel) {
			this.initialIndexLevel = initialIndexLevel;
			return this;
		}
		public InflationRateSpecificationBuilder setInterpolationMethod(FieldWithMetaInterpolationMethodEnum interpolationMethod) {
			this.interpolationMethod = interpolationMethod.toBuilder();
			return this;
		}
		public InflationRateSpecificationBuilder setInterpolationMethodRef(InterpolationMethodEnum interpolationMethod) {
			return setInterpolationMethod(FieldWithMetaInterpolationMethodEnum.builder().setValue(interpolationMethod).build());
		}
		public InflationRateSpecificationBuilder setMainPublication(FieldWithMetaString mainPublication) {
			this.mainPublication = mainPublication.toBuilder();
			return this;
		}
		public InflationRateSpecificationBuilder setMainPublicationRef(String mainPublication) {
			return setMainPublication(FieldWithMetaString.builder().setValue(mainPublication).build());
		}
		@Override public InflationRateSpecificationBuilder setAveragingMethod(AveragingMethodEnum averagingMethod) {
			this.averagingMethod = averagingMethod;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setFinalRateRounding(Rounding finalRateRounding) {
			this.finalRateRounding = finalRateRounding.toBuilder();
			return this;
		}
		@Override public InflationRateSpecificationBuilder setFinalRateRoundingBuilder(Rounding.RoundingBuilder finalRateRounding) {
			this.finalRateRounding = finalRateRounding;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setInitialRate(BigDecimal initialRate) {
			this.initialRate = initialRate;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setNegativeInterestRateTreatment(NegativeInterestRateTreatmentEnum negativeInterestRateTreatment) {
			this.negativeInterestRateTreatment = negativeInterestRateTreatment;
			return this;
		}
		@Override public InflationRateSpecificationBuilder addCapRateSchedule(StrikeSchedule capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addCapRateSchedule(StrikeSchedule capRateSchedule, int _idx) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			getIndex(this.capRateSchedule, _idx, () -> capRateSchedule.toBuilder());
			this.capRateSchedule.set(_idx, capRateSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addCapRateSchedule(List<StrikeSchedule> capRateSchedules) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : capRateSchedules) {
				this.capRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public InflationRateSpecificationBuilder addCapRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule);
			return this;
		}
		
		@Override public InflationRateSpecificationBuilder clearCapRateSchedule() {
			this.capRateSchedule = null;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex.toBuilder();
			return this;
		}
		@Override public InflationRateSpecificationBuilder setFloatingRateIndexRef(FloatingRateIndexEnum floatingRateIndex) {
			return setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum.builder().setValue(floatingRateIndex).build());
		}
		@Override public InflationRateSpecificationBuilder setFloatingRateMultiplierSchedule(Schedule floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule.toBuilder();
			return this;
		}
		@Override public InflationRateSpecificationBuilder setFloatingRateMultiplierScheduleBuilder(Schedule.ScheduleBuilder floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule;
			return this;
		}
		@Override public InflationRateSpecificationBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule, int _idx) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			getIndex(this.floorRateSchedule, _idx, () -> floorRateSchedule.toBuilder());
			this.floorRateSchedule.set(_idx, floorRateSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addFloorRateSchedule(List<StrikeSchedule> floorRateSchedules) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : floorRateSchedules) {
				this.floorRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public InflationRateSpecificationBuilder addFloorRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule);
			return this;
		}
		
		@Override public InflationRateSpecificationBuilder clearFloorRateSchedule() {
			this.floorRateSchedule = null;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setIndexTenor(Period indexTenor) {
			this.indexTenor = indexTenor.toBuilder();
			return this;
		}
		@Override public InflationRateSpecificationBuilder setIndexTenorBuilder(Period.PeriodBuilder indexTenor) {
			this.indexTenor = indexTenor;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public InflationRateSpecificationBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public InflationRateSpecificationBuilder setRateTreatment(RateTreatmentEnum rateTreatment) {
			this.rateTreatment = rateTreatment;
			return this;
		}
		@Override public InflationRateSpecificationBuilder addSpreadSchedule(SpreadSchedule spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addSpreadSchedule(SpreadSchedule spreadSchedule, int _idx) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			getIndex(this.spreadSchedule, _idx, () -> spreadSchedule.toBuilder());
			this.spreadSchedule.set(_idx, spreadSchedule.toBuilder());
			return this;
		}
		@Override public InflationRateSpecificationBuilder addSpreadSchedule(List<SpreadSchedule> spreadSchedules) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			for (SpreadSchedule toAdd : spreadSchedules) {
				this.spreadSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public InflationRateSpecificationBuilder addSpreadScheduleBuilder(SpreadSchedule.SpreadScheduleBuilder spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule);
			return this;
		}
		
		@Override public InflationRateSpecificationBuilder clearSpreadSchedule() {
			this.spreadSchedule = null;
			return this;
		}
		
		public InflationRateSpecification build() {
			return new InflationRateSpecification(this);
		}
	
		@Override
		public InflationRateSpecificationBuilder prune() {
			super.prune();
			if (indexSource!=null && !indexSource.prune().hasData()) indexSource = null;
			if (inflationLag!=null && !inflationLag.prune().hasData()) inflationLag = null;
			if (interpolationMethod!=null && !interpolationMethod.prune().hasData()) interpolationMethod = null;
			if (mainPublication!=null && !mainPublication.prune().hasData()) mainPublication = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getFallbackBondApplicable()!=null) return true;
			if (getIndexSource()!=null) return true;
			if (getInflationLag()!=null && getInflationLag().hasData()) return true;
			if (getInitialIndexLevel()!=null) return true;
			if (getInterpolationMethod()!=null) return true;
			if (getMainPublication()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("fallbackBondApplicable"), Boolean.class, fallbackBondApplicable, this);
			processor.processBasic(path.newSubPath("initialIndexLevel"), BigDecimal.class, initialIndexLevel, this);
			
			processRosetta(path.newSubPath("indexSource"), processor, FieldWithMetaString.class, indexSource);
			processRosetta(path.newSubPath("inflationLag"), processor, Offset.class, inflationLag);
			processRosetta(path.newSubPath("interpolationMethod"), processor, FieldWithMetaInterpolationMethodEnum.class, interpolationMethod);
			processRosetta(path.newSubPath("mainPublication"), processor, FieldWithMetaString.class, mainPublication);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			InflationRateSpecificationBuilder _that = (InflationRateSpecificationBuilder) o;
		
			if (fallbackBondApplicable != null ? !fallbackBondApplicable.equals(_that.fallbackBondApplicable) : _that.fallbackBondApplicable != null) return false;
			if (indexSource != null ? !indexSource.equals(_that.indexSource) : _that.indexSource != null) return false;
			if (inflationLag != null ? !inflationLag.equals(_that.inflationLag) : _that.inflationLag != null) return false;
			if (initialIndexLevel != null ? !initialIndexLevel.equals(_that.initialIndexLevel) : _that.initialIndexLevel != null) return false;
			if (interpolationMethod != null ? !interpolationMethod.equals(_that.interpolationMethod) : _that.interpolationMethod != null) return false;
			if (mainPublication != null ? !mainPublication.equals(_that.mainPublication) : _that.mainPublication != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (fallbackBondApplicable != null ? fallbackBondApplicable.hashCode() : 0);
			_result = 31 * _result + (indexSource != null ? indexSource.hashCode() : 0);
			_result = 31 * _result + (inflationLag != null ? inflationLag.hashCode() : 0);
			_result = 31 * _result + (initialIndexLevel != null ? initialIndexLevel.hashCode() : 0);
			_result = 31 * _result + (interpolationMethod != null ? interpolationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (mainPublication != null ? mainPublication.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InflationRateSpecificationBuilder {" +
				"fallbackBondApplicable=" + this.fallbackBondApplicable + ", " +
				"indexSource=" + this.indexSource + ", " +
				"inflationLag=" + this.inflationLag + ", " +
				"initialIndexLevel=" + this.initialIndexLevel + ", " +
				"interpolationMethod=" + this.interpolationMethod + ", " +
				"mainPublication=" + this.mainPublication +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("fallbackBondApplicable"), Boolean.class, fallbackBondApplicable, this);
		processor.processBasic(path.newSubPath("initialIndexLevel"), BigDecimal.class, initialIndexLevel, this);
		
		processRosetta(path.newSubPath("indexSource"), processor, FieldWithMetaString.class, indexSource);
		processRosetta(path.newSubPath("inflationLag"), processor, Offset.class, inflationLag);
		processRosetta(path.newSubPath("interpolationMethod"), processor, FieldWithMetaInterpolationMethodEnum.class, interpolationMethod);
		processRosetta(path.newSubPath("mainPublication"), processor, FieldWithMetaString.class, mainPublication);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		InflationRateSpecification _that = (InflationRateSpecification) o;
	
		if (fallbackBondApplicable != null ? !fallbackBondApplicable.equals(_that.fallbackBondApplicable) : _that.fallbackBondApplicable != null) return false;
		if (indexSource != null ? !indexSource.equals(_that.indexSource) : _that.indexSource != null) return false;
		if (inflationLag != null ? !inflationLag.equals(_that.inflationLag) : _that.inflationLag != null) return false;
		if (initialIndexLevel != null ? !initialIndexLevel.equals(_that.initialIndexLevel) : _that.initialIndexLevel != null) return false;
		if (interpolationMethod != null ? !interpolationMethod.equals(_that.interpolationMethod) : _that.interpolationMethod != null) return false;
		if (mainPublication != null ? !mainPublication.equals(_that.mainPublication) : _that.mainPublication != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (fallbackBondApplicable != null ? fallbackBondApplicable.hashCode() : 0);
		_result = 31 * _result + (indexSource != null ? indexSource.hashCode() : 0);
		_result = 31 * _result + (inflationLag != null ? inflationLag.hashCode() : 0);
		_result = 31 * _result + (initialIndexLevel != null ? initialIndexLevel.hashCode() : 0);
		_result = 31 * _result + (interpolationMethod != null ? interpolationMethod.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (mainPublication != null ? mainPublication.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InflationRateSpecification {" +
			"fallbackBondApplicable=" + this.fallbackBondApplicable + ", " +
			"indexSource=" + this.indexSource + ", " +
			"inflationLag=" + this.inflationLag + ", " +
			"initialIndexLevel=" + this.initialIndexLevel + ", " +
			"interpolationMethod=" + this.interpolationMethod + ", " +
			"mainPublication=" + this.mainPublication +
		'}' + " " + super.toString();
	}
}
