package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to qualify the intent associated with a transaction event.
 * 
 * @version 2.5.4
 */
public enum IntentEnum {
    

	/**
	 * The intent is to correct the event or associated execution/contract.
	 */
	@RosettaSynonym(value = "correction", source = "Rosetta_Workbench")
	CORRECTION,
	    

	/**
	 * The intent is to increase the notional or quantity associated with the contract or execution.
	 */
	@RosettaSynonym(value = "increase", source = "Rosetta_Workbench")
	INCREASE,
	    

	/**
	 * The intent is to reduce the notional or quantity associated with the contract (a.k.a. partially terminate it).
	 */
	@RosettaSynonym(value = "partialTermination", source = "Rosetta_Workbench")
	PARTIAL_TERMINATION,
	    

	/**
	 * The intent is to re-negotiate some of the terms of the contract.
	 */
	@RosettaSynonym(value = "renegotiation", source = "Rosetta_Workbench")
	RENEGOTIATION,
	    

	/**
	 * The intent is to terminate the contract.
	 */
	@RosettaSynonym(value = "termination", source = "Rosetta_Workbench")
	TERMINATION    
	;

	private final String displayName;
	
	IntentEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
