package org.isda.cdm;

import org.isda.cdm.Period;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum.FieldWithMetaFloatingRateIndexEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestRateCurveMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class InterestRateCurve extends RosettaModelObject {
	private final FieldWithMetaFloatingRateIndexEnum floatingRateIndex;
	private final Period tenor;
	private static InterestRateCurveMeta metaData = new InterestRateCurveMeta();
	
	InterestRateCurve(InterestRateCurveBuilder builder) {
		this.floatingRateIndex = ofNullable(builder.getFloatingRateIndex()).map(f->f.build()).orElse(null);
		this.tenor = ofNullable(builder.getTenor()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="floatingRateIndex", source="Rosetta_Workbench")
	public final FieldWithMetaFloatingRateIndexEnum getFloatingRateIndex() {
		return floatingRateIndex;
	}
	
	@RosettaSynonym(value="tenor", source="Rosetta_Workbench")
	public final Period getTenor() {
		return tenor;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestRateCurve> metaData() {
		return metaData;
	} 
	
	public InterestRateCurveBuilder toBuilder() {
		InterestRateCurveBuilder builder = new InterestRateCurveBuilder();
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getTenor()).ifPresent(builder::setTenor);
		return builder;
	}

	public static InterestRateCurveBuilder builder() {
		return new InterestRateCurveBuilder();
	}

	public static  class InterestRateCurveBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaFloatingRateIndexEnumBuilder floatingRateIndex;
		protected Period.PeriodBuilder tenor;
	
		public InterestRateCurveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestRateCurve> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaFloatingRateIndexEnumBuilder getFloatingRateIndex() {
			return floatingRateIndex;
		}
		
		public FieldWithMetaFloatingRateIndexEnumBuilder getOrCreateFloatingRateIndex() {
			if (floatingRateIndex!=null) {
				return floatingRateIndex;
			}
			else return floatingRateIndex = new FieldWithMetaFloatingRateIndexEnumBuilder();
		}
		
		
		public Period.PeriodBuilder getTenor() {
			return tenor;
		}
		
		public Period.PeriodBuilder getOrCreateTenor() {
			if (tenor!=null) {
				return tenor;
			}
			else return tenor = new Period.PeriodBuilder();
		}
		
	
		public InterestRateCurveBuilder setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex.toBuilder();
			return this;
		}
		public InterestRateCurveBuilder setFloatingRateIndexRef(FloatingRateIndexEnum floatingRateIndex) {
			return setFloatingRateIndex(FieldWithMetaFloatingRateIndexEnum.builder().setValue(floatingRateIndex).build());
		}
		public InterestRateCurveBuilder setTenor(Period tenor) {
			this.tenor = tenor.toBuilder();
			return this;
		}
		public InterestRateCurveBuilder setTenorBuilder(Period.PeriodBuilder tenor) {
			this.tenor = tenor;
			return this;
		}
		
		public InterestRateCurve build() {
			return new InterestRateCurve(this);
		}
	
		@Override
		public InterestRateCurveBuilder prune() {
			if (floatingRateIndex!=null && !floatingRateIndex.prune().hasData()) floatingRateIndex = null;
			if (tenor!=null && !tenor.prune().hasData()) tenor = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFloatingRateIndex()!=null) return true;
			if (getTenor()!=null && getTenor().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("floatingRateIndex"), processor, FieldWithMetaFloatingRateIndexEnum.class, floatingRateIndex);
			processRosetta(path.newSubPath("tenor"), processor, Period.class, tenor);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InterestRateCurveBuilder _that = (InterestRateCurveBuilder) o;
		
			if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
			if (tenor != null ? !tenor.equals(_that.tenor) : _that.tenor != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (tenor != null ? tenor.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestRateCurveBuilder {" +
				"floatingRateIndex=" + this.floatingRateIndex + ", " +
				"tenor=" + this.tenor +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("floatingRateIndex"), processor, FieldWithMetaFloatingRateIndexEnum.class, floatingRateIndex);
		processRosetta(path.newSubPath("tenor"), processor, Period.class, tenor);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InterestRateCurve _that = (InterestRateCurve) o;
	
		if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
		if (tenor != null ? !tenor.equals(_that.tenor) : _that.tenor != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (tenor != null ? tenor.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestRateCurve {" +
			"floatingRateIndex=" + this.floatingRateIndex + ", " +
			"tenor=" + this.tenor +
		'}';
	}
}
