package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.InterestShortfallCapEnum;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum;
import org.isda.cdm.metafields.FieldWithMetaFloatingRateIndexEnum.FieldWithMetaFloatingRateIndexEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.InterestShortFallMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the interest shortfall floating rate payment event.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="InterestShortFall", source="FpML_5_10")
public class InterestShortFall extends RosettaModelObject {
	private final Boolean compounding;
	private final InterestShortfallCapEnum interestShortfallCap;
	private final FieldWithMetaFloatingRateIndexEnum rateSource;
	private static InterestShortFallMeta metaData = new InterestShortFallMeta();
	
	InterestShortFall(InterestShortFallBuilder builder) {
		this.compounding = builder.getCompounding();
		this.interestShortfallCap = builder.getInterestShortfallCap();
		this.rateSource = ofNullable(builder.getRateSource()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="compounding", source="FpML_5_10")
	@RosettaSynonym(value="compounding", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="compounding", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="compounding", source="DTCC_11_0")
	@RosettaSynonym(value="compounding", source="DTCC_9_0")
	public final Boolean getCompounding() {
		return compounding;
	}
	
	/**
	 * Specifies the nature of the interest Shortfall cap (i.e. Fixed Cap or Variable Cap) in the case where it is applicable. ISDA 2003 Term: Interest Shortfall Cap.
	 */
	@RosettaSynonym(value="interestShortfallCap", source="FpML_5_10")
	@RosettaSynonym(value="interestShortfallCap", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="interestShortfallCap", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="interestShortfallCap", source="DTCC_11_0")
	@RosettaSynonym(value="interestShortfallCap", source="DTCC_9_0")
	public final InterestShortfallCapEnum getInterestShortfallCap() {
		return interestShortfallCap;
	}
	
	/**
	 * The rate source in the case of a variable cap.
	 */
	@RosettaSynonym(value="rateSource", source="FpML_5_10")
	@RosettaSynonym(value="rateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateSource", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="rateSource", source="DTCC_11_0")
	@RosettaSynonym(value="rateSource", source="DTCC_9_0")
	@RosettaSynonym(value="rateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateSource", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaFloatingRateIndexEnum getRateSource() {
		return rateSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends InterestShortFall> metaData() {
		return metaData;
	} 
	
	public InterestShortFallBuilder toBuilder() {
		InterestShortFallBuilder builder = new InterestShortFallBuilder();
		ofNullable(getCompounding()).ifPresent(builder::setCompounding);
		ofNullable(getInterestShortfallCap()).ifPresent(builder::setInterestShortfallCap);
		ofNullable(getRateSource()).ifPresent(builder::setRateSource);
		return builder;
	}

	public static InterestShortFallBuilder builder() {
		return new InterestShortFallBuilder();
	}

	public static  class InterestShortFallBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean compounding;
		protected InterestShortfallCapEnum interestShortfallCap;
		protected FieldWithMetaFloatingRateIndexEnumBuilder rateSource;
	
		public InterestShortFallBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends InterestShortFall> metaData() {
			return metaData;
		} 
	
		
		public Boolean getCompounding() {
			return compounding;
		}
		
		
		public InterestShortfallCapEnum getInterestShortfallCap() {
			return interestShortfallCap;
		}
		
		
		public FieldWithMetaFloatingRateIndexEnumBuilder getRateSource() {
			return rateSource;
		}
		
		public FieldWithMetaFloatingRateIndexEnumBuilder getOrCreateRateSource() {
			if (rateSource!=null) {
				return rateSource;
			}
			else return rateSource = new FieldWithMetaFloatingRateIndexEnumBuilder();
		}
		
	
		public InterestShortFallBuilder setCompounding(Boolean compounding) {
			this.compounding = compounding;
			return this;
		}
		public InterestShortFallBuilder setInterestShortfallCap(InterestShortfallCapEnum interestShortfallCap) {
			this.interestShortfallCap = interestShortfallCap;
			return this;
		}
		public InterestShortFallBuilder setRateSource(FieldWithMetaFloatingRateIndexEnum rateSource) {
			this.rateSource = rateSource.toBuilder();
			return this;
		}
		public InterestShortFallBuilder setRateSourceRef(FloatingRateIndexEnum rateSource) {
			return setRateSource(FieldWithMetaFloatingRateIndexEnum.builder().setValue(rateSource).build());
		}
		
		public InterestShortFall build() {
			return new InterestShortFall(this);
		}
	
		@Override
		public InterestShortFallBuilder prune() {
			if (rateSource!=null && !rateSource.prune().hasData()) rateSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCompounding()!=null) return true;
			if (getInterestShortfallCap()!=null) return true;
			if (getRateSource()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("compounding"), Boolean.class, compounding, this);
			processor.processBasic(path.newSubPath("interestShortfallCap"), InterestShortfallCapEnum.class, interestShortfallCap, this);
			
			processRosetta(path.newSubPath("rateSource"), processor, FieldWithMetaFloatingRateIndexEnum.class, rateSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			InterestShortFallBuilder _that = (InterestShortFallBuilder) o;
		
			if (compounding != null ? !compounding.equals(_that.compounding) : _that.compounding != null) return false;
			if (interestShortfallCap != null ? !interestShortfallCap.equals(_that.interestShortfallCap) : _that.interestShortfallCap != null) return false;
			if (rateSource != null ? !rateSource.equals(_that.rateSource) : _that.rateSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (compounding != null ? compounding.hashCode() : 0);
			_result = 31 * _result + (interestShortfallCap != null ? interestShortfallCap.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (rateSource != null ? rateSource.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "InterestShortFallBuilder {" +
				"compounding=" + this.compounding + ", " +
				"interestShortfallCap=" + this.interestShortfallCap + ", " +
				"rateSource=" + this.rateSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("compounding"), Boolean.class, compounding, this);
		processor.processBasic(path.newSubPath("interestShortfallCap"), InterestShortfallCapEnum.class, interestShortfallCap, this);
		
		processRosetta(path.newSubPath("rateSource"), processor, FieldWithMetaFloatingRateIndexEnum.class, rateSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		InterestShortFall _that = (InterestShortFall) o;
	
		if (compounding != null ? !compounding.equals(_that.compounding) : _that.compounding != null) return false;
		if (interestShortfallCap != null ? !interestShortfallCap.equals(_that.interestShortfallCap) : _that.interestShortfallCap != null) return false;
		if (rateSource != null ? !rateSource.equals(_that.rateSource) : _that.rateSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (compounding != null ? compounding.hashCode() : 0);
		_result = 31 * _result + (interestShortfallCap != null ? interestShortfallCap.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (rateSource != null ? rateSource.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "InterestShortFall {" +
			"compounding=" + this.compounding + ", " +
			"interestShortfallCap=" + this.interestShortfallCap + ", " +
			"rateSource=" + this.rateSource +
		'}';
	}
}
