package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.IssuerTradeIdMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class for a two-parts identifier, such as a USI.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="IssuerTradeId", source="FpML_5_10")
public class IssuerTradeId extends RosettaModelObject {
	private final FieldWithMetaString identifier;
	private final FieldWithMetaString issuer;
	private static IssuerTradeIdMeta metaData = new IssuerTradeIdMeta();
	
	IssuerTradeId(IssuerTradeIdBuilder builder) {
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.issuer = ofNullable(builder.getIssuer()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The identifier value. The CDM uses a neutral identifier attribute name rather than the FpML trade qualifier because of the focus that includes the pre-execution lifecycle, at which point a trade doesn&#39;t exist yet.
	 */
	@RosettaSynonym(value="tradeId", source="FpML_5_10")
	@RosettaSynonym(value="tradeId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tradeId", source="DTCC_11_0")
	@RosettaSynonym(value="tradeId", source="DTCC_9_0")
	@RosettaSynonym(value="tradeId", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIdentifier() {
		return identifier;
	}
	
	/**
	 * The party that assigns the trade identifier.
	 */
	@RosettaSynonym(value="issuer", source="FpML_5_10")
	@RosettaSynonym(value="issuer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="issuer", source="DTCC_11_0")
	@RosettaSynonym(value="issuer", source="DTCC_9_0")
	@RosettaSynonym(value="issuer", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIssuer() {
		return issuer;
	}
	
	
	@Override
	public RosettaMetaData<? extends IssuerTradeId> metaData() {
		return metaData;
	} 
	
	public IssuerTradeIdBuilder toBuilder() {
		IssuerTradeIdBuilder builder = new IssuerTradeIdBuilder();
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getIssuer()).ifPresent(builder::setIssuer);
		return builder;
	}

	public static IssuerTradeIdBuilder builder() {
		return new IssuerTradeIdBuilder();
	}

	public static  class IssuerTradeIdBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder identifier;
		protected FieldWithMetaStringBuilder issuer;
	
		public IssuerTradeIdBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends IssuerTradeId> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getIssuer() {
			return issuer;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIssuer() {
			if (issuer!=null) {
				return issuer;
			}
			else return issuer = new FieldWithMetaStringBuilder();
		}
		
	
		public IssuerTradeIdBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public IssuerTradeIdBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		public IssuerTradeIdBuilder setIssuer(FieldWithMetaString issuer) {
			this.issuer = issuer.toBuilder();
			return this;
		}
		public IssuerTradeIdBuilder setIssuerRef(String issuer) {
			return setIssuer(FieldWithMetaString.builder().setValue(issuer).build());
		}
		
		public IssuerTradeId build() {
			return new IssuerTradeId(this);
		}
	
		@Override
		public IssuerTradeIdBuilder prune() {
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			if (issuer!=null && !issuer.prune().hasData()) issuer = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null) return true;
			if (getIssuer()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
			processRosetta(path.newSubPath("issuer"), processor, FieldWithMetaString.class, issuer);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			IssuerTradeIdBuilder _that = (IssuerTradeIdBuilder) o;
		
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (issuer != null ? !issuer.equals(_that.issuer) : _that.issuer != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (issuer != null ? issuer.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "IssuerTradeIdBuilder {" +
				"identifier=" + this.identifier + ", " +
				"issuer=" + this.issuer +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		processRosetta(path.newSubPath("issuer"), processor, FieldWithMetaString.class, issuer);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		IssuerTradeId _that = (IssuerTradeId) o;
	
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (issuer != null ? !issuer.equals(_that.issuer) : _that.issuer != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (issuer != null ? issuer.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "IssuerTradeId {" +
			"identifier=" + this.identifier + ", " +
			"issuer=" + this.issuer +
		'}';
	}
}
