package org.isda.cdm;

import org.isda.cdm.TriggerEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.KnockMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Knock In means option to exercise comes into existence. Knock Out means option to exercise goes out of existence.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Knock", source="FpML_5_10")
public class Knock extends RosettaModelObject {
	private final TriggerEvent knockIn;
	private final TriggerEvent knockOut;
	private static KnockMeta metaData = new KnockMeta();
	
	Knock(KnockBuilder builder) {
		this.knockIn = ofNullable(builder.getKnockIn()).map(f->f.build()).orElse(null);
		this.knockOut = ofNullable(builder.getKnockOut()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The knock in.
	 */
	@RosettaSynonym(value="knockIn", source="FpML_5_10")
	@RosettaSynonym(value="knockIn", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="knockIn", source="DTCC_11_0")
	@RosettaSynonym(value="knockIn", source="DTCC_9_0")
	@RosettaSynonym(value="knockIn", source="CME_ClearedConfirm_1_17")
	public final TriggerEvent getKnockIn() {
		return knockIn;
	}
	
	/**
	 * The knock out.
	 */
	@RosettaSynonym(value="knockOut", source="FpML_5_10")
	@RosettaSynonym(value="knockOut", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="knockOut", source="DTCC_11_0")
	@RosettaSynonym(value="knockOut", source="DTCC_9_0")
	@RosettaSynonym(value="knockOut", source="CME_ClearedConfirm_1_17")
	public final TriggerEvent getKnockOut() {
		return knockOut;
	}
	
	
	@Override
	public RosettaMetaData<? extends Knock> metaData() {
		return metaData;
	} 
	
	public KnockBuilder toBuilder() {
		KnockBuilder builder = new KnockBuilder();
		ofNullable(getKnockIn()).ifPresent(builder::setKnockIn);
		ofNullable(getKnockOut()).ifPresent(builder::setKnockOut);
		return builder;
	}

	public static KnockBuilder builder() {
		return new KnockBuilder();
	}

	public static  class KnockBuilder extends RosettaModelObjectBuilder {
	
		protected TriggerEvent.TriggerEventBuilder knockIn;
		protected TriggerEvent.TriggerEventBuilder knockOut;
	
		public KnockBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Knock> metaData() {
			return metaData;
		} 
	
		
		public TriggerEvent.TriggerEventBuilder getKnockIn() {
			return knockIn;
		}
		
		public TriggerEvent.TriggerEventBuilder getOrCreateKnockIn() {
			if (knockIn!=null) {
				return knockIn;
			}
			else return knockIn = new TriggerEvent.TriggerEventBuilder();
		}
		
		
		public TriggerEvent.TriggerEventBuilder getKnockOut() {
			return knockOut;
		}
		
		public TriggerEvent.TriggerEventBuilder getOrCreateKnockOut() {
			if (knockOut!=null) {
				return knockOut;
			}
			else return knockOut = new TriggerEvent.TriggerEventBuilder();
		}
		
	
		public KnockBuilder setKnockIn(TriggerEvent knockIn) {
			this.knockIn = knockIn.toBuilder();
			return this;
		}
		public KnockBuilder setKnockInBuilder(TriggerEvent.TriggerEventBuilder knockIn) {
			this.knockIn = knockIn;
			return this;
		}
		public KnockBuilder setKnockOut(TriggerEvent knockOut) {
			this.knockOut = knockOut.toBuilder();
			return this;
		}
		public KnockBuilder setKnockOutBuilder(TriggerEvent.TriggerEventBuilder knockOut) {
			this.knockOut = knockOut;
			return this;
		}
		
		public Knock build() {
			return new Knock(this);
		}
	
		@Override
		public KnockBuilder prune() {
			if (knockIn!=null && !knockIn.prune().hasData()) knockIn = null;
			if (knockOut!=null && !knockOut.prune().hasData()) knockOut = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getKnockIn()!=null && getKnockIn().hasData()) return true;
			if (getKnockOut()!=null && getKnockOut().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("knockIn"), processor, TriggerEvent.class, knockIn);
			processRosetta(path.newSubPath("knockOut"), processor, TriggerEvent.class, knockOut);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			KnockBuilder _that = (KnockBuilder) o;
		
			if (knockIn != null ? !knockIn.equals(_that.knockIn) : _that.knockIn != null) return false;
			if (knockOut != null ? !knockOut.equals(_that.knockOut) : _that.knockOut != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (knockIn != null ? knockIn.hashCode() : 0);
			_result = 31 * _result + (knockOut != null ? knockOut.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "KnockBuilder {" +
				"knockIn=" + this.knockIn + ", " +
				"knockOut=" + this.knockOut +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("knockIn"), processor, TriggerEvent.class, knockIn);
		processRosetta(path.newSubPath("knockOut"), processor, TriggerEvent.class, knockOut);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Knock _that = (Knock) o;
	
		if (knockIn != null ? !knockIn.equals(_that.knockIn) : _that.knockIn != null) return false;
		if (knockOut != null ? !knockOut.equals(_that.knockOut) : _that.knockOut != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (knockIn != null ? knockIn.hashCode() : 0);
		_result = 31 * _result + (knockOut != null ? knockOut.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Knock {" +
			"knockIn=" + this.knockIn + ", " +
			"knockOut=" + this.knockOut +
		'}';
	}
}
