package org.isda.cdm;

import org.isda.cdm.CsaInitialMargin2016JapaneseLaw;
import org.isda.cdm.CsaInitialMargin2016NewYorkLaw;
import org.isda.cdm.CsaVariationMargin2016NewYorkLaw;
import org.isda.cdm.CsdInitialMargin2016EnglishLaw;
import org.isda.cdm.EquitySwapMasterConfirmation2018;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LegalAgreementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class to specify the elections and variables that characterize a legal agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class LegalAgreement extends LegalAgreementBase implements GlobalKey {
	private final CsaInitialMargin2016JapaneseLaw csaInitialMargin2016JapaneseLaw;
	private final CsaInitialMargin2016NewYorkLaw csaInitialMargin2016NewYorkLaw;
	private final CsaVariationMargin2016NewYorkLaw csaVariationMargin2016NewYorkLaw;
	private final CsdInitialMargin2016EnglishLaw csdInitialMargin2016EnglishLaw;
	private final EquitySwapMasterConfirmation2018 equitySwapMasterConfirmation2018;
	private final MetaFields meta;
	private static LegalAgreementMeta metaData = new LegalAgreementMeta();
	
	LegalAgreement(LegalAgreementBuilder builder) {
		super(builder);
		this.csaInitialMargin2016JapaneseLaw = ofNullable(builder.getCsaInitialMargin2016JapaneseLaw()).map(f->f.build()).orElse(null);
		this.csaInitialMargin2016NewYorkLaw = ofNullable(builder.getCsaInitialMargin2016NewYorkLaw()).map(f->f.build()).orElse(null);
		this.csaVariationMargin2016NewYorkLaw = ofNullable(builder.getCsaVariationMargin2016NewYorkLaw()).map(f->f.build()).orElse(null);
		this.csdInitialMargin2016EnglishLaw = ofNullable(builder.getCsdInitialMargin2016EnglishLaw()).map(f->f.build()).orElse(null);
		this.equitySwapMasterConfirmation2018 = ofNullable(builder.getEquitySwapMasterConfirmation2018()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The elections and variables specified in Paragraph 13 of the ISDA 2016 Credit Support Annex for Initial Margin, Japanese Law.
	 */
	@RosettaSynonym(value="answers", source="ISDA_Create_1_0")
	public final CsaInitialMargin2016JapaneseLaw getCsaInitialMargin2016JapaneseLaw() {
		return csaInitialMargin2016JapaneseLaw;
	}
	
	/**
	 * The elections and variables specified in Paragraph 13 of the ISDA 2016 Credit Support Annex for Initial Margin, New York Law.
	 */
	@RosettaSynonym(value="answers", source="ISDA_Create_1_0")
	public final CsaInitialMargin2016NewYorkLaw getCsaInitialMargin2016NewYorkLaw() {
		return csaInitialMargin2016NewYorkLaw;
	}
	
	/**
	 * The elections and variables specified in Paragraph 13 of the ISDA 2016 Credit Support Annex for Variation Margin, New York Law.
	 */
	@RosettaSynonym(value="ignore", source="ISDA_Create_1_0")
	public final CsaVariationMargin2016NewYorkLaw getCsaVariationMargin2016NewYorkLaw() {
		return csaVariationMargin2016NewYorkLaw;
	}
	
	/**
	 * The elections and variables specified in Paragraph 13 of the ISDA 2016 Credit Support Deed for Initial Margin, English Law.
	 */
	@RosettaSynonym(value="answers", source="ISDA_Create_1_0")
	public final CsdInitialMargin2016EnglishLaw getCsdInitialMargin2016EnglishLaw() {
		return csdInitialMargin2016EnglishLaw;
	}
	
	/**
	 * The General Terms and Relationship Supplement Elections specified in the 2018 ISDA CDM Equity Confirmation for Security Equity Swap.
	 */
	@RosettaSynonym(value="ignore", source="ISDA_Create_1_0")
	public final EquitySwapMasterConfirmation2018 getEquitySwapMasterConfirmation2018() {
		return equitySwapMasterConfirmation2018;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends LegalAgreement> metaData() {
		return metaData;
	} 
	
	public LegalAgreementBuilder toBuilder() {
		LegalAgreementBuilder builder = new LegalAgreementBuilder();
		ofNullable(getCsaInitialMargin2016JapaneseLaw()).ifPresent(builder::setCsaInitialMargin2016JapaneseLaw);
		ofNullable(getCsaInitialMargin2016NewYorkLaw()).ifPresent(builder::setCsaInitialMargin2016NewYorkLaw);
		ofNullable(getCsaVariationMargin2016NewYorkLaw()).ifPresent(builder::setCsaVariationMargin2016NewYorkLaw);
		ofNullable(getCsdInitialMargin2016EnglishLaw()).ifPresent(builder::setCsdInitialMargin2016EnglishLaw);
		ofNullable(getEquitySwapMasterConfirmation2018()).ifPresent(builder::setEquitySwapMasterConfirmation2018);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getAgreementDate()).ifPresent(builder::setAgreementDate);
		ofNullable(getAgreementType()).ifPresent(builder::setAgreementType);
		ofNullable(getContractualParty()).ifPresent(contractualParty -> contractualParty.forEach(builder::addContractualParty));
		ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
		ofNullable(getIdentifier()).ifPresent(identifier -> identifier.forEach(builder::addIdentifier));
		ofNullable(getLineage()).ifPresent(builder::setLineage);
		ofNullable(getOtherParty()).ifPresent(otherParty -> otherParty.forEach(builder::addOtherParty));
		ofNullable(getPartyInformation()).ifPresent(partyInformation -> partyInformation.forEach(builder::addPartyInformation));
		ofNullable(getUmbrellaAgreement()).ifPresent(builder::setUmbrellaAgreement);
		return builder;
	}

	public static LegalAgreementBuilder builder() {
		return new LegalAgreementBuilder();
	}

	public static  class LegalAgreementBuilder extends LegalAgreementBaseBuilder implements GlobalKeyBuilder<LegalAgreementBuilder> {
	
		protected CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder csaInitialMargin2016JapaneseLaw;
		protected CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder csaInitialMargin2016NewYorkLaw;
		protected CsaVariationMargin2016NewYorkLaw.CsaVariationMargin2016NewYorkLawBuilder csaVariationMargin2016NewYorkLaw;
		protected CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder csdInitialMargin2016EnglishLaw;
		protected EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder equitySwapMasterConfirmation2018;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public LegalAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LegalAgreement> metaData() {
			return metaData;
		} 
	
		
		public CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder getCsaInitialMargin2016JapaneseLaw() {
			return csaInitialMargin2016JapaneseLaw;
		}
		
		public CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder getOrCreateCsaInitialMargin2016JapaneseLaw() {
			if (csaInitialMargin2016JapaneseLaw!=null) {
				return csaInitialMargin2016JapaneseLaw;
			}
			else return csaInitialMargin2016JapaneseLaw = new CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder();
		}
		
		
		public CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder getCsaInitialMargin2016NewYorkLaw() {
			return csaInitialMargin2016NewYorkLaw;
		}
		
		public CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder getOrCreateCsaInitialMargin2016NewYorkLaw() {
			if (csaInitialMargin2016NewYorkLaw!=null) {
				return csaInitialMargin2016NewYorkLaw;
			}
			else return csaInitialMargin2016NewYorkLaw = new CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder();
		}
		
		
		public CsaVariationMargin2016NewYorkLaw.CsaVariationMargin2016NewYorkLawBuilder getCsaVariationMargin2016NewYorkLaw() {
			return csaVariationMargin2016NewYorkLaw;
		}
		
		public CsaVariationMargin2016NewYorkLaw.CsaVariationMargin2016NewYorkLawBuilder getOrCreateCsaVariationMargin2016NewYorkLaw() {
			if (csaVariationMargin2016NewYorkLaw!=null) {
				return csaVariationMargin2016NewYorkLaw;
			}
			else return csaVariationMargin2016NewYorkLaw = new CsaVariationMargin2016NewYorkLaw.CsaVariationMargin2016NewYorkLawBuilder();
		}
		
		
		public CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder getCsdInitialMargin2016EnglishLaw() {
			return csdInitialMargin2016EnglishLaw;
		}
		
		public CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder getOrCreateCsdInitialMargin2016EnglishLaw() {
			if (csdInitialMargin2016EnglishLaw!=null) {
				return csdInitialMargin2016EnglishLaw;
			}
			else return csdInitialMargin2016EnglishLaw = new CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder();
		}
		
		
		public EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder getEquitySwapMasterConfirmation2018() {
			return equitySwapMasterConfirmation2018;
		}
		
		public EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder getOrCreateEquitySwapMasterConfirmation2018() {
			if (equitySwapMasterConfirmation2018!=null) {
				return equitySwapMasterConfirmation2018;
			}
			else return equitySwapMasterConfirmation2018 = new EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public LegalAgreementBuilder setCsaInitialMargin2016JapaneseLaw(CsaInitialMargin2016JapaneseLaw csaInitialMargin2016JapaneseLaw) {
			this.csaInitialMargin2016JapaneseLaw = csaInitialMargin2016JapaneseLaw.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setCsaInitialMargin2016JapaneseLawBuilder(CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder csaInitialMargin2016JapaneseLaw) {
			this.csaInitialMargin2016JapaneseLaw = csaInitialMargin2016JapaneseLaw;
			return this;
		}
		public LegalAgreementBuilder setCsaInitialMargin2016NewYorkLaw(CsaInitialMargin2016NewYorkLaw csaInitialMargin2016NewYorkLaw) {
			this.csaInitialMargin2016NewYorkLaw = csaInitialMargin2016NewYorkLaw.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setCsaInitialMargin2016NewYorkLawBuilder(CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder csaInitialMargin2016NewYorkLaw) {
			this.csaInitialMargin2016NewYorkLaw = csaInitialMargin2016NewYorkLaw;
			return this;
		}
		public LegalAgreementBuilder setCsaVariationMargin2016NewYorkLaw(CsaVariationMargin2016NewYorkLaw csaVariationMargin2016NewYorkLaw) {
			this.csaVariationMargin2016NewYorkLaw = csaVariationMargin2016NewYorkLaw.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setCsaVariationMargin2016NewYorkLawBuilder(CsaVariationMargin2016NewYorkLaw.CsaVariationMargin2016NewYorkLawBuilder csaVariationMargin2016NewYorkLaw) {
			this.csaVariationMargin2016NewYorkLaw = csaVariationMargin2016NewYorkLaw;
			return this;
		}
		public LegalAgreementBuilder setCsdInitialMargin2016EnglishLaw(CsdInitialMargin2016EnglishLaw csdInitialMargin2016EnglishLaw) {
			this.csdInitialMargin2016EnglishLaw = csdInitialMargin2016EnglishLaw.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setCsdInitialMargin2016EnglishLawBuilder(CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder csdInitialMargin2016EnglishLaw) {
			this.csdInitialMargin2016EnglishLaw = csdInitialMargin2016EnglishLaw;
			return this;
		}
		public LegalAgreementBuilder setEquitySwapMasterConfirmation2018(EquitySwapMasterConfirmation2018 equitySwapMasterConfirmation2018) {
			this.equitySwapMasterConfirmation2018 = equitySwapMasterConfirmation2018.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setEquitySwapMasterConfirmation2018Builder(EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder equitySwapMasterConfirmation2018) {
			this.equitySwapMasterConfirmation2018 = equitySwapMasterConfirmation2018;
			return this;
		}
		public LegalAgreementBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public LegalAgreementBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public LegalAgreementBuilder setAgreementDate(Date agreementDate) {
			this.agreementDate = agreementDate;
			return this;
		}
		@Override public LegalAgreementBuilder setAgreementType(LegalAgreementType agreementType) {
			this.agreementType = agreementType.toBuilder();
			return this;
		}
		@Override public LegalAgreementBuilder setAgreementTypeBuilder(LegalAgreementType.LegalAgreementTypeBuilder agreementType) {
			this.agreementType = agreementType;
			return this;
		}
		@Override public LegalAgreementBuilder addContractualParty(ReferenceWithMetaParty contractualParty) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			this.contractualParty.add(contractualParty.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addContractualParty(ReferenceWithMetaParty contractualParty, int _idx) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			getIndex(this.contractualParty, _idx, () -> contractualParty.toBuilder());
			this.contractualParty.set(_idx, contractualParty.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addContractualParty(List<ReferenceWithMetaParty> contractualPartys) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			for (ReferenceWithMetaParty toAdd : contractualPartys) {
				this.contractualParty.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public LegalAgreementBuilder addContractualPartyBuilder(ReferenceWithMetaPartyBuilder contractualParty) {
			if(this.contractualParty == null){
				this.contractualParty = new ArrayList<>();
			}
			this.contractualParty.add(contractualParty);
			return this;
		}
		
		@Override public LegalAgreementBuilder addContractualPartyRef(Party.PartyBuilder contractualParty) {
			return addContractualParty(ReferenceWithMetaParty.builder().setValueBuilder(contractualParty).build());
		}
		
		@Override public LegalAgreementBuilder addContractualPartyRef(Party.PartyBuilder contractualParty, int _idx) {
			return addContractualParty(ReferenceWithMetaParty.builder().setValueBuilder(contractualParty).build(), _idx);
		}
		
		@Override public LegalAgreementBuilder addContractualPartyRef(List<Party> contractualPartys) {
			for (Party toAdd : contractualPartys) {
				addContractualPartyRef(toAdd);
			}
			return this;
		}
		
		@Override public LegalAgreementBuilder addContractualPartyRef(Party contractualParty) {
			return addContractualPartyRef(contractualParty.toBuilder());
		}
		
		@Override public LegalAgreementBuilder clearContractualParty() {
			this.contractualParty = null;
			return this;
		}
		@Override public LegalAgreementBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate;
			return this;
		}
		@Override public LegalAgreementBuilder addIdentifier(Identifier identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addIdentifier(Identifier identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addIdentifier(List<Identifier> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (Identifier toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public LegalAgreementBuilder addIdentifierBuilder(Identifier.IdentifierBuilder identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier);
			return this;
		}
		
		@Override public LegalAgreementBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		@Override public LegalAgreementBuilder setLineage(Lineage lineage) {
			this.lineage = lineage.toBuilder();
			return this;
		}
		@Override public LegalAgreementBuilder setLineageBuilder(Lineage.LineageBuilder lineage) {
			this.lineage = lineage;
			return this;
		}
		@Override public LegalAgreementBuilder addOtherParty(PartyRole otherParty) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			this.otherParty.add(otherParty.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addOtherParty(PartyRole otherParty, int _idx) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			getIndex(this.otherParty, _idx, () -> otherParty.toBuilder());
			this.otherParty.set(_idx, otherParty.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addOtherParty(List<PartyRole> otherPartys) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			for (PartyRole toAdd : otherPartys) {
				this.otherParty.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public LegalAgreementBuilder addOtherPartyBuilder(PartyRole.PartyRoleBuilder otherParty) {
			if(this.otherParty == null){
				this.otherParty = new ArrayList<>();
			}
			this.otherParty.add(otherParty);
			return this;
		}
		
		@Override public LegalAgreementBuilder clearOtherParty() {
			this.otherParty = null;
			return this;
		}
		@Override public LegalAgreementBuilder addPartyInformation(Party partyInformation) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			this.partyInformation.add(partyInformation.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addPartyInformation(Party partyInformation, int _idx) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			getIndex(this.partyInformation, _idx, () -> partyInformation.toBuilder());
			this.partyInformation.set(_idx, partyInformation.toBuilder());
			return this;
		}
		@Override public LegalAgreementBuilder addPartyInformation(List<Party> partyInformations) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			for (Party toAdd : partyInformations) {
				this.partyInformation.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public LegalAgreementBuilder addPartyInformationBuilder(Party.PartyBuilder partyInformation) {
			if(this.partyInformation == null){
				this.partyInformation = new ArrayList<>();
			}
			this.partyInformation.add(partyInformation);
			return this;
		}
		
		@Override public LegalAgreementBuilder clearPartyInformation() {
			this.partyInformation = null;
			return this;
		}
		@Override public LegalAgreementBuilder setUmbrellaAgreement(UmbrellaAgreement umbrellaAgreement) {
			this.umbrellaAgreement = umbrellaAgreement.toBuilder();
			return this;
		}
		@Override public LegalAgreementBuilder setUmbrellaAgreementBuilder(UmbrellaAgreement.UmbrellaAgreementBuilder umbrellaAgreement) {
			this.umbrellaAgreement = umbrellaAgreement;
			return this;
		}
		
		public LegalAgreement build() {
			return new LegalAgreement(this);
		}
	
		@Override
		public LegalAgreementBuilder prune() {
			super.prune();
			if (csaInitialMargin2016JapaneseLaw!=null && !csaInitialMargin2016JapaneseLaw.prune().hasData()) csaInitialMargin2016JapaneseLaw = null;
			if (csaInitialMargin2016NewYorkLaw!=null && !csaInitialMargin2016NewYorkLaw.prune().hasData()) csaInitialMargin2016NewYorkLaw = null;
			if (csaVariationMargin2016NewYorkLaw!=null && !csaVariationMargin2016NewYorkLaw.prune().hasData()) csaVariationMargin2016NewYorkLaw = null;
			if (csdInitialMargin2016EnglishLaw!=null && !csdInitialMargin2016EnglishLaw.prune().hasData()) csdInitialMargin2016EnglishLaw = null;
			if (equitySwapMasterConfirmation2018!=null && !equitySwapMasterConfirmation2018.prune().hasData()) equitySwapMasterConfirmation2018 = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getCsaInitialMargin2016JapaneseLaw()!=null && getCsaInitialMargin2016JapaneseLaw().hasData()) return true;
			if (getCsaInitialMargin2016NewYorkLaw()!=null && getCsaInitialMargin2016NewYorkLaw().hasData()) return true;
			if (getCsaVariationMargin2016NewYorkLaw()!=null && getCsaVariationMargin2016NewYorkLaw().hasData()) return true;
			if (getCsdInitialMargin2016EnglishLaw()!=null && getCsdInitialMargin2016EnglishLaw().hasData()) return true;
			if (getEquitySwapMasterConfirmation2018()!=null && getEquitySwapMasterConfirmation2018().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("csaInitialMargin2016JapaneseLaw"), processor, CsaInitialMargin2016JapaneseLaw.class, csaInitialMargin2016JapaneseLaw);
			processRosetta(path.newSubPath("csaInitialMargin2016NewYorkLaw"), processor, CsaInitialMargin2016NewYorkLaw.class, csaInitialMargin2016NewYorkLaw);
			processRosetta(path.newSubPath("csaVariationMargin2016NewYorkLaw"), processor, CsaVariationMargin2016NewYorkLaw.class, csaVariationMargin2016NewYorkLaw);
			processRosetta(path.newSubPath("csdInitialMargin2016EnglishLaw"), processor, CsdInitialMargin2016EnglishLaw.class, csdInitialMargin2016EnglishLaw);
			processRosetta(path.newSubPath("equitySwapMasterConfirmation2018"), processor, EquitySwapMasterConfirmation2018.class, equitySwapMasterConfirmation2018);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			LegalAgreementBuilder _that = (LegalAgreementBuilder) o;
		
			if (csaInitialMargin2016JapaneseLaw != null ? !csaInitialMargin2016JapaneseLaw.equals(_that.csaInitialMargin2016JapaneseLaw) : _that.csaInitialMargin2016JapaneseLaw != null) return false;
			if (csaInitialMargin2016NewYorkLaw != null ? !csaInitialMargin2016NewYorkLaw.equals(_that.csaInitialMargin2016NewYorkLaw) : _that.csaInitialMargin2016NewYorkLaw != null) return false;
			if (csaVariationMargin2016NewYorkLaw != null ? !csaVariationMargin2016NewYorkLaw.equals(_that.csaVariationMargin2016NewYorkLaw) : _that.csaVariationMargin2016NewYorkLaw != null) return false;
			if (csdInitialMargin2016EnglishLaw != null ? !csdInitialMargin2016EnglishLaw.equals(_that.csdInitialMargin2016EnglishLaw) : _that.csdInitialMargin2016EnglishLaw != null) return false;
			if (equitySwapMasterConfirmation2018 != null ? !equitySwapMasterConfirmation2018.equals(_that.equitySwapMasterConfirmation2018) : _that.equitySwapMasterConfirmation2018 != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (csaInitialMargin2016JapaneseLaw != null ? csaInitialMargin2016JapaneseLaw.hashCode() : 0);
			_result = 31 * _result + (csaInitialMargin2016NewYorkLaw != null ? csaInitialMargin2016NewYorkLaw.hashCode() : 0);
			_result = 31 * _result + (csaVariationMargin2016NewYorkLaw != null ? csaVariationMargin2016NewYorkLaw.hashCode() : 0);
			_result = 31 * _result + (csdInitialMargin2016EnglishLaw != null ? csdInitialMargin2016EnglishLaw.hashCode() : 0);
			_result = 31 * _result + (equitySwapMasterConfirmation2018 != null ? equitySwapMasterConfirmation2018.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LegalAgreementBuilder {" +
				"csaInitialMargin2016JapaneseLaw=" + this.csaInitialMargin2016JapaneseLaw + ", " +
				"csaInitialMargin2016NewYorkLaw=" + this.csaInitialMargin2016NewYorkLaw + ", " +
				"csaVariationMargin2016NewYorkLaw=" + this.csaVariationMargin2016NewYorkLaw + ", " +
				"csdInitialMargin2016EnglishLaw=" + this.csdInitialMargin2016EnglishLaw + ", " +
				"equitySwapMasterConfirmation2018=" + this.equitySwapMasterConfirmation2018 + ", " +
				"meta=" + this.meta +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("csaInitialMargin2016JapaneseLaw"), processor, CsaInitialMargin2016JapaneseLaw.class, csaInitialMargin2016JapaneseLaw);
		processRosetta(path.newSubPath("csaInitialMargin2016NewYorkLaw"), processor, CsaInitialMargin2016NewYorkLaw.class, csaInitialMargin2016NewYorkLaw);
		processRosetta(path.newSubPath("csaVariationMargin2016NewYorkLaw"), processor, CsaVariationMargin2016NewYorkLaw.class, csaVariationMargin2016NewYorkLaw);
		processRosetta(path.newSubPath("csdInitialMargin2016EnglishLaw"), processor, CsdInitialMargin2016EnglishLaw.class, csdInitialMargin2016EnglishLaw);
		processRosetta(path.newSubPath("equitySwapMasterConfirmation2018"), processor, EquitySwapMasterConfirmation2018.class, equitySwapMasterConfirmation2018);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		LegalAgreement _that = (LegalAgreement) o;
	
		if (csaInitialMargin2016JapaneseLaw != null ? !csaInitialMargin2016JapaneseLaw.equals(_that.csaInitialMargin2016JapaneseLaw) : _that.csaInitialMargin2016JapaneseLaw != null) return false;
		if (csaInitialMargin2016NewYorkLaw != null ? !csaInitialMargin2016NewYorkLaw.equals(_that.csaInitialMargin2016NewYorkLaw) : _that.csaInitialMargin2016NewYorkLaw != null) return false;
		if (csaVariationMargin2016NewYorkLaw != null ? !csaVariationMargin2016NewYorkLaw.equals(_that.csaVariationMargin2016NewYorkLaw) : _that.csaVariationMargin2016NewYorkLaw != null) return false;
		if (csdInitialMargin2016EnglishLaw != null ? !csdInitialMargin2016EnglishLaw.equals(_that.csdInitialMargin2016EnglishLaw) : _that.csdInitialMargin2016EnglishLaw != null) return false;
		if (equitySwapMasterConfirmation2018 != null ? !equitySwapMasterConfirmation2018.equals(_that.equitySwapMasterConfirmation2018) : _that.equitySwapMasterConfirmation2018 != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (csaInitialMargin2016JapaneseLaw != null ? csaInitialMargin2016JapaneseLaw.hashCode() : 0);
		_result = 31 * _result + (csaInitialMargin2016NewYorkLaw != null ? csaInitialMargin2016NewYorkLaw.hashCode() : 0);
		_result = 31 * _result + (csaVariationMargin2016NewYorkLaw != null ? csaVariationMargin2016NewYorkLaw.hashCode() : 0);
		_result = 31 * _result + (csdInitialMargin2016EnglishLaw != null ? csdInitialMargin2016EnglishLaw.hashCode() : 0);
		_result = 31 * _result + (equitySwapMasterConfirmation2018 != null ? equitySwapMasterConfirmation2018.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LegalAgreement {" +
			"csaInitialMargin2016JapaneseLaw=" + this.csaInitialMargin2016JapaneseLaw + ", " +
			"csaInitialMargin2016NewYorkLaw=" + this.csaInitialMargin2016NewYorkLaw + ", " +
			"csaVariationMargin2016NewYorkLaw=" + this.csaVariationMargin2016NewYorkLaw + ", " +
			"csdInitialMargin2016EnglishLaw=" + this.csdInitialMargin2016EnglishLaw + ", " +
			"equitySwapMasterConfirmation2018=" + this.equitySwapMasterConfirmation2018 + ", " +
			"meta=" + this.meta +
		'}' + " " + super.toString();
	}
}
