package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LegalEntityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a legal entity, with a required name and an optional entity identifier (such as the LEI).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class LegalEntity extends RosettaModelObject implements GlobalKey {
	private final List<FieldWithMetaString> entityId;
	private final MetaFields meta;
	private final FieldWithMetaString name;
	private static LegalEntityMeta metaData = new LegalEntityMeta();
	
	LegalEntity(LegalEntityBuilder builder) {
		this.entityId = ofNullable(builder.getEntityId()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.name = ofNullable(builder.getName()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A legal entity identifier (e.g. RED entity code).
	 */
	@RosettaSynonym(value="entityId", source="FpML_5_10")
	@RosettaSynonym(value="entityId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="entityId", source="DTCC_11_0")
	@RosettaSynonym(value="entityId", source="DTCC_9_0")
	@RosettaSynonym(value="entityId", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="entityId", source="FpML_5_10", path="referenceEntity")
	@RosettaSynonym(value="entityId", source="CME_SubmissionIRS_1_0", path="referenceEntity")
	@RosettaSynonym(value="entityId", source="DTCC_11_0", path="referenceEntity")
	@RosettaSynonym(value="entityId", source="DTCC_9_0", path="referenceEntity")
	@RosettaSynonym(value="entityId", source="CME_ClearedConfirm_1_17", path="referenceEntity")
	@RosettaSynonym(value="lei", source="ISDA_Create_1_0")
	public final List<FieldWithMetaString> getEntityId() {
		return entityId;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The legal entity name.
	 */
	@RosettaSynonym(value="partyName", source="FpML_5_10")
	@RosettaSynonym(value="partyName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyName", source="DTCC_11_0")
	@RosettaSynonym(value="partyName", source="DTCC_9_0")
	@RosettaSynonym(value="partyName", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="entityName", source="FpML_5_10")
	@RosettaSynonym(value="entityName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="entityName", source="DTCC_11_0")
	@RosettaSynonym(value="entityName", source="DTCC_9_0")
	@RosettaSynonym(value="entityName", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="entityName", source="FpML_5_10", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="CME_SubmissionIRS_1_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="DTCC_11_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="DTCC_9_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="CME_ClearedConfirm_1_17", path="referenceEntity")
	@RosettaSynonym(value="SID", source="CME_SubmissionIRS_1_0", path="Hdr")
	@RosettaSynonym(value="partyA_custodian_name", source="ISDA_Create_1_0", path="custodian_and_segregated_account_details")
	@RosettaSynonym(value="partyB_custodian_name", source="ISDA_Create_1_0", path="custodian_and_segregated_account_details")
	@RosettaSynonym(value="principal_name", source="ISDA_Create_1_0")
	@RosettaSynonym(value="executionVenue", source="Rosetta_Workbench")
	@RosettaSynonym(value="initialDesignation", source="FpML_5_10")
	public final FieldWithMetaString getName() {
		return name;
	}
	
	
	@Override
	public RosettaMetaData<? extends LegalEntity> metaData() {
		return metaData;
	} 
	
	public LegalEntityBuilder toBuilder() {
		LegalEntityBuilder builder = new LegalEntityBuilder();
		ofNullable(getEntityId()).ifPresent(entityId -> entityId.forEach(builder::addEntityId));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getName()).ifPresent(builder::setName);
		return builder;
	}

	public static LegalEntityBuilder builder() {
		return new LegalEntityBuilder();
	}

	public static  class LegalEntityBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<LegalEntityBuilder> {
	
		protected List<FieldWithMetaStringBuilder> entityId;
		protected MetaFields.MetaFieldsBuilder meta;
		protected FieldWithMetaStringBuilder name;
	
		public LegalEntityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LegalEntity> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getEntityId() {
			return entityId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateEntityId(int index) {
			if (entityId==null) {
				this.entityId = new ArrayList<>();
			}
			return getIndex(entityId, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getName() {
			return name;
		}
		
		public FieldWithMetaStringBuilder getOrCreateName() {
			if (name!=null) {
				return name;
			}
			else return name = new FieldWithMetaStringBuilder();
		}
		
	
		public LegalEntityBuilder addEntityId(FieldWithMetaString entityId) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			this.entityId.add(entityId.toBuilder());
			return this;
		}
		public LegalEntityBuilder addEntityId(FieldWithMetaString entityId, int _idx) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			getIndex(this.entityId, _idx, () -> entityId.toBuilder());
			this.entityId.set(_idx, entityId.toBuilder());
			return this;
		}
		public LegalEntityBuilder addEntityId(List<FieldWithMetaString> entityIds) {
			if(this.entityId == null){
				this.entityId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : entityIds) {
				this.entityId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public LegalEntityBuilder clearEntityId() {
			this.entityId = null;
			return this;
		}
		public LegalEntityBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public LegalEntityBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public LegalEntityBuilder setName(FieldWithMetaString name) {
			this.name = name.toBuilder();
			return this;
		}
		public LegalEntityBuilder setNameRef(String name) {
			return setName(FieldWithMetaString.builder().setValue(name).build());
		}
		
		public LegalEntity build() {
			return new LegalEntity(this);
		}
	
		@Override
		public LegalEntityBuilder prune() {
			if (entityId!=null) entityId = entityId.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (name!=null && !name.prune().hasData()) name = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEntityId()!=null && !getEntityId().isEmpty()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getName()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("entityId"), processor, FieldWithMetaString.class, entityId);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("name"), processor, FieldWithMetaString.class, name);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LegalEntityBuilder _that = (LegalEntityBuilder) o;
		
			if (!ListEquals.listEquals(entityId, _that.entityId)) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (entityId != null ? entityId.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (name != null ? name.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LegalEntityBuilder {" +
				"entityId=" + this.entityId + ", " +
				"meta=" + this.meta + ", " +
				"name=" + this.name +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("entityId"), processor, FieldWithMetaString.class, entityId);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("name"), processor, FieldWithMetaString.class, name);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		LegalEntity _that = (LegalEntity) o;
	
		if (!ListEquals.listEquals(entityId, _that.entityId)) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (entityId != null ? entityId.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (name != null ? name.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LegalEntity {" +
			"entityId=" + this.entityId + ", " +
			"meta=" + this.meta + ", " +
			"name=" + this.name +
		'}';
	}
}
