package org.isda.cdm;

import java.lang.Integer;
import java.math.BigDecimal;
import org.isda.cdm.CreditLimitUtilisation;
import org.isda.cdm.Velocity;
import org.isda.cdm.metafields.FieldWithMetaCreditLimitTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaCreditLimitTypeEnum.FieldWithMetaCreditLimitTypeEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LimitApplicableMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="LimitApplicable", source="FpML_5_10")
public class LimitApplicable extends RosettaModelObject {
	private final BigDecimal amountRemaining;
	private final BigDecimal amountUtilized;
	private final Integer clipSize;
	private final FieldWithMetaString currency;
	private final FieldWithMetaCreditLimitTypeEnum limitType;
	private final CreditLimitUtilisation utilization;
	private final Velocity velocity;
	private static LimitApplicableMeta metaData = new LimitApplicableMeta();
	
	LimitApplicable(LimitApplicableBuilder builder) {
		this.amountRemaining = builder.getAmountRemaining();
		this.amountUtilized = builder.getAmountUtilized();
		this.clipSize = builder.getClipSize();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
		this.limitType = ofNullable(builder.getLimitType()).map(f->f.build()).orElse(null);
		this.utilization = ofNullable(builder.getUtilization()).map(f->f.build()).orElse(null);
		this.velocity = ofNullable(builder.getVelocity()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The limit remaining for the limit level and limit type. This does not take into account any pending trades. While the attribute is of type integer in FpML and the CME schema, it has been specified to be of type number in the CDM to take into consideration java size limits as well as for consistency purposes with the way most monetary amounts are expressed.
	 */
	@RosettaSynonym(value="amountRemaining", source="FpML_5_10")
	@RosettaSynonym(value="amountRemaining", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="amountRemaining", source="Rosetta_Workbench")
	public final BigDecimal getAmountRemaining() {
		return amountRemaining;
	}
	
	/**
	 * The limit utilised by all the cleared trades for the limit level and limit type. While the attribute is of type integer in FpML and the CME schema, it has been specified to be of type number in the CDM to take into consideration java size limits as well as for consistency purposes with the way most monetary amounts are expressed.
	 */
	@RosettaSynonym(value="amountUtilized", source="FpML_5_10")
	@RosettaSynonym(value="amountUtilized", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="amountUtilized", source="Rosetta_Workbench")
	public final BigDecimal getAmountUtilized() {
		return amountUtilized;
	}
	
	/**
	 * This element is required in FpML, optional in CDM for the purpose of accommodating the CME data representation while making reference to the FpML one.
	 */
	@RosettaSynonym(value="clipSize", source="FpML_5_10")
	public final Integer getClipSize() {
		return clipSize;
	}
	
	/**
	 * The currency in which the applicable limit is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="currency", source="Rosetta_Workbench")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	/**
	 * Standard code to indicate which type of credit line is being referred to - i.e. IM, DV01, PV01, CS01, Notional, Clip Size, Notional, maximumOrderQuantity.
	 */
	@RosettaSynonym(value="limitType", source="FpML_5_10")
	@RosettaSynonym(value="limitType", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="limitType", source="Rosetta_Workbench")
	public final FieldWithMetaCreditLimitTypeEnum getLimitType() {
		return limitType;
	}
	
	@RosettaSynonym(value="utilization", source="FpML_5_10")
	public final CreditLimitUtilisation getUtilization() {
		return utilization;
	}
	
	@RosettaSynonym(value="velocity", source="FpML_5_10")
	public final Velocity getVelocity() {
		return velocity;
	}
	
	
	@Override
	public RosettaMetaData<? extends LimitApplicable> metaData() {
		return metaData;
	} 
	
	public LimitApplicableBuilder toBuilder() {
		LimitApplicableBuilder builder = new LimitApplicableBuilder();
		ofNullable(getAmountRemaining()).ifPresent(builder::setAmountRemaining);
		ofNullable(getAmountUtilized()).ifPresent(builder::setAmountUtilized);
		ofNullable(getClipSize()).ifPresent(builder::setClipSize);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getLimitType()).ifPresent(builder::setLimitType);
		ofNullable(getUtilization()).ifPresent(builder::setUtilization);
		ofNullable(getVelocity()).ifPresent(builder::setVelocity);
		return builder;
	}

	public static LimitApplicableBuilder builder() {
		return new LimitApplicableBuilder();
	}

	public static  class LimitApplicableBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal amountRemaining;
		protected BigDecimal amountUtilized;
		protected Integer clipSize;
		protected FieldWithMetaStringBuilder currency;
		protected FieldWithMetaCreditLimitTypeEnumBuilder limitType;
		protected CreditLimitUtilisation.CreditLimitUtilisationBuilder utilization;
		protected Velocity.VelocityBuilder velocity;
	
		public LimitApplicableBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends LimitApplicable> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getAmountRemaining() {
			return amountRemaining;
		}
		
		
		public BigDecimal getAmountUtilized() {
			return amountUtilized;
		}
		
		
		public Integer getClipSize() {
			return clipSize;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaCreditLimitTypeEnumBuilder getLimitType() {
			return limitType;
		}
		
		public FieldWithMetaCreditLimitTypeEnumBuilder getOrCreateLimitType() {
			if (limitType!=null) {
				return limitType;
			}
			else return limitType = new FieldWithMetaCreditLimitTypeEnumBuilder();
		}
		
		
		public CreditLimitUtilisation.CreditLimitUtilisationBuilder getUtilization() {
			return utilization;
		}
		
		public CreditLimitUtilisation.CreditLimitUtilisationBuilder getOrCreateUtilization() {
			if (utilization!=null) {
				return utilization;
			}
			else return utilization = new CreditLimitUtilisation.CreditLimitUtilisationBuilder();
		}
		
		
		public Velocity.VelocityBuilder getVelocity() {
			return velocity;
		}
		
		public Velocity.VelocityBuilder getOrCreateVelocity() {
			if (velocity!=null) {
				return velocity;
			}
			else return velocity = new Velocity.VelocityBuilder();
		}
		
	
		public LimitApplicableBuilder setAmountRemaining(BigDecimal amountRemaining) {
			this.amountRemaining = amountRemaining;
			return this;
		}
		public LimitApplicableBuilder setAmountUtilized(BigDecimal amountUtilized) {
			this.amountUtilized = amountUtilized;
			return this;
		}
		public LimitApplicableBuilder setClipSize(Integer clipSize) {
			this.clipSize = clipSize;
			return this;
		}
		public LimitApplicableBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public LimitApplicableBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		public LimitApplicableBuilder setLimitType(FieldWithMetaCreditLimitTypeEnum limitType) {
			this.limitType = limitType.toBuilder();
			return this;
		}
		public LimitApplicableBuilder setLimitTypeRef(CreditLimitTypeEnum limitType) {
			return setLimitType(FieldWithMetaCreditLimitTypeEnum.builder().setValue(limitType).build());
		}
		public LimitApplicableBuilder setUtilization(CreditLimitUtilisation utilization) {
			this.utilization = utilization.toBuilder();
			return this;
		}
		public LimitApplicableBuilder setUtilizationBuilder(CreditLimitUtilisation.CreditLimitUtilisationBuilder utilization) {
			this.utilization = utilization;
			return this;
		}
		public LimitApplicableBuilder setVelocity(Velocity velocity) {
			this.velocity = velocity.toBuilder();
			return this;
		}
		public LimitApplicableBuilder setVelocityBuilder(Velocity.VelocityBuilder velocity) {
			this.velocity = velocity;
			return this;
		}
		
		public LimitApplicable build() {
			return new LimitApplicable(this);
		}
	
		@Override
		public LimitApplicableBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			if (limitType!=null && !limitType.prune().hasData()) limitType = null;
			if (utilization!=null && !utilization.prune().hasData()) utilization = null;
			if (velocity!=null && !velocity.prune().hasData()) velocity = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmountRemaining()!=null) return true;
			if (getAmountUtilized()!=null) return true;
			if (getClipSize()!=null) return true;
			if (getCurrency()!=null) return true;
			if (getLimitType()!=null) return true;
			if (getUtilization()!=null && getUtilization().hasData()) return true;
			if (getVelocity()!=null && getVelocity().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("amountRemaining"), BigDecimal.class, amountRemaining, this);
			processor.processBasic(path.newSubPath("amountUtilized"), BigDecimal.class, amountUtilized, this);
			processor.processBasic(path.newSubPath("clipSize"), Integer.class, clipSize, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
			processRosetta(path.newSubPath("limitType"), processor, FieldWithMetaCreditLimitTypeEnum.class, limitType);
			processRosetta(path.newSubPath("utilization"), processor, CreditLimitUtilisation.class, utilization);
			processRosetta(path.newSubPath("velocity"), processor, Velocity.class, velocity);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LimitApplicableBuilder _that = (LimitApplicableBuilder) o;
		
			if (amountRemaining != null ? !amountRemaining.equals(_that.amountRemaining) : _that.amountRemaining != null) return false;
			if (amountUtilized != null ? !amountUtilized.equals(_that.amountUtilized) : _that.amountUtilized != null) return false;
			if (clipSize != null ? !clipSize.equals(_that.clipSize) : _that.clipSize != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			if (limitType != null ? !limitType.equals(_that.limitType) : _that.limitType != null) return false;
			if (utilization != null ? !utilization.equals(_that.utilization) : _that.utilization != null) return false;
			if (velocity != null ? !velocity.equals(_that.velocity) : _that.velocity != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amountRemaining != null ? amountRemaining.hashCode() : 0);
			_result = 31 * _result + (amountUtilized != null ? amountUtilized.hashCode() : 0);
			_result = 31 * _result + (clipSize != null ? clipSize.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			_result = 31 * _result + (limitType != null ? limitType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (utilization != null ? utilization.hashCode() : 0);
			_result = 31 * _result + (velocity != null ? velocity.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LimitApplicableBuilder {" +
				"amountRemaining=" + this.amountRemaining + ", " +
				"amountUtilized=" + this.amountUtilized + ", " +
				"clipSize=" + this.clipSize + ", " +
				"currency=" + this.currency + ", " +
				"limitType=" + this.limitType + ", " +
				"utilization=" + this.utilization + ", " +
				"velocity=" + this.velocity +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("amountRemaining"), BigDecimal.class, amountRemaining, this);
		processor.processBasic(path.newSubPath("amountUtilized"), BigDecimal.class, amountUtilized, this);
		processor.processBasic(path.newSubPath("clipSize"), Integer.class, clipSize, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		processRosetta(path.newSubPath("limitType"), processor, FieldWithMetaCreditLimitTypeEnum.class, limitType);
		processRosetta(path.newSubPath("utilization"), processor, CreditLimitUtilisation.class, utilization);
		processRosetta(path.newSubPath("velocity"), processor, Velocity.class, velocity);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		LimitApplicable _that = (LimitApplicable) o;
	
		if (amountRemaining != null ? !amountRemaining.equals(_that.amountRemaining) : _that.amountRemaining != null) return false;
		if (amountUtilized != null ? !amountUtilized.equals(_that.amountUtilized) : _that.amountUtilized != null) return false;
		if (clipSize != null ? !clipSize.equals(_that.clipSize) : _that.clipSize != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		if (limitType != null ? !limitType.equals(_that.limitType) : _that.limitType != null) return false;
		if (utilization != null ? !utilization.equals(_that.utilization) : _that.utilization != null) return false;
		if (velocity != null ? !velocity.equals(_that.velocity) : _that.velocity != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (amountRemaining != null ? amountRemaining.hashCode() : 0);
		_result = 31 * _result + (amountUtilized != null ? amountUtilized.hashCode() : 0);
		_result = 31 * _result + (clipSize != null ? clipSize.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		_result = 31 * _result + (limitType != null ? limitType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (utilization != null ? utilization.hashCode() : 0);
		_result = 31 * _result + (velocity != null ? velocity.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "LimitApplicable {" +
			"amountRemaining=" + this.amountRemaining + ", " +
			"amountUtilized=" + this.amountUtilized + ", " +
			"clipSize=" + this.clipSize + ", " +
			"currency=" + this.currency + ", " +
			"limitType=" + this.limitType + ", " +
			"utilization=" + this.utilization + ", " +
			"velocity=" + this.velocity +
		'}';
	}
}
