package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify the level at which the limit is set: customer business, proprietary business or account level. This is part of the CME specification for clearing credit limits, although not specified as a set of enumerated values as part of the clearing confirmation specification.
 * 
 * @version 2.5.4
 */
public enum LimitLevelEnum {
    

	/**
	 * The limit is set in relation to the proprietary business undertaken by the clearing counterparty.
	 */
	@RosettaSynonym(value = "ACCT", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "ACCT", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "ACCT", source = "CME_SubmissionIRS_1_0")
	ACCOUNT,
	    

	/**
	 * The limit is set in relation to the customer business undertaken by the clearing counterparty.
	 */
	@RosettaSynonym(value = "CUST", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "CUST", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "CUST", source = "CME_SubmissionIRS_1_0")
	CUSTOMER,
	    

	/**
	 * The limit is set at the account level in relation to the clearing counterparty.
	 */
	@RosettaSynonym(value = "HOUS", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "HOUS", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "HOUS", source = "CME_SubmissionIRS_1_0")
	HOUSE    
	;

	private final String displayName;
	
	LimitLevelEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
