package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaCashflow;
import org.isda.cdm.metafields.ReferenceWithMetaCashflow.ReferenceWithMetaCashflowBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaContract;
import org.isda.cdm.metafields.ReferenceWithMetaContract.ReferenceWithMetaContractBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaCreditDefaultPayout;
import org.isda.cdm.metafields.ReferenceWithMetaCreditDefaultPayout.ReferenceWithMetaCreditDefaultPayoutBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaEquityPayout;
import org.isda.cdm.metafields.ReferenceWithMetaEquityPayout.ReferenceWithMetaEquityPayoutBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaEvent;
import org.isda.cdm.metafields.ReferenceWithMetaEvent.ReferenceWithMetaEventBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaExecution;
import org.isda.cdm.metafields.ReferenceWithMetaExecution.ReferenceWithMetaExecutionBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaInterestRatePayout;
import org.isda.cdm.metafields.ReferenceWithMetaInterestRatePayout.ReferenceWithMetaInterestRatePayoutBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaLegalAgreement;
import org.isda.cdm.metafields.ReferenceWithMetaLegalAgreement.ReferenceWithMetaLegalAgreementBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaOptionPayout;
import org.isda.cdm.metafields.ReferenceWithMetaOptionPayout.ReferenceWithMetaOptionPayoutBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaPortfolioState;
import org.isda.cdm.metafields.ReferenceWithMetaPortfolioState.ReferenceWithMetaPortfolioStateBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaTransferPrimitive;
import org.isda.cdm.metafields.ReferenceWithMetaTransferPrimitive.ReferenceWithMetaTransferPrimitiveBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LineageMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to provide lineage information across lifecycle events through a pointer or set of pointers into the event(s), contract(s) and, possibly, payout components that the event is dependent on or relates to. As an example, if an inception event is corrected, the correction event will have a lineage into the initial event, which takes the form of a rosettaKey into that initial inception event. Two referencing mechanisms are provided as part of the CDM: either the rosettaKey, which corresponds to the hash value of the CDM class which is referred to, or a reference qualifier which is meant to provide support for the ingestion of xml documents with id/href mechanisms. The CDM recommends the use of the rosettaKey and provides a default implementation which is accessible in the generated code through org.isda.cdm.rosettakey.RosettaKeyValueHashFunction. If implementers want to use an alternative hashing mechanism, the API in which they need to plug it is com.rosetta.model.lib.HashFunction.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class Lineage extends RosettaModelObject {
	private final List<ReferenceWithMetaCashflow> cashflowReference;
	private final List<ReferenceWithMetaContract> contractReference;
	private final List<ReferenceWithMetaCreditDefaultPayout> creditDefaultPayoutReference;
	private final List<ReferenceWithMetaEquityPayout> equityPayoutReference;
	private final List<ReferenceWithMetaEvent> eventReference;
	private final List<ReferenceWithMetaExecution> executionReference;
	private final List<ReferenceWithMetaInterestRatePayout> interestRatePayoutReference;
	private final List<ReferenceWithMetaLegalAgreement> legalAgreement;
	private final List<ReferenceWithMetaOptionPayout> optionPayoutReference;
	private final List<ReferenceWithMetaPortfolioState> portfolioStateReference;
	private final List<ReferenceWithMetaTransferPrimitive> transferReference;
	private static LineageMeta metaData = new LineageMeta();
	
	Lineage(LineageBuilder builder) {
		this.cashflowReference = ofNullable(builder.getCashflowReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contractReference = ofNullable(builder.getContractReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.creditDefaultPayoutReference = ofNullable(builder.getCreditDefaultPayoutReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.equityPayoutReference = ofNullable(builder.getEquityPayoutReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.eventReference = ofNullable(builder.getEventReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.executionReference = ofNullable(builder.getExecutionReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.interestRatePayoutReference = ofNullable(builder.getInterestRatePayoutReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.legalAgreement = ofNullable(builder.getLegalAgreement()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.optionPayoutReference = ofNullable(builder.getOptionPayoutReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.portfolioStateReference = ofNullable(builder.getPortfolioStateReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.transferReference = ofNullable(builder.getTransferReference()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The reference to the instantiation of a Cashflow payout component object. An expected typical usage is to provide lineage for the payment of, say, the option premium or the swap initial fee. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="cashflowReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaCashflow> getCashflowReference() {
		return cashflowReference;
	}
	
	/**
	 * The reference to the instantiation of a Contract object. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="contractReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaContract> getContractReference() {
		return contractReference;
	}
	
	/**
	 * The reference to the instantiation of a CreditdefaultPayout component object. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="creditDefaultReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaCreditDefaultPayout> getCreditDefaultPayoutReference() {
		return creditDefaultPayoutReference;
	}
	
	/**
	 * The reference to the instantiation of a EquityPayout object. An expected typical usage is to provide lineage for the payment of, say, an equity dividend. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="equityPayoutReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaEquityPayout> getEquityPayoutReference() {
		return equityPayoutReference;
	}
	
	/**
	 * The reference to the instantiation of an Event object, either through a rosettaKey or an xml-derived id/href mechanism. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="eventReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaEvent> getEventReference() {
		return eventReference;
	}
	
	/**
	 * The reference to the instantiation of a Execution object. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="executionReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaExecution> getExecutionReference() {
		return executionReference;
	}
	
	/**
	 * The reference to the instantiation of a InterestRatePayout component object. An expected typical usage is to provide lineage for the payment of, say, an interest rate swap reset, with the ability to relate the gross cashflow amounts to the respective payout components. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="interestRatePayoutReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaInterestRatePayout> getInterestRatePayoutReference() {
		return interestRatePayoutReference;
	}
	
	/**
	 * The reference to the instantiation of a Legal Agreement object. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	public final List<ReferenceWithMetaLegalAgreement> getLegalAgreement() {
		return legalAgreement;
	}
	
	/**
	 * The reference to the instantiation of a OptionPayout component object. The definition associated to the Lineage class provides more details with respect to those referencing approaches, their expected usage and available implementation.
	 */
	@RosettaSynonym(value="optionPayoutReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaOptionPayout> getOptionPayoutReference() {
		return optionPayoutReference;
	}
	
	/**
	 * The refence to the previous state of a Portfolio, in a chain of Events leading up to a build of that Portfolio as the holding of Product(s) in specific Quantity(ies). As part of the PortfolioState object, a pointer to the previous PortfolioState is provided through a Lineage object, together with pointer(s) to the Event or set of Events leading up to the current (new) state.
	 */
	public final List<ReferenceWithMetaPortfolioState> getPortfolioStateReference() {
		return portfolioStateReference;
	}
	
	/**
	 * The reference to the instantiation of a TransferPrimitive object.
	 */
	@RosettaSynonym(value="transferReference", source="Rosetta_Workbench")
	public final List<ReferenceWithMetaTransferPrimitive> getTransferReference() {
		return transferReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends Lineage> metaData() {
		return metaData;
	} 
	
	public LineageBuilder toBuilder() {
		LineageBuilder builder = new LineageBuilder();
		ofNullable(getCashflowReference()).ifPresent(cashflowReference -> cashflowReference.forEach(builder::addCashflowReference));
		ofNullable(getContractReference()).ifPresent(contractReference -> contractReference.forEach(builder::addContractReference));
		ofNullable(getCreditDefaultPayoutReference()).ifPresent(creditDefaultPayoutReference -> creditDefaultPayoutReference.forEach(builder::addCreditDefaultPayoutReference));
		ofNullable(getEquityPayoutReference()).ifPresent(equityPayoutReference -> equityPayoutReference.forEach(builder::addEquityPayoutReference));
		ofNullable(getEventReference()).ifPresent(eventReference -> eventReference.forEach(builder::addEventReference));
		ofNullable(getExecutionReference()).ifPresent(executionReference -> executionReference.forEach(builder::addExecutionReference));
		ofNullable(getInterestRatePayoutReference()).ifPresent(interestRatePayoutReference -> interestRatePayoutReference.forEach(builder::addInterestRatePayoutReference));
		ofNullable(getLegalAgreement()).ifPresent(legalAgreement -> legalAgreement.forEach(builder::addLegalAgreement));
		ofNullable(getOptionPayoutReference()).ifPresent(optionPayoutReference -> optionPayoutReference.forEach(builder::addOptionPayoutReference));
		ofNullable(getPortfolioStateReference()).ifPresent(portfolioStateReference -> portfolioStateReference.forEach(builder::addPortfolioStateReference));
		ofNullable(getTransferReference()).ifPresent(transferReference -> transferReference.forEach(builder::addTransferReference));
		return builder;
	}

	public static LineageBuilder builder() {
		return new LineageBuilder();
	}

	public static  class LineageBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaCashflowBuilder> cashflowReference;
		protected List<ReferenceWithMetaContractBuilder> contractReference;
		protected List<ReferenceWithMetaCreditDefaultPayoutBuilder> creditDefaultPayoutReference;
		protected List<ReferenceWithMetaEquityPayoutBuilder> equityPayoutReference;
		protected List<ReferenceWithMetaEventBuilder> eventReference;
		protected List<ReferenceWithMetaExecutionBuilder> executionReference;
		protected List<ReferenceWithMetaInterestRatePayoutBuilder> interestRatePayoutReference;
		protected List<ReferenceWithMetaLegalAgreementBuilder> legalAgreement;
		protected List<ReferenceWithMetaOptionPayoutBuilder> optionPayoutReference;
		protected List<ReferenceWithMetaPortfolioStateBuilder> portfolioStateReference;
		protected List<ReferenceWithMetaTransferPrimitiveBuilder> transferReference;
	
		public LineageBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Lineage> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaCashflowBuilder> getCashflowReference() {
			return cashflowReference;
		}
		
		public ReferenceWithMetaCashflowBuilder getOrCreateCashflowReference(int index) {
			if (cashflowReference==null) {
				this.cashflowReference = new ArrayList<>();
			}
			return getIndex(cashflowReference, index, ()->new ReferenceWithMetaCashflowBuilder());
		}
		
		
		public List<ReferenceWithMetaContractBuilder> getContractReference() {
			return contractReference;
		}
		
		public ReferenceWithMetaContractBuilder getOrCreateContractReference(int index) {
			if (contractReference==null) {
				this.contractReference = new ArrayList<>();
			}
			return getIndex(contractReference, index, ()->new ReferenceWithMetaContractBuilder());
		}
		
		
		public List<ReferenceWithMetaCreditDefaultPayoutBuilder> getCreditDefaultPayoutReference() {
			return creditDefaultPayoutReference;
		}
		
		public ReferenceWithMetaCreditDefaultPayoutBuilder getOrCreateCreditDefaultPayoutReference(int index) {
			if (creditDefaultPayoutReference==null) {
				this.creditDefaultPayoutReference = new ArrayList<>();
			}
			return getIndex(creditDefaultPayoutReference, index, ()->new ReferenceWithMetaCreditDefaultPayoutBuilder());
		}
		
		
		public List<ReferenceWithMetaEquityPayoutBuilder> getEquityPayoutReference() {
			return equityPayoutReference;
		}
		
		public ReferenceWithMetaEquityPayoutBuilder getOrCreateEquityPayoutReference(int index) {
			if (equityPayoutReference==null) {
				this.equityPayoutReference = new ArrayList<>();
			}
			return getIndex(equityPayoutReference, index, ()->new ReferenceWithMetaEquityPayoutBuilder());
		}
		
		
		public List<ReferenceWithMetaEventBuilder> getEventReference() {
			return eventReference;
		}
		
		public ReferenceWithMetaEventBuilder getOrCreateEventReference(int index) {
			if (eventReference==null) {
				this.eventReference = new ArrayList<>();
			}
			return getIndex(eventReference, index, ()->new ReferenceWithMetaEventBuilder());
		}
		
		
		public List<ReferenceWithMetaExecutionBuilder> getExecutionReference() {
			return executionReference;
		}
		
		public ReferenceWithMetaExecutionBuilder getOrCreateExecutionReference(int index) {
			if (executionReference==null) {
				this.executionReference = new ArrayList<>();
			}
			return getIndex(executionReference, index, ()->new ReferenceWithMetaExecutionBuilder());
		}
		
		
		public List<ReferenceWithMetaInterestRatePayoutBuilder> getInterestRatePayoutReference() {
			return interestRatePayoutReference;
		}
		
		public ReferenceWithMetaInterestRatePayoutBuilder getOrCreateInterestRatePayoutReference(int index) {
			if (interestRatePayoutReference==null) {
				this.interestRatePayoutReference = new ArrayList<>();
			}
			return getIndex(interestRatePayoutReference, index, ()->new ReferenceWithMetaInterestRatePayoutBuilder());
		}
		
		
		public List<ReferenceWithMetaLegalAgreementBuilder> getLegalAgreement() {
			return legalAgreement;
		}
		
		public ReferenceWithMetaLegalAgreementBuilder getOrCreateLegalAgreement(int index) {
			if (legalAgreement==null) {
				this.legalAgreement = new ArrayList<>();
			}
			return getIndex(legalAgreement, index, ()->new ReferenceWithMetaLegalAgreementBuilder());
		}
		
		
		public List<ReferenceWithMetaOptionPayoutBuilder> getOptionPayoutReference() {
			return optionPayoutReference;
		}
		
		public ReferenceWithMetaOptionPayoutBuilder getOrCreateOptionPayoutReference(int index) {
			if (optionPayoutReference==null) {
				this.optionPayoutReference = new ArrayList<>();
			}
			return getIndex(optionPayoutReference, index, ()->new ReferenceWithMetaOptionPayoutBuilder());
		}
		
		
		public List<ReferenceWithMetaPortfolioStateBuilder> getPortfolioStateReference() {
			return portfolioStateReference;
		}
		
		public ReferenceWithMetaPortfolioStateBuilder getOrCreatePortfolioStateReference(int index) {
			if (portfolioStateReference==null) {
				this.portfolioStateReference = new ArrayList<>();
			}
			return getIndex(portfolioStateReference, index, ()->new ReferenceWithMetaPortfolioStateBuilder());
		}
		
		
		public List<ReferenceWithMetaTransferPrimitiveBuilder> getTransferReference() {
			return transferReference;
		}
		
		public ReferenceWithMetaTransferPrimitiveBuilder getOrCreateTransferReference(int index) {
			if (transferReference==null) {
				this.transferReference = new ArrayList<>();
			}
			return getIndex(transferReference, index, ()->new ReferenceWithMetaTransferPrimitiveBuilder());
		}
		
	
		public LineageBuilder addCashflowReference(ReferenceWithMetaCashflow cashflowReference) {
			if(this.cashflowReference == null){
				this.cashflowReference = new ArrayList<>();
			}
			this.cashflowReference.add(cashflowReference.toBuilder());
			return this;
		}
		public LineageBuilder addCashflowReference(ReferenceWithMetaCashflow cashflowReference, int _idx) {
			if(this.cashflowReference == null){
				this.cashflowReference = new ArrayList<>();
			}
			getIndex(this.cashflowReference, _idx, () -> cashflowReference.toBuilder());
			this.cashflowReference.set(_idx, cashflowReference.toBuilder());
			return this;
		}
		public LineageBuilder addCashflowReference(List<ReferenceWithMetaCashflow> cashflowReferences) {
			if(this.cashflowReference == null){
				this.cashflowReference = new ArrayList<>();
			}
			for (ReferenceWithMetaCashflow toAdd : cashflowReferences) {
				this.cashflowReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addCashflowReferenceBuilder(ReferenceWithMetaCashflowBuilder cashflowReference) {
			if(this.cashflowReference == null){
				this.cashflowReference = new ArrayList<>();
			}
			this.cashflowReference.add(cashflowReference);
			return this;
		}
		
		public LineageBuilder addCashflowReferenceRef(Cashflow.CashflowBuilder cashflowReference) {
			return addCashflowReference(ReferenceWithMetaCashflow.builder().setValueBuilder(cashflowReference).build());
		}
		
		public LineageBuilder addCashflowReferenceRef(Cashflow.CashflowBuilder cashflowReference, int _idx) {
			return addCashflowReference(ReferenceWithMetaCashflow.builder().setValueBuilder(cashflowReference).build(), _idx);
		}
		
		public LineageBuilder addCashflowReferenceRef(List<Cashflow> cashflowReferences) {
			for (Cashflow toAdd : cashflowReferences) {
				addCashflowReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addCashflowReferenceRef(Cashflow cashflowReference) {
			return addCashflowReferenceRef(cashflowReference.toBuilder());
		}
		
		public LineageBuilder clearCashflowReference() {
			this.cashflowReference = null;
			return this;
		}
		public LineageBuilder addContractReference(ReferenceWithMetaContract contractReference) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			this.contractReference.add(contractReference.toBuilder());
			return this;
		}
		public LineageBuilder addContractReference(ReferenceWithMetaContract contractReference, int _idx) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			getIndex(this.contractReference, _idx, () -> contractReference.toBuilder());
			this.contractReference.set(_idx, contractReference.toBuilder());
			return this;
		}
		public LineageBuilder addContractReference(List<ReferenceWithMetaContract> contractReferences) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			for (ReferenceWithMetaContract toAdd : contractReferences) {
				this.contractReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addContractReferenceBuilder(ReferenceWithMetaContractBuilder contractReference) {
			if(this.contractReference == null){
				this.contractReference = new ArrayList<>();
			}
			this.contractReference.add(contractReference);
			return this;
		}
		
		public LineageBuilder addContractReferenceRef(Contract.ContractBuilder contractReference) {
			return addContractReference(ReferenceWithMetaContract.builder().setValueBuilder(contractReference).build());
		}
		
		public LineageBuilder addContractReferenceRef(Contract.ContractBuilder contractReference, int _idx) {
			return addContractReference(ReferenceWithMetaContract.builder().setValueBuilder(contractReference).build(), _idx);
		}
		
		public LineageBuilder addContractReferenceRef(List<Contract> contractReferences) {
			for (Contract toAdd : contractReferences) {
				addContractReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addContractReferenceRef(Contract contractReference) {
			return addContractReferenceRef(contractReference.toBuilder());
		}
		
		public LineageBuilder clearContractReference() {
			this.contractReference = null;
			return this;
		}
		public LineageBuilder addCreditDefaultPayoutReference(ReferenceWithMetaCreditDefaultPayout creditDefaultPayoutReference) {
			if(this.creditDefaultPayoutReference == null){
				this.creditDefaultPayoutReference = new ArrayList<>();
			}
			this.creditDefaultPayoutReference.add(creditDefaultPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addCreditDefaultPayoutReference(ReferenceWithMetaCreditDefaultPayout creditDefaultPayoutReference, int _idx) {
			if(this.creditDefaultPayoutReference == null){
				this.creditDefaultPayoutReference = new ArrayList<>();
			}
			getIndex(this.creditDefaultPayoutReference, _idx, () -> creditDefaultPayoutReference.toBuilder());
			this.creditDefaultPayoutReference.set(_idx, creditDefaultPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addCreditDefaultPayoutReference(List<ReferenceWithMetaCreditDefaultPayout> creditDefaultPayoutReferences) {
			if(this.creditDefaultPayoutReference == null){
				this.creditDefaultPayoutReference = new ArrayList<>();
			}
			for (ReferenceWithMetaCreditDefaultPayout toAdd : creditDefaultPayoutReferences) {
				this.creditDefaultPayoutReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addCreditDefaultPayoutReferenceBuilder(ReferenceWithMetaCreditDefaultPayoutBuilder creditDefaultPayoutReference) {
			if(this.creditDefaultPayoutReference == null){
				this.creditDefaultPayoutReference = new ArrayList<>();
			}
			this.creditDefaultPayoutReference.add(creditDefaultPayoutReference);
			return this;
		}
		
		public LineageBuilder addCreditDefaultPayoutReferenceRef(CreditDefaultPayout.CreditDefaultPayoutBuilder creditDefaultPayoutReference) {
			return addCreditDefaultPayoutReference(ReferenceWithMetaCreditDefaultPayout.builder().setValueBuilder(creditDefaultPayoutReference).build());
		}
		
		public LineageBuilder addCreditDefaultPayoutReferenceRef(CreditDefaultPayout.CreditDefaultPayoutBuilder creditDefaultPayoutReference, int _idx) {
			return addCreditDefaultPayoutReference(ReferenceWithMetaCreditDefaultPayout.builder().setValueBuilder(creditDefaultPayoutReference).build(), _idx);
		}
		
		public LineageBuilder addCreditDefaultPayoutReferenceRef(List<CreditDefaultPayout> creditDefaultPayoutReferences) {
			for (CreditDefaultPayout toAdd : creditDefaultPayoutReferences) {
				addCreditDefaultPayoutReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addCreditDefaultPayoutReferenceRef(CreditDefaultPayout creditDefaultPayoutReference) {
			return addCreditDefaultPayoutReferenceRef(creditDefaultPayoutReference.toBuilder());
		}
		
		public LineageBuilder clearCreditDefaultPayoutReference() {
			this.creditDefaultPayoutReference = null;
			return this;
		}
		public LineageBuilder addEquityPayoutReference(ReferenceWithMetaEquityPayout equityPayoutReference) {
			if(this.equityPayoutReference == null){
				this.equityPayoutReference = new ArrayList<>();
			}
			this.equityPayoutReference.add(equityPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addEquityPayoutReference(ReferenceWithMetaEquityPayout equityPayoutReference, int _idx) {
			if(this.equityPayoutReference == null){
				this.equityPayoutReference = new ArrayList<>();
			}
			getIndex(this.equityPayoutReference, _idx, () -> equityPayoutReference.toBuilder());
			this.equityPayoutReference.set(_idx, equityPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addEquityPayoutReference(List<ReferenceWithMetaEquityPayout> equityPayoutReferences) {
			if(this.equityPayoutReference == null){
				this.equityPayoutReference = new ArrayList<>();
			}
			for (ReferenceWithMetaEquityPayout toAdd : equityPayoutReferences) {
				this.equityPayoutReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addEquityPayoutReferenceBuilder(ReferenceWithMetaEquityPayoutBuilder equityPayoutReference) {
			if(this.equityPayoutReference == null){
				this.equityPayoutReference = new ArrayList<>();
			}
			this.equityPayoutReference.add(equityPayoutReference);
			return this;
		}
		
		public LineageBuilder addEquityPayoutReferenceRef(EquityPayout.EquityPayoutBuilder equityPayoutReference) {
			return addEquityPayoutReference(ReferenceWithMetaEquityPayout.builder().setValueBuilder(equityPayoutReference).build());
		}
		
		public LineageBuilder addEquityPayoutReferenceRef(EquityPayout.EquityPayoutBuilder equityPayoutReference, int _idx) {
			return addEquityPayoutReference(ReferenceWithMetaEquityPayout.builder().setValueBuilder(equityPayoutReference).build(), _idx);
		}
		
		public LineageBuilder addEquityPayoutReferenceRef(List<EquityPayout> equityPayoutReferences) {
			for (EquityPayout toAdd : equityPayoutReferences) {
				addEquityPayoutReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addEquityPayoutReferenceRef(EquityPayout equityPayoutReference) {
			return addEquityPayoutReferenceRef(equityPayoutReference.toBuilder());
		}
		
		public LineageBuilder clearEquityPayoutReference() {
			this.equityPayoutReference = null;
			return this;
		}
		public LineageBuilder addEventReference(ReferenceWithMetaEvent eventReference) {
			if(this.eventReference == null){
				this.eventReference = new ArrayList<>();
			}
			this.eventReference.add(eventReference.toBuilder());
			return this;
		}
		public LineageBuilder addEventReference(ReferenceWithMetaEvent eventReference, int _idx) {
			if(this.eventReference == null){
				this.eventReference = new ArrayList<>();
			}
			getIndex(this.eventReference, _idx, () -> eventReference.toBuilder());
			this.eventReference.set(_idx, eventReference.toBuilder());
			return this;
		}
		public LineageBuilder addEventReference(List<ReferenceWithMetaEvent> eventReferences) {
			if(this.eventReference == null){
				this.eventReference = new ArrayList<>();
			}
			for (ReferenceWithMetaEvent toAdd : eventReferences) {
				this.eventReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addEventReferenceBuilder(ReferenceWithMetaEventBuilder eventReference) {
			if(this.eventReference == null){
				this.eventReference = new ArrayList<>();
			}
			this.eventReference.add(eventReference);
			return this;
		}
		
		public LineageBuilder addEventReferenceRef(Event.EventBuilder eventReference) {
			return addEventReference(ReferenceWithMetaEvent.builder().setValueBuilder(eventReference).build());
		}
		
		public LineageBuilder addEventReferenceRef(Event.EventBuilder eventReference, int _idx) {
			return addEventReference(ReferenceWithMetaEvent.builder().setValueBuilder(eventReference).build(), _idx);
		}
		
		public LineageBuilder addEventReferenceRef(List<Event> eventReferences) {
			for (Event toAdd : eventReferences) {
				addEventReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addEventReferenceRef(Event eventReference) {
			return addEventReferenceRef(eventReference.toBuilder());
		}
		
		public LineageBuilder clearEventReference() {
			this.eventReference = null;
			return this;
		}
		public LineageBuilder addExecutionReference(ReferenceWithMetaExecution executionReference) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			this.executionReference.add(executionReference.toBuilder());
			return this;
		}
		public LineageBuilder addExecutionReference(ReferenceWithMetaExecution executionReference, int _idx) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			getIndex(this.executionReference, _idx, () -> executionReference.toBuilder());
			this.executionReference.set(_idx, executionReference.toBuilder());
			return this;
		}
		public LineageBuilder addExecutionReference(List<ReferenceWithMetaExecution> executionReferences) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			for (ReferenceWithMetaExecution toAdd : executionReferences) {
				this.executionReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addExecutionReferenceBuilder(ReferenceWithMetaExecutionBuilder executionReference) {
			if(this.executionReference == null){
				this.executionReference = new ArrayList<>();
			}
			this.executionReference.add(executionReference);
			return this;
		}
		
		public LineageBuilder addExecutionReferenceRef(Execution.ExecutionBuilder executionReference) {
			return addExecutionReference(ReferenceWithMetaExecution.builder().setValueBuilder(executionReference).build());
		}
		
		public LineageBuilder addExecutionReferenceRef(Execution.ExecutionBuilder executionReference, int _idx) {
			return addExecutionReference(ReferenceWithMetaExecution.builder().setValueBuilder(executionReference).build(), _idx);
		}
		
		public LineageBuilder addExecutionReferenceRef(List<Execution> executionReferences) {
			for (Execution toAdd : executionReferences) {
				addExecutionReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addExecutionReferenceRef(Execution executionReference) {
			return addExecutionReferenceRef(executionReference.toBuilder());
		}
		
		public LineageBuilder clearExecutionReference() {
			this.executionReference = null;
			return this;
		}
		public LineageBuilder addInterestRatePayoutReference(ReferenceWithMetaInterestRatePayout interestRatePayoutReference) {
			if(this.interestRatePayoutReference == null){
				this.interestRatePayoutReference = new ArrayList<>();
			}
			this.interestRatePayoutReference.add(interestRatePayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addInterestRatePayoutReference(ReferenceWithMetaInterestRatePayout interestRatePayoutReference, int _idx) {
			if(this.interestRatePayoutReference == null){
				this.interestRatePayoutReference = new ArrayList<>();
			}
			getIndex(this.interestRatePayoutReference, _idx, () -> interestRatePayoutReference.toBuilder());
			this.interestRatePayoutReference.set(_idx, interestRatePayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addInterestRatePayoutReference(List<ReferenceWithMetaInterestRatePayout> interestRatePayoutReferences) {
			if(this.interestRatePayoutReference == null){
				this.interestRatePayoutReference = new ArrayList<>();
			}
			for (ReferenceWithMetaInterestRatePayout toAdd : interestRatePayoutReferences) {
				this.interestRatePayoutReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addInterestRatePayoutReferenceBuilder(ReferenceWithMetaInterestRatePayoutBuilder interestRatePayoutReference) {
			if(this.interestRatePayoutReference == null){
				this.interestRatePayoutReference = new ArrayList<>();
			}
			this.interestRatePayoutReference.add(interestRatePayoutReference);
			return this;
		}
		
		public LineageBuilder addInterestRatePayoutReferenceRef(InterestRatePayout.InterestRatePayoutBuilder interestRatePayoutReference) {
			return addInterestRatePayoutReference(ReferenceWithMetaInterestRatePayout.builder().setValueBuilder(interestRatePayoutReference).build());
		}
		
		public LineageBuilder addInterestRatePayoutReferenceRef(InterestRatePayout.InterestRatePayoutBuilder interestRatePayoutReference, int _idx) {
			return addInterestRatePayoutReference(ReferenceWithMetaInterestRatePayout.builder().setValueBuilder(interestRatePayoutReference).build(), _idx);
		}
		
		public LineageBuilder addInterestRatePayoutReferenceRef(List<InterestRatePayout> interestRatePayoutReferences) {
			for (InterestRatePayout toAdd : interestRatePayoutReferences) {
				addInterestRatePayoutReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addInterestRatePayoutReferenceRef(InterestRatePayout interestRatePayoutReference) {
			return addInterestRatePayoutReferenceRef(interestRatePayoutReference.toBuilder());
		}
		
		public LineageBuilder clearInterestRatePayoutReference() {
			this.interestRatePayoutReference = null;
			return this;
		}
		public LineageBuilder addLegalAgreement(ReferenceWithMetaLegalAgreement legalAgreement) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			this.legalAgreement.add(legalAgreement.toBuilder());
			return this;
		}
		public LineageBuilder addLegalAgreement(ReferenceWithMetaLegalAgreement legalAgreement, int _idx) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			getIndex(this.legalAgreement, _idx, () -> legalAgreement.toBuilder());
			this.legalAgreement.set(_idx, legalAgreement.toBuilder());
			return this;
		}
		public LineageBuilder addLegalAgreement(List<ReferenceWithMetaLegalAgreement> legalAgreements) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			for (ReferenceWithMetaLegalAgreement toAdd : legalAgreements) {
				this.legalAgreement.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addLegalAgreementBuilder(ReferenceWithMetaLegalAgreementBuilder legalAgreement) {
			if(this.legalAgreement == null){
				this.legalAgreement = new ArrayList<>();
			}
			this.legalAgreement.add(legalAgreement);
			return this;
		}
		
		public LineageBuilder addLegalAgreementRef(LegalAgreement.LegalAgreementBuilder legalAgreement) {
			return addLegalAgreement(ReferenceWithMetaLegalAgreement.builder().setValueBuilder(legalAgreement).build());
		}
		
		public LineageBuilder addLegalAgreementRef(LegalAgreement.LegalAgreementBuilder legalAgreement, int _idx) {
			return addLegalAgreement(ReferenceWithMetaLegalAgreement.builder().setValueBuilder(legalAgreement).build(), _idx);
		}
		
		public LineageBuilder addLegalAgreementRef(List<LegalAgreement> legalAgreements) {
			for (LegalAgreement toAdd : legalAgreements) {
				addLegalAgreementRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addLegalAgreementRef(LegalAgreement legalAgreement) {
			return addLegalAgreementRef(legalAgreement.toBuilder());
		}
		
		public LineageBuilder clearLegalAgreement() {
			this.legalAgreement = null;
			return this;
		}
		public LineageBuilder addOptionPayoutReference(ReferenceWithMetaOptionPayout optionPayoutReference) {
			if(this.optionPayoutReference == null){
				this.optionPayoutReference = new ArrayList<>();
			}
			this.optionPayoutReference.add(optionPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addOptionPayoutReference(ReferenceWithMetaOptionPayout optionPayoutReference, int _idx) {
			if(this.optionPayoutReference == null){
				this.optionPayoutReference = new ArrayList<>();
			}
			getIndex(this.optionPayoutReference, _idx, () -> optionPayoutReference.toBuilder());
			this.optionPayoutReference.set(_idx, optionPayoutReference.toBuilder());
			return this;
		}
		public LineageBuilder addOptionPayoutReference(List<ReferenceWithMetaOptionPayout> optionPayoutReferences) {
			if(this.optionPayoutReference == null){
				this.optionPayoutReference = new ArrayList<>();
			}
			for (ReferenceWithMetaOptionPayout toAdd : optionPayoutReferences) {
				this.optionPayoutReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addOptionPayoutReferenceBuilder(ReferenceWithMetaOptionPayoutBuilder optionPayoutReference) {
			if(this.optionPayoutReference == null){
				this.optionPayoutReference = new ArrayList<>();
			}
			this.optionPayoutReference.add(optionPayoutReference);
			return this;
		}
		
		public LineageBuilder addOptionPayoutReferenceRef(OptionPayout.OptionPayoutBuilder optionPayoutReference) {
			return addOptionPayoutReference(ReferenceWithMetaOptionPayout.builder().setValueBuilder(optionPayoutReference).build());
		}
		
		public LineageBuilder addOptionPayoutReferenceRef(OptionPayout.OptionPayoutBuilder optionPayoutReference, int _idx) {
			return addOptionPayoutReference(ReferenceWithMetaOptionPayout.builder().setValueBuilder(optionPayoutReference).build(), _idx);
		}
		
		public LineageBuilder addOptionPayoutReferenceRef(List<OptionPayout> optionPayoutReferences) {
			for (OptionPayout toAdd : optionPayoutReferences) {
				addOptionPayoutReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addOptionPayoutReferenceRef(OptionPayout optionPayoutReference) {
			return addOptionPayoutReferenceRef(optionPayoutReference.toBuilder());
		}
		
		public LineageBuilder clearOptionPayoutReference() {
			this.optionPayoutReference = null;
			return this;
		}
		public LineageBuilder addPortfolioStateReference(ReferenceWithMetaPortfolioState portfolioStateReference) {
			if(this.portfolioStateReference == null){
				this.portfolioStateReference = new ArrayList<>();
			}
			this.portfolioStateReference.add(portfolioStateReference.toBuilder());
			return this;
		}
		public LineageBuilder addPortfolioStateReference(ReferenceWithMetaPortfolioState portfolioStateReference, int _idx) {
			if(this.portfolioStateReference == null){
				this.portfolioStateReference = new ArrayList<>();
			}
			getIndex(this.portfolioStateReference, _idx, () -> portfolioStateReference.toBuilder());
			this.portfolioStateReference.set(_idx, portfolioStateReference.toBuilder());
			return this;
		}
		public LineageBuilder addPortfolioStateReference(List<ReferenceWithMetaPortfolioState> portfolioStateReferences) {
			if(this.portfolioStateReference == null){
				this.portfolioStateReference = new ArrayList<>();
			}
			for (ReferenceWithMetaPortfolioState toAdd : portfolioStateReferences) {
				this.portfolioStateReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addPortfolioStateReferenceBuilder(ReferenceWithMetaPortfolioStateBuilder portfolioStateReference) {
			if(this.portfolioStateReference == null){
				this.portfolioStateReference = new ArrayList<>();
			}
			this.portfolioStateReference.add(portfolioStateReference);
			return this;
		}
		
		public LineageBuilder addPortfolioStateReferenceRef(PortfolioState.PortfolioStateBuilder portfolioStateReference) {
			return addPortfolioStateReference(ReferenceWithMetaPortfolioState.builder().setValueBuilder(portfolioStateReference).build());
		}
		
		public LineageBuilder addPortfolioStateReferenceRef(PortfolioState.PortfolioStateBuilder portfolioStateReference, int _idx) {
			return addPortfolioStateReference(ReferenceWithMetaPortfolioState.builder().setValueBuilder(portfolioStateReference).build(), _idx);
		}
		
		public LineageBuilder addPortfolioStateReferenceRef(List<PortfolioState> portfolioStateReferences) {
			for (PortfolioState toAdd : portfolioStateReferences) {
				addPortfolioStateReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addPortfolioStateReferenceRef(PortfolioState portfolioStateReference) {
			return addPortfolioStateReferenceRef(portfolioStateReference.toBuilder());
		}
		
		public LineageBuilder clearPortfolioStateReference() {
			this.portfolioStateReference = null;
			return this;
		}
		public LineageBuilder addTransferReference(ReferenceWithMetaTransferPrimitive transferReference) {
			if(this.transferReference == null){
				this.transferReference = new ArrayList<>();
			}
			this.transferReference.add(transferReference.toBuilder());
			return this;
		}
		public LineageBuilder addTransferReference(ReferenceWithMetaTransferPrimitive transferReference, int _idx) {
			if(this.transferReference == null){
				this.transferReference = new ArrayList<>();
			}
			getIndex(this.transferReference, _idx, () -> transferReference.toBuilder());
			this.transferReference.set(_idx, transferReference.toBuilder());
			return this;
		}
		public LineageBuilder addTransferReference(List<ReferenceWithMetaTransferPrimitive> transferReferences) {
			if(this.transferReference == null){
				this.transferReference = new ArrayList<>();
			}
			for (ReferenceWithMetaTransferPrimitive toAdd : transferReferences) {
				this.transferReference.add(toAdd.toBuilder());
			}
			return this;
		}
		public LineageBuilder addTransferReferenceBuilder(ReferenceWithMetaTransferPrimitiveBuilder transferReference) {
			if(this.transferReference == null){
				this.transferReference = new ArrayList<>();
			}
			this.transferReference.add(transferReference);
			return this;
		}
		
		public LineageBuilder addTransferReferenceRef(TransferPrimitive.TransferPrimitiveBuilder transferReference) {
			return addTransferReference(ReferenceWithMetaTransferPrimitive.builder().setValueBuilder(transferReference).build());
		}
		
		public LineageBuilder addTransferReferenceRef(TransferPrimitive.TransferPrimitiveBuilder transferReference, int _idx) {
			return addTransferReference(ReferenceWithMetaTransferPrimitive.builder().setValueBuilder(transferReference).build(), _idx);
		}
		
		public LineageBuilder addTransferReferenceRef(List<TransferPrimitive> transferReferences) {
			for (TransferPrimitive toAdd : transferReferences) {
				addTransferReferenceRef(toAdd);
			}
			return this;
		}
		
		public LineageBuilder addTransferReferenceRef(TransferPrimitive transferReference) {
			return addTransferReferenceRef(transferReference.toBuilder());
		}
		
		public LineageBuilder clearTransferReference() {
			this.transferReference = null;
			return this;
		}
		
		public Lineage build() {
			return new Lineage(this);
		}
	
		@Override
		public LineageBuilder prune() {
			if (cashflowReference!=null) cashflowReference = cashflowReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contractReference!=null) contractReference = contractReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (creditDefaultPayoutReference!=null) creditDefaultPayoutReference = creditDefaultPayoutReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (equityPayoutReference!=null) equityPayoutReference = equityPayoutReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (eventReference!=null) eventReference = eventReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (executionReference!=null) executionReference = executionReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (interestRatePayoutReference!=null) interestRatePayoutReference = interestRatePayoutReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (legalAgreement!=null) legalAgreement = legalAgreement.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (optionPayoutReference!=null) optionPayoutReference = optionPayoutReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (portfolioStateReference!=null) portfolioStateReference = portfolioStateReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (transferReference!=null) transferReference = transferReference.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCashflowReference()!=null && getCashflowReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getContractReference()!=null && getContractReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCreditDefaultPayoutReference()!=null && getCreditDefaultPayoutReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEquityPayoutReference()!=null && getEquityPayoutReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getEventReference()!=null && getEventReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getExecutionReference()!=null && getExecutionReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getInterestRatePayoutReference()!=null && getInterestRatePayoutReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getLegalAgreement()!=null && getLegalAgreement().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOptionPayoutReference()!=null && getOptionPayoutReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPortfolioStateReference()!=null && getPortfolioStateReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTransferReference()!=null && getTransferReference().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("cashflowReference"), processor, ReferenceWithMetaCashflow.class, cashflowReference);
			processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
			processRosetta(path.newSubPath("creditDefaultPayoutReference"), processor, ReferenceWithMetaCreditDefaultPayout.class, creditDefaultPayoutReference);
			processRosetta(path.newSubPath("equityPayoutReference"), processor, ReferenceWithMetaEquityPayout.class, equityPayoutReference);
			processRosetta(path.newSubPath("eventReference"), processor, ReferenceWithMetaEvent.class, eventReference);
			processRosetta(path.newSubPath("executionReference"), processor, ReferenceWithMetaExecution.class, executionReference);
			processRosetta(path.newSubPath("interestRatePayoutReference"), processor, ReferenceWithMetaInterestRatePayout.class, interestRatePayoutReference);
			processRosetta(path.newSubPath("legalAgreement"), processor, ReferenceWithMetaLegalAgreement.class, legalAgreement);
			processRosetta(path.newSubPath("optionPayoutReference"), processor, ReferenceWithMetaOptionPayout.class, optionPayoutReference);
			processRosetta(path.newSubPath("portfolioStateReference"), processor, ReferenceWithMetaPortfolioState.class, portfolioStateReference);
			processRosetta(path.newSubPath("transferReference"), processor, ReferenceWithMetaTransferPrimitive.class, transferReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			LineageBuilder _that = (LineageBuilder) o;
		
			if (!ListEquals.listEquals(cashflowReference, _that.cashflowReference)) return false;
			if (!ListEquals.listEquals(contractReference, _that.contractReference)) return false;
			if (!ListEquals.listEquals(creditDefaultPayoutReference, _that.creditDefaultPayoutReference)) return false;
			if (!ListEquals.listEquals(equityPayoutReference, _that.equityPayoutReference)) return false;
			if (!ListEquals.listEquals(eventReference, _that.eventReference)) return false;
			if (!ListEquals.listEquals(executionReference, _that.executionReference)) return false;
			if (!ListEquals.listEquals(interestRatePayoutReference, _that.interestRatePayoutReference)) return false;
			if (!ListEquals.listEquals(legalAgreement, _that.legalAgreement)) return false;
			if (!ListEquals.listEquals(optionPayoutReference, _that.optionPayoutReference)) return false;
			if (!ListEquals.listEquals(portfolioStateReference, _that.portfolioStateReference)) return false;
			if (!ListEquals.listEquals(transferReference, _that.transferReference)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cashflowReference != null ? cashflowReference.hashCode() : 0);
			_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
			_result = 31 * _result + (creditDefaultPayoutReference != null ? creditDefaultPayoutReference.hashCode() : 0);
			_result = 31 * _result + (equityPayoutReference != null ? equityPayoutReference.hashCode() : 0);
			_result = 31 * _result + (eventReference != null ? eventReference.hashCode() : 0);
			_result = 31 * _result + (executionReference != null ? executionReference.hashCode() : 0);
			_result = 31 * _result + (interestRatePayoutReference != null ? interestRatePayoutReference.hashCode() : 0);
			_result = 31 * _result + (legalAgreement != null ? legalAgreement.hashCode() : 0);
			_result = 31 * _result + (optionPayoutReference != null ? optionPayoutReference.hashCode() : 0);
			_result = 31 * _result + (portfolioStateReference != null ? portfolioStateReference.hashCode() : 0);
			_result = 31 * _result + (transferReference != null ? transferReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LineageBuilder {" +
				"cashflowReference=" + this.cashflowReference + ", " +
				"contractReference=" + this.contractReference + ", " +
				"creditDefaultPayoutReference=" + this.creditDefaultPayoutReference + ", " +
				"equityPayoutReference=" + this.equityPayoutReference + ", " +
				"eventReference=" + this.eventReference + ", " +
				"executionReference=" + this.executionReference + ", " +
				"interestRatePayoutReference=" + this.interestRatePayoutReference + ", " +
				"legalAgreement=" + this.legalAgreement + ", " +
				"optionPayoutReference=" + this.optionPayoutReference + ", " +
				"portfolioStateReference=" + this.portfolioStateReference + ", " +
				"transferReference=" + this.transferReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("cashflowReference"), processor, ReferenceWithMetaCashflow.class, cashflowReference);
		processRosetta(path.newSubPath("contractReference"), processor, ReferenceWithMetaContract.class, contractReference);
		processRosetta(path.newSubPath("creditDefaultPayoutReference"), processor, ReferenceWithMetaCreditDefaultPayout.class, creditDefaultPayoutReference);
		processRosetta(path.newSubPath("equityPayoutReference"), processor, ReferenceWithMetaEquityPayout.class, equityPayoutReference);
		processRosetta(path.newSubPath("eventReference"), processor, ReferenceWithMetaEvent.class, eventReference);
		processRosetta(path.newSubPath("executionReference"), processor, ReferenceWithMetaExecution.class, executionReference);
		processRosetta(path.newSubPath("interestRatePayoutReference"), processor, ReferenceWithMetaInterestRatePayout.class, interestRatePayoutReference);
		processRosetta(path.newSubPath("legalAgreement"), processor, ReferenceWithMetaLegalAgreement.class, legalAgreement);
		processRosetta(path.newSubPath("optionPayoutReference"), processor, ReferenceWithMetaOptionPayout.class, optionPayoutReference);
		processRosetta(path.newSubPath("portfolioStateReference"), processor, ReferenceWithMetaPortfolioState.class, portfolioStateReference);
		processRosetta(path.newSubPath("transferReference"), processor, ReferenceWithMetaTransferPrimitive.class, transferReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Lineage _that = (Lineage) o;
	
		if (!ListEquals.listEquals(cashflowReference, _that.cashflowReference)) return false;
		if (!ListEquals.listEquals(contractReference, _that.contractReference)) return false;
		if (!ListEquals.listEquals(creditDefaultPayoutReference, _that.creditDefaultPayoutReference)) return false;
		if (!ListEquals.listEquals(equityPayoutReference, _that.equityPayoutReference)) return false;
		if (!ListEquals.listEquals(eventReference, _that.eventReference)) return false;
		if (!ListEquals.listEquals(executionReference, _that.executionReference)) return false;
		if (!ListEquals.listEquals(interestRatePayoutReference, _that.interestRatePayoutReference)) return false;
		if (!ListEquals.listEquals(legalAgreement, _that.legalAgreement)) return false;
		if (!ListEquals.listEquals(optionPayoutReference, _that.optionPayoutReference)) return false;
		if (!ListEquals.listEquals(portfolioStateReference, _that.portfolioStateReference)) return false;
		if (!ListEquals.listEquals(transferReference, _that.transferReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (cashflowReference != null ? cashflowReference.hashCode() : 0);
		_result = 31 * _result + (contractReference != null ? contractReference.hashCode() : 0);
		_result = 31 * _result + (creditDefaultPayoutReference != null ? creditDefaultPayoutReference.hashCode() : 0);
		_result = 31 * _result + (equityPayoutReference != null ? equityPayoutReference.hashCode() : 0);
		_result = 31 * _result + (eventReference != null ? eventReference.hashCode() : 0);
		_result = 31 * _result + (executionReference != null ? executionReference.hashCode() : 0);
		_result = 31 * _result + (interestRatePayoutReference != null ? interestRatePayoutReference.hashCode() : 0);
		_result = 31 * _result + (legalAgreement != null ? legalAgreement.hashCode() : 0);
		_result = 31 * _result + (optionPayoutReference != null ? optionPayoutReference.hashCode() : 0);
		_result = 31 * _result + (portfolioStateReference != null ? portfolioStateReference.hashCode() : 0);
		_result = 31 * _result + (transferReference != null ? transferReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Lineage {" +
			"cashflowReference=" + this.cashflowReference + ", " +
			"contractReference=" + this.contractReference + ", " +
			"creditDefaultPayoutReference=" + this.creditDefaultPayoutReference + ", " +
			"equityPayoutReference=" + this.equityPayoutReference + ", " +
			"eventReference=" + this.eventReference + ", " +
			"executionReference=" + this.executionReference + ", " +
			"interestRatePayoutReference=" + this.interestRatePayoutReference + ", " +
			"legalAgreement=" + this.legalAgreement + ", " +
			"optionPayoutReference=" + this.optionPayoutReference + ", " +
			"portfolioStateReference=" + this.portfolioStateReference + ", " +
			"transferReference=" + this.transferReference +
		'}';
	}
}
