package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.LoanMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;
		import com.rosetta.model.lib.records.Date;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Loan", source="FpML_5_10")
public class Loan extends IdentifiedProduct {
	private final List<LegalEntity> borrower;
	private final Date creditAgreementDate;
	private final FieldWithMetaString facilityType;
	private final FieldWithMetaString lien;
	private final FieldWithMetaString tranche;
	private static LoanMeta metaData = new LoanMeta();
	
	Loan(LoanBuilder builder) {
		super(builder);
		this.borrower = ofNullable(builder.getBorrower()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.creditAgreementDate = builder.getCreditAgreementDate();
		this.facilityType = ofNullable(builder.getFacilityType()).map(f->f.build()).orElse(null);
		this.lien = ofNullable(builder.getLien()).map(f->f.build()).orElse(null);
		this.tranche = ofNullable(builder.getTranche()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies the borrower. There can be more than one borrower. It is meant to be used in the event that there is no Bloomberg Id or the Secured List isn&#39;t applicable.
	 */
	@RosettaSynonym(value="borrower", source="FpML_5_10")
	@RosettaSynonym(value="borrower", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="borrower", source="DTCC_11_0")
	@RosettaSynonym(value="borrower", source="DTCC_9_0")
	@RosettaSynonym(value="borrower", source="CME_ClearedConfirm_1_17")
	public final List<LegalEntity> getBorrower() {
		return borrower;
	}
	
	/**
	 * The credit agreement date is the closing date (the date where the agreement has been signed) for the loans in the credit agreement. Funding of the facilities occurs on (or sometimes a little after) the Credit Agreement date. This underlier attribute is used to help identify which of the company&#39;s outstanding loans are being referenced by knowing to which credit agreement it belongs. ISDA Standards Terms Supplement term: Date of Original Credit Agreement.
	 */
	@RosettaSynonym(value="creditAgreementDate", source="FpML_5_10")
	@RosettaSynonym(value="creditAgreementDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditAgreementDate", source="DTCC_11_0")
	@RosettaSynonym(value="creditAgreementDate", source="DTCC_9_0")
	@RosettaSynonym(value="creditAgreementDate", source="CME_ClearedConfirm_1_17")
	public final Date getCreditAgreementDate() {
		return creditAgreementDate;
	}
	
	/**
	 * The type of loan facility (letter of credit, revolving, ...).
	 */
	@RosettaSynonym(value="facilityType", source="FpML_5_10")
	@RosettaSynonym(value="facilityType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="facilityType", source="DTCC_11_0")
	@RosettaSynonym(value="facilityType", source="DTCC_9_0")
	@RosettaSynonym(value="facilityType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getFacilityType() {
		return facilityType;
	}
	
	/**
	 * Specifies the seniority level of the lien.
	 */
	@RosettaSynonym(value="lien", source="FpML_5_10")
	@RosettaSynonym(value="lien", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="lien", source="DTCC_11_0")
	@RosettaSynonym(value="lien", source="DTCC_9_0")
	@RosettaSynonym(value="lien", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getLien() {
		return lien;
	}
	
	/**
	 * The loan tranche that is subject to the derivative transaction. It will typically be referenced as the Bloomberg tranche number. ISDA Standards Terms Supplement term: Bloomberg Tranche Number.
	 */
	@RosettaSynonym(value="tranche", source="FpML_5_10")
	@RosettaSynonym(value="tranche", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tranche", source="DTCC_11_0")
	@RosettaSynonym(value="tranche", source="DTCC_9_0")
	@RosettaSynonym(value="tranche", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getTranche() {
		return tranche;
	}
	
	
	@Override
	public RosettaMetaData<? extends Loan> metaData() {
		return metaData;
	} 
	
	public LoanBuilder toBuilder() {
		LoanBuilder builder = new LoanBuilder();
		ofNullable(getBorrower()).ifPresent(borrower -> borrower.forEach(builder::addBorrower));
		ofNullable(getCreditAgreementDate()).ifPresent(builder::setCreditAgreementDate);
		ofNullable(getFacilityType()).ifPresent(builder::setFacilityType);
		ofNullable(getLien()).ifPresent(builder::setLien);
		ofNullable(getTranche()).ifPresent(builder::setTranche);
		ofNullable(getProductIdentifier()).ifPresent(builder::setProductIdentifier);
		return builder;
	}

	public static LoanBuilder builder() {
		return new LoanBuilder();
	}

	public static  class LoanBuilder extends IdentifiedProductBuilder {
	
		protected List<LegalEntity.LegalEntityBuilder> borrower;
		protected Date creditAgreementDate;
		protected FieldWithMetaStringBuilder facilityType;
		protected FieldWithMetaStringBuilder lien;
		protected FieldWithMetaStringBuilder tranche;
	
		public LoanBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Loan> metaData() {
			return metaData;
		} 
	
		
		public List<LegalEntity.LegalEntityBuilder> getBorrower() {
			return borrower;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateBorrower(int index) {
			if (borrower==null) {
				this.borrower = new ArrayList<>();
			}
			return getIndex(borrower, index, ()->new LegalEntity.LegalEntityBuilder());
		}
		
		
		public Date getCreditAgreementDate() {
			return creditAgreementDate;
		}
		
		
		public FieldWithMetaStringBuilder getFacilityType() {
			return facilityType;
		}
		
		public FieldWithMetaStringBuilder getOrCreateFacilityType() {
			if (facilityType!=null) {
				return facilityType;
			}
			else return facilityType = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getLien() {
			return lien;
		}
		
		public FieldWithMetaStringBuilder getOrCreateLien() {
			if (lien!=null) {
				return lien;
			}
			else return lien = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getTranche() {
			return tranche;
		}
		
		public FieldWithMetaStringBuilder getOrCreateTranche() {
			if (tranche!=null) {
				return tranche;
			}
			else return tranche = new FieldWithMetaStringBuilder();
		}
		
	
		public LoanBuilder addBorrower(LegalEntity borrower) {
			if(this.borrower == null){
				this.borrower = new ArrayList<>();
			}
			this.borrower.add(borrower.toBuilder());
			return this;
		}
		public LoanBuilder addBorrower(LegalEntity borrower, int _idx) {
			if(this.borrower == null){
				this.borrower = new ArrayList<>();
			}
			getIndex(this.borrower, _idx, () -> borrower.toBuilder());
			this.borrower.set(_idx, borrower.toBuilder());
			return this;
		}
		public LoanBuilder addBorrower(List<LegalEntity> borrowers) {
			if(this.borrower == null){
				this.borrower = new ArrayList<>();
			}
			for (LegalEntity toAdd : borrowers) {
				this.borrower.add(toAdd.toBuilder());
			}
			return this;
		}
		public LoanBuilder addBorrowerBuilder(LegalEntity.LegalEntityBuilder borrower) {
			if(this.borrower == null){
				this.borrower = new ArrayList<>();
			}
			this.borrower.add(borrower);
			return this;
		}
		
		public LoanBuilder clearBorrower() {
			this.borrower = null;
			return this;
		}
		public LoanBuilder setCreditAgreementDate(Date creditAgreementDate) {
			this.creditAgreementDate = creditAgreementDate;
			return this;
		}
		public LoanBuilder setFacilityType(FieldWithMetaString facilityType) {
			this.facilityType = facilityType.toBuilder();
			return this;
		}
		public LoanBuilder setFacilityTypeRef(String facilityType) {
			return setFacilityType(FieldWithMetaString.builder().setValue(facilityType).build());
		}
		public LoanBuilder setLien(FieldWithMetaString lien) {
			this.lien = lien.toBuilder();
			return this;
		}
		public LoanBuilder setLienRef(String lien) {
			return setLien(FieldWithMetaString.builder().setValue(lien).build());
		}
		public LoanBuilder setTranche(FieldWithMetaString tranche) {
			this.tranche = tranche.toBuilder();
			return this;
		}
		public LoanBuilder setTrancheRef(String tranche) {
			return setTranche(FieldWithMetaString.builder().setValue(tranche).build());
		}
		@Override public LoanBuilder setProductIdentifier(ProductIdentifier productIdentifier) {
			this.productIdentifier = productIdentifier.toBuilder();
			return this;
		}
		@Override public LoanBuilder setProductIdentifierBuilder(ProductIdentifier.ProductIdentifierBuilder productIdentifier) {
			this.productIdentifier = productIdentifier;
			return this;
		}
		
		public Loan build() {
			return new Loan(this);
		}
	
		@Override
		public LoanBuilder prune() {
			super.prune();
			if (borrower!=null) borrower = borrower.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (facilityType!=null && !facilityType.prune().hasData()) facilityType = null;
			if (lien!=null && !lien.prune().hasData()) lien = null;
			if (tranche!=null && !tranche.prune().hasData()) tranche = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBorrower()!=null && getBorrower().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCreditAgreementDate()!=null) return true;
			if (getFacilityType()!=null) return true;
			if (getLien()!=null) return true;
			if (getTranche()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("creditAgreementDate"), Date.class, creditAgreementDate, this);
			
			processRosetta(path.newSubPath("borrower"), processor, LegalEntity.class, borrower);
			processRosetta(path.newSubPath("facilityType"), processor, FieldWithMetaString.class, facilityType);
			processRosetta(path.newSubPath("lien"), processor, FieldWithMetaString.class, lien);
			processRosetta(path.newSubPath("tranche"), processor, FieldWithMetaString.class, tranche);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			LoanBuilder _that = (LoanBuilder) o;
		
			if (!ListEquals.listEquals(borrower, _that.borrower)) return false;
			if (creditAgreementDate != null ? !creditAgreementDate.equals(_that.creditAgreementDate) : _that.creditAgreementDate != null) return false;
			if (facilityType != null ? !facilityType.equals(_that.facilityType) : _that.facilityType != null) return false;
			if (lien != null ? !lien.equals(_that.lien) : _that.lien != null) return false;
			if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (borrower != null ? borrower.hashCode() : 0);
			_result = 31 * _result + (creditAgreementDate != null ? creditAgreementDate.hashCode() : 0);
			_result = 31 * _result + (facilityType != null ? facilityType.hashCode() : 0);
			_result = 31 * _result + (lien != null ? lien.hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "LoanBuilder {" +
				"borrower=" + this.borrower + ", " +
				"creditAgreementDate=" + this.creditAgreementDate + ", " +
				"facilityType=" + this.facilityType + ", " +
				"lien=" + this.lien + ", " +
				"tranche=" + this.tranche +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("creditAgreementDate"), Date.class, creditAgreementDate, this);
		
		processRosetta(path.newSubPath("borrower"), processor, LegalEntity.class, borrower);
		processRosetta(path.newSubPath("facilityType"), processor, FieldWithMetaString.class, facilityType);
		processRosetta(path.newSubPath("lien"), processor, FieldWithMetaString.class, lien);
		processRosetta(path.newSubPath("tranche"), processor, FieldWithMetaString.class, tranche);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		Loan _that = (Loan) o;
	
		if (!ListEquals.listEquals(borrower, _that.borrower)) return false;
		if (creditAgreementDate != null ? !creditAgreementDate.equals(_that.creditAgreementDate) : _that.creditAgreementDate != null) return false;
		if (facilityType != null ? !facilityType.equals(_that.facilityType) : _that.facilityType != null) return false;
		if (lien != null ? !lien.equals(_that.lien) : _that.lien != null) return false;
		if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (borrower != null ? borrower.hashCode() : 0);
		_result = 31 * _result + (creditAgreementDate != null ? creditAgreementDate.hashCode() : 0);
		_result = 31 * _result + (facilityType != null ? facilityType.hashCode() : 0);
		_result = 31 * _result + (lien != null ? lien.hashCode() : 0);
		_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Loan {" +
			"borrower=" + this.borrower + ", " +
			"creditAgreementDate=" + this.creditAgreementDate + ", " +
			"facilityType=" + this.facilityType + ", " +
			"lien=" + this.lien + ", " +
			"tranche=" + this.tranche +
		'}' + " " + super.toString();
	}
}
