package org.isda.cdm;

import com.rosetta.model.lib.records.Date;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MandatoryEarlyTerminationAdjustedDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the adjusted dates associated with a mandatory early termination provision.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MandatoryEarlyTerminationAdjustedDates", source="FpML_5_10")
public class MandatoryEarlyTerminationAdjustedDates extends RosettaModelObject {
	private final Date adjustedCashSettlementPaymentDate;
	private final Date adjustedCashSettlementValuationDate;
	private final Date adjustedEarlyTerminationDate;
	private static MandatoryEarlyTerminationAdjustedDatesMeta metaData = new MandatoryEarlyTerminationAdjustedDatesMeta();
	
	MandatoryEarlyTerminationAdjustedDates(MandatoryEarlyTerminationAdjustedDatesBuilder builder) {
		this.adjustedCashSettlementPaymentDate = builder.getAdjustedCashSettlementPaymentDate();
		this.adjustedCashSettlementValuationDate = builder.getAdjustedCashSettlementValuationDate();
		this.adjustedEarlyTerminationDate = builder.getAdjustedEarlyTerminationDate();
	}
	
	/**
	 * The date on which the cash settlement amount is paid. This date should already be adjusted for any applicable business date convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementPaymentDate() {
		return adjustedCashSettlementPaymentDate;
	}
	
	/**
	 * The date by which the cash settlement amount must be agreed. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedCashSettlementValuationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedCashSettlementValuationDate() {
		return adjustedCashSettlementValuationDate;
	}
	
	/**
	 * The early termination date that is applicable if an early termination provision is exercised. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedEarlyTerminationDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedEarlyTerminationDate() {
		return adjustedEarlyTerminationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends MandatoryEarlyTerminationAdjustedDates> metaData() {
		return metaData;
	} 
	
	public MandatoryEarlyTerminationAdjustedDatesBuilder toBuilder() {
		MandatoryEarlyTerminationAdjustedDatesBuilder builder = new MandatoryEarlyTerminationAdjustedDatesBuilder();
		ofNullable(getAdjustedCashSettlementPaymentDate()).ifPresent(builder::setAdjustedCashSettlementPaymentDate);
		ofNullable(getAdjustedCashSettlementValuationDate()).ifPresent(builder::setAdjustedCashSettlementValuationDate);
		ofNullable(getAdjustedEarlyTerminationDate()).ifPresent(builder::setAdjustedEarlyTerminationDate);
		return builder;
	}

	public static MandatoryEarlyTerminationAdjustedDatesBuilder builder() {
		return new MandatoryEarlyTerminationAdjustedDatesBuilder();
	}

	public static  class MandatoryEarlyTerminationAdjustedDatesBuilder extends RosettaModelObjectBuilder {
	
		protected Date adjustedCashSettlementPaymentDate;
		protected Date adjustedCashSettlementValuationDate;
		protected Date adjustedEarlyTerminationDate;
	
		public MandatoryEarlyTerminationAdjustedDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MandatoryEarlyTerminationAdjustedDates> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedCashSettlementPaymentDate() {
			return adjustedCashSettlementPaymentDate;
		}
		
		
		public Date getAdjustedCashSettlementValuationDate() {
			return adjustedCashSettlementValuationDate;
		}
		
		
		public Date getAdjustedEarlyTerminationDate() {
			return adjustedEarlyTerminationDate;
		}
		
	
		public MandatoryEarlyTerminationAdjustedDatesBuilder setAdjustedCashSettlementPaymentDate(Date adjustedCashSettlementPaymentDate) {
			this.adjustedCashSettlementPaymentDate = adjustedCashSettlementPaymentDate;
			return this;
		}
		public MandatoryEarlyTerminationAdjustedDatesBuilder setAdjustedCashSettlementValuationDate(Date adjustedCashSettlementValuationDate) {
			this.adjustedCashSettlementValuationDate = adjustedCashSettlementValuationDate;
			return this;
		}
		public MandatoryEarlyTerminationAdjustedDatesBuilder setAdjustedEarlyTerminationDate(Date adjustedEarlyTerminationDate) {
			this.adjustedEarlyTerminationDate = adjustedEarlyTerminationDate;
			return this;
		}
		
		public MandatoryEarlyTerminationAdjustedDates build() {
			return new MandatoryEarlyTerminationAdjustedDates(this);
		}
	
		@Override
		public MandatoryEarlyTerminationAdjustedDatesBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedCashSettlementPaymentDate()!=null) return true;
			if (getAdjustedCashSettlementValuationDate()!=null) return true;
			if (getAdjustedEarlyTerminationDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
			processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
			processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MandatoryEarlyTerminationAdjustedDatesBuilder _that = (MandatoryEarlyTerminationAdjustedDatesBuilder) o;
		
			if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
			if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
			if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
			_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
			_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MandatoryEarlyTerminationAdjustedDatesBuilder {" +
				"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
				"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
				"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedCashSettlementPaymentDate"), Date.class, adjustedCashSettlementPaymentDate, this);
		processor.processBasic(path.newSubPath("adjustedCashSettlementValuationDate"), Date.class, adjustedCashSettlementValuationDate, this);
		processor.processBasic(path.newSubPath("adjustedEarlyTerminationDate"), Date.class, adjustedEarlyTerminationDate, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MandatoryEarlyTerminationAdjustedDates _that = (MandatoryEarlyTerminationAdjustedDates) o;
	
		if (adjustedCashSettlementPaymentDate != null ? !adjustedCashSettlementPaymentDate.equals(_that.adjustedCashSettlementPaymentDate) : _that.adjustedCashSettlementPaymentDate != null) return false;
		if (adjustedCashSettlementValuationDate != null ? !adjustedCashSettlementValuationDate.equals(_that.adjustedCashSettlementValuationDate) : _that.adjustedCashSettlementValuationDate != null) return false;
		if (adjustedEarlyTerminationDate != null ? !adjustedEarlyTerminationDate.equals(_that.adjustedEarlyTerminationDate) : _that.adjustedEarlyTerminationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedCashSettlementPaymentDate != null ? adjustedCashSettlementPaymentDate.hashCode() : 0);
		_result = 31 * _result + (adjustedCashSettlementValuationDate != null ? adjustedCashSettlementValuationDate.hashCode() : 0);
		_result = 31 * _result + (adjustedEarlyTerminationDate != null ? adjustedEarlyTerminationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MandatoryEarlyTerminationAdjustedDates {" +
			"adjustedCashSettlementPaymentDate=" + this.adjustedCashSettlementPaymentDate + ", " +
			"adjustedCashSettlementValuationDate=" + this.adjustedCashSettlementValuationDate + ", " +
			"adjustedEarlyTerminationDate=" + this.adjustedEarlyTerminationDate +
		'}';
	}
}
