package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * This indicator defines which type of assets (cash or securities) is specified to apply as margin to the repo transaction.
 * 
 * @version 2.5.4
 */
public enum MarginTypeEnum {
    

	/**
	 * When the margin type is Cash, the margin factor is applied to the cash value of the transaction.
	 */
	@RosettaSynonym(value = "Cash", source = "FpML_5_10")
	CASH,
	    

	/**
	 * When the margin type is Instrument, the margin factor is applied to the instrument value for the transaction. In the “instrument” case, the haircut would be applied to the securities.
	 */
	@RosettaSynonym(value = "Instrument", source = "FpML_5_10")
	INSTRUMENT    
	;

	private final String displayName;
	
	MarginTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
