package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the handling of an averaging date market disruption for an equity derivative transaction.
 * 
 * @version 2.5.4
 */
public enum MarketDisruptionEnum {
    

	/**
	 * As defined in section 6.7 paragraph (c) sub-paragraph (iii) of the ISDA 2002 Equity Derivative definitions.
	 */
	@RosettaSynonym(value = "ModifiedPostponement", source = "FpML_5_10")
	@RosettaSynonym(value = "ModifiedPostponement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ModifiedPostponement", source = "DTCC_11_0")
	@RosettaSynonym(value = "ModifiedPostponement", source = "DTCC_9_0")
	@RosettaSynonym(value = "ModifiedPostponement", source = "CME_ClearedConfirm_1_17")
	MODIFIED_POSTPONEMENT,
	    

	/**
	 * As defined in section 6.7 paragraph (c) sub-paragraph (i) of the ISDA 2002 Equity Derivative definitions.
	 */
	@RosettaSynonym(value = "Omission", source = "FpML_5_10")
	@RosettaSynonym(value = "Omission", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Omission", source = "DTCC_11_0")
	@RosettaSynonym(value = "Omission", source = "DTCC_9_0")
	@RosettaSynonym(value = "Omission", source = "CME_ClearedConfirm_1_17")
	OMISSION,
	    

	/**
	 * As defined in section 6.7 paragraph (c) sub-paragraph (ii) of the ISDA 2002 Equity Derivative definitions.
	 */
	@RosettaSynonym(value = "Postponement", source = "FpML_5_10")
	@RosettaSynonym(value = "Postponement", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Postponement", source = "DTCC_11_0")
	@RosettaSynonym(value = "Postponement", source = "DTCC_9_0")
	@RosettaSynonym(value = "Postponement", source = "CME_ClearedConfirm_1_17")
	POSTPONEMENT    
	;

	private final String displayName;
	
	MarketDisruptionEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
