package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaMasterAgreementTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaMasterAgreementTypeEnum.FieldWithMetaMasterAgreementTypeEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MasterAgreementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining the agreement executed between the parties and intended to govern all OTC derivatives transactions between those parties.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MasterAgreement", source="FpML_5_10")
public class MasterAgreement extends RosettaModelObject {
	private final Date masterAgreementDate;
	private final FieldWithMetaString masterAgreementId;
	private final FieldWithMetaMasterAgreementTypeEnum masterAgreementType;
	private final FieldWithMetaString masterAgreementVersion;
	private static MasterAgreementMeta metaData = new MasterAgreementMeta();
	
	MasterAgreement(MasterAgreementBuilder builder) {
		this.masterAgreementDate = builder.getMasterAgreementDate();
		this.masterAgreementId = ofNullable(builder.getMasterAgreementId()).map(f->f.build()).orElse(null);
		this.masterAgreementType = ofNullable(builder.getMasterAgreementType()).map(f->f.build()).orElse(null);
		this.masterAgreementVersion = ofNullable(builder.getMasterAgreementVersion()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the master agreement was signed.
	 */
	@RosettaSynonym(value="masterAgreementDate", source="FpML_5_10")
	@RosettaSynonym(value="masterAgreementDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterAgreementDate", source="DTCC_11_0")
	@RosettaSynonym(value="masterAgreementDate", source="DTCC_9_0")
	@RosettaSynonym(value="masterAgreementDate", source="CME_ClearedConfirm_1_17")
	public final Date getMasterAgreementDate() {
		return masterAgreementDate;
	}
	
	/**
	 * An identifier that has been created to identify the master agreement.
	 */
	@RosettaSynonym(value="masterAgreementId", source="FpML_5_10")
	@RosettaSynonym(value="masterAgreementId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterAgreementId", source="DTCC_11_0")
	@RosettaSynonym(value="masterAgreementId", source="DTCC_9_0")
	@RosettaSynonym(value="masterAgreementId", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getMasterAgreementId() {
		return masterAgreementId;
	}
	
	/**
	 * The agreement executed between the parties and intended to govern product-specific derivatives transactions between those parties.
	 */
	@RosettaSynonym(value="masterAgreementType", source="FpML_5_10")
	@RosettaSynonym(value="masterAgreementType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterAgreementType", source="DTCC_11_0")
	@RosettaSynonym(value="masterAgreementType", source="DTCC_9_0")
	@RosettaSynonym(value="masterAgreementType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMasterAgreementTypeEnum getMasterAgreementType() {
		return masterAgreementType;
	}
	
	/**
	 * The version of the master agreement.
	 */
	@RosettaSynonym(value="masterAgreementVersion", source="FpML_5_10")
	@RosettaSynonym(value="masterAgreementVersion", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterAgreementVersion", source="DTCC_11_0")
	@RosettaSynonym(value="masterAgreementVersion", source="DTCC_9_0")
	@RosettaSynonym(value="masterAgreementVersion", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getMasterAgreementVersion() {
		return masterAgreementVersion;
	}
	
	
	@Override
	public RosettaMetaData<? extends MasterAgreement> metaData() {
		return metaData;
	} 
	
	public MasterAgreementBuilder toBuilder() {
		MasterAgreementBuilder builder = new MasterAgreementBuilder();
		ofNullable(getMasterAgreementDate()).ifPresent(builder::setMasterAgreementDate);
		ofNullable(getMasterAgreementId()).ifPresent(builder::setMasterAgreementId);
		ofNullable(getMasterAgreementType()).ifPresent(builder::setMasterAgreementType);
		ofNullable(getMasterAgreementVersion()).ifPresent(builder::setMasterAgreementVersion);
		return builder;
	}

	public static MasterAgreementBuilder builder() {
		return new MasterAgreementBuilder();
	}

	public static  class MasterAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected Date masterAgreementDate;
		protected FieldWithMetaStringBuilder masterAgreementId;
		protected FieldWithMetaMasterAgreementTypeEnumBuilder masterAgreementType;
		protected FieldWithMetaStringBuilder masterAgreementVersion;
	
		public MasterAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MasterAgreement> metaData() {
			return metaData;
		} 
	
		
		public Date getMasterAgreementDate() {
			return masterAgreementDate;
		}
		
		
		public FieldWithMetaStringBuilder getMasterAgreementId() {
			return masterAgreementId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMasterAgreementId() {
			if (masterAgreementId!=null) {
				return masterAgreementId;
			}
			else return masterAgreementId = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaMasterAgreementTypeEnumBuilder getMasterAgreementType() {
			return masterAgreementType;
		}
		
		public FieldWithMetaMasterAgreementTypeEnumBuilder getOrCreateMasterAgreementType() {
			if (masterAgreementType!=null) {
				return masterAgreementType;
			}
			else return masterAgreementType = new FieldWithMetaMasterAgreementTypeEnumBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getMasterAgreementVersion() {
			return masterAgreementVersion;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMasterAgreementVersion() {
			if (masterAgreementVersion!=null) {
				return masterAgreementVersion;
			}
			else return masterAgreementVersion = new FieldWithMetaStringBuilder();
		}
		
	
		public MasterAgreementBuilder setMasterAgreementDate(Date masterAgreementDate) {
			this.masterAgreementDate = masterAgreementDate;
			return this;
		}
		public MasterAgreementBuilder setMasterAgreementId(FieldWithMetaString masterAgreementId) {
			this.masterAgreementId = masterAgreementId.toBuilder();
			return this;
		}
		public MasterAgreementBuilder setMasterAgreementIdRef(String masterAgreementId) {
			return setMasterAgreementId(FieldWithMetaString.builder().setValue(masterAgreementId).build());
		}
		public MasterAgreementBuilder setMasterAgreementType(FieldWithMetaMasterAgreementTypeEnum masterAgreementType) {
			this.masterAgreementType = masterAgreementType.toBuilder();
			return this;
		}
		public MasterAgreementBuilder setMasterAgreementTypeRef(MasterAgreementTypeEnum masterAgreementType) {
			return setMasterAgreementType(FieldWithMetaMasterAgreementTypeEnum.builder().setValue(masterAgreementType).build());
		}
		public MasterAgreementBuilder setMasterAgreementVersion(FieldWithMetaString masterAgreementVersion) {
			this.masterAgreementVersion = masterAgreementVersion.toBuilder();
			return this;
		}
		public MasterAgreementBuilder setMasterAgreementVersionRef(String masterAgreementVersion) {
			return setMasterAgreementVersion(FieldWithMetaString.builder().setValue(masterAgreementVersion).build());
		}
		
		public MasterAgreement build() {
			return new MasterAgreement(this);
		}
	
		@Override
		public MasterAgreementBuilder prune() {
			if (masterAgreementId!=null && !masterAgreementId.prune().hasData()) masterAgreementId = null;
			if (masterAgreementType!=null && !masterAgreementType.prune().hasData()) masterAgreementType = null;
			if (masterAgreementVersion!=null && !masterAgreementVersion.prune().hasData()) masterAgreementVersion = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMasterAgreementDate()!=null) return true;
			if (getMasterAgreementId()!=null) return true;
			if (getMasterAgreementType()!=null) return true;
			if (getMasterAgreementVersion()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("masterAgreementDate"), Date.class, masterAgreementDate, this);
			
			processRosetta(path.newSubPath("masterAgreementId"), processor, FieldWithMetaString.class, masterAgreementId);
			processRosetta(path.newSubPath("masterAgreementType"), processor, FieldWithMetaMasterAgreementTypeEnum.class, masterAgreementType);
			processRosetta(path.newSubPath("masterAgreementVersion"), processor, FieldWithMetaString.class, masterAgreementVersion);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MasterAgreementBuilder _that = (MasterAgreementBuilder) o;
		
			if (masterAgreementDate != null ? !masterAgreementDate.equals(_that.masterAgreementDate) : _that.masterAgreementDate != null) return false;
			if (masterAgreementId != null ? !masterAgreementId.equals(_that.masterAgreementId) : _that.masterAgreementId != null) return false;
			if (masterAgreementType != null ? !masterAgreementType.equals(_that.masterAgreementType) : _that.masterAgreementType != null) return false;
			if (masterAgreementVersion != null ? !masterAgreementVersion.equals(_that.masterAgreementVersion) : _that.masterAgreementVersion != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (masterAgreementDate != null ? masterAgreementDate.hashCode() : 0);
			_result = 31 * _result + (masterAgreementId != null ? masterAgreementId.hashCode() : 0);
			_result = 31 * _result + (masterAgreementType != null ? masterAgreementType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (masterAgreementVersion != null ? masterAgreementVersion.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MasterAgreementBuilder {" +
				"masterAgreementDate=" + this.masterAgreementDate + ", " +
				"masterAgreementId=" + this.masterAgreementId + ", " +
				"masterAgreementType=" + this.masterAgreementType + ", " +
				"masterAgreementVersion=" + this.masterAgreementVersion +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("masterAgreementDate"), Date.class, masterAgreementDate, this);
		
		processRosetta(path.newSubPath("masterAgreementId"), processor, FieldWithMetaString.class, masterAgreementId);
		processRosetta(path.newSubPath("masterAgreementType"), processor, FieldWithMetaMasterAgreementTypeEnum.class, masterAgreementType);
		processRosetta(path.newSubPath("masterAgreementVersion"), processor, FieldWithMetaString.class, masterAgreementVersion);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MasterAgreement _that = (MasterAgreement) o;
	
		if (masterAgreementDate != null ? !masterAgreementDate.equals(_that.masterAgreementDate) : _that.masterAgreementDate != null) return false;
		if (masterAgreementId != null ? !masterAgreementId.equals(_that.masterAgreementId) : _that.masterAgreementId != null) return false;
		if (masterAgreementType != null ? !masterAgreementType.equals(_that.masterAgreementType) : _that.masterAgreementType != null) return false;
		if (masterAgreementVersion != null ? !masterAgreementVersion.equals(_that.masterAgreementVersion) : _that.masterAgreementVersion != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (masterAgreementDate != null ? masterAgreementDate.hashCode() : 0);
		_result = 31 * _result + (masterAgreementId != null ? masterAgreementId.hashCode() : 0);
		_result = 31 * _result + (masterAgreementType != null ? masterAgreementType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (masterAgreementVersion != null ? masterAgreementVersion.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MasterAgreement {" +
			"masterAgreementDate=" + this.masterAgreementDate + ", " +
			"masterAgreementId=" + this.masterAgreementId + ", " +
			"masterAgreementType=" + this.masterAgreementType + ", " +
			"masterAgreementVersion=" + this.masterAgreementVersion +
		'}';
	}
}
