package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of the master agreement governing the transaction.
 * 
 * @version 2.5.4
 */
public enum MasterAgreementTypeEnum {
    

	/**
	 * AFB Master Agreement for Foreign Exchange and Derivatives Transactions
	 */
	@RosettaSynonym(value = "AFB", source = "FpML_5_10")
	@RosettaSynonym(value = "AFB", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AFB", source = "DTCC_11_0")
	@RosettaSynonym(value = "AFB", source = "DTCC_9_0")
	@RosettaSynonym(value = "AFB", source = "CME_ClearedConfirm_1_17")
	AFB,
	    

	/**
	 * A Bespoke (custom) Master Agreement, including one-off agreements for transactions
	 */
	@RosettaSynonym(value = "Bespoke", source = "FpML_5_10")
	@RosettaSynonym(value = "Bespoke", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Bespoke", source = "DTCC_11_0")
	@RosettaSynonym(value = "Bespoke", source = "DTCC_9_0")
	@RosettaSynonym(value = "Bespoke", source = "CME_ClearedConfirm_1_17")
	BESPOKE,
	    

	/**
	 * Clearing Master Agreement
	 */
	@RosettaSynonym(value = "CMA", source = "FpML_5_10")
	@RosettaSynonym(value = "CMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CMA", source = "CME_ClearedConfirm_1_17")
	CMA,
	    

	/**
	 * Contrato Marco de Operaciones Financieras
	 */
	@RosettaSynonym(value = "CMOF", source = "FpML_5_10")
	@RosettaSynonym(value = "CMOF", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CMOF", source = "DTCC_11_0")
	@RosettaSynonym(value = "CMOF", source = "DTCC_9_0")
	@RosettaSynonym(value = "CMOF", source = "CME_ClearedConfirm_1_17")
	CMOF,
	    

	/**
	 * EEI Master Power Purchase and Sale Agreement
	 */
	@RosettaSynonym(value = "EEIPower", source = "FpML_5_10")
	@RosettaSynonym(value = "EEIPower", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EEIPower", source = "DTCC_11_0")
	@RosettaSynonym(value = "EEIPower", source = "DTCC_9_0")
	@RosettaSynonym(value = "EEIPower", source = "CME_ClearedConfirm_1_17")
	EEI_POWER,
	    

	/**
	 * EFET General Agreement Concerning the Delivery and Acceptance of Electricity
	 */
	@RosettaSynonym(value = "EFETElectricity", source = "FpML_5_10")
	@RosettaSynonym(value = "EFETElectricity", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EFETElectricity", source = "DTCC_11_0")
	@RosettaSynonym(value = "EFETElectricity", source = "DTCC_9_0")
	@RosettaSynonym(value = "EFETElectricity", source = "CME_ClearedConfirm_1_17")
	EFET_ELECTRICITY,
	    

	/**
	 * EFET General Agreement Concerning The Delivery And Acceptance of Natural Gas
	 */
	@RosettaSynonym(value = "EFETGas", source = "FpML_5_10")
	@RosettaSynonym(value = "EFETGas", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EFETGas", source = "DTCC_11_0")
	@RosettaSynonym(value = "EFETGas", source = "DTCC_9_0")
	@RosettaSynonym(value = "EFETGas", source = "CME_ClearedConfirm_1_17")
	EFET_GAS,
	    

	/**
	 * European Master Agreement and the Derivatives Annex (Banking Federation of the European Union)
	 */
	@RosettaSynonym(value = "EMA", source = "FpML_5_10")
	@RosettaSynonym(value = "EMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "EMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "EMA", source = "CME_ClearedConfirm_1_17")
	EMA,
	    

	/**
	 * Master Agreement Relating to transactions on Forward Financial Instruments (Federation Bancaire Francaise)
	 */
	@RosettaSynonym(value = "FBF", source = "FpML_5_10")
	@RosettaSynonym(value = "FBF", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FBF", source = "DTCC_11_0")
	@RosettaSynonym(value = "FBF", source = "DTCC_9_0")
	@RosettaSynonym(value = "FBF", source = "CME_ClearedConfirm_1_17")
	FBF,
	    

	/**
	 * ICMA Global Master Agreement for REPO Trades
	 */
	@RosettaSynonym(value = "GMRA", source = "FpML_5_10")
	@RosettaSynonym(value = "GMRA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GMRA", source = "DTCC_11_0")
	@RosettaSynonym(value = "GMRA", source = "DTCC_9_0")
	@RosettaSynonym(value = "GMRA", source = "CME_ClearedConfirm_1_17")
	GMRA,
	    

	/**
	 * ISLA Global Master Agreement for Securities Lending
	 */
	@RosettaSynonym(value = "GMSLA", source = "FpML_5_10")
	@RosettaSynonym(value = "GMSLA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GMSLA", source = "DTCC_11_0")
	@RosettaSynonym(value = "GMSLA", source = "DTCC_9_0")
	@RosettaSynonym(value = "GMSLA", source = "CME_ClearedConfirm_1_17")
	GMSLA,
	    

	/**
	 * FOA Grid Trade Master Agreement
	 */
	@RosettaSynonym(value = "GTMA", source = "FpML_5_10")
	@RosettaSynonym(value = "GTMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GTMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "GTMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "GTMA", source = "CME_ClearedConfirm_1_17")
	GTMA,
	    

	/**
	 * GasEDI Base Contract for Short-term Sale and Purchase of Natural Gas
	 */
	@RosettaSynonym(value = "GasEDI", source = "FpML_5_10")
	@RosettaSynonym(value = "GasEDI", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GasEDI", source = "DTCC_11_0")
	@RosettaSynonym(value = "GasEDI", source = "DTCC_9_0")
	@RosettaSynonym(value = "GasEDI", source = "CME_ClearedConfirm_1_17")
	GAS_EDI,
	    

	/**
	 * German Master Agreement for Financial derivatives and Addendum for Options on Stock Exchange Indices or Securities
	 */
	@RosettaSynonym(value = "German", source = "FpML_5_10")
	@RosettaSynonym(value = "German", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "German", source = "DTCC_11_0")
	@RosettaSynonym(value = "German", source = "DTCC_9_0")
	@RosettaSynonym(value = "German", source = "CME_ClearedConfirm_1_17")
	GERMAN,
	    

	/**
	 * International Currency Options Market Master Agreement
	 */
	@RosettaSynonym(value = "ICOM", source = "FpML_5_10")
	@RosettaSynonym(value = "ICOM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ICOM", source = "DTCC_11_0")
	@RosettaSynonym(value = "ICOM", source = "DTCC_9_0")
	@RosettaSynonym(value = "ICOM", source = "CME_ClearedConfirm_1_17")
	ICOM,
	    

	/**
	 * International Emissions Trading Association Emissions Reduction Purchase Agreement
	 */
	@RosettaSynonym(value = "IETA-ERPA", source = "FpML_5_10")
	@RosettaSynonym(value = "IETA-ERPA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IETA-ERPA", source = "DTCC_11_0")
	@RosettaSynonym(value = "IETA-ERPA", source = "DTCC_9_0")
	@RosettaSynonym(value = "IETA-ERPA", source = "CME_ClearedConfirm_1_17")
	IETA_ERPA("IETA-ERPA"),
	

	/**
	 * International Emissions Trading Association Emissions Trading Master Agreement
	 */
	@RosettaSynonym(value = "IETA-ETMA", source = "FpML_5_10")
	@RosettaSynonym(value = "IETA-ETMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IETA-ETMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "IETA-ETMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "IETA-ETMA", source = "CME_ClearedConfirm_1_17")
	IETA_ETMA("IETA-ETMA"),
	

	/**
	 * International Emissions Trading Association International Emissions Trading Master Agreement
	 */
	@RosettaSynonym(value = "IETA-IETMA", source = "FpML_5_10")
	@RosettaSynonym(value = "IETA-IETMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IETA-IETMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "IETA-IETMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "IETA-IETMA", source = "CME_ClearedConfirm_1_17")
	IETA_IETMA("IETA-IETMA"),
	

	/**
	 * International Foreign Exchange Master Agreement
	 */
	@RosettaSynonym(value = "IFEMA", source = "FpML_5_10")
	@RosettaSynonym(value = "IFEMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IFEMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "IFEMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "IFEMA", source = "CME_ClearedConfirm_1_17")
	IFEMA,
	    

	/**
	 * International Foreign Exchange and Options Master Agreement
	 */
	@RosettaSynonym(value = "IFEOMA", source = "FpML_5_10")
	@RosettaSynonym(value = "IFEOMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IFEOMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "IFEOMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "IFEOMA", source = "CME_ClearedConfirm_1_17")
	IFEOMA,
	    

	/**
	 * ISDA Master Agreement
	 */
	@RosettaSynonym(value = "ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDA", source = "CME_ClearedConfirm_1_17")
	ISDA,
	    

	/**
	 * ISDA-FIA Cleared Derivatives Execution Agreement
	 */
	@RosettaSynonym(value = "ISDAFIA-CDEA", source = "FpML_5_10")
	@RosettaSynonym(value = "ISDAFIA-CDEA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISDAFIA-CDEA", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISDAFIA-CDEA", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISDAFIA-CDEA", source = "CME_ClearedConfirm_1_17")
	ISDAFIA_CDEA("ISDAFIA-CDEA"),
	

	/**
	 * Master agreement of Japan Securities Clearing Corporation
	 */
	@RosettaSynonym(value = "JSCC", source = "FpML_5_10")
	@RosettaSynonym(value = "JSCC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JSCC", source = "DTCC_11_0")
	@RosettaSynonym(value = "JSCC", source = "DTCC_9_0")
	@RosettaSynonym(value = "JSCC", source = "CME_ClearedConfirm_1_17")
	JSCC,
	    

	/**
	 * International Bullion Master Agreement Terms published by the London Bullion Market Association
	 */
	@RosettaSynonym(value = "LBMA", source = "FpML_5_10")
	@RosettaSynonym(value = "LBMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LBMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "LBMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "LBMA", source = "CME_ClearedConfirm_1_17")
	LBMA,
	    

	/**
	 * Leadership in Energy Automated Processing
	 */
	@RosettaSynonym(value = "LEAP", source = "FpML_5_10")
	@RosettaSynonym(value = "LEAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LEAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "LEAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "LEAP", source = "CME_ClearedConfirm_1_17")
	LEAP,
	    

	/**
	 * CTA Master Coal Purchase and Sales Agreement
	 */
	@RosettaSynonym(value = "MCPSA", source = "FpML_5_10")
	@RosettaSynonym(value = "MCPSA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MCPSA", source = "DTCC_11_0")
	@RosettaSynonym(value = "MCPSA", source = "DTCC_9_0")
	@RosettaSynonym(value = "MCPSA", source = "CME_ClearedConfirm_1_17")
	MCPSA,
	    

	/**
	 * NAESB Base Contract for Sale and Purchase of Natural Gas
	 */
	@RosettaSynonym(value = "NAESBGas", source = "FpML_5_10")
	@RosettaSynonym(value = "NAESBGas", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NAESBGas", source = "DTCC_11_0")
	@RosettaSynonym(value = "NAESBGas", source = "DTCC_9_0")
	@RosettaSynonym(value = "NAESBGas", source = "CME_ClearedConfirm_1_17")
	NAESB_GAS,
	    

	/**
	 * Short Term Flat NBP Trading Terms and Conditions
	 */
	@RosettaSynonym(value = "NBP", source = "FpML_5_10")
	@RosettaSynonym(value = "NBP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NBP", source = "DTCC_11_0")
	@RosettaSynonym(value = "NBP", source = "DTCC_9_0")
	@RosettaSynonym(value = "NBP", source = "CME_ClearedConfirm_1_17")
	NBP,
	    

	/**
	 * Standard Documentation for Derivative Transactions on the Russian Financial Markets
	 */
	@RosettaSynonym(value = "RussianDerivatives", source = "FpML_5_10")
	@RosettaSynonym(value = "RussianDerivatives", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RussianDerivatives", source = "DTCC_11_0")
	@RosettaSynonym(value = "RussianDerivatives", source = "DTCC_9_0")
	@RosettaSynonym(value = "RussianDerivatives", source = "CME_ClearedConfirm_1_17")
	RUSSIAN_DERIVATIVES,
	    

	/**
	 * Master Agreement and Contractual Terms for Repurchase Agreements on the Russian Financial Market
	 */
	@RosettaSynonym(value = "RussianRepo", source = "FpML_5_10")
	@RosettaSynonym(value = "RussianRepo", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RussianRepo", source = "DTCC_11_0")
	@RosettaSynonym(value = "RussianRepo", source = "DTCC_9_0")
	@RosettaSynonym(value = "RussianRepo", source = "CME_ClearedConfirm_1_17")
	RUSSIAN_REPO,
	    

	/**
	 * globalCOAL Standard Coal Trading Agreement
	 */
	@RosettaSynonym(value = "SCoTA", source = "FpML_5_10")
	@RosettaSynonym(value = "SCoTA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SCoTA", source = "DTCC_11_0")
	@RosettaSynonym(value = "SCoTA", source = "DTCC_9_0")
	@RosettaSynonym(value = "SCoTA", source = "CME_ClearedConfirm_1_17")
	S_CO_TA,
	    

	/**
	 * Swiss Master Agreement for OTC Derivatives Instruments
	 */
	@RosettaSynonym(value = "Swiss", source = "FpML_5_10")
	@RosettaSynonym(value = "Swiss", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Swiss", source = "DTCC_11_0")
	@RosettaSynonym(value = "Swiss", source = "DTCC_9_0")
	@RosettaSynonym(value = "Swiss", source = "CME_ClearedConfirm_1_17")
	SWISS,
	    

	/**
	 * TTF Hub Natural Gas Trading Terms and Conditions
	 */
	@RosettaSynonym(value = "TTF", source = "FpML_5_10")
	@RosettaSynonym(value = "TTF", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TTF", source = "DTCC_11_0")
	@RosettaSynonym(value = "TTF", source = "DTCC_9_0")
	@RosettaSynonym(value = "TTF", source = "CME_ClearedConfirm_1_17")
	TTF,
	    

	/**
	 * Zeebrugge Hub Natural Gas Trading Terms and Conditions
	 */
	@RosettaSynonym(value = "ZBT", source = "FpML_5_10")
	@RosettaSynonym(value = "ZBT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZBT", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZBT", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZBT", source = "CME_ClearedConfirm_1_17")
	ZBT    
	;

	private final String displayName;
	
	MasterAgreementTypeEnum() {
		this.displayName = null;
	}

	MasterAgreementTypeEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
