package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaMasterConfirmationAnnexTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaMasterConfirmationAnnexTypeEnum.FieldWithMetaMasterConfirmationAnnexTypeEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaMasterConfirmationTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaMasterConfirmationTypeEnum.FieldWithMetaMasterConfirmationTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MasterConfirmationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining the master confirmation agreement executed between the parties.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MasterConfirmation", source="FpML_5_10")
public class MasterConfirmation extends RosettaModelObject {
	private final Date masterConfirmationAnnexDate;
	private final FieldWithMetaMasterConfirmationAnnexTypeEnum masterConfirmationAnnexType;
	private final Date masterConfirmationDate;
	private final FieldWithMetaMasterConfirmationTypeEnum masterConfirmationType;
	private static MasterConfirmationMeta metaData = new MasterConfirmationMeta();
	
	MasterConfirmation(MasterConfirmationBuilder builder) {
		this.masterConfirmationAnnexDate = builder.getMasterConfirmationAnnexDate();
		this.masterConfirmationAnnexType = ofNullable(builder.getMasterConfirmationAnnexType()).map(f->f.build()).orElse(null);
		this.masterConfirmationDate = builder.getMasterConfirmationDate();
		this.masterConfirmationType = ofNullable(builder.getMasterConfirmationType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date that an annex to the master confirmation was executed between the parties.
	 */
	@RosettaSynonym(value="masterConfirmationAnnexDate", source="FpML_5_10")
	@RosettaSynonym(value="masterConfirmationAnnexDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterConfirmationAnnexDate", source="DTCC_11_0")
	@RosettaSynonym(value="masterConfirmationAnnexDate", source="DTCC_9_0")
	@RosettaSynonym(value="masterConfirmationAnnexDate", source="CME_ClearedConfirm_1_17")
	public final Date getMasterConfirmationAnnexDate() {
		return masterConfirmationAnnexDate;
	}
	
	/**
	 * The type of master confirmation annex executed between the parties.
	 */
	@RosettaSynonym(value="masterConfirmationAnnexType", source="FpML_5_10")
	@RosettaSynonym(value="masterConfirmationAnnexType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterConfirmationAnnexType", source="DTCC_11_0")
	@RosettaSynonym(value="masterConfirmationAnnexType", source="DTCC_9_0")
	@RosettaSynonym(value="masterConfirmationAnnexType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMasterConfirmationAnnexTypeEnum getMasterConfirmationAnnexType() {
		return masterConfirmationAnnexType;
	}
	
	/**
	 * The date of the confirmation executed between the parties and intended to govern all relevant transactions between those parties.
	 */
	@RosettaSynonym(value="masterConfirmationDate", source="FpML_5_10")
	@RosettaSynonym(value="masterConfirmationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterConfirmationDate", source="DTCC_11_0")
	@RosettaSynonym(value="masterConfirmationDate", source="DTCC_9_0")
	@RosettaSynonym(value="masterConfirmationDate", source="CME_ClearedConfirm_1_17")
	public final Date getMasterConfirmationDate() {
		return masterConfirmationDate;
	}
	
	/**
	 * The type of master confirmation executed between the parties.
	 */
	@RosettaSynonym(value="masterConfirmationType", source="FpML_5_10")
	@RosettaSynonym(value="masterConfirmationType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="masterConfirmationType", source="DTCC_11_0")
	@RosettaSynonym(value="masterConfirmationType", source="DTCC_9_0")
	@RosettaSynonym(value="masterConfirmationType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMasterConfirmationTypeEnum getMasterConfirmationType() {
		return masterConfirmationType;
	}
	
	
	@Override
	public RosettaMetaData<? extends MasterConfirmation> metaData() {
		return metaData;
	} 
	
	public MasterConfirmationBuilder toBuilder() {
		MasterConfirmationBuilder builder = new MasterConfirmationBuilder();
		ofNullable(getMasterConfirmationAnnexDate()).ifPresent(builder::setMasterConfirmationAnnexDate);
		ofNullable(getMasterConfirmationAnnexType()).ifPresent(builder::setMasterConfirmationAnnexType);
		ofNullable(getMasterConfirmationDate()).ifPresent(builder::setMasterConfirmationDate);
		ofNullable(getMasterConfirmationType()).ifPresent(builder::setMasterConfirmationType);
		return builder;
	}

	public static MasterConfirmationBuilder builder() {
		return new MasterConfirmationBuilder();
	}

	public static  class MasterConfirmationBuilder extends RosettaModelObjectBuilder {
	
		protected Date masterConfirmationAnnexDate;
		protected FieldWithMetaMasterConfirmationAnnexTypeEnumBuilder masterConfirmationAnnexType;
		protected Date masterConfirmationDate;
		protected FieldWithMetaMasterConfirmationTypeEnumBuilder masterConfirmationType;
	
		public MasterConfirmationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MasterConfirmation> metaData() {
			return metaData;
		} 
	
		
		public Date getMasterConfirmationAnnexDate() {
			return masterConfirmationAnnexDate;
		}
		
		
		public FieldWithMetaMasterConfirmationAnnexTypeEnumBuilder getMasterConfirmationAnnexType() {
			return masterConfirmationAnnexType;
		}
		
		public FieldWithMetaMasterConfirmationAnnexTypeEnumBuilder getOrCreateMasterConfirmationAnnexType() {
			if (masterConfirmationAnnexType!=null) {
				return masterConfirmationAnnexType;
			}
			else return masterConfirmationAnnexType = new FieldWithMetaMasterConfirmationAnnexTypeEnumBuilder();
		}
		
		
		public Date getMasterConfirmationDate() {
			return masterConfirmationDate;
		}
		
		
		public FieldWithMetaMasterConfirmationTypeEnumBuilder getMasterConfirmationType() {
			return masterConfirmationType;
		}
		
		public FieldWithMetaMasterConfirmationTypeEnumBuilder getOrCreateMasterConfirmationType() {
			if (masterConfirmationType!=null) {
				return masterConfirmationType;
			}
			else return masterConfirmationType = new FieldWithMetaMasterConfirmationTypeEnumBuilder();
		}
		
	
		public MasterConfirmationBuilder setMasterConfirmationAnnexDate(Date masterConfirmationAnnexDate) {
			this.masterConfirmationAnnexDate = masterConfirmationAnnexDate;
			return this;
		}
		public MasterConfirmationBuilder setMasterConfirmationAnnexType(FieldWithMetaMasterConfirmationAnnexTypeEnum masterConfirmationAnnexType) {
			this.masterConfirmationAnnexType = masterConfirmationAnnexType.toBuilder();
			return this;
		}
		public MasterConfirmationBuilder setMasterConfirmationAnnexTypeRef(MasterConfirmationAnnexTypeEnum masterConfirmationAnnexType) {
			return setMasterConfirmationAnnexType(FieldWithMetaMasterConfirmationAnnexTypeEnum.builder().setValue(masterConfirmationAnnexType).build());
		}
		public MasterConfirmationBuilder setMasterConfirmationDate(Date masterConfirmationDate) {
			this.masterConfirmationDate = masterConfirmationDate;
			return this;
		}
		public MasterConfirmationBuilder setMasterConfirmationType(FieldWithMetaMasterConfirmationTypeEnum masterConfirmationType) {
			this.masterConfirmationType = masterConfirmationType.toBuilder();
			return this;
		}
		public MasterConfirmationBuilder setMasterConfirmationTypeRef(MasterConfirmationTypeEnum masterConfirmationType) {
			return setMasterConfirmationType(FieldWithMetaMasterConfirmationTypeEnum.builder().setValue(masterConfirmationType).build());
		}
		
		public MasterConfirmation build() {
			return new MasterConfirmation(this);
		}
	
		@Override
		public MasterConfirmationBuilder prune() {
			if (masterConfirmationAnnexType!=null && !masterConfirmationAnnexType.prune().hasData()) masterConfirmationAnnexType = null;
			if (masterConfirmationType!=null && !masterConfirmationType.prune().hasData()) masterConfirmationType = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMasterConfirmationAnnexDate()!=null) return true;
			if (getMasterConfirmationAnnexType()!=null) return true;
			if (getMasterConfirmationDate()!=null) return true;
			if (getMasterConfirmationType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("masterConfirmationAnnexDate"), Date.class, masterConfirmationAnnexDate, this);
			processor.processBasic(path.newSubPath("masterConfirmationDate"), Date.class, masterConfirmationDate, this);
			
			processRosetta(path.newSubPath("masterConfirmationAnnexType"), processor, FieldWithMetaMasterConfirmationAnnexTypeEnum.class, masterConfirmationAnnexType);
			processRosetta(path.newSubPath("masterConfirmationType"), processor, FieldWithMetaMasterConfirmationTypeEnum.class, masterConfirmationType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MasterConfirmationBuilder _that = (MasterConfirmationBuilder) o;
		
			if (masterConfirmationAnnexDate != null ? !masterConfirmationAnnexDate.equals(_that.masterConfirmationAnnexDate) : _that.masterConfirmationAnnexDate != null) return false;
			if (masterConfirmationAnnexType != null ? !masterConfirmationAnnexType.equals(_that.masterConfirmationAnnexType) : _that.masterConfirmationAnnexType != null) return false;
			if (masterConfirmationDate != null ? !masterConfirmationDate.equals(_that.masterConfirmationDate) : _that.masterConfirmationDate != null) return false;
			if (masterConfirmationType != null ? !masterConfirmationType.equals(_that.masterConfirmationType) : _that.masterConfirmationType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (masterConfirmationAnnexDate != null ? masterConfirmationAnnexDate.hashCode() : 0);
			_result = 31 * _result + (masterConfirmationAnnexType != null ? masterConfirmationAnnexType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (masterConfirmationDate != null ? masterConfirmationDate.hashCode() : 0);
			_result = 31 * _result + (masterConfirmationType != null ? masterConfirmationType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MasterConfirmationBuilder {" +
				"masterConfirmationAnnexDate=" + this.masterConfirmationAnnexDate + ", " +
				"masterConfirmationAnnexType=" + this.masterConfirmationAnnexType + ", " +
				"masterConfirmationDate=" + this.masterConfirmationDate + ", " +
				"masterConfirmationType=" + this.masterConfirmationType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("masterConfirmationAnnexDate"), Date.class, masterConfirmationAnnexDate, this);
		processor.processBasic(path.newSubPath("masterConfirmationDate"), Date.class, masterConfirmationDate, this);
		
		processRosetta(path.newSubPath("masterConfirmationAnnexType"), processor, FieldWithMetaMasterConfirmationAnnexTypeEnum.class, masterConfirmationAnnexType);
		processRosetta(path.newSubPath("masterConfirmationType"), processor, FieldWithMetaMasterConfirmationTypeEnum.class, masterConfirmationType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MasterConfirmation _that = (MasterConfirmation) o;
	
		if (masterConfirmationAnnexDate != null ? !masterConfirmationAnnexDate.equals(_that.masterConfirmationAnnexDate) : _that.masterConfirmationAnnexDate != null) return false;
		if (masterConfirmationAnnexType != null ? !masterConfirmationAnnexType.equals(_that.masterConfirmationAnnexType) : _that.masterConfirmationAnnexType != null) return false;
		if (masterConfirmationDate != null ? !masterConfirmationDate.equals(_that.masterConfirmationDate) : _that.masterConfirmationDate != null) return false;
		if (masterConfirmationType != null ? !masterConfirmationType.equals(_that.masterConfirmationType) : _that.masterConfirmationType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (masterConfirmationAnnexDate != null ? masterConfirmationAnnexDate.hashCode() : 0);
		_result = 31 * _result + (masterConfirmationAnnexType != null ? masterConfirmationAnnexType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (masterConfirmationDate != null ? masterConfirmationDate.hashCode() : 0);
		_result = 31 * _result + (masterConfirmationType != null ? masterConfirmationType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MasterConfirmation {" +
			"masterConfirmationAnnexDate=" + this.masterConfirmationAnnexDate + ", " +
			"masterConfirmationAnnexType=" + this.masterConfirmationAnnexType + ", " +
			"masterConfirmationDate=" + this.masterConfirmationDate + ", " +
			"masterConfirmationType=" + this.masterConfirmationType +
		'}';
	}
}
