package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the identification the form of applicable matrix.
 * 
 * @version 2.5.4
 */
public enum MatrixTypeEnum {
    

	/**
	 * The ISDA-published Credit Derivatives Physical Settlement Matrix.
	 */
	@RosettaSynonym(value = "CreditDerivativesPhysicalSettlementMatrix", source = "FpML_5_10")
	@RosettaSynonym(value = "CreditDerivativesPhysicalSettlementMatrix", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CreditDerivativesPhysicalSettlementMatrix", source = "DTCC_11_0")
	@RosettaSynonym(value = "CreditDerivativesPhysicalSettlementMatrix", source = "DTCC_9_0")
	@RosettaSynonym(value = "CreditDerivativesPhysicalSettlementMatrix", source = "CME_ClearedConfirm_1_17")
	CREDIT_DERIVATIVES_PHYSICAL_SETTLEMENT_MATRIX,
	    

	/**
	 * The ISDA-published Equity Derivatives Matrix.
	 */
	@RosettaSynonym(value = "EquityDerivativesMatrix", source = "FpML_5_10")
	@RosettaSynonym(value = "EquityDerivativesMatrix", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EquityDerivativesMatrix", source = "DTCC_11_0")
	@RosettaSynonym(value = "EquityDerivativesMatrix", source = "DTCC_9_0")
	@RosettaSynonym(value = "EquityDerivativesMatrix", source = "CME_ClearedConfirm_1_17")
	EQUITY_DERIVATIVES_MATRIX,
	    

	/**
	 * The ISDA-published 2000 ISDA Definitions Settlement Matrix for Early Terminations and Swaptions.
	 */
	@RosettaSynonym(value = "SettlementMatrix", source = "FpML_5_10")
	@RosettaSynonym(value = "SettlementMatrix", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SettlementMatrix", source = "DTCC_11_0")
	@RosettaSynonym(value = "SettlementMatrix", source = "DTCC_9_0")
	@RosettaSynonym(value = "SettlementMatrix", source = "CME_ClearedConfirm_1_17")
	SETTLEMENT_MATRIX    
	;

	private final String displayName;
	
	MatrixTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
