package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MessageInformationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * This class corresponds to the components of the FpML MessageHeader.model.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class MessageInformation extends RosettaModelObject {
	private final List<FieldWithMetaString> copyTo;
	private final FieldWithMetaString messageId;
	private final FieldWithMetaString sentBy;
	private final List<FieldWithMetaString> sentTo;
	private static MessageInformationMeta metaData = new MessageInformationMeta();
	
	MessageInformation(MessageInformationBuilder builder) {
		this.copyTo = ofNullable(builder.getCopyTo()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.messageId = ofNullable(builder.getMessageId()).map(f->f.build()).orElse(null);
		this.sentBy = ofNullable(builder.getSentBy()).map(f->f.build()).orElse(null);
		this.sentTo = ofNullable(builder.getSentTo()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A unique identifier (within the specified coding scheme) giving the details of some party to whom a copy of this message will be sent for reference.
	 */
	public final List<FieldWithMetaString> getCopyTo() {
		return copyTo;
	}
	
	/**
	 * A unique identifier assigned to the message.
	 */
	@RosettaSynonym(value="messageId", source="Rosetta_Workbench", path="messageInformation")
	@RosettaSynonym(value="messageId", source="DTCC_11_0", path="header")
	@RosettaSynonym(value="messageId", source="DTCC_9_0", path="header")
	@RosettaSynonym(value="messageId", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getMessageId() {
		return messageId;
	}
	
	/**
	 * The identifier for the originator of a message instance.
	 */
	@RosettaSynonym(value="sentBy", source="Rosetta_Workbench", path="messageInformation")
	@RosettaSynonym(value="From", source="DTCC_11_0")
	@RosettaSynonym(value="From", source="DTCC_9_0")
	@RosettaSynonym(value="sentBy", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="SSub", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt.Hdr")
	public final FieldWithMetaString getSentBy() {
		return sentBy;
	}
	
	/**
	 * The identifier(s) for the recipient(s) of a message instance.
	 */
	@RosettaSynonym(value="sentTo", source="Rosetta_Workbench", path="messageInformation")
	@RosettaSynonym(value="sendTo", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="To", source="DTCC_11_0")
	@RosettaSynonym(value="To", source="DTCC_9_0")
	@RosettaSynonym(value="TID", source="CME_SubmissionIRS_1_0", path="TrdCaptRpt.Hdr")
	public final List<FieldWithMetaString> getSentTo() {
		return sentTo;
	}
	
	
	@Override
	public RosettaMetaData<? extends MessageInformation> metaData() {
		return metaData;
	} 
	
	public MessageInformationBuilder toBuilder() {
		MessageInformationBuilder builder = new MessageInformationBuilder();
		ofNullable(getCopyTo()).ifPresent(copyTo -> copyTo.forEach(builder::addCopyTo));
		ofNullable(getMessageId()).ifPresent(builder::setMessageId);
		ofNullable(getSentBy()).ifPresent(builder::setSentBy);
		ofNullable(getSentTo()).ifPresent(sentTo -> sentTo.forEach(builder::addSentTo));
		return builder;
	}

	public static MessageInformationBuilder builder() {
		return new MessageInformationBuilder();
	}

	public static  class MessageInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<FieldWithMetaStringBuilder> copyTo;
		protected FieldWithMetaStringBuilder messageId;
		protected FieldWithMetaStringBuilder sentBy;
		protected List<FieldWithMetaStringBuilder> sentTo;
	
		public MessageInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MessageInformation> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getCopyTo() {
			return copyTo;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCopyTo(int index) {
			if (copyTo==null) {
				this.copyTo = new ArrayList<>();
			}
			return getIndex(copyTo, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public FieldWithMetaStringBuilder getMessageId() {
			return messageId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMessageId() {
			if (messageId!=null) {
				return messageId;
			}
			else return messageId = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getSentBy() {
			return sentBy;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSentBy() {
			if (sentBy!=null) {
				return sentBy;
			}
			else return sentBy = new FieldWithMetaStringBuilder();
		}
		
		
		public List<FieldWithMetaStringBuilder> getSentTo() {
			return sentTo;
		}
		
		public FieldWithMetaStringBuilder getOrCreateSentTo(int index) {
			if (sentTo==null) {
				this.sentTo = new ArrayList<>();
			}
			return getIndex(sentTo, index, ()->new FieldWithMetaStringBuilder());
		}
		
	
		public MessageInformationBuilder addCopyTo(FieldWithMetaString copyTo) {
			if(this.copyTo == null){
				this.copyTo = new ArrayList<>();
			}
			this.copyTo.add(copyTo.toBuilder());
			return this;
		}
		public MessageInformationBuilder addCopyTo(FieldWithMetaString copyTo, int _idx) {
			if(this.copyTo == null){
				this.copyTo = new ArrayList<>();
			}
			getIndex(this.copyTo, _idx, () -> copyTo.toBuilder());
			this.copyTo.set(_idx, copyTo.toBuilder());
			return this;
		}
		public MessageInformationBuilder addCopyTo(List<FieldWithMetaString> copyTos) {
			if(this.copyTo == null){
				this.copyTo = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : copyTos) {
				this.copyTo.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public MessageInformationBuilder clearCopyTo() {
			this.copyTo = null;
			return this;
		}
		public MessageInformationBuilder setMessageId(FieldWithMetaString messageId) {
			this.messageId = messageId.toBuilder();
			return this;
		}
		public MessageInformationBuilder setMessageIdRef(String messageId) {
			return setMessageId(FieldWithMetaString.builder().setValue(messageId).build());
		}
		public MessageInformationBuilder setSentBy(FieldWithMetaString sentBy) {
			this.sentBy = sentBy.toBuilder();
			return this;
		}
		public MessageInformationBuilder setSentByRef(String sentBy) {
			return setSentBy(FieldWithMetaString.builder().setValue(sentBy).build());
		}
		public MessageInformationBuilder addSentTo(FieldWithMetaString sentTo) {
			if(this.sentTo == null){
				this.sentTo = new ArrayList<>();
			}
			this.sentTo.add(sentTo.toBuilder());
			return this;
		}
		public MessageInformationBuilder addSentTo(FieldWithMetaString sentTo, int _idx) {
			if(this.sentTo == null){
				this.sentTo = new ArrayList<>();
			}
			getIndex(this.sentTo, _idx, () -> sentTo.toBuilder());
			this.sentTo.set(_idx, sentTo.toBuilder());
			return this;
		}
		public MessageInformationBuilder addSentTo(List<FieldWithMetaString> sentTos) {
			if(this.sentTo == null){
				this.sentTo = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : sentTos) {
				this.sentTo.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public MessageInformationBuilder clearSentTo() {
			this.sentTo = null;
			return this;
		}
		
		public MessageInformation build() {
			return new MessageInformation(this);
		}
	
		@Override
		public MessageInformationBuilder prune() {
			if (copyTo!=null) copyTo = copyTo.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (messageId!=null && !messageId.prune().hasData()) messageId = null;
			if (sentBy!=null && !sentBy.prune().hasData()) sentBy = null;
			if (sentTo!=null) sentTo = sentTo.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCopyTo()!=null && !getCopyTo().isEmpty()) return true;
			if (getMessageId()!=null) return true;
			if (getSentBy()!=null) return true;
			if (getSentTo()!=null && !getSentTo().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("copyTo"), processor, FieldWithMetaString.class, copyTo);
			processRosetta(path.newSubPath("messageId"), processor, FieldWithMetaString.class, messageId);
			processRosetta(path.newSubPath("sentBy"), processor, FieldWithMetaString.class, sentBy);
			processRosetta(path.newSubPath("sentTo"), processor, FieldWithMetaString.class, sentTo);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MessageInformationBuilder _that = (MessageInformationBuilder) o;
		
			if (!ListEquals.listEquals(copyTo, _that.copyTo)) return false;
			if (messageId != null ? !messageId.equals(_that.messageId) : _that.messageId != null) return false;
			if (sentBy != null ? !sentBy.equals(_that.sentBy) : _that.sentBy != null) return false;
			if (!ListEquals.listEquals(sentTo, _that.sentTo)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (copyTo != null ? copyTo.hashCode() : 0);
			_result = 31 * _result + (messageId != null ? messageId.hashCode() : 0);
			_result = 31 * _result + (sentBy != null ? sentBy.hashCode() : 0);
			_result = 31 * _result + (sentTo != null ? sentTo.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MessageInformationBuilder {" +
				"copyTo=" + this.copyTo + ", " +
				"messageId=" + this.messageId + ", " +
				"sentBy=" + this.sentBy + ", " +
				"sentTo=" + this.sentTo +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("copyTo"), processor, FieldWithMetaString.class, copyTo);
		processRosetta(path.newSubPath("messageId"), processor, FieldWithMetaString.class, messageId);
		processRosetta(path.newSubPath("sentBy"), processor, FieldWithMetaString.class, sentBy);
		processRosetta(path.newSubPath("sentTo"), processor, FieldWithMetaString.class, sentTo);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MessageInformation _that = (MessageInformation) o;
	
		if (!ListEquals.listEquals(copyTo, _that.copyTo)) return false;
		if (messageId != null ? !messageId.equals(_that.messageId) : _that.messageId != null) return false;
		if (sentBy != null ? !sentBy.equals(_that.sentBy) : _that.sentBy != null) return false;
		if (!ListEquals.listEquals(sentTo, _that.sentTo)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (copyTo != null ? copyTo.hashCode() : 0);
		_result = 31 * _result + (messageId != null ? messageId.hashCode() : 0);
		_result = 31 * _result + (sentBy != null ? sentBy.hashCode() : 0);
		_result = 31 * _result + (sentTo != null ? sentTo.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MessageInformation {" +
			"copyTo=" + this.copyTo + ", " +
			"messageId=" + this.messageId + ", " +
			"sentBy=" + this.sentBy + ", " +
			"sentTo=" + this.sentTo +
		'}';
	}
}
