package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ElectiveAmountElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MinimumTransferAmountMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify amount of exposure reached before collateral has to be posted or returned. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (c)(vi)(B): Minimum Transfer Amount. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(vi)(B): Minimum Transfer Amount. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (c)(vi)(B): Minimum Transfer Amount.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class MinimumTransferAmount extends RosettaModelObject {
	private final List<ElectiveAmountElection> partyElection;
	private static MinimumTransferAmountMeta metaData = new MinimumTransferAmountMeta();
	
	MinimumTransferAmount(MinimumTransferAmountBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; minimum transfer amount elections.
	 */
	public final List<ElectiveAmountElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends MinimumTransferAmount> metaData() {
		return metaData;
	} 
	
	public MinimumTransferAmountBuilder toBuilder() {
		MinimumTransferAmountBuilder builder = new MinimumTransferAmountBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static MinimumTransferAmountBuilder builder() {
		return new MinimumTransferAmountBuilder();
	}

	public static  class MinimumTransferAmountBuilder extends RosettaModelObjectBuilder {
	
		protected List<ElectiveAmountElection.ElectiveAmountElectionBuilder> partyElection;
	
		public MinimumTransferAmountBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MinimumTransferAmount> metaData() {
			return metaData;
		} 
	
		
		public List<ElectiveAmountElection.ElectiveAmountElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public ElectiveAmountElection.ElectiveAmountElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new ElectiveAmountElection.ElectiveAmountElectionBuilder());
		}
		
	
		public MinimumTransferAmountBuilder addPartyElection(ElectiveAmountElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public MinimumTransferAmountBuilder addPartyElection(ElectiveAmountElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public MinimumTransferAmountBuilder addPartyElection(List<ElectiveAmountElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (ElectiveAmountElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public MinimumTransferAmountBuilder addPartyElectionBuilder(ElectiveAmountElection.ElectiveAmountElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public MinimumTransferAmountBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public MinimumTransferAmount build() {
			return new MinimumTransferAmount(this);
		}
	
		@Override
		public MinimumTransferAmountBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, ElectiveAmountElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MinimumTransferAmountBuilder _that = (MinimumTransferAmountBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MinimumTransferAmountBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, ElectiveAmountElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MinimumTransferAmount _that = (MinimumTransferAmount) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MinimumTransferAmount {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
