package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.AssetPool;
import org.isda.cdm.metafields.FieldWithMetaMortgageSectorEnum;
import org.isda.cdm.metafields.FieldWithMetaMortgageSectorEnum.FieldWithMetaMortgageSectorEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MortgageBackedSecurityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Mortgage", source="FpML_5_10")
public class MortgageBackedSecurity extends ProductIdentifier {
	private final AssetPool pool;
	private final FieldWithMetaMortgageSectorEnum sector;
	private final String tranche;
	private static MortgageBackedSecurityMeta metaData = new MortgageBackedSecurityMeta();
	
	MortgageBackedSecurity(MortgageBackedSecurityBuilder builder) {
		super(builder);
		this.pool = ofNullable(builder.getPool()).map(f->f.build()).orElse(null);
		this.sector = ofNullable(builder.getSector()).map(f->f.build()).orElse(null);
		this.tranche = builder.getTranche();
	}
	
	/**
	 * The mortgage pool that is underneath the mortgage obligation.
	 */
	@RosettaSynonym(value="pool", source="FpML_5_10")
	@RosettaSynonym(value="pool", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="pool", source="DTCC_11_0")
	@RosettaSynonym(value="pool", source="DTCC_9_0")
	@RosettaSynonym(value="pool", source="CME_ClearedConfirm_1_17")
	public final AssetPool getPool() {
		return pool;
	}
	
	/**
	 * The sector classification of the mortgage obligation.
	 */
	@RosettaSynonym(value="sector", source="FpML_5_10")
	@RosettaSynonym(value="sector", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sector", source="DTCC_11_0")
	@RosettaSynonym(value="sector", source="DTCC_9_0")
	@RosettaSynonym(value="sector", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaMortgageSectorEnum getSector() {
		return sector;
	}
	
	/**
	 * The mortgage obligation tranche that is subject to the derivative transaction.
	 */
	@RosettaSynonym(value="tranche", source="FpML_5_10")
	@RosettaSynonym(value="tranche", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="tranche", source="DTCC_11_0")
	@RosettaSynonym(value="tranche", source="DTCC_9_0")
	@RosettaSynonym(value="tranche", source="CME_ClearedConfirm_1_17")
	public final String getTranche() {
		return tranche;
	}
	
	
	@Override
	public RosettaMetaData<? extends MortgageBackedSecurity> metaData() {
		return metaData;
	} 
	
	public MortgageBackedSecurityBuilder toBuilder() {
		MortgageBackedSecurityBuilder builder = new MortgageBackedSecurityBuilder();
		ofNullable(getPool()).ifPresent(builder::setPool);
		ofNullable(getSector()).ifPresent(builder::setSector);
		ofNullable(getTranche()).ifPresent(builder::setTranche);
		ofNullable(getIdentifier()).ifPresent(identifier -> identifier.forEach(builder::addIdentifier));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getProductTaxonomy()).ifPresent(productTaxonomy -> productTaxonomy.forEach(builder::addProductTaxonomy));
		ofNullable(getSource()).ifPresent(builder::setSource);
		return builder;
	}

	public static MortgageBackedSecurityBuilder builder() {
		return new MortgageBackedSecurityBuilder();
	}

	public static  class MortgageBackedSecurityBuilder extends ProductIdentifierBuilder {
	
		protected AssetPool.AssetPoolBuilder pool;
		protected FieldWithMetaMortgageSectorEnumBuilder sector;
		protected String tranche;
	
		public MortgageBackedSecurityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MortgageBackedSecurity> metaData() {
			return metaData;
		} 
	
		
		public AssetPool.AssetPoolBuilder getPool() {
			return pool;
		}
		
		public AssetPool.AssetPoolBuilder getOrCreatePool() {
			if (pool!=null) {
				return pool;
			}
			else return pool = new AssetPool.AssetPoolBuilder();
		}
		
		
		public FieldWithMetaMortgageSectorEnumBuilder getSector() {
			return sector;
		}
		
		public FieldWithMetaMortgageSectorEnumBuilder getOrCreateSector() {
			if (sector!=null) {
				return sector;
			}
			else return sector = new FieldWithMetaMortgageSectorEnumBuilder();
		}
		
		
		public String getTranche() {
			return tranche;
		}
		
	
		public MortgageBackedSecurityBuilder setPool(AssetPool pool) {
			this.pool = pool.toBuilder();
			return this;
		}
		public MortgageBackedSecurityBuilder setPoolBuilder(AssetPool.AssetPoolBuilder pool) {
			this.pool = pool;
			return this;
		}
		public MortgageBackedSecurityBuilder setSector(FieldWithMetaMortgageSectorEnum sector) {
			this.sector = sector.toBuilder();
			return this;
		}
		public MortgageBackedSecurityBuilder setSectorRef(MortgageSectorEnum sector) {
			return setSector(FieldWithMetaMortgageSectorEnum.builder().setValue(sector).build());
		}
		public MortgageBackedSecurityBuilder setTranche(String tranche) {
			this.tranche = tranche;
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addIdentifier(FieldWithMetaString identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addIdentifier(FieldWithMetaString identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addIdentifier(List<FieldWithMetaString> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		
		@Override public MortgageBackedSecurityBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		@Override public MortgageBackedSecurityBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public MortgageBackedSecurityBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy.toBuilder());
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy, int _idx) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			getIndex(this.productTaxonomy, _idx, () -> productTaxonomy.toBuilder());
			this.productTaxonomy.set(_idx, productTaxonomy.toBuilder());
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addProductTaxonomy(List<ProductTaxonomy> productTaxonomys) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			for (ProductTaxonomy toAdd : productTaxonomys) {
				this.productTaxonomy.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public MortgageBackedSecurityBuilder addProductTaxonomyBuilder(ProductTaxonomy.ProductTaxonomyBuilder productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy);
			return this;
		}
		
		@Override public MortgageBackedSecurityBuilder clearProductTaxonomy() {
			this.productTaxonomy = null;
			return this;
		}
		@Override public MortgageBackedSecurityBuilder setSource(ProductIdSourceEnum source) {
			this.source = source;
			return this;
		}
		
		public MortgageBackedSecurity build() {
			return new MortgageBackedSecurity(this);
		}
	
		@Override
		public MortgageBackedSecurityBuilder prune() {
			super.prune();
			if (pool!=null && !pool.prune().hasData()) pool = null;
			if (sector!=null && !sector.prune().hasData()) sector = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getPool()!=null && getPool().hasData()) return true;
			if (getSector()!=null) return true;
			if (getTranche()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("tranche"), String.class, tranche, this);
			
			processRosetta(path.newSubPath("pool"), processor, AssetPool.class, pool);
			processRosetta(path.newSubPath("sector"), processor, FieldWithMetaMortgageSectorEnum.class, sector);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			MortgageBackedSecurityBuilder _that = (MortgageBackedSecurityBuilder) o;
		
			if (pool != null ? !pool.equals(_that.pool) : _that.pool != null) return false;
			if (sector != null ? !sector.equals(_that.sector) : _that.sector != null) return false;
			if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (pool != null ? pool.hashCode() : 0);
			_result = 31 * _result + (sector != null ? sector.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MortgageBackedSecurityBuilder {" +
				"pool=" + this.pool + ", " +
				"sector=" + this.sector + ", " +
				"tranche=" + this.tranche +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("tranche"), String.class, tranche, this);
		
		processRosetta(path.newSubPath("pool"), processor, AssetPool.class, pool);
		processRosetta(path.newSubPath("sector"), processor, FieldWithMetaMortgageSectorEnum.class, sector);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		MortgageBackedSecurity _that = (MortgageBackedSecurity) o;
	
		if (pool != null ? !pool.equals(_that.pool) : _that.pool != null) return false;
		if (sector != null ? !sector.equals(_that.sector) : _that.sector != null) return false;
		if (tranche != null ? !tranche.equals(_that.tranche) : _that.tranche != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (pool != null ? pool.hashCode() : 0);
		_result = 31 * _result + (sector != null ? sector.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MortgageBackedSecurity {" +
			"pool=" + this.pool + ", " +
			"sector=" + this.sector + ", " +
			"tranche=" + this.tranche +
		'}' + " " + super.toString();
	}
}
