package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.QuantifierEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MultipleCreditNotationsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify multiple credit notations alongside a conditional &#39;any&#39; or &#39;all&#39; qualifier.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class MultipleCreditNotations extends RosettaModelObject {
	private final QuantifierEnum condition;
	private final List<FieldWithMetaString> creditNotation;
	private static MultipleCreditNotationsMeta metaData = new MultipleCreditNotationsMeta();
	
	MultipleCreditNotations(MultipleCreditNotationsBuilder builder) {
		this.condition = builder.getCondition();
		this.creditNotation = ofNullable(builder.getCreditNotation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * An enumerated element, to qualify whether All or Any credit notation applies.
	 */
	@RosettaSynonym(value="condition", source="FpML_5_10")
	public final QuantifierEnum getCondition() {
		return condition;
	}
	
	/**
	 * At least two credit notations much be specified.
	 */
	@RosettaSynonym(value="debtType", source="FpML_5_10")
	public final List<FieldWithMetaString> getCreditNotation() {
		return creditNotation;
	}
	
	
	@Override
	public RosettaMetaData<? extends MultipleCreditNotations> metaData() {
		return metaData;
	} 
	
	public MultipleCreditNotationsBuilder toBuilder() {
		MultipleCreditNotationsBuilder builder = new MultipleCreditNotationsBuilder();
		ofNullable(getCondition()).ifPresent(builder::setCondition);
		ofNullable(getCreditNotation()).ifPresent(creditNotation -> creditNotation.forEach(builder::addCreditNotation));
		return builder;
	}

	public static MultipleCreditNotationsBuilder builder() {
		return new MultipleCreditNotationsBuilder();
	}

	public static  class MultipleCreditNotationsBuilder extends RosettaModelObjectBuilder {
	
		protected QuantifierEnum condition;
		protected List<FieldWithMetaStringBuilder> creditNotation;
	
		public MultipleCreditNotationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MultipleCreditNotations> metaData() {
			return metaData;
		} 
	
		
		public QuantifierEnum getCondition() {
			return condition;
		}
		
		
		public List<FieldWithMetaStringBuilder> getCreditNotation() {
			return creditNotation;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCreditNotation(int index) {
			if (creditNotation==null) {
				this.creditNotation = new ArrayList<>();
			}
			return getIndex(creditNotation, index, ()->new FieldWithMetaStringBuilder());
		}
		
	
		public MultipleCreditNotationsBuilder setCondition(QuantifierEnum condition) {
			this.condition = condition;
			return this;
		}
		public MultipleCreditNotationsBuilder addCreditNotation(FieldWithMetaString creditNotation) {
			if(this.creditNotation == null){
				this.creditNotation = new ArrayList<>();
			}
			this.creditNotation.add(creditNotation.toBuilder());
			return this;
		}
		public MultipleCreditNotationsBuilder addCreditNotation(FieldWithMetaString creditNotation, int _idx) {
			if(this.creditNotation == null){
				this.creditNotation = new ArrayList<>();
			}
			getIndex(this.creditNotation, _idx, () -> creditNotation.toBuilder());
			this.creditNotation.set(_idx, creditNotation.toBuilder());
			return this;
		}
		public MultipleCreditNotationsBuilder addCreditNotation(List<FieldWithMetaString> creditNotations) {
			if(this.creditNotation == null){
				this.creditNotation = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : creditNotations) {
				this.creditNotation.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public MultipleCreditNotationsBuilder clearCreditNotation() {
			this.creditNotation = null;
			return this;
		}
		
		public MultipleCreditNotations build() {
			return new MultipleCreditNotations(this);
		}
	
		@Override
		public MultipleCreditNotationsBuilder prune() {
			if (creditNotation!=null) creditNotation = creditNotation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCondition()!=null) return true;
			if (getCreditNotation()!=null && !getCreditNotation().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("condition"), QuantifierEnum.class, condition, this);
			
			processRosetta(path.newSubPath("creditNotation"), processor, FieldWithMetaString.class, creditNotation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MultipleCreditNotationsBuilder _that = (MultipleCreditNotationsBuilder) o;
		
			if (condition != null ? !condition.equals(_that.condition) : _that.condition != null) return false;
			if (!ListEquals.listEquals(creditNotation, _that.creditNotation)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (condition != null ? condition.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (creditNotation != null ? creditNotation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MultipleCreditNotationsBuilder {" +
				"condition=" + this.condition + ", " +
				"creditNotation=" + this.creditNotation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("condition"), QuantifierEnum.class, condition, this);
		
		processRosetta(path.newSubPath("creditNotation"), processor, FieldWithMetaString.class, creditNotation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MultipleCreditNotations _that = (MultipleCreditNotations) o;
	
		if (condition != null ? !condition.equals(_that.condition) : _that.condition != null) return false;
		if (!ListEquals.listEquals(creditNotation, _that.creditNotation)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (condition != null ? condition.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (creditNotation != null ? creditNotation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MultipleCreditNotations {" +
			"condition=" + this.condition + ", " +
			"creditNotation=" + this.creditNotation +
		'}';
	}
}
