package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.QuantifierEnum;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MultipleDebtTypesMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify multiple credit debt types alongside a conditional &#39;any&#39; or &#39;all&#39; qualifier.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class MultipleDebtTypes extends RosettaModelObject {
	private final QuantifierEnum condition;
	private final List<FieldWithMetaString> debtType;
	private static MultipleDebtTypesMeta metaData = new MultipleDebtTypesMeta();
	
	MultipleDebtTypes(MultipleDebtTypesBuilder builder) {
		this.condition = builder.getCondition();
		this.debtType = ofNullable(builder.getDebtType()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * An enumerated attribute, to qualify whether All or Any debt type applies.
	 */
	@RosettaSynonym(value="condition", source="FpML_5_10")
	public final QuantifierEnum getCondition() {
		return condition;
	}
	
	/**
	 * The type of debt, e.g. long term debt, deposit, ... FpML doesn&#39;t specific a scheme value, hence no enumeration is specified as part of the CDM. At least two debt types much be specified.
	 */
	@RosettaSynonym(value="debtType", source="FpML_5_10")
	public final List<FieldWithMetaString> getDebtType() {
		return debtType;
	}
	
	
	@Override
	public RosettaMetaData<? extends MultipleDebtTypes> metaData() {
		return metaData;
	} 
	
	public MultipleDebtTypesBuilder toBuilder() {
		MultipleDebtTypesBuilder builder = new MultipleDebtTypesBuilder();
		ofNullable(getCondition()).ifPresent(builder::setCondition);
		ofNullable(getDebtType()).ifPresent(debtType -> debtType.forEach(builder::addDebtType));
		return builder;
	}

	public static MultipleDebtTypesBuilder builder() {
		return new MultipleDebtTypesBuilder();
	}

	public static  class MultipleDebtTypesBuilder extends RosettaModelObjectBuilder {
	
		protected QuantifierEnum condition;
		protected List<FieldWithMetaStringBuilder> debtType;
	
		public MultipleDebtTypesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MultipleDebtTypes> metaData() {
			return metaData;
		} 
	
		
		public QuantifierEnum getCondition() {
			return condition;
		}
		
		
		public List<FieldWithMetaStringBuilder> getDebtType() {
			return debtType;
		}
		
		public FieldWithMetaStringBuilder getOrCreateDebtType(int index) {
			if (debtType==null) {
				this.debtType = new ArrayList<>();
			}
			return getIndex(debtType, index, ()->new FieldWithMetaStringBuilder());
		}
		
	
		public MultipleDebtTypesBuilder setCondition(QuantifierEnum condition) {
			this.condition = condition;
			return this;
		}
		public MultipleDebtTypesBuilder addDebtType(FieldWithMetaString debtType) {
			if(this.debtType == null){
				this.debtType = new ArrayList<>();
			}
			this.debtType.add(debtType.toBuilder());
			return this;
		}
		public MultipleDebtTypesBuilder addDebtType(FieldWithMetaString debtType, int _idx) {
			if(this.debtType == null){
				this.debtType = new ArrayList<>();
			}
			getIndex(this.debtType, _idx, () -> debtType.toBuilder());
			this.debtType.set(_idx, debtType.toBuilder());
			return this;
		}
		public MultipleDebtTypesBuilder addDebtType(List<FieldWithMetaString> debtTypes) {
			if(this.debtType == null){
				this.debtType = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : debtTypes) {
				this.debtType.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public MultipleDebtTypesBuilder clearDebtType() {
			this.debtType = null;
			return this;
		}
		
		public MultipleDebtTypes build() {
			return new MultipleDebtTypes(this);
		}
	
		@Override
		public MultipleDebtTypesBuilder prune() {
			if (debtType!=null) debtType = debtType.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCondition()!=null) return true;
			if (getDebtType()!=null && !getDebtType().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("condition"), QuantifierEnum.class, condition, this);
			
			processRosetta(path.newSubPath("debtType"), processor, FieldWithMetaString.class, debtType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			MultipleDebtTypesBuilder _that = (MultipleDebtTypesBuilder) o;
		
			if (condition != null ? !condition.equals(_that.condition) : _that.condition != null) return false;
			if (!ListEquals.listEquals(debtType, _that.debtType)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (condition != null ? condition.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (debtType != null ? debtType.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MultipleDebtTypesBuilder {" +
				"condition=" + this.condition + ", " +
				"debtType=" + this.debtType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("condition"), QuantifierEnum.class, condition, this);
		
		processRosetta(path.newSubPath("debtType"), processor, FieldWithMetaString.class, debtType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		MultipleDebtTypes _that = (MultipleDebtTypes) o;
	
		if (condition != null ? !condition.equals(_that.condition) : _that.condition != null) return false;
		if (!ListEquals.listEquals(debtType, _that.debtType)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (condition != null ? condition.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (debtType != null ? debtType.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MultipleDebtTypes {" +
			"condition=" + this.condition + ", " +
			"debtType=" + this.debtType +
		'}';
	}
}
