package org.isda.cdm;

import java.lang.Integer;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MultipleValuationDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="MultipleValuationDates", source="FpML_5_10")
public class MultipleValuationDates extends SingleValuationDate {
	private final Integer businessDaysThereafter;
	private final Integer numberValuationDates;
	private static MultipleValuationDatesMeta metaData = new MultipleValuationDatesMeta();
	
	MultipleValuationDates(MultipleValuationDatesBuilder builder) {
		super(builder);
		this.businessDaysThereafter = builder.getBusinessDaysThereafter();
		this.numberValuationDates = builder.getNumberValuationDates();
	}
	
	/**
	 * The number of business days between successive valuation dates when multiple valuation dates are applicable for cash settlement. ISDA 2003 Term: Business Days thereafter.
	 */
	@RosettaSynonym(value="businessDaysThereafter", source="FpML_5_10")
	@RosettaSynonym(value="businessDaysThereafter", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDaysThereafter", source="DTCC_11_0")
	@RosettaSynonym(value="businessDaysThereafter", source="DTCC_9_0")
	@RosettaSynonym(value="businessDaysThereafter", source="CME_ClearedConfirm_1_17")
	public final Integer getBusinessDaysThereafter() {
		return businessDaysThereafter;
	}
	
	/**
	 * Where multiple valuation dates are specified as being applicable for cash settlement, this element specifies (a) the number of applicable valuation dates, and (b) the number of business days after satisfaction of all conditions to settlement when the first such valuation date occurs, and (c) the number of business days thereafter of each successive valuation date. ISDA 2003 Term: Multiple Valuation Dates.
	 */
	@RosettaSynonym(value="numberValuationDates", source="FpML_5_10")
	@RosettaSynonym(value="numberValuationDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="numberValuationDates", source="DTCC_11_0")
	@RosettaSynonym(value="numberValuationDates", source="DTCC_9_0")
	@RosettaSynonym(value="numberValuationDates", source="CME_ClearedConfirm_1_17")
	public final Integer getNumberValuationDates() {
		return numberValuationDates;
	}
	
	
	@Override
	public RosettaMetaData<? extends MultipleValuationDates> metaData() {
		return metaData;
	} 
	
	public MultipleValuationDatesBuilder toBuilder() {
		MultipleValuationDatesBuilder builder = new MultipleValuationDatesBuilder();
		ofNullable(getBusinessDaysThereafter()).ifPresent(builder::setBusinessDaysThereafter);
		ofNullable(getNumberValuationDates()).ifPresent(builder::setNumberValuationDates);
		ofNullable(getBusinessDays()).ifPresent(builder::setBusinessDays);
		return builder;
	}

	public static MultipleValuationDatesBuilder builder() {
		return new MultipleValuationDatesBuilder();
	}

	public static  class MultipleValuationDatesBuilder extends SingleValuationDateBuilder {
	
		protected Integer businessDaysThereafter;
		protected Integer numberValuationDates;
	
		public MultipleValuationDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MultipleValuationDates> metaData() {
			return metaData;
		} 
	
		
		public Integer getBusinessDaysThereafter() {
			return businessDaysThereafter;
		}
		
		
		public Integer getNumberValuationDates() {
			return numberValuationDates;
		}
		
	
		public MultipleValuationDatesBuilder setBusinessDaysThereafter(Integer businessDaysThereafter) {
			this.businessDaysThereafter = businessDaysThereafter;
			return this;
		}
		public MultipleValuationDatesBuilder setNumberValuationDates(Integer numberValuationDates) {
			this.numberValuationDates = numberValuationDates;
			return this;
		}
		@Override public MultipleValuationDatesBuilder setBusinessDays(Integer businessDays) {
			this.businessDays = businessDays;
			return this;
		}
		
		public MultipleValuationDates build() {
			return new MultipleValuationDates(this);
		}
	
		@Override
		public MultipleValuationDatesBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getBusinessDaysThereafter()!=null) return true;
			if (getNumberValuationDates()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("businessDaysThereafter"), Integer.class, businessDaysThereafter, this);
			processor.processBasic(path.newSubPath("numberValuationDates"), Integer.class, numberValuationDates, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			MultipleValuationDatesBuilder _that = (MultipleValuationDatesBuilder) o;
		
			if (businessDaysThereafter != null ? !businessDaysThereafter.equals(_that.businessDaysThereafter) : _that.businessDaysThereafter != null) return false;
			if (numberValuationDates != null ? !numberValuationDates.equals(_that.numberValuationDates) : _that.numberValuationDates != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (businessDaysThereafter != null ? businessDaysThereafter.hashCode() : 0);
			_result = 31 * _result + (numberValuationDates != null ? numberValuationDates.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MultipleValuationDatesBuilder {" +
				"businessDaysThereafter=" + this.businessDaysThereafter + ", " +
				"numberValuationDates=" + this.numberValuationDates +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("businessDaysThereafter"), Integer.class, businessDaysThereafter, this);
		processor.processBasic(path.newSubPath("numberValuationDates"), Integer.class, numberValuationDates, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		MultipleValuationDates _that = (MultipleValuationDates) o;
	
		if (businessDaysThereafter != null ? !businessDaysThereafter.equals(_that.businessDaysThereafter) : _that.businessDaysThereafter != null) return false;
		if (numberValuationDates != null ? !numberValuationDates.equals(_that.numberValuationDates) : _that.numberValuationDates != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (businessDaysThereafter != null ? businessDaysThereafter.hashCode() : 0);
		_result = 31 * _result + (numberValuationDates != null ? numberValuationDates.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "MultipleValuationDates {" +
			"businessDaysThereafter=" + this.businessDaysThereafter + ", " +
			"numberValuationDates=" + this.numberValuationDates +
		'}' + " " + super.toString();
	}
}
