package org.isda.cdm;



import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.MutualFundMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify a mutual fund as having a product identifier. As a difference versus the FpML standard, the CDM structure of this class only includes the productIdentifier class, which consists of an identifier, productTaxonomy, and source of the identifier. The reason for this approach is to avoid the potential for conflicting information between the information associated with the contractual product and the reference information maintained by the relevant service provider.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class MutualFund extends IdentifiedProduct {
	private static MutualFundMeta metaData = new MutualFundMeta();
	
	MutualFund(MutualFundBuilder builder) {
		super(builder);
	}
	
	
	@Override
	public RosettaMetaData<? extends MutualFund> metaData() {
		return metaData;
	} 
	
	public MutualFundBuilder toBuilder() {
		MutualFundBuilder builder = new MutualFundBuilder();
		ofNullable(getProductIdentifier()).ifPresent(builder::setProductIdentifier);
		return builder;
	}

	public static MutualFundBuilder builder() {
		return new MutualFundBuilder();
	}

	public static  class MutualFundBuilder extends IdentifiedProductBuilder {
	
	
		public MutualFundBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends MutualFund> metaData() {
			return metaData;
		} 
	
	
		@Override public MutualFundBuilder setProductIdentifier(ProductIdentifier productIdentifier) {
			this.productIdentifier = productIdentifier.toBuilder();
			return this;
		}
		@Override public MutualFundBuilder setProductIdentifierBuilder(ProductIdentifier.ProductIdentifierBuilder productIdentifier) {
			this.productIdentifier = productIdentifier;
			return this;
		}
		
		public MutualFund build() {
			return new MutualFund(this);
		}
	
		@Override
		public MutualFundBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			MutualFundBuilder _that = (MutualFundBuilder) o;
		
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			return _result;
		}
		
		@Override
		public String toString() {
			return "MutualFundBuilder {" +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		MutualFund _that = (MutualFund) o;
	
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		return _result;
	}
	
	@Override
	public String toString() {
		return "MutualFund {" +
		'}' + " " + super.toString();
	}
}
