package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.String;
import java.util.List;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NaturalPersonMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the attributes that are specific to a natural person.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Person", source="FpML_5_10")
@RosettaSynonym(value="Person", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Person", source="DTCC_11_0")
@RosettaSynonym(value="Person", source="DTCC_9_0")
@RosettaSynonym(value="Person", source="CME_ClearedConfirm_1_17")
public class NaturalPerson extends RosettaModelObject implements GlobalKey {
	private final Date dateOfBirth;
	private final String firstName;
	private final String honorific;
	private final List<String> initial;
	private final MetaFields meta;
	private final List<String> middleName;
	private final String suffix;
	private final String surname;
	private static NaturalPersonMeta metaData = new NaturalPersonMeta();
	
	NaturalPerson(NaturalPersonBuilder builder) {
		this.dateOfBirth = builder.getDateOfBirth();
		this.firstName = builder.getFirstName();
		this.honorific = builder.getHonorific();
		this.initial = builder.getInitial();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.middleName = builder.getMiddleName();
		this.suffix = builder.getSuffix();
		this.surname = builder.getSurname();
	}
	
	/**
	 * The natural person&#39;s date of birth.
	 */
	@RosettaSynonym(value="dateOfBirth", source="FpML_5_10")
	@RosettaSynonym(value="dateOfBirth", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="dateOfBirth", source="DTCC_11_0")
	@RosettaSynonym(value="dateOfBirth", source="DTCC_9_0")
	@RosettaSynonym(value="dateOfBirth", source="CME_ClearedConfirm_1_17")
	public final Date getDateOfBirth() {
		return dateOfBirth;
	}
	
	/**
	 * The natural person&#39;s first name. It is optional in FpML.
	 */
	@RosettaSynonym(value="firstName", source="FpML_5_10")
	@RosettaSynonym(value="firstName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="firstName", source="DTCC_11_0")
	@RosettaSynonym(value="firstName", source="DTCC_9_0")
	@RosettaSynonym(value="firstName", source="CME_ClearedConfirm_1_17")
	public final String getFirstName() {
		return firstName;
	}
	
	/**
	 * An honorific title, such as Mr., Ms., Dr. etc.
	 */
	@RosettaSynonym(value="honorific", source="FpML_5_10")
	@RosettaSynonym(value="honorific", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="honorific", source="DTCC_11_0")
	@RosettaSynonym(value="honorific", source="DTCC_9_0")
	@RosettaSynonym(value="honorific", source="CME_ClearedConfirm_1_17")
	public final String getHonorific() {
		return honorific;
	}
	
	@RosettaSynonym(value="initial", source="FpML_5_10")
	@RosettaSynonym(value="initial", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initial", source="DTCC_11_0")
	@RosettaSynonym(value="initial", source="DTCC_9_0")
	@RosettaSynonym(value="initial", source="CME_ClearedConfirm_1_17")
	public final List<String> getInitial() {
		return initial;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	@RosettaSynonym(value="middleName", source="FpML_5_10")
	@RosettaSynonym(value="middleName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="middleName", source="DTCC_11_0")
	@RosettaSynonym(value="middleName", source="DTCC_9_0")
	@RosettaSynonym(value="middleName", source="CME_ClearedConfirm_1_17")
	public final List<String> getMiddleName() {
		return middleName;
	}
	
	/**
	 * Name suffix, such as Jr., III, etc.
	 */
	@RosettaSynonym(value="suffix", source="FpML_5_10")
	@RosettaSynonym(value="suffix", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="suffix", source="DTCC_11_0")
	@RosettaSynonym(value="suffix", source="DTCC_9_0")
	@RosettaSynonym(value="suffix", source="CME_ClearedConfirm_1_17")
	public final String getSuffix() {
		return suffix;
	}
	
	/**
	 * The natural person&#39;s surname.
	 */
	@RosettaSynonym(value="surname", source="FpML_5_10")
	@RosettaSynonym(value="surname", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="surname", source="DTCC_11_0")
	@RosettaSynonym(value="surname", source="DTCC_9_0")
	@RosettaSynonym(value="surname", source="CME_ClearedConfirm_1_17")
	public final String getSurname() {
		return surname;
	}
	
	
	@Override
	public RosettaMetaData<? extends NaturalPerson> metaData() {
		return metaData;
	} 
	
	public NaturalPersonBuilder toBuilder() {
		NaturalPersonBuilder builder = new NaturalPersonBuilder();
		ofNullable(getDateOfBirth()).ifPresent(builder::setDateOfBirth);
		ofNullable(getFirstName()).ifPresent(builder::setFirstName);
		ofNullable(getHonorific()).ifPresent(builder::setHonorific);
		ofNullable(getInitial()).ifPresent(initial -> initial.forEach(builder::addInitial));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getMiddleName()).ifPresent(middleName -> middleName.forEach(builder::addMiddleName));
		ofNullable(getSuffix()).ifPresent(builder::setSuffix);
		ofNullable(getSurname()).ifPresent(builder::setSurname);
		return builder;
	}

	public static NaturalPersonBuilder builder() {
		return new NaturalPersonBuilder();
	}

	public static  class NaturalPersonBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<NaturalPersonBuilder> {
	
		protected Date dateOfBirth;
		protected String firstName;
		protected String honorific;
		protected List<String> initial;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<String> middleName;
		protected String suffix;
		protected String surname;
	
		public NaturalPersonBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NaturalPerson> metaData() {
			return metaData;
		} 
	
		
		public Date getDateOfBirth() {
			return dateOfBirth;
		}
		
		
		public String getFirstName() {
			return firstName;
		}
		
		
		public String getHonorific() {
			return honorific;
		}
		
		
		public List<String> getInitial() {
			return initial;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<String> getMiddleName() {
			return middleName;
		}
		
		
		public String getSuffix() {
			return suffix;
		}
		
		
		public String getSurname() {
			return surname;
		}
		
	
		public NaturalPersonBuilder setDateOfBirth(Date dateOfBirth) {
			this.dateOfBirth = dateOfBirth;
			return this;
		}
		public NaturalPersonBuilder setFirstName(String firstName) {
			this.firstName = firstName;
			return this;
		}
		public NaturalPersonBuilder setHonorific(String honorific) {
			this.honorific = honorific;
			return this;
		}
		public NaturalPersonBuilder addInitial(String initial) {
			if(this.initial == null){
				this.initial = new ArrayList<>();
			}
			this.initial.add(initial);
			return this;
		}
		public NaturalPersonBuilder addInitial(String initial, int _idx) {
			if(this.initial == null){
				this.initial = new ArrayList<>();
			}
			getIndex(this.initial, _idx, () -> initial);
			this.initial.set(_idx, initial);
			return this;
		}
		public NaturalPersonBuilder addInitial(List<String> initials) {
			if(this.initial == null){
				this.initial = new ArrayList<>();
			}
			for (String toAdd : initials) {
				this.initial.add(toAdd);
			}
			return this;
		}
		
		public NaturalPersonBuilder clearInitial() {
			this.initial = null;
			return this;
		}
		public NaturalPersonBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public NaturalPersonBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public NaturalPersonBuilder addMiddleName(String middleName) {
			if(this.middleName == null){
				this.middleName = new ArrayList<>();
			}
			this.middleName.add(middleName);
			return this;
		}
		public NaturalPersonBuilder addMiddleName(String middleName, int _idx) {
			if(this.middleName == null){
				this.middleName = new ArrayList<>();
			}
			getIndex(this.middleName, _idx, () -> middleName);
			this.middleName.set(_idx, middleName);
			return this;
		}
		public NaturalPersonBuilder addMiddleName(List<String> middleNames) {
			if(this.middleName == null){
				this.middleName = new ArrayList<>();
			}
			for (String toAdd : middleNames) {
				this.middleName.add(toAdd);
			}
			return this;
		}
		
		public NaturalPersonBuilder clearMiddleName() {
			this.middleName = null;
			return this;
		}
		public NaturalPersonBuilder setSuffix(String suffix) {
			this.suffix = suffix;
			return this;
		}
		public NaturalPersonBuilder setSurname(String surname) {
			this.surname = surname;
			return this;
		}
		
		public NaturalPerson build() {
			return new NaturalPerson(this);
		}
	
		@Override
		public NaturalPersonBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDateOfBirth()!=null) return true;
			if (getFirstName()!=null) return true;
			if (getHonorific()!=null) return true;
			if (getInitial()!=null && !getInitial().isEmpty()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getMiddleName()!=null && !getMiddleName().isEmpty()) return true;
			if (getSuffix()!=null) return true;
			if (getSurname()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("dateOfBirth"), Date.class, dateOfBirth, this);
			processor.processBasic(path.newSubPath("firstName"), String.class, firstName, this);
			processor.processBasic(path.newSubPath("honorific"), String.class, honorific, this);
			processor.processBasic(path.newSubPath("initial"), String.class, initial, this);
			processor.processBasic(path.newSubPath("middleName"), String.class, middleName, this);
			processor.processBasic(path.newSubPath("suffix"), String.class, suffix, this);
			processor.processBasic(path.newSubPath("surname"), String.class, surname, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NaturalPersonBuilder _that = (NaturalPersonBuilder) o;
		
			if (dateOfBirth != null ? !dateOfBirth.equals(_that.dateOfBirth) : _that.dateOfBirth != null) return false;
			if (firstName != null ? !firstName.equals(_that.firstName) : _that.firstName != null) return false;
			if (honorific != null ? !honorific.equals(_that.honorific) : _that.honorific != null) return false;
			if (!ListEquals.listEquals(initial, _that.initial)) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(middleName, _that.middleName)) return false;
			if (suffix != null ? !suffix.equals(_that.suffix) : _that.suffix != null) return false;
			if (surname != null ? !surname.equals(_that.surname) : _that.surname != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (dateOfBirth != null ? dateOfBirth.hashCode() : 0);
			_result = 31 * _result + (firstName != null ? firstName.hashCode() : 0);
			_result = 31 * _result + (honorific != null ? honorific.hashCode() : 0);
			_result = 31 * _result + (initial != null ? initial.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (middleName != null ? middleName.hashCode() : 0);
			_result = 31 * _result + (suffix != null ? suffix.hashCode() : 0);
			_result = 31 * _result + (surname != null ? surname.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NaturalPersonBuilder {" +
				"dateOfBirth=" + this.dateOfBirth + ", " +
				"firstName=" + this.firstName + ", " +
				"honorific=" + this.honorific + ", " +
				"initial=" + this.initial + ", " +
				"meta=" + this.meta + ", " +
				"middleName=" + this.middleName + ", " +
				"suffix=" + this.suffix + ", " +
				"surname=" + this.surname +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("dateOfBirth"), Date.class, dateOfBirth, this);
		processor.processBasic(path.newSubPath("firstName"), String.class, firstName, this);
		processor.processBasic(path.newSubPath("honorific"), String.class, honorific, this);
		initial.stream().forEach(a->processor.processBasic(path.newSubPath("initial"), String.class, a, this));
		middleName.stream().forEach(a->processor.processBasic(path.newSubPath("middleName"), String.class, a, this));
		processor.processBasic(path.newSubPath("suffix"), String.class, suffix, this);
		processor.processBasic(path.newSubPath("surname"), String.class, surname, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NaturalPerson _that = (NaturalPerson) o;
	
		if (dateOfBirth != null ? !dateOfBirth.equals(_that.dateOfBirth) : _that.dateOfBirth != null) return false;
		if (firstName != null ? !firstName.equals(_that.firstName) : _that.firstName != null) return false;
		if (honorific != null ? !honorific.equals(_that.honorific) : _that.honorific != null) return false;
		if (!ListEquals.listEquals(initial, _that.initial)) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(middleName, _that.middleName)) return false;
		if (suffix != null ? !suffix.equals(_that.suffix) : _that.suffix != null) return false;
		if (surname != null ? !surname.equals(_that.surname) : _that.surname != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (dateOfBirth != null ? dateOfBirth.hashCode() : 0);
		_result = 31 * _result + (firstName != null ? firstName.hashCode() : 0);
		_result = 31 * _result + (honorific != null ? honorific.hashCode() : 0);
		_result = 31 * _result + (initial != null ? initial.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (middleName != null ? middleName.hashCode() : 0);
		_result = 31 * _result + (suffix != null ? suffix.hashCode() : 0);
		_result = 31 * _result + (surname != null ? surname.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NaturalPerson {" +
			"dateOfBirth=" + this.dateOfBirth + ", " +
			"firstName=" + this.firstName + ", " +
			"honorific=" + this.honorific + ", " +
			"initial=" + this.initial + ", " +
			"meta=" + this.meta + ", " +
			"middleName=" + this.middleName + ", " +
			"suffix=" + this.suffix + ", " +
			"surname=" + this.surname +
		'}';
	}
}
