package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values for the natural person&#39;s role.
 * 
 * @version 2.5.4
 */
public enum NaturalPersonRoleEnum {
    

	/**
	 * The person who arranged with a client to execute the trade.
	 */
	@RosettaSynonym(value = "Broker", source = "FpML_5_10")
	@RosettaSynonym(value = "Broker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Broker", source = "DTCC_11_0")
	@RosettaSynonym(value = "Broker", source = "DTCC_9_0")
	@RosettaSynonym(value = "Broker", source = "CME_ClearedConfirm_1_17")
	BROKER,
	    

	/**
	 * Acquirer of the legal title to the financial instrument.
	 */
	@RosettaSynonym(value = "Buyer", source = "FpML_5_10")
	@RosettaSynonym(value = "Buyer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Buyer", source = "DTCC_11_0")
	@RosettaSynonym(value = "Buyer", source = "DTCC_9_0")
	@RosettaSynonym(value = "Buyer", source = "CME_ClearedConfirm_1_17")
	BUYER,
	    

	/**
	 * The party or person with legal responsibility for authorization of the execution of the transaction.
	 */
	@RosettaSynonym(value = "DecisionMaker", source = "FpML_5_10")
	@RosettaSynonym(value = "DecisionMaker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DecisionMaker", source = "DTCC_11_0")
	@RosettaSynonym(value = "DecisionMaker", source = "DTCC_9_0")
	@RosettaSynonym(value = "DecisionMaker", source = "CME_ClearedConfirm_1_17")
	DECISION_MAKER,
	    

	/**
	 * Person within the firm who is responsible for execution of the transaction.
	 */
	@RosettaSynonym(value = "ExecutionWithinFirm", source = "FpML_5_10")
	@RosettaSynonym(value = "ExecutionWithinFirm", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExecutionWithinFirm", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExecutionWithinFirm", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExecutionWithinFirm", source = "CME_ClearedConfirm_1_17")
	EXECUTION_WITHIN_FIRM,
	    

	/**
	 * Person who is responsible for making the investment decision.
	 */
	@RosettaSynonym(value = "InvestmentDecisionMaker", source = "FpML_5_10")
	@RosettaSynonym(value = "InvestmentDecisionMaker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "InvestmentDecisionMaker", source = "DTCC_11_0")
	@RosettaSynonym(value = "InvestmentDecisionMaker", source = "DTCC_9_0")
	@RosettaSynonym(value = "InvestmentDecisionMaker", source = "CME_ClearedConfirm_1_17")
	INVESTMENT_DECISION_MAKER,
	    

	/**
	 * Seller of the legal title to the financial instrument.
	 */
	@RosettaSynonym(value = "Seller", source = "FpML_5_10")
	@RosettaSynonym(value = "Seller", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Seller", source = "DTCC_11_0")
	@RosettaSynonym(value = "Seller", source = "DTCC_9_0")
	@RosettaSynonym(value = "Seller", source = "CME_ClearedConfirm_1_17")
	SELLER,
	    

	/**
	 * The person who executed the trade.
	 */
	@RosettaSynonym(value = "Trader", source = "FpML_5_10")
	@RosettaSynonym(value = "Trader", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Trader", source = "DTCC_11_0")
	@RosettaSynonym(value = "Trader", source = "DTCC_9_0")
	@RosettaSynonym(value = "Trader", source = "CME_ClearedConfirm_1_17")
	TRADER    
	;

	private final String displayName;
	
	NaturalPersonRoleEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
