package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the method of calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).
 * 
 * @version 2.5.4
 */
public enum NegativeInterestRateTreatmentEnum {
    

	/**
	 * Negative Interest Rate Method. Per 2000 ISDA Definitions, Section 6.4 Negative Interest Rates, paragraphs (b) and (c).
	 */
	@RosettaSynonym(value = "NegativeInterestRateMethod", source = "FpML_5_10")
	@RosettaSynonym(value = "NegativeInterestRateMethod", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NegativeInterestRateMethod", source = "DTCC_11_0")
	@RosettaSynonym(value = "NegativeInterestRateMethod", source = "DTCC_9_0")
	@RosettaSynonym(value = "NegativeInterestRateMethod", source = "CME_ClearedConfirm_1_17")
	NEGATIVE_INTEREST_RATE_METHOD,
	    

	/**
	 * Zero Interest Rate Method. Per 2000 ISDA Definitions, Section 6.4. Negative Interest Rates, paragraphs (d) and (e).
	 */
	@RosettaSynonym(value = "ZeroInterestRateMethod", source = "FpML_5_10")
	@RosettaSynonym(value = "ZeroInterestRateMethod", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZeroInterestRateMethod", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZeroInterestRateMethod", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZeroInterestRateMethod", source = "CME_ClearedConfirm_1_17")
	ZERO_INTEREST_RATE_METHOD    
	;

	private final String displayName;
	
	NegativeInterestRateTreatmentEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
