package org.isda.cdm;

import org.isda.cdm.RefRate;
import org.isda.cdm.Term;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NmMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Nm extends RosettaModelObject {
	private final RefRate refRate;
	private final Term term;
	private static NmMeta metaData = new NmMeta();
	
	Nm(NmBuilder builder) {
		this.refRate = ofNullable(builder.getRefRate()).map(f->f.build()).orElse(null);
		this.term = ofNullable(builder.getTerm()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="RefRate", source="ISO20022")
	public final RefRate getRefRate() {
		return refRate;
	}
	
	@RosettaSynonym(value="Term", source="ISO20022")
	public final Term getTerm() {
		return term;
	}
	
	
	@Override
	public RosettaMetaData<? extends Nm> metaData() {
		return metaData;
	} 
	
	public NmBuilder toBuilder() {
		NmBuilder builder = new NmBuilder();
		ofNullable(getRefRate()).ifPresent(builder::setRefRate);
		ofNullable(getTerm()).ifPresent(builder::setTerm);
		return builder;
	}

	public static NmBuilder builder() {
		return new NmBuilder();
	}

	public static  class NmBuilder extends RosettaModelObjectBuilder {
	
		protected RefRate.RefRateBuilder refRate;
		protected Term.TermBuilder term;
	
		public NmBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Nm> metaData() {
			return metaData;
		} 
	
		
		public RefRate.RefRateBuilder getRefRate() {
			return refRate;
		}
		
		public RefRate.RefRateBuilder getOrCreateRefRate() {
			if (refRate!=null) {
				return refRate;
			}
			else return refRate = new RefRate.RefRateBuilder();
		}
		
		
		public Term.TermBuilder getTerm() {
			return term;
		}
		
		public Term.TermBuilder getOrCreateTerm() {
			if (term!=null) {
				return term;
			}
			else return term = new Term.TermBuilder();
		}
		
	
		public NmBuilder setRefRate(RefRate refRate) {
			this.refRate = refRate.toBuilder();
			return this;
		}
		public NmBuilder setRefRateBuilder(RefRate.RefRateBuilder refRate) {
			this.refRate = refRate;
			return this;
		}
		public NmBuilder setTerm(Term term) {
			this.term = term.toBuilder();
			return this;
		}
		public NmBuilder setTermBuilder(Term.TermBuilder term) {
			this.term = term;
			return this;
		}
		
		public Nm build() {
			return new Nm(this);
		}
	
		@Override
		public NmBuilder prune() {
			if (refRate!=null && !refRate.prune().hasData()) refRate = null;
			if (term!=null && !term.prune().hasData()) term = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getRefRate()!=null && getRefRate().hasData()) return true;
			if (getTerm()!=null && getTerm().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("refRate"), processor, RefRate.class, refRate);
			processRosetta(path.newSubPath("term"), processor, Term.class, term);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NmBuilder _that = (NmBuilder) o;
		
			if (refRate != null ? !refRate.equals(_that.refRate) : _that.refRate != null) return false;
			if (term != null ? !term.equals(_that.term) : _that.term != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (refRate != null ? refRate.hashCode() : 0);
			_result = 31 * _result + (term != null ? term.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NmBuilder {" +
				"refRate=" + this.refRate + ", " +
				"term=" + this.term +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("refRate"), processor, RefRate.class, refRate);
		processRosetta(path.newSubPath("term"), processor, Term.class, term);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Nm _that = (Nm) o;
	
		if (refRate != null ? !refRate.equals(_that.refRate) : _that.refRate != null) return false;
		if (term != null ? !term.equals(_that.term) : _that.term != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (refRate != null ? refRate.hashCode() : 0);
		_result = 31 * _result + (term != null ? term.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Nm {" +
			"refRate=" + this.refRate + ", " +
			"term=" + this.term +
		'}';
	}
}
