package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NonNegativeQuantityMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * Class to specify a quantity as a non-negative number, which condition is enforced through a data rule that only applies to the extending class.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NonNegativeQuantity extends Quantity {
	private static NonNegativeQuantityMeta metaData = new NonNegativeQuantityMeta();
	
	NonNegativeQuantity(NonNegativeQuantityBuilder builder) {
		super(builder);
	}
	
	
	@Override
	public RosettaMetaData<? extends NonNegativeQuantity> metaData() {
		return metaData;
	} 
	
	public NonNegativeQuantityBuilder toBuilder() {
		NonNegativeQuantityBuilder builder = new NonNegativeQuantityBuilder();
		ofNullable(getAmount()).ifPresent(builder::setAmount);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		return builder;
	}

	public static NonNegativeQuantityBuilder builder() {
		return new NonNegativeQuantityBuilder();
	}

	public static  class NonNegativeQuantityBuilder extends QuantityBuilder {
	
	
		public NonNegativeQuantityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NonNegativeQuantity> metaData() {
			return metaData;
		} 
	
	
		@Override public NonNegativeQuantityBuilder setAmount(BigDecimal amount) {
			this.amount = amount;
			return this;
		}
		@Override public NonNegativeQuantityBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		@Override public NonNegativeQuantityBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		@Override public NonNegativeQuantityBuilder setUnit(UnitEnum unit) {
			this.unit = unit;
			return this;
		}
		
		public NonNegativeQuantity build() {
			return new NonNegativeQuantity(this);
		}
	
		@Override
		public NonNegativeQuantityBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			NonNegativeQuantityBuilder _that = (NonNegativeQuantityBuilder) o;
		
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			return _result;
		}
		
		@Override
		public String toString() {
			return "NonNegativeQuantityBuilder {" +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		NonNegativeQuantity _that = (NonNegativeQuantity) o;
	
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		return _result;
	}
	
	@Override
	public String toString() {
		return "NonNegativeQuantity {" +
		'}' + " " + super.toString();
	}
}
