package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.NotificationTimeElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NotificationTimeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the time by which a demand for the Transfer of Eligible Credit Support (IM) or Posted Credit Support (IM) needs to be made in order for the transfer to take place in accordance with the Transfer Timing provisions. ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (d)(iii): Notification Time. | ISDA 2016 Japanese Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (e)(iii): Notification Time. | ISDA 2016 New York Law Credit Support Annex for Initial Margin, paragraph 13, General Principles, (d)(iii): Notification Time.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class NotificationTime extends RosettaModelObject {
	private final List<NotificationTimeElection> partyElections;
	private static NotificationTimeMeta metaData = new NotificationTimeMeta();
	
	NotificationTime(NotificationTimeBuilder builder) {
		this.partyElections = ofNullable(builder.getPartyElections()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; Notification Time election.
	 */
	public final List<NotificationTimeElection> getPartyElections() {
		return partyElections;
	}
	
	
	@Override
	public RosettaMetaData<? extends NotificationTime> metaData() {
		return metaData;
	} 
	
	public NotificationTimeBuilder toBuilder() {
		NotificationTimeBuilder builder = new NotificationTimeBuilder();
		ofNullable(getPartyElections()).ifPresent(partyElections -> partyElections.forEach(builder::addPartyElections));
		return builder;
	}

	public static NotificationTimeBuilder builder() {
		return new NotificationTimeBuilder();
	}

	public static  class NotificationTimeBuilder extends RosettaModelObjectBuilder {
	
		protected List<NotificationTimeElection.NotificationTimeElectionBuilder> partyElections;
	
		public NotificationTimeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NotificationTime> metaData() {
			return metaData;
		} 
	
		
		public List<NotificationTimeElection.NotificationTimeElectionBuilder> getPartyElections() {
			return partyElections;
		}
		
		public NotificationTimeElection.NotificationTimeElectionBuilder getOrCreatePartyElections(int index) {
			if (partyElections==null) {
				this.partyElections = new ArrayList<>();
			}
			return getIndex(partyElections, index, ()->new NotificationTimeElection.NotificationTimeElectionBuilder());
		}
		
	
		public NotificationTimeBuilder addPartyElections(NotificationTimeElection partyElections) {
			if(this.partyElections == null){
				this.partyElections = new ArrayList<>();
			}
			this.partyElections.add(partyElections.toBuilder());
			return this;
		}
		public NotificationTimeBuilder addPartyElections(NotificationTimeElection partyElections, int _idx) {
			if(this.partyElections == null){
				this.partyElections = new ArrayList<>();
			}
			getIndex(this.partyElections, _idx, () -> partyElections.toBuilder());
			this.partyElections.set(_idx, partyElections.toBuilder());
			return this;
		}
		public NotificationTimeBuilder addPartyElections(List<NotificationTimeElection> partyElectionss) {
			if(this.partyElections == null){
				this.partyElections = new ArrayList<>();
			}
			for (NotificationTimeElection toAdd : partyElectionss) {
				this.partyElections.add(toAdd.toBuilder());
			}
			return this;
		}
		public NotificationTimeBuilder addPartyElectionsBuilder(NotificationTimeElection.NotificationTimeElectionBuilder partyElections) {
			if(this.partyElections == null){
				this.partyElections = new ArrayList<>();
			}
			this.partyElections.add(partyElections);
			return this;
		}
		
		public NotificationTimeBuilder clearPartyElections() {
			this.partyElections = null;
			return this;
		}
		
		public NotificationTime build() {
			return new NotificationTime(this);
		}
	
		@Override
		public NotificationTimeBuilder prune() {
			if (partyElections!=null) partyElections = partyElections.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElections()!=null && getPartyElections().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElections"), processor, NotificationTimeElection.class, partyElections);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NotificationTimeBuilder _that = (NotificationTimeBuilder) o;
		
			if (!ListEquals.listEquals(partyElections, _that.partyElections)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElections != null ? partyElections.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NotificationTimeBuilder {" +
				"partyElections=" + this.partyElections +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElections"), processor, NotificationTimeElection.class, partyElections);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NotificationTime _that = (NotificationTime) o;
	
		if (!ListEquals.listEquals(partyElections, _that.partyElections)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElections != null ? partyElections.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NotificationTime {" +
			"partyElections=" + this.partyElections +
		'}';
	}
}
