package org.isda.cdm;

import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.NotifyingPartyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="NotifyingParty", source="FpML_5_10")
public class NotifyingParty extends RosettaModelObject {
	private final ReferenceWithMetaParty buyerPartyReference;
	private final ReferenceWithMetaParty sellerPartyReference;
	private static NotifyingPartyMeta metaData = new NotifyingPartyMeta();
	
	NotifyingParty(NotifyingPartyBuilder builder) {
		this.buyerPartyReference = ofNullable(builder.getBuyerPartyReference()).map(f->f.build()).orElse(null);
		this.sellerPartyReference = ofNullable(builder.getSellerPartyReference()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="buyerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="buyerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="buyerPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getBuyerPartyReference() {
		return buyerPartyReference;
	}
	
	@RosettaSynonym(value="sellerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="sellerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="sellerPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getSellerPartyReference() {
		return sellerPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends NotifyingParty> metaData() {
		return metaData;
	} 
	
	public NotifyingPartyBuilder toBuilder() {
		NotifyingPartyBuilder builder = new NotifyingPartyBuilder();
		ofNullable(getBuyerPartyReference()).ifPresent(builder::setBuyerPartyReference);
		ofNullable(getSellerPartyReference()).ifPresent(builder::setSellerPartyReference);
		return builder;
	}

	public static NotifyingPartyBuilder builder() {
		return new NotifyingPartyBuilder();
	}

	public static  class NotifyingPartyBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaPartyBuilder buyerPartyReference;
		protected ReferenceWithMetaPartyBuilder sellerPartyReference;
	
		public NotifyingPartyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends NotifyingParty> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaPartyBuilder getBuyerPartyReference() {
			return buyerPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateBuyerPartyReference() {
			if (buyerPartyReference!=null) {
				return buyerPartyReference;
			}
			else return buyerPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getSellerPartyReference() {
			return sellerPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateSellerPartyReference() {
			if (sellerPartyReference!=null) {
				return sellerPartyReference;
			}
			else return sellerPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public NotifyingPartyBuilder setBuyerPartyReference(ReferenceWithMetaParty buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference.toBuilder();
			return this;
		}
		public NotifyingPartyBuilder setBuyerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder buyerPartyReference) {
			this.buyerPartyReference = buyerPartyReference;
			return this;
		}
		public NotifyingPartyBuilder setBuyerPartyReferenceRef(Party.PartyBuilder buyerPartyReference) {
			return setBuyerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(buyerPartyReference).build());
		}
		public NotifyingPartyBuilder setBuyerPartyReferenceRef(Party buyerPartyReference) {
			return setBuyerPartyReferenceRef(Party.builder());
		}
		public NotifyingPartyBuilder setSellerPartyReference(ReferenceWithMetaParty sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference.toBuilder();
			return this;
		}
		public NotifyingPartyBuilder setSellerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder sellerPartyReference) {
			this.sellerPartyReference = sellerPartyReference;
			return this;
		}
		public NotifyingPartyBuilder setSellerPartyReferenceRef(Party.PartyBuilder sellerPartyReference) {
			return setSellerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(sellerPartyReference).build());
		}
		public NotifyingPartyBuilder setSellerPartyReferenceRef(Party sellerPartyReference) {
			return setSellerPartyReferenceRef(Party.builder());
		}
		
		public NotifyingParty build() {
			return new NotifyingParty(this);
		}
	
		@Override
		public NotifyingPartyBuilder prune() {
			if (buyerPartyReference!=null && !buyerPartyReference.prune().hasData()) buyerPartyReference = null;
			if (sellerPartyReference!=null && !sellerPartyReference.prune().hasData()) sellerPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBuyerPartyReference()!=null && getBuyerPartyReference().hasData()) return true;
			if (getSellerPartyReference()!=null && getSellerPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("buyerPartyReference"), processor, ReferenceWithMetaParty.class, buyerPartyReference);
			processRosetta(path.newSubPath("sellerPartyReference"), processor, ReferenceWithMetaParty.class, sellerPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			NotifyingPartyBuilder _that = (NotifyingPartyBuilder) o;
		
			if (buyerPartyReference != null ? !buyerPartyReference.equals(_that.buyerPartyReference) : _that.buyerPartyReference != null) return false;
			if (sellerPartyReference != null ? !sellerPartyReference.equals(_that.sellerPartyReference) : _that.sellerPartyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (buyerPartyReference != null ? buyerPartyReference.hashCode() : 0);
			_result = 31 * _result + (sellerPartyReference != null ? sellerPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "NotifyingPartyBuilder {" +
				"buyerPartyReference=" + this.buyerPartyReference + ", " +
				"sellerPartyReference=" + this.sellerPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("buyerPartyReference"), processor, ReferenceWithMetaParty.class, buyerPartyReference);
		processRosetta(path.newSubPath("sellerPartyReference"), processor, ReferenceWithMetaParty.class, sellerPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		NotifyingParty _that = (NotifyingParty) o;
	
		if (buyerPartyReference != null ? !buyerPartyReference.equals(_that.buyerPartyReference) : _that.buyerPartyReference != null) return false;
		if (sellerPartyReference != null ? !sellerPartyReference.equals(_that.sellerPartyReference) : _that.sellerPartyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (buyerPartyReference != null ? buyerPartyReference.hashCode() : 0);
		_result = 31 * _result + (sellerPartyReference != null ? sellerPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "NotifyingParty {" +
			"buyerPartyReference=" + this.buyerPartyReference + ", " +
			"sellerPartyReference=" + this.sellerPartyReference +
		'}';
	}
}
