package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the conditions that govern the adjustment to the number of units of the return swap.
 * 
 * @version 2.5.4
 */
public enum NotionalAdjustmentEnum {
    

	/**
	 * The adjustments to the number of units are governed by an execution clause.
	 */
	@RosettaSynonym(value = "Execution", source = "FpML_5_10")
	EXECUTION,
	    

	/**
	 * The adjustments to the number of units are governed by a portfolio rebalancing clause.
	 */
	@RosettaSynonym(value = "PortfolioRebalancing", source = "FpML_5_10")
	PORTFOLIO_REBALANCING,
	    

	/**
	 * The adjustments to the number of units are not governed by any specific clause.
	 */
	@RosettaSynonym(value = "Standard", source = "FpML_5_10")
	STANDARD    
	;

	private final String displayName;
	
	NotionalAdjustmentEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
