package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values used in both the obligations and deliverable obligations of the credit default swap to represent a class or type of securities which apply.
 * 
 * @version 2.5.4
 */
public enum ObligationCategoryEnum {
    

	/**
	 * ISDA term &#39;Payment&#39;.
	 */
	@RosettaSynonym(value = "Payment", source = "FpML_5_10")
	@RosettaSynonym(value = "Payment", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Payment", source = "DTCC_11_0")
	@RosettaSynonym(value = "Payment", source = "DTCC_9_0")
	@RosettaSynonym(value = "Payment", source = "CME_ClearedConfirm_1_17")
	PAYMENT,
	    

	/**
	 * ISDA term &#39;Borrowed Money&#39;.
	 */
	@RosettaSynonym(value = "BorrowedMoney", source = "FpML_5_10")
	@RosettaSynonym(value = "BorrowedMoney", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BorrowedMoney", source = "DTCC_11_0")
	@RosettaSynonym(value = "BorrowedMoney", source = "DTCC_9_0")
	@RosettaSynonym(value = "BorrowedMoney", source = "CME_ClearedConfirm_1_17")
	BORROWED_MONEY,
	    

	/**
	 * ISDA term &#39;Reference Obligations Only&#39;.
	 */
	@RosettaSynonym(value = "ReferenceObligationsOnly", source = "FpML_5_10")
	@RosettaSynonym(value = "ReferenceObligationsOnly", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ReferenceObligationsOnly", source = "DTCC_11_0")
	@RosettaSynonym(value = "ReferenceObligationsOnly", source = "DTCC_9_0")
	@RosettaSynonym(value = "ReferenceObligationsOnly", source = "CME_ClearedConfirm_1_17")
	REFERENCE_OBLIGATIONS_ONLY,
	    

	/**
	 * ISDA term &#39;Bond&#39;.
	 */
	@RosettaSynonym(value = "Bond", source = "FpML_5_10")
	@RosettaSynonym(value = "Bond", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Bond", source = "DTCC_11_0")
	@RosettaSynonym(value = "Bond", source = "DTCC_9_0")
	@RosettaSynonym(value = "Bond", source = "CME_ClearedConfirm_1_17")
	BOND,
	    

	/**
	 * ISDA term &#39;Loan&#39;.
	 */
	@RosettaSynonym(value = "Loan", source = "FpML_5_10")
	@RosettaSynonym(value = "Loan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Loan", source = "DTCC_11_0")
	@RosettaSynonym(value = "Loan", source = "DTCC_9_0")
	@RosettaSynonym(value = "Loan", source = "CME_ClearedConfirm_1_17")
	LOAN,
	    

	/**
	 * ISDA term &#39;Bond or Loan&#39;.
	 */
	@RosettaSynonym(value = "BondOrLoan", source = "FpML_5_10")
	@RosettaSynonym(value = "BondOrLoan", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BondOrLoan", source = "DTCC_11_0")
	@RosettaSynonym(value = "BondOrLoan", source = "DTCC_9_0")
	@RosettaSynonym(value = "BondOrLoan", source = "CME_ClearedConfirm_1_17")
	BOND_OR_LOAN    
	;

	private final String displayName;
	
	ObligationCategoryEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
