package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.ReferenceWithMetaPostingObligationsElection;
import org.isda.cdm.metafields.ReferenceWithMetaPostingObligationsElection.ReferenceWithMetaPostingObligationsElectionBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ObligorPostingObligationsMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the obligor(s) collateral posting obligations in accordance with the terms of the Japanese Law ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles, (ii).
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ObligorPostingObligations extends RosettaModelObject {
	private final List<ReferenceWithMetaPostingObligationsElection> partyElection;
	private static ObligorPostingObligationsMeta metaData = new ObligorPostingObligationsMeta();
	
	ObligorPostingObligations(ObligorPostingObligationsBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The obligor party(ies) to which the posting obligations apply to, which can be either one or both parties to the legal agreement.
	 */
	@RosettaSynonym(value="obligor", source="ISDA_Create_1_0")
	public final List<ReferenceWithMetaPostingObligationsElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends ObligorPostingObligations> metaData() {
		return metaData;
	} 
	
	public ObligorPostingObligationsBuilder toBuilder() {
		ObligorPostingObligationsBuilder builder = new ObligorPostingObligationsBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static ObligorPostingObligationsBuilder builder() {
		return new ObligorPostingObligationsBuilder();
	}

	public static  class ObligorPostingObligationsBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferenceWithMetaPostingObligationsElectionBuilder> partyElection;
	
		public ObligorPostingObligationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ObligorPostingObligations> metaData() {
			return metaData;
		} 
	
		
		public List<ReferenceWithMetaPostingObligationsElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public ReferenceWithMetaPostingObligationsElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new ReferenceWithMetaPostingObligationsElectionBuilder());
		}
		
	
		public ObligorPostingObligationsBuilder addPartyElection(ReferenceWithMetaPostingObligationsElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public ObligorPostingObligationsBuilder addPartyElection(ReferenceWithMetaPostingObligationsElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public ObligorPostingObligationsBuilder addPartyElection(List<ReferenceWithMetaPostingObligationsElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (ReferenceWithMetaPostingObligationsElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public ObligorPostingObligationsBuilder addPartyElectionBuilder(ReferenceWithMetaPostingObligationsElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public ObligorPostingObligationsBuilder addPartyElectionRef(PostingObligationsElection.PostingObligationsElectionBuilder partyElection) {
			return addPartyElection(ReferenceWithMetaPostingObligationsElection.builder().setValueBuilder(partyElection).build());
		}
		
		public ObligorPostingObligationsBuilder addPartyElectionRef(PostingObligationsElection.PostingObligationsElectionBuilder partyElection, int _idx) {
			return addPartyElection(ReferenceWithMetaPostingObligationsElection.builder().setValueBuilder(partyElection).build(), _idx);
		}
		
		public ObligorPostingObligationsBuilder addPartyElectionRef(List<PostingObligationsElection> partyElections) {
			for (PostingObligationsElection toAdd : partyElections) {
				addPartyElectionRef(toAdd);
			}
			return this;
		}
		
		public ObligorPostingObligationsBuilder addPartyElectionRef(PostingObligationsElection partyElection) {
			return addPartyElectionRef(partyElection.toBuilder());
		}
		
		public ObligorPostingObligationsBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public ObligorPostingObligations build() {
			return new ObligorPostingObligations(this);
		}
	
		@Override
		public ObligorPostingObligationsBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, ReferenceWithMetaPostingObligationsElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ObligorPostingObligationsBuilder _that = (ObligorPostingObligationsBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ObligorPostingObligationsBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, ReferenceWithMetaPostingObligationsElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ObligorPostingObligations _that = (ObligorPostingObligations) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ObligorPostingObligations {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
