package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionDenominationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * Class which corresponds to the FpML OptionDenomination.model group.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OptionDenomination extends RosettaModelObject {
	private final FieldWithMetaString entitlementCurrency;
	private final BigDecimal numberOfOptions;
	private final BigDecimal optionEntitlement;
	private static OptionDenominationMeta metaData = new OptionDenominationMeta();
	
	OptionDenomination(OptionDenominationBuilder builder) {
		this.entitlementCurrency = ofNullable(builder.getEntitlementCurrency()).map(f->f.build()).orElse(null);
		this.numberOfOptions = builder.getNumberOfOptions();
		this.optionEntitlement = builder.getOptionEntitlement();
	}
	
	/**
	 * The currency in which the option is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="entitlementCurrency", source="FpML_5_10")
	@RosettaSynonym(value="entitlementCurrency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="entitlementCurrency", source="DTCC_11_0")
	@RosettaSynonym(value="entitlementCurrency", source="DTCC_9_0")
	@RosettaSynonym(value="entitlementCurrency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getEntitlementCurrency() {
		return entitlementCurrency;
	}
	
	/**
	 * The number of options comprised in the option transaction.
	 */
	@RosettaSynonym(value="numberOfOptions", source="FpML_5_10")
	@RosettaSynonym(value="numberOfOptions", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="numberOfOptions", source="DTCC_11_0")
	@RosettaSynonym(value="numberOfOptions", source="DTCC_9_0")
	@RosettaSynonym(value="numberOfOptions", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getNumberOfOptions() {
		return numberOfOptions;
	}
	
	/**
	 * The number of units of underlier per option comprised in the option transaction.
	 */
	@RosettaSynonym(value="optionEntitlement", source="FpML_5_10")
	@RosettaSynonym(value="optionEntitlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="optionEntitlement", source="DTCC_11_0")
	@RosettaSynonym(value="optionEntitlement", source="DTCC_9_0")
	@RosettaSynonym(value="optionEntitlement", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getOptionEntitlement() {
		return optionEntitlement;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionDenomination> metaData() {
		return metaData;
	} 
	
	public OptionDenominationBuilder toBuilder() {
		OptionDenominationBuilder builder = new OptionDenominationBuilder();
		ofNullable(getEntitlementCurrency()).ifPresent(builder::setEntitlementCurrency);
		ofNullable(getNumberOfOptions()).ifPresent(builder::setNumberOfOptions);
		ofNullable(getOptionEntitlement()).ifPresent(builder::setOptionEntitlement);
		return builder;
	}

	public static OptionDenominationBuilder builder() {
		return new OptionDenominationBuilder();
	}

	public static  class OptionDenominationBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder entitlementCurrency;
		protected BigDecimal numberOfOptions;
		protected BigDecimal optionEntitlement;
	
		public OptionDenominationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionDenomination> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getEntitlementCurrency() {
			return entitlementCurrency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateEntitlementCurrency() {
			if (entitlementCurrency!=null) {
				return entitlementCurrency;
			}
			else return entitlementCurrency = new FieldWithMetaStringBuilder();
		}
		
		
		public BigDecimal getNumberOfOptions() {
			return numberOfOptions;
		}
		
		
		public BigDecimal getOptionEntitlement() {
			return optionEntitlement;
		}
		
	
		public OptionDenominationBuilder setEntitlementCurrency(FieldWithMetaString entitlementCurrency) {
			this.entitlementCurrency = entitlementCurrency.toBuilder();
			return this;
		}
		public OptionDenominationBuilder setEntitlementCurrencyRef(String entitlementCurrency) {
			return setEntitlementCurrency(FieldWithMetaString.builder().setValue(entitlementCurrency).build());
		}
		public OptionDenominationBuilder setNumberOfOptions(BigDecimal numberOfOptions) {
			this.numberOfOptions = numberOfOptions;
			return this;
		}
		public OptionDenominationBuilder setOptionEntitlement(BigDecimal optionEntitlement) {
			this.optionEntitlement = optionEntitlement;
			return this;
		}
		
		public OptionDenomination build() {
			return new OptionDenomination(this);
		}
	
		@Override
		public OptionDenominationBuilder prune() {
			if (entitlementCurrency!=null && !entitlementCurrency.prune().hasData()) entitlementCurrency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEntitlementCurrency()!=null) return true;
			if (getNumberOfOptions()!=null) return true;
			if (getOptionEntitlement()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("numberOfOptions"), BigDecimal.class, numberOfOptions, this);
			processor.processBasic(path.newSubPath("optionEntitlement"), BigDecimal.class, optionEntitlement, this);
			
			processRosetta(path.newSubPath("entitlementCurrency"), processor, FieldWithMetaString.class, entitlementCurrency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionDenominationBuilder _that = (OptionDenominationBuilder) o;
		
			if (entitlementCurrency != null ? !entitlementCurrency.equals(_that.entitlementCurrency) : _that.entitlementCurrency != null) return false;
			if (numberOfOptions != null ? !numberOfOptions.equals(_that.numberOfOptions) : _that.numberOfOptions != null) return false;
			if (optionEntitlement != null ? !optionEntitlement.equals(_that.optionEntitlement) : _that.optionEntitlement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (entitlementCurrency != null ? entitlementCurrency.hashCode() : 0);
			_result = 31 * _result + (numberOfOptions != null ? numberOfOptions.hashCode() : 0);
			_result = 31 * _result + (optionEntitlement != null ? optionEntitlement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionDenominationBuilder {" +
				"entitlementCurrency=" + this.entitlementCurrency + ", " +
				"numberOfOptions=" + this.numberOfOptions + ", " +
				"optionEntitlement=" + this.optionEntitlement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("numberOfOptions"), BigDecimal.class, numberOfOptions, this);
		processor.processBasic(path.newSubPath("optionEntitlement"), BigDecimal.class, optionEntitlement, this);
		
		processRosetta(path.newSubPath("entitlementCurrency"), processor, FieldWithMetaString.class, entitlementCurrency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionDenomination _that = (OptionDenomination) o;
	
		if (entitlementCurrency != null ? !entitlementCurrency.equals(_that.entitlementCurrency) : _that.entitlementCurrency != null) return false;
		if (numberOfOptions != null ? !numberOfOptions.equals(_that.numberOfOptions) : _that.numberOfOptions != null) return false;
		if (optionEntitlement != null ? !optionEntitlement.equals(_that.optionEntitlement) : _that.optionEntitlement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (entitlementCurrency != null ? entitlementCurrency.hashCode() : 0);
		_result = 31 * _result + (numberOfOptions != null ? numberOfOptions.hashCode() : 0);
		_result = 31 * _result + (optionEntitlement != null ? optionEntitlement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionDenomination {" +
			"entitlementCurrency=" + this.entitlementCurrency + ", " +
			"numberOfOptions=" + this.numberOfOptions + ", " +
			"optionEntitlement=" + this.optionEntitlement +
		'}';
	}
}
