package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionPhysicalSettlementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SwaptionPhysicalSettlement", source="FpML_5_10")
public class OptionPhysicalSettlement extends RosettaModelObject {
	private final Boolean clearedPhysicalSettlement;
	private final ReferenceWithMetaParty predeterminedClearingOrganizationPartyReference;
	private static OptionPhysicalSettlementMeta metaData = new OptionPhysicalSettlementMeta();
	
	OptionPhysicalSettlement(OptionPhysicalSettlementBuilder builder) {
		this.clearedPhysicalSettlement = builder.getClearedPhysicalSettlement();
		this.predeterminedClearingOrganizationPartyReference = ofNullable(builder.getPredeterminedClearingOrganizationPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies whether the swap resulting from physical settlement of the swaption transaction will clear through a clearing house. The meaning of Cleared Physical Settlement is defined in the 2006 ISDA Definitions, Section 15.2 (published in Supplement number 28).
	 */
	@RosettaSynonym(value="clearedPhysicalSettlement", source="FpML_5_10")
	@RosettaSynonym(value="clearedPhysicalSettlement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="clearedPhysicalSettlement", source="DTCC_11_0")
	@RosettaSynonym(value="clearedPhysicalSettlement", source="DTCC_9_0")
	@RosettaSynonym(value="clearedPhysicalSettlement", source="CME_ClearedConfirm_1_17")
	public final Boolean getClearedPhysicalSettlement() {
		return clearedPhysicalSettlement;
	}
	
	/**
	 * A reference to the clearing organization (CCP, DCO) to which the trade should be cleared.
	 */
	@RosettaSynonym(value="predeterminedClearingOrganizationPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="predeterminedClearingOrganizationPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="predeterminedClearingOrganizationPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="predeterminedClearingOrganizationPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="predeterminedClearingOrganizationPartyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getPredeterminedClearingOrganizationPartyReference() {
		return predeterminedClearingOrganizationPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionPhysicalSettlement> metaData() {
		return metaData;
	} 
	
	public OptionPhysicalSettlementBuilder toBuilder() {
		OptionPhysicalSettlementBuilder builder = new OptionPhysicalSettlementBuilder();
		ofNullable(getClearedPhysicalSettlement()).ifPresent(builder::setClearedPhysicalSettlement);
		ofNullable(getPredeterminedClearingOrganizationPartyReference()).ifPresent(builder::setPredeterminedClearingOrganizationPartyReference);
		return builder;
	}

	public static OptionPhysicalSettlementBuilder builder() {
		return new OptionPhysicalSettlementBuilder();
	}

	public static  class OptionPhysicalSettlementBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean clearedPhysicalSettlement;
		protected ReferenceWithMetaPartyBuilder predeterminedClearingOrganizationPartyReference;
	
		public OptionPhysicalSettlementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionPhysicalSettlement> metaData() {
			return metaData;
		} 
	
		
		public Boolean getClearedPhysicalSettlement() {
			return clearedPhysicalSettlement;
		}
		
		
		public ReferenceWithMetaPartyBuilder getPredeterminedClearingOrganizationPartyReference() {
			return predeterminedClearingOrganizationPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePredeterminedClearingOrganizationPartyReference() {
			if (predeterminedClearingOrganizationPartyReference!=null) {
				return predeterminedClearingOrganizationPartyReference;
			}
			else return predeterminedClearingOrganizationPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public OptionPhysicalSettlementBuilder setClearedPhysicalSettlement(Boolean clearedPhysicalSettlement) {
			this.clearedPhysicalSettlement = clearedPhysicalSettlement;
			return this;
		}
		public OptionPhysicalSettlementBuilder setPredeterminedClearingOrganizationPartyReference(ReferenceWithMetaParty predeterminedClearingOrganizationPartyReference) {
			this.predeterminedClearingOrganizationPartyReference = predeterminedClearingOrganizationPartyReference.toBuilder();
			return this;
		}
		public OptionPhysicalSettlementBuilder setPredeterminedClearingOrganizationPartyReferenceBuilder(ReferenceWithMetaPartyBuilder predeterminedClearingOrganizationPartyReference) {
			this.predeterminedClearingOrganizationPartyReference = predeterminedClearingOrganizationPartyReference;
			return this;
		}
		public OptionPhysicalSettlementBuilder setPredeterminedClearingOrganizationPartyReferenceRef(Party.PartyBuilder predeterminedClearingOrganizationPartyReference) {
			return setPredeterminedClearingOrganizationPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(predeterminedClearingOrganizationPartyReference).build());
		}
		public OptionPhysicalSettlementBuilder setPredeterminedClearingOrganizationPartyReferenceRef(Party predeterminedClearingOrganizationPartyReference) {
			return setPredeterminedClearingOrganizationPartyReferenceRef(Party.builder());
		}
		
		public OptionPhysicalSettlement build() {
			return new OptionPhysicalSettlement(this);
		}
	
		@Override
		public OptionPhysicalSettlementBuilder prune() {
			if (predeterminedClearingOrganizationPartyReference!=null && !predeterminedClearingOrganizationPartyReference.prune().hasData()) predeterminedClearingOrganizationPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getClearedPhysicalSettlement()!=null) return true;
			if (getPredeterminedClearingOrganizationPartyReference()!=null && getPredeterminedClearingOrganizationPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("clearedPhysicalSettlement"), Boolean.class, clearedPhysicalSettlement, this);
			
			processRosetta(path.newSubPath("predeterminedClearingOrganizationPartyReference"), processor, ReferenceWithMetaParty.class, predeterminedClearingOrganizationPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionPhysicalSettlementBuilder _that = (OptionPhysicalSettlementBuilder) o;
		
			if (clearedPhysicalSettlement != null ? !clearedPhysicalSettlement.equals(_that.clearedPhysicalSettlement) : _that.clearedPhysicalSettlement != null) return false;
			if (predeterminedClearingOrganizationPartyReference != null ? !predeterminedClearingOrganizationPartyReference.equals(_that.predeterminedClearingOrganizationPartyReference) : _that.predeterminedClearingOrganizationPartyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (clearedPhysicalSettlement != null ? clearedPhysicalSettlement.hashCode() : 0);
			_result = 31 * _result + (predeterminedClearingOrganizationPartyReference != null ? predeterminedClearingOrganizationPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionPhysicalSettlementBuilder {" +
				"clearedPhysicalSettlement=" + this.clearedPhysicalSettlement + ", " +
				"predeterminedClearingOrganizationPartyReference=" + this.predeterminedClearingOrganizationPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("clearedPhysicalSettlement"), Boolean.class, clearedPhysicalSettlement, this);
		
		processRosetta(path.newSubPath("predeterminedClearingOrganizationPartyReference"), processor, ReferenceWithMetaParty.class, predeterminedClearingOrganizationPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionPhysicalSettlement _that = (OptionPhysicalSettlement) o;
	
		if (clearedPhysicalSettlement != null ? !clearedPhysicalSettlement.equals(_that.clearedPhysicalSettlement) : _that.clearedPhysicalSettlement != null) return false;
		if (predeterminedClearingOrganizationPartyReference != null ? !predeterminedClearingOrganizationPartyReference.equals(_that.predeterminedClearingOrganizationPartyReference) : _that.predeterminedClearingOrganizationPartyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (clearedPhysicalSettlement != null ? clearedPhysicalSettlement.hashCode() : 0);
		_result = 31 * _result + (predeterminedClearingOrganizationPartyReference != null ? predeterminedClearingOrganizationPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionPhysicalSettlement {" +
			"clearedPhysicalSettlement=" + this.clearedPhysicalSettlement + ", " +
			"predeterminedClearingOrganizationPartyReference=" + this.predeterminedClearingOrganizationPartyReference +
		'}';
	}
}
