package org.isda.cdm;

import org.isda.cdm.AmericanExercise;
import org.isda.cdm.BermudaExercise;
import org.isda.cdm.EuropeanExercise;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionStyleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The qualification of the option style: American, Bermuda or European. FpML implements those features as part of a substitution group.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class OptionStyle extends RosettaModelObject {
	private final AmericanExercise americanExercise;
	private final BermudaExercise bermudaExercise;
	private final EuropeanExercise europeanExercise;
	private static OptionStyleMeta metaData = new OptionStyleMeta();
	
	OptionStyle(OptionStyleBuilder builder) {
		this.americanExercise = ofNullable(builder.getAmericanExercise()).map(f->f.build()).orElse(null);
		this.bermudaExercise = ofNullable(builder.getBermudaExercise()).map(f->f.build()).orElse(null);
		this.europeanExercise = ofNullable(builder.getEuropeanExercise()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="americanExercise", source="FpML_5_10")
	@RosettaSynonym(value="americanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="americanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="americanExercise", source="CME_ClearedConfirm_1_17")
	public final AmericanExercise getAmericanExercise() {
		return americanExercise;
	}
	
	@RosettaSynonym(value="bermudaExercise", source="FpML_5_10")
	@RosettaSynonym(value="bermudaExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_11_0")
	@RosettaSynonym(value="bermudaExercise", source="DTCC_9_0")
	@RosettaSynonym(value="bermudaExercise", source="CME_ClearedConfirm_1_17")
	public final BermudaExercise getBermudaExercise() {
		return bermudaExercise;
	}
	
	@RosettaSynonym(value="europeanExercise", source="FpML_5_10")
	@RosettaSynonym(value="europeanExercise", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_11_0")
	@RosettaSynonym(value="europeanExercise", source="DTCC_9_0")
	@RosettaSynonym(value="europeanExercise", source="CME_ClearedConfirm_1_17")
	public final EuropeanExercise getEuropeanExercise() {
		return europeanExercise;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionStyle> metaData() {
		return metaData;
	} 
	
	public OptionStyleBuilder toBuilder() {
		OptionStyleBuilder builder = new OptionStyleBuilder();
		ofNullable(getAmericanExercise()).ifPresent(builder::setAmericanExercise);
		ofNullable(getBermudaExercise()).ifPresent(builder::setBermudaExercise);
		ofNullable(getEuropeanExercise()).ifPresent(builder::setEuropeanExercise);
		return builder;
	}

	public static OptionStyleBuilder builder() {
		return new OptionStyleBuilder();
	}

	public static  class OptionStyleBuilder extends RosettaModelObjectBuilder {
	
		protected AmericanExercise.AmericanExerciseBuilder americanExercise;
		protected BermudaExercise.BermudaExerciseBuilder bermudaExercise;
		protected EuropeanExercise.EuropeanExerciseBuilder europeanExercise;
	
		public OptionStyleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionStyle> metaData() {
			return metaData;
		} 
	
		
		public AmericanExercise.AmericanExerciseBuilder getAmericanExercise() {
			return americanExercise;
		}
		
		public AmericanExercise.AmericanExerciseBuilder getOrCreateAmericanExercise() {
			if (americanExercise!=null) {
				return americanExercise;
			}
			else return americanExercise = new AmericanExercise.AmericanExerciseBuilder();
		}
		
		
		public BermudaExercise.BermudaExerciseBuilder getBermudaExercise() {
			return bermudaExercise;
		}
		
		public BermudaExercise.BermudaExerciseBuilder getOrCreateBermudaExercise() {
			if (bermudaExercise!=null) {
				return bermudaExercise;
			}
			else return bermudaExercise = new BermudaExercise.BermudaExerciseBuilder();
		}
		
		
		public EuropeanExercise.EuropeanExerciseBuilder getEuropeanExercise() {
			return europeanExercise;
		}
		
		public EuropeanExercise.EuropeanExerciseBuilder getOrCreateEuropeanExercise() {
			if (europeanExercise!=null) {
				return europeanExercise;
			}
			else return europeanExercise = new EuropeanExercise.EuropeanExerciseBuilder();
		}
		
	
		public OptionStyleBuilder setAmericanExercise(AmericanExercise americanExercise) {
			this.americanExercise = americanExercise.toBuilder();
			return this;
		}
		public OptionStyleBuilder setAmericanExerciseBuilder(AmericanExercise.AmericanExerciseBuilder americanExercise) {
			this.americanExercise = americanExercise;
			return this;
		}
		public OptionStyleBuilder setBermudaExercise(BermudaExercise bermudaExercise) {
			this.bermudaExercise = bermudaExercise.toBuilder();
			return this;
		}
		public OptionStyleBuilder setBermudaExerciseBuilder(BermudaExercise.BermudaExerciseBuilder bermudaExercise) {
			this.bermudaExercise = bermudaExercise;
			return this;
		}
		public OptionStyleBuilder setEuropeanExercise(EuropeanExercise europeanExercise) {
			this.europeanExercise = europeanExercise.toBuilder();
			return this;
		}
		public OptionStyleBuilder setEuropeanExerciseBuilder(EuropeanExercise.EuropeanExerciseBuilder europeanExercise) {
			this.europeanExercise = europeanExercise;
			return this;
		}
		
		public OptionStyle build() {
			return new OptionStyle(this);
		}
	
		@Override
		public OptionStyleBuilder prune() {
			if (americanExercise!=null && !americanExercise.prune().hasData()) americanExercise = null;
			if (bermudaExercise!=null && !bermudaExercise.prune().hasData()) bermudaExercise = null;
			if (europeanExercise!=null && !europeanExercise.prune().hasData()) europeanExercise = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmericanExercise()!=null && getAmericanExercise().hasData()) return true;
			if (getBermudaExercise()!=null && getBermudaExercise().hasData()) return true;
			if (getEuropeanExercise()!=null && getEuropeanExercise().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
			processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
			processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionStyleBuilder _that = (OptionStyleBuilder) o;
		
			if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
			if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
			if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
			_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
			_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionStyleBuilder {" +
				"americanExercise=" + this.americanExercise + ", " +
				"bermudaExercise=" + this.bermudaExercise + ", " +
				"europeanExercise=" + this.europeanExercise +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("americanExercise"), processor, AmericanExercise.class, americanExercise);
		processRosetta(path.newSubPath("bermudaExercise"), processor, BermudaExercise.class, bermudaExercise);
		processRosetta(path.newSubPath("europeanExercise"), processor, EuropeanExercise.class, europeanExercise);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionStyle _that = (OptionStyle) o;
	
		if (americanExercise != null ? !americanExercise.equals(_that.americanExercise) : _that.americanExercise != null) return false;
		if (bermudaExercise != null ? !bermudaExercise.equals(_that.bermudaExercise) : _that.bermudaExercise != null) return false;
		if (europeanExercise != null ? !europeanExercise.equals(_that.europeanExercise) : _that.europeanExercise != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (americanExercise != null ? americanExercise.hashCode() : 0);
		_result = 31 * _result + (bermudaExercise != null ? bermudaExercise.hashCode() : 0);
		_result = 31 * _result + (europeanExercise != null ? europeanExercise.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionStyle {" +
			"americanExercise=" + this.americanExercise + ", " +
			"bermudaExercise=" + this.bermudaExercise + ", " +
			"europeanExercise=" + this.europeanExercise +
		'}';
	}
}
