package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of the option. In FpML, OptionTypeEnum is a union with PutCallEnum, which specifies whether the option is a put or a call.
 * 
 * @version 2.5.4
 */
public enum OptionTypeEnum {
    

	/**
	 * A put option gives the holder the right to sell the underlying asset by a certain date for a certain price.
	 */
	@RosettaSynonym(value = "Put", source = "FpML_5_10")
	@RosettaSynonym(value = "Put", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Put", source = "DTCC_11_0")
	@RosettaSynonym(value = "Put", source = "DTCC_9_0")
	@RosettaSynonym(value = "Put", source = "CME_ClearedConfirm_1_17")
	PUT,
	    

	/**
	 * A call option gives the holder the right to buy the underlying asset by a certain date for a certain price.
	 */
	@RosettaSynonym(value = "Call", source = "FpML_5_10")
	@RosettaSynonym(value = "Call", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Call", source = "DTCC_11_0")
	@RosettaSynonym(value = "Call", source = "DTCC_9_0")
	@RosettaSynonym(value = "Call", source = "CME_ClearedConfirm_1_17")
	CALL,
	    

	/**
	 * A &#39;payer&#39; option: If you buy a &#39;payer&#39; option you have the right but not the obligation to enter into the underlying swap transaction as the &#39;fixed&#39; rate/price payer and receive float.
	 */
	@RosettaSynonym(value = "Payer", source = "FpML_5_10")
	@RosettaSynonym(value = "Payer", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Payer", source = "DTCC_11_0")
	@RosettaSynonym(value = "Payer", source = "DTCC_9_0")
	@RosettaSynonym(value = "Payer", source = "CME_ClearedConfirm_1_17")
	PAYER,
	    

	/**
	 * A &#39;receiver&#39; option: If you buy a &#39;receiver&#39; option you have the right but not the obligation to enter into the underlying swap transaction as the &#39;fixed&#39; rate/price receiver and pay float.
	 */
	@RosettaSynonym(value = "Receiver", source = "FpML_5_10")
	@RosettaSynonym(value = "Receiver", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Receiver", source = "DTCC_11_0")
	@RosettaSynonym(value = "Receiver", source = "DTCC_9_0")
	@RosettaSynonym(value = "Receiver", source = "CME_ClearedConfirm_1_17")
	RECEIVER,
	    

	/**
	 * A straddle strategy, which involves the simultaneous buying of a put and a call of the same underlier, at the same strike and same expiration date
	 */
	@RosettaSynonym(value = "Straddle", source = "FpML_5_10")
	@RosettaSynonym(value = "Straddle", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Straddle", source = "DTCC_11_0")
	@RosettaSynonym(value = "Straddle", source = "DTCC_9_0")
	@RosettaSynonym(value = "Straddle", source = "CME_ClearedConfirm_1_17")
	STRADDLE    
	;

	private final String displayName;
	
	OptionTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
