package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.EarlyTerminationEvent;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OptionalEarlyTerminationAdjustedDatesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining the adjusted dates associated with an optional early termination provision.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="OptionalEarlyTerminationAdjustedDates", source="FpML_5_10")
public class OptionalEarlyTerminationAdjustedDates extends RosettaModelObject {
	private final List<EarlyTerminationEvent> earlyTerminationEvent;
	private static OptionalEarlyTerminationAdjustedDatesMeta metaData = new OptionalEarlyTerminationAdjustedDatesMeta();
	
	OptionalEarlyTerminationAdjustedDates(OptionalEarlyTerminationAdjustedDatesBuilder builder) {
		this.earlyTerminationEvent = ofNullable(builder.getEarlyTerminationEvent()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The adjusted dates associated with an individual early termination date.
	 */
	@RosettaSynonym(value="earlyTerminationEvent", source="FpML_5_10")
	@RosettaSynonym(value="earlyTerminationEvent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="earlyTerminationEvent", source="DTCC_11_0")
	@RosettaSynonym(value="earlyTerminationEvent", source="DTCC_9_0")
	@RosettaSynonym(value="earlyTerminationEvent", source="CME_ClearedConfirm_1_17")
	public final List<EarlyTerminationEvent> getEarlyTerminationEvent() {
		return earlyTerminationEvent;
	}
	
	
	@Override
	public RosettaMetaData<? extends OptionalEarlyTerminationAdjustedDates> metaData() {
		return metaData;
	} 
	
	public OptionalEarlyTerminationAdjustedDatesBuilder toBuilder() {
		OptionalEarlyTerminationAdjustedDatesBuilder builder = new OptionalEarlyTerminationAdjustedDatesBuilder();
		ofNullable(getEarlyTerminationEvent()).ifPresent(earlyTerminationEvent -> earlyTerminationEvent.forEach(builder::addEarlyTerminationEvent));
		return builder;
	}

	public static OptionalEarlyTerminationAdjustedDatesBuilder builder() {
		return new OptionalEarlyTerminationAdjustedDatesBuilder();
	}

	public static  class OptionalEarlyTerminationAdjustedDatesBuilder extends RosettaModelObjectBuilder {
	
		protected List<EarlyTerminationEvent.EarlyTerminationEventBuilder> earlyTerminationEvent;
	
		public OptionalEarlyTerminationAdjustedDatesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OptionalEarlyTerminationAdjustedDates> metaData() {
			return metaData;
		} 
	
		
		public List<EarlyTerminationEvent.EarlyTerminationEventBuilder> getEarlyTerminationEvent() {
			return earlyTerminationEvent;
		}
		
		public EarlyTerminationEvent.EarlyTerminationEventBuilder getOrCreateEarlyTerminationEvent(int index) {
			if (earlyTerminationEvent==null) {
				this.earlyTerminationEvent = new ArrayList<>();
			}
			return getIndex(earlyTerminationEvent, index, ()->new EarlyTerminationEvent.EarlyTerminationEventBuilder());
		}
		
	
		public OptionalEarlyTerminationAdjustedDatesBuilder addEarlyTerminationEvent(EarlyTerminationEvent earlyTerminationEvent) {
			if(this.earlyTerminationEvent == null){
				this.earlyTerminationEvent = new ArrayList<>();
			}
			this.earlyTerminationEvent.add(earlyTerminationEvent.toBuilder());
			return this;
		}
		public OptionalEarlyTerminationAdjustedDatesBuilder addEarlyTerminationEvent(EarlyTerminationEvent earlyTerminationEvent, int _idx) {
			if(this.earlyTerminationEvent == null){
				this.earlyTerminationEvent = new ArrayList<>();
			}
			getIndex(this.earlyTerminationEvent, _idx, () -> earlyTerminationEvent.toBuilder());
			this.earlyTerminationEvent.set(_idx, earlyTerminationEvent.toBuilder());
			return this;
		}
		public OptionalEarlyTerminationAdjustedDatesBuilder addEarlyTerminationEvent(List<EarlyTerminationEvent> earlyTerminationEvents) {
			if(this.earlyTerminationEvent == null){
				this.earlyTerminationEvent = new ArrayList<>();
			}
			for (EarlyTerminationEvent toAdd : earlyTerminationEvents) {
				this.earlyTerminationEvent.add(toAdd.toBuilder());
			}
			return this;
		}
		public OptionalEarlyTerminationAdjustedDatesBuilder addEarlyTerminationEventBuilder(EarlyTerminationEvent.EarlyTerminationEventBuilder earlyTerminationEvent) {
			if(this.earlyTerminationEvent == null){
				this.earlyTerminationEvent = new ArrayList<>();
			}
			this.earlyTerminationEvent.add(earlyTerminationEvent);
			return this;
		}
		
		public OptionalEarlyTerminationAdjustedDatesBuilder clearEarlyTerminationEvent() {
			this.earlyTerminationEvent = null;
			return this;
		}
		
		public OptionalEarlyTerminationAdjustedDates build() {
			return new OptionalEarlyTerminationAdjustedDates(this);
		}
	
		@Override
		public OptionalEarlyTerminationAdjustedDatesBuilder prune() {
			if (earlyTerminationEvent!=null) earlyTerminationEvent = earlyTerminationEvent.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEarlyTerminationEvent()!=null && getEarlyTerminationEvent().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("earlyTerminationEvent"), processor, EarlyTerminationEvent.class, earlyTerminationEvent);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OptionalEarlyTerminationAdjustedDatesBuilder _that = (OptionalEarlyTerminationAdjustedDatesBuilder) o;
		
			if (!ListEquals.listEquals(earlyTerminationEvent, _that.earlyTerminationEvent)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (earlyTerminationEvent != null ? earlyTerminationEvent.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OptionalEarlyTerminationAdjustedDatesBuilder {" +
				"earlyTerminationEvent=" + this.earlyTerminationEvent +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("earlyTerminationEvent"), processor, EarlyTerminationEvent.class, earlyTerminationEvent);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OptionalEarlyTerminationAdjustedDates _that = (OptionalEarlyTerminationAdjustedDates) o;
	
		if (!ListEquals.listEquals(earlyTerminationEvent, _that.earlyTerminationEvent)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (earlyTerminationEvent != null ? earlyTerminationEvent.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OptionalEarlyTerminationAdjustedDates {" +
			"earlyTerminationEvent=" + this.earlyTerminationEvent +
		'}';
	}
}
