package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify the originating event that gave way to the trade.
 * 
 * @version 2.5.4
 */
public enum OriginatingEventEnum {
    

	/**
	 * The trade results from an allocation event.
	 */
	@RosettaSynonym(value = "1", source = "CME_SubmissionIRS_1_0")
	ALLOCATION,
	    

	/**
	 * The trade results from a trade amendment.
	 */
	@RosettaSynonym(value = "TRADE_AMEND", source = "CME_ClearedConfirm_1_17")
	AMENDMENT,
	    

	/**
	 * The trade results from a compression event.
	 */
	@RosettaSynonym(value = "RESIDUAL_TRADE", source = "CME_ClearedConfirm_1_17")
	COMPRESSION,
	    

	/**
	 * The trade results from an exercise event.
	 */
	@RosettaSynonym(value = "EXERCISE", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "Exercise", source = "Rosetta_Workbench")
	EXERCISE,
	    

	/**
	 * The trade results from a clearing event.
	 */
	@RosettaSynonym(value = "2", source = "CME_SubmissionIRS_1_0")
	CLEARING,
	    

	/**
	 * The trade results from a new trade event.
	 */
	@RosettaSynonym(value = "NEW_TRADE", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "NewTrade", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "0", source = "CME_SubmissionIRS_1_0")
	NEW_TRADE,
	    

	/**
	 * The trade presented results from a novation event.
	 */
	@RosettaSynonym(value = "ASSIGNMENT", source = "CME_ClearedConfirm_1_17")
	NOVATION,
	    

	/**
	 * The trade results from a partial exercise event.
	 */
	@RosettaSynonym(value = "PARTIAL_EXERCISE", source = "CME_ClearedConfirm_1_17")
	PARTIAL_EXERCISE,
	    

	/**
	 * The trade results from a partial novation event.
	 */
	@RosettaSynonym(value = "NOVATION", source = "CME_ClearedConfirm_1_17")
	PARTIAL_NOVATION,
	    

	/**
	 * The trade results from a transfer.
	 */
	@RosettaSynonym(value = "TRANSFER", source = "CME_ClearedConfirm_1_17")
	TRANSFER    
	;

	private final String displayName;
	
	OriginatingEventEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
