package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OtherAgreementMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining an agreement executed between parties.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="OtherAgreement", source="FpML_5_10")
public class OtherAgreement extends RosettaModelObject {
	private final Date date;
	private final FieldWithMetaString identifier;
	private final FieldWithMetaString type;
	private final FieldWithMetaString version;
	private static OtherAgreementMeta metaData = new OtherAgreementMeta();
	
	OtherAgreement(OtherAgreementBuilder builder) {
		this.date = builder.getDate();
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.type = ofNullable(builder.getType()).map(f->f.build()).orElse(null);
		this.version = ofNullable(builder.getVersion()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The date on which the agreement was signed.
	 */
	@RosettaSynonym(value="date", source="FpML_5_10")
	@RosettaSynonym(value="date", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="date", source="DTCC_11_0")
	@RosettaSynonym(value="date", source="DTCC_9_0")
	@RosettaSynonym(value="date", source="CME_ClearedConfirm_1_17")
	public final Date getDate() {
		return date;
	}
	
	/**
	 * An identifier that has been created to identify the agreement.
	 */
	@RosettaSynonym(value="identifier", source="FpML_5_10")
	@RosettaSynonym(value="identifier", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="identifier", source="DTCC_11_0")
	@RosettaSynonym(value="identifier", source="DTCC_9_0")
	@RosettaSynonym(value="identifier", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getIdentifier() {
		return identifier;
	}
	
	/**
	 * The agreement executed between the parties and intended to govern product-specific derivatives transactions between those parties.
	 */
	@RosettaSynonym(value="type", source="FpML_5_10")
	@RosettaSynonym(value="type", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="type", source="DTCC_11_0")
	@RosettaSynonym(value="type", source="DTCC_9_0")
	@RosettaSynonym(value="type", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getType() {
		return type;
	}
	
	/**
	 * The version of the agreement.
	 */
	@RosettaSynonym(value="version", source="FpML_5_10")
	@RosettaSynonym(value="version", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="version", source="DTCC_11_0")
	@RosettaSynonym(value="version", source="DTCC_9_0")
	@RosettaSynonym(value="version", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getVersion() {
		return version;
	}
	
	
	@Override
	public RosettaMetaData<? extends OtherAgreement> metaData() {
		return metaData;
	} 
	
	public OtherAgreementBuilder toBuilder() {
		OtherAgreementBuilder builder = new OtherAgreementBuilder();
		ofNullable(getDate()).ifPresent(builder::setDate);
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getType()).ifPresent(builder::setType);
		ofNullable(getVersion()).ifPresent(builder::setVersion);
		return builder;
	}

	public static OtherAgreementBuilder builder() {
		return new OtherAgreementBuilder();
	}

	public static  class OtherAgreementBuilder extends RosettaModelObjectBuilder {
	
		protected Date date;
		protected FieldWithMetaStringBuilder identifier;
		protected FieldWithMetaStringBuilder type;
		protected FieldWithMetaStringBuilder version;
	
		public OtherAgreementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends OtherAgreement> metaData() {
			return metaData;
		} 
	
		
		public Date getDate() {
			return date;
		}
		
		
		public FieldWithMetaStringBuilder getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getType() {
			return type;
		}
		
		public FieldWithMetaStringBuilder getOrCreateType() {
			if (type!=null) {
				return type;
			}
			else return type = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getVersion() {
			return version;
		}
		
		public FieldWithMetaStringBuilder getOrCreateVersion() {
			if (version!=null) {
				return version;
			}
			else return version = new FieldWithMetaStringBuilder();
		}
		
	
		public OtherAgreementBuilder setDate(Date date) {
			this.date = date;
			return this;
		}
		public OtherAgreementBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public OtherAgreementBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		public OtherAgreementBuilder setType(FieldWithMetaString type) {
			this.type = type.toBuilder();
			return this;
		}
		public OtherAgreementBuilder setTypeRef(String type) {
			return setType(FieldWithMetaString.builder().setValue(type).build());
		}
		public OtherAgreementBuilder setVersion(FieldWithMetaString version) {
			this.version = version.toBuilder();
			return this;
		}
		public OtherAgreementBuilder setVersionRef(String version) {
			return setVersion(FieldWithMetaString.builder().setValue(version).build());
		}
		
		public OtherAgreement build() {
			return new OtherAgreement(this);
		}
	
		@Override
		public OtherAgreementBuilder prune() {
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			if (type!=null && !type.prune().hasData()) type = null;
			if (version!=null && !version.prune().hasData()) version = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDate()!=null) return true;
			if (getIdentifier()!=null) return true;
			if (getType()!=null) return true;
			if (getVersion()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
			processRosetta(path.newSubPath("type"), processor, FieldWithMetaString.class, type);
			processRosetta(path.newSubPath("version"), processor, FieldWithMetaString.class, version);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OtherAgreementBuilder _that = (OtherAgreementBuilder) o;
		
			if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
			if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (type != null ? type.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OtherAgreementBuilder {" +
				"date=" + this.date + ", " +
				"identifier=" + this.identifier + ", " +
				"type=" + this.type + ", " +
				"version=" + this.version +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("date"), Date.class, date, this);
		
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		processRosetta(path.newSubPath("type"), processor, FieldWithMetaString.class, type);
		processRosetta(path.newSubPath("version"), processor, FieldWithMetaString.class, version);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		OtherAgreement _that = (OtherAgreement) o;
	
		if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
		if (version != null ? !version.equals(_that.version) : _that.version != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (type != null ? type.hashCode() : 0);
		_result = 31 * _result + (version != null ? version.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "OtherAgreement {" +
			"date=" + this.date + ", " +
			"identifier=" + this.identifier + ", " +
			"type=" + this.type + ", " +
			"version=" + this.version +
		'}';
	}
}
