package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.DerivInstrmAttrbts;
import org.isda.cdm.FinInstrmGnlAttrbts;
import org.isda.cdm.SchmeNm;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.OthrMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Othr extends RosettaModelObject {
	private final DerivInstrmAttrbts derivInstrmAttrbts;
	private final FinInstrmGnlAttrbts finInstrmGnlAttrbts;
	private final String id;
	private final SchmeNm schmeNm;
	private static OthrMeta metaData = new OthrMeta();
	
	Othr(OthrBuilder builder) {
		this.derivInstrmAttrbts = ofNullable(builder.getDerivInstrmAttrbts()).map(f->f.build()).orElse(null);
		this.finInstrmGnlAttrbts = ofNullable(builder.getFinInstrmGnlAttrbts()).map(f->f.build()).orElse(null);
		this.id = builder.getId();
		this.schmeNm = ofNullable(builder.getSchmeNm()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="DerivInstrmAttrbts", source="ISO20022")
	public final DerivInstrmAttrbts getDerivInstrmAttrbts() {
		return derivInstrmAttrbts;
	}
	
	@RosettaSynonym(value="FinInstrmGnlAttrbts", source="ISO20022")
	public final FinInstrmGnlAttrbts getFinInstrmGnlAttrbts() {
		return finInstrmGnlAttrbts;
	}
	
	@RosettaSynonym(value="Id", source="ISO20022")
	public final String getId() {
		return id;
	}
	
	@RosettaSynonym(value="SchmeNm", source="ISO20022")
	public final SchmeNm getSchmeNm() {
		return schmeNm;
	}
	
	
	@Override
	public RosettaMetaData<? extends Othr> metaData() {
		return metaData;
	} 
	
	public OthrBuilder toBuilder() {
		OthrBuilder builder = new OthrBuilder();
		ofNullable(getDerivInstrmAttrbts()).ifPresent(builder::setDerivInstrmAttrbts);
		ofNullable(getFinInstrmGnlAttrbts()).ifPresent(builder::setFinInstrmGnlAttrbts);
		ofNullable(getId()).ifPresent(builder::setId);
		ofNullable(getSchmeNm()).ifPresent(builder::setSchmeNm);
		return builder;
	}

	public static OthrBuilder builder() {
		return new OthrBuilder();
	}

	public static  class OthrBuilder extends RosettaModelObjectBuilder {
	
		protected DerivInstrmAttrbts.DerivInstrmAttrbtsBuilder derivInstrmAttrbts;
		protected FinInstrmGnlAttrbts.FinInstrmGnlAttrbtsBuilder finInstrmGnlAttrbts;
		protected String id;
		protected SchmeNm.SchmeNmBuilder schmeNm;
	
		public OthrBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Othr> metaData() {
			return metaData;
		} 
	
		
		public DerivInstrmAttrbts.DerivInstrmAttrbtsBuilder getDerivInstrmAttrbts() {
			return derivInstrmAttrbts;
		}
		
		public DerivInstrmAttrbts.DerivInstrmAttrbtsBuilder getOrCreateDerivInstrmAttrbts() {
			if (derivInstrmAttrbts!=null) {
				return derivInstrmAttrbts;
			}
			else return derivInstrmAttrbts = new DerivInstrmAttrbts.DerivInstrmAttrbtsBuilder();
		}
		
		
		public FinInstrmGnlAttrbts.FinInstrmGnlAttrbtsBuilder getFinInstrmGnlAttrbts() {
			return finInstrmGnlAttrbts;
		}
		
		public FinInstrmGnlAttrbts.FinInstrmGnlAttrbtsBuilder getOrCreateFinInstrmGnlAttrbts() {
			if (finInstrmGnlAttrbts!=null) {
				return finInstrmGnlAttrbts;
			}
			else return finInstrmGnlAttrbts = new FinInstrmGnlAttrbts.FinInstrmGnlAttrbtsBuilder();
		}
		
		
		public String getId() {
			return id;
		}
		
		
		public SchmeNm.SchmeNmBuilder getSchmeNm() {
			return schmeNm;
		}
		
		public SchmeNm.SchmeNmBuilder getOrCreateSchmeNm() {
			if (schmeNm!=null) {
				return schmeNm;
			}
			else return schmeNm = new SchmeNm.SchmeNmBuilder();
		}
		
	
		public OthrBuilder setDerivInstrmAttrbts(DerivInstrmAttrbts derivInstrmAttrbts) {
			this.derivInstrmAttrbts = derivInstrmAttrbts.toBuilder();
			return this;
		}
		public OthrBuilder setDerivInstrmAttrbtsBuilder(DerivInstrmAttrbts.DerivInstrmAttrbtsBuilder derivInstrmAttrbts) {
			this.derivInstrmAttrbts = derivInstrmAttrbts;
			return this;
		}
		public OthrBuilder setFinInstrmGnlAttrbts(FinInstrmGnlAttrbts finInstrmGnlAttrbts) {
			this.finInstrmGnlAttrbts = finInstrmGnlAttrbts.toBuilder();
			return this;
		}
		public OthrBuilder setFinInstrmGnlAttrbtsBuilder(FinInstrmGnlAttrbts.FinInstrmGnlAttrbtsBuilder finInstrmGnlAttrbts) {
			this.finInstrmGnlAttrbts = finInstrmGnlAttrbts;
			return this;
		}
		public OthrBuilder setId(String id) {
			this.id = id;
			return this;
		}
		public OthrBuilder setSchmeNm(SchmeNm schmeNm) {
			this.schmeNm = schmeNm.toBuilder();
			return this;
		}
		public OthrBuilder setSchmeNmBuilder(SchmeNm.SchmeNmBuilder schmeNm) {
			this.schmeNm = schmeNm;
			return this;
		}
		
		public Othr build() {
			return new Othr(this);
		}
	
		@Override
		public OthrBuilder prune() {
			if (derivInstrmAttrbts!=null && !derivInstrmAttrbts.prune().hasData()) derivInstrmAttrbts = null;
			if (finInstrmGnlAttrbts!=null && !finInstrmGnlAttrbts.prune().hasData()) finInstrmGnlAttrbts = null;
			if (schmeNm!=null && !schmeNm.prune().hasData()) schmeNm = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDerivInstrmAttrbts()!=null && getDerivInstrmAttrbts().hasData()) return true;
			if (getFinInstrmGnlAttrbts()!=null && getFinInstrmGnlAttrbts().hasData()) return true;
			if (getId()!=null) return true;
			if (getSchmeNm()!=null && getSchmeNm().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("id"), String.class, id, this);
			
			processRosetta(path.newSubPath("derivInstrmAttrbts"), processor, DerivInstrmAttrbts.class, derivInstrmAttrbts);
			processRosetta(path.newSubPath("finInstrmGnlAttrbts"), processor, FinInstrmGnlAttrbts.class, finInstrmGnlAttrbts);
			processRosetta(path.newSubPath("schmeNm"), processor, SchmeNm.class, schmeNm);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			OthrBuilder _that = (OthrBuilder) o;
		
			if (derivInstrmAttrbts != null ? !derivInstrmAttrbts.equals(_that.derivInstrmAttrbts) : _that.derivInstrmAttrbts != null) return false;
			if (finInstrmGnlAttrbts != null ? !finInstrmGnlAttrbts.equals(_that.finInstrmGnlAttrbts) : _that.finInstrmGnlAttrbts != null) return false;
			if (id != null ? !id.equals(_that.id) : _that.id != null) return false;
			if (schmeNm != null ? !schmeNm.equals(_that.schmeNm) : _that.schmeNm != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (derivInstrmAttrbts != null ? derivInstrmAttrbts.hashCode() : 0);
			_result = 31 * _result + (finInstrmGnlAttrbts != null ? finInstrmGnlAttrbts.hashCode() : 0);
			_result = 31 * _result + (id != null ? id.hashCode() : 0);
			_result = 31 * _result + (schmeNm != null ? schmeNm.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "OthrBuilder {" +
				"derivInstrmAttrbts=" + this.derivInstrmAttrbts + ", " +
				"finInstrmGnlAttrbts=" + this.finInstrmGnlAttrbts + ", " +
				"id=" + this.id + ", " +
				"schmeNm=" + this.schmeNm +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("id"), String.class, id, this);
		
		processRosetta(path.newSubPath("derivInstrmAttrbts"), processor, DerivInstrmAttrbts.class, derivInstrmAttrbts);
		processRosetta(path.newSubPath("finInstrmGnlAttrbts"), processor, FinInstrmGnlAttrbts.class, finInstrmGnlAttrbts);
		processRosetta(path.newSubPath("schmeNm"), processor, SchmeNm.class, schmeNm);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Othr _that = (Othr) o;
	
		if (derivInstrmAttrbts != null ? !derivInstrmAttrbts.equals(_that.derivInstrmAttrbts) : _that.derivInstrmAttrbts != null) return false;
		if (finInstrmGnlAttrbts != null ? !finInstrmGnlAttrbts.equals(_that.finInstrmGnlAttrbts) : _that.finInstrmGnlAttrbts != null) return false;
		if (id != null ? !id.equals(_that.id) : _that.id != null) return false;
		if (schmeNm != null ? !schmeNm.equals(_that.schmeNm) : _that.schmeNm != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (derivInstrmAttrbts != null ? derivInstrmAttrbts.hashCode() : 0);
		_result = 31 * _result + (finInstrmGnlAttrbts != null ? finInstrmGnlAttrbts.hashCode() : 0);
		_result = 31 * _result + (id != null ? id.hashCode() : 0);
		_result = 31 * _result + (schmeNm != null ? schmeNm.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Othr {" +
			"derivInstrmAttrbts=" + this.derivInstrmAttrbts + ", " +
			"finInstrmGnlAttrbts=" + this.finInstrmGnlAttrbts + ", " +
			"id=" + this.id + ", " +
			"schmeNm=" + this.schmeNm +
		'}';
	}
}
