package org.isda.cdm;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CategoryEnum;
import org.isda.cdm.RelatedParty;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PackageInformationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining additional information that may be recorded alongside a transaction package.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PackageInformation", source="FpML_5_10")
public class PackageInformation extends RosettaModelObject {
	private final List<CategoryEnum> category;
	private final Boolean intentToAllocate;
	private final List<RelatedParty> relatedParty;
	private static PackageInformationMeta metaData = new PackageInformationMeta();
	
	PackageInformation(PackageInformationBuilder builder) {
		this.category = builder.getCategory();
		this.intentToAllocate = builder.getIntentToAllocate();
		this.relatedParty = ofNullable(builder.getRelatedParty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	@RosettaSynonym(value="category", source="FpML_5_10")
	@RosettaSynonym(value="category", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="category", source="DTCC_11_0")
	@RosettaSynonym(value="category", source="DTCC_9_0")
	@RosettaSynonym(value="category", source="CME_ClearedConfirm_1_17")
	public final List<CategoryEnum> getCategory() {
		return category;
	}
	
	/**
	 * specifies whether the transaction package is anticipated to be allocated.
	 */
	@RosettaSynonym(value="intentToAllocate", source="FpML_5_10")
	@RosettaSynonym(value="intentToAllocate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="intentToAllocate", source="DTCC_11_0")
	@RosettaSynonym(value="intentToAllocate", source="DTCC_9_0")
	@RosettaSynonym(value="intentToAllocate", source="CME_ClearedConfirm_1_17")
	public final Boolean getIntentToAllocate() {
		return intentToAllocate;
	}
	
	/**
	 * This may be used to identify one or more parties that perform a role as part of the transaction.
	 */
	@RosettaSynonym(value="relatedParty", source="FpML_5_10")
	@RosettaSynonym(value="relatedParty", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relatedParty", source="DTCC_11_0")
	@RosettaSynonym(value="relatedParty", source="DTCC_9_0")
	@RosettaSynonym(value="relatedParty", source="CME_ClearedConfirm_1_17")
	public final List<RelatedParty> getRelatedParty() {
		return relatedParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends PackageInformation> metaData() {
		return metaData;
	} 
	
	public PackageInformationBuilder toBuilder() {
		PackageInformationBuilder builder = new PackageInformationBuilder();
		ofNullable(getCategory()).ifPresent(category -> category.forEach(builder::addCategory));
		ofNullable(getIntentToAllocate()).ifPresent(builder::setIntentToAllocate);
		ofNullable(getRelatedParty()).ifPresent(relatedParty -> relatedParty.forEach(builder::addRelatedParty));
		return builder;
	}

	public static PackageInformationBuilder builder() {
		return new PackageInformationBuilder();
	}

	public static  class PackageInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<CategoryEnum> category;
		protected Boolean intentToAllocate;
		protected List<RelatedParty.RelatedPartyBuilder> relatedParty;
	
		public PackageInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PackageInformation> metaData() {
			return metaData;
		} 
	
		
		public List<CategoryEnum> getCategory() {
			return category;
		}
		
		
		public Boolean getIntentToAllocate() {
			return intentToAllocate;
		}
		
		
		public List<RelatedParty.RelatedPartyBuilder> getRelatedParty() {
			return relatedParty;
		}
		
		public RelatedParty.RelatedPartyBuilder getOrCreateRelatedParty(int index) {
			if (relatedParty==null) {
				this.relatedParty = new ArrayList<>();
			}
			return getIndex(relatedParty, index, ()->new RelatedParty.RelatedPartyBuilder());
		}
		
	
		public PackageInformationBuilder addCategory(CategoryEnum category) {
			if(this.category == null){
				this.category = new ArrayList<>();
			}
			this.category.add(category);
			return this;
		}
		public PackageInformationBuilder addCategory(CategoryEnum category, int _idx) {
			if(this.category == null){
				this.category = new ArrayList<>();
			}
			getIndex(this.category, _idx, () -> category);
			this.category.set(_idx, category);
			return this;
		}
		public PackageInformationBuilder addCategory(List<CategoryEnum> categorys) {
			if(this.category == null){
				this.category = new ArrayList<>();
			}
			for (CategoryEnum toAdd : categorys) {
				this.category.add(toAdd);
			}
			return this;
		}
		
		public PackageInformationBuilder clearCategory() {
			this.category = null;
			return this;
		}
		public PackageInformationBuilder setIntentToAllocate(Boolean intentToAllocate) {
			this.intentToAllocate = intentToAllocate;
			return this;
		}
		public PackageInformationBuilder addRelatedParty(RelatedParty relatedParty) {
			if(this.relatedParty == null){
				this.relatedParty = new ArrayList<>();
			}
			this.relatedParty.add(relatedParty.toBuilder());
			return this;
		}
		public PackageInformationBuilder addRelatedParty(RelatedParty relatedParty, int _idx) {
			if(this.relatedParty == null){
				this.relatedParty = new ArrayList<>();
			}
			getIndex(this.relatedParty, _idx, () -> relatedParty.toBuilder());
			this.relatedParty.set(_idx, relatedParty.toBuilder());
			return this;
		}
		public PackageInformationBuilder addRelatedParty(List<RelatedParty> relatedPartys) {
			if(this.relatedParty == null){
				this.relatedParty = new ArrayList<>();
			}
			for (RelatedParty toAdd : relatedPartys) {
				this.relatedParty.add(toAdd.toBuilder());
			}
			return this;
		}
		public PackageInformationBuilder addRelatedPartyBuilder(RelatedParty.RelatedPartyBuilder relatedParty) {
			if(this.relatedParty == null){
				this.relatedParty = new ArrayList<>();
			}
			this.relatedParty.add(relatedParty);
			return this;
		}
		
		public PackageInformationBuilder clearRelatedParty() {
			this.relatedParty = null;
			return this;
		}
		
		public PackageInformation build() {
			return new PackageInformation(this);
		}
	
		@Override
		public PackageInformationBuilder prune() {
			if (relatedParty!=null) relatedParty = relatedParty.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCategory()!=null && !getCategory().isEmpty()) return true;
			if (getIntentToAllocate()!=null) return true;
			if (getRelatedParty()!=null && getRelatedParty().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("category"), CategoryEnum.class, category, this);
			processor.processBasic(path.newSubPath("intentToAllocate"), Boolean.class, intentToAllocate, this);
			
			processRosetta(path.newSubPath("relatedParty"), processor, RelatedParty.class, relatedParty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PackageInformationBuilder _that = (PackageInformationBuilder) o;
		
			if (!ListEquals.listEquals(category, _that.category)) return false;
			if (intentToAllocate != null ? !intentToAllocate.equals(_that.intentToAllocate) : _that.intentToAllocate != null) return false;
			if (!ListEquals.listEquals(relatedParty, _that.relatedParty)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (category != null ? category.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (intentToAllocate != null ? intentToAllocate.hashCode() : 0);
			_result = 31 * _result + (relatedParty != null ? relatedParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PackageInformationBuilder {" +
				"category=" + this.category + ", " +
				"intentToAllocate=" + this.intentToAllocate + ", " +
				"relatedParty=" + this.relatedParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		category.stream().forEach(a->processor.processBasic(path.newSubPath("category"), CategoryEnum.class, a, this));
		processor.processBasic(path.newSubPath("intentToAllocate"), Boolean.class, intentToAllocate, this);
		
		processRosetta(path.newSubPath("relatedParty"), processor, RelatedParty.class, relatedParty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PackageInformation _that = (PackageInformation) o;
	
		if (!ListEquals.listEquals(category, _that.category)) return false;
		if (intentToAllocate != null ? !intentToAllocate.equals(_that.intentToAllocate) : _that.intentToAllocate != null) return false;
		if (!ListEquals.listEquals(relatedParty, _that.relatedParty)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (category != null ? category.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		_result = 31 * _result + (intentToAllocate != null ? intentToAllocate.hashCode() : 0);
		_result = 31 * _result + (relatedParty != null ? relatedParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PackageInformation {" +
			"category=" + this.category + ", " +
			"intentToAllocate=" + this.intentToAllocate + ", " +
			"relatedParty=" + this.relatedParty +
		'}';
	}
}
