package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of package transaction.
 * 
 * @version 2.5.4
 */
public enum PackageTypeEnum {
    

	/**
	 * A strategy in which a firm either pays or receives fixed for intermediate term (the &#39;body&#39;), and does the opposite (receives or pays fixed) for a short and a long term (the &#39;wings&#39;&#39;).
	 */
	@RosettaSynonym(value = "Butterfly", source = "FpML_5_10")
	@RosettaSynonym(value = "Butterfly", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Butterfly", source = "DTCC_11_0")
	@RosettaSynonym(value = "Butterfly", source = "DTCC_9_0")
	@RosettaSynonym(value = "Butterfly", source = "CME_ClearedConfirm_1_17")
	BUTTERFLY,
	    

	/**
	 * A strategy in which a swap is used to Roll from one IMM date into another IMM swap.
	 */
	@RosettaSynonym(value = "CalendarRoll", source = "FpML_5_10")
	@RosettaSynonym(value = "CalendarRoll", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalendarRoll", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalendarRoll", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalendarRoll", source = "CME_ClearedConfirm_1_17")
	CALENDAR_ROLL,
	    

	/**
	 * A strategy in which 2 trades on different dates are done at the same time, e.g., Sept vs June.
	 */
	@RosettaSynonym(value = "CalendarSpread", source = "FpML_5_10")
	@RosettaSynonym(value = "CalendarSpread", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalendarSpread", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalendarSpread", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalendarSpread", source = "CME_ClearedConfirm_1_17")
	CALENDAR_SPREAD,
	    

	/**
	 * A package created for a particular client need e.g. portfolioCompression, termination.
	 */
	@RosettaSynonym(value = "Custom", source = "FpML_5_10")
	@RosettaSynonym(value = "Custom", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Custom", source = "DTCC_11_0")
	@RosettaSynonym(value = "Custom", source = "DTCC_9_0")
	@RosettaSynonym(value = "Custom", source = "CME_ClearedConfirm_1_17")
	CUSTOM,
	    

	/**
	 * A strategy in which a firms buys new version of index and sells and old version of the same index.
	 */
	@RosettaSynonym(value = "IndexRoll", source = "FpML_5_10")
	@RosettaSynonym(value = "IndexRoll", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IndexRoll", source = "DTCC_11_0")
	@RosettaSynonym(value = "IndexRoll", source = "DTCC_9_0")
	@RosettaSynonym(value = "IndexRoll", source = "CME_ClearedConfirm_1_17")
	INDEX_ROLL,
	    

	/**
	 * A package in which only a maximum of one of the components will be executed (used for credit limit checking/orders).
	 */
	@RosettaSynonym(value = "OneCancelsOthers", source = "FpML_5_10")
	@RosettaSynonym(value = "OneCancelsOthers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OneCancelsOthers", source = "DTCC_11_0")
	@RosettaSynonym(value = "OneCancelsOthers", source = "DTCC_9_0")
	@RosettaSynonym(value = "OneCancelsOthers", source = "CME_ClearedConfirm_1_17")
	ONE_CANCELS_OTHERS,
	    

	/**
	 * A strategy in which a firm either buys a treasury and enters a payer swap, or sells treasury and enters a receiver swap.
	 */
	@RosettaSynonym(value = "SwapSpread", source = "FpML_5_10")
	@RosettaSynonym(value = "SwapSpread", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SwapSpread", source = "DTCC_11_0")
	@RosettaSynonym(value = "SwapSpread", source = "DTCC_9_0")
	@RosettaSynonym(value = "SwapSpread", source = "CME_ClearedConfirm_1_17")
	SWAP_SPREAD,
	    

	/**
	 * A strategy in which a firm either pays or receives fixed for some term versus the opposite (receives or pays fixed) for different term. Typically the second term starts at the completion of the first.
	 */
	@RosettaSynonym(value = "Switch", source = "FpML_5_10")
	@RosettaSynonym(value = "Switch", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Switch", source = "DTCC_11_0")
	@RosettaSynonym(value = "Switch", source = "DTCC_9_0")
	@RosettaSynonym(value = "Switch", source = "CME_ClearedConfirm_1_17")
	SWITCH    
	;

	private final String displayName;
	
	PackageTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
