package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Account;
import org.isda.cdm.NaturalPerson;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify a party, without a qualification as to whether this party is a legal entity or a natural person, although the model provides the ability to associate a person (or set of persons) to a party, which use case would imply that such party would be a legal entity (even if not formally specified as such). 
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Party", source="FpML_5_10")
@RosettaSynonym(value="Party", source="DTCC_11_0")
@RosettaSynonym(value="Party", source="DTCC_9_0")
@RosettaSynonym(value="Party", source="CME_ClearedConfirm_1_17")
@RosettaSynonym(value="Party", source="CME_SubmissionIRS_1_0")
public class Party extends RosettaModelObject implements GlobalKey {
	private final Account account;
	private final MetaFields meta;
	private final FieldWithMetaString name;
	private final List<FieldWithMetaString> partyId;
	private final List<NaturalPerson> person;
	private static PartyMeta metaData = new PartyMeta();
	
	Party(PartyBuilder builder) {
		this.account = ofNullable(builder.getAccount()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.name = ofNullable(builder.getName()).map(f->f.build()).orElse(null);
		this.partyId = ofNullable(builder.getPartyId()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.person = ofNullable(builder.getPerson()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The account that might be associated with the party. At most one account can be specified, as it is expected that this information is used in the context of a contract or legal document where only one account per party can be associated with such object.
	 */
	@RosettaSynonym(value="account", source="FpML_5_10")
	@RosettaSynonym(value="account", source="DTCC_11_0")
	@RosettaSynonym(value="account", source="DTCC_9_0")
	@RosettaSynonym(value="account", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="account", source="CME_SubmissionIRS_1_0")
	public final Account getAccount() {
		return account;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The party name.
	 */
	@RosettaSynonym(value="partyName", source="FpML_5_10")
	@RosettaSynonym(value="partyName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyName", source="DTCC_11_0")
	@RosettaSynonym(value="partyName", source="DTCC_9_0")
	@RosettaSynonym(value="partyName", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="entityName", source="FpML_5_10")
	@RosettaSynonym(value="entityName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="entityName", source="DTCC_11_0")
	@RosettaSynonym(value="entityName", source="DTCC_9_0")
	@RosettaSynonym(value="entityName", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="entityName", source="FpML_5_10", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="CME_SubmissionIRS_1_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="DTCC_11_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="DTCC_9_0", path="referenceEntity")
	@RosettaSynonym(value="entityName", source="CME_ClearedConfirm_1_17", path="referenceEntity")
	@RosettaSynonym(value="SID", source="CME_SubmissionIRS_1_0", path="Hdr")
	@RosettaSynonym(value="partyA_name", source="ISDA_Create_1_0", path="answers.parties")
	@RosettaSynonym(value="partyB_name", source="ISDA_Create_1_0", path="answers.parties")
	public final FieldWithMetaString getName() {
		return name;
	}
	
	/**
	 * The identifier associated with a party, e.g. the 20 digits LEI code.
	 */
	@RosettaSynonym(value="partyId", source="FpML_5_10")
	@RosettaSynonym(value="partyId", source="DTCC_11_0")
	@RosettaSynonym(value="partyId", source="DTCC_9_0")
	@RosettaSynonym(value="partyId", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="partyId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyId", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="partyId", source="DTCC_9_0", maps=2)
	public final List<FieldWithMetaString> getPartyId() {
		return partyId;
	}
	
	/**
	 * The person(s) who might be associated with the party as part of the execution, contract or legal document.
	 */
	@RosettaSynonym(value="person", source="FpML_5_10")
	@RosettaSynonym(value="person", source="DTCC_11_0")
	@RosettaSynonym(value="person", source="DTCC_9_0")
	@RosettaSynonym(value="person", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="person", source="CME_SubmissionIRS_1_0")
	public final List<NaturalPerson> getPerson() {
		return person;
	}
	
	
	@Override
	public RosettaMetaData<? extends Party> metaData() {
		return metaData;
	} 
	
	public PartyBuilder toBuilder() {
		PartyBuilder builder = new PartyBuilder();
		ofNullable(getAccount()).ifPresent(builder::setAccount);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getName()).ifPresent(builder::setName);
		ofNullable(getPartyId()).ifPresent(partyId -> partyId.forEach(builder::addPartyId));
		ofNullable(getPerson()).ifPresent(person -> person.forEach(builder::addPerson));
		return builder;
	}

	public static PartyBuilder builder() {
		return new PartyBuilder();
	}

	public static  class PartyBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PartyBuilder> {
	
		protected Account.AccountBuilder account;
		protected MetaFields.MetaFieldsBuilder meta;
		protected FieldWithMetaStringBuilder name;
		protected List<FieldWithMetaStringBuilder> partyId;
		protected List<NaturalPerson.NaturalPersonBuilder> person;
	
		public PartyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Party> metaData() {
			return metaData;
		} 
	
		
		public Account.AccountBuilder getAccount() {
			return account;
		}
		
		public Account.AccountBuilder getOrCreateAccount() {
			if (account!=null) {
				return account;
			}
			else return account = new Account.AccountBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getName() {
			return name;
		}
		
		public FieldWithMetaStringBuilder getOrCreateName() {
			if (name!=null) {
				return name;
			}
			else return name = new FieldWithMetaStringBuilder();
		}
		
		
		public List<FieldWithMetaStringBuilder> getPartyId() {
			return partyId;
		}
		
		public FieldWithMetaStringBuilder getOrCreatePartyId(int index) {
			if (partyId==null) {
				this.partyId = new ArrayList<>();
			}
			return getIndex(partyId, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public List<NaturalPerson.NaturalPersonBuilder> getPerson() {
			return person;
		}
		
		public NaturalPerson.NaturalPersonBuilder getOrCreatePerson(int index) {
			if (person==null) {
				this.person = new ArrayList<>();
			}
			return getIndex(person, index, ()->new NaturalPerson.NaturalPersonBuilder());
		}
		
	
		public PartyBuilder setAccount(Account account) {
			this.account = account.toBuilder();
			return this;
		}
		public PartyBuilder setAccountBuilder(Account.AccountBuilder account) {
			this.account = account;
			return this;
		}
		public PartyBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PartyBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PartyBuilder setName(FieldWithMetaString name) {
			this.name = name.toBuilder();
			return this;
		}
		public PartyBuilder setNameRef(String name) {
			return setName(FieldWithMetaString.builder().setValue(name).build());
		}
		public PartyBuilder addPartyId(FieldWithMetaString partyId) {
			if(this.partyId == null){
				this.partyId = new ArrayList<>();
			}
			this.partyId.add(partyId.toBuilder());
			return this;
		}
		public PartyBuilder addPartyId(FieldWithMetaString partyId, int _idx) {
			if(this.partyId == null){
				this.partyId = new ArrayList<>();
			}
			getIndex(this.partyId, _idx, () -> partyId.toBuilder());
			this.partyId.set(_idx, partyId.toBuilder());
			return this;
		}
		public PartyBuilder addPartyId(List<FieldWithMetaString> partyIds) {
			if(this.partyId == null){
				this.partyId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : partyIds) {
				this.partyId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public PartyBuilder clearPartyId() {
			this.partyId = null;
			return this;
		}
		public PartyBuilder addPerson(NaturalPerson person) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			this.person.add(person.toBuilder());
			return this;
		}
		public PartyBuilder addPerson(NaturalPerson person, int _idx) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			getIndex(this.person, _idx, () -> person.toBuilder());
			this.person.set(_idx, person.toBuilder());
			return this;
		}
		public PartyBuilder addPerson(List<NaturalPerson> persons) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			for (NaturalPerson toAdd : persons) {
				this.person.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyBuilder addPersonBuilder(NaturalPerson.NaturalPersonBuilder person) {
			if(this.person == null){
				this.person = new ArrayList<>();
			}
			this.person.add(person);
			return this;
		}
		
		public PartyBuilder clearPerson() {
			this.person = null;
			return this;
		}
		
		public Party build() {
			return new Party(this);
		}
	
		@Override
		public PartyBuilder prune() {
			if (account!=null && !account.prune().hasData()) account = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (name!=null && !name.prune().hasData()) name = null;
			if (partyId!=null) partyId = partyId.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (person!=null) person = person.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccount()!=null && getAccount().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getName()!=null) return true;
			if (getPartyId()!=null && !getPartyId().isEmpty()) return true;
			if (getPerson()!=null && getPerson().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("account"), processor, Account.class, account);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("name"), processor, FieldWithMetaString.class, name);
			processRosetta(path.newSubPath("partyId"), processor, FieldWithMetaString.class, partyId);
			processRosetta(path.newSubPath("person"), processor, NaturalPerson.class, person);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyBuilder _that = (PartyBuilder) o;
		
			if (account != null ? !account.equals(_that.account) : _that.account != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			if (!ListEquals.listEquals(partyId, _that.partyId)) return false;
			if (!ListEquals.listEquals(person, _that.person)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (account != null ? account.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (name != null ? name.hashCode() : 0);
			_result = 31 * _result + (partyId != null ? partyId.hashCode() : 0);
			_result = 31 * _result + (person != null ? person.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyBuilder {" +
				"account=" + this.account + ", " +
				"meta=" + this.meta + ", " +
				"name=" + this.name + ", " +
				"partyId=" + this.partyId + ", " +
				"person=" + this.person +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("account"), processor, Account.class, account);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("name"), processor, FieldWithMetaString.class, name);
		processRosetta(path.newSubPath("partyId"), processor, FieldWithMetaString.class, partyId);
		processRosetta(path.newSubPath("person"), processor, NaturalPerson.class, person);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Party _that = (Party) o;
	
		if (account != null ? !account.equals(_that.account) : _that.account != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		if (!ListEquals.listEquals(partyId, _that.partyId)) return false;
		if (!ListEquals.listEquals(person, _that.person)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (account != null ? account.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (name != null ? name.hashCode() : 0);
		_result = 31 * _result + (partyId != null ? partyId.hashCode() : 0);
		_result = 31 * _result + (person != null ? person.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Party {" +
			"account=" + this.account + ", " +
			"meta=" + this.meta + ", " +
			"name=" + this.name + ", " +
			"partyId=" + this.partyId + ", " +
			"person=" + this.person +
		'}';
	}
}
