package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaIdentifier;
import org.isda.cdm.metafields.FieldWithMetaIdentifier.FieldWithMetaIdentifierBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyAgreementIdentifierMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a legal agreement identifier issued by the indicated party.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PartyDocumentIdentifier", source="FpML_5_10")
public class PartyAgreementIdentifier extends RosettaModelObject implements GlobalKey {
	private final List<FieldWithMetaIdentifier> documentIdentifier;
	private final MetaFields meta;
	private final String partyReference;
	private static PartyAgreementIdentifierMeta metaData = new PartyAgreementIdentifierMeta();
	
	PartyAgreementIdentifier(PartyAgreementIdentifierBuilder builder) {
		this.documentIdentifier = ofNullable(builder.getDocumentIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.partyReference = builder.getPartyReference();
	}
	
	/**
	 * While FpML specifies the document identifier with a value and an associated scheme, the CDM makes use of the Identifier, which has an explicit issuer. The issuer of this identifier is not necessarily the same as the party reference.
	 */
	@RosettaSynonym(value="documentId", source="FpML_5_10")
	public final List<FieldWithMetaIdentifier> getDocumentIdentifier() {
		return documentIdentifier;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * Reference to the party that issued the document identifier.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	public final String getPartyReference() {
		return partyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyAgreementIdentifier> metaData() {
		return metaData;
	} 
	
	public PartyAgreementIdentifierBuilder toBuilder() {
		PartyAgreementIdentifierBuilder builder = new PartyAgreementIdentifierBuilder();
		ofNullable(getDocumentIdentifier()).ifPresent(documentIdentifier -> documentIdentifier.forEach(builder::addDocumentIdentifier));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		return builder;
	}

	public static PartyAgreementIdentifierBuilder builder() {
		return new PartyAgreementIdentifierBuilder();
	}

	public static  class PartyAgreementIdentifierBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PartyAgreementIdentifierBuilder> {
	
		protected List<FieldWithMetaIdentifierBuilder> documentIdentifier;
		protected MetaFields.MetaFieldsBuilder meta;
		protected String partyReference;
	
		public PartyAgreementIdentifierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyAgreementIdentifier> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaIdentifierBuilder> getDocumentIdentifier() {
			return documentIdentifier;
		}
		
		public FieldWithMetaIdentifierBuilder getOrCreateDocumentIdentifier(int index) {
			if (documentIdentifier==null) {
				this.documentIdentifier = new ArrayList<>();
			}
			return getIndex(documentIdentifier, index, ()->new FieldWithMetaIdentifierBuilder());
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public String getPartyReference() {
			return partyReference;
		}
		
	
		public PartyAgreementIdentifierBuilder addDocumentIdentifier(FieldWithMetaIdentifier documentIdentifier) {
			if(this.documentIdentifier == null){
				this.documentIdentifier = new ArrayList<>();
			}
			this.documentIdentifier.add(documentIdentifier.toBuilder());
			return this;
		}
		public PartyAgreementIdentifierBuilder addDocumentIdentifier(FieldWithMetaIdentifier documentIdentifier, int _idx) {
			if(this.documentIdentifier == null){
				this.documentIdentifier = new ArrayList<>();
			}
			getIndex(this.documentIdentifier, _idx, () -> documentIdentifier.toBuilder());
			this.documentIdentifier.set(_idx, documentIdentifier.toBuilder());
			return this;
		}
		public PartyAgreementIdentifierBuilder addDocumentIdentifier(List<FieldWithMetaIdentifier> documentIdentifiers) {
			if(this.documentIdentifier == null){
				this.documentIdentifier = new ArrayList<>();
			}
			for (FieldWithMetaIdentifier toAdd : documentIdentifiers) {
				this.documentIdentifier.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyAgreementIdentifierBuilder addDocumentIdentifierBuilder(FieldWithMetaIdentifierBuilder documentIdentifier) {
			if(this.documentIdentifier == null){
				this.documentIdentifier = new ArrayList<>();
			}
			this.documentIdentifier.add(documentIdentifier);
			return this;
		}
		
		public PartyAgreementIdentifierBuilder addDocumentIdentifierRef(Identifier.IdentifierBuilder documentIdentifier) {
			return addDocumentIdentifier(FieldWithMetaIdentifier.builder().setValueBuilder(documentIdentifier).build());
		}
		
		public PartyAgreementIdentifierBuilder addDocumentIdentifierRef(Identifier.IdentifierBuilder documentIdentifier, int _idx) {
			return addDocumentIdentifier(FieldWithMetaIdentifier.builder().setValueBuilder(documentIdentifier).build(), _idx);
		}
		
		public PartyAgreementIdentifierBuilder addDocumentIdentifierRef(List<Identifier> documentIdentifiers) {
			for (Identifier toAdd : documentIdentifiers) {
				addDocumentIdentifierRef(toAdd);
			}
			return this;
		}
		
		public PartyAgreementIdentifierBuilder addDocumentIdentifierRef(Identifier documentIdentifier) {
			return addDocumentIdentifierRef(documentIdentifier.toBuilder());
		}
		
		public PartyAgreementIdentifierBuilder clearDocumentIdentifier() {
			this.documentIdentifier = null;
			return this;
		}
		public PartyAgreementIdentifierBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PartyAgreementIdentifierBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PartyAgreementIdentifierBuilder setPartyReference(String partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		
		public PartyAgreementIdentifier build() {
			return new PartyAgreementIdentifier(this);
		}
	
		@Override
		public PartyAgreementIdentifierBuilder prune() {
			if (documentIdentifier!=null) documentIdentifier = documentIdentifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDocumentIdentifier()!=null && getDocumentIdentifier().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPartyReference()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("partyReference"), String.class, partyReference, this);
			
			processRosetta(path.newSubPath("documentIdentifier"), processor, FieldWithMetaIdentifier.class, documentIdentifier);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyAgreementIdentifierBuilder _that = (PartyAgreementIdentifierBuilder) o;
		
			if (!ListEquals.listEquals(documentIdentifier, _that.documentIdentifier)) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (documentIdentifier != null ? documentIdentifier.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyAgreementIdentifierBuilder {" +
				"documentIdentifier=" + this.documentIdentifier + ", " +
				"meta=" + this.meta + ", " +
				"partyReference=" + this.partyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("partyReference"), String.class, partyReference, this);
		
		processRosetta(path.newSubPath("documentIdentifier"), processor, FieldWithMetaIdentifier.class, documentIdentifier);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyAgreementIdentifier _that = (PartyAgreementIdentifier) o;
	
		if (!ListEquals.listEquals(documentIdentifier, _that.documentIdentifier)) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (documentIdentifier != null ? documentIdentifier.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyAgreementIdentifier {" +
			"documentIdentifier=" + this.documentIdentifier + ", " +
			"meta=" + this.meta + ", " +
			"partyReference=" + this.partyReference +
		'}';
	}
}
