package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.NaturalPersonRole;
import org.isda.cdm.RelatedParty;
import org.isda.cdm.metafields.FieldWithMetaCategoryEnum;
import org.isda.cdm.metafields.FieldWithMetaCategoryEnum.FieldWithMetaCategoryEnumBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyContractInformationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class defining party-specific additional information that may be recorded with respect to a contract.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PartyTradeInformation", source="FpML_5_10")
public class PartyContractInformation extends RosettaModelObject {
	private final ReferenceWithMetaAccount accountReference;
	private final FieldWithMetaCategoryEnum category;
	private final List<NaturalPersonRole> naturalPersonRole;
	private final ReferenceWithMetaParty partyReference;
	private final RelatedParty relatedParty;
	private static PartyContractInformationMeta metaData = new PartyContractInformationMeta();
	
	PartyContractInformation(PartyContractInformationBuilder builder) {
		this.accountReference = ofNullable(builder.getAccountReference()).map(f->f.build()).orElse(null);
		this.category = ofNullable(builder.getCategory()).map(f->f.build()).orElse(null);
		this.naturalPersonRole = ofNullable(builder.getNaturalPersonRole()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
		this.relatedParty = ofNullable(builder.getRelatedParty()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Reference to an account.
	 */
	@RosettaSynonym(value="accountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getAccountReference() {
		return accountReference;
	}
	
	/**
	 * The qualification of the trade by the counterparty, e.g. customer or principal. This information is relevant for a number of marketplace processes, such as regulatory reporting or clearing.
	 */
	@RosettaSynonym(value="category", source="FpML_5_10")
	@RosettaSynonym(value="category", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="category", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaCategoryEnum getCategory() {
		return category;
	}
	
	/**
	 * The role(s) that natural person(s) may have in relation to the contract.
	 */
	@RosettaSynonym(value="relatedPerson", source="FpML_5_10")
	@RosettaSynonym(value="relatedPerson", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relatedPerson", source="DTCC_11_0")
	@RosettaSynonym(value="relatedPerson", source="DTCC_9_0")
	@RosettaSynonym(value="relatedPerson", source="CME_ClearedConfirm_1_17")
	public final List<NaturalPersonRole> getNaturalPersonRole() {
		return naturalPersonRole;
	}
	
	/**
	 * The reference to the party that owns this party contract information or, in the case of shared trades information, the reference that originated such information.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0")
	@RosettaSynonym(value="partyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	@RosettaSynonym(value="relatedParty", source="FpML_5_10")
	@RosettaSynonym(value="relatedParty", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="relatedParty", source="CME_ClearedConfirm_1_17")
	public final RelatedParty getRelatedParty() {
		return relatedParty;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyContractInformation> metaData() {
		return metaData;
	} 
	
	public PartyContractInformationBuilder toBuilder() {
		PartyContractInformationBuilder builder = new PartyContractInformationBuilder();
		ofNullable(getAccountReference()).ifPresent(builder::setAccountReference);
		ofNullable(getCategory()).ifPresent(builder::setCategory);
		ofNullable(getNaturalPersonRole()).ifPresent(naturalPersonRole -> naturalPersonRole.forEach(builder::addNaturalPersonRole));
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		ofNullable(getRelatedParty()).ifPresent(builder::setRelatedParty);
		return builder;
	}

	public static PartyContractInformationBuilder builder() {
		return new PartyContractInformationBuilder();
	}

	public static  class PartyContractInformationBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaAccountBuilder accountReference;
		protected FieldWithMetaCategoryEnumBuilder category;
		protected List<NaturalPersonRole.NaturalPersonRoleBuilder> naturalPersonRole;
		protected ReferenceWithMetaPartyBuilder partyReference;
		protected RelatedParty.RelatedPartyBuilder relatedParty;
	
		public PartyContractInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyContractInformation> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaAccountBuilder getAccountReference() {
			return accountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateAccountReference() {
			if (accountReference!=null) {
				return accountReference;
			}
			else return accountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public FieldWithMetaCategoryEnumBuilder getCategory() {
			return category;
		}
		
		public FieldWithMetaCategoryEnumBuilder getOrCreateCategory() {
			if (category!=null) {
				return category;
			}
			else return category = new FieldWithMetaCategoryEnumBuilder();
		}
		
		
		public List<NaturalPersonRole.NaturalPersonRoleBuilder> getNaturalPersonRole() {
			return naturalPersonRole;
		}
		
		public NaturalPersonRole.NaturalPersonRoleBuilder getOrCreateNaturalPersonRole(int index) {
			if (naturalPersonRole==null) {
				this.naturalPersonRole = new ArrayList<>();
			}
			return getIndex(naturalPersonRole, index, ()->new NaturalPersonRole.NaturalPersonRoleBuilder());
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public RelatedParty.RelatedPartyBuilder getRelatedParty() {
			return relatedParty;
		}
		
		public RelatedParty.RelatedPartyBuilder getOrCreateRelatedParty() {
			if (relatedParty!=null) {
				return relatedParty;
			}
			else return relatedParty = new RelatedParty.RelatedPartyBuilder();
		}
		
	
		public PartyContractInformationBuilder setAccountReference(ReferenceWithMetaAccount accountReference) {
			this.accountReference = accountReference.toBuilder();
			return this;
		}
		public PartyContractInformationBuilder setAccountReferenceBuilder(ReferenceWithMetaAccountBuilder accountReference) {
			this.accountReference = accountReference;
			return this;
		}
		public PartyContractInformationBuilder setAccountReferenceRef(Account.AccountBuilder accountReference) {
			return setAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(accountReference).build());
		}
		public PartyContractInformationBuilder setAccountReferenceRef(Account accountReference) {
			return setAccountReferenceRef(Account.builder());
		}
		public PartyContractInformationBuilder setCategory(FieldWithMetaCategoryEnum category) {
			this.category = category.toBuilder();
			return this;
		}
		public PartyContractInformationBuilder setCategoryRef(CategoryEnum category) {
			return setCategory(FieldWithMetaCategoryEnum.builder().setValue(category).build());
		}
		public PartyContractInformationBuilder addNaturalPersonRole(NaturalPersonRole naturalPersonRole) {
			if(this.naturalPersonRole == null){
				this.naturalPersonRole = new ArrayList<>();
			}
			this.naturalPersonRole.add(naturalPersonRole.toBuilder());
			return this;
		}
		public PartyContractInformationBuilder addNaturalPersonRole(NaturalPersonRole naturalPersonRole, int _idx) {
			if(this.naturalPersonRole == null){
				this.naturalPersonRole = new ArrayList<>();
			}
			getIndex(this.naturalPersonRole, _idx, () -> naturalPersonRole.toBuilder());
			this.naturalPersonRole.set(_idx, naturalPersonRole.toBuilder());
			return this;
		}
		public PartyContractInformationBuilder addNaturalPersonRole(List<NaturalPersonRole> naturalPersonRoles) {
			if(this.naturalPersonRole == null){
				this.naturalPersonRole = new ArrayList<>();
			}
			for (NaturalPersonRole toAdd : naturalPersonRoles) {
				this.naturalPersonRole.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyContractInformationBuilder addNaturalPersonRoleBuilder(NaturalPersonRole.NaturalPersonRoleBuilder naturalPersonRole) {
			if(this.naturalPersonRole == null){
				this.naturalPersonRole = new ArrayList<>();
			}
			this.naturalPersonRole.add(naturalPersonRole);
			return this;
		}
		
		public PartyContractInformationBuilder clearNaturalPersonRole() {
			this.naturalPersonRole = null;
			return this;
		}
		public PartyContractInformationBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public PartyContractInformationBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public PartyContractInformationBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public PartyContractInformationBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		public PartyContractInformationBuilder setRelatedParty(RelatedParty relatedParty) {
			this.relatedParty = relatedParty.toBuilder();
			return this;
		}
		public PartyContractInformationBuilder setRelatedPartyBuilder(RelatedParty.RelatedPartyBuilder relatedParty) {
			this.relatedParty = relatedParty;
			return this;
		}
		
		public PartyContractInformation build() {
			return new PartyContractInformation(this);
		}
	
		@Override
		public PartyContractInformationBuilder prune() {
			if (accountReference!=null && !accountReference.prune().hasData()) accountReference = null;
			if (category!=null && !category.prune().hasData()) category = null;
			if (naturalPersonRole!=null) naturalPersonRole = naturalPersonRole.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			if (relatedParty!=null && !relatedParty.prune().hasData()) relatedParty = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccountReference()!=null && getAccountReference().hasData()) return true;
			if (getCategory()!=null) return true;
			if (getNaturalPersonRole()!=null && getNaturalPersonRole().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			if (getRelatedParty()!=null && getRelatedParty().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("accountReference"), processor, ReferenceWithMetaAccount.class, accountReference);
			processRosetta(path.newSubPath("category"), processor, FieldWithMetaCategoryEnum.class, category);
			processRosetta(path.newSubPath("naturalPersonRole"), processor, NaturalPersonRole.class, naturalPersonRole);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
			processRosetta(path.newSubPath("relatedParty"), processor, RelatedParty.class, relatedParty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyContractInformationBuilder _that = (PartyContractInformationBuilder) o;
		
			if (accountReference != null ? !accountReference.equals(_that.accountReference) : _that.accountReference != null) return false;
			if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
			if (!ListEquals.listEquals(naturalPersonRole, _that.naturalPersonRole)) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			if (relatedParty != null ? !relatedParty.equals(_that.relatedParty) : _that.relatedParty != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accountReference != null ? accountReference.hashCode() : 0);
			_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (naturalPersonRole != null ? naturalPersonRole.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			_result = 31 * _result + (relatedParty != null ? relatedParty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyContractInformationBuilder {" +
				"accountReference=" + this.accountReference + ", " +
				"category=" + this.category + ", " +
				"naturalPersonRole=" + this.naturalPersonRole + ", " +
				"partyReference=" + this.partyReference + ", " +
				"relatedParty=" + this.relatedParty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("accountReference"), processor, ReferenceWithMetaAccount.class, accountReference);
		processRosetta(path.newSubPath("category"), processor, FieldWithMetaCategoryEnum.class, category);
		processRosetta(path.newSubPath("naturalPersonRole"), processor, NaturalPersonRole.class, naturalPersonRole);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		processRosetta(path.newSubPath("relatedParty"), processor, RelatedParty.class, relatedParty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyContractInformation _that = (PartyContractInformation) o;
	
		if (accountReference != null ? !accountReference.equals(_that.accountReference) : _that.accountReference != null) return false;
		if (category != null ? !category.equals(_that.category) : _that.category != null) return false;
		if (!ListEquals.listEquals(naturalPersonRole, _that.naturalPersonRole)) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		if (relatedParty != null ? !relatedParty.equals(_that.relatedParty) : _that.relatedParty != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accountReference != null ? accountReference.hashCode() : 0);
		_result = 31 * _result + (category != null ? category.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (naturalPersonRole != null ? naturalPersonRole.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		_result = 31 * _result + (relatedParty != null ? relatedParty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyContractInformation {" +
			"accountReference=" + this.accountReference + ", " +
			"category=" + this.category + ", " +
			"naturalPersonRole=" + this.naturalPersonRole + ", " +
			"partyReference=" + this.partyReference + ", " +
			"relatedParty=" + this.relatedParty +
		'}';
	}
}
