package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CustomisedWorkflow;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyCustomisedWorkflowMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.Objects;

/**
 * A class to specify a party-related, non-standardized data in a generic form.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PartyCustomisedWorkflow extends RosettaModelObject {
	private final List<CustomisedWorkflow> customisedWorkflow;
	private final String partyName;
	private final ReferenceWithMetaParty partyReference;
	private static PartyCustomisedWorkflowMeta metaData = new PartyCustomisedWorkflowMeta();
	
	PartyCustomisedWorkflow(PartyCustomisedWorkflowBuilder builder) {
		this.customisedWorkflow = ofNullable(builder.getCustomisedWorkflow()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.partyName = builder.getPartyName();
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Non-standardized data in a generic form.
	 */
	public final List<CustomisedWorkflow> getCustomisedWorkflow() {
		return customisedWorkflow;
	}
	
	/**
	 * The party name to which the workflow pertains to.
	 */
	@RosettaSynonym(value="ID", source="CME_SubmissionIRS_1_0", path="Pty")
	public final String getPartyName() {
		return partyName;
	}
	
	/**
	 * Reference to the party to which the workflow pertains to.
	 */
	@RosettaSynonym(value="partyReference", source="DTCC_11_0")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyCustomisedWorkflow> metaData() {
		return metaData;
	} 
	
	public PartyCustomisedWorkflowBuilder toBuilder() {
		PartyCustomisedWorkflowBuilder builder = new PartyCustomisedWorkflowBuilder();
		ofNullable(getCustomisedWorkflow()).ifPresent(customisedWorkflow -> customisedWorkflow.forEach(builder::addCustomisedWorkflow));
		ofNullable(getPartyName()).ifPresent(builder::setPartyName);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		return builder;
	}

	public static PartyCustomisedWorkflowBuilder builder() {
		return new PartyCustomisedWorkflowBuilder();
	}

	public static  class PartyCustomisedWorkflowBuilder extends RosettaModelObjectBuilder {
	
		protected List<CustomisedWorkflow.CustomisedWorkflowBuilder> customisedWorkflow;
		protected String partyName;
		protected ReferenceWithMetaPartyBuilder partyReference;
	
		public PartyCustomisedWorkflowBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyCustomisedWorkflow> metaData() {
			return metaData;
		} 
	
		
		public List<CustomisedWorkflow.CustomisedWorkflowBuilder> getCustomisedWorkflow() {
			return customisedWorkflow;
		}
		
		public CustomisedWorkflow.CustomisedWorkflowBuilder getOrCreateCustomisedWorkflow(int index) {
			if (customisedWorkflow==null) {
				this.customisedWorkflow = new ArrayList<>();
			}
			return getIndex(customisedWorkflow, index, ()->new CustomisedWorkflow.CustomisedWorkflowBuilder());
		}
		
		
		public String getPartyName() {
			return partyName;
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public PartyCustomisedWorkflowBuilder addCustomisedWorkflow(CustomisedWorkflow customisedWorkflow) {
			if(this.customisedWorkflow == null){
				this.customisedWorkflow = new ArrayList<>();
			}
			this.customisedWorkflow.add(customisedWorkflow.toBuilder());
			return this;
		}
		public PartyCustomisedWorkflowBuilder addCustomisedWorkflow(CustomisedWorkflow customisedWorkflow, int _idx) {
			if(this.customisedWorkflow == null){
				this.customisedWorkflow = new ArrayList<>();
			}
			getIndex(this.customisedWorkflow, _idx, () -> customisedWorkflow.toBuilder());
			this.customisedWorkflow.set(_idx, customisedWorkflow.toBuilder());
			return this;
		}
		public PartyCustomisedWorkflowBuilder addCustomisedWorkflow(List<CustomisedWorkflow> customisedWorkflows) {
			if(this.customisedWorkflow == null){
				this.customisedWorkflow = new ArrayList<>();
			}
			for (CustomisedWorkflow toAdd : customisedWorkflows) {
				this.customisedWorkflow.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyCustomisedWorkflowBuilder addCustomisedWorkflowBuilder(CustomisedWorkflow.CustomisedWorkflowBuilder customisedWorkflow) {
			if(this.customisedWorkflow == null){
				this.customisedWorkflow = new ArrayList<>();
			}
			this.customisedWorkflow.add(customisedWorkflow);
			return this;
		}
		
		public PartyCustomisedWorkflowBuilder clearCustomisedWorkflow() {
			this.customisedWorkflow = null;
			return this;
		}
		public PartyCustomisedWorkflowBuilder setPartyName(String partyName) {
			this.partyName = partyName;
			return this;
		}
		public PartyCustomisedWorkflowBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public PartyCustomisedWorkflowBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public PartyCustomisedWorkflowBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public PartyCustomisedWorkflowBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		
		public PartyCustomisedWorkflow build() {
			return new PartyCustomisedWorkflow(this);
		}
	
		@Override
		public PartyCustomisedWorkflowBuilder prune() {
			if (customisedWorkflow!=null) customisedWorkflow = customisedWorkflow.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCustomisedWorkflow()!=null && getCustomisedWorkflow().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPartyName()!=null) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("partyName"), String.class, partyName, this);
			
			processRosetta(path.newSubPath("customisedWorkflow"), processor, CustomisedWorkflow.class, customisedWorkflow);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyCustomisedWorkflowBuilder _that = (PartyCustomisedWorkflowBuilder) o;
		
			if (!ListEquals.listEquals(customisedWorkflow, _that.customisedWorkflow)) return false;
			if (partyName != null ? !partyName.equals(_that.partyName) : _that.partyName != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (customisedWorkflow != null ? customisedWorkflow.hashCode() : 0);
			_result = 31 * _result + (partyName != null ? partyName.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyCustomisedWorkflowBuilder {" +
				"customisedWorkflow=" + this.customisedWorkflow + ", " +
				"partyName=" + this.partyName + ", " +
				"partyReference=" + this.partyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("partyName"), String.class, partyName, this);
		
		processRosetta(path.newSubPath("customisedWorkflow"), processor, CustomisedWorkflow.class, customisedWorkflow);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyCustomisedWorkflow _that = (PartyCustomisedWorkflow) o;
	
		if (!ListEquals.listEquals(customisedWorkflow, _that.customisedWorkflow)) return false;
		if (partyName != null ? !partyName.equals(_that.partyName) : _that.partyName != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (customisedWorkflow != null ? customisedWorkflow.hashCode() : 0);
		_result = 31 * _result + (partyName != null ? partyName.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyCustomisedWorkflow {" +
			"customisedWorkflow=" + this.customisedWorkflow + ", " +
			"partyName=" + this.partyName + ", " +
			"partyReference=" + this.partyReference +
		'}';
	}
}
