package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.TerminationCurrencyElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PartyTerminationCurrencyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Termination Currency election by the respective parties to the agreement.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PartyTerminationCurrency extends RosettaModelObject {
	private final List<TerminationCurrencyElection> partyElection;
	private static PartyTerminationCurrencyMeta metaData = new PartyTerminationCurrencyMeta();
	
	PartyTerminationCurrency(PartyTerminationCurrencyBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; Termination Currency election.
	 */
	public final List<TerminationCurrencyElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends PartyTerminationCurrency> metaData() {
		return metaData;
	} 
	
	public PartyTerminationCurrencyBuilder toBuilder() {
		PartyTerminationCurrencyBuilder builder = new PartyTerminationCurrencyBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static PartyTerminationCurrencyBuilder builder() {
		return new PartyTerminationCurrencyBuilder();
	}

	public static  class PartyTerminationCurrencyBuilder extends RosettaModelObjectBuilder {
	
		protected List<TerminationCurrencyElection.TerminationCurrencyElectionBuilder> partyElection;
	
		public PartyTerminationCurrencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PartyTerminationCurrency> metaData() {
			return metaData;
		} 
	
		
		public List<TerminationCurrencyElection.TerminationCurrencyElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public TerminationCurrencyElection.TerminationCurrencyElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new TerminationCurrencyElection.TerminationCurrencyElectionBuilder());
		}
		
	
		public PartyTerminationCurrencyBuilder addPartyElection(TerminationCurrencyElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public PartyTerminationCurrencyBuilder addPartyElection(TerminationCurrencyElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public PartyTerminationCurrencyBuilder addPartyElection(List<TerminationCurrencyElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (TerminationCurrencyElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public PartyTerminationCurrencyBuilder addPartyElectionBuilder(TerminationCurrencyElection.TerminationCurrencyElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public PartyTerminationCurrencyBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public PartyTerminationCurrency build() {
			return new PartyTerminationCurrency(this);
		}
	
		@Override
		public PartyTerminationCurrencyBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, TerminationCurrencyElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PartyTerminationCurrencyBuilder _that = (PartyTerminationCurrencyBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PartyTerminationCurrencyBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, TerminationCurrencyElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PartyTerminationCurrency _that = (PartyTerminationCurrency) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PartyTerminationCurrency {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
