package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PassThroughItem;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PassThroughMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Type which contains pass through payments.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PassThrough", source="FpML_5_10")
public class PassThrough extends RosettaModelObject {
	private final List<PassThroughItem> passThroughItem;
	private static PassThroughMeta metaData = new PassThroughMeta();
	
	PassThrough(PassThroughBuilder builder) {
		this.passThroughItem = ofNullable(builder.getPassThroughItem()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * One to many pass through payment items.
	 */
	@RosettaSynonym(value="passThroughItem", source="FpML_5_10")
	@RosettaSynonym(value="passThroughItem", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="passThroughItem", source="DTCC_11_0")
	@RosettaSynonym(value="passThroughItem", source="DTCC_9_0")
	@RosettaSynonym(value="passThroughItem", source="CME_ClearedConfirm_1_17")
	public final List<PassThroughItem> getPassThroughItem() {
		return passThroughItem;
	}
	
	
	@Override
	public RosettaMetaData<? extends PassThrough> metaData() {
		return metaData;
	} 
	
	public PassThroughBuilder toBuilder() {
		PassThroughBuilder builder = new PassThroughBuilder();
		ofNullable(getPassThroughItem()).ifPresent(passThroughItem -> passThroughItem.forEach(builder::addPassThroughItem));
		return builder;
	}

	public static PassThroughBuilder builder() {
		return new PassThroughBuilder();
	}

	public static  class PassThroughBuilder extends RosettaModelObjectBuilder {
	
		protected List<PassThroughItem.PassThroughItemBuilder> passThroughItem;
	
		public PassThroughBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PassThrough> metaData() {
			return metaData;
		} 
	
		
		public List<PassThroughItem.PassThroughItemBuilder> getPassThroughItem() {
			return passThroughItem;
		}
		
		public PassThroughItem.PassThroughItemBuilder getOrCreatePassThroughItem(int index) {
			if (passThroughItem==null) {
				this.passThroughItem = new ArrayList<>();
			}
			return getIndex(passThroughItem, index, ()->new PassThroughItem.PassThroughItemBuilder());
		}
		
	
		public PassThroughBuilder addPassThroughItem(PassThroughItem passThroughItem) {
			if(this.passThroughItem == null){
				this.passThroughItem = new ArrayList<>();
			}
			this.passThroughItem.add(passThroughItem.toBuilder());
			return this;
		}
		public PassThroughBuilder addPassThroughItem(PassThroughItem passThroughItem, int _idx) {
			if(this.passThroughItem == null){
				this.passThroughItem = new ArrayList<>();
			}
			getIndex(this.passThroughItem, _idx, () -> passThroughItem.toBuilder());
			this.passThroughItem.set(_idx, passThroughItem.toBuilder());
			return this;
		}
		public PassThroughBuilder addPassThroughItem(List<PassThroughItem> passThroughItems) {
			if(this.passThroughItem == null){
				this.passThroughItem = new ArrayList<>();
			}
			for (PassThroughItem toAdd : passThroughItems) {
				this.passThroughItem.add(toAdd.toBuilder());
			}
			return this;
		}
		public PassThroughBuilder addPassThroughItemBuilder(PassThroughItem.PassThroughItemBuilder passThroughItem) {
			if(this.passThroughItem == null){
				this.passThroughItem = new ArrayList<>();
			}
			this.passThroughItem.add(passThroughItem);
			return this;
		}
		
		public PassThroughBuilder clearPassThroughItem() {
			this.passThroughItem = null;
			return this;
		}
		
		public PassThrough build() {
			return new PassThrough(this);
		}
	
		@Override
		public PassThroughBuilder prune() {
			if (passThroughItem!=null) passThroughItem = passThroughItem.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPassThroughItem()!=null && getPassThroughItem().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("passThroughItem"), processor, PassThroughItem.class, passThroughItem);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PassThroughBuilder _that = (PassThroughBuilder) o;
		
			if (!ListEquals.listEquals(passThroughItem, _that.passThroughItem)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (passThroughItem != null ? passThroughItem.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PassThroughBuilder {" +
				"passThroughItem=" + this.passThroughItem +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("passThroughItem"), processor, PassThroughItem.class, passThroughItem);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PassThrough _that = (PassThrough) o;
	
		if (!ListEquals.listEquals(passThroughItem, _that.passThroughItem)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (passThroughItem != null ? passThroughItem.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PassThrough {" +
			"passThroughItem=" + this.passThroughItem +
		'}';
	}
}
