package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether payments occur relative to the calculation period start date or end date, each reset date, valuation date or the last pricing date.
 * 
 * @version 2.5.4
 */
public enum PayRelativeToEnum {
    

	/**
	 * Payments will occur relative to the first day of each calculation period.
	 */
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "FpML_5_10")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "CME_ClearedConfirm_1_17")
	CALCULATION_PERIOD_START_DATE,
	    

	/**
	 * Payments will occur relative to the last day of each calculation period.
	 */
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "FpML_5_10")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "CME_ClearedConfirm_1_17")
	CALCULATION_PERIOD_END_DATE,
	    

	/**
	 * Payments will occur relative to the last Pricing Date of each Calculation Period.
	 */
	@RosettaSynonym(value = "LastPricingDate", source = "FpML_5_10")
	@RosettaSynonym(value = "LastPricingDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LastPricingDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "LastPricingDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "LastPricingDate", source = "CME_ClearedConfirm_1_17")
	LAST_PRICING_DATE,
	    

	/**
	 * Payments will occur relative to the reset date.
	 */
	@RosettaSynonym(value = "ResetDate", source = "FpML_5_10")
	@RosettaSynonym(value = "ResetDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ResetDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "ResetDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "ResetDate", source = "CME_ClearedConfirm_1_17")
	RESET_DATE,
	    

	/**
	 * Payments will occur relative to the valuation date.
	 */
	@RosettaSynonym(value = "ValuationDate", source = "FpML_5_10")
	@RosettaSynonym(value = "ValuationDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ValuationDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "ValuationDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "ValuationDate", source = "CME_ClearedConfirm_1_17")
	VALUATION_DATE    
	;

	private final String displayName;
	
	PayRelativeToEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
