package org.isda.cdm;

import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PayerReceiverMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class to represent the FpML PayerReceiver.model.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PayerReceiver extends RosettaModelObject {
	private final ReferenceWithMetaAccount payerAccountReference;
	private final ReferenceWithMetaParty payerPartyReference;
	private final ReferenceWithMetaAccount receiverAccountReference;
	private final ReferenceWithMetaParty receiverPartyReference;
	private static PayerReceiverMeta metaData = new PayerReceiverMeta();
	
	PayerReceiver(PayerReceiverBuilder builder) {
		this.payerAccountReference = ofNullable(builder.getPayerAccountReference()).map(f->f.build()).orElse(null);
		this.payerPartyReference = ofNullable(builder.getPayerPartyReference()).map(f->f.build()).orElse(null);
		this.receiverAccountReference = ofNullable(builder.getReceiverAccountReference()).map(f->f.build()).orElse(null);
		this.receiverPartyReference = ofNullable(builder.getReceiverPartyReference()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A reference to the account responsible for making the payments defined by this structure.
	 */
	@RosettaSynonym(value="payerAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="payerAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="payerAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="payerAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="payerAccountReference", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="buyerAccountReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="buyerAccountReference", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="buyerAccountReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="buyerAccountReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="buyerAccountReference", source="CME_ClearedConfirm_1_17", maps=2)
	public final ReferenceWithMetaAccount getPayerAccountReference() {
		return payerAccountReference;
	}
	
	/**
	 * A reference to the party responsible for making the payments defined by this structure.
	 */
	@RosettaSynonym(value="payerPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="payerPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="payerPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="payerPartyReference", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="buyerPartyReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="CME_ClearedConfirm_1_17", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="FpML_5_10", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_11_0", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="DTCC_9_0", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="buyerPartyReference", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="payerPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyerPartyReference", source="CME_SubmissionIRS_1_0", maps=2)
	public final ReferenceWithMetaParty getPayerPartyReference() {
		return payerPartyReference;
	}
	
	/**
	 * A reference to the account that receives the payments corresponding to this structure.
	 */
	@RosettaSynonym(value="receiverAccountReference", source="FpML_5_10")
	@RosettaSynonym(value="receiverAccountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="receiverAccountReference", source="DTCC_11_0")
	@RosettaSynonym(value="receiverAccountReference", source="DTCC_9_0")
	@RosettaSynonym(value="receiverAccountReference", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="sellerAccountReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="sellerAccountReference", source="CME_SubmissionIRS_1_0", maps=2)
	@RosettaSynonym(value="sellerAccountReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="sellerAccountReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="sellerAccountReference", source="CME_ClearedConfirm_1_17", maps=2)
	public final ReferenceWithMetaAccount getReceiverAccountReference() {
		return receiverAccountReference;
	}
	
	/**
	 * A reference to the party that receives the payments corresponding to this structure.
	 */
	@RosettaSynonym(value="receiverPartyReference", source="FpML_5_10")
	@RosettaSynonym(value="receiverPartyReference", source="DTCC_11_0")
	@RosettaSynonym(value="receiverPartyReference", source="DTCC_9_0")
	@RosettaSynonym(value="receiverPartyReference", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="sellerPartyReference", source="FpML_5_10", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_11_0", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_9_0", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="CME_ClearedConfirm_1_17", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="FpML_5_10", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_11_0", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="DTCC_9_0", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="sellerPartyReference", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap.generalTerms", maps=2)
	@RosettaSynonym(value="receiverPartyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sellerPartyReference", source="CME_SubmissionIRS_1_0", maps=2)
	public final ReferenceWithMetaParty getReceiverPartyReference() {
		return receiverPartyReference;
	}
	
	
	@Override
	public RosettaMetaData<? extends PayerReceiver> metaData() {
		return metaData;
	} 
	
	public PayerReceiverBuilder toBuilder() {
		PayerReceiverBuilder builder = new PayerReceiverBuilder();
		ofNullable(getPayerAccountReference()).ifPresent(builder::setPayerAccountReference);
		ofNullable(getPayerPartyReference()).ifPresent(builder::setPayerPartyReference);
		ofNullable(getReceiverAccountReference()).ifPresent(builder::setReceiverAccountReference);
		ofNullable(getReceiverPartyReference()).ifPresent(builder::setReceiverPartyReference);
		return builder;
	}

	public static PayerReceiverBuilder builder() {
		return new PayerReceiverBuilder();
	}

	public static  class PayerReceiverBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaAccountBuilder payerAccountReference;
		protected ReferenceWithMetaPartyBuilder payerPartyReference;
		protected ReferenceWithMetaAccountBuilder receiverAccountReference;
		protected ReferenceWithMetaPartyBuilder receiverPartyReference;
	
		public PayerReceiverBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PayerReceiver> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaAccountBuilder getPayerAccountReference() {
			return payerAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreatePayerAccountReference() {
			if (payerAccountReference!=null) {
				return payerAccountReference;
			}
			else return payerAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getPayerPartyReference() {
			return payerPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePayerPartyReference() {
			if (payerPartyReference!=null) {
				return payerPartyReference;
			}
			else return payerPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public ReferenceWithMetaAccountBuilder getReceiverAccountReference() {
			return receiverAccountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateReceiverAccountReference() {
			if (receiverAccountReference!=null) {
				return receiverAccountReference;
			}
			else return receiverAccountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getReceiverPartyReference() {
			return receiverPartyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreateReceiverPartyReference() {
			if (receiverPartyReference!=null) {
				return receiverPartyReference;
			}
			else return receiverPartyReference = new ReferenceWithMetaPartyBuilder();
		}
		
	
		public PayerReceiverBuilder setPayerAccountReference(ReferenceWithMetaAccount payerAccountReference) {
			this.payerAccountReference = payerAccountReference.toBuilder();
			return this;
		}
		public PayerReceiverBuilder setPayerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder payerAccountReference) {
			this.payerAccountReference = payerAccountReference;
			return this;
		}
		public PayerReceiverBuilder setPayerAccountReferenceRef(Account.AccountBuilder payerAccountReference) {
			return setPayerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(payerAccountReference).build());
		}
		public PayerReceiverBuilder setPayerAccountReferenceRef(Account payerAccountReference) {
			return setPayerAccountReferenceRef(Account.builder());
		}
		public PayerReceiverBuilder setPayerPartyReference(ReferenceWithMetaParty payerPartyReference) {
			this.payerPartyReference = payerPartyReference.toBuilder();
			return this;
		}
		public PayerReceiverBuilder setPayerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder payerPartyReference) {
			this.payerPartyReference = payerPartyReference;
			return this;
		}
		public PayerReceiverBuilder setPayerPartyReferenceRef(Party.PartyBuilder payerPartyReference) {
			return setPayerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(payerPartyReference).build());
		}
		public PayerReceiverBuilder setPayerPartyReferenceRef(Party payerPartyReference) {
			return setPayerPartyReferenceRef(Party.builder());
		}
		public PayerReceiverBuilder setReceiverAccountReference(ReferenceWithMetaAccount receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference.toBuilder();
			return this;
		}
		public PayerReceiverBuilder setReceiverAccountReferenceBuilder(ReferenceWithMetaAccountBuilder receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference;
			return this;
		}
		public PayerReceiverBuilder setReceiverAccountReferenceRef(Account.AccountBuilder receiverAccountReference) {
			return setReceiverAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(receiverAccountReference).build());
		}
		public PayerReceiverBuilder setReceiverAccountReferenceRef(Account receiverAccountReference) {
			return setReceiverAccountReferenceRef(Account.builder());
		}
		public PayerReceiverBuilder setReceiverPartyReference(ReferenceWithMetaParty receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference.toBuilder();
			return this;
		}
		public PayerReceiverBuilder setReceiverPartyReferenceBuilder(ReferenceWithMetaPartyBuilder receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference;
			return this;
		}
		public PayerReceiverBuilder setReceiverPartyReferenceRef(Party.PartyBuilder receiverPartyReference) {
			return setReceiverPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(receiverPartyReference).build());
		}
		public PayerReceiverBuilder setReceiverPartyReferenceRef(Party receiverPartyReference) {
			return setReceiverPartyReferenceRef(Party.builder());
		}
		
		public PayerReceiver build() {
			return new PayerReceiver(this);
		}
	
		@Override
		public PayerReceiverBuilder prune() {
			if (payerAccountReference!=null && !payerAccountReference.prune().hasData()) payerAccountReference = null;
			if (payerPartyReference!=null && !payerPartyReference.prune().hasData()) payerPartyReference = null;
			if (receiverAccountReference!=null && !receiverAccountReference.prune().hasData()) receiverAccountReference = null;
			if (receiverPartyReference!=null && !receiverPartyReference.prune().hasData()) receiverPartyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPayerAccountReference()!=null && getPayerAccountReference().hasData()) return true;
			if (getPayerPartyReference()!=null && getPayerPartyReference().hasData()) return true;
			if (getReceiverAccountReference()!=null && getReceiverAccountReference().hasData()) return true;
			if (getReceiverPartyReference()!=null && getReceiverPartyReference().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("payerAccountReference"), processor, ReferenceWithMetaAccount.class, payerAccountReference);
			processRosetta(path.newSubPath("payerPartyReference"), processor, ReferenceWithMetaParty.class, payerPartyReference);
			processRosetta(path.newSubPath("receiverAccountReference"), processor, ReferenceWithMetaAccount.class, receiverAccountReference);
			processRosetta(path.newSubPath("receiverPartyReference"), processor, ReferenceWithMetaParty.class, receiverPartyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PayerReceiverBuilder _that = (PayerReceiverBuilder) o;
		
			if (payerAccountReference != null ? !payerAccountReference.equals(_that.payerAccountReference) : _that.payerAccountReference != null) return false;
			if (payerPartyReference != null ? !payerPartyReference.equals(_that.payerPartyReference) : _that.payerPartyReference != null) return false;
			if (receiverAccountReference != null ? !receiverAccountReference.equals(_that.receiverAccountReference) : _that.receiverAccountReference != null) return false;
			if (receiverPartyReference != null ? !receiverPartyReference.equals(_that.receiverPartyReference) : _that.receiverPartyReference != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (payerAccountReference != null ? payerAccountReference.hashCode() : 0);
			_result = 31 * _result + (payerPartyReference != null ? payerPartyReference.hashCode() : 0);
			_result = 31 * _result + (receiverAccountReference != null ? receiverAccountReference.hashCode() : 0);
			_result = 31 * _result + (receiverPartyReference != null ? receiverPartyReference.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PayerReceiverBuilder {" +
				"payerAccountReference=" + this.payerAccountReference + ", " +
				"payerPartyReference=" + this.payerPartyReference + ", " +
				"receiverAccountReference=" + this.receiverAccountReference + ", " +
				"receiverPartyReference=" + this.receiverPartyReference +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("payerAccountReference"), processor, ReferenceWithMetaAccount.class, payerAccountReference);
		processRosetta(path.newSubPath("payerPartyReference"), processor, ReferenceWithMetaParty.class, payerPartyReference);
		processRosetta(path.newSubPath("receiverAccountReference"), processor, ReferenceWithMetaAccount.class, receiverAccountReference);
		processRosetta(path.newSubPath("receiverPartyReference"), processor, ReferenceWithMetaParty.class, receiverPartyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PayerReceiver _that = (PayerReceiver) o;
	
		if (payerAccountReference != null ? !payerAccountReference.equals(_that.payerAccountReference) : _that.payerAccountReference != null) return false;
		if (payerPartyReference != null ? !payerPartyReference.equals(_that.payerPartyReference) : _that.payerPartyReference != null) return false;
		if (receiverAccountReference != null ? !receiverAccountReference.equals(_that.receiverAccountReference) : _that.receiverAccountReference != null) return false;
		if (receiverPartyReference != null ? !receiverPartyReference.equals(_that.receiverPartyReference) : _that.receiverPartyReference != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (payerAccountReference != null ? payerAccountReference.hashCode() : 0);
		_result = 31 * _result + (payerPartyReference != null ? payerPartyReference.hashCode() : 0);
		_result = 31 * _result + (receiverAccountReference != null ? receiverAccountReference.hashCode() : 0);
		_result = 31 * _result + (receiverPartyReference != null ? receiverPartyReference.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PayerReceiver {" +
			"payerAccountReference=" + this.payerAccountReference + ", " +
			"payerPartyReference=" + this.payerPartyReference + ", " +
			"receiverAccountReference=" + this.receiverAccountReference + ", " +
			"receiverPartyReference=" + this.receiverPartyReference +
		'}';
	}
}
