package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.CalculationPeriod;
import org.isda.cdm.Money;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PaymentCalculationPeriodMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;

/**
 * A class defining the adjusted payment date and associated calculation period parameters required to calculate the actual or projected payment amount. This class forms part of the cashflow representation of a swap stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PaymentCalculationPeriod", source="FpML_5_10")
@RosettaSynonym(value="PaymentCalculationPeriod", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="PaymentCalculationPeriod", source="DTCC_11_0")
@RosettaSynonym(value="PaymentCalculationPeriod", source="DTCC_9_0")
@RosettaSynonym(value="PaymentCalculationPeriod", source="CME_ClearedConfirm_1_17")
public class PaymentCalculationPeriod extends RosettaModelObject implements GlobalKey {
	private final Date adjustedPaymentDate;
	private final List<CalculationPeriod> calculationPeriod;
	private final BigDecimal discountFactor;
	private final BigDecimal fixedPaymentAmount;
	private final Money forecastPaymentAmount;
	private final MetaFields meta;
	private final Money presentValueAmount;
	private final Date unadjustedPaymentDate;
	private static PaymentCalculationPeriodMeta metaData = new PaymentCalculationPeriodMeta();
	
	PaymentCalculationPeriod(PaymentCalculationPeriodBuilder builder) {
		this.adjustedPaymentDate = builder.getAdjustedPaymentDate();
		this.calculationPeriod = ofNullable(builder.getCalculationPeriod()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.discountFactor = builder.getDiscountFactor();
		this.fixedPaymentAmount = builder.getFixedPaymentAmount();
		this.forecastPaymentAmount = ofNullable(builder.getForecastPaymentAmount()).map(f->f.build()).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.presentValueAmount = ofNullable(builder.getPresentValueAmount()).map(f->f.build()).orElse(null);
		this.unadjustedPaymentDate = builder.getUnadjustedPaymentDate();
	}
	
	/**
	 * The adjusted payment date. This date should already be adjusted for any applicable business day convention. This component is not intended for use in trade confirmation but may be specified to allow the fee structure to also serve as a cashflow type component.
	 */
	@RosettaSynonym(value="adjustedPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedPaymentDate() {
		return adjustedPaymentDate;
	}
	
	/**
	 * The parameters used in the calculation of a fixed or floating rate calculation period amount. A list of calculation period elements may be ordered in the document by ascending start date. An FpML document which contains an unordered list of calculation periods is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="calculationPeriod", source="FpML_5_10")
	@RosettaSynonym(value="calculationPeriod", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="calculationPeriod", source="DTCC_11_0")
	@RosettaSynonym(value="calculationPeriod", source="DTCC_9_0")
	@RosettaSynonym(value="calculationPeriod", source="CME_ClearedConfirm_1_17")
	public final List<CalculationPeriod> getCalculationPeriod() {
		return calculationPeriod;
	}
	
	/**
	 * A decimal value representing the discount factor used to calculate the present value of cash flow.
	 */
	@RosettaSynonym(value="discountFactor", source="FpML_5_10")
	@RosettaSynonym(value="discountFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_11_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_9_0")
	@RosettaSynonym(value="discountFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDiscountFactor() {
		return discountFactor;
	}
	
	/**
	 * A known fixed payment amount.
	 */
	@RosettaSynonym(value="fixedPaymentAmount", source="FpML_5_10")
	@RosettaSynonym(value="fixedPaymentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixedPaymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="fixedPaymentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="fixedPaymentAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getFixedPaymentAmount() {
		return fixedPaymentAmount;
	}
	
	/**
	 * A monetary amount representing the forecast of the future value of the payment.
	 */
	@RosettaSynonym(value="forecastPaymentAmount", source="FpML_5_10")
	@RosettaSynonym(value="forecastPaymentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="forecastPaymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="forecastPaymentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="forecastPaymentAmount", source="CME_ClearedConfirm_1_17")
	public final Money getForecastPaymentAmount() {
		return forecastPaymentAmount;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A monetary amount representing the present value of the forecast payment.
	 */
	@RosettaSynonym(value="presentValueAmount", source="FpML_5_10")
	@RosettaSynonym(value="presentValueAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_11_0")
	@RosettaSynonym(value="presentValueAmount", source="DTCC_9_0")
	@RosettaSynonym(value="presentValueAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPresentValueAmount() {
		return presentValueAmount;
	}
	
	/**
	 * The unadjusted payment date.
	 */
	@RosettaSynonym(value="unadjustedPaymentDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedPaymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedPaymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedPaymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedPaymentDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedPaymentDate() {
		return unadjustedPaymentDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends PaymentCalculationPeriod> metaData() {
		return metaData;
	} 
	
	public PaymentCalculationPeriodBuilder toBuilder() {
		PaymentCalculationPeriodBuilder builder = new PaymentCalculationPeriodBuilder();
		ofNullable(getAdjustedPaymentDate()).ifPresent(builder::setAdjustedPaymentDate);
		ofNullable(getCalculationPeriod()).ifPresent(calculationPeriod -> calculationPeriod.forEach(builder::addCalculationPeriod));
		ofNullable(getDiscountFactor()).ifPresent(builder::setDiscountFactor);
		ofNullable(getFixedPaymentAmount()).ifPresent(builder::setFixedPaymentAmount);
		ofNullable(getForecastPaymentAmount()).ifPresent(builder::setForecastPaymentAmount);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPresentValueAmount()).ifPresent(builder::setPresentValueAmount);
		ofNullable(getUnadjustedPaymentDate()).ifPresent(builder::setUnadjustedPaymentDate);
		return builder;
	}

	public static PaymentCalculationPeriodBuilder builder() {
		return new PaymentCalculationPeriodBuilder();
	}

	public static  class PaymentCalculationPeriodBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PaymentCalculationPeriodBuilder> {
	
		protected Date adjustedPaymentDate;
		protected List<CalculationPeriod.CalculationPeriodBuilder> calculationPeriod;
		protected BigDecimal discountFactor;
		protected BigDecimal fixedPaymentAmount;
		protected Money.MoneyBuilder forecastPaymentAmount;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder presentValueAmount;
		protected Date unadjustedPaymentDate;
	
		public PaymentCalculationPeriodBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PaymentCalculationPeriod> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedPaymentDate() {
			return adjustedPaymentDate;
		}
		
		
		public List<CalculationPeriod.CalculationPeriodBuilder> getCalculationPeriod() {
			return calculationPeriod;
		}
		
		public CalculationPeriod.CalculationPeriodBuilder getOrCreateCalculationPeriod(int index) {
			if (calculationPeriod==null) {
				this.calculationPeriod = new ArrayList<>();
			}
			return getIndex(calculationPeriod, index, ()->new CalculationPeriod.CalculationPeriodBuilder());
		}
		
		
		public BigDecimal getDiscountFactor() {
			return discountFactor;
		}
		
		
		public BigDecimal getFixedPaymentAmount() {
			return fixedPaymentAmount;
		}
		
		
		public Money.MoneyBuilder getForecastPaymentAmount() {
			return forecastPaymentAmount;
		}
		
		public Money.MoneyBuilder getOrCreateForecastPaymentAmount() {
			if (forecastPaymentAmount!=null) {
				return forecastPaymentAmount;
			}
			else return forecastPaymentAmount = new Money.MoneyBuilder();
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getPresentValueAmount() {
			return presentValueAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePresentValueAmount() {
			if (presentValueAmount!=null) {
				return presentValueAmount;
			}
			else return presentValueAmount = new Money.MoneyBuilder();
		}
		
		
		public Date getUnadjustedPaymentDate() {
			return unadjustedPaymentDate;
		}
		
	
		public PaymentCalculationPeriodBuilder setAdjustedPaymentDate(Date adjustedPaymentDate) {
			this.adjustedPaymentDate = adjustedPaymentDate;
			return this;
		}
		public PaymentCalculationPeriodBuilder addCalculationPeriod(CalculationPeriod calculationPeriod) {
			if(this.calculationPeriod == null){
				this.calculationPeriod = new ArrayList<>();
			}
			this.calculationPeriod.add(calculationPeriod.toBuilder());
			return this;
		}
		public PaymentCalculationPeriodBuilder addCalculationPeriod(CalculationPeriod calculationPeriod, int _idx) {
			if(this.calculationPeriod == null){
				this.calculationPeriod = new ArrayList<>();
			}
			getIndex(this.calculationPeriod, _idx, () -> calculationPeriod.toBuilder());
			this.calculationPeriod.set(_idx, calculationPeriod.toBuilder());
			return this;
		}
		public PaymentCalculationPeriodBuilder addCalculationPeriod(List<CalculationPeriod> calculationPeriods) {
			if(this.calculationPeriod == null){
				this.calculationPeriod = new ArrayList<>();
			}
			for (CalculationPeriod toAdd : calculationPeriods) {
				this.calculationPeriod.add(toAdd.toBuilder());
			}
			return this;
		}
		public PaymentCalculationPeriodBuilder addCalculationPeriodBuilder(CalculationPeriod.CalculationPeriodBuilder calculationPeriod) {
			if(this.calculationPeriod == null){
				this.calculationPeriod = new ArrayList<>();
			}
			this.calculationPeriod.add(calculationPeriod);
			return this;
		}
		
		public PaymentCalculationPeriodBuilder clearCalculationPeriod() {
			this.calculationPeriod = null;
			return this;
		}
		public PaymentCalculationPeriodBuilder setDiscountFactor(BigDecimal discountFactor) {
			this.discountFactor = discountFactor;
			return this;
		}
		public PaymentCalculationPeriodBuilder setFixedPaymentAmount(BigDecimal fixedPaymentAmount) {
			this.fixedPaymentAmount = fixedPaymentAmount;
			return this;
		}
		public PaymentCalculationPeriodBuilder setForecastPaymentAmount(Money forecastPaymentAmount) {
			this.forecastPaymentAmount = forecastPaymentAmount.toBuilder();
			return this;
		}
		public PaymentCalculationPeriodBuilder setForecastPaymentAmountBuilder(Money.MoneyBuilder forecastPaymentAmount) {
			this.forecastPaymentAmount = forecastPaymentAmount;
			return this;
		}
		public PaymentCalculationPeriodBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PaymentCalculationPeriodBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PaymentCalculationPeriodBuilder setPresentValueAmount(Money presentValueAmount) {
			this.presentValueAmount = presentValueAmount.toBuilder();
			return this;
		}
		public PaymentCalculationPeriodBuilder setPresentValueAmountBuilder(Money.MoneyBuilder presentValueAmount) {
			this.presentValueAmount = presentValueAmount;
			return this;
		}
		public PaymentCalculationPeriodBuilder setUnadjustedPaymentDate(Date unadjustedPaymentDate) {
			this.unadjustedPaymentDate = unadjustedPaymentDate;
			return this;
		}
		
		public PaymentCalculationPeriod build() {
			return new PaymentCalculationPeriod(this);
		}
	
		@Override
		public PaymentCalculationPeriodBuilder prune() {
			if (calculationPeriod!=null) calculationPeriod = calculationPeriod.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (forecastPaymentAmount!=null && !forecastPaymentAmount.prune().hasData()) forecastPaymentAmount = null;
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (presentValueAmount!=null && !presentValueAmount.prune().hasData()) presentValueAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedPaymentDate()!=null) return true;
			if (getCalculationPeriod()!=null && getCalculationPeriod().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getDiscountFactor()!=null) return true;
			if (getFixedPaymentAmount()!=null) return true;
			if (getForecastPaymentAmount()!=null && getForecastPaymentAmount().hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPresentValueAmount()!=null && getPresentValueAmount().hasData()) return true;
			if (getUnadjustedPaymentDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedPaymentDate"), Date.class, adjustedPaymentDate, this);
			processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
			processor.processBasic(path.newSubPath("fixedPaymentAmount"), BigDecimal.class, fixedPaymentAmount, this);
			processor.processBasic(path.newSubPath("unadjustedPaymentDate"), Date.class, unadjustedPaymentDate, this);
			
			processRosetta(path.newSubPath("calculationPeriod"), processor, CalculationPeriod.class, calculationPeriod);
			processRosetta(path.newSubPath("forecastPaymentAmount"), processor, Money.class, forecastPaymentAmount);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PaymentCalculationPeriodBuilder _that = (PaymentCalculationPeriodBuilder) o;
		
			if (adjustedPaymentDate != null ? !adjustedPaymentDate.equals(_that.adjustedPaymentDate) : _that.adjustedPaymentDate != null) return false;
			if (!ListEquals.listEquals(calculationPeriod, _that.calculationPeriod)) return false;
			if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
			if (fixedPaymentAmount != null ? !fixedPaymentAmount.equals(_that.fixedPaymentAmount) : _that.fixedPaymentAmount != null) return false;
			if (forecastPaymentAmount != null ? !forecastPaymentAmount.equals(_that.forecastPaymentAmount) : _that.forecastPaymentAmount != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
			if (unadjustedPaymentDate != null ? !unadjustedPaymentDate.equals(_that.unadjustedPaymentDate) : _that.unadjustedPaymentDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedPaymentDate != null ? adjustedPaymentDate.hashCode() : 0);
			_result = 31 * _result + (calculationPeriod != null ? calculationPeriod.hashCode() : 0);
			_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
			_result = 31 * _result + (fixedPaymentAmount != null ? fixedPaymentAmount.hashCode() : 0);
			_result = 31 * _result + (forecastPaymentAmount != null ? forecastPaymentAmount.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
			_result = 31 * _result + (unadjustedPaymentDate != null ? unadjustedPaymentDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaymentCalculationPeriodBuilder {" +
				"adjustedPaymentDate=" + this.adjustedPaymentDate + ", " +
				"calculationPeriod=" + this.calculationPeriod + ", " +
				"discountFactor=" + this.discountFactor + ", " +
				"fixedPaymentAmount=" + this.fixedPaymentAmount + ", " +
				"forecastPaymentAmount=" + this.forecastPaymentAmount + ", " +
				"meta=" + this.meta + ", " +
				"presentValueAmount=" + this.presentValueAmount + ", " +
				"unadjustedPaymentDate=" + this.unadjustedPaymentDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedPaymentDate"), Date.class, adjustedPaymentDate, this);
		processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
		processor.processBasic(path.newSubPath("fixedPaymentAmount"), BigDecimal.class, fixedPaymentAmount, this);
		processor.processBasic(path.newSubPath("unadjustedPaymentDate"), Date.class, unadjustedPaymentDate, this);
		
		processRosetta(path.newSubPath("calculationPeriod"), processor, CalculationPeriod.class, calculationPeriod);
		processRosetta(path.newSubPath("forecastPaymentAmount"), processor, Money.class, forecastPaymentAmount);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("presentValueAmount"), processor, Money.class, presentValueAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PaymentCalculationPeriod _that = (PaymentCalculationPeriod) o;
	
		if (adjustedPaymentDate != null ? !adjustedPaymentDate.equals(_that.adjustedPaymentDate) : _that.adjustedPaymentDate != null) return false;
		if (!ListEquals.listEquals(calculationPeriod, _that.calculationPeriod)) return false;
		if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
		if (fixedPaymentAmount != null ? !fixedPaymentAmount.equals(_that.fixedPaymentAmount) : _that.fixedPaymentAmount != null) return false;
		if (forecastPaymentAmount != null ? !forecastPaymentAmount.equals(_that.forecastPaymentAmount) : _that.forecastPaymentAmount != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (presentValueAmount != null ? !presentValueAmount.equals(_that.presentValueAmount) : _that.presentValueAmount != null) return false;
		if (unadjustedPaymentDate != null ? !unadjustedPaymentDate.equals(_that.unadjustedPaymentDate) : _that.unadjustedPaymentDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedPaymentDate != null ? adjustedPaymentDate.hashCode() : 0);
		_result = 31 * _result + (calculationPeriod != null ? calculationPeriod.hashCode() : 0);
		_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
		_result = 31 * _result + (fixedPaymentAmount != null ? fixedPaymentAmount.hashCode() : 0);
		_result = 31 * _result + (forecastPaymentAmount != null ? forecastPaymentAmount.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (presentValueAmount != null ? presentValueAmount.hashCode() : 0);
		_result = 31 * _result + (unadjustedPaymentDate != null ? unadjustedPaymentDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PaymentCalculationPeriod {" +
			"adjustedPaymentDate=" + this.adjustedPaymentDate + ", " +
			"calculationPeriod=" + this.calculationPeriod + ", " +
			"discountFactor=" + this.discountFactor + ", " +
			"fixedPaymentAmount=" + this.fixedPaymentAmount + ", " +
			"forecastPaymentAmount=" + this.forecastPaymentAmount + ", " +
			"meta=" + this.meta + ", " +
			"presentValueAmount=" + this.presentValueAmount + ", " +
			"unadjustedPaymentDate=" + this.unadjustedPaymentDate +
		'}';
	}
}
