package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Money;
import org.isda.cdm.PaymentRule;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PaymentDetailMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PaymentDetail", source="FpML_5_10")
@RosettaSynonym(value="PaymentDetail", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="PaymentDetail", source="DTCC_11_0")
@RosettaSynonym(value="PaymentDetail", source="DTCC_9_0")
@RosettaSynonym(value="PaymentDetail", source="CME_ClearedConfirm_1_17")
public class PaymentDetail extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final Money paymentAmount;
	private final AdjustableOrRelativeDate paymentDate;
	private final PaymentRule paymentRule;
	private static PaymentDetailMeta metaData = new PaymentDetailMeta();
	
	PaymentDetail(PaymentDetailBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.paymentAmount = ofNullable(builder.getPaymentAmount()).map(f->f.build()).orElse(null);
		this.paymentDate = ofNullable(builder.getPaymentDate()).map(f->f.build()).orElse(null);
		this.paymentRule = ofNullable(builder.getPaymentRule()).map(f->f.build()).orElse(null);
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * A fixed payment amount.
	 */
	@RosettaSynonym(value="paymentAmount", source="FpML_5_10")
	@RosettaSynonym(value="paymentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="paymentAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPaymentAmount() {
		return paymentAmount;
	}
	
	@RosettaSynonym(value="paymentDate", source="FpML_5_10")
	@RosettaSynonym(value="paymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getPaymentDate() {
		return paymentDate;
	}
	
	/**
	 * The calculation rule.
	 */
	@RosettaSynonym(value="paymentRule", source="FpML_5_10")
	@RosettaSynonym(value="paymentRule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentRule", source="DTCC_11_0")
	@RosettaSynonym(value="paymentRule", source="DTCC_9_0")
	@RosettaSynonym(value="paymentRule", source="CME_ClearedConfirm_1_17")
	public final PaymentRule getPaymentRule() {
		return paymentRule;
	}
	
	
	@Override
	public RosettaMetaData<? extends PaymentDetail> metaData() {
		return metaData;
	} 
	
	public PaymentDetailBuilder toBuilder() {
		PaymentDetailBuilder builder = new PaymentDetailBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPaymentAmount()).ifPresent(builder::setPaymentAmount);
		ofNullable(getPaymentDate()).ifPresent(builder::setPaymentDate);
		ofNullable(getPaymentRule()).ifPresent(builder::setPaymentRule);
		return builder;
	}

	public static PaymentDetailBuilder builder() {
		return new PaymentDetailBuilder();
	}

	public static  class PaymentDetailBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PaymentDetailBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder paymentAmount;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate;
		protected PaymentRule.PaymentRuleBuilder paymentRule;
	
		public PaymentDetailBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PaymentDetail> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getPaymentAmount() {
			return paymentAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePaymentAmount() {
			if (paymentAmount!=null) {
				return paymentAmount;
			}
			else return paymentAmount = new Money.MoneyBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getPaymentDate() {
			return paymentDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreatePaymentDate() {
			if (paymentDate!=null) {
				return paymentDate;
			}
			else return paymentDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
		
		public PaymentRule.PaymentRuleBuilder getPaymentRule() {
			return paymentRule;
		}
		
		public PaymentRule.PaymentRuleBuilder getOrCreatePaymentRule() {
			if (paymentRule!=null) {
				return paymentRule;
			}
			else return paymentRule = new PaymentRule.PaymentRuleBuilder();
		}
		
	
		public PaymentDetailBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PaymentDetailBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PaymentDetailBuilder setPaymentAmount(Money paymentAmount) {
			this.paymentAmount = paymentAmount.toBuilder();
			return this;
		}
		public PaymentDetailBuilder setPaymentAmountBuilder(Money.MoneyBuilder paymentAmount) {
			this.paymentAmount = paymentAmount;
			return this;
		}
		public PaymentDetailBuilder setPaymentDate(AdjustableOrRelativeDate paymentDate) {
			this.paymentDate = paymentDate.toBuilder();
			return this;
		}
		public PaymentDetailBuilder setPaymentDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate) {
			this.paymentDate = paymentDate;
			return this;
		}
		public PaymentDetailBuilder setPaymentRule(PaymentRule paymentRule) {
			this.paymentRule = paymentRule.toBuilder();
			return this;
		}
		public PaymentDetailBuilder setPaymentRuleBuilder(PaymentRule.PaymentRuleBuilder paymentRule) {
			this.paymentRule = paymentRule;
			return this;
		}
		
		public PaymentDetail build() {
			return new PaymentDetail(this);
		}
	
		@Override
		public PaymentDetailBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (paymentAmount!=null && !paymentAmount.prune().hasData()) paymentAmount = null;
			if (paymentDate!=null && !paymentDate.prune().hasData()) paymentDate = null;
			if (paymentRule!=null && !paymentRule.prune().hasData()) paymentRule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPaymentAmount()!=null && getPaymentAmount().hasData()) return true;
			if (getPaymentDate()!=null && getPaymentDate().hasData()) return true;
			if (getPaymentRule()!=null && getPaymentRule().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("paymentAmount"), processor, Money.class, paymentAmount);
			processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
			processRosetta(path.newSubPath("paymentRule"), processor, PaymentRule.class, paymentRule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PaymentDetailBuilder _that = (PaymentDetailBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (paymentAmount != null ? !paymentAmount.equals(_that.paymentAmount) : _that.paymentAmount != null) return false;
			if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
			if (paymentRule != null ? !paymentRule.equals(_that.paymentRule) : _that.paymentRule != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (paymentAmount != null ? paymentAmount.hashCode() : 0);
			_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
			_result = 31 * _result + (paymentRule != null ? paymentRule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaymentDetailBuilder {" +
				"meta=" + this.meta + ", " +
				"paymentAmount=" + this.paymentAmount + ", " +
				"paymentDate=" + this.paymentDate + ", " +
				"paymentRule=" + this.paymentRule +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("paymentAmount"), processor, Money.class, paymentAmount);
		processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
		processRosetta(path.newSubPath("paymentRule"), processor, PaymentRule.class, paymentRule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PaymentDetail _that = (PaymentDetail) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (paymentAmount != null ? !paymentAmount.equals(_that.paymentAmount) : _that.paymentAmount != null) return false;
		if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
		if (paymentRule != null ? !paymentRule.equals(_that.paymentRule) : _that.paymentRule != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (paymentAmount != null ? paymentAmount.hashCode() : 0);
		_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
		_result = 31 * _result + (paymentRule != null ? paymentRule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PaymentDetail {" +
			"meta=" + this.meta + ", " +
			"paymentAmount=" + this.paymentAmount + ", " +
			"paymentDate=" + this.paymentDate + ", " +
			"paymentRule=" + this.paymentRule +
		'}';
	}
}
