package org.isda.cdm;

import org.isda.cdm.PercentageRule;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PaymentRuleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the payment calculation rule. As of FpML 5.10, percentage rule is the only calculation rule that has been specified as part of the standard.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PaymentRule", source="FpML_5_10")
public class PaymentRule extends RosettaModelObject {
	private final PercentageRule percentageRule;
	private static PaymentRuleMeta metaData = new PaymentRuleMeta();
	
	PaymentRule(PaymentRuleBuilder builder) {
		this.percentageRule = ofNullable(builder.getPercentageRule()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * This attribute is not present as part of the FpML construct, as the payment rule is specialised by means of runtime type extension through the xsi:type.
	 */
	public final PercentageRule getPercentageRule() {
		return percentageRule;
	}
	
	
	@Override
	public RosettaMetaData<? extends PaymentRule> metaData() {
		return metaData;
	} 
	
	public PaymentRuleBuilder toBuilder() {
		PaymentRuleBuilder builder = new PaymentRuleBuilder();
		ofNullable(getPercentageRule()).ifPresent(builder::setPercentageRule);
		return builder;
	}

	public static PaymentRuleBuilder builder() {
		return new PaymentRuleBuilder();
	}

	public static  class PaymentRuleBuilder extends RosettaModelObjectBuilder {
	
		protected PercentageRule.PercentageRuleBuilder percentageRule;
	
		public PaymentRuleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PaymentRule> metaData() {
			return metaData;
		} 
	
		
		public PercentageRule.PercentageRuleBuilder getPercentageRule() {
			return percentageRule;
		}
		
		public PercentageRule.PercentageRuleBuilder getOrCreatePercentageRule() {
			if (percentageRule!=null) {
				return percentageRule;
			}
			else return percentageRule = new PercentageRule.PercentageRuleBuilder();
		}
		
	
		public PaymentRuleBuilder setPercentageRule(PercentageRule percentageRule) {
			this.percentageRule = percentageRule.toBuilder();
			return this;
		}
		public PaymentRuleBuilder setPercentageRuleBuilder(PercentageRule.PercentageRuleBuilder percentageRule) {
			this.percentageRule = percentageRule;
			return this;
		}
		
		public PaymentRule build() {
			return new PaymentRule(this);
		}
	
		@Override
		public PaymentRuleBuilder prune() {
			if (percentageRule!=null && !percentageRule.prune().hasData()) percentageRule = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPercentageRule()!=null && getPercentageRule().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("percentageRule"), processor, PercentageRule.class, percentageRule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PaymentRuleBuilder _that = (PaymentRuleBuilder) o;
		
			if (percentageRule != null ? !percentageRule.equals(_that.percentageRule) : _that.percentageRule != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (percentageRule != null ? percentageRule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaymentRuleBuilder {" +
				"percentageRule=" + this.percentageRule +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("percentageRule"), processor, PercentageRule.class, percentageRule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PaymentRule _that = (PaymentRule) o;
	
		if (percentageRule != null ? !percentageRule.equals(_that.percentageRule) : _that.percentageRule != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (percentageRule != null ? percentageRule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PaymentRule {" +
			"percentageRule=" + this.percentageRule +
		'}';
	}
}
