package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify the type of payment.
 * 
 * @version 2.5.4
 */
public enum PaymentTypeEnum {
    

	NET_CASHFLOW,
	    

	/**
	 * A cash flow associated with an amendment lifecycle event.
	 */
	@RosettaSynonym(value = "AmendmentFee", source = "FpML_5_10")
	@RosettaSynonym(value = "AmendmentFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AmendmentFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "AmendmentFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "AmendmentFee", source = "CME_ClearedConfirm_1_17")
	AMENDMENT_FEE,
	    

	/**
	 * A cash flow resulting from the assignment of a contract to a new counterparty.
	 */
	@RosettaSynonym(value = "AssignmentFee", source = "FpML_5_10")
	@RosettaSynonym(value = "AssignmentFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AssignmentFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "AssignmentFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "AssignmentFee", source = "CME_ClearedConfirm_1_17")
	ASSIGNMENT_FEE,
	    

	/**
	 * The brokerage commission.
	 */
	@RosettaSynonym(value = "brokerageCommission", source = "Rosetta_Workbench")
	BROKERAGE_COMMISSION,
	    

	/**
	 * A cash flow corresponding to the periodic accrued interests.
	 */
	@RosettaSynonym(value = "Coupon", source = "FpML_5_10")
	@RosettaSynonym(value = "Coupon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Coupon", source = "DTCC_11_0")
	@RosettaSynonym(value = "Coupon", source = "DTCC_9_0")
	@RosettaSynonym(value = "Coupon", source = "CME_ClearedConfirm_1_17")
	COUPON,
	    

	/**
	 * A cashflow resulting from a credit event.
	 */
	@RosettaSynonym(value = "CreditEvent", source = "FpML_5_10")
	@RosettaSynonym(value = "CreditEvent", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CreditEvent", source = "DTCC_11_0")
	@RosettaSynonym(value = "CreditEvent", source = "DTCC_9_0")
	@RosettaSynonym(value = "CreditEvent", source = "CME_ClearedConfirm_1_17")
	CREDIT_EVENT,
	    

	/**
	 * A cash flow corresponding to the synthetic dividend of an equity underlier asset traded through a derivative instrument.
	 */
	@RosettaSynonym(value = "DividendReturn", source = "FpML_5_10")
	@RosettaSynonym(value = "DividendReturn", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DividendReturn", source = "DTCC_11_0")
	@RosettaSynonym(value = "DividendReturn", source = "DTCC_9_0")
	@RosettaSynonym(value = "DividendReturn", source = "CME_ClearedConfirm_1_17")
	DIVIDEND_RETURN,
	    

	/**
	 * A cash flow associated with an exercise lifecycle event.
	 */
	@RosettaSynonym(value = "ExerciseFee", source = "FpML_5_10")
	@RosettaSynonym(value = "ExerciseFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExerciseFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExerciseFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExerciseFee", source = "CME_ClearedConfirm_1_17")
	EXERCISE_FEE,
	    

	/**
	 * A generic term for describing a non-scheduled cashflow that can be associated either with the initial contract, with some later corrections to it (e.g. a correction to the day count fraction that has a cashflow impact) or with some lifecycle events. Fees that are specifically associated with termination and partial termination, increase, amendment, and exercise events are qualified accordingly.
	 */
	@RosettaSynonym(value = "Fee", source = "FpML_5_10")
	@RosettaSynonym(value = "Fee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Fee", source = "DTCC_11_0")
	@RosettaSynonym(value = "Fee", source = "DTCC_9_0")
	@RosettaSynonym(value = "Fee", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "fee", source = "Rosetta_Workbench")
	FEE,
	    

	/**
	 * A cash flow associated with an increase lifecycle event.
	 */
	@RosettaSynonym(value = "IncreaseFee", source = "FpML_5_10")
	@RosettaSynonym(value = "IncreaseFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IncreaseFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "IncreaseFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "IncreaseFee", source = "CME_ClearedConfirm_1_17")
	INCREASE_FEE,
	    

	/**
	 * Interest, without qualification as to whether it a gross or net interest relates cashflow.
	 */
	@RosettaSynonym(value = "interest", source = "Rosetta_Workbench")
	INTEREST,
	    

	/**
	 * A cash flow corresponding to the return of the interest rate portion of a derivative instrument that has different types of underlying assets, such as a total return swap.
	 */
	@RosettaSynonym(value = "InterestReturn", source = "FpML_5_10")
	@RosettaSynonym(value = "InterestReturn", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "InterestReturn", source = "DTCC_11_0")
	@RosettaSynonym(value = "InterestReturn", source = "DTCC_9_0")
	@RosettaSynonym(value = "InterestReturn", source = "CME_ClearedConfirm_1_17")
	INTEREST_RETURN,
	    

	/**
	 * Net interest across payout components. Applicable to products such as interest rate swaps.
	 */
	@RosettaSynonym(value = "netInterest", source = "Rosetta_Workbench")
	NET_INTEREST,
	    

	/**
	 * The novation fee.
	 */
	@RosettaSynonym(value = "novationFee", source = "Rosetta_Workbench")
	NOVATION_FEE,
	    

	/**
	 * A cash flow associated with a partial termination lifecycle event.
	 */
	@RosettaSynonym(value = "PartialTerminationFee", source = "FpML_5_10")
	@RosettaSynonym(value = "PartialTerminationFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PartialTerminationFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "PartialTerminationFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "PartialTerminationFee", source = "CME_ClearedConfirm_1_17")
	PARTIAL_TERMINATION_FEE,
	    

	/**
	 * The premium associated with an OTC contract such as an option or a cap/floor.
	 */
	@RosettaSynonym(value = "PREMIUM", source = "FpML_5_10")
	@RosettaSynonym(value = "PREMIUM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PREMIUM", source = "DTCC_11_0")
	@RosettaSynonym(value = "PREMIUM", source = "DTCC_9_0")
	@RosettaSynonym(value = "PREMIUM", source = "CME_ClearedConfirm_1_17")
	@RosettaSynonym(value = "premium", source = "Rosetta_Workbench")
	PREMIUM,
	    

	/**
	 * A cash flow corresponding to the return of the price portion of a derivative instrument that has different types of underlying assets, such as a total return swap.
	 */
	@RosettaSynonym(value = "PriceReturn", source = "FpML_5_10")
	@RosettaSynonym(value = "PriceReturn", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PriceReturn", source = "DTCC_11_0")
	@RosettaSynonym(value = "PriceReturn", source = "DTCC_9_0")
	@RosettaSynonym(value = "PriceReturn", source = "CME_ClearedConfirm_1_17")
	PRICE_RETURN,
	    

	/**
	 * A cash flow which amount typically corresponds to the notional of the contract and that is exchanged between the parties on trade inception and reverted back when the contract is terminated.
	 */
	@RosettaSynonym(value = "PrincipleExchange", source = "FpML_5_10")
	@RosettaSynonym(value = "PrincipleExchange", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PrincipleExchange", source = "DTCC_11_0")
	@RosettaSynonym(value = "PrincipleExchange", source = "DTCC_9_0")
	@RosettaSynonym(value = "PrincipleExchange", source = "CME_ClearedConfirm_1_17")
	PRINCIPAL_EXCHANGE,
	    

	/**
	 * A cash flow associated with a termination lifecycle event.
	 */
	@RosettaSynonym(value = "TerminationFee", source = "FpML_5_10")
	@RosettaSynonym(value = "TerminationFee", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TerminationFee", source = "DTCC_11_0")
	@RosettaSynonym(value = "TerminationFee", source = "DTCC_9_0")
	@RosettaSynonym(value = "TerminationFee", source = "CME_ClearedConfirm_1_17")
	TERMINATION_FEE,
	    

	/**
	 * An upfront cashflow associated to the swap to adjust for a difference between the swap price and the current market price.
	 */
	@RosettaSynonym(value = "UPFRONT_FEE", source = "CME_ClearedConfirm_1_17")
	UPFRONT_FEE    
	;

	private final String displayName;
	
	PaymentTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
