package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.metafields.ReferenceWithMetaMoney;
import org.isda.cdm.metafields.ReferenceWithMetaMoney.ReferenceWithMetaMoneyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PercentageRuleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a content model for a calculation rule defined as percentage of the notional amount.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PercentageRule", source="FpML_5_10")
public class PercentageRule extends RosettaModelObject {
	private final ReferenceWithMetaMoney notionalAmountReference;
	private final BigDecimal paymentPercent;
	private static PercentageRuleMeta metaData = new PercentageRuleMeta();
	
	PercentageRule(PercentageRuleBuilder builder) {
		this.notionalAmountReference = ofNullable(builder.getNotionalAmountReference()).map(f->f.build()).orElse(null);
		this.paymentPercent = builder.getPaymentPercent();
	}
	
	/**
	 * A reference to the notional amount.
	 */
	@RosettaSynonym(value="notionalAmountReference", source="FpML_5_10")
	@RosettaSynonym(value="notionalAmountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="notionalAmountReference", source="DTCC_11_0")
	@RosettaSynonym(value="notionalAmountReference", source="DTCC_9_0")
	@RosettaSynonym(value="notionalAmountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaMoney getNotionalAmountReference() {
		return notionalAmountReference;
	}
	
	/**
	 * A percentage of the notional amount.
	 */
	@RosettaSynonym(value="paymentPercent", source="FpML_5_10")
	@RosettaSynonym(value="paymentPercent", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentPercent", source="DTCC_11_0")
	@RosettaSynonym(value="paymentPercent", source="DTCC_9_0")
	@RosettaSynonym(value="paymentPercent", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getPaymentPercent() {
		return paymentPercent;
	}
	
	
	@Override
	public RosettaMetaData<? extends PercentageRule> metaData() {
		return metaData;
	} 
	
	public PercentageRuleBuilder toBuilder() {
		PercentageRuleBuilder builder = new PercentageRuleBuilder();
		ofNullable(getNotionalAmountReference()).ifPresent(builder::setNotionalAmountReference);
		ofNullable(getPaymentPercent()).ifPresent(builder::setPaymentPercent);
		return builder;
	}

	public static PercentageRuleBuilder builder() {
		return new PercentageRuleBuilder();
	}

	public static  class PercentageRuleBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaMoneyBuilder notionalAmountReference;
		protected BigDecimal paymentPercent;
	
		public PercentageRuleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PercentageRule> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaMoneyBuilder getNotionalAmountReference() {
			return notionalAmountReference;
		}
		
		public ReferenceWithMetaMoneyBuilder getOrCreateNotionalAmountReference() {
			if (notionalAmountReference!=null) {
				return notionalAmountReference;
			}
			else return notionalAmountReference = new ReferenceWithMetaMoneyBuilder();
		}
		
		
		public BigDecimal getPaymentPercent() {
			return paymentPercent;
		}
		
	
		public PercentageRuleBuilder setNotionalAmountReference(ReferenceWithMetaMoney notionalAmountReference) {
			this.notionalAmountReference = notionalAmountReference.toBuilder();
			return this;
		}
		public PercentageRuleBuilder setNotionalAmountReferenceBuilder(ReferenceWithMetaMoneyBuilder notionalAmountReference) {
			this.notionalAmountReference = notionalAmountReference;
			return this;
		}
		public PercentageRuleBuilder setNotionalAmountReferenceRef(Money.MoneyBuilder notionalAmountReference) {
			return setNotionalAmountReference(ReferenceWithMetaMoney.builder().setValueBuilder(notionalAmountReference).build());
		}
		public PercentageRuleBuilder setNotionalAmountReferenceRef(Money notionalAmountReference) {
			return setNotionalAmountReferenceRef(Money.builder());
		}
		public PercentageRuleBuilder setPaymentPercent(BigDecimal paymentPercent) {
			this.paymentPercent = paymentPercent;
			return this;
		}
		
		public PercentageRule build() {
			return new PercentageRule(this);
		}
	
		@Override
		public PercentageRuleBuilder prune() {
			if (notionalAmountReference!=null && !notionalAmountReference.prune().hasData()) notionalAmountReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNotionalAmountReference()!=null && getNotionalAmountReference().hasData()) return true;
			if (getPaymentPercent()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("paymentPercent"), BigDecimal.class, paymentPercent, this);
			
			processRosetta(path.newSubPath("notionalAmountReference"), processor, ReferenceWithMetaMoney.class, notionalAmountReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PercentageRuleBuilder _that = (PercentageRuleBuilder) o;
		
			if (notionalAmountReference != null ? !notionalAmountReference.equals(_that.notionalAmountReference) : _that.notionalAmountReference != null) return false;
			if (paymentPercent != null ? !paymentPercent.equals(_that.paymentPercent) : _that.paymentPercent != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (notionalAmountReference != null ? notionalAmountReference.hashCode() : 0);
			_result = 31 * _result + (paymentPercent != null ? paymentPercent.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PercentageRuleBuilder {" +
				"notionalAmountReference=" + this.notionalAmountReference + ", " +
				"paymentPercent=" + this.paymentPercent +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("paymentPercent"), BigDecimal.class, paymentPercent, this);
		
		processRosetta(path.newSubPath("notionalAmountReference"), processor, ReferenceWithMetaMoney.class, notionalAmountReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PercentageRule _that = (PercentageRule) o;
	
		if (notionalAmountReference != null ? !notionalAmountReference.equals(_that.notionalAmountReference) : _that.notionalAmountReference != null) return false;
		if (paymentPercent != null ? !paymentPercent.equals(_that.paymentPercent) : _that.paymentPercent != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (notionalAmountReference != null ? notionalAmountReference.hashCode() : 0);
		_result = 31 * _result + (paymentPercent != null ? paymentPercent.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PercentageRule {" +
			"notionalAmountReference=" + this.notionalAmountReference + ", " +
			"paymentPercent=" + this.paymentPercent +
		'}';
	}
}
