package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify a time period containing the additional value of Term.
 * 
 * @version 2.5.4
 */
public enum PeriodExtendedEnum {
    

	/**
	 * Term. The period commencing on the effective date and ending on the termination date. The T period always appears in association with periodMultiplier = 1, and the notation is intended for use in contexts where the interval thus qualified (e.g. accrual period, payment period, reset period, ...) spans the entire term of the trade.
	 */
	@RosettaSynonym(value = "T", source = "FpML_5_10")
	@RosettaSynonym(value = "T", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "T", source = "DTCC_11_0")
	@RosettaSynonym(value = "T", source = "DTCC_9_0")
	@RosettaSynonym(value = "T", source = "CME_ClearedConfirm_1_17")
	T,
	    

	/**
	 * Day
	 */
	@RosettaSynonym(value = "D", source = "FpML_5_10")
	@RosettaSynonym(value = "D", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "D", source = "DTCC_11_0")
	@RosettaSynonym(value = "D", source = "DTCC_9_0")
	@RosettaSynonym(value = "D", source = "CME_ClearedConfirm_1_17")
	D,
	    

	/**
	 * Week
	 */
	@RosettaSynonym(value = "W", source = "FpML_5_10")
	@RosettaSynonym(value = "W", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "W", source = "DTCC_11_0")
	@RosettaSynonym(value = "W", source = "DTCC_9_0")
	@RosettaSynonym(value = "W", source = "CME_ClearedConfirm_1_17")
	W,
	    

	/**
	 * Month
	 */
	@RosettaSynonym(value = "M", source = "FpML_5_10")
	@RosettaSynonym(value = "M", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "M", source = "DTCC_11_0")
	@RosettaSynonym(value = "M", source = "DTCC_9_0")
	@RosettaSynonym(value = "M", source = "CME_ClearedConfirm_1_17")
	M,
	    

	/**
	 * Year
	 */
	@RosettaSynonym(value = "Y", source = "FpML_5_10")
	@RosettaSynonym(value = "Y", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Y", source = "DTCC_11_0")
	@RosettaSynonym(value = "Y", source = "DTCC_9_0")
	@RosettaSynonym(value = "Y", source = "CME_ClearedConfirm_1_17")
	Y    
	;

	private final String displayName;
	
	PeriodExtendedEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
