package org.isda.cdm;


/**
 * Enumeration to describe the different (risk) states of a Position, whether executed, settled, matured...etc
 * 
 * @version 2.5.4
 */
public enum PositionStatusEnum {
    

	/**
	 * The position has been executed, which is the point at which risk has been transferred.
	 */
	EXECUTED,
	    

	/**
	 * Contract has been formed, in case position is on a contractual product.
	 */
	FORMED,
	    

	/**
	 * The position has settled, in case product is subject to settlement after execution, such as securities.
	 */
	SETTLED,
	    

	/**
	 * The position has been cancelled, in case of a cancellation event following an execution.
	 */
	CANCELLED    
	;

	private final String displayName;
	
	PositionStatusEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
