package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.Pric;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PricMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Pric extends RosettaModelObject {
	private final String bsisPts;
	private final Pric pric;
	private static PricMeta metaData = new PricMeta();
	
	Pric(PricBuilder builder) {
		this.bsisPts = builder.getBsisPts();
		this.pric = ofNullable(builder.getPric()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="BsisPts", source="ISO20022")
	public final String getBsisPts() {
		return bsisPts;
	}
	
	@RosettaSynonym(value="Pric", source="ISO20022")
	public final Pric getPric() {
		return pric;
	}
	
	
	@Override
	public RosettaMetaData<? extends Pric> metaData() {
		return metaData;
	} 
	
	public PricBuilder toBuilder() {
		PricBuilder builder = new PricBuilder();
		ofNullable(getBsisPts()).ifPresent(builder::setBsisPts);
		ofNullable(getPric()).ifPresent(builder::setPric);
		return builder;
	}

	public static PricBuilder builder() {
		return new PricBuilder();
	}

	public static  class PricBuilder extends RosettaModelObjectBuilder {
	
		protected String bsisPts;
		protected Pric.PricBuilder pric;
	
		public PricBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Pric> metaData() {
			return metaData;
		} 
	
		
		public String getBsisPts() {
			return bsisPts;
		}
		
		
		public Pric.PricBuilder getPric() {
			return pric;
		}
		
		public Pric.PricBuilder getOrCreatePric() {
			if (pric!=null) {
				return pric;
			}
			else return pric = new Pric.PricBuilder();
		}
		
	
		public PricBuilder setBsisPts(String bsisPts) {
			this.bsisPts = bsisPts;
			return this;
		}
		public PricBuilder setPric(Pric pric) {
			this.pric = pric.toBuilder();
			return this;
		}
		public PricBuilder setPricBuilder(Pric.PricBuilder pric) {
			this.pric = pric;
			return this;
		}
		
		public Pric build() {
			return new Pric(this);
		}
	
		@Override
		public PricBuilder prune() {
			if (pric!=null && !pric.prune().hasData()) pric = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBsisPts()!=null) return true;
			if (getPric()!=null && getPric().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("bsisPts"), String.class, bsisPts, this);
			
			processRosetta(path.newSubPath("pric"), processor, Pric.class, pric);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PricBuilder _that = (PricBuilder) o;
		
			if (bsisPts != null ? !bsisPts.equals(_that.bsisPts) : _that.bsisPts != null) return false;
			if (pric != null ? !pric.equals(_that.pric) : _that.pric != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bsisPts != null ? bsisPts.hashCode() : 0);
			_result = 31 * _result + (pric != null ? pric.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PricBuilder {" +
				"bsisPts=" + this.bsisPts + ", " +
				"pric=" + this.pric +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("bsisPts"), String.class, bsisPts, this);
		
		processRosetta(path.newSubPath("pric"), processor, Pric.class, pric);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Pric _that = (Pric) o;
	
		if (bsisPts != null ? !bsisPts.equals(_that.bsisPts) : _that.bsisPts != null) return false;
		if (pric != null ? !pric.equals(_that.pric) : _that.pric != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bsisPts != null ? bsisPts.hashCode() : 0);
		_result = 31 * _result + (pric != null ? pric.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Pric {" +
			"bsisPts=" + this.bsisPts + ", " +
			"pric=" + this.pric +
		'}';
	}
}
