package org.isda.cdm;

import org.isda.cdm.FallbackReferencePrice;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PriceSourceDisruptionMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining the parameters used to get a price quote to replace the settlement rate option that is disrupted.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PriceSourceDisruption", source="FpML_5_10")
public class PriceSourceDisruption extends RosettaModelObject {
	private final FallbackReferencePrice fallbackReferencePrice;
	private static PriceSourceDisruptionMeta metaData = new PriceSourceDisruptionMeta();
	
	PriceSourceDisruption(PriceSourceDisruptionBuilder builder) {
		this.fallbackReferencePrice = ofNullable(builder.getFallbackReferencePrice()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The method, prioritised by the order it is listed in this element, to get a replacement rate for the disrupted settlement rate option.
	 */
	@RosettaSynonym(value="fallbackReferencePrice", source="FpML_5_10")
	@RosettaSynonym(value="fallbackReferencePrice", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fallbackReferencePrice", source="DTCC_11_0")
	@RosettaSynonym(value="fallbackReferencePrice", source="DTCC_9_0")
	@RosettaSynonym(value="fallbackReferencePrice", source="CME_ClearedConfirm_1_17")
	public final FallbackReferencePrice getFallbackReferencePrice() {
		return fallbackReferencePrice;
	}
	
	
	@Override
	public RosettaMetaData<? extends PriceSourceDisruption> metaData() {
		return metaData;
	} 
	
	public PriceSourceDisruptionBuilder toBuilder() {
		PriceSourceDisruptionBuilder builder = new PriceSourceDisruptionBuilder();
		ofNullable(getFallbackReferencePrice()).ifPresent(builder::setFallbackReferencePrice);
		return builder;
	}

	public static PriceSourceDisruptionBuilder builder() {
		return new PriceSourceDisruptionBuilder();
	}

	public static  class PriceSourceDisruptionBuilder extends RosettaModelObjectBuilder {
	
		protected FallbackReferencePrice.FallbackReferencePriceBuilder fallbackReferencePrice;
	
		public PriceSourceDisruptionBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PriceSourceDisruption> metaData() {
			return metaData;
		} 
	
		
		public FallbackReferencePrice.FallbackReferencePriceBuilder getFallbackReferencePrice() {
			return fallbackReferencePrice;
		}
		
		public FallbackReferencePrice.FallbackReferencePriceBuilder getOrCreateFallbackReferencePrice() {
			if (fallbackReferencePrice!=null) {
				return fallbackReferencePrice;
			}
			else return fallbackReferencePrice = new FallbackReferencePrice.FallbackReferencePriceBuilder();
		}
		
	
		public PriceSourceDisruptionBuilder setFallbackReferencePrice(FallbackReferencePrice fallbackReferencePrice) {
			this.fallbackReferencePrice = fallbackReferencePrice.toBuilder();
			return this;
		}
		public PriceSourceDisruptionBuilder setFallbackReferencePriceBuilder(FallbackReferencePrice.FallbackReferencePriceBuilder fallbackReferencePrice) {
			this.fallbackReferencePrice = fallbackReferencePrice;
			return this;
		}
		
		public PriceSourceDisruption build() {
			return new PriceSourceDisruption(this);
		}
	
		@Override
		public PriceSourceDisruptionBuilder prune() {
			if (fallbackReferencePrice!=null && !fallbackReferencePrice.prune().hasData()) fallbackReferencePrice = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFallbackReferencePrice()!=null && getFallbackReferencePrice().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fallbackReferencePrice"), processor, FallbackReferencePrice.class, fallbackReferencePrice);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PriceSourceDisruptionBuilder _that = (PriceSourceDisruptionBuilder) o;
		
			if (fallbackReferencePrice != null ? !fallbackReferencePrice.equals(_that.fallbackReferencePrice) : _that.fallbackReferencePrice != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fallbackReferencePrice != null ? fallbackReferencePrice.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PriceSourceDisruptionBuilder {" +
				"fallbackReferencePrice=" + this.fallbackReferencePrice +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fallbackReferencePrice"), processor, FallbackReferencePrice.class, fallbackReferencePrice);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PriceSourceDisruption _that = (PriceSourceDisruption) o;
	
		if (fallbackReferencePrice != null ? !fallbackReferencePrice.equals(_that.fallbackReferencePrice) : _that.fallbackReferencePrice != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fallbackReferencePrice != null ? fallbackReferencePrice.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PriceSourceDisruption {" +
			"fallbackReferencePrice=" + this.fallbackReferencePrice +
		'}';
	}
}
