package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AllocationPrimitive;
import org.isda.cdm.ContractFormation;
import org.isda.cdm.ExecutionPrimitive;
import org.isda.cdm.ExercisePrimitive;
import org.isda.cdm.Inception;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.ResetPrimitive;
import org.isda.cdm.TermsChangePrimitive;
import org.isda.cdm.TransferPrimitive;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PrimitiveEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the set of elemental/primitives components that are used to specify lifecycle events. Lifecycle events make use of one or several of those elemental components specified as part of this class.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class PrimitiveEvent extends RosettaModelObject {
	private final List<AllocationPrimitive> allocation;
	private final List<ContractFormation> contractFormation;
	private final List<ExecutionPrimitive> execution;
	private final ExercisePrimitive exercise;
	private final List<Inception> inception;
	private final List<ObservationPrimitive> observation;
	private final List<QuantityChangePrimitive> quantityChange;
	private final List<ResetPrimitive> reset;
	private final TermsChangePrimitive termsChange;
	private final List<TransferPrimitive> transfer;
	private static PrimitiveEventMeta metaData = new PrimitiveEventMeta();
	
	PrimitiveEvent(PrimitiveEventBuilder builder) {
		this.allocation = ofNullable(builder.getAllocation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.contractFormation = ofNullable(builder.getContractFormation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.execution = ofNullable(builder.getExecution()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.exercise = ofNullable(builder.getExercise()).map(f->f.build()).orElse(null);
		this.inception = ofNullable(builder.getInception()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.observation = ofNullable(builder.getObservation()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.quantityChange = ofNullable(builder.getQuantityChange()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.reset = ofNullable(builder.getReset()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.termsChange = ofNullable(builder.getTermsChange()).map(f->f.build()).orElse(null);
		this.transfer = ofNullable(builder.getTransfer()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	@RosettaSynonym(value="allocation", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<AllocationPrimitive> getAllocation() {
		return allocation;
	}
	
	@RosettaSynonym(value="ignore", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="ignore", source="FpML_5_10")
	public final List<ContractFormation> getContractFormation() {
		return contractFormation;
	}
	
	@RosettaSynonym(value="ignore", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="ignore", source="FpML_5_10")
	public final List<ExecutionPrimitive> getExecution() {
		return execution;
	}
	
	@RosettaSynonym(value="exercise", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final ExercisePrimitive getExercise() {
		return exercise;
	}
	
	@RosettaSynonym(value="inception", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<Inception> getInception() {
		return inception;
	}
	
	@RosettaSynonym(value="observation", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<ObservationPrimitive> getObservation() {
		return observation;
	}
	
	@RosettaSynonym(value="quantityChange", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<QuantityChangePrimitive> getQuantityChange() {
		return quantityChange;
	}
	
	@RosettaSynonym(value="reset", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<ResetPrimitive> getReset() {
		return reset;
	}
	
	@RosettaSynonym(value="termsChange", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="DTCC_11_0")
	@RosettaSynonym(value="ignore", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	public final TermsChangePrimitive getTermsChange() {
		return termsChange;
	}
	
	@RosettaSynonym(value="transfer", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="Payment", source="DTCC_11_0", path="Body.OTC_Matching")
	@RosettaSynonym(value="Payment", source="DTCC_9_0", path="Body.OTC_Matching")
	public final List<TransferPrimitive> getTransfer() {
		return transfer;
	}
	
	
	@Override
	public RosettaMetaData<? extends PrimitiveEvent> metaData() {
		return metaData;
	} 
	
	public PrimitiveEventBuilder toBuilder() {
		PrimitiveEventBuilder builder = new PrimitiveEventBuilder();
		ofNullable(getAllocation()).ifPresent(allocation -> allocation.forEach(builder::addAllocation));
		ofNullable(getContractFormation()).ifPresent(contractFormation -> contractFormation.forEach(builder::addContractFormation));
		ofNullable(getExecution()).ifPresent(execution -> execution.forEach(builder::addExecution));
		ofNullable(getExercise()).ifPresent(builder::setExercise);
		ofNullable(getInception()).ifPresent(inception -> inception.forEach(builder::addInception));
		ofNullable(getObservation()).ifPresent(observation -> observation.forEach(builder::addObservation));
		ofNullable(getQuantityChange()).ifPresent(quantityChange -> quantityChange.forEach(builder::addQuantityChange));
		ofNullable(getReset()).ifPresent(reset -> reset.forEach(builder::addReset));
		ofNullable(getTermsChange()).ifPresent(builder::setTermsChange);
		ofNullable(getTransfer()).ifPresent(transfer -> transfer.forEach(builder::addTransfer));
		return builder;
	}

	public static PrimitiveEventBuilder builder() {
		return new PrimitiveEventBuilder();
	}

	public static  class PrimitiveEventBuilder extends RosettaModelObjectBuilder {
	
		protected List<AllocationPrimitive.AllocationPrimitiveBuilder> allocation;
		protected List<ContractFormation.ContractFormationBuilder> contractFormation;
		protected List<ExecutionPrimitive.ExecutionPrimitiveBuilder> execution;
		protected ExercisePrimitive.ExercisePrimitiveBuilder exercise;
		protected List<Inception.InceptionBuilder> inception;
		protected List<ObservationPrimitive.ObservationPrimitiveBuilder> observation;
		protected List<QuantityChangePrimitive.QuantityChangePrimitiveBuilder> quantityChange;
		protected List<ResetPrimitive.ResetPrimitiveBuilder> reset;
		protected TermsChangePrimitive.TermsChangePrimitiveBuilder termsChange;
		protected List<TransferPrimitive.TransferPrimitiveBuilder> transfer;
	
		public PrimitiveEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PrimitiveEvent> metaData() {
			return metaData;
		} 
	
		
		public List<AllocationPrimitive.AllocationPrimitiveBuilder> getAllocation() {
			return allocation;
		}
		
		public AllocationPrimitive.AllocationPrimitiveBuilder getOrCreateAllocation(int index) {
			if (allocation==null) {
				this.allocation = new ArrayList<>();
			}
			return getIndex(allocation, index, ()->new AllocationPrimitive.AllocationPrimitiveBuilder());
		}
		
		
		public List<ContractFormation.ContractFormationBuilder> getContractFormation() {
			return contractFormation;
		}
		
		public ContractFormation.ContractFormationBuilder getOrCreateContractFormation(int index) {
			if (contractFormation==null) {
				this.contractFormation = new ArrayList<>();
			}
			return getIndex(contractFormation, index, ()->new ContractFormation.ContractFormationBuilder());
		}
		
		
		public List<ExecutionPrimitive.ExecutionPrimitiveBuilder> getExecution() {
			return execution;
		}
		
		public ExecutionPrimitive.ExecutionPrimitiveBuilder getOrCreateExecution(int index) {
			if (execution==null) {
				this.execution = new ArrayList<>();
			}
			return getIndex(execution, index, ()->new ExecutionPrimitive.ExecutionPrimitiveBuilder());
		}
		
		
		public ExercisePrimitive.ExercisePrimitiveBuilder getExercise() {
			return exercise;
		}
		
		public ExercisePrimitive.ExercisePrimitiveBuilder getOrCreateExercise() {
			if (exercise!=null) {
				return exercise;
			}
			else return exercise = new ExercisePrimitive.ExercisePrimitiveBuilder();
		}
		
		
		public List<Inception.InceptionBuilder> getInception() {
			return inception;
		}
		
		public Inception.InceptionBuilder getOrCreateInception(int index) {
			if (inception==null) {
				this.inception = new ArrayList<>();
			}
			return getIndex(inception, index, ()->new Inception.InceptionBuilder());
		}
		
		
		public List<ObservationPrimitive.ObservationPrimitiveBuilder> getObservation() {
			return observation;
		}
		
		public ObservationPrimitive.ObservationPrimitiveBuilder getOrCreateObservation(int index) {
			if (observation==null) {
				this.observation = new ArrayList<>();
			}
			return getIndex(observation, index, ()->new ObservationPrimitive.ObservationPrimitiveBuilder());
		}
		
		
		public List<QuantityChangePrimitive.QuantityChangePrimitiveBuilder> getQuantityChange() {
			return quantityChange;
		}
		
		public QuantityChangePrimitive.QuantityChangePrimitiveBuilder getOrCreateQuantityChange(int index) {
			if (quantityChange==null) {
				this.quantityChange = new ArrayList<>();
			}
			return getIndex(quantityChange, index, ()->new QuantityChangePrimitive.QuantityChangePrimitiveBuilder());
		}
		
		
		public List<ResetPrimitive.ResetPrimitiveBuilder> getReset() {
			return reset;
		}
		
		public ResetPrimitive.ResetPrimitiveBuilder getOrCreateReset(int index) {
			if (reset==null) {
				this.reset = new ArrayList<>();
			}
			return getIndex(reset, index, ()->new ResetPrimitive.ResetPrimitiveBuilder());
		}
		
		
		public TermsChangePrimitive.TermsChangePrimitiveBuilder getTermsChange() {
			return termsChange;
		}
		
		public TermsChangePrimitive.TermsChangePrimitiveBuilder getOrCreateTermsChange() {
			if (termsChange!=null) {
				return termsChange;
			}
			else return termsChange = new TermsChangePrimitive.TermsChangePrimitiveBuilder();
		}
		
		
		public List<TransferPrimitive.TransferPrimitiveBuilder> getTransfer() {
			return transfer;
		}
		
		public TransferPrimitive.TransferPrimitiveBuilder getOrCreateTransfer(int index) {
			if (transfer==null) {
				this.transfer = new ArrayList<>();
			}
			return getIndex(transfer, index, ()->new TransferPrimitive.TransferPrimitiveBuilder());
		}
		
	
		public PrimitiveEventBuilder addAllocation(AllocationPrimitive allocation) {
			if(this.allocation == null){
				this.allocation = new ArrayList<>();
			}
			this.allocation.add(allocation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addAllocation(AllocationPrimitive allocation, int _idx) {
			if(this.allocation == null){
				this.allocation = new ArrayList<>();
			}
			getIndex(this.allocation, _idx, () -> allocation.toBuilder());
			this.allocation.set(_idx, allocation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addAllocation(List<AllocationPrimitive> allocations) {
			if(this.allocation == null){
				this.allocation = new ArrayList<>();
			}
			for (AllocationPrimitive toAdd : allocations) {
				this.allocation.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addAllocationBuilder(AllocationPrimitive.AllocationPrimitiveBuilder allocation) {
			if(this.allocation == null){
				this.allocation = new ArrayList<>();
			}
			this.allocation.add(allocation);
			return this;
		}
		
		public PrimitiveEventBuilder clearAllocation() {
			this.allocation = null;
			return this;
		}
		public PrimitiveEventBuilder addContractFormation(ContractFormation contractFormation) {
			if(this.contractFormation == null){
				this.contractFormation = new ArrayList<>();
			}
			this.contractFormation.add(contractFormation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addContractFormation(ContractFormation contractFormation, int _idx) {
			if(this.contractFormation == null){
				this.contractFormation = new ArrayList<>();
			}
			getIndex(this.contractFormation, _idx, () -> contractFormation.toBuilder());
			this.contractFormation.set(_idx, contractFormation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addContractFormation(List<ContractFormation> contractFormations) {
			if(this.contractFormation == null){
				this.contractFormation = new ArrayList<>();
			}
			for (ContractFormation toAdd : contractFormations) {
				this.contractFormation.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addContractFormationBuilder(ContractFormation.ContractFormationBuilder contractFormation) {
			if(this.contractFormation == null){
				this.contractFormation = new ArrayList<>();
			}
			this.contractFormation.add(contractFormation);
			return this;
		}
		
		public PrimitiveEventBuilder clearContractFormation() {
			this.contractFormation = null;
			return this;
		}
		public PrimitiveEventBuilder addExecution(ExecutionPrimitive execution) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			this.execution.add(execution.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addExecution(ExecutionPrimitive execution, int _idx) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			getIndex(this.execution, _idx, () -> execution.toBuilder());
			this.execution.set(_idx, execution.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addExecution(List<ExecutionPrimitive> executions) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			for (ExecutionPrimitive toAdd : executions) {
				this.execution.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addExecutionBuilder(ExecutionPrimitive.ExecutionPrimitiveBuilder execution) {
			if(this.execution == null){
				this.execution = new ArrayList<>();
			}
			this.execution.add(execution);
			return this;
		}
		
		public PrimitiveEventBuilder clearExecution() {
			this.execution = null;
			return this;
		}
		public PrimitiveEventBuilder setExercise(ExercisePrimitive exercise) {
			this.exercise = exercise.toBuilder();
			return this;
		}
		public PrimitiveEventBuilder setExerciseBuilder(ExercisePrimitive.ExercisePrimitiveBuilder exercise) {
			this.exercise = exercise;
			return this;
		}
		public PrimitiveEventBuilder addInception(Inception inception) {
			if(this.inception == null){
				this.inception = new ArrayList<>();
			}
			this.inception.add(inception.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addInception(Inception inception, int _idx) {
			if(this.inception == null){
				this.inception = new ArrayList<>();
			}
			getIndex(this.inception, _idx, () -> inception.toBuilder());
			this.inception.set(_idx, inception.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addInception(List<Inception> inceptions) {
			if(this.inception == null){
				this.inception = new ArrayList<>();
			}
			for (Inception toAdd : inceptions) {
				this.inception.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addInceptionBuilder(Inception.InceptionBuilder inception) {
			if(this.inception == null){
				this.inception = new ArrayList<>();
			}
			this.inception.add(inception);
			return this;
		}
		
		public PrimitiveEventBuilder clearInception() {
			this.inception = null;
			return this;
		}
		public PrimitiveEventBuilder addObservation(ObservationPrimitive observation) {
			if(this.observation == null){
				this.observation = new ArrayList<>();
			}
			this.observation.add(observation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addObservation(ObservationPrimitive observation, int _idx) {
			if(this.observation == null){
				this.observation = new ArrayList<>();
			}
			getIndex(this.observation, _idx, () -> observation.toBuilder());
			this.observation.set(_idx, observation.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addObservation(List<ObservationPrimitive> observations) {
			if(this.observation == null){
				this.observation = new ArrayList<>();
			}
			for (ObservationPrimitive toAdd : observations) {
				this.observation.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addObservationBuilder(ObservationPrimitive.ObservationPrimitiveBuilder observation) {
			if(this.observation == null){
				this.observation = new ArrayList<>();
			}
			this.observation.add(observation);
			return this;
		}
		
		public PrimitiveEventBuilder clearObservation() {
			this.observation = null;
			return this;
		}
		public PrimitiveEventBuilder addQuantityChange(QuantityChangePrimitive quantityChange) {
			if(this.quantityChange == null){
				this.quantityChange = new ArrayList<>();
			}
			this.quantityChange.add(quantityChange.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addQuantityChange(QuantityChangePrimitive quantityChange, int _idx) {
			if(this.quantityChange == null){
				this.quantityChange = new ArrayList<>();
			}
			getIndex(this.quantityChange, _idx, () -> quantityChange.toBuilder());
			this.quantityChange.set(_idx, quantityChange.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addQuantityChange(List<QuantityChangePrimitive> quantityChanges) {
			if(this.quantityChange == null){
				this.quantityChange = new ArrayList<>();
			}
			for (QuantityChangePrimitive toAdd : quantityChanges) {
				this.quantityChange.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addQuantityChangeBuilder(QuantityChangePrimitive.QuantityChangePrimitiveBuilder quantityChange) {
			if(this.quantityChange == null){
				this.quantityChange = new ArrayList<>();
			}
			this.quantityChange.add(quantityChange);
			return this;
		}
		
		public PrimitiveEventBuilder clearQuantityChange() {
			this.quantityChange = null;
			return this;
		}
		public PrimitiveEventBuilder addReset(ResetPrimitive reset) {
			if(this.reset == null){
				this.reset = new ArrayList<>();
			}
			this.reset.add(reset.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addReset(ResetPrimitive reset, int _idx) {
			if(this.reset == null){
				this.reset = new ArrayList<>();
			}
			getIndex(this.reset, _idx, () -> reset.toBuilder());
			this.reset.set(_idx, reset.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addReset(List<ResetPrimitive> resets) {
			if(this.reset == null){
				this.reset = new ArrayList<>();
			}
			for (ResetPrimitive toAdd : resets) {
				this.reset.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addResetBuilder(ResetPrimitive.ResetPrimitiveBuilder reset) {
			if(this.reset == null){
				this.reset = new ArrayList<>();
			}
			this.reset.add(reset);
			return this;
		}
		
		public PrimitiveEventBuilder clearReset() {
			this.reset = null;
			return this;
		}
		public PrimitiveEventBuilder setTermsChange(TermsChangePrimitive termsChange) {
			this.termsChange = termsChange.toBuilder();
			return this;
		}
		public PrimitiveEventBuilder setTermsChangeBuilder(TermsChangePrimitive.TermsChangePrimitiveBuilder termsChange) {
			this.termsChange = termsChange;
			return this;
		}
		public PrimitiveEventBuilder addTransfer(TransferPrimitive transfer) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			this.transfer.add(transfer.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addTransfer(TransferPrimitive transfer, int _idx) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			getIndex(this.transfer, _idx, () -> transfer.toBuilder());
			this.transfer.set(_idx, transfer.toBuilder());
			return this;
		}
		public PrimitiveEventBuilder addTransfer(List<TransferPrimitive> transfers) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			for (TransferPrimitive toAdd : transfers) {
				this.transfer.add(toAdd.toBuilder());
			}
			return this;
		}
		public PrimitiveEventBuilder addTransferBuilder(TransferPrimitive.TransferPrimitiveBuilder transfer) {
			if(this.transfer == null){
				this.transfer = new ArrayList<>();
			}
			this.transfer.add(transfer);
			return this;
		}
		
		public PrimitiveEventBuilder clearTransfer() {
			this.transfer = null;
			return this;
		}
		
		public PrimitiveEvent build() {
			return new PrimitiveEvent(this);
		}
	
		@Override
		public PrimitiveEventBuilder prune() {
			if (allocation!=null) allocation = allocation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (contractFormation!=null) contractFormation = contractFormation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (execution!=null) execution = execution.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (exercise!=null && !exercise.prune().hasData()) exercise = null;
			if (inception!=null) inception = inception.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (observation!=null) observation = observation.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (quantityChange!=null) quantityChange = quantityChange.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (reset!=null) reset = reset.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (termsChange!=null && !termsChange.prune().hasData()) termsChange = null;
			if (transfer!=null) transfer = transfer.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAllocation()!=null && getAllocation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getContractFormation()!=null && getContractFormation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getExecution()!=null && getExecution().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getExercise()!=null && getExercise().hasData()) return true;
			if (getInception()!=null && getInception().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getObservation()!=null && getObservation().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getQuantityChange()!=null && getQuantityChange().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getReset()!=null && getReset().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTermsChange()!=null && getTermsChange().hasData()) return true;
			if (getTransfer()!=null && getTransfer().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("allocation"), processor, AllocationPrimitive.class, allocation);
			processRosetta(path.newSubPath("contractFormation"), processor, ContractFormation.class, contractFormation);
			processRosetta(path.newSubPath("execution"), processor, ExecutionPrimitive.class, execution);
			processRosetta(path.newSubPath("exercise"), processor, ExercisePrimitive.class, exercise);
			processRosetta(path.newSubPath("inception"), processor, Inception.class, inception);
			processRosetta(path.newSubPath("observation"), processor, ObservationPrimitive.class, observation);
			processRosetta(path.newSubPath("quantityChange"), processor, QuantityChangePrimitive.class, quantityChange);
			processRosetta(path.newSubPath("reset"), processor, ResetPrimitive.class, reset);
			processRosetta(path.newSubPath("termsChange"), processor, TermsChangePrimitive.class, termsChange);
			processRosetta(path.newSubPath("transfer"), processor, TransferPrimitive.class, transfer);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PrimitiveEventBuilder _that = (PrimitiveEventBuilder) o;
		
			if (!ListEquals.listEquals(allocation, _that.allocation)) return false;
			if (!ListEquals.listEquals(contractFormation, _that.contractFormation)) return false;
			if (!ListEquals.listEquals(execution, _that.execution)) return false;
			if (exercise != null ? !exercise.equals(_that.exercise) : _that.exercise != null) return false;
			if (!ListEquals.listEquals(inception, _that.inception)) return false;
			if (!ListEquals.listEquals(observation, _that.observation)) return false;
			if (!ListEquals.listEquals(quantityChange, _that.quantityChange)) return false;
			if (!ListEquals.listEquals(reset, _that.reset)) return false;
			if (termsChange != null ? !termsChange.equals(_that.termsChange) : _that.termsChange != null) return false;
			if (!ListEquals.listEquals(transfer, _that.transfer)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (allocation != null ? allocation.hashCode() : 0);
			_result = 31 * _result + (contractFormation != null ? contractFormation.hashCode() : 0);
			_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
			_result = 31 * _result + (exercise != null ? exercise.hashCode() : 0);
			_result = 31 * _result + (inception != null ? inception.hashCode() : 0);
			_result = 31 * _result + (observation != null ? observation.hashCode() : 0);
			_result = 31 * _result + (quantityChange != null ? quantityChange.hashCode() : 0);
			_result = 31 * _result + (reset != null ? reset.hashCode() : 0);
			_result = 31 * _result + (termsChange != null ? termsChange.hashCode() : 0);
			_result = 31 * _result + (transfer != null ? transfer.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PrimitiveEventBuilder {" +
				"allocation=" + this.allocation + ", " +
				"contractFormation=" + this.contractFormation + ", " +
				"execution=" + this.execution + ", " +
				"exercise=" + this.exercise + ", " +
				"inception=" + this.inception + ", " +
				"observation=" + this.observation + ", " +
				"quantityChange=" + this.quantityChange + ", " +
				"reset=" + this.reset + ", " +
				"termsChange=" + this.termsChange + ", " +
				"transfer=" + this.transfer +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("allocation"), processor, AllocationPrimitive.class, allocation);
		processRosetta(path.newSubPath("contractFormation"), processor, ContractFormation.class, contractFormation);
		processRosetta(path.newSubPath("execution"), processor, ExecutionPrimitive.class, execution);
		processRosetta(path.newSubPath("exercise"), processor, ExercisePrimitive.class, exercise);
		processRosetta(path.newSubPath("inception"), processor, Inception.class, inception);
		processRosetta(path.newSubPath("observation"), processor, ObservationPrimitive.class, observation);
		processRosetta(path.newSubPath("quantityChange"), processor, QuantityChangePrimitive.class, quantityChange);
		processRosetta(path.newSubPath("reset"), processor, ResetPrimitive.class, reset);
		processRosetta(path.newSubPath("termsChange"), processor, TermsChangePrimitive.class, termsChange);
		processRosetta(path.newSubPath("transfer"), processor, TransferPrimitive.class, transfer);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PrimitiveEvent _that = (PrimitiveEvent) o;
	
		if (!ListEquals.listEquals(allocation, _that.allocation)) return false;
		if (!ListEquals.listEquals(contractFormation, _that.contractFormation)) return false;
		if (!ListEquals.listEquals(execution, _that.execution)) return false;
		if (exercise != null ? !exercise.equals(_that.exercise) : _that.exercise != null) return false;
		if (!ListEquals.listEquals(inception, _that.inception)) return false;
		if (!ListEquals.listEquals(observation, _that.observation)) return false;
		if (!ListEquals.listEquals(quantityChange, _that.quantityChange)) return false;
		if (!ListEquals.listEquals(reset, _that.reset)) return false;
		if (termsChange != null ? !termsChange.equals(_that.termsChange) : _that.termsChange != null) return false;
		if (!ListEquals.listEquals(transfer, _that.transfer)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (allocation != null ? allocation.hashCode() : 0);
		_result = 31 * _result + (contractFormation != null ? contractFormation.hashCode() : 0);
		_result = 31 * _result + (execution != null ? execution.hashCode() : 0);
		_result = 31 * _result + (exercise != null ? exercise.hashCode() : 0);
		_result = 31 * _result + (inception != null ? inception.hashCode() : 0);
		_result = 31 * _result + (observation != null ? observation.hashCode() : 0);
		_result = 31 * _result + (quantityChange != null ? quantityChange.hashCode() : 0);
		_result = 31 * _result + (reset != null ? reset.hashCode() : 0);
		_result = 31 * _result + (termsChange != null ? termsChange.hashCode() : 0);
		_result = 31 * _result + (transfer != null ? transfer.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PrimitiveEvent {" +
			"allocation=" + this.allocation + ", " +
			"contractFormation=" + this.contractFormation + ", " +
			"execution=" + this.execution + ", " +
			"exercise=" + this.exercise + ", " +
			"inception=" + this.inception + ", " +
			"observation=" + this.observation + ", " +
			"quantityChange=" + this.quantityChange + ", " +
			"reset=" + this.reset + ", " +
			"termsChange=" + this.termsChange + ", " +
			"transfer=" + this.transfer +
		'}';
	}
}
