package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.Money;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PrincipalExchangeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class for defining a principal exchange amount and adjusted exchange date. This class forms part of the cashflow representation of a swap stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PrincipalExchange", source="FpML_5_10")
@RosettaSynonym(value="PrincipalExchange", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="PrincipalExchange", source="DTCC_11_0")
@RosettaSynonym(value="PrincipalExchange", source="DTCC_9_0")
@RosettaSynonym(value="PrincipalExchange", source="CME_ClearedConfirm_1_17")
public class PrincipalExchange extends RosettaModelObject implements GlobalKey {
	private final Date adjustedPrincipalExchangeDate;
	private final BigDecimal discountFactor;
	private final MetaFields meta;
	private final Money presentValuePrincipalExchangeAmount;
	private final BigDecimal principalExchangeAmount;
	private final Date unadjustedPrincipalExchangeDate;
	private static PrincipalExchangeMeta metaData = new PrincipalExchangeMeta();
	
	PrincipalExchange(PrincipalExchangeBuilder builder) {
		this.adjustedPrincipalExchangeDate = builder.getAdjustedPrincipalExchangeDate();
		this.discountFactor = builder.getDiscountFactor();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.presentValuePrincipalExchangeAmount = ofNullable(builder.getPresentValuePrincipalExchangeAmount()).map(f->f.build()).orElse(null);
		this.principalExchangeAmount = builder.getPrincipalExchangeAmount();
		this.unadjustedPrincipalExchangeDate = builder.getUnadjustedPrincipalExchangeDate();
	}
	
	/**
	 * The adjusted principal exchange date. This date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedPrincipalExchangeDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedPrincipalExchangeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedPrincipalExchangeDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedPrincipalExchangeDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedPrincipalExchangeDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedPrincipalExchangeDate() {
		return adjustedPrincipalExchangeDate;
	}
	
	/**
	 * The value representing the discount factor used to calculate the present value of the principal exchange amount.
	 */
	@RosettaSynonym(value="discountFactor", source="FpML_5_10")
	@RosettaSynonym(value="discountFactor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_11_0")
	@RosettaSynonym(value="discountFactor", source="DTCC_9_0")
	@RosettaSynonym(value="discountFactor", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getDiscountFactor() {
		return discountFactor;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The amount representing the present value of the principal exchange.
	 */
	@RosettaSynonym(value="presentValuePrincipalExchangeAmount", source="FpML_5_10")
	@RosettaSynonym(value="presentValuePrincipalExchangeAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="presentValuePrincipalExchangeAmount", source="DTCC_11_0")
	@RosettaSynonym(value="presentValuePrincipalExchangeAmount", source="DTCC_9_0")
	@RosettaSynonym(value="presentValuePrincipalExchangeAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPresentValuePrincipalExchangeAmount() {
		return presentValuePrincipalExchangeAmount;
	}
	
	/**
	 * The principal exchange amount. This amount should be positive if the stream payer is paying the exchange amount and signed negative if they are receiving it.
	 */
	@RosettaSynonym(value="principalExchangeAmount", source="FpML_5_10")
	@RosettaSynonym(value="principalExchangeAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="principalExchangeAmount", source="DTCC_11_0")
	@RosettaSynonym(value="principalExchangeAmount", source="DTCC_9_0")
	@RosettaSynonym(value="principalExchangeAmount", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getPrincipalExchangeAmount() {
		return principalExchangeAmount;
	}
	
	/**
	 * The non adjusted principal exchange date.
	 */
	@RosettaSynonym(value="unadjustedPrincipalExchangeDate", source="FpML_5_10")
	@RosettaSynonym(value="unadjustedPrincipalExchangeDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="unadjustedPrincipalExchangeDate", source="DTCC_11_0")
	@RosettaSynonym(value="unadjustedPrincipalExchangeDate", source="DTCC_9_0")
	@RosettaSynonym(value="unadjustedPrincipalExchangeDate", source="CME_ClearedConfirm_1_17")
	public final Date getUnadjustedPrincipalExchangeDate() {
		return unadjustedPrincipalExchangeDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends PrincipalExchange> metaData() {
		return metaData;
	} 
	
	public PrincipalExchangeBuilder toBuilder() {
		PrincipalExchangeBuilder builder = new PrincipalExchangeBuilder();
		ofNullable(getAdjustedPrincipalExchangeDate()).ifPresent(builder::setAdjustedPrincipalExchangeDate);
		ofNullable(getDiscountFactor()).ifPresent(builder::setDiscountFactor);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPresentValuePrincipalExchangeAmount()).ifPresent(builder::setPresentValuePrincipalExchangeAmount);
		ofNullable(getPrincipalExchangeAmount()).ifPresent(builder::setPrincipalExchangeAmount);
		ofNullable(getUnadjustedPrincipalExchangeDate()).ifPresent(builder::setUnadjustedPrincipalExchangeDate);
		return builder;
	}

	public static PrincipalExchangeBuilder builder() {
		return new PrincipalExchangeBuilder();
	}

	public static  class PrincipalExchangeBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PrincipalExchangeBuilder> {
	
		protected Date adjustedPrincipalExchangeDate;
		protected BigDecimal discountFactor;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder presentValuePrincipalExchangeAmount;
		protected BigDecimal principalExchangeAmount;
		protected Date unadjustedPrincipalExchangeDate;
	
		public PrincipalExchangeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PrincipalExchange> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedPrincipalExchangeDate() {
			return adjustedPrincipalExchangeDate;
		}
		
		
		public BigDecimal getDiscountFactor() {
			return discountFactor;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getPresentValuePrincipalExchangeAmount() {
			return presentValuePrincipalExchangeAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePresentValuePrincipalExchangeAmount() {
			if (presentValuePrincipalExchangeAmount!=null) {
				return presentValuePrincipalExchangeAmount;
			}
			else return presentValuePrincipalExchangeAmount = new Money.MoneyBuilder();
		}
		
		
		public BigDecimal getPrincipalExchangeAmount() {
			return principalExchangeAmount;
		}
		
		
		public Date getUnadjustedPrincipalExchangeDate() {
			return unadjustedPrincipalExchangeDate;
		}
		
	
		public PrincipalExchangeBuilder setAdjustedPrincipalExchangeDate(Date adjustedPrincipalExchangeDate) {
			this.adjustedPrincipalExchangeDate = adjustedPrincipalExchangeDate;
			return this;
		}
		public PrincipalExchangeBuilder setDiscountFactor(BigDecimal discountFactor) {
			this.discountFactor = discountFactor;
			return this;
		}
		public PrincipalExchangeBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PrincipalExchangeBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public PrincipalExchangeBuilder setPresentValuePrincipalExchangeAmount(Money presentValuePrincipalExchangeAmount) {
			this.presentValuePrincipalExchangeAmount = presentValuePrincipalExchangeAmount.toBuilder();
			return this;
		}
		public PrincipalExchangeBuilder setPresentValuePrincipalExchangeAmountBuilder(Money.MoneyBuilder presentValuePrincipalExchangeAmount) {
			this.presentValuePrincipalExchangeAmount = presentValuePrincipalExchangeAmount;
			return this;
		}
		public PrincipalExchangeBuilder setPrincipalExchangeAmount(BigDecimal principalExchangeAmount) {
			this.principalExchangeAmount = principalExchangeAmount;
			return this;
		}
		public PrincipalExchangeBuilder setUnadjustedPrincipalExchangeDate(Date unadjustedPrincipalExchangeDate) {
			this.unadjustedPrincipalExchangeDate = unadjustedPrincipalExchangeDate;
			return this;
		}
		
		public PrincipalExchange build() {
			return new PrincipalExchange(this);
		}
	
		@Override
		public PrincipalExchangeBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (presentValuePrincipalExchangeAmount!=null && !presentValuePrincipalExchangeAmount.prune().hasData()) presentValuePrincipalExchangeAmount = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedPrincipalExchangeDate()!=null) return true;
			if (getDiscountFactor()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPresentValuePrincipalExchangeAmount()!=null && getPresentValuePrincipalExchangeAmount().hasData()) return true;
			if (getPrincipalExchangeAmount()!=null) return true;
			if (getUnadjustedPrincipalExchangeDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedPrincipalExchangeDate"), Date.class, adjustedPrincipalExchangeDate, this);
			processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
			processor.processBasic(path.newSubPath("principalExchangeAmount"), BigDecimal.class, principalExchangeAmount, this);
			processor.processBasic(path.newSubPath("unadjustedPrincipalExchangeDate"), Date.class, unadjustedPrincipalExchangeDate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("presentValuePrincipalExchangeAmount"), processor, Money.class, presentValuePrincipalExchangeAmount);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PrincipalExchangeBuilder _that = (PrincipalExchangeBuilder) o;
		
			if (adjustedPrincipalExchangeDate != null ? !adjustedPrincipalExchangeDate.equals(_that.adjustedPrincipalExchangeDate) : _that.adjustedPrincipalExchangeDate != null) return false;
			if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (presentValuePrincipalExchangeAmount != null ? !presentValuePrincipalExchangeAmount.equals(_that.presentValuePrincipalExchangeAmount) : _that.presentValuePrincipalExchangeAmount != null) return false;
			if (principalExchangeAmount != null ? !principalExchangeAmount.equals(_that.principalExchangeAmount) : _that.principalExchangeAmount != null) return false;
			if (unadjustedPrincipalExchangeDate != null ? !unadjustedPrincipalExchangeDate.equals(_that.unadjustedPrincipalExchangeDate) : _that.unadjustedPrincipalExchangeDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedPrincipalExchangeDate != null ? adjustedPrincipalExchangeDate.hashCode() : 0);
			_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (presentValuePrincipalExchangeAmount != null ? presentValuePrincipalExchangeAmount.hashCode() : 0);
			_result = 31 * _result + (principalExchangeAmount != null ? principalExchangeAmount.hashCode() : 0);
			_result = 31 * _result + (unadjustedPrincipalExchangeDate != null ? unadjustedPrincipalExchangeDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PrincipalExchangeBuilder {" +
				"adjustedPrincipalExchangeDate=" + this.adjustedPrincipalExchangeDate + ", " +
				"discountFactor=" + this.discountFactor + ", " +
				"meta=" + this.meta + ", " +
				"presentValuePrincipalExchangeAmount=" + this.presentValuePrincipalExchangeAmount + ", " +
				"principalExchangeAmount=" + this.principalExchangeAmount + ", " +
				"unadjustedPrincipalExchangeDate=" + this.unadjustedPrincipalExchangeDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedPrincipalExchangeDate"), Date.class, adjustedPrincipalExchangeDate, this);
		processor.processBasic(path.newSubPath("discountFactor"), BigDecimal.class, discountFactor, this);
		processor.processBasic(path.newSubPath("principalExchangeAmount"), BigDecimal.class, principalExchangeAmount, this);
		processor.processBasic(path.newSubPath("unadjustedPrincipalExchangeDate"), Date.class, unadjustedPrincipalExchangeDate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("presentValuePrincipalExchangeAmount"), processor, Money.class, presentValuePrincipalExchangeAmount);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PrincipalExchange _that = (PrincipalExchange) o;
	
		if (adjustedPrincipalExchangeDate != null ? !adjustedPrincipalExchangeDate.equals(_that.adjustedPrincipalExchangeDate) : _that.adjustedPrincipalExchangeDate != null) return false;
		if (discountFactor != null ? !discountFactor.equals(_that.discountFactor) : _that.discountFactor != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (presentValuePrincipalExchangeAmount != null ? !presentValuePrincipalExchangeAmount.equals(_that.presentValuePrincipalExchangeAmount) : _that.presentValuePrincipalExchangeAmount != null) return false;
		if (principalExchangeAmount != null ? !principalExchangeAmount.equals(_that.principalExchangeAmount) : _that.principalExchangeAmount != null) return false;
		if (unadjustedPrincipalExchangeDate != null ? !unadjustedPrincipalExchangeDate.equals(_that.unadjustedPrincipalExchangeDate) : _that.unadjustedPrincipalExchangeDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedPrincipalExchangeDate != null ? adjustedPrincipalExchangeDate.hashCode() : 0);
		_result = 31 * _result + (discountFactor != null ? discountFactor.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (presentValuePrincipalExchangeAmount != null ? presentValuePrincipalExchangeAmount.hashCode() : 0);
		_result = 31 * _result + (principalExchangeAmount != null ? principalExchangeAmount.hashCode() : 0);
		_result = 31 * _result + (unadjustedPrincipalExchangeDate != null ? unadjustedPrincipalExchangeDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PrincipalExchange {" +
			"adjustedPrincipalExchangeDate=" + this.adjustedPrincipalExchangeDate + ", " +
			"discountFactor=" + this.discountFactor + ", " +
			"meta=" + this.meta + ", " +
			"presentValuePrincipalExchangeAmount=" + this.presentValuePrincipalExchangeAmount + ", " +
			"principalExchangeAmount=" + this.principalExchangeAmount + ", " +
			"unadjustedPrincipalExchangeDate=" + this.unadjustedPrincipalExchangeDate +
		'}';
	}
}
