package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PrincipalExchangesMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class defining which principal exchanges occur for the stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PrincipalExchanges", source="FpML_5_10")
@RosettaSynonym(value="PrincipalExchanges", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="PrincipalExchanges", source="DTCC_11_0")
@RosettaSynonym(value="PrincipalExchanges", source="DTCC_9_0")
@RosettaSynonym(value="PrincipalExchanges", source="CME_ClearedConfirm_1_17")
public class PrincipalExchanges extends RosettaModelObject implements GlobalKey {
	private final Boolean finalExchange;
	private final Boolean initialExchange;
	private final Boolean intermediateExchange;
	private final MetaFields meta;
	private static PrincipalExchangesMeta metaData = new PrincipalExchangesMeta();
	
	PrincipalExchanges(PrincipalExchangesBuilder builder) {
		this.finalExchange = builder.getFinalExchange();
		this.initialExchange = builder.getInitialExchange();
		this.intermediateExchange = builder.getIntermediateExchange();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * A true/false flag to indicate whether there is a final exchange of principal on the termination date.
	 */
	@RosettaSynonym(value="finalExchange", source="FpML_5_10")
	@RosettaSynonym(value="finalExchange", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="finalExchange", source="DTCC_11_0")
	@RosettaSynonym(value="finalExchange", source="DTCC_9_0")
	@RosettaSynonym(value="finalExchange", source="CME_ClearedConfirm_1_17")
	public final Boolean getFinalExchange() {
		return finalExchange;
	}
	
	/**
	 * A true/false flag to indicate whether there is an initial exchange of principal on the effective date.
	 */
	@RosettaSynonym(value="initialExchange", source="FpML_5_10")
	@RosettaSynonym(value="initialExchange", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialExchange", source="DTCC_11_0")
	@RosettaSynonym(value="initialExchange", source="DTCC_9_0")
	@RosettaSynonym(value="initialExchange", source="CME_ClearedConfirm_1_17")
	public final Boolean getInitialExchange() {
		return initialExchange;
	}
	
	/**
	 * A true/false flag to indicate whether there are intermediate or interim exchanges of principal during the term of the swap.
	 */
	@RosettaSynonym(value="intermediateExchange", source="FpML_5_10")
	@RosettaSynonym(value="intermediateExchange", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="intermediateExchange", source="DTCC_11_0")
	@RosettaSynonym(value="intermediateExchange", source="DTCC_9_0")
	@RosettaSynonym(value="intermediateExchange", source="CME_ClearedConfirm_1_17")
	public final Boolean getIntermediateExchange() {
		return intermediateExchange;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends PrincipalExchanges> metaData() {
		return metaData;
	} 
	
	public PrincipalExchangesBuilder toBuilder() {
		PrincipalExchangesBuilder builder = new PrincipalExchangesBuilder();
		ofNullable(getFinalExchange()).ifPresent(builder::setFinalExchange);
		ofNullable(getInitialExchange()).ifPresent(builder::setInitialExchange);
		ofNullable(getIntermediateExchange()).ifPresent(builder::setIntermediateExchange);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static PrincipalExchangesBuilder builder() {
		return new PrincipalExchangesBuilder();
	}

	public static  class PrincipalExchangesBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<PrincipalExchangesBuilder> {
	
		protected Boolean finalExchange;
		protected Boolean initialExchange;
		protected Boolean intermediateExchange;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public PrincipalExchangesBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PrincipalExchanges> metaData() {
			return metaData;
		} 
	
		
		public Boolean getFinalExchange() {
			return finalExchange;
		}
		
		
		public Boolean getInitialExchange() {
			return initialExchange;
		}
		
		
		public Boolean getIntermediateExchange() {
			return intermediateExchange;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public PrincipalExchangesBuilder setFinalExchange(Boolean finalExchange) {
			this.finalExchange = finalExchange;
			return this;
		}
		public PrincipalExchangesBuilder setInitialExchange(Boolean initialExchange) {
			this.initialExchange = initialExchange;
			return this;
		}
		public PrincipalExchangesBuilder setIntermediateExchange(Boolean intermediateExchange) {
			this.intermediateExchange = intermediateExchange;
			return this;
		}
		public PrincipalExchangesBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public PrincipalExchangesBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public PrincipalExchanges build() {
			return new PrincipalExchanges(this);
		}
	
		@Override
		public PrincipalExchangesBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFinalExchange()!=null) return true;
			if (getInitialExchange()!=null) return true;
			if (getIntermediateExchange()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("finalExchange"), Boolean.class, finalExchange, this);
			processor.processBasic(path.newSubPath("initialExchange"), Boolean.class, initialExchange, this);
			processor.processBasic(path.newSubPath("intermediateExchange"), Boolean.class, intermediateExchange, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PrincipalExchangesBuilder _that = (PrincipalExchangesBuilder) o;
		
			if (finalExchange != null ? !finalExchange.equals(_that.finalExchange) : _that.finalExchange != null) return false;
			if (initialExchange != null ? !initialExchange.equals(_that.initialExchange) : _that.initialExchange != null) return false;
			if (intermediateExchange != null ? !intermediateExchange.equals(_that.intermediateExchange) : _that.intermediateExchange != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (finalExchange != null ? finalExchange.hashCode() : 0);
			_result = 31 * _result + (initialExchange != null ? initialExchange.hashCode() : 0);
			_result = 31 * _result + (intermediateExchange != null ? intermediateExchange.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PrincipalExchangesBuilder {" +
				"finalExchange=" + this.finalExchange + ", " +
				"initialExchange=" + this.initialExchange + ", " +
				"intermediateExchange=" + this.intermediateExchange + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("finalExchange"), Boolean.class, finalExchange, this);
		processor.processBasic(path.newSubPath("initialExchange"), Boolean.class, initialExchange, this);
		processor.processBasic(path.newSubPath("intermediateExchange"), Boolean.class, intermediateExchange, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PrincipalExchanges _that = (PrincipalExchanges) o;
	
		if (finalExchange != null ? !finalExchange.equals(_that.finalExchange) : _that.finalExchange != null) return false;
		if (initialExchange != null ? !initialExchange.equals(_that.initialExchange) : _that.initialExchange != null) return false;
		if (intermediateExchange != null ? !intermediateExchange.equals(_that.intermediateExchange) : _that.intermediateExchange != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (finalExchange != null ? finalExchange.hashCode() : 0);
		_result = 31 * _result + (initialExchange != null ? initialExchange.hashCode() : 0);
		_result = 31 * _result + (intermediateExchange != null ? intermediateExchange.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PrincipalExchanges {" +
			"finalExchange=" + this.finalExchange + ", " +
			"initialExchange=" + this.initialExchange + ", " +
			"intermediateExchange=" + this.intermediateExchange + ", " +
			"meta=" + this.meta +
		'}';
	}
}
