package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ProcessAgentElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProcessAgentMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Process Agent that might be appointed by the parties to the agreement in accordance with the ISDA 2016 English Law CSA, paragraph 11(h). ISDA 2016 English Law Credit Support Deed for Initial Margin, paragraph 13, General Principles, (t): Process Agent.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ProcessAgent extends RosettaModelObject {
	private final List<ProcessAgentElection> partyElection;
	private static ProcessAgentMeta metaData = new ProcessAgentMeta();
	
	ProcessAgent(ProcessAgentBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; Process Agent election.
	 */
	public final List<ProcessAgentElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProcessAgent> metaData() {
		return metaData;
	} 
	
	public ProcessAgentBuilder toBuilder() {
		ProcessAgentBuilder builder = new ProcessAgentBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static ProcessAgentBuilder builder() {
		return new ProcessAgentBuilder();
	}

	public static  class ProcessAgentBuilder extends RosettaModelObjectBuilder {
	
		protected List<ProcessAgentElection.ProcessAgentElectionBuilder> partyElection;
	
		public ProcessAgentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProcessAgent> metaData() {
			return metaData;
		} 
	
		
		public List<ProcessAgentElection.ProcessAgentElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public ProcessAgentElection.ProcessAgentElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new ProcessAgentElection.ProcessAgentElectionBuilder());
		}
		
	
		public ProcessAgentBuilder addPartyElection(ProcessAgentElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public ProcessAgentBuilder addPartyElection(ProcessAgentElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public ProcessAgentBuilder addPartyElection(List<ProcessAgentElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (ProcessAgentElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public ProcessAgentBuilder addPartyElectionBuilder(ProcessAgentElection.ProcessAgentElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public ProcessAgentBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public ProcessAgent build() {
			return new ProcessAgent(this);
		}
	
		@Override
		public ProcessAgentBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, ProcessAgentElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProcessAgentBuilder _that = (ProcessAgentBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProcessAgentBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, ProcessAgentElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProcessAgent _that = (ProcessAgent) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProcessAgent {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
