package org.isda.cdm;


/**
 * The enumerated values to specify the product identifier source. FpML doesn&#39;t specify a list of values.
 * 
 * @version 2.5.4
 */
public enum ProductIdSourceEnum {
    

	/**
	 * The Committee on Uniform Security Identification Procedures
	 */
	CUSIP,
	    

	/**
	 * The Stock Exchange Daily Official List, a list of security identifiers used in the United Kingdom and Ireland for clearing purposes.
	 */
	SEDOL,
	    

	/**
	 * The ISO 6166 product identifier
	 */
	ISIN,
	    

	/**
	 * The Reuters Instrument Code
	 */
	RIC,
	    

	/**
	 * The Financial Instrument Global Identifier
	 */
	FIGI,
	    

	/**
	 * The German Wertpapierkennnummer securities identification code. It is composed of six digits or capital letters (excluding I and O), and no check digit.
	 */
	WERTPAPIER,
	    

	/**
	 * The identification code issued by the French Société Interprofessionnelle pour la Compensation des Valeurs Mobilières.
	 */
	SICOVAM    
	;

	private final String displayName;
	
	ProductIdSourceEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
