package org.isda.cdm;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.metafields.FieldWithMetaAssetClassEnum;
import org.isda.cdm.metafields.FieldWithMetaAssetClassEnum.FieldWithMetaAssetClassEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProductIdentificationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaQualified;
import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to combine the CDM product qualifier with other product qualifiers, such as the FpML ones. While the CDM product qualifier is derived by the CDM from the product payout features, the other product identification elements are assigned by some external sources and correspond to values specified by other data representation protocols.
 * 
 * @version 2.5.4
 */
@RosettaClass
@RosettaQualified(attribute="productQualifier",qualifiedClass=EconomicTerms.class)
public class ProductIdentification extends RosettaModelObject {
	private final FieldWithMetaAssetClassEnum primaryAssetClass;
	private final List<FieldWithMetaString> productId;
	private final String productQualifier;
	private final List<FieldWithMetaString> productType;
	private final List<FieldWithMetaAssetClassEnum> secondaryAssetClass;
	private static ProductIdentificationMeta metaData = new ProductIdentificationMeta();
	
	ProductIdentification(ProductIdentificationBuilder builder) {
		this.primaryAssetClass = ofNullable(builder.getPrimaryAssetClass()).map(f->f.build()).orElse(null);
		this.productId = ofNullable(builder.getProductId()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.productQualifier = builder.getProductQualifier();
		this.productType = ofNullable(builder.getProductType()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.secondaryAssetClass = ofNullable(builder.getSecondaryAssetClass()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * A classification of the most important risk class of the trade. FpML defines a simple asset class categorisation using a coding scheme.
	 */
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17", path="swaption")
	@RosettaSynonym(value="primaryAssetClass", source="FpML_5_10")
	@RosettaSynonym(value="primaryAssetClass", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="primaryAssetClass", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaAssetClassEnum getPrimaryAssetClass() {
		return primaryAssetClass;
	}
	
	/**
	 * A product reference identifier. The product Id is an identifier that describes the key economic characteristics of the trade type, with the exception of concepts such as size (notional, quantity, number of units) and price (fixed rate, strike, etc.) that are negotiated for each transaction. It can be used to hold identifiers such as the &#39;UPI&#39; (universal product identifier) required by certain regulatory reporting rules. It can also be used to hold identifiers of benchmark products or product temnplates used by certain trading systems or facilities. FpML does not define the domain values associated with this element. Note that the domain values for this element are not strictly an enumerated list.
	 */
	@RosettaSynonym(value="productId", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productId", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productId", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="productId", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="productId", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="swaption")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="trade.swaption.swap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="trade.swaption.swap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="trade.swaption.swap")
	@RosettaSynonym(value="productId", source="FpML_5_10", path="swaption.swap")
	@RosettaSynonym(value="productId", source="CME_SubmissionIRS_1_0", path="swaption.swap")
	@RosettaSynonym(value="productId", source="CME_ClearedConfirm_1_17", path="swaption.swap")
	@RosettaSynonym(value="instrumentId", source="DTCC_11_0", path="payment.calculationDetails.calculationElements.underlier.index")
	@RosettaSynonym(value="instrumentId", source="DTCC_9_0", path="payment.calculationDetails.calculationElements.underlier.index")
	public final List<FieldWithMetaString> getProductId() {
		return productId;
	}
	
	/**
	 * The CDM product qualifier, which corresponds to the outcome of the isProduct qualification logic. This value is derived by the CDM from the product payout features.
	 */
	public final String getProductQualifier() {
		return productQualifier;
	}
	
	/**
	 * A classification of the type of product. FpML defines a simple product categorisation using a coding scheme.
	 */
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="swaption")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.swaption.swap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="trade.swaption.swap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="trade.swaption.swap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="swaption.swap")
	@RosettaSynonym(value="productType", source="CME_SubmissionIRS_1_0", path="swaption.swap")
	@RosettaSynonym(value="productType", source="CME_ClearedConfirm_1_17", path="swaption.swap")
	@RosettaSynonym(value="ProductType", source="DTCC_11_0")
	@RosettaSynonym(value="ProductType", source="DTCC_9_0")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.returnSwap")
	@RosettaSynonym(value="productType", source="FpML_5_10", path="trade.equitySwapTransactionSupplement")
	public final List<FieldWithMetaString> getProductType() {
		return productType;
	}
	
	/**
	 * A classification of additional risk classes of the trade, if any. FpML defines a simple asset class categorisation using a coding scheme.
	 */
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="trade.swap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.swap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.swap")
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="swap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="swap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="swap")
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="DTCC_11_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="DTCC_9_0", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="DTCC_11_0", path="creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="DTCC_9_0", path="creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="creditDefaultSwap")
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="trade.swaption")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="trade.swaption")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="trade.swaption")
	@RosettaSynonym(value="secondaryAssetClass", source="FpML_5_10", path="swaption")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_SubmissionIRS_1_0", path="swaption")
	@RosettaSynonym(value="secondaryAssetClass", source="CME_ClearedConfirm_1_17", path="swaption")
	public final List<FieldWithMetaAssetClassEnum> getSecondaryAssetClass() {
		return secondaryAssetClass;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProductIdentification> metaData() {
		return metaData;
	} 
	
	public ProductIdentificationBuilder toBuilder() {
		ProductIdentificationBuilder builder = new ProductIdentificationBuilder();
		ofNullable(getPrimaryAssetClass()).ifPresent(builder::setPrimaryAssetClass);
		ofNullable(getProductId()).ifPresent(productId -> productId.forEach(builder::addProductId));
		ofNullable(getProductQualifier()).ifPresent(builder::setProductQualifier);
		ofNullable(getProductType()).ifPresent(productType -> productType.forEach(builder::addProductType));
		ofNullable(getSecondaryAssetClass()).ifPresent(secondaryAssetClass -> secondaryAssetClass.forEach(builder::addSecondaryAssetClass));
		return builder;
	}

	public static ProductIdentificationBuilder builder() {
		return new ProductIdentificationBuilder();
	}

	public static  class ProductIdentificationBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaAssetClassEnumBuilder primaryAssetClass;
		protected List<FieldWithMetaStringBuilder> productId;
		protected String productQualifier;
		protected List<FieldWithMetaStringBuilder> productType;
		protected List<FieldWithMetaAssetClassEnumBuilder> secondaryAssetClass;
	
		public ProductIdentificationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProductIdentification> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaAssetClassEnumBuilder getPrimaryAssetClass() {
			return primaryAssetClass;
		}
		
		public FieldWithMetaAssetClassEnumBuilder getOrCreatePrimaryAssetClass() {
			if (primaryAssetClass!=null) {
				return primaryAssetClass;
			}
			else return primaryAssetClass = new FieldWithMetaAssetClassEnumBuilder();
		}
		
		
		public List<FieldWithMetaStringBuilder> getProductId() {
			return productId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateProductId(int index) {
			if (productId==null) {
				this.productId = new ArrayList<>();
			}
			return getIndex(productId, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public String getProductQualifier() {
			return productQualifier;
		}
		
		
		public List<FieldWithMetaStringBuilder> getProductType() {
			return productType;
		}
		
		public FieldWithMetaStringBuilder getOrCreateProductType(int index) {
			if (productType==null) {
				this.productType = new ArrayList<>();
			}
			return getIndex(productType, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public List<FieldWithMetaAssetClassEnumBuilder> getSecondaryAssetClass() {
			return secondaryAssetClass;
		}
		
		public FieldWithMetaAssetClassEnumBuilder getOrCreateSecondaryAssetClass(int index) {
			if (secondaryAssetClass==null) {
				this.secondaryAssetClass = new ArrayList<>();
			}
			return getIndex(secondaryAssetClass, index, ()->new FieldWithMetaAssetClassEnumBuilder());
		}
		
	
		public ProductIdentificationBuilder setPrimaryAssetClass(FieldWithMetaAssetClassEnum primaryAssetClass) {
			this.primaryAssetClass = primaryAssetClass.toBuilder();
			return this;
		}
		public ProductIdentificationBuilder setPrimaryAssetClassRef(AssetClassEnum primaryAssetClass) {
			return setPrimaryAssetClass(FieldWithMetaAssetClassEnum.builder().setValue(primaryAssetClass).build());
		}
		public ProductIdentificationBuilder addProductId(FieldWithMetaString productId) {
			if(this.productId == null){
				this.productId = new ArrayList<>();
			}
			this.productId.add(productId.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addProductId(FieldWithMetaString productId, int _idx) {
			if(this.productId == null){
				this.productId = new ArrayList<>();
			}
			getIndex(this.productId, _idx, () -> productId.toBuilder());
			this.productId.set(_idx, productId.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addProductId(List<FieldWithMetaString> productIds) {
			if(this.productId == null){
				this.productId = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : productIds) {
				this.productId.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public ProductIdentificationBuilder clearProductId() {
			this.productId = null;
			return this;
		}
		public ProductIdentificationBuilder setProductQualifier(String productQualifier) {
			this.productQualifier = productQualifier;
			return this;
		}
		public ProductIdentificationBuilder addProductType(FieldWithMetaString productType) {
			if(this.productType == null){
				this.productType = new ArrayList<>();
			}
			this.productType.add(productType.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addProductType(FieldWithMetaString productType, int _idx) {
			if(this.productType == null){
				this.productType = new ArrayList<>();
			}
			getIndex(this.productType, _idx, () -> productType.toBuilder());
			this.productType.set(_idx, productType.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addProductType(List<FieldWithMetaString> productTypes) {
			if(this.productType == null){
				this.productType = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : productTypes) {
				this.productType.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public ProductIdentificationBuilder clearProductType() {
			this.productType = null;
			return this;
		}
		public ProductIdentificationBuilder addSecondaryAssetClass(FieldWithMetaAssetClassEnum secondaryAssetClass) {
			if(this.secondaryAssetClass == null){
				this.secondaryAssetClass = new ArrayList<>();
			}
			this.secondaryAssetClass.add(secondaryAssetClass.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addSecondaryAssetClass(FieldWithMetaAssetClassEnum secondaryAssetClass, int _idx) {
			if(this.secondaryAssetClass == null){
				this.secondaryAssetClass = new ArrayList<>();
			}
			getIndex(this.secondaryAssetClass, _idx, () -> secondaryAssetClass.toBuilder());
			this.secondaryAssetClass.set(_idx, secondaryAssetClass.toBuilder());
			return this;
		}
		public ProductIdentificationBuilder addSecondaryAssetClass(List<FieldWithMetaAssetClassEnum> secondaryAssetClasss) {
			if(this.secondaryAssetClass == null){
				this.secondaryAssetClass = new ArrayList<>();
			}
			for (FieldWithMetaAssetClassEnum toAdd : secondaryAssetClasss) {
				this.secondaryAssetClass.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public ProductIdentificationBuilder clearSecondaryAssetClass() {
			this.secondaryAssetClass = null;
			return this;
		}
		
		public ProductIdentification build() {
			return new ProductIdentification(this);
		}
	
		@Override
		public ProductIdentificationBuilder prune() {
			if (primaryAssetClass!=null && !primaryAssetClass.prune().hasData()) primaryAssetClass = null;
			if (productId!=null) productId = productId.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (productType!=null) productType = productType.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (secondaryAssetClass!=null) secondaryAssetClass = secondaryAssetClass.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrimaryAssetClass()!=null) return true;
			if (getProductId()!=null && !getProductId().isEmpty()) return true;
			if (getProductQualifier()!=null) return true;
			if (getProductType()!=null && !getProductType().isEmpty()) return true;
			if (getSecondaryAssetClass()!=null && !getSecondaryAssetClass().isEmpty()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("productQualifier"), String.class, productQualifier, this);
			
			processRosetta(path.newSubPath("primaryAssetClass"), processor, FieldWithMetaAssetClassEnum.class, primaryAssetClass);
			processRosetta(path.newSubPath("productId"), processor, FieldWithMetaString.class, productId);
			processRosetta(path.newSubPath("productType"), processor, FieldWithMetaString.class, productType);
			processRosetta(path.newSubPath("secondaryAssetClass"), processor, FieldWithMetaAssetClassEnum.class, secondaryAssetClass);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProductIdentificationBuilder _that = (ProductIdentificationBuilder) o;
		
			if (primaryAssetClass != null ? !primaryAssetClass.equals(_that.primaryAssetClass) : _that.primaryAssetClass != null) return false;
			if (!ListEquals.listEquals(productId, _that.productId)) return false;
			if (productQualifier != null ? !productQualifier.equals(_that.productQualifier) : _that.productQualifier != null) return false;
			if (!ListEquals.listEquals(productType, _that.productType)) return false;
			if (!ListEquals.listEquals(secondaryAssetClass, _that.secondaryAssetClass)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (primaryAssetClass != null ? primaryAssetClass.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (productId != null ? productId.hashCode() : 0);
			_result = 31 * _result + (productQualifier != null ? productQualifier.hashCode() : 0);
			_result = 31 * _result + (productType != null ? productType.hashCode() : 0);
			_result = 31 * _result + (secondaryAssetClass != null ? secondaryAssetClass.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProductIdentificationBuilder {" +
				"primaryAssetClass=" + this.primaryAssetClass + ", " +
				"productId=" + this.productId + ", " +
				"productQualifier=" + this.productQualifier + ", " +
				"productType=" + this.productType + ", " +
				"secondaryAssetClass=" + this.secondaryAssetClass +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("productQualifier"), String.class, productQualifier, this);
		
		processRosetta(path.newSubPath("primaryAssetClass"), processor, FieldWithMetaAssetClassEnum.class, primaryAssetClass);
		processRosetta(path.newSubPath("productId"), processor, FieldWithMetaString.class, productId);
		processRosetta(path.newSubPath("productType"), processor, FieldWithMetaString.class, productType);
		processRosetta(path.newSubPath("secondaryAssetClass"), processor, FieldWithMetaAssetClassEnum.class, secondaryAssetClass);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProductIdentification _that = (ProductIdentification) o;
	
		if (primaryAssetClass != null ? !primaryAssetClass.equals(_that.primaryAssetClass) : _that.primaryAssetClass != null) return false;
		if (!ListEquals.listEquals(productId, _that.productId)) return false;
		if (productQualifier != null ? !productQualifier.equals(_that.productQualifier) : _that.productQualifier != null) return false;
		if (!ListEquals.listEquals(productType, _that.productType)) return false;
		if (!ListEquals.listEquals(secondaryAssetClass, _that.secondaryAssetClass)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (primaryAssetClass != null ? primaryAssetClass.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (productId != null ? productId.hashCode() : 0);
		_result = 31 * _result + (productQualifier != null ? productQualifier.hashCode() : 0);
		_result = 31 * _result + (productType != null ? productType.hashCode() : 0);
		_result = 31 * _result + (secondaryAssetClass != null ? secondaryAssetClass.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProductIdentification {" +
			"primaryAssetClass=" + this.primaryAssetClass + ", " +
			"productId=" + this.productId + ", " +
			"productQualifier=" + this.productQualifier + ", " +
			"productType=" + this.productType + ", " +
			"secondaryAssetClass=" + this.secondaryAssetClass +
		'}';
	}
}
