package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ProductIdSourceEnum;
import org.isda.cdm.ProductTaxonomy;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProductIdentifierMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * The product identifier, composed of an identifier, a source and a product taxonomy. The source is optional because it can be specified through the scheme that is associated with the identifier. As FpML doesn&#39;t specify a scheme as part of the standard, that scheme cannot be mapped to the CDM ProductIdSourceEnum. The associated rosettaKey denotes the ability to associate a hash value to the ProductIdentifier instantiations for the purpose of model cross-referencing, in support of functionality such as the event effect and the lineage.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ProductIdentifier extends RosettaModelObject implements GlobalKey {
	private final List<FieldWithMetaString> identifier;
	private final MetaFields meta;
	private final List<ProductTaxonomy> productTaxonomy;
	private final ProductIdSourceEnum source;
	private static ProductIdentifierMeta metaData = new ProductIdentifierMeta();
	
	ProductIdentifier(ProductIdentifierBuilder builder) {
		this.identifier = ofNullable(builder.getIdentifier()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.productTaxonomy = ofNullable(builder.getProductTaxonomy()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.source = builder.getSource();
	}
	
	/**
	 *  A unique identifier of an asset provided by a public source that is specified in the source attribute.
	 */
	@RosettaSynonym(value="instrumentId", source="FpML_5_10")
	@RosettaSynonym(value="instrumentId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="instrumentId", source="Rosetta_Workbench")
	@RosettaSynonym(value="instrumentId", source="DTCC_11_0")
	@RosettaSynonym(value="instrumentId", source="DTCC_9_0")
	@RosettaSynonym(value="instrumentId", source="Rosetta_Workbench")
	public final List<FieldWithMetaString> getIdentifier() {
		return identifier;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The product taxonomy value(s) associated with a product.
	 */
	public final List<ProductTaxonomy> getProductTaxonomy() {
		return productTaxonomy;
	}
	
	/**
	 * The identifier source.
	 */
	public final ProductIdSourceEnum getSource() {
		return source;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProductIdentifier> metaData() {
		return metaData;
	} 
	
	public ProductIdentifierBuilder toBuilder() {
		ProductIdentifierBuilder builder = new ProductIdentifierBuilder();
		ofNullable(getIdentifier()).ifPresent(identifier -> identifier.forEach(builder::addIdentifier));
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getProductTaxonomy()).ifPresent(productTaxonomy -> productTaxonomy.forEach(builder::addProductTaxonomy));
		ofNullable(getSource()).ifPresent(builder::setSource);
		return builder;
	}

	public static ProductIdentifierBuilder builder() {
		return new ProductIdentifierBuilder();
	}

	public static  class ProductIdentifierBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ProductIdentifierBuilder> {
	
		protected List<FieldWithMetaStringBuilder> identifier;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<ProductTaxonomy.ProductTaxonomyBuilder> productTaxonomy;
		protected ProductIdSourceEnum source;
	
		public ProductIdentifierBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProductIdentifier> metaData() {
			return metaData;
		} 
	
		
		public List<FieldWithMetaStringBuilder> getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier(int index) {
			if (identifier==null) {
				this.identifier = new ArrayList<>();
			}
			return getIndex(identifier, index, ()->new FieldWithMetaStringBuilder());
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<ProductTaxonomy.ProductTaxonomyBuilder> getProductTaxonomy() {
			return productTaxonomy;
		}
		
		public ProductTaxonomy.ProductTaxonomyBuilder getOrCreateProductTaxonomy(int index) {
			if (productTaxonomy==null) {
				this.productTaxonomy = new ArrayList<>();
			}
			return getIndex(productTaxonomy, index, ()->new ProductTaxonomy.ProductTaxonomyBuilder());
		}
		
		
		public ProductIdSourceEnum getSource() {
			return source;
		}
		
	
		public ProductIdentifierBuilder addIdentifier(FieldWithMetaString identifier) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			this.identifier.add(identifier.toBuilder());
			return this;
		}
		public ProductIdentifierBuilder addIdentifier(FieldWithMetaString identifier, int _idx) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			getIndex(this.identifier, _idx, () -> identifier.toBuilder());
			this.identifier.set(_idx, identifier.toBuilder());
			return this;
		}
		public ProductIdentifierBuilder addIdentifier(List<FieldWithMetaString> identifiers) {
			if(this.identifier == null){
				this.identifier = new ArrayList<>();
			}
			for (FieldWithMetaString toAdd : identifiers) {
				this.identifier.add(toAdd.toBuilder());
			}
			return this;
		}
		
		public ProductIdentifierBuilder clearIdentifier() {
			this.identifier = null;
			return this;
		}
		public ProductIdentifierBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ProductIdentifierBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ProductIdentifierBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy.toBuilder());
			return this;
		}
		public ProductIdentifierBuilder addProductTaxonomy(ProductTaxonomy productTaxonomy, int _idx) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			getIndex(this.productTaxonomy, _idx, () -> productTaxonomy.toBuilder());
			this.productTaxonomy.set(_idx, productTaxonomy.toBuilder());
			return this;
		}
		public ProductIdentifierBuilder addProductTaxonomy(List<ProductTaxonomy> productTaxonomys) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			for (ProductTaxonomy toAdd : productTaxonomys) {
				this.productTaxonomy.add(toAdd.toBuilder());
			}
			return this;
		}
		public ProductIdentifierBuilder addProductTaxonomyBuilder(ProductTaxonomy.ProductTaxonomyBuilder productTaxonomy) {
			if(this.productTaxonomy == null){
				this.productTaxonomy = new ArrayList<>();
			}
			this.productTaxonomy.add(productTaxonomy);
			return this;
		}
		
		public ProductIdentifierBuilder clearProductTaxonomy() {
			this.productTaxonomy = null;
			return this;
		}
		public ProductIdentifierBuilder setSource(ProductIdSourceEnum source) {
			this.source = source;
			return this;
		}
		
		public ProductIdentifier build() {
			return new ProductIdentifier(this);
		}
	
		@Override
		public ProductIdentifierBuilder prune() {
			if (identifier!=null) identifier = identifier.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (productTaxonomy!=null) productTaxonomy = productTaxonomy.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null && !getIdentifier().isEmpty()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getProductTaxonomy()!=null && getProductTaxonomy().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getSource()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("source"), ProductIdSourceEnum.class, source, this);
			
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("productTaxonomy"), processor, ProductTaxonomy.class, productTaxonomy);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProductIdentifierBuilder _that = (ProductIdentifierBuilder) o;
		
			if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(productTaxonomy, _that.productTaxonomy)) return false;
			if (source != null ? !source.equals(_that.source) : _that.source != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (productTaxonomy != null ? productTaxonomy.hashCode() : 0);
			_result = 31 * _result + (source != null ? source.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProductIdentifierBuilder {" +
				"identifier=" + this.identifier + ", " +
				"meta=" + this.meta + ", " +
				"productTaxonomy=" + this.productTaxonomy + ", " +
				"source=" + this.source +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("source"), ProductIdSourceEnum.class, source, this);
		
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("productTaxonomy"), processor, ProductTaxonomy.class, productTaxonomy);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProductIdentifier _that = (ProductIdentifier) o;
	
		if (!ListEquals.listEquals(identifier, _that.identifier)) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(productTaxonomy, _that.productTaxonomy)) return false;
		if (source != null ? !source.equals(_that.source) : _that.source != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (productTaxonomy != null ? productTaxonomy.hashCode() : 0);
		_result = 31 * _result + (source != null ? source.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProductIdentifier {" +
			"identifier=" + this.identifier + ", " +
			"meta=" + this.meta + ", " +
			"productTaxonomy=" + this.productTaxonomy + ", " +
			"source=" + this.source +
		'}';
	}
}
