package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.TaxonomySourceEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ProductTaxonomyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The product taxonomy, which is composed of a taxonomy value and a taxonomy source.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class ProductTaxonomy extends RosettaModelObject {
	private final TaxonomySourceEnum taxonomySource;
	private final String taxonomyValue;
	private static ProductTaxonomyMeta metaData = new ProductTaxonomyMeta();
	
	ProductTaxonomy(ProductTaxonomyBuilder builder) {
		this.taxonomySource = builder.getTaxonomySource();
		this.taxonomyValue = builder.getTaxonomyValue();
	}
	
	/**
	 * The taxonomy source.
	 */
	public final TaxonomySourceEnum getTaxonomySource() {
		return taxonomySource;
	}
	
	/**
	 * The taxonomy value.
	 */
	public final String getTaxonomyValue() {
		return taxonomyValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends ProductTaxonomy> metaData() {
		return metaData;
	} 
	
	public ProductTaxonomyBuilder toBuilder() {
		ProductTaxonomyBuilder builder = new ProductTaxonomyBuilder();
		ofNullable(getTaxonomySource()).ifPresent(builder::setTaxonomySource);
		ofNullable(getTaxonomyValue()).ifPresent(builder::setTaxonomyValue);
		return builder;
	}

	public static ProductTaxonomyBuilder builder() {
		return new ProductTaxonomyBuilder();
	}

	public static  class ProductTaxonomyBuilder extends RosettaModelObjectBuilder {
	
		protected TaxonomySourceEnum taxonomySource;
		protected String taxonomyValue;
	
		public ProductTaxonomyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ProductTaxonomy> metaData() {
			return metaData;
		} 
	
		
		public TaxonomySourceEnum getTaxonomySource() {
			return taxonomySource;
		}
		
		
		public String getTaxonomyValue() {
			return taxonomyValue;
		}
		
	
		public ProductTaxonomyBuilder setTaxonomySource(TaxonomySourceEnum taxonomySource) {
			this.taxonomySource = taxonomySource;
			return this;
		}
		public ProductTaxonomyBuilder setTaxonomyValue(String taxonomyValue) {
			this.taxonomyValue = taxonomyValue;
			return this;
		}
		
		public ProductTaxonomy build() {
			return new ProductTaxonomy(this);
		}
	
		@Override
		public ProductTaxonomyBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getTaxonomySource()!=null) return true;
			if (getTaxonomyValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("taxonomySource"), TaxonomySourceEnum.class, taxonomySource, this);
			processor.processBasic(path.newSubPath("taxonomyValue"), String.class, taxonomyValue, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ProductTaxonomyBuilder _that = (ProductTaxonomyBuilder) o;
		
			if (taxonomySource != null ? !taxonomySource.equals(_that.taxonomySource) : _that.taxonomySource != null) return false;
			if (taxonomyValue != null ? !taxonomyValue.equals(_that.taxonomyValue) : _that.taxonomyValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (taxonomySource != null ? taxonomySource.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (taxonomyValue != null ? taxonomyValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ProductTaxonomyBuilder {" +
				"taxonomySource=" + this.taxonomySource + ", " +
				"taxonomyValue=" + this.taxonomyValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("taxonomySource"), TaxonomySourceEnum.class, taxonomySource, this);
		processor.processBasic(path.newSubPath("taxonomyValue"), String.class, taxonomyValue, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ProductTaxonomy _that = (ProductTaxonomy) o;
	
		if (taxonomySource != null ? !taxonomySource.equals(_that.taxonomySource) : _that.taxonomySource != null) return false;
		if (taxonomyValue != null ? !taxonomyValue.equals(_that.taxonomyValue) : _that.taxonomyValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (taxonomySource != null ? taxonomySource.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (taxonomyValue != null ? taxonomyValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ProductTaxonomy {" +
			"taxonomySource=" + this.taxonomySource + ", " +
			"taxonomyValue=" + this.taxonomyValue +
		'}';
	}
}
