package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.Othr;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PrsnMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Prsn extends RosettaModelObject {
	private final String ctryOfBrnch;
	private final Othr othr;
	private static PrsnMeta metaData = new PrsnMeta();
	
	Prsn(PrsnBuilder builder) {
		this.ctryOfBrnch = builder.getCtryOfBrnch();
		this.othr = ofNullable(builder.getOthr()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="CtryOfBrnch", source="ISO20022")
	public final String getCtryOfBrnch() {
		return ctryOfBrnch;
	}
	
	@RosettaSynonym(value="Othr", source="ISO20022")
	public final Othr getOthr() {
		return othr;
	}
	
	
	@Override
	public RosettaMetaData<? extends Prsn> metaData() {
		return metaData;
	} 
	
	public PrsnBuilder toBuilder() {
		PrsnBuilder builder = new PrsnBuilder();
		ofNullable(getCtryOfBrnch()).ifPresent(builder::setCtryOfBrnch);
		ofNullable(getOthr()).ifPresent(builder::setOthr);
		return builder;
	}

	public static PrsnBuilder builder() {
		return new PrsnBuilder();
	}

	public static  class PrsnBuilder extends RosettaModelObjectBuilder {
	
		protected String ctryOfBrnch;
		protected Othr.OthrBuilder othr;
	
		public PrsnBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Prsn> metaData() {
			return metaData;
		} 
	
		
		public String getCtryOfBrnch() {
			return ctryOfBrnch;
		}
		
		
		public Othr.OthrBuilder getOthr() {
			return othr;
		}
		
		public Othr.OthrBuilder getOrCreateOthr() {
			if (othr!=null) {
				return othr;
			}
			else return othr = new Othr.OthrBuilder();
		}
		
	
		public PrsnBuilder setCtryOfBrnch(String ctryOfBrnch) {
			this.ctryOfBrnch = ctryOfBrnch;
			return this;
		}
		public PrsnBuilder setOthr(Othr othr) {
			this.othr = othr.toBuilder();
			return this;
		}
		public PrsnBuilder setOthrBuilder(Othr.OthrBuilder othr) {
			this.othr = othr;
			return this;
		}
		
		public Prsn build() {
			return new Prsn(this);
		}
	
		@Override
		public PrsnBuilder prune() {
			if (othr!=null && !othr.prune().hasData()) othr = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCtryOfBrnch()!=null) return true;
			if (getOthr()!=null && getOthr().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("ctryOfBrnch"), String.class, ctryOfBrnch, this);
			
			processRosetta(path.newSubPath("othr"), processor, Othr.class, othr);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PrsnBuilder _that = (PrsnBuilder) o;
		
			if (ctryOfBrnch != null ? !ctryOfBrnch.equals(_that.ctryOfBrnch) : _that.ctryOfBrnch != null) return false;
			if (othr != null ? !othr.equals(_that.othr) : _that.othr != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (ctryOfBrnch != null ? ctryOfBrnch.hashCode() : 0);
			_result = 31 * _result + (othr != null ? othr.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PrsnBuilder {" +
				"ctryOfBrnch=" + this.ctryOfBrnch + ", " +
				"othr=" + this.othr +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("ctryOfBrnch"), String.class, ctryOfBrnch, this);
		
		processRosetta(path.newSubPath("othr"), processor, Othr.class, othr);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Prsn _that = (Prsn) o;
	
		if (ctryOfBrnch != null ? !ctryOfBrnch.equals(_that.ctryOfBrnch) : _that.ctryOfBrnch != null) return false;
		if (othr != null ? !othr.equals(_that.othr) : _that.othr != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (ctryOfBrnch != null ? ctryOfBrnch.hashCode() : 0);
		_result = 31 * _result + (othr != null ? othr.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Prsn {" +
			"ctryOfBrnch=" + this.ctryOfBrnch + ", " +
			"othr=" + this.othr +
		'}';
	}
}
