package org.isda.cdm;

import java.lang.Boolean;
import java.lang.Integer;
import java.lang.String;
import java.util.List;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.PubliclyAvailableInformationMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="PubliclyAvailableInformation", source="FpML_5_10")
public class PubliclyAvailableInformation extends RosettaModelObject {
	private final List<String> publicSource;
	private final Integer specifiedNumber;
	private final Boolean standardPublicSources;
	private static PubliclyAvailableInformationMeta metaData = new PubliclyAvailableInformationMeta();
	
	PubliclyAvailableInformation(PubliclyAvailableInformationBuilder builder) {
		this.publicSource = builder.getPublicSource();
		this.specifiedNumber = builder.getSpecifiedNumber();
		this.standardPublicSources = builder.getStandardPublicSources();
	}
	
	/**
	 * A public information source, e.g. a particular newspaper or electronic news service, that may publish relevant information used in the determination of whether or not a credit event has occurred. ISDA 2003 Term: Public Source.
	 */
	@RosettaSynonym(value="publicSource", source="FpML_5_10")
	@RosettaSynonym(value="publicSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="publicSource", source="DTCC_11_0")
	@RosettaSynonym(value="publicSource", source="DTCC_9_0")
	@RosettaSynonym(value="publicSource", source="CME_ClearedConfirm_1_17")
	public final List<String> getPublicSource() {
		return publicSource;
	}
	
	/**
	 * The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two. ISDA 2003 Term: Specified Number.
	 */
	@RosettaSynonym(value="specifiedNumber", source="FpML_5_10")
	@RosettaSynonym(value="specifiedNumber", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="specifiedNumber", source="DTCC_11_0")
	@RosettaSynonym(value="specifiedNumber", source="DTCC_9_0")
	@RosettaSynonym(value="specifiedNumber", source="CME_ClearedConfirm_1_17")
	public final Integer getSpecifiedNumber() {
		return specifiedNumber;
	}
	
	/**
	 * If this element is specified and set to &#39;true&#39;, indicates that ISDA defined Standard Public Sources are applicable.
	 */
	@RosettaSynonym(value="standardPublicSources", source="FpML_5_10")
	@RosettaSynonym(value="standardPublicSources", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="standardPublicSources", source="DTCC_11_0")
	@RosettaSynonym(value="standardPublicSources", source="DTCC_9_0")
	@RosettaSynonym(value="standardPublicSources", source="CME_ClearedConfirm_1_17")
	public final Boolean getStandardPublicSources() {
		return standardPublicSources;
	}
	
	
	@Override
	public RosettaMetaData<? extends PubliclyAvailableInformation> metaData() {
		return metaData;
	} 
	
	public PubliclyAvailableInformationBuilder toBuilder() {
		PubliclyAvailableInformationBuilder builder = new PubliclyAvailableInformationBuilder();
		ofNullable(getPublicSource()).ifPresent(publicSource -> publicSource.forEach(builder::addPublicSource));
		ofNullable(getSpecifiedNumber()).ifPresent(builder::setSpecifiedNumber);
		ofNullable(getStandardPublicSources()).ifPresent(builder::setStandardPublicSources);
		return builder;
	}

	public static PubliclyAvailableInformationBuilder builder() {
		return new PubliclyAvailableInformationBuilder();
	}

	public static  class PubliclyAvailableInformationBuilder extends RosettaModelObjectBuilder {
	
		protected List<String> publicSource;
		protected Integer specifiedNumber;
		protected Boolean standardPublicSources;
	
		public PubliclyAvailableInformationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends PubliclyAvailableInformation> metaData() {
			return metaData;
		} 
	
		
		public List<String> getPublicSource() {
			return publicSource;
		}
		
		
		public Integer getSpecifiedNumber() {
			return specifiedNumber;
		}
		
		
		public Boolean getStandardPublicSources() {
			return standardPublicSources;
		}
		
	
		public PubliclyAvailableInformationBuilder addPublicSource(String publicSource) {
			if(this.publicSource == null){
				this.publicSource = new ArrayList<>();
			}
			this.publicSource.add(publicSource);
			return this;
		}
		public PubliclyAvailableInformationBuilder addPublicSource(String publicSource, int _idx) {
			if(this.publicSource == null){
				this.publicSource = new ArrayList<>();
			}
			getIndex(this.publicSource, _idx, () -> publicSource);
			this.publicSource.set(_idx, publicSource);
			return this;
		}
		public PubliclyAvailableInformationBuilder addPublicSource(List<String> publicSources) {
			if(this.publicSource == null){
				this.publicSource = new ArrayList<>();
			}
			for (String toAdd : publicSources) {
				this.publicSource.add(toAdd);
			}
			return this;
		}
		
		public PubliclyAvailableInformationBuilder clearPublicSource() {
			this.publicSource = null;
			return this;
		}
		public PubliclyAvailableInformationBuilder setSpecifiedNumber(Integer specifiedNumber) {
			this.specifiedNumber = specifiedNumber;
			return this;
		}
		public PubliclyAvailableInformationBuilder setStandardPublicSources(Boolean standardPublicSources) {
			this.standardPublicSources = standardPublicSources;
			return this;
		}
		
		public PubliclyAvailableInformation build() {
			return new PubliclyAvailableInformation(this);
		}
	
		@Override
		public PubliclyAvailableInformationBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPublicSource()!=null && !getPublicSource().isEmpty()) return true;
			if (getSpecifiedNumber()!=null) return true;
			if (getStandardPublicSources()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("publicSource"), String.class, publicSource, this);
			processor.processBasic(path.newSubPath("specifiedNumber"), Integer.class, specifiedNumber, this);
			processor.processBasic(path.newSubPath("standardPublicSources"), Boolean.class, standardPublicSources, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			PubliclyAvailableInformationBuilder _that = (PubliclyAvailableInformationBuilder) o;
		
			if (!ListEquals.listEquals(publicSource, _that.publicSource)) return false;
			if (specifiedNumber != null ? !specifiedNumber.equals(_that.specifiedNumber) : _that.specifiedNumber != null) return false;
			if (standardPublicSources != null ? !standardPublicSources.equals(_that.standardPublicSources) : _that.standardPublicSources != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (publicSource != null ? publicSource.hashCode() : 0);
			_result = 31 * _result + (specifiedNumber != null ? specifiedNumber.hashCode() : 0);
			_result = 31 * _result + (standardPublicSources != null ? standardPublicSources.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PubliclyAvailableInformationBuilder {" +
				"publicSource=" + this.publicSource + ", " +
				"specifiedNumber=" + this.specifiedNumber + ", " +
				"standardPublicSources=" + this.standardPublicSources +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		publicSource.stream().forEach(a->processor.processBasic(path.newSubPath("publicSource"), String.class, a, this));
		processor.processBasic(path.newSubPath("specifiedNumber"), Integer.class, specifiedNumber, this);
		processor.processBasic(path.newSubPath("standardPublicSources"), Boolean.class, standardPublicSources, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		PubliclyAvailableInformation _that = (PubliclyAvailableInformation) o;
	
		if (!ListEquals.listEquals(publicSource, _that.publicSource)) return false;
		if (specifiedNumber != null ? !specifiedNumber.equals(_that.specifiedNumber) : _that.specifiedNumber != null) return false;
		if (standardPublicSources != null ? !standardPublicSources.equals(_that.standardPublicSources) : _that.standardPublicSources != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (publicSource != null ? publicSource.hashCode() : 0);
		_result = 31 * _result + (specifiedNumber != null ? specifiedNumber.hashCode() : 0);
		_result = 31 * _result + (standardPublicSources != null ? standardPublicSources.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "PubliclyAvailableInformation {" +
			"publicSource=" + this.publicSource + ", " +
			"specifiedNumber=" + this.specifiedNumber + ", " +
			"standardPublicSources=" + this.standardPublicSources +
		'}';
	}
}
