package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QtyMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Qty extends RosettaModelObject {
	private final String unit;
	private static QtyMeta metaData = new QtyMeta();
	
	Qty(QtyBuilder builder) {
		this.unit = builder.getUnit();
	}
	
	@RosettaSynonym(value="Unit", source="ISO20022")
	public final String getUnit() {
		return unit;
	}
	
	
	@Override
	public RosettaMetaData<? extends Qty> metaData() {
		return metaData;
	} 
	
	public QtyBuilder toBuilder() {
		QtyBuilder builder = new QtyBuilder();
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		return builder;
	}

	public static QtyBuilder builder() {
		return new QtyBuilder();
	}

	public static  class QtyBuilder extends RosettaModelObjectBuilder {
	
		protected String unit;
	
		public QtyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Qty> metaData() {
			return metaData;
		} 
	
		
		public String getUnit() {
			return unit;
		}
		
	
		public QtyBuilder setUnit(String unit) {
			this.unit = unit;
			return this;
		}
		
		public Qty build() {
			return new Qty(this);
		}
	
		@Override
		public QtyBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUnit()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unit"), String.class, unit, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QtyBuilder _that = (QtyBuilder) o;
		
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (unit != null ? unit.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QtyBuilder {" +
				"unit=" + this.unit +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unit"), String.class, unit, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Qty _that = (Qty) o;
	
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (unit != null ? unit.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Qty {" +
			"unit=" + this.unit +
		'}';
	}
}
