package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.FxRate;
import org.isda.cdm.FxSpotRateSource;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.QuantoMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Determines the currency rate that the seller of the equity amounts will apply at each valuation date for converting the respective amounts into a currency that is different from the currency denomination of the underlier.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Quanto", source="FpML_5_10")
public class Quanto extends RosettaModelObject {
	private final List<FxRate> fxRate;
	private final FxSpotRateSource fxSpotRateSource;
	private static QuantoMeta metaData = new QuantoMeta();
	
	Quanto(QuantoBuilder builder) {
		this.fxRate = ofNullable(builder.getFxRate()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.fxSpotRateSource = ofNullable(builder.getFxSpotRateSource()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Specifies a currency conversion rate.
	 */
	@RosettaSynonym(value="fxRate", source="FpML_5_10")
	@RosettaSynonym(value="fxRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxRate", source="DTCC_11_0")
	@RosettaSynonym(value="fxRate", source="DTCC_9_0")
	@RosettaSynonym(value="fxRate", source="CME_ClearedConfirm_1_17")
	public final List<FxRate> getFxRate() {
		return fxRate;
	}
	
	/**
	 * Specifies the methodology (reference source and, optionally, fixing time) to be used for determining a currency conversion rate.
	 */
	@RosettaSynonym(value="fxSpotRateSource", source="FpML_5_10")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_11_0")
	@RosettaSynonym(value="fxSpotRateSource", source="DTCC_9_0")
	@RosettaSynonym(value="fxSpotRateSource", source="CME_ClearedConfirm_1_17")
	public final FxSpotRateSource getFxSpotRateSource() {
		return fxSpotRateSource;
	}
	
	
	@Override
	public RosettaMetaData<? extends Quanto> metaData() {
		return metaData;
	} 
	
	public QuantoBuilder toBuilder() {
		QuantoBuilder builder = new QuantoBuilder();
		ofNullable(getFxRate()).ifPresent(fxRate -> fxRate.forEach(builder::addFxRate));
		ofNullable(getFxSpotRateSource()).ifPresent(builder::setFxSpotRateSource);
		return builder;
	}

	public static QuantoBuilder builder() {
		return new QuantoBuilder();
	}

	public static  class QuantoBuilder extends RosettaModelObjectBuilder {
	
		protected List<FxRate.FxRateBuilder> fxRate;
		protected FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource;
	
		public QuantoBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Quanto> metaData() {
			return metaData;
		} 
	
		
		public List<FxRate.FxRateBuilder> getFxRate() {
			return fxRate;
		}
		
		public FxRate.FxRateBuilder getOrCreateFxRate(int index) {
			if (fxRate==null) {
				this.fxRate = new ArrayList<>();
			}
			return getIndex(fxRate, index, ()->new FxRate.FxRateBuilder());
		}
		
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getFxSpotRateSource() {
			return fxSpotRateSource;
		}
		
		public FxSpotRateSource.FxSpotRateSourceBuilder getOrCreateFxSpotRateSource() {
			if (fxSpotRateSource!=null) {
				return fxSpotRateSource;
			}
			else return fxSpotRateSource = new FxSpotRateSource.FxSpotRateSourceBuilder();
		}
		
	
		public QuantoBuilder addFxRate(FxRate fxRate) {
			if(this.fxRate == null){
				this.fxRate = new ArrayList<>();
			}
			this.fxRate.add(fxRate.toBuilder());
			return this;
		}
		public QuantoBuilder addFxRate(FxRate fxRate, int _idx) {
			if(this.fxRate == null){
				this.fxRate = new ArrayList<>();
			}
			getIndex(this.fxRate, _idx, () -> fxRate.toBuilder());
			this.fxRate.set(_idx, fxRate.toBuilder());
			return this;
		}
		public QuantoBuilder addFxRate(List<FxRate> fxRates) {
			if(this.fxRate == null){
				this.fxRate = new ArrayList<>();
			}
			for (FxRate toAdd : fxRates) {
				this.fxRate.add(toAdd.toBuilder());
			}
			return this;
		}
		public QuantoBuilder addFxRateBuilder(FxRate.FxRateBuilder fxRate) {
			if(this.fxRate == null){
				this.fxRate = new ArrayList<>();
			}
			this.fxRate.add(fxRate);
			return this;
		}
		
		public QuantoBuilder clearFxRate() {
			this.fxRate = null;
			return this;
		}
		public QuantoBuilder setFxSpotRateSource(FxSpotRateSource fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource.toBuilder();
			return this;
		}
		public QuantoBuilder setFxSpotRateSourceBuilder(FxSpotRateSource.FxSpotRateSourceBuilder fxSpotRateSource) {
			this.fxSpotRateSource = fxSpotRateSource;
			return this;
		}
		
		public Quanto build() {
			return new Quanto(this);
		}
	
		@Override
		public QuantoBuilder prune() {
			if (fxRate!=null) fxRate = fxRate.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (fxSpotRateSource!=null && !fxSpotRateSource.prune().hasData()) fxSpotRateSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFxRate()!=null && getFxRate().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFxSpotRateSource()!=null && getFxSpotRateSource().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("fxRate"), processor, FxRate.class, fxRate);
			processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			QuantoBuilder _that = (QuantoBuilder) o;
		
			if (!ListEquals.listEquals(fxRate, _that.fxRate)) return false;
			if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (fxRate != null ? fxRate.hashCode() : 0);
			_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "QuantoBuilder {" +
				"fxRate=" + this.fxRate + ", " +
				"fxSpotRateSource=" + this.fxSpotRateSource +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("fxRate"), processor, FxRate.class, fxRate);
		processRosetta(path.newSubPath("fxSpotRateSource"), processor, FxSpotRateSource.class, fxSpotRateSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Quanto _that = (Quanto) o;
	
		if (!ListEquals.listEquals(fxRate, _that.fxRate)) return false;
		if (fxSpotRateSource != null ? !fxSpotRateSource.equals(_that.fxSpotRateSource) : _that.fxSpotRateSource != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (fxRate != null ? fxRate.hashCode() : 0);
		_result = 31 * _result + (fxSpotRateSource != null ? fxSpotRateSource.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Quanto {" +
			"fxRate=" + this.fxRate + ", " +
			"fxSpotRateSource=" + this.fxSpotRateSource +
		'}';
	}
}
