package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of quotation rate to be obtained from each cash settlement reference bank.
 * 
 * @version 2.5.4
 */
public enum QuotationRateTypeEnum {
    

	/**
	 * A bid rate.
	 */
	@RosettaSynonym(value = "Bid", source = "FpML_5_10")
	@RosettaSynonym(value = "Bid", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Bid", source = "DTCC_11_0")
	@RosettaSynonym(value = "Bid", source = "DTCC_9_0")
	@RosettaSynonym(value = "Bid", source = "CME_ClearedConfirm_1_17")
	BID,
	    

	/**
	 * An ask rate.
	 */
	@RosettaSynonym(value = "Ask", source = "FpML_5_10")
	@RosettaSynonym(value = "Ask", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Ask", source = "DTCC_11_0")
	@RosettaSynonym(value = "Ask", source = "DTCC_9_0")
	@RosettaSynonym(value = "Ask", source = "CME_ClearedConfirm_1_17")
	ASK,
	    

	/**
	 * A mid-market rate.
	 */
	@RosettaSynonym(value = "Mid", source = "FpML_5_10")
	@RosettaSynonym(value = "Mid", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Mid", source = "DTCC_11_0")
	@RosettaSynonym(value = "Mid", source = "DTCC_9_0")
	@RosettaSynonym(value = "Mid", source = "CME_ClearedConfirm_1_17")
	MID,
	    

	/**
	 * If optional early termination is applicable to a swap transaction, the rate, which may be a bid or ask rate, which would result, if seller is in-the-money, in the higher absolute value of the cash settlement amount, or, is seller is out-of-the-money, in the lower absolute value of the cash settlement amount.
	 */
	@RosettaSynonym(value = "ExercisingPartyPays", source = "FpML_5_10")
	@RosettaSynonym(value = "ExercisingPartyPays", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ExercisingPartyPays", source = "DTCC_11_0")
	@RosettaSynonym(value = "ExercisingPartyPays", source = "DTCC_9_0")
	@RosettaSynonym(value = "ExercisingPartyPays", source = "CME_ClearedConfirm_1_17")
	EXERCISING_PARTY_PAYS    
	;

	private final String displayName;
	
	QuotationRateTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
