package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the side from which perspective a value is quoted.
 * 
 * @version 2.5.4
 */
public enum QuotationSideEnum {
    

	/**
	 * A value &#39;bid&#39; by a buyer for an asset, i.e. the value a buyer is willing to pay.
	 */
	@RosettaSynonym(value = "Bid", source = "FpML_5_10")
	@RosettaSynonym(value = "Bid", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Bid", source = "DTCC_11_0")
	@RosettaSynonym(value = "Bid", source = "DTCC_9_0")
	@RosettaSynonym(value = "Bid", source = "CME_ClearedConfirm_1_17")
	BID,
	    

	/**
	 * A value &#39;asked&#39; by a seller for an asset, i.e. the value at which a seller is willing to sell.
	 */
	@RosettaSynonym(value = "Ask", source = "FpML_5_10")
	@RosettaSynonym(value = "Ask", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Ask", source = "DTCC_11_0")
	@RosettaSynonym(value = "Ask", source = "DTCC_9_0")
	@RosettaSynonym(value = "Ask", source = "CME_ClearedConfirm_1_17")
	ASK,
	    

	/**
	 * A value midway between the bid and the ask value.
	 */
	@RosettaSynonym(value = "Mid", source = "FpML_5_10")
	@RosettaSynonym(value = "Mid", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Mid", source = "DTCC_11_0")
	@RosettaSynonym(value = "Mid", source = "DTCC_9_0")
	@RosettaSynonym(value = "Mid", source = "CME_ClearedConfirm_1_17")
	MID    
	;

	private final String displayName;
	
	QuotationSideEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
