package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the actual quotation style (e.g. PointsUpFront, TradedSpread) used to quote a credit default swap fee leg.
 * 
 * @version 2.5.4
 */
public enum QuotationStyleEnum {
    

	/**
	 * When quotation style is &#39;PointsUpFront&#39;, the initialPoints element of the Credit Default Swap feeLeg should be populated
	 */
	@RosettaSynonym(value = "PointsUpFront", source = "FpML_5_10")
	@RosettaSynonym(value = "PointsUpFront", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PointsUpFront", source = "DTCC_11_0")
	@RosettaSynonym(value = "PointsUpFront", source = "DTCC_9_0")
	@RosettaSynonym(value = "PointsUpFront", source = "CME_ClearedConfirm_1_17")
	POINTS_UP_FRONT,
	    

	/**
	 * When quotation style is &#39;TradedSpread&#39;, the marketFixedRate element of the Credit Default Swap feeLeg should be populated
	 */
	@RosettaSynonym(value = "TradedSpread", source = "FpML_5_10")
	@RosettaSynonym(value = "TradedSpread", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TradedSpread", source = "DTCC_11_0")
	@RosettaSynonym(value = "TradedSpread", source = "DTCC_9_0")
	@RosettaSynonym(value = "TradedSpread", source = "CME_ClearedConfirm_1_17")
	TRADED_SPREAD,
	    

	/**
	 * When quotation style is &#39;Price&#39;, the marketPrice element of the Credit Default Swap feeLeg should be populated
	 */
	@RosettaSynonym(value = "Price", source = "FpML_5_10")
	@RosettaSynonym(value = "Price", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Price", source = "DTCC_11_0")
	@RosettaSynonym(value = "Price", source = "DTCC_9_0")
	@RosettaSynonym(value = "Price", source = "CME_ClearedConfirm_1_17")
	PRICE    
	;

	private final String displayName;
	
	QuotationStyleEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
