package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify how an exchange rate is quoted.
 * 
 * @version 2.5.4
 */
public enum QuoteBasisEnum {
    

	/**
	 * The amount of currency1 for one unit of currency2
	 */
	@RosettaSynonym(value = "Currency1PerCurrency2", source = "FpML_5_10")
	@RosettaSynonym(value = "Currency1PerCurrency2", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Currency1PerCurrency2", source = "DTCC_11_0")
	@RosettaSynonym(value = "Currency1PerCurrency2", source = "DTCC_9_0")
	@RosettaSynonym(value = "Currency1PerCurrency2", source = "CME_ClearedConfirm_1_17")
	CURRENCY_1_PER_CURRENCY_2,
	    

	/**
	 * The amount of currency2 for one unit of currency1
	 */
	@RosettaSynonym(value = "Currency2PerCurrency1", source = "FpML_5_10")
	@RosettaSynonym(value = "Currency2PerCurrency1", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Currency2PerCurrency1", source = "DTCC_11_0")
	@RosettaSynonym(value = "Currency2PerCurrency1", source = "DTCC_9_0")
	@RosettaSynonym(value = "Currency2PerCurrency1", source = "CME_ClearedConfirm_1_17")
	CURRENCY_2_PER_CURRENCY_1    
	;

	private final String displayName;
	
	QuoteBasisEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
