package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.lang.Integer;
import java.math.BigDecimal;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaRateObservation;
import org.isda.cdm.metafields.ReferenceWithMetaRateObservation.ReferenceWithMetaRateObservationBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RateObservationMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * A class defining parameters associated with an individual observation or fixing. This class forms part of the cashflow representation of a stream.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="RateObservation", source="FpML_5_10")
@RosettaSynonym(value="RateObservation", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="RateObservation", source="DTCC_11_0")
@RosettaSynonym(value="RateObservation", source="DTCC_9_0")
@RosettaSynonym(value="RateObservation", source="CME_ClearedConfirm_1_17")
public class RateObservation extends RosettaModelObject implements GlobalKey {
	private final Date adjustedFixingDate;
	private final BigDecimal forecastRate;
	private final MetaFields meta;
	private final Integer observationWeight;
	private final BigDecimal observedRate;
	private final ReferenceWithMetaRateObservation rateReference;
	private final Date resetDate;
	private final BigDecimal treatedForecastRate;
	private final BigDecimal treatedRate;
	private static RateObservationMeta metaData = new RateObservationMeta();
	
	RateObservation(RateObservationBuilder builder) {
		this.adjustedFixingDate = builder.getAdjustedFixingDate();
		this.forecastRate = builder.getForecastRate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.observationWeight = builder.getObservationWeight();
		this.observedRate = builder.getObservedRate();
		this.rateReference = ofNullable(builder.getRateReference()).map(f->f.build()).orElse(null);
		this.resetDate = builder.getResetDate();
		this.treatedForecastRate = builder.getTreatedForecastRate();
		this.treatedRate = builder.getTreatedRate();
	}
	
	/**
	 * The adjusted fixing date, i.e. the actual date the rate is observed. The date should already be adjusted for any applicable business day convention.
	 */
	@RosettaSynonym(value="adjustedFixingDate", source="FpML_5_10")
	@RosettaSynonym(value="adjustedFixingDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="adjustedFixingDate", source="DTCC_11_0")
	@RosettaSynonym(value="adjustedFixingDate", source="DTCC_9_0")
	@RosettaSynonym(value="adjustedFixingDate", source="CME_ClearedConfirm_1_17")
	public final Date getAdjustedFixingDate() {
		return adjustedFixingDate;
	}
	
	/**
	 * The value representing the forecast rate used to calculate the forecast future value of the accrual period.A value of 1% should be represented as 0.01.
	 */
	@RosettaSynonym(value="forecastRate", source="FpML_5_10")
	@RosettaSynonym(value="forecastRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="forecastRate", source="DTCC_11_0")
	@RosettaSynonym(value="forecastRate", source="DTCC_9_0")
	@RosettaSynonym(value="forecastRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getForecastRate() {
		return forecastRate;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The number of days weighting to be associated with the rate observation, i.e. the number of days such rate is in effect. This is applicable in the case of a weighted average method of calculation where more than one reset date is established for a single calculation period.
	 */
	@RosettaSynonym(value="observationWeight", source="FpML_5_10")
	@RosettaSynonym(value="observationWeight", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="observationWeight", source="DTCC_11_0")
	@RosettaSynonym(value="observationWeight", source="DTCC_9_0")
	@RosettaSynonym(value="observationWeight", source="CME_ClearedConfirm_1_17")
	public final Integer getObservationWeight() {
		return observationWeight;
	}
	
	/**
	 * The actual observed rate before any required rate treatment is applied, e.g. before converting a rate quoted on a discount basis to an equivalent yield. An observed rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="observedRate", source="FpML_5_10")
	@RosettaSynonym(value="observedRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="observedRate", source="DTCC_11_0")
	@RosettaSynonym(value="observedRate", source="DTCC_9_0")
	@RosettaSynonym(value="observedRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getObservedRate() {
		return observedRate;
	}
	
	/**
	 * A pointer style reference to a floating rate component defined as part of a stub calculation period amount component. It is only required when it is necessary to distinguish two rate observations for the same fixing date which could occur when linear interpolation of two different rates occurs for a stub calculation period.
	 */
	@RosettaSynonym(value="rateReference", source="FpML_5_10")
	@RosettaSynonym(value="rateReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateReference", source="DTCC_11_0")
	@RosettaSynonym(value="rateReference", source="DTCC_9_0")
	@RosettaSynonym(value="rateReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaRateObservation getRateReference() {
		return rateReference;
	}
	
	/**
	 * The reset date.
	 */
	@RosettaSynonym(value="resetDate", source="FpML_5_10")
	@RosettaSynonym(value="resetDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="resetDate", source="DTCC_11_0")
	@RosettaSynonym(value="resetDate", source="DTCC_9_0")
	@RosettaSynonym(value="resetDate", source="CME_ClearedConfirm_1_17")
	public final Date getResetDate() {
		return resetDate;
	}
	
	/**
	 * The value representing the forecast rate after applying rate treatment rules. A value of 1% should be represented as 0.01.
	 */
	@RosettaSynonym(value="treatedForecastRate", source="FpML_5_10")
	@RosettaSynonym(value="treatedForecastRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="treatedForecastRate", source="DTCC_11_0")
	@RosettaSynonym(value="treatedForecastRate", source="DTCC_9_0")
	@RosettaSynonym(value="treatedForecastRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getTreatedForecastRate() {
		return treatedForecastRate;
	}
	
	/**
	 * The observed rate after any required rate treatment is applied. A treated rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="treatedRate", source="FpML_5_10")
	@RosettaSynonym(value="treatedRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="treatedRate", source="DTCC_11_0")
	@RosettaSynonym(value="treatedRate", source="DTCC_9_0")
	@RosettaSynonym(value="treatedRate", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getTreatedRate() {
		return treatedRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends RateObservation> metaData() {
		return metaData;
	} 
	
	public RateObservationBuilder toBuilder() {
		RateObservationBuilder builder = new RateObservationBuilder();
		ofNullable(getAdjustedFixingDate()).ifPresent(builder::setAdjustedFixingDate);
		ofNullable(getForecastRate()).ifPresent(builder::setForecastRate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getObservationWeight()).ifPresent(builder::setObservationWeight);
		ofNullable(getObservedRate()).ifPresent(builder::setObservedRate);
		ofNullable(getRateReference()).ifPresent(builder::setRateReference);
		ofNullable(getResetDate()).ifPresent(builder::setResetDate);
		ofNullable(getTreatedForecastRate()).ifPresent(builder::setTreatedForecastRate);
		ofNullable(getTreatedRate()).ifPresent(builder::setTreatedRate);
		return builder;
	}

	public static RateObservationBuilder builder() {
		return new RateObservationBuilder();
	}

	public static  class RateObservationBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<RateObservationBuilder> {
	
		protected Date adjustedFixingDate;
		protected BigDecimal forecastRate;
		protected MetaFields.MetaFieldsBuilder meta;
		protected Integer observationWeight;
		protected BigDecimal observedRate;
		protected ReferenceWithMetaRateObservationBuilder rateReference;
		protected Date resetDate;
		protected BigDecimal treatedForecastRate;
		protected BigDecimal treatedRate;
	
		public RateObservationBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RateObservation> metaData() {
			return metaData;
		} 
	
		
		public Date getAdjustedFixingDate() {
			return adjustedFixingDate;
		}
		
		
		public BigDecimal getForecastRate() {
			return forecastRate;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Integer getObservationWeight() {
			return observationWeight;
		}
		
		
		public BigDecimal getObservedRate() {
			return observedRate;
		}
		
		
		public ReferenceWithMetaRateObservationBuilder getRateReference() {
			return rateReference;
		}
		
		public ReferenceWithMetaRateObservationBuilder getOrCreateRateReference() {
			if (rateReference!=null) {
				return rateReference;
			}
			else return rateReference = new ReferenceWithMetaRateObservationBuilder();
		}
		
		
		public Date getResetDate() {
			return resetDate;
		}
		
		
		public BigDecimal getTreatedForecastRate() {
			return treatedForecastRate;
		}
		
		
		public BigDecimal getTreatedRate() {
			return treatedRate;
		}
		
	
		public RateObservationBuilder setAdjustedFixingDate(Date adjustedFixingDate) {
			this.adjustedFixingDate = adjustedFixingDate;
			return this;
		}
		public RateObservationBuilder setForecastRate(BigDecimal forecastRate) {
			this.forecastRate = forecastRate;
			return this;
		}
		public RateObservationBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public RateObservationBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public RateObservationBuilder setObservationWeight(Integer observationWeight) {
			this.observationWeight = observationWeight;
			return this;
		}
		public RateObservationBuilder setObservedRate(BigDecimal observedRate) {
			this.observedRate = observedRate;
			return this;
		}
		public RateObservationBuilder setRateReference(ReferenceWithMetaRateObservation rateReference) {
			this.rateReference = rateReference.toBuilder();
			return this;
		}
		public RateObservationBuilder setRateReferenceBuilder(ReferenceWithMetaRateObservationBuilder rateReference) {
			this.rateReference = rateReference;
			return this;
		}
		public RateObservationBuilder setRateReferenceRef(RateObservation.RateObservationBuilder rateReference) {
			return setRateReference(ReferenceWithMetaRateObservation.builder().setValueBuilder(rateReference).build());
		}
		public RateObservationBuilder setRateReferenceRef(RateObservation rateReference) {
			return setRateReferenceRef(RateObservation.builder());
		}
		public RateObservationBuilder setResetDate(Date resetDate) {
			this.resetDate = resetDate;
			return this;
		}
		public RateObservationBuilder setTreatedForecastRate(BigDecimal treatedForecastRate) {
			this.treatedForecastRate = treatedForecastRate;
			return this;
		}
		public RateObservationBuilder setTreatedRate(BigDecimal treatedRate) {
			this.treatedRate = treatedRate;
			return this;
		}
		
		public RateObservation build() {
			return new RateObservation(this);
		}
	
		@Override
		public RateObservationBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (rateReference!=null && !rateReference.prune().hasData()) rateReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdjustedFixingDate()!=null) return true;
			if (getForecastRate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getObservationWeight()!=null) return true;
			if (getObservedRate()!=null) return true;
			if (getRateReference()!=null && getRateReference().hasData()) return true;
			if (getResetDate()!=null) return true;
			if (getTreatedForecastRate()!=null) return true;
			if (getTreatedRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("adjustedFixingDate"), Date.class, adjustedFixingDate, this);
			processor.processBasic(path.newSubPath("forecastRate"), BigDecimal.class, forecastRate, this);
			processor.processBasic(path.newSubPath("observationWeight"), Integer.class, observationWeight, this);
			processor.processBasic(path.newSubPath("observedRate"), BigDecimal.class, observedRate, this);
			processor.processBasic(path.newSubPath("resetDate"), Date.class, resetDate, this);
			processor.processBasic(path.newSubPath("treatedForecastRate"), BigDecimal.class, treatedForecastRate, this);
			processor.processBasic(path.newSubPath("treatedRate"), BigDecimal.class, treatedRate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("rateReference"), processor, ReferenceWithMetaRateObservation.class, rateReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RateObservationBuilder _that = (RateObservationBuilder) o;
		
			if (adjustedFixingDate != null ? !adjustedFixingDate.equals(_that.adjustedFixingDate) : _that.adjustedFixingDate != null) return false;
			if (forecastRate != null ? !forecastRate.equals(_that.forecastRate) : _that.forecastRate != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (observationWeight != null ? !observationWeight.equals(_that.observationWeight) : _that.observationWeight != null) return false;
			if (observedRate != null ? !observedRate.equals(_that.observedRate) : _that.observedRate != null) return false;
			if (rateReference != null ? !rateReference.equals(_that.rateReference) : _that.rateReference != null) return false;
			if (resetDate != null ? !resetDate.equals(_that.resetDate) : _that.resetDate != null) return false;
			if (treatedForecastRate != null ? !treatedForecastRate.equals(_that.treatedForecastRate) : _that.treatedForecastRate != null) return false;
			if (treatedRate != null ? !treatedRate.equals(_that.treatedRate) : _that.treatedRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (adjustedFixingDate != null ? adjustedFixingDate.hashCode() : 0);
			_result = 31 * _result + (forecastRate != null ? forecastRate.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (observationWeight != null ? observationWeight.hashCode() : 0);
			_result = 31 * _result + (observedRate != null ? observedRate.hashCode() : 0);
			_result = 31 * _result + (rateReference != null ? rateReference.hashCode() : 0);
			_result = 31 * _result + (resetDate != null ? resetDate.hashCode() : 0);
			_result = 31 * _result + (treatedForecastRate != null ? treatedForecastRate.hashCode() : 0);
			_result = 31 * _result + (treatedRate != null ? treatedRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RateObservationBuilder {" +
				"adjustedFixingDate=" + this.adjustedFixingDate + ", " +
				"forecastRate=" + this.forecastRate + ", " +
				"meta=" + this.meta + ", " +
				"observationWeight=" + this.observationWeight + ", " +
				"observedRate=" + this.observedRate + ", " +
				"rateReference=" + this.rateReference + ", " +
				"resetDate=" + this.resetDate + ", " +
				"treatedForecastRate=" + this.treatedForecastRate + ", " +
				"treatedRate=" + this.treatedRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("adjustedFixingDate"), Date.class, adjustedFixingDate, this);
		processor.processBasic(path.newSubPath("forecastRate"), BigDecimal.class, forecastRate, this);
		processor.processBasic(path.newSubPath("observationWeight"), Integer.class, observationWeight, this);
		processor.processBasic(path.newSubPath("observedRate"), BigDecimal.class, observedRate, this);
		processor.processBasic(path.newSubPath("resetDate"), Date.class, resetDate, this);
		processor.processBasic(path.newSubPath("treatedForecastRate"), BigDecimal.class, treatedForecastRate, this);
		processor.processBasic(path.newSubPath("treatedRate"), BigDecimal.class, treatedRate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("rateReference"), processor, ReferenceWithMetaRateObservation.class, rateReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RateObservation _that = (RateObservation) o;
	
		if (adjustedFixingDate != null ? !adjustedFixingDate.equals(_that.adjustedFixingDate) : _that.adjustedFixingDate != null) return false;
		if (forecastRate != null ? !forecastRate.equals(_that.forecastRate) : _that.forecastRate != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (observationWeight != null ? !observationWeight.equals(_that.observationWeight) : _that.observationWeight != null) return false;
		if (observedRate != null ? !observedRate.equals(_that.observedRate) : _that.observedRate != null) return false;
		if (rateReference != null ? !rateReference.equals(_that.rateReference) : _that.rateReference != null) return false;
		if (resetDate != null ? !resetDate.equals(_that.resetDate) : _that.resetDate != null) return false;
		if (treatedForecastRate != null ? !treatedForecastRate.equals(_that.treatedForecastRate) : _that.treatedForecastRate != null) return false;
		if (treatedRate != null ? !treatedRate.equals(_that.treatedRate) : _that.treatedRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (adjustedFixingDate != null ? adjustedFixingDate.hashCode() : 0);
		_result = 31 * _result + (forecastRate != null ? forecastRate.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (observationWeight != null ? observationWeight.hashCode() : 0);
		_result = 31 * _result + (observedRate != null ? observedRate.hashCode() : 0);
		_result = 31 * _result + (rateReference != null ? rateReference.hashCode() : 0);
		_result = 31 * _result + (resetDate != null ? resetDate.hashCode() : 0);
		_result = 31 * _result + (treatedForecastRate != null ? treatedForecastRate.hashCode() : 0);
		_result = 31 * _result + (treatedRate != null ? treatedRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RateObservation {" +
			"adjustedFixingDate=" + this.adjustedFixingDate + ", " +
			"forecastRate=" + this.forecastRate + ", " +
			"meta=" + this.meta + ", " +
			"observationWeight=" + this.observationWeight + ", " +
			"observedRate=" + this.observedRate + ", " +
			"rateReference=" + this.rateReference + ", " +
			"resetDate=" + this.resetDate + ", " +
			"treatedForecastRate=" + this.treatedForecastRate + ", " +
			"treatedRate=" + this.treatedRate +
		'}';
	}
}
