package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the methods for converting rates from one basis to another.
 * 
 * @version 2.5.4
 */
public enum RateTreatmentEnum {
    

	/**
	 * Bond Equivalent Yield. Per Annex to the 2000 ISDA Definitions (June 2000 Version), Section 7.3. Certain General Definitions Relating to Floating Rate Options, paragraph (g).
	 */
	@RosettaSynonym(value = "BondEquivalentYield", source = "FpML_5_10")
	@RosettaSynonym(value = "BondEquivalentYield", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BondEquivalentYield", source = "DTCC_11_0")
	@RosettaSynonym(value = "BondEquivalentYield", source = "DTCC_9_0")
	@RosettaSynonym(value = "BondEquivalentYield", source = "CME_ClearedConfirm_1_17")
	BOND_EQUIVALENT_YIELD,
	    

	/**
	 * Money Market Yield. Per Annex to the 2000 ISDA Definitions (June 2000 Version), Section 7.3. Certain General Definitions Relating to Floating Rate Options, paragraph (h).
	 */
	@RosettaSynonym(value = "MoneyMarketYield", source = "FpML_5_10")
	@RosettaSynonym(value = "MoneyMarketYield", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MoneyMarketYield", source = "DTCC_11_0")
	@RosettaSynonym(value = "MoneyMarketYield", source = "DTCC_9_0")
	@RosettaSynonym(value = "MoneyMarketYield", source = "CME_ClearedConfirm_1_17")
	MONEY_MARKET_YIELD    
	;

	private final String displayName;
	
	RateTreatmentEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
