package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RefRateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class RefRate extends RosettaModelObject {
	private final String indx;
	private final String nm;
	private static RefRateMeta metaData = new RefRateMeta();
	
	RefRate(RefRateBuilder builder) {
		this.indx = builder.getIndx();
		this.nm = builder.getNm();
	}
	
	@RosettaSynonym(value="Indx", source="ISO20022")
	public final String getIndx() {
		return indx;
	}
	
	@RosettaSynonym(value="Nm", source="ISO20022")
	public final String getNm() {
		return nm;
	}
	
	
	@Override
	public RosettaMetaData<? extends RefRate> metaData() {
		return metaData;
	} 
	
	public RefRateBuilder toBuilder() {
		RefRateBuilder builder = new RefRateBuilder();
		ofNullable(getIndx()).ifPresent(builder::setIndx);
		ofNullable(getNm()).ifPresent(builder::setNm);
		return builder;
	}

	public static RefRateBuilder builder() {
		return new RefRateBuilder();
	}

	public static  class RefRateBuilder extends RosettaModelObjectBuilder {
	
		protected String indx;
		protected String nm;
	
		public RefRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RefRate> metaData() {
			return metaData;
		} 
	
		
		public String getIndx() {
			return indx;
		}
		
		
		public String getNm() {
			return nm;
		}
		
	
		public RefRateBuilder setIndx(String indx) {
			this.indx = indx;
			return this;
		}
		public RefRateBuilder setNm(String nm) {
			this.nm = nm;
			return this;
		}
		
		public RefRate build() {
			return new RefRate(this);
		}
	
		@Override
		public RefRateBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIndx()!=null) return true;
			if (getNm()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("indx"), String.class, indx, this);
			processor.processBasic(path.newSubPath("nm"), String.class, nm, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RefRateBuilder _that = (RefRateBuilder) o;
		
			if (indx != null ? !indx.equals(_that.indx) : _that.indx != null) return false;
			if (nm != null ? !nm.equals(_that.nm) : _that.nm != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
			_result = 31 * _result + (nm != null ? nm.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RefRateBuilder {" +
				"indx=" + this.indx + ", " +
				"nm=" + this.nm +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("indx"), String.class, indx, this);
		processor.processBasic(path.newSubPath("nm"), String.class, nm, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RefRate _that = (RefRate) o;
	
		if (indx != null ? !indx.equals(_that.indx) : _that.indx != null) return false;
		if (nm != null ? !nm.equals(_that.nm) : _that.nm != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
		_result = 31 * _result + (nm != null ? nm.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RefRate {" +
			"indx=" + this.indx + ", " +
			"nm=" + this.nm +
		'}';
	}
}
