package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferenceBankMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to describe an institution (party) identified by means of a coding scheme and an optional name.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferenceBank", source="FpML_5_10")
public class ReferenceBank extends RosettaModelObject {
	private final FieldWithMetaString referenceBankId;
	private final String referenceBankName;
	private static ReferenceBankMeta metaData = new ReferenceBankMeta();
	
	ReferenceBank(ReferenceBankBuilder builder) {
		this.referenceBankId = ofNullable(builder.getReferenceBankId()).map(f->f.build()).orElse(null);
		this.referenceBankName = builder.getReferenceBankName();
	}
	
	/**
	 * An institution (party) identifier, e.g. a bank identifier code (BIC). FpML specifies a referenceBankIdScheme.
	 */
	@RosettaSynonym(value="referenceBankId", source="FpML_5_10")
	@RosettaSynonym(value="referenceBankId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceBankId", source="DTCC_11_0")
	@RosettaSynonym(value="referenceBankId", source="DTCC_9_0")
	@RosettaSynonym(value="referenceBankId", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getReferenceBankId() {
		return referenceBankId;
	}
	
	/**
	 * The name of the institution (party). A free format string. FpML does not define usage rules for the element.
	 */
	@RosettaSynonym(value="referenceBankName", source="FpML_5_10")
	@RosettaSynonym(value="referenceBankName", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceBankName", source="DTCC_11_0")
	@RosettaSynonym(value="referenceBankName", source="DTCC_9_0")
	@RosettaSynonym(value="referenceBankName", source="CME_ClearedConfirm_1_17")
	public final String getReferenceBankName() {
		return referenceBankName;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferenceBank> metaData() {
		return metaData;
	} 
	
	public ReferenceBankBuilder toBuilder() {
		ReferenceBankBuilder builder = new ReferenceBankBuilder();
		ofNullable(getReferenceBankId()).ifPresent(builder::setReferenceBankId);
		ofNullable(getReferenceBankName()).ifPresent(builder::setReferenceBankName);
		return builder;
	}

	public static ReferenceBankBuilder builder() {
		return new ReferenceBankBuilder();
	}

	public static  class ReferenceBankBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder referenceBankId;
		protected String referenceBankName;
	
		public ReferenceBankBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferenceBank> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getReferenceBankId() {
			return referenceBankId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateReferenceBankId() {
			if (referenceBankId!=null) {
				return referenceBankId;
			}
			else return referenceBankId = new FieldWithMetaStringBuilder();
		}
		
		
		public String getReferenceBankName() {
			return referenceBankName;
		}
		
	
		public ReferenceBankBuilder setReferenceBankId(FieldWithMetaString referenceBankId) {
			this.referenceBankId = referenceBankId.toBuilder();
			return this;
		}
		public ReferenceBankBuilder setReferenceBankIdRef(String referenceBankId) {
			return setReferenceBankId(FieldWithMetaString.builder().setValue(referenceBankId).build());
		}
		public ReferenceBankBuilder setReferenceBankName(String referenceBankName) {
			this.referenceBankName = referenceBankName;
			return this;
		}
		
		public ReferenceBank build() {
			return new ReferenceBank(this);
		}
	
		@Override
		public ReferenceBankBuilder prune() {
			if (referenceBankId!=null && !referenceBankId.prune().hasData()) referenceBankId = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getReferenceBankId()!=null) return true;
			if (getReferenceBankName()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("referenceBankName"), String.class, referenceBankName, this);
			
			processRosetta(path.newSubPath("referenceBankId"), processor, FieldWithMetaString.class, referenceBankId);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferenceBankBuilder _that = (ReferenceBankBuilder) o;
		
			if (referenceBankId != null ? !referenceBankId.equals(_that.referenceBankId) : _that.referenceBankId != null) return false;
			if (referenceBankName != null ? !referenceBankName.equals(_that.referenceBankName) : _that.referenceBankName != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (referenceBankId != null ? referenceBankId.hashCode() : 0);
			_result = 31 * _result + (referenceBankName != null ? referenceBankName.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferenceBankBuilder {" +
				"referenceBankId=" + this.referenceBankId + ", " +
				"referenceBankName=" + this.referenceBankName +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("referenceBankName"), String.class, referenceBankName, this);
		
		processRosetta(path.newSubPath("referenceBankId"), processor, FieldWithMetaString.class, referenceBankId);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferenceBank _that = (ReferenceBank) o;
	
		if (referenceBankId != null ? !referenceBankId.equals(_that.referenceBankId) : _that.referenceBankId != null) return false;
		if (referenceBankName != null ? !referenceBankName.equals(_that.referenceBankName) : _that.referenceBankName != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (referenceBankId != null ? referenceBankId.hashCode() : 0);
		_result = 31 * _result + (referenceBankName != null ? referenceBankName.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferenceBank {" +
			"referenceBankId=" + this.referenceBankId + ", " +
			"referenceBankName=" + this.referenceBankName +
		'}';
	}
}
