package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.ReferenceObligation;
import org.isda.cdm.metafields.FieldWithMetaEntityTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaEntityTypeEnum.FieldWithMetaEntityTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferencePairMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferencePair", source="FpML_5_10")
public class ReferencePair extends RosettaModelObject {
	private final FieldWithMetaEntityTypeEnum entityType;
	private final Boolean noReferenceObligation;
	private final LegalEntity referenceEntity;
	private final ReferenceObligation referenceObligation;
	private static ReferencePairMeta metaData = new ReferencePairMeta();
	
	ReferencePair(ReferencePairBuilder builder) {
		this.entityType = ofNullable(builder.getEntityType()).map(f->f.build()).orElse(null);
		this.noReferenceObligation = builder.getNoReferenceObligation();
		this.referenceEntity = ofNullable(builder.getReferenceEntity()).map(f->f.build()).orElse(null);
		this.referenceObligation = ofNullable(builder.getReferenceObligation()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Defines the reference entity types corresponding to a list of types in the ISDA First to Default documentation.
	 */
	@RosettaSynonym(value="entityType", source="FpML_5_10")
	@RosettaSynonym(value="entityType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="entityType", source="DTCC_11_0")
	@RosettaSynonym(value="entityType", source="DTCC_9_0")
	@RosettaSynonym(value="entityType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaEntityTypeEnum getEntityType() {
		return entityType;
	}
	
	/**
	 * Used to indicate that there is no Reference Obligation associated with this Credit Default Swap and that there will never be one.
	 */
	@RosettaSynonym(value="noReferenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="noReferenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="noReferenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="noReferenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="noReferenceObligation", source="CME_ClearedConfirm_1_17")
	public final Boolean getNoReferenceObligation() {
		return noReferenceObligation;
	}
	
	/**
	 * The corporate or sovereign entity on which you are buying or selling protection and any successor that assumes all or substantially all of its contractual and other obligations. It is vital to use the correct legal name of the entity and to be careful not to choose a subsidiary if you really want to trade protection on a parent company. Please note, Reference Entities cannot be senior or subordinated. It is the obligations of the Reference Entities that can be senior or subordinated. ISDA 2003 Term: Reference Entity.
	 */
	public final LegalEntity getReferenceEntity() {
		return referenceEntity;
	}
	
	/**
	 * The Reference Obligation is a financial instrument that is either issued or guaranteed by the reference entity. It serves to clarify the precise reference entity protection is being offered upon, and its legal position with regard to other related firms (parents/subsidiaries). Furthermore the Reference Obligation is ALWAYS deliverable and establishes the Pari Passu ranking (as the deliverable bonds must rank equal to the reference obligation). ISDA 2003 Term: Reference Obligation.
	 */
	@RosettaSynonym(value="referenceObligation", source="FpML_5_10")
	@RosettaSynonym(value="referenceObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referenceObligation", source="DTCC_11_0")
	@RosettaSynonym(value="referenceObligation", source="DTCC_9_0")
	@RosettaSynonym(value="referenceObligation", source="CME_ClearedConfirm_1_17")
	public final ReferenceObligation getReferenceObligation() {
		return referenceObligation;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferencePair> metaData() {
		return metaData;
	} 
	
	public ReferencePairBuilder toBuilder() {
		ReferencePairBuilder builder = new ReferencePairBuilder();
		ofNullable(getEntityType()).ifPresent(builder::setEntityType);
		ofNullable(getNoReferenceObligation()).ifPresent(builder::setNoReferenceObligation);
		ofNullable(getReferenceEntity()).ifPresent(builder::setReferenceEntity);
		ofNullable(getReferenceObligation()).ifPresent(builder::setReferenceObligation);
		return builder;
	}

	public static ReferencePairBuilder builder() {
		return new ReferencePairBuilder();
	}

	public static  class ReferencePairBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaEntityTypeEnumBuilder entityType;
		protected Boolean noReferenceObligation;
		protected LegalEntity.LegalEntityBuilder referenceEntity;
		protected ReferenceObligation.ReferenceObligationBuilder referenceObligation;
	
		public ReferencePairBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferencePair> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaEntityTypeEnumBuilder getEntityType() {
			return entityType;
		}
		
		public FieldWithMetaEntityTypeEnumBuilder getOrCreateEntityType() {
			if (entityType!=null) {
				return entityType;
			}
			else return entityType = new FieldWithMetaEntityTypeEnumBuilder();
		}
		
		
		public Boolean getNoReferenceObligation() {
			return noReferenceObligation;
		}
		
		
		public LegalEntity.LegalEntityBuilder getReferenceEntity() {
			return referenceEntity;
		}
		
		public LegalEntity.LegalEntityBuilder getOrCreateReferenceEntity() {
			if (referenceEntity!=null) {
				return referenceEntity;
			}
			else return referenceEntity = new LegalEntity.LegalEntityBuilder();
		}
		
		
		public ReferenceObligation.ReferenceObligationBuilder getReferenceObligation() {
			return referenceObligation;
		}
		
		public ReferenceObligation.ReferenceObligationBuilder getOrCreateReferenceObligation() {
			if (referenceObligation!=null) {
				return referenceObligation;
			}
			else return referenceObligation = new ReferenceObligation.ReferenceObligationBuilder();
		}
		
	
		public ReferencePairBuilder setEntityType(FieldWithMetaEntityTypeEnum entityType) {
			this.entityType = entityType.toBuilder();
			return this;
		}
		public ReferencePairBuilder setEntityTypeRef(EntityTypeEnum entityType) {
			return setEntityType(FieldWithMetaEntityTypeEnum.builder().setValue(entityType).build());
		}
		public ReferencePairBuilder setNoReferenceObligation(Boolean noReferenceObligation) {
			this.noReferenceObligation = noReferenceObligation;
			return this;
		}
		public ReferencePairBuilder setReferenceEntity(LegalEntity referenceEntity) {
			this.referenceEntity = referenceEntity.toBuilder();
			return this;
		}
		public ReferencePairBuilder setReferenceEntityBuilder(LegalEntity.LegalEntityBuilder referenceEntity) {
			this.referenceEntity = referenceEntity;
			return this;
		}
		public ReferencePairBuilder setReferenceObligation(ReferenceObligation referenceObligation) {
			this.referenceObligation = referenceObligation.toBuilder();
			return this;
		}
		public ReferencePairBuilder setReferenceObligationBuilder(ReferenceObligation.ReferenceObligationBuilder referenceObligation) {
			this.referenceObligation = referenceObligation;
			return this;
		}
		
		public ReferencePair build() {
			return new ReferencePair(this);
		}
	
		@Override
		public ReferencePairBuilder prune() {
			if (entityType!=null && !entityType.prune().hasData()) entityType = null;
			if (referenceEntity!=null && !referenceEntity.prune().hasData()) referenceEntity = null;
			if (referenceObligation!=null && !referenceObligation.prune().hasData()) referenceObligation = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEntityType()!=null) return true;
			if (getNoReferenceObligation()!=null) return true;
			if (getReferenceEntity()!=null && getReferenceEntity().hasData()) return true;
			if (getReferenceObligation()!=null && getReferenceObligation().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("noReferenceObligation"), Boolean.class, noReferenceObligation, this);
			
			processRosetta(path.newSubPath("entityType"), processor, FieldWithMetaEntityTypeEnum.class, entityType);
			processRosetta(path.newSubPath("referenceEntity"), processor, LegalEntity.class, referenceEntity);
			processRosetta(path.newSubPath("referenceObligation"), processor, ReferenceObligation.class, referenceObligation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferencePairBuilder _that = (ReferencePairBuilder) o;
		
			if (entityType != null ? !entityType.equals(_that.entityType) : _that.entityType != null) return false;
			if (noReferenceObligation != null ? !noReferenceObligation.equals(_that.noReferenceObligation) : _that.noReferenceObligation != null) return false;
			if (referenceEntity != null ? !referenceEntity.equals(_that.referenceEntity) : _that.referenceEntity != null) return false;
			if (referenceObligation != null ? !referenceObligation.equals(_that.referenceObligation) : _that.referenceObligation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (entityType != null ? entityType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (noReferenceObligation != null ? noReferenceObligation.hashCode() : 0);
			_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
			_result = 31 * _result + (referenceObligation != null ? referenceObligation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferencePairBuilder {" +
				"entityType=" + this.entityType + ", " +
				"noReferenceObligation=" + this.noReferenceObligation + ", " +
				"referenceEntity=" + this.referenceEntity + ", " +
				"referenceObligation=" + this.referenceObligation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("noReferenceObligation"), Boolean.class, noReferenceObligation, this);
		
		processRosetta(path.newSubPath("entityType"), processor, FieldWithMetaEntityTypeEnum.class, entityType);
		processRosetta(path.newSubPath("referenceEntity"), processor, LegalEntity.class, referenceEntity);
		processRosetta(path.newSubPath("referenceObligation"), processor, ReferenceObligation.class, referenceObligation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferencePair _that = (ReferencePair) o;
	
		if (entityType != null ? !entityType.equals(_that.entityType) : _that.entityType != null) return false;
		if (noReferenceObligation != null ? !noReferenceObligation.equals(_that.noReferenceObligation) : _that.noReferenceObligation != null) return false;
		if (referenceEntity != null ? !referenceEntity.equals(_that.referenceEntity) : _that.referenceEntity != null) return false;
		if (referenceObligation != null ? !referenceObligation.equals(_that.referenceObligation) : _that.referenceObligation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (entityType != null ? entityType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (noReferenceObligation != null ? noReferenceObligation.hashCode() : 0);
		_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
		_result = 31 * _result + (referenceObligation != null ? referenceObligation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferencePair {" +
			"entityType=" + this.entityType + ", " +
			"noReferenceObligation=" + this.noReferenceObligation + ", " +
			"referenceEntity=" + this.referenceEntity + ", " +
			"referenceObligation=" + this.referenceObligation +
		'}';
	}
}
