package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.ReferencePoolItem;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ReferencePoolMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * This type contains all the reference pool items to define the reference entity and reference obligation(s) in the basket.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ReferencePool", source="FpML_5_10")
public class ReferencePool extends RosettaModelObject {
	private final List<ReferencePoolItem> referencePoolItem;
	private static ReferencePoolMeta metaData = new ReferencePoolMeta();
	
	ReferencePool(ReferencePoolBuilder builder) {
		this.referencePoolItem = ofNullable(builder.getReferencePoolItem()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * This type contains all the constituent weight and reference information.
	 */
	@RosettaSynonym(value="referencePoolItem", source="FpML_5_10")
	@RosettaSynonym(value="referencePoolItem", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="referencePoolItem", source="DTCC_11_0")
	@RosettaSynonym(value="referencePoolItem", source="DTCC_9_0")
	@RosettaSynonym(value="referencePoolItem", source="CME_ClearedConfirm_1_17")
	public final List<ReferencePoolItem> getReferencePoolItem() {
		return referencePoolItem;
	}
	
	
	@Override
	public RosettaMetaData<? extends ReferencePool> metaData() {
		return metaData;
	} 
	
	public ReferencePoolBuilder toBuilder() {
		ReferencePoolBuilder builder = new ReferencePoolBuilder();
		ofNullable(getReferencePoolItem()).ifPresent(referencePoolItem -> referencePoolItem.forEach(builder::addReferencePoolItem));
		return builder;
	}

	public static ReferencePoolBuilder builder() {
		return new ReferencePoolBuilder();
	}

	public static  class ReferencePoolBuilder extends RosettaModelObjectBuilder {
	
		protected List<ReferencePoolItem.ReferencePoolItemBuilder> referencePoolItem;
	
		public ReferencePoolBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ReferencePool> metaData() {
			return metaData;
		} 
	
		
		public List<ReferencePoolItem.ReferencePoolItemBuilder> getReferencePoolItem() {
			return referencePoolItem;
		}
		
		public ReferencePoolItem.ReferencePoolItemBuilder getOrCreateReferencePoolItem(int index) {
			if (referencePoolItem==null) {
				this.referencePoolItem = new ArrayList<>();
			}
			return getIndex(referencePoolItem, index, ()->new ReferencePoolItem.ReferencePoolItemBuilder());
		}
		
	
		public ReferencePoolBuilder addReferencePoolItem(ReferencePoolItem referencePoolItem) {
			if(this.referencePoolItem == null){
				this.referencePoolItem = new ArrayList<>();
			}
			this.referencePoolItem.add(referencePoolItem.toBuilder());
			return this;
		}
		public ReferencePoolBuilder addReferencePoolItem(ReferencePoolItem referencePoolItem, int _idx) {
			if(this.referencePoolItem == null){
				this.referencePoolItem = new ArrayList<>();
			}
			getIndex(this.referencePoolItem, _idx, () -> referencePoolItem.toBuilder());
			this.referencePoolItem.set(_idx, referencePoolItem.toBuilder());
			return this;
		}
		public ReferencePoolBuilder addReferencePoolItem(List<ReferencePoolItem> referencePoolItems) {
			if(this.referencePoolItem == null){
				this.referencePoolItem = new ArrayList<>();
			}
			for (ReferencePoolItem toAdd : referencePoolItems) {
				this.referencePoolItem.add(toAdd.toBuilder());
			}
			return this;
		}
		public ReferencePoolBuilder addReferencePoolItemBuilder(ReferencePoolItem.ReferencePoolItemBuilder referencePoolItem) {
			if(this.referencePoolItem == null){
				this.referencePoolItem = new ArrayList<>();
			}
			this.referencePoolItem.add(referencePoolItem);
			return this;
		}
		
		public ReferencePoolBuilder clearReferencePoolItem() {
			this.referencePoolItem = null;
			return this;
		}
		
		public ReferencePool build() {
			return new ReferencePool(this);
		}
	
		@Override
		public ReferencePoolBuilder prune() {
			if (referencePoolItem!=null) referencePoolItem = referencePoolItem.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getReferencePoolItem()!=null && getReferencePoolItem().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("referencePoolItem"), processor, ReferencePoolItem.class, referencePoolItem);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ReferencePoolBuilder _that = (ReferencePoolBuilder) o;
		
			if (!ListEquals.listEquals(referencePoolItem, _that.referencePoolItem)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (referencePoolItem != null ? referencePoolItem.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ReferencePoolBuilder {" +
				"referencePoolItem=" + this.referencePoolItem +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("referencePoolItem"), processor, ReferencePoolItem.class, referencePoolItem);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ReferencePool _that = (ReferencePool) o;
	
		if (!ListEquals.listEquals(referencePoolItem, _that.referencePoolItem)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (referencePoolItem != null ? referencePoolItem.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ReferencePool {" +
			"referencePoolItem=" + this.referencePoolItem +
		'}';
	}
}
