package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.RegimeElection;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RegimeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the regulatory regimes elections by the respective parties a legal agreement. 2016 ISDA Credit Support Annex for Initial Margin: Regime.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Regime", source="AcadiaSoft_AM_1_0")
public class Regime extends RosettaModelObject {
	private final List<RegimeElection> partyElection;
	private static RegimeMeta metaData = new RegimeMeta();
	
	Regime(RegimeBuilder builder) {
		this.partyElection = ofNullable(builder.getPartyElection()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The parties&#39; regulatory regime election.
	 */
	public final List<RegimeElection> getPartyElection() {
		return partyElection;
	}
	
	
	@Override
	public RosettaMetaData<? extends Regime> metaData() {
		return metaData;
	} 
	
	public RegimeBuilder toBuilder() {
		RegimeBuilder builder = new RegimeBuilder();
		ofNullable(getPartyElection()).ifPresent(partyElection -> partyElection.forEach(builder::addPartyElection));
		return builder;
	}

	public static RegimeBuilder builder() {
		return new RegimeBuilder();
	}

	public static  class RegimeBuilder extends RosettaModelObjectBuilder {
	
		protected List<RegimeElection.RegimeElectionBuilder> partyElection;
	
		public RegimeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Regime> metaData() {
			return metaData;
		} 
	
		
		public List<RegimeElection.RegimeElectionBuilder> getPartyElection() {
			return partyElection;
		}
		
		public RegimeElection.RegimeElectionBuilder getOrCreatePartyElection(int index) {
			if (partyElection==null) {
				this.partyElection = new ArrayList<>();
			}
			return getIndex(partyElection, index, ()->new RegimeElection.RegimeElectionBuilder());
		}
		
	
		public RegimeBuilder addPartyElection(RegimeElection partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection.toBuilder());
			return this;
		}
		public RegimeBuilder addPartyElection(RegimeElection partyElection, int _idx) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			getIndex(this.partyElection, _idx, () -> partyElection.toBuilder());
			this.partyElection.set(_idx, partyElection.toBuilder());
			return this;
		}
		public RegimeBuilder addPartyElection(List<RegimeElection> partyElections) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			for (RegimeElection toAdd : partyElections) {
				this.partyElection.add(toAdd.toBuilder());
			}
			return this;
		}
		public RegimeBuilder addPartyElectionBuilder(RegimeElection.RegimeElectionBuilder partyElection) {
			if(this.partyElection == null){
				this.partyElection = new ArrayList<>();
			}
			this.partyElection.add(partyElection);
			return this;
		}
		
		public RegimeBuilder clearPartyElection() {
			this.partyElection = null;
			return this;
		}
		
		public Regime build() {
			return new Regime(this);
		}
	
		@Override
		public RegimeBuilder prune() {
			if (partyElection!=null) partyElection = partyElection.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyElection()!=null && getPartyElection().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("partyElection"), processor, RegimeElection.class, partyElection);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RegimeBuilder _that = (RegimeBuilder) o;
		
			if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RegimeBuilder {" +
				"partyElection=" + this.partyElection +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("partyElection"), processor, RegimeElection.class, partyElection);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Regime _that = (Regime) o;
	
		if (!ListEquals.listEquals(partyElection, _that.partyElection)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyElection != null ? partyElection.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Regime {" +
			"partyElection=" + this.partyElection +
		'}';
	}
}
