package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the regulatory regimes. The display name corresponds to the regulatory regime name or acronym specified as part of the Regime table in the ISDA CSA for Initial Margin, paragraph 13, General Principles. 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles: Regime. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles: Regime.
 * 
 * @version 2.5.4
 */
public enum RegulatoryRegimeEnum {
    

	/**
	 * Australian Prudential Standard CPS 226 Margining and risk mitigation for non-centrally cleared derivatives. 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "APRA", source = "AcadiaSoft_AM_1_0")
	AUSTRALIA_MARGIN_RULES("Australia"),
	

	/**
	 * Guideline E-22, Margin Requirements for Non-Centrally Cleared Derivatives issued by the Canadian Office of the Superintendent of Financial Institutions in February 2016. 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "OSFI", source = "AcadiaSoft_AM_1_0")
	CANADA_MARGIN_RULES("Canada"),
	

	/**
	 * Margin requirements adopted by the U.S. Commodity Futures Trading Commission pursuant to CEA § 4s(e). 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "CFTC", source = "AcadiaSoft_AM_1_0")
	CFTC_MARGIN_RULES("CFTC"),
	

	/**
	 * Regulation (EU) No 648/2012 of the European Parliament and of the Council of 4 July 2012 on OTC derivatives, central counterparties and trade repositories (including the EMIR RTS, which means the published regulatory technical standards on risk-mitigation techniques for OTC-derivative contracts not cleared by a CCP under Article 11(15) of EMIR). 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	EMIR_MARGIN_RULES("EMIR"),
	

	/**
	 * Chapter CR-G-14 &#39;Non-centrally Cleared OTC Derivatives Transactions – Margin and Other Risk Mitigation Standards&#39; in the Banking Supervisory Policy Manual issued by the Hong Kong Monetary Authority. 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "HKMA", source = "AcadiaSoft_AM_1_0")
	HONG_KONG_MARGIN_RULES("Hong Kong"),
	

	/**
	 * Margin rules adopted by the Financial Services Agency of Japan pursuant to Article 40, Item 2 of the Financial Instruments and Exchange Act (kin’yuu shouhin torihiki hou) (Act No. 25 of 1948) and by the Ministry of Agriculture, Forestry and Fisheries and the Ministry of Economy, Trade and Industry pursuant to the Commodity Derivatives Act (shouhin sakimono torihiki hou) (Act No. 239 of 1950) (including their subordinated regulations and the related supervisory guidelines). 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "JFSA", source = "AcadiaSoft_AM_1_0")
	JAPAN_MARGIN_RULES("Japan"),
	

	/**
	 * Margin requirements adopted by a &#39;prudential regulator&#39; (as defined in CEA § 1a(39)) pursuant to CEA § 4s(e) and Exchange Act § 15F(e). 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "USPR", source = "AcadiaSoft_AM_1_0")
	US_PRUDENTIAL_MARGIN_RULES("Prudential"),
	

	/**
	 * Margin requirements adopted by the U.S. Securities and Exchange Commission pursuant to Exchange Act § 15F(e). 2016 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles. | 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "SEC", source = "AcadiaSoft_AM_1_0")
	SEC_MARGIN_RULES("SEC"),
	

	/**
	 * Guidelines on Margin Requirements for Non-centrally Cleared OTC Derivatives Contracts issued by the Monetary Authority of Singapore (MAS) pursuant to section 321 of the Securities and Futures Act, Chapter 289 of Singapore. 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	@RosettaSynonym(value = "MAS", source = "AcadiaSoft_AM_1_0")
	SINGAPORE_MARGIN_RULES("Singapore"),
	

	/**
	 * Margin rules adopted by the Swiss Federal Council pursuant to Article 110-111 of the Financial Market Infrastructure Act as well as Articles 100 to 107 and Annexes 3 to 5 of the Financial Market Infrastructure Ordinance. 2018 ISDA Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	SWITZERLAND_MARGIN_RULES("Switzerland")
	;

	private final String displayName;
	
	RegulatoryRegimeEnum() {
		this.displayName = null;
	}

	RegulatoryRegimeEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
