package org.isda.cdm;

import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RelatedPartyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.stream.Collectors;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="RelatedParty", source="FpML_5_10")
public class RelatedParty extends RosettaModelObject {
	private final ReferenceWithMetaAccount accountReference;
	private final ReferenceWithMetaParty partyReference;
	private final PartyRoleEnum role;
	private static RelatedPartyMeta metaData = new RelatedPartyMeta();
	
	RelatedParty(RelatedPartyBuilder builder) {
		this.accountReference = ofNullable(builder.getAccountReference()).map(f->f.build()).orElse(null);
		this.partyReference = ofNullable(builder.getPartyReference()).map(f->f.build()).orElse(null);
		this.role = builder.getRole();
	}
	
	/**
	 * Reference to an account.
	 */
	@RosettaSynonym(value="accountReference", source="FpML_5_10")
	@RosettaSynonym(value="accountReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="accountReference", source="DTCC_11_0")
	@RosettaSynonym(value="accountReference", source="DTCC_9_0")
	@RosettaSynonym(value="accountReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaAccount getAccountReference() {
		return accountReference;
	}
	
	/**
	 * Reference to a party.
	 */
	@RosettaSynonym(value="partyReference", source="FpML_5_10")
	@RosettaSynonym(value="partyReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="partyReference", source="DTCC_11_0")
	@RosettaSynonym(value="partyReference", source="DTCC_9_0")
	@RosettaSynonym(value="partyReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaParty getPartyReference() {
		return partyReference;
	}
	
	/**
	 * The category of the relationship. The related party performs the role specified in this field for the base party. For example, if the role is ,Guarantor,, the related party acts as a guarantor for the base party.
	 */
	@RosettaSynonym(value="role", source="FpML_5_10")
	@RosettaSynonym(value="role", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="role", source="DTCC_11_0")
	@RosettaSynonym(value="role", source="DTCC_9_0")
	@RosettaSynonym(value="role", source="CME_ClearedConfirm_1_17")
	public final PartyRoleEnum getRole() {
		return role;
	}
	
	
	@Override
	public RosettaMetaData<? extends RelatedParty> metaData() {
		return metaData;
	} 
	
	public RelatedPartyBuilder toBuilder() {
		RelatedPartyBuilder builder = new RelatedPartyBuilder();
		ofNullable(getAccountReference()).ifPresent(builder::setAccountReference);
		ofNullable(getPartyReference()).ifPresent(builder::setPartyReference);
		ofNullable(getRole()).ifPresent(builder::setRole);
		return builder;
	}

	public static RelatedPartyBuilder builder() {
		return new RelatedPartyBuilder();
	}

	public static  class RelatedPartyBuilder extends RosettaModelObjectBuilder {
	
		protected ReferenceWithMetaAccountBuilder accountReference;
		protected ReferenceWithMetaPartyBuilder partyReference;
		protected PartyRoleEnum role;
	
		public RelatedPartyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends RelatedParty> metaData() {
			return metaData;
		} 
	
		
		public ReferenceWithMetaAccountBuilder getAccountReference() {
			return accountReference;
		}
		
		public ReferenceWithMetaAccountBuilder getOrCreateAccountReference() {
			if (accountReference!=null) {
				return accountReference;
			}
			else return accountReference = new ReferenceWithMetaAccountBuilder();
		}
		
		
		public ReferenceWithMetaPartyBuilder getPartyReference() {
			return partyReference;
		}
		
		public ReferenceWithMetaPartyBuilder getOrCreatePartyReference() {
			if (partyReference!=null) {
				return partyReference;
			}
			else return partyReference = new ReferenceWithMetaPartyBuilder();
		}
		
		
		public PartyRoleEnum getRole() {
			return role;
		}
		
	
		public RelatedPartyBuilder setAccountReference(ReferenceWithMetaAccount accountReference) {
			this.accountReference = accountReference.toBuilder();
			return this;
		}
		public RelatedPartyBuilder setAccountReferenceBuilder(ReferenceWithMetaAccountBuilder accountReference) {
			this.accountReference = accountReference;
			return this;
		}
		public RelatedPartyBuilder setAccountReferenceRef(Account.AccountBuilder accountReference) {
			return setAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(accountReference).build());
		}
		public RelatedPartyBuilder setAccountReferenceRef(Account accountReference) {
			return setAccountReferenceRef(Account.builder());
		}
		public RelatedPartyBuilder setPartyReference(ReferenceWithMetaParty partyReference) {
			this.partyReference = partyReference.toBuilder();
			return this;
		}
		public RelatedPartyBuilder setPartyReferenceBuilder(ReferenceWithMetaPartyBuilder partyReference) {
			this.partyReference = partyReference;
			return this;
		}
		public RelatedPartyBuilder setPartyReferenceRef(Party.PartyBuilder partyReference) {
			return setPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(partyReference).build());
		}
		public RelatedPartyBuilder setPartyReferenceRef(Party partyReference) {
			return setPartyReferenceRef(Party.builder());
		}
		public RelatedPartyBuilder setRole(PartyRoleEnum role) {
			this.role = role;
			return this;
		}
		
		public RelatedParty build() {
			return new RelatedParty(this);
		}
	
		@Override
		public RelatedPartyBuilder prune() {
			if (accountReference!=null && !accountReference.prune().hasData()) accountReference = null;
			if (partyReference!=null && !partyReference.prune().hasData()) partyReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAccountReference()!=null && getAccountReference().hasData()) return true;
			if (getPartyReference()!=null && getPartyReference().hasData()) return true;
			if (getRole()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("role"), PartyRoleEnum.class, role, this);
			
			processRosetta(path.newSubPath("accountReference"), processor, ReferenceWithMetaAccount.class, accountReference);
			processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RelatedPartyBuilder _that = (RelatedPartyBuilder) o;
		
			if (accountReference != null ? !accountReference.equals(_that.accountReference) : _that.accountReference != null) return false;
			if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
			if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (accountReference != null ? accountReference.hashCode() : 0);
			_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
			_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RelatedPartyBuilder {" +
				"accountReference=" + this.accountReference + ", " +
				"partyReference=" + this.partyReference + ", " +
				"role=" + this.role +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("role"), PartyRoleEnum.class, role, this);
		
		processRosetta(path.newSubPath("accountReference"), processor, ReferenceWithMetaAccount.class, accountReference);
		processRosetta(path.newSubPath("partyReference"), processor, ReferenceWithMetaParty.class, partyReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		RelatedParty _that = (RelatedParty) o;
	
		if (accountReference != null ? !accountReference.equals(_that.accountReference) : _that.accountReference != null) return false;
		if (partyReference != null ? !partyReference.equals(_that.partyReference) : _that.partyReference != null) return false;
		if (role != null ? !role.equals(_that.role) : _that.role != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (accountReference != null ? accountReference.hashCode() : 0);
		_result = 31 * _result + (partyReference != null ? partyReference.hashCode() : 0);
		_result = 31 * _result + (role != null ? role.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "RelatedParty {" +
			"accountReference=" + this.accountReference + ", " +
			"partyReference=" + this.partyReference + ", " +
			"role=" + this.role +
		'}';
	}
}
