package org.isda.cdm;

import java.lang.Boolean;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RepresentationsMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Representations", source="FpML_5_10")
public class Representations extends RosettaModelObject {
	private final Boolean additionalAcknowledgements;
	private final Boolean agreementsRegardingHedging;
	private final Boolean indexDisclaimer;
	private final Boolean nonReliance;
	private static RepresentationsMeta metaData = new RepresentationsMeta();
	
	Representations(RepresentationsBuilder builder) {
		this.additionalAcknowledgements = builder.getAdditionalAcknowledgements();
		this.agreementsRegardingHedging = builder.getAgreementsRegardingHedging();
		this.indexDisclaimer = builder.getIndexDisclaimer();
		this.nonReliance = builder.getNonReliance();
	}
	
	/**
	 * If true, then additional acknowledgements are applicable.
	 */
	@RosettaSynonym(value="additionalAcknowledgements", source="FpML_5_10")
	public final Boolean getAdditionalAcknowledgements() {
		return additionalAcknowledgements;
	}
	
	/**
	 * If true, then agreements regarding hedging are applicable.
	 */
	@RosettaSynonym(value="agreementsRegardingHedging", source="FpML_5_10")
	public final Boolean getAgreementsRegardingHedging() {
		return agreementsRegardingHedging;
	}
	
	/**
	 * If present and true, then index disclaimer is applicable.
	 */
	@RosettaSynonym(value="indexDisclaimer", source="FpML_5_10")
	public final Boolean getIndexDisclaimer() {
		return indexDisclaimer;
	}
	
	/**
	 * If true, then non reliance is applicable.
	 */
	@RosettaSynonym(value="nonReliance", source="FpML_5_10")
	public final Boolean getNonReliance() {
		return nonReliance;
	}
	
	
	@Override
	public RosettaMetaData<? extends Representations> metaData() {
		return metaData;
	} 
	
	public RepresentationsBuilder toBuilder() {
		RepresentationsBuilder builder = new RepresentationsBuilder();
		ofNullable(getAdditionalAcknowledgements()).ifPresent(builder::setAdditionalAcknowledgements);
		ofNullable(getAgreementsRegardingHedging()).ifPresent(builder::setAgreementsRegardingHedging);
		ofNullable(getIndexDisclaimer()).ifPresent(builder::setIndexDisclaimer);
		ofNullable(getNonReliance()).ifPresent(builder::setNonReliance);
		return builder;
	}

	public static RepresentationsBuilder builder() {
		return new RepresentationsBuilder();
	}

	public static  class RepresentationsBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean additionalAcknowledgements;
		protected Boolean agreementsRegardingHedging;
		protected Boolean indexDisclaimer;
		protected Boolean nonReliance;
	
		public RepresentationsBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Representations> metaData() {
			return metaData;
		} 
	
		
		public Boolean getAdditionalAcknowledgements() {
			return additionalAcknowledgements;
		}
		
		
		public Boolean getAgreementsRegardingHedging() {
			return agreementsRegardingHedging;
		}
		
		
		public Boolean getIndexDisclaimer() {
			return indexDisclaimer;
		}
		
		
		public Boolean getNonReliance() {
			return nonReliance;
		}
		
	
		public RepresentationsBuilder setAdditionalAcknowledgements(Boolean additionalAcknowledgements) {
			this.additionalAcknowledgements = additionalAcknowledgements;
			return this;
		}
		public RepresentationsBuilder setAgreementsRegardingHedging(Boolean agreementsRegardingHedging) {
			this.agreementsRegardingHedging = agreementsRegardingHedging;
			return this;
		}
		public RepresentationsBuilder setIndexDisclaimer(Boolean indexDisclaimer) {
			this.indexDisclaimer = indexDisclaimer;
			return this;
		}
		public RepresentationsBuilder setNonReliance(Boolean nonReliance) {
			this.nonReliance = nonReliance;
			return this;
		}
		
		public Representations build() {
			return new Representations(this);
		}
	
		@Override
		public RepresentationsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAdditionalAcknowledgements()!=null) return true;
			if (getAgreementsRegardingHedging()!=null) return true;
			if (getIndexDisclaimer()!=null) return true;
			if (getNonReliance()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("additionalAcknowledgements"), Boolean.class, additionalAcknowledgements, this);
			processor.processBasic(path.newSubPath("agreementsRegardingHedging"), Boolean.class, agreementsRegardingHedging, this);
			processor.processBasic(path.newSubPath("indexDisclaimer"), Boolean.class, indexDisclaimer, this);
			processor.processBasic(path.newSubPath("nonReliance"), Boolean.class, nonReliance, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RepresentationsBuilder _that = (RepresentationsBuilder) o;
		
			if (additionalAcknowledgements != null ? !additionalAcknowledgements.equals(_that.additionalAcknowledgements) : _that.additionalAcknowledgements != null) return false;
			if (agreementsRegardingHedging != null ? !agreementsRegardingHedging.equals(_that.agreementsRegardingHedging) : _that.agreementsRegardingHedging != null) return false;
			if (indexDisclaimer != null ? !indexDisclaimer.equals(_that.indexDisclaimer) : _that.indexDisclaimer != null) return false;
			if (nonReliance != null ? !nonReliance.equals(_that.nonReliance) : _that.nonReliance != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (additionalAcknowledgements != null ? additionalAcknowledgements.hashCode() : 0);
			_result = 31 * _result + (agreementsRegardingHedging != null ? agreementsRegardingHedging.hashCode() : 0);
			_result = 31 * _result + (indexDisclaimer != null ? indexDisclaimer.hashCode() : 0);
			_result = 31 * _result + (nonReliance != null ? nonReliance.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RepresentationsBuilder {" +
				"additionalAcknowledgements=" + this.additionalAcknowledgements + ", " +
				"agreementsRegardingHedging=" + this.agreementsRegardingHedging + ", " +
				"indexDisclaimer=" + this.indexDisclaimer + ", " +
				"nonReliance=" + this.nonReliance +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("additionalAcknowledgements"), Boolean.class, additionalAcknowledgements, this);
		processor.processBasic(path.newSubPath("agreementsRegardingHedging"), Boolean.class, agreementsRegardingHedging, this);
		processor.processBasic(path.newSubPath("indexDisclaimer"), Boolean.class, indexDisclaimer, this);
		processor.processBasic(path.newSubPath("nonReliance"), Boolean.class, nonReliance, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Representations _that = (Representations) o;
	
		if (additionalAcknowledgements != null ? !additionalAcknowledgements.equals(_that.additionalAcknowledgements) : _that.additionalAcknowledgements != null) return false;
		if (agreementsRegardingHedging != null ? !agreementsRegardingHedging.equals(_that.agreementsRegardingHedging) : _that.agreementsRegardingHedging != null) return false;
		if (indexDisclaimer != null ? !indexDisclaimer.equals(_that.indexDisclaimer) : _that.indexDisclaimer != null) return false;
		if (nonReliance != null ? !nonReliance.equals(_that.nonReliance) : _that.nonReliance != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (additionalAcknowledgements != null ? additionalAcknowledgements.hashCode() : 0);
		_result = 31 * _result + (agreementsRegardingHedging != null ? agreementsRegardingHedging.hashCode() : 0);
		_result = 31 * _result + (indexDisclaimer != null ? indexDisclaimer.hashCode() : 0);
		_result = 31 * _result + (nonReliance != null ? nonReliance.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Representations {" +
			"additionalAcknowledgements=" + this.additionalAcknowledgements + ", " +
			"agreementsRegardingHedging=" + this.agreementsRegardingHedging + ", " +
			"indexDisclaimer=" + this.indexDisclaimer + ", " +
			"nonReliance=" + this.nonReliance +
		'}';
	}
}
