package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether resets occur relative to the first or last day of a calculation period.
 * 
 * @version 2.5.4
 */
public enum ResetRelativeToEnum {
    

	/**
	 * Resets occur relative to the first day of a calculation period.
	 */
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "FpML_5_10")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalculationPeriodStartDate", source = "CME_ClearedConfirm_1_17")
	CALCULATION_PERIOD_START_DATE,
	    

	/**
	 * Resets occur relative to the last day of a calculation period.
	 */
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "FpML_5_10")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CalculationPeriodEndDate", source = "CME_ClearedConfirm_1_17")
	CALCULATION_PERIOD_END_DATE    
	;

	private final String displayName;
	
	ResetRelativeToEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
