package org.isda.cdm;

import java.lang.String;
import java.math.BigDecimal;
import org.isda.cdm.ResourceLength;
import org.isda.cdm.metafields.FieldWithMetaResourceTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaResourceTypeEnum.FieldWithMetaResourceTypeEnumBuilder;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResourceMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * Describes the resource that contains the media representation of a business event (i.e used for stating the Publicly Available Information). For example, can describe a file or a URL that represents the event. This type is an extended version of a type defined by RIXML (www.rixml.org).  Rosetta restricts the FpML implementation by not providing the ability to associated a document in hexadecimalBinary or base64Binary until such time that actual use cases will come up.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Resource", source="FpML_5_10")
public class Resource extends RosettaModelObject {
	private final String comments;
	private final FieldWithMetaString language;
	private final ResourceLength length;
	private final FieldWithMetaString mimeType;
	private final String name;
	private final FieldWithMetaString resourceId;
	private final FieldWithMetaResourceTypeEnum resourceType;
	private final BigDecimal sizeInBytes;
	private final String string;
	private final String url;
	private static ResourceMeta metaData = new ResourceMeta();
	
	Resource(ResourceBuilder builder) {
		this.comments = builder.getComments();
		this.language = ofNullable(builder.getLanguage()).map(f->f.build()).orElse(null);
		this.length = ofNullable(builder.getLength()).map(f->f.build()).orElse(null);
		this.mimeType = ofNullable(builder.getMimeType()).map(f->f.build()).orElse(null);
		this.name = builder.getName();
		this.resourceId = ofNullable(builder.getResourceId()).map(f->f.build()).orElse(null);
		this.resourceType = ofNullable(builder.getResourceType()).map(f->f.build()).orElse(null);
		this.sizeInBytes = builder.getSizeInBytes();
		this.string = builder.getString();
		this.url = builder.getUrl();
	}
	
	/**
	 * Any additional comments that are deemed necessary. For example, which software version is required to open the document? Or, how does this resource relate to the others for this event?
	 */
	@RosettaSynonym(value="comments", source="FpML_5_10")
	@RosettaSynonym(value="comments", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="comments", source="DTCC_11_0")
	@RosettaSynonym(value="comments", source="DTCC_9_0")
	@RosettaSynonym(value="comments", source="CME_ClearedConfirm_1_17")
	public final String getComments() {
		return comments;
	}
	
	/**
	 * Indicates the language of the resource, described using the ISO 639-2/T Code.
	 */
	@RosettaSynonym(value="language", source="FpML_5_10")
	@RosettaSynonym(value="language", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="language", source="DTCC_11_0")
	@RosettaSynonym(value="language", source="DTCC_9_0")
	@RosettaSynonym(value="language", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getLanguage() {
		return language;
	}
	
	/**
	 * Indicates the length of the resource. For example, if the resource were a PDF file, the length would be in pages.
	 */
	@RosettaSynonym(value="length", source="FpML_5_10")
	@RosettaSynonym(value="length", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="length", source="DTCC_11_0")
	@RosettaSynonym(value="length", source="DTCC_9_0")
	@RosettaSynonym(value="length", source="CME_ClearedConfirm_1_17")
	public final ResourceLength getLength() {
		return length;
	}
	
	/**
	 * Indicates the type of media used to store the content. mimeType is used to determine the software product(s) that can read the content. MIME Types are described in RFC 2046.
	 */
	@RosettaSynonym(value="mimeType", source="FpML_5_10")
	@RosettaSynonym(value="mimeType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mimeType", source="DTCC_11_0")
	@RosettaSynonym(value="mimeType", source="DTCC_9_0")
	@RosettaSynonym(value="mimeType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getMimeType() {
		return mimeType;
	}
	
	/**
	 * The name of the resource.  It is specified as a NormalizedString in FpML.
	 */
	@RosettaSynonym(value="name", source="FpML_5_10")
	@RosettaSynonym(value="name", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="name", source="DTCC_11_0")
	@RosettaSynonym(value="name", source="DTCC_9_0")
	@RosettaSynonym(value="name", source="CME_ClearedConfirm_1_17")
	public final String getName() {
		return name;
	}
	
	/**
	 * The unique identifier of the resource within the event. FpML specifies this element of type resourceIdScheme but with no specified value.
	 */
	@RosettaSynonym(value="resourceId", source="FpML_5_10")
	@RosettaSynonym(value="resourceId", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="resourceId", source="DTCC_11_0")
	@RosettaSynonym(value="resourceId", source="DTCC_9_0")
	@RosettaSynonym(value="resourceId", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getResourceId() {
		return resourceId;
	}
	
	/**
	 * A description of the type of the resource, e.g. a confirmation.
	 */
	@RosettaSynonym(value="resourceType", source="FpML_5_10")
	@RosettaSynonym(value="resourceType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="resourceType", source="DTCC_11_0")
	@RosettaSynonym(value="resourceType", source="DTCC_9_0")
	@RosettaSynonym(value="resourceType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaResourceTypeEnum getResourceType() {
		return resourceType;
	}
	
	/**
	 * Indicates the size of the resource in bytes. It could be used by the end user to estimate the download time and storage needs.
	 */
	@RosettaSynonym(value="sizeInBytes", source="FpML_5_10")
	@RosettaSynonym(value="sizeInBytes", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="sizeInBytes", source="DTCC_11_0")
	@RosettaSynonym(value="sizeInBytes", source="DTCC_9_0")
	@RosettaSynonym(value="sizeInBytes", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getSizeInBytes() {
		return sizeInBytes;
	}
	
	/**
	 * Provides extra information as string. In case the extra information is in XML format, a CDATA section must be placed around the source message to prevent its interpretation as XML content.
	 */
	@RosettaSynonym(value="string", source="FpML_5_10")
	@RosettaSynonym(value="string", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="string", source="DTCC_11_0")
	@RosettaSynonym(value="string", source="DTCC_9_0")
	@RosettaSynonym(value="string", source="CME_ClearedConfirm_1_17")
	public final String getString() {
		return string;
	}
	
	/**
	 * Indicates where the resource can be found, as a URL that references the information on a web server accessible to the message recipient.
	 */
	@RosettaSynonym(value="url", source="FpML_5_10")
	@RosettaSynonym(value="url", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="url", source="DTCC_11_0")
	@RosettaSynonym(value="url", source="DTCC_9_0")
	@RosettaSynonym(value="url", source="CME_ClearedConfirm_1_17")
	public final String getUrl() {
		return url;
	}
	
	
	@Override
	public RosettaMetaData<? extends Resource> metaData() {
		return metaData;
	} 
	
	public ResourceBuilder toBuilder() {
		ResourceBuilder builder = new ResourceBuilder();
		ofNullable(getComments()).ifPresent(builder::setComments);
		ofNullable(getLanguage()).ifPresent(builder::setLanguage);
		ofNullable(getLength()).ifPresent(builder::setLength);
		ofNullable(getMimeType()).ifPresent(builder::setMimeType);
		ofNullable(getName()).ifPresent(builder::setName);
		ofNullable(getResourceId()).ifPresent(builder::setResourceId);
		ofNullable(getResourceType()).ifPresent(builder::setResourceType);
		ofNullable(getSizeInBytes()).ifPresent(builder::setSizeInBytes);
		ofNullable(getString()).ifPresent(builder::setString);
		ofNullable(getUrl()).ifPresent(builder::setUrl);
		return builder;
	}

	public static ResourceBuilder builder() {
		return new ResourceBuilder();
	}

	public static  class ResourceBuilder extends RosettaModelObjectBuilder {
	
		protected String comments;
		protected FieldWithMetaStringBuilder language;
		protected ResourceLength.ResourceLengthBuilder length;
		protected FieldWithMetaStringBuilder mimeType;
		protected String name;
		protected FieldWithMetaStringBuilder resourceId;
		protected FieldWithMetaResourceTypeEnumBuilder resourceType;
		protected BigDecimal sizeInBytes;
		protected String string;
		protected String url;
	
		public ResourceBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Resource> metaData() {
			return metaData;
		} 
	
		
		public String getComments() {
			return comments;
		}
		
		
		public FieldWithMetaStringBuilder getLanguage() {
			return language;
		}
		
		public FieldWithMetaStringBuilder getOrCreateLanguage() {
			if (language!=null) {
				return language;
			}
			else return language = new FieldWithMetaStringBuilder();
		}
		
		
		public ResourceLength.ResourceLengthBuilder getLength() {
			return length;
		}
		
		public ResourceLength.ResourceLengthBuilder getOrCreateLength() {
			if (length!=null) {
				return length;
			}
			else return length = new ResourceLength.ResourceLengthBuilder();
		}
		
		
		public FieldWithMetaStringBuilder getMimeType() {
			return mimeType;
		}
		
		public FieldWithMetaStringBuilder getOrCreateMimeType() {
			if (mimeType!=null) {
				return mimeType;
			}
			else return mimeType = new FieldWithMetaStringBuilder();
		}
		
		
		public String getName() {
			return name;
		}
		
		
		public FieldWithMetaStringBuilder getResourceId() {
			return resourceId;
		}
		
		public FieldWithMetaStringBuilder getOrCreateResourceId() {
			if (resourceId!=null) {
				return resourceId;
			}
			else return resourceId = new FieldWithMetaStringBuilder();
		}
		
		
		public FieldWithMetaResourceTypeEnumBuilder getResourceType() {
			return resourceType;
		}
		
		public FieldWithMetaResourceTypeEnumBuilder getOrCreateResourceType() {
			if (resourceType!=null) {
				return resourceType;
			}
			else return resourceType = new FieldWithMetaResourceTypeEnumBuilder();
		}
		
		
		public BigDecimal getSizeInBytes() {
			return sizeInBytes;
		}
		
		
		public String getString() {
			return string;
		}
		
		
		public String getUrl() {
			return url;
		}
		
	
		public ResourceBuilder setComments(String comments) {
			this.comments = comments;
			return this;
		}
		public ResourceBuilder setLanguage(FieldWithMetaString language) {
			this.language = language.toBuilder();
			return this;
		}
		public ResourceBuilder setLanguageRef(String language) {
			return setLanguage(FieldWithMetaString.builder().setValue(language).build());
		}
		public ResourceBuilder setLength(ResourceLength length) {
			this.length = length.toBuilder();
			return this;
		}
		public ResourceBuilder setLengthBuilder(ResourceLength.ResourceLengthBuilder length) {
			this.length = length;
			return this;
		}
		public ResourceBuilder setMimeType(FieldWithMetaString mimeType) {
			this.mimeType = mimeType.toBuilder();
			return this;
		}
		public ResourceBuilder setMimeTypeRef(String mimeType) {
			return setMimeType(FieldWithMetaString.builder().setValue(mimeType).build());
		}
		public ResourceBuilder setName(String name) {
			this.name = name;
			return this;
		}
		public ResourceBuilder setResourceId(FieldWithMetaString resourceId) {
			this.resourceId = resourceId.toBuilder();
			return this;
		}
		public ResourceBuilder setResourceIdRef(String resourceId) {
			return setResourceId(FieldWithMetaString.builder().setValue(resourceId).build());
		}
		public ResourceBuilder setResourceType(FieldWithMetaResourceTypeEnum resourceType) {
			this.resourceType = resourceType.toBuilder();
			return this;
		}
		public ResourceBuilder setResourceTypeRef(ResourceTypeEnum resourceType) {
			return setResourceType(FieldWithMetaResourceTypeEnum.builder().setValue(resourceType).build());
		}
		public ResourceBuilder setSizeInBytes(BigDecimal sizeInBytes) {
			this.sizeInBytes = sizeInBytes;
			return this;
		}
		public ResourceBuilder setString(String string) {
			this.string = string;
			return this;
		}
		public ResourceBuilder setUrl(String url) {
			this.url = url;
			return this;
		}
		
		public Resource build() {
			return new Resource(this);
		}
	
		@Override
		public ResourceBuilder prune() {
			if (language!=null && !language.prune().hasData()) language = null;
			if (length!=null && !length.prune().hasData()) length = null;
			if (mimeType!=null && !mimeType.prune().hasData()) mimeType = null;
			if (resourceId!=null && !resourceId.prune().hasData()) resourceId = null;
			if (resourceType!=null && !resourceType.prune().hasData()) resourceType = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getComments()!=null) return true;
			if (getLanguage()!=null) return true;
			if (getLength()!=null && getLength().hasData()) return true;
			if (getMimeType()!=null) return true;
			if (getName()!=null) return true;
			if (getResourceId()!=null) return true;
			if (getResourceType()!=null) return true;
			if (getSizeInBytes()!=null) return true;
			if (getString()!=null) return true;
			if (getUrl()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("comments"), String.class, comments, this);
			processor.processBasic(path.newSubPath("name"), String.class, name, this);
			processor.processBasic(path.newSubPath("sizeInBytes"), BigDecimal.class, sizeInBytes, this);
			processor.processBasic(path.newSubPath("string"), String.class, string, this);
			processor.processBasic(path.newSubPath("url"), String.class, url, this);
			
			processRosetta(path.newSubPath("language"), processor, FieldWithMetaString.class, language);
			processRosetta(path.newSubPath("length"), processor, ResourceLength.class, length);
			processRosetta(path.newSubPath("mimeType"), processor, FieldWithMetaString.class, mimeType);
			processRosetta(path.newSubPath("resourceId"), processor, FieldWithMetaString.class, resourceId);
			processRosetta(path.newSubPath("resourceType"), processor, FieldWithMetaResourceTypeEnum.class, resourceType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ResourceBuilder _that = (ResourceBuilder) o;
		
			if (comments != null ? !comments.equals(_that.comments) : _that.comments != null) return false;
			if (language != null ? !language.equals(_that.language) : _that.language != null) return false;
			if (length != null ? !length.equals(_that.length) : _that.length != null) return false;
			if (mimeType != null ? !mimeType.equals(_that.mimeType) : _that.mimeType != null) return false;
			if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
			if (resourceId != null ? !resourceId.equals(_that.resourceId) : _that.resourceId != null) return false;
			if (resourceType != null ? !resourceType.equals(_that.resourceType) : _that.resourceType != null) return false;
			if (sizeInBytes != null ? !sizeInBytes.equals(_that.sizeInBytes) : _that.sizeInBytes != null) return false;
			if (string != null ? !string.equals(_that.string) : _that.string != null) return false;
			if (url != null ? !url.equals(_that.url) : _that.url != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (comments != null ? comments.hashCode() : 0);
			_result = 31 * _result + (language != null ? language.hashCode() : 0);
			_result = 31 * _result + (length != null ? length.hashCode() : 0);
			_result = 31 * _result + (mimeType != null ? mimeType.hashCode() : 0);
			_result = 31 * _result + (name != null ? name.hashCode() : 0);
			_result = 31 * _result + (resourceId != null ? resourceId.hashCode() : 0);
			_result = 31 * _result + (resourceType != null ? resourceType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (sizeInBytes != null ? sizeInBytes.hashCode() : 0);
			_result = 31 * _result + (string != null ? string.hashCode() : 0);
			_result = 31 * _result + (url != null ? url.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResourceBuilder {" +
				"comments=" + this.comments + ", " +
				"language=" + this.language + ", " +
				"length=" + this.length + ", " +
				"mimeType=" + this.mimeType + ", " +
				"name=" + this.name + ", " +
				"resourceId=" + this.resourceId + ", " +
				"resourceType=" + this.resourceType + ", " +
				"sizeInBytes=" + this.sizeInBytes + ", " +
				"string=" + this.string + ", " +
				"url=" + this.url +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("comments"), String.class, comments, this);
		processor.processBasic(path.newSubPath("name"), String.class, name, this);
		processor.processBasic(path.newSubPath("sizeInBytes"), BigDecimal.class, sizeInBytes, this);
		processor.processBasic(path.newSubPath("string"), String.class, string, this);
		processor.processBasic(path.newSubPath("url"), String.class, url, this);
		
		processRosetta(path.newSubPath("language"), processor, FieldWithMetaString.class, language);
		processRosetta(path.newSubPath("length"), processor, ResourceLength.class, length);
		processRosetta(path.newSubPath("mimeType"), processor, FieldWithMetaString.class, mimeType);
		processRosetta(path.newSubPath("resourceId"), processor, FieldWithMetaString.class, resourceId);
		processRosetta(path.newSubPath("resourceType"), processor, FieldWithMetaResourceTypeEnum.class, resourceType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Resource _that = (Resource) o;
	
		if (comments != null ? !comments.equals(_that.comments) : _that.comments != null) return false;
		if (language != null ? !language.equals(_that.language) : _that.language != null) return false;
		if (length != null ? !length.equals(_that.length) : _that.length != null) return false;
		if (mimeType != null ? !mimeType.equals(_that.mimeType) : _that.mimeType != null) return false;
		if (name != null ? !name.equals(_that.name) : _that.name != null) return false;
		if (resourceId != null ? !resourceId.equals(_that.resourceId) : _that.resourceId != null) return false;
		if (resourceType != null ? !resourceType.equals(_that.resourceType) : _that.resourceType != null) return false;
		if (sizeInBytes != null ? !sizeInBytes.equals(_that.sizeInBytes) : _that.sizeInBytes != null) return false;
		if (string != null ? !string.equals(_that.string) : _that.string != null) return false;
		if (url != null ? !url.equals(_that.url) : _that.url != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (comments != null ? comments.hashCode() : 0);
		_result = 31 * _result + (language != null ? language.hashCode() : 0);
		_result = 31 * _result + (length != null ? length.hashCode() : 0);
		_result = 31 * _result + (mimeType != null ? mimeType.hashCode() : 0);
		_result = 31 * _result + (name != null ? name.hashCode() : 0);
		_result = 31 * _result + (resourceId != null ? resourceId.hashCode() : 0);
		_result = 31 * _result + (resourceType != null ? resourceType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (sizeInBytes != null ? sizeInBytes.hashCode() : 0);
		_result = 31 * _result + (string != null ? string.hashCode() : 0);
		_result = 31 * _result + (url != null ? url.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Resource {" +
			"comments=" + this.comments + ", " +
			"language=" + this.language + ", " +
			"length=" + this.length + ", " +
			"mimeType=" + this.mimeType + ", " +
			"name=" + this.name + ", " +
			"resourceId=" + this.resourceId + ", " +
			"resourceType=" + this.resourceType + ", " +
			"sizeInBytes=" + this.sizeInBytes + ", " +
			"string=" + this.string + ", " +
			"url=" + this.url +
		'}';
	}
}
