package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.LengthUnitEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ResourceLengthMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class to indicate the length of the resource.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ResourceLength", source="FpML_5_10")
public class ResourceLength extends RosettaModelObject {
	private final LengthUnitEnum lengthUnit;
	private final BigDecimal lengthValue;
	private static ResourceLengthMeta metaData = new ResourceLengthMeta();
	
	ResourceLength(ResourceLengthBuilder builder) {
		this.lengthUnit = builder.getLengthUnit();
		this.lengthValue = builder.getLengthValue();
	}
	
	/**
	 * The length unit of the resource. For example, pages (pdf, text documents) or time (audio, video files).
	 */
	@RosettaSynonym(value="lengthUnit", source="FpML_5_10")
	@RosettaSynonym(value="lengthUnit", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="lengthUnit", source="DTCC_11_0")
	@RosettaSynonym(value="lengthUnit", source="DTCC_9_0")
	@RosettaSynonym(value="lengthUnit", source="CME_ClearedConfirm_1_17")
	public final LengthUnitEnum getLengthUnit() {
		return lengthUnit;
	}
	
	/**
	 * The length value of the resource.
	 */
	@RosettaSynonym(value="lengthValue", source="FpML_5_10")
	@RosettaSynonym(value="lengthValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="lengthValue", source="DTCC_11_0")
	@RosettaSynonym(value="lengthValue", source="DTCC_9_0")
	@RosettaSynonym(value="lengthValue", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getLengthValue() {
		return lengthValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends ResourceLength> metaData() {
		return metaData;
	} 
	
	public ResourceLengthBuilder toBuilder() {
		ResourceLengthBuilder builder = new ResourceLengthBuilder();
		ofNullable(getLengthUnit()).ifPresent(builder::setLengthUnit);
		ofNullable(getLengthValue()).ifPresent(builder::setLengthValue);
		return builder;
	}

	public static ResourceLengthBuilder builder() {
		return new ResourceLengthBuilder();
	}

	public static  class ResourceLengthBuilder extends RosettaModelObjectBuilder {
	
		protected LengthUnitEnum lengthUnit;
		protected BigDecimal lengthValue;
	
		public ResourceLengthBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ResourceLength> metaData() {
			return metaData;
		} 
	
		
		public LengthUnitEnum getLengthUnit() {
			return lengthUnit;
		}
		
		
		public BigDecimal getLengthValue() {
			return lengthValue;
		}
		
	
		public ResourceLengthBuilder setLengthUnit(LengthUnitEnum lengthUnit) {
			this.lengthUnit = lengthUnit;
			return this;
		}
		public ResourceLengthBuilder setLengthValue(BigDecimal lengthValue) {
			this.lengthValue = lengthValue;
			return this;
		}
		
		public ResourceLength build() {
			return new ResourceLength(this);
		}
	
		@Override
		public ResourceLengthBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLengthUnit()!=null) return true;
			if (getLengthValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("lengthUnit"), LengthUnitEnum.class, lengthUnit, this);
			processor.processBasic(path.newSubPath("lengthValue"), BigDecimal.class, lengthValue, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ResourceLengthBuilder _that = (ResourceLengthBuilder) o;
		
			if (lengthUnit != null ? !lengthUnit.equals(_that.lengthUnit) : _that.lengthUnit != null) return false;
			if (lengthValue != null ? !lengthValue.equals(_that.lengthValue) : _that.lengthValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (lengthUnit != null ? lengthUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (lengthValue != null ? lengthValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ResourceLengthBuilder {" +
				"lengthUnit=" + this.lengthUnit + ", " +
				"lengthValue=" + this.lengthValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("lengthUnit"), LengthUnitEnum.class, lengthUnit, this);
		processor.processBasic(path.newSubPath("lengthValue"), BigDecimal.class, lengthValue, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ResourceLength _that = (ResourceLength) o;
	
		if (lengthUnit != null ? !lengthUnit.equals(_that.lengthUnit) : _that.lengthUnit != null) return false;
		if (lengthValue != null ? !lengthValue.equals(_that.lengthValue) : _that.lengthValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (lengthUnit != null ? lengthUnit.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (lengthValue != null ? lengthValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ResourceLength {" +
			"lengthUnit=" + this.lengthUnit + ", " +
			"lengthValue=" + this.lengthValue +
		'}';
	}
}
