package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of a resource (e.g. document).
 * 
 * @version 2.5.4
 */
public enum ResourceTypeEnum {
    

	/**
	 * Document describing the legal terms of a transaction.
	 */
	@RosettaSynonym(value = "Confirmation", source = "FpML_5_10")
	@RosettaSynonym(value = "Confirmation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Confirmation", source = "DTCC_11_0")
	@RosettaSynonym(value = "Confirmation", source = "DTCC_9_0")
	@RosettaSynonym(value = "Confirmation", source = "CME_ClearedConfirm_1_17")
	CONFIRMATION,
	    

	/**
	 * Document providing supplemental material economic terms to the FpML data representation. The initial intended usage is to fulfill the CFTC Part 45 rule requirement to report ‘Any other terms(s) of the swap matched or affirmed by the counterparties in verifying the swap’ when the reporting is done via the generic FpML representation.
	 */
	@RosettaSynonym(value = "SupplementalMaterialEconomicTerms", source = "FpML_5_10")
	@RosettaSynonym(value = "SupplementalMaterialEconomicTerms", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SupplementalMaterialEconomicTerms", source = "DTCC_11_0")
	@RosettaSynonym(value = "SupplementalMaterialEconomicTerms", source = "DTCC_9_0")
	@RosettaSynonym(value = "SupplementalMaterialEconomicTerms", source = "CME_ClearedConfirm_1_17")
	SUPPLEMENTAL_MATERIAL_ECONOMIC_TERMS,
	    

	/**
	 * Document describing the economic characteristics of a transaction.
	 */
	@RosettaSynonym(value = "TermSheet", source = "FpML_5_10")
	@RosettaSynonym(value = "TermSheet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TermSheet", source = "DTCC_11_0")
	@RosettaSynonym(value = "TermSheet", source = "DTCC_9_0")
	@RosettaSynonym(value = "TermSheet", source = "CME_ClearedConfirm_1_17")
	TERM_SHEET    
	;

	private final String displayName;
	
	ResourceTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
