package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.metafields.FieldWithMetaRestructuringEnum;
import org.isda.cdm.metafields.FieldWithMetaRestructuringEnum.FieldWithMetaRestructuringEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.RestructuringMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Restructuring", source="FpML_5_10")
public class Restructuring extends RosettaModelObject {
	private final Boolean applicable;
	private final Boolean multipleCreditEventNotices;
	private final Boolean multipleHolderObligation;
	private final FieldWithMetaRestructuringEnum restructuringType;
	private static RestructuringMeta metaData = new RestructuringMeta();
	
	Restructuring(RestructuringBuilder builder) {
		this.applicable = builder.getApplicable();
		this.multipleCreditEventNotices = builder.getMultipleCreditEventNotices();
		this.multipleHolderObligation = builder.getMultipleHolderObligation();
		this.restructuringType = ofNullable(builder.getRestructuringType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates whether the restructuring provision is applicable.
	 */
	@RosettaSynonym(value="applicable", source="FpML_5_10")
	@RosettaSynonym(value="applicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="applicable", source="DTCC_11_0")
	@RosettaSynonym(value="applicable", source="DTCC_9_0")
	@RosettaSynonym(value="applicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * Presence of this element and value set to &#39;true&#39; indicates that Section 3.9 of the 2003 Credit Derivatives Definitions shall apply. Absence of this element indicates that Section 3.9 shall not apply. NOTE: Not allowed under ISDA Credit 1999.
	 */
	@RosettaSynonym(value="multipleCreditEventNotices", source="FpML_5_10")
	@RosettaSynonym(value="multipleCreditEventNotices", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="multipleCreditEventNotices", source="DTCC_11_0")
	@RosettaSynonym(value="multipleCreditEventNotices", source="DTCC_9_0")
	@RosettaSynonym(value="multipleCreditEventNotices", source="CME_ClearedConfirm_1_17")
	public final Boolean getMultipleCreditEventNotices() {
		return multipleCreditEventNotices;
	}
	
	/**
	 * In relation to a restructuring credit event, unless multiple holder obligation is not specified restructurings are limited to multiple holder obligations. A multiple holder obligation means an obligation that is held by more than three holders that are not affiliates of each other and where at least two thirds of the holders must agree to the event that constitutes the restructuring credit event. ISDA 2003 Term: Multiple Holder Obligation.
	 */
	@RosettaSynonym(value="multipleHolderObligation", source="FpML_5_10")
	@RosettaSynonym(value="multipleHolderObligation", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="multipleHolderObligation", source="DTCC_11_0")
	@RosettaSynonym(value="multipleHolderObligation", source="DTCC_9_0")
	@RosettaSynonym(value="multipleHolderObligation", source="CME_ClearedConfirm_1_17")
	public final Boolean getMultipleHolderObligation() {
		return multipleHolderObligation;
	}
	
	/**
	 * Specifies the type of restructuring that is applicable.
	 */
	@RosettaSynonym(value="restructuringType", source="FpML_5_10")
	@RosettaSynonym(value="restructuringType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="restructuringType", source="DTCC_11_0")
	@RosettaSynonym(value="restructuringType", source="DTCC_9_0")
	@RosettaSynonym(value="restructuringType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaRestructuringEnum getRestructuringType() {
		return restructuringType;
	}
	
	
	@Override
	public RosettaMetaData<? extends Restructuring> metaData() {
		return metaData;
	} 
	
	public RestructuringBuilder toBuilder() {
		RestructuringBuilder builder = new RestructuringBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getMultipleCreditEventNotices()).ifPresent(builder::setMultipleCreditEventNotices);
		ofNullable(getMultipleHolderObligation()).ifPresent(builder::setMultipleHolderObligation);
		ofNullable(getRestructuringType()).ifPresent(builder::setRestructuringType);
		return builder;
	}

	public static RestructuringBuilder builder() {
		return new RestructuringBuilder();
	}

	public static  class RestructuringBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected Boolean multipleCreditEventNotices;
		protected Boolean multipleHolderObligation;
		protected FieldWithMetaRestructuringEnumBuilder restructuringType;
	
		public RestructuringBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Restructuring> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public Boolean getMultipleCreditEventNotices() {
			return multipleCreditEventNotices;
		}
		
		
		public Boolean getMultipleHolderObligation() {
			return multipleHolderObligation;
		}
		
		
		public FieldWithMetaRestructuringEnumBuilder getRestructuringType() {
			return restructuringType;
		}
		
		public FieldWithMetaRestructuringEnumBuilder getOrCreateRestructuringType() {
			if (restructuringType!=null) {
				return restructuringType;
			}
			else return restructuringType = new FieldWithMetaRestructuringEnumBuilder();
		}
		
	
		public RestructuringBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public RestructuringBuilder setMultipleCreditEventNotices(Boolean multipleCreditEventNotices) {
			this.multipleCreditEventNotices = multipleCreditEventNotices;
			return this;
		}
		public RestructuringBuilder setMultipleHolderObligation(Boolean multipleHolderObligation) {
			this.multipleHolderObligation = multipleHolderObligation;
			return this;
		}
		public RestructuringBuilder setRestructuringType(FieldWithMetaRestructuringEnum restructuringType) {
			this.restructuringType = restructuringType.toBuilder();
			return this;
		}
		public RestructuringBuilder setRestructuringTypeRef(RestructuringEnum restructuringType) {
			return setRestructuringType(FieldWithMetaRestructuringEnum.builder().setValue(restructuringType).build());
		}
		
		public Restructuring build() {
			return new Restructuring(this);
		}
	
		@Override
		public RestructuringBuilder prune() {
			if (restructuringType!=null && !restructuringType.prune().hasData()) restructuringType = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getMultipleCreditEventNotices()!=null) return true;
			if (getMultipleHolderObligation()!=null) return true;
			if (getRestructuringType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			processor.processBasic(path.newSubPath("multipleCreditEventNotices"), Boolean.class, multipleCreditEventNotices, this);
			processor.processBasic(path.newSubPath("multipleHolderObligation"), Boolean.class, multipleHolderObligation, this);
			
			processRosetta(path.newSubPath("restructuringType"), processor, FieldWithMetaRestructuringEnum.class, restructuringType);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			RestructuringBuilder _that = (RestructuringBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (multipleCreditEventNotices != null ? !multipleCreditEventNotices.equals(_that.multipleCreditEventNotices) : _that.multipleCreditEventNotices != null) return false;
			if (multipleHolderObligation != null ? !multipleHolderObligation.equals(_that.multipleHolderObligation) : _that.multipleHolderObligation != null) return false;
			if (restructuringType != null ? !restructuringType.equals(_that.restructuringType) : _that.restructuringType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (multipleCreditEventNotices != null ? multipleCreditEventNotices.hashCode() : 0);
			_result = 31 * _result + (multipleHolderObligation != null ? multipleHolderObligation.hashCode() : 0);
			_result = 31 * _result + (restructuringType != null ? restructuringType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "RestructuringBuilder {" +
				"applicable=" + this.applicable + ", " +
				"multipleCreditEventNotices=" + this.multipleCreditEventNotices + ", " +
				"multipleHolderObligation=" + this.multipleHolderObligation + ", " +
				"restructuringType=" + this.restructuringType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		processor.processBasic(path.newSubPath("multipleCreditEventNotices"), Boolean.class, multipleCreditEventNotices, this);
		processor.processBasic(path.newSubPath("multipleHolderObligation"), Boolean.class, multipleHolderObligation, this);
		
		processRosetta(path.newSubPath("restructuringType"), processor, FieldWithMetaRestructuringEnum.class, restructuringType);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Restructuring _that = (Restructuring) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (multipleCreditEventNotices != null ? !multipleCreditEventNotices.equals(_that.multipleCreditEventNotices) : _that.multipleCreditEventNotices != null) return false;
		if (multipleHolderObligation != null ? !multipleHolderObligation.equals(_that.multipleHolderObligation) : _that.multipleHolderObligation != null) return false;
		if (restructuringType != null ? !restructuringType.equals(_that.restructuringType) : _that.restructuringType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (multipleCreditEventNotices != null ? multipleCreditEventNotices.hashCode() : 0);
		_result = 31 * _result + (multipleHolderObligation != null ? multipleHolderObligation.hashCode() : 0);
		_result = 31 * _result + (restructuringType != null ? restructuringType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Restructuring {" +
			"applicable=" + this.applicable + ", " +
			"multipleCreditEventNotices=" + this.multipleCreditEventNotices + ", " +
			"multipleHolderObligation=" + this.multipleHolderObligation + ", " +
			"restructuringType=" + this.restructuringType +
		'}';
	}
}
