package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the form of the restructuring credit event that is applicable to the credit default swap.
 * 
 * @version 2.5.4
 */
public enum RestructuringEnum {
    

	/**
	 * Restructuring (Section 4.7) and Modified Restructuring Maturity Limitation and Conditionally Transferable Obligation (2014 Definitions: Section 3.31, 2003 Definitions: 2.32) apply.
	 */
	@RosettaSynonym(value = "ModModR", source = "FpML_5_10")
	@RosettaSynonym(value = "ModModR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ModModR", source = "DTCC_11_0")
	@RosettaSynonym(value = "ModModR", source = "DTCC_9_0")
	@RosettaSynonym(value = "ModModR", source = "CME_ClearedConfirm_1_17")
	MOD_MOD_R,
	    

	/**
	 * Restructuring (Section 4.7) and Restructuring Maturity Limitation and Fully Transferable Obligation (2014 Definitions: Section 3.31, 2003 Definitions: 2.32) apply.
	 */
	@RosettaSynonym(value = "ModR", source = "FpML_5_10")
	@RosettaSynonym(value = "ModR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ModR", source = "DTCC_11_0")
	@RosettaSynonym(value = "ModR", source = "DTCC_9_0")
	@RosettaSynonym(value = "ModR", source = "CME_ClearedConfirm_1_17")
	MOD_R,
	    

	/**
	 * Restructuring as defined in the applicable ISDA Credit Derivatives Definitions. (2003 or 2014).
	 */
	@RosettaSynonym(value = "R", source = "FpML_5_10")
	@RosettaSynonym(value = "R", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "R", source = "DTCC_11_0")
	@RosettaSynonym(value = "R", source = "DTCC_9_0")
	@RosettaSynonym(value = "R", source = "CME_ClearedConfirm_1_17")
	R    
	;

	private final String displayName;
	
	RestructuringEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
