package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the type of return associated the equity payout.
 * 
 * @version 2.5.4
 */
public enum ReturnTypeEnum {
    

	/**
	 * Dividend return, i.e. excluding price variations.
	 */
	@RosettaSynonym(value = "Dividend", source = "FpML_5_10")
	DIVIDEND,
	    

	/**
	 * Price return, i.e. excluding dividends.
	 */
	@RosettaSynonym(value = "Price", source = "FpML_5_10")
	PRICE,
	    

	/**
	 * Total return, i.e. including dividend and price components.
	 */
	@RosettaSynonym(value = "Total", source = "FpML_5_10")
	TOTAL,
	    

	/**
	 * Variance return.
	 */
	VARIANCE,
	    

	/**
	 * Volatility return.
	 */
	VOLATILITY    
	;

	private final String displayName;
	
	ReturnTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
