package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the period term as part of a periodic schedule, i.e. the calculation period end date within the regular part of the calculation period. The value could be a rule, e.g. IMM Settlement Dates, which is the 3rd Wednesday of the month, or it could be a specific day of the month, such as the first day of the applicable month.
 * 
 * @version 2.5.4
 */
public enum RollConventionEnum {
    

	/**
	 * Rolls on month end dates irrespective of the length of the month and the previous roll day.
	 */
	@RosettaSynonym(value = "EOM", source = "FpML_5_10")
	@RosettaSynonym(value = "EOM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EOM", source = "DTCC_11_0")
	@RosettaSynonym(value = "EOM", source = "DTCC_9_0")
	@RosettaSynonym(value = "EOM", source = "CME_ClearedConfirm_1_17")
	EOM,
	    

	/**
	 * Roll days are determined according to the FRN Convention or Euro-dollar Convention as described in ISDA 2000 definitions.
	 */
	@RosettaSynonym(value = "FRN", source = "FpML_5_10")
	@RosettaSynonym(value = "FRN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FRN", source = "DTCC_11_0")
	@RosettaSynonym(value = "FRN", source = "DTCC_9_0")
	@RosettaSynonym(value = "FRN", source = "CME_ClearedConfirm_1_17")
	FRN,
	    

	/**
	 * IMM Settlement Dates. The third Wednesday of the (delivery) month.
	 */
	@RosettaSynonym(value = "IMM", source = "FpML_5_10")
	@RosettaSynonym(value = "IMM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IMM", source = "DTCC_11_0")
	@RosettaSynonym(value = "IMM", source = "DTCC_9_0")
	@RosettaSynonym(value = "IMM", source = "CME_ClearedConfirm_1_17")
	IMM,
	    

	/**
	 * The last trading day/expiration day of the Canadian Derivatives Exchange (Bourse de Montreal Inc) Three-month Canadian Bankers&#39; Acceptance Futures (Ticker Symbol BAX). The second London banking day prior to the third Wednesday of the contract month. If the determined day is a Bourse or bank holiday in Montreal or Toronto, the last trading day shall be the previous bank business day. Per Canadian Derivatives Exchange BAX contract specification.
	 */
	@RosettaSynonym(value = "IMMCAD", source = "FpML_5_10")
	@RosettaSynonym(value = "IMMCAD", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IMMCAD", source = "DTCC_11_0")
	@RosettaSynonym(value = "IMMCAD", source = "DTCC_9_0")
	@RosettaSynonym(value = "IMMCAD", source = "CME_ClearedConfirm_1_17")
	IMMCAD,
	    

	/**
	 * The last trading day of the Sydney Futures Exchange 90 Day Bank Accepted Bills Futures contract (see http://www.sfe.com.au/content/sfe/trading/con_specs.pdf). One Sydney business day preceding the second Friday of the relevant settlement.
	 */
	@RosettaSynonym(value = "IMMAUD", source = "FpML_5_10")
	@RosettaSynonym(value = "IMMAUD", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IMMAUD", source = "DTCC_11_0")
	@RosettaSynonym(value = "IMMAUD", source = "DTCC_9_0")
	@RosettaSynonym(value = "IMMAUD", source = "CME_ClearedConfirm_1_17")
	IMMAUD,
	    

	/**
	 * The last trading day of the Sydney Futures Exchange NZ 90 Day Bank Bill Futures contract (see http://www.sfe.com.au/content/sfe/trading/con_specs.pdf). The first Wednesday after the ninth day of the relevant settlement month.
	 */
	@RosettaSynonym(value = "IMMNZD", source = "FpML_5_10")
	@RosettaSynonym(value = "IMMNZD", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IMMNZD", source = "DTCC_11_0")
	@RosettaSynonym(value = "IMMNZD", source = "DTCC_9_0")
	@RosettaSynonym(value = "IMMNZD", source = "CME_ClearedConfirm_1_17")
	IMMNZD,
	    

	/**
	 * Sydney Futures Exchange 90-Day Bank Accepted Bill Futures Settlement Dates. The second Friday of the (delivery) month
	 */
	@RosettaSynonym(value = "SFE", source = "FpML_5_10")
	@RosettaSynonym(value = "SFE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SFE", source = "DTCC_11_0")
	@RosettaSynonym(value = "SFE", source = "DTCC_9_0")
	@RosettaSynonym(value = "SFE", source = "CME_ClearedConfirm_1_17")
	SFE,
	    

	/**
	 * The roll convention is not required. For example, in the case of a daily calculation frequency.
	 */
	@RosettaSynonym(value = "NONE", source = "FpML_5_10")
	@RosettaSynonym(value = "NONE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NONE", source = "DTCC_11_0")
	@RosettaSynonym(value = "NONE", source = "DTCC_9_0")
	@RosettaSynonym(value = "NONE", source = "CME_ClearedConfirm_1_17")
	NONE,
	    

	/**
	 * 13-week and 26-week U.S. Treasury Bill Auction Dates. Each Monday except for U.S. (New York) holidays when it will occur on a Tuesday.
	 */
	@RosettaSynonym(value = "TBILL", source = "FpML_5_10")
	@RosettaSynonym(value = "TBILL", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TBILL", source = "DTCC_11_0")
	@RosettaSynonym(value = "TBILL", source = "DTCC_9_0")
	@RosettaSynonym(value = "TBILL", source = "CME_ClearedConfirm_1_17")
	TBILL,
	    

	/**
	 * Rolls on the 1st day of the month.
	 */
	@RosettaSynonym(value = "1", source = "FpML_5_10")
	@RosettaSynonym(value = "1", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "1", source = "DTCC_11_0")
	@RosettaSynonym(value = "1", source = "DTCC_9_0")
	@RosettaSynonym(value = "1", source = "CME_ClearedConfirm_1_17")
	_1("1"),
	

	/**
	 * Rolls on the 2nd day of the month.
	 */
	@RosettaSynonym(value = "2", source = "FpML_5_10")
	@RosettaSynonym(value = "2", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "2", source = "DTCC_11_0")
	@RosettaSynonym(value = "2", source = "DTCC_9_0")
	@RosettaSynonym(value = "2", source = "CME_ClearedConfirm_1_17")
	_2("2"),
	

	/**
	 * Rolls on the 3rd day of the month.
	 */
	@RosettaSynonym(value = "3", source = "FpML_5_10")
	@RosettaSynonym(value = "3", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "3", source = "DTCC_11_0")
	@RosettaSynonym(value = "3", source = "DTCC_9_0")
	@RosettaSynonym(value = "3", source = "CME_ClearedConfirm_1_17")
	_3("3"),
	

	/**
	 * Rolls on the 4th day of the month.
	 */
	@RosettaSynonym(value = "4", source = "FpML_5_10")
	@RosettaSynonym(value = "4", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "4", source = "DTCC_11_0")
	@RosettaSynonym(value = "4", source = "DTCC_9_0")
	@RosettaSynonym(value = "4", source = "CME_ClearedConfirm_1_17")
	_4("4"),
	

	/**
	 * Rolls on the 5th day of the month.
	 */
	@RosettaSynonym(value = "5", source = "FpML_5_10")
	@RosettaSynonym(value = "5", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "5", source = "DTCC_11_0")
	@RosettaSynonym(value = "5", source = "DTCC_9_0")
	@RosettaSynonym(value = "5", source = "CME_ClearedConfirm_1_17")
	_5("5"),
	

	/**
	 * Rolls on the 6th day of the month.
	 */
	@RosettaSynonym(value = "6", source = "FpML_5_10")
	@RosettaSynonym(value = "6", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "6", source = "DTCC_11_0")
	@RosettaSynonym(value = "6", source = "DTCC_9_0")
	@RosettaSynonym(value = "6", source = "CME_ClearedConfirm_1_17")
	_6("6"),
	

	/**
	 * Rolls on the 7th day of the month.
	 */
	@RosettaSynonym(value = "7", source = "FpML_5_10")
	@RosettaSynonym(value = "7", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "7", source = "DTCC_11_0")
	@RosettaSynonym(value = "7", source = "DTCC_9_0")
	@RosettaSynonym(value = "7", source = "CME_ClearedConfirm_1_17")
	_7("7"),
	

	/**
	 * Rolls on the 8th day of the month.
	 */
	@RosettaSynonym(value = "8", source = "FpML_5_10")
	@RosettaSynonym(value = "8", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "8", source = "DTCC_11_0")
	@RosettaSynonym(value = "8", source = "DTCC_9_0")
	@RosettaSynonym(value = "8", source = "CME_ClearedConfirm_1_17")
	_8("8"),
	

	/**
	 * Rolls on the 9th day of the month.
	 */
	@RosettaSynonym(value = "9", source = "FpML_5_10")
	@RosettaSynonym(value = "9", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "9", source = "DTCC_11_0")
	@RosettaSynonym(value = "9", source = "DTCC_9_0")
	@RosettaSynonym(value = "9", source = "CME_ClearedConfirm_1_17")
	_9("9"),
	

	/**
	 * Rolls on the 10th day of the month.
	 */
	@RosettaSynonym(value = "10", source = "FpML_5_10")
	@RosettaSynonym(value = "10", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "10", source = "DTCC_11_0")
	@RosettaSynonym(value = "10", source = "DTCC_9_0")
	@RosettaSynonym(value = "10", source = "CME_ClearedConfirm_1_17")
	_10("10"),
	

	/**
	 * Rolls on the 11th day of the month.
	 */
	@RosettaSynonym(value = "11", source = "FpML_5_10")
	@RosettaSynonym(value = "11", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "11", source = "DTCC_11_0")
	@RosettaSynonym(value = "11", source = "DTCC_9_0")
	@RosettaSynonym(value = "11", source = "CME_ClearedConfirm_1_17")
	_11("11"),
	

	/**
	 * Rolls on the 12th day of the month.
	 */
	@RosettaSynonym(value = "12", source = "FpML_5_10")
	@RosettaSynonym(value = "12", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "12", source = "DTCC_11_0")
	@RosettaSynonym(value = "12", source = "DTCC_9_0")
	@RosettaSynonym(value = "12", source = "CME_ClearedConfirm_1_17")
	_12("12"),
	

	/**
	 * Rolls on the 13th day of the month.
	 */
	@RosettaSynonym(value = "13", source = "FpML_5_10")
	@RosettaSynonym(value = "13", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "13", source = "DTCC_11_0")
	@RosettaSynonym(value = "13", source = "DTCC_9_0")
	@RosettaSynonym(value = "13", source = "CME_ClearedConfirm_1_17")
	_13("13"),
	

	/**
	 * Rolls on the 14th day of the month.
	 */
	@RosettaSynonym(value = "14", source = "FpML_5_10")
	@RosettaSynonym(value = "14", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "14", source = "DTCC_11_0")
	@RosettaSynonym(value = "14", source = "DTCC_9_0")
	@RosettaSynonym(value = "14", source = "CME_ClearedConfirm_1_17")
	_14("14"),
	

	/**
	 * Rolls on the 15th day of the month.
	 */
	@RosettaSynonym(value = "15", source = "FpML_5_10")
	@RosettaSynonym(value = "15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "15", source = "DTCC_11_0")
	@RosettaSynonym(value = "15", source = "DTCC_9_0")
	@RosettaSynonym(value = "15", source = "CME_ClearedConfirm_1_17")
	_15("15"),
	

	/**
	 * Rolls on the 16th day of the month.
	 */
	@RosettaSynonym(value = "16", source = "FpML_5_10")
	@RosettaSynonym(value = "16", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "16", source = "DTCC_11_0")
	@RosettaSynonym(value = "16", source = "DTCC_9_0")
	@RosettaSynonym(value = "16", source = "CME_ClearedConfirm_1_17")
	_16("16"),
	

	/**
	 * Rolls on the 17th day of the month.
	 */
	@RosettaSynonym(value = "17", source = "FpML_5_10")
	@RosettaSynonym(value = "17", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "17", source = "DTCC_11_0")
	@RosettaSynonym(value = "17", source = "DTCC_9_0")
	@RosettaSynonym(value = "17", source = "CME_ClearedConfirm_1_17")
	_17("17"),
	

	/**
	 * Rolls on the 18th day of the month.
	 */
	@RosettaSynonym(value = "18", source = "FpML_5_10")
	@RosettaSynonym(value = "18", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "18", source = "DTCC_11_0")
	@RosettaSynonym(value = "18", source = "DTCC_9_0")
	@RosettaSynonym(value = "18", source = "CME_ClearedConfirm_1_17")
	_18("18"),
	

	/**
	 * Rolls on the 19th day of the month.
	 */
	@RosettaSynonym(value = "19", source = "FpML_5_10")
	@RosettaSynonym(value = "19", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "19", source = "DTCC_11_0")
	@RosettaSynonym(value = "19", source = "DTCC_9_0")
	@RosettaSynonym(value = "19", source = "CME_ClearedConfirm_1_17")
	_19("19"),
	

	/**
	 * Rolls on the 20th day of the month.
	 */
	@RosettaSynonym(value = "20", source = "FpML_5_10")
	@RosettaSynonym(value = "20", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "20", source = "DTCC_11_0")
	@RosettaSynonym(value = "20", source = "DTCC_9_0")
	@RosettaSynonym(value = "20", source = "CME_ClearedConfirm_1_17")
	_20("20"),
	

	/**
	 * Rolls on the 21st day of the month.
	 */
	@RosettaSynonym(value = "21", source = "FpML_5_10")
	@RosettaSynonym(value = "21", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "21", source = "DTCC_11_0")
	@RosettaSynonym(value = "21", source = "DTCC_9_0")
	@RosettaSynonym(value = "21", source = "CME_ClearedConfirm_1_17")
	_21("21"),
	

	/**
	 * Rolls on the 22nd day of the month.
	 */
	@RosettaSynonym(value = "22", source = "FpML_5_10")
	@RosettaSynonym(value = "22", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "22", source = "DTCC_11_0")
	@RosettaSynonym(value = "22", source = "DTCC_9_0")
	@RosettaSynonym(value = "22", source = "CME_ClearedConfirm_1_17")
	_22("22"),
	

	/**
	 * Rolls on the 23rd day of the month.
	 */
	@RosettaSynonym(value = "23", source = "FpML_5_10")
	@RosettaSynonym(value = "23", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "23", source = "DTCC_11_0")
	@RosettaSynonym(value = "23", source = "DTCC_9_0")
	@RosettaSynonym(value = "23", source = "CME_ClearedConfirm_1_17")
	_23("23"),
	

	/**
	 * Rolls on the 24th day of the month.
	 */
	@RosettaSynonym(value = "24", source = "FpML_5_10")
	@RosettaSynonym(value = "24", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "24", source = "DTCC_11_0")
	@RosettaSynonym(value = "24", source = "DTCC_9_0")
	@RosettaSynonym(value = "24", source = "CME_ClearedConfirm_1_17")
	_24("24"),
	

	/**
	 * Rolls on the 25th day of the month.
	 */
	@RosettaSynonym(value = "25", source = "FpML_5_10")
	@RosettaSynonym(value = "25", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "25", source = "DTCC_11_0")
	@RosettaSynonym(value = "25", source = "DTCC_9_0")
	@RosettaSynonym(value = "25", source = "CME_ClearedConfirm_1_17")
	_25("25"),
	

	/**
	 * Rolls on the 26th day of the month.
	 */
	@RosettaSynonym(value = "26", source = "FpML_5_10")
	@RosettaSynonym(value = "26", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "26", source = "DTCC_11_0")
	@RosettaSynonym(value = "26", source = "DTCC_9_0")
	@RosettaSynonym(value = "26", source = "CME_ClearedConfirm_1_17")
	_26("26"),
	

	/**
	 * Rolls on the 27th day of the month.
	 */
	@RosettaSynonym(value = "27", source = "FpML_5_10")
	@RosettaSynonym(value = "27", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "27", source = "DTCC_11_0")
	@RosettaSynonym(value = "27", source = "DTCC_9_0")
	@RosettaSynonym(value = "27", source = "CME_ClearedConfirm_1_17")
	_27("27"),
	

	/**
	 * Rolls on the 28th day of the month.
	 */
	@RosettaSynonym(value = "28", source = "FpML_5_10")
	@RosettaSynonym(value = "28", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "28", source = "DTCC_11_0")
	@RosettaSynonym(value = "28", source = "DTCC_9_0")
	@RosettaSynonym(value = "28", source = "CME_ClearedConfirm_1_17")
	_28("28"),
	

	/**
	 * Rolls on the 29th day of the month.
	 */
	@RosettaSynonym(value = "29", source = "FpML_5_10")
	@RosettaSynonym(value = "29", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "29", source = "DTCC_11_0")
	@RosettaSynonym(value = "29", source = "DTCC_9_0")
	@RosettaSynonym(value = "29", source = "CME_ClearedConfirm_1_17")
	_29("29"),
	

	/**
	 * Rolls on the 30th day of the month.
	 */
	@RosettaSynonym(value = "30", source = "FpML_5_10")
	@RosettaSynonym(value = "30", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "30", source = "DTCC_11_0")
	@RosettaSynonym(value = "30", source = "DTCC_9_0")
	@RosettaSynonym(value = "30", source = "CME_ClearedConfirm_1_17")
	_30("30"),
	

	/**
	 * Rolling weekly on a Monday.
	 */
	@RosettaSynonym(value = "MON", source = "FpML_5_10")
	@RosettaSynonym(value = "MON", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MON", source = "DTCC_11_0")
	@RosettaSynonym(value = "MON", source = "DTCC_9_0")
	@RosettaSynonym(value = "MON", source = "CME_ClearedConfirm_1_17")
	MON,
	    

	/**
	 * Rolling weekly on a Tuesday
	 */
	@RosettaSynonym(value = "TUE", source = "FpML_5_10")
	@RosettaSynonym(value = "TUE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TUE", source = "DTCC_11_0")
	@RosettaSynonym(value = "TUE", source = "DTCC_9_0")
	@RosettaSynonym(value = "TUE", source = "CME_ClearedConfirm_1_17")
	TUE,
	    

	/**
	 * Rolling weekly on a Wednesday
	 */
	@RosettaSynonym(value = "WED", source = "FpML_5_10")
	@RosettaSynonym(value = "WED", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "WED", source = "DTCC_11_0")
	@RosettaSynonym(value = "WED", source = "DTCC_9_0")
	@RosettaSynonym(value = "WED", source = "CME_ClearedConfirm_1_17")
	WED,
	    

	/**
	 * Rolling weekly on a Thursday
	 */
	@RosettaSynonym(value = "THU", source = "FpML_5_10")
	@RosettaSynonym(value = "THU", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THU", source = "DTCC_11_0")
	@RosettaSynonym(value = "THU", source = "DTCC_9_0")
	@RosettaSynonym(value = "THU", source = "CME_ClearedConfirm_1_17")
	THU,
	    

	/**
	 * Rolling weekly on a Friday
	 */
	@RosettaSynonym(value = "FRI", source = "FpML_5_10")
	@RosettaSynonym(value = "FRI", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "FRI", source = "DTCC_11_0")
	@RosettaSynonym(value = "FRI", source = "DTCC_9_0")
	@RosettaSynonym(value = "FRI", source = "CME_ClearedConfirm_1_17")
	FRI,
	    

	/**
	 * Rolling weekly on a Saturday
	 */
	@RosettaSynonym(value = "SAT", source = "FpML_5_10")
	@RosettaSynonym(value = "SAT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SAT", source = "DTCC_11_0")
	@RosettaSynonym(value = "SAT", source = "DTCC_9_0")
	@RosettaSynonym(value = "SAT", source = "CME_ClearedConfirm_1_17")
	SAT,
	    

	/**
	 * Rolling weekly on a Sunday
	 */
	@RosettaSynonym(value = "SUN", source = "FpML_5_10")
	@RosettaSynonym(value = "SUN", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SUN", source = "DTCC_11_0")
	@RosettaSynonym(value = "SUN", source = "DTCC_9_0")
	@RosettaSynonym(value = "SUN", source = "CME_ClearedConfirm_1_17")
	SUN    
	;

	private final String displayName;
	
	RollConventionEnum() {
		this.displayName = null;
	}

	RollConventionEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
