package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Step;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ScheduleMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a schedule of rates or amounts in terms of an initial value and then a series of step date and value pairs. On each step date the rate or amount changes to the new step value. The series of step date and value pairs are optional. If not specified, this implies that the initial value remains unchanged over time.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Schedule", source="FpML_5_10")
@RosettaSynonym(value="Schedule", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Schedule", source="DTCC_11_0")
@RosettaSynonym(value="Schedule", source="DTCC_9_0")
@RosettaSynonym(value="Schedule", source="CME_ClearedConfirm_1_17")
public class Schedule extends RosettaModelObject implements GlobalKey {
	private final BigDecimal initialValue;
	private final MetaFields meta;
	private final List<Step> step;
	private static ScheduleMeta metaData = new ScheduleMeta();
	
	Schedule(ScheduleBuilder builder) {
		this.initialValue = builder.getInitialValue();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.step = ofNullable(builder.getStep()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The initial rate or amount, as the case may be. An initial rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="initialValue", source="FpML_5_10")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="initialValue", source="FpML_5_10", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="initialValue", source="FpML_5_10", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialValue", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialValue", source="DTCC_11_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialValue", source="DTCC_9_0", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="initialValue", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.floatingRateCalculation")
	@RosettaSynonym(value="fixedRate", source="FpML_5_10", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="fixedRate", source="CME_SubmissionIRS_1_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="fixedRate", source="DTCC_11_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="fixedRate", source="DTCC_9_0", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="fixedRate", source="CME_ClearedConfirm_1_17", path="periodicPayment.fixedAmountCalculation")
	@RosettaSynonym(value="fixedRate", source="FpML_5_10")
	@RosettaSynonym(value="fixedRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="fixedRate", source="DTCC_11_0")
	@RosettaSynonym(value="fixedRate", source="DTCC_9_0")
	@RosettaSynonym(value="fixedRate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="initialValue", source="FpML_5_10", path="fixedRateSchedule")
	public final BigDecimal getInitialValue() {
		return initialValue;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The schedule of step date and value pairs. On each step date the associated step value becomes effective A list of steps may be ordered in the document by ascending step date. An FpML document containing an unordered list of steps is still regarded as a conformant document.
	 */
	@RosettaSynonym(value="step", source="FpML_5_10", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="step", source="CME_SubmissionIRS_1_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="step", source="DTCC_11_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="step", source="DTCC_9_0", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="step", source="CME_ClearedConfirm_1_17", path="calculationPeriodAmount.calculation.fixedRateSchedule")
	@RosettaSynonym(value="step", source="FpML_5_10")
	@RosettaSynonym(value="step", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="step", source="DTCC_11_0")
	@RosettaSynonym(value="step", source="DTCC_9_0")
	@RosettaSynonym(value="step", source="CME_ClearedConfirm_1_17")
	public final List<Step> getStep() {
		return step;
	}
	
	
	@Override
	public RosettaMetaData<? extends Schedule> metaData() {
		return metaData;
	} 
	
	public ScheduleBuilder toBuilder() {
		ScheduleBuilder builder = new ScheduleBuilder();
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static ScheduleBuilder builder() {
		return new ScheduleBuilder();
	}

	public static  class ScheduleBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<ScheduleBuilder> {
	
		protected BigDecimal initialValue;
		protected MetaFields.MetaFieldsBuilder meta;
		protected List<Step.StepBuilder> step;
	
		public ScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Schedule> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getInitialValue() {
			return initialValue;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public List<Step.StepBuilder> getStep() {
			return step;
		}
		
		public Step.StepBuilder getOrCreateStep(int index) {
			if (step==null) {
				this.step = new ArrayList<>();
			}
			return getIndex(step, index, ()->new Step.StepBuilder());
		}
		
	
		public ScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		public ScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public ScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public ScheduleBuilder addStep(Step step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		public ScheduleBuilder addStep(Step step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		public ScheduleBuilder addStep(List<Step> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (Step toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		public ScheduleBuilder addStepBuilder(Step.StepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		public ScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public Schedule build() {
			return new Schedule(this);
		}
	
		@Override
		public ScheduleBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (step!=null) step = step.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getInitialValue()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getStep()!=null && getStep().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("step"), processor, Step.class, step);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ScheduleBuilder _that = (ScheduleBuilder) o;
		
			if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (!ListEquals.listEquals(step, _that.step)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (step != null ? step.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ScheduleBuilder {" +
				"initialValue=" + this.initialValue + ", " +
				"meta=" + this.meta + ", " +
				"step=" + this.step +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("initialValue"), BigDecimal.class, initialValue, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("step"), processor, Step.class, step);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Schedule _that = (Schedule) o;
	
		if (initialValue != null ? !initialValue.equals(_that.initialValue) : _that.initialValue != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (!ListEquals.listEquals(step, _that.step)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (initialValue != null ? initialValue.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (step != null ? step.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Schedule {" +
			"initialValue=" + this.initialValue + ", " +
			"meta=" + this.meta + ", " +
			"step=" + this.step +
		'}';
	}
}
