package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SchmeNmMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class SchmeNm extends RosettaModelObject {
	private final String prtry;
	private static SchmeNmMeta metaData = new SchmeNmMeta();
	
	SchmeNm(SchmeNmBuilder builder) {
		this.prtry = builder.getPrtry();
	}
	
	@RosettaSynonym(value="Prtry", source="ISO20022")
	public final String getPrtry() {
		return prtry;
	}
	
	
	@Override
	public RosettaMetaData<? extends SchmeNm> metaData() {
		return metaData;
	} 
	
	public SchmeNmBuilder toBuilder() {
		SchmeNmBuilder builder = new SchmeNmBuilder();
		ofNullable(getPrtry()).ifPresent(builder::setPrtry);
		return builder;
	}

	public static SchmeNmBuilder builder() {
		return new SchmeNmBuilder();
	}

	public static  class SchmeNmBuilder extends RosettaModelObjectBuilder {
	
		protected String prtry;
	
		public SchmeNmBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SchmeNm> metaData() {
			return metaData;
		} 
	
		
		public String getPrtry() {
			return prtry;
		}
		
	
		public SchmeNmBuilder setPrtry(String prtry) {
			this.prtry = prtry;
			return this;
		}
		
		public SchmeNm build() {
			return new SchmeNm(this);
		}
	
		@Override
		public SchmeNmBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPrtry()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("prtry"), String.class, prtry, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SchmeNmBuilder _that = (SchmeNmBuilder) o;
		
			if (prtry != null ? !prtry.equals(_that.prtry) : _that.prtry != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (prtry != null ? prtry.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SchmeNmBuilder {" +
				"prtry=" + this.prtry +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("prtry"), String.class, prtry, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SchmeNm _that = (SchmeNm) o;
	
		if (prtry != null ? !prtry.equals(_that.prtry) : _that.prtry != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (prtry != null ? prtry.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SchmeNm {" +
			"prtry=" + this.prtry +
		'}';
	}
}
