package org.isda.cdm;

import org.isda.cdm.Bond;
import org.isda.cdm.ConvertibleBond;
import org.isda.cdm.Equity;
import org.isda.cdm.ExchangeTradedFund;
import org.isda.cdm.MortgageBackedSecurity;
import org.isda.cdm.MutualFund;
import org.isda.cdm.Warrant;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Security extends RosettaModelObject {
	private final Bond bond;
	private final ConvertibleBond convertibleBond;
	private final Equity equity;
	private final ExchangeTradedFund exchangeTradedFund;
	private final MortgageBackedSecurity mortgageBackedSecurity;
	private final MutualFund mutualFund;
	private final Warrant warrant;
	private static SecurityMeta metaData = new SecurityMeta();
	
	Security(SecurityBuilder builder) {
		this.bond = ofNullable(builder.getBond()).map(f->f.build()).orElse(null);
		this.convertibleBond = ofNullable(builder.getConvertibleBond()).map(f->f.build()).orElse(null);
		this.equity = ofNullable(builder.getEquity()).map(f->f.build()).orElse(null);
		this.exchangeTradedFund = ofNullable(builder.getExchangeTradedFund()).map(f->f.build()).orElse(null);
		this.mortgageBackedSecurity = ofNullable(builder.getMortgageBackedSecurity()).map(f->f.build()).orElse(null);
		this.mutualFund = ofNullable(builder.getMutualFund()).map(f->f.build()).orElse(null);
		this.warrant = ofNullable(builder.getWarrant()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="", source="FpML_5_10")
	@RosettaSynonym(value="bond", source="FpML_5_10")
	@RosettaSynonym(value="bond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="bond", source="DTCC_11_0")
	@RosettaSynonym(value="bond", source="DTCC_9_0")
	@RosettaSynonym(value="bond", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="bond", source="Rosetta_Workbench")
	public final Bond getBond() {
		return bond;
	}
	
	@RosettaSynonym(value="convertibleBond", source="FpML_5_10")
	@RosettaSynonym(value="convertibleBond", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="convertibleBond", source="DTCC_11_0")
	@RosettaSynonym(value="convertibleBond", source="DTCC_9_0")
	@RosettaSynonym(value="convertibleBond", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="convertibleBond", source="Rosetta_Workbench")
	public final ConvertibleBond getConvertibleBond() {
		return convertibleBond;
	}
	
	@RosettaSynonym(value="equity", source="FpML_5_10")
	public final Equity getEquity() {
		return equity;
	}
	
	@RosettaSynonym(value="exchangeTradedFund", source="FpML_5_10")
	public final ExchangeTradedFund getExchangeTradedFund() {
		return exchangeTradedFund;
	}
	
	@RosettaSynonym(value="mortgage", source="FpML_5_10")
	@RosettaSynonym(value="mortgage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="mortgage", source="DTCC_11_0")
	@RosettaSynonym(value="mortgage", source="DTCC_9_0")
	@RosettaSynonym(value="mortgage", source="CME_ClearedConfirm_1_17")
	public final MortgageBackedSecurity getMortgageBackedSecurity() {
		return mortgageBackedSecurity;
	}
	
	@RosettaSynonym(value="mutualFund", source="FpML_5_10")
	public final MutualFund getMutualFund() {
		return mutualFund;
	}
	
	@RosettaSynonym(value="warrant", source="FpML_5_10")
	public final Warrant getWarrant() {
		return warrant;
	}
	
	
	@Override
	public RosettaMetaData<? extends Security> metaData() {
		return metaData;
	} 
	
	public SecurityBuilder toBuilder() {
		SecurityBuilder builder = new SecurityBuilder();
		ofNullable(getBond()).ifPresent(builder::setBond);
		ofNullable(getConvertibleBond()).ifPresent(builder::setConvertibleBond);
		ofNullable(getEquity()).ifPresent(builder::setEquity);
		ofNullable(getExchangeTradedFund()).ifPresent(builder::setExchangeTradedFund);
		ofNullable(getMortgageBackedSecurity()).ifPresent(builder::setMortgageBackedSecurity);
		ofNullable(getMutualFund()).ifPresent(builder::setMutualFund);
		ofNullable(getWarrant()).ifPresent(builder::setWarrant);
		return builder;
	}

	public static SecurityBuilder builder() {
		return new SecurityBuilder();
	}

	public static  class SecurityBuilder extends RosettaModelObjectBuilder {
	
		protected Bond.BondBuilder bond;
		protected ConvertibleBond.ConvertibleBondBuilder convertibleBond;
		protected Equity.EquityBuilder equity;
		protected ExchangeTradedFund.ExchangeTradedFundBuilder exchangeTradedFund;
		protected MortgageBackedSecurity.MortgageBackedSecurityBuilder mortgageBackedSecurity;
		protected MutualFund.MutualFundBuilder mutualFund;
		protected Warrant.WarrantBuilder warrant;
	
		public SecurityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Security> metaData() {
			return metaData;
		} 
	
		
		public Bond.BondBuilder getBond() {
			return bond;
		}
		
		public Bond.BondBuilder getOrCreateBond() {
			if (bond!=null) {
				return bond;
			}
			else return bond = new Bond.BondBuilder();
		}
		
		
		public ConvertibleBond.ConvertibleBondBuilder getConvertibleBond() {
			return convertibleBond;
		}
		
		public ConvertibleBond.ConvertibleBondBuilder getOrCreateConvertibleBond() {
			if (convertibleBond!=null) {
				return convertibleBond;
			}
			else return convertibleBond = new ConvertibleBond.ConvertibleBondBuilder();
		}
		
		
		public Equity.EquityBuilder getEquity() {
			return equity;
		}
		
		public Equity.EquityBuilder getOrCreateEquity() {
			if (equity!=null) {
				return equity;
			}
			else return equity = new Equity.EquityBuilder();
		}
		
		
		public ExchangeTradedFund.ExchangeTradedFundBuilder getExchangeTradedFund() {
			return exchangeTradedFund;
		}
		
		public ExchangeTradedFund.ExchangeTradedFundBuilder getOrCreateExchangeTradedFund() {
			if (exchangeTradedFund!=null) {
				return exchangeTradedFund;
			}
			else return exchangeTradedFund = new ExchangeTradedFund.ExchangeTradedFundBuilder();
		}
		
		
		public MortgageBackedSecurity.MortgageBackedSecurityBuilder getMortgageBackedSecurity() {
			return mortgageBackedSecurity;
		}
		
		public MortgageBackedSecurity.MortgageBackedSecurityBuilder getOrCreateMortgageBackedSecurity() {
			if (mortgageBackedSecurity!=null) {
				return mortgageBackedSecurity;
			}
			else return mortgageBackedSecurity = new MortgageBackedSecurity.MortgageBackedSecurityBuilder();
		}
		
		
		public MutualFund.MutualFundBuilder getMutualFund() {
			return mutualFund;
		}
		
		public MutualFund.MutualFundBuilder getOrCreateMutualFund() {
			if (mutualFund!=null) {
				return mutualFund;
			}
			else return mutualFund = new MutualFund.MutualFundBuilder();
		}
		
		
		public Warrant.WarrantBuilder getWarrant() {
			return warrant;
		}
		
		public Warrant.WarrantBuilder getOrCreateWarrant() {
			if (warrant!=null) {
				return warrant;
			}
			else return warrant = new Warrant.WarrantBuilder();
		}
		
	
		public SecurityBuilder setBond(Bond bond) {
			this.bond = bond.toBuilder();
			return this;
		}
		public SecurityBuilder setBondBuilder(Bond.BondBuilder bond) {
			this.bond = bond;
			return this;
		}
		public SecurityBuilder setConvertibleBond(ConvertibleBond convertibleBond) {
			this.convertibleBond = convertibleBond.toBuilder();
			return this;
		}
		public SecurityBuilder setConvertibleBondBuilder(ConvertibleBond.ConvertibleBondBuilder convertibleBond) {
			this.convertibleBond = convertibleBond;
			return this;
		}
		public SecurityBuilder setEquity(Equity equity) {
			this.equity = equity.toBuilder();
			return this;
		}
		public SecurityBuilder setEquityBuilder(Equity.EquityBuilder equity) {
			this.equity = equity;
			return this;
		}
		public SecurityBuilder setExchangeTradedFund(ExchangeTradedFund exchangeTradedFund) {
			this.exchangeTradedFund = exchangeTradedFund.toBuilder();
			return this;
		}
		public SecurityBuilder setExchangeTradedFundBuilder(ExchangeTradedFund.ExchangeTradedFundBuilder exchangeTradedFund) {
			this.exchangeTradedFund = exchangeTradedFund;
			return this;
		}
		public SecurityBuilder setMortgageBackedSecurity(MortgageBackedSecurity mortgageBackedSecurity) {
			this.mortgageBackedSecurity = mortgageBackedSecurity.toBuilder();
			return this;
		}
		public SecurityBuilder setMortgageBackedSecurityBuilder(MortgageBackedSecurity.MortgageBackedSecurityBuilder mortgageBackedSecurity) {
			this.mortgageBackedSecurity = mortgageBackedSecurity;
			return this;
		}
		public SecurityBuilder setMutualFund(MutualFund mutualFund) {
			this.mutualFund = mutualFund.toBuilder();
			return this;
		}
		public SecurityBuilder setMutualFundBuilder(MutualFund.MutualFundBuilder mutualFund) {
			this.mutualFund = mutualFund;
			return this;
		}
		public SecurityBuilder setWarrant(Warrant warrant) {
			this.warrant = warrant.toBuilder();
			return this;
		}
		public SecurityBuilder setWarrantBuilder(Warrant.WarrantBuilder warrant) {
			this.warrant = warrant;
			return this;
		}
		
		public Security build() {
			return new Security(this);
		}
	
		@Override
		public SecurityBuilder prune() {
			if (bond!=null && !bond.prune().hasData()) bond = null;
			if (convertibleBond!=null && !convertibleBond.prune().hasData()) convertibleBond = null;
			if (equity!=null && !equity.prune().hasData()) equity = null;
			if (exchangeTradedFund!=null && !exchangeTradedFund.prune().hasData()) exchangeTradedFund = null;
			if (mortgageBackedSecurity!=null && !mortgageBackedSecurity.prune().hasData()) mortgageBackedSecurity = null;
			if (mutualFund!=null && !mutualFund.prune().hasData()) mutualFund = null;
			if (warrant!=null && !warrant.prune().hasData()) warrant = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBond()!=null && getBond().hasData()) return true;
			if (getConvertibleBond()!=null && getConvertibleBond().hasData()) return true;
			if (getEquity()!=null && getEquity().hasData()) return true;
			if (getExchangeTradedFund()!=null && getExchangeTradedFund().hasData()) return true;
			if (getMortgageBackedSecurity()!=null && getMortgageBackedSecurity().hasData()) return true;
			if (getMutualFund()!=null && getMutualFund().hasData()) return true;
			if (getWarrant()!=null && getWarrant().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
			processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
			processRosetta(path.newSubPath("equity"), processor, Equity.class, equity);
			processRosetta(path.newSubPath("exchangeTradedFund"), processor, ExchangeTradedFund.class, exchangeTradedFund);
			processRosetta(path.newSubPath("mortgageBackedSecurity"), processor, MortgageBackedSecurity.class, mortgageBackedSecurity);
			processRosetta(path.newSubPath("mutualFund"), processor, MutualFund.class, mutualFund);
			processRosetta(path.newSubPath("warrant"), processor, Warrant.class, warrant);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityBuilder _that = (SecurityBuilder) o;
		
			if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
			if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
			if (equity != null ? !equity.equals(_that.equity) : _that.equity != null) return false;
			if (exchangeTradedFund != null ? !exchangeTradedFund.equals(_that.exchangeTradedFund) : _that.exchangeTradedFund != null) return false;
			if (mortgageBackedSecurity != null ? !mortgageBackedSecurity.equals(_that.mortgageBackedSecurity) : _that.mortgageBackedSecurity != null) return false;
			if (mutualFund != null ? !mutualFund.equals(_that.mutualFund) : _that.mutualFund != null) return false;
			if (warrant != null ? !warrant.equals(_that.warrant) : _that.warrant != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
			_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
			_result = 31 * _result + (equity != null ? equity.hashCode() : 0);
			_result = 31 * _result + (exchangeTradedFund != null ? exchangeTradedFund.hashCode() : 0);
			_result = 31 * _result + (mortgageBackedSecurity != null ? mortgageBackedSecurity.hashCode() : 0);
			_result = 31 * _result + (mutualFund != null ? mutualFund.hashCode() : 0);
			_result = 31 * _result + (warrant != null ? warrant.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityBuilder {" +
				"bond=" + this.bond + ", " +
				"convertibleBond=" + this.convertibleBond + ", " +
				"equity=" + this.equity + ", " +
				"exchangeTradedFund=" + this.exchangeTradedFund + ", " +
				"mortgageBackedSecurity=" + this.mortgageBackedSecurity + ", " +
				"mutualFund=" + this.mutualFund + ", " +
				"warrant=" + this.warrant +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("bond"), processor, Bond.class, bond);
		processRosetta(path.newSubPath("convertibleBond"), processor, ConvertibleBond.class, convertibleBond);
		processRosetta(path.newSubPath("equity"), processor, Equity.class, equity);
		processRosetta(path.newSubPath("exchangeTradedFund"), processor, ExchangeTradedFund.class, exchangeTradedFund);
		processRosetta(path.newSubPath("mortgageBackedSecurity"), processor, MortgageBackedSecurity.class, mortgageBackedSecurity);
		processRosetta(path.newSubPath("mutualFund"), processor, MutualFund.class, mutualFund);
		processRosetta(path.newSubPath("warrant"), processor, Warrant.class, warrant);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Security _that = (Security) o;
	
		if (bond != null ? !bond.equals(_that.bond) : _that.bond != null) return false;
		if (convertibleBond != null ? !convertibleBond.equals(_that.convertibleBond) : _that.convertibleBond != null) return false;
		if (equity != null ? !equity.equals(_that.equity) : _that.equity != null) return false;
		if (exchangeTradedFund != null ? !exchangeTradedFund.equals(_that.exchangeTradedFund) : _that.exchangeTradedFund != null) return false;
		if (mortgageBackedSecurity != null ? !mortgageBackedSecurity.equals(_that.mortgageBackedSecurity) : _that.mortgageBackedSecurity != null) return false;
		if (mutualFund != null ? !mutualFund.equals(_that.mutualFund) : _that.mutualFund != null) return false;
		if (warrant != null ? !warrant.equals(_that.warrant) : _that.warrant != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (bond != null ? bond.hashCode() : 0);
		_result = 31 * _result + (convertibleBond != null ? convertibleBond.hashCode() : 0);
		_result = 31 * _result + (equity != null ? equity.hashCode() : 0);
		_result = 31 * _result + (exchangeTradedFund != null ? exchangeTradedFund.hashCode() : 0);
		_result = 31 * _result + (mortgageBackedSecurity != null ? mortgageBackedSecurity.hashCode() : 0);
		_result = 31 * _result + (mutualFund != null ? mutualFund.hashCode() : 0);
		_result = 31 * _result + (warrant != null ? warrant.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Security {" +
			"bond=" + this.bond + ", " +
			"convertibleBond=" + this.convertibleBond + ", " +
			"equity=" + this.equity + ", " +
			"exchangeTradedFund=" + this.exchangeTradedFund + ", " +
			"mortgageBackedSecurity=" + this.mortgageBackedSecurity + ", " +
			"mutualFund=" + this.mutualFund + ", " +
			"warrant=" + this.warrant +
		'}';
	}
}
