package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Security;
import org.isda.cdm.TransferorTransferee;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityTransferBreakdownMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import java.math.BigDecimal;

/**
 * @version 2.5.4
 */
@RosettaClass
public class SecurityTransferBreakdown extends RosettaModelObject {
	private final BigDecimal quantity;
	private final List<Security> security;
	private final TransferorTransferee transferorTransferee;
	private static SecurityTransferBreakdownMeta metaData = new SecurityTransferBreakdownMeta();
	
	SecurityTransferBreakdown(SecurityTransferBreakdownBuilder builder) {
		this.quantity = builder.getQuantity();
		this.security = ofNullable(builder.getSecurity()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.transferorTransferee = ofNullable(builder.getTransferorTransferee()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="quantity", source="Rosetta_Workbench")
	public final BigDecimal getQuantity() {
		return quantity;
	}
	
	/**
	 * The securities that are being transfered
	 */
	@RosettaSynonym(value="security", source="Rosetta_Workbench")
	public final List<Security> getSecurity() {
		return security;
	}
	
	/**
	 * The transferee and transferor party information.
	 */
	public final TransferorTransferee getTransferorTransferee() {
		return transferorTransferee;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityTransferBreakdown> metaData() {
		return metaData;
	} 
	
	public SecurityTransferBreakdownBuilder toBuilder() {
		SecurityTransferBreakdownBuilder builder = new SecurityTransferBreakdownBuilder();
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getSecurity()).ifPresent(security -> security.forEach(builder::addSecurity));
		ofNullable(getTransferorTransferee()).ifPresent(builder::setTransferorTransferee);
		return builder;
	}

	public static SecurityTransferBreakdownBuilder builder() {
		return new SecurityTransferBreakdownBuilder();
	}

	public static  class SecurityTransferBreakdownBuilder extends RosettaModelObjectBuilder {
	
		protected BigDecimal quantity;
		protected List<Security.SecurityBuilder> security;
		protected TransferorTransferee.TransferorTransfereeBuilder transferorTransferee;
	
		public SecurityTransferBreakdownBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityTransferBreakdown> metaData() {
			return metaData;
		} 
	
		
		public BigDecimal getQuantity() {
			return quantity;
		}
		
		
		public List<Security.SecurityBuilder> getSecurity() {
			return security;
		}
		
		public Security.SecurityBuilder getOrCreateSecurity(int index) {
			if (security==null) {
				this.security = new ArrayList<>();
			}
			return getIndex(security, index, ()->new Security.SecurityBuilder());
		}
		
		
		public TransferorTransferee.TransferorTransfereeBuilder getTransferorTransferee() {
			return transferorTransferee;
		}
		
		public TransferorTransferee.TransferorTransfereeBuilder getOrCreateTransferorTransferee() {
			if (transferorTransferee!=null) {
				return transferorTransferee;
			}
			else return transferorTransferee = new TransferorTransferee.TransferorTransfereeBuilder();
		}
		
	
		public SecurityTransferBreakdownBuilder setQuantity(BigDecimal quantity) {
			this.quantity = quantity;
			return this;
		}
		public SecurityTransferBreakdownBuilder addSecurity(Security security) {
			if(this.security == null){
				this.security = new ArrayList<>();
			}
			this.security.add(security.toBuilder());
			return this;
		}
		public SecurityTransferBreakdownBuilder addSecurity(Security security, int _idx) {
			if(this.security == null){
				this.security = new ArrayList<>();
			}
			getIndex(this.security, _idx, () -> security.toBuilder());
			this.security.set(_idx, security.toBuilder());
			return this;
		}
		public SecurityTransferBreakdownBuilder addSecurity(List<Security> securitys) {
			if(this.security == null){
				this.security = new ArrayList<>();
			}
			for (Security toAdd : securitys) {
				this.security.add(toAdd.toBuilder());
			}
			return this;
		}
		public SecurityTransferBreakdownBuilder addSecurityBuilder(Security.SecurityBuilder security) {
			if(this.security == null){
				this.security = new ArrayList<>();
			}
			this.security.add(security);
			return this;
		}
		
		public SecurityTransferBreakdownBuilder clearSecurity() {
			this.security = null;
			return this;
		}
		public SecurityTransferBreakdownBuilder setTransferorTransferee(TransferorTransferee transferorTransferee) {
			this.transferorTransferee = transferorTransferee.toBuilder();
			return this;
		}
		public SecurityTransferBreakdownBuilder setTransferorTransfereeBuilder(TransferorTransferee.TransferorTransfereeBuilder transferorTransferee) {
			this.transferorTransferee = transferorTransferee;
			return this;
		}
		
		public SecurityTransferBreakdown build() {
			return new SecurityTransferBreakdown(this);
		}
	
		@Override
		public SecurityTransferBreakdownBuilder prune() {
			if (security!=null) security = security.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (transferorTransferee!=null && !transferorTransferee.prune().hasData()) transferorTransferee = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getQuantity()!=null) return true;
			if (getSecurity()!=null && getSecurity().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTransferorTransferee()!=null && getTransferorTransferee().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
			
			processRosetta(path.newSubPath("security"), processor, Security.class, security);
			processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SecurityTransferBreakdownBuilder _that = (SecurityTransferBreakdownBuilder) o;
		
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (!ListEquals.listEquals(security, _that.security)) return false;
			if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (security != null ? security.hashCode() : 0);
			_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityTransferBreakdownBuilder {" +
				"quantity=" + this.quantity + ", " +
				"security=" + this.security + ", " +
				"transferorTransferee=" + this.transferorTransferee +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
		
		processRosetta(path.newSubPath("security"), processor, Security.class, security);
		processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SecurityTransferBreakdown _that = (SecurityTransferBreakdown) o;
	
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (!ListEquals.listEquals(security, _that.security)) return false;
		if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (security != null ? security.hashCode() : 0);
		_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityTransferBreakdown {" +
			"quantity=" + this.quantity + ", " +
			"security=" + this.security + ", " +
			"transferorTransferee=" + this.transferorTransferee +
		'}';
	}
}
