package org.isda.cdm;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AssetTransferTypeEnum;
import org.isda.cdm.Security;
import org.isda.cdm.SecurityTransferBreakdown;
import org.isda.cdm.TransferorTransferee;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SecurityTransferComponentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * @version 2.5.4
 */
@RosettaClass
public class SecurityTransferComponent extends TransferBase {
	private final AssetTransferTypeEnum assetTransferType;
	private final List<SecurityTransferBreakdown> breakdown;
	private final BigDecimal quantity;
	private final Security security;
	private final TransferorTransferee transferorTransferee;
	private static SecurityTransferComponentMeta metaData = new SecurityTransferComponentMeta();
	
	SecurityTransferComponent(SecurityTransferComponentBuilder builder) {
		super(builder);
		this.assetTransferType = builder.getAssetTransferType();
		this.breakdown = ofNullable(builder.getBreakdown()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.quantity = builder.getQuantity();
		this.security = ofNullable(builder.getSecurity()).map(f->f.build()).orElse(null);
		this.transferorTransferee = ofNullable(builder.getTransferorTransferee()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="assetTransferType", source="Rosetta_Workbench")
	public final AssetTransferTypeEnum getAssetTransferType() {
		return assetTransferType;
	}
	
	/**
	 * The security transfer breakdown, when the transfer corresponds to a net transfer across several components which breakdown is deemed relevant (e.g. the net security transfer related to several contracts).
	 */
	@RosettaSynonym(value="breakdown", source="Rosetta_Workbench")
	public final List<SecurityTransferBreakdown> getBreakdown() {
		return breakdown;
	}
	
	@RosettaSynonym(value="quantity", source="Rosetta_Workbench")
	public final BigDecimal getQuantity() {
		return quantity;
	}
	
	/**
	 * The security that is being transfered
	 */
	@RosettaSynonym(value="security", source="Rosetta_Workbench")
	public final Security getSecurity() {
		return security;
	}
	
	/**
	 * The transferee and transferor party information.
	 */
	public final TransferorTransferee getTransferorTransferee() {
		return transferorTransferee;
	}
	
	
	@Override
	public RosettaMetaData<? extends SecurityTransferComponent> metaData() {
		return metaData;
	} 
	
	public SecurityTransferComponentBuilder toBuilder() {
		SecurityTransferComponentBuilder builder = new SecurityTransferComponentBuilder();
		ofNullable(getAssetTransferType()).ifPresent(builder::setAssetTransferType);
		ofNullable(getBreakdown()).ifPresent(breakdown -> breakdown.forEach(builder::addBreakdown));
		ofNullable(getQuantity()).ifPresent(builder::setQuantity);
		ofNullable(getSecurity()).ifPresent(builder::setSecurity);
		ofNullable(getTransferorTransferee()).ifPresent(builder::setTransferorTransferee);
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static SecurityTransferComponentBuilder builder() {
		return new SecurityTransferComponentBuilder();
	}

	public static  class SecurityTransferComponentBuilder extends TransferBaseBuilder {
	
		protected AssetTransferTypeEnum assetTransferType;
		protected List<SecurityTransferBreakdown.SecurityTransferBreakdownBuilder> breakdown;
		protected BigDecimal quantity;
		protected Security.SecurityBuilder security;
		protected TransferorTransferee.TransferorTransfereeBuilder transferorTransferee;
	
		public SecurityTransferComponentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SecurityTransferComponent> metaData() {
			return metaData;
		} 
	
		
		public AssetTransferTypeEnum getAssetTransferType() {
			return assetTransferType;
		}
		
		
		public List<SecurityTransferBreakdown.SecurityTransferBreakdownBuilder> getBreakdown() {
			return breakdown;
		}
		
		public SecurityTransferBreakdown.SecurityTransferBreakdownBuilder getOrCreateBreakdown(int index) {
			if (breakdown==null) {
				this.breakdown = new ArrayList<>();
			}
			return getIndex(breakdown, index, ()->new SecurityTransferBreakdown.SecurityTransferBreakdownBuilder());
		}
		
		
		public BigDecimal getQuantity() {
			return quantity;
		}
		
		
		public Security.SecurityBuilder getSecurity() {
			return security;
		}
		
		public Security.SecurityBuilder getOrCreateSecurity() {
			if (security!=null) {
				return security;
			}
			else return security = new Security.SecurityBuilder();
		}
		
		
		public TransferorTransferee.TransferorTransfereeBuilder getTransferorTransferee() {
			return transferorTransferee;
		}
		
		public TransferorTransferee.TransferorTransfereeBuilder getOrCreateTransferorTransferee() {
			if (transferorTransferee!=null) {
				return transferorTransferee;
			}
			else return transferorTransferee = new TransferorTransferee.TransferorTransfereeBuilder();
		}
		
	
		public SecurityTransferComponentBuilder setAssetTransferType(AssetTransferTypeEnum assetTransferType) {
			this.assetTransferType = assetTransferType;
			return this;
		}
		public SecurityTransferComponentBuilder addBreakdown(SecurityTransferBreakdown breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown.toBuilder());
			return this;
		}
		public SecurityTransferComponentBuilder addBreakdown(SecurityTransferBreakdown breakdown, int _idx) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			getIndex(this.breakdown, _idx, () -> breakdown.toBuilder());
			this.breakdown.set(_idx, breakdown.toBuilder());
			return this;
		}
		public SecurityTransferComponentBuilder addBreakdown(List<SecurityTransferBreakdown> breakdowns) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			for (SecurityTransferBreakdown toAdd : breakdowns) {
				this.breakdown.add(toAdd.toBuilder());
			}
			return this;
		}
		public SecurityTransferComponentBuilder addBreakdownBuilder(SecurityTransferBreakdown.SecurityTransferBreakdownBuilder breakdown) {
			if(this.breakdown == null){
				this.breakdown = new ArrayList<>();
			}
			this.breakdown.add(breakdown);
			return this;
		}
		
		public SecurityTransferComponentBuilder clearBreakdown() {
			this.breakdown = null;
			return this;
		}
		public SecurityTransferComponentBuilder setQuantity(BigDecimal quantity) {
			this.quantity = quantity;
			return this;
		}
		public SecurityTransferComponentBuilder setSecurity(Security security) {
			this.security = security.toBuilder();
			return this;
		}
		public SecurityTransferComponentBuilder setSecurityBuilder(Security.SecurityBuilder security) {
			this.security = security;
			return this;
		}
		public SecurityTransferComponentBuilder setTransferorTransferee(TransferorTransferee transferorTransferee) {
			this.transferorTransferee = transferorTransferee.toBuilder();
			return this;
		}
		public SecurityTransferComponentBuilder setTransferorTransfereeBuilder(TransferorTransferee.TransferorTransfereeBuilder transferorTransferee) {
			this.transferorTransferee = transferorTransferee;
			return this;
		}
		@Override public SecurityTransferComponentBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public SecurityTransferComponentBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public SecurityTransferComponentBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public SecurityTransferComponentBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public SecurityTransferComponent build() {
			return new SecurityTransferComponent(this);
		}
	
		@Override
		public SecurityTransferComponentBuilder prune() {
			super.prune();
			if (breakdown!=null) breakdown = breakdown.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (security!=null && !security.prune().hasData()) security = null;
			if (transferorTransferee!=null && !transferorTransferee.prune().hasData()) transferorTransferee = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getAssetTransferType()!=null) return true;
			if (getBreakdown()!=null && getBreakdown().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getQuantity()!=null) return true;
			if (getSecurity()!=null && getSecurity().hasData()) return true;
			if (getTransferorTransferee()!=null && getTransferorTransferee().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			processor.processBasic(path.newSubPath("assetTransferType"), AssetTransferTypeEnum.class, assetTransferType, this);
			processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
			
			processRosetta(path.newSubPath("breakdown"), processor, SecurityTransferBreakdown.class, breakdown);
			processRosetta(path.newSubPath("security"), processor, Security.class, security);
			processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			SecurityTransferComponentBuilder _that = (SecurityTransferComponentBuilder) o;
		
			if (assetTransferType != null ? !assetTransferType.equals(_that.assetTransferType) : _that.assetTransferType != null) return false;
			if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
			if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
			if (security != null ? !security.equals(_that.security) : _that.security != null) return false;
			if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (assetTransferType != null ? assetTransferType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
			_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
			_result = 31 * _result + (security != null ? security.hashCode() : 0);
			_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityTransferComponentBuilder {" +
				"assetTransferType=" + this.assetTransferType + ", " +
				"breakdown=" + this.breakdown + ", " +
				"quantity=" + this.quantity + ", " +
				"security=" + this.security + ", " +
				"transferorTransferee=" + this.transferorTransferee +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		processor.processBasic(path.newSubPath("assetTransferType"), AssetTransferTypeEnum.class, assetTransferType, this);
		processor.processBasic(path.newSubPath("quantity"), BigDecimal.class, quantity, this);
		
		processRosetta(path.newSubPath("breakdown"), processor, SecurityTransferBreakdown.class, breakdown);
		processRosetta(path.newSubPath("security"), processor, Security.class, security);
		processRosetta(path.newSubPath("transferorTransferee"), processor, TransferorTransferee.class, transferorTransferee);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		SecurityTransferComponent _that = (SecurityTransferComponent) o;
	
		if (assetTransferType != null ? !assetTransferType.equals(_that.assetTransferType) : _that.assetTransferType != null) return false;
		if (!ListEquals.listEquals(breakdown, _that.breakdown)) return false;
		if (quantity != null ? !quantity.equals(_that.quantity) : _that.quantity != null) return false;
		if (security != null ? !security.equals(_that.security) : _that.security != null) return false;
		if (transferorTransferee != null ? !transferorTransferee.equals(_that.transferorTransferee) : _that.transferorTransferee != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (assetTransferType != null ? assetTransferType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (breakdown != null ? breakdown.hashCode() : 0);
		_result = 31 * _result + (quantity != null ? quantity.hashCode() : 0);
		_result = 31 * _result + (security != null ? security.hashCode() : 0);
		_result = 31 * _result + (transferorTransferee != null ? transferorTransferee.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SecurityTransferComponent {" +
			"assetTransferType=" + this.assetTransferType + ", " +
			"breakdown=" + this.breakdown + ", " +
			"quantity=" + this.quantity + ", " +
			"security=" + this.security + ", " +
			"transferorTransferee=" + this.transferorTransferee +
		'}' + " " + super.toString();
	}
}
