package org.isda.cdm;

import org.isda.cdm.AcctOwnr;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SellrMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Sellr extends RosettaModelObject {
	private final AcctOwnr acctOwnr;
	private static SellrMeta metaData = new SellrMeta();
	
	Sellr(SellrBuilder builder) {
		this.acctOwnr = ofNullable(builder.getAcctOwnr()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="AcctOwnr", source="ISO20022")
	public final AcctOwnr getAcctOwnr() {
		return acctOwnr;
	}
	
	
	@Override
	public RosettaMetaData<? extends Sellr> metaData() {
		return metaData;
	} 
	
	public SellrBuilder toBuilder() {
		SellrBuilder builder = new SellrBuilder();
		ofNullable(getAcctOwnr()).ifPresent(builder::setAcctOwnr);
		return builder;
	}

	public static SellrBuilder builder() {
		return new SellrBuilder();
	}

	public static  class SellrBuilder extends RosettaModelObjectBuilder {
	
		protected AcctOwnr.AcctOwnrBuilder acctOwnr;
	
		public SellrBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Sellr> metaData() {
			return metaData;
		} 
	
		
		public AcctOwnr.AcctOwnrBuilder getAcctOwnr() {
			return acctOwnr;
		}
		
		public AcctOwnr.AcctOwnrBuilder getOrCreateAcctOwnr() {
			if (acctOwnr!=null) {
				return acctOwnr;
			}
			else return acctOwnr = new AcctOwnr.AcctOwnrBuilder();
		}
		
	
		public SellrBuilder setAcctOwnr(AcctOwnr acctOwnr) {
			this.acctOwnr = acctOwnr.toBuilder();
			return this;
		}
		public SellrBuilder setAcctOwnrBuilder(AcctOwnr.AcctOwnrBuilder acctOwnr) {
			this.acctOwnr = acctOwnr;
			return this;
		}
		
		public Sellr build() {
			return new Sellr(this);
		}
	
		@Override
		public SellrBuilder prune() {
			if (acctOwnr!=null && !acctOwnr.prune().hasData()) acctOwnr = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAcctOwnr()!=null && getAcctOwnr().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("acctOwnr"), processor, AcctOwnr.class, acctOwnr);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SellrBuilder _that = (SellrBuilder) o;
		
			if (acctOwnr != null ? !acctOwnr.equals(_that.acctOwnr) : _that.acctOwnr != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (acctOwnr != null ? acctOwnr.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SellrBuilder {" +
				"acctOwnr=" + this.acctOwnr +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("acctOwnr"), processor, AcctOwnr.class, acctOwnr);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Sellr _that = (Sellr) o;
	
		if (acctOwnr != null ? !acctOwnr.equals(_that.acctOwnr) : _that.acctOwnr != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (acctOwnr != null ? acctOwnr.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Sellr {" +
			"acctOwnr=" + this.acctOwnr +
		'}';
	}
}
