package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.FieldWithMetaSettledEntityMatrixSourceEnum;
import org.isda.cdm.metafields.FieldWithMetaSettledEntityMatrixSourceEnum.FieldWithMetaSettledEntityMatrixSourceEnumBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SettledEntityMatrixMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the Relevant Settled Entity Matrix.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SettledEntityMatrix", source="FpML_5_10")
public class SettledEntityMatrix extends RosettaModelObject {
	private final FieldWithMetaSettledEntityMatrixSourceEnum matrixSource;
	private final Date publicationDate;
	private static SettledEntityMatrixMeta metaData = new SettledEntityMatrixMeta();
	
	SettledEntityMatrix(SettledEntityMatrixBuilder builder) {
		this.matrixSource = ofNullable(builder.getMatrixSource()).map(f->f.build()).orElse(null);
		this.publicationDate = builder.getPublicationDate();
	}
	
	/**
	 * Relevant settled entity matrix source.
	 */
	@RosettaSynonym(value="matrixSource", source="FpML_5_10")
	@RosettaSynonym(value="matrixSource", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="matrixSource", source="DTCC_11_0")
	@RosettaSynonym(value="matrixSource", source="DTCC_9_0")
	@RosettaSynonym(value="matrixSource", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaSettledEntityMatrixSourceEnum getMatrixSource() {
		return matrixSource;
	}
	
	/**
	 * Specifies the publication date of the applicable version of the matrix. When this element is omitted, the Standard Terms Supplement defines rules for which version of the matrix is applicable.
	 */
	@RosettaSynonym(value="publicationDate", source="FpML_5_10")
	@RosettaSynonym(value="publicationDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_11_0")
	@RosettaSynonym(value="publicationDate", source="DTCC_9_0")
	@RosettaSynonym(value="publicationDate", source="CME_ClearedConfirm_1_17")
	public final Date getPublicationDate() {
		return publicationDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends SettledEntityMatrix> metaData() {
		return metaData;
	} 
	
	public SettledEntityMatrixBuilder toBuilder() {
		SettledEntityMatrixBuilder builder = new SettledEntityMatrixBuilder();
		ofNullable(getMatrixSource()).ifPresent(builder::setMatrixSource);
		ofNullable(getPublicationDate()).ifPresent(builder::setPublicationDate);
		return builder;
	}

	public static SettledEntityMatrixBuilder builder() {
		return new SettledEntityMatrixBuilder();
	}

	public static  class SettledEntityMatrixBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaSettledEntityMatrixSourceEnumBuilder matrixSource;
		protected Date publicationDate;
	
		public SettledEntityMatrixBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SettledEntityMatrix> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaSettledEntityMatrixSourceEnumBuilder getMatrixSource() {
			return matrixSource;
		}
		
		public FieldWithMetaSettledEntityMatrixSourceEnumBuilder getOrCreateMatrixSource() {
			if (matrixSource!=null) {
				return matrixSource;
			}
			else return matrixSource = new FieldWithMetaSettledEntityMatrixSourceEnumBuilder();
		}
		
		
		public Date getPublicationDate() {
			return publicationDate;
		}
		
	
		public SettledEntityMatrixBuilder setMatrixSource(FieldWithMetaSettledEntityMatrixSourceEnum matrixSource) {
			this.matrixSource = matrixSource.toBuilder();
			return this;
		}
		public SettledEntityMatrixBuilder setMatrixSourceRef(SettledEntityMatrixSourceEnum matrixSource) {
			return setMatrixSource(FieldWithMetaSettledEntityMatrixSourceEnum.builder().setValue(matrixSource).build());
		}
		public SettledEntityMatrixBuilder setPublicationDate(Date publicationDate) {
			this.publicationDate = publicationDate;
			return this;
		}
		
		public SettledEntityMatrix build() {
			return new SettledEntityMatrix(this);
		}
	
		@Override
		public SettledEntityMatrixBuilder prune() {
			if (matrixSource!=null && !matrixSource.prune().hasData()) matrixSource = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMatrixSource()!=null) return true;
			if (getPublicationDate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
			
			processRosetta(path.newSubPath("matrixSource"), processor, FieldWithMetaSettledEntityMatrixSourceEnum.class, matrixSource);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SettledEntityMatrixBuilder _that = (SettledEntityMatrixBuilder) o;
		
			if (matrixSource != null ? !matrixSource.equals(_that.matrixSource) : _that.matrixSource != null) return false;
			if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (matrixSource != null ? matrixSource.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SettledEntityMatrixBuilder {" +
				"matrixSource=" + this.matrixSource + ", " +
				"publicationDate=" + this.publicationDate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("publicationDate"), Date.class, publicationDate, this);
		
		processRosetta(path.newSubPath("matrixSource"), processor, FieldWithMetaSettledEntityMatrixSourceEnum.class, matrixSource);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SettledEntityMatrix _that = (SettledEntityMatrix) o;
	
		if (matrixSource != null ? !matrixSource.equals(_that.matrixSource) : _that.matrixSource != null) return false;
		if (publicationDate != null ? !publicationDate.equals(_that.publicationDate) : _that.publicationDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (matrixSource != null ? matrixSource.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (publicationDate != null ? publicationDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SettledEntityMatrix {" +
			"matrixSource=" + this.matrixSource + ", " +
			"publicationDate=" + this.publicationDate +
		'}';
	}
}
