package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the relevant settled entity matrix source.
 * 
 * @version 2.5.4
 */
public enum SettledEntityMatrixSourceEnum {
    

	/**
	 * The Relevant Settled Entity Matrix shall be the list agreed for this purpose by the parties. The list is not included as part of the electronic confirmation.
	 */
	@RosettaSynonym(value = "ConfirmationAnnex", source = "FpML_5_10")
	@RosettaSynonym(value = "ConfirmationAnnex", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ConfirmationAnnex", source = "DTCC_11_0")
	@RosettaSynonym(value = "ConfirmationAnnex", source = "DTCC_9_0")
	@RosettaSynonym(value = "ConfirmationAnnex", source = "CME_ClearedConfirm_1_17")
	CONFIRMATION_ANNEX,
	    

	/**
	 * The term is not applicable.
	 */
	@RosettaSynonym(value = "NotApplicable", source = "FpML_5_10")
	@RosettaSynonym(value = "NotApplicable", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NotApplicable", source = "DTCC_11_0")
	@RosettaSynonym(value = "NotApplicable", source = "DTCC_9_0")
	@RosettaSynonym(value = "NotApplicable", source = "CME_ClearedConfirm_1_17")
	NOT_APPLICABLE,
	    

	/**
	 * The Settled Entity Matrix published by the Index Publisher.
	 */
	@RosettaSynonym(value = "Publisher", source = "FpML_5_10")
	@RosettaSynonym(value = "Publisher", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Publisher", source = "DTCC_11_0")
	@RosettaSynonym(value = "Publisher", source = "DTCC_9_0")
	@RosettaSynonym(value = "Publisher", source = "CME_ClearedConfirm_1_17")
	PUBLISHER    
	;

	private final String displayName;
	
	SettledEntityMatrixSourceEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
