package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify how the option is to be settled when exercised.
 * 
 * @version 2.5.4
 */
public enum SettlementTypeEnum {
    

	/**
	 * The intrinsic value of the option will be delivered by way of a cash settlement amount determined, (i) by reference to the differential between the strike price and the settlement price; or (ii) in accordance with a bilateral agreement between the parties.
	 */
	@RosettaSynonym(value = "Cash", source = "FpML_5_10")
	@RosettaSynonym(value = "Cash", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Cash", source = "DTCC_11_0")
	@RosettaSynonym(value = "Cash", source = "DTCC_9_0")
	@RosettaSynonym(value = "Cash", source = "CME_ClearedConfirm_1_17")
	CASH,
	    

	/**
	 * The securities underlying the transaction will be delivered by (i) in the case of a call, the seller to the buyer, or (ii) in the case of a put, the buyer to the seller versus a settlement amount equivalent to the strike price per share.
	 */
	@RosettaSynonym(value = "Physical", source = "FpML_5_10")
	@RosettaSynonym(value = "Physical", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Physical", source = "DTCC_11_0")
	@RosettaSynonym(value = "Physical", source = "DTCC_9_0")
	@RosettaSynonym(value = "Physical", source = "CME_ClearedConfirm_1_17")
	PHYSICAL,
	    

	/**
	 * Allow Election of either Cash or Physical settlement.
	 */
	@RosettaSynonym(value = "Election", source = "FpML_5_10")
	@RosettaSynonym(value = "Election", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Election", source = "DTCC_11_0")
	@RosettaSynonym(value = "Election", source = "DTCC_9_0")
	@RosettaSynonym(value = "Election", source = "CME_ClearedConfirm_1_17")
	ELECTION,
	    

	/**
	 * Allow use of either Cash or Physical settlement without prior Election.
	 */
	@RosettaSynonym(value = "CashOrPhysical", source = "FpML_5_10")
	@RosettaSynonym(value = "CashOrPhysical", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CashOrPhysical", source = "DTCC_11_0")
	@RosettaSynonym(value = "CashOrPhysical", source = "DTCC_9_0")
	@RosettaSynonym(value = "CashOrPhysical", source = "CME_ClearedConfirm_1_17")
	CASH_OR_PHYSICAL    
	;

	private final String displayName;
	
	SettlementTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
