package org.isda.cdm;


/**
 * The enumerated values to specify the SIMM normalized exceptions applicable to the ISDA 2018 Standard CSA. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles. | ISDA 2018 Credit Support Annex for Initial Margin, paragraph 13, General Principles.
 * 
 * @version 2.5.4
 */
public enum SimmExceptionEnum {
    

	/**
	 * The ISDA Standard Initial Margin Model exception is applicable. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles. | ISDA 2018 Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	APPLICABLE,
	    

	/**
	 * The ISDA Standard Initial Margin Model exception is not applicable. ISDA 2016 Credit Support Annex for Initial Margin, paragraph 13, General Principles. | ISDA 2018 Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	NOT_APPLICABLE,
	    

	/**
	 * The ISDA Standard Initial Margin Model exception is applicable as a Fallback to Mandatory Method. ISDA 2018 Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	FALL_BACK_TO_MANDATORY_METHOD,
	    

	/**
	 * The ISDA Standard Initial Margin Model exception is applicable as a Mandatory Method. This means that the method applicable is to determine the Margin Amount (IM) by reference to the methodology prescribed pursuant to the applicable regulatory regime which uses a standardized initial margin schedule (such that prescribed percentages are applied to notional amounts before being adjusted, including by a net-to-gross ratio (NGR)). ISDA 2018 Credit Support Annex for Initial Margin, paragraph 13, General Principles.
	 */
	MANDATORY_METHOD    
	;

	private final String displayName;
	
	SimmExceptionEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
