package org.isda.cdm;

import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Money;
import org.isda.cdm.metafields.MetaFields;
import org.isda.cdm.metafields.ReferenceWithMetaAccount;
import org.isda.cdm.metafields.ReferenceWithMetaAccount.ReferenceWithMetaAccountBuilder;
import org.isda.cdm.metafields.ReferenceWithMetaParty;
import org.isda.cdm.metafields.ReferenceWithMetaParty.ReferenceWithMetaPartyBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SimplePaymentMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * A class to specified payments in a simpler fashion than the Payment type. This construct should be used from the FpML version 4.3 onwards.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SimplePayment", source="FpML_5_10")
@RosettaSynonym(value="SimplePayment", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="SimplePayment", source="DTCC_11_0")
@RosettaSynonym(value="SimplePayment", source="DTCC_9_0")
@RosettaSynonym(value="SimplePayment", source="CME_ClearedConfirm_1_17")
public class SimplePayment extends PayerReceiver implements GlobalKey {
	private final MetaFields meta;
	private final Money paymentAmount;
	private final AdjustableOrRelativeDate paymentDate;
	private static SimplePaymentMeta metaData = new SimplePaymentMeta();
	
	SimplePayment(SimplePaymentBuilder builder) {
		super(builder);
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.paymentAmount = ofNullable(builder.getPaymentAmount()).map(f->f.build()).orElse(null);
		this.paymentDate = ofNullable(builder.getPaymentDate()).map(f->f.build()).orElse(null);
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The payment amount.
	 */
	@RosettaSynonym(value="paymentAmount", source="FpML_5_10")
	@RosettaSynonym(value="paymentAmount", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_11_0")
	@RosettaSynonym(value="paymentAmount", source="DTCC_9_0")
	@RosettaSynonym(value="paymentAmount", source="CME_ClearedConfirm_1_17")
	public final Money getPaymentAmount() {
		return paymentAmount;
	}
	
	/**
	 * The payment date. This date is subject to adjustment in accordance with any applicable business day convention.
	 */
	@RosettaSynonym(value="paymentDate", source="FpML_5_10")
	@RosettaSynonym(value="paymentDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_11_0")
	@RosettaSynonym(value="paymentDate", source="DTCC_9_0")
	@RosettaSynonym(value="paymentDate", source="CME_ClearedConfirm_1_17")
	public final AdjustableOrRelativeDate getPaymentDate() {
		return paymentDate;
	}
	
	
	@Override
	public RosettaMetaData<? extends SimplePayment> metaData() {
		return metaData;
	} 
	
	public SimplePaymentBuilder toBuilder() {
		SimplePaymentBuilder builder = new SimplePaymentBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getPaymentAmount()).ifPresent(builder::setPaymentAmount);
		ofNullable(getPaymentDate()).ifPresent(builder::setPaymentDate);
		ofNullable(getPayerAccountReference()).ifPresent(builder::setPayerAccountReference);
		ofNullable(getPayerPartyReference()).ifPresent(builder::setPayerPartyReference);
		ofNullable(getReceiverAccountReference()).ifPresent(builder::setReceiverAccountReference);
		ofNullable(getReceiverPartyReference()).ifPresent(builder::setReceiverPartyReference);
		return builder;
	}

	public static SimplePaymentBuilder builder() {
		return new SimplePaymentBuilder();
	}

	public static  class SimplePaymentBuilder extends PayerReceiverBuilder implements GlobalKeyBuilder<SimplePaymentBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected Money.MoneyBuilder paymentAmount;
		protected AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate;
	
		public SimplePaymentBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SimplePayment> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Money.MoneyBuilder getPaymentAmount() {
			return paymentAmount;
		}
		
		public Money.MoneyBuilder getOrCreatePaymentAmount() {
			if (paymentAmount!=null) {
				return paymentAmount;
			}
			else return paymentAmount = new Money.MoneyBuilder();
		}
		
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getPaymentDate() {
			return paymentDate;
		}
		
		public AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder getOrCreatePaymentDate() {
			if (paymentDate!=null) {
				return paymentDate;
			}
			else return paymentDate = new AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder();
		}
		
	
		public SimplePaymentBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public SimplePaymentBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public SimplePaymentBuilder setPaymentAmount(Money paymentAmount) {
			this.paymentAmount = paymentAmount.toBuilder();
			return this;
		}
		public SimplePaymentBuilder setPaymentAmountBuilder(Money.MoneyBuilder paymentAmount) {
			this.paymentAmount = paymentAmount;
			return this;
		}
		public SimplePaymentBuilder setPaymentDate(AdjustableOrRelativeDate paymentDate) {
			this.paymentDate = paymentDate.toBuilder();
			return this;
		}
		public SimplePaymentBuilder setPaymentDateBuilder(AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder paymentDate) {
			this.paymentDate = paymentDate;
			return this;
		}
		@Override public SimplePaymentBuilder setPayerAccountReference(ReferenceWithMetaAccount payerAccountReference) {
			this.payerAccountReference = payerAccountReference.toBuilder();
			return this;
		}
		@Override public SimplePaymentBuilder setPayerAccountReferenceBuilder(ReferenceWithMetaAccountBuilder payerAccountReference) {
			this.payerAccountReference = payerAccountReference;
			return this;
		}
		@Override public SimplePaymentBuilder setPayerAccountReferenceRef(Account.AccountBuilder payerAccountReference) {
			return setPayerAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(payerAccountReference).build());
		}
		@Override public SimplePaymentBuilder setPayerAccountReferenceRef(Account payerAccountReference) {
			return setPayerAccountReferenceRef(Account.builder());
		}
		@Override public SimplePaymentBuilder setPayerPartyReference(ReferenceWithMetaParty payerPartyReference) {
			this.payerPartyReference = payerPartyReference.toBuilder();
			return this;
		}
		@Override public SimplePaymentBuilder setPayerPartyReferenceBuilder(ReferenceWithMetaPartyBuilder payerPartyReference) {
			this.payerPartyReference = payerPartyReference;
			return this;
		}
		@Override public SimplePaymentBuilder setPayerPartyReferenceRef(Party.PartyBuilder payerPartyReference) {
			return setPayerPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(payerPartyReference).build());
		}
		@Override public SimplePaymentBuilder setPayerPartyReferenceRef(Party payerPartyReference) {
			return setPayerPartyReferenceRef(Party.builder());
		}
		@Override public SimplePaymentBuilder setReceiverAccountReference(ReferenceWithMetaAccount receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference.toBuilder();
			return this;
		}
		@Override public SimplePaymentBuilder setReceiverAccountReferenceBuilder(ReferenceWithMetaAccountBuilder receiverAccountReference) {
			this.receiverAccountReference = receiverAccountReference;
			return this;
		}
		@Override public SimplePaymentBuilder setReceiverAccountReferenceRef(Account.AccountBuilder receiverAccountReference) {
			return setReceiverAccountReference(ReferenceWithMetaAccount.builder().setValueBuilder(receiverAccountReference).build());
		}
		@Override public SimplePaymentBuilder setReceiverAccountReferenceRef(Account receiverAccountReference) {
			return setReceiverAccountReferenceRef(Account.builder());
		}
		@Override public SimplePaymentBuilder setReceiverPartyReference(ReferenceWithMetaParty receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference.toBuilder();
			return this;
		}
		@Override public SimplePaymentBuilder setReceiverPartyReferenceBuilder(ReferenceWithMetaPartyBuilder receiverPartyReference) {
			this.receiverPartyReference = receiverPartyReference;
			return this;
		}
		@Override public SimplePaymentBuilder setReceiverPartyReferenceRef(Party.PartyBuilder receiverPartyReference) {
			return setReceiverPartyReference(ReferenceWithMetaParty.builder().setValueBuilder(receiverPartyReference).build());
		}
		@Override public SimplePaymentBuilder setReceiverPartyReferenceRef(Party receiverPartyReference) {
			return setReceiverPartyReferenceRef(Party.builder());
		}
		
		public SimplePayment build() {
			return new SimplePayment(this);
		}
	
		@Override
		public SimplePaymentBuilder prune() {
			super.prune();
			if (meta!=null && !meta.prune().hasData()) meta = null;
			if (paymentAmount!=null && !paymentAmount.prune().hasData()) paymentAmount = null;
			if (paymentDate!=null && !paymentDate.prune().hasData()) paymentDate = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getPaymentAmount()!=null && getPaymentAmount().hasData()) return true;
			if (getPaymentDate()!=null && getPaymentDate().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
			processRosetta(path.newSubPath("paymentAmount"), processor, Money.class, paymentAmount);
			processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			SimplePaymentBuilder _that = (SimplePaymentBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (paymentAmount != null ? !paymentAmount.equals(_that.paymentAmount) : _that.paymentAmount != null) return false;
			if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (paymentAmount != null ? paymentAmount.hashCode() : 0);
			_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SimplePaymentBuilder {" +
				"meta=" + this.meta + ", " +
				"paymentAmount=" + this.paymentAmount + ", " +
				"paymentDate=" + this.paymentDate +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		processRosetta(path.newSubPath("paymentAmount"), processor, Money.class, paymentAmount);
		processRosetta(path.newSubPath("paymentDate"), processor, AdjustableOrRelativeDate.class, paymentDate);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		SimplePayment _that = (SimplePayment) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (paymentAmount != null ? !paymentAmount.equals(_that.paymentAmount) : _that.paymentAmount != null) return false;
		if (paymentDate != null ? !paymentDate.equals(_that.paymentDate) : _that.paymentDate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (paymentAmount != null ? paymentAmount.hashCode() : 0);
		_result = 31 * _result + (paymentDate != null ? paymentDate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SimplePayment {" +
			"meta=" + this.meta + ", " +
			"paymentAmount=" + this.paymentAmount + ", " +
			"paymentDate=" + this.paymentDate +
		'}' + " " + super.toString();
	}
}
