package org.isda.cdm;

import java.lang.Integer;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SingleValuationDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify the number of business days after satisfaction of all conditions to settlement.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SingleValuationDate", source="FpML_5_10")
public class SingleValuationDate extends RosettaModelObject {
	private final Integer businessDays;
	private static SingleValuationDateMeta metaData = new SingleValuationDateMeta();
	
	SingleValuationDate(SingleValuationDateBuilder builder) {
		this.businessDays = builder.getBusinessDays();
	}
	
	/**
	 * A number of business days. Its precise meaning is dependant on the context in which this element is used. ISDA 2003 Term: Business Day.
	 */
	@RosettaSynonym(value="businessDays", source="FpML_5_10")
	@RosettaSynonym(value="businessDays", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="businessDays", source="DTCC_11_0")
	@RosettaSynonym(value="businessDays", source="DTCC_9_0")
	@RosettaSynonym(value="businessDays", source="CME_ClearedConfirm_1_17")
	public final Integer getBusinessDays() {
		return businessDays;
	}
	
	
	@Override
	public RosettaMetaData<? extends SingleValuationDate> metaData() {
		return metaData;
	} 
	
	public SingleValuationDateBuilder toBuilder() {
		SingleValuationDateBuilder builder = new SingleValuationDateBuilder();
		ofNullable(getBusinessDays()).ifPresent(builder::setBusinessDays);
		return builder;
	}

	public static SingleValuationDateBuilder builder() {
		return new SingleValuationDateBuilder();
	}

	public static  class SingleValuationDateBuilder extends RosettaModelObjectBuilder {
	
		protected Integer businessDays;
	
		public SingleValuationDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SingleValuationDate> metaData() {
			return metaData;
		} 
	
		
		public Integer getBusinessDays() {
			return businessDays;
		}
		
	
		public SingleValuationDateBuilder setBusinessDays(Integer businessDays) {
			this.businessDays = businessDays;
			return this;
		}
		
		public SingleValuationDate build() {
			return new SingleValuationDate(this);
		}
	
		@Override
		public SingleValuationDateBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBusinessDays()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("businessDays"), Integer.class, businessDays, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SingleValuationDateBuilder _that = (SingleValuationDateBuilder) o;
		
			if (businessDays != null ? !businessDays.equals(_that.businessDays) : _that.businessDays != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (businessDays != null ? businessDays.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SingleValuationDateBuilder {" +
				"businessDays=" + this.businessDays +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("businessDays"), Integer.class, businessDays, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SingleValuationDate _that = (SingleValuationDate) o;
	
		if (businessDays != null ? !businessDays.equals(_that.businessDays) : _that.businessDays != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (businessDays != null ? businessDays.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SingleValuationDate {" +
			"businessDays=" + this.businessDays +
		'}';
	}
}
