package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.Indx;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SnglMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Sngl extends RosettaModelObject {
	private final Indx indx;
	private final String isin;
	private static SnglMeta metaData = new SnglMeta();
	
	Sngl(SnglBuilder builder) {
		this.indx = ofNullable(builder.getIndx()).map(f->f.build()).orElse(null);
		this.isin = builder.getIsin();
	}
	
	@RosettaSynonym(value="Indx", source="ISO20022")
	public final Indx getIndx() {
		return indx;
	}
	
	@RosettaSynonym(value="ISIN", source="ISO20022")
	public final String getIsin() {
		return isin;
	}
	
	
	@Override
	public RosettaMetaData<? extends Sngl> metaData() {
		return metaData;
	} 
	
	public SnglBuilder toBuilder() {
		SnglBuilder builder = new SnglBuilder();
		ofNullable(getIndx()).ifPresent(builder::setIndx);
		ofNullable(getIsin()).ifPresent(builder::setIsin);
		return builder;
	}

	public static SnglBuilder builder() {
		return new SnglBuilder();
	}

	public static  class SnglBuilder extends RosettaModelObjectBuilder {
	
		protected Indx.IndxBuilder indx;
		protected String isin;
	
		public SnglBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Sngl> metaData() {
			return metaData;
		} 
	
		
		public Indx.IndxBuilder getIndx() {
			return indx;
		}
		
		public Indx.IndxBuilder getOrCreateIndx() {
			if (indx!=null) {
				return indx;
			}
			else return indx = new Indx.IndxBuilder();
		}
		
		
		public String getIsin() {
			return isin;
		}
		
	
		public SnglBuilder setIndx(Indx indx) {
			this.indx = indx.toBuilder();
			return this;
		}
		public SnglBuilder setIndxBuilder(Indx.IndxBuilder indx) {
			this.indx = indx;
			return this;
		}
		public SnglBuilder setIsin(String isin) {
			this.isin = isin;
			return this;
		}
		
		public Sngl build() {
			return new Sngl(this);
		}
	
		@Override
		public SnglBuilder prune() {
			if (indx!=null && !indx.prune().hasData()) indx = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIndx()!=null && getIndx().hasData()) return true;
			if (getIsin()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("isin"), String.class, isin, this);
			
			processRosetta(path.newSubPath("indx"), processor, Indx.class, indx);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SnglBuilder _that = (SnglBuilder) o;
		
			if (indx != null ? !indx.equals(_that.indx) : _that.indx != null) return false;
			if (isin != null ? !isin.equals(_that.isin) : _that.isin != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
			_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SnglBuilder {" +
				"indx=" + this.indx + ", " +
				"isin=" + this.isin +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("isin"), String.class, isin, this);
		
		processRosetta(path.newSubPath("indx"), processor, Indx.class, indx);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Sngl _that = (Sngl) o;
	
		if (indx != null ? !indx.equals(_that.indx) : _that.indx != null) return false;
		if (isin != null ? !isin.equals(_that.isin) : _that.isin != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
		_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Sngl {" +
			"indx=" + this.indx + ", " +
			"isin=" + this.isin +
		'}';
	}
}
