package org.isda.cdm;

import java.lang.Boolean;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SpecifiedCurrencyMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SpecifiedCurrency", source="FpML_5_10")
public class SpecifiedCurrency extends RosettaModelObject {
	private final Boolean applicable;
	private final FieldWithMetaString currency;
	private static SpecifiedCurrencyMeta metaData = new SpecifiedCurrencyMeta();
	
	SpecifiedCurrency(SpecifiedCurrencyBuilder builder) {
		this.applicable = builder.getApplicable();
		this.currency = ofNullable(builder.getCurrency()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * Indicates whether the specified currency provision is applicable.
	 */
	@RosettaSynonym(value="applicable", source="FpML_5_10")
	@RosettaSynonym(value="applicable", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="applicable", source="DTCC_11_0")
	@RosettaSynonym(value="applicable", source="DTCC_9_0")
	@RosettaSynonym(value="applicable", source="CME_ClearedConfirm_1_17")
	public final Boolean getApplicable() {
		return applicable;
	}
	
	/**
	 * The currency in which the the specified currency is denominated. The list of valid currencies is not presently positioned as an enumeration as part of the CDM because that scope is limited to the values specified by ISDA and FpML. As a result, implementers have to make reference to the relevant standard, such as the ISO 4217 standard for currency codes.
	 */
	@RosettaSynonym(value="currency", source="FpML_5_10")
	@RosettaSynonym(value="currency", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="currency", source="DTCC_11_0")
	@RosettaSynonym(value="currency", source="DTCC_9_0")
	@RosettaSynonym(value="currency", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaString getCurrency() {
		return currency;
	}
	
	
	@Override
	public RosettaMetaData<? extends SpecifiedCurrency> metaData() {
		return metaData;
	} 
	
	public SpecifiedCurrencyBuilder toBuilder() {
		SpecifiedCurrencyBuilder builder = new SpecifiedCurrencyBuilder();
		ofNullable(getApplicable()).ifPresent(builder::setApplicable);
		ofNullable(getCurrency()).ifPresent(builder::setCurrency);
		return builder;
	}

	public static SpecifiedCurrencyBuilder builder() {
		return new SpecifiedCurrencyBuilder();
	}

	public static  class SpecifiedCurrencyBuilder extends RosettaModelObjectBuilder {
	
		protected Boolean applicable;
		protected FieldWithMetaStringBuilder currency;
	
		public SpecifiedCurrencyBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SpecifiedCurrency> metaData() {
			return metaData;
		} 
	
		
		public Boolean getApplicable() {
			return applicable;
		}
		
		
		public FieldWithMetaStringBuilder getCurrency() {
			return currency;
		}
		
		public FieldWithMetaStringBuilder getOrCreateCurrency() {
			if (currency!=null) {
				return currency;
			}
			else return currency = new FieldWithMetaStringBuilder();
		}
		
	
		public SpecifiedCurrencyBuilder setApplicable(Boolean applicable) {
			this.applicable = applicable;
			return this;
		}
		public SpecifiedCurrencyBuilder setCurrency(FieldWithMetaString currency) {
			this.currency = currency.toBuilder();
			return this;
		}
		public SpecifiedCurrencyBuilder setCurrencyRef(String currency) {
			return setCurrency(FieldWithMetaString.builder().setValue(currency).build());
		}
		
		public SpecifiedCurrency build() {
			return new SpecifiedCurrency(this);
		}
	
		@Override
		public SpecifiedCurrencyBuilder prune() {
			if (currency!=null && !currency.prune().hasData()) currency = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getApplicable()!=null) return true;
			if (getCurrency()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
			
			processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SpecifiedCurrencyBuilder _that = (SpecifiedCurrencyBuilder) o;
		
			if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
			if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
			_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpecifiedCurrencyBuilder {" +
				"applicable=" + this.applicable + ", " +
				"currency=" + this.currency +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("applicable"), Boolean.class, applicable, this);
		
		processRosetta(path.newSubPath("currency"), processor, FieldWithMetaString.class, currency);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SpecifiedCurrency _that = (SpecifiedCurrency) o;
	
		if (applicable != null ? !applicable.equals(_that.applicable) : _that.applicable != null) return false;
		if (currency != null ? !currency.equals(_that.currency) : _that.currency != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (applicable != null ? applicable.hashCode() : 0);
		_result = 31 * _result + (currency != null ? currency.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SpecifiedCurrency {" +
			"applicable=" + this.applicable + ", " +
			"currency=" + this.currency +
		'}';
	}
}
