package org.isda.cdm;

import org.isda.cdm.metafields.FieldWithMetaSpreadScheduleTypeEnum;
import org.isda.cdm.metafields.FieldWithMetaSpreadScheduleTypeEnum.FieldWithMetaSpreadScheduleTypeEnumBuilder;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SpreadScheduleMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Adds an optional spread type element to the Schedule to identify a long or short spread value.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="SpreadSchedule", source="FpML_5_10")
public class SpreadSchedule extends Schedule {
	private final FieldWithMetaSpreadScheduleTypeEnum type;
	private static SpreadScheduleMeta metaData = new SpreadScheduleMeta();
	
	SpreadSchedule(SpreadScheduleBuilder builder) {
		super(builder);
		this.type = ofNullable(builder.getType()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * An element which purpose is to identify a long or short spread value.
	 */
	@RosettaSynonym(value="SpreadScheduleType", source="FpML_5_10")
	@RosettaSynonym(value="SpreadScheduleType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="SpreadScheduleType", source="DTCC_11_0")
	@RosettaSynonym(value="SpreadScheduleType", source="DTCC_9_0")
	@RosettaSynonym(value="SpreadScheduleType", source="CME_ClearedConfirm_1_17")
	public final FieldWithMetaSpreadScheduleTypeEnum getType() {
		return type;
	}
	
	
	@Override
	public RosettaMetaData<? extends SpreadSchedule> metaData() {
		return metaData;
	} 
	
	public SpreadScheduleBuilder toBuilder() {
		SpreadScheduleBuilder builder = new SpreadScheduleBuilder();
		ofNullable(getType()).ifPresent(builder::setType);
		ofNullable(getInitialValue()).ifPresent(builder::setInitialValue);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStep()).ifPresent(step -> step.forEach(builder::addStep));
		return builder;
	}

	public static SpreadScheduleBuilder builder() {
		return new SpreadScheduleBuilder();
	}

	public static  class SpreadScheduleBuilder extends ScheduleBuilder {
	
		protected FieldWithMetaSpreadScheduleTypeEnumBuilder type;
	
		public SpreadScheduleBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SpreadSchedule> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaSpreadScheduleTypeEnumBuilder getType() {
			return type;
		}
		
		public FieldWithMetaSpreadScheduleTypeEnumBuilder getOrCreateType() {
			if (type!=null) {
				return type;
			}
			else return type = new FieldWithMetaSpreadScheduleTypeEnumBuilder();
		}
		
	
		public SpreadScheduleBuilder setType(FieldWithMetaSpreadScheduleTypeEnum type) {
			this.type = type.toBuilder();
			return this;
		}
		public SpreadScheduleBuilder setTypeRef(SpreadScheduleTypeEnum type) {
			return setType(FieldWithMetaSpreadScheduleTypeEnum.builder().setValue(type).build());
		}
		@Override public SpreadScheduleBuilder setInitialValue(BigDecimal initialValue) {
			this.initialValue = initialValue;
			return this;
		}
		@Override public SpreadScheduleBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		@Override public SpreadScheduleBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		@Override public SpreadScheduleBuilder addStep(Step step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step.toBuilder());
			return this;
		}
		@Override public SpreadScheduleBuilder addStep(Step step, int _idx) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			getIndex(this.step, _idx, () -> step.toBuilder());
			this.step.set(_idx, step.toBuilder());
			return this;
		}
		@Override public SpreadScheduleBuilder addStep(List<Step> steps) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			for (Step toAdd : steps) {
				this.step.add(toAdd.toBuilder());
			}
			return this;
		}
		@Override public SpreadScheduleBuilder addStepBuilder(Step.StepBuilder step) {
			if(this.step == null){
				this.step = new ArrayList<>();
			}
			this.step.add(step);
			return this;
		}
		
		@Override public SpreadScheduleBuilder clearStep() {
			this.step = null;
			return this;
		}
		
		public SpreadSchedule build() {
			return new SpreadSchedule(this);
		}
	
		@Override
		public SpreadScheduleBuilder prune() {
			super.prune();
			if (type!=null && !type.prune().hasData()) type = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("type"), processor, FieldWithMetaSpreadScheduleTypeEnum.class, type);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			SpreadScheduleBuilder _that = (SpreadScheduleBuilder) o;
		
			if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpreadScheduleBuilder {" +
				"type=" + this.type +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("type"), processor, FieldWithMetaSpreadScheduleTypeEnum.class, type);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		SpreadSchedule _that = (SpreadSchedule) o;
	
		if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SpreadSchedule {" +
			"type=" + this.type +
		'}' + " " + super.toString();
	}
}
