package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether a trade is settling using standard settlement instructions as well as whether it is a candidate for settlement netting.
 * 
 * @version 2.5.4
 */
public enum StandardSettlementStyleEnum {
    

	/**
	 * This trade will settle using standard predetermined funds settlement instructions.
	 */
	@RosettaSynonym(value = "Standard", source = "FpML_5_10")
	@RosettaSynonym(value = "Standard", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Standard", source = "DTCC_11_0")
	@RosettaSynonym(value = "Standard", source = "DTCC_9_0")
	@RosettaSynonym(value = "Standard", source = "CME_ClearedConfirm_1_17")
	STANDARD,
	    

	/**
	 * This trade is a candidate for settlement netting.
	 */
	@RosettaSynonym(value = "Net", source = "FpML_5_10")
	@RosettaSynonym(value = "Net", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Net", source = "DTCC_11_0")
	@RosettaSynonym(value = "Net", source = "DTCC_9_0")
	@RosettaSynonym(value = "Net", source = "CME_ClearedConfirm_1_17")
	NET,
	    

	/**
	 * This trade will settle using standard predetermined funds settlement instructions and is a candidate for settlement netting.
	 */
	@RosettaSynonym(value = "StandardAndNet", source = "FpML_5_10")
	@RosettaSynonym(value = "StandardAndNet", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "StandardAndNet", source = "DTCC_11_0")
	@RosettaSynonym(value = "StandardAndNet", source = "DTCC_9_0")
	@RosettaSynonym(value = "StandardAndNet", source = "CME_ClearedConfirm_1_17")
	STANDARD_AND_NET    
	;

	private final String displayName;
	
	StandardSettlementStyleEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
