package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StepMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class defining a step date and step value pair. This step definitions are used to define varying rate or amount schedules, e.g. a notional amortisation or a step-up coupon schedule.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Step", source="FpML_5_10")
@RosettaSynonym(value="Step", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Step", source="DTCC_11_0")
@RosettaSynonym(value="Step", source="DTCC_9_0")
@RosettaSynonym(value="Step", source="CME_ClearedConfirm_1_17")
public class Step extends RosettaModelObject implements GlobalKey {
	private final MetaFields meta;
	private final Date stepDate;
	private final BigDecimal stepValue;
	private static StepMeta metaData = new StepMeta();
	
	Step(StepBuilder builder) {
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.stepDate = builder.getStepDate();
		this.stepValue = builder.getStepValue();
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The date on which the associated step value becomes effective.
	 */
	@RosettaSynonym(value="stepDate", source="FpML_5_10")
	@RosettaSynonym(value="stepDate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stepDate", source="DTCC_11_0")
	@RosettaSynonym(value="stepDate", source="DTCC_9_0")
	@RosettaSynonym(value="stepDate", source="CME_ClearedConfirm_1_17")
	public final Date getStepDate() {
		return stepDate;
	}
	
	/**
	 * The rate of amount which becomes effective on the associated step date. A rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="stepValue", source="FpML_5_10")
	@RosettaSynonym(value="stepValue", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="stepValue", source="DTCC_11_0")
	@RosettaSynonym(value="stepValue", source="DTCC_9_0")
	@RosettaSynonym(value="stepValue", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getStepValue() {
		return stepValue;
	}
	
	
	@Override
	public RosettaMetaData<? extends Step> metaData() {
		return metaData;
	} 
	
	public StepBuilder toBuilder() {
		StepBuilder builder = new StepBuilder();
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getStepDate()).ifPresent(builder::setStepDate);
		ofNullable(getStepValue()).ifPresent(builder::setStepValue);
		return builder;
	}

	public static StepBuilder builder() {
		return new StepBuilder();
	}

	public static  class StepBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<StepBuilder> {
	
		protected MetaFields.MetaFieldsBuilder meta;
		protected Date stepDate;
		protected BigDecimal stepValue;
	
		public StepBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Step> metaData() {
			return metaData;
		} 
	
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public Date getStepDate() {
			return stepDate;
		}
		
		
		public BigDecimal getStepValue() {
			return stepValue;
		}
		
	
		public StepBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public StepBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public StepBuilder setStepDate(Date stepDate) {
			this.stepDate = stepDate;
			return this;
		}
		public StepBuilder setStepValue(BigDecimal stepValue) {
			this.stepValue = stepValue;
			return this;
		}
		
		public Step build() {
			return new Step(this);
		}
	
		@Override
		public StepBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getStepDate()!=null) return true;
			if (getStepValue()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("stepDate"), Date.class, stepDate, this);
			processor.processBasic(path.newSubPath("stepValue"), BigDecimal.class, stepValue, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StepBuilder _that = (StepBuilder) o;
		
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (stepDate != null ? !stepDate.equals(_that.stepDate) : _that.stepDate != null) return false;
			if (stepValue != null ? !stepValue.equals(_that.stepValue) : _that.stepValue != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (stepDate != null ? stepDate.hashCode() : 0);
			_result = 31 * _result + (stepValue != null ? stepValue.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StepBuilder {" +
				"meta=" + this.meta + ", " +
				"stepDate=" + this.stepDate + ", " +
				"stepValue=" + this.stepValue +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("stepDate"), Date.class, stepDate, this);
		processor.processBasic(path.newSubPath("stepValue"), BigDecimal.class, stepValue, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Step _that = (Step) o;
	
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (stepDate != null ? !stepDate.equals(_that.stepDate) : _that.stepDate != null) return false;
		if (stepValue != null ? !stepValue.equals(_that.stepValue) : _that.stepValue != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (stepDate != null ? stepDate.hashCode() : 0);
		_result = 31 * _result + (stepValue != null ? stepValue.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Step {" +
			"meta=" + this.meta + ", " +
			"stepDate=" + this.stepDate + ", " +
			"stepValue=" + this.stepValue +
		'}';
	}
}
