package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether a percentage rate change, used to calculate a change in notional outstanding, is expressed as a percentage of the initial notional amount or the previously outstanding notional amount.
 * 
 * @version 2.5.4
 */
public enum StepRelativeToEnum {
    

	/**
	 * Change in notional to be applied is calculated by multiplying the percentage rate by the initial notional amount.
	 */
	@RosettaSynonym(value = "Initial", source = "FpML_5_10")
	@RosettaSynonym(value = "Initial", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Initial", source = "DTCC_11_0")
	@RosettaSynonym(value = "Initial", source = "DTCC_9_0")
	@RosettaSynonym(value = "Initial", source = "CME_ClearedConfirm_1_17")
	INITIAL,
	    

	/**
	 * Change in notional to be applied is calculated by multiplying the percentage rate by the previously outstanding notional amount.
	 */
	@RosettaSynonym(value = "Flat", source = "FpML_5_10")
	@RosettaSynonym(value = "Flat", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Flat", source = "DTCC_11_0")
	@RosettaSynonym(value = "Flat", source = "DTCC_9_0")
	@RosettaSynonym(value = "Flat", source = "CME_ClearedConfirm_1_17")
	PREVIOUS    
	;

	private final String displayName;
	
	StepRelativeToEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
