package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.PayerReceiverEnum;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StrikeMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;

/**
 * A class describing a single cap or floor rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Strike", source="FpML_5_10")
@RosettaSynonym(value="Strike", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="Strike", source="DTCC_11_0")
@RosettaSynonym(value="Strike", source="DTCC_9_0")
@RosettaSynonym(value="Strike", source="CME_ClearedConfirm_1_17")
public class Strike extends RosettaModelObject implements GlobalKey {
	private final PayerReceiverEnum buyer;
	private final MetaFields meta;
	private final PayerReceiverEnum seller;
	private final BigDecimal strikeRate;
	private static StrikeMeta metaData = new StrikeMeta();
	
	Strike(StrikeBuilder builder) {
		this.buyer = builder.getBuyer();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
		this.seller = builder.getSeller();
		this.strikeRate = builder.getStrikeRate();
	}
	
	/**
	 * The buyer of the option.
	 */
	@RosettaSynonym(value="buyer", source="FpML_5_10")
	@RosettaSynonym(value="buyer", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="buyer", source="DTCC_11_0")
	@RosettaSynonym(value="buyer", source="DTCC_9_0")
	@RosettaSynonym(value="buyer", source="CME_ClearedConfirm_1_17")
	public final PayerReceiverEnum getBuyer() {
		return buyer;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	/**
	 * The party that has sold.
	 */
	@RosettaSynonym(value="seller", source="FpML_5_10")
	@RosettaSynonym(value="seller", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="seller", source="DTCC_11_0")
	@RosettaSynonym(value="seller", source="DTCC_9_0")
	@RosettaSynonym(value="seller", source="CME_ClearedConfirm_1_17")
	public final PayerReceiverEnum getSeller() {
		return seller;
	}
	
	/**
	 * The rate for a cap or floor.
	 */
	@RosettaSynonym(value="strikeRate", source="FpML_5_10")
	@RosettaSynonym(value="strikeRate", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="strikeRate", source="DTCC_11_0")
	@RosettaSynonym(value="strikeRate", source="DTCC_9_0")
	@RosettaSynonym(value="strikeRate", source="CME_ClearedConfirm_1_17")
	@RosettaSynonym(value="StrikePrice", source="FIX_5_0_SP2")
	public final BigDecimal getStrikeRate() {
		return strikeRate;
	}
	
	
	@Override
	public RosettaMetaData<? extends Strike> metaData() {
		return metaData;
	} 
	
	public StrikeBuilder toBuilder() {
		StrikeBuilder builder = new StrikeBuilder();
		ofNullable(getBuyer()).ifPresent(builder::setBuyer);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		ofNullable(getSeller()).ifPresent(builder::setSeller);
		ofNullable(getStrikeRate()).ifPresent(builder::setStrikeRate);
		return builder;
	}

	public static StrikeBuilder builder() {
		return new StrikeBuilder();
	}

	public static  class StrikeBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<StrikeBuilder> {
	
		protected PayerReceiverEnum buyer;
		protected MetaFields.MetaFieldsBuilder meta;
		protected PayerReceiverEnum seller;
		protected BigDecimal strikeRate;
	
		public StrikeBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Strike> metaData() {
			return metaData;
		} 
	
		
		public PayerReceiverEnum getBuyer() {
			return buyer;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
		
		public PayerReceiverEnum getSeller() {
			return seller;
		}
		
		
		public BigDecimal getStrikeRate() {
			return strikeRate;
		}
		
	
		public StrikeBuilder setBuyer(PayerReceiverEnum buyer) {
			this.buyer = buyer;
			return this;
		}
		public StrikeBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public StrikeBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		public StrikeBuilder setSeller(PayerReceiverEnum seller) {
			this.seller = seller;
			return this;
		}
		public StrikeBuilder setStrikeRate(BigDecimal strikeRate) {
			this.strikeRate = strikeRate;
			return this;
		}
		
		public Strike build() {
			return new Strike(this);
		}
	
		@Override
		public StrikeBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBuyer()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			if (getSeller()!=null) return true;
			if (getStrikeRate()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("buyer"), PayerReceiverEnum.class, buyer, this);
			processor.processBasic(path.newSubPath("seller"), PayerReceiverEnum.class, seller, this);
			processor.processBasic(path.newSubPath("strikeRate"), BigDecimal.class, strikeRate, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StrikeBuilder _that = (StrikeBuilder) o;
		
			if (buyer != null ? !buyer.equals(_that.buyer) : _that.buyer != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			if (seller != null ? !seller.equals(_that.seller) : _that.seller != null) return false;
			if (strikeRate != null ? !strikeRate.equals(_that.strikeRate) : _that.strikeRate != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (buyer != null ? buyer.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			_result = 31 * _result + (seller != null ? seller.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (strikeRate != null ? strikeRate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrikeBuilder {" +
				"buyer=" + this.buyer + ", " +
				"meta=" + this.meta + ", " +
				"seller=" + this.seller + ", " +
				"strikeRate=" + this.strikeRate +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("buyer"), PayerReceiverEnum.class, buyer, this);
		processor.processBasic(path.newSubPath("seller"), PayerReceiverEnum.class, seller, this);
		processor.processBasic(path.newSubPath("strikeRate"), BigDecimal.class, strikeRate, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Strike _that = (Strike) o;
	
		if (buyer != null ? !buyer.equals(_that.buyer) : _that.buyer != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		if (seller != null ? !seller.equals(_that.seller) : _that.seller != null) return false;
		if (strikeRate != null ? !strikeRate.equals(_that.strikeRate) : _that.strikeRate != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (buyer != null ? buyer.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		_result = 31 * _result + (seller != null ? seller.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (strikeRate != null ? strikeRate.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Strike {" +
			"buyer=" + this.buyer + ", " +
			"meta=" + this.meta + ", " +
			"seller=" + this.seller + ", " +
			"strikeRate=" + this.strikeRate +
		'}';
	}
}
