package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.FloatingRateIndexEnum;
import org.isda.cdm.Period;
import org.isda.cdm.RateTreatmentEnum;
import org.isda.cdm.Schedule;
import org.isda.cdm.SpreadSchedule;
import org.isda.cdm.StrikeSchedule;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.StubFloatingRateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;
		import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;

/**
 * A class defining a floating rate.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="StubFloatingRate", source="FpML_5_10")
@RosettaSynonym(value="StubFloatingRate", source="CME_SubmissionIRS_1_0")
@RosettaSynonym(value="StubFloatingRate", source="DTCC_11_0")
@RosettaSynonym(value="StubFloatingRate", source="DTCC_9_0")
@RosettaSynonym(value="StubFloatingRate", source="CME_ClearedConfirm_1_17")
public class StubFloatingRate extends RosettaModelObject {
	private final List<StrikeSchedule> capRateSchedule;
	private final FloatingRateIndexEnum floatingRateIndex;
	private final Schedule floatingRateMultiplierSchedule;
	private final List<StrikeSchedule> floorRateSchedule;
	private final Period indexTenor;
	private final RateTreatmentEnum rateTreatment;
	private final List<SpreadSchedule> spreadSchedule;
	private static StubFloatingRateMeta metaData = new StubFloatingRateMeta();
	
	StubFloatingRate(StubFloatingRateBuilder builder) {
		this.capRateSchedule = ofNullable(builder.getCapRateSchedule()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.floatingRateIndex = builder.getFloatingRateIndex();
		this.floatingRateMultiplierSchedule = ofNullable(builder.getFloatingRateMultiplierSchedule()).map(f->f.build()).orElse(null);
		this.floorRateSchedule = ofNullable(builder.getFloorRateSchedule()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.indexTenor = ofNullable(builder.getIndexTenor()).map(f->f.build()).orElse(null);
		this.rateTreatment = builder.getRateTreatment();
		this.spreadSchedule = ofNullable(builder.getSpreadSchedule()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The cap rate or cap rate schedule, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. A cap rate schedule is expressed as explicit cap rates and dates and the step dates may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="capRateSchedule", source="FpML_5_10")
	@RosettaSynonym(value="capRateSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="capRateSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="capRateSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="capRateSchedule", source="CME_ClearedConfirm_1_17")
	public final List<StrikeSchedule> getCapRateSchedule() {
		return capRateSchedule;
	}
	
	/**
	 * The floating rate index.
	 */
	@RosettaSynonym(value="floatingRateIndex", source="FpML_5_10")
	@RosettaSynonym(value="floatingRateIndex", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRateIndex", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRateIndex", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRateIndex", source="CME_ClearedConfirm_1_17")
	public final FloatingRateIndexEnum getFloatingRateIndex() {
		return floatingRateIndex;
	}
	
	/**
	 * A rate multiplier or multiplier schedule to apply to the floating rate. A multiplier schedule is expressed as explicit multipliers and dates. In the case of a schedule, the step dates may be subject to adjustment in accordance with any adjustments specified in the calculationPeriodDatesAdjustments. The multiplier can be a positive or negative decimal. This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.
	 */
	@RosettaSynonym(value="floatingRateMultiplierSchedule", source="FpML_5_10")
	@RosettaSynonym(value="floatingRateMultiplierSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floatingRateMultiplierSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="floatingRateMultiplierSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="floatingRateMultiplierSchedule", source="CME_ClearedConfirm_1_17")
	public final Schedule getFloatingRateMultiplierSchedule() {
		return floatingRateMultiplierSchedule;
	}
	
	/**
	 * The floor rate or floor rate schedule, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. A floor rate schedule is expressed as explicit floor rates and dates and the step dates may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.
	 */
	@RosettaSynonym(value="floorRateSchedule", source="FpML_5_10")
	@RosettaSynonym(value="floorRateSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="floorRateSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="floorRateSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="floorRateSchedule", source="CME_ClearedConfirm_1_17")
	public final List<StrikeSchedule> getFloorRateSchedule() {
		return floorRateSchedule;
	}
	
	/**
	 * The ISDA Designated Maturity, i.e. the tenor of the floating rate.
	 */
	@RosettaSynonym(value="indexTenor", source="FpML_5_10")
	@RosettaSynonym(value="indexTenor", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="indexTenor", source="DTCC_11_0")
	@RosettaSynonym(value="indexTenor", source="DTCC_9_0")
	@RosettaSynonym(value="indexTenor", source="CME_ClearedConfirm_1_17")
	public final Period getIndexTenor() {
		return indexTenor;
	}
	
	/**
	 * The specification of any rate conversion which needs to be applied to the observed rate before being used in any calculations. The two common conversions are for securities quoted on a bank discount basis which will need to be converted to either a Money Market Yield or Bond Equivalent Yield. See the Annex to the 2000 ISDA Definitions, Section 7.3. Certain General Definitions Relating to Floating Rate Options, paragraphs (g) and (h) for definitions of these terms.
	 */
	@RosettaSynonym(value="rateTreatment", source="FpML_5_10")
	@RosettaSynonym(value="rateTreatment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="rateTreatment", source="DTCC_11_0")
	@RosettaSynonym(value="rateTreatment", source="DTCC_9_0")
	@RosettaSynonym(value="rateTreatment", source="CME_ClearedConfirm_1_17")
	public final RateTreatmentEnum getRateTreatment() {
		return rateTreatment;
	}
	
	/**
	 * The ISDA Spread or a Spread schedule expressed as explicit spreads and dates. In the case of a schedule, the step dates may be subject to adjustment in accordance with any adjustments specified in calculationPeriodDatesAdjustments. The spread is a per annum rate, expressed as a decimal. For purposes of determining a calculation period amount, if positive the spread will be added to the floating rate and if negative the spread will be subtracted from the floating rate. A positive 10 basis point (0.1%) spread would be represented as 0.001.
	 */
	@RosettaSynonym(value="spreadSchedule", source="FpML_5_10")
	@RosettaSynonym(value="spreadSchedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="spreadSchedule", source="DTCC_11_0")
	@RosettaSynonym(value="spreadSchedule", source="DTCC_9_0")
	@RosettaSynonym(value="spreadSchedule", source="CME_ClearedConfirm_1_17")
	public final List<SpreadSchedule> getSpreadSchedule() {
		return spreadSchedule;
	}
	
	
	@Override
	public RosettaMetaData<? extends StubFloatingRate> metaData() {
		return metaData;
	} 
	
	public StubFloatingRateBuilder toBuilder() {
		StubFloatingRateBuilder builder = new StubFloatingRateBuilder();
		ofNullable(getCapRateSchedule()).ifPresent(capRateSchedule -> capRateSchedule.forEach(builder::addCapRateSchedule));
		ofNullable(getFloatingRateIndex()).ifPresent(builder::setFloatingRateIndex);
		ofNullable(getFloatingRateMultiplierSchedule()).ifPresent(builder::setFloatingRateMultiplierSchedule);
		ofNullable(getFloorRateSchedule()).ifPresent(floorRateSchedule -> floorRateSchedule.forEach(builder::addFloorRateSchedule));
		ofNullable(getIndexTenor()).ifPresent(builder::setIndexTenor);
		ofNullable(getRateTreatment()).ifPresent(builder::setRateTreatment);
		ofNullable(getSpreadSchedule()).ifPresent(spreadSchedule -> spreadSchedule.forEach(builder::addSpreadSchedule));
		return builder;
	}

	public static StubFloatingRateBuilder builder() {
		return new StubFloatingRateBuilder();
	}

	public static  class StubFloatingRateBuilder extends RosettaModelObjectBuilder {
	
		protected List<StrikeSchedule.StrikeScheduleBuilder> capRateSchedule;
		protected FloatingRateIndexEnum floatingRateIndex;
		protected Schedule.ScheduleBuilder floatingRateMultiplierSchedule;
		protected List<StrikeSchedule.StrikeScheduleBuilder> floorRateSchedule;
		protected Period.PeriodBuilder indexTenor;
		protected RateTreatmentEnum rateTreatment;
		protected List<SpreadSchedule.SpreadScheduleBuilder> spreadSchedule;
	
		public StubFloatingRateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends StubFloatingRate> metaData() {
			return metaData;
		} 
	
		
		public List<StrikeSchedule.StrikeScheduleBuilder> getCapRateSchedule() {
			return capRateSchedule;
		}
		
		public StrikeSchedule.StrikeScheduleBuilder getOrCreateCapRateSchedule(int index) {
			if (capRateSchedule==null) {
				this.capRateSchedule = new ArrayList<>();
			}
			return getIndex(capRateSchedule, index, ()->new StrikeSchedule.StrikeScheduleBuilder());
		}
		
		
		public FloatingRateIndexEnum getFloatingRateIndex() {
			return floatingRateIndex;
		}
		
		
		public Schedule.ScheduleBuilder getFloatingRateMultiplierSchedule() {
			return floatingRateMultiplierSchedule;
		}
		
		public Schedule.ScheduleBuilder getOrCreateFloatingRateMultiplierSchedule() {
			if (floatingRateMultiplierSchedule!=null) {
				return floatingRateMultiplierSchedule;
			}
			else return floatingRateMultiplierSchedule = new Schedule.ScheduleBuilder();
		}
		
		
		public List<StrikeSchedule.StrikeScheduleBuilder> getFloorRateSchedule() {
			return floorRateSchedule;
		}
		
		public StrikeSchedule.StrikeScheduleBuilder getOrCreateFloorRateSchedule(int index) {
			if (floorRateSchedule==null) {
				this.floorRateSchedule = new ArrayList<>();
			}
			return getIndex(floorRateSchedule, index, ()->new StrikeSchedule.StrikeScheduleBuilder());
		}
		
		
		public Period.PeriodBuilder getIndexTenor() {
			return indexTenor;
		}
		
		public Period.PeriodBuilder getOrCreateIndexTenor() {
			if (indexTenor!=null) {
				return indexTenor;
			}
			else return indexTenor = new Period.PeriodBuilder();
		}
		
		
		public RateTreatmentEnum getRateTreatment() {
			return rateTreatment;
		}
		
		
		public List<SpreadSchedule.SpreadScheduleBuilder> getSpreadSchedule() {
			return spreadSchedule;
		}
		
		public SpreadSchedule.SpreadScheduleBuilder getOrCreateSpreadSchedule(int index) {
			if (spreadSchedule==null) {
				this.spreadSchedule = new ArrayList<>();
			}
			return getIndex(spreadSchedule, index, ()->new SpreadSchedule.SpreadScheduleBuilder());
		}
		
	
		public StubFloatingRateBuilder addCapRateSchedule(StrikeSchedule capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addCapRateSchedule(StrikeSchedule capRateSchedule, int _idx) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			getIndex(this.capRateSchedule, _idx, () -> capRateSchedule.toBuilder());
			this.capRateSchedule.set(_idx, capRateSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addCapRateSchedule(List<StrikeSchedule> capRateSchedules) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : capRateSchedules) {
				this.capRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		public StubFloatingRateBuilder addCapRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder capRateSchedule) {
			if(this.capRateSchedule == null){
				this.capRateSchedule = new ArrayList<>();
			}
			this.capRateSchedule.add(capRateSchedule);
			return this;
		}
		
		public StubFloatingRateBuilder clearCapRateSchedule() {
			this.capRateSchedule = null;
			return this;
		}
		public StubFloatingRateBuilder setFloatingRateIndex(FloatingRateIndexEnum floatingRateIndex) {
			this.floatingRateIndex = floatingRateIndex;
			return this;
		}
		public StubFloatingRateBuilder setFloatingRateMultiplierSchedule(Schedule floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule.toBuilder();
			return this;
		}
		public StubFloatingRateBuilder setFloatingRateMultiplierScheduleBuilder(Schedule.ScheduleBuilder floatingRateMultiplierSchedule) {
			this.floatingRateMultiplierSchedule = floatingRateMultiplierSchedule;
			return this;
		}
		public StubFloatingRateBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addFloorRateSchedule(StrikeSchedule floorRateSchedule, int _idx) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			getIndex(this.floorRateSchedule, _idx, () -> floorRateSchedule.toBuilder());
			this.floorRateSchedule.set(_idx, floorRateSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addFloorRateSchedule(List<StrikeSchedule> floorRateSchedules) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			for (StrikeSchedule toAdd : floorRateSchedules) {
				this.floorRateSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		public StubFloatingRateBuilder addFloorRateScheduleBuilder(StrikeSchedule.StrikeScheduleBuilder floorRateSchedule) {
			if(this.floorRateSchedule == null){
				this.floorRateSchedule = new ArrayList<>();
			}
			this.floorRateSchedule.add(floorRateSchedule);
			return this;
		}
		
		public StubFloatingRateBuilder clearFloorRateSchedule() {
			this.floorRateSchedule = null;
			return this;
		}
		public StubFloatingRateBuilder setIndexTenor(Period indexTenor) {
			this.indexTenor = indexTenor.toBuilder();
			return this;
		}
		public StubFloatingRateBuilder setIndexTenorBuilder(Period.PeriodBuilder indexTenor) {
			this.indexTenor = indexTenor;
			return this;
		}
		public StubFloatingRateBuilder setRateTreatment(RateTreatmentEnum rateTreatment) {
			this.rateTreatment = rateTreatment;
			return this;
		}
		public StubFloatingRateBuilder addSpreadSchedule(SpreadSchedule spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addSpreadSchedule(SpreadSchedule spreadSchedule, int _idx) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			getIndex(this.spreadSchedule, _idx, () -> spreadSchedule.toBuilder());
			this.spreadSchedule.set(_idx, spreadSchedule.toBuilder());
			return this;
		}
		public StubFloatingRateBuilder addSpreadSchedule(List<SpreadSchedule> spreadSchedules) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			for (SpreadSchedule toAdd : spreadSchedules) {
				this.spreadSchedule.add(toAdd.toBuilder());
			}
			return this;
		}
		public StubFloatingRateBuilder addSpreadScheduleBuilder(SpreadSchedule.SpreadScheduleBuilder spreadSchedule) {
			if(this.spreadSchedule == null){
				this.spreadSchedule = new ArrayList<>();
			}
			this.spreadSchedule.add(spreadSchedule);
			return this;
		}
		
		public StubFloatingRateBuilder clearSpreadSchedule() {
			this.spreadSchedule = null;
			return this;
		}
		
		public StubFloatingRate build() {
			return new StubFloatingRate(this);
		}
	
		@Override
		public StubFloatingRateBuilder prune() {
			if (capRateSchedule!=null) capRateSchedule = capRateSchedule.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (floatingRateMultiplierSchedule!=null && !floatingRateMultiplierSchedule.prune().hasData()) floatingRateMultiplierSchedule = null;
			if (floorRateSchedule!=null) floorRateSchedule = floorRateSchedule.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (indexTenor!=null && !indexTenor.prune().hasData()) indexTenor = null;
			if (spreadSchedule!=null) spreadSchedule = spreadSchedule.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCapRateSchedule()!=null && getCapRateSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFloatingRateIndex()!=null) return true;
			if (getFloatingRateMultiplierSchedule()!=null && getFloatingRateMultiplierSchedule().hasData()) return true;
			if (getFloorRateSchedule()!=null && getFloorRateSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getIndexTenor()!=null && getIndexTenor().hasData()) return true;
			if (getRateTreatment()!=null) return true;
			if (getSpreadSchedule()!=null && getSpreadSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("floatingRateIndex"), FloatingRateIndexEnum.class, floatingRateIndex, this);
			processor.processBasic(path.newSubPath("rateTreatment"), RateTreatmentEnum.class, rateTreatment, this);
			
			processRosetta(path.newSubPath("capRateSchedule"), processor, StrikeSchedule.class, capRateSchedule);
			processRosetta(path.newSubPath("floatingRateMultiplierSchedule"), processor, Schedule.class, floatingRateMultiplierSchedule);
			processRosetta(path.newSubPath("floorRateSchedule"), processor, StrikeSchedule.class, floorRateSchedule);
			processRosetta(path.newSubPath("indexTenor"), processor, Period.class, indexTenor);
			processRosetta(path.newSubPath("spreadSchedule"), processor, SpreadSchedule.class, spreadSchedule);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			StubFloatingRateBuilder _that = (StubFloatingRateBuilder) o;
		
			if (!ListEquals.listEquals(capRateSchedule, _that.capRateSchedule)) return false;
			if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
			if (floatingRateMultiplierSchedule != null ? !floatingRateMultiplierSchedule.equals(_that.floatingRateMultiplierSchedule) : _that.floatingRateMultiplierSchedule != null) return false;
			if (!ListEquals.listEquals(floorRateSchedule, _that.floorRateSchedule)) return false;
			if (indexTenor != null ? !indexTenor.equals(_that.indexTenor) : _that.indexTenor != null) return false;
			if (rateTreatment != null ? !rateTreatment.equals(_that.rateTreatment) : _that.rateTreatment != null) return false;
			if (!ListEquals.listEquals(spreadSchedule, _that.spreadSchedule)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (capRateSchedule != null ? capRateSchedule.hashCode() : 0);
			_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateMultiplierSchedule != null ? floatingRateMultiplierSchedule.hashCode() : 0);
			_result = 31 * _result + (floorRateSchedule != null ? floorRateSchedule.hashCode() : 0);
			_result = 31 * _result + (indexTenor != null ? indexTenor.hashCode() : 0);
			_result = 31 * _result + (rateTreatment != null ? rateTreatment.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadSchedule != null ? spreadSchedule.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StubFloatingRateBuilder {" +
				"capRateSchedule=" + this.capRateSchedule + ", " +
				"floatingRateIndex=" + this.floatingRateIndex + ", " +
				"floatingRateMultiplierSchedule=" + this.floatingRateMultiplierSchedule + ", " +
				"floorRateSchedule=" + this.floorRateSchedule + ", " +
				"indexTenor=" + this.indexTenor + ", " +
				"rateTreatment=" + this.rateTreatment + ", " +
				"spreadSchedule=" + this.spreadSchedule +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("floatingRateIndex"), FloatingRateIndexEnum.class, floatingRateIndex, this);
		processor.processBasic(path.newSubPath("rateTreatment"), RateTreatmentEnum.class, rateTreatment, this);
		
		processRosetta(path.newSubPath("capRateSchedule"), processor, StrikeSchedule.class, capRateSchedule);
		processRosetta(path.newSubPath("floatingRateMultiplierSchedule"), processor, Schedule.class, floatingRateMultiplierSchedule);
		processRosetta(path.newSubPath("floorRateSchedule"), processor, StrikeSchedule.class, floorRateSchedule);
		processRosetta(path.newSubPath("indexTenor"), processor, Period.class, indexTenor);
		processRosetta(path.newSubPath("spreadSchedule"), processor, SpreadSchedule.class, spreadSchedule);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		StubFloatingRate _that = (StubFloatingRate) o;
	
		if (!ListEquals.listEquals(capRateSchedule, _that.capRateSchedule)) return false;
		if (floatingRateIndex != null ? !floatingRateIndex.equals(_that.floatingRateIndex) : _that.floatingRateIndex != null) return false;
		if (floatingRateMultiplierSchedule != null ? !floatingRateMultiplierSchedule.equals(_that.floatingRateMultiplierSchedule) : _that.floatingRateMultiplierSchedule != null) return false;
		if (!ListEquals.listEquals(floorRateSchedule, _that.floorRateSchedule)) return false;
		if (indexTenor != null ? !indexTenor.equals(_that.indexTenor) : _that.indexTenor != null) return false;
		if (rateTreatment != null ? !rateTreatment.equals(_that.rateTreatment) : _that.rateTreatment != null) return false;
		if (!ListEquals.listEquals(spreadSchedule, _that.spreadSchedule)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (capRateSchedule != null ? capRateSchedule.hashCode() : 0);
		_result = 31 * _result + (floatingRateIndex != null ? floatingRateIndex.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (floatingRateMultiplierSchedule != null ? floatingRateMultiplierSchedule.hashCode() : 0);
		_result = 31 * _result + (floorRateSchedule != null ? floorRateSchedule.hashCode() : 0);
		_result = 31 * _result + (indexTenor != null ? indexTenor.hashCode() : 0);
		_result = 31 * _result + (rateTreatment != null ? rateTreatment.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (spreadSchedule != null ? spreadSchedule.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "StubFloatingRate {" +
			"capRateSchedule=" + this.capRateSchedule + ", " +
			"floatingRateIndex=" + this.floatingRateIndex + ", " +
			"floatingRateMultiplierSchedule=" + this.floatingRateMultiplierSchedule + ", " +
			"floorRateSchedule=" + this.floorRateSchedule + ", " +
			"indexTenor=" + this.indexTenor + ", " +
			"rateTreatment=" + this.rateTreatment + ", " +
			"spreadSchedule=" + this.spreadSchedule +
		'}';
	}
}
