package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify how to deal with a non standard calculation period within a swap stream.
 * 
 * @version 2.5.4
 */
public enum StubPeriodTypeEnum {
    

	/**
	 * If there is a non regular period remaining it is left shorter than the streams calculation period frequency and placed at the start of the stream.
	 */
	@RosettaSynonym(value = "ShortInitial", source = "FpML_5_10")
	@RosettaSynonym(value = "ShortInitial", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ShortInitial", source = "DTCC_11_0")
	@RosettaSynonym(value = "ShortInitial", source = "DTCC_9_0")
	@RosettaSynonym(value = "ShortInitial", source = "CME_ClearedConfirm_1_17")
	SHORT_INITIAL,
	    

	/**
	 * If there is a non regular period remaining it is left shorter than the streams calculation period frequency and placed at the end of the stream.
	 */
	@RosettaSynonym(value = "ShortFinal", source = "FpML_5_10")
	@RosettaSynonym(value = "ShortFinal", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ShortFinal", source = "DTCC_11_0")
	@RosettaSynonym(value = "ShortFinal", source = "DTCC_9_0")
	@RosettaSynonym(value = "ShortFinal", source = "CME_ClearedConfirm_1_17")
	SHORT_FINAL,
	    

	/**
	 * If there is a non regular period remaining it is placed at the start of the stream and combined with the adjacent calculation period to give a long first calculation period.
	 */
	@RosettaSynonym(value = "LongInitial", source = "FpML_5_10")
	@RosettaSynonym(value = "LongInitial", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LongInitial", source = "DTCC_11_0")
	@RosettaSynonym(value = "LongInitial", source = "DTCC_9_0")
	@RosettaSynonym(value = "LongInitial", source = "CME_ClearedConfirm_1_17")
	LONG_INITIAL,
	    

	/**
	 * If there is a non regular period remaining it is placed at the end of the stream and combined with the adjacent calculation period to give a long last calculation period.
	 */
	@RosettaSynonym(value = "LongFinal", source = "FpML_5_10")
	@RosettaSynonym(value = "LongFinal", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "LongFinal", source = "DTCC_11_0")
	@RosettaSynonym(value = "LongFinal", source = "DTCC_9_0")
	@RosettaSynonym(value = "LongFinal", source = "CME_ClearedConfirm_1_17")
	LONG_FINAL    
	;

	private final String displayName;
	
	StubPeriodTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
