package org.isda.cdm;

import org.isda.cdm.SwpIn;
import org.isda.cdm.SwpOut;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SwpMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Swp extends RosettaModelObject {
	private final SwpIn swpIn;
	private final SwpOut swpOut;
	private static SwpMeta metaData = new SwpMeta();
	
	Swp(SwpBuilder builder) {
		this.swpIn = ofNullable(builder.getSwpIn()).map(f->f.build()).orElse(null);
		this.swpOut = ofNullable(builder.getSwpOut()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="SwpIn", source="ISO20022")
	public final SwpIn getSwpIn() {
		return swpIn;
	}
	
	@RosettaSynonym(value="SwpOut", source="ISO20022")
	public final SwpOut getSwpOut() {
		return swpOut;
	}
	
	
	@Override
	public RosettaMetaData<? extends Swp> metaData() {
		return metaData;
	} 
	
	public SwpBuilder toBuilder() {
		SwpBuilder builder = new SwpBuilder();
		ofNullable(getSwpIn()).ifPresent(builder::setSwpIn);
		ofNullable(getSwpOut()).ifPresent(builder::setSwpOut);
		return builder;
	}

	public static SwpBuilder builder() {
		return new SwpBuilder();
	}

	public static  class SwpBuilder extends RosettaModelObjectBuilder {
	
		protected SwpIn.SwpInBuilder swpIn;
		protected SwpOut.SwpOutBuilder swpOut;
	
		public SwpBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Swp> metaData() {
			return metaData;
		} 
	
		
		public SwpIn.SwpInBuilder getSwpIn() {
			return swpIn;
		}
		
		public SwpIn.SwpInBuilder getOrCreateSwpIn() {
			if (swpIn!=null) {
				return swpIn;
			}
			else return swpIn = new SwpIn.SwpInBuilder();
		}
		
		
		public SwpOut.SwpOutBuilder getSwpOut() {
			return swpOut;
		}
		
		public SwpOut.SwpOutBuilder getOrCreateSwpOut() {
			if (swpOut!=null) {
				return swpOut;
			}
			else return swpOut = new SwpOut.SwpOutBuilder();
		}
		
	
		public SwpBuilder setSwpIn(SwpIn swpIn) {
			this.swpIn = swpIn.toBuilder();
			return this;
		}
		public SwpBuilder setSwpInBuilder(SwpIn.SwpInBuilder swpIn) {
			this.swpIn = swpIn;
			return this;
		}
		public SwpBuilder setSwpOut(SwpOut swpOut) {
			this.swpOut = swpOut.toBuilder();
			return this;
		}
		public SwpBuilder setSwpOutBuilder(SwpOut.SwpOutBuilder swpOut) {
			this.swpOut = swpOut;
			return this;
		}
		
		public Swp build() {
			return new Swp(this);
		}
	
		@Override
		public SwpBuilder prune() {
			if (swpIn!=null && !swpIn.prune().hasData()) swpIn = null;
			if (swpOut!=null && !swpOut.prune().hasData()) swpOut = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSwpIn()!=null && getSwpIn().hasData()) return true;
			if (getSwpOut()!=null && getSwpOut().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("swpIn"), processor, SwpIn.class, swpIn);
			processRosetta(path.newSubPath("swpOut"), processor, SwpOut.class, swpOut);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SwpBuilder _that = (SwpBuilder) o;
		
			if (swpIn != null ? !swpIn.equals(_that.swpIn) : _that.swpIn != null) return false;
			if (swpOut != null ? !swpOut.equals(_that.swpOut) : _that.swpOut != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (swpIn != null ? swpIn.hashCode() : 0);
			_result = 31 * _result + (swpOut != null ? swpOut.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SwpBuilder {" +
				"swpIn=" + this.swpIn + ", " +
				"swpOut=" + this.swpOut +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("swpIn"), processor, SwpIn.class, swpIn);
		processRosetta(path.newSubPath("swpOut"), processor, SwpOut.class, swpOut);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Swp _that = (Swp) o;
	
		if (swpIn != null ? !swpIn.equals(_that.swpIn) : _that.swpIn != null) return false;
		if (swpOut != null ? !swpOut.equals(_that.swpOut) : _that.swpOut != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (swpIn != null ? swpIn.hashCode() : 0);
		_result = 31 * _result + (swpOut != null ? swpOut.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Swp {" +
			"swpIn=" + this.swpIn + ", " +
			"swpOut=" + this.swpOut +
		'}';
	}
}
