package org.isda.cdm;

import org.isda.cdm.Sngl;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.SwpOutMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class SwpOut extends RosettaModelObject {
	private final Sngl sngl;
	private static SwpOutMeta metaData = new SwpOutMeta();
	
	SwpOut(SwpOutBuilder builder) {
		this.sngl = ofNullable(builder.getSngl()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Sngl", source="ISO20022")
	public final Sngl getSngl() {
		return sngl;
	}
	
	
	@Override
	public RosettaMetaData<? extends SwpOut> metaData() {
		return metaData;
	} 
	
	public SwpOutBuilder toBuilder() {
		SwpOutBuilder builder = new SwpOutBuilder();
		ofNullable(getSngl()).ifPresent(builder::setSngl);
		return builder;
	}

	public static SwpOutBuilder builder() {
		return new SwpOutBuilder();
	}

	public static  class SwpOutBuilder extends RosettaModelObjectBuilder {
	
		protected Sngl.SnglBuilder sngl;
	
		public SwpOutBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends SwpOut> metaData() {
			return metaData;
		} 
	
		
		public Sngl.SnglBuilder getSngl() {
			return sngl;
		}
		
		public Sngl.SnglBuilder getOrCreateSngl() {
			if (sngl!=null) {
				return sngl;
			}
			else return sngl = new Sngl.SnglBuilder();
		}
		
	
		public SwpOutBuilder setSngl(Sngl sngl) {
			this.sngl = sngl.toBuilder();
			return this;
		}
		public SwpOutBuilder setSnglBuilder(Sngl.SnglBuilder sngl) {
			this.sngl = sngl;
			return this;
		}
		
		public SwpOut build() {
			return new SwpOut(this);
		}
	
		@Override
		public SwpOutBuilder prune() {
			if (sngl!=null && !sngl.prune().hasData()) sngl = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSngl()!=null && getSngl().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("sngl"), processor, Sngl.class, sngl);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			SwpOutBuilder _that = (SwpOutBuilder) o;
		
			if (sngl != null ? !sngl.equals(_that.sngl) : _that.sngl != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (sngl != null ? sngl.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SwpOutBuilder {" +
				"sngl=" + this.sngl +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("sngl"), processor, Sngl.class, sngl);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		SwpOut _that = (SwpOut) o;
	
		if (sngl != null ? !sngl.equals(_that.sngl) : _that.sngl != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (sngl != null ? sngl.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "SwpOut {" +
			"sngl=" + this.sngl +
		'}';
	}
}
