package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.TelephoneTypeEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TelephoneNumberMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify a telephone number as a type of phone number (e.g. work, personal, ...) alongside with the actual number.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TelephoneNumber extends RosettaModelObject {
	private final String number;
	private final TelephoneTypeEnum type;
	private static TelephoneNumberMeta metaData = new TelephoneNumberMeta();
	
	TelephoneNumber(TelephoneNumberBuilder builder) {
		this.number = builder.getNumber();
		this.type = builder.getType();
	}
	
	/**
	 * The actual telephone number.
	 */
	public final String getNumber() {
		return number;
	}
	
	/**
	 * The type of telephone number, e.g. work, mobile.
	 */
	public final TelephoneTypeEnum getType() {
		return type;
	}
	
	
	@Override
	public RosettaMetaData<? extends TelephoneNumber> metaData() {
		return metaData;
	} 
	
	public TelephoneNumberBuilder toBuilder() {
		TelephoneNumberBuilder builder = new TelephoneNumberBuilder();
		ofNullable(getNumber()).ifPresent(builder::setNumber);
		ofNullable(getType()).ifPresent(builder::setType);
		return builder;
	}

	public static TelephoneNumberBuilder builder() {
		return new TelephoneNumberBuilder();
	}

	public static  class TelephoneNumberBuilder extends RosettaModelObjectBuilder {
	
		protected String number;
		protected TelephoneTypeEnum type;
	
		public TelephoneNumberBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TelephoneNumber> metaData() {
			return metaData;
		} 
	
		
		public String getNumber() {
			return number;
		}
		
		
		public TelephoneTypeEnum getType() {
			return type;
		}
		
	
		public TelephoneNumberBuilder setNumber(String number) {
			this.number = number;
			return this;
		}
		public TelephoneNumberBuilder setType(TelephoneTypeEnum type) {
			this.type = type;
			return this;
		}
		
		public TelephoneNumber build() {
			return new TelephoneNumber(this);
		}
	
		@Override
		public TelephoneNumberBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getNumber()!=null) return true;
			if (getType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("number"), String.class, number, this);
			processor.processBasic(path.newSubPath("type"), TelephoneTypeEnum.class, type, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TelephoneNumberBuilder _that = (TelephoneNumberBuilder) o;
		
			if (number != null ? !number.equals(_that.number) : _that.number != null) return false;
			if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (number != null ? number.hashCode() : 0);
			_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TelephoneNumberBuilder {" +
				"number=" + this.number + ", " +
				"type=" + this.type +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("number"), String.class, number, this);
		processor.processBasic(path.newSubPath("type"), TelephoneTypeEnum.class, type, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TelephoneNumber _that = (TelephoneNumber) o;
	
		if (number != null ? !number.equals(_that.number) : _that.number != null) return false;
		if (type != null ? !type.equals(_that.type) : _that.type != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (number != null ? number.hashCode() : 0);
		_result = 31 * _result + (type != null ? type.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TelephoneNumber {" +
			"number=" + this.number + ", " +
			"type=" + this.type +
		'}';
	}
}
