package org.isda.cdm;

import java.lang.String;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TermMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Term extends RosettaModelObject {
	private final String unit;
	private final String val;
	private static TermMeta metaData = new TermMeta();
	
	Term(TermBuilder builder) {
		this.unit = builder.getUnit();
		this.val = builder.getVal();
	}
	
	@RosettaSynonym(value="Unit", source="ISO20022")
	public final String getUnit() {
		return unit;
	}
	
	@RosettaSynonym(value="Val", source="ISO20022")
	public final String getVal() {
		return val;
	}
	
	
	@Override
	public RosettaMetaData<? extends Term> metaData() {
		return metaData;
	} 
	
	public TermBuilder toBuilder() {
		TermBuilder builder = new TermBuilder();
		ofNullable(getUnit()).ifPresent(builder::setUnit);
		ofNullable(getVal()).ifPresent(builder::setVal);
		return builder;
	}

	public static TermBuilder builder() {
		return new TermBuilder();
	}

	public static  class TermBuilder extends RosettaModelObjectBuilder {
	
		protected String unit;
		protected String val;
	
		public TermBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Term> metaData() {
			return metaData;
		} 
	
		
		public String getUnit() {
			return unit;
		}
		
		
		public String getVal() {
			return val;
		}
		
	
		public TermBuilder setUnit(String unit) {
			this.unit = unit;
			return this;
		}
		public TermBuilder setVal(String val) {
			this.val = val;
			return this;
		}
		
		public Term build() {
			return new Term(this);
		}
	
		@Override
		public TermBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUnit()!=null) return true;
			if (getVal()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("unit"), String.class, unit, this);
			processor.processBasic(path.newSubPath("val"), String.class, val, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TermBuilder _that = (TermBuilder) o;
		
			if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
			if (val != null ? !val.equals(_that.val) : _that.val != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (unit != null ? unit.hashCode() : 0);
			_result = 31 * _result + (val != null ? val.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TermBuilder {" +
				"unit=" + this.unit + ", " +
				"val=" + this.val +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("unit"), String.class, unit, this);
		processor.processBasic(path.newSubPath("val"), String.class, val, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Term _that = (Term) o;
	
		if (unit != null ? !unit.equals(_that.unit) : _that.unit != null) return false;
		if (val != null ? !val.equals(_that.val) : _that.val != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (unit != null ? unit.hashCode() : 0);
		_result = 31 * _result + (val != null ? val.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Term {" +
			"unit=" + this.unit + ", " +
			"val=" + this.val +
		'}';
	}
}
