package org.isda.cdm;

import org.isda.cdm.Trade;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TermsChangePrimitiveMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * The primitive event to represent change(s) to the contractual terms and the clearing submission and acceptance process.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TermsChangePrimitive extends RosettaModelObject {
	private final Trade after;
	private final Trade before;
	private static TermsChangePrimitiveMeta metaData = new TermsChangePrimitiveMeta();
	
	TermsChangePrimitive(TermsChangePrimitiveBuilder builder) {
		this.after = ofNullable(builder.getAfter()).map(f->f.build()).orElse(null);
		this.before = ofNullable(builder.getBefore()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="after", source="Rosetta_Workbench")
	public final Trade getAfter() {
		return after;
	}
	
	@RosettaSynonym(value="before", source="Rosetta_Workbench")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final Trade getBefore() {
		return before;
	}
	
	
	@Override
	public RosettaMetaData<? extends TermsChangePrimitive> metaData() {
		return metaData;
	} 
	
	public TermsChangePrimitiveBuilder toBuilder() {
		TermsChangePrimitiveBuilder builder = new TermsChangePrimitiveBuilder();
		ofNullable(getAfter()).ifPresent(builder::setAfter);
		ofNullable(getBefore()).ifPresent(builder::setBefore);
		return builder;
	}

	public static TermsChangePrimitiveBuilder builder() {
		return new TermsChangePrimitiveBuilder();
	}

	public static  class TermsChangePrimitiveBuilder extends RosettaModelObjectBuilder {
	
		protected Trade.TradeBuilder after;
		protected Trade.TradeBuilder before;
	
		public TermsChangePrimitiveBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TermsChangePrimitive> metaData() {
			return metaData;
		} 
	
		
		public Trade.TradeBuilder getAfter() {
			return after;
		}
		
		public Trade.TradeBuilder getOrCreateAfter() {
			if (after!=null) {
				return after;
			}
			else return after = new Trade.TradeBuilder();
		}
		
		
		public Trade.TradeBuilder getBefore() {
			return before;
		}
		
		public Trade.TradeBuilder getOrCreateBefore() {
			if (before!=null) {
				return before;
			}
			else return before = new Trade.TradeBuilder();
		}
		
	
		public TermsChangePrimitiveBuilder setAfter(Trade after) {
			this.after = after.toBuilder();
			return this;
		}
		public TermsChangePrimitiveBuilder setAfterBuilder(Trade.TradeBuilder after) {
			this.after = after;
			return this;
		}
		public TermsChangePrimitiveBuilder setBefore(Trade before) {
			this.before = before.toBuilder();
			return this;
		}
		public TermsChangePrimitiveBuilder setBeforeBuilder(Trade.TradeBuilder before) {
			this.before = before;
			return this;
		}
		
		public TermsChangePrimitive build() {
			return new TermsChangePrimitive(this);
		}
	
		@Override
		public TermsChangePrimitiveBuilder prune() {
			if (after!=null && !after.prune().hasData()) after = null;
			if (before!=null && !before.prune().hasData()) before = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAfter()!=null && getAfter().hasData()) return true;
			if (getBefore()!=null && getBefore().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("after"), processor, Trade.class, after);
			processRosetta(path.newSubPath("before"), processor, Trade.class, before);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TermsChangePrimitiveBuilder _that = (TermsChangePrimitiveBuilder) o;
		
			if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
			if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (after != null ? after.hashCode() : 0);
			_result = 31 * _result + (before != null ? before.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TermsChangePrimitiveBuilder {" +
				"after=" + this.after + ", " +
				"before=" + this.before +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("after"), processor, Trade.class, after);
		processRosetta(path.newSubPath("before"), processor, Trade.class, before);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TermsChangePrimitive _that = (TermsChangePrimitive) o;
	
		if (after != null ? !after.equals(_that.after) : _that.after != null) return false;
		if (before != null ? !before.equals(_that.before) : _that.before != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (after != null ? after.hashCode() : 0);
		_result = 31 * _result + (before != null ? before.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TermsChangePrimitive {" +
			"after=" + this.after + ", " +
			"before=" + this.before +
		'}';
	}
}
