package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify points in the day when option exercise and valuation can occur.
 * 
 * @version 2.5.4
 */
public enum TimeTypeEnum {
    

	/**
	 * The official closing time of the exchange on the valuation date.
	 */
	@RosettaSynonym(value = "Close", source = "FpML_5_10")
	@RosettaSynonym(value = "Close", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Close", source = "DTCC_11_0")
	@RosettaSynonym(value = "Close", source = "DTCC_9_0")
	@RosettaSynonym(value = "Close", source = "CME_ClearedConfirm_1_17")
	CLOSE,
	    

	/**
	 * The official opening time of the exchange on the valuation date.
	 */
	@RosettaSynonym(value = "Open", source = "FpML_5_10")
	@RosettaSynonym(value = "Open", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Open", source = "DTCC_11_0")
	@RosettaSynonym(value = "Open", source = "DTCC_9_0")
	@RosettaSynonym(value = "Open", source = "CME_ClearedConfirm_1_17")
	OPEN,
	    

	/**
	 * The time at which the official settlement price is determined.
	 */
	@RosettaSynonym(value = "OSP", source = "FpML_5_10")
	@RosettaSynonym(value = "OSP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "OSP", source = "DTCC_11_0")
	@RosettaSynonym(value = "OSP", source = "DTCC_9_0")
	@RosettaSynonym(value = "OSP", source = "CME_ClearedConfirm_1_17")
	OSP,
	    

	/**
	 * The time specified in the element equityExpirationTime or valuationTime (as appropriate).
	 */
	@RosettaSynonym(value = "SpecificTime", source = "FpML_5_10")
	@RosettaSynonym(value = "SpecificTime", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SpecificTime", source = "DTCC_11_0")
	@RosettaSynonym(value = "SpecificTime", source = "DTCC_9_0")
	@RosettaSynonym(value = "SpecificTime", source = "CME_ClearedConfirm_1_17")
	SPECIFIC_TIME,
	    

	/**
	 * The time at which the official settlement price (following the auction by the exchange) is determined by the exchange.
	 */
	@RosettaSynonym(value = "XETRA", source = "FpML_5_10")
	@RosettaSynonym(value = "XETRA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "XETRA", source = "DTCC_11_0")
	@RosettaSynonym(value = "XETRA", source = "DTCC_9_0")
	@RosettaSynonym(value = "XETRA", source = "CME_ClearedConfirm_1_17")
	XETRA,
	    

	/**
	 * The official closing time of the derivatives exchange on which a derivative contract is listed on that security underlier.
	 */
	@RosettaSynonym(value = "DerivativesClose", source = "FpML_5_10")
	@RosettaSynonym(value = "DerivativesClose", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DerivativesClose", source = "DTCC_11_0")
	@RosettaSynonym(value = "DerivativesClose", source = "DTCC_9_0")
	@RosettaSynonym(value = "DerivativesClose", source = "CME_ClearedConfirm_1_17")
	DERIVATIVES_CLOSE,
	    

	/**
	 * The time is determined as provided in the relevant Master Confirmation.
	 */
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "FpML_5_10")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "DTCC_11_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "DTCC_9_0")
	@RosettaSynonym(value = "AsSpecifiedInMasterConfirmation", source = "CME_ClearedConfirm_1_17")
	AS_SPECIFIED_IN_MASTER_CONFIRMATION    
	;

	private final String displayName;
	
	TimeTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
