package org.isda.cdm;

import java.time.LocalTime;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TimeZoneMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.time.LocalTime;
		import static java.util.Optional.ofNullable;

/**
 * The time alongside with the timezone location information. This class makes use of the FpML TimezoneLocation construct.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TimeZone extends RosettaModelObject {
	private final FieldWithMetaString location;
	private final LocalTime time;
	private static TimeZoneMeta metaData = new TimeZoneMeta();
	
	TimeZone(TimeZoneBuilder builder) {
		this.location = ofNullable(builder.getLocation()).map(f->f.build()).orElse(null);
		this.time = builder.getTime();
	}
	
	/**
	 * FpML specifies the timezoneLocationScheme by reference to the Time Zone Database (a.k.a. tz database) maintained by IANA, the Internet Assigned Numbers Authority.
	 */
	@RosettaSynonym(value="location", source="Rosetta_Workbench")
	public final FieldWithMetaString getLocation() {
		return location;
	}
	
	/**
	 * The observation time.
	 */
	@RosettaSynonym(value="time", source="Rosetta_Workbench")
	public final LocalTime getTime() {
		return time;
	}
	
	
	@Override
	public RosettaMetaData<? extends TimeZone> metaData() {
		return metaData;
	} 
	
	public TimeZoneBuilder toBuilder() {
		TimeZoneBuilder builder = new TimeZoneBuilder();
		ofNullable(getLocation()).ifPresent(builder::setLocation);
		ofNullable(getTime()).ifPresent(builder::setTime);
		return builder;
	}

	public static TimeZoneBuilder builder() {
		return new TimeZoneBuilder();
	}

	public static  class TimeZoneBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder location;
		protected LocalTime time;
	
		public TimeZoneBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TimeZone> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getLocation() {
			return location;
		}
		
		public FieldWithMetaStringBuilder getOrCreateLocation() {
			if (location!=null) {
				return location;
			}
			else return location = new FieldWithMetaStringBuilder();
		}
		
		
		public LocalTime getTime() {
			return time;
		}
		
	
		public TimeZoneBuilder setLocation(FieldWithMetaString location) {
			this.location = location.toBuilder();
			return this;
		}
		public TimeZoneBuilder setLocationRef(String location) {
			return setLocation(FieldWithMetaString.builder().setValue(location).build());
		}
		public TimeZoneBuilder setTime(LocalTime time) {
			this.time = time;
			return this;
		}
		
		public TimeZone build() {
			return new TimeZone(this);
		}
	
		@Override
		public TimeZoneBuilder prune() {
			if (location!=null && !location.prune().hasData()) location = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getLocation()!=null) return true;
			if (getTime()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("time"), LocalTime.class, time, this);
			
			processRosetta(path.newSubPath("location"), processor, FieldWithMetaString.class, location);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TimeZoneBuilder _that = (TimeZoneBuilder) o;
		
			if (location != null ? !location.equals(_that.location) : _that.location != null) return false;
			if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (location != null ? location.hashCode() : 0);
			_result = 31 * _result + (time != null ? time.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TimeZoneBuilder {" +
				"location=" + this.location + ", " +
				"time=" + this.time +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("time"), LocalTime.class, time, this);
		
		processRosetta(path.newSubPath("location"), processor, FieldWithMetaString.class, location);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TimeZone _that = (TimeZone) o;
	
		if (location != null ? !location.equals(_that.location) : _that.location != null) return false;
		if (time != null ? !time.equals(_that.time) : _that.time != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (location != null ? location.hashCode() : 0);
		_result = 31 * _result + (time != null ? time.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TimeZone {" +
			"location=" + this.location + ", " +
			"time=" + this.time +
		'}';
	}
}
