package org.isda.cdm;

import com.rosetta.model.lib.records.Date;
import org.isda.cdm.metafields.MetaFields;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TradeDateMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import org.isda.cdm.metafields.MetaFields;
import com.rosetta.model.lib.GlobalKey;
import com.rosetta.model.lib.GlobalKeyBuilder;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import com.rosetta.model.lib.records.Date;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify the contract&#39;s trade date alongside an identifier.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TradeDate extends RosettaModelObject implements GlobalKey {
	private final Date date;
	private final MetaFields meta;
	private static TradeDateMeta metaData = new TradeDateMeta();
	
	TradeDate(TradeDateBuilder builder) {
		this.date = builder.getDate();
		this.meta = ofNullable(builder.getMeta()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The trade date associated with the contract. In the case of a novated trade, this date corresponds to the novation date.
	 */
	@RosettaSynonym(value="tradeDate", source="FpML_5_10", path="trade.tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_SubmissionIRS_1_0", path="trade.tradeHeader")
	@RosettaSynonym(value="tradeDate", source="DTCC_11_0", path="trade.tradeHeader")
	@RosettaSynonym(value="tradeDate", source="DTCC_9_0", path="trade.tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_ClearedConfirm_1_17", path="trade.tradeHeader")
	@RosettaSynonym(value="tradeDate", source="FpML_5_10", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_SubmissionIRS_1_0", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="DTCC_11_0", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="DTCC_9_0", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_ClearedConfirm_1_17", path="tradeHeader")
	@RosettaSynonym(value="tradeDate", source="CME_SubmissionIRS_1_0", path="Instrmt.SecXML.FpML.trade.tradeHeader")
	public final Date getDate() {
		return date;
	}
	
	public final MetaFields getMeta() {
		return meta;
	}
	
	
	@Override
	public RosettaMetaData<? extends TradeDate> metaData() {
		return metaData;
	} 
	
	public TradeDateBuilder toBuilder() {
		TradeDateBuilder builder = new TradeDateBuilder();
		ofNullable(getDate()).ifPresent(builder::setDate);
		ofNullable(getMeta()).ifPresent(builder::setMeta);
		return builder;
	}

	public static TradeDateBuilder builder() {
		return new TradeDateBuilder();
	}

	public static  class TradeDateBuilder extends RosettaModelObjectBuilder implements GlobalKeyBuilder<TradeDateBuilder> {
	
		protected Date date;
		protected MetaFields.MetaFieldsBuilder meta;
	
		public TradeDateBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TradeDate> metaData() {
			return metaData;
		} 
	
		
		public Date getDate() {
			return date;
		}
		
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			if (meta!=null) {
				return meta;
			}
			else return meta = new MetaFields.MetaFieldsBuilder();
		}
		
	
		public TradeDateBuilder setDate(Date date) {
			this.date = date;
			return this;
		}
		public TradeDateBuilder setMeta(MetaFields meta) {
			this.meta = meta.toBuilder();
			return this;
		}
		public TradeDateBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public TradeDate build() {
			return new TradeDate(this);
		}
	
		@Override
		public TradeDateBuilder prune() {
			if (meta!=null && !meta.prune().hasData()) meta = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDate()!=null) return true;
			if (getMeta()!=null && getMeta().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("date"), Date.class, date, this);
			
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TradeDateBuilder _that = (TradeDateBuilder) o;
		
			if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
			if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (date != null ? date.hashCode() : 0);
			_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TradeDateBuilder {" +
				"date=" + this.date + ", " +
				"meta=" + this.meta +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("date"), Date.class, date, this);
		
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TradeDate _that = (TradeDate) o;
	
		if (date != null ? !date.equals(_that.date) : _that.date != null) return false;
		if (meta != null ? !meta.equals(_that.meta) : _that.meta != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (date != null ? date.hashCode() : 0);
		_result = 31 * _result + (meta != null ? meta.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TradeDate {" +
			"date=" + this.date + ", " +
			"meta=" + this.meta +
		'}';
	}
}
