package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.PartyCustomisedWorkflow;
import org.isda.cdm.WarehouseIdentityEnum;
import org.isda.cdm.WorkflowStatusEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TradeWarehouseWorkflowMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * A class to specify trade warehouse workflow information: the identity of the trade warehouse, the contract status at the warehouse and party-specific workflow information.
 * 
 * @version 2.5.4
 */
@RosettaClass
public class TradeWarehouseWorkflow extends RosettaModelObject {
	private final List<PartyCustomisedWorkflow> partyCustomisedWorkflow;
	private final WarehouseIdentityEnum warehouseIdentity;
	private final WorkflowStatusEnum warehouseStatus;
	private static TradeWarehouseWorkflowMeta metaData = new TradeWarehouseWorkflowMeta();
	
	TradeWarehouseWorkflow(TradeWarehouseWorkflowBuilder builder) {
		this.partyCustomisedWorkflow = ofNullable(builder.getPartyCustomisedWorkflow()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.warehouseIdentity = builder.getWarehouseIdentity();
		this.warehouseStatus = builder.getWarehouseStatus();
	}
	
	/**
	 * Non-standardized workflow data related to a party.
	 */
	@RosettaSynonym(value="WorkflowData", source="DTCC_11_0")
	@RosettaSynonym(value="WorkflowData", source="DTCC_9_0")
	@RosettaSynonym(value="ignore", source="CME_SubmissionIRS_1_0")
	public final List<PartyCustomisedWorkflow> getPartyCustomisedWorkflow() {
		return partyCustomisedWorkflow;
	}
	
	@RosettaSynonym(value="WarehousePositionType", source="DTCC_11_0", path="WarehouseState")
	@RosettaSynonym(value="WarehousePositionType", source="DTCC_9_0", path="WarehouseState")
	public final WarehouseIdentityEnum getWarehouseIdentity() {
		return warehouseIdentity;
	}
	
	@RosettaSynonym(value="WarehouseStatus", source="DTCC_11_0", path="WarehouseState")
	@RosettaSynonym(value="WarehouseStatus", source="DTCC_9_0", path="WarehouseState")
	public final WorkflowStatusEnum getWarehouseStatus() {
		return warehouseStatus;
	}
	
	
	@Override
	public RosettaMetaData<? extends TradeWarehouseWorkflow> metaData() {
		return metaData;
	} 
	
	public TradeWarehouseWorkflowBuilder toBuilder() {
		TradeWarehouseWorkflowBuilder builder = new TradeWarehouseWorkflowBuilder();
		ofNullable(getPartyCustomisedWorkflow()).ifPresent(partyCustomisedWorkflow -> partyCustomisedWorkflow.forEach(builder::addPartyCustomisedWorkflow));
		ofNullable(getWarehouseIdentity()).ifPresent(builder::setWarehouseIdentity);
		ofNullable(getWarehouseStatus()).ifPresent(builder::setWarehouseStatus);
		return builder;
	}

	public static TradeWarehouseWorkflowBuilder builder() {
		return new TradeWarehouseWorkflowBuilder();
	}

	public static  class TradeWarehouseWorkflowBuilder extends RosettaModelObjectBuilder {
	
		protected List<PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder> partyCustomisedWorkflow;
		protected WarehouseIdentityEnum warehouseIdentity;
		protected WorkflowStatusEnum warehouseStatus;
	
		public TradeWarehouseWorkflowBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TradeWarehouseWorkflow> metaData() {
			return metaData;
		} 
	
		
		public List<PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder> getPartyCustomisedWorkflow() {
			return partyCustomisedWorkflow;
		}
		
		public PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder getOrCreatePartyCustomisedWorkflow(int index) {
			if (partyCustomisedWorkflow==null) {
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			return getIndex(partyCustomisedWorkflow, index, ()->new PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder());
		}
		
		
		public WarehouseIdentityEnum getWarehouseIdentity() {
			return warehouseIdentity;
		}
		
		
		public WorkflowStatusEnum getWarehouseStatus() {
			return warehouseStatus;
		}
		
	
		public TradeWarehouseWorkflowBuilder addPartyCustomisedWorkflow(PartyCustomisedWorkflow partyCustomisedWorkflow) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			this.partyCustomisedWorkflow.add(partyCustomisedWorkflow.toBuilder());
			return this;
		}
		public TradeWarehouseWorkflowBuilder addPartyCustomisedWorkflow(PartyCustomisedWorkflow partyCustomisedWorkflow, int _idx) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			getIndex(this.partyCustomisedWorkflow, _idx, () -> partyCustomisedWorkflow.toBuilder());
			this.partyCustomisedWorkflow.set(_idx, partyCustomisedWorkflow.toBuilder());
			return this;
		}
		public TradeWarehouseWorkflowBuilder addPartyCustomisedWorkflow(List<PartyCustomisedWorkflow> partyCustomisedWorkflows) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			for (PartyCustomisedWorkflow toAdd : partyCustomisedWorkflows) {
				this.partyCustomisedWorkflow.add(toAdd.toBuilder());
			}
			return this;
		}
		public TradeWarehouseWorkflowBuilder addPartyCustomisedWorkflowBuilder(PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder partyCustomisedWorkflow) {
			if(this.partyCustomisedWorkflow == null){
				this.partyCustomisedWorkflow = new ArrayList<>();
			}
			this.partyCustomisedWorkflow.add(partyCustomisedWorkflow);
			return this;
		}
		
		public TradeWarehouseWorkflowBuilder clearPartyCustomisedWorkflow() {
			this.partyCustomisedWorkflow = null;
			return this;
		}
		public TradeWarehouseWorkflowBuilder setWarehouseIdentity(WarehouseIdentityEnum warehouseIdentity) {
			this.warehouseIdentity = warehouseIdentity;
			return this;
		}
		public TradeWarehouseWorkflowBuilder setWarehouseStatus(WorkflowStatusEnum warehouseStatus) {
			this.warehouseStatus = warehouseStatus;
			return this;
		}
		
		public TradeWarehouseWorkflow build() {
			return new TradeWarehouseWorkflow(this);
		}
	
		@Override
		public TradeWarehouseWorkflowBuilder prune() {
			if (partyCustomisedWorkflow!=null) partyCustomisedWorkflow = partyCustomisedWorkflow.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPartyCustomisedWorkflow()!=null && getPartyCustomisedWorkflow().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getWarehouseIdentity()!=null) return true;
			if (getWarehouseStatus()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("warehouseIdentity"), WarehouseIdentityEnum.class, warehouseIdentity, this);
			processor.processBasic(path.newSubPath("warehouseStatus"), WorkflowStatusEnum.class, warehouseStatus, this);
			
			processRosetta(path.newSubPath("partyCustomisedWorkflow"), processor, PartyCustomisedWorkflow.class, partyCustomisedWorkflow);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TradeWarehouseWorkflowBuilder _that = (TradeWarehouseWorkflowBuilder) o;
		
			if (!ListEquals.listEquals(partyCustomisedWorkflow, _that.partyCustomisedWorkflow)) return false;
			if (warehouseIdentity != null ? !warehouseIdentity.equals(_that.warehouseIdentity) : _that.warehouseIdentity != null) return false;
			if (warehouseStatus != null ? !warehouseStatus.equals(_that.warehouseStatus) : _that.warehouseStatus != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (partyCustomisedWorkflow != null ? partyCustomisedWorkflow.hashCode() : 0);
			_result = 31 * _result + (warehouseIdentity != null ? warehouseIdentity.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (warehouseStatus != null ? warehouseStatus.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TradeWarehouseWorkflowBuilder {" +
				"partyCustomisedWorkflow=" + this.partyCustomisedWorkflow + ", " +
				"warehouseIdentity=" + this.warehouseIdentity + ", " +
				"warehouseStatus=" + this.warehouseStatus +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("warehouseIdentity"), WarehouseIdentityEnum.class, warehouseIdentity, this);
		processor.processBasic(path.newSubPath("warehouseStatus"), WorkflowStatusEnum.class, warehouseStatus, this);
		
		processRosetta(path.newSubPath("partyCustomisedWorkflow"), processor, PartyCustomisedWorkflow.class, partyCustomisedWorkflow);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TradeWarehouseWorkflow _that = (TradeWarehouseWorkflow) o;
	
		if (!ListEquals.listEquals(partyCustomisedWorkflow, _that.partyCustomisedWorkflow)) return false;
		if (warehouseIdentity != null ? !warehouseIdentity.equals(_that.warehouseIdentity) : _that.warehouseIdentity != null) return false;
		if (warehouseStatus != null ? !warehouseStatus.equals(_that.warehouseStatus) : _that.warehouseStatus != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (partyCustomisedWorkflow != null ? partyCustomisedWorkflow.hashCode() : 0);
		_result = 31 * _result + (warehouseIdentity != null ? warehouseIdentity.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (warehouseStatus != null ? warehouseStatus.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TradeWarehouseWorkflow {" +
			"partyCustomisedWorkflow=" + this.partyCustomisedWorkflow + ", " +
			"warehouseIdentity=" + this.warehouseIdentity + ", " +
			"warehouseStatus=" + this.warehouseStatus +
		'}';
	}
}
