package org.isda.cdm;

import org.isda.cdm.TransferCalculation;
import org.isda.cdm.metafields.FieldWithMetaString;
import org.isda.cdm.metafields.FieldWithMetaString.FieldWithMetaStringBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransferBaseMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class TransferBase extends RosettaModelObject {
	private final FieldWithMetaString identifier;
	private final TransferCalculation transferCalculation;
	private static TransferBaseMeta metaData = new TransferBaseMeta();
	
	TransferBase(TransferBaseBuilder builder) {
		this.identifier = ofNullable(builder.getIdentifier()).map(f->f.build()).orElse(null);
		this.transferCalculation = ofNullable(builder.getTransferCalculation()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The identifier that can be associated with each of the transfer components
	 */
	@RosettaSynonym(value="identifier", source="DTCC_11_0")
	@RosettaSynonym(value="identifier", source="DTCC_9_0")
	public final FieldWithMetaString getIdentifier() {
		return identifier;
	}
	
	/**
	 * The calculation details underlying the transfer amount, when applicable.
	 */
	@RosettaSynonym(value="calculationDetails", source="DTCC_11_0")
	@RosettaSynonym(value="calculationDetails", source="DTCC_9_0")
	public final TransferCalculation getTransferCalculation() {
		return transferCalculation;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransferBase> metaData() {
		return metaData;
	} 
	
	public TransferBaseBuilder toBuilder() {
		TransferBaseBuilder builder = new TransferBaseBuilder();
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static TransferBaseBuilder builder() {
		return new TransferBaseBuilder();
	}

	public static  class TransferBaseBuilder extends RosettaModelObjectBuilder {
	
		protected FieldWithMetaStringBuilder identifier;
		protected TransferCalculation.TransferCalculationBuilder transferCalculation;
	
		public TransferBaseBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransferBase> metaData() {
			return metaData;
		} 
	
		
		public FieldWithMetaStringBuilder getIdentifier() {
			return identifier;
		}
		
		public FieldWithMetaStringBuilder getOrCreateIdentifier() {
			if (identifier!=null) {
				return identifier;
			}
			else return identifier = new FieldWithMetaStringBuilder();
		}
		
		
		public TransferCalculation.TransferCalculationBuilder getTransferCalculation() {
			return transferCalculation;
		}
		
		public TransferCalculation.TransferCalculationBuilder getOrCreateTransferCalculation() {
			if (transferCalculation!=null) {
				return transferCalculation;
			}
			else return transferCalculation = new TransferCalculation.TransferCalculationBuilder();
		}
		
	
		public TransferBaseBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		public TransferBaseBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		public TransferBaseBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		public TransferBaseBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public TransferBase build() {
			return new TransferBase(this);
		}
	
		@Override
		public TransferBaseBuilder prune() {
			if (identifier!=null && !identifier.prune().hasData()) identifier = null;
			if (transferCalculation!=null && !transferCalculation.prune().hasData()) transferCalculation = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null) return true;
			if (getTransferCalculation()!=null && getTransferCalculation().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
			processRosetta(path.newSubPath("transferCalculation"), processor, TransferCalculation.class, transferCalculation);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TransferBaseBuilder _that = (TransferBaseBuilder) o;
		
			if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
			if (transferCalculation != null ? !transferCalculation.equals(_that.transferCalculation) : _that.transferCalculation != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			_result = 31 * _result + (transferCalculation != null ? transferCalculation.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransferBaseBuilder {" +
				"identifier=" + this.identifier + ", " +
				"transferCalculation=" + this.transferCalculation +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("identifier"), processor, FieldWithMetaString.class, identifier);
		processRosetta(path.newSubPath("transferCalculation"), processor, TransferCalculation.class, transferCalculation);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TransferBase _that = (TransferBase) o;
	
		if (identifier != null ? !identifier.equals(_that.identifier) : _that.identifier != null) return false;
		if (transferCalculation != null ? !transferCalculation.equals(_that.transferCalculation) : _that.transferCalculation != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
		_result = 31 * _result + (transferCalculation != null ? transferCalculation.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransferBase {" +
			"identifier=" + this.identifier + ", " +
			"transferCalculation=" + this.transferCalculation +
		'}';
	}
}
