package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.Lineage;
import org.isda.cdm.metafields.FieldWithMetaString;


import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TransferBreakdownMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class TransferBreakdown extends TransferBase {
	private final List<Lineage> lineage;
	private static TransferBreakdownMeta metaData = new TransferBreakdownMeta();
	
	TransferBreakdown(TransferBreakdownBuilder builder) {
		super(builder);
		this.lineage = ofNullable(builder.getLineage()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
	}
	
	/**
	 * The lineage into the transfer components that might be associated with each of the transfer components.
	 */
	@RosettaSynonym(value="lineage", source="Rosetta_Workbench")
	public final List<Lineage> getLineage() {
		return lineage;
	}
	
	
	@Override
	public RosettaMetaData<? extends TransferBreakdown> metaData() {
		return metaData;
	} 
	
	public TransferBreakdownBuilder toBuilder() {
		TransferBreakdownBuilder builder = new TransferBreakdownBuilder();
		ofNullable(getLineage()).ifPresent(lineage -> lineage.forEach(builder::addLineage));
		ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		ofNullable(getTransferCalculation()).ifPresent(builder::setTransferCalculation);
		return builder;
	}

	public static TransferBreakdownBuilder builder() {
		return new TransferBreakdownBuilder();
	}

	public static  class TransferBreakdownBuilder extends TransferBaseBuilder {
	
		protected List<Lineage.LineageBuilder> lineage;
	
		public TransferBreakdownBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TransferBreakdown> metaData() {
			return metaData;
		} 
	
		
		public List<Lineage.LineageBuilder> getLineage() {
			return lineage;
		}
		
		public Lineage.LineageBuilder getOrCreateLineage(int index) {
			if (lineage==null) {
				this.lineage = new ArrayList<>();
			}
			return getIndex(lineage, index, ()->new Lineage.LineageBuilder());
		}
		
	
		public TransferBreakdownBuilder addLineage(Lineage lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage.toBuilder());
			return this;
		}
		public TransferBreakdownBuilder addLineage(Lineage lineage, int _idx) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			getIndex(this.lineage, _idx, () -> lineage.toBuilder());
			this.lineage.set(_idx, lineage.toBuilder());
			return this;
		}
		public TransferBreakdownBuilder addLineage(List<Lineage> lineages) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			for (Lineage toAdd : lineages) {
				this.lineage.add(toAdd.toBuilder());
			}
			return this;
		}
		public TransferBreakdownBuilder addLineageBuilder(Lineage.LineageBuilder lineage) {
			if(this.lineage == null){
				this.lineage = new ArrayList<>();
			}
			this.lineage.add(lineage);
			return this;
		}
		
		public TransferBreakdownBuilder clearLineage() {
			this.lineage = null;
			return this;
		}
		@Override public TransferBreakdownBuilder setIdentifier(FieldWithMetaString identifier) {
			this.identifier = identifier.toBuilder();
			return this;
		}
		@Override public TransferBreakdownBuilder setIdentifierRef(String identifier) {
			return setIdentifier(FieldWithMetaString.builder().setValue(identifier).build());
		}
		@Override public TransferBreakdownBuilder setTransferCalculation(TransferCalculation transferCalculation) {
			this.transferCalculation = transferCalculation.toBuilder();
			return this;
		}
		@Override public TransferBreakdownBuilder setTransferCalculationBuilder(TransferCalculation.TransferCalculationBuilder transferCalculation) {
			this.transferCalculation = transferCalculation;
			return this;
		}
		
		public TransferBreakdown build() {
			return new TransferBreakdown(this);
		}
	
		@Override
		public TransferBreakdownBuilder prune() {
			super.prune();
			if (lineage!=null) lineage = lineage.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			if (getLineage()!=null && getLineage().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
			processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			TransferBreakdownBuilder _that = (TransferBreakdownBuilder) o;
		
			if (!ListEquals.listEquals(lineage, _that.lineage)) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TransferBreakdownBuilder {" +
				"lineage=" + this.lineage +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
		processRosetta(path.newSubPath("lineage"), processor, Lineage.class, lineage);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		TransferBreakdown _that = (TransferBreakdown) o;
	
		if (!ListEquals.listEquals(lineage, _that.lineage)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		_result = 31 * _result + (lineage != null ? lineage.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TransferBreakdown {" +
			"lineage=" + this.lineage +
		'}' + " " + super.toString();
	}
}
