package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to specify how the transfer will settle, e.g. DvP.
 * 
 * @version 2.5.4
 */
public enum TransferSettlementEnum {
    

	/**
	 * Simultaneous transfer of two assets, typically securities, as a way to avoid settlement risk.
	 */
	@RosettaSynonym(value = "DeliveryVersusDelivery", source = "Rosetta_Workbench")
	DELIVERY_VERSUS_DELIVERY,
	    

	/**
	 * Settlement in which the transfer of the asset and the cash settlement are simultaneous.
	 */
	@RosettaSynonym(value = "DeliveryVersusPayment", source = "Rosetta_Workbench")
	DELIVERY_VERSUS_PAYMENT,
	    

	/**
	 * Simultaneous transfer of cashflows.
	 */
	@RosettaSynonym(value = "PaymentVersusPayment", source = "Rosetta_Workbench")
	@RosettaSynonym(value = "CentralSettlement", source = "DTCC_11_0")
	@RosettaSynonym(value = "CentralSettlement", source = "DTCC_9_0")
	PAYMENT_VERSUS_PAYMENT,
	    

	/**
	 * No central settlement.
	 */
	@RosettaSynonym(value = "NotCentralSettlement", source = "DTCC_11_0")
	@RosettaSynonym(value = "NotCentralSettlement", source = "DTCC_9_0")
	NOT_CENTRAL_SETTLEMENT    
	;

	private final String displayName;
	
	TransferSettlementEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
