package org.isda.cdm;

import java.math.BigDecimal;
import org.isda.cdm.CreditEvents;
import org.isda.cdm.TriggerTimeTypeEnum;
import org.isda.cdm.TriggerTypeEnum;
import org.isda.cdm.metafields.ReferenceWithMetaCreditEvents;
import org.isda.cdm.metafields.ReferenceWithMetaCreditEvents.ReferenceWithMetaCreditEventsBuilder;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TriggerMeta;
import org.isda.cdm.metafields.*;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.math.BigDecimal;
		import static java.util.Optional.ofNullable;
		import java.util.stream.Collectors;

/**
 * Trigger point at which feature is effective.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Trigger", source="FpML_5_10")
public class Trigger extends RosettaModelObject {
	private final CreditEvents creditEvents;
	private final ReferenceWithMetaCreditEvents creditEventsReference;
	private final BigDecimal level;
	private final BigDecimal levelPercentage;
	private final TriggerTimeTypeEnum triggerTimeType;
	private final TriggerTypeEnum triggerType;
	private static TriggerMeta metaData = new TriggerMeta();
	
	Trigger(TriggerBuilder builder) {
		this.creditEvents = ofNullable(builder.getCreditEvents()).map(f->f.build()).orElse(null);
		this.creditEventsReference = ofNullable(builder.getCreditEventsReference()).map(f->f.build()).orElse(null);
		this.level = builder.getLevel();
		this.levelPercentage = builder.getLevelPercentage();
		this.triggerTimeType = builder.getTriggerTimeType();
		this.triggerType = builder.getTriggerType();
	}
	
	@RosettaSynonym(value="creditEvents", source="FpML_5_10")
	@RosettaSynonym(value="creditEvents", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditEvents", source="DTCC_11_0")
	@RosettaSynonym(value="creditEvents", source="DTCC_9_0")
	@RosettaSynonym(value="creditEvents", source="CME_ClearedConfirm_1_17")
	public final CreditEvents getCreditEvents() {
		return creditEvents;
	}
	
	@RosettaSynonym(value="creditEventsReference", source="FpML_5_10")
	@RosettaSynonym(value="creditEventsReference", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="creditEventsReference", source="DTCC_11_0")
	@RosettaSynonym(value="creditEventsReference", source="DTCC_9_0")
	@RosettaSynonym(value="creditEventsReference", source="CME_ClearedConfirm_1_17")
	public final ReferenceWithMetaCreditEvents getCreditEventsReference() {
		return creditEventsReference;
	}
	
	/**
	 * The trigger level.
	 */
	@RosettaSynonym(value="level", source="FpML_5_10")
	@RosettaSynonym(value="level", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="level", source="DTCC_11_0")
	@RosettaSynonym(value="level", source="DTCC_9_0")
	@RosettaSynonym(value="level", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getLevel() {
		return level;
	}
	
	/**
	 * The trigger level percentage.
	 */
	@RosettaSynonym(value="levelPercentage", source="FpML_5_10")
	@RosettaSynonym(value="levelPercentage", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="levelPercentage", source="DTCC_11_0")
	@RosettaSynonym(value="levelPercentage", source="DTCC_9_0")
	@RosettaSynonym(value="levelPercentage", source="CME_ClearedConfirm_1_17")
	public final BigDecimal getLevelPercentage() {
		return levelPercentage;
	}
	
	/**
	 * The valuation time type of knock condition.
	 */
	@RosettaSynonym(value="triggerTimeType", source="FpML_5_10")
	@RosettaSynonym(value="triggerTimeType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="triggerTimeType", source="DTCC_11_0")
	@RosettaSynonym(value="triggerTimeType", source="DTCC_9_0")
	@RosettaSynonym(value="triggerTimeType", source="CME_ClearedConfirm_1_17")
	public final TriggerTimeTypeEnum getTriggerTimeType() {
		return triggerTimeType;
	}
	
	/**
	 * The Triggering condition.
	 */
	@RosettaSynonym(value="triggerType", source="FpML_5_10")
	@RosettaSynonym(value="triggerType", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="triggerType", source="DTCC_11_0")
	@RosettaSynonym(value="triggerType", source="DTCC_9_0")
	@RosettaSynonym(value="triggerType", source="CME_ClearedConfirm_1_17")
	public final TriggerTypeEnum getTriggerType() {
		return triggerType;
	}
	
	
	@Override
	public RosettaMetaData<? extends Trigger> metaData() {
		return metaData;
	} 
	
	public TriggerBuilder toBuilder() {
		TriggerBuilder builder = new TriggerBuilder();
		ofNullable(getCreditEvents()).ifPresent(builder::setCreditEvents);
		ofNullable(getCreditEventsReference()).ifPresent(builder::setCreditEventsReference);
		ofNullable(getLevel()).ifPresent(builder::setLevel);
		ofNullable(getLevelPercentage()).ifPresent(builder::setLevelPercentage);
		ofNullable(getTriggerTimeType()).ifPresent(builder::setTriggerTimeType);
		ofNullable(getTriggerType()).ifPresent(builder::setTriggerType);
		return builder;
	}

	public static TriggerBuilder builder() {
		return new TriggerBuilder();
	}

	public static  class TriggerBuilder extends RosettaModelObjectBuilder {
	
		protected CreditEvents.CreditEventsBuilder creditEvents;
		protected ReferenceWithMetaCreditEventsBuilder creditEventsReference;
		protected BigDecimal level;
		protected BigDecimal levelPercentage;
		protected TriggerTimeTypeEnum triggerTimeType;
		protected TriggerTypeEnum triggerType;
	
		public TriggerBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Trigger> metaData() {
			return metaData;
		} 
	
		
		public CreditEvents.CreditEventsBuilder getCreditEvents() {
			return creditEvents;
		}
		
		public CreditEvents.CreditEventsBuilder getOrCreateCreditEvents() {
			if (creditEvents!=null) {
				return creditEvents;
			}
			else return creditEvents = new CreditEvents.CreditEventsBuilder();
		}
		
		
		public ReferenceWithMetaCreditEventsBuilder getCreditEventsReference() {
			return creditEventsReference;
		}
		
		public ReferenceWithMetaCreditEventsBuilder getOrCreateCreditEventsReference() {
			if (creditEventsReference!=null) {
				return creditEventsReference;
			}
			else return creditEventsReference = new ReferenceWithMetaCreditEventsBuilder();
		}
		
		
		public BigDecimal getLevel() {
			return level;
		}
		
		
		public BigDecimal getLevelPercentage() {
			return levelPercentage;
		}
		
		
		public TriggerTimeTypeEnum getTriggerTimeType() {
			return triggerTimeType;
		}
		
		
		public TriggerTypeEnum getTriggerType() {
			return triggerType;
		}
		
	
		public TriggerBuilder setCreditEvents(CreditEvents creditEvents) {
			this.creditEvents = creditEvents.toBuilder();
			return this;
		}
		public TriggerBuilder setCreditEventsBuilder(CreditEvents.CreditEventsBuilder creditEvents) {
			this.creditEvents = creditEvents;
			return this;
		}
		public TriggerBuilder setCreditEventsReference(ReferenceWithMetaCreditEvents creditEventsReference) {
			this.creditEventsReference = creditEventsReference.toBuilder();
			return this;
		}
		public TriggerBuilder setCreditEventsReferenceBuilder(ReferenceWithMetaCreditEventsBuilder creditEventsReference) {
			this.creditEventsReference = creditEventsReference;
			return this;
		}
		public TriggerBuilder setCreditEventsReferenceRef(CreditEvents.CreditEventsBuilder creditEventsReference) {
			return setCreditEventsReference(ReferenceWithMetaCreditEvents.builder().setValueBuilder(creditEventsReference).build());
		}
		public TriggerBuilder setCreditEventsReferenceRef(CreditEvents creditEventsReference) {
			return setCreditEventsReferenceRef(CreditEvents.builder());
		}
		public TriggerBuilder setLevel(BigDecimal level) {
			this.level = level;
			return this;
		}
		public TriggerBuilder setLevelPercentage(BigDecimal levelPercentage) {
			this.levelPercentage = levelPercentage;
			return this;
		}
		public TriggerBuilder setTriggerTimeType(TriggerTimeTypeEnum triggerTimeType) {
			this.triggerTimeType = triggerTimeType;
			return this;
		}
		public TriggerBuilder setTriggerType(TriggerTypeEnum triggerType) {
			this.triggerType = triggerType;
			return this;
		}
		
		public Trigger build() {
			return new Trigger(this);
		}
	
		@Override
		public TriggerBuilder prune() {
			if (creditEvents!=null && !creditEvents.prune().hasData()) creditEvents = null;
			if (creditEventsReference!=null && !creditEventsReference.prune().hasData()) creditEventsReference = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCreditEvents()!=null && getCreditEvents().hasData()) return true;
			if (getCreditEventsReference()!=null && getCreditEventsReference().hasData()) return true;
			if (getLevel()!=null) return true;
			if (getLevelPercentage()!=null) return true;
			if (getTriggerTimeType()!=null) return true;
			if (getTriggerType()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("level"), BigDecimal.class, level, this);
			processor.processBasic(path.newSubPath("levelPercentage"), BigDecimal.class, levelPercentage, this);
			processor.processBasic(path.newSubPath("triggerTimeType"), TriggerTimeTypeEnum.class, triggerTimeType, this);
			processor.processBasic(path.newSubPath("triggerType"), TriggerTypeEnum.class, triggerType, this);
			
			processRosetta(path.newSubPath("creditEvents"), processor, CreditEvents.class, creditEvents);
			processRosetta(path.newSubPath("creditEventsReference"), processor, ReferenceWithMetaCreditEvents.class, creditEventsReference);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TriggerBuilder _that = (TriggerBuilder) o;
		
			if (creditEvents != null ? !creditEvents.equals(_that.creditEvents) : _that.creditEvents != null) return false;
			if (creditEventsReference != null ? !creditEventsReference.equals(_that.creditEventsReference) : _that.creditEventsReference != null) return false;
			if (level != null ? !level.equals(_that.level) : _that.level != null) return false;
			if (levelPercentage != null ? !levelPercentage.equals(_that.levelPercentage) : _that.levelPercentage != null) return false;
			if (triggerTimeType != null ? !triggerTimeType.equals(_that.triggerTimeType) : _that.triggerTimeType != null) return false;
			if (triggerType != null ? !triggerType.equals(_that.triggerType) : _that.triggerType != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (creditEvents != null ? creditEvents.hashCode() : 0);
			_result = 31 * _result + (creditEventsReference != null ? creditEventsReference.hashCode() : 0);
			_result = 31 * _result + (level != null ? level.hashCode() : 0);
			_result = 31 * _result + (levelPercentage != null ? levelPercentage.hashCode() : 0);
			_result = 31 * _result + (triggerTimeType != null ? triggerTimeType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (triggerType != null ? triggerType.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TriggerBuilder {" +
				"creditEvents=" + this.creditEvents + ", " +
				"creditEventsReference=" + this.creditEventsReference + ", " +
				"level=" + this.level + ", " +
				"levelPercentage=" + this.levelPercentage + ", " +
				"triggerTimeType=" + this.triggerTimeType + ", " +
				"triggerType=" + this.triggerType +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("level"), BigDecimal.class, level, this);
		processor.processBasic(path.newSubPath("levelPercentage"), BigDecimal.class, levelPercentage, this);
		processor.processBasic(path.newSubPath("triggerTimeType"), TriggerTimeTypeEnum.class, triggerTimeType, this);
		processor.processBasic(path.newSubPath("triggerType"), TriggerTypeEnum.class, triggerType, this);
		
		processRosetta(path.newSubPath("creditEvents"), processor, CreditEvents.class, creditEvents);
		processRosetta(path.newSubPath("creditEventsReference"), processor, ReferenceWithMetaCreditEvents.class, creditEventsReference);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Trigger _that = (Trigger) o;
	
		if (creditEvents != null ? !creditEvents.equals(_that.creditEvents) : _that.creditEvents != null) return false;
		if (creditEventsReference != null ? !creditEventsReference.equals(_that.creditEventsReference) : _that.creditEventsReference != null) return false;
		if (level != null ? !level.equals(_that.level) : _that.level != null) return false;
		if (levelPercentage != null ? !levelPercentage.equals(_that.levelPercentage) : _that.levelPercentage != null) return false;
		if (triggerTimeType != null ? !triggerTimeType.equals(_that.triggerTimeType) : _that.triggerTimeType != null) return false;
		if (triggerType != null ? !triggerType.equals(_that.triggerType) : _that.triggerType != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (creditEvents != null ? creditEvents.hashCode() : 0);
		_result = 31 * _result + (creditEventsReference != null ? creditEventsReference.hashCode() : 0);
		_result = 31 * _result + (level != null ? level.hashCode() : 0);
		_result = 31 * _result + (levelPercentage != null ? levelPercentage.hashCode() : 0);
		_result = 31 * _result + (triggerTimeType != null ? triggerTimeType.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (triggerType != null ? triggerType.getClass().getName().hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Trigger {" +
			"creditEvents=" + this.creditEvents + ", " +
			"creditEventsReference=" + this.creditEventsReference + ", " +
			"level=" + this.level + ", " +
			"levelPercentage=" + this.levelPercentage + ", " +
			"triggerTimeType=" + this.triggerTimeType + ", " +
			"triggerType=" + this.triggerType +
		'}';
	}
}
