package org.isda.cdm;

import java.util.ArrayList;
import java.util.List;
import org.isda.cdm.AveragingSchedule;
import org.isda.cdm.DateList;
import org.isda.cdm.FeaturePayment;
import org.isda.cdm.Trigger;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TriggerEventMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import java.util.List;
		import java.util.ArrayList;
		import java.util.stream.Collectors;
		import java.util.Objects;
		import static java.util.Optional.ofNullable;

/**
 * Observation point for trigger.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="TriggerEvent", source="FpML_5_10")
public class TriggerEvent extends RosettaModelObject {
	private final FeaturePayment featurePayment;
	private final List<AveragingSchedule> schedule;
	private final Trigger trigger;
	private final DateList triggerDates;
	private static TriggerEventMeta metaData = new TriggerEventMeta();
	
	TriggerEvent(TriggerEventBuilder builder) {
		this.featurePayment = ofNullable(builder.getFeaturePayment()).map(f->f.build()).orElse(null);
		this.schedule = ofNullable(builder.getSchedule()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(Collectors.toList())).orElse(null);
		this.trigger = ofNullable(builder.getTrigger()).map(f->f.build()).orElse(null);
		this.triggerDates = ofNullable(builder.getTriggerDates()).map(f->f.build()).orElse(null);
	}
	
	/**
	 * The feature payment, i.e. the payment made following trigger occurrence.
	 */
	@RosettaSynonym(value="featurePayment", source="FpML_5_10")
	@RosettaSynonym(value="featurePayment", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="featurePayment", source="DTCC_11_0")
	@RosettaSynonym(value="featurePayment", source="DTCC_9_0")
	@RosettaSynonym(value="featurePayment", source="CME_ClearedConfirm_1_17")
	public final FeaturePayment getFeaturePayment() {
		return featurePayment;
	}
	
	/**
	 * A derivative schedule.
	 */
	@RosettaSynonym(value="schedule", source="FpML_5_10")
	@RosettaSynonym(value="schedule", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="schedule", source="DTCC_11_0")
	@RosettaSynonym(value="schedule", source="DTCC_9_0")
	@RosettaSynonym(value="schedule", source="CME_ClearedConfirm_1_17")
	public final List<AveragingSchedule> getSchedule() {
		return schedule;
	}
	
	/**
	 * The trigger level
	 */
	@RosettaSynonym(value="trigger", source="FpML_5_10")
	@RosettaSynonym(value="trigger", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="trigger", source="DTCC_11_0")
	@RosettaSynonym(value="trigger", source="DTCC_9_0")
	@RosettaSynonym(value="trigger", source="CME_ClearedConfirm_1_17")
	public final Trigger getTrigger() {
		return trigger;
	}
	
	/**
	 * The trigger Dates.
	 */
	@RosettaSynonym(value="triggerDates", source="FpML_5_10")
	@RosettaSynonym(value="triggerDates", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="triggerDates", source="DTCC_11_0")
	@RosettaSynonym(value="triggerDates", source="DTCC_9_0")
	@RosettaSynonym(value="triggerDates", source="CME_ClearedConfirm_1_17")
	public final DateList getTriggerDates() {
		return triggerDates;
	}
	
	
	@Override
	public RosettaMetaData<? extends TriggerEvent> metaData() {
		return metaData;
	} 
	
	public TriggerEventBuilder toBuilder() {
		TriggerEventBuilder builder = new TriggerEventBuilder();
		ofNullable(getFeaturePayment()).ifPresent(builder::setFeaturePayment);
		ofNullable(getSchedule()).ifPresent(schedule -> schedule.forEach(builder::addSchedule));
		ofNullable(getTrigger()).ifPresent(builder::setTrigger);
		ofNullable(getTriggerDates()).ifPresent(builder::setTriggerDates);
		return builder;
	}

	public static TriggerEventBuilder builder() {
		return new TriggerEventBuilder();
	}

	public static  class TriggerEventBuilder extends RosettaModelObjectBuilder {
	
		protected FeaturePayment.FeaturePaymentBuilder featurePayment;
		protected List<AveragingSchedule.AveragingScheduleBuilder> schedule;
		protected Trigger.TriggerBuilder trigger;
		protected DateList.DateListBuilder triggerDates;
	
		public TriggerEventBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends TriggerEvent> metaData() {
			return metaData;
		} 
	
		
		public FeaturePayment.FeaturePaymentBuilder getFeaturePayment() {
			return featurePayment;
		}
		
		public FeaturePayment.FeaturePaymentBuilder getOrCreateFeaturePayment() {
			if (featurePayment!=null) {
				return featurePayment;
			}
			else return featurePayment = new FeaturePayment.FeaturePaymentBuilder();
		}
		
		
		public List<AveragingSchedule.AveragingScheduleBuilder> getSchedule() {
			return schedule;
		}
		
		public AveragingSchedule.AveragingScheduleBuilder getOrCreateSchedule(int index) {
			if (schedule==null) {
				this.schedule = new ArrayList<>();
			}
			return getIndex(schedule, index, ()->new AveragingSchedule.AveragingScheduleBuilder());
		}
		
		
		public Trigger.TriggerBuilder getTrigger() {
			return trigger;
		}
		
		public Trigger.TriggerBuilder getOrCreateTrigger() {
			if (trigger!=null) {
				return trigger;
			}
			else return trigger = new Trigger.TriggerBuilder();
		}
		
		
		public DateList.DateListBuilder getTriggerDates() {
			return triggerDates;
		}
		
		public DateList.DateListBuilder getOrCreateTriggerDates() {
			if (triggerDates!=null) {
				return triggerDates;
			}
			else return triggerDates = new DateList.DateListBuilder();
		}
		
	
		public TriggerEventBuilder setFeaturePayment(FeaturePayment featurePayment) {
			this.featurePayment = featurePayment.toBuilder();
			return this;
		}
		public TriggerEventBuilder setFeaturePaymentBuilder(FeaturePayment.FeaturePaymentBuilder featurePayment) {
			this.featurePayment = featurePayment;
			return this;
		}
		public TriggerEventBuilder addSchedule(AveragingSchedule schedule) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			this.schedule.add(schedule.toBuilder());
			return this;
		}
		public TriggerEventBuilder addSchedule(AveragingSchedule schedule, int _idx) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			getIndex(this.schedule, _idx, () -> schedule.toBuilder());
			this.schedule.set(_idx, schedule.toBuilder());
			return this;
		}
		public TriggerEventBuilder addSchedule(List<AveragingSchedule> schedules) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			for (AveragingSchedule toAdd : schedules) {
				this.schedule.add(toAdd.toBuilder());
			}
			return this;
		}
		public TriggerEventBuilder addScheduleBuilder(AveragingSchedule.AveragingScheduleBuilder schedule) {
			if(this.schedule == null){
				this.schedule = new ArrayList<>();
			}
			this.schedule.add(schedule);
			return this;
		}
		
		public TriggerEventBuilder clearSchedule() {
			this.schedule = null;
			return this;
		}
		public TriggerEventBuilder setTrigger(Trigger trigger) {
			this.trigger = trigger.toBuilder();
			return this;
		}
		public TriggerEventBuilder setTriggerBuilder(Trigger.TriggerBuilder trigger) {
			this.trigger = trigger;
			return this;
		}
		public TriggerEventBuilder setTriggerDates(DateList triggerDates) {
			this.triggerDates = triggerDates.toBuilder();
			return this;
		}
		public TriggerEventBuilder setTriggerDatesBuilder(DateList.DateListBuilder triggerDates) {
			this.triggerDates = triggerDates;
			return this;
		}
		
		public TriggerEvent build() {
			return new TriggerEvent(this);
		}
	
		@Override
		public TriggerEventBuilder prune() {
			if (featurePayment!=null && !featurePayment.prune().hasData()) featurePayment = null;
			if (schedule!=null) schedule = schedule.stream().filter(b->b!=null).map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (trigger!=null && !trigger.prune().hasData()) trigger = null;
			if (triggerDates!=null && !triggerDates.prune().hasData()) triggerDates = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getFeaturePayment()!=null && getFeaturePayment().hasData()) return true;
			if (getSchedule()!=null && getSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTrigger()!=null && getTrigger().hasData()) return true;
			if (getTriggerDates()!=null && getTriggerDates().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("featurePayment"), processor, FeaturePayment.class, featurePayment);
			processRosetta(path.newSubPath("schedule"), processor, AveragingSchedule.class, schedule);
			processRosetta(path.newSubPath("trigger"), processor, Trigger.class, trigger);
			processRosetta(path.newSubPath("triggerDates"), processor, DateList.class, triggerDates);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TriggerEventBuilder _that = (TriggerEventBuilder) o;
		
			if (featurePayment != null ? !featurePayment.equals(_that.featurePayment) : _that.featurePayment != null) return false;
			if (!ListEquals.listEquals(schedule, _that.schedule)) return false;
			if (trigger != null ? !trigger.equals(_that.trigger) : _that.trigger != null) return false;
			if (triggerDates != null ? !triggerDates.equals(_that.triggerDates) : _that.triggerDates != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (featurePayment != null ? featurePayment.hashCode() : 0);
			_result = 31 * _result + (schedule != null ? schedule.hashCode() : 0);
			_result = 31 * _result + (trigger != null ? trigger.hashCode() : 0);
			_result = 31 * _result + (triggerDates != null ? triggerDates.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TriggerEventBuilder {" +
				"featurePayment=" + this.featurePayment + ", " +
				"schedule=" + this.schedule + ", " +
				"trigger=" + this.trigger + ", " +
				"triggerDates=" + this.triggerDates +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("featurePayment"), processor, FeaturePayment.class, featurePayment);
		processRosetta(path.newSubPath("schedule"), processor, AveragingSchedule.class, schedule);
		processRosetta(path.newSubPath("trigger"), processor, Trigger.class, trigger);
		processRosetta(path.newSubPath("triggerDates"), processor, DateList.class, triggerDates);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		TriggerEvent _that = (TriggerEvent) o;
	
		if (featurePayment != null ? !featurePayment.equals(_that.featurePayment) : _that.featurePayment != null) return false;
		if (!ListEquals.listEquals(schedule, _that.schedule)) return false;
		if (trigger != null ? !trigger.equals(_that.trigger) : _that.trigger != null) return false;
		if (triggerDates != null ? !triggerDates.equals(_that.triggerDates) : _that.triggerDates != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (featurePayment != null ? featurePayment.hashCode() : 0);
		_result = 31 * _result + (schedule != null ? schedule.hashCode() : 0);
		_result = 31 * _result + (trigger != null ? trigger.hashCode() : 0);
		_result = 31 * _result + (triggerDates != null ? triggerDates.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "TriggerEvent {" +
			"featurePayment=" + this.featurePayment + ", " +
			"schedule=" + this.schedule + ", " +
			"trigger=" + this.trigger + ", " +
			"triggerDates=" + this.triggerDates +
		'}';
	}
}
