package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the time of day which would be considered for valuing the knock event.
 * 
 * @version 2.5.4
 */
public enum TriggerTimeTypeEnum {
    

	/**
	 * The close of trading on a day would be considered for valuation.
	 */
	@RosettaSynonym(value = "Closing", source = "FpML_5_10")
	@RosettaSynonym(value = "Closing", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Closing", source = "DTCC_11_0")
	@RosettaSynonym(value = "Closing", source = "DTCC_9_0")
	@RosettaSynonym(value = "Closing", source = "CME_ClearedConfirm_1_17")
	CLOSING,
	    

	/**
	 * At any time during the Knock Determination period (continuous barrier).
	 */
	@RosettaSynonym(value = "Anytime", source = "FpML_5_10")
	@RosettaSynonym(value = "Anytime", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Anytime", source = "DTCC_11_0")
	@RosettaSynonym(value = "Anytime", source = "DTCC_9_0")
	@RosettaSynonym(value = "Anytime", source = "CME_ClearedConfirm_1_17")
	ANYTIME    
	;

	private final String displayName;
	
	TriggerTimeTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
