package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify whether an option will trigger or expire depending upon whether the spot rate is above or below the barrier rate.
 * 
 * @version 2.5.4
 */
public enum TriggerTypeEnum {
    

	/**
	 * The underlier price must be equal to or less than the Trigger level.
	 */
	@RosettaSynonym(value = "EqualOrLess", source = "FpML_5_10")
	@RosettaSynonym(value = "EqualOrLess", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EqualOrLess", source = "DTCC_11_0")
	@RosettaSynonym(value = "EqualOrLess", source = "DTCC_9_0")
	@RosettaSynonym(value = "EqualOrLess", source = "CME_ClearedConfirm_1_17")
	EQUAL_OR_LESS,
	    

	/**
	 * The underlier price must be equal to or greater than the Trigger level.
	 */
	@RosettaSynonym(value = "EqualOrGreater", source = "FpML_5_10")
	@RosettaSynonym(value = "EqualOrGreater", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EqualOrGreater", source = "DTCC_11_0")
	@RosettaSynonym(value = "EqualOrGreater", source = "DTCC_9_0")
	@RosettaSynonym(value = "EqualOrGreater", source = "CME_ClearedConfirm_1_17")
	EQUAL_OR_GREATER,
	    

	/**
	 * The underlier price must be equal to the Trigger level.
	 */
	@RosettaSynonym(value = "Equal", source = "FpML_5_10")
	@RosettaSynonym(value = "Equal", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Equal", source = "DTCC_11_0")
	@RosettaSynonym(value = "Equal", source = "DTCC_9_0")
	@RosettaSynonym(value = "Equal", source = "CME_ClearedConfirm_1_17")
	EQUAL,
	    

	/**
	 * The underlier price must be less than the Trigger level.
	 */
	@RosettaSynonym(value = "Less", source = "FpML_5_10")
	@RosettaSynonym(value = "Less", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Less", source = "DTCC_11_0")
	@RosettaSynonym(value = "Less", source = "DTCC_9_0")
	@RosettaSynonym(value = "Less", source = "CME_ClearedConfirm_1_17")
	LESS,
	    

	/**
	 * The underlier price must be greater than the Trigger level.
	 */
	@RosettaSynonym(value = "Greater", source = "FpML_5_10")
	@RosettaSynonym(value = "Greater", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Greater", source = "DTCC_11_0")
	@RosettaSynonym(value = "Greater", source = "DTCC_9_0")
	@RosettaSynonym(value = "Greater", source = "CME_ClearedConfirm_1_17")
	GREATER    
	;

	private final String displayName;
	
	TriggerTypeEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
