package org.isda.cdm;

import java.lang.String;
import org.isda.cdm.New;
import org.isda.cdm.Pric;
import org.isda.cdm.Qty;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.TxMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class Tx extends RosettaModelObject {
	private final String ctryOfBrnch;
	private final New newTx;
	private final Pric pric;
	private final Qty qty;
	private final String tradDt;
	private final String tradVn;
	private final String tradgCpcty;
	private static TxMeta metaData = new TxMeta();
	
	Tx(TxBuilder builder) {
		this.ctryOfBrnch = builder.getCtryOfBrnch();
		this.newTx = ofNullable(builder.getNewTx()).map(f->f.build()).orElse(null);
		this.pric = ofNullable(builder.getPric()).map(f->f.build()).orElse(null);
		this.qty = ofNullable(builder.getQty()).map(f->f.build()).orElse(null);
		this.tradDt = builder.getTradDt();
		this.tradVn = builder.getTradVn();
		this.tradgCpcty = builder.getTradgCpcty();
	}
	
	@RosettaSynonym(value="CtryOfBrnch", source="ISO20022")
	public final String getCtryOfBrnch() {
		return ctryOfBrnch;
	}
	
	@RosettaSynonym(value="New", source="ISO20022")
	public final New getNewTx() {
		return newTx;
	}
	
	@RosettaSynonym(value="Pric", source="ISO20022")
	public final Pric getPric() {
		return pric;
	}
	
	@RosettaSynonym(value="Qty", source="ISO20022")
	public final Qty getQty() {
		return qty;
	}
	
	@RosettaSynonym(value="TradDt", source="ISO20022")
	public final String getTradDt() {
		return tradDt;
	}
	
	@RosettaSynonym(value="TradVn", source="ISO20022")
	public final String getTradVn() {
		return tradVn;
	}
	
	@RosettaSynonym(value="TradgCpcty", source="ISO20022")
	public final String getTradgCpcty() {
		return tradgCpcty;
	}
	
	
	@Override
	public RosettaMetaData<? extends Tx> metaData() {
		return metaData;
	} 
	
	public TxBuilder toBuilder() {
		TxBuilder builder = new TxBuilder();
		ofNullable(getCtryOfBrnch()).ifPresent(builder::setCtryOfBrnch);
		ofNullable(getNewTx()).ifPresent(builder::setNewTx);
		ofNullable(getPric()).ifPresent(builder::setPric);
		ofNullable(getQty()).ifPresent(builder::setQty);
		ofNullable(getTradDt()).ifPresent(builder::setTradDt);
		ofNullable(getTradVn()).ifPresent(builder::setTradVn);
		ofNullable(getTradgCpcty()).ifPresent(builder::setTradgCpcty);
		return builder;
	}

	public static TxBuilder builder() {
		return new TxBuilder();
	}

	public static  class TxBuilder extends RosettaModelObjectBuilder {
	
		protected String ctryOfBrnch;
		protected New.NewBuilder newTx;
		protected Pric.PricBuilder pric;
		protected Qty.QtyBuilder qty;
		protected String tradDt;
		protected String tradVn;
		protected String tradgCpcty;
	
		public TxBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Tx> metaData() {
			return metaData;
		} 
	
		
		public String getCtryOfBrnch() {
			return ctryOfBrnch;
		}
		
		
		public New.NewBuilder getNewTx() {
			return newTx;
		}
		
		public New.NewBuilder getOrCreateNewTx() {
			if (newTx!=null) {
				return newTx;
			}
			else return newTx = new New.NewBuilder();
		}
		
		
		public Pric.PricBuilder getPric() {
			return pric;
		}
		
		public Pric.PricBuilder getOrCreatePric() {
			if (pric!=null) {
				return pric;
			}
			else return pric = new Pric.PricBuilder();
		}
		
		
		public Qty.QtyBuilder getQty() {
			return qty;
		}
		
		public Qty.QtyBuilder getOrCreateQty() {
			if (qty!=null) {
				return qty;
			}
			else return qty = new Qty.QtyBuilder();
		}
		
		
		public String getTradDt() {
			return tradDt;
		}
		
		
		public String getTradVn() {
			return tradVn;
		}
		
		
		public String getTradgCpcty() {
			return tradgCpcty;
		}
		
	
		public TxBuilder setCtryOfBrnch(String ctryOfBrnch) {
			this.ctryOfBrnch = ctryOfBrnch;
			return this;
		}
		public TxBuilder setNewTx(New newTx) {
			this.newTx = newTx.toBuilder();
			return this;
		}
		public TxBuilder setNewTxBuilder(New.NewBuilder newTx) {
			this.newTx = newTx;
			return this;
		}
		public TxBuilder setPric(Pric pric) {
			this.pric = pric.toBuilder();
			return this;
		}
		public TxBuilder setPricBuilder(Pric.PricBuilder pric) {
			this.pric = pric;
			return this;
		}
		public TxBuilder setQty(Qty qty) {
			this.qty = qty.toBuilder();
			return this;
		}
		public TxBuilder setQtyBuilder(Qty.QtyBuilder qty) {
			this.qty = qty;
			return this;
		}
		public TxBuilder setTradDt(String tradDt) {
			this.tradDt = tradDt;
			return this;
		}
		public TxBuilder setTradVn(String tradVn) {
			this.tradVn = tradVn;
			return this;
		}
		public TxBuilder setTradgCpcty(String tradgCpcty) {
			this.tradgCpcty = tradgCpcty;
			return this;
		}
		
		public Tx build() {
			return new Tx(this);
		}
	
		@Override
		public TxBuilder prune() {
			if (newTx!=null && !newTx.prune().hasData()) newTx = null;
			if (pric!=null && !pric.prune().hasData()) pric = null;
			if (qty!=null && !qty.prune().hasData()) qty = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCtryOfBrnch()!=null) return true;
			if (getNewTx()!=null && getNewTx().hasData()) return true;
			if (getPric()!=null && getPric().hasData()) return true;
			if (getQty()!=null && getQty().hasData()) return true;
			if (getTradDt()!=null) return true;
			if (getTradVn()!=null) return true;
			if (getTradgCpcty()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("ctryOfBrnch"), String.class, ctryOfBrnch, this);
			processor.processBasic(path.newSubPath("tradDt"), String.class, tradDt, this);
			processor.processBasic(path.newSubPath("tradVn"), String.class, tradVn, this);
			processor.processBasic(path.newSubPath("tradgCpcty"), String.class, tradgCpcty, this);
			
			processRosetta(path.newSubPath("newTx"), processor, New.class, newTx);
			processRosetta(path.newSubPath("pric"), processor, Pric.class, pric);
			processRosetta(path.newSubPath("qty"), processor, Qty.class, qty);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			TxBuilder _that = (TxBuilder) o;
		
			if (ctryOfBrnch != null ? !ctryOfBrnch.equals(_that.ctryOfBrnch) : _that.ctryOfBrnch != null) return false;
			if (newTx != null ? !newTx.equals(_that.newTx) : _that.newTx != null) return false;
			if (pric != null ? !pric.equals(_that.pric) : _that.pric != null) return false;
			if (qty != null ? !qty.equals(_that.qty) : _that.qty != null) return false;
			if (tradDt != null ? !tradDt.equals(_that.tradDt) : _that.tradDt != null) return false;
			if (tradVn != null ? !tradVn.equals(_that.tradVn) : _that.tradVn != null) return false;
			if (tradgCpcty != null ? !tradgCpcty.equals(_that.tradgCpcty) : _that.tradgCpcty != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (ctryOfBrnch != null ? ctryOfBrnch.hashCode() : 0);
			_result = 31 * _result + (newTx != null ? newTx.hashCode() : 0);
			_result = 31 * _result + (pric != null ? pric.hashCode() : 0);
			_result = 31 * _result + (qty != null ? qty.hashCode() : 0);
			_result = 31 * _result + (tradDt != null ? tradDt.hashCode() : 0);
			_result = 31 * _result + (tradVn != null ? tradVn.hashCode() : 0);
			_result = 31 * _result + (tradgCpcty != null ? tradgCpcty.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "TxBuilder {" +
				"ctryOfBrnch=" + this.ctryOfBrnch + ", " +
				"newTx=" + this.newTx + ", " +
				"pric=" + this.pric + ", " +
				"qty=" + this.qty + ", " +
				"tradDt=" + this.tradDt + ", " +
				"tradVn=" + this.tradVn + ", " +
				"tradgCpcty=" + this.tradgCpcty +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("ctryOfBrnch"), String.class, ctryOfBrnch, this);
		processor.processBasic(path.newSubPath("tradDt"), String.class, tradDt, this);
		processor.processBasic(path.newSubPath("tradVn"), String.class, tradVn, this);
		processor.processBasic(path.newSubPath("tradgCpcty"), String.class, tradgCpcty, this);
		
		processRosetta(path.newSubPath("newTx"), processor, New.class, newTx);
		processRosetta(path.newSubPath("pric"), processor, Pric.class, pric);
		processRosetta(path.newSubPath("qty"), processor, Qty.class, qty);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Tx _that = (Tx) o;
	
		if (ctryOfBrnch != null ? !ctryOfBrnch.equals(_that.ctryOfBrnch) : _that.ctryOfBrnch != null) return false;
		if (newTx != null ? !newTx.equals(_that.newTx) : _that.newTx != null) return false;
		if (pric != null ? !pric.equals(_that.pric) : _that.pric != null) return false;
		if (qty != null ? !qty.equals(_that.qty) : _that.qty != null) return false;
		if (tradDt != null ? !tradDt.equals(_that.tradDt) : _that.tradDt != null) return false;
		if (tradVn != null ? !tradVn.equals(_that.tradVn) : _that.tradVn != null) return false;
		if (tradgCpcty != null ? !tradgCpcty.equals(_that.tradgCpcty) : _that.tradgCpcty != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (ctryOfBrnch != null ? ctryOfBrnch.hashCode() : 0);
		_result = 31 * _result + (newTx != null ? newTx.hashCode() : 0);
		_result = 31 * _result + (pric != null ? pric.hashCode() : 0);
		_result = 31 * _result + (qty != null ? qty.hashCode() : 0);
		_result = 31 * _result + (tradDt != null ? tradDt.hashCode() : 0);
		_result = 31 * _result + (tradVn != null ? tradVn.hashCode() : 0);
		_result = 31 * _result + (tradgCpcty != null ? tradgCpcty.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Tx {" +
			"ctryOfBrnch=" + this.ctryOfBrnch + ", " +
			"newTx=" + this.newTx + ", " +
			"pric=" + this.pric + ", " +
			"qty=" + this.qty + ", " +
			"tradDt=" + this.tradDt + ", " +
			"tradVn=" + this.tradVn + ", " +
			"tradgCpcty=" + this.tradgCpcty +
		'}';
	}
}
