package org.isda.cdm;

import org.isda.cdm.Swp;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.UndrlygInstrmMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass
public class UndrlygInstrm extends RosettaModelObject {
	private final Swp swp;
	private static UndrlygInstrmMeta metaData = new UndrlygInstrmMeta();
	
	UndrlygInstrm(UndrlygInstrmBuilder builder) {
		this.swp = ofNullable(builder.getSwp()).map(f->f.build()).orElse(null);
	}
	
	@RosettaSynonym(value="Swp", source="ISO20022")
	public final Swp getSwp() {
		return swp;
	}
	
	
	@Override
	public RosettaMetaData<? extends UndrlygInstrm> metaData() {
		return metaData;
	} 
	
	public UndrlygInstrmBuilder toBuilder() {
		UndrlygInstrmBuilder builder = new UndrlygInstrmBuilder();
		ofNullable(getSwp()).ifPresent(builder::setSwp);
		return builder;
	}

	public static UndrlygInstrmBuilder builder() {
		return new UndrlygInstrmBuilder();
	}

	public static  class UndrlygInstrmBuilder extends RosettaModelObjectBuilder {
	
		protected Swp.SwpBuilder swp;
	
		public UndrlygInstrmBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends UndrlygInstrm> metaData() {
			return metaData;
		} 
	
		
		public Swp.SwpBuilder getSwp() {
			return swp;
		}
		
		public Swp.SwpBuilder getOrCreateSwp() {
			if (swp!=null) {
				return swp;
			}
			else return swp = new Swp.SwpBuilder();
		}
		
	
		public UndrlygInstrmBuilder setSwp(Swp swp) {
			this.swp = swp.toBuilder();
			return this;
		}
		public UndrlygInstrmBuilder setSwpBuilder(Swp.SwpBuilder swp) {
			this.swp = swp;
			return this;
		}
		
		public UndrlygInstrm build() {
			return new UndrlygInstrm(this);
		}
	
		@Override
		public UndrlygInstrmBuilder prune() {
			if (swp!=null && !swp.prune().hasData()) swp = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSwp()!=null && getSwp().hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			
			processRosetta(path.newSubPath("swp"), processor, Swp.class, swp);
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			UndrlygInstrmBuilder _that = (UndrlygInstrmBuilder) o;
		
			if (swp != null ? !swp.equals(_that.swp) : _that.swp != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (swp != null ? swp.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "UndrlygInstrmBuilder {" +
				"swp=" + this.swp +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		
		processRosetta(path.newSubPath("swp"), processor, Swp.class, swp);
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		UndrlygInstrm _that = (UndrlygInstrm) o;
	
		if (swp != null ? !swp.equals(_that.swp) : _that.swp != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (swp != null ? swp.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "UndrlygInstrm {" +
			"swp=" + this.swp +
		'}';
	}
}
