package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumeration values to qualify the units of measure. FpML makes use of the priceQuoteUnitsScheme for specifying the units as part of its commodity implementation. Its scope is however deemed too broad for the CDM, as it includes values such as Amount and BasisPoints.  As a result, it is deemed inappropriate as a reference scheme for that enumeration.
 * 
 * @version 2.5.4
 */
public enum UnitEnum {
    

	/**
	 * Megawatt-hour
	 */
	@RosettaSynonym(value = "MWh", source = "Rosetta_Workbench")
	M_WH,
	    

	/**
	 * Million British thermal units
	 */
	@RosettaSynonym(value = "MMBTU", source = "Rosetta_Workbench")
	MMBTU,
	    

	/**
	 * Barrel
	 */
	@RosettaSynonym(value = "BBL", source = "Rosetta_Workbench")
	BBL,
	    

	/**
	 * Gallon
	 */
	@RosettaSynonym(value = "GAL", source = "Rosetta_Workbench")
	GAL,
	    

	/**
	 * Bushel
	 */
	@RosettaSynonym(value = "BSH", source = "Rosetta_Workbench")
	BSH    
	;

	private final String displayName;
	
	UnitEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
