package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the ISDA defined methodology for determining the final price of the reference obligation for purposes of cash settlement.
 * 
 * @version 2.5.4
 */
public enum ValuationMethodEnum {
    

	@RosettaSynonym(value = "Market", source = "FpML_5_10")
	@RosettaSynonym(value = "Market", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Market", source = "DTCC_11_0")
	@RosettaSynonym(value = "Market", source = "DTCC_9_0")
	@RosettaSynonym(value = "Market", source = "CME_ClearedConfirm_1_17")
	MARKET,
	    

	@RosettaSynonym(value = "Highest", source = "FpML_5_10")
	@RosettaSynonym(value = "Highest", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "Highest", source = "DTCC_11_0")
	@RosettaSynonym(value = "Highest", source = "DTCC_9_0")
	@RosettaSynonym(value = "Highest", source = "CME_ClearedConfirm_1_17")
	HIGHEST,
	    

	@RosettaSynonym(value = "AverageMarket", source = "FpML_5_10")
	@RosettaSynonym(value = "AverageMarket", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AverageMarket", source = "DTCC_11_0")
	@RosettaSynonym(value = "AverageMarket", source = "DTCC_9_0")
	@RosettaSynonym(value = "AverageMarket", source = "CME_ClearedConfirm_1_17")
	AVERAGE_MARKET,
	    

	@RosettaSynonym(value = "AverageHighest", source = "FpML_5_10")
	@RosettaSynonym(value = "AverageHighest", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AverageHighest", source = "DTCC_11_0")
	@RosettaSynonym(value = "AverageHighest", source = "DTCC_9_0")
	@RosettaSynonym(value = "AverageHighest", source = "CME_ClearedConfirm_1_17")
	AVERAGE_HIGHEST,
	    

	@RosettaSynonym(value = "BlendedMarket", source = "FpML_5_10")
	@RosettaSynonym(value = "BlendedMarket", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BlendedMarket", source = "DTCC_11_0")
	@RosettaSynonym(value = "BlendedMarket", source = "DTCC_9_0")
	@RosettaSynonym(value = "BlendedMarket", source = "CME_ClearedConfirm_1_17")
	BLENDED_MARKET,
	    

	@RosettaSynonym(value = "BlendedHighest", source = "FpML_5_10")
	@RosettaSynonym(value = "BlendedHighest", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BlendedHighest", source = "DTCC_11_0")
	@RosettaSynonym(value = "BlendedHighest", source = "DTCC_9_0")
	@RosettaSynonym(value = "BlendedHighest", source = "CME_ClearedConfirm_1_17")
	BLENDED_HIGHEST,
	    

	@RosettaSynonym(value = "AverageBlendedMarket", source = "FpML_5_10")
	@RosettaSynonym(value = "AverageBlendedMarket", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AverageBlendedMarket", source = "DTCC_11_0")
	@RosettaSynonym(value = "AverageBlendedMarket", source = "DTCC_9_0")
	@RosettaSynonym(value = "AverageBlendedMarket", source = "CME_ClearedConfirm_1_17")
	AVERAGE_BLENDED_MARKET,
	    

	@RosettaSynonym(value = "AverageBlendedHighest", source = "FpML_5_10")
	@RosettaSynonym(value = "AverageBlendedHighest", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AverageBlendedHighest", source = "DTCC_11_0")
	@RosettaSynonym(value = "AverageBlendedHighest", source = "DTCC_9_0")
	@RosettaSynonym(value = "AverageBlendedHighest", source = "CME_ClearedConfirm_1_17")
	AVERAGE_BLENDED_HIGHEST    
	;

	private final String displayName;
	
	ValuationMethodEnum() {
		this.displayName = null;
	}


	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
