package org.isda.cdm;

import java.lang.Integer;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.ValuationPostponementMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * Specifies how long to wait to get a quote from a settlement rate option upon a price source disruption.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="ValuationPostponement", source="FpML_5_10")
public class ValuationPostponement extends RosettaModelObject {
	private final Integer maximumDaysOfPostponement;
	private static ValuationPostponementMeta metaData = new ValuationPostponementMeta();
	
	ValuationPostponement(ValuationPostponementBuilder builder) {
		this.maximumDaysOfPostponement = builder.getMaximumDaysOfPostponement();
	}
	
	/**
	 * The maximum number of days to wait for a quote from the disrupted settlement rate option before proceeding to the next method.
	 */
	@RosettaSynonym(value="maximumDaysOfPostponement", source="FpML_5_10")
	@RosettaSynonym(value="maximumDaysOfPostponement", source="CME_SubmissionIRS_1_0")
	@RosettaSynonym(value="maximumDaysOfPostponement", source="DTCC_11_0")
	@RosettaSynonym(value="maximumDaysOfPostponement", source="DTCC_9_0")
	@RosettaSynonym(value="maximumDaysOfPostponement", source="CME_ClearedConfirm_1_17")
	public final Integer getMaximumDaysOfPostponement() {
		return maximumDaysOfPostponement;
	}
	
	
	@Override
	public RosettaMetaData<? extends ValuationPostponement> metaData() {
		return metaData;
	} 
	
	public ValuationPostponementBuilder toBuilder() {
		ValuationPostponementBuilder builder = new ValuationPostponementBuilder();
		ofNullable(getMaximumDaysOfPostponement()).ifPresent(builder::setMaximumDaysOfPostponement);
		return builder;
	}

	public static ValuationPostponementBuilder builder() {
		return new ValuationPostponementBuilder();
	}

	public static  class ValuationPostponementBuilder extends RosettaModelObjectBuilder {
	
		protected Integer maximumDaysOfPostponement;
	
		public ValuationPostponementBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends ValuationPostponement> metaData() {
			return metaData;
		} 
	
		
		public Integer getMaximumDaysOfPostponement() {
			return maximumDaysOfPostponement;
		}
		
	
		public ValuationPostponementBuilder setMaximumDaysOfPostponement(Integer maximumDaysOfPostponement) {
			this.maximumDaysOfPostponement = maximumDaysOfPostponement;
			return this;
		}
		
		public ValuationPostponement build() {
			return new ValuationPostponement(this);
		}
	
		@Override
		public ValuationPostponementBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMaximumDaysOfPostponement()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("maximumDaysOfPostponement"), Integer.class, maximumDaysOfPostponement, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			ValuationPostponementBuilder _that = (ValuationPostponementBuilder) o;
		
			if (maximumDaysOfPostponement != null ? !maximumDaysOfPostponement.equals(_that.maximumDaysOfPostponement) : _that.maximumDaysOfPostponement != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (maximumDaysOfPostponement != null ? maximumDaysOfPostponement.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ValuationPostponementBuilder {" +
				"maximumDaysOfPostponement=" + this.maximumDaysOfPostponement +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("maximumDaysOfPostponement"), Integer.class, maximumDaysOfPostponement, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		ValuationPostponement _that = (ValuationPostponement) o;
	
		if (maximumDaysOfPostponement != null ? !maximumDaysOfPostponement.equals(_that.maximumDaysOfPostponement) : _that.maximumDaysOfPostponement != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (maximumDaysOfPostponement != null ? maximumDaysOfPostponement.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "ValuationPostponement {" +
			"maximumDaysOfPostponement=" + this.maximumDaysOfPostponement +
		'}';
	}
}
