package org.isda.cdm;

import java.lang.Integer;
import org.isda.cdm.PeriodTimeEnum;


import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.VelocityMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Velocity", source="FpML_5_10")
public class Velocity extends RosettaModelObject {
	private final PeriodTimeEnum period;
	private final Integer periodMultiplier;
	private static VelocityMeta metaData = new VelocityMeta();
	
	Velocity(VelocityBuilder builder) {
		this.period = builder.getPeriod();
		this.periodMultiplier = builder.getPeriodMultiplier();
	}
	
	public final PeriodTimeEnum getPeriod() {
		return period;
	}
	
	public final Integer getPeriodMultiplier() {
		return periodMultiplier;
	}
	
	
	@Override
	public RosettaMetaData<? extends Velocity> metaData() {
		return metaData;
	} 
	
	public VelocityBuilder toBuilder() {
		VelocityBuilder builder = new VelocityBuilder();
		ofNullable(getPeriod()).ifPresent(builder::setPeriod);
		ofNullable(getPeriodMultiplier()).ifPresent(builder::setPeriodMultiplier);
		return builder;
	}

	public static VelocityBuilder builder() {
		return new VelocityBuilder();
	}

	public static  class VelocityBuilder extends RosettaModelObjectBuilder {
	
		protected PeriodTimeEnum period;
		protected Integer periodMultiplier;
	
		public VelocityBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Velocity> metaData() {
			return metaData;
		} 
	
		
		public PeriodTimeEnum getPeriod() {
			return period;
		}
		
		
		public Integer getPeriodMultiplier() {
			return periodMultiplier;
		}
		
	
		public VelocityBuilder setPeriod(PeriodTimeEnum period) {
			this.period = period;
			return this;
		}
		public VelocityBuilder setPeriodMultiplier(Integer periodMultiplier) {
			this.periodMultiplier = periodMultiplier;
			return this;
		}
		
		public Velocity build() {
			return new Velocity(this);
		}
	
		@Override
		public VelocityBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPeriod()!=null) return true;
			if (getPeriodMultiplier()!=null) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
		
			processor.processBasic(path.newSubPath("period"), PeriodTimeEnum.class, period, this);
			processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
		
			VelocityBuilder _that = (VelocityBuilder) o;
		
			if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
			if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "VelocityBuilder {" +
				"period=" + this.period + ", " +
				"periodMultiplier=" + this.periodMultiplier +
			'}';
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		
		processor.processBasic(path.newSubPath("period"), PeriodTimeEnum.class, period, this);
		processor.processBasic(path.newSubPath("periodMultiplier"), Integer.class, periodMultiplier, this);
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
	
		Velocity _that = (Velocity) o;
	
		if (period != null ? !period.equals(_that.period) : _that.period != null) return false;
		if (periodMultiplier != null ? !periodMultiplier.equals(_that.periodMultiplier) : _that.periodMultiplier != null) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = 0;
		_result = 31 * _result + (period != null ? period.getClass().getName().hashCode() : 0);
		_result = 31 * _result + (periodMultiplier != null ? periodMultiplier.hashCode() : 0);
		return _result;
	}
	
	@Override
	public String toString() {
		return "Velocity {" +
			"period=" + this.period + ", " +
			"periodMultiplier=" + this.periodMultiplier +
		'}';
	}
}
