package org.isda.cdm;



import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import org.isda.cdm.meta.WarrantMeta;
import com.rosetta.model.lib.annotations.RosettaClass;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.*;

import com.rosetta.model.lib.annotations.RosettaSynonym;
import com.rosetta.model.lib.qualify.Qualified;
import com.rosetta.util.ListEquals;

import static java.util.Optional.ofNullable;

/**
 * A class to specify a warrant as having a product identifier. As a difference versus the FpML standard, the CDM structure of this class only includes the productIdentifier class, which consists of an identifier, productTaxonomy, and source of the identifier. The reason for this approach is to avoid the potential for conflicting information between the information associated with the contractual product and the reference information maintained by the relevant service provider.
 * 
 * @version 2.5.4
 */
@RosettaClass

@RosettaSynonym(value="Warrant", source="FpML_5_10")
public class Warrant extends IdentifiedProduct {
	private static WarrantMeta metaData = new WarrantMeta();
	
	Warrant(WarrantBuilder builder) {
		super(builder);
	}
	
	
	@Override
	public RosettaMetaData<? extends Warrant> metaData() {
		return metaData;
	} 
	
	public WarrantBuilder toBuilder() {
		WarrantBuilder builder = new WarrantBuilder();
		ofNullable(getProductIdentifier()).ifPresent(builder::setProductIdentifier);
		return builder;
	}

	public static WarrantBuilder builder() {
		return new WarrantBuilder();
	}

	public static  class WarrantBuilder extends IdentifiedProductBuilder {
	
	
		public WarrantBuilder() {
		}
				
		@Override
		public RosettaMetaData<? extends Warrant> metaData() {
			return metaData;
		} 
	
	
		@Override public WarrantBuilder setProductIdentifier(ProductIdentifier productIdentifier) {
			this.productIdentifier = productIdentifier.toBuilder();
			return this;
		}
		@Override public WarrantBuilder setProductIdentifierBuilder(ProductIdentifier.ProductIdentifierBuilder productIdentifier) {
			this.productIdentifier = productIdentifier;
			return this;
		}
		
		public Warrant build() {
			return new Warrant(this);
		}
	
		@Override
		public WarrantBuilder prune() {
			super.prune();
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (super.hasData()) return true;
			return false;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			super.process(path, processor);
		
			
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
		
			WarrantBuilder _that = (WarrantBuilder) o;
		
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = super.hashCode();
			return _result;
		}
		
		@Override
		public String toString() {
			return "WarrantBuilder {" +
			'}' + " " + super.toString();
		}
	}

	@Override
	public void process(RosettaPath path, Processor processor) {
		super.process(path, processor);
		
		
	}
	
	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		if (!super.equals(o)) return false;
	
		Warrant _that = (Warrant) o;
	
		return true;
	}
	
	@Override
	public int hashCode() {
		int _result = super.hashCode();
		return _result;
	}
	
	@Override
	public String toString() {
		return "Warrant {" +
		'}' + " " + super.toString();
	}
}
