package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class BasisSwapPriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, BigDecimal, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public BasisSwapPriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "BasisSwapPrice"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#BasisSwapPrice";
	}
	
	
	@Override
	public BlueprintInstance<Payout, BigDecimal, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.88/@nodes/@node", "IsIRSwapBasis", new IsIRSwapBasisRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, FloatingRateSpecification, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.88/@nodes/@next/@node", "->interestRatePayout->rateSpecification->floatingRate", new StringIdentifier("->interestRatePayout->rateSpecification->floatingRate"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate)))
			.then(actionFactory.<FloatingRateSpecification, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.88/@nodes/@next/@next/@node", "->initialRate", new StringIdentifier("Price"), floatingRateSpecification -> MapperS.of(floatingRateSpecification).<BigDecimal>map("getInitialRate", FloatingRateSpecification::getInitialRate)))
			.toBlueprint(getURI(), getName());
	}
}
