package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import org.isda.cdm.Address;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.ExecutingEntity;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class BranchInEEARule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Boolean, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public BranchInEEARule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "BranchInEEA"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#BranchInEEA";
	}
	
	
	@Override
	public BlueprintInstance<Event, Boolean, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(getExecutingEntityForReportingParty())
			.then(actionFactory.<ExecutingEntity, Address, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.32/@nodes/@next/@next/@node", "->addressOfBranch", new StringIdentifier("->addressOfBranch"), executingEntity -> MapperS.of(executingEntity).<Address>map("getAddressOfBranch", ExecutingEntity::getAddressOfBranch)))
			.then(getIsAddressInEEA())
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Contract, ExecutingEntity, INKEY, INKEY> getExecutingEntityForReportingParty() {
		Blueprint<Contract, ExecutingEntity, INKEY, INKEY> rule = new ExecutingEntityForReportingPartyRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Address, Boolean, INKEY, INKEY> getIsAddressInEEA() {
		Blueprint<Address, Boolean, INKEY, INKEY> rule = new IsAddressInEEARule<>(actionFactory);
		return rule.blueprint();
	}
}
