package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.Payout;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class BuyerSellerRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public BuyerSellerRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "BuyerSeller"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#BuyerSeller";
	}
	
	
	@Override
	public BlueprintInstance<Event, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, Payout, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.73/@nodes/@next/@node", "->contractualProduct->economicTerms->payout", new StringIdentifier("->contractualProduct->economicTerms->payout"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout)))
			.then(BlueprintBuilder.<Payout, String, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, getFixedFloatBuyerSeller()),
				startsWith(actionFactory, getFixedFixedBuyerSeller()),
				startsWith(actionFactory, getSingleCurrencyBasisSwap()),
				startsWith(actionFactory, getCrossCurrencySwapBuyerSeller()),
				startsWith(actionFactory, getCreditDefaultSwapBuyerSeller())
				)
			)
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Payout, String, INKEY, INKEY> getCrossCurrencySwapBuyerSeller() {
		Blueprint<Payout, String, INKEY, INKEY> rule = new CrossCurrencySwapBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, String, INKEY, INKEY> getFixedFloatBuyerSeller() {
		Blueprint<Payout, String, INKEY, INKEY> rule = new FixedFloatBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, String, INKEY, INKEY> getFixedFixedBuyerSeller() {
		Blueprint<Payout, String, INKEY, INKEY> rule = new FixedFixedBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, String, INKEY, INKEY> getCreditDefaultSwapBuyerSeller() {
		Blueprint<Payout, String, INKEY, INKEY> rule = new CreditDefaultSwapBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, String, INKEY, INKEY> getSingleCurrencyBasisSwap() {
		Blueprint<Payout, String, INKEY, INKEY> rule = new SingleCurrencyBasisSwapRule<>(actionFactory);
		return rule.blueprint();
	}
}
