package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.Address;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.ExecutingEntity;
import org.isda.cdm.Party;
import org.isda.cdm.metafields.*;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class BuyerSellerToBuyerAndSellerRule<INKEY extends Comparable<INKEY>> implements Blueprint<BuyerSeller, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public BuyerSellerToBuyerAndSellerRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "BuyerSellerToBuyerAndSeller"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#BuyerSellerToBuyerAndSeller";
	}
	
	
	@Override
	public BlueprintInstance<BuyerSeller, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, BlueprintBuilder.<BuyerSeller, String, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, actionFactory.<BuyerSeller, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.0/@node", "->buyerPartyReference", new StringIdentifier("->buyerPartyReference"), buyerSeller -> MapperS.of(buyerSeller).map("getBuyerPartyReference", BuyerSeller::getBuyerPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
				.then(new Filter<Party, INKEY>("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.0/@next/@node", "->partyId->scheme=\"iso17442\"", party -> areEqual(MapperS.of(party).<FieldWithMetaString>mapC("getPartyId", Party::getPartyId).map("getMeta", a->a.getMeta()).map("getScheme", a->a.getScheme()), MapperS.of("iso17442")).get()))
				.then(actionFactory.<Party, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.0/@next/@next/@node", "->partyId", new StringIdentifier("Buyer identification code"), party -> MapperS.of(party).mapC("getPartyId", Party::getPartyId).<String>map("getValue", FieldWithMeta::getValue))),
				startsWith(actionFactory, actionFactory.<BuyerSeller, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.1/@node", "->buyerPartyReference", new StringIdentifier("->buyerPartyReference"), buyerSeller -> MapperS.of(buyerSeller).map("getBuyerPartyReference", BuyerSeller::getBuyerPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
				.then(new Filter<Party, INKEY>("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.1/@next/@node", "->partyId->scheme=\"iso17442\"", party -> areEqual(MapperS.of(party).<FieldWithMetaString>mapC("getPartyId", Party::getPartyId).map("getMeta", a->a.getMeta()).map("getScheme", a->a.getScheme()), MapperS.of("iso17442")).get()))
				.then(actionFactory.<Party, ExecutingEntity, INKEY>newRosettaLookup("model-reg-reporting.rosetta#BuyerSellerToBuyerAndSeller.ExecutingEntity", "ExecutingEntity", new StringIdentifier("Lookup ExecutingEntity"), "ExecutingEntity"))
				.then(actionFactory.<ExecutingEntity, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.1/@next/@next/@next/@node", "->addressOfBranch->country", new StringIdentifier("Country of the branch for the buyer"), executingEntity -> MapperS.of(executingEntity).<Address>map("getAddressOfBranch", ExecutingEntity::getAddressOfBranch).map("getCountry", Address::getCountry).<String>map("getValue", FieldWithMeta::getValue))),
				startsWith(actionFactory, actionFactory.<BuyerSeller, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.2/@node", "->sellerPartyReference", new StringIdentifier("->sellerPartyReference"), buyerSeller -> MapperS.of(buyerSeller).map("getSellerPartyReference", BuyerSeller::getSellerPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
				.then(new Filter<Party, INKEY>("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.2/@next/@node", "->partyId->scheme=\"iso17442\"", party -> areEqual(MapperS.of(party).<FieldWithMetaString>mapC("getPartyId", Party::getPartyId).map("getMeta", a->a.getMeta()).map("getScheme", a->a.getScheme()), MapperS.of("iso17442")).get()))
				.then(actionFactory.<Party, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.2/@next/@next/@node", "->partyId", new StringIdentifier("Seller identification code"), party -> MapperS.of(party).mapC("getPartyId", Party::getPartyId).<String>map("getValue", FieldWithMeta::getValue))),
				startsWith(actionFactory, actionFactory.<BuyerSeller, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.3/@node", "->sellerPartyReference", new StringIdentifier("->sellerPartyReference"), buyerSeller -> MapperS.of(buyerSeller).map("getSellerPartyReference", BuyerSeller::getSellerPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
				.then(new Filter<Party, INKEY>("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.3/@next/@node", "->partyId->scheme=\"iso17442\"", party -> areEqual(MapperS.of(party).<FieldWithMetaString>mapC("getPartyId", Party::getPartyId).map("getMeta", a->a.getMeta()).map("getScheme", a->a.getScheme()), MapperS.of("iso17442")).get()))
				.then(actionFactory.<Party, ExecutingEntity, INKEY>newRosettaLookup("model-reg-reporting.rosetta#BuyerSellerToBuyerAndSeller.ExecutingEntity", "ExecutingEntity", new StringIdentifier("Lookup ExecutingEntity"), "ExecutingEntity"))
				.then(actionFactory.<ExecutingEntity, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.85/@nodes/@node/@bps.3/@next/@next/@next/@node", "->addressOfBranch->country", new StringIdentifier("Country of the branch for the seller"), executingEntity -> MapperS.of(executingEntity).<Address>map("getAddressOfBranch", ExecutingEntity::getAddressOfBranch).map("getCountry", Address::getCountry).<String>map("getValue", FieldWithMeta::getValue)))
				)
			)
			.toBlueprint(getURI(), getName());
	}
}
