package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintIfThen;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class CDSPriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, Number, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public CDSPriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "CDSPrice"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#CDSPrice";
	}
	
	
	@Override
	public BlueprintInstance<Payout, Number, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.90/@nodes/@node", "IsCreditDefaultSwap", new IsCreditDefaultSwapRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, RateSpecification, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@node", "->interestRatePayout->rateSpecification", new StringIdentifier("->interestRatePayout->rateSpecification"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification)))
			.then(ifElse(actionFactory, getURI(), getName(),
							new BlueprintIfThen(startsWith(actionFactory, new Filter<RateSpecification, INKEY>("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@next/@node/@bps.0/@ifNode/@node", "->fixedRate->initialValue<>0", rateSpecification -> notEqual(MapperS.of(rateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).<BigDecimal>map("getInitialValue", Schedule::getInitialValue), MapperS.of(Integer.valueOf(0))).get())),
								startsWith(actionFactory, actionFactory.<RateSpecification, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@next/@node/@bps.0/@thenNode/@node", "->fixedRate->initialValue", new StringIdentifier("Price"), rateSpecification -> MapperS.of(rateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).<BigDecimal>map("getInitialValue", Schedule::getInitialValue)))),
							new BlueprintIfThen(startsWith(actionFactory, actionFactory.<RateSpecification, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@next/@node/@bps.1/@ifNode/@node", "->floatingRate->initialRate", new StringIdentifier("->floatingRate->initialRate"), rateSpecification -> MapperS.of(rateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<BigDecimal>map("getInitialRate", FloatingRateSpecification::getInitialRate))),
								startsWith(actionFactory, actionFactory.<RateSpecification, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@next/@node/@bps.1/@thenNode/@node", "->floatingRate->initialRate", new StringIdentifier("Price"), rateSpecification -> MapperS.of(rateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<BigDecimal>map("getInitialRate", FloatingRateSpecification::getInitialRate))))
							,
								new BlueprintIfThen(
									startsWith(actionFactory, actionFactory.<RateSpecification, Integer, INKEY> newRosettaReturn("model-reg-reporting.rosetta#//@elements.90/@nodes/@next/@next/@node/@elseNode/@node", "0",  new StringIdentifier("Price"),  () -> MapperS.of(Integer.valueOf(0)))))
					))
			.toBlueprint(getURI(), getName());
	}
}
