package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import org.isda.cdm.BuyerSeller;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.Payout;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class CreditDefaultSwapBuyerSellerRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public CreditDefaultSwapBuyerSellerRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "CreditDefaultSwapBuyerSeller"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#CreditDefaultSwapBuyerSeller";
	}
	
	
	@Override
	public BlueprintInstance<Payout, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.83/@nodes/@node", "IsCreditDefaultSwap", new IsCreditDefaultSwapRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, BuyerSeller, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.83/@nodes/@next/@node", "->creditDefaultPayout->generalTerms->buyerSeller", new StringIdentifier("->creditDefaultPayout->generalTerms->buyerSeller"), payout -> MapperS.of(payout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<GeneralTerms>map("getGeneralTerms", CreditDefaultPayout::getGeneralTerms).<BuyerSeller>map("getBuyerSeller", GeneralTerms::getBuyerSeller)))
			.then(getBuyerSellerToBuyerAndSeller())
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <BuyerSeller, String, INKEY, INKEY> getBuyerSellerToBuyerAndSeller() {
		Blueprint<BuyerSeller, String, INKEY, INKEY> rule = new BuyerSellerToBuyerAndSellerRule<>(actionFactory);
		return rule.blueprint();
	}
}
