package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.ReduceBy;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.Payout;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class CrossCurrencySwapBuyerSellerRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public CrossCurrencySwapBuyerSellerRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "CrossCurrencySwapBuyerSeller"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#CrossCurrencySwapBuyerSeller";
	}
	
	
	@Override
	public BlueprintInstance<Payout, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.81/@nodes/@node", "IsXCCYSwap", new IsXCCYSwapRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, InterestRatePayout, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.81/@nodes/@next/@node", "->interestRatePayout", new StringIdentifier("->interestRatePayout"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout)))
			.then(new ReduceBy<InterestRatePayout, String, INKEY>("model-reg-reporting.rosetta#//@elements.81/@nodes/@next/@next/@node", "->quantity->notionalSchedule->notionalStepSchedule->currency", ReduceBy.Action.MINBY, 
								interestRatePayout -> MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue).get()))
			.then(actionFactory.<InterestRatePayout, PayerReceiver, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.81/@nodes/@next/@next/@next/@node", "->payerReceiver", new StringIdentifier("->payerReceiver"), interestRatePayout -> MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver)))
			.then(getPayerReceiverToBuyerSeller())
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <PayerReceiver, String, INKEY, INKEY> getPayerReceiverToBuyerSeller() {
		Blueprint<PayerReceiver, String, INKEY, INKEY> rule = new PayerReceiverToBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
}
