package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.records.Date;
import org.isda.cdm.Event;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;

/**
 * @version 2.5.4
 */
public class ESMAEMIR_ITS_9Report<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Object, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public ESMAEMIR_ITS_9Report(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "ESMAEMIR_ITS_9"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#//@elements.21";
	}
	
	
	@Override
	public BlueprintInstance<Event, Object, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getEmirInvestmentFirm())
			.then(BlueprintBuilder.<Event, Object, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, getReportingCounterpartyID()),
				startsWith(actionFactory, getBuyerSeller()),
				startsWith(actionFactory, getMaturityDate()),
				startsWith(actionFactory, getPrice())
				)
			)
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getReportingCounterpartyID() {
		Blueprint<Event, String, INKEY, INKEY> rule = new ReportingCounterpartyIDRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getBuyerSeller() {
		Blueprint<Event, String, INKEY, INKEY> rule = new BuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Object, INKEY, INKEY> getPrice() {
		Blueprint<Event, Object, INKEY, INKEY> rule = new PriceRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getEmirInvestmentFirm() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new EmirInvestmentFirmRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Date, INKEY, INKEY> getMaturityDate() {
		Blueprint<Event, Date, INKEY, INKEY> rule = new MaturityDateRule<>(actionFactory);
		return rule.blueprint();
	}
}
