package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import org.isda.cdm.Event;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;

/**
 * @version 2.5.4
 */
public class ESMAMiFIR_RTS_22Report<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Object, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public ESMAMiFIR_RTS_22Report(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "ESMAMiFIR_RTS_22"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#//@elements.20";
	}
	
	
	@Override
	public BlueprintInstance<Event, Object, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getMifirInvestmentFirm())
			.then(getReportableTransaction())
			.then(getReportableProduct())
			.then(BlueprintBuilder.<Event, Object, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, getReportStatus()),
				startsWith(actionFactory, getTransactionReferenceNumber()),
				startsWith(actionFactory, getTradingVenueTransactionIdentificationCode()),
				startsWith(actionFactory, getExecutingEntityIdentificationCode()),
				startsWith(actionFactory, getIsInvestmentFirm()),
				startsWith(actionFactory, getSubmittingEntityIdentificationCode()),
				startsWith(actionFactory, getBuyerSeller()),
				startsWith(actionFactory, getTransmissionOfOrderIndicator()),
				startsWith(actionFactory, getTradingDateTime()),
				startsWith(actionFactory, getTradingCapacity()),
				startsWith(actionFactory, getQuantity()),
				startsWith(actionFactory, getPrice()),
				startsWith(actionFactory, getVenue()),
				startsWith(actionFactory, getCountryOfTheBranchMembership()),
				startsWith(actionFactory, getInstrumentIdentificationCode()),
				startsWith(actionFactory, getInstrumentFullName()),
				startsWith(actionFactory, getInstrumentClassification()),
				startsWith(actionFactory, getNotionalCurrency1()),
				startsWith(actionFactory, getNotionalCurrency2()),
				startsWith(actionFactory, getPriceMultiplier()),
				startsWith(actionFactory, getUnderlyingInstrumentCode()),
				startsWith(actionFactory, getUnderlyingIndexName()),
				startsWith(actionFactory, getUnderlyingIndexTermPeriod()),
				startsWith(actionFactory, getUnderlyingIndexTermMultiplier()),
				startsWith(actionFactory, getExpiryDate()),
				startsWith(actionFactory, getDeliveryType()),
				startsWith(actionFactory, getInvestmentDecisionWithinFirm()),
				startsWith(actionFactory, getPersonResponsibleForInvestmentDecisionCountry()),
				startsWith(actionFactory, getExecutionWithinFirm()),
				startsWith(actionFactory, getPersonResponsibleForExecutionCountry()),
				startsWith(actionFactory, getCommodityDerivativeIndicator()),
				startsWith(actionFactory, getSecuritiesFinancingTransactionIndicator())
				)
			)
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getUnderlyingInstrumentCode() {
		Blueprint<Event, String, INKEY, INKEY> rule = new UnderlyingInstrumentCodeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getExpiryDate() {
		Blueprint<Event, String, INKEY, INKEY> rule = new ExpiryDateRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getReportableTransaction() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new ReportableTransactionRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getCommodityDerivativeIndicator() {
		Blueprint<Event, String, INKEY, INKEY> rule = new CommodityDerivativeIndicatorRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getMifirInvestmentFirm() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new MifirInvestmentFirmRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getUnderlyingIndexTermPeriod() {
		Blueprint<Event, String, INKEY, INKEY> rule = new UnderlyingIndexTermPeriodRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getNotionalCurrency2() {
		Blueprint<Event, String, INKEY, INKEY> rule = new NotionalCurrency2Rule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getNotionalCurrency1() {
		Blueprint<Event, String, INKEY, INKEY> rule = new NotionalCurrency1Rule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getBuyerSeller() {
		Blueprint<Event, String, INKEY, INKEY> rule = new BuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getPersonResponsibleForExecutionCountry() {
		Blueprint<Event, String, INKEY, INKEY> rule = new PersonResponsibleForExecutionCountryRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getReportableProduct() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new ReportableProductRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getUnderlyingIndexName() {
		Blueprint<Event, String, INKEY, INKEY> rule = new UnderlyingIndexNameRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, ZonedDateTime, INKEY, INKEY> getTradingDateTime() {
		Blueprint<Event, ZonedDateTime, INKEY, INKEY> rule = new TradingDateTimeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getTradingCapacity() {
		Blueprint<Event, String, INKEY, INKEY> rule = new TradingCapacityRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getReportStatus() {
		Blueprint<Event, String, INKEY, INKEY> rule = new ReportStatusRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getExecutionWithinFirm() {
		Blueprint<Event, String, INKEY, INKEY> rule = new ExecutionWithinFirmRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getDeliveryType() {
		Blueprint<Event, String, INKEY, INKEY> rule = new DeliveryTypeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getIsInvestmentFirm() {
		Blueprint<Event, String, INKEY, INKEY> rule = new IsInvestmentFirmRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getVenue() {
		Blueprint<Event, String, INKEY, INKEY> rule = new VenueRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getPersonResponsibleForInvestmentDecisionCountry() {
		Blueprint<Event, String, INKEY, INKEY> rule = new PersonResponsibleForInvestmentDecisionCountryRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getSecuritiesFinancingTransactionIndicator() {
		Blueprint<Event, String, INKEY, INKEY> rule = new SecuritiesFinancingTransactionIndicatorRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getExecutingEntityIdentificationCode() {
		Blueprint<Event, String, INKEY, INKEY> rule = new ExecutingEntityIdentificationCodeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, BigDecimal, INKEY, INKEY> getQuantity() {
		Blueprint<Event, BigDecimal, INKEY, INKEY> rule = new QuantityRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getInstrumentFullName() {
		Blueprint<Event, String, INKEY, INKEY> rule = new InstrumentFullNameRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getInstrumentClassification() {
		Blueprint<Event, String, INKEY, INKEY> rule = new InstrumentClassificationRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getSubmittingEntityIdentificationCode() {
		Blueprint<Event, String, INKEY, INKEY> rule = new SubmittingEntityIdentificationCodeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getTransmissionOfOrderIndicator() {
		Blueprint<Event, String, INKEY, INKEY> rule = new TransmissionOfOrderIndicatorRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getInvestmentDecisionWithinFirm() {
		Blueprint<Event, String, INKEY, INKEY> rule = new InvestmentDecisionWithinFirmRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getPriceMultiplier() {
		Blueprint<Event, String, INKEY, INKEY> rule = new PriceMultiplierRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getInstrumentIdentificationCode() {
		Blueprint<Event, String, INKEY, INKEY> rule = new InstrumentIdentificationCodeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Object, INKEY, INKEY> getPrice() {
		Blueprint<Event, Object, INKEY, INKEY> rule = new PriceRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getCountryOfTheBranchMembership() {
		Blueprint<Event, String, INKEY, INKEY> rule = new CountryOfTheBranchMembershipRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getUnderlyingIndexTermMultiplier() {
		Blueprint<Event, String, INKEY, INKEY> rule = new UnderlyingIndexTermMultiplierRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getTransactionReferenceNumber() {
		Blueprint<Event, String, INKEY, INKEY> rule = new TransactionReferenceNumberRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, String, INKEY, INKEY> getTradingVenueTransactionIdentificationCode() {
		Blueprint<Event, String, INKEY, INKEY> rule = new TradingVenueTransactionIdentificationCodeRule<>(actionFactory);
		return rule.blueprint();
	}
}
