package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.Contract;
import org.isda.cdm.ExecutingEntity;
import org.isda.cdm.Party;
import org.isda.cdm.PartyContractInformation;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.RelatedParty;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class ExecutingEntityForReportingPartyRule<INKEY extends Comparable<INKEY>> implements Blueprint<Contract, ExecutingEntity, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public ExecutingEntityForReportingPartyRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "ExecutingEntityForReportingParty"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#ExecutingEntityForReportingParty";
	}
	
	
	@Override
	public BlueprintInstance<Contract, ExecutingEntity, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, actionFactory.<Contract, PartyContractInformation, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.35/@nodes/@node", "->partyContractInformation", new StringIdentifier("->partyContractInformation"), contract -> MapperS.of(contract).<PartyContractInformation>mapC("getPartyContractInformation", Contract::getPartyContractInformation)))
			.then(new Filter<PartyContractInformation, INKEY>("model-reg-reporting.rosetta#//@elements.35/@nodes/@next/@node", "->relatedParty->role=PartyRoleEnum", partyContractInformation -> areEqual(MapperS.of(partyContractInformation).<RelatedParty>map("getRelatedParty", PartyContractInformation::getRelatedParty).<PartyRoleEnum>map("getRole", RelatedParty::getRole), MapperS.of(PartyRoleEnum.REPORTING_PARTY)).get()))
			.then(actionFactory.<PartyContractInformation, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.35/@nodes/@next/@next/@node", "->partyReference", new StringIdentifier("->partyReference"), partyContractInformation -> MapperS.of(partyContractInformation).map("getPartyReference", PartyContractInformation::getPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
			.then(actionFactory.<Party, ExecutingEntity, INKEY>newRosettaLookup("model-reg-reporting.rosetta#ExecutingEntityForReportingParty.ExecutingEntity", "ExecutingEntity", new StringIdentifier("Lookup ExecutingEntity"), "ExecutingEntity"))
			.toBlueprint(getURI(), getName());
	}
}
