package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.Party;
import org.isda.cdm.PartyContractInformation;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.RelatedParty;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class ExecutingEntityIdentificationCodeRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public ExecutingEntityIdentificationCodeRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "ExecutingEntityIdentificationCode"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#ExecutingEntityIdentificationCode";
	}
	
	
	@Override
	public BlueprintInstance<Event, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, PartyContractInformation, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.41/@nodes/@next/@node", "->partyContractInformation", new StringIdentifier("->partyContractInformation"), contract -> MapperS.of(contract).<PartyContractInformation>mapC("getPartyContractInformation", Contract::getPartyContractInformation)))
			.then(new Filter<PartyContractInformation, INKEY>("model-reg-reporting.rosetta#//@elements.41/@nodes/@next/@next/@node", "->relatedParty->role=PartyRoleEnum", partyContractInformation -> areEqual(MapperS.of(partyContractInformation).<RelatedParty>map("getRelatedParty", PartyContractInformation::getRelatedParty).<PartyRoleEnum>map("getRole", RelatedParty::getRole), MapperS.of(PartyRoleEnum.REPORTING_PARTY)).get()))
			.then(actionFactory.<PartyContractInformation, Party, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.41/@nodes/@next/@next/@next/@node", "->partyReference", new StringIdentifier("->partyReference"), partyContractInformation -> MapperS.of(partyContractInformation).map("getPartyReference", PartyContractInformation::getPartyReference).<Party>map("getValue", FieldWithMeta::getValue)))
			.then(actionFactory.<Party, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.41/@nodes/@next/@next/@next/@next/@node", "->partyId", new StringIdentifier("Executing Entity Identification Code"), party -> MapperS.of(party).mapC("getPartyId", Party::getPartyId).<String>map("getValue", FieldWithMeta::getValue)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
