package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.ReduceBy;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class FixedFixedPriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, BigDecimal, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public FixedFixedPriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "FixedFixedPrice"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#FixedFixedPrice";
	}
	
	
	@Override
	public BlueprintInstance<Payout, BigDecimal, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.89/@nodes/@node", "IsFixedFixed", new IsFixedFixedRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, Schedule, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.89/@nodes/@next/@node", "->interestRatePayout->rateSpecification->fixedRate", new StringIdentifier("->interestRatePayout->rateSpecification->fixedRate"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate)))
			.then(new ReduceBy<Schedule, BigDecimal, INKEY>("model-reg-reporting.rosetta#//@elements.89/@nodes/@next/@next/@node", "->initialValue", ReduceBy.Action.MAXBY, 
								schedule -> MapperS.of(schedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue).get()))
			.then(actionFactory.<Schedule, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.89/@nodes/@next/@next/@next/@node", "->initialValue", new StringIdentifier("Price"), schedule -> MapperS.of(schedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue)))
			.toBlueprint(getURI(), getName());
	}
}
