package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class FixedFloatBuyerSellerRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public FixedFloatBuyerSellerRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "FixedFloatBuyerSeller"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#FixedFloatBuyerSeller";
	}
	
	
	@Override
	public BlueprintInstance<Payout, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.74/@nodes/@node", "IsFixedFloat", new IsFixedFloatRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, InterestRatePayout, INKEY>newRosettaMultipleMapper("model-reg-reporting.rosetta#//@elements.74/@nodes/@next/@node", "->interestRatePayout", new StringIdentifier("->interestRatePayout"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout)))
			.then(new Filter<InterestRatePayout, INKEY>("model-reg-reporting.rosetta#//@elements.74/@nodes/@next/@next/@node", "->rateSpecification->fixedRate exists", interestRatePayout -> exists(MapperS.of(interestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate), false).get()))
			.then(actionFactory.<InterestRatePayout, PayerReceiver, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.74/@nodes/@next/@next/@next/@node", "->payerReceiver", new StringIdentifier("->payerReceiver"), interestRatePayout -> MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver)))
			.then(getPayerReceiverToBuyerSeller())
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <PayerReceiver, String, INKEY, INKEY> getPayerReceiverToBuyerSeller() {
		Blueprint<PayerReceiver, String, INKEY, INKEY> rule = new PayerReceiverToBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
}
