package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class FixedRatePriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, BigDecimal, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public FixedRatePriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "FixedRatePrice"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#FixedRatePrice";
	}
	
	
	@Override
	public BlueprintInstance<Event, BigDecimal, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(new Filter<Contract, INKEY>("model-reg-reporting.rosetta#//@elements.49/@nodes/@next/@node", "->contractualProduct->economicTerms->payout->interestRatePayout->rateSpecification->fixedRate exists", contract -> exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate), false).get()))
			.then(actionFactory.<Contract, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.49/@nodes/@next/@next/@node", "->contractualProduct->economicTerms->payout->interestRatePayout->rateSpecification->fixedRate->initialValue", new StringIdentifier("Price"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).<BigDecimal>map("getInitialValue", Schedule::getInitialValue)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
