package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.SpreadSchedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class FloatingRatePriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, BigDecimal, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public FloatingRatePriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "FloatingRatePrice"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#FloatingRatePrice";
	}
	
	
	@Override
	public BlueprintInstance<Event, BigDecimal, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, RateSpecification, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.50/@nodes/@next/@node", "->contractualProduct->economicTerms->payout->interestRatePayout->rateSpecification", new StringIdentifier("->contractualProduct->economicTerms->payout->interestRatePayout->rateSpecification"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification)))
			.then(actionFactory.<RateSpecification, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.50/@nodes/@next/@next/@node", "->floatingRate->spreadSchedule->initialValue", new StringIdentifier("Price"), rateSpecification -> MapperS.of(rateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<SpreadSchedule>mapC("getSpreadSchedule", FloatingRate::getSpreadSchedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
