package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.Event;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class HasContractRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Boolean, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public HasContractRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "HasContract"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#HasContract";
	}
	
	
	@Override
	public BlueprintInstance<Event, Boolean, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, Boolean, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.23/@nodes/@next/@node", "->contractualProduct exists", new StringIdentifier("->contractualProduct exists"), contract -> exists(MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct), false)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
