package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.records.Date;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class MaturityDateRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Date, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public MaturityDateRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "MaturityDate"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#MaturityDate";
	}
	
	
	@Override
	public BlueprintInstance<Event, Date, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, Date, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.36/@nodes/@next/@node", "->contractualProduct->economicTerms->terminationDate->adjustableDate->unadjustedDate", new StringIdentifier("->contractualProduct->economicTerms->terminationDate->adjustableDate->unadjustedDate"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
