package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.Payout;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class PriceRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Object, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public PriceRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "Price"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#Price";
	}
	
	
	@Override
	public BlueprintInstance<Event, Object, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, Payout, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.86/@nodes/@next/@node", "->contractualProduct->economicTerms->payout", new StringIdentifier("->contractualProduct->economicTerms->payout"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout)))
			.then(BlueprintBuilder.<Payout, Object, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, getFixedFixedPrice()),
				startsWith(actionFactory, getFixedFloatPrice()),
				startsWith(actionFactory, getBasisSwapPrice()),
				startsWith(actionFactory, getCDSPrice())
				)
			)
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Payout, BigDecimal, INKEY, INKEY> getFixedFixedPrice() {
		Blueprint<Payout, BigDecimal, INKEY, INKEY> rule = new FixedFixedPriceRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, Number, INKEY, INKEY> getCDSPrice() {
		Blueprint<Payout, Number, INKEY, INKEY> rule = new CDSPriceRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, BigDecimal, INKEY, INKEY> getBasisSwapPrice() {
		Blueprint<Payout, BigDecimal, INKEY, INKEY> rule = new BasisSwapPriceRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Payout, BigDecimal, INKEY, INKEY> getFixedFloatPrice() {
		Blueprint<Payout, BigDecimal, INKEY, INKEY> rule = new FixedFloatPriceRule<>(actionFactory);
		return rule.blueprint();
	}
}
