package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.Quantity;
import org.isda.cdm.QuantityNotation;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class QuantityRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, BigDecimal, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public QuantityRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "Quantity"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#Quantity";
	}
	
	
	@Override
	public BlueprintInstance<Event, BigDecimal, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, BigDecimal, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.47/@nodes/@next/@node", "->contractualQuantity->quantityNotation->quantity->amount", new StringIdentifier("Quantity"), contract -> MapperS.of(contract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
