package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import org.isda.cdm.Event;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;

/**
 * @version 2.5.4
 */
public class ReportableTransactionRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, Event, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public ReportableTransactionRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "ReportableTransaction"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#ReportableTransaction";
	}
	
	
	@Override
	public BlueprintInstance<Event, Event, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, BlueprintBuilder.<Event, Event, INKEY, INKEY>and(actionFactory,
				startsWith(actionFactory, getNewTrade()),
				startsWith(actionFactory, getQuantityChange())
				)
			)
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getNewTrade() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new NewTradeRule<>(actionFactory);
		return rule.blueprint();
	}
	
	protected BlueprintInstance <Event, Event, INKEY, INKEY> getQuantityChange() {
		Blueprint<Event, Event, INKEY, INKEY> rule = new QuantityChangeRule<>(actionFactory);
		return rule.blueprint();
	}
}
