package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.FilterByRule;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.SpreadSchedule;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class SingleCurrencyBasisSwapRule<INKEY extends Comparable<INKEY>> implements Blueprint<Payout, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public SingleCurrencyBasisSwapRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "SingleCurrencyBasisSwap"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#SingleCurrencyBasisSwap";
	}
	
	
	@Override
	public BlueprintInstance<Payout, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, new FilterByRule<Payout, INKEY>("model-reg-reporting.rosetta#//@elements.79/@nodes/@node", "IsIRSwapBasis", new IsIRSwapBasisRule<INKEY>(actionFactory).blueprint()))
			.then(actionFactory.<Payout, InterestRatePayout, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.79/@nodes/@next/@node", "->interestRatePayout", new StringIdentifier("->interestRatePayout"), payout -> MapperS.of(payout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout)))
			.then(new Filter<InterestRatePayout, INKEY>("model-reg-reporting.rosetta#//@elements.79/@nodes/@next/@next/@node", "->rateSpecification->floatingRate->spreadSchedule->initialValue exists", interestRatePayout -> exists(MapperS.of(interestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<SpreadSchedule>mapC("getSpreadSchedule", FloatingRate::getSpreadSchedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue), false).get()))
			.then(actionFactory.<InterestRatePayout, PayerReceiver, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.79/@nodes/@next/@next/@next/@node", "->payerReceiver", new StringIdentifier("->payerReceiver"), interestRatePayout -> MapperS.of(interestRatePayout).<PayerReceiver>map("getPayerReceiver", InterestRatePayout::getPayerReceiver)))
			.then(getPayerReceiverToBuyerSeller())
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <PayerReceiver, String, INKEY, INKEY> getPayerReceiverToBuyerSeller() {
		Blueprint<PayerReceiver, String, INKEY, INKEY> rule = new PayerReceiverToBuyerSellerRule<>(actionFactory);
		return rule.blueprint();
	}
}
