package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.Filter;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import java.time.ZonedDateTime;
import org.isda.cdm.Event;
import org.isda.cdm.EventTimestamp;
import org.isda.cdm.EventTimestampQualificationEnum;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class TradingDateTimeRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, ZonedDateTime, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public TradingDateTimeRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "TradingDateTime"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#TradingDateTime";
	}
	
	
	@Override
	public BlueprintInstance<Event, ZonedDateTime, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, actionFactory.<Event, EventTimestamp, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.45/@nodes/@node", "->timestamp", new StringIdentifier("->timestamp"), event -> MapperS.of(event).<EventTimestamp>mapC("getTimestamp", Event::getTimestamp)))
			.then(new Filter<EventTimestamp, INKEY>("model-reg-reporting.rosetta#//@elements.45/@nodes/@next/@node", "->qualification=EventTimestampQualificationEnum", eventTimestamp -> areEqual(MapperS.of(eventTimestamp).<EventTimestampQualificationEnum>map("getQualification", EventTimestamp::getQualification), MapperS.of(EventTimestampQualificationEnum.EXECUTION_DATE_TIME)).get()))
			.then(actionFactory.<EventTimestamp, ZonedDateTime, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.45/@nodes/@next/@next/@node", "->dateTime", new StringIdentifier("Trading Date Time"), eventTimestamp -> MapperS.of(eventTimestamp).<ZonedDateTime>map("getDateTime", EventTimestamp::getDateTime)))
			.toBlueprint(getURI(), getName());
	}
}
