package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.AssignedIdentifier;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.Identifier;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class TradingVenueTransactionIdentificationCodeRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public TradingVenueTransactionIdentificationCodeRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "TradingVenueTransactionIdentificationCode"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#TradingVenueTransactionIdentificationCode";
	}
	
	
	@Override
	public BlueprintInstance<Event, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.40/@nodes/@next/@node", "->contractIdentifier->assignedIdentifier->identifier", new StringIdentifier("Trading Venue Transaction Identification Code"), contract -> MapperS.of(contract).<Identifier>mapC("getContractIdentifier", Contract::getContractIdentifier).<AssignedIdentifier>mapC("getAssignedIdentifier", Identifier::getAssignedIdentifier).map("getIdentifier", AssignedIdentifier::getIdentifier).<String>map("getValue", FieldWithMeta::getValue)))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
