package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.Basket;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.Event;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.ProductIdentification;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class UnderlyingIndexNameRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public UnderlyingIndexNameRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "UnderlyingIndexName"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#UnderlyingIndexName";
	}
	
	
	@Override
	public BlueprintInstance<Event, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, getContractForEvent())
			.then(actionFactory.<Contract, Underlier, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.60/@nodes/@next/@node", "->contractualProduct->economicTerms->payout->equityPayout->underlier", new StringIdentifier("->contractualProduct->economicTerms->payout->equityPayout->underlier"), contract -> MapperS.of(contract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier)))
			.then(actionFactory.<Underlier, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.60/@nodes/@next/@next/@node", "choice", new StringIdentifier("Underlying Index Name"), underlier -> doIf(exists(MapperS.of(underlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier), false),MapperS.of(underlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct).<ProductIdentification>map("getProductIdentification", ContractualProduct::getProductIdentification).mapC("getProductId", ProductIdentification::getProductId).<String>map("getValue", FieldWithMeta::getValue),doIf(exists(MapperS.of(underlier).<Basket>map("getBasket", Underlier::getBasket), false),MapperS.of("Add basket id path here"),MapperS.of("NOAP")))))
			.toBlueprint(getURI(), getName());
	}
	
	protected BlueprintInstance <Event, Contract, INKEY, INKEY> getContractForEvent() {
		Blueprint<Event, Contract, INKEY, INKEY> rule = new ContractForEventRule<>(actionFactory);
		return rule.blueprint();
	}
}
