package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.AllocationInstructions;
import org.isda.cdm.Event;
import org.isda.cdm.Execution;
import org.isda.cdm.functions.NewAllocationPrimitive;


public class Allocate implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewAllocationPrimitive newAllocationPrimitive;

	/**
	* @param execution Block trade to be allocated.
	* @param allocationInstructions Allocation instructions to be applied.
	* @param previousEvent Previous event for lineage purposes.
	* @return allocationEvent Allocation event containing both the original block trade (with the status set to Allocated) and the set of allocated trades.
	*/
	public Event evaluate(Execution execution, AllocationInstructions allocationInstructions, Event previousEvent) {
		
		Event allocationEvent = doEvaluate(execution, allocationInstructions, previousEvent).build();
		
		objectValidator.validateAndFailOnErorr(Event.class, allocationEvent);
		return allocationEvent;
	}
	
	protected Event.EventBuilder doEvaluate(Execution execution, AllocationInstructions allocationInstructions, Event previousEvent) {
		Event.EventBuilder allocationEventHolder = Event.builder();
		@SuppressWarnings("unused") Event allocationEvent = allocationEventHolder.build();
		allocationEventHolder
			.getOrCreatePrimitive()
			.addAllocation(MapperS.of(newAllocationPrimitive.evaluate(MapperS.of(execution).get(), MapperS.of(allocationInstructions).get())).get());
		;
		allocationEvent = allocationEventHolder.build();
		allocationEventHolder
			.getOrCreateLineage()
			.addEventReferenceRef(MapperS.of(previousEvent).get());
		;
		allocationEvent = allocationEventHolder.build();
		allocationEventHolder
			.getOrCreateLineage()
			.addExecutionReferenceRef(MapperS.of(execution).get());
		;
		return allocationEventHolder;
	}
	
}
