package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.math.BigDecimal;
import org.isda.cdm.ActualPrice;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.Money;
import org.isda.cdm.Payout;
import org.isda.cdm.PayoutBase;
import org.isda.cdm.Price;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.QuantityNotationEnum;
import org.isda.cdm.ResolvablePayoutQuantity;
import org.isda.cdm.functions.Abs;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;

public class EquityCashSettlementAmount implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected Abs abs;

	/**
	* @param contractState 
	* @param date 
	* @return equityCashSettlementAmount 
	*/
	public Money evaluate(ContractState contractState, Date date) {
		// pre-conditions
		
			assert
				areEqual(MapperS.of(equityPayout(contractState, date).get()).<ResolvablePayoutQuantity>map("getPayoutQuantity", PayoutBase::getPayoutQuantity).<QuantityNotationEnum>map("getQuantityNotationTag", ResolvablePayoutQuantity::getQuantityNotationTag), MapperS.of(QuantityNotationEnum.NUMBER_OF_SECURITIES)).get()
				: "";
		
		Money equityCashSettlementAmount = doEvaluate(contractState, date).build();
		
		objectValidator.validateAndFailOnErorr(Money.class, equityCashSettlementAmount);
		return equityCashSettlementAmount;
	}
	
	protected Money.MoneyBuilder doEvaluate(ContractState contractState, Date date) {
		Money.MoneyBuilder equityCashSettlementAmountHolder = Money.builder();
		@SuppressWarnings("unused") Money equityCashSettlementAmount = equityCashSettlementAmountHolder.build();
		equityCashSettlementAmountHolder
			.setAmount(MapperS.of(abs.evaluate(MapperS.of(contractState).<Contract>map("getUpdatedContract", ContractState::getUpdatedContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<BigDecimal>map("getPerformance", EquityPayout::getPerformance).get())).get());
		;
		equityCashSettlementAmount = equityCashSettlementAmountHolder.build();
		equityCashSettlementAmountHolder
			.setCurrencyRef(MapperS.of(equityPayout(contractState, date).get()).<PriceReturnTerms>map("getPriceReturnTerms", EquityPayout::getPriceReturnTerms).<Price>map("getInitialPrice", PriceReturnTerms::getInitialPrice).<ActualPrice>map("getNetPrice", Price::getNetPrice).map("getCurrency", ActualPrice::getCurrency).<String>map("getValue", FieldWithMeta::getValue).get());
		;
		return equityCashSettlementAmountHolder;
	}
	
	
	protected MapperBuilder<EquityPayout> equityPayout(ContractState contractState, Date date) {
		return MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout);
	}
}
