package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.math.BigDecimalExtensions;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.PayoutBase;
import org.isda.cdm.Quantity;
import org.isda.cdm.ResolvablePayoutQuantity;
import org.isda.cdm.functions.EquityNotionalAmount;
import org.isda.cdm.functions.RateOfReturn;
import org.isda.cdm.functions.ResolveEquityPeriodEndPrice;
import org.isda.cdm.functions.ResolveEquityPeriodStartPrice;
import org.isda.cdm.functions.ResolveQuantity;


public class EquityPerformance implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected EquityNotionalAmount equityNotionalAmount;
	@Inject protected RateOfReturn rateOfReturn;
	@Inject protected ResolveEquityPeriodEndPrice resolveEquityPeriodEndPrice;
	@Inject protected ResolveEquityPeriodStartPrice resolveEquityPeriodStartPrice;
	@Inject protected ResolveQuantity resolveQuantity;

	/**
	* @param equityPayout 
	* @param observation 
	* @param date 
	* @return equityPerformance 
	*/
	public BigDecimal evaluate(EquityPayout equityPayout, BigDecimal observation, Date date) {
		
		BigDecimal equityPerformance = doEvaluate(equityPayout, observation, date);
		
		return equityPerformance;
	}
	
	protected BigDecimal doEvaluate(EquityPayout equityPayout, BigDecimal observation, Date date) {
		Mapper<BigDecimal> equityPerformanceHolder = null;
		equityPerformanceHolder = MapperS.of(BigDecimalExtensions.multiply(rateOfReturn(equityPayout, observation, date).get(), notionalAmount(equityPayout, observation, date).get()));
		return equityPerformanceHolder.get();
	}
	
	
	protected Mapper<BigDecimal> periodStartPrice(EquityPayout equityPayout, BigDecimal observation, Date date) {
		return MapperS.of(resolveEquityPeriodStartPrice.evaluate(MapperS.of(equityPayout).get(), MapperS.of(date).get()));
	}
	
	protected Mapper<BigDecimal> periodEndPrice(EquityPayout equityPayout, BigDecimal observation, Date date) {
		return MapperS.of(resolveEquityPeriodEndPrice.evaluate(MapperS.of(equityPayout).get(), MapperS.of(date).get()));
	}
	
	protected MapperBuilder<Quantity> quantity(EquityPayout equityPayout, BigDecimal observation, Date date) {
		return MapperS.of(resolveQuantity.evaluate(MapperS.of(equityPayout).<ResolvablePayoutQuantity>map("getPayoutQuantity", PayoutBase::getPayoutQuantity).get(), MapperS.of(date).get()));
	}
	
	protected Mapper<BigDecimal> rateOfReturn(EquityPayout equityPayout, BigDecimal observation, Date date) {
		return MapperS.of(rateOfReturn.evaluate(MapperS.of(periodStartPrice(equityPayout, observation, date).get()).get(), MapperS.of(periodEndPrice(equityPayout, observation, date).get()).get()));
	}
	
	protected Mapper<BigDecimal> notionalAmount(EquityPayout equityPayout, BigDecimal observation, Date date) {
		return MapperS.of(equityNotionalAmount.evaluate(MapperS.of(quantity(equityPayout, observation, date).get()).get(), MapperS.of(periodEndPrice(equityPayout, observation, date).get()).get()));
	}
}
