package org.isda.cdm.functions;

import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import org.isda.cdm.Portfolio;
import org.isda.cdm.PortfolioState;


@ImplementedBy(EvaluatePortfolioStateImpl.class)
public abstract class EvaluatePortfolioState implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param portfolio Portfolio containing the aggregation parameters to be used to calculate the new portfolio state.
	* @return portfolioState Portfolio state containing the aggregated positions based on the input aggregation parameters.
	*/
	public PortfolioState evaluate(Portfolio portfolio) {
		
		PortfolioState portfolioState = doEvaluate(portfolio).build();
		
		objectValidator.validateAndFailOnErorr(PortfolioState.class, portfolioState);
		return portfolioState;
	}
	
	protected abstract PortfolioState.PortfolioStateBuilder doEvaluate(Portfolio portfolio);
}
