package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Event;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionPrimitive;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.ExecutionState;
import org.isda.cdm.Lineage;
import org.isda.cdm.Party;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.Product;
import org.isda.cdm.functions.NewExecutionPrimitive;

import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class Execute implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewExecutionPrimitive newExecutionPrimitive;

	/**
	* @param product The financial product, either contractual or fungible, that is the subject of the execution.
	* @param quantity The quantity of the financial product being executed.
	* @param partyA The parties to that execution, for now assumed Principal-Principal, i.e. no Agent.
	* @param partyB The parties to that execution, for now assumed Principal-Principal, i.e. no Agent.
	* @return execution 
	*/
	public Event evaluate(Product product, ExecutionQuantity quantity, Party partyA, Party partyB) {
		
		Event execution = doEvaluate(product, quantity, partyA, partyB).build();
		
		// post-conditions

		assert
			notExists(MapperS.of(execution).<Lineage>map("getLineage", Event::getLineage)).get()
			: "";
		objectValidator.validateAndFailOnErorr(Event.class, execution);
		return execution;
	}
	
	protected Event.EventBuilder doEvaluate(Product product, ExecutionQuantity quantity, Party partyA, Party partyB) {
		Event.EventBuilder executionHolder = Event.builder();
		@SuppressWarnings("unused") Event execution = executionHolder.build();
		executionHolder
			.getOrCreatePrimitive()
			.addExecution(MapperS.of(newExecutionPrimitive.evaluate(MapperS.of(product).get(), MapperS.of(partyA).get(), MapperS.of(partyB).get(), MapperS.of(quantity).get())).get());
		;
		execution = executionHolder.build();
		executionHolder
			.getOrCreateEventEffect()
			.addExecutionRef(MapperS.of(execution).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ExecutionPrimitive>mapC("getExecution", PrimitiveEvent::getExecution).<ExecutionState>map("getAfter", ExecutionPrimitive::getAfter).<Execution>map("getExecution", ExecutionState::getExecution).getMulti());
		;
		return executionHolder;
	}
	
}
