package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.math.BigDecimalExtensions;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.functions.DayCountFraction;


public class FixedAmount implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected DayCountFraction dayCountFraction;

	/**
	* @param interestRatePayout 
	* @param date 
	* @return fixedAmount 
	*/
	public BigDecimal evaluate(InterestRatePayout interestRatePayout, Date date) {
		
		BigDecimal fixedAmount = doEvaluate(interestRatePayout, date);
		
		return fixedAmount;
	}
	
	protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, Date date) {
		Mapper<BigDecimal> fixedAmountHolder = null;
		fixedAmountHolder = MapperS.of(BigDecimalExtensions.multiply(BigDecimalExtensions.multiply(calculationAmount(interestRatePayout, date).get(), fixedRate(interestRatePayout, date).get()), dayCountFraction(interestRatePayout, date).get()));
		return fixedAmountHolder.get();
	}
	
	
	protected Mapper<BigDecimal> calculationAmount(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue);
	}
	
	protected Mapper<BigDecimal> fixedRate(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(interestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate).<BigDecimal>map("getInitialValue", Schedule::getInitialValue);
	}
	
	protected Mapper<BigDecimal> dayCountFraction(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(dayCountFraction.evaluate(MapperS.of(interestRatePayout).get(), MapperS.of(interestRatePayout).map("getDayCountFraction", InterestRatePayout::getDayCountFraction).<DayCountFractionEnum>map("getValue", FieldWithMeta::getValue).get(), MapperS.of(date).get()));
	}
}
