package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.math.BigDecimalExtensions;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.FloatingRateIndexEnum;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.functions.DayCountFraction;
import org.isda.cdm.functions.GetRateSchedule;
import org.isda.cdm.functions.ResolveRateIndex;


public class FloatingAmount implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected DayCountFraction dayCountFraction;
	@Inject protected GetRateSchedule getRateSchedule;
	@Inject protected ResolveRateIndex resolveRateIndex;

	/**
	* @param interestRatePayout 
	* @param date 
	* @return floatingAmount 
	*/
	public BigDecimal evaluate(InterestRatePayout interestRatePayout, Date date) {
		
		BigDecimal floatingAmount = doEvaluate(interestRatePayout, date);
		
		return floatingAmount;
	}
	
	protected BigDecimal doEvaluate(InterestRatePayout interestRatePayout, Date date) {
		Mapper<BigDecimal> floatingAmountHolder = null;
		floatingAmountHolder = MapperS.of(BigDecimalExtensions.multiply(BigDecimalExtensions.multiply(calculationAmount(interestRatePayout, date).get(), (BigDecimalExtensions.add(floatingRate(interestRatePayout, date).get(), spread(interestRatePayout, date).get()))), dayCountFraction(interestRatePayout, date).get()));
		return floatingAmountHolder.get();
	}
	
	
	protected Mapper<BigDecimal> calculationAmount(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(interestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue);
	}
	
	protected Mapper<BigDecimal> floatingRate(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(resolveRateIndex.evaluate(MapperS.of(interestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).map("getFloatingRateIndex", FloatingRate::getFloatingRateIndex).<FloatingRateIndexEnum>map("getValue", FieldWithMeta::getValue).get()));
	}
	
	protected Mapper<BigDecimal> spread(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(getRateSchedule.evaluate(MapperS.of(interestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).get())).<BigDecimal>map("getInitialValue", Schedule::getInitialValue);
	}
	
	protected Mapper<BigDecimal> dayCountFraction(InterestRatePayout interestRatePayout, Date date) {
		return MapperS.of(dayCountFraction.evaluate(MapperS.of(interestRatePayout).get(), MapperS.of(interestRatePayout).map("getDayCountFraction", InterestRatePayout::getDayCountFraction).<DayCountFractionEnum>map("getValue", FieldWithMeta::getValue).get(), MapperS.of(date).get()));
	}
}
