package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractFormation;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.Event;
import org.isda.cdm.EventEffect;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionPrimitive;
import org.isda.cdm.ExecutionState;
import org.isda.cdm.LegalAgreement;
import org.isda.cdm.LegalAgreementBase;
import org.isda.cdm.Party;
import org.isda.cdm.PostInceptionState;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.Product;
import org.isda.cdm.ProductIdentifier;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.functions.NewContractFormationPrimitive;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.contains;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class FormContract implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewContractFormationPrimitive newContractFormationPrimitive;

	/**
	* @param executionEvent 
	* @param legalAgreement 
	* @return contractFormationEvent 
	*/
	public Event evaluate(Event executionEvent, LegalAgreement legalAgreement) {
		// pre-conditions
		
			assert
				exists(MapperS.of(execution(executionEvent, legalAgreement).get()), false).and(exists(MapperS.of(execution(executionEvent, legalAgreement).get()).<Product>map("getProduct", Execution::getProduct).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct), false)).get()
				: "Input event must be an execution must be on a contractual product.";
		
			assert
				doIf(exists(MapperS.of(legalAgreement), false),contains(MapperS.of(legalAgreement).mapC("getContractualParty", LegalAgreementBase::getContractualParty).<Party>map("getValue", FieldWithMeta::getValue), MapperS.of(execution(executionEvent, legalAgreement).get()).mapC("getParty", Execution::getParty).<Party>map("getValue", FieldWithMeta::getValue))).get()
				: "Ensures that parties to the Contract are defined in the Legal Agreement, if one is provided.";
		
		Event contractFormationEvent = doEvaluate(executionEvent, legalAgreement).build();
		
		// post-conditions

		assert
			areEqual(MapperS.of(contractFormationEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getContract", EventEffect::getContract).<Contract>map("getValue", FieldWithMeta::getValue), MapperS.of(contractFormationEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ContractFormation>mapC("getContractFormation", PrimitiveEvent::getContractFormation).<PostInceptionState>map("getAfter", ContractFormation::getAfter).<Contract>map("getContract", ContractState::getContract)).and(areEqual(MapperS.of(contractFormationEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getEffectedExecution", EventEffect::getEffectedExecution).<Execution>map("getValue", FieldWithMeta::getValue), MapperS.of(contractFormationEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ContractFormation>mapC("getContractFormation", PrimitiveEvent::getContractFormation).<ExecutionState>map("getBefore", ContractFormation::getBefore).<Execution>map("getExecution", ExecutionState::getExecution))).and(notExists(MapperS.of(contractFormationEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getEffectedContract", EventEffect::getEffectedContract).<Contract>map("getValue", FieldWithMeta::getValue))).and(notExists(MapperS.of(contractFormationEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getProductIdentifier", EventEffect::getProductIdentifier).<ProductIdentifier>map("getValue", FieldWithMeta::getValue))).and(notExists(MapperS.of(contractFormationEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getTransfer", EventEffect::getTransfer).<TransferPrimitive>map("getValue", FieldWithMeta::getValue))).get()
			: "Event effect must point to the new execution and contract, and exclude other effects.";
		objectValidator.validateAndFailOnErorr(Event.class, contractFormationEvent);
		return contractFormationEvent;
	}
	
	protected Event.EventBuilder doEvaluate(Event executionEvent, LegalAgreement legalAgreement) {
		Event.EventBuilder contractFormationEventHolder = Event.builder();
		@SuppressWarnings("unused") Event contractFormationEvent = contractFormationEventHolder.build();
		contractFormationEventHolder
			.getOrCreatePrimitive()
			.addContractFormation(MapperS.of(newContractFormationPrimitive.evaluate(MapperS.of(executionEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ExecutionPrimitive>mapC("getExecution", PrimitiveEvent::getExecution).<ExecutionState>map("getAfter", ExecutionPrimitive::getAfter).get(), MapperS.of(execution(executionEvent, legalAgreement).get()).mapC("getParty", Execution::getParty).<Party>map("getValue", FieldWithMeta::getValue).getMulti(), MapperS.of(legalAgreement).get())).get());
		;
		contractFormationEvent = contractFormationEventHolder.build();
		contractFormationEventHolder
			.addParty(MapperS.of(execution(executionEvent, legalAgreement).get()).mapC("getParty", Execution::getParty).<Party>map("getValue", FieldWithMeta::getValue).getMulti());
		;
		contractFormationEvent = contractFormationEventHolder.build();
		contractFormationEventHolder
			.getOrCreateLineage()
			.addEventReferenceRef(MapperS.of(executionEvent).get());
		;
		return contractFormationEventHolder;
	}
	
	
	protected MapperBuilder<Execution> execution(Event executionEvent, LegalAgreement legalAgreement) {
		return MapperS.of(executionEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ExecutionPrimitive>mapC("getExecution", PrimitiveEvent::getExecution).<ExecutionState>map("getAfter", ExecutionPrimitive::getAfter).<Execution>map("getExecution", ExecutionState::getExecution);
	}
}
