package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.math.BigDecimalExtensions;
import java.math.BigDecimal;
import org.isda.cdm.Cashflow;
import org.isda.cdm.ExchangeRate;
import org.isda.cdm.ForeignExchange;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.Money;
import org.isda.cdm.Product;
import org.isda.cdm.QuoteBasisEnum;
import org.isda.cdm.QuotedCurrencyPair;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.functions.InterpolateForwardRate;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;

public class FxMarkToMarket implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected InterpolateForwardRate interpolateForwardRate;

	/**
	* @param forwardPayout 
	* @return value 
	*/
	public BigDecimal evaluate(ForwardPayout forwardPayout) {
		
		BigDecimal value = doEvaluate(forwardPayout);
		
		return value;
	}
	
	protected BigDecimal doEvaluate(ForwardPayout forwardPayout) {
		Mapper<BigDecimal> valueHolder = null;
		valueHolder = MapperS.of(BigDecimalExtensions.multiply((BigDecimalExtensions.subtract(BigDecimalExtensions.divide(quotedQuantity(forwardPayout).get(), interpolatedRate(forwardPayout).get()), baseQuantity(forwardPayout).get())), interpolatedRate(forwardPayout).get()));
		return valueHolder.get();
	}
	
	
	protected MapperBuilder<ForeignExchange> forwardFX(ForwardPayout forwardPayout) {
		return MapperS.of(forwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange);
	}
	
	protected Mapper<BigDecimal> quotedQuantity(ForwardPayout forwardPayout) {
		return doIf(areEqual(MapperS.of(forwardFX(forwardPayout).get()).<ExchangeRate>map("getExchangeRate", ForeignExchange::getExchangeRate).<QuotedCurrencyPair>map("getQuotedCurrencyPair", ExchangeRate::getQuotedCurrencyPair).<QuoteBasisEnum>map("getQuoteBasis", QuotedCurrencyPair::getQuoteBasis), MapperS.of(QuoteBasisEnum.CURRENCY_2_PER_CURRENCY_1)),MapperS.of(forwardFX(forwardPayout).get()).<Cashflow>map("getExchangedCurrency2", ForeignExchange::getExchangedCurrency2).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).<BigDecimal>map("getAmount", Money::getAmount),MapperS.of(forwardFX(forwardPayout).get()).<Cashflow>map("getExchangedCurrency1", ForeignExchange::getExchangedCurrency1).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).<BigDecimal>map("getAmount", Money::getAmount));
	}
	
	protected Mapper<BigDecimal> baseQuantity(ForwardPayout forwardPayout) {
		return doIf(areEqual(MapperS.of(forwardFX(forwardPayout).get()).<ExchangeRate>map("getExchangeRate", ForeignExchange::getExchangeRate).<QuotedCurrencyPair>map("getQuotedCurrencyPair", ExchangeRate::getQuotedCurrencyPair).<QuoteBasisEnum>map("getQuoteBasis", QuotedCurrencyPair::getQuoteBasis), MapperS.of(QuoteBasisEnum.CURRENCY_2_PER_CURRENCY_1)),MapperS.of(forwardFX(forwardPayout).get()).<Cashflow>map("getExchangedCurrency1", ForeignExchange::getExchangedCurrency1).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).<BigDecimal>map("getAmount", Money::getAmount),MapperS.of(forwardFX(forwardPayout).get()).<Cashflow>map("getExchangedCurrency2", ForeignExchange::getExchangedCurrency2).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).<BigDecimal>map("getAmount", Money::getAmount));
	}
	
	protected Mapper<BigDecimal> interpolatedRate(ForwardPayout forwardPayout) {
		return MapperS.of(interpolateForwardRate.evaluate(MapperS.of(forwardPayout).get()));
	}
}
