package org.isda.cdm.functions;

import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import org.isda.cdm.AllocationInstructions;
import org.isda.cdm.AllocationPrimitive;
import org.isda.cdm.Execution;


@ImplementedBy(NewAllocationPrimitiveImpl.class)
public abstract class NewAllocationPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param execution Block trade to be allocated.
	* @param allocationInstructions Allocation instructions to be applied.
	* @return primitive Allocation primitive containing both the original block trade (with the status set to Allocated) and the set of allocated trades.
	*/
	public AllocationPrimitive evaluate(Execution execution, AllocationInstructions allocationInstructions) {
		
		AllocationPrimitive primitive = doEvaluate(execution, allocationInstructions).build();
		
		objectValidator.validateAndFailOnErorr(AllocationPrimitive.class, primitive);
		return primitive;
	}
	
	protected abstract AllocationPrimitive.AllocationPrimitiveBuilder doEvaluate(Execution execution, AllocationInstructions allocationInstructions);
}
