package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.Cashflow;
import org.isda.cdm.ContractState;
import org.isda.cdm.Money;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.TransferStatusEnum;
import org.isda.cdm.functions.ResolveCashflow;


public class NewCashTransferPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ResolveCashflow resolveCashflow;

	/**
	* @param contractState 
	* @param date 
	* @return transfer 
	*/
	public TransferPrimitive evaluate(ContractState contractState, Date date) {
		
		TransferPrimitive transfer = doEvaluate(contractState, date).build();
		
		objectValidator.validateAndFailOnErorr(TransferPrimitive.class, transfer);
		return transfer;
	}
	
	protected TransferPrimitive.TransferPrimitiveBuilder doEvaluate(ContractState contractState, Date date) {
		TransferPrimitive.TransferPrimitiveBuilder transferHolder = TransferPrimitive.builder();
		@SuppressWarnings("unused") TransferPrimitive transfer = transferHolder.build();
		transferHolder
			.setStatus(MapperS.of(TransferStatusEnum.INSTRUCTED).get());
		;
		transfer = transferHolder.build();
		transferHolder
			.getOrCreateCashTransfer(0)
			.setAmount(MapperS.of(cashflow(contractState, date).get()).<Money>map("getCashflowAmount", Cashflow::getCashflowAmount).get());
		;
		transfer = transferHolder.build();
		transferHolder
			.getOrCreateCashTransfer(0)
			.setPayerReceiver(MapperS.of(cashflow(contractState, date).get()).<PayerReceiver>map("getPayerReceiver", Cashflow::getPayerReceiver).get());
		;
		transfer = transferHolder.build();
		transferHolder
			.setSettlementDate(MapperS.of(cashflow(contractState, date).get()).<AdjustableOrAdjustedOrRelativeDate>map("getCashflowDate", Cashflow::getCashflowDate).get());
		;
		return transferHolder;
	}
	
	
	protected MapperBuilder<Cashflow> cashflow(ContractState contractState, Date date) {
		return MapperS.of(resolveCashflow.evaluate(MapperS.of(contractState).get(), MapperS.of(date).get()));
	}
}
