package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperC;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import java.util.List;
import org.isda.cdm.ContractFormation;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionState;
import org.isda.cdm.LegalAgreement;
import org.isda.cdm.Party;
import org.isda.cdm.Product;

import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

public class NewContractFormationPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param executionState 
	* @param parties 
	* @param legalAgreement 
	* @return contractFormation 
	*/
	public ContractFormation evaluate(ExecutionState executionState, List<Party> parties, LegalAgreement legalAgreement) {
		
		ContractFormation contractFormation = doEvaluate(executionState, parties, legalAgreement).build();
		
		objectValidator.validateAndFailOnErorr(ContractFormation.class, contractFormation);
		return contractFormation;
	}
	
	protected ContractFormation.ContractFormationBuilder doEvaluate(ExecutionState executionState, List<Party> parties, LegalAgreement legalAgreement) {
		ContractFormation.ContractFormationBuilder contractFormationHolder = ContractFormation.builder();
		@SuppressWarnings("unused") ContractFormation contractFormation = contractFormationHolder.build();
		contractFormationHolder
			.setBefore(MapperS.of(executionState).get());
		;
		contractFormation = contractFormationHolder.build();
		contractFormationHolder
			.getOrCreateAfter().getOrCreateContract()
			.setContractualProduct(MapperS.of(executionState).<Execution>map("getExecution", ExecutionState::getExecution).<Product>map("getProduct", Execution::getProduct).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct).get());
		;
		contractFormation = contractFormationHolder.build();
		contractFormationHolder
			.getOrCreateAfter().getOrCreateContract().getOrCreateDocumentation()
			.addLegalAgreementRef(doIf(exists(MapperS.of(legalAgreement), false),MapperS.of(legalAgreement)).get());
		;
		contractFormation = contractFormationHolder.build();
		contractFormationHolder
			.getOrCreateAfter().getOrCreateContract()
			.addParty(MapperC.of(parties).getMulti());
		;
		return contractFormationHolder;
	}
	
}
