package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Cashflow;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.Equity;
import org.isda.cdm.EquitySwapMasterConfirmation2018;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.functions.NewFloatingPayout;
import org.isda.cdm.functions.NewSingleNameEquityPayout;

import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class NewEquitySwapProduct implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewFloatingPayout newFloatingPayout;
	@Inject protected NewSingleNameEquityPayout newSingleNameEquityPayout;

	/**
	* @param underlier The underlying Equity asset for the swap.
	* @param masterConfirmation An (optional) pointer to the Master Confirmation Agreement, if any, that holds further inputs to the Equity Swap
	* @return product 
	*/
	public Product evaluate(Equity underlier, EquitySwapMasterConfirmation2018 masterConfirmation) {
		
		Product product = doEvaluate(underlier, masterConfirmation).build();
		
		// post-conditions

		assert
			doIf(notExists(MapperS.of(masterConfirmation)),notExists(MapperS.of(payout(product.toBuilder(), underlier, masterConfirmation).build()).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout)).and(notExists(MapperS.of(payout(product.toBuilder(), underlier, masterConfirmation).build()).<Cashflow>mapC("getCashflow", Payout::getCashflow))).and(notExists(MapperS.of(payout(product.toBuilder(), underlier, masterConfirmation).build()).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout))).and(notExists(MapperS.of(payout(product.toBuilder(), underlier, masterConfirmation).build()).<ForwardPayout>mapC("getForwardPayout", Payout::getForwardPayout))).and(notExists(MapperS.of(payout(product.toBuilder(), underlier, masterConfirmation).build()).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout)))).get()
			: "Other payout types must be absent.";

		assert
			exists(MapperS.of(product).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct), true).get()
			: "Non-contractual product types must be absent.";
		objectValidator.validateAndFailOnErorr(Product.class, product);
		return product;
	}
	
	protected Product.ProductBuilder doEvaluate(Equity underlier, EquitySwapMasterConfirmation2018 masterConfirmation) {
		Product.ProductBuilder productHolder = Product.builder();
		@SuppressWarnings("unused") Product product = productHolder.build();
		productHolder
			.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout()
			.addEquityPayout(MapperS.of(newSingleNameEquityPayout.evaluate(MapperS.of(underlier).get(), MapperS.of(masterConfirmation).get())).get());
		;
		product = productHolder.build();
		productHolder
			.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout()
			.addInterestRatePayout(doIf(exists(MapperS.of(masterConfirmation), false),MapperS.of(newFloatingPayout.evaluate(MapperS.of(masterConfirmation).get()))).get());
		;
		return productHolder;
	}
	
	
	protected Payout.PayoutBuilder payout(Product.ProductBuilder product, Equity underlier, EquitySwapMasterConfirmation2018 masterConfirmation) {
		return product.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout();
	}
}
