package org.isda.cdm.functions;

import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.EquitySwapMasterConfirmation2018;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.PaymentDates;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

@ImplementedBy(NewFloatingPayoutImpl.class)
public abstract class NewFloatingPayout implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param masterConfirmation 
	* @return interestRatePayout 
	*/
	public InterestRatePayout evaluate(EquitySwapMasterConfirmation2018 masterConfirmation) {
		
		InterestRatePayout interestRatePayout = doEvaluate(masterConfirmation).build();
		
		// post-conditions

		assert
			doIf(exists(MapperS.of(masterConfirmation), false),areEqual(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates), MapperS.of(masterConfirmation).<CalculationPeriodDates>map("getEquityCalculationPeriod", EquitySwapMasterConfirmation2018::getEquityCalculationPeriod)).and(areEqual(MapperS.of(interestRatePayout).<PaymentDates>map("getPaymentDates", InterestRatePayout::getPaymentDates), MapperS.of(masterConfirmation).<PaymentDates>map("getEquityCashSettlementDates", EquitySwapMasterConfirmation2018::getEquityCashSettlementDates)))).get()
			: "Interest rate payout must inherit terms from the Master Confirmation Agreement when it exists.";
		objectValidator.validateAndFailOnErorr(InterestRatePayout.class, interestRatePayout);
		return interestRatePayout;
	}
	
	protected abstract InterestRatePayout.InterestRatePayoutBuilder doEvaluate(EquitySwapMasterConfirmation2018 masterConfirmation);
}
