package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.Execution;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.Trade;
import org.isda.cdm.functions.ExtractQuantity;
import org.isda.cdm.functions.GreaterThan;
import org.isda.cdm.functions.Plus;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

public class NewQuantityChangePrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ExtractQuantity extractQuantity;
	@Inject protected GreaterThan greaterThan;
	@Inject protected Plus plus;

	/**
	* @param trade 
	* @param quantityChange 
	* @return quantityChangePrimitive 
	*/
	public QuantityChangePrimitive evaluate(Trade trade, ContractualQuantity quantityChange) {
		// pre-conditions
		
			assert
				MapperS.of(greaterThan.evaluate(MapperS.of(quantityChange).get(), MapperS.of(BigDecimal.valueOf(0.0)).get())).get()
				: "Non-zero quantity change";
		
		QuantityChangePrimitive quantityChangePrimitive = doEvaluate(trade, quantityChange).build();
		
		// post-conditions

		assert
			areEqual(MapperS.of(extractQuantity.evaluate(MapperS.of(quantityChangePrimitive).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).get())), MapperS.of(plus.evaluate(MapperS.of(extractQuantity.evaluate(MapperS.of(trade).get())).get(), MapperS.of(quantityChange).get()))).get()
			: "The resulting quantity must equal the original quantity plus the quantity change.";

		assert
			doIf(exists(MapperS.of(quantityChangePrimitive).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution), false),exists(MapperS.of(quantityChangePrimitive).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Execution>map("getExecution", Trade::getExecution), false).and(doIf(exists(MapperS.of(quantityChangePrimitive).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract), false),exists(MapperS.of(quantityChangePrimitive).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract), false)))).get()
			: "The input and output types need to be consistent, if we started with an Execution, we should end with an Execution, likewise with Contract.";
		objectValidator.validateAndFailOnErorr(QuantityChangePrimitive.class, quantityChangePrimitive);
		return quantityChangePrimitive;
	}
	
	protected QuantityChangePrimitive.QuantityChangePrimitiveBuilder doEvaluate(Trade trade, ContractualQuantity quantityChange) {
		QuantityChangePrimitive.QuantityChangePrimitiveBuilder quantityChangePrimitiveHolder = QuantityChangePrimitive.builder();
		@SuppressWarnings("unused") QuantityChangePrimitive quantityChangePrimitive = quantityChangePrimitiveHolder.build();
		quantityChangePrimitiveHolder
			.setBefore(MapperS.of(trade).get());
		;
		return quantityChangePrimitiveHolder;
	}
	
}
