package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.ResetPrimitive;
import org.isda.cdm.functions.ResolveUpdatedContract;


public class NewResetPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ResolveUpdatedContract resolveUpdatedContract;

	/**
	* @param contractState 
	* @param observation 
	* @param date 
	* @return resetPrimitive 
	*/
	public ResetPrimitive evaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		
		ResetPrimitive resetPrimitive = doEvaluate(contractState, observation, date).build();
		
		objectValidator.validateAndFailOnErorr(ResetPrimitive.class, resetPrimitive);
		return resetPrimitive;
	}
	
	protected ResetPrimitive.ResetPrimitiveBuilder doEvaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		ResetPrimitive.ResetPrimitiveBuilder resetPrimitiveHolder = ResetPrimitive.builder();
		@SuppressWarnings("unused") ResetPrimitive resetPrimitive = resetPrimitiveHolder.build();
		resetPrimitiveHolder
			.setBefore(MapperS.of(contractState).get());
		;
		resetPrimitive = resetPrimitiveHolder.build();
		resetPrimitiveHolder
			.getOrCreateAfter()
			.setContract(MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract).get());
		;
		resetPrimitive = resetPrimitiveHolder.build();
		resetPrimitiveHolder
			.getOrCreateAfter()
			.setUpdatedContract(MapperS.of(resolveUpdatedContract.evaluate(MapperS.of(contractState).get(), MapperS.of(observation).get(), MapperS.of(date).get())).get());
		;
		return resetPrimitiveHolder;
	}
	
	
	protected MapperBuilder<Contract> contract(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract);
	}
}
