package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.Equity;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.EquitySwapMasterConfirmation2018;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Product;
import org.isda.cdm.ReturnTypeEnum;
import org.isda.cdm.Security;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;

public class NewSingleNameEquityPayout implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param underlier 
	* @param masterConfirmation 
	* @return equityPayout 
	*/
	public EquityPayout evaluate(Equity underlier, EquitySwapMasterConfirmation2018 masterConfirmation) {
		
		EquityPayout equityPayout = doEvaluate(underlier, masterConfirmation).build();
		
		// post-conditions

		assert
			areEqual(MapperS.of(equityPayout).<Underlier>map("getUnderlier", EquityPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Equity>map("getEquity", Security::getEquity), MapperS.of(underlier)).get()
			: "Equity payout must be single name on the provided equity underlier.";
		objectValidator.validateAndFailOnErorr(EquityPayout.class, equityPayout);
		return equityPayout;
	}
	
	protected EquityPayout.EquityPayoutBuilder doEvaluate(Equity underlier, EquitySwapMasterConfirmation2018 masterConfirmation) {
		EquityPayout.EquityPayoutBuilder equityPayoutHolder = EquityPayout.builder();
		@SuppressWarnings("unused") EquityPayout equityPayout = equityPayoutHolder.build();
		equityPayoutHolder
			.setReturnType(MapperS.of(masterConfirmation).<ReturnTypeEnum>map("getTypeOfSwapElection", EquitySwapMasterConfirmation2018::getTypeOfSwapElection).get());
		;
		equityPayout = equityPayoutHolder.build();
		equityPayoutHolder
			.setCalculationPeriodDates(MapperS.of(masterConfirmation).<CalculationPeriodDates>map("getEquityCalculationPeriod", EquitySwapMasterConfirmation2018::getEquityCalculationPeriod).get());
		;
		equityPayout = equityPayoutHolder.build();
		equityPayoutHolder
			.setPaymentDates(MapperS.of(masterConfirmation).<PaymentDates>map("getEquityCashSettlementDates", EquitySwapMasterConfirmation2018::getEquityCashSettlementDates).get());
		;
		equityPayout = equityPayoutHolder.build();
		equityPayoutHolder
			.setSettlementTerms(MapperS.of(masterConfirmation).<SettlementTerms>map("getSettlementTerms", EquitySwapMasterConfirmation2018::getSettlementTerms).get());
		;
		return equityPayoutHolder;
	}
	
}
