package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.CashTransferComponent;
import org.isda.cdm.CommodityTransferComponent;
import org.isda.cdm.Execution;
import org.isda.cdm.Product;
import org.isda.cdm.Security;
import org.isda.cdm.SecurityTransferComponent;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.TransferSettlementEnum;
import org.isda.cdm.TransferStatusEnum;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class NewTransferPrimitive implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param execution 
	* @return transferPrimitive Transfer primitive generated from execution.
	*/
	public TransferPrimitive evaluate(Execution execution) {
		
		TransferPrimitive transferPrimitive = doEvaluate(execution).build();
		
		// post-conditions

		assert
			doIf(exists(MapperS.of(execution).<Product>map("getProduct", Execution::getProduct).<Security>map("getSecurity", Product::getSecurity), false).and(areEqual(MapperS.of(execution).<SettlementTerms>map("getSettlementTerms", Execution::getSettlementTerms).<TransferSettlementEnum>map("getTransferSettlementType", SettlementTerms::getTransferSettlementType), MapperS.of(TransferSettlementEnum.DELIVERY_VERSUS_PAYMENT))),exists(MapperS.of(transferPrimitive).<CashTransferComponent>mapC("getCashTransfer", TransferPrimitive::getCashTransfer), false).and(exists(MapperS.of(transferPrimitive).<SecurityTransferComponent>mapC("getSecurityTransfer", TransferPrimitive::getSecurityTransfer), false)).and(notExists(MapperS.of(transferPrimitive).<CommodityTransferComponent>mapC("getCommodityTransfer", TransferPrimitive::getCommodityTransfer)))).get()
			: "";
		objectValidator.validateAndFailOnErorr(TransferPrimitive.class, transferPrimitive);
		return transferPrimitive;
	}
	
	protected TransferPrimitive.TransferPrimitiveBuilder doEvaluate(Execution execution) {
		TransferPrimitive.TransferPrimitiveBuilder transferPrimitiveHolder = TransferPrimitive.builder();
		@SuppressWarnings("unused") TransferPrimitive transferPrimitive = transferPrimitiveHolder.build();
		transferPrimitiveHolder
			.setStatus(MapperS.of(TransferStatusEnum.INSTRUCTED).get());
		;
		return transferPrimitiveHolder;
	}
	
}
