package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Cashflow;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.functions.EquityCashSettlementAmount;
import org.isda.cdm.functions.ResolveCashSettlementDate;

import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

public class ResolveCashflow implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected EquityCashSettlementAmount equityCashSettlementAmount;
	@Inject protected ResolveCashSettlementDate resolveCashSettlementDate;

	/**
	* @param contractState 
	* @param date 
	* @return cashflow 
	*/
	public Cashflow evaluate(ContractState contractState, Date date) {
		
		Cashflow cashflow = doEvaluate(contractState, date).build();
		
		objectValidator.validateAndFailOnErorr(Cashflow.class, cashflow);
		return cashflow;
	}
	
	protected Cashflow.CashflowBuilder doEvaluate(ContractState contractState, Date date) {
		Cashflow.CashflowBuilder cashflowHolder = Cashflow.builder();
		@SuppressWarnings("unused") Cashflow cashflow = cashflowHolder.build();
		cashflowHolder
			.setCashflowAmount(doIf(exists(MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout), false),MapperS.of(equityCashSettlementAmount.evaluate(MapperS.of(contractState).get(), MapperS.of(date).get()))).get());
		;
		cashflow = cashflowHolder.build();
		cashflowHolder
			.getOrCreateCashflowDate()
			.setAdjustedDateRef(MapperS.of(resolveCashSettlementDate.evaluate(MapperS.of(contractState).get())).get());
		;
		return cashflowHolder;
	}
	
}
