package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperMaths;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.Boolean;
import java.lang.SuppressWarnings;
import java.math.BigDecimal;
import org.isda.cdm.CalculationPeriodData;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.Payout;
import org.isda.cdm.functions.CalculationPeriod;
import org.isda.cdm.functions.EquityPerformance;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.doIf;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

public class ResolveEquityContract implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected CalculationPeriod calculationPeriod;
	@Inject protected EquityPerformance equityPerformance;

	/**
	* @param contractState 
	* @param observation 
	* @param date 
	* @return updatedContract 
	*/
	public Contract evaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		// pre-conditions
		
			assert
				exists(MapperS.of(equityPayout(contractState, observation, date).get()), false).get()
				: "";
		
		Contract updatedContract = doEvaluate(contractState, observation, date).build();
		
		objectValidator.validateAndFailOnErorr(Contract.class, updatedContract);
		return updatedContract;
	}
	
	protected Contract.ContractBuilder doEvaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		Contract.ContractBuilder updatedContractHolder = Contract.builder();
		@SuppressWarnings("unused") Contract updatedContract = updatedContractHolder.build();
		updatedEquityPayout(updatedContractHolder, contractState, observation, date)
			.getOrCreatePriceReturnTerms().getOrCreateValuationPriceFinal().getOrCreateNetPrice()
			.setAmount(doIf(MapperS.of(calculationPeriod.evaluate(MapperS.of(equityPayout(contractState, observation, date).get()).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).get(), MapperS.of(periodEndDate(contractState, observation, date).get()).get())).<Boolean>map("getIsLastPeriod", CalculationPeriodData::getIsLastPeriod),MapperS.of(price(contractState, observation, date).get())).get());
		;
		updatedContract = updatedContractHolder.build();
		updatedEquityPayout(updatedContractHolder, contractState, observation, date)
			.getOrCreatePriceReturnTerms().getOrCreateValuationPriceInterim().getOrCreateNetPrice()
			.setAmount(doIf(areEqual(MapperS.of(calculationPeriod.evaluate(MapperS.of(equityPayout(contractState, observation, date).get()).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).get(), MapperS.of(periodEndDate(contractState, observation, date).get()).get())).<Boolean>map("getIsLastPeriod", CalculationPeriodData::getIsLastPeriod), MapperS.of(Boolean.valueOf(false))),MapperS.of(price(contractState, observation, date).get())).get());
		;
		updatedContract = updatedContractHolder.build();
		updatedContractHolder
			.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout().getOrCreateEquityPayout(0)
			.setPerformance(MapperS.of(equityPerformance(contractState, observation, date).get()).get());
		;
		updatedContract = updatedContractHolder.build();
		updatedContractHolder
			.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout().getOrCreateEquityPayout(0).getOrCreatePayoutQuantity().getOrCreateQuantityMultiplier()
			.setMultiplierValue(MapperMaths.<BigDecimal, Integer, BigDecimal>add(MapperS.of(Integer.valueOf(1)), MapperMaths.<BigDecimal, BigDecimal, Integer>divide(MapperS.of(equityPerformance(contractState, observation, date).get()), MapperS.of(Integer.valueOf(100)))).get());
		;
		return updatedContractHolder;
	}
	
	
	protected Mapper<BigDecimal> price(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(observation).<BigDecimal>map("getObservation", ObservationPrimitive::getObservation);
	}
	
	protected MapperBuilder<EquityPayout> equityPayout(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout);
	}
	
	protected EquityPayout.EquityPayoutBuilder updatedEquityPayout(Contract.ContractBuilder updatedContract, ContractState contractState, ObservationPrimitive observation, Date date) {
		return updatedContract.getOrCreateContractualProduct().getOrCreateEconomicTerms().getOrCreatePayout().getOrCreateEquityPayout(0);
	}
	
	protected Mapper<Date> periodEndDate(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(calculationPeriod.evaluate(MapperS.of(equityPayout(contractState, observation, date).get()).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).get(), MapperS.of(date).get())).<Date>map("getEndDate", CalculationPeriodData::getEndDate);
	}
	
	protected Mapper<BigDecimal> equityPerformance(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(equityPerformance.evaluate(MapperS.of(equityPayout(contractState, observation, date).get()).get(), MapperS.of(observation).<BigDecimal>map("getObservation", ObservationPrimitive::getObservation).get(), MapperS.of(periodEndDate(contractState, observation, date).get()).get()));
	}
}
