package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.Mapper;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;
import org.isda.cdm.CalculationPeriodData;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.functions.CalculationPeriod;


public class ResolveEquityPeriodEndPrice implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected CalculationPeriod calculationPeriod;

	/**
	* @param equityPayout 
	* @param date 
	* @return price 
	*/
	public BigDecimal evaluate(EquityPayout equityPayout, Date date) {
		
		BigDecimal price = doEvaluate(equityPayout, date);
		
		return price;
	}
	
	protected BigDecimal doEvaluate(EquityPayout equityPayout, Date date) {
		Mapper<BigDecimal> priceHolder = null;
		priceHolder = MapperS.of(calculationPeriod(equityPayout, date).get().toBuilder().getIsLastPeriod() ? priceReturnTerms(equityPayout, date).get().toBuilder().getOrCreateValuationPriceFinal().getOrCreateNetPrice().getAmount() : priceReturnTerms(equityPayout, date).get().toBuilder().getOrCreateValuationPriceInterim().getOrCreateNetPrice().getAmount());
		return priceHolder.get();
	}
	
	
	protected MapperBuilder<PriceReturnTerms> priceReturnTerms(EquityPayout equityPayout, Date date) {
		return MapperS.of(equityPayout).<PriceReturnTerms>map("getPriceReturnTerms", EquityPayout::getPriceReturnTerms);
	}
	
	protected MapperBuilder<CalculationPeriodData> calculationPeriod(EquityPayout equityPayout, Date date) {
		return MapperS.of(calculationPeriod.evaluate(MapperS.of(equityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).get(), MapperS.of(date).get()));
	}
}
