package org.isda.cdm.functions;

import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import org.isda.cdm.Quantity;
import org.isda.cdm.ResolvablePayoutQuantity;


@ImplementedBy(ResolveQuantityImpl.class)
public abstract class ResolveQuantity implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param resolvableQuantity The quantity to resolve.
	* @param date 
	* @return quantity The resolved quantity, containing just a number and (optionally) a unit or currency.
	*/
	public Quantity evaluate(ResolvablePayoutQuantity resolvableQuantity, Date date) {
		
		Quantity quantity = doEvaluate(resolvableQuantity, date).build();
		
		objectValidator.validateAndFailOnErorr(Quantity.class, quantity);
		return quantity;
	}
	
	protected abstract Quantity.QuantityBuilder doEvaluate(ResolvablePayoutQuantity resolvableQuantity, Date date);
}
