package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.Payout;
import org.isda.cdm.functions.ResolveEquityContract;

import static com.rosetta.model.lib.functions.ExpressionOperators.exists;

public class ResolveUpdatedContract implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ResolveEquityContract resolveEquityContract;

	/**
	* @param contractState 
	* @param observation 
	* @param date 
	* @return updatedContract An object that follows the structure of the Contract with only the updated values are set.
	*/
	public Contract evaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		
		Contract updatedContract = doEvaluate(contractState, observation, date).build();
		
		objectValidator.validateAndFailOnErorr(Contract.class, updatedContract);
		return updatedContract;
	}
	
	protected Contract.ContractBuilder doEvaluate(ContractState contractState, ObservationPrimitive observation, Date date) {
		Contract.ContractBuilder updatedContractHolder = Contract.builder();
		@SuppressWarnings("unused") Contract updatedContract = updatedContractHolder.build();
		updatedContractHolder = exists(payout(contractState, observation, date).get().toBuilder().getOrCreateEquityPayout(0)) ? resolveEquityContract.evaluate(contractState,observation,date).toBuilder() : null
		;
		return updatedContractHolder;
	}
	
	
	protected MapperBuilder<Payout> payout(ContractState contractState, ObservationPrimitive observation, Date date) {
		return MapperS.of(contractState).<Contract>map("getContract", ContractState::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout);
	}
}
