package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Event;
import org.isda.cdm.Execution;
import org.isda.cdm.Product;
import org.isda.cdm.Security;
import org.isda.cdm.functions.NewTransferPrimitive;

import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

public class Settle implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewTransferPrimitive newTransferPrimitive;

	/**
	* @param execution Execution to be settled.
	* @param previousEvent Previous event for lineage purposes.
	* @return transferEvent Transfer event composed of cash and security components created from a security settled via delivery vs payment.
	*/
	public Event evaluate(Execution execution, Event previousEvent) {
		// pre-conditions
		
			assert
				exists(MapperS.of(execution).<Product>map("getProduct", Execution::getProduct).<Security>map("getSecurity", Product::getSecurity), false).get()
				: "";
		
		Event transferEvent = doEvaluate(execution, previousEvent).build();
		
		objectValidator.validateAndFailOnErorr(Event.class, transferEvent);
		return transferEvent;
	}
	
	protected Event.EventBuilder doEvaluate(Execution execution, Event previousEvent) {
		Event.EventBuilder transferEventHolder = Event.builder();
		@SuppressWarnings("unused") Event transferEvent = transferEventHolder.build();
		transferEventHolder
			.getOrCreatePrimitive()
			.addTransfer(MapperS.of(newTransferPrimitive.evaluate(MapperS.of(execution).get())).get());
		;
		transferEvent = transferEventHolder.build();
		transferEventHolder
			.getOrCreateLineage()
			.addEventReferenceRef(MapperS.of(previousEvent).get());
		;
		transferEvent = transferEventHolder.build();
		transferEventHolder
			.getOrCreateLineage()
			.addExecutionReferenceRef(MapperS.of(execution).get());
		;
		return transferEventHolder;
	}
	
}
