package org.isda.cdm.functions;

import com.google.inject.Inject;
import com.rosetta.model.lib.functions.MapperBuilder;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ModelObjectValidator;
import java.lang.SuppressWarnings;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.Event;
import org.isda.cdm.EventEffect;
import org.isda.cdm.Execution;
import org.isda.cdm.Lineage;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProductIdentifier;
import org.isda.cdm.ResetPrimitive;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.functions.NewCashTransferPrimitive;

import static com.rosetta.model.lib.validation.ValidatorHelper.areEqual;
import static com.rosetta.model.lib.validation.ValidatorHelper.exists;
import static com.rosetta.model.lib.validation.ValidatorHelper.notExists;

public class TransferCash implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected NewCashTransferPrimitive newCashTransferPrimitive;

	/**
	* @param resetEvent 
	* @return transferEvent 
	*/
	public Event evaluate(Event resetEvent) {
		// pre-conditions
		
			assert
				exists(MapperS.of(resetEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ResetPrimitive>mapC("getReset", PrimitiveEvent::getReset), false).get()
				: "There must be a reset primitive on the event input, and the contract being referenced must be consistent between those events.";
		
		Event transferEvent = doEvaluate(resetEvent).build();
		
		// post-conditions

		assert
			exists(MapperS.of(transferEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer), true).get()
			: "Event must contain a transfer primitive populated with the cashflow, and no other primitive.";

		assert
			areEqual(MapperS.of(transferEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getEffectedContract", EventEffect::getEffectedContract).<Contract>map("getValue", FieldWithMeta::getValue), MapperS.of(contractState(resetEvent).get()).<Contract>map("getContract", ContractState::getContract)).and(areEqual(MapperS.of(transferEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getContract", EventEffect::getContract).<Contract>map("getValue", FieldWithMeta::getValue), MapperS.of(contractState(resetEvent).get()).<Contract>map("getContract", ContractState::getContract))).and(notExists(MapperS.of(transferEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getEffectedExecution", EventEffect::getEffectedExecution).<Execution>map("getValue", FieldWithMeta::getValue))).and(notExists(MapperS.of(transferEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getExecution", EventEffect::getExecution).<Execution>map("getValue", FieldWithMeta::getValue))).and(notExists(MapperS.of(transferEvent).<EventEffect>map("getEventEffect", Event::getEventEffect).mapC("getProductIdentifier", EventEffect::getProductIdentifier).<ProductIdentifier>map("getValue", FieldWithMeta::getValue))).get()
			: "Event effect must include the contract generating the cashflow.";

		assert
			areEqual(MapperS.of(transferEvent).<Lineage>map("getLineage", Event::getLineage).mapC("getContractReference", Lineage::getContractReference).<Contract>map("getValue", FieldWithMeta::getValue), MapperS.of(contractState(resetEvent).get()).<Contract>map("getContract", ContractState::getContract)).and(areEqual(MapperS.of(transferEvent).<Lineage>map("getLineage", Event::getLineage).mapC("getEventReference", Lineage::getEventReference).<Event>map("getValue", FieldWithMeta::getValue), MapperS.of(resetEvent))).get()
			: "Event lineage must point to the contract that generated the cashflow and the reset event.";
		objectValidator.validateAndFailOnErorr(Event.class, transferEvent);
		return transferEvent;
	}
	
	protected Event.EventBuilder doEvaluate(Event resetEvent) {
		Event.EventBuilder transferEventHolder = Event.builder();
		@SuppressWarnings("unused") Event transferEvent = transferEventHolder.build();
		transferEventHolder
			.getOrCreatePrimitive()
			.addTransfer(MapperS.of(newCashTransferPrimitive.evaluate(MapperS.of(contractState(resetEvent).get()).get(), MapperS.of(resetEvent).<Date>map("getEventDate", Event::getEventDate).get())).get());
		;
		return transferEventHolder;
	}
	
	
	protected MapperBuilder<ContractState> contractState(Event resetEvent) {
		return MapperS.of(resetEvent).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ResetPrimitive>mapC("getReset", PrimitiveEvent::getReset).<ContractState>map("getAfter", ResetPrimitive::getAfter);
	}
}
