package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.BusinessCenterTime;
import org.isda.cdm.CashSettlementTerms;
import org.isda.cdm.Money;
import org.isda.cdm.QuotationRateTypeEnum;
import org.isda.cdm.ValuationDate;
import org.isda.cdm.ValuationMethodEnum;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=CashSettlementTerms.class)
public class CashSettlementTermsMeta implements RosettaMetaData<CashSettlementTerms> {

	@Override
	public List<Validator<? super CashSettlementTerms>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.CashSettlementTermsCashSettlementBusinessDaysDataRule(),
			new org.isda.cdm.validation.datarule.CashSettlementTermsRecoveryFactorDataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd37DataRule()
		);
	}

	@Override
	public List<Validator<? super CashSettlementTerms>> choiceRuleValidators() {
		return Arrays.asList(
			new org.isda.cdm.validation.choicerule.CashSettlementTermsChoiceRule()
		);
	}

	@Override
	public List<Function<? super CashSettlementTerms, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super CashSettlementTerms> validator() {
		return new org.isda.cdm.validation.CashSettlementTermsValidator();
	}
	
	@Override
	public ValidatorWithArg<? super CashSettlementTerms, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.CashSettlementTermsOnlyExistsValidator();
	}
}
