package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.Account;
import org.isda.cdm.CalculationAgent;
import org.isda.cdm.ClosedState;
import org.isda.cdm.Collateral;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.Documentation;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.GoverningLawEnum;
import org.isda.cdm.Identifier;
import org.isda.cdm.Party;
import org.isda.cdm.PartyContractInformation;
import org.isda.cdm.PartyRole;
import org.isda.cdm.TradeDate;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=Contract.class)
public class ContractMeta implements RosettaMetaData<Contract> {

	@Override
	public List<Validator<? super Contract>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.DeliverableObligationsPhysicalSettlementMatrixDataRule(),
			new org.isda.cdm.validation.datarule.ObligationsPhysicalSettlementMatrixDataRule(),
			new org.isda.cdm.validation.datarule.CreditEventsPhysicalSettlementMatrixDataRule(),
			new org.isda.cdm.validation.datarule.RestructuringPhysicalSettlementMatrixDataRule(),
			new org.isda.cdm.validation.datarule.AdditionalFixedPaymentsMortgagesDataRule(),
			new org.isda.cdm.validation.datarule.FloatingAmountEventsMortgagesDataRule(),
			new org.isda.cdm.validation.datarule.CreditEventsMortgagesDataRule(),
			new org.isda.cdm.validation.datarule.ContractHedgingPartyDataRule(),
			new org.isda.cdm.validation.datarule.ContractDeterminingPartyDataRule(),
			new org.isda.cdm.validation.datarule.ContractBarrierDerterminationAgentDataRule(),
			new org.isda.cdm.validation.datarule.ContractClearedDateDataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd1DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd7DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd8DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd11DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd19DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd20DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd23DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd24DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd25DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd32DataRule()
		);
	}

	@Override
	public List<Validator<? super Contract>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super Contract, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super Contract> validator() {
		return new org.isda.cdm.validation.ContractValidator();
	}
	
	@Override
	public ValidatorWithArg<? super Contract, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.ContractOnlyExistsValidator();
	}
}
