package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.CashSettlementTerms;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.GeneralTerms;
import org.isda.cdm.PhysicalSettlementTerms;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.TransactedPrice;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=CreditDefaultPayout.class)
public class CreditDefaultPayoutMeta implements RosettaMetaData<CreditDefaultPayout> {

	@Override
	public List<Validator<? super CreditDefaultPayout>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.FpMLCd12DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd13DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd14DataRule()
		);
	}

	@Override
	public List<Validator<? super CreditDefaultPayout>> choiceRuleValidators() {
		return Arrays.asList(
			new org.isda.cdm.validation.choicerule.CreditDefaultPayoutChoiceRule()
		);
	}

	@Override
	public List<Function<? super CreditDefaultPayout, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super CreditDefaultPayout> validator() {
		return new org.isda.cdm.validation.CreditDefaultPayoutValidator();
	}
	
	@Override
	public ValidatorWithArg<? super CreditDefaultPayout, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.CreditDefaultPayoutOnlyExistsValidator();
	}
}
