package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.CancelableProvision;
import org.isda.cdm.EarlyTerminationProvision;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.ExtendibleProvision;
import org.isda.cdm.ExtraordinaryEvents;
import org.isda.cdm.Payout;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=EconomicTerms.class)
public class EconomicTermsMeta implements RosettaMetaData<EconomicTerms> {

	@Override
	public List<Validator<? super EconomicTerms>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.EconomicTermsExtraordinaryEventsDataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd2628DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd27DataRule(),
			new org.isda.cdm.validation.datarule.FpMLCd30DataRule()
		);
	}

	@Override
	public List<Validator<? super EconomicTerms>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super EconomicTerms, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
			new org.isda.cdm.qualify.product.IsCreditDefaultSwap(),
			new org.isda.cdm.qualify.product.IsCreditDefaultSwaption(),
			new org.isda.cdm.qualify.product.IsEquitySwapPriceReturnBasicPerformanceSingleName(),
			new org.isda.cdm.qualify.product.IsEquitySwapParameterReturnDividendSingleName(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapBasis(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapFixedFloatPlainVanilla(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapFixedFloat(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapFixedFixed(),
			new org.isda.cdm.qualify.product.IsInterestRateCrossCurrencyBasis(),
			new org.isda.cdm.qualify.product.IsInterestRateCrossCurrencyFixedFloat(),
			new org.isda.cdm.qualify.product.IsInterestRateCrossCurrencyFixedFixed(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapFixedFloatZeroCoupon(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapFixedFloatOIS(),
			new org.isda.cdm.qualify.product.IsInterestRateIRSwapBasisOIS(),
			new org.isda.cdm.qualify.product.IsInterestRateInflationSwapFixedFloatYearOnYear(),
			new org.isda.cdm.qualify.product.IsInterestRateInflationSwapFixedFloatZeroCoupon(),
			new org.isda.cdm.qualify.product.IsInterestRateInflationSwapBasisYearOnYear(),
			new org.isda.cdm.qualify.product.IsInterestRateInflationSwapBasisZeroCoupon(),
			new org.isda.cdm.qualify.product.IsInterestRateOptionSwaption(),
			new org.isda.cdm.qualify.product.IsInterestRateOptionDebtOption(),
			new org.isda.cdm.qualify.product.IsInterestRateCapFloor(),
			new org.isda.cdm.qualify.product.IsInterestRateFra(),
			new org.isda.cdm.qualify.product.IsForeignExchange(),
			new org.isda.cdm.qualify.product.IsForeignExchangeNDF(),
			new org.isda.cdm.qualify.product.IsRepurchaseAgreement()
		);
	}
	
	@Override
	public Validator<? super EconomicTerms> validator() {
		return new org.isda.cdm.validation.EconomicTermsValidator();
	}
	
	@Override
	public ValidatorWithArg<? super EconomicTerms, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.EconomicTermsOnlyExistsValidator();
	}
}
