package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.DividendReturnTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.PriceReturnTerms;
import org.isda.cdm.ReturnTypeEnum;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.Underlier;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=EquityPayout.class)
public class EquityPayoutMeta implements RosettaMetaData<EquityPayout> {

	@Override
	public List<Validator<? super EquityPayout>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.EquityPayoutDividendReturnDataRule(),
			new org.isda.cdm.validation.datarule.EquityPayoutPriceReturnDataRule(),
			new org.isda.cdm.validation.datarule.EquityPayoutTotalReturnDataRule(),
			new org.isda.cdm.validation.datarule.EquityPayoutSingleUnderlierDataRule()
		);
	}

	@Override
	public List<Validator<? super EquityPayout>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super EquityPayout, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super EquityPayout> validator() {
		return new org.isda.cdm.validation.EquityPayoutValidator();
	}
	
	@Override
	public ValidatorWithArg<? super EquityPayout, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.EquityPayoutOnlyExistsValidator();
	}
}
