package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.Account;
import org.isda.cdm.ActionEnum;
import org.isda.cdm.Event;
import org.isda.cdm.EventEffect;
import org.isda.cdm.EventTimestamp;
import org.isda.cdm.EventWorkflow;
import org.isda.cdm.Identifier;
import org.isda.cdm.IntentEnum;
import org.isda.cdm.Lineage;
import org.isda.cdm.MessageInformation;
import org.isda.cdm.Party;
import org.isda.cdm.PrimitiveEvent;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=Event.class)
public class EventMeta implements RosettaMetaData<Event> {

	@Override
	public List<Validator<? super Event>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.EventIntentDataRule()
		);
	}

	@Override
	public List<Validator<? super Event>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super Event, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
			new org.isda.cdm.qualify.event.IsAllocation(),
			new org.isda.cdm.qualify.event.IsCashflowCalculationNotification(),
			new org.isda.cdm.qualify.event.IsCashTransfer(),
			new org.isda.cdm.qualify.event.IsCashAndSecurityTransfer(),
			new org.isda.cdm.qualify.event.IsClearedTrade(),
			new org.isda.cdm.qualify.event.IsClearingRejection(),
			new org.isda.cdm.qualify.event.IsClearingSubmission(),
			new org.isda.cdm.qualify.event.IsCompression(),
			new org.isda.cdm.qualify.event.IsDerivedObservation(),
			new org.isda.cdm.qualify.event.IsExercise(),
			new org.isda.cdm.qualify.event.IsIncrease(),
			new org.isda.cdm.qualify.event.IsMultipleTransfers(),
			new org.isda.cdm.qualify.event.IsNewTrade(),
			new org.isda.cdm.qualify.event.IsNovation(),
			new org.isda.cdm.qualify.event.IsObservation(),
			new org.isda.cdm.qualify.event.IsPartialNovation(),
			new org.isda.cdm.qualify.event.IsPartialTermination(),
			new org.isda.cdm.qualify.event.IsReset(),
			new org.isda.cdm.qualify.event.IsSecurityTransfer(),
			new org.isda.cdm.qualify.event.IsSecuritySettlement(),
			new org.isda.cdm.qualify.event.IsTermination(),
			new org.isda.cdm.qualify.event.IsTradeWarehousePositionNotification()
		);
	}
	
	@Override
	public Validator<? super Event> validator() {
		return new org.isda.cdm.validation.EventValidator();
	}
	
	@Override
	public ValidatorWithArg<? super Event, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.EventOnlyExistsValidator();
	}
}
