package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.ClosedState;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.ExecutionTypeEnum;
import org.isda.cdm.Identifier;
import org.isda.cdm.LegalEntity;
import org.isda.cdm.Party;
import org.isda.cdm.PartyRole;
import org.isda.cdm.Price;
import org.isda.cdm.Product;
import org.isda.cdm.Quantity;
import org.isda.cdm.SettlementTerms;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=Execution.class)
public class ExecutionMeta implements RosettaMetaData<Execution> {

	@Override
	public List<Validator<? super Execution>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.ExecutionExecutionVenueDataRule(),
			new org.isda.cdm.validation.datarule.ExecutionSecurityPartyRoleDataRule(),
			new org.isda.cdm.validation.datarule.ExecutionSecurityPartyRoleBuyerSellerDataRule(),
			new org.isda.cdm.validation.datarule.ExecutionQuantityDataRule(),
			new org.isda.cdm.validation.datarule.ExecutionPriceDataRule(),
			new org.isda.cdm.validation.datarule.ExecutionSettlementTermsDataRule()
		);
	}

	@Override
	public List<Validator<? super Execution>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super Execution, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super Execution> validator() {
		return new org.isda.cdm.validation.ExecutionValidator();
	}
	
	@Override
	public ValidatorWithArg<? super Execution, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.ExecutionOnlyExistsValidator();
	}
}
