package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.BondReference;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CashflowRepresentation;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.DiscountingMethod;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.ResetDates;
import org.isda.cdm.StubPeriod;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=InterestRatePayout.class)
public class InterestRatePayoutMeta implements RosettaMetaData<InterestRatePayout> {

	@Override
	public List<Validator<? super InterestRatePayout>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.InterestRatePayoutFutureValueNotionalDataRule(),
			new org.isda.cdm.validation.datarule.FutureValueNotionalTerminationDateDataRule(),
			new org.isda.cdm.validation.datarule.InterestRatePayoutActualQuantityDataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd6DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd8DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd23DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd24DataRule(),
			new org.isda.cdm.validation.datarule.InitialStubFinalStubDataRule(),
			new org.isda.cdm.validation.datarule.SettlementProvisionSettlementCurrencyDataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd71DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd72DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd9DataRule(),
			new org.isda.cdm.validation.datarule.FpMLIrd29DataRule(),
			new org.isda.cdm.validation.datarule.CalculationPeriodDatesFirstCompoundingPeriodEndDateDataRule()
		);
	}

	@Override
	public List<Validator<? super InterestRatePayout>> choiceRuleValidators() {
		return Arrays.asList(
			new org.isda.cdm.validation.choicerule.InterestRatePayoutChoiceRule()
		);
	}

	@Override
	public List<Function<? super InterestRatePayout, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super InterestRatePayout> validator() {
		return new org.isda.cdm.validation.InterestRatePayoutValidator();
	}
	
	@Override
	public ValidatorWithArg<? super InterestRatePayout, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.InterestRatePayoutOnlyExistsValidator();
	}
}
