package org.isda.cdm.meta;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import org.isda.cdm.Cashflow;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.SecurityPayout;


/**
 * @version 2.5.4
 */
@RosettaMeta(model=Payout.class)
public class PayoutMeta implements RosettaMetaData<Payout> {

	@Override
	public List<Validator<? super Payout>> dataRules() {
		return Arrays.asList(
			new org.isda.cdm.validation.datarule.InterestRatePayoutQuantityDataRule(),
			new org.isda.cdm.validation.datarule.InterestRatePayoutDayCountFractionDataRule(),
			new org.isda.cdm.validation.datarule.InterestRatePayoutPaymentDatesDataRule(),
			new org.isda.cdm.validation.datarule.FinalStubLastRegularPaymentDateDataRule(),
			new org.isda.cdm.validation.datarule.PaymentDatesPayRelativeToDataRule(),
			new org.isda.cdm.validation.datarule.PaymentDatesPaymentDatesAdjustmentsDataRule(),
			new org.isda.cdm.validation.datarule.PaymentDatesPaymentFrequencyDataRule(),
			new org.isda.cdm.validation.datarule.TransactedPriceMarketPriceDataRule()
		);
	}

	@Override
	public List<Validator<? super Payout>> choiceRuleValidators() {
		return Arrays.asList(
		);
	}

	@Override
	public List<Function<? super Payout, QualifyResult>> getQualifyFunctions() {
		return Arrays.asList(
		);
	}
	
	@Override
	public Validator<? super Payout> validator() {
		return new org.isda.cdm.validation.PayoutValidator();
	}
	
	@Override
	public ValidatorWithArg<? super Payout, String> onlyExistsValidator() {
		return new org.isda.cdm.validation.exists.PayoutOnlyExistsValidator();
	}
}
