package org.isda.cdm.metafields;
	
import static java.util.Optional.ofNullable;
	
import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.ReferenceWithMetaBuilder;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;

public class BasicReferenceWithMetaString extends RosettaModelObject implements ReferenceWithMeta<String>{
	private final String globalReference;
	private final String externalReference;
	private final String value;
	
	private BasicReferenceWithMetaString(BasicReferenceWithMetaStringBuilder builder){
		value = builder.getValue();
		globalReference = builder.globalReference;
		externalReference = builder.externalReference;
	}
	
	public String getValue() {
		return value;
	}
	
	public String getGlobalReference() {
		return globalReference;
	}
	
	public String getExternalReference() {
		return externalReference;
	}
	
	private static BasicRosettaMetaData<BasicReferenceWithMetaString> metaData = new BasicRosettaMetaData<>();
	
	@Override
	public RosettaMetaData<? extends BasicReferenceWithMetaString> metaData() {
		return metaData;
	}
	
	public BasicReferenceWithMetaStringBuilder toBuilder() {
		BasicReferenceWithMetaStringBuilder builder = new BasicReferenceWithMetaStringBuilder();
		builder.setValue(value);
		builder.setGlobalReference(globalReference);
		builder.setExternalReference(externalReference);
		return builder;
	}
	
	public static BasicReferenceWithMetaStringBuilder builder() {
		return new BasicReferenceWithMetaStringBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("value"), String.class, value, this);
		processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((globalReference == null) ? 0 : globalReference.hashCode());
		_result = prime * _result + ((externalReference == null) ? 0 : externalReference.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		BasicReferenceWithMetaString other = (BasicReferenceWithMetaString) obj;
		if (globalReference != null ? !globalReference.equals(other.globalReference) : other.globalReference!=null) return false;
		if (externalReference != null ? !externalReference.equals(other.externalReference) : other.externalReference!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "BasicReferenceWitMeta {" +
			"globalReference=" + this.globalReference + ", " +
			"externalReference=" + this.externalReference + ", " +
			"value=" + this.value + ", " +
		'}';
	}
	
	public static class BasicReferenceWithMetaStringBuilder extends RosettaModelObjectBuilder implements ReferenceWithMetaBuilder<String>{
		private String value;
		private String globalReference;
		private String externalReference;
		
		public BasicReferenceWithMetaStringBuilder() {}
		
		@Override
		public RosettaMetaData<? extends BasicReferenceWithMetaString> metaData() {
			return metaData;
		}
		
		public String getValue() {
			return value;
		}
		
		public String getGlobalReference() {
			return globalReference;
		}
		
		public String getExternalReference() {
			return externalReference;
		}
		
		public BasicReferenceWithMetaStringBuilder setValue(String value) {
			this.value = value;
			return this;
		}
		
		public BasicReferenceWithMetaStringBuilder setGlobalReference(String reference) {
			this.globalReference = reference;
			return this;
		}
		
		public BasicReferenceWithMetaStringBuilder setExternalReference(String reference) {
			this.externalReference = reference;
			return this;
		}
		
		public BasicReferenceWithMetaString build() {
			return new BasicReferenceWithMetaString(this);
		}
		
		@Override
		public BasicReferenceWithMetaStringBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			return !(value==null && globalReference==null && externalReference==null);
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("value"), String.class, value, this);
			processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
		}
	}
}