package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.*;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;

public class FieldWithMetaCategoryEnum extends RosettaModelObject implements FieldWithMeta<CategoryEnum>{
	private final CategoryEnum value;
	private final MetaFields meta;
	private static BasicRosettaMetaData<FieldWithMetaCategoryEnum> metaData = new BasicRosettaMetaData<>();
				
	@Override
	public RosettaMetaData<? extends FieldWithMetaCategoryEnum> metaData() {
		return metaData;
	}
	
	private FieldWithMetaCategoryEnum(FieldWithMetaCategoryEnumBuilder builder) {
		value = builder.getValue();
		meta = ofNullable(builder.getMeta()).map(MetaFields.MetaFieldsBuilder::build).orElse(null);
	}
	
	public CategoryEnum getValue() {
		return value;
	}
	
	public MetaFields getMeta() {
		return meta;
	}
	
	public FieldWithMetaCategoryEnumBuilder toBuilder() {
		FieldWithMetaCategoryEnumBuilder builder = new FieldWithMetaCategoryEnumBuilder();
		builder.setValue(value);
		builder.setMeta(meta);
		return builder;
	}
	
	public static FieldWithMetaCategoryEnumBuilder builder() {
		return new FieldWithMetaCategoryEnumBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("value"), CategoryEnum.class, value, this);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		FieldWithMetaCategoryEnum other = (FieldWithMetaCategoryEnum) obj;
		if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "FieldWithMetaCategoryEnum {" +
			"value=" + this.value + ", " +
			"meta=" + this.meta + ", " +
		'}';
	}
	
	public static class FieldWithMetaCategoryEnumBuilder extends RosettaModelObjectBuilder{
		private CategoryEnum  value;
		private MetaFields.MetaFieldsBuilder meta;
		
		public FieldWithMetaCategoryEnumBuilder() {}
		
		@Override
		public RosettaMetaData<? extends FieldWithMetaCategoryEnum> metaData() {
			return metaData;
		}
		
		public CategoryEnum getValue() {
			return value;
		}
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			return meta=ofNullable(meta).orElseGet(MetaFields::builder);
		}
		
		public FieldWithMetaCategoryEnumBuilder setValue(CategoryEnum value) {
			this.value = value;
			return this;
		}
		
		public FieldWithMetaCategoryEnumBuilder setMeta(MetaFields meta) {
			this.meta = ofNullable(meta).map(MetaFields::toBuilder).orElse(null);
			return this;
		}
		
		public FieldWithMetaCategoryEnumBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public FieldWithMetaCategoryEnum build() {
			return new FieldWithMetaCategoryEnum(this);
		}
		
		@Override
		public FieldWithMetaCategoryEnumBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			return value!=null || value!=null;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("value"), CategoryEnum.class, value, this);
		}
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int _result = 1;
			_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
			_result = prime * _result + ((value == null) ? 0 : value.hashCode());
			return _result;
		}
	
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || getClass() != obj.getClass())
				return false;
			FieldWithMetaCategoryEnumBuilder other = (FieldWithMetaCategoryEnumBuilder) obj;
			if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
			if (value != null ? !value.equals(other.value) : other.value!=null) return false;
			return true;
		}
	
		@Override
		public String toString() {
			return "FieldWithMetaCategoryEnumBuilder {" +
				"value=" + this.value + ", " +
				"meta=" + this.meta + ", " +
			'}';
		}
	}
}
