package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.*;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;

public class FieldWithMetaIdentifier extends RosettaModelObject implements FieldWithMeta<Identifier>{
	private final Identifier value;
	private final MetaFields meta;
	private static BasicRosettaMetaData<FieldWithMetaIdentifier> metaData = new BasicRosettaMetaData<>();
				
	@Override
	public RosettaMetaData<? extends FieldWithMetaIdentifier> metaData() {
		return metaData;
	}
	
	private FieldWithMetaIdentifier(FieldWithMetaIdentifierBuilder builder) {
		value = ofNullable(builder.getValue()).map(v->v.build()).orElse(null);
		meta = ofNullable(builder.getMeta()).map(MetaFields.MetaFieldsBuilder::build).orElse(null);
	}
	
	public Identifier getValue() {
		return value;
	}
	
	public MetaFields getMeta() {
		return meta;
	}
	
	public FieldWithMetaIdentifierBuilder toBuilder() {
		FieldWithMetaIdentifierBuilder builder = new FieldWithMetaIdentifierBuilder();
		builder.setValue(value);
		builder.setMeta(meta);
		return builder;
	}
	
	public static FieldWithMetaIdentifierBuilder builder() {
		return new FieldWithMetaIdentifierBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
		processRosetta(path.newSubPath("value"), processor, Identifier.class, value);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		FieldWithMetaIdentifier other = (FieldWithMetaIdentifier) obj;
		if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "FieldWithMetaIdentifier {" +
			"value=" + this.value + ", " +
			"meta=" + this.meta + ", " +
		'}';
	}
	
	public static class FieldWithMetaIdentifierBuilder extends RosettaModelObjectBuilder{
		private Identifier.IdentifierBuilder  value;
		private MetaFields.MetaFieldsBuilder meta;
		
		public FieldWithMetaIdentifierBuilder() {}
		
		@Override
		public RosettaMetaData<? extends FieldWithMetaIdentifier> metaData() {
			return metaData;
		}
		
		public Identifier.IdentifierBuilder  getValue() {
			return value;
		}
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			return meta=ofNullable(meta).orElseGet(MetaFields::builder);
		}
		
		public FieldWithMetaIdentifierBuilder setValueBuilder(Identifier.IdentifierBuilder value) {
			this.value = value;
			return this;
		}
		
		public FieldWithMetaIdentifierBuilder setValue(Identifier value) {
			this.value = ofNullable(value).map(t->t.toBuilder()).orElse(null);
			return this;
		}
		
		public FieldWithMetaIdentifierBuilder setMeta(MetaFields meta) {
			this.meta = ofNullable(meta).map(MetaFields::toBuilder).orElse(null);
			return this;
		}
		
		public FieldWithMetaIdentifierBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public FieldWithMetaIdentifier build() {
			return new FieldWithMetaIdentifier(this);
		}
		
		@Override
		public FieldWithMetaIdentifierBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			return value!=null || value!=null;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
			processRosetta(path.newSubPath("value"), processor, Identifier.class, value);
		}
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int _result = 1;
			_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
			_result = prime * _result + ((value == null) ? 0 : value.hashCode());
			return _result;
		}
	
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || getClass() != obj.getClass())
				return false;
			FieldWithMetaIdentifierBuilder other = (FieldWithMetaIdentifierBuilder) obj;
			if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
			if (value != null ? !value.equals(other.value) : other.value!=null) return false;
			return true;
		}
	
		@Override
		public String toString() {
			return "FieldWithMetaIdentifierBuilder {" +
				"value=" + this.value + ", " +
				"meta=" + this.meta + ", " +
			'}';
		}
	}
}
