package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.FieldWithMeta;
import org.isda.cdm.*;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;
import com.rosetta.model.lib.records.Date;
import java.math.BigDecimal;

public class FieldWithMetaIndexAnnexSourceEnum extends RosettaModelObject implements FieldWithMeta<IndexAnnexSourceEnum>{
	private final IndexAnnexSourceEnum value;
	private final MetaFields meta;
	private static BasicRosettaMetaData<FieldWithMetaIndexAnnexSourceEnum> metaData = new BasicRosettaMetaData<>();
				
	@Override
	public RosettaMetaData<? extends FieldWithMetaIndexAnnexSourceEnum> metaData() {
		return metaData;
	}
	
	private FieldWithMetaIndexAnnexSourceEnum(FieldWithMetaIndexAnnexSourceEnumBuilder builder) {
		value = builder.getValue();
		meta = ofNullable(builder.getMeta()).map(MetaFields.MetaFieldsBuilder::build).orElse(null);
	}
	
	public IndexAnnexSourceEnum getValue() {
		return value;
	}
	
	public MetaFields getMeta() {
		return meta;
	}
	
	public FieldWithMetaIndexAnnexSourceEnumBuilder toBuilder() {
		FieldWithMetaIndexAnnexSourceEnumBuilder builder = new FieldWithMetaIndexAnnexSourceEnumBuilder();
		builder.setValue(value);
		builder.setMeta(meta);
		return builder;
	}
	
	public static FieldWithMetaIndexAnnexSourceEnumBuilder builder() {
		return new FieldWithMetaIndexAnnexSourceEnumBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("value"), IndexAnnexSourceEnum.class, value, this);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		FieldWithMetaIndexAnnexSourceEnum other = (FieldWithMetaIndexAnnexSourceEnum) obj;
		if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "FieldWithMetaIndexAnnexSourceEnum {" +
			"value=" + this.value + ", " +
			"meta=" + this.meta + ", " +
		'}';
	}
	
	public static class FieldWithMetaIndexAnnexSourceEnumBuilder extends RosettaModelObjectBuilder{
		private IndexAnnexSourceEnum  value;
		private MetaFields.MetaFieldsBuilder meta;
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder() {}
		
		@Override
		public RosettaMetaData<? extends FieldWithMetaIndexAnnexSourceEnum> metaData() {
			return metaData;
		}
		
		public IndexAnnexSourceEnum getValue() {
			return value;
		}
		
		public MetaFields.MetaFieldsBuilder getMeta() {
			return meta;
		}
		
		public MetaFields.MetaFieldsBuilder getOrCreateMeta() {
			return meta=ofNullable(meta).orElseGet(MetaFields::builder);
		}
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder setValue(IndexAnnexSourceEnum value) {
			this.value = value;
			return this;
		}
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder setMeta(MetaFields meta) {
			this.meta = ofNullable(meta).map(MetaFields::toBuilder).orElse(null);
			return this;
		}
		
		public FieldWithMetaIndexAnnexSourceEnumBuilder setMetaBuilder(MetaFields.MetaFieldsBuilder meta) {
			this.meta = meta;
			return this;
		}
		
		public FieldWithMetaIndexAnnexSourceEnum build() {
			return new FieldWithMetaIndexAnnexSourceEnum(this);
		}
		
		@Override
		public FieldWithMetaIndexAnnexSourceEnumBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			return value!=null || value!=null;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("meta"), processor, MetaFields.class, meta, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("value"), IndexAnnexSourceEnum.class, value, this);
		}
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int _result = 1;
			_result = prime * _result + ((meta == null) ? 0 : meta.hashCode());
			_result = prime * _result + ((value == null) ? 0 : value.hashCode());
			return _result;
		}
	
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || getClass() != obj.getClass())
				return false;
			FieldWithMetaIndexAnnexSourceEnumBuilder other = (FieldWithMetaIndexAnnexSourceEnumBuilder) obj;
			if (meta != null ? !meta.equals(other.meta) : other.meta!=null) return false;
			if (value != null ? !value.equals(other.value) : other.value!=null) return false;
			return true;
		}
	
		@Override
		public String toString() {
			return "FieldWithMetaIndexAnnexSourceEnumBuilder {" +
				"value=" + this.value + ", " +
				"meta=" + this.meta + ", " +
			'}';
		}
	}
}
