package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;

public class MetaFields extends RosettaModelObject implements com.rosetta.model.lib.meta.MetaFieldsI{
	private final java.lang.String scheme;
	private final java.lang.String globalKey;
	private final java.lang.String externalKey;
	private static BasicRosettaMetaData<MetaFields> metaData = new BasicRosettaMetaData<>();
	
	@Override
	public RosettaMetaData<? extends MetaFields> metaData() {
		return metaData;
	}
	
	private MetaFields(MetaFieldsBuilder builder) {
		scheme = builder.getScheme();
		globalKey = builder.getGlobalKey();
		externalKey = builder.getExternalKey();
	}
	
	public java.lang.String getScheme() {
		return scheme;
	}
	public java.lang.String getGlobalKey() {
		return globalKey;
	}
	public java.lang.String getExternalKey() {
		return externalKey;
	}
	
	public MetaFieldsBuilder toBuilder() {
		MetaFieldsBuilder builder = new MetaFieldsBuilder();
		ofNullable(getScheme()).ifPresent(builder::setScheme);
		ofNullable(getGlobalKey()).ifPresent(builder::setGlobalKey);
		ofNullable(getExternalKey()).ifPresent(builder::setExternalKey);
		return builder;
	}
	
	public static MetaFieldsBuilder builder() {
		return new MetaFieldsBuilder();
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((scheme == null) ? 0 : scheme.hashCode());
		_result = prime * _result + ((globalKey == null) ? 0 : globalKey.hashCode());
		_result = prime * _result + ((externalKey == null) ? 0 : externalKey.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		MetaFields other = (MetaFields) obj;
		if (scheme != null ? !scheme.equals(other.scheme) : other.scheme!=null) return false;
		if (globalKey != null ? !globalKey.equals(other.globalKey) : other.globalKey!=null) return false;
		if (externalKey != null ? !externalKey.equals(other.externalKey) : other.externalKey!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "MetaFields {" +
		"scheme=" + this.scheme + ", " +
		"globalKey=" + this.globalKey + ", " +
		"externalKey=" + this.externalKey + ", " +
		'}';
	}
	
	public static class MetaFieldsBuilder extends RosettaModelObjectBuilder implements com.rosetta.model.lib.meta.MetaFieldsI.MetaFieldsBuilderI{
		private java.lang.String scheme;
		private java.lang.String globalKey;
		private java.lang.String externalKey;
		
		@Override
		public RosettaMetaData<? extends MetaFields> metaData() {
			return metaData;
		}
		
		public java.lang.String getScheme() {
			return scheme;
		}
		public java.lang.String getGlobalKey() {
			return globalKey;
		}
		public java.lang.String getExternalKey() {
			return externalKey;
		}
		
		public MetaFieldsBuilder setScheme(java.lang.String scheme) {
			this.scheme = scheme;
			return this;
		}
		public MetaFieldsBuilder setGlobalKey(java.lang.String globalKey) {
			this.globalKey = globalKey;
			return this;
		}
		public MetaFieldsBuilder setExternalKey(java.lang.String externalKey) {
			this.externalKey = externalKey;
			return this;
		}
		
		public MetaFields build() {
			return new MetaFields(this);
		}
		
		@Override
		public MetaFieldsBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			return scheme!=null || 
			globalKey!=null || 
			externalKey!=null;
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("scheme"), String.class, scheme, this, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("globalKey"), String.class, globalKey, this, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("externalKey"), String.class, externalKey, this, AttributeMeta.IS_META);
		}
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int _result = 1;
			_result = prime * _result + ((scheme == null) ? 0 : scheme.hashCode());
			_result = prime * _result + ((globalKey == null) ? 0 : globalKey.hashCode());
			_result = prime * _result + ((externalKey == null) ? 0 : externalKey.hashCode());
			return _result;
		}
	
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || getClass() != obj.getClass())
				return false;
			MetaFieldsBuilder other = (MetaFieldsBuilder) obj;
			if (scheme != null ? !scheme.equals(other.scheme) : other.scheme!=null) return false;
			if (globalKey != null ? !globalKey.equals(other.globalKey) : other.globalKey!=null) return false;
			if (externalKey != null ? !externalKey.equals(other.externalKey) : other.externalKey!=null) return false;
			return true;
		}
	
		@Override
		public String toString() {
			return "MetaFieldsBuilder {" +
			"scheme=" + this.scheme + ", " +
			"globalKey=" + this.globalKey + ", " +
			"externalKey=" + this.externalKey + ", " +
			'}';
		}
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("scheme"), String.class, scheme, this, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("globalKey"), String.class, globalKey, this, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("externalKey"), String.class, externalKey, this, AttributeMeta.IS_META);
	}
}
