package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.ReferenceWithMetaBuilder;
import org.isda.cdm.*;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;

public class ReferenceWithMetaCashflow extends RosettaModelObject implements ReferenceWithMeta<Cashflow>{
	private final String globalReference;
	private final String externalReference;
	private final Cashflow value;
	
	private ReferenceWithMetaCashflow(ReferenceWithMetaCashflowBuilder builder) {
		value = ofNullable(builder.getValue()).map(v->v.build()).orElse(null);
		globalReference = builder.globalReference;
		externalReference = builder.externalReference;
	}
	
	public Cashflow getValue() {
		return value;
	}
	
	public String getGlobalReference() {
		return globalReference;
	}
	
	public String getExternalReference() {
		return externalReference;
	}
	
	private static BasicRosettaMetaData<ReferenceWithMetaCashflow> metaData = new BasicRosettaMetaData<>();
							
	@Override
	public RosettaMetaData<? extends ReferenceWithMetaCashflow> metaData() {
		return metaData;
	}
	
	public ReferenceWithMetaCashflowBuilder toBuilder() {
		ReferenceWithMetaCashflowBuilder builder = new ReferenceWithMetaCashflowBuilder();
		builder.setValue(value);
		builder.setGlobalReference(globalReference);
		builder.setExternalReference(externalReference);
		return builder;
	}
	
	public static ReferenceWithMetaCashflowBuilder builder() {
		return new ReferenceWithMetaCashflowBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("value"), processor, Cashflow.class, value);
		processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((globalReference == null) ? 0 : globalReference.hashCode());
		_result = prime * _result + ((externalReference == null) ? 0 : externalReference.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		ReferenceWithMetaCashflow other = (ReferenceWithMetaCashflow) obj;
		if (globalReference != null ? !globalReference.equals(other.globalReference) : other.globalReference!=null) return false;
		if (externalReference != null ? !externalReference.equals(other.externalReference) : other.externalReference!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "ReferenceWitMeta {" +
			"globalReference=" + this.globalReference + ", " +
			"externalReference=" + this.externalReference + ", " +
			"value=" + this.value + ", " +
		'}';
	}
	
	public static class ReferenceWithMetaCashflowBuilder extends RosettaModelObjectBuilder implements ReferenceWithMetaBuilder<Cashflow>{
		private Cashflow.CashflowBuilder value;
		private String globalReference;
		private String externalReference;
		
		public ReferenceWithMetaCashflowBuilder() {}
		
		@Override
		public RosettaMetaData<? extends ReferenceWithMetaCashflow> metaData() {
			return metaData;
		}
		
		public Cashflow.CashflowBuilder getValue() {
			return value;
		}
		
		public String getGlobalReference() {
			return globalReference;
		}
		
		public String getExternalReference() {
			return externalReference;
		}
		
		@SuppressWarnings("unchecked")
		public ReferenceWithMetaCashflowBuilder setValue(Cashflow value) {
			this.value = ofNullable(value).map(t->t.toBuilder()).orElse(null);
			return this;
		}
		
		public ReferenceWithMetaCashflowBuilder setValueBuilder(Cashflow.CashflowBuilder value) {
			this.value = value;
			return this;
		}
		
		public ReferenceWithMetaCashflowBuilder setGlobalReference(String reference) {
			this.globalReference = reference;
			return this;
		}
		
		public ReferenceWithMetaCashflowBuilder setExternalReference(String reference) {
			this.externalReference = reference;
			return this;
		}
		
		public ReferenceWithMetaCashflow build() {
			return new ReferenceWithMetaCashflow(this);
		}
		
		@Override
		public ReferenceWithMetaCashflowBuilder prune() {
			if (value!=null && !value.hasData()) value = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			return !(value==null && globalReference==null && externalReference==null);
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("value"), processor, Cashflow.class, value);
			processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
		}
	}
}
