package org.isda.cdm.metafields;

import static java.util.Optional.ofNullable;

import com.rosetta.model.lib.process.*;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.meta.ReferenceWithMeta;
import com.rosetta.model.lib.meta.ReferenceWithMetaBuilder;
import org.isda.cdm.*;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.meta.BasicRosettaMetaData;

public class ReferenceWithMetaOptionPayout extends RosettaModelObject implements ReferenceWithMeta<OptionPayout>{
	private final String globalReference;
	private final String externalReference;
	private final OptionPayout value;
	
	private ReferenceWithMetaOptionPayout(ReferenceWithMetaOptionPayoutBuilder builder) {
		value = ofNullable(builder.getValue()).map(v->v.build()).orElse(null);
		globalReference = builder.globalReference;
		externalReference = builder.externalReference;
	}
	
	public OptionPayout getValue() {
		return value;
	}
	
	public String getGlobalReference() {
		return globalReference;
	}
	
	public String getExternalReference() {
		return externalReference;
	}
	
	private static BasicRosettaMetaData<ReferenceWithMetaOptionPayout> metaData = new BasicRosettaMetaData<>();
							
	@Override
	public RosettaMetaData<? extends ReferenceWithMetaOptionPayout> metaData() {
		return metaData;
	}
	
	public ReferenceWithMetaOptionPayoutBuilder toBuilder() {
		ReferenceWithMetaOptionPayoutBuilder builder = new ReferenceWithMetaOptionPayoutBuilder();
		builder.setValue(value);
		builder.setGlobalReference(globalReference);
		builder.setExternalReference(externalReference);
		return builder;
	}
	
	public static ReferenceWithMetaOptionPayoutBuilder builder() {
		return new ReferenceWithMetaOptionPayoutBuilder();
	}
	
	@Override
	protected void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("value"), processor, OptionPayout.class, value);
		processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
		processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int _result = 1;
		_result = prime * _result + ((globalReference == null) ? 0 : globalReference.hashCode());
		_result = prime * _result + ((externalReference == null) ? 0 : externalReference.hashCode());
		_result = prime * _result + ((value == null) ? 0 : value.hashCode());
		return _result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || getClass() != obj.getClass())
			return false;
		ReferenceWithMetaOptionPayout other = (ReferenceWithMetaOptionPayout) obj;
		if (globalReference != null ? !globalReference.equals(other.globalReference) : other.globalReference!=null) return false;
		if (externalReference != null ? !externalReference.equals(other.externalReference) : other.externalReference!=null) return false;
		if (value != null ? !value.equals(other.value) : other.value!=null) return false;
		return true;
	}

	@Override
	public String toString() {
		return "ReferenceWitMeta {" +
			"globalReference=" + this.globalReference + ", " +
			"externalReference=" + this.externalReference + ", " +
			"value=" + this.value + ", " +
		'}';
	}
	
	public static class ReferenceWithMetaOptionPayoutBuilder extends RosettaModelObjectBuilder implements ReferenceWithMetaBuilder<OptionPayout>{
		private OptionPayout.OptionPayoutBuilder value;
		private String globalReference;
		private String externalReference;
		
		public ReferenceWithMetaOptionPayoutBuilder() {}
		
		@Override
		public RosettaMetaData<? extends ReferenceWithMetaOptionPayout> metaData() {
			return metaData;
		}
		
		public OptionPayout.OptionPayoutBuilder getValue() {
			return value;
		}
		
		public String getGlobalReference() {
			return globalReference;
		}
		
		public String getExternalReference() {
			return externalReference;
		}
		
		@SuppressWarnings("unchecked")
		public ReferenceWithMetaOptionPayoutBuilder setValue(OptionPayout value) {
			this.value = ofNullable(value).map(t->t.toBuilder()).orElse(null);
			return this;
		}
		
		public ReferenceWithMetaOptionPayoutBuilder setValueBuilder(OptionPayout.OptionPayoutBuilder value) {
			this.value = value;
			return this;
		}
		
		public ReferenceWithMetaOptionPayoutBuilder setGlobalReference(String reference) {
			this.globalReference = reference;
			return this;
		}
		
		public ReferenceWithMetaOptionPayoutBuilder setExternalReference(String reference) {
			this.externalReference = reference;
			return this;
		}
		
		public ReferenceWithMetaOptionPayout build() {
			return new ReferenceWithMetaOptionPayout(this);
		}
		
		@Override
		public ReferenceWithMetaOptionPayoutBuilder prune() {
			if (value!=null && !value.hasData()) value = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			return !(value==null && globalReference==null && externalReference==null);
		}
		
		@Override
		public void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("value"), processor, OptionPayout.class, value);
			processor.processBasic(path.newSubPath("globalReference"), String.class, globalReference, this, AttributeMeta.IS_META);
			processor.processBasic(path.newSubPath("externalReference"), String.class, externalReference, this, AttributeMeta.IS_META);
		}
	}
}
