package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.CashTransferComponent;
import org.isda.cdm.CommodityTransferComponent;
import org.isda.cdm.Event;
import org.isda.cdm.Party;
import org.isda.cdm.PayerReceiver;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.SecurityTransferComponent;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.TransferorTransferee;
import org.isda.cdm.metafields.*;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("CashAndSecurityTransfer")
public class IsCashAndSecurityTransfer implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent CashAndSecurityTransfer <The qualification of a security settlement from the fact that (i) it is composed of a cashTransfer component and a securityTransfer component, and (ii) the cash and security move in the same direction.>\n" + 
		"	Event -> primitive -> transfer -> cashTransfer exists\n" + 
		"	and Event -> primitive -> transfer -> securityTransfer exists\n" + 
		"	and Event -> primitive -> transfer -> cashTransfer -> payerReceiver -> payerPartyReference -> reference = Event -> primitive -> transfer -> securityTransfer -> transferorTransferee -> transferorPartyReference -> reference\n" + 
		"	and Event -> primitive -> transfer -> commodityTransfer is absent";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> transfer -> cashTransfer exists\n" + 
		"	and Event -> primitive -> transfer -> securityTransfer exists\n" + 
		"	and Event -> primitive -> transfer -> cashTransfer -> payerReceiver -> payerPartyReference -> reference = Event -> primitive -> transfer -> securityTransfer -> transferorTransferee -> transferorPartyReference -> reference\n" + 
		"	and Event -> primitive -> transfer -> commodityTransfer is absent";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<CashTransferComponent>mapC("getCashTransfer", TransferPrimitive::getCashTransfer), false).and(exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<SecurityTransferComponent>mapC("getSecurityTransfer", TransferPrimitive::getSecurityTransfer), false)).and(areEqual(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<CashTransferComponent>mapC("getCashTransfer", TransferPrimitive::getCashTransfer).<PayerReceiver>map("getPayerReceiver", CashTransferComponent::getPayerReceiver).<ReferenceWithMetaParty>map("getPayerPartyReference", PayerReceiver::getPayerPartyReference).map("getReference", a->a.getGlobalReference()), MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<SecurityTransferComponent>mapC("getSecurityTransfer", TransferPrimitive::getSecurityTransfer).<TransferorTransferee>map("getTransferorTransferee", SecurityTransferComponent::getTransferorTransferee).<ReferenceWithMetaParty>map("getTransferorPartyReference", TransferorTransferee::getTransferorPartyReference).map("getReference", a->a.getGlobalReference()))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer).<CommodityTransferComponent>mapC("getCommodityTransfer", TransferPrimitive::getCommodityTransfer)));
		
		return QualifyResult.builder()
				.setName("CashAndSecurityTransfer")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
