package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.MessageInformation;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("CashflowCalculationNotification")
public class IsCashflowCalculationNotification implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent CashflowCalculationNotification <null>\n" + 
		"	Event -> primitive -> transfer only exists\n" + 
		"	and Event -> messageInformation -> sentBy = \"DTCC\"";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> transfer only exists\n" + 
		"	and Event -> messageInformation -> sentBy = \"DTCC\"";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer), true).and(areEqual(MapperS.of(event).<MessageInformation>map("getMessageInformation", Event::getMessageInformation).map("getSentBy", MessageInformation::getSentBy).<String>map("getValue", FieldWithMeta::getValue), MapperS.of("DTCC")));
		
		return QualifyResult.builder()
				.setName("CashflowCalculationNotification")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
