package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.EventWorkflow;
import org.isda.cdm.Inception;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.WorkflowStatusEnum;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("ClearedTrade")
public class IsClearedTrade implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent ClearedTrade <The cleared trade at the CCP.>\n" + 
		"	Event -> primitive -> inception only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Cleared";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> inception only exists\n" + 
		"	and Event -> eventWorkflow -> workflowStatus = WorkflowStatusEnum.Cleared";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception), true).and(areEqual(MapperS.of(event).<EventWorkflow>map("getEventWorkflow", Event::getEventWorkflow).<WorkflowStatusEnum>mapC("getWorkflowStatus", EventWorkflow::getWorkflowStatus), MapperS.of(WorkflowStatusEnum.CLEARED)));
		
		return QualifyResult.builder()
				.setName("ClearedTrade")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
