package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Contract;
import org.isda.cdm.Event;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.Trade;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("Compression")
public class IsCompression implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent Compression <The qualification of a compression event from the fact that (i) the quantityChange primitive exists, and (ii) there are multiple contracts (or contract references) specified in the before state.>\n" + 
		"	Event -> primitive -> quantityChange exists\n" + 
		"	and Event -> primitive -> quantityChange -> before -> contract multiple exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> quantityChange exists\n" + 
		"	and Event -> primitive -> quantityChange -> before -> contract multiple exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange), false).and(multipleExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract), false));
		
		return QualifyResult.builder()
				.setName("Compression")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
