package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("DerivedObservation")
public class IsDerivedObservation implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent DerivedObservation <The qualification of a derived observation event from the fact that (i) the only primitive is the observation, (ii) there are multiple instances of such observations, and (iii) there is an associated call function to compute, say, an interpolation.>\n" + 
		"	Event -> primitive -> observation only multiple exists\n" + 
		"	and Event -> functionCall exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> observation only multiple exists\n" + 
		"	and Event -> functionCall exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = multipleExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ObservationPrimitive>mapC("getObservation", PrimitiveEvent::getObservation), true).and(exists(MapperS.of(event).<String>map("getFunctionCall", Event::getFunctionCall), false));
		
		return QualifyResult.builder()
				.setName("DerivedObservation")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
