package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.ExercisePrimitive;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("Exercise")
public class IsExercise implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent Exercise <The qualification of an exercise event from the fact that (i) the only primitive is the exercise, and (ii) the remaining quantity = 0, and (iv) the closedState of the contract is Terminated.>\n" + 
		"	Event -> primitive -> exercise only exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> exercise only exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ExercisePrimitive>map("getExercise", PrimitiveEvent::getExercise), true);
		
		return QualifyResult.builder()
				.setName("Exercise")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
