package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.TransferPrimitive;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("MultipleTransfers")
public class IsMultipleTransfers implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent MultipleTransfers <The qualification an event that combines multiple transfers from the fact that multiple transfer primitives exist.>\n" + 
		"	Event -> primitive -> transfer multiple exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> transfer multiple exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = multipleExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<TransferPrimitive>mapC("getTransfer", PrimitiveEvent::getTransfer), false);
		
		return QualifyResult.builder()
				.setName("MultipleTransfers")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
