package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.ClosedState;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.Event;
import org.isda.cdm.Inception;
import org.isda.cdm.PostInceptionState;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.Trade;
import org.isda.cdm.TradeWarehouseWorkflow;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("NewTrade")
public class IsNewTrade implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent NewTrade <null>\n" + 
		"	Event -> primitive -> inception only exists\n" + 
		"	and Event -> primitive -> inception -> after -> tradeWarehouseWorkflow is absent\n" + 
		"	and Event -> primitive -> quantityChange -> after -> contract -> closedState is absent\n" + 
		"	and Event -> primitive -> inception -> after -> contract -> clearedDate is absent";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> inception only exists\n" + 
		"	and Event -> primitive -> inception -> after -> tradeWarehouseWorkflow is absent\n" + 
		"	and Event -> primitive -> quantityChange -> after -> contract -> closedState is absent\n" + 
		"	and Event -> primitive -> inception -> after -> contract -> clearedDate is absent";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception), true).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<TradeWarehouseWorkflow>map("getTradeWarehouseWorkflow", PostInceptionState::getTradeWarehouseWorkflow))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ClosedState>map("getClosedState", Contract::getClosedState))).and(notExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<Contract>map("getContract", ContractState::getContract).<Date>map("getClearedDate", Contract::getClearedDate)));
		
		return QualifyResult.builder()
				.setName("NewTrade")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
