package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("Observation")
public class IsObservation implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent Observation <The qualification of an observation event from the fact that (i) the only primitive is the observation, and (ii) that there is only a single instance of such observation.>\n" + 
		"	Event -> primitive -> observation only single exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> observation only single exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = singleExists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ObservationPrimitive>mapC("getObservation", PrimitiveEvent::getObservation), true);
		
		return QualifyResult.builder()
				.setName("Observation")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
