package org.isda.cdm.qualify.event;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Event;
import org.isda.cdm.Inception;
import org.isda.cdm.PostInceptionState;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.TradeWarehouseWorkflow;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("TradeWarehousePositionNotification")
public class IsTradeWarehousePositionNotification implements Function<Event, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isEvent TradeWarehousePositionNotification <null>\n" + 
		"	Event -> primitive -> inception exists\n" + 
		"	and Event -> primitive -> inception -> after -> tradeWarehouseWorkflow exists";
	
	private static final String EXPR_DEFINITION = "Event -> primitive -> inception exists\n" + 
		"	and Event -> primitive -> inception -> after -> tradeWarehouseWorkflow exists";
	
	@Override
	public QualifyResult apply(Event event) {
		ComparisonResult exprResult = exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception), false).and(exists(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception).<PostInceptionState>map("getAfter", Inception::getAfter).<TradeWarehouseWorkflow>map("getTradeWarehouseWorkflow", PostInceptionState::getTradeWarehouseWorkflow), false));
		
		return QualifyResult.builder()
				.setName("TradeWarehousePositionNotification")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
