package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Cashflow;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("CreditDefaultSwap")
public class IsCreditDefaultSwap implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct CreditDefaultSwap <This product qualification is temporary until such time that the ISDA Credit Group specifies a proper taxonomy for credit derivatives that is based upon economic terms.>\n" + 
		"	EconomicTerms -> payout -> creditDefaultPayout exists\n" + 
		"	and ( EconomicTerms -> payout -> interestRatePayout\n" + 
		"		or EconomicTerms -> payout -> cashflow ) exists\n" + 
		"	and EconomicTerms -> payout -> equityPayout is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> creditDefaultPayout exists\n" + 
		"	and ( EconomicTerms -> payout -> interestRatePayout\n" + 
		"		or EconomicTerms -> payout -> cashflow ) exists\n" + 
		"	and EconomicTerms -> payout -> equityPayout is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout), false).and(exists(MapperTree.orDifferent(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<Cashflow>mapC("getCashflow", Payout::getCashflow))), false)).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout)));
		
		return QualifyResult.builder()
				.setName("CreditDefaultSwap")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
