package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.ForeignExchange;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.FxCashSettlement;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("ForeignExchange")
public class IsForeignExchange implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct ForeignExchange <null>\n" + 
		"	EconomicTerms -> payout -> forwardPayout -> underlier -> singleUnderlier -> underlyingProduct -> foreignExchange exists and\n" + 
		"	EconomicTerms -> payout -> forwardPayout -> settlementTerms -> fxSettlementTerms is absent";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> forwardPayout -> underlier -> singleUnderlier -> underlyingProduct -> foreignExchange exists and\n" + 
		"	EconomicTerms -> payout -> forwardPayout -> settlementTerms -> fxSettlementTerms is absent";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<ForwardPayout>mapC("getForwardPayout", Payout::getForwardPayout).<Underlier>map("getUnderlier", ForwardPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ForeignExchange>map("getForeignExchange", Product::getForeignExchange), false).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<ForwardPayout>mapC("getForwardPayout", Payout::getForwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<FxCashSettlement>map("getFxSettlementTerms", OptionSettlement::getFxSettlementTerms)));
		
		return QualifyResult.builder()
				.setName("ForeignExchange")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
