package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.ForwardPayout;
import org.isda.cdm.FxCashSettlement;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.Payout;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("ForeignExchange_NDF")
public class IsForeignExchangeNDF implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct ForeignExchange_NDF <null>\n" + 
		"	EconomicTerms -> payout -> forwardPayout -> settlementTerms -> fxSettlementTerms exists";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> forwardPayout -> settlementTerms -> fxSettlementTerms exists";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<ForwardPayout>mapC("getForwardPayout", Payout::getForwardPayout).<OptionSettlement>map("getSettlementTerms", ForwardPayout::getSettlementTerms).<FxCashSettlement>map("getFxSettlementTerms", OptionSettlement::getFxSettlementTerms), false);
		
		return QualifyResult.builder()
				.setName("ForeignExchange_NDF")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
