package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.StrikeSchedule;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_CapFloor")
public class IsInterestRateCapFloor implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_CapFloor <null>\n" + 
		"	EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> capRateSchedule exists\n" + 
		"		or EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> floorRateSchedule exists";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> capRateSchedule exists\n" + 
		"		or EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate -> floorRateSchedule exists";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<StrikeSchedule>mapC("getCapRateSchedule", FloatingRate::getCapRateSchedule), false).or(exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).<StrikeSchedule>mapC("getFloorRateSchedule", FloatingRate::getFloorRateSchedule), false));
		
		return QualifyResult.builder()
				.setName("InterestRate_CapFloor")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
