package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.CrossCurrencyTerms;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.FloatingRateSpecification;
import org.isda.cdm.InflationRateSpecification;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.PrincipalExchanges;
import org.isda.cdm.RateSpecification;
import org.isda.cdm.Schedule;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_IRSwap_Basis")
public class IsInterestRateIRSwapBasis implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_IRSwap_Basis <null>\n" + 
		"	EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate count = 2\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> floatingRate count = 2\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> fixedRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> rateSpecification -> inflationRate is absent\n" + 
		"	and EconomicTerms -> payout -> interestRatePayout -> crossCurrencyTerms -> principalExchanges is absent\n" + 
		"	and EconomicTerms -> payout -> optionPayout is absent";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<FloatingRateSpecification>map("getFloatingRate", RateSpecification::getFloatingRate).resultCount()), MapperS.of(Integer.valueOf(2))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<Schedule>map("getFixedRate", RateSpecification::getFixedRate))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<RateSpecification>map("getRateSpecification", InterestRatePayout::getRateSpecification).<InflationRateSpecification>map("getInflationRate", RateSpecification::getInflationRate))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CrossCurrencyTerms>map("getCrossCurrencyTerms", InterestRatePayout::getCrossCurrencyTerms).<PrincipalExchanges>map("getPrincipalExchanges", CrossCurrencyTerms::getPrincipalExchanges))).and(notExists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout)));
		
		return QualifyResult.builder()
				.setName("InterestRate_IRSwap_Basis")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
