package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.Bond;
import org.isda.cdm.ConvertibleBond;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.Security;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_Option_DebtOption")
public class IsInterestRateOptionDebtOption implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_Option_DebtOption <null>\n" + 
		"	EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> bond exists\n" + 
		"		or EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> convertibleBond exists";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> bond exists\n" + 
		"		or EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> security -> convertibleBond exists";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<Underlier>map("getUnderlier", OptionPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<Bond>map("getBond", Security::getBond), false).or(exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<Underlier>map("getUnderlier", OptionPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<Security>map("getSecurity", Product::getSecurity).<ConvertibleBond>map("getConvertibleBond", Security::getConvertibleBond), false));
		
		return QualifyResult.builder()
				.setName("InterestRate_Option_DebtOption")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
