package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.Product;
import org.isda.cdm.SingleUnderlier;
import org.isda.cdm.Underlier;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("InterestRate_Option_Swaption")
public class IsInterestRateOptionSwaption implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct InterestRate_Option_Swaption <null>\n" + 
		"	EconomicTerms -> payout -> optionPayout exists\n" + 
		"	and EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> contractualProduct -> economicTerms -> payout -> interestRatePayout count = 2";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> optionPayout exists\n" + 
		"	and EconomicTerms -> payout -> optionPayout -> underlier -> singleUnderlier -> underlyingProduct -> contractualProduct -> economicTerms -> payout -> interestRatePayout count = 2";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout), false).and(areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<Underlier>map("getUnderlier", OptionPayout::getUnderlier).<SingleUnderlier>map("getSingleUnderlier", Underlier::getSingleUnderlier).<Product>map("getUnderlyingProduct", SingleUnderlier::getUnderlyingProduct).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).resultCount()), MapperS.of(Integer.valueOf(2))));
		
		return QualifyResult.builder()
				.setName("InterestRate_Option_Swaption")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
