package org.isda.cdm.qualify.product;

import com.rosetta.model.lib.annotations.RosettaQualifiable;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.ComparisonResult;
import java.util.function.Function;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.SecurityLeg;
import org.isda.cdm.SecurityPayout;
import org.isda.cdm.validation.datarule.*;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaQualifiable("RepurchaseAgreement")
public class IsRepurchaseAgreement implements Function<EconomicTerms, QualifyResult>{
	
	private static final String QUALIFY_DEFINITION = "isProduct RepurchaseAgreement <null>\n" + 
		"	EconomicTerms -> payout -> securityPayout -> securityLeg count >= 1 and\n" + 
		"	EconomicTerms -> payout -> interestRatePayout count = 1";
	
	private static final String EXPR_DEFINITION = "EconomicTerms -> payout -> securityPayout -> securityLeg count >= 1 and\n" + 
		"	EconomicTerms -> payout -> interestRatePayout count = 1";
	
	@Override
	public QualifyResult apply(EconomicTerms economicTerms) {
		ComparisonResult exprResult = greaterThanEquals(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<SecurityPayout>mapC("getSecurityPayout", Payout::getSecurityPayout).<SecurityLeg>mapC("getSecurityLeg", SecurityPayout::getSecurityLeg).resultCount()), MapperS.of(Integer.valueOf(1))).and(areEqual(MapperS.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).resultCount()), MapperS.of(Integer.valueOf(1))));
		
		return QualifyResult.builder()
				.setName("RepurchaseAgreement")
				.setDefinition(QUALIFY_DEFINITION)
				.setExpressionResult(EXPR_DEFINITION, exprResult)
				.build();
	}
}
