package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Account;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AccountValidator implements Validator<Account> {

	@Override
	public ValidationResult<Account> validate(RosettaPath path, Account o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("accountNumber", o.getAccountNumber()!=null ? 1 : 0, 1, 1),
				checkCardinality("accountName", o.getAccountName()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountType", o.getAccountType()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountBeneficiary", o.getAccountBeneficiary()!=null ? 1 : 0, 0, 1),
				checkCardinality("servicingParty", o.getServicingParty()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Account", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Account", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Account> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Account.AccountBuilder o = (Account.AccountBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("accountNumber", o.getAccountNumber()!=null ? 1 : 0, 1, 1),
				checkCardinality("accountName", o.getAccountName()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountType", o.getAccountType()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountBeneficiary", o.getAccountBeneficiary()!=null ? 1 : 0, 0, 1),
				checkCardinality("servicingParty", o.getServicingParty()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Account", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Account", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
