package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.AdditionalFixedPayments;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AdditionalFixedPaymentsValidator implements Validator<AdditionalFixedPayments> {

	@Override
	public ValidationResult<AdditionalFixedPayments> validate(RosettaPath path, AdditionalFixedPayments o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestShortfallReimbursement", o.getInterestShortfallReimbursement()!=null ? 1 : 0, 0, 1),
				checkCardinality("principalShortfallReimbursement", o.getPrincipalShortfallReimbursement()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedownReimbursement", o.getWritedownReimbursement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalFixedPayments", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalFixedPayments", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AdditionalFixedPayments> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		AdditionalFixedPayments.AdditionalFixedPaymentsBuilder o = (AdditionalFixedPayments.AdditionalFixedPaymentsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestShortfallReimbursement", o.getInterestShortfallReimbursement()!=null ? 1 : 0, 0, 1),
				checkCardinality("principalShortfallReimbursement", o.getPrincipalShortfallReimbursement()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedownReimbursement", o.getWritedownReimbursement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalFixedPayments", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalFixedPayments", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
