package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.AdditionalRegime;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AdditionalRegimeValidator implements Validator<AdditionalRegime> {

	@Override
	public ValidationResult<AdditionalRegime> validate(RosettaPath path, AdditionalRegime o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("regime", o.getRegime()!=null ? 1 : 0, 1, 1),
				checkCardinality("regimeTerms", o.getRegimeTerms()==null?0:o.getRegimeTerms().size(), 2, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalRegime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalRegime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AdditionalRegime> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		AdditionalRegime.AdditionalRegimeBuilder o = (AdditionalRegime.AdditionalRegimeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("regime", o.getRegime()!=null ? 1 : 0, 1, 1),
				checkCardinality("regimeTerms", o.getRegimeTerms()==null?0:o.getRegimeTerms().size(), 2, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalRegime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalRegime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
