package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.AdditionalTerminationEvent;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AdditionalTerminationEventValidator implements Validator<AdditionalTerminationEvent> {

	@Override
	public ValidationResult<AdditionalTerminationEvent> validate(RosettaPath path, AdditionalTerminationEvent o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("name", o.getName()!=null ? 1 : 0, 1, 1),
				checkCardinality("applicable", o.getApplicable()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalTerminationEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalTerminationEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AdditionalTerminationEvent> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		AdditionalTerminationEvent.AdditionalTerminationEventBuilder o = (AdditionalTerminationEvent.AdditionalTerminationEventBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("name", o.getName()!=null ? 1 : 0, 1, 1),
				checkCardinality("applicable", o.getApplicable()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AdditionalTerminationEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AdditionalTerminationEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
