package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.AddtlAttrbts;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AddtlAttrbtsValidator implements Validator<AddtlAttrbts> {

	@Override
	public ValidationResult<AddtlAttrbts> validate(RosettaPath path, AddtlAttrbts o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("rskRdcgTx", o.getRskRdcgTx()!=null ? 1 : 0, 1, 1),
				checkCardinality("sctiesFincgTxInd", o.getSctiesFincgTxInd()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AddtlAttrbts", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AddtlAttrbts", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AddtlAttrbts> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		AddtlAttrbts.AddtlAttrbtsBuilder o = (AddtlAttrbts.AddtlAttrbtsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("rskRdcgTx", o.getRskRdcgTx()!=null ? 1 : 0, 1, 1),
				checkCardinality("sctiesFincgTxInd", o.getSctiesFincgTxInd()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AddtlAttrbts", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AddtlAttrbts", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
