package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.AveragingObservationList;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class AveragingObservationListValidator implements Validator<AveragingObservationList> {

	@Override
	public ValidationResult<AveragingObservationList> validate(RosettaPath path, AveragingObservationList o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("averagingObservation", o.getAveragingObservation()==null?0:o.getAveragingObservation().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AveragingObservationList", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AveragingObservationList", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AveragingObservationList> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		AveragingObservationList.AveragingObservationListBuilder o = (AveragingObservationList.AveragingObservationListBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("averagingObservation", o.getAveragingObservation()==null?0:o.getAveragingObservation().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AveragingObservationList", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("AveragingObservationList", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
