package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.BondPriceAndYieldModel;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class BondPriceAndYieldModelValidator implements Validator<BondPriceAndYieldModel> {

	@Override
	public ValidationResult<BondPriceAndYieldModel> validate(RosettaPath path, BondPriceAndYieldModel o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cleanOrDirtyPrice", o.getCleanOrDirtyPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("relativePrice", o.getRelativePrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("yieldToMaturity", o.getYieldToMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("inflationFactor", o.getInflationFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("allInPrice", o.getAllInPrice()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BondPriceAndYieldModel", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BondPriceAndYieldModel", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<BondPriceAndYieldModel> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		BondPriceAndYieldModel.BondPriceAndYieldModelBuilder o = (BondPriceAndYieldModel.BondPriceAndYieldModelBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cleanOrDirtyPrice", o.getCleanOrDirtyPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("relativePrice", o.getRelativePrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("yieldToMaturity", o.getYieldToMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("inflationFactor", o.getInflationFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("allInPrice", o.getAllInPrice()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BondPriceAndYieldModel", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BondPriceAndYieldModel", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
