package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.BusinessCenters;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class BusinessCentersValidator implements Validator<BusinessCenters> {

	@Override
	public ValidationResult<BusinessCenters> validate(RosettaPath path, BusinessCenters o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("businessCenter", o.getBusinessCenter()==null?0:o.getBusinessCenter().size(), 0, 0),
				checkCardinality("businessCentersReference", o.getBusinessCentersReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BusinessCenters", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BusinessCenters", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<BusinessCenters> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		BusinessCenters.BusinessCentersBuilder o = (BusinessCenters.BusinessCentersBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("businessCenter", o.getBusinessCenter()==null?0:o.getBusinessCenter().size(), 0, 0),
				checkCardinality("businessCentersReference", o.getBusinessCentersReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BusinessCenters", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BusinessCenters", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
