package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.BuyerSeller;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class BuyerSellerValidator implements Validator<BuyerSeller> {

	@Override
	public ValidationResult<BuyerSeller> validate(RosettaPath path, BuyerSeller o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("buyerPartyReference", o.getBuyerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("buyerAccountReference", o.getBuyerAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("sellerPartyReference", o.getSellerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("sellerAccountReference", o.getSellerAccountReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BuyerSeller", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BuyerSeller", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<BuyerSeller> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		BuyerSeller.BuyerSellerBuilder o = (BuyerSeller.BuyerSellerBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("buyerPartyReference", o.getBuyerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("buyerAccountReference", o.getBuyerAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("sellerPartyReference", o.getSellerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("sellerAccountReference", o.getSellerAccountReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("BuyerSeller", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("BuyerSeller", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
