package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CalculationPeriodData;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CalculationPeriodDataValidator implements Validator<CalculationPeriodData> {

	@Override
	public ValidationResult<CalculationPeriodData> validate(RosettaPath path, CalculationPeriodData o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("startDate", o.getStartDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("endDate", o.getEndDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("daysInPeriod", o.getDaysInPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("daysInLeapYearPeriod", o.getDaysInLeapYearPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("isFirstPeriod", o.getIsFirstPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("isLastPeriod", o.getIsLastPeriod()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CalculationPeriodData", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CalculationPeriodData", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CalculationPeriodData> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CalculationPeriodData.CalculationPeriodDataBuilder o = (CalculationPeriodData.CalculationPeriodDataBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("startDate", o.getStartDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("endDate", o.getEndDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("daysInPeriod", o.getDaysInPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("daysInLeapYearPeriod", o.getDaysInLeapYearPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("isFirstPeriod", o.getIsFirstPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("isLastPeriod", o.getIsLastPeriod()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CalculationPeriodData", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CalculationPeriodData", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
