package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CashPriceMethod;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CashPriceMethodValidator implements Validator<CashPriceMethod> {

	@Override
	public ValidationResult<CashPriceMethod> validate(RosettaPath path, CashPriceMethod o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cashSettlementReferenceBanks", o.getCashSettlementReferenceBanks()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementCurrency", o.getCashSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("quotationRateType", o.getQuotationRateType()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CashPriceMethod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CashPriceMethod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CashPriceMethod> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CashPriceMethod.CashPriceMethodBuilder o = (CashPriceMethod.CashPriceMethodBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cashSettlementReferenceBanks", o.getCashSettlementReferenceBanks()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementCurrency", o.getCashSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("quotationRateType", o.getQuotationRateType()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CashPriceMethod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CashPriceMethod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
