package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CashSettlementTerms;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CashSettlementTermsValidator implements Validator<CashSettlementTerms> {

	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("valuationDate", o.getValuationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationTime", o.getValuationTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationMethod", o.getQuotationMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationAmount", o.getQuotationAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("minimumQuotationAmout", o.getMinimumQuotationAmout()!=null ? 1 : 0, 0, 1),
				checkCardinality("dealer", o.getDealer()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementBusinessDays", o.getCashSettlementBusinessDays()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementAmount", o.getCashSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("recoveryFactor", o.getRecoveryFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixedSettlement", o.getFixedSettlement()!=null ? 1 : 0, 0, 1),
				checkCardinality("accruedInterest", o.getAccruedInterest()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationMethod", o.getValuationMethod()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CashSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CashSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CashSettlementTerms.CashSettlementTermsBuilder o = (CashSettlementTerms.CashSettlementTermsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("valuationDate", o.getValuationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationTime", o.getValuationTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationMethod", o.getQuotationMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationAmount", o.getQuotationAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("minimumQuotationAmout", o.getMinimumQuotationAmout()!=null ? 1 : 0, 0, 1),
				checkCardinality("dealer", o.getDealer()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementBusinessDays", o.getCashSettlementBusinessDays()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementAmount", o.getCashSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("recoveryFactor", o.getRecoveryFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixedSettlement", o.getFixedSettlement()!=null ? 1 : 0, 0, 1),
				checkCardinality("accruedInterest", o.getAccruedInterest()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationMethod", o.getValuationMethod()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CashSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CashSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
