package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CleanOrDirtyPrice;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CleanOrDirtyPriceValidator implements Validator<CleanOrDirtyPrice> {

	@Override
	public ValidationResult<CleanOrDirtyPrice> validate(RosettaPath path, CleanOrDirtyPrice o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cleanPrice", o.getCleanPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("dirtyPrice", o.getDirtyPrice()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CleanOrDirtyPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CleanOrDirtyPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CleanOrDirtyPrice> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CleanOrDirtyPrice.CleanOrDirtyPriceBuilder o = (CleanOrDirtyPrice.CleanOrDirtyPriceBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cleanPrice", o.getCleanPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("dirtyPrice", o.getDirtyPrice()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CleanOrDirtyPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CleanOrDirtyPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
