package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ClosedState;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ClosedStateValidator implements Validator<ClosedState> {

	@Override
	public ValidationResult<ClosedState> validate(RosettaPath path, ClosedState o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("state", o.getState()!=null ? 1 : 0, 1, 1),
				checkCardinality("activityDate", o.getActivityDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("lastPaymentDate", o.getLastPaymentDate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ClosedState", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ClosedState", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ClosedState> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ClosedState.ClosedStateBuilder o = (ClosedState.ClosedStateBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("state", o.getState()!=null ? 1 : 0, 1, 1),
				checkCardinality("activityDate", o.getActivityDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("lastPaymentDate", o.getLastPaymentDate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ClosedState", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ClosedState", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
