package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CommodityTransferBreakdown;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CommodityTransferBreakdownValidator implements Validator<CommodityTransferBreakdown> {

	@Override
	public ValidationResult<CommodityTransferBreakdown> validate(RosettaPath path, CommodityTransferBreakdown o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("transferorTransferee", o.getTransferorTransferee()!=null ? 1 : 0, 1, 1),
				checkCardinality("commodity", o.getCommodity()!=null ? 1 : 0, 1, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 1, 1),
				checkCardinality("unit", o.getUnit()!=null ? 1 : 0, 1, 1),
				checkCardinality("timeUnit", o.getTimeUnit()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CommodityTransferBreakdown", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CommodityTransferBreakdown", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CommodityTransferBreakdown> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CommodityTransferBreakdown.CommodityTransferBreakdownBuilder o = (CommodityTransferBreakdown.CommodityTransferBreakdownBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("transferorTransferee", o.getTransferorTransferee()!=null ? 1 : 0, 1, 1),
				checkCardinality("commodity", o.getCommodity()!=null ? 1 : 0, 1, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 1, 1),
				checkCardinality("unit", o.getUnit()!=null ? 1 : 0, 1, 1),
				checkCardinality("timeUnit", o.getTimeUnit()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CommodityTransferBreakdown", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CommodityTransferBreakdown", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
