package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Composite;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CompositeValidator implements Validator<Composite> {

	@Override
	public ValidationResult<Composite> validate(RosettaPath path, Composite o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("determinationMethod", o.getDeterminationMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("relativeDate", o.getRelativeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Composite", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Composite", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Composite> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Composite.CompositeBuilder o = (Composite.CompositeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("determinationMethod", o.getDeterminationMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("relativeDate", o.getRelativeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Composite", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Composite", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
