package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ConditionsElections;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ConditionsElectionsValidator implements Validator<ConditionsElections> {

	@Override
	public ValidationResult<ConditionsElections> validate(RosettaPath path, ConditionsElections o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("party", o.getParty()!=null ? 1 : 0, 1, 1),
				checkCardinality("illegality", o.getIllegality()!=null ? 1 : 0, 1, 1),
				checkCardinality("forceMajeure", o.getForceMajeure()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxEvent", o.getTaxEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxEventUponMerger", o.getTaxEventUponMerger()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditEventUponMerger", o.getCreditEventUponMerger()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalTerminationEvents", o.getAdditionalTerminationEvents()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalLanguage", o.getAdditionalLanguage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ConditionsElections", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ConditionsElections", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ConditionsElections> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ConditionsElections.ConditionsElectionsBuilder o = (ConditionsElections.ConditionsElectionsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("party", o.getParty()!=null ? 1 : 0, 1, 1),
				checkCardinality("illegality", o.getIllegality()!=null ? 1 : 0, 1, 1),
				checkCardinality("forceMajeure", o.getForceMajeure()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxEvent", o.getTaxEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxEventUponMerger", o.getTaxEventUponMerger()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditEventUponMerger", o.getCreditEventUponMerger()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalTerminationEvents", o.getAdditionalTerminationEvents()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalLanguage", o.getAdditionalLanguage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ConditionsElections", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ConditionsElections", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
