package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ContactInformation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ContactInformationValidator implements Validator<ContactInformation> {

	@Override
	public ValidationResult<ContactInformation> validate(RosettaPath path, ContactInformation o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("telephone", o.getTelephone()==null?0:o.getTelephone().size(), 0, 0),
				checkCardinality("address", o.getAddress()==null?0:o.getAddress().size(), 0, 0),
				checkCardinality("email", o.getEmail()==null?0:o.getEmail().size(), 0, 0),
				checkCardinality("webPage", o.getWebPage()==null?0:o.getWebPage().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContactInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ContactInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ContactInformation> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ContactInformation.ContactInformationBuilder o = (ContactInformation.ContactInformationBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("telephone", o.getTelephone()==null?0:o.getTelephone().size(), 0, 0),
				checkCardinality("address", o.getAddress()==null?0:o.getAddress().size(), 0, 0),
				checkCardinality("email", o.getEmail()==null?0:o.getEmail().size(), 0, 0),
				checkCardinality("webPage", o.getWebPage()==null?0:o.getWebPage().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContactInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ContactInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
