package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Contract;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ContractValidator implements Validator<Contract> {

	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contractIdentifier", o.getContractIdentifier()==null?0:o.getContractIdentifier().size(), 1, 0),
				checkCardinality("tradeDate", o.getTradeDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("clearedDate", o.getClearedDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("contractualProduct", o.getContractualProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractualQuantity", o.getContractualQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("collateral", o.getCollateral()!=null ? 1 : 0, 0, 1),
				checkCardinality("documentation", o.getDocumentation()!=null ? 1 : 0, 0, 1),
				checkCardinality("governingLaw", o.getGoverningLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("account", o.getAccount()==null?0:o.getAccount().size(), 0, 0),
				checkCardinality("partyRole", o.getPartyRole()==null?0:o.getPartyRole().size(), 0, 0),
				checkCardinality("calculationAgent", o.getCalculationAgent()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyContractInformation", o.getPartyContractInformation()==null?0:o.getPartyContractInformation().size(), 0, 0),
				checkCardinality("closedState", o.getClosedState()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Contract", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Contract", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Contract.ContractBuilder o = (Contract.ContractBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contractIdentifier", o.getContractIdentifier()==null?0:o.getContractIdentifier().size(), 1, 0),
				checkCardinality("tradeDate", o.getTradeDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("clearedDate", o.getClearedDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("contractualProduct", o.getContractualProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractualQuantity", o.getContractualQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("collateral", o.getCollateral()!=null ? 1 : 0, 0, 1),
				checkCardinality("documentation", o.getDocumentation()!=null ? 1 : 0, 0, 1),
				checkCardinality("governingLaw", o.getGoverningLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("account", o.getAccount()==null?0:o.getAccount().size(), 0, 0),
				checkCardinality("partyRole", o.getPartyRole()==null?0:o.getPartyRole().size(), 0, 0),
				checkCardinality("calculationAgent", o.getCalculationAgent()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyContractInformation", o.getPartyContractInformation()==null?0:o.getPartyContractInformation().size(), 0, 0),
				checkCardinality("closedState", o.getClosedState()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Contract", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Contract", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
