package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ContractualMatrix;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ContractualMatrixValidator implements Validator<ContractualMatrix> {

	@Override
	public ValidationResult<ContractualMatrix> validate(RosettaPath path, ContractualMatrix o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("matrixType", o.getMatrixType()!=null ? 1 : 0, 1, 1),
				checkCardinality("publicationDate", o.getPublicationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("matrixTerm", o.getMatrixTerm()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContractualMatrix", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ContractualMatrix", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ContractualMatrix> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ContractualMatrix.ContractualMatrixBuilder o = (ContractualMatrix.ContractualMatrixBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("matrixType", o.getMatrixType()!=null ? 1 : 0, 1, 1),
				checkCardinality("publicationDate", o.getPublicationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("matrixTerm", o.getMatrixTerm()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContractualMatrix", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ContractualMatrix", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
