package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CreditDefaultPayout;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CreditDefaultPayoutValidator implements Validator<CreditDefaultPayout> {

	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, CreditDefaultPayout o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("generalTerms", o.getGeneralTerms()!=null ? 1 : 0, 1, 1),
				checkCardinality("protectionTerms", o.getProtectionTerms()==null?0:o.getProtectionTerms().size(), 1, 0),
				checkCardinality("cashSettlementTerms", o.getCashSettlementTerms()==null?0:o.getCashSettlementTerms().size(), 0, 0),
				checkCardinality("physicalSettlementTerms", o.getPhysicalSettlementTerms()==null?0:o.getPhysicalSettlementTerms().size(), 0, 0),
				checkCardinality("transactedPrice", o.getTransactedPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditDefaultPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditDefaultPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CreditDefaultPayout.CreditDefaultPayoutBuilder o = (CreditDefaultPayout.CreditDefaultPayoutBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("generalTerms", o.getGeneralTerms()!=null ? 1 : 0, 1, 1),
				checkCardinality("protectionTerms", o.getProtectionTerms()==null?0:o.getProtectionTerms().size(), 1, 0),
				checkCardinality("cashSettlementTerms", o.getCashSettlementTerms()==null?0:o.getCashSettlementTerms().size(), 0, 0),
				checkCardinality("physicalSettlementTerms", o.getPhysicalSettlementTerms()==null?0:o.getPhysicalSettlementTerms().size(), 0, 0),
				checkCardinality("transactedPrice", o.getTransactedPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditDefaultPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditDefaultPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
