package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CreditEvents;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CreditEventsValidator implements Validator<CreditEvents> {

	@Override
	public ValidationResult<CreditEvents> validate(RosettaPath path, CreditEvents o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bankruptcy", o.getBankruptcy()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPay", o.getFailureToPay()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPayPrincipal", o.getFailureToPayPrincipal()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPayInterest", o.getFailureToPayInterest()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligationDefault", o.getObligationDefault()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligationAcceleration", o.getObligationAcceleration()!=null ? 1 : 0, 0, 1),
				checkCardinality("repudiationMoratorium", o.getRepudiationMoratorium()!=null ? 1 : 0, 0, 1),
				checkCardinality("restructuring", o.getRestructuring()!=null ? 1 : 0, 0, 1),
				checkCardinality("governmentalIntervention", o.getGovernmentalIntervention()!=null ? 1 : 0, 0, 1),
				checkCardinality("distressedRatingsDowngrade", o.getDistressedRatingsDowngrade()!=null ? 1 : 0, 0, 1),
				checkCardinality("maturityExtension", o.getMaturityExtension()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedown", o.getWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("impliedWritedown", o.getImpliedWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("defaultRequirement", o.getDefaultRequirement()!=null ? 1 : 0, 0, 1),
				checkCardinality("creditEventNotice", o.getCreditEventNotice()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditEvents> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CreditEvents.CreditEventsBuilder o = (CreditEvents.CreditEventsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bankruptcy", o.getBankruptcy()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPay", o.getFailureToPay()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPayPrincipal", o.getFailureToPayPrincipal()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToPayInterest", o.getFailureToPayInterest()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligationDefault", o.getObligationDefault()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligationAcceleration", o.getObligationAcceleration()!=null ? 1 : 0, 0, 1),
				checkCardinality("repudiationMoratorium", o.getRepudiationMoratorium()!=null ? 1 : 0, 0, 1),
				checkCardinality("restructuring", o.getRestructuring()!=null ? 1 : 0, 0, 1),
				checkCardinality("governmentalIntervention", o.getGovernmentalIntervention()!=null ? 1 : 0, 0, 1),
				checkCardinality("distressedRatingsDowngrade", o.getDistressedRatingsDowngrade()!=null ? 1 : 0, 0, 1),
				checkCardinality("maturityExtension", o.getMaturityExtension()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedown", o.getWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("impliedWritedown", o.getImpliedWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("defaultRequirement", o.getDefaultRequirement()!=null ? 1 : 0, 0, 1),
				checkCardinality("creditEventNotice", o.getCreditEventNotice()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
