package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CreditLimitUtilisationPosition;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CreditLimitUtilisationPositionValidator implements Validator<CreditLimitUtilisationPosition> {

	@Override
	public ValidationResult<CreditLimitUtilisationPosition> validate(RosettaPath path, CreditLimitUtilisationPosition o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("shortPosition", o.getShortPosition()!=null ? 1 : 0, 0, 1),
				checkCardinality("longPosition", o.getLongPosition()!=null ? 1 : 0, 0, 1),
				checkCardinality("global", o.getGlobal()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditLimitUtilisationPosition", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditLimitUtilisationPosition", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditLimitUtilisationPosition> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder o = (CreditLimitUtilisationPosition.CreditLimitUtilisationPositionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("shortPosition", o.getShortPosition()!=null ? 1 : 0, 0, 1),
				checkCardinality("longPosition", o.getLongPosition()!=null ? 1 : 0, 0, 1),
				checkCardinality("global", o.getGlobal()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditLimitUtilisationPosition", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditLimitUtilisationPosition", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
