package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CreditLimit;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CreditLimitValidator implements Validator<CreditLimit> {

	@Override
	public ValidationResult<CreditLimit> validate(RosettaPath path, CreditLimit o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("limitId", o.getLimitId()!=null ? 1 : 0, 0, 1),
				checkCardinality("applicableProduct", o.getApplicableProduct()!=null ? 1 : 0, 0, 1),
				checkCardinality("currency", o.getCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("tenor", o.getTenor()!=null ? 1 : 0, 0, 1),
				checkCardinality("limitApplicable", o.getLimitApplicable()==null?0:o.getLimitApplicable().size(), 0, 0),
				checkCardinality("expirationDateTime", o.getExpirationDateTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditLimit", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditLimit", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditLimit> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CreditLimit.CreditLimitBuilder o = (CreditLimit.CreditLimitBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("limitId", o.getLimitId()!=null ? 1 : 0, 0, 1),
				checkCardinality("applicableProduct", o.getApplicableProduct()!=null ? 1 : 0, 0, 1),
				checkCardinality("currency", o.getCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("tenor", o.getTenor()!=null ? 1 : 0, 0, 1),
				checkCardinality("limitApplicable", o.getLimitApplicable()==null?0:o.getLimitApplicable().size(), 0, 0),
				checkCardinality("expirationDateTime", o.getExpirationDateTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditLimit", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditLimit", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
