package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CreditSupportObligationsVariationMargin;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CreditSupportObligationsVariationMarginValidator implements Validator<CreditSupportObligationsVariationMargin> {

	@Override
	public ValidationResult<CreditSupportObligationsVariationMargin> validate(RosettaPath path, CreditSupportObligationsVariationMargin o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("eligibleCollateral", o.getEligibleCollateral()!=null ? 1 : 0, 1, 1),
				checkCardinality("ineligibleCreditSupport", o.getIneligibleCreditSupport()!=null ? 1 : 0, 1, 1),
				checkCardinality("majorCurrency", o.getMajorCurrency()==null?0:o.getMajorCurrency().size(), 1, 0),
				checkCardinality("fxHaircut", o.getFxHaircut()!=null ? 1 : 0, 1, 1),
				checkCardinality("minimumTransferAmount", o.getMinimumTransferAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("rounding", o.getRounding()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditSupportObligationsVariationMargin", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditSupportObligationsVariationMargin", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditSupportObligationsVariationMargin> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder o = (CreditSupportObligationsVariationMargin.CreditSupportObligationsVariationMarginBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("eligibleCollateral", o.getEligibleCollateral()!=null ? 1 : 0, 1, 1),
				checkCardinality("ineligibleCreditSupport", o.getIneligibleCreditSupport()!=null ? 1 : 0, 1, 1),
				checkCardinality("majorCurrency", o.getMajorCurrency()==null?0:o.getMajorCurrency().size(), 1, 0),
				checkCardinality("fxHaircut", o.getFxHaircut()!=null ? 1 : 0, 1, 1),
				checkCardinality("minimumTransferAmount", o.getMinimumTransferAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("rounding", o.getRounding()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CreditSupportObligationsVariationMargin", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CreditSupportObligationsVariationMargin", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
