package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Csa2016;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class Csa2016Validator implements Validator<Csa2016> {

	@Override
	public ValidationResult<Csa2016> validate(RosettaPath path, Csa2016 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("baseCurrency", o.getBaseCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalObligations", o.getAdditionalObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("conditionsPrecedent", o.getConditionsPrecedent()!=null ? 1 : 0, 1, 1),
				checkCardinality("substitution", o.getSubstitution()!=null ? 1 : 0, 1, 1),
				checkCardinality("disputeResolution", o.getDisputeResolution()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalRepresentation", o.getAdditionalRepresentation()!=null ? 1 : 0, 1, 1),
				checkCardinality("demandsAndNotices", o.getDemandsAndNotices()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressesForTransfer", o.getAddressesForTransfer()!=null ? 1 : 0, 1, 1),
				checkCardinality("bespokeProvision", o.getBespokeProvision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Csa2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Csa2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Csa2016> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Csa2016.Csa2016Builder o = (Csa2016.Csa2016Builder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("baseCurrency", o.getBaseCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalObligations", o.getAdditionalObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("conditionsPrecedent", o.getConditionsPrecedent()!=null ? 1 : 0, 1, 1),
				checkCardinality("substitution", o.getSubstitution()!=null ? 1 : 0, 1, 1),
				checkCardinality("disputeResolution", o.getDisputeResolution()!=null ? 1 : 0, 1, 1),
				checkCardinality("additionalRepresentation", o.getAdditionalRepresentation()!=null ? 1 : 0, 1, 1),
				checkCardinality("demandsAndNotices", o.getDemandsAndNotices()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressesForTransfer", o.getAddressesForTransfer()!=null ? 1 : 0, 1, 1),
				checkCardinality("bespokeProvision", o.getBespokeProvision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Csa2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Csa2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
