package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CsaInitialMargin2016JapaneseLaw;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CsaInitialMargin2016JapaneseLawValidator implements Validator<CsaInitialMargin2016JapaneseLaw> {

	@Override
	public ValidationResult<CsaInitialMargin2016JapaneseLaw> validate(RosettaPath path, CsaInitialMargin2016JapaneseLaw o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("obligorPostingObligations", o.getObligorPostingObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManagementAgreeement", o.getCollateralManagementAgreeement()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManager", o.getCollateralManager()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherEligibleSupport", o.getOtherEligibleSupport()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementDay", o.getCashSettlementDay()!=null ? 1 : 0, 0, 1),
				checkCardinality("securitiesSettlementDay", o.getSecuritiesSettlementDay()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligorRightsEvent", o.getObligorRightsEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("obligorAdditionalRightsEvent", o.getObligorAdditionalRightsEvent()!=null ? 1 : 0, 0, 1),
				checkCardinality("automaticSetOff", o.getAutomaticSetOff()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManagementArrangement", o.getCollateralManagementArrangement()!=null ? 1 : 0, 1, 1),
				checkCardinality("distributionAndInterestPayment", o.getDistributionAndInterestPayment()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherEligibleAndPostedSupport", o.getOtherEligibleAndPostedSupport()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherCsa", o.getOtherCsa()!=null ? 1 : 0, 1, 1),
				checkCardinality("japaneseLawCsa", o.getJapaneseLawCsa()!=null ? 1 : 0, 1, 1),
				checkCardinality("trustSchemeAddendum", o.getTrustSchemeAddendum()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaInitialMargin2016JapaneseLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaInitialMargin2016JapaneseLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaInitialMargin2016JapaneseLaw> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder o = (CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("obligorPostingObligations", o.getObligorPostingObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManagementAgreeement", o.getCollateralManagementAgreeement()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManager", o.getCollateralManager()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherEligibleSupport", o.getOtherEligibleSupport()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementDay", o.getCashSettlementDay()!=null ? 1 : 0, 0, 1),
				checkCardinality("securitiesSettlementDay", o.getSecuritiesSettlementDay()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligorRightsEvent", o.getObligorRightsEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("obligorAdditionalRightsEvent", o.getObligorAdditionalRightsEvent()!=null ? 1 : 0, 0, 1),
				checkCardinality("automaticSetOff", o.getAutomaticSetOff()!=null ? 1 : 0, 1, 1),
				checkCardinality("collateralManagementArrangement", o.getCollateralManagementArrangement()!=null ? 1 : 0, 1, 1),
				checkCardinality("distributionAndInterestPayment", o.getDistributionAndInterestPayment()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherEligibleAndPostedSupport", o.getOtherEligibleAndPostedSupport()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherCsa", o.getOtherCsa()!=null ? 1 : 0, 1, 1),
				checkCardinality("japaneseLawCsa", o.getJapaneseLawCsa()!=null ? 1 : 0, 1, 1),
				checkCardinality("trustSchemeAddendum", o.getTrustSchemeAddendum()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaInitialMargin2016JapaneseLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaInitialMargin2016JapaneseLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
