package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CsaInitialMargin2016;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CsaInitialMargin2016Validator implements Validator<CsaInitialMargin2016> {

	@Override
	public ValidationResult<CsaInitialMargin2016> validate(RosettaPath path, CsaInitialMargin2016 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("regime", o.getRegime()!=null ? 1 : 0, 1, 1),
				checkCardinality("oneWayProvisions", o.getOneWayProvisions()!=null ? 1 : 0, 1, 1),
				checkCardinality("method", o.getMethod()!=null ? 1 : 0, 1, 1),
				checkCardinality("identifiedCrossCurrencySwap", o.getIdentifiedCrossCurrencySwap()!=null ? 1 : 0, 1, 1),
				checkCardinality("sensitivityToEquity", o.getSensitivityToEquity()!=null ? 1 : 0, 1, 1),
				checkCardinality("sensitivityToCommodity", o.getSensitivityToCommodity()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxHaircutCurrency", o.getFxHaircutCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditSupportObligations", o.getCreditSupportObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("calculationDateLocation", o.getCalculationDateLocation()!=null ? 1 : 0, 1, 1),
				checkCardinality("notificationTime", o.getNotificationTime()!=null ? 1 : 0, 1, 1),
				checkCardinality("terminationCurrency", o.getTerminationCurrency()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaInitialMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaInitialMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaInitialMargin2016> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CsaInitialMargin2016.CsaInitialMargin2016Builder o = (CsaInitialMargin2016.CsaInitialMargin2016Builder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("regime", o.getRegime()!=null ? 1 : 0, 1, 1),
				checkCardinality("oneWayProvisions", o.getOneWayProvisions()!=null ? 1 : 0, 1, 1),
				checkCardinality("method", o.getMethod()!=null ? 1 : 0, 1, 1),
				checkCardinality("identifiedCrossCurrencySwap", o.getIdentifiedCrossCurrencySwap()!=null ? 1 : 0, 1, 1),
				checkCardinality("sensitivityToEquity", o.getSensitivityToEquity()!=null ? 1 : 0, 1, 1),
				checkCardinality("sensitivityToCommodity", o.getSensitivityToCommodity()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxHaircutCurrency", o.getFxHaircutCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditSupportObligations", o.getCreditSupportObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("calculationDateLocation", o.getCalculationDateLocation()!=null ? 1 : 0, 1, 1),
				checkCardinality("notificationTime", o.getNotificationTime()!=null ? 1 : 0, 1, 1),
				checkCardinality("terminationCurrency", o.getTerminationCurrency()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaInitialMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaInitialMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
