package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CsaVariationMargin2016;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CsaVariationMargin2016Validator implements Validator<CsaVariationMargin2016> {

	@Override
	public ValidationResult<CsaVariationMargin2016> validate(RosettaPath path, CsaVariationMargin2016 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("creditSupportObligations", o.getCreditSupportObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationAgent", o.getValuationAgent()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationDateLocation", o.getValuationDateLocation()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationTime", o.getValuationTime()==null?0:o.getValuationTime().size(), 1, 0),
				checkCardinality("notificationTime", o.getNotificationTime()!=null ? 1 : 0, 1, 1),
				checkCardinality("holdingAndUsingPostedCollateral", o.getHoldingAndUsingPostedCollateral()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditSupportOffsets", o.getCreditSupportOffsets()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherCsa", o.getOtherCsa()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaVariationMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaVariationMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaVariationMargin2016> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CsaVariationMargin2016.CsaVariationMargin2016Builder o = (CsaVariationMargin2016.CsaVariationMargin2016Builder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("creditSupportObligations", o.getCreditSupportObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationAgent", o.getValuationAgent()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationDateLocation", o.getValuationDateLocation()!=null ? 1 : 0, 1, 1),
				checkCardinality("valuationTime", o.getValuationTime()==null?0:o.getValuationTime().size(), 1, 0),
				checkCardinality("notificationTime", o.getNotificationTime()!=null ? 1 : 0, 1, 1),
				checkCardinality("holdingAndUsingPostedCollateral", o.getHoldingAndUsingPostedCollateral()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditSupportOffsets", o.getCreditSupportOffsets()!=null ? 1 : 0, 1, 1),
				checkCardinality("otherCsa", o.getOtherCsa()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsaVariationMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsaVariationMargin2016", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
