package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.CsdInitialMargin2016EnglishLaw;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CsdInitialMargin2016EnglishLawValidator implements Validator<CsdInitialMargin2016EnglishLaw> {

	@Override
	public ValidationResult<CsdInitialMargin2016EnglishLaw> validate(RosettaPath path, CsdInitialMargin2016EnglishLaw o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("chargorPostingObligations", o.getChargorPostingObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("transferTiming", o.getTransferTiming()!=null ? 1 : 0, 0, 1),
				checkCardinality("chargorRightsEvent", o.getChargorRightsEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("chargorAdditionalRightsEvent", o.getChargorAdditionalRightsEvent()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryInLieuRight", o.getDeliveryInLieuRight()!=null ? 1 : 0, 1, 1),
				checkCardinality("custodyArrangements", o.getCustodyArrangements()!=null ? 1 : 0, 1, 1),
				checkCardinality("processAgent", o.getProcessAgent()!=null ? 1 : 0, 1, 1),
				checkCardinality("appropriatedCollateralValuation", o.getAppropriatedCollateralValuation()!=null ? 1 : 0, 1, 1),
				checkCardinality("japaneseSecuritiesProvisions", o.getJapaneseSecuritiesProvisions()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsdInitialMargin2016EnglishLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsdInitialMargin2016EnglishLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsdInitialMargin2016EnglishLaw> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder o = (CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("chargorPostingObligations", o.getChargorPostingObligations()!=null ? 1 : 0, 1, 1),
				checkCardinality("transferTiming", o.getTransferTiming()!=null ? 1 : 0, 0, 1),
				checkCardinality("chargorRightsEvent", o.getChargorRightsEvent()!=null ? 1 : 0, 1, 1),
				checkCardinality("chargorAdditionalRightsEvent", o.getChargorAdditionalRightsEvent()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryInLieuRight", o.getDeliveryInLieuRight()!=null ? 1 : 0, 1, 1),
				checkCardinality("custodyArrangements", o.getCustodyArrangements()!=null ? 1 : 0, 1, 1),
				checkCardinality("processAgent", o.getProcessAgent()!=null ? 1 : 0, 1, 1),
				checkCardinality("appropriatedCollateralValuation", o.getAppropriatedCollateralValuation()!=null ? 1 : 0, 1, 1),
				checkCardinality("japaneseSecuritiesProvisions", o.getJapaneseSecuritiesProvisions()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CsdInitialMargin2016EnglishLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("CsdInitialMargin2016EnglishLaw", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
