package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Curve;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class CurveValidator implements Validator<Curve> {

	@Override
	public ValidationResult<Curve> validate(RosettaPath path, Curve o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestRateCurve", o.getInterestRateCurve()!=null ? 1 : 0, 0, 1),
				checkCardinality("commodityCurve", o.getCommodityCurve()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Curve", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Curve", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Curve> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Curve.CurveBuilder o = (Curve.CurveBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestRateCurve", o.getInterestRateCurve()!=null ? 1 : 0, 0, 1),
				checkCardinality("commodityCurve", o.getCommodityCurve()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Curve", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Curve", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
