package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.DistributionAndInterestPayment;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class DistributionAndInterestPaymentValidator implements Validator<DistributionAndInterestPayment> {

	@Override
	public ValidationResult<DistributionAndInterestPayment> validate(RosettaPath path, DistributionAndInterestPayment o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestRate", o.getInterestRate()==null?0:o.getInterestRate().size(), 1, 0),
				checkCardinality("interestPaymentTransfer", o.getInterestPaymentTransfer()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestPaymentNetting", o.getInterestPaymentNetting()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestAmount", o.getInterestAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestAdjustment", o.getInterestAdjustment()!=null ? 1 : 0, 1, 1),
				checkCardinality("negativeInterest", o.getNegativeInterest()!=null ? 1 : 0, 1, 1),
				checkCardinality("dailyInterestCompounding", o.getDailyInterestCompounding()!=null ? 1 : 0, 1, 1),
				checkCardinality("alternativeProvision", o.getAlternativeProvision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DistributionAndInterestPayment", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("DistributionAndInterestPayment", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<DistributionAndInterestPayment> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder o = (DistributionAndInterestPayment.DistributionAndInterestPaymentBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("interestRate", o.getInterestRate()==null?0:o.getInterestRate().size(), 1, 0),
				checkCardinality("interestPaymentTransfer", o.getInterestPaymentTransfer()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestPaymentNetting", o.getInterestPaymentNetting()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestAmount", o.getInterestAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("interestAdjustment", o.getInterestAdjustment()!=null ? 1 : 0, 1, 1),
				checkCardinality("negativeInterest", o.getNegativeInterest()!=null ? 1 : 0, 1, 1),
				checkCardinality("dailyInterestCompounding", o.getDailyInterestCompounding()!=null ? 1 : 0, 1, 1),
				checkCardinality("alternativeProvision", o.getAlternativeProvision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DistributionAndInterestPayment", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("DistributionAndInterestPayment", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
