package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.DividendDateReference;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class DividendDateReferenceValidator implements Validator<DividendDateReference> {

	@Override
	public ValidationResult<DividendDateReference> validate(RosettaPath path, DividendDateReference o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("dateReference", o.getDateReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("paymentDateOffset", o.getPaymentDateOffset()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DividendDateReference", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("DividendDateReference", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<DividendDateReference> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		DividendDateReference.DividendDateReferenceBuilder o = (DividendDateReference.DividendDateReferenceBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("dateReference", o.getDateReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("paymentDateOffset", o.getPaymentDateOffset()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DividendDateReference", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("DividendDateReference", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
