package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EligibilityToHoldCollateral;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EligibilityToHoldCollateralValidator implements Validator<EligibilityToHoldCollateral> {

	@Override
	public ValidationResult<EligibilityToHoldCollateral> validate(RosettaPath path, EligibilityToHoldCollateral o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyTerms", o.getPartyTerms()==null?0:o.getPartyTerms().size(), 1, 0),
				checkCardinality("custodianTerms", o.getCustodianTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("eligibleCountry", o.getEligibleCountry()==null?0:o.getEligibleCountry().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EligibilityToHoldCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EligibilityToHoldCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EligibilityToHoldCollateral> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder o = (EligibilityToHoldCollateral.EligibilityToHoldCollateralBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyTerms", o.getPartyTerms()==null?0:o.getPartyTerms().size(), 1, 0),
				checkCardinality("custodianTerms", o.getCustodianTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("eligibleCountry", o.getEligibleCountry()==null?0:o.getEligibleCountry().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EligibilityToHoldCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EligibilityToHoldCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
