package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EligibleCollateral;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EligibleCollateralValidator implements Validator<EligibleCollateral> {

	@Override
	public ValidationResult<EligibleCollateral> validate(RosettaPath path, EligibleCollateral o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("eligibleAsset", o.getEligibleAsset()!=null ? 1 : 0, 1, 1),
				checkCardinality("lowerMaturity", o.getLowerMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("higherMaturity", o.getHigherMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("minimumRating", o.getMinimumRating()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationPercentage", o.getValuationPercentage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EligibleCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EligibleCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EligibleCollateral> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EligibleCollateral.EligibleCollateralBuilder o = (EligibleCollateral.EligibleCollateralBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("eligibleAsset", o.getEligibleAsset()!=null ? 1 : 0, 1, 1),
				checkCardinality("lowerMaturity", o.getLowerMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("higherMaturity", o.getHigherMaturity()!=null ? 1 : 0, 0, 1),
				checkCardinality("minimumRating", o.getMinimumRating()!=null ? 1 : 0, 0, 1),
				checkCardinality("valuationPercentage", o.getValuationPercentage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EligibleCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EligibleCollateral", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
