package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EquityCorporateEvents;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EquityCorporateEventsValidator implements Validator<EquityCorporateEvents> {

	@Override
	public ValidationResult<EquityCorporateEvents> validate(RosettaPath path, EquityCorporateEvents o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("shareForShare", o.getShareForShare()!=null ? 1 : 0, 1, 1),
				checkCardinality("shareForOther", o.getShareForOther()!=null ? 1 : 0, 1, 1),
				checkCardinality("shareForCombined", o.getShareForCombined()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EquityCorporateEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EquityCorporateEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EquityCorporateEvents> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EquityCorporateEvents.EquityCorporateEventsBuilder o = (EquityCorporateEvents.EquityCorporateEventsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("shareForShare", o.getShareForShare()!=null ? 1 : 0, 1, 1),
				checkCardinality("shareForOther", o.getShareForOther()!=null ? 1 : 0, 1, 1),
				checkCardinality("shareForCombined", o.getShareForCombined()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EquityCorporateEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EquityCorporateEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
