package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EquitySwapMasterConfirmation2018;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EquitySwapMasterConfirmation2018Validator implements Validator<EquitySwapMasterConfirmation2018> {

	@Override
	public ValidationResult<EquitySwapMasterConfirmation2018> validate(RosettaPath path, EquitySwapMasterConfirmation2018 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("typeOfSwapElection", o.getTypeOfSwapElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("pricingMethodElection", o.getPricingMethodElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("linearInterpolationElection", o.getLinearInterpolationElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 1, 1),
				checkCardinality("equityCalculationPeriod", o.getEquityCalculationPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("equityCashSettlementDates", o.getEquityCashSettlementDates()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EquitySwapMasterConfirmation2018", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EquitySwapMasterConfirmation2018", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EquitySwapMasterConfirmation2018> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder o = (EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("typeOfSwapElection", o.getTypeOfSwapElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("pricingMethodElection", o.getPricingMethodElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("linearInterpolationElection", o.getLinearInterpolationElection()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 1, 1),
				checkCardinality("equityCalculationPeriod", o.getEquityCalculationPeriod()!=null ? 1 : 0, 1, 1),
				checkCardinality("equityCashSettlementDates", o.getEquityCashSettlementDates()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EquitySwapMasterConfirmation2018", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EquitySwapMasterConfirmation2018", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
