package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EventEffect;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EventEffectValidator implements Validator<EventEffect> {

	@Override
	public ValidationResult<EventEffect> validate(RosettaPath path, EventEffect o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("effectedContract", o.getEffectedContract()==null?0:o.getEffectedContract().size(), 0, 0),
				checkCardinality("effectedExecution", o.getEffectedExecution()==null?0:o.getEffectedExecution().size(), 0, 0),
				checkCardinality("contract", o.getContract()==null?0:o.getContract().size(), 0, 0),
				checkCardinality("execution", o.getExecution()==null?0:o.getExecution().size(), 0, 0),
				checkCardinality("productIdentifier", o.getProductIdentifier()==null?0:o.getProductIdentifier().size(), 0, 0),
				checkCardinality("transfer", o.getTransfer()==null?0:o.getTransfer().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EventEffect", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EventEffect", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EventEffect> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EventEffect.EventEffectBuilder o = (EventEffect.EventEffectBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("effectedContract", o.getEffectedContract()==null?0:o.getEffectedContract().size(), 0, 0),
				checkCardinality("effectedExecution", o.getEffectedExecution()==null?0:o.getEffectedExecution().size(), 0, 0),
				checkCardinality("contract", o.getContract()==null?0:o.getContract().size(), 0, 0),
				checkCardinality("execution", o.getExecution()==null?0:o.getExecution().size(), 0, 0),
				checkCardinality("productIdentifier", o.getProductIdentifier()==null?0:o.getProductIdentifier().size(), 0, 0),
				checkCardinality("transfer", o.getTransfer()==null?0:o.getTransfer().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EventEffect", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EventEffect", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
