package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Event;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EventValidator implements Validator<Event> {

	@Override
	public ValidationResult<Event> validate(RosettaPath path, Event o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("messageInformation", o.getMessageInformation()!=null ? 1 : 0, 0, 1),
				checkCardinality("timestamp", o.getTimestamp()==null?0:o.getTimestamp().size(), 1, 0),
				checkCardinality("eventIdentifier", o.getEventIdentifier()==null?0:o.getEventIdentifier().size(), 1, 0),
				checkCardinality("eventQualifier", o.getEventQualifier()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventDate", o.getEventDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("action", o.getAction()!=null ? 1 : 0, 1, 1),
				checkCardinality("intent", o.getIntent()!=null ? 1 : 0, 0, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("account", o.getAccount()==null?0:o.getAccount().size(), 0, 0),
				checkCardinality("lineage", o.getLineage()!=null ? 1 : 0, 0, 1),
				checkCardinality("primitive", o.getPrimitive()!=null ? 1 : 0, 1, 1),
				checkCardinality("functionCall", o.getFunctionCall()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventEffect", o.getEventEffect()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventWorkflow", o.getEventWorkflow()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Event", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Event", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Event> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Event.EventBuilder o = (Event.EventBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("messageInformation", o.getMessageInformation()!=null ? 1 : 0, 0, 1),
				checkCardinality("timestamp", o.getTimestamp()==null?0:o.getTimestamp().size(), 1, 0),
				checkCardinality("eventIdentifier", o.getEventIdentifier()==null?0:o.getEventIdentifier().size(), 1, 0),
				checkCardinality("eventQualifier", o.getEventQualifier()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventDate", o.getEventDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("action", o.getAction()!=null ? 1 : 0, 1, 1),
				checkCardinality("intent", o.getIntent()!=null ? 1 : 0, 0, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("account", o.getAccount()==null?0:o.getAccount().size(), 0, 0),
				checkCardinality("lineage", o.getLineage()!=null ? 1 : 0, 0, 1),
				checkCardinality("primitive", o.getPrimitive()!=null ? 1 : 0, 1, 1),
				checkCardinality("functionCall", o.getFunctionCall()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventEffect", o.getEventEffect()!=null ? 1 : 0, 0, 1),
				checkCardinality("eventWorkflow", o.getEventWorkflow()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Event", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Event", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
