package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.EventWorkflow;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class EventWorkflowValidator implements Validator<EventWorkflow> {

	@Override
	public ValidationResult<EventWorkflow> validate(RosettaPath path, EventWorkflow o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("workflowStatus", o.getWorkflowStatus()==null?0:o.getWorkflowStatus().size(), 0, 0),
				checkCardinality("comment", o.getComment()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyCustomisedWorkflow", o.getPartyCustomisedWorkflow()==null?0:o.getPartyCustomisedWorkflow().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EventWorkflow", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EventWorkflow", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EventWorkflow> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		EventWorkflow.EventWorkflowBuilder o = (EventWorkflow.EventWorkflowBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("workflowStatus", o.getWorkflowStatus()==null?0:o.getWorkflowStatus().size(), 0, 0),
				checkCardinality("comment", o.getComment()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyCustomisedWorkflow", o.getPartyCustomisedWorkflow()==null?0:o.getPartyCustomisedWorkflow().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EventWorkflow", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("EventWorkflow", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
