package org.isda.cdm.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ExchangeRate;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.stream.Collectors.joining;

public class ExchangeRateValidator implements Validator<ExchangeRate> {

	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, ExchangeRate o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("quotedCurrencyPair", o.getQuotedCurrencyPair()!=null ? 1 : 0, 1, 1),
				checkCardinality("rate", o.getRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("spotRate", o.getSpotRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("forwardPoints", o.getForwardPoints()!=null ? 1 : 0, 0, 1),
				checkCardinality("pointValue", o.getPointValue()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossRate", o.getCrossRate()==null?0:o.getCrossRate().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExchangeRate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExchangeRate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExchangeRate.ExchangeRateBuilder o = (ExchangeRate.ExchangeRateBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("quotedCurrencyPair", o.getQuotedCurrencyPair()!=null ? 1 : 0, 1, 1),
				checkCardinality("rate", o.getRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("spotRate", o.getSpotRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("forwardPoints", o.getForwardPoints()!=null ? 1 : 0, 0, 1),
				checkCardinality("pointValue", o.getPointValue()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossRate", o.getCrossRate()==null?0:o.getCrossRate().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExchangeRate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExchangeRate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
