package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExecutingEntity;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExecutingEntityValidator implements Validator<ExecutingEntity> {

	@Override
	public ValidationResult<ExecutingEntity> validate(RosettaPath path, ExecutingEntity o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("isInvestmentFirm", o.getIsInvestmentFirm()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressOfBranch", o.getAddressOfBranch()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressOfIncorporation", o.getAddressOfIncorporation()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExecutingEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExecutingEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExecutingEntity> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExecutingEntity.ExecutingEntityBuilder o = (ExecutingEntity.ExecutingEntityBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("isInvestmentFirm", o.getIsInvestmentFirm()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressOfBranch", o.getAddressOfBranch()!=null ? 1 : 0, 1, 1),
				checkCardinality("addressOfIncorporation", o.getAddressOfIncorporation()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExecutingEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExecutingEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
