package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Execution;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExecutionValidator implements Validator<Execution> {

	@Override
	public ValidationResult<Execution> validate(RosettaPath path, Execution o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("executionType", o.getExecutionType()!=null ? 1 : 0, 1, 1),
				checkCardinality("executionVenue", o.getExecutionVenue()!=null ? 1 : 0, 0, 1),
				checkCardinality("identifier", o.getIdentifier()==null?0:o.getIdentifier().size(), 1, 0),
				checkCardinality("tradeDate", o.getTradeDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("product", o.getProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("partyRole", o.getPartyRole()==null?0:o.getPartyRole().size(), 0, 0),
				checkCardinality("closedState", o.getClosedState()!=null ? 1 : 0, 0, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("executionQuantity", o.getExecutionQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("price", o.getPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Execution", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Execution", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Execution.ExecutionBuilder o = (Execution.ExecutionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("executionType", o.getExecutionType()!=null ? 1 : 0, 1, 1),
				checkCardinality("executionVenue", o.getExecutionVenue()!=null ? 1 : 0, 0, 1),
				checkCardinality("identifier", o.getIdentifier()==null?0:o.getIdentifier().size(), 1, 0),
				checkCardinality("tradeDate", o.getTradeDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("product", o.getProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("party", o.getParty()==null?0:o.getParty().size(), 0, 0),
				checkCardinality("partyRole", o.getPartyRole()==null?0:o.getPartyRole().size(), 0, 0),
				checkCardinality("closedState", o.getClosedState()!=null ? 1 : 0, 0, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("executionQuantity", o.getExecutionQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("price", o.getPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Execution", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Execution", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
