package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExerciseFeeSchedule;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExerciseFeeScheduleValidator implements Validator<ExerciseFeeSchedule> {

	@Override
	public ValidationResult<ExerciseFeeSchedule> validate(RosettaPath path, ExerciseFeeSchedule o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalReference", o.getNotionalReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("feeAmountSchedule", o.getFeeAmountSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("feeRateSchedule", o.getFeeRateSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("feePaymentDate", o.getFeePaymentDate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseFeeSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseFeeSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExerciseFeeSchedule> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExerciseFeeSchedule.ExerciseFeeScheduleBuilder o = (ExerciseFeeSchedule.ExerciseFeeScheduleBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalReference", o.getNotionalReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("feeAmountSchedule", o.getFeeAmountSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("feeRateSchedule", o.getFeeRateSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("feePaymentDate", o.getFeePaymentDate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseFeeSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseFeeSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
