package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExerciseFee;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExerciseFeeValidator implements Validator<ExerciseFee> {

	@Override
	public ValidationResult<ExerciseFee> validate(RosettaPath path, ExerciseFee o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalReference", o.getNotionalReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("feeAmount", o.getFeeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("feeRate", o.getFeeRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("feePaymentDate", o.getFeePaymentDate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseFee", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseFee", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExerciseFee> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExerciseFee.ExerciseFeeBuilder o = (ExerciseFee.ExerciseFeeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalReference", o.getNotionalReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("feeAmount", o.getFeeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("feeRate", o.getFeeRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("feePaymentDate", o.getFeePaymentDate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseFee", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseFee", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
