package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExerciseOutcome;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExerciseOutcomeValidator implements Validator<ExerciseOutcome> {

	@Override
	public ValidationResult<ExerciseOutcome> validate(RosettaPath path, ExerciseOutcome o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contract", o.getContract()!=null ? 1 : 0, 1, 1),
				checkCardinality("physicalExercise", o.getPhysicalExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashExercise", o.getCashExercise()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseOutcome", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseOutcome", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExerciseOutcome> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExerciseOutcome.ExerciseOutcomeBuilder o = (ExerciseOutcome.ExerciseOutcomeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contract", o.getContract()!=null ? 1 : 0, 1, 1),
				checkCardinality("physicalExercise", o.getPhysicalExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashExercise", o.getCashExercise()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseOutcome", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseOutcome", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
