package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExercisePrimitive;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExercisePrimitiveValidator implements Validator<ExercisePrimitive> {

	@Override
	public ValidationResult<ExercisePrimitive> validate(RosettaPath path, ExercisePrimitive o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exerciseDate", o.getExerciseDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("exerciseTime", o.getExerciseTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("fullExercise", o.getFullExercise()!=null ? 1 : 0, 1, 1),
				checkCardinality("before", o.getBefore()!=null ? 1 : 0, 1, 1),
				checkCardinality("after", o.getAfter()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExercisePrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExercisePrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExercisePrimitive> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExercisePrimitive.ExercisePrimitiveBuilder o = (ExercisePrimitive.ExercisePrimitiveBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exerciseDate", o.getExerciseDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("exerciseTime", o.getExerciseTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("fullExercise", o.getFullExercise()!=null ? 1 : 0, 1, 1),
				checkCardinality("before", o.getBefore()!=null ? 1 : 0, 1, 1),
				checkCardinality("after", o.getAfter()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExercisePrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExercisePrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
