package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExerciseProcedure;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExerciseProcedureValidator implements Validator<ExerciseProcedure> {

	@Override
	public ValidationResult<ExerciseProcedure> validate(RosettaPath path, ExerciseProcedure o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("manualExercise", o.getManualExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("automaticExercise", o.getAutomaticExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("followUpConfirmation", o.getFollowUpConfirmation()!=null ? 1 : 0, 1, 1),
				checkCardinality("limitedRightToConfirm", o.getLimitedRightToConfirm()!=null ? 1 : 0, 0, 1),
				checkCardinality("splitTicket", o.getSplitTicket()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseProcedure", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseProcedure", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExerciseProcedure> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExerciseProcedure.ExerciseProcedureBuilder o = (ExerciseProcedure.ExerciseProcedureBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("manualExercise", o.getManualExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("automaticExercise", o.getAutomaticExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("followUpConfirmation", o.getFollowUpConfirmation()!=null ? 1 : 0, 1, 1),
				checkCardinality("limitedRightToConfirm", o.getLimitedRightToConfirm()!=null ? 1 : 0, 0, 1),
				checkCardinality("splitTicket", o.getSplitTicket()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExerciseProcedure", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExerciseProcedure", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
