package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ExtraordinaryEvents;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ExtraordinaryEventsValidator implements Validator<ExtraordinaryEvents> {

	@Override
	public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, ExtraordinaryEvents o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("mergerEvents", o.getMergerEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("tenderOfferEvents", o.getTenderOfferEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("compositionOfCombinedConsideration", o.getCompositionOfCombinedConsideration()!=null ? 1 : 0, 0, 1),
				checkCardinality("indexAdjustmentEvents", o.getIndexAdjustmentEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalDisruptionEvents", o.getAdditionalDisruptionEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToDeliver", o.getFailureToDeliver()!=null ? 1 : 0, 0, 1),
				checkCardinality("representations", o.getRepresentations()!=null ? 1 : 0, 0, 1),
				checkCardinality("nationalizationOrInsolvency", o.getNationalizationOrInsolvency()!=null ? 1 : 0, 0, 1),
				checkCardinality("delisting", o.getDelisting()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExtraordinaryEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExtraordinaryEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ExtraordinaryEvents.ExtraordinaryEventsBuilder o = (ExtraordinaryEvents.ExtraordinaryEventsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("mergerEvents", o.getMergerEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("tenderOfferEvents", o.getTenderOfferEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("compositionOfCombinedConsideration", o.getCompositionOfCombinedConsideration()!=null ? 1 : 0, 0, 1),
				checkCardinality("indexAdjustmentEvents", o.getIndexAdjustmentEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalDisruptionEvents", o.getAdditionalDisruptionEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("failureToDeliver", o.getFailureToDeliver()!=null ? 1 : 0, 0, 1),
				checkCardinality("representations", o.getRepresentations()!=null ? 1 : 0, 0, 1),
				checkCardinality("nationalizationOrInsolvency", o.getNationalizationOrInsolvency()!=null ? 1 : 0, 0, 1),
				checkCardinality("delisting", o.getDelisting()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ExtraordinaryEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ExtraordinaryEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
