package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.FailureToPay;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class FailureToPayValidator implements Validator<FailureToPay> {

	@Override
	public ValidationResult<FailureToPay> validate(RosettaPath path, FailureToPay o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("applicable", o.getApplicable()!=null ? 1 : 0, 1, 1),
				checkCardinality("gracePeriodExtension", o.getGracePeriodExtension()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentRequirement", o.getPaymentRequirement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FailureToPay", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FailureToPay", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FailureToPay> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FailureToPay.FailureToPayBuilder o = (FailureToPay.FailureToPayBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("applicable", o.getApplicable()!=null ? 1 : 0, 1, 1),
				checkCardinality("gracePeriodExtension", o.getGracePeriodExtension()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentRequirement", o.getPaymentRequirement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FailureToPay", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FailureToPay", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
