package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.FloatingAmountEvents;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class FloatingAmountEventsValidator implements Validator<FloatingAmountEvents> {

	@Override
	public ValidationResult<FloatingAmountEvents> validate(RosettaPath path, FloatingAmountEvents o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("failureToPayPrincipal", o.getFailureToPayPrincipal()!=null ? 1 : 0, 0, 1),
				checkCardinality("interestShortfall", o.getInterestShortfall()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedown", o.getWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("impliedWritedown", o.getImpliedWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmountProvisions", o.getFloatingAmountProvisions()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalFixedPayments", o.getAdditionalFixedPayments()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FloatingAmountEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FloatingAmountEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FloatingAmountEvents> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FloatingAmountEvents.FloatingAmountEventsBuilder o = (FloatingAmountEvents.FloatingAmountEventsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("failureToPayPrincipal", o.getFailureToPayPrincipal()!=null ? 1 : 0, 0, 1),
				checkCardinality("interestShortfall", o.getInterestShortfall()!=null ? 1 : 0, 0, 1),
				checkCardinality("writedown", o.getWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("impliedWritedown", o.getImpliedWritedown()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmountProvisions", o.getFloatingAmountProvisions()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalFixedPayments", o.getAdditionalFixedPayments()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FloatingAmountEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FloatingAmountEvents", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
