package org.isda.cdm.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ForeignExchange;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.stream.Collectors.joining;

public class ForeignExchangeValidator implements Validator<ForeignExchange> {

	@Override
	public ValidationResult<ForeignExchange> validate(RosettaPath path, ForeignExchange o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exchangedCurrency1", o.getExchangedCurrency1()!=null ? 1 : 0, 1, 1),
				checkCardinality("exchangedCurrency2", o.getExchangedCurrency2()!=null ? 1 : 0, 1, 1),
				checkCardinality("tenorPeriod", o.getTenorPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("exchangeRate", o.getExchangeRate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ForeignExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ForeignExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ForeignExchange> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ForeignExchange.ForeignExchangeBuilder o = (ForeignExchange.ForeignExchangeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exchangedCurrency1", o.getExchangedCurrency1()!=null ? 1 : 0, 1, 1),
				checkCardinality("exchangedCurrency2", o.getExchangedCurrency2()!=null ? 1 : 0, 1, 1),
				checkCardinality("tenorPeriod", o.getTenorPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("exchangeRate", o.getExchangeRate()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ForeignExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ForeignExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
