package org.isda.cdm.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ForwardPayout;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.stream.Collectors.joining;

public class ForwardPayoutValidator implements Validator<ForwardPayout> {

	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, ForwardPayout o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("underlier", o.getUnderlier()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ForwardPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ForwardPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ForwardPayout> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ForwardPayout.ForwardPayoutBuilder o = (ForwardPayout.ForwardPayoutBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("underlier", o.getUnderlier()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementTerms", o.getSettlementTerms()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ForwardPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ForwardPayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
