package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.FxCashSettlement;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class FxCashSettlementValidator implements Validator<FxCashSettlement> {

	@Override
	public ValidationResult<FxCashSettlement> validate(RosettaPath path, FxCashSettlement o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("referenceCurrency", o.getReferenceCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixing", o.getFixing()==null?0:o.getFixing().size(), 0, 0),
				checkCardinality("rateSourceFixing", o.getRateSourceFixing()==null?0:o.getRateSourceFixing().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FxCashSettlement> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FxCashSettlement.FxCashSettlementBuilder o = (FxCashSettlement.FxCashSettlementBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("referenceCurrency", o.getReferenceCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixing", o.getFixing()==null?0:o.getFixing().size(), 0, 0),
				checkCardinality("rateSourceFixing", o.getRateSourceFixing()==null?0:o.getRateSourceFixing().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
