package org.isda.cdm.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.FxFixing;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.stream.Collectors.joining;

public class FxFixingValidator implements Validator<FxFixing> {

	@Override
	public ValidationResult<FxFixing> validate(RosettaPath path, FxFixing o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("quotedCurrencyPair", o.getQuotedCurrencyPair()!=null ? 1 : 0, 1, 1),
				checkCardinality("fixingDate", o.getFixingDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxFixing", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxFixing", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FxFixing> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FxFixing.FxFixingBuilder o = (FxFixing.FxFixingBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("quotedCurrencyPair", o.getQuotedCurrencyPair()!=null ? 1 : 0, 1, 1),
				checkCardinality("fixingDate", o.getFixingDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxFixing", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxFixing", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
