package org.isda.cdm.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.FxInformationSource;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.stream.Collectors.joining;

public class FxInformationSourceValidator implements Validator<FxInformationSource> {

	@Override
	public ValidationResult<FxInformationSource> validate(RosettaPath path, FxInformationSource o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("fixingTime", o.getFixingTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxInformationSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxInformationSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FxInformationSource> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FxInformationSource.FxInformationSourceBuilder o = (FxInformationSource.FxInformationSourceBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("fixingTime", o.getFixingTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxInformationSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxInformationSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
