package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.FxLinkedNotionalSchedule;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class FxLinkedNotionalScheduleValidator implements Validator<FxLinkedNotionalSchedule> {

	@Override
	public ValidationResult<FxLinkedNotionalSchedule> validate(RosettaPath path, FxLinkedNotionalSchedule o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("constantNotionalScheduleReference", o.getConstantNotionalScheduleReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("initialValue", o.getInitialValue()!=null ? 1 : 0, 0, 1),
				checkCardinality("varyingNotionalCurrency", o.getVaryingNotionalCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("varyingNotionalFixingDates", o.getVaryingNotionalFixingDates()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("varyingNotionalInterimExchangePaymentDates", o.getVaryingNotionalInterimExchangePaymentDates()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxLinkedNotionalSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxLinkedNotionalSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FxLinkedNotionalSchedule> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder o = (FxLinkedNotionalSchedule.FxLinkedNotionalScheduleBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("constantNotionalScheduleReference", o.getConstantNotionalScheduleReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("initialValue", o.getInitialValue()!=null ? 1 : 0, 0, 1),
				checkCardinality("varyingNotionalCurrency", o.getVaryingNotionalCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("varyingNotionalFixingDates", o.getVaryingNotionalFixingDates()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxSpotRateSource", o.getFxSpotRateSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("varyingNotionalInterimExchangePaymentDates", o.getVaryingNotionalInterimExchangePaymentDates()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxLinkedNotionalSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxLinkedNotionalSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
