package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.FxSpotRateSource;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class FxSpotRateSourceValidator implements Validator<FxSpotRateSource> {

	@Override
	public ValidationResult<FxSpotRateSource> validate(RosettaPath path, FxSpotRateSource o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("primaryRateSource", o.getPrimaryRateSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("secondaryRateSource", o.getSecondaryRateSource()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixingTime", o.getFixingTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxSpotRateSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxSpotRateSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FxSpotRateSource> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		FxSpotRateSource.FxSpotRateSourceBuilder o = (FxSpotRateSource.FxSpotRateSourceBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("primaryRateSource", o.getPrimaryRateSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("secondaryRateSource", o.getSecondaryRateSource()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixingTime", o.getFixingTime()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FxSpotRateSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("FxSpotRateSource", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
