package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.IneligibleCreditSupport;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class IneligibleCreditSupportValidator implements Validator<IneligibleCreditSupport> {

	@Override
	public ValidationResult<IneligibleCreditSupport> validate(RosettaPath path, IneligibleCreditSupport o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("specifiedParty", o.getSpecifiedParty()==null?0:o.getSpecifiedParty().size(), 0, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("IneligibleCreditSupport", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("IneligibleCreditSupport", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<IneligibleCreditSupport> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		IneligibleCreditSupport.IneligibleCreditSupportBuilder o = (IneligibleCreditSupport.IneligibleCreditSupportBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("specifiedParty", o.getSpecifiedParty()==null?0:o.getSpecifiedParty().size(), 0, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("IneligibleCreditSupport", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("IneligibleCreditSupport", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
