package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.InitialMarginCalculation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class InitialMarginCalculationValidator implements Validator<InitialMarginCalculation> {

	@Override
	public ValidationResult<InitialMarginCalculation> validate(RosettaPath path, InitialMarginCalculation o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("marginRatio", o.getMarginRatio()!=null ? 1 : 0, 0, 1),
				checkCardinality("marginRatioThreshold", o.getMarginRatioThreshold()==null?0:o.getMarginRatioThreshold().size(), 0, 2),
				checkCardinality("haircut", o.getHaircut()!=null ? 1 : 0, 0, 1),
				checkCardinality("haircutThreshold", o.getHaircutThreshold()==null?0:o.getHaircutThreshold().size(), 0, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("InitialMarginCalculation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("InitialMarginCalculation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<InitialMarginCalculation> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		InitialMarginCalculation.InitialMarginCalculationBuilder o = (InitialMarginCalculation.InitialMarginCalculationBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("marginRatio", o.getMarginRatio()!=null ? 1 : 0, 0, 1),
				checkCardinality("marginRatioThreshold", o.getMarginRatioThreshold()==null?0:o.getMarginRatioThreshold().size(), 0, 2),
				checkCardinality("haircut", o.getHaircut()!=null ? 1 : 0, 0, 1),
				checkCardinality("haircutThreshold", o.getHaircutThreshold()==null?0:o.getHaircutThreshold().size(), 0, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("InitialMarginCalculation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("InitialMarginCalculation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
