package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.InterestRatePayout;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class InterestRatePayoutValidator implements Validator<InterestRatePayout> {

	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("payerReceiver", o.getPayerReceiver()!=null ? 1 : 0, 0, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("rateSpecification", o.getRateSpecification()!=null ? 1 : 0, 1, 1),
				checkCardinality("dayCountFraction", o.getDayCountFraction()!=null ? 1 : 0, 0, 1),
				checkCardinality("calculationPeriodDates", o.getCalculationPeriodDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDates", o.getPaymentDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDate", o.getPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDelay", o.getPaymentDelay()!=null ? 1 : 0, 0, 1),
				checkCardinality("resetDates", o.getResetDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountingMethod", o.getDiscountingMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("compoundingMethod", o.getCompoundingMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashflowRepresentation", o.getCashflowRepresentation()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossCurrencyTerms", o.getCrossCurrencyTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("stubPeriod", o.getStubPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("bondReference", o.getBondReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixedAmount", o.getFixedAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmount", o.getFloatingAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("InterestRatePayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("InterestRatePayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		InterestRatePayout.InterestRatePayoutBuilder o = (InterestRatePayout.InterestRatePayoutBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("payerReceiver", o.getPayerReceiver()!=null ? 1 : 0, 0, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 0, 1),
				checkCardinality("rateSpecification", o.getRateSpecification()!=null ? 1 : 0, 1, 1),
				checkCardinality("dayCountFraction", o.getDayCountFraction()!=null ? 1 : 0, 0, 1),
				checkCardinality("calculationPeriodDates", o.getCalculationPeriodDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDates", o.getPaymentDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDate", o.getPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("paymentDelay", o.getPaymentDelay()!=null ? 1 : 0, 0, 1),
				checkCardinality("resetDates", o.getResetDates()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountingMethod", o.getDiscountingMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("compoundingMethod", o.getCompoundingMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashflowRepresentation", o.getCashflowRepresentation()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossCurrencyTerms", o.getCrossCurrencyTerms()!=null ? 1 : 0, 0, 1),
				checkCardinality("stubPeriod", o.getStubPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("bondReference", o.getBondReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("fixedAmount", o.getFixedAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmount", o.getFloatingAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("InterestRatePayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("InterestRatePayout", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
