package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.LastRegularPaymentDate;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class LastRegularPaymentDateValidator implements Validator<LastRegularPaymentDate> {

	@Override
	public ValidationResult<LastRegularPaymentDate> validate(RosettaPath path, LastRegularPaymentDate o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("lastRegularPaymentDate", o.getLastRegularPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("finalPaymentDate", o.getFinalPaymentDate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LastRegularPaymentDate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LastRegularPaymentDate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LastRegularPaymentDate> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		LastRegularPaymentDate.LastRegularPaymentDateBuilder o = (LastRegularPaymentDate.LastRegularPaymentDateBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("lastRegularPaymentDate", o.getLastRegularPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("finalPaymentDate", o.getFinalPaymentDate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LastRegularPaymentDate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LastRegularPaymentDate", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
