package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.LegalAgreementBase;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class LegalAgreementBaseValidator implements Validator<LegalAgreementBase> {

	@Override
	public ValidationResult<LegalAgreementBase> validate(RosettaPath path, LegalAgreementBase o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("agreementDate", o.getAgreementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("identifier", o.getIdentifier()==null?0:o.getIdentifier().size(), 0, 0),
				checkCardinality("lineage", o.getLineage()!=null ? 1 : 0, 0, 1),
				checkCardinality("agreementType", o.getAgreementType()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractualParty", o.getContractualParty()==null?0:o.getContractualParty().size(), 2, 2),
				checkCardinality("otherParty", o.getOtherParty()==null?0:o.getOtherParty().size(), 0, 0),
				checkCardinality("umbrellaAgreement", o.getUmbrellaAgreement()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyInformation", o.getPartyInformation()==null?0:o.getPartyInformation().size(), 2, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalAgreementBase", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalAgreementBase", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LegalAgreementBase> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		LegalAgreementBase.LegalAgreementBaseBuilder o = (LegalAgreementBase.LegalAgreementBaseBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("agreementDate", o.getAgreementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("effectiveDate", o.getEffectiveDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("identifier", o.getIdentifier()==null?0:o.getIdentifier().size(), 0, 0),
				checkCardinality("lineage", o.getLineage()!=null ? 1 : 0, 0, 1),
				checkCardinality("agreementType", o.getAgreementType()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractualParty", o.getContractualParty()==null?0:o.getContractualParty().size(), 2, 2),
				checkCardinality("otherParty", o.getOtherParty()==null?0:o.getOtherParty().size(), 0, 0),
				checkCardinality("umbrellaAgreement", o.getUmbrellaAgreement()!=null ? 1 : 0, 0, 1),
				checkCardinality("partyInformation", o.getPartyInformation()==null?0:o.getPartyInformation().size(), 2, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalAgreementBase", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalAgreementBase", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
