package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.LegalAgreement;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class LegalAgreementValidator implements Validator<LegalAgreement> {

	@Override
	public ValidationResult<LegalAgreement> validate(RosettaPath path, LegalAgreement o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("csdInitialMargin2016EnglishLaw", o.getCsdInitialMargin2016EnglishLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaInitialMargin2016JapaneseLaw", o.getCsaInitialMargin2016JapaneseLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaInitialMargin2016NewYorkLaw", o.getCsaInitialMargin2016NewYorkLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaVariationMargin2016NewYorkLaw", o.getCsaVariationMargin2016NewYorkLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("equitySwapMasterConfirmation2018", o.getEquitySwapMasterConfirmation2018()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalAgreement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalAgreement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LegalAgreement> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		LegalAgreement.LegalAgreementBuilder o = (LegalAgreement.LegalAgreementBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("csdInitialMargin2016EnglishLaw", o.getCsdInitialMargin2016EnglishLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaInitialMargin2016JapaneseLaw", o.getCsaInitialMargin2016JapaneseLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaInitialMargin2016NewYorkLaw", o.getCsaInitialMargin2016NewYorkLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("csaVariationMargin2016NewYorkLaw", o.getCsaVariationMargin2016NewYorkLaw()!=null ? 1 : 0, 0, 1),
				checkCardinality("equitySwapMasterConfirmation2018", o.getEquitySwapMasterConfirmation2018()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalAgreement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalAgreement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
