package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.LegalEntity;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class LegalEntityValidator implements Validator<LegalEntity> {

	@Override
	public ValidationResult<LegalEntity> validate(RosettaPath path, LegalEntity o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("entityId", o.getEntityId()==null?0:o.getEntityId().size(), 0, 0),
				checkCardinality("name", o.getName()!=null ? 1 : 0, 1, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LegalEntity> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		LegalEntity.LegalEntityBuilder o = (LegalEntity.LegalEntityBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("entityId", o.getEntityId()==null?0:o.getEntityId().size(), 0, 0),
				checkCardinality("name", o.getName()!=null ? 1 : 0, 1, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("LegalEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("LegalEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
