package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Lineage;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class LineageValidator implements Validator<Lineage> {

	@Override
	public ValidationResult<Lineage> validate(RosettaPath path, Lineage o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contractReference", o.getContractReference()==null?0:o.getContractReference().size(), 0, 0),
				checkCardinality("eventReference", o.getEventReference()==null?0:o.getEventReference().size(), 0, 0),
				checkCardinality("executionReference", o.getExecutionReference()==null?0:o.getExecutionReference().size(), 0, 0),
				checkCardinality("cashflowReference", o.getCashflowReference()==null?0:o.getCashflowReference().size(), 0, 0),
				checkCardinality("creditDefaultPayoutReference", o.getCreditDefaultPayoutReference()==null?0:o.getCreditDefaultPayoutReference().size(), 0, 0),
				checkCardinality("interestRatePayoutReference", o.getInterestRatePayoutReference()==null?0:o.getInterestRatePayoutReference().size(), 0, 0),
				checkCardinality("optionPayoutReference", o.getOptionPayoutReference()==null?0:o.getOptionPayoutReference().size(), 0, 0),
				checkCardinality("equityPayoutReference", o.getEquityPayoutReference()==null?0:o.getEquityPayoutReference().size(), 0, 0),
				checkCardinality("transferReference", o.getTransferReference()==null?0:o.getTransferReference().size(), 0, 0),
				checkCardinality("legalAgreement", o.getLegalAgreement()==null?0:o.getLegalAgreement().size(), 0, 0),
				checkCardinality("portfolioStateReference", o.getPortfolioStateReference()==null?0:o.getPortfolioStateReference().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Lineage", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Lineage", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Lineage> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Lineage.LineageBuilder o = (Lineage.LineageBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("contractReference", o.getContractReference()==null?0:o.getContractReference().size(), 0, 0),
				checkCardinality("eventReference", o.getEventReference()==null?0:o.getEventReference().size(), 0, 0),
				checkCardinality("executionReference", o.getExecutionReference()==null?0:o.getExecutionReference().size(), 0, 0),
				checkCardinality("cashflowReference", o.getCashflowReference()==null?0:o.getCashflowReference().size(), 0, 0),
				checkCardinality("creditDefaultPayoutReference", o.getCreditDefaultPayoutReference()==null?0:o.getCreditDefaultPayoutReference().size(), 0, 0),
				checkCardinality("interestRatePayoutReference", o.getInterestRatePayoutReference()==null?0:o.getInterestRatePayoutReference().size(), 0, 0),
				checkCardinality("optionPayoutReference", o.getOptionPayoutReference()==null?0:o.getOptionPayoutReference().size(), 0, 0),
				checkCardinality("equityPayoutReference", o.getEquityPayoutReference()==null?0:o.getEquityPayoutReference().size(), 0, 0),
				checkCardinality("transferReference", o.getTransferReference()==null?0:o.getTransferReference().size(), 0, 0),
				checkCardinality("legalAgreement", o.getLegalAgreement()==null?0:o.getLegalAgreement().size(), 0, 0),
				checkCardinality("portfolioStateReference", o.getPortfolioStateReference()==null?0:o.getPortfolioStateReference().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Lineage", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Lineage", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
