package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ManualExercise;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ManualExerciseValidator implements Validator<ManualExercise> {

	@Override
	public ValidationResult<ManualExercise> validate(RosettaPath path, ManualExercise o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exerciseNotice", o.getExerciseNotice()!=null ? 1 : 0, 0, 1),
				checkCardinality("fallbackExercise", o.getFallbackExercise()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ManualExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ManualExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ManualExercise> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ManualExercise.ManualExerciseBuilder o = (ManualExercise.ManualExerciseBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("exerciseNotice", o.getExerciseNotice()!=null ? 1 : 0, 0, 1),
				checkCardinality("fallbackExercise", o.getFallbackExercise()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ManualExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ManualExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
