package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Method;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class MethodValidator implements Validator<Method> {

	@Override
	public ValidationResult<Method> validate(RosettaPath path, Method o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("version", o.getVersion()!=null ? 1 : 0, 1, 1),
				checkCardinality("calculationCurrency", o.getCalculationCurrency()==null?0:o.getCalculationCurrency().size(), 1, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Method", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Method", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Method> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Method.MethodBuilder o = (Method.MethodBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("version", o.getVersion()!=null ? 1 : 0, 1, 1),
				checkCardinality("calculationCurrency", o.getCalculationCurrency()==null?0:o.getCalculationCurrency().size(), 1, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Method", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Method", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
