package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.MortgageBackedSecurity;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class MortgageBackedSecurityValidator implements Validator<MortgageBackedSecurity> {

	@Override
	public ValidationResult<MortgageBackedSecurity> validate(RosettaPath path, MortgageBackedSecurity o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("pool", o.getPool()!=null ? 1 : 0, 0, 1),
				checkCardinality("sector", o.getSector()!=null ? 1 : 0, 0, 1),
				checkCardinality("tranche", o.getTranche()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MortgageBackedSecurity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("MortgageBackedSecurity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<MortgageBackedSecurity> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		MortgageBackedSecurity.MortgageBackedSecurityBuilder o = (MortgageBackedSecurity.MortgageBackedSecurityBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("pool", o.getPool()!=null ? 1 : 0, 0, 1),
				checkCardinality("sector", o.getSector()!=null ? 1 : 0, 0, 1),
				checkCardinality("tranche", o.getTranche()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MortgageBackedSecurity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("MortgageBackedSecurity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
