package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.MultipleExercise;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class MultipleExerciseValidator implements Validator<MultipleExercise> {

	@Override
	public ValidationResult<MultipleExercise> validate(RosettaPath path, MultipleExercise o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("maximumNotionalAmount", o.getMaximumNotionalAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("maximumNumberOfOptions", o.getMaximumNumberOfOptions()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MultipleExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("MultipleExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<MultipleExercise> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		MultipleExercise.MultipleExerciseBuilder o = (MultipleExercise.MultipleExerciseBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("maximumNotionalAmount", o.getMaximumNotionalAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("maximumNumberOfOptions", o.getMaximumNumberOfOptions()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MultipleExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("MultipleExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
