package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Nm;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class NmValidator implements Validator<Nm> {

	@Override
	public ValidationResult<Nm> validate(RosettaPath path, Nm o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("refRate", o.getRefRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("term", o.getTerm()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Nm", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Nm", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Nm> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Nm.NmBuilder o = (Nm.NmBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("refRate", o.getRefRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("term", o.getTerm()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Nm", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Nm", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
