package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.NonDeliverableSettlement;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class NonDeliverableSettlementValidator implements Validator<NonDeliverableSettlement> {

	@Override
	public ValidationResult<NonDeliverableSettlement> validate(RosettaPath path, NonDeliverableSettlement o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("referenceCurrency", o.getReferenceCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxFixingDate", o.getFxFixingDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxFixingSchedule", o.getFxFixingSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementRateOption", o.getSettlementRateOption()!=null ? 1 : 0, 1, 1),
				checkCardinality("priceSourceDisruption", o.getPriceSourceDisruption()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NonDeliverableSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NonDeliverableSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<NonDeliverableSettlement> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		NonDeliverableSettlement.NonDeliverableSettlementBuilder o = (NonDeliverableSettlement.NonDeliverableSettlementBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("referenceCurrency", o.getReferenceCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("fxFixingDate", o.getFxFixingDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxFixingSchedule", o.getFxFixingSchedule()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementRateOption", o.getSettlementRateOption()!=null ? 1 : 0, 1, 1),
				checkCardinality("priceSourceDisruption", o.getPriceSourceDisruption()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NonDeliverableSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NonDeliverableSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
