package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.NonNegativeStepSchedule;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class NonNegativeStepScheduleValidator implements Validator<NonNegativeStepSchedule> {

	@Override
	public ValidationResult<NonNegativeStepSchedule> validate(RosettaPath path, NonNegativeStepSchedule o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("step", o.getStep()==null?0:o.getStep().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NonNegativeStepSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NonNegativeStepSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<NonNegativeStepSchedule> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		NonNegativeStepSchedule.NonNegativeStepScheduleBuilder o = (NonNegativeStepSchedule.NonNegativeStepScheduleBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("step", o.getStep()==null?0:o.getStep().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NonNegativeStepSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NonNegativeStepSchedule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
