package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.NotionalStepRule;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class NotionalStepRuleValidator implements Validator<NotionalStepRule> {

	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, NotionalStepRule o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("calculationPeriodDatesReference", o.getCalculationPeriodDatesReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("stepFrequency", o.getStepFrequency()!=null ? 1 : 0, 1, 1),
				checkCardinality("firstNotionalStepDate", o.getFirstNotionalStepDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("lastNotionalStepDate", o.getLastNotionalStepDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("notionalStepAmount", o.getNotionalStepAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("notionalStepRate", o.getNotionalStepRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("stepRelativeTo", o.getStepRelativeTo()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NotionalStepRule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NotionalStepRule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		NotionalStepRule.NotionalStepRuleBuilder o = (NotionalStepRule.NotionalStepRuleBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("calculationPeriodDatesReference", o.getCalculationPeriodDatesReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("stepFrequency", o.getStepFrequency()!=null ? 1 : 0, 1, 1),
				checkCardinality("firstNotionalStepDate", o.getFirstNotionalStepDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("lastNotionalStepDate", o.getLastNotionalStepDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("notionalStepAmount", o.getNotionalStepAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("notionalStepRate", o.getNotionalStepRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("stepRelativeTo", o.getStepRelativeTo()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NotionalStepRule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("NotionalStepRule", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
