package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ObservationPrimitive;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ObservationPrimitiveValidator implements Validator<ObservationPrimitive> {

	@Override
	public ValidationResult<ObservationPrimitive> validate(RosettaPath path, ObservationPrimitive o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("source", o.getSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("observation", o.getObservation()!=null ? 1 : 0, 1, 1),
				checkCardinality("date", o.getDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("time", o.getTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("side", o.getSide()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ObservationPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ObservationPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ObservationPrimitive> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ObservationPrimitive.ObservationPrimitiveBuilder o = (ObservationPrimitive.ObservationPrimitiveBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("source", o.getSource()!=null ? 1 : 0, 1, 1),
				checkCardinality("observation", o.getObservation()!=null ? 1 : 0, 1, 1),
				checkCardinality("date", o.getDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("time", o.getTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("side", o.getSide()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ObservationPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ObservationPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
