package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.OptionCashSettlement;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class OptionCashSettlementValidator implements Validator<OptionCashSettlement> {

	@Override
	public ValidationResult<OptionCashSettlement> validate(RosettaPath path, OptionCashSettlement o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cashSettlementValuationTime", o.getCashSettlementValuationTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementValuationDate", o.getCashSettlementValuationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementPaymentDate", o.getCashSettlementPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashPriceMethod", o.getCashPriceMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashPriceAlternateMethod", o.getCashPriceAlternateMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("parYieldCurveAdjustedMethod", o.getParYieldCurveAdjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("zeroCouponYieldAdjustedMethod", o.getZeroCouponYieldAdjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("parYieldCurveUnadjustedMethod", o.getParYieldCurveUnadjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossCurrencyMethod", o.getCrossCurrencyMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("collateralizedCashPriceMethod", o.getCollateralizedCashPriceMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionCashSettlement> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		OptionCashSettlement.OptionCashSettlementBuilder o = (OptionCashSettlement.OptionCashSettlementBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("cashSettlementValuationTime", o.getCashSettlementValuationTime()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementValuationDate", o.getCashSettlementValuationDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashSettlementPaymentDate", o.getCashSettlementPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashPriceMethod", o.getCashPriceMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("cashPriceAlternateMethod", o.getCashPriceAlternateMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("parYieldCurveAdjustedMethod", o.getParYieldCurveAdjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("zeroCouponYieldAdjustedMethod", o.getZeroCouponYieldAdjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("parYieldCurveUnadjustedMethod", o.getParYieldCurveUnadjustedMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("crossCurrencyMethod", o.getCrossCurrencyMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("collateralizedCashPriceMethod", o.getCollateralizedCashPriceMethod()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionCashSettlement", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
