package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.OptionExercise;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class OptionExerciseValidator implements Validator<OptionExercise> {

	@Override
	public ValidationResult<OptionExercise> validate(RosettaPath path, OptionExercise o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("optionStyle", o.getOptionStyle()!=null ? 1 : 0, 1, 1),
				checkCardinality("strike", o.getStrike()!=null ? 1 : 0, 0, 1),
				checkCardinality("exerciseProcedure", o.getExerciseProcedure()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlement", o.getSettlement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionExercise> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		OptionExercise.OptionExerciseBuilder o = (OptionExercise.OptionExerciseBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("optionStyle", o.getOptionStyle()!=null ? 1 : 0, 1, 1),
				checkCardinality("strike", o.getStrike()!=null ? 1 : 0, 0, 1),
				checkCardinality("exerciseProcedure", o.getExerciseProcedure()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlement", o.getSettlement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionExercise", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
