package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.OptionStrike;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class OptionStrikeValidator implements Validator<OptionStrike> {

	@Override
	public ValidationResult<OptionStrike> validate(RosettaPath path, OptionStrike o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("spread", o.getSpread()!=null ? 1 : 0, 0, 1),
				checkCardinality("price", o.getPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("currency", o.getCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("percentage", o.getPercentage()!=null ? 1 : 0, 0, 1),
				checkCardinality("strikeReference", o.getStrikeReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("referenceSwapCurve", o.getReferenceSwapCurve()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionStrike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionStrike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionStrike> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		OptionStrike.OptionStrikeBuilder o = (OptionStrike.OptionStrikeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("spread", o.getSpread()!=null ? 1 : 0, 0, 1),
				checkCardinality("price", o.getPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("currency", o.getCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("percentage", o.getPercentage()!=null ? 1 : 0, 0, 1),
				checkCardinality("strikeReference", o.getStrikeReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("referenceSwapCurve", o.getReferenceSwapCurve()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OptionStrike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("OptionStrike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
