package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PartyContractInformation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PartyContractInformationValidator implements Validator<PartyContractInformation> {

	@Override
	public ValidationResult<PartyContractInformation> validate(RosettaPath path, PartyContractInformation o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyReference", o.getPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("naturalPersonRole", o.getNaturalPersonRole()==null?0:o.getNaturalPersonRole().size(), 0, 0),
				checkCardinality("relatedParty", o.getRelatedParty()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountReference", o.getAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("category", o.getCategory()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PartyContractInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PartyContractInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PartyContractInformation> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PartyContractInformation.PartyContractInformationBuilder o = (PartyContractInformation.PartyContractInformationBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyReference", o.getPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("naturalPersonRole", o.getNaturalPersonRole()==null?0:o.getNaturalPersonRole().size(), 0, 0),
				checkCardinality("relatedParty", o.getRelatedParty()!=null ? 1 : 0, 0, 1),
				checkCardinality("accountReference", o.getAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("category", o.getCategory()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PartyContractInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PartyContractInformation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
