package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PassThrough;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PassThroughValidator implements Validator<PassThrough> {

	@Override
	public ValidationResult<PassThrough> validate(RosettaPath path, PassThrough o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("passThroughItem", o.getPassThroughItem()==null?0:o.getPassThroughItem().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PassThrough", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PassThrough", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PassThrough> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PassThrough.PassThroughBuilder o = (PassThrough.PassThroughBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("passThroughItem", o.getPassThroughItem()==null?0:o.getPassThroughItem().size(), 1, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PassThrough", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PassThrough", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
