package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PayerReceiver;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PayerReceiverValidator implements Validator<PayerReceiver> {

	@Override
	public ValidationResult<PayerReceiver> validate(RosettaPath path, PayerReceiver o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("payerPartyReference", o.getPayerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("payerAccountReference", o.getPayerAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("receiverPartyReference", o.getReceiverPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("receiverAccountReference", o.getReceiverAccountReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PayerReceiver", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PayerReceiver", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PayerReceiver> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PayerReceiver.PayerReceiverBuilder o = (PayerReceiver.PayerReceiverBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("payerPartyReference", o.getPayerPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("payerAccountReference", o.getPayerAccountReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("receiverPartyReference", o.getReceiverPartyReference()!=null ? 1 : 0, 1, 1),
				checkCardinality("receiverAccountReference", o.getReceiverAccountReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PayerReceiver", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PayerReceiver", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
