package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PaymentCalculationPeriod;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PaymentCalculationPeriodValidator implements Validator<PaymentCalculationPeriod> {

	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, PaymentCalculationPeriod o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("unadjustedPaymentDate", o.getUnadjustedPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("adjustedPaymentDate", o.getAdjustedPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("calculationPeriod", o.getCalculationPeriod()==null?0:o.getCalculationPeriod().size(), 1, 0),
				checkCardinality("fixedPaymentAmount", o.getFixedPaymentAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountFactor", o.getDiscountFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("forecastPaymentAmount", o.getForecastPaymentAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("presentValueAmount", o.getPresentValueAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PaymentCalculationPeriod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PaymentCalculationPeriod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PaymentCalculationPeriod.PaymentCalculationPeriodBuilder o = (PaymentCalculationPeriod.PaymentCalculationPeriodBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("unadjustedPaymentDate", o.getUnadjustedPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("adjustedPaymentDate", o.getAdjustedPaymentDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("calculationPeriod", o.getCalculationPeriod()==null?0:o.getCalculationPeriod().size(), 1, 0),
				checkCardinality("fixedPaymentAmount", o.getFixedPaymentAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountFactor", o.getDiscountFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("forecastPaymentAmount", o.getForecastPaymentAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("presentValueAmount", o.getPresentValueAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PaymentCalculationPeriod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PaymentCalculationPeriod", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
