package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PhysicalSettlementTerms;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PhysicalSettlementTermsValidator implements Validator<PhysicalSettlementTerms> {

	@Override
	public ValidationResult<PhysicalSettlementTerms> validate(RosettaPath path, PhysicalSettlementTerms o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("physicalSettlementPeriod", o.getPhysicalSettlementPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliverableObligations", o.getDeliverableObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("escrow", o.getEscrow()!=null ? 1 : 0, 0, 1),
				checkCardinality("sixtyBusinessDaySettlementCap", o.getSixtyBusinessDaySettlementCap()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PhysicalSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PhysicalSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PhysicalSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PhysicalSettlementTerms.PhysicalSettlementTermsBuilder o = (PhysicalSettlementTerms.PhysicalSettlementTermsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("physicalSettlementPeriod", o.getPhysicalSettlementPeriod()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliverableObligations", o.getDeliverableObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("escrow", o.getEscrow()!=null ? 1 : 0, 0, 1),
				checkCardinality("sixtyBusinessDaySettlementCap", o.getSixtyBusinessDaySettlementCap()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PhysicalSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PhysicalSettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
