package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Position;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PositionValidator implements Validator<Position> {

	@Override
	public ValidationResult<Position> validate(RosettaPath path, Position o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("product", o.getProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 1, 1),
				checkCardinality("cashBalance", o.getCashBalance()!=null ? 1 : 0, 0, 1),
				checkCardinality("positionStatus", o.getPositionStatus()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractReference", o.getContractReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Position", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Position", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Position> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Position.PositionBuilder o = (Position.PositionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("product", o.getProduct()!=null ? 1 : 0, 1, 1),
				checkCardinality("quantity", o.getQuantity()!=null ? 1 : 0, 1, 1),
				checkCardinality("cashBalance", o.getCashBalance()!=null ? 1 : 0, 0, 1),
				checkCardinality("positionStatus", o.getPositionStatus()!=null ? 1 : 0, 1, 1),
				checkCardinality("contractReference", o.getContractReference()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Position", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Position", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
