package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PostingObligationsElection;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PostingObligationsElectionValidator implements Validator<PostingObligationsElection> {

	@Override
	public ValidationResult<PostingObligationsElection> validate(RosettaPath path, PostingObligationsElection o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("party", o.getParty()!=null ? 1 : 0, 1, 1),
				checkCardinality("asPermitted", o.getAsPermitted()!=null ? 1 : 0, 1, 1),
				checkCardinality("eligibleCollateral", o.getEligibleCollateral()==null?0:o.getEligibleCollateral().size(), 0, 0),
				checkCardinality("excludedCollateral", o.getExcludedCollateral()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalLanguage", o.getAdditionalLanguage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PostingObligationsElection", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PostingObligationsElection", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PostingObligationsElection> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PostingObligationsElection.PostingObligationsElectionBuilder o = (PostingObligationsElection.PostingObligationsElectionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("party", o.getParty()!=null ? 1 : 0, 1, 1),
				checkCardinality("asPermitted", o.getAsPermitted()!=null ? 1 : 0, 1, 1),
				checkCardinality("eligibleCollateral", o.getEligibleCollateral()==null?0:o.getEligibleCollateral().size(), 0, 0),
				checkCardinality("excludedCollateral", o.getExcludedCollateral()!=null ? 1 : 0, 0, 1),
				checkCardinality("additionalLanguage", o.getAdditionalLanguage()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PostingObligationsElection", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PostingObligationsElection", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
