package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PrimitiveEvent;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PrimitiveEventValidator implements Validator<PrimitiveEvent> {

	@Override
	public ValidationResult<PrimitiveEvent> validate(RosettaPath path, PrimitiveEvent o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("execution", o.getExecution()==null?0:o.getExecution().size(), 0, 0),
				checkCardinality("contractFormation", o.getContractFormation()==null?0:o.getContractFormation().size(), 0, 0),
				checkCardinality("allocation", o.getAllocation()==null?0:o.getAllocation().size(), 0, 0),
				checkCardinality("exercise", o.getExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("inception", o.getInception()==null?0:o.getInception().size(), 0, 0),
				checkCardinality("observation", o.getObservation()==null?0:o.getObservation().size(), 0, 0),
				checkCardinality("quantityChange", o.getQuantityChange()==null?0:o.getQuantityChange().size(), 0, 0),
				checkCardinality("reset", o.getReset()==null?0:o.getReset().size(), 0, 0),
				checkCardinality("termsChange", o.getTermsChange()!=null ? 1 : 0, 0, 1),
				checkCardinality("transfer", o.getTransfer()==null?0:o.getTransfer().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PrimitiveEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PrimitiveEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PrimitiveEvent> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PrimitiveEvent.PrimitiveEventBuilder o = (PrimitiveEvent.PrimitiveEventBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("execution", o.getExecution()==null?0:o.getExecution().size(), 0, 0),
				checkCardinality("contractFormation", o.getContractFormation()==null?0:o.getContractFormation().size(), 0, 0),
				checkCardinality("allocation", o.getAllocation()==null?0:o.getAllocation().size(), 0, 0),
				checkCardinality("exercise", o.getExercise()!=null ? 1 : 0, 0, 1),
				checkCardinality("inception", o.getInception()==null?0:o.getInception().size(), 0, 0),
				checkCardinality("observation", o.getObservation()==null?0:o.getObservation().size(), 0, 0),
				checkCardinality("quantityChange", o.getQuantityChange()==null?0:o.getQuantityChange().size(), 0, 0),
				checkCardinality("reset", o.getReset()==null?0:o.getReset().size(), 0, 0),
				checkCardinality("termsChange", o.getTermsChange()!=null ? 1 : 0, 0, 1),
				checkCardinality("transfer", o.getTransfer()==null?0:o.getTransfer().size(), 0, 0)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PrimitiveEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PrimitiveEvent", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
