package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.PrincipalExchange;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class PrincipalExchangeValidator implements Validator<PrincipalExchange> {

	@Override
	public ValidationResult<PrincipalExchange> validate(RosettaPath path, PrincipalExchange o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("unadjustedPrincipalExchangeDate", o.getUnadjustedPrincipalExchangeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("adjustedPrincipalExchangeDate", o.getAdjustedPrincipalExchangeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("principalExchangeAmount", o.getPrincipalExchangeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountFactor", o.getDiscountFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("presentValuePrincipalExchangeAmount", o.getPresentValuePrincipalExchangeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PrincipalExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PrincipalExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PrincipalExchange> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		PrincipalExchange.PrincipalExchangeBuilder o = (PrincipalExchange.PrincipalExchangeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("unadjustedPrincipalExchangeDate", o.getUnadjustedPrincipalExchangeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("adjustedPrincipalExchangeDate", o.getAdjustedPrincipalExchangeDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("principalExchangeAmount", o.getPrincipalExchangeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("discountFactor", o.getDiscountFactor()!=null ? 1 : 0, 0, 1),
				checkCardinality("presentValuePrincipalExchangeAmount", o.getPresentValuePrincipalExchangeAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("PrincipalExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("PrincipalExchange", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
