package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ProductTaxonomy;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ProductTaxonomyValidator implements Validator<ProductTaxonomy> {

	@Override
	public ValidationResult<ProductTaxonomy> validate(RosettaPath path, ProductTaxonomy o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("taxonomyValue", o.getTaxonomyValue()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxonomySource", o.getTaxonomySource()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ProductTaxonomy", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ProductTaxonomy", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ProductTaxonomy> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ProductTaxonomy.ProductTaxonomyBuilder o = (ProductTaxonomy.ProductTaxonomyBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("taxonomyValue", o.getTaxonomyValue()!=null ? 1 : 0, 1, 1),
				checkCardinality("taxonomySource", o.getTaxonomySource()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ProductTaxonomy", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ProductTaxonomy", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
