package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ProtectionTerms;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ProtectionTermsValidator implements Validator<ProtectionTerms> {

	@Override
	public ValidationResult<ProtectionTerms> validate(RosettaPath path, ProtectionTerms o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalAmount", o.getNotionalAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditEvents", o.getCreditEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligations", o.getObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmountEvents", o.getFloatingAmountEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ProtectionTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ProtectionTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ProtectionTerms> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ProtectionTerms.ProtectionTermsBuilder o = (ProtectionTerms.ProtectionTermsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("notionalAmount", o.getNotionalAmount()!=null ? 1 : 0, 1, 1),
				checkCardinality("creditEvents", o.getCreditEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("obligations", o.getObligations()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingAmountEvents", o.getFloatingAmountEvents()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ProtectionTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ProtectionTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
