package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.RateSpecification;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class RateSpecificationValidator implements Validator<RateSpecification> {

	@Override
	public ValidationResult<RateSpecification> validate(RosettaPath path, RateSpecification o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("fixedRate", o.getFixedRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingRate", o.getFloatingRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("inflationRate", o.getInflationRate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("RateSpecification", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("RateSpecification", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<RateSpecification> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		RateSpecification.RateSpecificationBuilder o = (RateSpecification.RateSpecificationBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("fixedRate", o.getFixedRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("floatingRate", o.getFloatingRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("inflationRate", o.getInflationRate()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("RateSpecification", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("RateSpecification", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
