package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ReferenceBank;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ReferenceBankValidator implements Validator<ReferenceBank> {

	@Override
	public ValidationResult<ReferenceBank> validate(RosettaPath path, ReferenceBank o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("referenceBankId", o.getReferenceBankId()!=null ? 1 : 0, 1, 1),
				checkCardinality("referenceBankName", o.getReferenceBankName()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ReferenceBank", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ReferenceBank", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ReferenceBank> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ReferenceBank.ReferenceBankBuilder o = (ReferenceBank.ReferenceBankBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("referenceBankId", o.getReferenceBankId()!=null ? 1 : 0, 1, 1),
				checkCardinality("referenceBankName", o.getReferenceBankName()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ReferenceBank", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ReferenceBank", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
