package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ReferenceObligation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ReferenceObligationValidator implements Validator<ReferenceObligation> {

	@Override
	public ValidationResult<ReferenceObligation> validate(RosettaPath path, ReferenceObligation o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bond", o.getBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("convertibleBond", o.getConvertibleBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("mortgageBackedSecurity", o.getMortgageBackedSecurity()!=null ? 1 : 0, 0, 1),
				checkCardinality("loan", o.getLoan()!=null ? 1 : 0, 0, 1),
				checkCardinality("primaryObligor", o.getPrimaryObligor()!=null ? 1 : 0, 0, 1),
				checkCardinality("primaryObligorReference", o.getPrimaryObligorReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("guarantor", o.getGuarantor()!=null ? 1 : 0, 0, 1),
				checkCardinality("guarantorReference", o.getGuarantorReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("standardReferenceObligation", o.getStandardReferenceObligation()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ReferenceObligation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ReferenceObligation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ReferenceObligation> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ReferenceObligation.ReferenceObligationBuilder o = (ReferenceObligation.ReferenceObligationBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bond", o.getBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("convertibleBond", o.getConvertibleBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("mortgageBackedSecurity", o.getMortgageBackedSecurity()!=null ? 1 : 0, 0, 1),
				checkCardinality("loan", o.getLoan()!=null ? 1 : 0, 0, 1),
				checkCardinality("primaryObligor", o.getPrimaryObligor()!=null ? 1 : 0, 0, 1),
				checkCardinality("primaryObligorReference", o.getPrimaryObligorReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("guarantor", o.getGuarantor()!=null ? 1 : 0, 0, 1),
				checkCardinality("guarantorReference", o.getGuarantorReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("standardReferenceObligation", o.getStandardReferenceObligation()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ReferenceObligation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ReferenceObligation", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
