package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Regime;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class RegimeValidator implements Validator<Regime> {

	@Override
	public ValidationResult<Regime> validate(RosettaPath path, Regime o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyElection", o.getPartyElection()==null?0:o.getPartyElection().size(), 2, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Regime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Regime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Regime> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Regime.RegimeBuilder o = (Regime.RegimeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyElection", o.getPartyElection()==null?0:o.getPartyElection().size(), 2, 2)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Regime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Regime", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
