package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.ResetFrequency;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class ResetFrequencyValidator implements Validator<ResetFrequency> {

	@Override
	public ValidationResult<ResetFrequency> validate(RosettaPath path, ResetFrequency o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("weeklyRollConvention", o.getWeeklyRollConvention()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ResetFrequency", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ResetFrequency", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ResetFrequency> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		ResetFrequency.ResetFrequencyBuilder o = (ResetFrequency.ResetFrequencyBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("weeklyRollConvention", o.getWeeklyRollConvention()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ResetFrequency", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("ResetFrequency", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
