package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Rounding;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class RoundingValidator implements Validator<Rounding> {

	@Override
	public ValidationResult<Rounding> validate(RosettaPath path, Rounding o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("roundingDirection", o.getRoundingDirection()!=null ? 1 : 0, 1, 1),
				checkCardinality("precision", o.getPrecision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Rounding", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Rounding", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Rounding> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Rounding.RoundingBuilder o = (Rounding.RoundingBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("roundingDirection", o.getRoundingDirection()!=null ? 1 : 0, 1, 1),
				checkCardinality("precision", o.getPrecision()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Rounding", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Rounding", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
