package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.SecurityLeg;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SecurityLegValidator implements Validator<SecurityLeg> {

	@Override
	public ValidationResult<SecurityLeg> validate(RosettaPath path, SecurityLeg o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("buyerSeller", o.getBuyerSeller()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementAmount", o.getSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxRate", o.getFxRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryDate", o.getDeliveryDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryMethod", o.getDeliveryMethod()!=null ? 1 : 0, 1, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SecurityLeg", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SecurityLeg", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<SecurityLeg> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		SecurityLeg.SecurityLegBuilder o = (SecurityLeg.SecurityLegBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("buyerSeller", o.getBuyerSeller()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("settlementAmount", o.getSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 0, 1),
				checkCardinality("fxRate", o.getFxRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryDate", o.getDeliveryDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("deliveryMethod", o.getDeliveryMethod()!=null ? 1 : 0, 1, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SecurityLeg", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SecurityLeg", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
