package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Security;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SecurityValidator implements Validator<Security> {

	@Override
	public ValidationResult<Security> validate(RosettaPath path, Security o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bond", o.getBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("convertibleBond", o.getConvertibleBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("equity", o.getEquity()!=null ? 1 : 0, 0, 1),
				checkCardinality("exchangeTradedFund", o.getExchangeTradedFund()!=null ? 1 : 0, 0, 1),
				checkCardinality("mortgageBackedSecurity", o.getMortgageBackedSecurity()!=null ? 1 : 0, 0, 1),
				checkCardinality("mutualFund", o.getMutualFund()!=null ? 1 : 0, 0, 1),
				checkCardinality("warrant", o.getWarrant()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Security", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Security", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Security> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Security.SecurityBuilder o = (Security.SecurityBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("bond", o.getBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("convertibleBond", o.getConvertibleBond()!=null ? 1 : 0, 0, 1),
				checkCardinality("equity", o.getEquity()!=null ? 1 : 0, 0, 1),
				checkCardinality("exchangeTradedFund", o.getExchangeTradedFund()!=null ? 1 : 0, 0, 1),
				checkCardinality("mortgageBackedSecurity", o.getMortgageBackedSecurity()!=null ? 1 : 0, 0, 1),
				checkCardinality("mutualFund", o.getMutualFund()!=null ? 1 : 0, 0, 1),
				checkCardinality("warrant", o.getWarrant()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Security", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Security", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
