package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.SettlementProvision;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SettlementProvisionValidator implements Validator<SettlementProvision> {

	@Override
	public ValidationResult<SettlementProvision> validate(RosettaPath path, SettlementProvision o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("nonDeliverableSettlement", o.getNonDeliverableSettlement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SettlementProvision", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SettlementProvision", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<SettlementProvision> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		SettlementProvision.SettlementProvisionBuilder o = (SettlementProvision.SettlementProvisionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementCurrency", o.getSettlementCurrency()!=null ? 1 : 0, 1, 1),
				checkCardinality("nonDeliverableSettlement", o.getNonDeliverableSettlement()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SettlementProvision", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SettlementProvision", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
