package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.SettlementTerms;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SettlementTermsValidator implements Validator<SettlementTerms> {

	@Override
	public ValidationResult<SettlementTerms> validate(RosettaPath path, SettlementTerms o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementType", o.getSettlementType()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("valueDate", o.getValueDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementAmount", o.getSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("transferSettlementType", o.getTransferSettlementType()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<SettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		SettlementTerms.SettlementTermsBuilder o = (SettlementTerms.SettlementTermsBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("settlementType", o.getSettlementType()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("valueDate", o.getValueDate()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementAmount", o.getSettlementAmount()!=null ? 1 : 0, 0, 1),
				checkCardinality("transferSettlementType", o.getTransferSettlementType()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SettlementTerms", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
