package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.SimmVersion;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SimmVersionValidator implements Validator<SimmVersion> {

	@Override
	public ValidationResult<SimmVersion> validate(RosettaPath path, SimmVersion o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("isSpecified", o.getIsSpecified()!=null ? 1 : 0, 1, 1),
				checkCardinality("version", o.getVersion()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SimmVersion", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SimmVersion", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<SimmVersion> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		SimmVersion.SimmVersionBuilder o = (SimmVersion.SimmVersionBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("isSpecified", o.getIsSpecified()!=null ? 1 : 0, 1, 1),
				checkCardinality("version", o.getVersion()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SimmVersion", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("SimmVersion", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
