package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Strike;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class StrikeValidator implements Validator<Strike> {

	@Override
	public ValidationResult<Strike> validate(RosettaPath path, Strike o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("strikeRate", o.getStrikeRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("buyer", o.getBuyer()!=null ? 1 : 0, 0, 1),
				checkCardinality("seller", o.getSeller()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Strike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Strike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Strike> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Strike.StrikeBuilder o = (Strike.StrikeBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("strikeRate", o.getStrikeRate()!=null ? 1 : 0, 1, 1),
				checkCardinality("buyer", o.getBuyer()!=null ? 1 : 0, 0, 1),
				checkCardinality("seller", o.getSeller()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Strike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Strike", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
