package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.Swp;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class SwpValidator implements Validator<Swp> {

	@Override
	public ValidationResult<Swp> validate(RosettaPath path, Swp o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("swpIn", o.getSwpIn()!=null ? 1 : 0, 1, 1),
				checkCardinality("swpOut", o.getSwpOut()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Swp", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Swp", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Swp> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		Swp.SwpBuilder o = (Swp.SwpBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("swpIn", o.getSwpIn()!=null ? 1 : 0, 1, 1),
				checkCardinality("swpOut", o.getSwpOut()!=null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Swp", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("Swp", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
