package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.TransactedPrice;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class TransactedPriceValidator implements Validator<TransactedPrice> {

	@Override
	public ValidationResult<TransactedPrice> validate(RosettaPath path, TransactedPrice o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("marketFixedRate", o.getMarketFixedRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("initialPoints", o.getInitialPoints()!=null ? 1 : 0, 0, 1),
				checkCardinality("marketPrice", o.getMarketPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationStyle", o.getQuotationStyle()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TransactedPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("TransactedPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<TransactedPrice> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		TransactedPrice.TransactedPriceBuilder o = (TransactedPrice.TransactedPriceBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("marketFixedRate", o.getMarketFixedRate()!=null ? 1 : 0, 0, 1),
				checkCardinality("initialPoints", o.getInitialPoints()!=null ? 1 : 0, 0, 1),
				checkCardinality("marketPrice", o.getMarketPrice()!=null ? 1 : 0, 0, 1),
				checkCardinality("quotationStyle", o.getQuotationStyle()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TransactedPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("TransactedPrice", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
