package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.TransferPrimitive;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class TransferPrimitiveValidator implements Validator<TransferPrimitive> {

	@Override
	public ValidationResult<TransferPrimitive> validate(RosettaPath path, TransferPrimitive o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("identifier", o.getIdentifier()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementType", o.getSettlementType()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("cashTransfer", o.getCashTransfer()==null?0:o.getCashTransfer().size(), 0, 0),
				checkCardinality("securityTransfer", o.getSecurityTransfer()==null?0:o.getSecurityTransfer().size(), 0, 0),
				checkCardinality("commodityTransfer", o.getCommodityTransfer()==null?0:o.getCommodityTransfer().size(), 0, 0),
				checkCardinality("status", o.getStatus()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementReference", o.getSettlementReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TransferPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("TransferPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<TransferPrimitive> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		TransferPrimitive.TransferPrimitiveBuilder o = (TransferPrimitive.TransferPrimitiveBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("identifier", o.getIdentifier()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementType", o.getSettlementType()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementDate", o.getSettlementDate()!=null ? 1 : 0, 1, 1),
				checkCardinality("cashTransfer", o.getCashTransfer()==null?0:o.getCashTransfer().size(), 0, 0),
				checkCardinality("securityTransfer", o.getSecurityTransfer()==null?0:o.getSecurityTransfer().size(), 0, 0),
				checkCardinality("commodityTransfer", o.getCommodityTransfer()==null?0:o.getCommodityTransfer().size(), 0, 0),
				checkCardinality("status", o.getStatus()!=null ? 1 : 0, 0, 1),
				checkCardinality("settlementReference", o.getSettlementReference()!=null ? 1 : 0, 0, 1),
				checkCardinality("meta", o.getMeta()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TransferPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("TransferPrimitive", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
