package org.isda.cdm.validation;

import static com.rosetta.model.lib.validation.ValidatorHelper.checkCardinality;
import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.joining;
import static java.util.Optional.ofNullable;
import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;


import org.isda.cdm.UmbrellaAgreementEntity;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;


public class UmbrellaAgreementEntityValidator implements Validator<UmbrellaAgreementEntity> {

	@Override
	public ValidationResult<UmbrellaAgreementEntity> validate(RosettaPath path, UmbrellaAgreementEntity o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("terms", o.getTerms()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("UmbrellaAgreementEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("UmbrellaAgreementEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<UmbrellaAgreementEntity> validate(RosettaPath path, RosettaModelObjectBuilder b) {
		UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder o = (UmbrellaAgreementEntity.UmbrellaAgreementEntityBuilder) b;
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("terms", o.getTerms()!=null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("UmbrellaAgreementEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "", error);
		}
		return success("UmbrellaAgreementEntity", ValidationType.MODEL_INSTANCE, o.getClass().getSimpleName(), path, "");
	}

}
