package org.isda.cdm.validation.choicerule;

import org.isda.cdm.AdjustableDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("AdjustableDate_choice")
public class AdjustableDateChoiceRule implements Validator<AdjustableDate> {
	
	private static final String NAME = "AdjustableDate_choice";
	
    @Override
    public ValidationResult<AdjustableDate> validate(RosettaPath path, AdjustableDate object) {
		List<String> choiceFieldNames = Arrays.asList("dateAdjustments", "dateAdjustmentsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateAdjustments())) populatedFieldNames.add("dateAdjustments");
		if (isSet(object.getDateAdjustmentsReference())) populatedFieldNames.add("dateAdjustmentsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableDate>(NAME, "AdjustableDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<AdjustableDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	AdjustableDate.AdjustableDateBuilder object = (AdjustableDate.AdjustableDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("dateAdjustments", "dateAdjustmentsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateAdjustments())) populatedFieldNames.add("dateAdjustments");
		if (isSet(object.getDateAdjustmentsReference())) populatedFieldNames.add("dateAdjustmentsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableDate>(NAME, "AdjustableDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
