package org.isda.cdm.validation.choicerule;

import org.isda.cdm.AdjustableOrRelativeDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("AdjustableOrRelativeDate_choice")
public class AdjustableOrRelativeDateChoiceRule implements Validator<AdjustableOrRelativeDate> {
	
	private static final String NAME = "AdjustableOrRelativeDate_choice";
	
    @Override
    public ValidationResult<AdjustableOrRelativeDate> validate(RosettaPath path, AdjustableOrRelativeDate object) {
		List<String> choiceFieldNames = Arrays.asList("adjustableDate", "relativeDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDate())) populatedFieldNames.add("adjustableDate");
		if (isSet(object.getRelativeDate())) populatedFieldNames.add("relativeDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableOrRelativeDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableOrRelativeDate>(NAME, "AdjustableOrRelativeDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<AdjustableOrRelativeDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder object = (AdjustableOrRelativeDate.AdjustableOrRelativeDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("adjustableDate", "relativeDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDate())) populatedFieldNames.add("adjustableDate");
		if (isSet(object.getRelativeDate())) populatedFieldNames.add("relativeDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableOrRelativeDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableOrRelativeDate>(NAME, "AdjustableOrRelativeDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
