package org.isda.cdm.validation.choicerule;

import org.isda.cdm.AdjustableOrRelativeDates;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("AdjustableOrRelativeDates_choice")
public class AdjustableOrRelativeDatesChoiceRule implements Validator<AdjustableOrRelativeDates> {
	
	private static final String NAME = "AdjustableOrRelativeDates_choice";
	
    @Override
    public ValidationResult<AdjustableOrRelativeDates> validate(RosettaPath path, AdjustableOrRelativeDates object) {
		List<String> choiceFieldNames = Arrays.asList("adjustableDates", "relativeDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDates())) populatedFieldNames.add("adjustableDates");
		if (isSet(object.getRelativeDates())) populatedFieldNames.add("relativeDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableOrRelativeDates", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableOrRelativeDates>(NAME, "AdjustableOrRelativeDates", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<AdjustableOrRelativeDates> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder object = (AdjustableOrRelativeDates.AdjustableOrRelativeDatesBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("adjustableDates", "relativeDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDates())) populatedFieldNames.add("adjustableDates");
		if (isSet(object.getRelativeDates())) populatedFieldNames.add("relativeDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AdjustableOrRelativeDates", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AdjustableOrRelativeDates>(NAME, "AdjustableOrRelativeDates", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
