package org.isda.cdm.validation.choicerule;

import org.isda.cdm.AmendmentEffectiveDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("AmendmentEffectiveDate_oneOf")
public class AmendmentEffectiveDateOneOfRule implements Validator<AmendmentEffectiveDate> {
	
	private static final String NAME = "AmendmentEffectiveDate_oneOf";
	
    @Override
    public ValidationResult<AmendmentEffectiveDate> validate(RosettaPath path, AmendmentEffectiveDate object) {
		List<String> choiceFieldNames = Arrays.asList("date", "specificDate", "customProvision");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDate())) populatedFieldNames.add("date");
		if (isSet(object.getSpecificDate())) populatedFieldNames.add("specificDate");
		if (isSet(object.getCustomProvision())) populatedFieldNames.add("customProvision");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AmendmentEffectiveDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AmendmentEffectiveDate>(NAME, "AmendmentEffectiveDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<AmendmentEffectiveDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	AmendmentEffectiveDate.AmendmentEffectiveDateBuilder object = (AmendmentEffectiveDate.AmendmentEffectiveDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("date", "specificDate", "customProvision");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDate())) populatedFieldNames.add("date");
		if (isSet(object.getSpecificDate())) populatedFieldNames.add("specificDate");
		if (isSet(object.getCustomProvision())) populatedFieldNames.add("customProvision");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AmendmentEffectiveDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AmendmentEffectiveDate>(NAME, "AmendmentEffectiveDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
