package org.isda.cdm.validation.choicerule;

import org.isda.cdm.AveragingPeriod;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("AveragingPeriod_choice")
public class AveragingPeriodChoiceRule implements Validator<AveragingPeriod> {
	
	private static final String NAME = "AveragingPeriod_choice";
	
    @Override
    public ValidationResult<AveragingPeriod> validate(RosettaPath path, AveragingPeriod object) {
		List<String> choiceFieldNames = Arrays.asList("averagingDateTimes", "averagingObservations");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAveragingDateTimes())) populatedFieldNames.add("averagingDateTimes");
		if (isSet(object.getAveragingObservations())) populatedFieldNames.add("averagingObservations");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AveragingPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AveragingPeriod>(NAME, "AveragingPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<AveragingPeriod> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	AveragingPeriod.AveragingPeriodBuilder object = (AveragingPeriod.AveragingPeriodBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("averagingDateTimes", "averagingObservations");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAveragingDateTimes())) populatedFieldNames.add("averagingDateTimes");
		if (isSet(object.getAveragingObservations())) populatedFieldNames.add("averagingObservations");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "AveragingPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<AveragingPeriod>(NAME, "AveragingPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
