package org.isda.cdm.validation.choicerule;

import org.isda.cdm.BasketReferenceInformation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("BasketReferenceInformation_choice")
public class BasketReferenceInformationChoiceRule implements Validator<BasketReferenceInformation> {
	
	private static final String NAME = "BasketReferenceInformation_choice";
	
    @Override
    public ValidationResult<BasketReferenceInformation> validate(RosettaPath path, BasketReferenceInformation object) {
		List<String> choiceFieldNames = Arrays.asList("nthToDefault", "tranche");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNthToDefault())) populatedFieldNames.add("nthToDefault");
		if (isSet(object.getTranche())) populatedFieldNames.add("tranche");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BasketReferenceInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BasketReferenceInformation>(NAME, "BasketReferenceInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<BasketReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	BasketReferenceInformation.BasketReferenceInformationBuilder object = (BasketReferenceInformation.BasketReferenceInformationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("nthToDefault", "tranche");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNthToDefault())) populatedFieldNames.add("nthToDefault");
		if (isSet(object.getTranche())) populatedFieldNames.add("tranche");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BasketReferenceInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BasketReferenceInformation>(NAME, "BasketReferenceInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
