package org.isda.cdm.validation.choicerule;

import org.isda.cdm.BondChoiceModel;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("BondChoiceModel_oneOf")
public class BondChoiceModelOneOfRule implements Validator<BondChoiceModel> {
	
	private static final String NAME = "BondChoiceModel_oneOf";
	
    @Override
    public ValidationResult<BondChoiceModel> validate(RosettaPath path, BondChoiceModel object) {
		List<String> choiceFieldNames = Arrays.asList("bond", "convertibleBond");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBond())) populatedFieldNames.add("bond");
		if (isSet(object.getConvertibleBond())) populatedFieldNames.add("convertibleBond");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondChoiceModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondChoiceModel>(NAME, "BondChoiceModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<BondChoiceModel> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	BondChoiceModel.BondChoiceModelBuilder object = (BondChoiceModel.BondChoiceModelBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("bond", "convertibleBond");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBond())) populatedFieldNames.add("bond");
		if (isSet(object.getConvertibleBond())) populatedFieldNames.add("convertibleBond");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondChoiceModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondChoiceModel>(NAME, "BondChoiceModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
