package org.isda.cdm.validation.choicerule;

import org.isda.cdm.BondEquityModel;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("BondEquityModel_oneOf")
public class BondEquityModelOneOfRule implements Validator<BondEquityModel> {
	
	private static final String NAME = "BondEquityModel_oneOf";
	
    @Override
    public ValidationResult<BondEquityModel> validate(RosettaPath path, BondEquityModel object) {
		List<String> choiceFieldNames = Arrays.asList("bondchoiceModel", "equity");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBondchoiceModel())) populatedFieldNames.add("bondchoiceModel");
		if (isSet(object.getEquity())) populatedFieldNames.add("equity");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondEquityModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondEquityModel>(NAME, "BondEquityModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<BondEquityModel> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	BondEquityModel.BondEquityModelBuilder object = (BondEquityModel.BondEquityModelBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("bondchoiceModel", "equity");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBondchoiceModel())) populatedFieldNames.add("bondchoiceModel");
		if (isSet(object.getEquity())) populatedFieldNames.add("equity");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondEquityModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondEquityModel>(NAME, "BondEquityModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
