package org.isda.cdm.validation.choicerule;

import org.isda.cdm.BondOptionStrike;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("BondOptionStrike_oneOf")
public class BondOptionStrikeOneOfRule implements Validator<BondOptionStrike> {
	
	private static final String NAME = "BondOptionStrike_oneOf";
	
    @Override
    public ValidationResult<BondOptionStrike> validate(RosettaPath path, BondOptionStrike object) {
		List<String> choiceFieldNames = Arrays.asList("referenceSwapCurve", "price");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceSwapCurve())) populatedFieldNames.add("referenceSwapCurve");
		if (isSet(object.getPrice())) populatedFieldNames.add("price");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondOptionStrike", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondOptionStrike>(NAME, "BondOptionStrike", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<BondOptionStrike> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	BondOptionStrike.BondOptionStrikeBuilder object = (BondOptionStrike.BondOptionStrikeBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("referenceSwapCurve", "price");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceSwapCurve())) populatedFieldNames.add("referenceSwapCurve");
		if (isSet(object.getPrice())) populatedFieldNames.add("price");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BondOptionStrike", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BondOptionStrike>(NAME, "BondOptionStrike", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
