package org.isda.cdm.validation.choicerule;

import org.isda.cdm.BusinessCenters;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("BusinessCenters_choice")
public class BusinessCentersChoiceRule implements Validator<BusinessCenters> {
	
	private static final String NAME = "BusinessCenters_choice";
	
    @Override
    public ValidationResult<BusinessCenters> validate(RosettaPath path, BusinessCenters object) {
		List<String> choiceFieldNames = Arrays.asList("businessCenter", "businessCentersReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessCenter())) populatedFieldNames.add("businessCenter");
		if (isSet(object.getBusinessCentersReference())) populatedFieldNames.add("businessCentersReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BusinessCenters", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BusinessCenters>(NAME, "BusinessCenters", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<BusinessCenters> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	BusinessCenters.BusinessCentersBuilder object = (BusinessCenters.BusinessCentersBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("businessCenter", "businessCentersReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessCenter())) populatedFieldNames.add("businessCenter");
		if (isSet(object.getBusinessCentersReference())) populatedFieldNames.add("businessCentersReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "BusinessCenters", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<BusinessCenters>(NAME, "BusinessCenters", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
