package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CalculationAgent;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CalculationAgent_choice")
public class CalculationAgentChoiceRule implements Validator<CalculationAgent> {
	
	private static final String NAME = "CalculationAgent_choice";
	
    @Override
    public ValidationResult<CalculationAgent> validate(RosettaPath path, CalculationAgent object) {
		List<String> choiceFieldNames = Arrays.asList("calculationAgentPartyReference", "calculationAgentParty");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCalculationAgentPartyReference())) populatedFieldNames.add("calculationAgentPartyReference");
		if (isSet(object.getCalculationAgentParty())) populatedFieldNames.add("calculationAgentParty");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationAgent", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationAgent>(NAME, "CalculationAgent", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CalculationAgent> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CalculationAgent.CalculationAgentBuilder object = (CalculationAgent.CalculationAgentBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("calculationAgentPartyReference", "calculationAgentParty");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCalculationAgentPartyReference())) populatedFieldNames.add("calculationAgentPartyReference");
		if (isSet(object.getCalculationAgentParty())) populatedFieldNames.add("calculationAgentParty");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationAgent", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationAgent>(NAME, "CalculationAgent", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
