package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CalculationDateLocationElection;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CalculationDateLocationElection_choice")
public class CalculationDateLocationElectionChoiceRule implements Validator<CalculationDateLocationElection> {
	
	private static final String NAME = "CalculationDateLocationElection_choice";
	
    @Override
    public ValidationResult<CalculationDateLocationElection> validate(RosettaPath path, CalculationDateLocationElection object) {
		List<String> choiceFieldNames = Arrays.asList("businessCenter", "customLocation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessCenter())) populatedFieldNames.add("businessCenter");
		if (isSet(object.getCustomLocation())) populatedFieldNames.add("customLocation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationDateLocationElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationDateLocationElection>(NAME, "CalculationDateLocationElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CalculationDateLocationElection> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CalculationDateLocationElection.CalculationDateLocationElectionBuilder object = (CalculationDateLocationElection.CalculationDateLocationElectionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("businessCenter", "customLocation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessCenter())) populatedFieldNames.add("businessCenter");
		if (isSet(object.getCustomLocation())) populatedFieldNames.add("customLocation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationDateLocationElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationDateLocationElection>(NAME, "CalculationDateLocationElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
