package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CalculationPeriod;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CalculationPeriod_choice1")
public class CalculationPeriodChoice1ChoiceRule implements Validator<CalculationPeriod> {
	
	private static final String NAME = "CalculationPeriod_choice1";
	
    @Override
    public ValidationResult<CalculationPeriod> validate(RosettaPath path, CalculationPeriod object) {
		List<String> choiceFieldNames = Arrays.asList("notionalAmount", "fxLinkedNotionalAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNotionalAmount())) populatedFieldNames.add("notionalAmount");
		if (isSet(object.getFxLinkedNotionalAmount())) populatedFieldNames.add("fxLinkedNotionalAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationPeriod>(NAME, "CalculationPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CalculationPeriod.CalculationPeriodBuilder object = (CalculationPeriod.CalculationPeriodBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("notionalAmount", "fxLinkedNotionalAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNotionalAmount())) populatedFieldNames.add("notionalAmount");
		if (isSet(object.getFxLinkedNotionalAmount())) populatedFieldNames.add("fxLinkedNotionalAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CalculationPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CalculationPeriod>(NAME, "CalculationPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
