package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CashSettlementPaymentDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CashSettlementPaymentDate_choice")
public class CashSettlementPaymentDateChoiceRule implements Validator<CashSettlementPaymentDate> {
	
	private static final String NAME = "CashSettlementPaymentDate_choice";
	
    @Override
    public ValidationResult<CashSettlementPaymentDate> validate(RosettaPath path, CashSettlementPaymentDate object) {
		List<String> choiceFieldNames = Arrays.asList("adjustableDates", "relativeDate", "businessDateRange");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDates())) populatedFieldNames.add("adjustableDates");
		if (isSet(object.getRelativeDate())) populatedFieldNames.add("relativeDate");
		if (isSet(object.getBusinessDateRange())) populatedFieldNames.add("businessDateRange");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CashSettlementPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CashSettlementPaymentDate>(NAME, "CashSettlementPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CashSettlementPaymentDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CashSettlementPaymentDate.CashSettlementPaymentDateBuilder object = (CashSettlementPaymentDate.CashSettlementPaymentDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("adjustableDates", "relativeDate", "businessDateRange");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdjustableDates())) populatedFieldNames.add("adjustableDates");
		if (isSet(object.getRelativeDate())) populatedFieldNames.add("relativeDate");
		if (isSet(object.getBusinessDateRange())) populatedFieldNames.add("businessDateRange");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CashSettlementPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CashSettlementPaymentDate>(NAME, "CashSettlementPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
