package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CashSettlementTerms;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CashSettlementTerms_choice")
public class CashSettlementTermsChoiceRule implements Validator<CashSettlementTerms> {
	
	private static final String NAME = "CashSettlementTerms_choice";
	
    @Override
    public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms object) {
		List<String> choiceFieldNames = Arrays.asList("cashSettlementAmount", "recoveryFactor");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementAmount())) populatedFieldNames.add("cashSettlementAmount");
		if (isSet(object.getRecoveryFactor())) populatedFieldNames.add("recoveryFactor");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CashSettlementTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CashSettlementTerms>(NAME, "CashSettlementTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CashSettlementTerms.CashSettlementTermsBuilder object = (CashSettlementTerms.CashSettlementTermsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("cashSettlementAmount", "recoveryFactor");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementAmount())) populatedFieldNames.add("cashSettlementAmount");
		if (isSet(object.getRecoveryFactor())) populatedFieldNames.add("recoveryFactor");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CashSettlementTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CashSettlementTerms>(NAME, "CashSettlementTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
