package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ContractualQuantity;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ContractualQuantity_choice")
public class ContractualQuantityChoiceRule implements Validator<ContractualQuantity> {
	
	private static final String NAME = "ContractualQuantity_choice";
	
    @Override
    public ValidationResult<ContractualQuantity> validate(RosettaPath path, ContractualQuantity object) {
		List<String> choiceFieldNames = Arrays.asList("quantity", "notionalAmount", "notionalSchedule", "fxLinkedNotional");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getQuantity())) populatedFieldNames.add("quantity");
		if (isSet(object.getNotionalAmount())) populatedFieldNames.add("notionalAmount");
		if (isSet(object.getNotionalSchedule())) populatedFieldNames.add("notionalSchedule");
		if (isSet(object.getFxLinkedNotional())) populatedFieldNames.add("fxLinkedNotional");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ContractualQuantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ContractualQuantity>(NAME, "ContractualQuantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ContractualQuantity> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ContractualQuantity.ContractualQuantityBuilder object = (ContractualQuantity.ContractualQuantityBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("quantity", "notionalAmount", "notionalSchedule", "fxLinkedNotional");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getQuantity())) populatedFieldNames.add("quantity");
		if (isSet(object.getNotionalAmount())) populatedFieldNames.add("notionalAmount");
		if (isSet(object.getNotionalSchedule())) populatedFieldNames.add("notionalSchedule");
		if (isSet(object.getFxLinkedNotional())) populatedFieldNames.add("fxLinkedNotional");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ContractualQuantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ContractualQuantity>(NAME, "ContractualQuantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
