package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CreditDefaultPayout;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CreditDefaultPayout_choice")
public class CreditDefaultPayoutChoiceRule implements Validator<CreditDefaultPayout> {
	
	private static final String NAME = "CreditDefaultPayout_choice";
	
    @Override
    public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, CreditDefaultPayout object) {
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTerms", "physicalSettlementTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTerms())) populatedFieldNames.add("cashSettlementTerms");
		if (isSet(object.getPhysicalSettlementTerms())) populatedFieldNames.add("physicalSettlementTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditDefaultPayout", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditDefaultPayout>(NAME, "CreditDefaultPayout", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CreditDefaultPayout> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CreditDefaultPayout.CreditDefaultPayoutBuilder object = (CreditDefaultPayout.CreditDefaultPayoutBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTerms", "physicalSettlementTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTerms())) populatedFieldNames.add("cashSettlementTerms");
		if (isSet(object.getPhysicalSettlementTerms())) populatedFieldNames.add("physicalSettlementTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditDefaultPayout", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditDefaultPayout>(NAME, "CreditDefaultPayout", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
