package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CreditNotations;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CreditNotations_oneOf")
public class CreditNotationsOneOfRule implements Validator<CreditNotations> {
	
	private static final String NAME = "CreditNotations_oneOf";
	
    @Override
    public ValidationResult<CreditNotations> validate(RosettaPath path, CreditNotations object) {
		List<String> choiceFieldNames = Arrays.asList("creditNotation", "creditNotations");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCreditNotation())) populatedFieldNames.add("creditNotation");
		if (isSet(object.getCreditNotations())) populatedFieldNames.add("creditNotations");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditNotations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditNotations>(NAME, "CreditNotations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CreditNotations> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CreditNotations.CreditNotationsBuilder object = (CreditNotations.CreditNotationsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("creditNotation", "creditNotations");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCreditNotation())) populatedFieldNames.add("creditNotation");
		if (isSet(object.getCreditNotations())) populatedFieldNames.add("creditNotations");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditNotations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditNotations>(NAME, "CreditNotations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
