package org.isda.cdm.validation.choicerule;

import org.isda.cdm.CreditRatingDebt;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("CreditRatingDebt_oneOf")
public class CreditRatingDebtOneOfRule implements Validator<CreditRatingDebt> {
	
	private static final String NAME = "CreditRatingDebt_oneOf";
	
    @Override
    public ValidationResult<CreditRatingDebt> validate(RosettaPath path, CreditRatingDebt object) {
		List<String> choiceFieldNames = Arrays.asList("debtType", "debtTypes");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDebtType())) populatedFieldNames.add("debtType");
		if (isSet(object.getDebtTypes())) populatedFieldNames.add("debtTypes");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditRatingDebt", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditRatingDebt>(NAME, "CreditRatingDebt", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<CreditRatingDebt> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	CreditRatingDebt.CreditRatingDebtBuilder object = (CreditRatingDebt.CreditRatingDebtBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("debtType", "debtTypes");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDebtType())) populatedFieldNames.add("debtType");
		if (isSet(object.getDebtTypes())) populatedFieldNames.add("debtTypes");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "CreditRatingDebt", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<CreditRatingDebt>(NAME, "CreditRatingDebt", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
