package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Curve;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Curve_oneOf")
public class CurveOneOfRule implements Validator<Curve> {
	
	private static final String NAME = "Curve_oneOf";
	
    @Override
    public ValidationResult<Curve> validate(RosettaPath path, Curve object) {
		List<String> choiceFieldNames = Arrays.asList("interestRateCurve", "commodityCurve");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getInterestRateCurve())) populatedFieldNames.add("interestRateCurve");
		if (isSet(object.getCommodityCurve())) populatedFieldNames.add("commodityCurve");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Curve", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Curve>(NAME, "Curve", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Curve> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Curve.CurveBuilder object = (Curve.CurveBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("interestRateCurve", "commodityCurve");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getInterestRateCurve())) populatedFieldNames.add("interestRateCurve");
		if (isSet(object.getCommodityCurve())) populatedFieldNames.add("commodityCurve");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Curve", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Curve>(NAME, "Curve", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
