package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DeliverableObligations;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DeliverableObligations_choice")
public class DeliverableObligationsChoiceRule implements Validator<DeliverableObligations> {
	
	private static final String NAME = "DeliverableObligations_choice";
	
    @Override
    public ValidationResult<DeliverableObligations> validate(RosettaPath path, DeliverableObligations object) {
		List<String> choiceFieldNames = Arrays.asList("fullFaithAndCreditObLiability", "generalFundObligationLiability", "revenueObligationLiability");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFullFaithAndCreditObLiability())) populatedFieldNames.add("fullFaithAndCreditObLiability");
		if (isSet(object.getGeneralFundObligationLiability())) populatedFieldNames.add("generalFundObligationLiability");
		if (isSet(object.getRevenueObligationLiability())) populatedFieldNames.add("revenueObligationLiability");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DeliverableObligations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DeliverableObligations>(NAME, "DeliverableObligations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DeliverableObligations> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DeliverableObligations.DeliverableObligationsBuilder object = (DeliverableObligations.DeliverableObligationsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("fullFaithAndCreditObLiability", "generalFundObligationLiability", "revenueObligationLiability");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFullFaithAndCreditObLiability())) populatedFieldNames.add("fullFaithAndCreditObLiability");
		if (isSet(object.getGeneralFundObligationLiability())) populatedFieldNames.add("generalFundObligationLiability");
		if (isSet(object.getRevenueObligationLiability())) populatedFieldNames.add("revenueObligationLiability");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DeliverableObligations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DeliverableObligations>(NAME, "DeliverableObligations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
