package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DeliveryAmount;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DeliveryAmount_oneOf")
public class DeliveryAmountOneOfRule implements Validator<DeliveryAmount> {
	
	private static final String NAME = "DeliveryAmount_oneOf";
	
    @Override
    public ValidationResult<DeliveryAmount> validate(RosettaPath path, DeliveryAmount object) {
		List<String> choiceFieldNames = Arrays.asList("standardElection", "customElection");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardElection())) populatedFieldNames.add("standardElection");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DeliveryAmount", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DeliveryAmount>(NAME, "DeliveryAmount", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DeliveryAmount> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DeliveryAmount.DeliveryAmountBuilder object = (DeliveryAmount.DeliveryAmountBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("standardElection", "customElection");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardElection())) populatedFieldNames.add("standardElection");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DeliveryAmount", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DeliveryAmount>(NAME, "DeliveryAmount", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
