package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DisputeResolution;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DisputeResolution_oneOf")
public class DisputeResolutionOneOfRule implements Validator<DisputeResolution> {
	
	private static final String NAME = "DisputeResolution_oneOf";
	
    @Override
    public ValidationResult<DisputeResolution> validate(RosettaPath path, DisputeResolution object) {
		List<String> choiceFieldNames = Arrays.asList("resolutionTime", "otherTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getResolutionTime())) populatedFieldNames.add("resolutionTime");
		if (isSet(object.getOtherTerms())) populatedFieldNames.add("otherTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DisputeResolution", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DisputeResolution>(NAME, "DisputeResolution", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DisputeResolution> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DisputeResolution.DisputeResolutionBuilder object = (DisputeResolution.DisputeResolutionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("resolutionTime", "otherTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getResolutionTime())) populatedFieldNames.add("resolutionTime");
		if (isSet(object.getOtherTerms())) populatedFieldNames.add("otherTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DisputeResolution", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DisputeResolution>(NAME, "DisputeResolution", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
