package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DividendCurrency;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DividendCurrency_oneOf")
public class DividendCurrencyOneOfRule implements Validator<DividendCurrency> {
	
	private static final String NAME = "DividendCurrency_oneOf";
	
    @Override
    public ValidationResult<DividendCurrency> validate(RosettaPath path, DividendCurrency object) {
		List<String> choiceFieldNames = Arrays.asList("currency", "determinationMethod", "currencyReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		if (isSet(object.getDeterminationMethod())) populatedFieldNames.add("determinationMethod");
		if (isSet(object.getCurrencyReference())) populatedFieldNames.add("currencyReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DividendCurrency", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DividendCurrency>(NAME, "DividendCurrency", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DividendCurrency> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DividendCurrency.DividendCurrencyBuilder object = (DividendCurrency.DividendCurrencyBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("currency", "determinationMethod", "currencyReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		if (isSet(object.getDeterminationMethod())) populatedFieldNames.add("determinationMethod");
		if (isSet(object.getCurrencyReference())) populatedFieldNames.add("currencyReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DividendCurrency", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DividendCurrency>(NAME, "DividendCurrency", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
