package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DividendPaymentDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DividendPaymentDate_oneOf")
public class DividendPaymentDateOneOfRule implements Validator<DividendPaymentDate> {
	
	private static final String NAME = "DividendPaymentDate_oneOf";
	
    @Override
    public ValidationResult<DividendPaymentDate> validate(RosettaPath path, DividendPaymentDate object) {
		List<String> choiceFieldNames = Arrays.asList("dividendDateReference", "adjustableDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDividendDateReference())) populatedFieldNames.add("dividendDateReference");
		if (isSet(object.getAdjustableDate())) populatedFieldNames.add("adjustableDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DividendPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DividendPaymentDate>(NAME, "DividendPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DividendPaymentDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DividendPaymentDate.DividendPaymentDateBuilder object = (DividendPaymentDate.DividendPaymentDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("dividendDateReference", "adjustableDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDividendDateReference())) populatedFieldNames.add("dividendDateReference");
		if (isSet(object.getAdjustableDate())) populatedFieldNames.add("adjustableDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DividendPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DividendPaymentDate>(NAME, "DividendPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
