package org.isda.cdm.validation.choicerule;

import org.isda.cdm.DocumentationIdentification;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("DocumentationIdentification_choice")
public class DocumentationIdentificationChoiceRule implements Validator<DocumentationIdentification> {
	
	private static final String NAME = "DocumentationIdentification_choice";
	
    @Override
    public ValidationResult<DocumentationIdentification> validate(RosettaPath path, DocumentationIdentification object) {
		List<String> choiceFieldNames = Arrays.asList("masterConfirmation", "brokerConfirmation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMasterConfirmation())) populatedFieldNames.add("masterConfirmation");
		if (isSet(object.getBrokerConfirmation())) populatedFieldNames.add("brokerConfirmation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DocumentationIdentification", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DocumentationIdentification>(NAME, "DocumentationIdentification", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<DocumentationIdentification> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	DocumentationIdentification.DocumentationIdentificationBuilder object = (DocumentationIdentification.DocumentationIdentificationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("masterConfirmation", "brokerConfirmation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMasterConfirmation())) populatedFieldNames.add("masterConfirmation");
		if (isSet(object.getBrokerConfirmation())) populatedFieldNames.add("brokerConfirmation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "DocumentationIdentification", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<DocumentationIdentification>(NAME, "DocumentationIdentification", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
