package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ElectiveAmountElection;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ElectiveAmountElection_choice")
public class ElectiveAmountElectionChoiceRule implements Validator<ElectiveAmountElection> {
	
	private static final String NAME = "ElectiveAmountElection_choice";
	
    @Override
    public ValidationResult<ElectiveAmountElection> validate(RosettaPath path, ElectiveAmountElection object) {
		List<String> choiceFieldNames = Arrays.asList("amount", "customElection", "noAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmount())) populatedFieldNames.add("amount");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		if (isSet(object.getNoAmount())) populatedFieldNames.add("noAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ElectiveAmountElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ElectiveAmountElection>(NAME, "ElectiveAmountElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ElectiveAmountElection> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ElectiveAmountElection.ElectiveAmountElectionBuilder object = (ElectiveAmountElection.ElectiveAmountElectionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("amount", "customElection", "noAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmount())) populatedFieldNames.add("amount");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		if (isSet(object.getNoAmount())) populatedFieldNames.add("noAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ElectiveAmountElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ElectiveAmountElection>(NAME, "ElectiveAmountElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
