package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExerciseFee;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExerciseFee_choice")
public class ExerciseFeeChoiceRule implements Validator<ExerciseFee> {
	
	private static final String NAME = "ExerciseFee_choice";
	
    @Override
    public ValidationResult<ExerciseFee> validate(RosettaPath path, ExerciseFee object) {
		List<String> choiceFieldNames = Arrays.asList("feeAmount", "feeRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFeeAmount())) populatedFieldNames.add("feeAmount");
		if (isSet(object.getFeeRate())) populatedFieldNames.add("feeRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseFee", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseFee>(NAME, "ExerciseFee", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExerciseFee> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExerciseFee.ExerciseFeeBuilder object = (ExerciseFee.ExerciseFeeBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("feeAmount", "feeRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFeeAmount())) populatedFieldNames.add("feeAmount");
		if (isSet(object.getFeeRate())) populatedFieldNames.add("feeRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseFee", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseFee>(NAME, "ExerciseFee", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
