package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExerciseFeeSchedule;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExerciseFeeSchedule_choice")
public class ExerciseFeeScheduleChoiceRule implements Validator<ExerciseFeeSchedule> {
	
	private static final String NAME = "ExerciseFeeSchedule_choice";
	
    @Override
    public ValidationResult<ExerciseFeeSchedule> validate(RosettaPath path, ExerciseFeeSchedule object) {
		List<String> choiceFieldNames = Arrays.asList("feeAmountSchedule", "feeRateSchedule");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFeeAmountSchedule())) populatedFieldNames.add("feeAmountSchedule");
		if (isSet(object.getFeeRateSchedule())) populatedFieldNames.add("feeRateSchedule");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseFeeSchedule", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseFeeSchedule>(NAME, "ExerciseFeeSchedule", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExerciseFeeSchedule> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExerciseFeeSchedule.ExerciseFeeScheduleBuilder object = (ExerciseFeeSchedule.ExerciseFeeScheduleBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("feeAmountSchedule", "feeRateSchedule");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFeeAmountSchedule())) populatedFieldNames.add("feeAmountSchedule");
		if (isSet(object.getFeeRateSchedule())) populatedFieldNames.add("feeRateSchedule");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseFeeSchedule", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseFeeSchedule>(NAME, "ExerciseFeeSchedule", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
