package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExerciseOutcome;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExerciseOutcome_choice")
public class ExerciseOutcomeChoiceRule implements Validator<ExerciseOutcome> {
	
	private static final String NAME = "ExerciseOutcome_choice";
	
    @Override
    public ValidationResult<ExerciseOutcome> validate(RosettaPath path, ExerciseOutcome object) {
		List<String> choiceFieldNames = Arrays.asList("physicalExercise", "cashExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPhysicalExercise())) populatedFieldNames.add("physicalExercise");
		if (isSet(object.getCashExercise())) populatedFieldNames.add("cashExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseOutcome", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseOutcome>(NAME, "ExerciseOutcome", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExerciseOutcome> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExerciseOutcome.ExerciseOutcomeBuilder object = (ExerciseOutcome.ExerciseOutcomeBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("physicalExercise", "cashExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPhysicalExercise())) populatedFieldNames.add("physicalExercise");
		if (isSet(object.getCashExercise())) populatedFieldNames.add("cashExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseOutcome", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseOutcome>(NAME, "ExerciseOutcome", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
