package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExerciseProcedure;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExerciseProcedure_choice")
public class ExerciseProcedureChoiceRule implements Validator<ExerciseProcedure> {
	
	private static final String NAME = "ExerciseProcedure_choice";
	
    @Override
    public ValidationResult<ExerciseProcedure> validate(RosettaPath path, ExerciseProcedure object) {
		List<String> choiceFieldNames = Arrays.asList("manualExercise", "automaticExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getManualExercise())) populatedFieldNames.add("manualExercise");
		if (isSet(object.getAutomaticExercise())) populatedFieldNames.add("automaticExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseProcedure", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseProcedure>(NAME, "ExerciseProcedure", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExerciseProcedure> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExerciseProcedure.ExerciseProcedureBuilder object = (ExerciseProcedure.ExerciseProcedureBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("manualExercise", "automaticExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getManualExercise())) populatedFieldNames.add("manualExercise");
		if (isSet(object.getAutomaticExercise())) populatedFieldNames.add("automaticExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExerciseProcedure", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExerciseProcedure>(NAME, "ExerciseProcedure", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
