package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExtendibleProvision;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExtendibleProvision_choice")
public class ExtendibleProvisionChoiceRule implements Validator<ExtendibleProvision> {
	
	private static final String NAME = "ExtendibleProvision_choice";
	
    @Override
    public ValidationResult<ExtendibleProvision> validate(RosettaPath path, ExtendibleProvision object) {
		List<String> choiceFieldNames = Arrays.asList("americanExercise", "bermudaExercise", "europeanExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmericanExercise())) populatedFieldNames.add("americanExercise");
		if (isSet(object.getBermudaExercise())) populatedFieldNames.add("bermudaExercise");
		if (isSet(object.getEuropeanExercise())) populatedFieldNames.add("europeanExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExtendibleProvision", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExtendibleProvision>(NAME, "ExtendibleProvision", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExtendibleProvision> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExtendibleProvision.ExtendibleProvisionBuilder object = (ExtendibleProvision.ExtendibleProvisionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("americanExercise", "bermudaExercise", "europeanExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmericanExercise())) populatedFieldNames.add("americanExercise");
		if (isSet(object.getBermudaExercise())) populatedFieldNames.add("bermudaExercise");
		if (isSet(object.getEuropeanExercise())) populatedFieldNames.add("europeanExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExtendibleProvision", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExtendibleProvision>(NAME, "ExtendibleProvision", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
