package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ExtraordinaryEvents;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ExtraordinaryEvents_choice")
public class ExtraordinaryEventsChoiceRule implements Validator<ExtraordinaryEvents> {
	
	private static final String NAME = "ExtraordinaryEvents_choice";
	
    @Override
    public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, ExtraordinaryEvents object) {
		List<String> choiceFieldNames = Arrays.asList("additionalDisruptionEvents", "failureToDeliver");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdditionalDisruptionEvents())) populatedFieldNames.add("additionalDisruptionEvents");
		if (isSet(object.getFailureToDeliver())) populatedFieldNames.add("failureToDeliver");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExtraordinaryEvents", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExtraordinaryEvents>(NAME, "ExtraordinaryEvents", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ExtraordinaryEvents.ExtraordinaryEventsBuilder object = (ExtraordinaryEvents.ExtraordinaryEventsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("additionalDisruptionEvents", "failureToDeliver");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAdditionalDisruptionEvents())) populatedFieldNames.add("additionalDisruptionEvents");
		if (isSet(object.getFailureToDeliver())) populatedFieldNames.add("failureToDeliver");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ExtraordinaryEvents", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ExtraordinaryEvents>(NAME, "ExtraordinaryEvents", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
