package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FeaturePayment;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FeaturePayment_choice")
public class FeaturePaymentChoiceRule implements Validator<FeaturePayment> {
	
	private static final String NAME = "FeaturePayment_choice";
	
    @Override
    public ValidationResult<FeaturePayment> validate(RosettaPath path, FeaturePayment object) {
		List<String> choiceFieldNames = Arrays.asList("levelPercentage", "amount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLevelPercentage())) populatedFieldNames.add("levelPercentage");
		if (isSet(object.getAmount())) populatedFieldNames.add("amount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FeaturePayment", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FeaturePayment>(NAME, "FeaturePayment", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FeaturePayment> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FeaturePayment.FeaturePaymentBuilder object = (FeaturePayment.FeaturePaymentBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("levelPercentage", "amount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLevelPercentage())) populatedFieldNames.add("levelPercentage");
		if (isSet(object.getAmount())) populatedFieldNames.add("amount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FeaturePayment", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FeaturePayment>(NAME, "FeaturePayment", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
