package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FxCashSettlement;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FxCashSettlement_choice")
public class FxCashSettlementChoiceRule implements Validator<FxCashSettlement> {
	
	private static final String NAME = "FxCashSettlement_choice";
	
    @Override
    public ValidationResult<FxCashSettlement> validate(RosettaPath path, FxCashSettlement object) {
		List<String> choiceFieldNames = Arrays.asList("fixing", "rateSourceFixing");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFixing())) populatedFieldNames.add("fixing");
		if (isSet(object.getRateSourceFixing())) populatedFieldNames.add("rateSourceFixing");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxCashSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxCashSettlement>(NAME, "FxCashSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FxCashSettlement> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FxCashSettlement.FxCashSettlementBuilder object = (FxCashSettlement.FxCashSettlementBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("fixing", "rateSourceFixing");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFixing())) populatedFieldNames.add("fixing");
		if (isSet(object.getRateSourceFixing())) populatedFieldNames.add("rateSourceFixing");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxCashSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxCashSettlement>(NAME, "FxCashSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
