package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FxFeature;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FxFeature_choice")
public class FxFeatureChoiceRule implements Validator<FxFeature> {
	
	private static final String NAME = "FxFeature_choice";
	
    @Override
    public ValidationResult<FxFeature> validate(RosettaPath path, FxFeature object) {
		List<String> choiceFieldNames = Arrays.asList("composite", "quanto", "crossCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getComposite())) populatedFieldNames.add("composite");
		if (isSet(object.getQuanto())) populatedFieldNames.add("quanto");
		if (isSet(object.getCrossCurrency())) populatedFieldNames.add("crossCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxFeature", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxFeature>(NAME, "FxFeature", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FxFeature> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FxFeature.FxFeatureBuilder object = (FxFeature.FxFeatureBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("composite", "quanto", "crossCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getComposite())) populatedFieldNames.add("composite");
		if (isSet(object.getQuanto())) populatedFieldNames.add("quanto");
		if (isSet(object.getCrossCurrency())) populatedFieldNames.add("crossCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxFeature", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxFeature>(NAME, "FxFeature", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
