package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FxFixingDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FxFixingDate_choice2")
public class FxFixingDateChoice2ChoiceRule implements Validator<FxFixingDate> {
	
	private static final String NAME = "FxFixingDate_choice2";
	
    @Override
    public ValidationResult<FxFixingDate> validate(RosettaPath path, FxFixingDate object) {
		List<String> choiceFieldNames = Arrays.asList("dateRelativeToPaymentDates", "dateRelativeToCalculationPeriodDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateRelativeToPaymentDates())) populatedFieldNames.add("dateRelativeToPaymentDates");
		if (isSet(object.getDateRelativeToCalculationPeriodDates())) populatedFieldNames.add("dateRelativeToCalculationPeriodDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxFixingDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxFixingDate>(NAME, "FxFixingDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FxFixingDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FxFixingDate.FxFixingDateBuilder object = (FxFixingDate.FxFixingDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("dateRelativeToPaymentDates", "dateRelativeToCalculationPeriodDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateRelativeToPaymentDates())) populatedFieldNames.add("dateRelativeToPaymentDates");
		if (isSet(object.getDateRelativeToCalculationPeriodDates())) populatedFieldNames.add("dateRelativeToCalculationPeriodDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxFixingDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxFixingDate>(NAME, "FxFixingDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
