package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FxHaircutCurrency;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FxHaircutCurrency_oneOf")
public class FxHaircutCurrencyOneOfRule implements Validator<FxHaircutCurrency> {
	
	private static final String NAME = "FxHaircutCurrency_oneOf";
	
    @Override
    public ValidationResult<FxHaircutCurrency> validate(RosettaPath path, FxHaircutCurrency object) {
		List<String> choiceFieldNames = Arrays.asList("isTerminationCurrency", "fxDesignatedCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIsTerminationCurrency())) populatedFieldNames.add("isTerminationCurrency");
		if (isSet(object.getFxDesignatedCurrency())) populatedFieldNames.add("fxDesignatedCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxHaircutCurrency", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxHaircutCurrency>(NAME, "FxHaircutCurrency", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FxHaircutCurrency> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FxHaircutCurrency.FxHaircutCurrencyBuilder object = (FxHaircutCurrency.FxHaircutCurrencyBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("isTerminationCurrency", "fxDesignatedCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIsTerminationCurrency())) populatedFieldNames.add("isTerminationCurrency");
		if (isSet(object.getFxDesignatedCurrency())) populatedFieldNames.add("fxDesignatedCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxHaircutCurrency", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxHaircutCurrency>(NAME, "FxHaircutCurrency", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
