package org.isda.cdm.validation.choicerule;

import org.isda.cdm.FxSettlementRateSource;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("FxSettlementRateSource_choice")
public class FxSettlementRateSourceChoiceRule implements Validator<FxSettlementRateSource> {
	
	private static final String NAME = "FxSettlementRateSource_choice";
	
    @Override
    public ValidationResult<FxSettlementRateSource> validate(RosettaPath path, FxSettlementRateSource object) {
		List<String> choiceFieldNames = Arrays.asList("settlementRateOption", "nonstandardSettlementRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementRateOption())) populatedFieldNames.add("settlementRateOption");
		if (isSet(object.getNonstandardSettlementRate())) populatedFieldNames.add("nonstandardSettlementRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxSettlementRateSource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxSettlementRateSource>(NAME, "FxSettlementRateSource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<FxSettlementRateSource> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	FxSettlementRateSource.FxSettlementRateSourceBuilder object = (FxSettlementRateSource.FxSettlementRateSourceBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("settlementRateOption", "nonstandardSettlementRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementRateOption())) populatedFieldNames.add("settlementRateOption");
		if (isSet(object.getNonstandardSettlementRate())) populatedFieldNames.add("nonstandardSettlementRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "FxSettlementRateSource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<FxSettlementRateSource>(NAME, "FxSettlementRateSource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
