package org.isda.cdm.validation.choicerule;

import org.isda.cdm.GeneralTerms;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("GeneralTerms_choice")
public class GeneralTermsChoiceRule implements Validator<GeneralTerms> {
	
	private static final String NAME = "GeneralTerms_choice";
	
    @Override
    public ValidationResult<GeneralTerms> validate(RosettaPath path, GeneralTerms object) {
		List<String> choiceFieldNames = Arrays.asList("referenceInformation", "indexReferenceInformation", "basketReferenceInformation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceInformation())) populatedFieldNames.add("referenceInformation");
		if (isSet(object.getIndexReferenceInformation())) populatedFieldNames.add("indexReferenceInformation");
		if (isSet(object.getBasketReferenceInformation())) populatedFieldNames.add("basketReferenceInformation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "GeneralTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<GeneralTerms>(NAME, "GeneralTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<GeneralTerms> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	GeneralTerms.GeneralTermsBuilder object = (GeneralTerms.GeneralTermsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("referenceInformation", "indexReferenceInformation", "basketReferenceInformation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceInformation())) populatedFieldNames.add("referenceInformation");
		if (isSet(object.getIndexReferenceInformation())) populatedFieldNames.add("indexReferenceInformation");
		if (isSet(object.getBasketReferenceInformation())) populatedFieldNames.add("basketReferenceInformation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "GeneralTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<GeneralTerms>(NAME, "GeneralTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
