package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Identifier;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Identifier_choice")
public class IdentifierChoiceRule implements Validator<Identifier> {
	
	private static final String NAME = "Identifier_choice";
	
    @Override
    public ValidationResult<Identifier> validate(RosettaPath path, Identifier object) {
		List<String> choiceFieldNames = Arrays.asList("issuerReference", "issuer");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIssuerReference())) populatedFieldNames.add("issuerReference");
		if (isSet(object.getIssuer())) populatedFieldNames.add("issuer");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Identifier", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Identifier>(NAME, "Identifier", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Identifier> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Identifier.IdentifierBuilder object = (Identifier.IdentifierBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("issuerReference", "issuer");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIssuerReference())) populatedFieldNames.add("issuerReference");
		if (isSet(object.getIssuer())) populatedFieldNames.add("issuer");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Identifier", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Identifier>(NAME, "Identifier", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
