package org.isda.cdm.validation.choicerule;

import org.isda.cdm.InitialMarginCalculation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("InitialMarginCalculation_choice")
public class InitialMarginCalculationChoiceRule implements Validator<InitialMarginCalculation> {
	
	private static final String NAME = "InitialMarginCalculation_choice";
	
    @Override
    public ValidationResult<InitialMarginCalculation> validate(RosettaPath path, InitialMarginCalculation object) {
		List<String> choiceFieldNames = Arrays.asList("marginRatio", "haircut");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMarginRatio())) populatedFieldNames.add("marginRatio");
		if (isSet(object.getHaircut())) populatedFieldNames.add("haircut");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "InitialMarginCalculation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<InitialMarginCalculation>(NAME, "InitialMarginCalculation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<InitialMarginCalculation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	InitialMarginCalculation.InitialMarginCalculationBuilder object = (InitialMarginCalculation.InitialMarginCalculationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("marginRatio", "haircut");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMarginRatio())) populatedFieldNames.add("marginRatio");
		if (isSet(object.getHaircut())) populatedFieldNames.add("haircut");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "InitialMarginCalculation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<InitialMarginCalculation>(NAME, "InitialMarginCalculation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
