package org.isda.cdm.validation.choicerule;

import org.isda.cdm.InterestRatePayout;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("InterestRatePayout_choice")
public class InterestRatePayoutChoiceRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "InterestRatePayout_choice";
	
    @Override
    public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout object) {
		List<String> choiceFieldNames = Arrays.asList("paymentDates", "paymentDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPaymentDates())) populatedFieldNames.add("paymentDates");
		if (isSet(object.getPaymentDate())) populatedFieldNames.add("paymentDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "InterestRatePayout", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<InterestRatePayout>(NAME, "InterestRatePayout", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	InterestRatePayout.InterestRatePayoutBuilder object = (InterestRatePayout.InterestRatePayoutBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("paymentDates", "paymentDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPaymentDates())) populatedFieldNames.add("paymentDates");
		if (isSet(object.getPaymentDate())) populatedFieldNames.add("paymentDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "InterestRatePayout", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<InterestRatePayout>(NAME, "InterestRatePayout", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
