package org.isda.cdm.validation.choicerule;

import org.isda.cdm.LastRegularPaymentDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("LastRegularPaymentDate_oneOf")
public class LastRegularPaymentDateOneOfRule implements Validator<LastRegularPaymentDate> {
	
	private static final String NAME = "LastRegularPaymentDate_oneOf";
	
    @Override
    public ValidationResult<LastRegularPaymentDate> validate(RosettaPath path, LastRegularPaymentDate object) {
		List<String> choiceFieldNames = Arrays.asList("lastRegularPaymentDate", "finalPaymentDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLastRegularPaymentDate())) populatedFieldNames.add("lastRegularPaymentDate");
		if (isSet(object.getFinalPaymentDate())) populatedFieldNames.add("finalPaymentDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LastRegularPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LastRegularPaymentDate>(NAME, "LastRegularPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<LastRegularPaymentDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	LastRegularPaymentDate.LastRegularPaymentDateBuilder object = (LastRegularPaymentDate.LastRegularPaymentDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("lastRegularPaymentDate", "finalPaymentDate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLastRegularPaymentDate())) populatedFieldNames.add("lastRegularPaymentDate");
		if (isSet(object.getFinalPaymentDate())) populatedFieldNames.add("finalPaymentDate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LastRegularPaymentDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LastRegularPaymentDate>(NAME, "LastRegularPaymentDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
