package org.isda.cdm.validation.choicerule;

import org.isda.cdm.LegalAgreement;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("LegalAgreement_oneOf")
public class LegalAgreementOneOfRule implements Validator<LegalAgreement> {
	
	private static final String NAME = "LegalAgreement_oneOf";
	
    @Override
    public ValidationResult<LegalAgreement> validate(RosettaPath path, LegalAgreement object) {
		List<String> choiceFieldNames = Arrays.asList("csdInitialMargin2016EnglishLaw", "csaInitialMargin2016JapaneseLaw", "csaInitialMargin2016NewYorkLaw", "csaVariationMargin2016NewYorkLaw", "equitySwapMasterConfirmation2018", "agreementDate", "effectiveDate", "identifier", "lineage", "agreementType", "contractualParty", "otherParty", "umbrellaAgreement", "partyInformation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCsdInitialMargin2016EnglishLaw())) populatedFieldNames.add("csdInitialMargin2016EnglishLaw");
		if (isSet(object.getCsaInitialMargin2016JapaneseLaw())) populatedFieldNames.add("csaInitialMargin2016JapaneseLaw");
		if (isSet(object.getCsaInitialMargin2016NewYorkLaw())) populatedFieldNames.add("csaInitialMargin2016NewYorkLaw");
		if (isSet(object.getCsaVariationMargin2016NewYorkLaw())) populatedFieldNames.add("csaVariationMargin2016NewYorkLaw");
		if (isSet(object.getEquitySwapMasterConfirmation2018())) populatedFieldNames.add("equitySwapMasterConfirmation2018");
		if (isSet(object.getAgreementDate())) populatedFieldNames.add("agreementDate");
		if (isSet(object.getEffectiveDate())) populatedFieldNames.add("effectiveDate");
		if (isSet(object.getIdentifier())) populatedFieldNames.add("identifier");
		if (isSet(object.getLineage())) populatedFieldNames.add("lineage");
		if (isSet(object.getAgreementType())) populatedFieldNames.add("agreementType");
		if (isSet(object.getContractualParty())) populatedFieldNames.add("contractualParty");
		if (isSet(object.getOtherParty())) populatedFieldNames.add("otherParty");
		if (isSet(object.getUmbrellaAgreement())) populatedFieldNames.add("umbrellaAgreement");
		if (isSet(object.getPartyInformation())) populatedFieldNames.add("partyInformation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LegalAgreement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LegalAgreement>(NAME, "LegalAgreement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<LegalAgreement> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	LegalAgreement.LegalAgreementBuilder object = (LegalAgreement.LegalAgreementBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("csdInitialMargin2016EnglishLaw", "csaInitialMargin2016JapaneseLaw", "csaInitialMargin2016NewYorkLaw", "csaVariationMargin2016NewYorkLaw", "equitySwapMasterConfirmation2018", "agreementDate", "effectiveDate", "identifier", "lineage", "agreementType", "contractualParty", "otherParty", "umbrellaAgreement", "partyInformation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCsdInitialMargin2016EnglishLaw())) populatedFieldNames.add("csdInitialMargin2016EnglishLaw");
		if (isSet(object.getCsaInitialMargin2016JapaneseLaw())) populatedFieldNames.add("csaInitialMargin2016JapaneseLaw");
		if (isSet(object.getCsaInitialMargin2016NewYorkLaw())) populatedFieldNames.add("csaInitialMargin2016NewYorkLaw");
		if (isSet(object.getCsaVariationMargin2016NewYorkLaw())) populatedFieldNames.add("csaVariationMargin2016NewYorkLaw");
		if (isSet(object.getEquitySwapMasterConfirmation2018())) populatedFieldNames.add("equitySwapMasterConfirmation2018");
		if (isSet(object.getAgreementDate())) populatedFieldNames.add("agreementDate");
		if (isSet(object.getEffectiveDate())) populatedFieldNames.add("effectiveDate");
		if (isSet(object.getIdentifier())) populatedFieldNames.add("identifier");
		if (isSet(object.getLineage())) populatedFieldNames.add("lineage");
		if (isSet(object.getAgreementType())) populatedFieldNames.add("agreementType");
		if (isSet(object.getContractualParty())) populatedFieldNames.add("contractualParty");
		if (isSet(object.getOtherParty())) populatedFieldNames.add("otherParty");
		if (isSet(object.getUmbrellaAgreement())) populatedFieldNames.add("umbrellaAgreement");
		if (isSet(object.getPartyInformation())) populatedFieldNames.add("partyInformation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LegalAgreement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LegalAgreement>(NAME, "LegalAgreement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
