package org.isda.cdm.validation.choicerule;

import org.isda.cdm.LimitApplicable;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("LimitApplicable_choice")
public class LimitApplicableChoiceRule implements Validator<LimitApplicable> {
	
	private static final String NAME = "LimitApplicable_choice";
	
    @Override
    public ValidationResult<LimitApplicable> validate(RosettaPath path, LimitApplicable object) {
		List<String> choiceFieldNames = Arrays.asList("amountUtilized", "utilization");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmountUtilized())) populatedFieldNames.add("amountUtilized");
		if (isSet(object.getUtilization())) populatedFieldNames.add("utilization");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LimitApplicable", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LimitApplicable>(NAME, "LimitApplicable", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<LimitApplicable> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	LimitApplicable.LimitApplicableBuilder object = (LimitApplicable.LimitApplicableBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("amountUtilized", "utilization");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmountUtilized())) populatedFieldNames.add("amountUtilized");
		if (isSet(object.getUtilization())) populatedFieldNames.add("utilization");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "LimitApplicable", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<LimitApplicable>(NAME, "LimitApplicable", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
