package org.isda.cdm.validation.choicerule;

import org.isda.cdm.MultipleExercise;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("MultipleExercise_oneOf")
public class MultipleExerciseOneOfRule implements Validator<MultipleExercise> {
	
	private static final String NAME = "MultipleExercise_oneOf";
	
    @Override
    public ValidationResult<MultipleExercise> validate(RosettaPath path, MultipleExercise object) {
		List<String> choiceFieldNames = Arrays.asList("maximumNotionalAmount", "maximumNumberOfOptions", "notionaReference", "integralMultipleAmount", "minimumNotionalAmount", "minimumNumberOfOptions");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMaximumNotionalAmount())) populatedFieldNames.add("maximumNotionalAmount");
		if (isSet(object.getMaximumNumberOfOptions())) populatedFieldNames.add("maximumNumberOfOptions");
		if (isSet(object.getNotionaReference())) populatedFieldNames.add("notionaReference");
		if (isSet(object.getIntegralMultipleAmount())) populatedFieldNames.add("integralMultipleAmount");
		if (isSet(object.getMinimumNotionalAmount())) populatedFieldNames.add("minimumNotionalAmount");
		if (isSet(object.getMinimumNumberOfOptions())) populatedFieldNames.add("minimumNumberOfOptions");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "MultipleExercise", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<MultipleExercise>(NAME, "MultipleExercise", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<MultipleExercise> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	MultipleExercise.MultipleExerciseBuilder object = (MultipleExercise.MultipleExerciseBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("maximumNotionalAmount", "maximumNumberOfOptions", "notionaReference", "integralMultipleAmount", "minimumNotionalAmount", "minimumNumberOfOptions");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMaximumNotionalAmount())) populatedFieldNames.add("maximumNotionalAmount");
		if (isSet(object.getMaximumNumberOfOptions())) populatedFieldNames.add("maximumNumberOfOptions");
		if (isSet(object.getNotionaReference())) populatedFieldNames.add("notionaReference");
		if (isSet(object.getIntegralMultipleAmount())) populatedFieldNames.add("integralMultipleAmount");
		if (isSet(object.getMinimumNotionalAmount())) populatedFieldNames.add("minimumNotionalAmount");
		if (isSet(object.getMinimumNumberOfOptions())) populatedFieldNames.add("minimumNumberOfOptions");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "MultipleExercise", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<MultipleExercise>(NAME, "MultipleExercise", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
