package org.isda.cdm.validation.choicerule;

import org.isda.cdm.NaturalPerson;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("NaturalPerson_choice")
public class NaturalPersonChoiceRule implements Validator<NaturalPerson> {
	
	private static final String NAME = "NaturalPerson_choice";
	
    @Override
    public ValidationResult<NaturalPerson> validate(RosettaPath path, NaturalPerson object) {
		List<String> choiceFieldNames = Arrays.asList("middleName", "initial");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMiddleName())) populatedFieldNames.add("middleName");
		if (isSet(object.getInitial())) populatedFieldNames.add("initial");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NaturalPerson", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NaturalPerson>(NAME, "NaturalPerson", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<NaturalPerson> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	NaturalPerson.NaturalPersonBuilder object = (NaturalPerson.NaturalPersonBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("middleName", "initial");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMiddleName())) populatedFieldNames.add("middleName");
		if (isSet(object.getInitial())) populatedFieldNames.add("initial");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NaturalPerson", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NaturalPerson>(NAME, "NaturalPerson", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
