package org.isda.cdm.validation.choicerule;

import org.isda.cdm.NonDeliverableSettlement;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("NonDeliverableSettlement_choice")
public class NonDeliverableSettlementChoiceRule implements Validator<NonDeliverableSettlement> {
	
	private static final String NAME = "NonDeliverableSettlement_choice";
	
    @Override
    public ValidationResult<NonDeliverableSettlement> validate(RosettaPath path, NonDeliverableSettlement object) {
		List<String> choiceFieldNames = Arrays.asList("fxFixingDate", "fxFixingSchedule");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFxFixingDate())) populatedFieldNames.add("fxFixingDate");
		if (isSet(object.getFxFixingSchedule())) populatedFieldNames.add("fxFixingSchedule");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NonDeliverableSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NonDeliverableSettlement>(NAME, "NonDeliverableSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<NonDeliverableSettlement> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	NonDeliverableSettlement.NonDeliverableSettlementBuilder object = (NonDeliverableSettlement.NonDeliverableSettlementBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("fxFixingDate", "fxFixingSchedule");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFxFixingDate())) populatedFieldNames.add("fxFixingDate");
		if (isSet(object.getFxFixingSchedule())) populatedFieldNames.add("fxFixingSchedule");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NonDeliverableSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NonDeliverableSettlement>(NAME, "NonDeliverableSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
