package org.isda.cdm.validation.choicerule;

import org.isda.cdm.NotificationTimeElection;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("NotificationTimeElection_choice")
public class NotificationTimeElectionChoiceRule implements Validator<NotificationTimeElection> {
	
	private static final String NAME = "NotificationTimeElection_choice";
	
    @Override
    public ValidationResult<NotificationTimeElection> validate(RosettaPath path, NotificationTimeElection object) {
		List<String> choiceFieldNames = Arrays.asList("notificationTime", "customNotification");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNotificationTime())) populatedFieldNames.add("notificationTime");
		if (isSet(object.getCustomNotification())) populatedFieldNames.add("customNotification");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NotificationTimeElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NotificationTimeElection>(NAME, "NotificationTimeElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<NotificationTimeElection> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	NotificationTimeElection.NotificationTimeElectionBuilder object = (NotificationTimeElection.NotificationTimeElectionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("notificationTime", "customNotification");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getNotificationTime())) populatedFieldNames.add("notificationTime");
		if (isSet(object.getCustomNotification())) populatedFieldNames.add("customNotification");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "NotificationTimeElection", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<NotificationTimeElection>(NAME, "NotificationTimeElection", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
