package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Obligations;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Obligations_choice")
public class ObligationsChoiceRule implements Validator<Obligations> {
	
	private static final String NAME = "Obligations_choice";
	
    @Override
    public ValidationResult<Obligations> validate(RosettaPath path, Obligations object) {
		List<String> choiceFieldNames = Arrays.asList("fullFaithAndCreditObLiability", "generalFundObligationLiability", "revenueObligationLiability");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFullFaithAndCreditObLiability())) populatedFieldNames.add("fullFaithAndCreditObLiability");
		if (isSet(object.getGeneralFundObligationLiability())) populatedFieldNames.add("generalFundObligationLiability");
		if (isSet(object.getRevenueObligationLiability())) populatedFieldNames.add("revenueObligationLiability");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Obligations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Obligations>(NAME, "Obligations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Obligations> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Obligations.ObligationsBuilder object = (Obligations.ObligationsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("fullFaithAndCreditObLiability", "generalFundObligationLiability", "revenueObligationLiability");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFullFaithAndCreditObLiability())) populatedFieldNames.add("fullFaithAndCreditObLiability");
		if (isSet(object.getGeneralFundObligationLiability())) populatedFieldNames.add("generalFundObligationLiability");
		if (isSet(object.getRevenueObligationLiability())) populatedFieldNames.add("revenueObligationLiability");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Obligations", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Obligations>(NAME, "Obligations", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
