package org.isda.cdm.validation.choicerule;

import org.isda.cdm.OptionCashSettlement;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("OptionCashSettlement_choice")
public class OptionCashSettlementChoiceRule implements Validator<OptionCashSettlement> {
	
	private static final String NAME = "OptionCashSettlement_choice";
	
    @Override
    public ValidationResult<OptionCashSettlement> validate(RosettaPath path, OptionCashSettlement object) {
		List<String> choiceFieldNames = Arrays.asList("cashPriceMethod", "cashPriceAlternateMethod", "parYieldCurveAdjustedMethod", "zeroCouponYieldAdjustedMethod", "parYieldCurveUnadjustedMethod", "crossCurrencyMethod", "collateralizedCashPriceMethod");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashPriceMethod())) populatedFieldNames.add("cashPriceMethod");
		if (isSet(object.getCashPriceAlternateMethod())) populatedFieldNames.add("cashPriceAlternateMethod");
		if (isSet(object.getParYieldCurveAdjustedMethod())) populatedFieldNames.add("parYieldCurveAdjustedMethod");
		if (isSet(object.getZeroCouponYieldAdjustedMethod())) populatedFieldNames.add("zeroCouponYieldAdjustedMethod");
		if (isSet(object.getParYieldCurveUnadjustedMethod())) populatedFieldNames.add("parYieldCurveUnadjustedMethod");
		if (isSet(object.getCrossCurrencyMethod())) populatedFieldNames.add("crossCurrencyMethod");
		if (isSet(object.getCollateralizedCashPriceMethod())) populatedFieldNames.add("collateralizedCashPriceMethod");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionCashSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionCashSettlement>(NAME, "OptionCashSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<OptionCashSettlement> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	OptionCashSettlement.OptionCashSettlementBuilder object = (OptionCashSettlement.OptionCashSettlementBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("cashPriceMethod", "cashPriceAlternateMethod", "parYieldCurveAdjustedMethod", "zeroCouponYieldAdjustedMethod", "parYieldCurveUnadjustedMethod", "crossCurrencyMethod", "collateralizedCashPriceMethod");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashPriceMethod())) populatedFieldNames.add("cashPriceMethod");
		if (isSet(object.getCashPriceAlternateMethod())) populatedFieldNames.add("cashPriceAlternateMethod");
		if (isSet(object.getParYieldCurveAdjustedMethod())) populatedFieldNames.add("parYieldCurveAdjustedMethod");
		if (isSet(object.getZeroCouponYieldAdjustedMethod())) populatedFieldNames.add("zeroCouponYieldAdjustedMethod");
		if (isSet(object.getParYieldCurveUnadjustedMethod())) populatedFieldNames.add("parYieldCurveUnadjustedMethod");
		if (isSet(object.getCrossCurrencyMethod())) populatedFieldNames.add("crossCurrencyMethod");
		if (isSet(object.getCollateralizedCashPriceMethod())) populatedFieldNames.add("collateralizedCashPriceMethod");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionCashSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionCashSettlement>(NAME, "OptionCashSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
