package org.isda.cdm.validation.choicerule;

import org.isda.cdm.OptionSettlement;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("OptionSettlement_choice")
public class OptionSettlementChoiceRule implements Validator<OptionSettlement> {
	
	private static final String NAME = "OptionSettlement_choice";
	
    @Override
    public ValidationResult<OptionSettlement> validate(RosettaPath path, OptionSettlement object) {
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTerms", "physicalSettlementTerms", "fxSettlementTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTerms())) populatedFieldNames.add("cashSettlementTerms");
		if (isSet(object.getPhysicalSettlementTerms())) populatedFieldNames.add("physicalSettlementTerms");
		if (isSet(object.getFxSettlementTerms())) populatedFieldNames.add("fxSettlementTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionSettlement>(NAME, "OptionSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<OptionSettlement> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	OptionSettlement.OptionSettlementBuilder object = (OptionSettlement.OptionSettlementBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTerms", "physicalSettlementTerms", "fxSettlementTerms");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTerms())) populatedFieldNames.add("cashSettlementTerms");
		if (isSet(object.getPhysicalSettlementTerms())) populatedFieldNames.add("physicalSettlementTerms");
		if (isSet(object.getFxSettlementTerms())) populatedFieldNames.add("fxSettlementTerms");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionSettlement", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionSettlement>(NAME, "OptionSettlement", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
