package org.isda.cdm.validation.choicerule;

import org.isda.cdm.OptionStrike;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("OptionStrike_oneOf")
public class OptionStrikeOneOfRule implements Validator<OptionStrike> {
	
	private static final String NAME = "OptionStrike_oneOf";
	
    @Override
    public ValidationResult<OptionStrike> validate(RosettaPath path, OptionStrike object) {
		List<String> choiceFieldNames = Arrays.asList("spread", "price", "currency", "percentage", "strikeReference", "referenceSwapCurve");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSpread())) populatedFieldNames.add("spread");
		if (isSet(object.getPrice())) populatedFieldNames.add("price");
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		if (isSet(object.getPercentage())) populatedFieldNames.add("percentage");
		if (isSet(object.getStrikeReference())) populatedFieldNames.add("strikeReference");
		if (isSet(object.getReferenceSwapCurve())) populatedFieldNames.add("referenceSwapCurve");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionStrike", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionStrike>(NAME, "OptionStrike", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<OptionStrike> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	OptionStrike.OptionStrikeBuilder object = (OptionStrike.OptionStrikeBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("spread", "price", "currency", "percentage", "strikeReference", "referenceSwapCurve");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSpread())) populatedFieldNames.add("spread");
		if (isSet(object.getPrice())) populatedFieldNames.add("price");
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		if (isSet(object.getPercentage())) populatedFieldNames.add("percentage");
		if (isSet(object.getStrikeReference())) populatedFieldNames.add("strikeReference");
		if (isSet(object.getReferenceSwapCurve())) populatedFieldNames.add("referenceSwapCurve");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionStrike", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionStrike>(NAME, "OptionStrike", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
