package org.isda.cdm.validation.choicerule;

import org.isda.cdm.OptionalEarlyTermination;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("OptionalEarlyTermination_choice")
public class OptionalEarlyTerminationChoiceRule implements Validator<OptionalEarlyTermination> {
	
	private static final String NAME = "OptionalEarlyTermination_choice";
	
    @Override
    public ValidationResult<OptionalEarlyTermination> validate(RosettaPath path, OptionalEarlyTermination object) {
		List<String> choiceFieldNames = Arrays.asList("americanExercise", "bermudaExercise", "europeanExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmericanExercise())) populatedFieldNames.add("americanExercise");
		if (isSet(object.getBermudaExercise())) populatedFieldNames.add("bermudaExercise");
		if (isSet(object.getEuropeanExercise())) populatedFieldNames.add("europeanExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionalEarlyTermination", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionalEarlyTermination>(NAME, "OptionalEarlyTermination", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<OptionalEarlyTermination> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	OptionalEarlyTermination.OptionalEarlyTerminationBuilder object = (OptionalEarlyTermination.OptionalEarlyTerminationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("americanExercise", "bermudaExercise", "europeanExercise");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getAmericanExercise())) populatedFieldNames.add("americanExercise");
		if (isSet(object.getBermudaExercise())) populatedFieldNames.add("bermudaExercise");
		if (isSet(object.getEuropeanExercise())) populatedFieldNames.add("europeanExercise");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "OptionalEarlyTermination", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<OptionalEarlyTermination>(NAME, "OptionalEarlyTermination", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
