package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PartialExercise;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PartialExercise_choice")
public class PartialExerciseChoiceRule implements Validator<PartialExercise> {
	
	private static final String NAME = "PartialExercise_choice";
	
    @Override
    public ValidationResult<PartialExercise> validate(RosettaPath path, PartialExercise object) {
		List<String> choiceFieldNames = Arrays.asList("minimumNotionalAmount", "minimumNumberOfOptions");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMinimumNotionalAmount())) populatedFieldNames.add("minimumNotionalAmount");
		if (isSet(object.getMinimumNumberOfOptions())) populatedFieldNames.add("minimumNumberOfOptions");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartialExercise", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartialExercise>(NAME, "PartialExercise", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PartialExercise> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PartialExercise.PartialExerciseBuilder object = (PartialExercise.PartialExerciseBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("minimumNotionalAmount", "minimumNumberOfOptions");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getMinimumNotionalAmount())) populatedFieldNames.add("minimumNotionalAmount");
		if (isSet(object.getMinimumNumberOfOptions())) populatedFieldNames.add("minimumNumberOfOptions");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartialExercise", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartialExercise>(NAME, "PartialExercise", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
