package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PartyContactInformation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PartyContactInformation_choice")
public class PartyContactInformationChoiceRule implements Validator<PartyContactInformation> {
	
	private static final String NAME = "PartyContactInformation_choice";
	
    @Override
    public ValidationResult<PartyContactInformation> validate(RosettaPath path, PartyContactInformation object) {
		List<String> choiceFieldNames = Arrays.asList("contactInformation", "address");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getContactInformation())) populatedFieldNames.add("contactInformation");
		if (isSet(object.getAddress())) populatedFieldNames.add("address");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartyContactInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartyContactInformation>(NAME, "PartyContactInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PartyContactInformation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PartyContactInformation.PartyContactInformationBuilder object = (PartyContactInformation.PartyContactInformationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("contactInformation", "address");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getContactInformation())) populatedFieldNames.add("contactInformation");
		if (isSet(object.getAddress())) populatedFieldNames.add("address");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartyContactInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartyContactInformation>(NAME, "PartyContactInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
