package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PartyCustomisedWorkflow;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PartyCustomisedWorkflow_choice")
public class PartyCustomisedWorkflowChoiceRule implements Validator<PartyCustomisedWorkflow> {
	
	private static final String NAME = "PartyCustomisedWorkflow_choice";
	
    @Override
    public ValidationResult<PartyCustomisedWorkflow> validate(RosettaPath path, PartyCustomisedWorkflow object) {
		List<String> choiceFieldNames = Arrays.asList("partyName", "partyReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPartyName())) populatedFieldNames.add("partyName");
		if (isSet(object.getPartyReference())) populatedFieldNames.add("partyReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartyCustomisedWorkflow", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartyCustomisedWorkflow>(NAME, "PartyCustomisedWorkflow", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PartyCustomisedWorkflow> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder object = (PartyCustomisedWorkflow.PartyCustomisedWorkflowBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("partyName", "partyReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPartyName())) populatedFieldNames.add("partyName");
		if (isSet(object.getPartyReference())) populatedFieldNames.add("partyReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PartyCustomisedWorkflow", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PartyCustomisedWorkflow>(NAME, "PartyCustomisedWorkflow", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
