package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PaymentCalculationPeriod;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PaymentCalculationPeriod_choice")
public class PaymentCalculationPeriodChoiceRule implements Validator<PaymentCalculationPeriod> {
	
	private static final String NAME = "PaymentCalculationPeriod_choice";
	
    @Override
    public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, PaymentCalculationPeriod object) {
		List<String> choiceFieldNames = Arrays.asList("calculationPeriod", "fixedPaymentAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCalculationPeriod())) populatedFieldNames.add("calculationPeriod");
		if (isSet(object.getFixedPaymentAmount())) populatedFieldNames.add("fixedPaymentAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PaymentCalculationPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PaymentCalculationPeriod>(NAME, "PaymentCalculationPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PaymentCalculationPeriod.PaymentCalculationPeriodBuilder object = (PaymentCalculationPeriod.PaymentCalculationPeriodBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("calculationPeriod", "fixedPaymentAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCalculationPeriod())) populatedFieldNames.add("calculationPeriod");
		if (isSet(object.getFixedPaymentAmount())) populatedFieldNames.add("fixedPaymentAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PaymentCalculationPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PaymentCalculationPeriod>(NAME, "PaymentCalculationPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
