package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PhysicalSettlementPeriod;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PhysicalSettlementPeriod_oneOf")
public class PhysicalSettlementPeriodOneOfRule implements Validator<PhysicalSettlementPeriod> {
	
	private static final String NAME = "PhysicalSettlementPeriod_oneOf";
	
    @Override
    public ValidationResult<PhysicalSettlementPeriod> validate(RosettaPath path, PhysicalSettlementPeriod object) {
		List<String> choiceFieldNames = Arrays.asList("businessDaysNotSpecified", "businessDays", "maximumBusinessDays");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessDaysNotSpecified())) populatedFieldNames.add("businessDaysNotSpecified");
		if (isSet(object.getBusinessDays())) populatedFieldNames.add("businessDays");
		if (isSet(object.getMaximumBusinessDays())) populatedFieldNames.add("maximumBusinessDays");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PhysicalSettlementPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PhysicalSettlementPeriod>(NAME, "PhysicalSettlementPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PhysicalSettlementPeriod> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder object = (PhysicalSettlementPeriod.PhysicalSettlementPeriodBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("businessDaysNotSpecified", "businessDays", "maximumBusinessDays");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBusinessDaysNotSpecified())) populatedFieldNames.add("businessDaysNotSpecified");
		if (isSet(object.getBusinessDays())) populatedFieldNames.add("businessDays");
		if (isSet(object.getMaximumBusinessDays())) populatedFieldNames.add("maximumBusinessDays");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PhysicalSettlementPeriod", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PhysicalSettlementPeriod>(NAME, "PhysicalSettlementPeriod", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
