package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Product;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Product_oneOf")
public class ProductOneOfRule implements Validator<Product> {
	
	private static final String NAME = "Product_oneOf";
	
    @Override
    public ValidationResult<Product> validate(RosettaPath path, Product object) {
		List<String> choiceFieldNames = Arrays.asList("contractualProduct", "index", "loan", "foreignExchange", "security");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getContractualProduct())) populatedFieldNames.add("contractualProduct");
		if (isSet(object.getIndex())) populatedFieldNames.add("index");
		if (isSet(object.getLoan())) populatedFieldNames.add("loan");
		if (isSet(object.getForeignExchange())) populatedFieldNames.add("foreignExchange");
		if (isSet(object.getSecurity())) populatedFieldNames.add("security");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Product", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Product>(NAME, "Product", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Product> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Product.ProductBuilder object = (Product.ProductBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("contractualProduct", "index", "loan", "foreignExchange", "security");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getContractualProduct())) populatedFieldNames.add("contractualProduct");
		if (isSet(object.getIndex())) populatedFieldNames.add("index");
		if (isSet(object.getLoan())) populatedFieldNames.add("loan");
		if (isSet(object.getForeignExchange())) populatedFieldNames.add("foreignExchange");
		if (isSet(object.getSecurity())) populatedFieldNames.add("security");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Product", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Product>(NAME, "Product", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
