package org.isda.cdm.validation.choicerule;

import org.isda.cdm.PubliclyAvailableInformation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("PubliclyAvailableInformation_choice")
public class PubliclyAvailableInformationChoiceRule implements Validator<PubliclyAvailableInformation> {
	
	private static final String NAME = "PubliclyAvailableInformation_choice";
	
    @Override
    public ValidationResult<PubliclyAvailableInformation> validate(RosettaPath path, PubliclyAvailableInformation object) {
		List<String> choiceFieldNames = Arrays.asList("standardPublicSources", "publicSource");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardPublicSources())) populatedFieldNames.add("standardPublicSources");
		if (isSet(object.getPublicSource())) populatedFieldNames.add("publicSource");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PubliclyAvailableInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PubliclyAvailableInformation>(NAME, "PubliclyAvailableInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<PubliclyAvailableInformation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	PubliclyAvailableInformation.PubliclyAvailableInformationBuilder object = (PubliclyAvailableInformation.PubliclyAvailableInformationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("standardPublicSources", "publicSource");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardPublicSources())) populatedFieldNames.add("standardPublicSources");
		if (isSet(object.getPublicSource())) populatedFieldNames.add("publicSource");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "PubliclyAvailableInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<PubliclyAvailableInformation>(NAME, "PubliclyAvailableInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
