package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Quantity;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Quantity_unit")
public class QuantityUnitChoiceRule implements Validator<Quantity> {
	
	private static final String NAME = "Quantity_unit";
	
    @Override
    public ValidationResult<Quantity> validate(RosettaPath path, Quantity object) {
		List<String> choiceFieldNames = Arrays.asList("unit", "currency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getUnit())) populatedFieldNames.add("unit");
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Quantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Quantity>(NAME, "Quantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Quantity> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Quantity.QuantityBuilder object = (Quantity.QuantityBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("unit", "currency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getUnit())) populatedFieldNames.add("unit");
		if (isSet(object.getCurrency())) populatedFieldNames.add("currency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Quantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Quantity>(NAME, "Quantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
