package org.isda.cdm.validation.choicerule;

import org.isda.cdm.RateSpecification;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("RateSpecification_oneOf")
public class RateSpecificationOneOfRule implements Validator<RateSpecification> {
	
	private static final String NAME = "RateSpecification_oneOf";
	
    @Override
    public ValidationResult<RateSpecification> validate(RosettaPath path, RateSpecification object) {
		List<String> choiceFieldNames = Arrays.asList("fixedRate", "floatingRate", "inflationRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFixedRate())) populatedFieldNames.add("fixedRate");
		if (isSet(object.getFloatingRate())) populatedFieldNames.add("floatingRate");
		if (isSet(object.getInflationRate())) populatedFieldNames.add("inflationRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "RateSpecification", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<RateSpecification>(NAME, "RateSpecification", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<RateSpecification> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	RateSpecification.RateSpecificationBuilder object = (RateSpecification.RateSpecificationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("fixedRate", "floatingRate", "inflationRate");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFixedRate())) populatedFieldNames.add("fixedRate");
		if (isSet(object.getFloatingRate())) populatedFieldNames.add("floatingRate");
		if (isSet(object.getInflationRate())) populatedFieldNames.add("inflationRate");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "RateSpecification", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<RateSpecification>(NAME, "RateSpecification", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
