package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ReferenceInformation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ReferenceInformation_choice")
public class ReferenceInformationChoiceRule implements Validator<ReferenceInformation> {
	
	private static final String NAME = "ReferenceInformation_choice";
	
    @Override
    public ValidationResult<ReferenceInformation> validate(RosettaPath path, ReferenceInformation object) {
		List<String> choiceFieldNames = Arrays.asList("referenceObligation", "noReferenceObligation", "unknownReferenceObligation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceObligation())) populatedFieldNames.add("referenceObligation");
		if (isSet(object.getNoReferenceObligation())) populatedFieldNames.add("noReferenceObligation");
		if (isSet(object.getUnknownReferenceObligation())) populatedFieldNames.add("unknownReferenceObligation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferenceInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferenceInformation>(NAME, "ReferenceInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ReferenceInformation.ReferenceInformationBuilder object = (ReferenceInformation.ReferenceInformationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("referenceObligation", "noReferenceObligation", "unknownReferenceObligation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceObligation())) populatedFieldNames.add("referenceObligation");
		if (isSet(object.getNoReferenceObligation())) populatedFieldNames.add("noReferenceObligation");
		if (isSet(object.getUnknownReferenceObligation())) populatedFieldNames.add("unknownReferenceObligation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferenceInformation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferenceInformation>(NAME, "ReferenceInformation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
