package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ReferenceObligation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ReferenceObligation_choice2")
public class ReferenceObligationChoice2ChoiceRule implements Validator<ReferenceObligation> {
	
	private static final String NAME = "ReferenceObligation_choice2";
	
    @Override
    public ValidationResult<ReferenceObligation> validate(RosettaPath path, ReferenceObligation object) {
		List<String> choiceFieldNames = Arrays.asList("primaryObligor", "primaryObligorReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPrimaryObligor())) populatedFieldNames.add("primaryObligor");
		if (isSet(object.getPrimaryObligorReference())) populatedFieldNames.add("primaryObligorReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferenceObligation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferenceObligation>(NAME, "ReferenceObligation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ReferenceObligation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ReferenceObligation.ReferenceObligationBuilder object = (ReferenceObligation.ReferenceObligationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("primaryObligor", "primaryObligorReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getPrimaryObligor())) populatedFieldNames.add("primaryObligor");
		if (isSet(object.getPrimaryObligorReference())) populatedFieldNames.add("primaryObligorReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferenceObligation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferenceObligation>(NAME, "ReferenceObligation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
