package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ReferencePair;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ReferencePair_choice")
public class ReferencePairChoiceRule implements Validator<ReferencePair> {
	
	private static final String NAME = "ReferencePair_choice";
	
    @Override
    public ValidationResult<ReferencePair> validate(RosettaPath path, ReferencePair object) {
		List<String> choiceFieldNames = Arrays.asList("referenceObligation", "noReferenceObligation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceObligation())) populatedFieldNames.add("referenceObligation");
		if (isSet(object.getNoReferenceObligation())) populatedFieldNames.add("noReferenceObligation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferencePair", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferencePair>(NAME, "ReferencePair", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ReferencePair> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ReferencePair.ReferencePairBuilder object = (ReferencePair.ReferencePairBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("referenceObligation", "noReferenceObligation");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getReferenceObligation())) populatedFieldNames.add("referenceObligation");
		if (isSet(object.getNoReferenceObligation())) populatedFieldNames.add("noReferenceObligation");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferencePair", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferencePair>(NAME, "ReferencePair", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
