package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ReferencePoolItem;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ReferencePoolItem_choice")
public class ReferencePoolItemChoiceRule implements Validator<ReferencePoolItem> {
	
	private static final String NAME = "ReferencePoolItem_choice";
	
    @Override
    public ValidationResult<ReferencePoolItem> validate(RosettaPath path, ReferencePoolItem object) {
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTermsReference", "physicalSettlementTermsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTermsReference())) populatedFieldNames.add("cashSettlementTermsReference");
		if (isSet(object.getPhysicalSettlementTermsReference())) populatedFieldNames.add("physicalSettlementTermsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferencePoolItem", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferencePoolItem>(NAME, "ReferencePoolItem", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ReferencePoolItem> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ReferencePoolItem.ReferencePoolItemBuilder object = (ReferencePoolItem.ReferencePoolItemBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("cashSettlementTermsReference", "physicalSettlementTermsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getCashSettlementTermsReference())) populatedFieldNames.add("cashSettlementTermsReference");
		if (isSet(object.getPhysicalSettlementTermsReference())) populatedFieldNames.add("physicalSettlementTermsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReferencePoolItem", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReferencePoolItem>(NAME, "ReferencePoolItem", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
