package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ResolvablePayoutQuantity;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ResolvablePayoutQuantity_choice1")
public class ResolvablePayoutQuantityChoice1ChoiceRule implements Validator<ResolvablePayoutQuantity> {
	
	private static final String NAME = "ResolvablePayoutQuantity_choice1";
	
    @Override
    public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, ResolvablePayoutQuantity object) {
		List<String> choiceFieldNames = Arrays.asList("quantityNotationTag", "quantityReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getQuantityNotationTag())) populatedFieldNames.add("quantityNotationTag");
		if (isSet(object.getQuantityReference())) populatedFieldNames.add("quantityReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ResolvablePayoutQuantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ResolvablePayoutQuantity>(NAME, "ResolvablePayoutQuantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder object = (ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("quantityNotationTag", "quantityReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getQuantityNotationTag())) populatedFieldNames.add("quantityNotationTag");
		if (isSet(object.getQuantityReference())) populatedFieldNames.add("quantityReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ResolvablePayoutQuantity", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ResolvablePayoutQuantity>(NAME, "ResolvablePayoutQuantity", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
