package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Resource;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Resource_choice")
public class ResourceChoiceRule implements Validator<Resource> {
	
	private static final String NAME = "Resource_choice";
	
    @Override
    public ValidationResult<Resource> validate(RosettaPath path, Resource object) {
		List<String> choiceFieldNames = Arrays.asList("string", "url");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getString())) populatedFieldNames.add("string");
		if (isSet(object.getUrl())) populatedFieldNames.add("url");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Resource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Resource>(NAME, "Resource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Resource> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Resource.ResourceBuilder object = (Resource.ResourceBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("string", "url");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getString())) populatedFieldNames.add("string");
		if (isSet(object.getUrl())) populatedFieldNames.add("url");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Resource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Resource>(NAME, "Resource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
