package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ReturnAmount;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ReturnAmount_oneOf")
public class ReturnAmountOneOfRule implements Validator<ReturnAmount> {
	
	private static final String NAME = "ReturnAmount_oneOf";
	
    @Override
    public ValidationResult<ReturnAmount> validate(RosettaPath path, ReturnAmount object) {
		List<String> choiceFieldNames = Arrays.asList("includesDefaultLanguage", "customElection");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIncludesDefaultLanguage())) populatedFieldNames.add("includesDefaultLanguage");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReturnAmount", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReturnAmount>(NAME, "ReturnAmount", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ReturnAmount> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ReturnAmount.ReturnAmountBuilder object = (ReturnAmount.ReturnAmountBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("includesDefaultLanguage", "customElection");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getIncludesDefaultLanguage())) populatedFieldNames.add("includesDefaultLanguage");
		if (isSet(object.getCustomElection())) populatedFieldNames.add("customElection");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ReturnAmount", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ReturnAmount>(NAME, "ReturnAmount", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
