package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SecurityLeg;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SecurityLeg_choice")
public class SecurityLegChoiceRule implements Validator<SecurityLeg> {
	
	private static final String NAME = "SecurityLeg_choice";
	
    @Override
    public ValidationResult<SecurityLeg> validate(RosettaPath path, SecurityLeg object) {
		List<String> choiceFieldNames = Arrays.asList("settlementAmount", "settlementCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementAmount())) populatedFieldNames.add("settlementAmount");
		if (isSet(object.getSettlementCurrency())) populatedFieldNames.add("settlementCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SecurityLeg", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SecurityLeg>(NAME, "SecurityLeg", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SecurityLeg> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SecurityLeg.SecurityLegBuilder object = (SecurityLeg.SecurityLegBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("settlementAmount", "settlementCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementAmount())) populatedFieldNames.add("settlementAmount");
		if (isSet(object.getSettlementCurrency())) populatedFieldNames.add("settlementCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SecurityLeg", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SecurityLeg>(NAME, "SecurityLeg", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
