package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Security;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Security_oneOf")
public class SecurityOneOfRule implements Validator<Security> {
	
	private static final String NAME = "Security_oneOf";
	
    @Override
    public ValidationResult<Security> validate(RosettaPath path, Security object) {
		List<String> choiceFieldNames = Arrays.asList("bond", "convertibleBond", "equity", "exchangeTradedFund", "mortgageBackedSecurity", "mutualFund", "warrant");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBond())) populatedFieldNames.add("bond");
		if (isSet(object.getConvertibleBond())) populatedFieldNames.add("convertibleBond");
		if (isSet(object.getEquity())) populatedFieldNames.add("equity");
		if (isSet(object.getExchangeTradedFund())) populatedFieldNames.add("exchangeTradedFund");
		if (isSet(object.getMortgageBackedSecurity())) populatedFieldNames.add("mortgageBackedSecurity");
		if (isSet(object.getMutualFund())) populatedFieldNames.add("mutualFund");
		if (isSet(object.getWarrant())) populatedFieldNames.add("warrant");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Security", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Security>(NAME, "Security", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Security> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Security.SecurityBuilder object = (Security.SecurityBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("bond", "convertibleBond", "equity", "exchangeTradedFund", "mortgageBackedSecurity", "mutualFund", "warrant");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBond())) populatedFieldNames.add("bond");
		if (isSet(object.getConvertibleBond())) populatedFieldNames.add("convertibleBond");
		if (isSet(object.getEquity())) populatedFieldNames.add("equity");
		if (isSet(object.getExchangeTradedFund())) populatedFieldNames.add("exchangeTradedFund");
		if (isSet(object.getMortgageBackedSecurity())) populatedFieldNames.add("mortgageBackedSecurity");
		if (isSet(object.getMutualFund())) populatedFieldNames.add("mutualFund");
		if (isSet(object.getWarrant())) populatedFieldNames.add("warrant");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Security", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Security>(NAME, "Security", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
