package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SecurityValuationModel;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SecurityValuationModel_oneOf")
public class SecurityValuationModelOneOfRule implements Validator<SecurityValuationModel> {
	
	private static final String NAME = "SecurityValuationModel_oneOf";
	
    @Override
    public ValidationResult<SecurityValuationModel> validate(RosettaPath path, SecurityValuationModel object) {
		List<String> choiceFieldNames = Arrays.asList("bondValuationModel", "unitContractValuationModel");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBondValuationModel())) populatedFieldNames.add("bondValuationModel");
		if (isSet(object.getUnitContractValuationModel())) populatedFieldNames.add("unitContractValuationModel");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SecurityValuationModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SecurityValuationModel>(NAME, "SecurityValuationModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SecurityValuationModel> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SecurityValuationModel.SecurityValuationModelBuilder object = (SecurityValuationModel.SecurityValuationModelBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("bondValuationModel", "unitContractValuationModel");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getBondValuationModel())) populatedFieldNames.add("bondValuationModel");
		if (isSet(object.getUnitContractValuationModel())) populatedFieldNames.add("unitContractValuationModel");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SecurityValuationModel", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SecurityValuationModel>(NAME, "SecurityValuationModel", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
