package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SensitivityMethodology;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SensitivityMethodology_oneOf")
public class SensitivityMethodologyOneOfRule implements Validator<SensitivityMethodology> {
	
	private static final String NAME = "SensitivityMethodology_oneOf";
	
    @Override
    public ValidationResult<SensitivityMethodology> validate(RosettaPath path, SensitivityMethodology object) {
		List<String> choiceFieldNames = Arrays.asList("specifiedMethodology", "customMethodology");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSpecifiedMethodology())) populatedFieldNames.add("specifiedMethodology");
		if (isSet(object.getCustomMethodology())) populatedFieldNames.add("customMethodology");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SensitivityMethodology", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SensitivityMethodology>(NAME, "SensitivityMethodology", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SensitivityMethodology> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SensitivityMethodology.SensitivityMethodologyBuilder object = (SensitivityMethodology.SensitivityMethodologyBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("specifiedMethodology", "customMethodology");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSpecifiedMethodology())) populatedFieldNames.add("specifiedMethodology");
		if (isSet(object.getCustomMethodology())) populatedFieldNames.add("customMethodology");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SensitivityMethodology", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SensitivityMethodology>(NAME, "SensitivityMethodology", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
