package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SettlementRateSource;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SettlementRateSource_oneOf")
public class SettlementRateSourceOneOfRule implements Validator<SettlementRateSource> {
	
	private static final String NAME = "SettlementRateSource_oneOf";
	
    @Override
    public ValidationResult<SettlementRateSource> validate(RosettaPath path, SettlementRateSource object) {
		List<String> choiceFieldNames = Arrays.asList("informationSource", "cashSettlementReferenceBanks");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getInformationSource())) populatedFieldNames.add("informationSource");
		if (isSet(object.getCashSettlementReferenceBanks())) populatedFieldNames.add("cashSettlementReferenceBanks");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SettlementRateSource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SettlementRateSource>(NAME, "SettlementRateSource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SettlementRateSource> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SettlementRateSource.SettlementRateSourceBuilder object = (SettlementRateSource.SettlementRateSourceBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("informationSource", "cashSettlementReferenceBanks");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getInformationSource())) populatedFieldNames.add("informationSource");
		if (isSet(object.getCashSettlementReferenceBanks())) populatedFieldNames.add("cashSettlementReferenceBanks");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SettlementRateSource", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SettlementRateSource>(NAME, "SettlementRateSource", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
