package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SettlementTerms;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SettlementTerms_amountCurrency")
public class SettlementTermsAmountCurrencyChoiceRule implements Validator<SettlementTerms> {
	
	private static final String NAME = "SettlementTerms_amountCurrency";
	
    @Override
    public ValidationResult<SettlementTerms> validate(RosettaPath path, SettlementTerms object) {
		List<String> choiceFieldNames = Arrays.asList("settlementAmount", "settlementCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementAmount())) populatedFieldNames.add("settlementAmount");
		if (isSet(object.getSettlementCurrency())) populatedFieldNames.add("settlementCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SettlementTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SettlementTerms>(NAME, "SettlementTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SettlementTerms.SettlementTermsBuilder object = (SettlementTerms.SettlementTermsBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("settlementAmount", "settlementCurrency");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSettlementAmount())) populatedFieldNames.add("settlementAmount");
		if (isSet(object.getSettlementCurrency())) populatedFieldNames.add("settlementCurrency");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.OPTIONAL;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SettlementTerms", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SettlementTerms>(NAME, "SettlementTerms", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
