package org.isda.cdm.validation.choicerule;

import org.isda.cdm.SimmException;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("SimmException_oneOf")
public class SimmExceptionOneOfRule implements Validator<SimmException> {
	
	private static final String NAME = "SimmException_oneOf";
	
    @Override
    public ValidationResult<SimmException> validate(RosettaPath path, SimmException object) {
		List<String> choiceFieldNames = Arrays.asList("standardisedException", "asSpecified");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardisedException())) populatedFieldNames.add("standardisedException");
		if (isSet(object.getAsSpecified())) populatedFieldNames.add("asSpecified");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SimmException", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SimmException>(NAME, "SimmException", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<SimmException> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	SimmException.SimmExceptionBuilder object = (SimmException.SimmExceptionBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("standardisedException", "asSpecified");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getStandardisedException())) populatedFieldNames.add("standardisedException");
		if (isSet(object.getAsSpecified())) populatedFieldNames.add("asSpecified");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "SimmException", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<SimmException>(NAME, "SimmException", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
