package org.isda.cdm.validation.choicerule;

import org.isda.cdm.StubValue;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("StubValue_oneOf")
public class StubValueOneOfRule implements Validator<StubValue> {
	
	private static final String NAME = "StubValue_oneOf";
	
    @Override
    public ValidationResult<StubValue> validate(RosettaPath path, StubValue object) {
		List<String> choiceFieldNames = Arrays.asList("floatingRate", "stubRate", "stubAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFloatingRate())) populatedFieldNames.add("floatingRate");
		if (isSet(object.getStubRate())) populatedFieldNames.add("stubRate");
		if (isSet(object.getStubAmount())) populatedFieldNames.add("stubAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "StubValue", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<StubValue>(NAME, "StubValue", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<StubValue> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	StubValue.StubValueBuilder object = (StubValue.StubValueBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("floatingRate", "stubRate", "stubAmount");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getFloatingRate())) populatedFieldNames.add("floatingRate");
		if (isSet(object.getStubRate())) populatedFieldNames.add("stubRate");
		if (isSet(object.getStubAmount())) populatedFieldNames.add("stubAmount");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "StubValue", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<StubValue>(NAME, "StubValue", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
