package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Trade;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Trade_oneOf")
public class TradeOneOfRule implements Validator<Trade> {
	
	private static final String NAME = "Trade_oneOf";
	
    @Override
    public ValidationResult<Trade> validate(RosettaPath path, Trade object) {
		List<String> choiceFieldNames = Arrays.asList("execution", "contract");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getExecution())) populatedFieldNames.add("execution");
		if (isSet(object.getContract())) populatedFieldNames.add("contract");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Trade", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Trade>(NAME, "Trade", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Trade> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Trade.TradeBuilder object = (Trade.TradeBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("execution", "contract");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getExecution())) populatedFieldNames.add("execution");
		if (isSet(object.getContract())) populatedFieldNames.add("contract");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Trade", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Trade>(NAME, "Trade", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
