package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Trigger;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Trigger_choice1")
public class TriggerChoice1ChoiceRule implements Validator<Trigger> {
	
	private static final String NAME = "Trigger_choice1";
	
    @Override
    public ValidationResult<Trigger> validate(RosettaPath path, Trigger object) {
		List<String> choiceFieldNames = Arrays.asList("level", "levelPercentage", "creditEvents", "creditEventsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLevel())) populatedFieldNames.add("level");
		if (isSet(object.getLevelPercentage())) populatedFieldNames.add("levelPercentage");
		if (isSet(object.getCreditEvents())) populatedFieldNames.add("creditEvents");
		if (isSet(object.getCreditEventsReference())) populatedFieldNames.add("creditEventsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Trigger", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Trigger>(NAME, "Trigger", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Trigger> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Trigger.TriggerBuilder object = (Trigger.TriggerBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("level", "levelPercentage", "creditEvents", "creditEventsReference");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getLevel())) populatedFieldNames.add("level");
		if (isSet(object.getLevelPercentage())) populatedFieldNames.add("levelPercentage");
		if (isSet(object.getCreditEvents())) populatedFieldNames.add("creditEvents");
		if (isSet(object.getCreditEventsReference())) populatedFieldNames.add("creditEventsReference");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Trigger", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Trigger>(NAME, "Trigger", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
