package org.isda.cdm.validation.choicerule;

import org.isda.cdm.Underlier;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("Underlier_oneOf")
public class UnderlierOneOfRule implements Validator<Underlier> {
	
	private static final String NAME = "Underlier_oneOf";
	
    @Override
    public ValidationResult<Underlier> validate(RosettaPath path, Underlier object) {
		List<String> choiceFieldNames = Arrays.asList("singleUnderlier", "basket");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSingleUnderlier())) populatedFieldNames.add("singleUnderlier");
		if (isSet(object.getBasket())) populatedFieldNames.add("basket");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Underlier", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Underlier>(NAME, "Underlier", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<Underlier> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	Underlier.UnderlierBuilder object = (Underlier.UnderlierBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("singleUnderlier", "basket");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSingleUnderlier())) populatedFieldNames.add("singleUnderlier");
		if (isSet(object.getBasket())) populatedFieldNames.add("basket");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "Underlier", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<Underlier>(NAME, "Underlier", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
