package org.isda.cdm.validation.choicerule;

import org.isda.cdm.ValuationDate;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("ValuationDate_oneOf")
public class ValuationDateOneOfRule implements Validator<ValuationDate> {
	
	private static final String NAME = "ValuationDate_oneOf";
	
    @Override
    public ValidationResult<ValuationDate> validate(RosettaPath path, ValuationDate object) {
		List<String> choiceFieldNames = Arrays.asList("singleValuationDate", "multipleValuationDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSingleValuationDate())) populatedFieldNames.add("singleValuationDate");
		if (isSet(object.getMultipleValuationDates())) populatedFieldNames.add("multipleValuationDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ValuationDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ValuationDate>(NAME, "ValuationDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<ValuationDate> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	ValuationDate.ValuationDateBuilder object = (ValuationDate.ValuationDateBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("singleValuationDate", "multipleValuationDates");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getSingleValuationDate())) populatedFieldNames.add("singleValuationDate");
		if (isSet(object.getMultipleValuationDates())) populatedFieldNames.add("multipleValuationDates");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "ValuationDate", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<ValuationDate>(NAME, "ValuationDate", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
