package org.isda.cdm.validation.choicerule;

import org.isda.cdm.WeightedAveragingObservation;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedList;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaChoiceRule;
import com.rosetta.model.lib.path.RosettaPath;
			
import static com.rosetta.model.lib.validation.ExistenceChecker.isSet;

/**
 * @version 2.5.4
 */
@RosettaChoiceRule("WeightedAveragingObservation_choice")
public class WeightedAveragingObservationChoiceRule implements Validator<WeightedAveragingObservation> {
	
	private static final String NAME = "WeightedAveragingObservation_choice";
	
    @Override
    public ValidationResult<WeightedAveragingObservation> validate(RosettaPath path, WeightedAveragingObservation object) {
		List<String> choiceFieldNames = Arrays.asList("dateTime", "observationNumber");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateTime())) populatedFieldNames.add("dateTime");
		if (isSet(object.getObservationNumber())) populatedFieldNames.add("observationNumber");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "WeightedAveragingObservation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<WeightedAveragingObservation>(NAME, "WeightedAveragingObservation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
    
    @Override
    public ValidationResult<WeightedAveragingObservation> validate(RosettaPath path, RosettaModelObjectBuilder builder) {
    	WeightedAveragingObservation.WeightedAveragingObservationBuilder object = (WeightedAveragingObservation.WeightedAveragingObservationBuilder) builder;
		List<String> choiceFieldNames = Arrays.asList("dateTime", "observationNumber");
		List<String> populatedFieldNames = new LinkedList<>();
		if (isSet(object.getDateTime())) populatedFieldNames.add("dateTime");
		if (isSet(object.getObservationNumber())) populatedFieldNames.add("observationNumber");
		
		ChoiceRuleValidationMethod validationMethod = ChoiceRuleValidationMethod.REQUIRED;
		
		if (validationMethod.check(populatedFieldNames.size())) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.CHOICE_RULE, "WeightedAveragingObservation", path, "");
		}
		return new ValidationResult.ChoiceRuleFailure<WeightedAveragingObservation>(NAME, "WeightedAveragingObservation", choiceFieldNames, path, populatedFieldNames, validationMethod);
    }
}
