package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ActualPrice;
import org.isda.cdm.PriceExpressionEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ActualPrice_currency")
public class ActualPriceCurrencyDataRule implements Validator<ActualPrice> {
	
	private static final String NAME = "ActualPrice_currency";
	private static final String DEFINITION = "when ActualPrice -> priceExpression = PriceExpressionEnum.PercentageOfNotional\n" + 
		"then ActualPrice -> currency is absent";
	
	@Override
	public ValidationResult<ActualPrice> validate(RosettaPath path, ActualPrice actualPrice) {
		ComparisonResult result = executeDataRule(actualPrice);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ActualPrice", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ActualPrice", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ActualPrice> validate(RosettaPath path, RosettaModelObjectBuilder actualPrice) {
		ComparisonResult result = executeDataRule((ActualPrice)actualPrice.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ActualPrice", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ActualPrice", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ActualPrice actualPrice) {
		if (ruleIsApplicable(actualPrice).get()) {
			return evaluateThenExpression(actualPrice);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ActualPrice actualPrice) {
		return areEqual(MapperS.of(actualPrice).<PriceExpressionEnum>map("getPriceExpression", ActualPrice::getPriceExpression), MapperS.of(PriceExpressionEnum.PERCENTAGE_OF_NOTIONAL));
	}
	
	private ComparisonResult evaluateThenExpression(ActualPrice actualPrice) {
		return notExists(MapperS.of(actualPrice).map("getCurrency", ActualPrice::getCurrency).<String>map("getValue", FieldWithMeta::getValue));
	}
}
