package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.AdditionalDisruptionEvents;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AdditionalDisruptionEvents_maximumStockLoanRate")
public class AdditionalDisruptionEventsMaximumStockLoanRateDataRule implements Validator<AdditionalDisruptionEvents> {
	
	private static final String NAME = "AdditionalDisruptionEvents_maximumStockLoanRate";
	private static final String DEFINITION = "when AdditionalDisruptionEvents -> maximumStockLoanRate exists\n" + 
		"then AdditionalDisruptionEvents -> maximumStockLoanRate >= 0\n" + 
		"	and AdditionalDisruptionEvents -> maximumStockLoanRate <= 1";
	
	@Override
	public ValidationResult<AdditionalDisruptionEvents> validate(RosettaPath path, AdditionalDisruptionEvents additionalDisruptionEvents) {
		ComparisonResult result = executeDataRule(additionalDisruptionEvents);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalDisruptionEvents", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalDisruptionEvents", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AdditionalDisruptionEvents> validate(RosettaPath path, RosettaModelObjectBuilder additionalDisruptionEvents) {
		ComparisonResult result = executeDataRule((AdditionalDisruptionEvents)additionalDisruptionEvents.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalDisruptionEvents", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalDisruptionEvents", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AdditionalDisruptionEvents additionalDisruptionEvents) {
		if (ruleIsApplicable(additionalDisruptionEvents).get()) {
			return evaluateThenExpression(additionalDisruptionEvents);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AdditionalDisruptionEvents additionalDisruptionEvents) {
		return exists(MapperS.of(additionalDisruptionEvents).<BigDecimal>map("getMaximumStockLoanRate", AdditionalDisruptionEvents::getMaximumStockLoanRate), false);
	}
	
	private ComparisonResult evaluateThenExpression(AdditionalDisruptionEvents additionalDisruptionEvents) {
		return greaterThanEquals(MapperS.of(additionalDisruptionEvents).<BigDecimal>map("getMaximumStockLoanRate", AdditionalDisruptionEvents::getMaximumStockLoanRate), MapperS.of(Integer.valueOf(0))).and(lessThanEquals(MapperS.of(additionalDisruptionEvents).<BigDecimal>map("getMaximumStockLoanRate", AdditionalDisruptionEvents::getMaximumStockLoanRate), MapperS.of(Integer.valueOf(1))));
	}
}
