package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdditionalRightsEvent;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AdditionalRightsEvent_qualification")
public class AdditionalRightsEventQualificationDataRule implements Validator<AdditionalRightsEvent> {
	
	private static final String NAME = "AdditionalRightsEvent_qualification";
	private static final String DEFINITION = "when AdditionalRightsEvent -> qualification exists\n" + 
		"then AdditionalRightsEvent -> isApplicable = True";
	
	@Override
	public ValidationResult<AdditionalRightsEvent> validate(RosettaPath path, AdditionalRightsEvent additionalRightsEvent) {
		ComparisonResult result = executeDataRule(additionalRightsEvent);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalRightsEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalRightsEvent", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AdditionalRightsEvent> validate(RosettaPath path, RosettaModelObjectBuilder additionalRightsEvent) {
		ComparisonResult result = executeDataRule((AdditionalRightsEvent)additionalRightsEvent.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalRightsEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalRightsEvent", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AdditionalRightsEvent additionalRightsEvent) {
		if (ruleIsApplicable(additionalRightsEvent).get()) {
			return evaluateThenExpression(additionalRightsEvent);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AdditionalRightsEvent additionalRightsEvent) {
		return exists(MapperS.of(additionalRightsEvent).<String>map("getQualification", AdditionalRightsEvent::getQualification), false);
	}
	
	private ComparisonResult evaluateThenExpression(AdditionalRightsEvent additionalRightsEvent) {
		return areEqual(MapperS.of(additionalRightsEvent).<Boolean>map("getIsApplicable", AdditionalRightsEvent::getIsApplicable), MapperS.of(Boolean.valueOf(true)));
	}
}
