package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdditionalType;
import org.isda.cdm.AdditionalTypeEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AdditionalType_customValue")
public class AdditionalTypeCustomValueDataRule implements Validator<AdditionalType> {
	
	private static final String NAME = "AdditionalType_customValue";
	private static final String DEFINITION = "when AdditionalType -> standardValue = AdditionalTypeEnum.Other\n" + 
		"then AdditionalType -> customValue exists";
	
	@Override
	public ValidationResult<AdditionalType> validate(RosettaPath path, AdditionalType additionalType) {
		ComparisonResult result = executeDataRule(additionalType);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalType", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalType", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AdditionalType> validate(RosettaPath path, RosettaModelObjectBuilder additionalType) {
		ComparisonResult result = executeDataRule((AdditionalType)additionalType.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AdditionalType", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdditionalType", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AdditionalType additionalType) {
		if (ruleIsApplicable(additionalType).get()) {
			return evaluateThenExpression(additionalType);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AdditionalType additionalType) {
		return areEqual(MapperS.of(additionalType).<AdditionalTypeEnum>map("getStandardValue", AdditionalType::getStandardValue), MapperS.of(AdditionalTypeEnum.OTHER));
	}
	
	private ComparisonResult evaluateThenExpression(AdditionalType additionalType) {
		return exists(MapperS.of(additionalType).<String>map("getCustomValue", AdditionalType::getCustomValue), false);
	}
}
