package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableOrAdjustedDate;
import org.isda.cdm.BusinessDayAdjustments;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AdjustableOrAdjustedDate_adjustedDate")
public class AdjustableOrAdjustedDateAdjustedDateDataRule implements Validator<AdjustableOrAdjustedDate> {
	
	private static final String NAME = "AdjustableOrAdjustedDate_adjustedDate";
	private static final String DEFINITION = "when AdjustableOrAdjustedDate exists\n" + 
		"then AdjustableOrAdjustedDate -> adjustedDate exists \n" + 
		"		or ( AdjustableOrAdjustedDate -> unadjustedDate and AdjustableOrAdjustedDate -> dateAdjustments and AdjustableOrAdjustedDate -> adjustedDate ) exists\n" + 
		"		or ( ( AdjustableOrAdjustedDate -> unadjustedDate and AdjustableOrAdjustedDate -> dateAdjustments ) exists \n" + 
		"			and AdjustableOrAdjustedDate -> adjustedDate is absent )";
	
	@Override
	public ValidationResult<AdjustableOrAdjustedDate> validate(RosettaPath path, AdjustableOrAdjustedDate adjustableOrAdjustedDate) {
		ComparisonResult result = executeDataRule(adjustableOrAdjustedDate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdjustableOrAdjustedDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AdjustableOrAdjustedDate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AdjustableOrAdjustedDate> validate(RosettaPath path, RosettaModelObjectBuilder adjustableOrAdjustedDate) {
		ComparisonResult result = executeDataRule((AdjustableOrAdjustedDate)adjustableOrAdjustedDate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AdjustableOrAdjustedDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdjustableOrAdjustedDate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AdjustableOrAdjustedDate adjustableOrAdjustedDate) {
		if (ruleIsApplicable(adjustableOrAdjustedDate).get()) {
			return evaluateThenExpression(adjustableOrAdjustedDate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AdjustableOrAdjustedDate adjustableOrAdjustedDate) {
		return exists(MapperS.of(adjustableOrAdjustedDate), false);
	}
	
	private ComparisonResult evaluateThenExpression(AdjustableOrAdjustedDate adjustableOrAdjustedDate) {
		return exists(MapperS.of(adjustableOrAdjustedDate).map("getAdjustedDate", AdjustableOrAdjustedDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue), false).or(exists(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.of(MapperS.of(adjustableOrAdjustedDate).<Date>map("getUnadjustedDate", AdjustableOrAdjustedDate::getUnadjustedDate)), MapperTree.of(MapperS.of(adjustableOrAdjustedDate).<BusinessDayAdjustments>map("getDateAdjustments", AdjustableOrAdjustedDate::getDateAdjustments))), MapperTree.of(MapperS.of(adjustableOrAdjustedDate).map("getAdjustedDate", AdjustableOrAdjustedDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue))), false)).or(exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(adjustableOrAdjustedDate).<Date>map("getUnadjustedDate", AdjustableOrAdjustedDate::getUnadjustedDate)), MapperTree.of(MapperS.of(adjustableOrAdjustedDate).<BusinessDayAdjustments>map("getDateAdjustments", AdjustableOrAdjustedDate::getDateAdjustments))), false).and(notExists(MapperS.of(adjustableOrAdjustedDate).map("getAdjustedDate", AdjustableOrAdjustedDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue))));
	}
}
