package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableOrAdjustedOrRelativeDate;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.RelativeDateOffset;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AdjustableOrAdjustedOrRelativeDate_adjustedDate")
public class AdjustableOrAdjustedOrRelativeDateAdjustedDateDataRule implements Validator<AdjustableOrAdjustedOrRelativeDate> {
	
	private static final String NAME = "AdjustableOrAdjustedOrRelativeDate_adjustedDate";
	private static final String DEFINITION = "when AdjustableOrAdjustedOrRelativeDate exists\n" + 
		"then AdjustableOrAdjustedOrRelativeDate -> adjustedDate exists\n" + 
		"		or AdjustableOrAdjustedOrRelativeDate -> relativeDate exists\n" + 
		"		or ( AdjustableOrAdjustedOrRelativeDate -> unadjustedDate ) exists\n" + 
		"		or ( ( AdjustableOrAdjustedOrRelativeDate -> unadjustedDate and AdjustableOrAdjustedOrRelativeDate -> dateAdjustments ) exists \n" + 
		"			and AdjustableOrAdjustedOrRelativeDate -> adjustedDate is absent )";
	
	@Override
	public ValidationResult<AdjustableOrAdjustedOrRelativeDate> validate(RosettaPath path, AdjustableOrAdjustedOrRelativeDate adjustableOrAdjustedOrRelativeDate) {
		ComparisonResult result = executeDataRule(adjustableOrAdjustedOrRelativeDate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdjustableOrAdjustedOrRelativeDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AdjustableOrAdjustedOrRelativeDate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AdjustableOrAdjustedOrRelativeDate> validate(RosettaPath path, RosettaModelObjectBuilder adjustableOrAdjustedOrRelativeDate) {
		ComparisonResult result = executeDataRule((AdjustableOrAdjustedOrRelativeDate)adjustableOrAdjustedOrRelativeDate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AdjustableOrAdjustedOrRelativeDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AdjustableOrAdjustedOrRelativeDate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AdjustableOrAdjustedOrRelativeDate adjustableOrAdjustedOrRelativeDate) {
		if (ruleIsApplicable(adjustableOrAdjustedOrRelativeDate).get()) {
			return evaluateThenExpression(adjustableOrAdjustedOrRelativeDate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AdjustableOrAdjustedOrRelativeDate adjustableOrAdjustedOrRelativeDate) {
		return exists(MapperS.of(adjustableOrAdjustedOrRelativeDate), false);
	}
	
	private ComparisonResult evaluateThenExpression(AdjustableOrAdjustedOrRelativeDate adjustableOrAdjustedOrRelativeDate) {
		return exists(MapperS.of(adjustableOrAdjustedOrRelativeDate).map("getAdjustedDate", AdjustableOrAdjustedOrRelativeDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue), false).or(exists(MapperS.of(adjustableOrAdjustedOrRelativeDate).<RelativeDateOffset>map("getRelativeDate", AdjustableOrAdjustedOrRelativeDate::getRelativeDate), false)).or(exists(MapperS.of(adjustableOrAdjustedOrRelativeDate).<Date>map("getUnadjustedDate", AdjustableOrAdjustedOrRelativeDate::getUnadjustedDate), false)).or(exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(adjustableOrAdjustedOrRelativeDate).<Date>map("getUnadjustedDate", AdjustableOrAdjustedOrRelativeDate::getUnadjustedDate)), MapperTree.of(MapperS.of(adjustableOrAdjustedOrRelativeDate).<BusinessDayAdjustments>map("getDateAdjustments", AdjustableOrAdjustedOrRelativeDate::getDateAdjustments))), false).and(notExists(MapperS.of(adjustableOrAdjustedOrRelativeDate).map("getAdjustedDate", AdjustableOrAdjustedOrRelativeDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue))));
	}
}
