package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AllocationOutcome;
import org.isda.cdm.AllocationPrimitive;
import org.isda.cdm.Contract;
import org.isda.cdm.Execution;
import org.isda.cdm.Trade;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AllocationPrimitive_executionType")
public class AllocationPrimitiveExecutionTypeDataRule implements Validator<AllocationPrimitive> {
	
	private static final String NAME = "AllocationPrimitive_executionType";
	private static final String DEFINITION = "when AllocationPrimitive exists and AllocationPrimitive -> before -> execution exists\n" + 
		"then AllocationPrimitive -> after -> originalTrade -> execution exists\n" + 
		"		and AllocationPrimitive -> after -> allocatedTrade -> execution exists\n" + 
		"		and AllocationPrimitive -> after -> allocatedTrade -> contract is absent";
	
	@Override
	public ValidationResult<AllocationPrimitive> validate(RosettaPath path, AllocationPrimitive allocationPrimitive) {
		ComparisonResult result = executeDataRule(allocationPrimitive);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AllocationPrimitive", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AllocationPrimitive", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AllocationPrimitive> validate(RosettaPath path, RosettaModelObjectBuilder allocationPrimitive) {
		ComparisonResult result = executeDataRule((AllocationPrimitive)allocationPrimitive.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AllocationPrimitive", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AllocationPrimitive", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AllocationPrimitive allocationPrimitive) {
		if (ruleIsApplicable(allocationPrimitive).get()) {
			return evaluateThenExpression(allocationPrimitive);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AllocationPrimitive allocationPrimitive) {
		return exists(MapperS.of(allocationPrimitive), false).and(exists(MapperS.of(allocationPrimitive).<Trade>map("getBefore", AllocationPrimitive::getBefore).<Execution>map("getExecution", Trade::getExecution), false));
	}
	
	private ComparisonResult evaluateThenExpression(AllocationPrimitive allocationPrimitive) {
		return exists(MapperS.of(allocationPrimitive).<AllocationOutcome>map("getAfter", AllocationPrimitive::getAfter).<Trade>map("getOriginalTrade", AllocationOutcome::getOriginalTrade).<Execution>map("getExecution", Trade::getExecution), false).and(exists(MapperS.of(allocationPrimitive).<AllocationOutcome>map("getAfter", AllocationPrimitive::getAfter).<Trade>mapC("getAllocatedTrade", AllocationOutcome::getAllocatedTrade).<Execution>map("getExecution", Trade::getExecution), false)).and(notExists(MapperS.of(allocationPrimitive).<AllocationOutcome>map("getAfter", AllocationPrimitive::getAfter).<Trade>mapC("getAllocatedTrade", AllocationOutcome::getAllocatedTrade).<Contract>map("getContract", Trade::getContract)));
	}
}
