package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AppropriatedCollateralValuation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AppropriatedCollateralValuation_notSpecified")
public class AppropriatedCollateralValuationNotSpecifiedDataRule implements Validator<AppropriatedCollateralValuation> {
	
	private static final String NAME = "AppropriatedCollateralValuation_notSpecified";
	private static final String DEFINITION = "when AppropriatedCollateralValuation -> isSpecified = False\n" + 
		"then AppropriatedCollateralValuation -> election is absent";
	
	@Override
	public ValidationResult<AppropriatedCollateralValuation> validate(RosettaPath path, AppropriatedCollateralValuation appropriatedCollateralValuation) {
		ComparisonResult result = executeDataRule(appropriatedCollateralValuation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AppropriatedCollateralValuation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AppropriatedCollateralValuation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AppropriatedCollateralValuation> validate(RosettaPath path, RosettaModelObjectBuilder appropriatedCollateralValuation) {
		ComparisonResult result = executeDataRule((AppropriatedCollateralValuation)appropriatedCollateralValuation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AppropriatedCollateralValuation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AppropriatedCollateralValuation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AppropriatedCollateralValuation appropriatedCollateralValuation) {
		if (ruleIsApplicable(appropriatedCollateralValuation).get()) {
			return evaluateThenExpression(appropriatedCollateralValuation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AppropriatedCollateralValuation appropriatedCollateralValuation) {
		return areEqual(MapperS.of(appropriatedCollateralValuation).<Boolean>map("getIsSpecified", AppropriatedCollateralValuation::getIsSpecified), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(AppropriatedCollateralValuation appropriatedCollateralValuation) {
		return notExists(MapperS.of(appropriatedCollateralValuation).<String>map("getElection", AppropriatedCollateralValuation::getElection));
	}
}
