package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AssetPool;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("AssetPool_effectiveDate")
public class AssetPoolEffectiveDateDataRule implements Validator<AssetPool> {
	
	private static final String NAME = "AssetPool_effectiveDate";
	private static final String DEFINITION = "when AssetPool -> version is absent\n" + 
		"then AssetPool -> effectiveDate is absent";
	
	@Override
	public ValidationResult<AssetPool> validate(RosettaPath path, AssetPool assetPool) {
		ComparisonResult result = executeDataRule(assetPool);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "AssetPool", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "AssetPool", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<AssetPool> validate(RosettaPath path, RosettaModelObjectBuilder assetPool) {
		ComparisonResult result = executeDataRule((AssetPool)assetPool.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AssetPool", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "AssetPool", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(AssetPool assetPool) {
		if (ruleIsApplicable(assetPool).get()) {
			return evaluateThenExpression(assetPool);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(AssetPool assetPool) {
		return notExists(MapperS.of(assetPool).<String>map("getVersion", AssetPool::getVersion));
	}
	
	private ComparisonResult evaluateThenExpression(AssetPool assetPool) {
		return notExists(MapperS.of(assetPool).<Date>map("getEffectiveDate", AssetPool::getEffectiveDate));
	}
}
