package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.BasketReferenceInformation;
import org.isda.cdm.GeneralTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("BasketReferenceInformation_nameOrId")
public class BasketReferenceInformationNameOrIdDataRule implements Validator<GeneralTerms> {
	
	private static final String NAME = "BasketReferenceInformation_nameOrId";
	private static final String DEFINITION = "when GeneralTerms -> basketReferenceInformation exists\n" + 
		"then GeneralTerms -> basketReferenceInformation -> basketName exists\n" + 
		"		or GeneralTerms -> basketReferenceInformation -> basketId exists";
	
	@Override
	public ValidationResult<GeneralTerms> validate(RosettaPath path, GeneralTerms generalTerms) {
		ComparisonResult result = executeDataRule(generalTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "GeneralTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "GeneralTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<GeneralTerms> validate(RosettaPath path, RosettaModelObjectBuilder generalTerms) {
		ComparisonResult result = executeDataRule((GeneralTerms)generalTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "GeneralTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "GeneralTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(GeneralTerms generalTerms) {
		if (ruleIsApplicable(generalTerms).get()) {
			return evaluateThenExpression(generalTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(GeneralTerms generalTerms) {
		return exists(MapperS.of(generalTerms).<BasketReferenceInformation>map("getBasketReferenceInformation", GeneralTerms::getBasketReferenceInformation), false);
	}
	
	private ComparisonResult evaluateThenExpression(GeneralTerms generalTerms) {
		return exists(MapperS.of(generalTerms).<BasketReferenceInformation>map("getBasketReferenceInformation", GeneralTerms::getBasketReferenceInformation).map("getBasketName", BasketReferenceInformation::getBasketName).<String>map("getValue", FieldWithMeta::getValue), false).or(exists(MapperS.of(generalTerms).<BasketReferenceInformation>map("getBasketReferenceInformation", GeneralTerms::getBasketReferenceInformation).mapC("getBasketId", BasketReferenceInformation::getBasketId).<String>map("getValue", FieldWithMeta::getValue), false));
	}
}
