package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.BasketReferenceInformation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("BasketReferenceInformation_nthToDefault")
public class BasketReferenceInformationNthToDefaultDataRule implements Validator<BasketReferenceInformation> {
	
	private static final String NAME = "BasketReferenceInformation_nthToDefault";
	private static final String DEFINITION = "when BasketReferenceInformation -> mthToDefault exists\n" + 
		"then BasketReferenceInformation -> nthToDefault exists";
	
	@Override
	public ValidationResult<BasketReferenceInformation> validate(RosettaPath path, BasketReferenceInformation basketReferenceInformation) {
		ComparisonResult result = executeDataRule(basketReferenceInformation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "BasketReferenceInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "BasketReferenceInformation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<BasketReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder basketReferenceInformation) {
		ComparisonResult result = executeDataRule((BasketReferenceInformation)basketReferenceInformation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "BasketReferenceInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "BasketReferenceInformation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(BasketReferenceInformation basketReferenceInformation) {
		if (ruleIsApplicable(basketReferenceInformation).get()) {
			return evaluateThenExpression(basketReferenceInformation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(BasketReferenceInformation basketReferenceInformation) {
		return exists(MapperS.of(basketReferenceInformation).<Integer>map("getMthToDefault", BasketReferenceInformation::getMthToDefault), false);
	}
	
	private ComparisonResult evaluateThenExpression(BasketReferenceInformation basketReferenceInformation) {
		return exists(MapperS.of(basketReferenceInformation).<Integer>map("getNthToDefault", BasketReferenceInformation::getNthToDefault), false);
	}
}
