package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.AdjustableOrRelativeDate;
import org.isda.cdm.AdjustedRelativeDateOffset;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Payout;
import org.isda.cdm.RelativeDateOffset;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CalculationPeriodDates_effectiveDate_terminationDate")
public class CalculationPeriodDatesEffectiveDateTerminationDateDataRule implements Validator<EconomicTerms> {
	
	private static final String NAME = "CalculationPeriodDates_effectiveDate_terminationDate";
	private static final String DEFINITION = "when effectiveDate exists\n" + 
		"	and terminationDate exists\n" + 
		"then effectiveDate > terminationDate";
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, EconomicTerms economicTerms) {
		ComparisonResult result = executeDataRule(economicTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, RosettaModelObjectBuilder economicTerms) {
		ComparisonResult result = executeDataRule((EconomicTerms)economicTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EconomicTerms economicTerms) {
		if (ruleIsApplicable(economicTerms).get()) {
			return evaluateThenExpression(economicTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EconomicTerms economicTerms) {
		return exists(MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))), false).and(exists(MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))), false));
	}
	
	private ComparisonResult evaluateThenExpression(EconomicTerms economicTerms) {
		return greaterThan(MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getEffectiveDate", EconomicTerms::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getEffectiveDate", CalculationPeriodDates::getEffectiveDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<AdjustableOrRelativeDate>map("getTerminationDate", EconomicTerms::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).map("getAdjustedDate", AdjustableDate::getAdjustedDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.or(MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustableDate>map("getAdjustableDate", AdjustableOrRelativeDate::getAdjustableDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate)), MapperTree.of(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout).<CalculationPeriodDates>map("getCalculationPeriodDates", EquityPayout::getCalculationPeriodDates).<AdjustableOrRelativeDate>map("getTerminationDate", CalculationPeriodDates::getTerminationDate).<AdjustedRelativeDateOffset>map("getRelativeDate", AdjustableOrRelativeDate::getRelativeDate).<Date>map("getAdjustedDate", RelativeDateOffset::getAdjustedDate)))))))))));
	}
}
