package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CalculationPeriodDates;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.InterestRatePayout;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CalculationPeriodDates_firstCompoundingPeriodEndDate")
public class CalculationPeriodDatesFirstCompoundingPeriodEndDateDataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "CalculationPeriodDates_firstCompoundingPeriodEndDate";
	private static final String DEFINITION = "when InterestRatePayout -> compoundingMethod is absent\n" + 
		"		or InterestRatePayout -> compoundingMethod = CompoundingMethodEnum.None\n" + 
		"then InterestRatePayout -> calculationPeriodDates -> firstCompoundingPeriodEndDate is absent";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return notExists(MapperS.of(interestRatePayout).<CompoundingMethodEnum>map("getCompoundingMethod", InterestRatePayout::getCompoundingMethod)).or(areEqual(MapperS.of(interestRatePayout).<CompoundingMethodEnum>map("getCompoundingMethod", InterestRatePayout::getCompoundingMethod), MapperS.of(CompoundingMethodEnum.NONE)));
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return notExists(MapperS.of(interestRatePayout).<CalculationPeriodDates>map("getCalculationPeriodDates", InterestRatePayout::getCalculationPeriodDates).<Date>map("getFirstCompoundingPeriodEndDate", CalculationPeriodDates::getFirstCompoundingPeriodEndDate));
	}
}
