package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CashSettlementTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CashSettlementTerms_cashSettlementBusinessDays")
public class CashSettlementTermsCashSettlementBusinessDaysDataRule implements Validator<CashSettlementTerms> {
	
	private static final String NAME = "CashSettlementTerms_cashSettlementBusinessDays";
	private static final String DEFINITION = "when CashSettlementTerms -> cashSettlementBusinessDays exists\n" + 
		"then CashSettlementTerms -> cashSettlementBusinessDays >= 0";
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms cashSettlementTerms) {
		ComparisonResult result = executeDataRule(cashSettlementTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder cashSettlementTerms) {
		ComparisonResult result = executeDataRule((CashSettlementTerms)cashSettlementTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CashSettlementTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CashSettlementTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CashSettlementTerms cashSettlementTerms) {
		if (ruleIsApplicable(cashSettlementTerms).get()) {
			return evaluateThenExpression(cashSettlementTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CashSettlementTerms cashSettlementTerms) {
		return exists(MapperS.of(cashSettlementTerms).<Integer>map("getCashSettlementBusinessDays", CashSettlementTerms::getCashSettlementBusinessDays), false);
	}
	
	private ComparisonResult evaluateThenExpression(CashSettlementTerms cashSettlementTerms) {
		return greaterThanEquals(MapperS.of(cashSettlementTerms).<Integer>map("getCashSettlementBusinessDays", CashSettlementTerms::getCashSettlementBusinessDays), MapperS.of(Integer.valueOf(0)));
	}
}
