package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.TradeDate;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Contract_clearedDate")
public class ContractClearedDateDataRule implements Validator<Contract> {
	
	private static final String NAME = "Contract_clearedDate";
	private static final String DEFINITION = "when Contract -> clearedDate exists\n" + 
		"then Contract -> clearedDate >= Contract -> tradeDate -> date";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return exists(MapperS.of(contract).<Date>map("getClearedDate", Contract::getClearedDate), false);
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return greaterThanEquals(MapperS.of(contract).<Date>map("getClearedDate", Contract::getClearedDate), MapperS.of(contract).<TradeDate>map("getTradeDate", Contract::getTradeDate).<Date>map("getDate", TradeDate::getDate));
	}
}
