package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Contract_hedgingParty")
public class ContractHedgingPartyDataRule implements Validator<Contract> {
	
	private static final String NAME = "Contract_hedgingParty";
	private static final String DEFINITION = "when Contract -> partyRole -> role = PartyRoleEnum.HedgingParty\n" + 
		"then Contract -> partyRole -> role count <= 2";
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract contract) {
		ComparisonResult result = executeDataRule(contract);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder contract) {
		ComparisonResult result = executeDataRule((Contract)contract.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Contract", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Contract", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Contract contract) {
		if (ruleIsApplicable(contract).get()) {
			return evaluateThenExpression(contract);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Contract contract) {
		return areEqual(MapperS.of(contract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.HEDGING_PARTY));
	}
	
	private ComparisonResult evaluateThenExpression(Contract contract) {
		return lessThanEquals(MapperS.of(MapperS.of(contract).<PartyRole>mapC("getPartyRole", Contract::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole).resultCount()), MapperS.of(Integer.valueOf(2)));
	}
}
