package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.CrossRate;

import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CrossRate_rule_1")
public class CrossRateRule1DataRule implements Validator<CrossRate> {
	
	private static final String NAME = "CrossRate_rule_1";
	private static final String DEFINITION = "if CrossRate -> forwardPoints exists then CrossRate -> spotRate exists";
	
	@Override
	public ValidationResult<CrossRate> validate(RosettaPath path, CrossRate crossRate) {
		ComparisonResult result = executeDataRule(crossRate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CrossRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CrossRate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CrossRate> validate(RosettaPath path, RosettaModelObjectBuilder crossRate) {
		ComparisonResult result = executeDataRule((CrossRate)crossRate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationType.DATA_RULE, "CrossRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationType.DATA_RULE,  "CrossRate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CrossRate crossRate) {
		if (ruleIsApplicable(crossRate).get()) {
			return evaluateThenExpression(crossRate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CrossRate crossRate) {
		return exists(MapperS.of(crossRate).<BigDecimal>map("getForwardPoints", CrossRate::getForwardPoints), false);
	}
	
	private ComparisonResult evaluateThenExpression(CrossRate crossRate) {
		return exists(MapperS.of(crossRate).<BigDecimal>map("getSpotRate", CrossRate::getSpotRate), false);
	}
}
