package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CustodianRisk;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("CustodianRisk_specified")
public class CustodianRiskSpecifiedDataRule implements Validator<CustodianRisk> {
	
	private static final String NAME = "CustodianRisk_specified";
	private static final String DEFINITION = "when CustodianRisk -> isSpecified = False\n" + 
		"then CustodianRisk -> qualification is absent";
	
	@Override
	public ValidationResult<CustodianRisk> validate(RosettaPath path, CustodianRisk custodianRisk) {
		ComparisonResult result = executeDataRule(custodianRisk);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CustodianRisk", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CustodianRisk", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CustodianRisk> validate(RosettaPath path, RosettaModelObjectBuilder custodianRisk) {
		ComparisonResult result = executeDataRule((CustodianRisk)custodianRisk.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CustodianRisk", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CustodianRisk", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CustodianRisk custodianRisk) {
		if (ruleIsApplicable(custodianRisk).get()) {
			return evaluateThenExpression(custodianRisk);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CustodianRisk custodianRisk) {
		return areEqual(MapperS.of(custodianRisk).<Boolean>map("getIsSpecified", CustodianRisk::getIsSpecified), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(CustodianRisk custodianRisk) {
		return notExists(MapperS.of(custodianRisk).<String>map("getQualification", CustodianRisk::getQualification));
	}
}
