package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.DayCountFractionEnum;
import org.isda.cdm.DiscountingMethod;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Discounting_discountRate")
public class DiscountingDiscountRateDataRule implements Validator<DiscountingMethod> {
	
	private static final String NAME = "Discounting_discountRate";
	private static final String DEFINITION = "when DiscountingMethod -> discountRateDayCountFraction exists\n" + 
		"then DiscountingMethod -> discountRate exists";
	
	@Override
	public ValidationResult<DiscountingMethod> validate(RosettaPath path, DiscountingMethod discountingMethod) {
		ComparisonResult result = executeDataRule(discountingMethod);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "DiscountingMethod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "DiscountingMethod", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<DiscountingMethod> validate(RosettaPath path, RosettaModelObjectBuilder discountingMethod) {
		ComparisonResult result = executeDataRule((DiscountingMethod)discountingMethod.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DiscountingMethod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "DiscountingMethod", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(DiscountingMethod discountingMethod) {
		if (ruleIsApplicable(discountingMethod).get()) {
			return evaluateThenExpression(discountingMethod);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(DiscountingMethod discountingMethod) {
		return exists(MapperS.of(discountingMethod).map("getDiscountRateDayCountFraction", DiscountingMethod::getDiscountRateDayCountFraction).<DayCountFractionEnum>map("getValue", FieldWithMeta::getValue), false);
	}
	
	private ComparisonResult evaluateThenExpression(DiscountingMethod discountingMethod) {
		return exists(MapperS.of(discountingMethod).<BigDecimal>map("getDiscountRate", DiscountingMethod::getDiscountRate), false);
	}
}
