package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.DividendDateReference;
import org.isda.cdm.DividendDateReferenceEnum;
import org.isda.cdm.Offset;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("DividendDateReference_paymentDateOffset")
public class DividendDateReferencePaymentDateOffsetDataRule implements Validator<DividendDateReference> {
	
	private static final String NAME = "DividendDateReference_paymentDateOffset";
	private static final String DEFINITION = "when DividendDateReference -> paymentDateOffset exists\n" + 
		"then DividendDateReference -> dateReference = DividendDateReferenceEnum.SharePayment";
	
	@Override
	public ValidationResult<DividendDateReference> validate(RosettaPath path, DividendDateReference dividendDateReference) {
		ComparisonResult result = executeDataRule(dividendDateReference);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendDateReference", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendDateReference", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<DividendDateReference> validate(RosettaPath path, RosettaModelObjectBuilder dividendDateReference) {
		ComparisonResult result = executeDataRule((DividendDateReference)dividendDateReference.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendDateReference", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendDateReference", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(DividendDateReference dividendDateReference) {
		if (ruleIsApplicable(dividendDateReference).get()) {
			return evaluateThenExpression(dividendDateReference);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(DividendDateReference dividendDateReference) {
		return exists(MapperS.of(dividendDateReference).<Offset>map("getPaymentDateOffset", DividendDateReference::getPaymentDateOffset), false);
	}
	
	private ComparisonResult evaluateThenExpression(DividendDateReference dividendDateReference) {
		return areEqual(MapperS.of(dividendDateReference).<DividendDateReferenceEnum>map("getDateReference", DividendDateReference::getDateReference), MapperS.of(DividendDateReferenceEnum.SHARE_PAYMENT));
	}
}
