package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.DividendPayout;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("DividendPayout_dividendPayoutRatioNonCash")
public class DividendPayoutDividendPayoutRatioNonCashDataRule implements Validator<DividendPayout> {
	
	private static final String NAME = "DividendPayout_dividendPayoutRatioNonCash";
	private static final String DEFINITION = "when DividendPayout -> dividendPayoutRatioNonCash exists\n" + 
		"then DividendPayout -> dividendPayoutRatioNonCash >= 0\n" + 
		"		and DividendPayout -> dividendPayoutRatio <= 1";
	
	@Override
	public ValidationResult<DividendPayout> validate(RosettaPath path, DividendPayout dividendPayout) {
		ComparisonResult result = executeDataRule(dividendPayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendPayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<DividendPayout> validate(RosettaPath path, RosettaModelObjectBuilder dividendPayout) {
		ComparisonResult result = executeDataRule((DividendPayout)dividendPayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendPayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendPayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(DividendPayout dividendPayout) {
		if (ruleIsApplicable(dividendPayout).get()) {
			return evaluateThenExpression(dividendPayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(DividendPayout dividendPayout) {
		return exists(MapperS.of(dividendPayout).<BigDecimal>map("getDividendPayoutRatioNonCash", DividendPayout::getDividendPayoutRatioNonCash), false);
	}
	
	private ComparisonResult evaluateThenExpression(DividendPayout dividendPayout) {
		return greaterThanEquals(MapperS.of(dividendPayout).<BigDecimal>map("getDividendPayoutRatioNonCash", DividendPayout::getDividendPayoutRatioNonCash), MapperS.of(Integer.valueOf(0))).and(lessThanEquals(MapperS.of(dividendPayout).<BigDecimal>map("getDividendPayoutRatio", DividendPayout::getDividendPayoutRatio), MapperS.of(Integer.valueOf(1))));
	}
}
