package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.DividendPeriodEnum;
import org.isda.cdm.DividendReturnTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("DividendReturnTerms_dividendPeriod")
public class DividendReturnTermsDividendPeriodDataRule implements Validator<DividendReturnTerms> {
	
	private static final String NAME = "DividendReturnTerms_dividendPeriod";
	private static final String DEFINITION = "when DividendReturnTerms -> dividendPeriod exists\n" + 
		"then ( DividendReturnTerms -> dividendPeriodEffectiveDate\n" + 
		"		and DividendReturnTerms -> dividendPeriodEndDate ) is absent";
	
	@Override
	public ValidationResult<DividendReturnTerms> validate(RosettaPath path, DividendReturnTerms dividendReturnTerms) {
		ComparisonResult result = executeDataRule(dividendReturnTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendReturnTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendReturnTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<DividendReturnTerms> validate(RosettaPath path, RosettaModelObjectBuilder dividendReturnTerms) {
		ComparisonResult result = executeDataRule((DividendReturnTerms)dividendReturnTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DividendReturnTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "DividendReturnTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(DividendReturnTerms dividendReturnTerms) {
		if (ruleIsApplicable(dividendReturnTerms).get()) {
			return evaluateThenExpression(dividendReturnTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(DividendReturnTerms dividendReturnTerms) {
		return exists(MapperS.of(dividendReturnTerms).<DividendPeriodEnum>map("getDividendPeriod", DividendReturnTerms::getDividendPeriod), false);
	}
	
	private ComparisonResult evaluateThenExpression(DividendReturnTerms dividendReturnTerms) {
		return notExists(MapperTree.and(MapperTree.of(MapperS.of(dividendReturnTerms).map("getDividendPeriodEffectiveDate", DividendReturnTerms::getDividendPeriodEffectiveDate).<Date>map("getValue", FieldWithMeta::getValue)), MapperTree.of(MapperS.of(dividendReturnTerms).map("getDividendPeriodEndDate", DividendReturnTerms::getDividendPeriodEndDate).<Date>map("getValue", FieldWithMeta::getValue))));
	}
}
