package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.EarlyTerminationProvision;
import org.isda.cdm.MandatoryEarlyTermination;
import org.isda.cdm.OptionalEarlyTermination;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("EarlyTerminationProvision_mandatoryEarlyTermination")
public class EarlyTerminationProvisionMandatoryEarlyTerminationDataRule implements Validator<EarlyTerminationProvision> {
	
	private static final String NAME = "EarlyTerminationProvision_mandatoryEarlyTermination";
	private static final String DEFINITION = "when EarlyTerminationProvision exists\n" + 
		"then (EarlyTerminationProvision -> mandatoryEarlyTermination or EarlyTerminationProvision -> optionalEarlyTermination) exists\n" + 
		"		or (EarlyTerminationProvision -> mandatoryEarlyTermination and EarlyTerminationProvision -> optionalEarlyTermination) exists";
	
	@Override
	public ValidationResult<EarlyTerminationProvision> validate(RosettaPath path, EarlyTerminationProvision earlyTerminationProvision) {
		ComparisonResult result = executeDataRule(earlyTerminationProvision);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EarlyTerminationProvision", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EarlyTerminationProvision", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EarlyTerminationProvision> validate(RosettaPath path, RosettaModelObjectBuilder earlyTerminationProvision) {
		ComparisonResult result = executeDataRule((EarlyTerminationProvision)earlyTerminationProvision.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EarlyTerminationProvision", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EarlyTerminationProvision", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EarlyTerminationProvision earlyTerminationProvision) {
		if (ruleIsApplicable(earlyTerminationProvision).get()) {
			return evaluateThenExpression(earlyTerminationProvision);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EarlyTerminationProvision earlyTerminationProvision) {
		return exists(MapperS.of(earlyTerminationProvision), false);
	}
	
	private ComparisonResult evaluateThenExpression(EarlyTerminationProvision earlyTerminationProvision) {
		return exists(MapperTree.orDifferent(MapperTree.of(MapperS.of(earlyTerminationProvision).<MandatoryEarlyTermination>map("getMandatoryEarlyTermination", EarlyTerminationProvision::getMandatoryEarlyTermination)), MapperTree.of(MapperS.of(earlyTerminationProvision).<OptionalEarlyTermination>map("getOptionalEarlyTermination", EarlyTerminationProvision::getOptionalEarlyTermination))), false).or(exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(earlyTerminationProvision).<MandatoryEarlyTermination>map("getMandatoryEarlyTermination", EarlyTerminationProvision::getMandatoryEarlyTermination)), MapperTree.of(MapperS.of(earlyTerminationProvision).<OptionalEarlyTermination>map("getOptionalEarlyTermination", EarlyTerminationProvision::getOptionalEarlyTermination))), false));
	}
}
