package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.EquityPayout;
import org.isda.cdm.ExtraordinaryEvents;
import org.isda.cdm.Payout;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("EconomicTerms_extraordinaryEvents")
public class EconomicTermsExtraordinaryEventsDataRule implements Validator<EconomicTerms> {
	
	private static final String NAME = "EconomicTerms_extraordinaryEvents";
	private static final String DEFINITION = "when EconomicTerms -> extraordinaryEvents exists\n" + 
		"then EconomicTerms -> payout -> equityPayout exists";
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, EconomicTerms economicTerms) {
		ComparisonResult result = executeDataRule(economicTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EconomicTerms> validate(RosettaPath path, RosettaModelObjectBuilder economicTerms) {
		ComparisonResult result = executeDataRule((EconomicTerms)economicTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EconomicTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EconomicTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EconomicTerms economicTerms) {
		if (ruleIsApplicable(economicTerms).get()) {
			return evaluateThenExpression(economicTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EconomicTerms economicTerms) {
		return exists(MapperS.of(economicTerms).<ExtraordinaryEvents>map("getExtraordinaryEvents", EconomicTerms::getExtraordinaryEvents), false);
	}
	
	private ComparisonResult evaluateThenExpression(EconomicTerms economicTerms) {
		return exists(MapperS.of(economicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<EquityPayout>mapC("getEquityPayout", Payout::getEquityPayout), false);
	}
}
