package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ElectiveAmountElection;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ElectiveAmountElection_noAmount")
public class ElectiveAmountElectionNoAmountDataRule implements Validator<ElectiveAmountElection> {
	
	private static final String NAME = "ElectiveAmountElection_noAmount";
	private static final String DEFINITION = "when ElectiveAmountElection -> noAmount exists\n" + 
		"then ElectiveAmountElection -> noAmount = 0";
	
	@Override
	public ValidationResult<ElectiveAmountElection> validate(RosettaPath path, ElectiveAmountElection electiveAmountElection) {
		ComparisonResult result = executeDataRule(electiveAmountElection);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ElectiveAmountElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ElectiveAmountElection", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ElectiveAmountElection> validate(RosettaPath path, RosettaModelObjectBuilder electiveAmountElection) {
		ComparisonResult result = executeDataRule((ElectiveAmountElection)electiveAmountElection.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ElectiveAmountElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ElectiveAmountElection", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ElectiveAmountElection electiveAmountElection) {
		if (ruleIsApplicable(electiveAmountElection).get()) {
			return evaluateThenExpression(electiveAmountElection);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ElectiveAmountElection electiveAmountElection) {
		return exists(MapperS.of(electiveAmountElection).<Integer>map("getNoAmount", ElectiveAmountElection::getNoAmount), false);
	}
	
	private ComparisonResult evaluateThenExpression(ElectiveAmountElection electiveAmountElection) {
		return areEqual(MapperS.of(electiveAmountElection).<Integer>map("getNoAmount", ElectiveAmountElection::getNoAmount), MapperS.of(Integer.valueOf(0)));
	}
}
