package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AllocationPrimitive;
import org.isda.cdm.Event;
import org.isda.cdm.ExercisePrimitive;
import org.isda.cdm.Inception;
import org.isda.cdm.IntentEnum;
import org.isda.cdm.ObservationPrimitive;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ResetPrimitive;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Event_intent")
public class EventIntentDataRule implements Validator<Event> {
	
	private static final String NAME = "Event_intent";
	private static final String DEFINITION = "when ( Event -> primitive -> allocation\n" + 
		"		or Event -> primitive -> exercise\n" + 
		"		or Event -> primitive -> inception\n" + 
		"		or Event -> primitive -> observation\n" + 
		"		or Event -> primitive -> reset ) exists\n" + 
		"then Event -> intent is absent";
	
	@Override
	public ValidationResult<Event> validate(RosettaPath path, Event event) {
		ComparisonResult result = executeDataRule(event);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Event", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Event", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Event> validate(RosettaPath path, RosettaModelObjectBuilder event) {
		ComparisonResult result = executeDataRule((Event)event.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Event", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Event", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Event event) {
		if (ruleIsApplicable(event).get()) {
			return evaluateThenExpression(event);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Event event) {
		return exists(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.orDifferent(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<AllocationPrimitive>mapC("getAllocation", PrimitiveEvent::getAllocation)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ExercisePrimitive>map("getExercise", PrimitiveEvent::getExercise))), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<Inception>mapC("getInception", PrimitiveEvent::getInception))), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ObservationPrimitive>mapC("getObservation", PrimitiveEvent::getObservation))), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<ResetPrimitive>mapC("getReset", PrimitiveEvent::getReset))), false);
	}
	
	private ComparisonResult evaluateThenExpression(Event event) {
		return notExists(MapperS.of(event).<IntentEnum>map("getIntent", Event::getIntent));
	}
}
