package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.ExchangeRate;

import static com.rosetta.model.lib.validation.ValidatorHelper.exists;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ExchangeRate_forwardPoints_spotRate")
public class ExchangeRateForwardPointsSpotRateDataRule implements Validator<ExchangeRate> {
	
	private static final String NAME = "ExchangeRate_forwardPoints_spotRate";
	private static final String DEFINITION = "if ExchangeRate -> forwardPoints exists then ExchangeRate -> spotRate exists";
	
	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, ExchangeRate exchangeRate) {
		ComparisonResult result = executeDataRule(exchangeRate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ExchangeRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ExchangeRate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, RosettaModelObjectBuilder exchangeRate) {
		ComparisonResult result = executeDataRule((ExchangeRate)exchangeRate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationType.DATA_RULE, "ExchangeRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationType.DATA_RULE,  "ExchangeRate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ExchangeRate exchangeRate) {
		if (ruleIsApplicable(exchangeRate).get()) {
			return evaluateThenExpression(exchangeRate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ExchangeRate exchangeRate) {
		return exists(MapperS.of(exchangeRate).<BigDecimal>map("getForwardPoints", ExchangeRate::getForwardPoints), false);
	}
	
	private ComparisonResult evaluateThenExpression(ExchangeRate exchangeRate) {
		return exists(MapperS.of(exchangeRate).<BigDecimal>map("getSpotRate", ExchangeRate::getSpotRate), false);
	}
}
