package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Execution;
import org.isda.cdm.ExecutionTypeEnum;
import org.isda.cdm.LegalEntity;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Execution_executionVenue")
public class ExecutionExecutionVenueDataRule implements Validator<Execution> {
	
	private static final String NAME = "Execution_executionVenue";
	private static final String DEFINITION = "when Execution -> executionType = ExecutionTypeEnum.Electronic\n" + 
		"then Execution -> executionVenue exists";
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, Execution execution) {
		ComparisonResult result = executeDataRule(execution);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Execution", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Execution", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, RosettaModelObjectBuilder execution) {
		ComparisonResult result = executeDataRule((Execution)execution.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Execution", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Execution", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Execution execution) {
		if (ruleIsApplicable(execution).get()) {
			return evaluateThenExpression(execution);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Execution execution) {
		return areEqual(MapperS.of(execution).<ExecutionTypeEnum>map("getExecutionType", Execution::getExecutionType), MapperS.of(ExecutionTypeEnum.ELECTRONIC));
	}
	
	private ComparisonResult evaluateThenExpression(Execution execution) {
		return exists(MapperS.of(execution).<LegalEntity>map("getExecutionVenue", Execution::getExecutionVenue), false);
	}
}
