package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Execution;
import org.isda.cdm.PartyRole;
import org.isda.cdm.PartyRoleEnum;
import org.isda.cdm.Product;
import org.isda.cdm.Security;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Execution_security_partyRole_buyerSeller")
public class ExecutionSecurityPartyRoleBuyerSellerDataRule implements Validator<Execution> {
	
	private static final String NAME = "Execution_security_partyRole_buyerSeller";
	private static final String DEFINITION = "when Execution -> product -> security exists\n" + 
		"then Execution -> partyRole -> role contains PartyRoleEnum.Buyer and Execution -> partyRole -> role contains PartyRoleEnum.Seller";
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, Execution execution) {
		ComparisonResult result = executeDataRule(execution);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Execution", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Execution", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, RosettaModelObjectBuilder execution) {
		ComparisonResult result = executeDataRule((Execution)execution.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Execution", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Execution", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Execution execution) {
		if (ruleIsApplicable(execution).get()) {
			return evaluateThenExpression(execution);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Execution execution) {
		return exists(MapperS.of(execution).<Product>map("getProduct", Execution::getProduct).<Security>map("getSecurity", Product::getSecurity), false);
	}
	
	private ComparisonResult evaluateThenExpression(Execution execution) {
		return contains(MapperS.of(execution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.BUYER)).and(contains(MapperS.of(execution).<PartyRole>mapC("getPartyRole", Execution::getPartyRole).<PartyRoleEnum>map("getRole", PartyRole::getRole), MapperS.of(PartyRoleEnum.SELLER)));
	}
}
