package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.FeaturePayment;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FeaturePayment_amount")
public class FeaturePaymentAmountDataRule implements Validator<FeaturePayment> {
	
	private static final String NAME = "FeaturePayment_amount";
	private static final String DEFINITION = "when FeaturePayment -> amount exists\n" + 
		"then FeaturePayment -> amount >= 0.0";
	
	@Override
	public ValidationResult<FeaturePayment> validate(RosettaPath path, FeaturePayment featurePayment) {
		ComparisonResult result = executeDataRule(featurePayment);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FeaturePayment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FeaturePayment", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FeaturePayment> validate(RosettaPath path, RosettaModelObjectBuilder featurePayment) {
		ComparisonResult result = executeDataRule((FeaturePayment)featurePayment.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FeaturePayment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FeaturePayment", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FeaturePayment featurePayment) {
		if (ruleIsApplicable(featurePayment).get()) {
			return evaluateThenExpression(featurePayment);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FeaturePayment featurePayment) {
		return exists(MapperS.of(featurePayment).<BigDecimal>map("getAmount", FeaturePayment::getAmount), false);
	}
	
	private ComparisonResult evaluateThenExpression(FeaturePayment featurePayment) {
		return greaterThanEquals(MapperS.of(featurePayment).<BigDecimal>map("getAmount", FeaturePayment::getAmount), MapperS.of(new BigDecimal(0.0)));
	}
}
