package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRateDefinition;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FloatingRateDefinition_FloatingRateMultiplier")
public class FloatingRateDefinitionFloatingRateMultiplierDataRule implements Validator<FloatingRateDefinition> {
	
	private static final String NAME = "FloatingRateDefinition_FloatingRateMultiplier";
	private static final String DEFINITION = "when FloatingRateDefinition -> floatingRateMultiplier exists\n" + 
		"then FloatingRateDefinition -> floatingRateMultiplier <> 1";
	
	@Override
	public ValidationResult<FloatingRateDefinition> validate(RosettaPath path, FloatingRateDefinition floatingRateDefinition) {
		ComparisonResult result = executeDataRule(floatingRateDefinition);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FloatingRateDefinition", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FloatingRateDefinition", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FloatingRateDefinition> validate(RosettaPath path, RosettaModelObjectBuilder floatingRateDefinition) {
		ComparisonResult result = executeDataRule((FloatingRateDefinition)floatingRateDefinition.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FloatingRateDefinition", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FloatingRateDefinition", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FloatingRateDefinition floatingRateDefinition) {
		if (ruleIsApplicable(floatingRateDefinition).get()) {
			return evaluateThenExpression(floatingRateDefinition);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FloatingRateDefinition floatingRateDefinition) {
		return exists(MapperS.of(floatingRateDefinition).<BigDecimal>map("getFloatingRateMultiplier", FloatingRateDefinition::getFloatingRateMultiplier), false);
	}
	
	private ComparisonResult evaluateThenExpression(FloatingRateDefinition floatingRateDefinition) {
		return notEqual(MapperS.of(floatingRateDefinition).<BigDecimal>map("getFloatingRateMultiplier", FloatingRateDefinition::getFloatingRateMultiplier), MapperS.of(Integer.valueOf(1)));
	}
}
