package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.FloatingRate;
import org.isda.cdm.Schedule;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FloatingRate_floatingRateMultiplierSchedule")
public class FloatingRateFloatingRateMultiplierScheduleDataRule implements Validator<FloatingRate> {
	
	private static final String NAME = "FloatingRate_floatingRateMultiplierSchedule";
	private static final String DEFINITION = "when FloatingRate -> floatingRateMultiplierSchedule exists\n" + 
		"then FloatingRate -> floatingRateMultiplierSchedule -> initialValue <> 1.0";
	
	@Override
	public ValidationResult<FloatingRate> validate(RosettaPath path, FloatingRate floatingRate) {
		ComparisonResult result = executeDataRule(floatingRate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FloatingRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FloatingRate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FloatingRate> validate(RosettaPath path, RosettaModelObjectBuilder floatingRate) {
		ComparisonResult result = executeDataRule((FloatingRate)floatingRate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FloatingRate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FloatingRate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FloatingRate floatingRate) {
		if (ruleIsApplicable(floatingRate).get()) {
			return evaluateThenExpression(floatingRate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FloatingRate floatingRate) {
		return exists(MapperS.of(floatingRate).<Schedule>map("getFloatingRateMultiplierSchedule", FloatingRate::getFloatingRateMultiplierSchedule), false);
	}
	
	private ComparisonResult evaluateThenExpression(FloatingRate floatingRate) {
		return notEqual(MapperS.of(floatingRate).<Schedule>map("getFloatingRateMultiplierSchedule", FloatingRate::getFloatingRateMultiplierSchedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue), MapperS.of(new BigDecimal(1.0)));
	}
}
