package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.BasketReferenceInformation;
import org.isda.cdm.GeneralTerms;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_42")
public class FpMLCd42DataRule implements Validator<GeneralTerms> {
	
	private static final String NAME = "FpML_cd_42";
	private static final String DEFINITION = "when GeneralTerms -> basketReferenceInformation is absent\n" + 
		"then GeneralTerms -> substitution is absent";
	
	@Override
	public ValidationResult<GeneralTerms> validate(RosettaPath path, GeneralTerms generalTerms) {
		ComparisonResult result = executeDataRule(generalTerms);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "GeneralTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "GeneralTerms", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<GeneralTerms> validate(RosettaPath path, RosettaModelObjectBuilder generalTerms) {
		ComparisonResult result = executeDataRule((GeneralTerms)generalTerms.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "GeneralTerms", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "GeneralTerms", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(GeneralTerms generalTerms) {
		if (ruleIsApplicable(generalTerms).get()) {
			return evaluateThenExpression(generalTerms);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(GeneralTerms generalTerms) {
		return notExists(MapperS.of(generalTerms).<BasketReferenceInformation>map("getBasketReferenceInformation", GeneralTerms::getBasketReferenceInformation));
	}
	
	private ComparisonResult evaluateThenExpression(GeneralTerms generalTerms) {
		return notExists(MapperS.of(generalTerms).<Boolean>map("getSubstitution", GeneralTerms::getSubstitution));
	}
}
