package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.ConstituentWeight;
import org.isda.cdm.ReferencePool;
import org.isda.cdm.ReferencePoolItem;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_cd_44_openUnits")
public class FpMLCd44OpenUnitsDataRule implements Validator<ReferencePool> {
	
	private static final String NAME = "FpML_cd_44_openUnits";
	private static final String DEFINITION = "when ReferencePool -> referencePoolItem -> constituentWeight -> openUnits exists\n" + 
		"then ReferencePool -> referencePoolItem -> constituentWeight -> basketPercentage is absent";
	
	@Override
	public ValidationResult<ReferencePool> validate(RosettaPath path, ReferencePool referencePool) {
		ComparisonResult result = executeDataRule(referencePool);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ReferencePool", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ReferencePool", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ReferencePool> validate(RosettaPath path, RosettaModelObjectBuilder referencePool) {
		ComparisonResult result = executeDataRule((ReferencePool)referencePool.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ReferencePool", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ReferencePool", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ReferencePool referencePool) {
		if (ruleIsApplicable(referencePool).get()) {
			return evaluateThenExpression(referencePool);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ReferencePool referencePool) {
		return exists(MapperS.of(referencePool).<ReferencePoolItem>mapC("getReferencePoolItem", ReferencePool::getReferencePoolItem).<ConstituentWeight>map("getConstituentWeight", ReferencePoolItem::getConstituentWeight).<BigDecimal>map("getOpenUnits", ConstituentWeight::getOpenUnits), false);
	}
	
	private ComparisonResult evaluateThenExpression(ReferencePool referencePool) {
		return notExists(MapperS.of(referencePool).<ReferencePoolItem>mapC("getReferencePoolItem", ReferencePool::getReferencePoolItem).<ConstituentWeight>map("getConstituentWeight", ReferencePoolItem::getConstituentWeight).<BigDecimal>map("getBasketPercentage", ConstituentWeight::getBasketPercentage));
	}
}
