package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AdjustableDate;
import org.isda.cdm.CalculationPeriodDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_22")
public class FpMLIrd22DataRule implements Validator<CalculationPeriodDates> {
	
	private static final String NAME = "FpML_ird_22";
	private static final String DEFINITION = "when CalculationPeriodDates -> firstPeriodStartDate exists\n" + 
		"		and CalculationPeriodDates -> firstRegularPeriodStartDate exists\n" + 
		"then CalculationPeriodDates -> firstPeriodStartDate -> unadjustedDate < CalculationPeriodDates -> firstRegularPeriodStartDate";
	
	@Override
	public ValidationResult<CalculationPeriodDates> validate(RosettaPath path, CalculationPeriodDates calculationPeriodDates) {
		ComparisonResult result = executeDataRule(calculationPeriodDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CalculationPeriodDates> validate(RosettaPath path, RosettaModelObjectBuilder calculationPeriodDates) {
		ComparisonResult result = executeDataRule((CalculationPeriodDates)calculationPeriodDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CalculationPeriodDates calculationPeriodDates) {
		if (ruleIsApplicable(calculationPeriodDates).get()) {
			return evaluateThenExpression(calculationPeriodDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CalculationPeriodDates calculationPeriodDates) {
		return exists(MapperS.of(calculationPeriodDates).<AdjustableDate>map("getFirstPeriodStartDate", CalculationPeriodDates::getFirstPeriodStartDate), false).and(exists(MapperS.of(calculationPeriodDates).<Date>map("getFirstRegularPeriodStartDate", CalculationPeriodDates::getFirstRegularPeriodStartDate), false));
	}
	
	private ComparisonResult evaluateThenExpression(CalculationPeriodDates calculationPeriodDates) {
		return lessThan(MapperS.of(calculationPeriodDates).<AdjustableDate>map("getFirstPeriodStartDate", CalculationPeriodDates::getFirstPeriodStartDate).<Date>map("getUnadjustedDate", AdjustableDate::getUnadjustedDate), MapperS.of(calculationPeriodDates).<Date>map("getFirstRegularPeriodStartDate", CalculationPeriodDates::getFirstRegularPeriodStartDate));
	}
}
