package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.Schedule;
import org.isda.cdm.Step;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_25")
public class FpMLIrd25DataRule implements Validator<Schedule> {
	
	private static final String NAME = "FpML_ird_25";
	private static final String DEFINITION = "when Schedule -> step is absent\n" + 
		"then Schedule -> initialValue <> 0.0";
	
	@Override
	public ValidationResult<Schedule> validate(RosettaPath path, Schedule schedule) {
		ComparisonResult result = executeDataRule(schedule);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Schedule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Schedule", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Schedule> validate(RosettaPath path, RosettaModelObjectBuilder schedule) {
		ComparisonResult result = executeDataRule((Schedule)schedule.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Schedule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Schedule", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Schedule schedule) {
		if (ruleIsApplicable(schedule).get()) {
			return evaluateThenExpression(schedule);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Schedule schedule) {
		return notExists(MapperS.of(schedule).<Step>mapC("getStep", Schedule::getStep));
	}
	
	private ComparisonResult evaluateThenExpression(Schedule schedule) {
		return notEqual(MapperS.of(schedule).<BigDecimal>map("getInitialValue", Schedule::getInitialValue), MapperS.of(new BigDecimal(0.0)));
	}
}
