package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CashSettlementPaymentDate;
import org.isda.cdm.MandatoryEarlyTermination;
import org.isda.cdm.OptionCashSettlement;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_27")
public class FpMLIrd27DataRule implements Validator<MandatoryEarlyTermination> {
	
	private static final String NAME = "FpML_ird_27";
	private static final String DEFINITION = "when MandatoryEarlyTermination -> cashSettlement exists\n" + 
		"then MandatoryEarlyTermination -> cashSettlement -> cashSettlementPaymentDate is absent";
	
	@Override
	public ValidationResult<MandatoryEarlyTermination> validate(RosettaPath path, MandatoryEarlyTermination mandatoryEarlyTermination) {
		ComparisonResult result = executeDataRule(mandatoryEarlyTermination);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "MandatoryEarlyTermination", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "MandatoryEarlyTermination", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<MandatoryEarlyTermination> validate(RosettaPath path, RosettaModelObjectBuilder mandatoryEarlyTermination) {
		ComparisonResult result = executeDataRule((MandatoryEarlyTermination)mandatoryEarlyTermination.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MandatoryEarlyTermination", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "MandatoryEarlyTermination", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(MandatoryEarlyTermination mandatoryEarlyTermination) {
		if (ruleIsApplicable(mandatoryEarlyTermination).get()) {
			return evaluateThenExpression(mandatoryEarlyTermination);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(MandatoryEarlyTermination mandatoryEarlyTermination) {
		return exists(MapperS.of(mandatoryEarlyTermination).<OptionCashSettlement>map("getCashSettlement", MandatoryEarlyTermination::getCashSettlement), false);
	}
	
	private ComparisonResult evaluateThenExpression(MandatoryEarlyTermination mandatoryEarlyTermination) {
		return notExists(MapperS.of(mandatoryEarlyTermination).<OptionCashSettlement>map("getCashSettlement", MandatoryEarlyTermination::getCashSettlement).<CashSettlementPaymentDate>map("getCashSettlementPaymentDate", OptionCashSettlement::getCashSettlementPaymentDate));
	}
}
