package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.PaymentCalculationPeriod;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_34")
public class FpMLIrd34DataRule implements Validator<PaymentCalculationPeriod> {
	
	private static final String NAME = "FpML_ird_34";
	private static final String DEFINITION = "when PaymentCalculationPeriod exists\n" + 
		"then PaymentCalculationPeriod -> unadjustedPaymentDate exists\n" + 
		"		or PaymentCalculationPeriod -> adjustedPaymentDate exists";
	
	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, PaymentCalculationPeriod paymentCalculationPeriod) {
		ComparisonResult result = executeDataRule(paymentCalculationPeriod);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentCalculationPeriod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentCalculationPeriod", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder paymentCalculationPeriod) {
		ComparisonResult result = executeDataRule((PaymentCalculationPeriod)paymentCalculationPeriod.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentCalculationPeriod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentCalculationPeriod", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PaymentCalculationPeriod paymentCalculationPeriod) {
		if (ruleIsApplicable(paymentCalculationPeriod).get()) {
			return evaluateThenExpression(paymentCalculationPeriod);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PaymentCalculationPeriod paymentCalculationPeriod) {
		return exists(MapperS.of(paymentCalculationPeriod), false);
	}
	
	private ComparisonResult evaluateThenExpression(PaymentCalculationPeriod paymentCalculationPeriod) {
		return exists(MapperS.of(paymentCalculationPeriod).<Date>map("getUnadjustedPaymentDate", PaymentCalculationPeriod::getUnadjustedPaymentDate), false).or(exists(MapperS.of(paymentCalculationPeriod).<Date>map("getAdjustedPaymentDate", PaymentCalculationPeriod::getAdjustedPaymentDate), false));
	}
}
