package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.EarlyTerminationEvent;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_39")
public class FpMLIrd39DataRule implements Validator<EarlyTerminationEvent> {
	
	private static final String NAME = "FpML_ird_39";
	private static final String DEFINITION = "when EarlyTerminationEvent exists\n" + 
		"then EarlyTerminationEvent -> adjustedExerciseDate <= EarlyTerminationEvent -> adjustedEarlyTerminationDate";
	
	@Override
	public ValidationResult<EarlyTerminationEvent> validate(RosettaPath path, EarlyTerminationEvent earlyTerminationEvent) {
		ComparisonResult result = executeDataRule(earlyTerminationEvent);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "EarlyTerminationEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "EarlyTerminationEvent", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<EarlyTerminationEvent> validate(RosettaPath path, RosettaModelObjectBuilder earlyTerminationEvent) {
		ComparisonResult result = executeDataRule((EarlyTerminationEvent)earlyTerminationEvent.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "EarlyTerminationEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "EarlyTerminationEvent", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(EarlyTerminationEvent earlyTerminationEvent) {
		if (ruleIsApplicable(earlyTerminationEvent).get()) {
			return evaluateThenExpression(earlyTerminationEvent);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(EarlyTerminationEvent earlyTerminationEvent) {
		return exists(MapperS.of(earlyTerminationEvent), false);
	}
	
	private ComparisonResult evaluateThenExpression(EarlyTerminationEvent earlyTerminationEvent) {
		return lessThanEquals(MapperS.of(earlyTerminationEvent).<Date>map("getAdjustedExerciseDate", EarlyTerminationEvent::getAdjustedExerciseDate), MapperS.of(earlyTerminationEvent).<Date>map("getAdjustedEarlyTerminationDate", EarlyTerminationEvent::getAdjustedEarlyTerminationDate));
	}
}
