package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ExtensionEvent;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_42")
public class FpMLIrd42DataRule implements Validator<ExtensionEvent> {
	
	private static final String NAME = "FpML_ird_42";
	private static final String DEFINITION = "when ExtensionEvent exists\n" + 
		"then ExtensionEvent -> adjustedExerciseDate < ExtensionEvent -> adjustedExtendedTerminationDate";
	
	@Override
	public ValidationResult<ExtensionEvent> validate(RosettaPath path, ExtensionEvent extensionEvent) {
		ComparisonResult result = executeDataRule(extensionEvent);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ExtensionEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ExtensionEvent", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ExtensionEvent> validate(RosettaPath path, RosettaModelObjectBuilder extensionEvent) {
		ComparisonResult result = executeDataRule((ExtensionEvent)extensionEvent.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ExtensionEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ExtensionEvent", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ExtensionEvent extensionEvent) {
		if (ruleIsApplicable(extensionEvent).get()) {
			return evaluateThenExpression(extensionEvent);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ExtensionEvent extensionEvent) {
		return exists(MapperS.of(extensionEvent), false);
	}
	
	private ComparisonResult evaluateThenExpression(ExtensionEvent extensionEvent) {
		return lessThan(MapperS.of(extensionEvent).<Date>map("getAdjustedExerciseDate", ExtensionEvent::getAdjustedExerciseDate), MapperS.of(extensionEvent).<Date>map("getAdjustedExtendedTerminationDate", ExtensionEvent::getAdjustedExtendedTerminationDate));
	}
}
