package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Frequency;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.ResetFrequency;
import org.isda.cdm.WeeklyRollConventionEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_49")
public class FpMLIrd49DataRule implements Validator<ResetFrequency> {
	
	private static final String NAME = "FpML_ird_49";
	private static final String DEFINITION = "when ResetFrequency -> weeklyRollConvention exists\n" + 
		"then ResetFrequency -> period = PeriodExtendedEnum.W";
	
	@Override
	public ValidationResult<ResetFrequency> validate(RosettaPath path, ResetFrequency resetFrequency) {
		ComparisonResult result = executeDataRule(resetFrequency);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetFrequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetFrequency", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ResetFrequency> validate(RosettaPath path, RosettaModelObjectBuilder resetFrequency) {
		ComparisonResult result = executeDataRule((ResetFrequency)resetFrequency.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetFrequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetFrequency", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ResetFrequency resetFrequency) {
		if (ruleIsApplicable(resetFrequency).get()) {
			return evaluateThenExpression(resetFrequency);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ResetFrequency resetFrequency) {
		return exists(MapperS.of(resetFrequency).<WeeklyRollConventionEnum>map("getWeeklyRollConvention", ResetFrequency::getWeeklyRollConvention), false);
	}
	
	private ComparisonResult evaluateThenExpression(ResetFrequency resetFrequency) {
		return areEqual(MapperS.of(resetFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.W));
	}
}
