package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CalculationPeriodFrequency;
import org.isda.cdm.Frequency;
import org.isda.cdm.PeriodExtendedEnum;
import org.isda.cdm.RollConventionEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_60")
public class FpMLIrd60DataRule implements Validator<CalculationPeriodFrequency> {
	
	private static final String NAME = "FpML_ird_60";
	private static final String DEFINITION = "when CalculationPeriodFrequency -> period = PeriodExtendedEnum.T\n" + 
		"then CalculationPeriodFrequency -> rollConvention = RollConventionEnum.NONE";
	
	@Override
	public ValidationResult<CalculationPeriodFrequency> validate(RosettaPath path, CalculationPeriodFrequency calculationPeriodFrequency) {
		ComparisonResult result = executeDataRule(calculationPeriodFrequency);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodFrequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodFrequency", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<CalculationPeriodFrequency> validate(RosettaPath path, RosettaModelObjectBuilder calculationPeriodFrequency) {
		ComparisonResult result = executeDataRule((CalculationPeriodFrequency)calculationPeriodFrequency.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CalculationPeriodFrequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "CalculationPeriodFrequency", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(CalculationPeriodFrequency calculationPeriodFrequency) {
		if (ruleIsApplicable(calculationPeriodFrequency).get()) {
			return evaluateThenExpression(calculationPeriodFrequency);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(CalculationPeriodFrequency calculationPeriodFrequency) {
		return areEqual(MapperS.of(calculationPeriodFrequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.T));
	}
	
	private ComparisonResult evaluateThenExpression(CalculationPeriodFrequency calculationPeriodFrequency) {
		return areEqual(MapperS.of(calculationPeriodFrequency).<RollConventionEnum>map("getRollConvention", CalculationPeriodFrequency::getRollConvention), MapperS.of(RollConventionEnum.NONE));
	}
}
