package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.CompoundingMethodEnum;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.ResetDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FpML_ird_9")
public class FpMLIrd9DataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "FpML_ird_9";
	private static final String DEFINITION = "when InterestRatePayout -> compoundingMethod exists\n" + 
		"then InterestRatePayout -> resetDates exists";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<CompoundingMethodEnum>map("getCompoundingMethod", InterestRatePayout::getCompoundingMethod), false);
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<ResetDates>map("getResetDates", InterestRatePayout::getResetDates), false);
	}
}
