package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Frequency;
import org.isda.cdm.PeriodExtendedEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Frequency_period")
public class FrequencyPeriodDataRule implements Validator<Frequency> {
	
	private static final String NAME = "Frequency_period";
	private static final String DEFINITION = "when Frequency -> period = PeriodExtendedEnum.T\n" + 
		"then Frequency -> periodMultiplier = 1";
	
	@Override
	public ValidationResult<Frequency> validate(RosettaPath path, Frequency frequency) {
		ComparisonResult result = executeDataRule(frequency);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Frequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Frequency", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Frequency> validate(RosettaPath path, RosettaModelObjectBuilder frequency) {
		ComparisonResult result = executeDataRule((Frequency)frequency.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Frequency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Frequency", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Frequency frequency) {
		if (ruleIsApplicable(frequency).get()) {
			return evaluateThenExpression(frequency);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Frequency frequency) {
		return areEqual(MapperS.of(frequency).<PeriodExtendedEnum>map("getPeriod", Frequency::getPeriod), MapperS.of(PeriodExtendedEnum.T));
	}
	
	private ComparisonResult evaluateThenExpression(Frequency frequency) {
		return areEqual(MapperS.of(frequency).<Integer>map("getPeriodMultiplier", Frequency::getPeriodMultiplier), MapperS.of(Integer.valueOf(1)));
	}
}
