package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.FutureValueAmount;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FutureValueAmount_calculationPeriodNumberOfDays")
public class FutureValueAmountCalculationPeriodNumberOfDaysDataRule implements Validator<FutureValueAmount> {
	
	private static final String NAME = "FutureValueAmount_calculationPeriodNumberOfDays";
	private static final String DEFINITION = "when FutureValueAmount exists\n" + 
		"then FutureValueAmount -> calculationPeriodNumberOfDays >= 0";
	
	@Override
	public ValidationResult<FutureValueAmount> validate(RosettaPath path, FutureValueAmount futureValueAmount) {
		ComparisonResult result = executeDataRule(futureValueAmount);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FutureValueAmount", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FutureValueAmount", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FutureValueAmount> validate(RosettaPath path, RosettaModelObjectBuilder futureValueAmount) {
		ComparisonResult result = executeDataRule((FutureValueAmount)futureValueAmount.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FutureValueAmount", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FutureValueAmount", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FutureValueAmount futureValueAmount) {
		if (ruleIsApplicable(futureValueAmount).get()) {
			return evaluateThenExpression(futureValueAmount);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FutureValueAmount futureValueAmount) {
		return exists(MapperS.of(futureValueAmount), false);
	}
	
	private ComparisonResult evaluateThenExpression(FutureValueAmount futureValueAmount) {
		return greaterThanEquals(MapperS.of(futureValueAmount).<Integer>map("getCalculationPeriodNumberOfDays", FutureValueAmount::getCalculationPeriodNumberOfDays), MapperS.of(Integer.valueOf(0)));
	}
}
