package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.FxHaircutCurrency;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("FxHaircut_fxDesignatedCurrency")
public class FxHaircutFxDesignatedCurrencyDataRule implements Validator<FxHaircutCurrency> {
	
	private static final String NAME = "FxHaircut_fxDesignatedCurrency";
	private static final String DEFINITION = "when FxHaircutCurrency -> isTerminationCurrency = False\n" + 
		"then FxHaircutCurrency -> fxDesignatedCurrency exists";
	
	@Override
	public ValidationResult<FxHaircutCurrency> validate(RosettaPath path, FxHaircutCurrency fxHaircutCurrency) {
		ComparisonResult result = executeDataRule(fxHaircutCurrency);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FxHaircutCurrency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FxHaircutCurrency", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FxHaircutCurrency> validate(RosettaPath path, RosettaModelObjectBuilder fxHaircutCurrency) {
		ComparisonResult result = executeDataRule((FxHaircutCurrency)fxHaircutCurrency.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FxHaircutCurrency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FxHaircutCurrency", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FxHaircutCurrency fxHaircutCurrency) {
		if (ruleIsApplicable(fxHaircutCurrency).get()) {
			return evaluateThenExpression(fxHaircutCurrency);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FxHaircutCurrency fxHaircutCurrency) {
		return areEqual(MapperS.of(fxHaircutCurrency).<Boolean>map("getIsTerminationCurrency", FxHaircutCurrency::getIsTerminationCurrency), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(FxHaircutCurrency fxHaircutCurrency) {
		return exists(MapperS.of(fxHaircutCurrency).map("getFxDesignatedCurrency", FxHaircutCurrency::getFxDesignatedCurrency).<String>map("getValue", FieldWithMeta::getValue), false);
	}
}
