package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.IndexReferenceInformation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("IndexReferenceInformation_indexAnnexVersion")
public class IndexReferenceInformationIndexAnnexVersionDataRule implements Validator<IndexReferenceInformation> {
	
	private static final String NAME = "IndexReferenceInformation_indexAnnexVersion";
	private static final String DEFINITION = "when IndexReferenceInformation -> indexAnnexVersion exists\n" + 
		"then IndexReferenceInformation -> indexAnnexVersion >= 0";
	
	@Override
	public ValidationResult<IndexReferenceInformation> validate(RosettaPath path, IndexReferenceInformation indexReferenceInformation) {
		ComparisonResult result = executeDataRule(indexReferenceInformation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "IndexReferenceInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "IndexReferenceInformation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<IndexReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder indexReferenceInformation) {
		ComparisonResult result = executeDataRule((IndexReferenceInformation)indexReferenceInformation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "IndexReferenceInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "IndexReferenceInformation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(IndexReferenceInformation indexReferenceInformation) {
		if (ruleIsApplicable(indexReferenceInformation).get()) {
			return evaluateThenExpression(indexReferenceInformation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(IndexReferenceInformation indexReferenceInformation) {
		return exists(MapperS.of(indexReferenceInformation).<Integer>map("getIndexAnnexVersion", IndexReferenceInformation::getIndexAnnexVersion), false);
	}
	
	private ComparisonResult evaluateThenExpression(IndexReferenceInformation indexReferenceInformation) {
		return greaterThanEquals(MapperS.of(indexReferenceInformation).<Integer>map("getIndexAnnexVersion", IndexReferenceInformation::getIndexAnnexVersion), MapperS.of(Integer.valueOf(0)));
	}
}
