package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.InitialMargin;
import org.isda.cdm.Money;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("InitialMargin_marginThreshold")
public class InitialMarginMarginThresholdDataRule implements Validator<InitialMargin> {
	
	private static final String NAME = "InitialMargin_marginThreshold";
	private static final String DEFINITION = "when InitialMargin -> marginThreshold exists\n" + 
		"then InitialMargin -> marginThreshold -> amount > 0";
	
	@Override
	public ValidationResult<InitialMargin> validate(RosettaPath path, InitialMargin initialMargin) {
		ComparisonResult result = executeDataRule(initialMargin);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InitialMargin", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InitialMargin", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InitialMargin> validate(RosettaPath path, RosettaModelObjectBuilder initialMargin) {
		ComparisonResult result = executeDataRule((InitialMargin)initialMargin.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InitialMargin", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InitialMargin", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InitialMargin initialMargin) {
		if (ruleIsApplicable(initialMargin).get()) {
			return evaluateThenExpression(initialMargin);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InitialMargin initialMargin) {
		return exists(MapperS.of(initialMargin).<Money>map("getMarginThreshold", InitialMargin::getMarginThreshold), false);
	}
	
	private ComparisonResult evaluateThenExpression(InitialMargin initialMargin) {
		return greaterThan(MapperS.of(initialMargin).<Money>map("getMarginThreshold", InitialMargin::getMarginThreshold).<BigDecimal>map("getAmount", Money::getAmount), MapperS.of(Integer.valueOf(0)));
	}
}
