package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.StubPeriod;
import org.isda.cdm.StubValue;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("InitialStub_FinalStub")
public class InitialStubFinalStubDataRule implements Validator<InterestRatePayout> {
	
	private static final String NAME = "InitialStub_FinalStub";
	private static final String DEFINITION = "when InterestRatePayout -> stubPeriod exists\n" + 
		"then InterestRatePayout -> stubPeriod -> initialStub exists\n" + 
		"		or InterestRatePayout -> stubPeriod -> finalStub exists";
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout interestRatePayout) {
		ComparisonResult result = executeDataRule(interestRatePayout);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder interestRatePayout) {
		ComparisonResult result = executeDataRule((InterestRatePayout)interestRatePayout.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "InterestRatePayout", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "InterestRatePayout", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(InterestRatePayout interestRatePayout) {
		if (ruleIsApplicable(interestRatePayout).get()) {
			return evaluateThenExpression(interestRatePayout);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<StubPeriod>map("getStubPeriod", InterestRatePayout::getStubPeriod), false);
	}
	
	private ComparisonResult evaluateThenExpression(InterestRatePayout interestRatePayout) {
		return exists(MapperS.of(interestRatePayout).<StubPeriod>map("getStubPeriod", InterestRatePayout::getStubPeriod).<StubValue>map("getInitialStub", StubPeriod::getInitialStub), false).or(exists(MapperS.of(interestRatePayout).<StubPeriod>map("getStubPeriod", InterestRatePayout::getStubPeriod).<StubValue>map("getFinalStub", StubPeriod::getFinalStub), false));
	}
}
