package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.MultipleExercise;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("MultipleExercise_maximumNumberOfOptions")
public class MultipleExerciseMaximumNumberOfOptionsDataRule implements Validator<MultipleExercise> {
	
	private static final String NAME = "MultipleExercise_maximumNumberOfOptions";
	private static final String DEFINITION = "when MultipleExercise -> maximumNumberOfOptions exists\n" + 
		"then MultipleExercise -> maximumNotionalAmount >= 0.0";
	
	@Override
	public ValidationResult<MultipleExercise> validate(RosettaPath path, MultipleExercise multipleExercise) {
		ComparisonResult result = executeDataRule(multipleExercise);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "MultipleExercise", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "MultipleExercise", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<MultipleExercise> validate(RosettaPath path, RosettaModelObjectBuilder multipleExercise) {
		ComparisonResult result = executeDataRule((MultipleExercise)multipleExercise.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MultipleExercise", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "MultipleExercise", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(MultipleExercise multipleExercise) {
		if (ruleIsApplicable(multipleExercise).get()) {
			return evaluateThenExpression(multipleExercise);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(MultipleExercise multipleExercise) {
		return exists(MapperS.of(multipleExercise).<BigDecimal>map("getMaximumNumberOfOptions", MultipleExercise::getMaximumNumberOfOptions), false);
	}
	
	private ComparisonResult evaluateThenExpression(MultipleExercise multipleExercise) {
		return greaterThanEquals(MapperS.of(multipleExercise).<BigDecimal>map("getMaximumNotionalAmount", MultipleExercise::getMaximumNotionalAmount), MapperS.of(new BigDecimal(0.0)));
	}
}
