package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.MultipleValuationDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("MultipleValuationDates_businessDaysThereafter")
public class MultipleValuationDatesBusinessDaysThereafterDataRule implements Validator<MultipleValuationDates> {
	
	private static final String NAME = "MultipleValuationDates_businessDaysThereafter";
	private static final String DEFINITION = "when MultipleValuationDates -> businessDaysThereafter exists\n" + 
		"then MultipleValuationDates -> businessDaysThereafter >= 0";
	
	@Override
	public ValidationResult<MultipleValuationDates> validate(RosettaPath path, MultipleValuationDates multipleValuationDates) {
		ComparisonResult result = executeDataRule(multipleValuationDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "MultipleValuationDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "MultipleValuationDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<MultipleValuationDates> validate(RosettaPath path, RosettaModelObjectBuilder multipleValuationDates) {
		ComparisonResult result = executeDataRule((MultipleValuationDates)multipleValuationDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MultipleValuationDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "MultipleValuationDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(MultipleValuationDates multipleValuationDates) {
		if (ruleIsApplicable(multipleValuationDates).get()) {
			return evaluateThenExpression(multipleValuationDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(MultipleValuationDates multipleValuationDates) {
		return exists(MapperS.of(multipleValuationDates).<Integer>map("getBusinessDaysThereafter", MultipleValuationDates::getBusinessDaysThereafter), false);
	}
	
	private ComparisonResult evaluateThenExpression(MultipleValuationDates multipleValuationDates) {
		return greaterThanEquals(MapperS.of(multipleValuationDates).<Integer>map("getBusinessDaysThereafter", MultipleValuationDates::getBusinessDaysThereafter), MapperS.of(Integer.valueOf(0)));
	}
}
