package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.Quantity;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("NonNegativeQuantity_amount")
public class NonNegativeQuantityAmountDataRule implements Validator<NonNegativeQuantity> {
	
	private static final String NAME = "NonNegativeQuantity_amount";
	private static final String DEFINITION = "when NonNegativeQuantity exists\n" + 
		"then NonNegativeQuantity -> amount >= 0.0";
	
	@Override
	public ValidationResult<NonNegativeQuantity> validate(RosettaPath path, NonNegativeQuantity nonNegativeQuantity) {
		ComparisonResult result = executeDataRule(nonNegativeQuantity);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeQuantity", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeQuantity", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<NonNegativeQuantity> validate(RosettaPath path, RosettaModelObjectBuilder nonNegativeQuantity) {
		ComparisonResult result = executeDataRule((NonNegativeQuantity)nonNegativeQuantity.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeQuantity", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeQuantity", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(NonNegativeQuantity nonNegativeQuantity) {
		if (ruleIsApplicable(nonNegativeQuantity).get()) {
			return evaluateThenExpression(nonNegativeQuantity);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(NonNegativeQuantity nonNegativeQuantity) {
		return exists(MapperS.of(nonNegativeQuantity), false);
	}
	
	private ComparisonResult evaluateThenExpression(NonNegativeQuantity nonNegativeQuantity) {
		return greaterThanEquals(MapperS.of(nonNegativeQuantity).<BigDecimal>map("getAmount", Quantity::getAmount), MapperS.of(new BigDecimal(0.0)));
	}
}
