package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.NonNegativeSchedule;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("NonNegativeSchedule_initialValue")
public class NonNegativeScheduleInitialValueDataRule implements Validator<NonNegativeSchedule> {
	
	private static final String NAME = "NonNegativeSchedule_initialValue";
	private static final String DEFINITION = "when NonNegativeSchedule exists\n" + 
		"then NonNegativeSchedule -> initialValue >= 0.0";
	
	@Override
	public ValidationResult<NonNegativeSchedule> validate(RosettaPath path, NonNegativeSchedule nonNegativeSchedule) {
		ComparisonResult result = executeDataRule(nonNegativeSchedule);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeSchedule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeSchedule", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<NonNegativeSchedule> validate(RosettaPath path, RosettaModelObjectBuilder nonNegativeSchedule) {
		ComparisonResult result = executeDataRule((NonNegativeSchedule)nonNegativeSchedule.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeSchedule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeSchedule", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(NonNegativeSchedule nonNegativeSchedule) {
		if (ruleIsApplicable(nonNegativeSchedule).get()) {
			return evaluateThenExpression(nonNegativeSchedule);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(NonNegativeSchedule nonNegativeSchedule) {
		return exists(MapperS.of(nonNegativeSchedule), false);
	}
	
	private ComparisonResult evaluateThenExpression(NonNegativeSchedule nonNegativeSchedule) {
		return greaterThanEquals(MapperS.of(nonNegativeSchedule).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue), MapperS.of(new BigDecimal(0.0)));
	}
}
