package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.NonNegativeStep;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("NonNegativeStep_stepValue")
public class NonNegativeStepStepValueDataRule implements Validator<NonNegativeStep> {
	
	private static final String NAME = "NonNegativeStep_stepValue";
	private static final String DEFINITION = "when NonNegativeStep exists\n" + 
		"then NonNegativeStep -> stepValue >= 0.0";
	
	@Override
	public ValidationResult<NonNegativeStep> validate(RosettaPath path, NonNegativeStep nonNegativeStep) {
		ComparisonResult result = executeDataRule(nonNegativeStep);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeStep", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeStep", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<NonNegativeStep> validate(RosettaPath path, RosettaModelObjectBuilder nonNegativeStep) {
		ComparisonResult result = executeDataRule((NonNegativeStep)nonNegativeStep.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "NonNegativeStep", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "NonNegativeStep", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(NonNegativeStep nonNegativeStep) {
		if (ruleIsApplicable(nonNegativeStep).get()) {
			return evaluateThenExpression(nonNegativeStep);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(NonNegativeStep nonNegativeStep) {
		return exists(MapperS.of(nonNegativeStep), false);
	}
	
	private ComparisonResult evaluateThenExpression(NonNegativeStep nonNegativeStep) {
		return greaterThanEquals(MapperS.of(nonNegativeStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue), MapperS.of(new BigDecimal(0.0)));
	}
}
