package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.StepRelativeToEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("NotionalStepRule_notionalStepRate")
public class NotionalStepRuleNotionalStepRateDataRule implements Validator<NotionalStepRule> {
	
	private static final String NAME = "NotionalStepRule_notionalStepRate";
	private static final String DEFINITION = "when NotionalStepRule -> notionalStepAmount is absent\n" + 
		"then NotionalStepRule -> notionalStepRate exists\n" + 
		"		and NotionalStepRule -> stepRelativeTo exists";
	
	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, NotionalStepRule notionalStepRule) {
		ComparisonResult result = executeDataRule(notionalStepRule);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "NotionalStepRule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "NotionalStepRule", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, RosettaModelObjectBuilder notionalStepRule) {
		ComparisonResult result = executeDataRule((NotionalStepRule)notionalStepRule.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "NotionalStepRule", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "NotionalStepRule", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(NotionalStepRule notionalStepRule) {
		if (ruleIsApplicable(notionalStepRule).get()) {
			return evaluateThenExpression(notionalStepRule);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(NotionalStepRule notionalStepRule) {
		return notExists(MapperS.of(notionalStepRule).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount));
	}
	
	private ComparisonResult evaluateThenExpression(NotionalStepRule notionalStepRule) {
		return exists(MapperS.of(notionalStepRule).<BigDecimal>map("getNotionalStepRate", NotionalStepRule::getNotionalStepRate), false).and(exists(MapperS.of(notionalStepRule).<StepRelativeToEnum>map("getStepRelativeTo", NotionalStepRule::getStepRelativeTo), false));
	}
}
