package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Curve;
import org.isda.cdm.InformationSource;
import org.isda.cdm.ObservationSource;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ObservationSource_content")
public class ObservationSourceContentDataRule implements Validator<ObservationSource> {
	
	private static final String NAME = "ObservationSource_content";
	private static final String DEFINITION = "when ObservationSource exists\n" + 
		"then ( ObservationSource -> curve and ObservationSource -> informationSource ) exists\n" + 
		"		or ObservationSource -> curve exists\n" + 
		"		or ObservationSource -> informationSource exists";
	
	@Override
	public ValidationResult<ObservationSource> validate(RosettaPath path, ObservationSource observationSource) {
		ComparisonResult result = executeDataRule(observationSource);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ObservationSource", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ObservationSource", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ObservationSource> validate(RosettaPath path, RosettaModelObjectBuilder observationSource) {
		ComparisonResult result = executeDataRule((ObservationSource)observationSource.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ObservationSource", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ObservationSource", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ObservationSource observationSource) {
		if (ruleIsApplicable(observationSource).get()) {
			return evaluateThenExpression(observationSource);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ObservationSource observationSource) {
		return exists(MapperS.of(observationSource), false);
	}
	
	private ComparisonResult evaluateThenExpression(ObservationSource observationSource) {
		return exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(observationSource).<Curve>map("getCurve", ObservationSource::getCurve)), MapperTree.of(MapperS.of(observationSource).<InformationSource>map("getInformationSource", ObservationSource::getInformationSource))), false).or(exists(MapperS.of(observationSource).<Curve>map("getCurve", ObservationSource::getCurve), false)).or(exists(MapperS.of(observationSource).<InformationSource>map("getInformationSource", ObservationSource::getInformationSource), false));
	}
}
