package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.DayTypeEnum;
import org.isda.cdm.Offset;
import org.isda.cdm.Period;
import org.isda.cdm.PeriodEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Offset_DayType")
public class OffsetDayTypeDataRule implements Validator<Offset> {
	
	private static final String NAME = "Offset_DayType";
	private static final String DEFINITION = "when Offset -> period <> PeriodEnum.D \n" + 
		"		or Offset -> periodMultiplier = 0\n" + 
		"then Offset -> dayType is absent";
	
	@Override
	public ValidationResult<Offset> validate(RosettaPath path, Offset offset) {
		ComparisonResult result = executeDataRule(offset);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Offset", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Offset", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Offset> validate(RosettaPath path, RosettaModelObjectBuilder offset) {
		ComparisonResult result = executeDataRule((Offset)offset.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Offset", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Offset", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Offset offset) {
		if (ruleIsApplicable(offset).get()) {
			return evaluateThenExpression(offset);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Offset offset) {
		return notEqual(MapperS.of(offset).<PeriodEnum>map("getPeriod", Period::getPeriod), MapperS.of(PeriodEnum.D)).or(areEqual(MapperS.of(offset).<Integer>map("getPeriodMultiplier", Period::getPeriodMultiplier), MapperS.of(Integer.valueOf(0))));
	}
	
	private ComparisonResult evaluateThenExpression(Offset offset) {
		return notExists(MapperS.of(offset).<DayTypeEnum>map("getDayType", Offset::getDayType));
	}
}
