package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.OneWayProvisions;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("OneWayProvisions_postingPartyReference_absent")
public class OneWayProvisionsPostingPartyReferenceAbsentDataRule implements Validator<OneWayProvisions> {
	
	private static final String NAME = "OneWayProvisions_postingPartyReference_absent";
	private static final String DEFINITION = "when OneWayProvisions -> isApplicable = False\n" + 
		"then OneWayProvisions -> postingParty is absent";
	
	@Override
	public ValidationResult<OneWayProvisions> validate(RosettaPath path, OneWayProvisions oneWayProvisions) {
		ComparisonResult result = executeDataRule(oneWayProvisions);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "OneWayProvisions", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "OneWayProvisions", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<OneWayProvisions> validate(RosettaPath path, RosettaModelObjectBuilder oneWayProvisions) {
		ComparisonResult result = executeDataRule((OneWayProvisions)oneWayProvisions.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "OneWayProvisions", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "OneWayProvisions", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(OneWayProvisions oneWayProvisions) {
		if (ruleIsApplicable(oneWayProvisions).get()) {
			return evaluateThenExpression(oneWayProvisions);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(OneWayProvisions oneWayProvisions) {
		return areEqual(MapperS.of(oneWayProvisions).<Boolean>map("getIsApplicable", OneWayProvisions::getIsApplicable), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(OneWayProvisions oneWayProvisions) {
		return notExists(MapperS.of(oneWayProvisions).<String>map("getPostingParty", OneWayProvisions::getPostingParty));
	}
}
