package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.OptionDenomination;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("OptionDenomination_optionEntitlement")
public class OptionDenominationOptionEntitlementDataRule implements Validator<OptionDenomination> {
	
	private static final String NAME = "OptionDenomination_optionEntitlement";
	private static final String DEFINITION = "when OptionDenomination -> optionEntitlement exists\n" + 
		"then OptionDenomination -> optionEntitlement >= 0.0";
	
	@Override
	public ValidationResult<OptionDenomination> validate(RosettaPath path, OptionDenomination optionDenomination) {
		ComparisonResult result = executeDataRule(optionDenomination);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "OptionDenomination", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "OptionDenomination", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<OptionDenomination> validate(RosettaPath path, RosettaModelObjectBuilder optionDenomination) {
		ComparisonResult result = executeDataRule((OptionDenomination)optionDenomination.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "OptionDenomination", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "OptionDenomination", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(OptionDenomination optionDenomination) {
		if (ruleIsApplicable(optionDenomination).get()) {
			return evaluateThenExpression(optionDenomination);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(OptionDenomination optionDenomination) {
		return exists(MapperS.of(optionDenomination).<BigDecimal>map("getOptionEntitlement", OptionDenomination::getOptionEntitlement), false);
	}
	
	private ComparisonResult evaluateThenExpression(OptionDenomination optionDenomination) {
		return greaterThanEquals(MapperS.of(optionDenomination).<BigDecimal>map("getOptionEntitlement", OptionDenomination::getOptionEntitlement), MapperS.of(new BigDecimal(0.0)));
	}
}
