package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.OptionCashSettlement;
import org.isda.cdm.OptionSettlement;
import org.isda.cdm.SettlementTerms;
import org.isda.cdm.SettlementTypeEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("OptionSettlement_cashSettlementTerms")
public class OptionSettlementCashSettlementTermsDataRule implements Validator<OptionSettlement> {
	
	private static final String NAME = "OptionSettlement_cashSettlementTerms";
	private static final String DEFINITION = "when OptionSettlement -> cashSettlementTerms exists\n" + 
		"then OptionSettlement -> settlementType <> SettlementTypeEnum.Physical";
	
	@Override
	public ValidationResult<OptionSettlement> validate(RosettaPath path, OptionSettlement optionSettlement) {
		ComparisonResult result = executeDataRule(optionSettlement);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "OptionSettlement", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "OptionSettlement", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<OptionSettlement> validate(RosettaPath path, RosettaModelObjectBuilder optionSettlement) {
		ComparisonResult result = executeDataRule((OptionSettlement)optionSettlement.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "OptionSettlement", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "OptionSettlement", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(OptionSettlement optionSettlement) {
		if (ruleIsApplicable(optionSettlement).get()) {
			return evaluateThenExpression(optionSettlement);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(OptionSettlement optionSettlement) {
		return exists(MapperS.of(optionSettlement).<OptionCashSettlement>map("getCashSettlementTerms", OptionSettlement::getCashSettlementTerms), false);
	}
	
	private ComparisonResult evaluateThenExpression(OptionSettlement optionSettlement) {
		return notEqual(MapperS.of(optionSettlement).<SettlementTypeEnum>map("getSettlementType", SettlementTerms::getSettlementType), MapperS.of(SettlementTypeEnum.PHYSICAL));
	}
}
