package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.BusinessDayAdjustments;
import org.isda.cdm.Offset;
import org.isda.cdm.PaymentDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PaymentDates_dateRelativeTo")
public class PaymentDatesDateRelativeToDataRule implements Validator<PaymentDates> {
	
	private static final String NAME = "PaymentDates_dateRelativeTo";
	private static final String DEFINITION = "when PaymentDates -> dateRelativeTo exists\n" + 
		"then ( PaymentDates -> paymentDaysOffset\n" + 
		"		and PaymentDates -> paymentDatesAdjustments ) exists";
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, PaymentDates paymentDates) {
		ComparisonResult result = executeDataRule(paymentDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, RosettaModelObjectBuilder paymentDates) {
		ComparisonResult result = executeDataRule((PaymentDates)paymentDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PaymentDates paymentDates) {
		if (ruleIsApplicable(paymentDates).get()) {
			return evaluateThenExpression(paymentDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PaymentDates paymentDates) {
		return exists(MapperS.of(paymentDates).map("getDateRelativeTo", PaymentDates::getDateRelativeTo).<Date>map("getValue", FieldWithMeta::getValue), false);
	}
	
	private ComparisonResult evaluateThenExpression(PaymentDates paymentDates) {
		return exists(MapperTree.andDifferent(MapperTree.of(MapperS.of(paymentDates).<Offset>map("getPaymentDaysOffset", PaymentDates::getPaymentDaysOffset)), MapperTree.of(MapperS.of(paymentDates).<BusinessDayAdjustments>map("getPaymentDatesAdjustments", PaymentDates::getPaymentDatesAdjustments))), false);
	}
}
