package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Offset;
import org.isda.cdm.PaymentDates;
import org.isda.cdm.Period;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PaymentDates_paymentDaysOffset")
public class PaymentDatesPaymentDaysOffsetDataRule implements Validator<PaymentDates> {
	
	private static final String NAME = "PaymentDates_paymentDaysOffset";
	private static final String DEFINITION = "when PaymentDates -> paymentDaysOffset exists\n" + 
		"then PaymentDates -> paymentDaysOffset -> periodMultiplier <> 0";
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, PaymentDates paymentDates) {
		ComparisonResult result = executeDataRule(paymentDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, RosettaModelObjectBuilder paymentDates) {
		ComparisonResult result = executeDataRule((PaymentDates)paymentDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PaymentDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PaymentDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PaymentDates paymentDates) {
		if (ruleIsApplicable(paymentDates).get()) {
			return evaluateThenExpression(paymentDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PaymentDates paymentDates) {
		return exists(MapperS.of(paymentDates).<Offset>map("getPaymentDaysOffset", PaymentDates::getPaymentDaysOffset), false);
	}
	
	private ComparisonResult evaluateThenExpression(PaymentDates paymentDates) {
		return notEqual(MapperS.of(paymentDates).<Offset>map("getPaymentDaysOffset", PaymentDates::getPaymentDaysOffset).<Integer>map("getPeriodMultiplier", Period::getPeriodMultiplier), MapperS.of(Integer.valueOf(0)));
	}
}
