package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Period;
import org.isda.cdm.PeriodEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Period_period")
public class PeriodPeriodDataRule implements Validator<Period> {
	
	private static final String NAME = "Period_period";
	private static final String DEFINITION = "when Period -> periodMultiplier = 0\n" + 
		"then Period -> period = PeriodEnum.D";
	
	@Override
	public ValidationResult<Period> validate(RosettaPath path, Period period) {
		ComparisonResult result = executeDataRule(period);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Period", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Period", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Period> validate(RosettaPath path, RosettaModelObjectBuilder period) {
		ComparisonResult result = executeDataRule((Period)period.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Period", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Period", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Period period) {
		if (ruleIsApplicable(period).get()) {
			return evaluateThenExpression(period);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Period period) {
		return areEqual(MapperS.of(period).<Integer>map("getPeriodMultiplier", Period::getPeriodMultiplier), MapperS.of(Integer.valueOf(0)));
	}
	
	private ComparisonResult evaluateThenExpression(Period period) {
		return areEqual(MapperS.of(period).<PeriodEnum>map("getPeriod", Period::getPeriod), MapperS.of(PeriodEnum.D));
	}
}
