package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.PhysicalExercise;
import org.isda.cdm.Product;
import org.isda.cdm.Quantity;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PhysicalExercise_quantity")
public class PhysicalExerciseQuantityDataRule implements Validator<PhysicalExercise> {
	
	private static final String NAME = "PhysicalExercise_quantity";
	private static final String DEFINITION = "when PhysicalExercise -> product -> contractualProduct exists\n" + 
		"then PhysicalExercise -> quantity is absent";
	
	@Override
	public ValidationResult<PhysicalExercise> validate(RosettaPath path, PhysicalExercise physicalExercise) {
		ComparisonResult result = executeDataRule(physicalExercise);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PhysicalExercise", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PhysicalExercise", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PhysicalExercise> validate(RosettaPath path, RosettaModelObjectBuilder physicalExercise) {
		ComparisonResult result = executeDataRule((PhysicalExercise)physicalExercise.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PhysicalExercise", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PhysicalExercise", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PhysicalExercise physicalExercise) {
		if (ruleIsApplicable(physicalExercise).get()) {
			return evaluateThenExpression(physicalExercise);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PhysicalExercise physicalExercise) {
		return exists(MapperS.of(physicalExercise).<Product>map("getProduct", PhysicalExercise::getProduct).<ContractualProduct>map("getContractualProduct", Product::getContractualProduct), false);
	}
	
	private ComparisonResult evaluateThenExpression(PhysicalExercise physicalExercise) {
		return notExists(MapperS.of(physicalExercise).<Quantity>map("getQuantity", PhysicalExercise::getQuantity));
	}
}
