package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.PhysicalSettlementPeriod;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PhysicalSettlementPeriod_businessDays")
public class PhysicalSettlementPeriodBusinessDaysDataRule implements Validator<PhysicalSettlementPeriod> {
	
	private static final String NAME = "PhysicalSettlementPeriod_businessDays";
	private static final String DEFINITION = "when PhysicalSettlementPeriod -> businessDays exists\n" + 
		"then PhysicalSettlementPeriod -> businessDays >= 0";
	
	@Override
	public ValidationResult<PhysicalSettlementPeriod> validate(RosettaPath path, PhysicalSettlementPeriod physicalSettlementPeriod) {
		ComparisonResult result = executeDataRule(physicalSettlementPeriod);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PhysicalSettlementPeriod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PhysicalSettlementPeriod", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PhysicalSettlementPeriod> validate(RosettaPath path, RosettaModelObjectBuilder physicalSettlementPeriod) {
		ComparisonResult result = executeDataRule((PhysicalSettlementPeriod)physicalSettlementPeriod.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PhysicalSettlementPeriod", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PhysicalSettlementPeriod", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PhysicalSettlementPeriod physicalSettlementPeriod) {
		if (ruleIsApplicable(physicalSettlementPeriod).get()) {
			return evaluateThenExpression(physicalSettlementPeriod);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PhysicalSettlementPeriod physicalSettlementPeriod) {
		return exists(MapperS.of(physicalSettlementPeriod).<Integer>map("getBusinessDays", PhysicalSettlementPeriod::getBusinessDays), false);
	}
	
	private ComparisonResult evaluateThenExpression(PhysicalSettlementPeriod physicalSettlementPeriod) {
		return greaterThanEquals(MapperS.of(physicalSettlementPeriod).<Integer>map("getBusinessDays", PhysicalSettlementPeriod::getBusinessDays), MapperS.of(Integer.valueOf(0)));
	}
}
