package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Event;
import org.isda.cdm.Lineage;
import org.isda.cdm.PortfolioState;
import org.isda.cdm.Position;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PortfolioState_initialisation")
public class PortfolioStateInitialisationDataRule implements Validator<PortfolioState> {
	
	private static final String NAME = "PortfolioState_initialisation";
	private static final String DEFINITION = "when PortfolioState exists and PortfolioState -> lineage -> portfolioStateReference is absent\n" + 
		"then PortfolioState -> positions is absent\n" + 
		"	and PortfolioState -> lineage -> eventReference is absent";
	
	@Override
	public ValidationResult<PortfolioState> validate(RosettaPath path, PortfolioState portfolioState) {
		ComparisonResult result = executeDataRule(portfolioState);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PortfolioState", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PortfolioState", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PortfolioState> validate(RosettaPath path, RosettaModelObjectBuilder portfolioState) {
		ComparisonResult result = executeDataRule((PortfolioState)portfolioState.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PortfolioState", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PortfolioState", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PortfolioState portfolioState) {
		if (ruleIsApplicable(portfolioState).get()) {
			return evaluateThenExpression(portfolioState);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PortfolioState portfolioState) {
		return exists(MapperS.of(portfolioState), false).and(notExists(MapperS.of(portfolioState).<Lineage>map("getLineage", PortfolioState::getLineage).mapC("getPortfolioStateReference", Lineage::getPortfolioStateReference).<PortfolioState>map("getValue", FieldWithMeta::getValue)));
	}
	
	private ComparisonResult evaluateThenExpression(PortfolioState portfolioState) {
		return notExists(MapperS.of(portfolioState).<Position>mapC("getPositions", PortfolioState::getPositions)).and(notExists(MapperS.of(portfolioState).<Lineage>map("getLineage", PortfolioState::getLineage).mapC("getEventReference", Lineage::getEventReference).<Event>map("getValue", FieldWithMeta::getValue)));
	}
}
