package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.Position;
import org.isda.cdm.PositionStatusEnum;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Position_ContractFormed")
public class PositionContractFormedDataRule implements Validator<Position> {
	
	private static final String NAME = "Position_ContractFormed";
	private static final String DEFINITION = "when Position -> positionStatus = PositionStatusEnum.Formed\n" + 
		"then Position -> contractReference exists";
	
	@Override
	public ValidationResult<Position> validate(RosettaPath path, Position position) {
		ComparisonResult result = executeDataRule(position);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Position", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Position", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Position> validate(RosettaPath path, RosettaModelObjectBuilder position) {
		ComparisonResult result = executeDataRule((Position)position.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Position", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Position", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Position position) {
		if (ruleIsApplicable(position).get()) {
			return evaluateThenExpression(position);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Position position) {
		return areEqual(MapperS.of(position).<PositionStatusEnum>map("getPositionStatus", Position::getPositionStatus), MapperS.of(PositionStatusEnum.FORMED));
	}
	
	private ComparisonResult evaluateThenExpression(Position position) {
		return exists(MapperS.of(position).map("getContractReference", Position::getContractReference).<Contract>map("getValue", FieldWithMeta::getValue), false);
	}
}
