package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.EligibleCollateral;
import org.isda.cdm.PostingObligationsElection;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PostingObligationsElection_eligibleCollateral")
public class PostingObligationsElectionEligibleCollateralDataRule implements Validator<PostingObligationsElection> {
	
	private static final String NAME = "PostingObligationsElection_eligibleCollateral";
	private static final String DEFINITION = "when PostingObligationsElection -> asPermitted = False\n" + 
		"then PostingObligationsElection -> eligibleCollateral exists";
	
	@Override
	public ValidationResult<PostingObligationsElection> validate(RosettaPath path, PostingObligationsElection postingObligationsElection) {
		ComparisonResult result = executeDataRule(postingObligationsElection);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PostingObligationsElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PostingObligationsElection", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PostingObligationsElection> validate(RosettaPath path, RosettaModelObjectBuilder postingObligationsElection) {
		ComparisonResult result = executeDataRule((PostingObligationsElection)postingObligationsElection.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PostingObligationsElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PostingObligationsElection", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PostingObligationsElection postingObligationsElection) {
		if (ruleIsApplicable(postingObligationsElection).get()) {
			return evaluateThenExpression(postingObligationsElection);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PostingObligationsElection postingObligationsElection) {
		return areEqual(MapperS.of(postingObligationsElection).<Boolean>map("getAsPermitted", PostingObligationsElection::getAsPermitted), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(PostingObligationsElection postingObligationsElection) {
		return exists(MapperS.of(postingObligationsElection).<EligibleCollateral>mapC("getEligibleCollateral", PostingObligationsElection::getEligibleCollateral), false);
	}
}
