package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ActualPrice;
import org.isda.cdm.Price;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Price_cleanNetPrice_netPrice")
public class PriceCleanNetPriceNetPriceDataRule implements Validator<Price> {
	
	private static final String NAME = "Price_cleanNetPrice_netPrice";
	private static final String DEFINITION = "when Price exists\n" + 
		"then Price -> netPrice exists or Price -> cleanNetPrice exists";
	
	@Override
	public ValidationResult<Price> validate(RosettaPath path, Price price) {
		ComparisonResult result = executeDataRule(price);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Price", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Price", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Price> validate(RosettaPath path, RosettaModelObjectBuilder price) {
		ComparisonResult result = executeDataRule((Price)price.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Price", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Price", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Price price) {
		if (ruleIsApplicable(price).get()) {
			return evaluateThenExpression(price);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Price price) {
		return exists(MapperS.of(price), false);
	}
	
	private ComparisonResult evaluateThenExpression(Price price) {
		return exists(MapperS.of(price).<ActualPrice>map("getNetPrice", Price::getNetPrice), false).or(exists(MapperS.of(price).<ActualPrice>map("getCleanNetPrice", Price::getCleanNetPrice), false));
	}
}
