package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.ProcessAgentElection;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ProcessAgentElection_applicable")
public class ProcessAgentElectionApplicableDataRule implements Validator<ProcessAgentElection> {
	
	private static final String NAME = "ProcessAgentElection_applicable";
	private static final String DEFINITION = "when ProcessAgentElection -> isApplicable = True\n" + 
		"then ProcessAgentElection -> specification exists";
	
	@Override
	public ValidationResult<ProcessAgentElection> validate(RosettaPath path, ProcessAgentElection processAgentElection) {
		ComparisonResult result = executeDataRule(processAgentElection);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ProcessAgentElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ProcessAgentElection", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ProcessAgentElection> validate(RosettaPath path, RosettaModelObjectBuilder processAgentElection) {
		ComparisonResult result = executeDataRule((ProcessAgentElection)processAgentElection.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ProcessAgentElection", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ProcessAgentElection", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ProcessAgentElection processAgentElection) {
		if (ruleIsApplicable(processAgentElection).get()) {
			return evaluateThenExpression(processAgentElection);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ProcessAgentElection processAgentElection) {
		return areEqual(MapperS.of(processAgentElection).<Boolean>map("getIsApplicable", ProcessAgentElection::getIsApplicable), MapperS.of(Boolean.valueOf(true)));
	}
	
	private ComparisonResult evaluateThenExpression(ProcessAgentElection processAgentElection) {
		return exists(MapperS.of(processAgentElection).<String>map("getSpecification", ProcessAgentElection::getSpecification), false);
	}
}
