package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.PubliclyAvailableInformation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("PubliclyAvailableInformation_specifiedNumber")
public class PubliclyAvailableInformationSpecifiedNumberDataRule implements Validator<PubliclyAvailableInformation> {
	
	private static final String NAME = "PubliclyAvailableInformation_specifiedNumber";
	private static final String DEFINITION = "when PubliclyAvailableInformation -> specifiedNumber exists\n" + 
		"then PubliclyAvailableInformation -> specifiedNumber >= 0";
	
	@Override
	public ValidationResult<PubliclyAvailableInformation> validate(RosettaPath path, PubliclyAvailableInformation publiclyAvailableInformation) {
		ComparisonResult result = executeDataRule(publiclyAvailableInformation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "PubliclyAvailableInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "PubliclyAvailableInformation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<PubliclyAvailableInformation> validate(RosettaPath path, RosettaModelObjectBuilder publiclyAvailableInformation) {
		ComparisonResult result = executeDataRule((PubliclyAvailableInformation)publiclyAvailableInformation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PubliclyAvailableInformation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "PubliclyAvailableInformation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(PubliclyAvailableInformation publiclyAvailableInformation) {
		if (ruleIsApplicable(publiclyAvailableInformation).get()) {
			return evaluateThenExpression(publiclyAvailableInformation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(PubliclyAvailableInformation publiclyAvailableInformation) {
		return exists(MapperS.of(publiclyAvailableInformation).<Integer>map("getSpecifiedNumber", PubliclyAvailableInformation::getSpecifiedNumber), false);
	}
	
	private ComparisonResult evaluateThenExpression(PubliclyAvailableInformation publiclyAvailableInformation) {
		return greaterThanEquals(MapperS.of(publiclyAvailableInformation).<Integer>map("getSpecifiedNumber", PubliclyAvailableInformation::getSpecifiedNumber), MapperS.of(Integer.valueOf(0)));
	}
}
