package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.RateObservation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("RateObservation_observationWeight")
public class RateObservationObservationWeightDataRule implements Validator<RateObservation> {
	
	private static final String NAME = "RateObservation_observationWeight";
	private static final String DEFINITION = "when RateObservation -> observationWeight exists\n" + 
		"then RateObservation -> observationWeight >= 0";
	
	@Override
	public ValidationResult<RateObservation> validate(RosettaPath path, RateObservation rateObservation) {
		ComparisonResult result = executeDataRule(rateObservation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "RateObservation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "RateObservation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<RateObservation> validate(RosettaPath path, RosettaModelObjectBuilder rateObservation) {
		ComparisonResult result = executeDataRule((RateObservation)rateObservation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RateObservation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "RateObservation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(RateObservation rateObservation) {
		if (ruleIsApplicable(rateObservation).get()) {
			return evaluateThenExpression(rateObservation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(RateObservation rateObservation) {
		return exists(MapperS.of(rateObservation).<Integer>map("getObservationWeight", RateObservation::getObservationWeight), false);
	}
	
	private ComparisonResult evaluateThenExpression(RateObservation rateObservation) {
		return greaterThanEquals(MapperS.of(rateObservation).<Integer>map("getObservationWeight", RateObservation::getObservationWeight), MapperS.of(Integer.valueOf(0)));
	}
}
