package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.RelatedAgreement;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("RelatedAgreement_notSpecified")
public class RelatedAgreementNotSpecifiedDataRule implements Validator<RelatedAgreement> {
	
	private static final String NAME = "RelatedAgreement_notSpecified";
	private static final String DEFINITION = "when RelatedAgreement -> isSpecified = False\n" + 
		"then RelatedAgreement -> legalDocument is absent";
	
	@Override
	public ValidationResult<RelatedAgreement> validate(RosettaPath path, RelatedAgreement relatedAgreement) {
		ComparisonResult result = executeDataRule(relatedAgreement);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "RelatedAgreement", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "RelatedAgreement", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<RelatedAgreement> validate(RosettaPath path, RosettaModelObjectBuilder relatedAgreement) {
		ComparisonResult result = executeDataRule((RelatedAgreement)relatedAgreement.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RelatedAgreement", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "RelatedAgreement", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(RelatedAgreement relatedAgreement) {
		if (ruleIsApplicable(relatedAgreement).get()) {
			return evaluateThenExpression(relatedAgreement);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(RelatedAgreement relatedAgreement) {
		return areEqual(MapperS.of(relatedAgreement).<Boolean>map("getIsSpecified", RelatedAgreement::getIsSpecified), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(RelatedAgreement relatedAgreement) {
		return notExists(MapperS.of(relatedAgreement).<String>map("getLegalDocument", RelatedAgreement::getLegalDocument));
	}
}
