package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.RelativeDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("RelativeDates_periodSkip")
public class RelativeDatesPeriodSkipDataRule implements Validator<RelativeDates> {
	
	private static final String NAME = "RelativeDates_periodSkip";
	private static final String DEFINITION = "when RelativeDates -> periodSkip exists\n" + 
		"then RelativeDates -> periodSkip > 1";
	
	@Override
	public ValidationResult<RelativeDates> validate(RosettaPath path, RelativeDates relativeDates) {
		ComparisonResult result = executeDataRule(relativeDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "RelativeDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "RelativeDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<RelativeDates> validate(RosettaPath path, RosettaModelObjectBuilder relativeDates) {
		ComparisonResult result = executeDataRule((RelativeDates)relativeDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RelativeDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "RelativeDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(RelativeDates relativeDates) {
		if (ruleIsApplicable(relativeDates).get()) {
			return evaluateThenExpression(relativeDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(RelativeDates relativeDates) {
		return exists(MapperS.of(relativeDates).<Integer>map("getPeriodSkip", RelativeDates::getPeriodSkip), false);
	}
	
	private ComparisonResult evaluateThenExpression(RelativeDates relativeDates) {
		return greaterThan(MapperS.of(relativeDates).<Integer>map("getPeriodSkip", RelativeDates::getPeriodSkip), MapperS.of(Integer.valueOf(1)));
	}
}
