package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Offset;
import org.isda.cdm.Period;
import org.isda.cdm.ResetDates;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ResetDates_rateCutOffDaysOffset")
public class ResetDatesRateCutOffDaysOffsetDataRule implements Validator<ResetDates> {
	
	private static final String NAME = "ResetDates_rateCutOffDaysOffset";
	private static final String DEFINITION = "when ResetDates -> rateCutOffDaysOffset exists\n" + 
		"then ResetDates -> rateCutOffDaysOffset -> periodMultiplier < 0";
	
	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, ResetDates resetDates) {
		ComparisonResult result = executeDataRule(resetDates);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetDates", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, RosettaModelObjectBuilder resetDates) {
		ComparisonResult result = executeDataRule((ResetDates)resetDates.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetDates", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetDates", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ResetDates resetDates) {
		if (ruleIsApplicable(resetDates).get()) {
			return evaluateThenExpression(resetDates);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ResetDates resetDates) {
		return exists(MapperS.of(resetDates).<Offset>map("getRateCutOffDaysOffset", ResetDates::getRateCutOffDaysOffset), false);
	}
	
	private ComparisonResult evaluateThenExpression(ResetDates resetDates) {
		return lessThan(MapperS.of(resetDates).<Offset>map("getRateCutOffDaysOffset", ResetDates::getRateCutOffDaysOffset).<Integer>map("getPeriodMultiplier", Period::getPeriodMultiplier), MapperS.of(Integer.valueOf(0)));
	}
}
