package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractState;
import org.isda.cdm.ResetPrimitive;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ResetPrimitive_contract")
public class ResetPrimitiveContractDataRule implements Validator<ResetPrimitive> {
	
	private static final String NAME = "ResetPrimitive_contract";
	private static final String DEFINITION = "when ResetPrimitive exists\n" + 
		"then ResetPrimitive -> before -> contract = ResetPrimitive -> after -> contract";
	
	@Override
	public ValidationResult<ResetPrimitive> validate(RosettaPath path, ResetPrimitive resetPrimitive) {
		ComparisonResult result = executeDataRule(resetPrimitive);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetPrimitive", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetPrimitive", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ResetPrimitive> validate(RosettaPath path, RosettaModelObjectBuilder resetPrimitive) {
		ComparisonResult result = executeDataRule((ResetPrimitive)resetPrimitive.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ResetPrimitive", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResetPrimitive", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ResetPrimitive resetPrimitive) {
		if (ruleIsApplicable(resetPrimitive).get()) {
			return evaluateThenExpression(resetPrimitive);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ResetPrimitive resetPrimitive) {
		return exists(MapperS.of(resetPrimitive), false);
	}
	
	private ComparisonResult evaluateThenExpression(ResetPrimitive resetPrimitive) {
		return areEqual(MapperS.of(resetPrimitive).<ContractState>map("getBefore", ResetPrimitive::getBefore).<Contract>map("getContract", ContractState::getContract), MapperS.of(resetPrimitive).<ContractState>map("getAfter", ResetPrimitive::getAfter).<Contract>map("getContract", ContractState::getContract));
	}
}
