package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.QuantityMultiplier;
import org.isda.cdm.ResolvablePayoutQuantity;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ResolvablePayoutQuantity_quantityMultiplier")
public class ResolvablePayoutQuantityQuantityMultiplierDataRule implements Validator<ResolvablePayoutQuantity> {
	
	private static final String NAME = "ResolvablePayoutQuantity_quantityMultiplier";
	private static final String DEFINITION = "when ResolvablePayoutQuantity -> quantityMultiplier exists\n" + 
		"then ResolvablePayoutQuantity -> quantityReference exists";
	
	@Override
	public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, ResolvablePayoutQuantity resolvablePayoutQuantity) {
		ComparisonResult result = executeDataRule(resolvablePayoutQuantity);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResolvablePayoutQuantity", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "ResolvablePayoutQuantity", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, RosettaModelObjectBuilder resolvablePayoutQuantity) {
		ComparisonResult result = executeDataRule((ResolvablePayoutQuantity)resolvablePayoutQuantity.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ResolvablePayoutQuantity", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "ResolvablePayoutQuantity", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(ResolvablePayoutQuantity resolvablePayoutQuantity) {
		if (ruleIsApplicable(resolvablePayoutQuantity).get()) {
			return evaluateThenExpression(resolvablePayoutQuantity);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(ResolvablePayoutQuantity resolvablePayoutQuantity) {
		return exists(MapperS.of(resolvablePayoutQuantity).<QuantityMultiplier>map("getQuantityMultiplier", ResolvablePayoutQuantity::getQuantityMultiplier), false);
	}
	
	private ComparisonResult evaluateThenExpression(ResolvablePayoutQuantity resolvablePayoutQuantity) {
		return exists(MapperS.of(resolvablePayoutQuantity).map("getQuantityReference", ResolvablePayoutQuantity::getQuantityReference).<ResolvablePayoutQuantity>map("getValue", FieldWithMeta::getValue), false);
	}
}
