package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.RightsEvent;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("RightsEvent_customElection")
public class RightsEventCustomElectionDataRule implements Validator<RightsEvent> {
	
	private static final String NAME = "RightsEvent_customElection";
	private static final String DEFINITION = "when RightsEvent -> customElection exists\n" + 
		"then RightsEvent -> includeCoolingOffLanguage = False";
	
	@Override
	public ValidationResult<RightsEvent> validate(RosettaPath path, RightsEvent rightsEvent) {
		ComparisonResult result = executeDataRule(rightsEvent);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "RightsEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "RightsEvent", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<RightsEvent> validate(RosettaPath path, RosettaModelObjectBuilder rightsEvent) {
		ComparisonResult result = executeDataRule((RightsEvent)rightsEvent.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RightsEvent", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "RightsEvent", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(RightsEvent rightsEvent) {
		if (ruleIsApplicable(rightsEvent).get()) {
			return evaluateThenExpression(rightsEvent);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(RightsEvent rightsEvent) {
		return exists(MapperS.of(rightsEvent).<String>map("getCustomElection", RightsEvent::getCustomElection), false);
	}
	
	private ComparisonResult evaluateThenExpression(RightsEvent rightsEvent) {
		return areEqual(MapperS.of(rightsEvent).<Boolean>map("getIncludeCoolingOffLanguage", RightsEvent::getIncludeCoolingOffLanguage), MapperS.of(Boolean.valueOf(false)));
	}
}
