package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.SimmCalculationCurrency;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("SimmCalculationCurrency_notBaseCurrency")
public class SimmCalculationCurrencyNotBaseCurrencyDataRule implements Validator<SimmCalculationCurrency> {
	
	private static final String NAME = "SimmCalculationCurrency_notBaseCurrency";
	private static final String DEFINITION = "when SimmCalculationCurrency -> isBaseCurrency = False\n" + 
		"then SimmCalculationCurrency -> currency exists";
	
	@Override
	public ValidationResult<SimmCalculationCurrency> validate(RosettaPath path, SimmCalculationCurrency simmCalculationCurrency) {
		ComparisonResult result = executeDataRule(simmCalculationCurrency);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimmCalculationCurrency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "SimmCalculationCurrency", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<SimmCalculationCurrency> validate(RosettaPath path, RosettaModelObjectBuilder simmCalculationCurrency) {
		ComparisonResult result = executeDataRule((SimmCalculationCurrency)simmCalculationCurrency.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SimmCalculationCurrency", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimmCalculationCurrency", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(SimmCalculationCurrency simmCalculationCurrency) {
		if (ruleIsApplicable(simmCalculationCurrency).get()) {
			return evaluateThenExpression(simmCalculationCurrency);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(SimmCalculationCurrency simmCalculationCurrency) {
		return areEqual(MapperS.of(simmCalculationCurrency).<Boolean>map("getIsBaseCurrency", SimmCalculationCurrency::getIsBaseCurrency), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(SimmCalculationCurrency simmCalculationCurrency) {
		return exists(MapperS.of(simmCalculationCurrency).map("getCurrency", SimmCalculationCurrency::getCurrency).<String>map("getValue", FieldWithMeta::getValue), false);
	}
}
