package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.SimmVersion;
import org.isda.cdm.SpecifiedSimmVersion;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("SimmVersion_version2")
public class SimmVersionVersion2DataRule implements Validator<SimmVersion> {
	
	private static final String NAME = "SimmVersion_version2";
	private static final String DEFINITION = "when SimmVersion -> isSpecified = True\n" + 
		"then SimmVersion -> version exists";
	
	@Override
	public ValidationResult<SimmVersion> validate(RosettaPath path, SimmVersion simmVersion) {
		ComparisonResult result = executeDataRule(simmVersion);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimmVersion", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "SimmVersion", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<SimmVersion> validate(RosettaPath path, RosettaModelObjectBuilder simmVersion) {
		ComparisonResult result = executeDataRule((SimmVersion)simmVersion.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SimmVersion", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimmVersion", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(SimmVersion simmVersion) {
		if (ruleIsApplicable(simmVersion).get()) {
			return evaluateThenExpression(simmVersion);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(SimmVersion simmVersion) {
		return areEqual(MapperS.of(simmVersion).<Boolean>map("getIsSpecified", SimmVersion::getIsSpecified), MapperS.of(Boolean.valueOf(true)));
	}
	
	private ComparisonResult evaluateThenExpression(SimmVersion simmVersion) {
		return exists(MapperS.of(simmVersion).<SpecifiedSimmVersion>map("getVersion", SimmVersion::getVersion), false);
	}
}
