package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.Money;
import org.isda.cdm.SimplePayment;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("SimplePayment_PaymentAmount")
public class SimplePaymentPaymentAmountDataRule implements Validator<SimplePayment> {
	
	private static final String NAME = "SimplePayment_PaymentAmount";
	private static final String DEFINITION = "when SimplePayment -> paymentAmount exists\n" + 
		"then SimplePayment -> paymentAmount -> amount > 0.0";
	
	@Override
	public ValidationResult<SimplePayment> validate(RosettaPath path, SimplePayment simplePayment) {
		ComparisonResult result = executeDataRule(simplePayment);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimplePayment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "SimplePayment", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<SimplePayment> validate(RosettaPath path, RosettaModelObjectBuilder simplePayment) {
		ComparisonResult result = executeDataRule((SimplePayment)simplePayment.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SimplePayment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "SimplePayment", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(SimplePayment simplePayment) {
		if (ruleIsApplicable(simplePayment).get()) {
			return evaluateThenExpression(simplePayment);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(SimplePayment simplePayment) {
		return exists(MapperS.of(simplePayment).<Money>map("getPaymentAmount", SimplePayment::getPaymentAmount), false);
	}
	
	private ComparisonResult evaluateThenExpression(SimplePayment simplePayment) {
		return greaterThan(MapperS.of(simplePayment).<Money>map("getPaymentAmount", SimplePayment::getPaymentAmount).<BigDecimal>map("getAmount", Money::getAmount), MapperS.of(new BigDecimal(0.0)));
	}
}
