package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.SingleValuationDate;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("SingleValuationDate_businessDays")
public class SingleValuationDateBusinessDaysDataRule implements Validator<SingleValuationDate> {
	
	private static final String NAME = "SingleValuationDate_businessDays";
	private static final String DEFINITION = "when SingleValuationDate -> businessDays exists\n" + 
		"then SingleValuationDate -> businessDays >= 0";
	
	@Override
	public ValidationResult<SingleValuationDate> validate(RosettaPath path, SingleValuationDate singleValuationDate) {
		ComparisonResult result = executeDataRule(singleValuationDate);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "SingleValuationDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "SingleValuationDate", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<SingleValuationDate> validate(RosettaPath path, RosettaModelObjectBuilder singleValuationDate) {
		ComparisonResult result = executeDataRule((SingleValuationDate)singleValuationDate.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SingleValuationDate", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "SingleValuationDate", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(SingleValuationDate singleValuationDate) {
		if (ruleIsApplicable(singleValuationDate).get()) {
			return evaluateThenExpression(singleValuationDate);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(SingleValuationDate singleValuationDate) {
		return exists(MapperS.of(singleValuationDate).<Integer>map("getBusinessDays", SingleValuationDate::getBusinessDays), false);
	}
	
	private ComparisonResult evaluateThenExpression(SingleValuationDate singleValuationDate) {
		return greaterThanEquals(MapperS.of(singleValuationDate).<Integer>map("getBusinessDays", SingleValuationDate::getBusinessDays), MapperS.of(Integer.valueOf(0)));
	}
}
