package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.AmendmentEffectiveDate;
import org.isda.cdm.PartyTerminationCurrency;
import org.isda.cdm.TerminationCurrencyAmendment;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("TerminationCurrencyAmendment_applicable")
public class TerminationCurrencyAmendmentApplicableDataRule implements Validator<TerminationCurrencyAmendment> {
	
	private static final String NAME = "TerminationCurrencyAmendment_applicable";
	private static final String DEFINITION = "when TerminationCurrencyAmendment -> isApplicable = False\n" + 
		"then ( TerminationCurrencyAmendment -> effectiveDate \n" + 
		"	and TerminationCurrencyAmendment -> terminationCurrency\n" + 
		"	and TerminationCurrencyAmendment -> bothPartiesCurrency	) is absent";
	
	@Override
	public ValidationResult<TerminationCurrencyAmendment> validate(RosettaPath path, TerminationCurrencyAmendment terminationCurrencyAmendment) {
		ComparisonResult result = executeDataRule(terminationCurrencyAmendment);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "TerminationCurrencyAmendment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "TerminationCurrencyAmendment", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<TerminationCurrencyAmendment> validate(RosettaPath path, RosettaModelObjectBuilder terminationCurrencyAmendment) {
		ComparisonResult result = executeDataRule((TerminationCurrencyAmendment)terminationCurrencyAmendment.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "TerminationCurrencyAmendment", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "TerminationCurrencyAmendment", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(TerminationCurrencyAmendment terminationCurrencyAmendment) {
		if (ruleIsApplicable(terminationCurrencyAmendment).get()) {
			return evaluateThenExpression(terminationCurrencyAmendment);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(TerminationCurrencyAmendment terminationCurrencyAmendment) {
		return areEqual(MapperS.of(terminationCurrencyAmendment).<Boolean>map("getIsApplicable", TerminationCurrencyAmendment::getIsApplicable), MapperS.of(Boolean.valueOf(false)));
	}
	
	private ComparisonResult evaluateThenExpression(TerminationCurrencyAmendment terminationCurrencyAmendment) {
		return notExists(MapperTree.andDifferent(MapperTree.andDifferent(MapperTree.of(MapperS.of(terminationCurrencyAmendment).<AmendmentEffectiveDate>map("getEffectiveDate", TerminationCurrencyAmendment::getEffectiveDate)), MapperTree.of(MapperS.of(terminationCurrencyAmendment).<PartyTerminationCurrency>map("getTerminationCurrency", TerminationCurrencyAmendment::getTerminationCurrency))), MapperTree.of(MapperS.of(terminationCurrencyAmendment).map("getBothPartiesCurrency", TerminationCurrencyAmendment::getBothPartiesCurrency).<String>map("getValue", FieldWithMeta::getValue))));
	}
}
