package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import java.math.BigDecimal;
import org.isda.cdm.Tranche;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("Tranche_exhaustionPoint")
public class TrancheExhaustionPointDataRule implements Validator<Tranche> {
	
	private static final String NAME = "Tranche_exhaustionPoint";
	private static final String DEFINITION = "when Tranche exists\n" + 
		"then Tranche -> exhaustionPoint >= 0.0 and Tranche -> exhaustionPoint <= 1.0";
	
	@Override
	public ValidationResult<Tranche> validate(RosettaPath path, Tranche tranche) {
		ComparisonResult result = executeDataRule(tranche);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "Tranche", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "Tranche", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<Tranche> validate(RosettaPath path, RosettaModelObjectBuilder tranche) {
		ComparisonResult result = executeDataRule((Tranche)tranche.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "Tranche", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "Tranche", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(Tranche tranche) {
		if (ruleIsApplicable(tranche).get()) {
			return evaluateThenExpression(tranche);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(Tranche tranche) {
		return exists(MapperS.of(tranche), false);
	}
	
	private ComparisonResult evaluateThenExpression(Tranche tranche) {
		return greaterThanEquals(MapperS.of(tranche).<BigDecimal>map("getExhaustionPoint", Tranche::getExhaustionPoint), MapperS.of(new BigDecimal(0.0))).and(lessThanEquals(MapperS.of(tranche).<BigDecimal>map("getExhaustionPoint", Tranche::getExhaustionPoint), MapperS.of(new BigDecimal(1.0))));
	}
}
