package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.FallbackReferencePrice;
import org.isda.cdm.ValuationPostponement;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("ValuationPostponement_MaximumDaysOfPostponement")
public class ValuationPostponementMaximumDaysOfPostponementDataRule implements Validator<FallbackReferencePrice> {
	
	private static final String NAME = "ValuationPostponement_MaximumDaysOfPostponement";
	private static final String DEFINITION = "when FallbackReferencePrice -> valuationPostponement exists\n" + 
		"then FallbackReferencePrice -> valuationPostponement -> maximumDaysOfPostponement > 0";
	
	@Override
	public ValidationResult<FallbackReferencePrice> validate(RosettaPath path, FallbackReferencePrice fallbackReferencePrice) {
		ComparisonResult result = executeDataRule(fallbackReferencePrice);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "FallbackReferencePrice", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "FallbackReferencePrice", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<FallbackReferencePrice> validate(RosettaPath path, RosettaModelObjectBuilder fallbackReferencePrice) {
		ComparisonResult result = executeDataRule((FallbackReferencePrice)fallbackReferencePrice.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "FallbackReferencePrice", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "FallbackReferencePrice", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(FallbackReferencePrice fallbackReferencePrice) {
		if (ruleIsApplicable(fallbackReferencePrice).get()) {
			return evaluateThenExpression(fallbackReferencePrice);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(FallbackReferencePrice fallbackReferencePrice) {
		return exists(MapperS.of(fallbackReferencePrice).<ValuationPostponement>map("getValuationPostponement", FallbackReferencePrice::getValuationPostponement), false);
	}
	
	private ComparisonResult evaluateThenExpression(FallbackReferencePrice fallbackReferencePrice) {
		return greaterThan(MapperS.of(fallbackReferencePrice).<ValuationPostponement>map("getValuationPostponement", FallbackReferencePrice::getValuationPostponement).<Integer>map("getMaximumDaysOfPostponement", ValuationPostponement::getMaximumDaysOfPostponement), MapperS.of(Integer.valueOf(0)));
	}
}
