package org.isda.cdm.validation.datarule;

import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ComparisonResult;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.Validator;
import org.isda.cdm.WeightedAveragingObservation;

import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
@RosettaDataRule("WeightedAveragingObservation_observationNumber")
public class WeightedAveragingObservationObservationNumberDataRule implements Validator<WeightedAveragingObservation> {
	
	private static final String NAME = "WeightedAveragingObservation_observationNumber";
	private static final String DEFINITION = "when WeightedAveragingObservation -> observationNumber exists\n" + 
		"then WeightedAveragingObservation -> observationNumber >= 0";
	
	@Override
	public ValidationResult<WeightedAveragingObservation> validate(RosettaPath path, WeightedAveragingObservation weightedAveragingObservation) {
		ComparisonResult result = executeDataRule(weightedAveragingObservation);
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE,  "WeightedAveragingObservation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE, "WeightedAveragingObservation", path, DEFINITION, result.getError());
	}
	
	@Override
	public ValidationResult<WeightedAveragingObservation> validate(RosettaPath path, RosettaModelObjectBuilder weightedAveragingObservation) {
		ComparisonResult result = executeDataRule((WeightedAveragingObservation)weightedAveragingObservation.build());
		if (result.get()) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "WeightedAveragingObservation", path, DEFINITION);
		}
		
		return ValidationResult.failure(NAME, ValidationResult.ValidationType.DATA_RULE,  "WeightedAveragingObservation", path, DEFINITION, result.getError());
	}
	
	private ComparisonResult executeDataRule(WeightedAveragingObservation weightedAveragingObservation) {
		if (ruleIsApplicable(weightedAveragingObservation).get()) {
			return evaluateThenExpression(weightedAveragingObservation);
		}
		return ComparisonResult.success();
	}
	
	private ComparisonResult ruleIsApplicable(WeightedAveragingObservation weightedAveragingObservation) {
		return exists(MapperS.of(weightedAveragingObservation).<Integer>map("getObservationNumber", WeightedAveragingObservation::getObservationNumber), false);
	}
	
	private ComparisonResult evaluateThenExpression(WeightedAveragingObservation weightedAveragingObservation) {
		return greaterThanEquals(MapperS.of(weightedAveragingObservation).<Integer>map("getObservationNumber", WeightedAveragingObservation::getObservationNumber), MapperS.of(Integer.valueOf(0)));
	}
}
