package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.AdditionalDisruptionEvents;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class AdditionalDisruptionEventsOnlyExistsValidator implements ValidatorWithArg<AdditionalDisruptionEvents, String> {

	@Override
	public ValidationResult<AdditionalDisruptionEvents> validate(RosettaPath path, AdditionalDisruptionEvents o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("changeInLaw", ExistenceChecker.isSet(o.getChangeInLaw()))
				.put("failureToDeliver", ExistenceChecker.isSet(o.getFailureToDeliver()))
				.put("insolvencyFiling", ExistenceChecker.isSet(o.getInsolvencyFiling()))
				.put("hedgingDisruption", ExistenceChecker.isSet(o.getHedgingDisruption()))
				.put("increasedCostOfHedging", ExistenceChecker.isSet(o.getIncreasedCostOfHedging()))
				.put("foreignOwnershipEvent", ExistenceChecker.isSet(o.getForeignOwnershipEvent()))
				.put("lossOfStockBorrow", ExistenceChecker.isSet(o.getLossOfStockBorrow()))
				.put("maximumStockLoanRate", ExistenceChecker.isSet(o.getMaximumStockLoanRate()))
				.put("increasedCostOfStockBorrow", ExistenceChecker.isSet(o.getIncreasedCostOfStockBorrow()))
				.put("initialStockLoanRate", ExistenceChecker.isSet(o.getInitialStockLoanRate()))
				.put("determiningPartyReference", ExistenceChecker.isSet(o.getDeterminingPartyReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("AdditionalDisruptionEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("AdditionalDisruptionEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AdditionalDisruptionEvents> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder o = (AdditionalDisruptionEvents.AdditionalDisruptionEventsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("changeInLaw", ExistenceChecker.isSet(o.getChangeInLaw()))
				.put("failureToDeliver", ExistenceChecker.isSet(o.getFailureToDeliver()))
				.put("insolvencyFiling", ExistenceChecker.isSet(o.getInsolvencyFiling()))
				.put("hedgingDisruption", ExistenceChecker.isSet(o.getHedgingDisruption()))
				.put("increasedCostOfHedging", ExistenceChecker.isSet(o.getIncreasedCostOfHedging()))
				.put("foreignOwnershipEvent", ExistenceChecker.isSet(o.getForeignOwnershipEvent()))
				.put("lossOfStockBorrow", ExistenceChecker.isSet(o.getLossOfStockBorrow()))
				.put("maximumStockLoanRate", ExistenceChecker.isSet(o.getMaximumStockLoanRate()))
				.put("increasedCostOfStockBorrow", ExistenceChecker.isSet(o.getIncreasedCostOfStockBorrow()))
				.put("initialStockLoanRate", ExistenceChecker.isSet(o.getInitialStockLoanRate()))
				.put("determiningPartyReference", ExistenceChecker.isSet(o.getDeterminingPartyReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("AdditionalDisruptionEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("AdditionalDisruptionEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
