package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.AggregationParameters;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class AggregationParametersOnlyExistsValidator implements ValidatorWithArg<AggregationParameters, String> {

	@Override
	public ValidationResult<AggregationParameters> validate(RosettaPath path, AggregationParameters o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dateTime", ExistenceChecker.isSet(o.getDateTime()))
				.put("totalPosition", ExistenceChecker.isSet(o.getTotalPosition()))
				.put("positionStatus", ExistenceChecker.isSet(o.getPositionStatus()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("product", ExistenceChecker.isSet(o.getProduct()))
				.put("productQualifier", ExistenceChecker.isSet(o.getProductQualifier()))
				.put("contractReference", ExistenceChecker.isSet(o.getContractReference()))
				.put("executionReference", ExistenceChecker.isSet(o.getExecutionReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("AggregationParameters", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("AggregationParameters", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<AggregationParameters> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		AggregationParameters.AggregationParametersBuilder o = (AggregationParameters.AggregationParametersBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dateTime", ExistenceChecker.isSet(o.getDateTime()))
				.put("totalPosition", ExistenceChecker.isSet(o.getTotalPosition()))
				.put("positionStatus", ExistenceChecker.isSet(o.getPositionStatus()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("product", ExistenceChecker.isSet(o.getProduct()))
				.put("productQualifier", ExistenceChecker.isSet(o.getProductQualifier()))
				.put("contractReference", ExistenceChecker.isSet(o.getContractReference()))
				.put("executionReference", ExistenceChecker.isSet(o.getExecutionReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("AggregationParameters", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("AggregationParameters", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
