package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.BermudaExercise;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class BermudaExerciseOnlyExistsValidator implements ValidatorWithArg<BermudaExercise, String> {

	@Override
	public ValidationResult<BermudaExercise> validate(RosettaPath path, BermudaExercise o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bermudaExerciseDates", ExistenceChecker.isSet(o.getBermudaExerciseDates()))
				.put("relevantUnderlyingDate", ExistenceChecker.isSet(o.getRelevantUnderlyingDate()))
				.put("earliestExerciseTime", ExistenceChecker.isSet(o.getEarliestExerciseTime()))
				.put("latestExerciseTime", ExistenceChecker.isSet(o.getLatestExerciseTime()))
				.put("expirationTime", ExistenceChecker.isSet(o.getExpirationTime()))
				.put("multipleExercise", ExistenceChecker.isSet(o.getMultipleExercise()))
				.put("exerciseFeeSchedule", ExistenceChecker.isSet(o.getExerciseFeeSchedule()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("BermudaExercise", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("BermudaExercise", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<BermudaExercise> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		BermudaExercise.BermudaExerciseBuilder o = (BermudaExercise.BermudaExerciseBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bermudaExerciseDates", ExistenceChecker.isSet(o.getBermudaExerciseDates()))
				.put("relevantUnderlyingDate", ExistenceChecker.isSet(o.getRelevantUnderlyingDate()))
				.put("earliestExerciseTime", ExistenceChecker.isSet(o.getEarliestExerciseTime()))
				.put("latestExerciseTime", ExistenceChecker.isSet(o.getLatestExerciseTime()))
				.put("expirationTime", ExistenceChecker.isSet(o.getExpirationTime()))
				.put("multipleExercise", ExistenceChecker.isSet(o.getMultipleExercise()))
				.put("exerciseFeeSchedule", ExistenceChecker.isSet(o.getExerciseFeeSchedule()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("BermudaExercise", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("BermudaExercise", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
