package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.BondPriceAndYieldModel;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class BondPriceAndYieldModelOnlyExistsValidator implements ValidatorWithArg<BondPriceAndYieldModel, String> {

	@Override
	public ValidationResult<BondPriceAndYieldModel> validate(RosettaPath path, BondPriceAndYieldModel o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("cleanOrDirtyPrice", ExistenceChecker.isSet(o.getCleanOrDirtyPrice()))
				.put("relativePrice", ExistenceChecker.isSet(o.getRelativePrice()))
				.put("yieldToMaturity", ExistenceChecker.isSet(o.getYieldToMaturity()))
				.put("inflationFactor", ExistenceChecker.isSet(o.getInflationFactor()))
				.put("allInPrice", ExistenceChecker.isSet(o.getAllInPrice()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("BondPriceAndYieldModel", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("BondPriceAndYieldModel", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<BondPriceAndYieldModel> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		BondPriceAndYieldModel.BondPriceAndYieldModelBuilder o = (BondPriceAndYieldModel.BondPriceAndYieldModelBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("cleanOrDirtyPrice", ExistenceChecker.isSet(o.getCleanOrDirtyPrice()))
				.put("relativePrice", ExistenceChecker.isSet(o.getRelativePrice()))
				.put("yieldToMaturity", ExistenceChecker.isSet(o.getYieldToMaturity()))
				.put("inflationFactor", ExistenceChecker.isSet(o.getInflationFactor()))
				.put("allInPrice", ExistenceChecker.isSet(o.getAllInPrice()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("BondPriceAndYieldModel", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("BondPriceAndYieldModel", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
