package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CalculationPeriod;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CalculationPeriodOnlyExistsValidator implements ValidatorWithArg<CalculationPeriod, String> {

	@Override
	public ValidationResult<CalculationPeriod> validate(RosettaPath path, CalculationPeriod o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedStartDate", ExistenceChecker.isSet(o.getUnadjustedStartDate()))
				.put("unadjustedEndDate", ExistenceChecker.isSet(o.getUnadjustedEndDate()))
				.put("calculationPeriodNumberOfDays", ExistenceChecker.isSet(o.getCalculationPeriodNumberOfDays()))
				.put("notionalAmount", ExistenceChecker.isSet(o.getNotionalAmount()))
				.put("fxLinkedNotionalAmount", ExistenceChecker.isSet(o.getFxLinkedNotionalAmount()))
				.put("floatingRateDefinition", ExistenceChecker.isSet(o.getFloatingRateDefinition()))
				.put("fixedRate", ExistenceChecker.isSet(o.getFixedRate()))
				.put("dayCountYearFraction", ExistenceChecker.isSet(o.getDayCountYearFraction()))
				.put("forecastAmount", ExistenceChecker.isSet(o.getForecastAmount()))
				.put("forecastRate", ExistenceChecker.isSet(o.getForecastRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CalculationPeriod.CalculationPeriodBuilder o = (CalculationPeriod.CalculationPeriodBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedStartDate", ExistenceChecker.isSet(o.getUnadjustedStartDate()))
				.put("unadjustedEndDate", ExistenceChecker.isSet(o.getUnadjustedEndDate()))
				.put("calculationPeriodNumberOfDays", ExistenceChecker.isSet(o.getCalculationPeriodNumberOfDays()))
				.put("notionalAmount", ExistenceChecker.isSet(o.getNotionalAmount()))
				.put("fxLinkedNotionalAmount", ExistenceChecker.isSet(o.getFxLinkedNotionalAmount()))
				.put("floatingRateDefinition", ExistenceChecker.isSet(o.getFloatingRateDefinition()))
				.put("fixedRate", ExistenceChecker.isSet(o.getFixedRate()))
				.put("dayCountYearFraction", ExistenceChecker.isSet(o.getDayCountYearFraction()))
				.put("forecastAmount", ExistenceChecker.isSet(o.getForecastAmount()))
				.put("forecastRate", ExistenceChecker.isSet(o.getForecastRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
