package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CashSettlementTerms;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CashSettlementTermsOnlyExistsValidator implements ValidatorWithArg<CashSettlementTerms, String> {

	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, CashSettlementTerms o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("valuationDate", ExistenceChecker.isSet(o.getValuationDate()))
				.put("valuationTime", ExistenceChecker.isSet(o.getValuationTime()))
				.put("quotationMethod", ExistenceChecker.isSet(o.getQuotationMethod()))
				.put("quotationAmount", ExistenceChecker.isSet(o.getQuotationAmount()))
				.put("minimumQuotationAmout", ExistenceChecker.isSet(o.getMinimumQuotationAmout()))
				.put("dealer", ExistenceChecker.isSet(o.getDealer()))
				.put("cashSettlementBusinessDays", ExistenceChecker.isSet(o.getCashSettlementBusinessDays()))
				.put("cashSettlementAmount", ExistenceChecker.isSet(o.getCashSettlementAmount()))
				.put("recoveryFactor", ExistenceChecker.isSet(o.getRecoveryFactor()))
				.put("fixedSettlement", ExistenceChecker.isSet(o.getFixedSettlement()))
				.put("accruedInterest", ExistenceChecker.isSet(o.getAccruedInterest()))
				.put("valuationMethod", ExistenceChecker.isSet(o.getValuationMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CashSettlementTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CashSettlementTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CashSettlementTerms> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CashSettlementTerms.CashSettlementTermsBuilder o = (CashSettlementTerms.CashSettlementTermsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("valuationDate", ExistenceChecker.isSet(o.getValuationDate()))
				.put("valuationTime", ExistenceChecker.isSet(o.getValuationTime()))
				.put("quotationMethod", ExistenceChecker.isSet(o.getQuotationMethod()))
				.put("quotationAmount", ExistenceChecker.isSet(o.getQuotationAmount()))
				.put("minimumQuotationAmout", ExistenceChecker.isSet(o.getMinimumQuotationAmout()))
				.put("dealer", ExistenceChecker.isSet(o.getDealer()))
				.put("cashSettlementBusinessDays", ExistenceChecker.isSet(o.getCashSettlementBusinessDays()))
				.put("cashSettlementAmount", ExistenceChecker.isSet(o.getCashSettlementAmount()))
				.put("recoveryFactor", ExistenceChecker.isSet(o.getRecoveryFactor()))
				.put("fixedSettlement", ExistenceChecker.isSet(o.getFixedSettlement()))
				.put("accruedInterest", ExistenceChecker.isSet(o.getAccruedInterest()))
				.put("valuationMethod", ExistenceChecker.isSet(o.getValuationMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CashSettlementTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CashSettlementTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
