package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Cashflow;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CashflowOnlyExistsValidator implements ValidatorWithArg<Cashflow, String> {

	@Override
	public ValidationResult<Cashflow> validate(RosettaPath path, Cashflow o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("paymentDiscounting", ExistenceChecker.isSet(o.getPaymentDiscounting()))
				.put("cashflowAmount", ExistenceChecker.isSet(o.getCashflowAmount()))
				.put("premiumExpression", ExistenceChecker.isSet(o.getPremiumExpression()))
				.put("cashflowDate", ExistenceChecker.isSet(o.getCashflowDate()))
				.put("cashflowCalculation", ExistenceChecker.isSet(o.getCashflowCalculation()))
				.put("cashflowType", ExistenceChecker.isSet(o.getCashflowType()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("presentValueAmount", ExistenceChecker.isSet(o.getPresentValueAmount()))
				.put("paymentDelay", ExistenceChecker.isSet(o.getPaymentDelay()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Cashflow", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Cashflow", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Cashflow> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Cashflow.CashflowBuilder o = (Cashflow.CashflowBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("paymentDiscounting", ExistenceChecker.isSet(o.getPaymentDiscounting()))
				.put("cashflowAmount", ExistenceChecker.isSet(o.getCashflowAmount()))
				.put("premiumExpression", ExistenceChecker.isSet(o.getPremiumExpression()))
				.put("cashflowDate", ExistenceChecker.isSet(o.getCashflowDate()))
				.put("cashflowCalculation", ExistenceChecker.isSet(o.getCashflowCalculation()))
				.put("cashflowType", ExistenceChecker.isSet(o.getCashflowType()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("presentValueAmount", ExistenceChecker.isSet(o.getPresentValueAmount()))
				.put("paymentDelay", ExistenceChecker.isSet(o.getPaymentDelay()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Cashflow", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Cashflow", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
