package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ConditionsElections;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ConditionsElectionsOnlyExistsValidator implements ValidatorWithArg<ConditionsElections, String> {

	@Override
	public ValidationResult<ConditionsElections> validate(RosettaPath path, ConditionsElections o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("illegality", ExistenceChecker.isSet(o.getIllegality()))
				.put("forceMajeure", ExistenceChecker.isSet(o.getForceMajeure()))
				.put("taxEvent", ExistenceChecker.isSet(o.getTaxEvent()))
				.put("taxEventUponMerger", ExistenceChecker.isSet(o.getTaxEventUponMerger()))
				.put("creditEventUponMerger", ExistenceChecker.isSet(o.getCreditEventUponMerger()))
				.put("additionalTerminationEvents", ExistenceChecker.isSet(o.getAdditionalTerminationEvents()))
				.put("additionalLanguage", ExistenceChecker.isSet(o.getAdditionalLanguage()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ConditionsElections", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ConditionsElections", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ConditionsElections> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ConditionsElections.ConditionsElectionsBuilder o = (ConditionsElections.ConditionsElectionsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("illegality", ExistenceChecker.isSet(o.getIllegality()))
				.put("forceMajeure", ExistenceChecker.isSet(o.getForceMajeure()))
				.put("taxEvent", ExistenceChecker.isSet(o.getTaxEvent()))
				.put("taxEventUponMerger", ExistenceChecker.isSet(o.getTaxEventUponMerger()))
				.put("creditEventUponMerger", ExistenceChecker.isSet(o.getCreditEventUponMerger()))
				.put("additionalTerminationEvents", ExistenceChecker.isSet(o.getAdditionalTerminationEvents()))
				.put("additionalLanguage", ExistenceChecker.isSet(o.getAdditionalLanguage()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ConditionsElections", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ConditionsElections", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
