package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Contract;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ContractOnlyExistsValidator implements ValidatorWithArg<Contract, String> {

	@Override
	public ValidationResult<Contract> validate(RosettaPath path, Contract o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("contractIdentifier", ExistenceChecker.isSet(o.getContractIdentifier()))
				.put("tradeDate", ExistenceChecker.isSet(o.getTradeDate()))
				.put("clearedDate", ExistenceChecker.isSet(o.getClearedDate()))
				.put("contractualProduct", ExistenceChecker.isSet(o.getContractualProduct()))
				.put("contractualQuantity", ExistenceChecker.isSet(o.getContractualQuantity()))
				.put("collateral", ExistenceChecker.isSet(o.getCollateral()))
				.put("documentation", ExistenceChecker.isSet(o.getDocumentation()))
				.put("governingLaw", ExistenceChecker.isSet(o.getGoverningLaw()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("account", ExistenceChecker.isSet(o.getAccount()))
				.put("partyRole", ExistenceChecker.isSet(o.getPartyRole()))
				.put("calculationAgent", ExistenceChecker.isSet(o.getCalculationAgent()))
				.put("partyContractInformation", ExistenceChecker.isSet(o.getPartyContractInformation()))
				.put("closedState", ExistenceChecker.isSet(o.getClosedState()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Contract", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Contract", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Contract> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Contract.ContractBuilder o = (Contract.ContractBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("contractIdentifier", ExistenceChecker.isSet(o.getContractIdentifier()))
				.put("tradeDate", ExistenceChecker.isSet(o.getTradeDate()))
				.put("clearedDate", ExistenceChecker.isSet(o.getClearedDate()))
				.put("contractualProduct", ExistenceChecker.isSet(o.getContractualProduct()))
				.put("contractualQuantity", ExistenceChecker.isSet(o.getContractualQuantity()))
				.put("collateral", ExistenceChecker.isSet(o.getCollateral()))
				.put("documentation", ExistenceChecker.isSet(o.getDocumentation()))
				.put("governingLaw", ExistenceChecker.isSet(o.getGoverningLaw()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("account", ExistenceChecker.isSet(o.getAccount()))
				.put("partyRole", ExistenceChecker.isSet(o.getPartyRole()))
				.put("calculationAgent", ExistenceChecker.isSet(o.getCalculationAgent()))
				.put("partyContractInformation", ExistenceChecker.isSet(o.getPartyContractInformation()))
				.put("closedState", ExistenceChecker.isSet(o.getClosedState()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Contract", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Contract", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
