package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CreditEvents;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CreditEventsOnlyExistsValidator implements ValidatorWithArg<CreditEvents, String> {

	@Override
	public ValidationResult<CreditEvents> validate(RosettaPath path, CreditEvents o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bankruptcy", ExistenceChecker.isSet(o.getBankruptcy()))
				.put("failureToPay", ExistenceChecker.isSet(o.getFailureToPay()))
				.put("failureToPayPrincipal", ExistenceChecker.isSet(o.getFailureToPayPrincipal()))
				.put("failureToPayInterest", ExistenceChecker.isSet(o.getFailureToPayInterest()))
				.put("obligationDefault", ExistenceChecker.isSet(o.getObligationDefault()))
				.put("obligationAcceleration", ExistenceChecker.isSet(o.getObligationAcceleration()))
				.put("repudiationMoratorium", ExistenceChecker.isSet(o.getRepudiationMoratorium()))
				.put("restructuring", ExistenceChecker.isSet(o.getRestructuring()))
				.put("governmentalIntervention", ExistenceChecker.isSet(o.getGovernmentalIntervention()))
				.put("distressedRatingsDowngrade", ExistenceChecker.isSet(o.getDistressedRatingsDowngrade()))
				.put("maturityExtension", ExistenceChecker.isSet(o.getMaturityExtension()))
				.put("writedown", ExistenceChecker.isSet(o.getWritedown()))
				.put("impliedWritedown", ExistenceChecker.isSet(o.getImpliedWritedown()))
				.put("defaultRequirement", ExistenceChecker.isSet(o.getDefaultRequirement()))
				.put("creditEventNotice", ExistenceChecker.isSet(o.getCreditEventNotice()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CreditEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CreditEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CreditEvents> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CreditEvents.CreditEventsBuilder o = (CreditEvents.CreditEventsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bankruptcy", ExistenceChecker.isSet(o.getBankruptcy()))
				.put("failureToPay", ExistenceChecker.isSet(o.getFailureToPay()))
				.put("failureToPayPrincipal", ExistenceChecker.isSet(o.getFailureToPayPrincipal()))
				.put("failureToPayInterest", ExistenceChecker.isSet(o.getFailureToPayInterest()))
				.put("obligationDefault", ExistenceChecker.isSet(o.getObligationDefault()))
				.put("obligationAcceleration", ExistenceChecker.isSet(o.getObligationAcceleration()))
				.put("repudiationMoratorium", ExistenceChecker.isSet(o.getRepudiationMoratorium()))
				.put("restructuring", ExistenceChecker.isSet(o.getRestructuring()))
				.put("governmentalIntervention", ExistenceChecker.isSet(o.getGovernmentalIntervention()))
				.put("distressedRatingsDowngrade", ExistenceChecker.isSet(o.getDistressedRatingsDowngrade()))
				.put("maturityExtension", ExistenceChecker.isSet(o.getMaturityExtension()))
				.put("writedown", ExistenceChecker.isSet(o.getWritedown()))
				.put("impliedWritedown", ExistenceChecker.isSet(o.getImpliedWritedown()))
				.put("defaultRequirement", ExistenceChecker.isSet(o.getDefaultRequirement()))
				.put("creditEventNotice", ExistenceChecker.isSet(o.getCreditEventNotice()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CreditEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CreditEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
