package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Csa2016;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class Csa2016OnlyExistsValidator implements ValidatorWithArg<Csa2016, String> {

	@Override
	public ValidationResult<Csa2016> validate(RosettaPath path, Csa2016 o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("baseCurrency", ExistenceChecker.isSet(o.getBaseCurrency()))
				.put("additionalObligations", ExistenceChecker.isSet(o.getAdditionalObligations()))
				.put("conditionsPrecedent", ExistenceChecker.isSet(o.getConditionsPrecedent()))
				.put("substitution", ExistenceChecker.isSet(o.getSubstitution()))
				.put("disputeResolution", ExistenceChecker.isSet(o.getDisputeResolution()))
				.put("additionalRepresentation", ExistenceChecker.isSet(o.getAdditionalRepresentation()))
				.put("demandsAndNotices", ExistenceChecker.isSet(o.getDemandsAndNotices()))
				.put("addressesForTransfer", ExistenceChecker.isSet(o.getAddressesForTransfer()))
				.put("bespokeProvision", ExistenceChecker.isSet(o.getBespokeProvision()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Csa2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Csa2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Csa2016> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Csa2016.Csa2016Builder o = (Csa2016.Csa2016Builder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("baseCurrency", ExistenceChecker.isSet(o.getBaseCurrency()))
				.put("additionalObligations", ExistenceChecker.isSet(o.getAdditionalObligations()))
				.put("conditionsPrecedent", ExistenceChecker.isSet(o.getConditionsPrecedent()))
				.put("substitution", ExistenceChecker.isSet(o.getSubstitution()))
				.put("disputeResolution", ExistenceChecker.isSet(o.getDisputeResolution()))
				.put("additionalRepresentation", ExistenceChecker.isSet(o.getAdditionalRepresentation()))
				.put("demandsAndNotices", ExistenceChecker.isSet(o.getDemandsAndNotices()))
				.put("addressesForTransfer", ExistenceChecker.isSet(o.getAddressesForTransfer()))
				.put("bespokeProvision", ExistenceChecker.isSet(o.getBespokeProvision()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Csa2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Csa2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
