package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CsaInitialMargin2016JapaneseLaw;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CsaInitialMargin2016JapaneseLawOnlyExistsValidator implements ValidatorWithArg<CsaInitialMargin2016JapaneseLaw, String> {

	@Override
	public ValidationResult<CsaInitialMargin2016JapaneseLaw> validate(RosettaPath path, CsaInitialMargin2016JapaneseLaw o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("obligorPostingObligations", ExistenceChecker.isSet(o.getObligorPostingObligations()))
				.put("collateralManagementAgreeement", ExistenceChecker.isSet(o.getCollateralManagementAgreeement()))
				.put("collateralManager", ExistenceChecker.isSet(o.getCollateralManager()))
				.put("otherEligibleSupport", ExistenceChecker.isSet(o.getOtherEligibleSupport()))
				.put("cashSettlementDay", ExistenceChecker.isSet(o.getCashSettlementDay()))
				.put("securitiesSettlementDay", ExistenceChecker.isSet(o.getSecuritiesSettlementDay()))
				.put("obligorRightsEvent", ExistenceChecker.isSet(o.getObligorRightsEvent()))
				.put("obligorAdditionalRightsEvent", ExistenceChecker.isSet(o.getObligorAdditionalRightsEvent()))
				.put("automaticSetOff", ExistenceChecker.isSet(o.getAutomaticSetOff()))
				.put("collateralManagementArrangement", ExistenceChecker.isSet(o.getCollateralManagementArrangement()))
				.put("distributionAndInterestPayment", ExistenceChecker.isSet(o.getDistributionAndInterestPayment()))
				.put("otherEligibleAndPostedSupport", ExistenceChecker.isSet(o.getOtherEligibleAndPostedSupport()))
				.put("otherCsa", ExistenceChecker.isSet(o.getOtherCsa()))
				.put("japaneseLawCsa", ExistenceChecker.isSet(o.getJapaneseLawCsa()))
				.put("trustSchemeAddendum", ExistenceChecker.isSet(o.getTrustSchemeAddendum()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016JapaneseLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016JapaneseLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaInitialMargin2016JapaneseLaw> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder o = (CsaInitialMargin2016JapaneseLaw.CsaInitialMargin2016JapaneseLawBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("obligorPostingObligations", ExistenceChecker.isSet(o.getObligorPostingObligations()))
				.put("collateralManagementAgreeement", ExistenceChecker.isSet(o.getCollateralManagementAgreeement()))
				.put("collateralManager", ExistenceChecker.isSet(o.getCollateralManager()))
				.put("otherEligibleSupport", ExistenceChecker.isSet(o.getOtherEligibleSupport()))
				.put("cashSettlementDay", ExistenceChecker.isSet(o.getCashSettlementDay()))
				.put("securitiesSettlementDay", ExistenceChecker.isSet(o.getSecuritiesSettlementDay()))
				.put("obligorRightsEvent", ExistenceChecker.isSet(o.getObligorRightsEvent()))
				.put("obligorAdditionalRightsEvent", ExistenceChecker.isSet(o.getObligorAdditionalRightsEvent()))
				.put("automaticSetOff", ExistenceChecker.isSet(o.getAutomaticSetOff()))
				.put("collateralManagementArrangement", ExistenceChecker.isSet(o.getCollateralManagementArrangement()))
				.put("distributionAndInterestPayment", ExistenceChecker.isSet(o.getDistributionAndInterestPayment()))
				.put("otherEligibleAndPostedSupport", ExistenceChecker.isSet(o.getOtherEligibleAndPostedSupport()))
				.put("otherCsa", ExistenceChecker.isSet(o.getOtherCsa()))
				.put("japaneseLawCsa", ExistenceChecker.isSet(o.getJapaneseLawCsa()))
				.put("trustSchemeAddendum", ExistenceChecker.isSet(o.getTrustSchemeAddendum()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016JapaneseLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016JapaneseLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
