package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CsaInitialMargin2016NewYorkLaw;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CsaInitialMargin2016NewYorkLawOnlyExistsValidator implements ValidatorWithArg<CsaInitialMargin2016NewYorkLaw, String> {

	@Override
	public ValidationResult<CsaInitialMargin2016NewYorkLaw> validate(RosettaPath path, CsaInitialMargin2016NewYorkLaw o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("pledgorPostingObligations", ExistenceChecker.isSet(o.getPledgorPostingObligations()))
				.put("otherEligibleSupport", ExistenceChecker.isSet(o.getOtherEligibleSupport()))
				.put("transferTiming", ExistenceChecker.isSet(o.getTransferTiming()))
				.put("pledgorRightsEvent", ExistenceChecker.isSet(o.getPledgorRightsEvent()))
				.put("pledgorAdditionalRightsEvent", ExistenceChecker.isSet(o.getPledgorAdditionalRightsEvent()))
				.put("deliveryInLieuRight", ExistenceChecker.isSet(o.getDeliveryInLieuRight()))
				.put("custodyArrangements", ExistenceChecker.isSet(o.getCustodyArrangements()))
				.put("otherEligibleAndPostedSupport", ExistenceChecker.isSet(o.getOtherEligibleAndPostedSupport()))
				.put("japaneseSecuritiesProvisions", ExistenceChecker.isSet(o.getJapaneseSecuritiesProvisions()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016NewYorkLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016NewYorkLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaInitialMargin2016NewYorkLaw> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder o = (CsaInitialMargin2016NewYorkLaw.CsaInitialMargin2016NewYorkLawBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("pledgorPostingObligations", ExistenceChecker.isSet(o.getPledgorPostingObligations()))
				.put("otherEligibleSupport", ExistenceChecker.isSet(o.getOtherEligibleSupport()))
				.put("transferTiming", ExistenceChecker.isSet(o.getTransferTiming()))
				.put("pledgorRightsEvent", ExistenceChecker.isSet(o.getPledgorRightsEvent()))
				.put("pledgorAdditionalRightsEvent", ExistenceChecker.isSet(o.getPledgorAdditionalRightsEvent()))
				.put("deliveryInLieuRight", ExistenceChecker.isSet(o.getDeliveryInLieuRight()))
				.put("custodyArrangements", ExistenceChecker.isSet(o.getCustodyArrangements()))
				.put("otherEligibleAndPostedSupport", ExistenceChecker.isSet(o.getOtherEligibleAndPostedSupport()))
				.put("japaneseSecuritiesProvisions", ExistenceChecker.isSet(o.getJapaneseSecuritiesProvisions()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016NewYorkLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016NewYorkLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
