package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CsaInitialMargin2016;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CsaInitialMargin2016OnlyExistsValidator implements ValidatorWithArg<CsaInitialMargin2016, String> {

	@Override
	public ValidationResult<CsaInitialMargin2016> validate(RosettaPath path, CsaInitialMargin2016 o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("regime", ExistenceChecker.isSet(o.getRegime()))
				.put("oneWayProvisions", ExistenceChecker.isSet(o.getOneWayProvisions()))
				.put("method", ExistenceChecker.isSet(o.getMethod()))
				.put("identifiedCrossCurrencySwap", ExistenceChecker.isSet(o.getIdentifiedCrossCurrencySwap()))
				.put("sensitivityToEquity", ExistenceChecker.isSet(o.getSensitivityToEquity()))
				.put("sensitivityToCommodity", ExistenceChecker.isSet(o.getSensitivityToCommodity()))
				.put("fxHaircutCurrency", ExistenceChecker.isSet(o.getFxHaircutCurrency()))
				.put("creditSupportObligations", ExistenceChecker.isSet(o.getCreditSupportObligations()))
				.put("calculationDateLocation", ExistenceChecker.isSet(o.getCalculationDateLocation()))
				.put("notificationTime", ExistenceChecker.isSet(o.getNotificationTime()))
				.put("terminationCurrency", ExistenceChecker.isSet(o.getTerminationCurrency()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaInitialMargin2016> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CsaInitialMargin2016.CsaInitialMargin2016Builder o = (CsaInitialMargin2016.CsaInitialMargin2016Builder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("regime", ExistenceChecker.isSet(o.getRegime()))
				.put("oneWayProvisions", ExistenceChecker.isSet(o.getOneWayProvisions()))
				.put("method", ExistenceChecker.isSet(o.getMethod()))
				.put("identifiedCrossCurrencySwap", ExistenceChecker.isSet(o.getIdentifiedCrossCurrencySwap()))
				.put("sensitivityToEquity", ExistenceChecker.isSet(o.getSensitivityToEquity()))
				.put("sensitivityToCommodity", ExistenceChecker.isSet(o.getSensitivityToCommodity()))
				.put("fxHaircutCurrency", ExistenceChecker.isSet(o.getFxHaircutCurrency()))
				.put("creditSupportObligations", ExistenceChecker.isSet(o.getCreditSupportObligations()))
				.put("calculationDateLocation", ExistenceChecker.isSet(o.getCalculationDateLocation()))
				.put("notificationTime", ExistenceChecker.isSet(o.getNotificationTime()))
				.put("terminationCurrency", ExistenceChecker.isSet(o.getTerminationCurrency()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaInitialMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaInitialMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
