package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CsaVariationMargin2016;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CsaVariationMargin2016OnlyExistsValidator implements ValidatorWithArg<CsaVariationMargin2016, String> {

	@Override
	public ValidationResult<CsaVariationMargin2016> validate(RosettaPath path, CsaVariationMargin2016 o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("creditSupportObligations", ExistenceChecker.isSet(o.getCreditSupportObligations()))
				.put("valuationAgent", ExistenceChecker.isSet(o.getValuationAgent()))
				.put("valuationDateLocation", ExistenceChecker.isSet(o.getValuationDateLocation()))
				.put("valuationTime", ExistenceChecker.isSet(o.getValuationTime()))
				.put("notificationTime", ExistenceChecker.isSet(o.getNotificationTime()))
				.put("holdingAndUsingPostedCollateral", ExistenceChecker.isSet(o.getHoldingAndUsingPostedCollateral()))
				.put("creditSupportOffsets", ExistenceChecker.isSet(o.getCreditSupportOffsets()))
				.put("otherCsa", ExistenceChecker.isSet(o.getOtherCsa()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaVariationMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaVariationMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsaVariationMargin2016> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CsaVariationMargin2016.CsaVariationMargin2016Builder o = (CsaVariationMargin2016.CsaVariationMargin2016Builder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("creditSupportObligations", ExistenceChecker.isSet(o.getCreditSupportObligations()))
				.put("valuationAgent", ExistenceChecker.isSet(o.getValuationAgent()))
				.put("valuationDateLocation", ExistenceChecker.isSet(o.getValuationDateLocation()))
				.put("valuationTime", ExistenceChecker.isSet(o.getValuationTime()))
				.put("notificationTime", ExistenceChecker.isSet(o.getNotificationTime()))
				.put("holdingAndUsingPostedCollateral", ExistenceChecker.isSet(o.getHoldingAndUsingPostedCollateral()))
				.put("creditSupportOffsets", ExistenceChecker.isSet(o.getCreditSupportOffsets()))
				.put("otherCsa", ExistenceChecker.isSet(o.getOtherCsa()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsaVariationMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsaVariationMargin2016", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
