package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.CsdInitialMargin2016EnglishLaw;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class CsdInitialMargin2016EnglishLawOnlyExistsValidator implements ValidatorWithArg<CsdInitialMargin2016EnglishLaw, String> {

	@Override
	public ValidationResult<CsdInitialMargin2016EnglishLaw> validate(RosettaPath path, CsdInitialMargin2016EnglishLaw o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("chargorPostingObligations", ExistenceChecker.isSet(o.getChargorPostingObligations()))
				.put("transferTiming", ExistenceChecker.isSet(o.getTransferTiming()))
				.put("chargorRightsEvent", ExistenceChecker.isSet(o.getChargorRightsEvent()))
				.put("chargorAdditionalRightsEvent", ExistenceChecker.isSet(o.getChargorAdditionalRightsEvent()))
				.put("deliveryInLieuRight", ExistenceChecker.isSet(o.getDeliveryInLieuRight()))
				.put("custodyArrangements", ExistenceChecker.isSet(o.getCustodyArrangements()))
				.put("processAgent", ExistenceChecker.isSet(o.getProcessAgent()))
				.put("appropriatedCollateralValuation", ExistenceChecker.isSet(o.getAppropriatedCollateralValuation()))
				.put("japaneseSecuritiesProvisions", ExistenceChecker.isSet(o.getJapaneseSecuritiesProvisions()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsdInitialMargin2016EnglishLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsdInitialMargin2016EnglishLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<CsdInitialMargin2016EnglishLaw> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder o = (CsdInitialMargin2016EnglishLaw.CsdInitialMargin2016EnglishLawBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("chargorPostingObligations", ExistenceChecker.isSet(o.getChargorPostingObligations()))
				.put("transferTiming", ExistenceChecker.isSet(o.getTransferTiming()))
				.put("chargorRightsEvent", ExistenceChecker.isSet(o.getChargorRightsEvent()))
				.put("chargorAdditionalRightsEvent", ExistenceChecker.isSet(o.getChargorAdditionalRightsEvent()))
				.put("deliveryInLieuRight", ExistenceChecker.isSet(o.getDeliveryInLieuRight()))
				.put("custodyArrangements", ExistenceChecker.isSet(o.getCustodyArrangements()))
				.put("processAgent", ExistenceChecker.isSet(o.getProcessAgent()))
				.put("appropriatedCollateralValuation", ExistenceChecker.isSet(o.getAppropriatedCollateralValuation()))
				.put("japaneseSecuritiesProvisions", ExistenceChecker.isSet(o.getJapaneseSecuritiesProvisions()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("CsdInitialMargin2016EnglishLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("CsdInitialMargin2016EnglishLaw", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
