package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.DividendReturnTerms;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class DividendReturnTermsOnlyExistsValidator implements ValidatorWithArg<DividendReturnTerms, String> {

	@Override
	public ValidationResult<DividendReturnTerms> validate(RosettaPath path, DividendReturnTerms o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dividendPayout", ExistenceChecker.isSet(o.getDividendPayout()))
				.put("dividendReinvestment", ExistenceChecker.isSet(o.getDividendReinvestment()))
				.put("dividendEntitlement", ExistenceChecker.isSet(o.getDividendEntitlement()))
				.put("dividendAmountType", ExistenceChecker.isSet(o.getDividendAmountType()))
				.put("dividendPaymentDate", ExistenceChecker.isSet(o.getDividendPaymentDate()))
				.put("dividendPeriod", ExistenceChecker.isSet(o.getDividendPeriod()))
				.put("dividendPeriodEffectiveDate", ExistenceChecker.isSet(o.getDividendPeriodEffectiveDate()))
				.put("dividendPeriodEndDate", ExistenceChecker.isSet(o.getDividendPeriodEndDate()))
				.put("extraordinaryDividendsParty", ExistenceChecker.isSet(o.getExtraordinaryDividendsParty()))
				.put("excessDividendAmount", ExistenceChecker.isSet(o.getExcessDividendAmount()))
				.put("dividendCurrency", ExistenceChecker.isSet(o.getDividendCurrency()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("DividendReturnTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("DividendReturnTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<DividendReturnTerms> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		DividendReturnTerms.DividendReturnTermsBuilder o = (DividendReturnTerms.DividendReturnTermsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dividendPayout", ExistenceChecker.isSet(o.getDividendPayout()))
				.put("dividendReinvestment", ExistenceChecker.isSet(o.getDividendReinvestment()))
				.put("dividendEntitlement", ExistenceChecker.isSet(o.getDividendEntitlement()))
				.put("dividendAmountType", ExistenceChecker.isSet(o.getDividendAmountType()))
				.put("dividendPaymentDate", ExistenceChecker.isSet(o.getDividendPaymentDate()))
				.put("dividendPeriod", ExistenceChecker.isSet(o.getDividendPeriod()))
				.put("dividendPeriodEffectiveDate", ExistenceChecker.isSet(o.getDividendPeriodEffectiveDate()))
				.put("dividendPeriodEndDate", ExistenceChecker.isSet(o.getDividendPeriodEndDate()))
				.put("extraordinaryDividendsParty", ExistenceChecker.isSet(o.getExtraordinaryDividendsParty()))
				.put("excessDividendAmount", ExistenceChecker.isSet(o.getExcessDividendAmount()))
				.put("dividendCurrency", ExistenceChecker.isSet(o.getDividendCurrency()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("DividendReturnTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("DividendReturnTerms", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
