package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.DocumentationIdentification;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class DocumentationIdentificationOnlyExistsValidator implements ValidatorWithArg<DocumentationIdentification, String> {

	@Override
	public ValidationResult<DocumentationIdentification> validate(RosettaPath path, DocumentationIdentification o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("masterAgreement", ExistenceChecker.isSet(o.getMasterAgreement()))
				.put("masterConfirmation", ExistenceChecker.isSet(o.getMasterConfirmation()))
				.put("brokerConfirmation", ExistenceChecker.isSet(o.getBrokerConfirmation()))
				.put("contractualDefinitions", ExistenceChecker.isSet(o.getContractualDefinitions()))
				.put("contractualTermsSupplement", ExistenceChecker.isSet(o.getContractualTermsSupplement()))
				.put("contractualMatrix", ExistenceChecker.isSet(o.getContractualMatrix()))
				.put("creditSupportAgreement", ExistenceChecker.isSet(o.getCreditSupportAgreement()))
				.put("otherAgreement", ExistenceChecker.isSet(o.getOtherAgreement()))
				.put("attachment", ExistenceChecker.isSet(o.getAttachment()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("DocumentationIdentification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("DocumentationIdentification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<DocumentationIdentification> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		DocumentationIdentification.DocumentationIdentificationBuilder o = (DocumentationIdentification.DocumentationIdentificationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("masterAgreement", ExistenceChecker.isSet(o.getMasterAgreement()))
				.put("masterConfirmation", ExistenceChecker.isSet(o.getMasterConfirmation()))
				.put("brokerConfirmation", ExistenceChecker.isSet(o.getBrokerConfirmation()))
				.put("contractualDefinitions", ExistenceChecker.isSet(o.getContractualDefinitions()))
				.put("contractualTermsSupplement", ExistenceChecker.isSet(o.getContractualTermsSupplement()))
				.put("contractualMatrix", ExistenceChecker.isSet(o.getContractualMatrix()))
				.put("creditSupportAgreement", ExistenceChecker.isSet(o.getCreditSupportAgreement()))
				.put("otherAgreement", ExistenceChecker.isSet(o.getOtherAgreement()))
				.put("attachment", ExistenceChecker.isSet(o.getAttachment()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("DocumentationIdentification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("DocumentationIdentification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
