package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.EarlyTerminationEvent;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class EarlyTerminationEventOnlyExistsValidator implements ValidatorWithArg<EarlyTerminationEvent, String> {

	@Override
	public ValidationResult<EarlyTerminationEvent> validate(RosettaPath path, EarlyTerminationEvent o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("adjustedExerciseDate", ExistenceChecker.isSet(o.getAdjustedExerciseDate()))
				.put("adjustedEarlyTerminationDate", ExistenceChecker.isSet(o.getAdjustedEarlyTerminationDate()))
				.put("adjustedCashSettlementValuationDate", ExistenceChecker.isSet(o.getAdjustedCashSettlementValuationDate()))
				.put("adjustedCashSettlementPaymentDate", ExistenceChecker.isSet(o.getAdjustedCashSettlementPaymentDate()))
				.put("adjustedExerciseFeePaymentDate", ExistenceChecker.isSet(o.getAdjustedExerciseFeePaymentDate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EarlyTerminationEvent", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EarlyTerminationEvent", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EarlyTerminationEvent> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		EarlyTerminationEvent.EarlyTerminationEventBuilder o = (EarlyTerminationEvent.EarlyTerminationEventBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("adjustedExerciseDate", ExistenceChecker.isSet(o.getAdjustedExerciseDate()))
				.put("adjustedEarlyTerminationDate", ExistenceChecker.isSet(o.getAdjustedEarlyTerminationDate()))
				.put("adjustedCashSettlementValuationDate", ExistenceChecker.isSet(o.getAdjustedCashSettlementValuationDate()))
				.put("adjustedCashSettlementPaymentDate", ExistenceChecker.isSet(o.getAdjustedCashSettlementPaymentDate()))
				.put("adjustedExerciseFeePaymentDate", ExistenceChecker.isSet(o.getAdjustedExerciseFeePaymentDate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EarlyTerminationEvent", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EarlyTerminationEvent", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
