package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.EquityPayout;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class EquityPayoutOnlyExistsValidator implements ValidatorWithArg<EquityPayout, String> {

	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, EquityPayout o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("underlier", ExistenceChecker.isSet(o.getUnderlier()))
				.put("returnType", ExistenceChecker.isSet(o.getReturnType()))
				.put("dividendReturnTerms", ExistenceChecker.isSet(o.getDividendReturnTerms()))
				.put("priceReturnTerms", ExistenceChecker.isSet(o.getPriceReturnTerms()))
				.put("calculationPeriodDates", ExistenceChecker.isSet(o.getCalculationPeriodDates()))
				.put("paymentDates", ExistenceChecker.isSet(o.getPaymentDates()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.put("rateOfReturn", ExistenceChecker.isSet(o.getRateOfReturn()))
				.put("performance", ExistenceChecker.isSet(o.getPerformance()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EquityPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EquityPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EquityPayout> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		EquityPayout.EquityPayoutBuilder o = (EquityPayout.EquityPayoutBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("underlier", ExistenceChecker.isSet(o.getUnderlier()))
				.put("returnType", ExistenceChecker.isSet(o.getReturnType()))
				.put("dividendReturnTerms", ExistenceChecker.isSet(o.getDividendReturnTerms()))
				.put("priceReturnTerms", ExistenceChecker.isSet(o.getPriceReturnTerms()))
				.put("calculationPeriodDates", ExistenceChecker.isSet(o.getCalculationPeriodDates()))
				.put("paymentDates", ExistenceChecker.isSet(o.getPaymentDates()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.put("rateOfReturn", ExistenceChecker.isSet(o.getRateOfReturn()))
				.put("performance", ExistenceChecker.isSet(o.getPerformance()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EquityPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EquityPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
