package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.EquitySwapMasterConfirmation2018;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class EquitySwapMasterConfirmation2018OnlyExistsValidator implements ValidatorWithArg<EquitySwapMasterConfirmation2018, String> {

	@Override
	public ValidationResult<EquitySwapMasterConfirmation2018> validate(RosettaPath path, EquitySwapMasterConfirmation2018 o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("typeOfSwapElection", ExistenceChecker.isSet(o.getTypeOfSwapElection()))
				.put("pricingMethodElection", ExistenceChecker.isSet(o.getPricingMethodElection()))
				.put("linearInterpolationElection", ExistenceChecker.isSet(o.getLinearInterpolationElection()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.put("equityCalculationPeriod", ExistenceChecker.isSet(o.getEquityCalculationPeriod()))
				.put("equityCashSettlementDates", ExistenceChecker.isSet(o.getEquityCashSettlementDates()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EquitySwapMasterConfirmation2018", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EquitySwapMasterConfirmation2018", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<EquitySwapMasterConfirmation2018> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder o = (EquitySwapMasterConfirmation2018.EquitySwapMasterConfirmation2018Builder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("typeOfSwapElection", ExistenceChecker.isSet(o.getTypeOfSwapElection()))
				.put("pricingMethodElection", ExistenceChecker.isSet(o.getPricingMethodElection()))
				.put("linearInterpolationElection", ExistenceChecker.isSet(o.getLinearInterpolationElection()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.put("equityCalculationPeriod", ExistenceChecker.isSet(o.getEquityCalculationPeriod()))
				.put("equityCashSettlementDates", ExistenceChecker.isSet(o.getEquityCashSettlementDates()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("EquitySwapMasterConfirmation2018", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("EquitySwapMasterConfirmation2018", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
