package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Event;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class EventOnlyExistsValidator implements ValidatorWithArg<Event, String> {

	@Override
	public ValidationResult<Event> validate(RosettaPath path, Event o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("messageInformation", ExistenceChecker.isSet(o.getMessageInformation()))
				.put("timestamp", ExistenceChecker.isSet(o.getTimestamp()))
				.put("eventIdentifier", ExistenceChecker.isSet(o.getEventIdentifier()))
				.put("eventQualifier", ExistenceChecker.isSet(o.getEventQualifier()))
				.put("eventDate", ExistenceChecker.isSet(o.getEventDate()))
				.put("effectiveDate", ExistenceChecker.isSet(o.getEffectiveDate()))
				.put("action", ExistenceChecker.isSet(o.getAction()))
				.put("intent", ExistenceChecker.isSet(o.getIntent()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("account", ExistenceChecker.isSet(o.getAccount()))
				.put("lineage", ExistenceChecker.isSet(o.getLineage()))
				.put("primitive", ExistenceChecker.isSet(o.getPrimitive()))
				.put("functionCall", ExistenceChecker.isSet(o.getFunctionCall()))
				.put("eventEffect", ExistenceChecker.isSet(o.getEventEffect()))
				.put("eventWorkflow", ExistenceChecker.isSet(o.getEventWorkflow()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Event", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Event", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Event> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Event.EventBuilder o = (Event.EventBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("messageInformation", ExistenceChecker.isSet(o.getMessageInformation()))
				.put("timestamp", ExistenceChecker.isSet(o.getTimestamp()))
				.put("eventIdentifier", ExistenceChecker.isSet(o.getEventIdentifier()))
				.put("eventQualifier", ExistenceChecker.isSet(o.getEventQualifier()))
				.put("eventDate", ExistenceChecker.isSet(o.getEventDate()))
				.put("effectiveDate", ExistenceChecker.isSet(o.getEffectiveDate()))
				.put("action", ExistenceChecker.isSet(o.getAction()))
				.put("intent", ExistenceChecker.isSet(o.getIntent()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("account", ExistenceChecker.isSet(o.getAccount()))
				.put("lineage", ExistenceChecker.isSet(o.getLineage()))
				.put("primitive", ExistenceChecker.isSet(o.getPrimitive()))
				.put("functionCall", ExistenceChecker.isSet(o.getFunctionCall()))
				.put("eventEffect", ExistenceChecker.isSet(o.getEventEffect()))
				.put("eventWorkflow", ExistenceChecker.isSet(o.getEventWorkflow()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Event", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Event", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
