package org.isda.cdm.validation.exists;

import com.google.common.collect.ImmutableMap;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import org.isda.cdm.ExchangeRate;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ExchangeRateOnlyExistsValidator implements ValidatorWithArg<ExchangeRate, String> {

	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, ExchangeRate o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("quotedCurrencyPair", ExistenceChecker.isSet(o.getQuotedCurrencyPair()))
				.put("rate", ExistenceChecker.isSet(o.getRate()))
				.put("spotRate", ExistenceChecker.isSet(o.getSpotRate()))
				.put("forwardPoints", ExistenceChecker.isSet(o.getForwardPoints()))
				.put("pointValue", ExistenceChecker.isSet(o.getPointValue()))
				.put("crossRate", ExistenceChecker.isSet(o.getCrossRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExchangeRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExchangeRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExchangeRate> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ExchangeRate.ExchangeRateBuilder o = (ExchangeRate.ExchangeRateBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("quotedCurrencyPair", ExistenceChecker.isSet(o.getQuotedCurrencyPair()))
				.put("rate", ExistenceChecker.isSet(o.getRate()))
				.put("spotRate", ExistenceChecker.isSet(o.getSpotRate()))
				.put("forwardPoints", ExistenceChecker.isSet(o.getForwardPoints()))
				.put("pointValue", ExistenceChecker.isSet(o.getPointValue()))
				.put("crossRate", ExistenceChecker.isSet(o.getCrossRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExchangeRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExchangeRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
