package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Execution;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ExecutionOnlyExistsValidator implements ValidatorWithArg<Execution, String> {

	@Override
	public ValidationResult<Execution> validate(RosettaPath path, Execution o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("executionType", ExistenceChecker.isSet(o.getExecutionType()))
				.put("executionVenue", ExistenceChecker.isSet(o.getExecutionVenue()))
				.put("identifier", ExistenceChecker.isSet(o.getIdentifier()))
				.put("tradeDate", ExistenceChecker.isSet(o.getTradeDate()))
				.put("product", ExistenceChecker.isSet(o.getProduct()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("partyRole", ExistenceChecker.isSet(o.getPartyRole()))
				.put("closedState", ExistenceChecker.isSet(o.getClosedState()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("executionQuantity", ExistenceChecker.isSet(o.getExecutionQuantity()))
				.put("price", ExistenceChecker.isSet(o.getPrice()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Execution", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Execution", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Execution> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Execution.ExecutionBuilder o = (Execution.ExecutionBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("executionType", ExistenceChecker.isSet(o.getExecutionType()))
				.put("executionVenue", ExistenceChecker.isSet(o.getExecutionVenue()))
				.put("identifier", ExistenceChecker.isSet(o.getIdentifier()))
				.put("tradeDate", ExistenceChecker.isSet(o.getTradeDate()))
				.put("product", ExistenceChecker.isSet(o.getProduct()))
				.put("party", ExistenceChecker.isSet(o.getParty()))
				.put("partyRole", ExistenceChecker.isSet(o.getPartyRole()))
				.put("closedState", ExistenceChecker.isSet(o.getClosedState()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("executionQuantity", ExistenceChecker.isSet(o.getExecutionQuantity()))
				.put("price", ExistenceChecker.isSet(o.getPrice()))
				.put("settlementTerms", ExistenceChecker.isSet(o.getSettlementTerms()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Execution", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Execution", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
