package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ExtendibleProvision;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ExtendibleProvisionOnlyExistsValidator implements ValidatorWithArg<ExtendibleProvision, String> {

	@Override
	public ValidationResult<ExtendibleProvision> validate(RosettaPath path, ExtendibleProvision o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("americanExercise", ExistenceChecker.isSet(o.getAmericanExercise()))
				.put("bermudaExercise", ExistenceChecker.isSet(o.getBermudaExercise()))
				.put("europeanExercise", ExistenceChecker.isSet(o.getEuropeanExercise()))
				.put("exerciseNotice", ExistenceChecker.isSet(o.getExerciseNotice()))
				.put("followUpConfirmation", ExistenceChecker.isSet(o.getFollowUpConfirmation()))
				.put("extendibleProvisionAdjustedDates", ExistenceChecker.isSet(o.getExtendibleProvisionAdjustedDates()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExtendibleProvision", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExtendibleProvision", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExtendibleProvision> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ExtendibleProvision.ExtendibleProvisionBuilder o = (ExtendibleProvision.ExtendibleProvisionBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("americanExercise", ExistenceChecker.isSet(o.getAmericanExercise()))
				.put("bermudaExercise", ExistenceChecker.isSet(o.getBermudaExercise()))
				.put("europeanExercise", ExistenceChecker.isSet(o.getEuropeanExercise()))
				.put("exerciseNotice", ExistenceChecker.isSet(o.getExerciseNotice()))
				.put("followUpConfirmation", ExistenceChecker.isSet(o.getFollowUpConfirmation()))
				.put("extendibleProvisionAdjustedDates", ExistenceChecker.isSet(o.getExtendibleProvisionAdjustedDates()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExtendibleProvision", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExtendibleProvision", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
