package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ExtraordinaryEvents;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ExtraordinaryEventsOnlyExistsValidator implements ValidatorWithArg<ExtraordinaryEvents, String> {

	@Override
	public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, ExtraordinaryEvents o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("mergerEvents", ExistenceChecker.isSet(o.getMergerEvents()))
				.put("tenderOfferEvents", ExistenceChecker.isSet(o.getTenderOfferEvents()))
				.put("compositionOfCombinedConsideration", ExistenceChecker.isSet(o.getCompositionOfCombinedConsideration()))
				.put("indexAdjustmentEvents", ExistenceChecker.isSet(o.getIndexAdjustmentEvents()))
				.put("additionalDisruptionEvents", ExistenceChecker.isSet(o.getAdditionalDisruptionEvents()))
				.put("failureToDeliver", ExistenceChecker.isSet(o.getFailureToDeliver()))
				.put("representations", ExistenceChecker.isSet(o.getRepresentations()))
				.put("nationalizationOrInsolvency", ExistenceChecker.isSet(o.getNationalizationOrInsolvency()))
				.put("delisting", ExistenceChecker.isSet(o.getDelisting()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExtraordinaryEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExtraordinaryEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ExtraordinaryEvents> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ExtraordinaryEvents.ExtraordinaryEventsBuilder o = (ExtraordinaryEvents.ExtraordinaryEventsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("mergerEvents", ExistenceChecker.isSet(o.getMergerEvents()))
				.put("tenderOfferEvents", ExistenceChecker.isSet(o.getTenderOfferEvents()))
				.put("compositionOfCombinedConsideration", ExistenceChecker.isSet(o.getCompositionOfCombinedConsideration()))
				.put("indexAdjustmentEvents", ExistenceChecker.isSet(o.getIndexAdjustmentEvents()))
				.put("additionalDisruptionEvents", ExistenceChecker.isSet(o.getAdditionalDisruptionEvents()))
				.put("failureToDeliver", ExistenceChecker.isSet(o.getFailureToDeliver()))
				.put("representations", ExistenceChecker.isSet(o.getRepresentations()))
				.put("nationalizationOrInsolvency", ExistenceChecker.isSet(o.getNationalizationOrInsolvency()))
				.put("delisting", ExistenceChecker.isSet(o.getDelisting()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ExtraordinaryEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ExtraordinaryEvents", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
