package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.FallbackReferencePrice;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class FallbackReferencePriceOnlyExistsValidator implements ValidatorWithArg<FallbackReferencePrice, String> {

	@Override
	public ValidationResult<FallbackReferencePrice> validate(RosettaPath path, FallbackReferencePrice o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("valuationPostponement", ExistenceChecker.isSet(o.getValuationPostponement()))
				.put("fallBackSettlementRateOption", ExistenceChecker.isSet(o.getFallBackSettlementRateOption()))
				.put("fallbackSurveyValuationPostponement", ExistenceChecker.isSet(o.getFallbackSurveyValuationPostponement()))
				.put("calculationAgentDetermination", ExistenceChecker.isSet(o.getCalculationAgentDetermination()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FallbackReferencePrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FallbackReferencePrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FallbackReferencePrice> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		FallbackReferencePrice.FallbackReferencePriceBuilder o = (FallbackReferencePrice.FallbackReferencePriceBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("valuationPostponement", ExistenceChecker.isSet(o.getValuationPostponement()))
				.put("fallBackSettlementRateOption", ExistenceChecker.isSet(o.getFallBackSettlementRateOption()))
				.put("fallbackSurveyValuationPostponement", ExistenceChecker.isSet(o.getFallbackSurveyValuationPostponement()))
				.put("calculationAgentDetermination", ExistenceChecker.isSet(o.getCalculationAgentDetermination()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FallbackReferencePrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FallbackReferencePrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
