package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.FeaturePayment;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class FeaturePaymentOnlyExistsValidator implements ValidatorWithArg<FeaturePayment, String> {

	@Override
	public ValidationResult<FeaturePayment> validate(RosettaPath path, FeaturePayment o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("levelPercentage", ExistenceChecker.isSet(o.getLevelPercentage()))
				.put("amount", ExistenceChecker.isSet(o.getAmount()))
				.put("time", ExistenceChecker.isSet(o.getTime()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("paymentDate", ExistenceChecker.isSet(o.getPaymentDate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FeaturePayment", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FeaturePayment", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FeaturePayment> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		FeaturePayment.FeaturePaymentBuilder o = (FeaturePayment.FeaturePaymentBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("levelPercentage", ExistenceChecker.isSet(o.getLevelPercentage()))
				.put("amount", ExistenceChecker.isSet(o.getAmount()))
				.put("time", ExistenceChecker.isSet(o.getTime()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("paymentDate", ExistenceChecker.isSet(o.getPaymentDate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FeaturePayment", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FeaturePayment", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
