package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.FloatingRateDefinition;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class FloatingRateDefinitionOnlyExistsValidator implements ValidatorWithArg<FloatingRateDefinition, String> {

	@Override
	public ValidationResult<FloatingRateDefinition> validate(RosettaPath path, FloatingRateDefinition o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculatedRate", ExistenceChecker.isSet(o.getCalculatedRate()))
				.put("rateObservation", ExistenceChecker.isSet(o.getRateObservation()))
				.put("floatingRateMultiplier", ExistenceChecker.isSet(o.getFloatingRateMultiplier()))
				.put("spread", ExistenceChecker.isSet(o.getSpread()))
				.put("capRate", ExistenceChecker.isSet(o.getCapRate()))
				.put("floorRate", ExistenceChecker.isSet(o.getFloorRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FloatingRateDefinition", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FloatingRateDefinition", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FloatingRateDefinition> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		FloatingRateDefinition.FloatingRateDefinitionBuilder o = (FloatingRateDefinition.FloatingRateDefinitionBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculatedRate", ExistenceChecker.isSet(o.getCalculatedRate()))
				.put("rateObservation", ExistenceChecker.isSet(o.getRateObservation()))
				.put("floatingRateMultiplier", ExistenceChecker.isSet(o.getFloatingRateMultiplier()))
				.put("spread", ExistenceChecker.isSet(o.getSpread()))
				.put("capRate", ExistenceChecker.isSet(o.getCapRate()))
				.put("floorRate", ExistenceChecker.isSet(o.getFloorRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FloatingRateDefinition", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FloatingRateDefinition", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
