package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.FloatingRate;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class FloatingRateOnlyExistsValidator implements ValidatorWithArg<FloatingRate, String> {

	@Override
	public ValidationResult<FloatingRate> validate(RosettaPath path, FloatingRate o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("floatingRateIndex", ExistenceChecker.isSet(o.getFloatingRateIndex()))
				.put("indexTenor", ExistenceChecker.isSet(o.getIndexTenor()))
				.put("floatingRateMultiplierSchedule", ExistenceChecker.isSet(o.getFloatingRateMultiplierSchedule()))
				.put("spreadSchedule", ExistenceChecker.isSet(o.getSpreadSchedule()))
				.put("rateTreatment", ExistenceChecker.isSet(o.getRateTreatment()))
				.put("capRateSchedule", ExistenceChecker.isSet(o.getCapRateSchedule()))
				.put("floorRateSchedule", ExistenceChecker.isSet(o.getFloorRateSchedule()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FloatingRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FloatingRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<FloatingRate> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		FloatingRate.FloatingRateBuilder o = (FloatingRate.FloatingRateBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("floatingRateIndex", ExistenceChecker.isSet(o.getFloatingRateIndex()))
				.put("indexTenor", ExistenceChecker.isSet(o.getIndexTenor()))
				.put("floatingRateMultiplierSchedule", ExistenceChecker.isSet(o.getFloatingRateMultiplierSchedule()))
				.put("spreadSchedule", ExistenceChecker.isSet(o.getSpreadSchedule()))
				.put("rateTreatment", ExistenceChecker.isSet(o.getRateTreatment()))
				.put("capRateSchedule", ExistenceChecker.isSet(o.getCapRateSchedule()))
				.put("floorRateSchedule", ExistenceChecker.isSet(o.getFloorRateSchedule()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("FloatingRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("FloatingRate", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
