package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.IndexReferenceInformation;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class IndexReferenceInformationOnlyExistsValidator implements ValidatorWithArg<IndexReferenceInformation, String> {

	@Override
	public ValidationResult<IndexReferenceInformation> validate(RosettaPath path, IndexReferenceInformation o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("indexName", ExistenceChecker.isSet(o.getIndexName()))
				.put("indexId", ExistenceChecker.isSet(o.getIndexId()))
				.put("indexSeries", ExistenceChecker.isSet(o.getIndexSeries()))
				.put("indexAnnexVersion", ExistenceChecker.isSet(o.getIndexAnnexVersion()))
				.put("indexAnnexDate", ExistenceChecker.isSet(o.getIndexAnnexDate()))
				.put("indexAnnexSource", ExistenceChecker.isSet(o.getIndexAnnexSource()))
				.put("excludedReferenceEntity", ExistenceChecker.isSet(o.getExcludedReferenceEntity()))
				.put("tranche", ExistenceChecker.isSet(o.getTranche()))
				.put("settledEntityMatrix", ExistenceChecker.isSet(o.getSettledEntityMatrix()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("IndexReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("IndexReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<IndexReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		IndexReferenceInformation.IndexReferenceInformationBuilder o = (IndexReferenceInformation.IndexReferenceInformationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("indexName", ExistenceChecker.isSet(o.getIndexName()))
				.put("indexId", ExistenceChecker.isSet(o.getIndexId()))
				.put("indexSeries", ExistenceChecker.isSet(o.getIndexSeries()))
				.put("indexAnnexVersion", ExistenceChecker.isSet(o.getIndexAnnexVersion()))
				.put("indexAnnexDate", ExistenceChecker.isSet(o.getIndexAnnexDate()))
				.put("indexAnnexSource", ExistenceChecker.isSet(o.getIndexAnnexSource()))
				.put("excludedReferenceEntity", ExistenceChecker.isSet(o.getExcludedReferenceEntity()))
				.put("tranche", ExistenceChecker.isSet(o.getTranche()))
				.put("settledEntityMatrix", ExistenceChecker.isSet(o.getSettledEntityMatrix()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("IndexReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("IndexReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
