package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.InflationRateSpecification;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class InflationRateSpecificationOnlyExistsValidator implements ValidatorWithArg<InflationRateSpecification, String> {

	@Override
	public ValidationResult<InflationRateSpecification> validate(RosettaPath path, InflationRateSpecification o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("inflationLag", ExistenceChecker.isSet(o.getInflationLag()))
				.put("indexSource", ExistenceChecker.isSet(o.getIndexSource()))
				.put("mainPublication", ExistenceChecker.isSet(o.getMainPublication()))
				.put("interpolationMethod", ExistenceChecker.isSet(o.getInterpolationMethod()))
				.put("initialIndexLevel", ExistenceChecker.isSet(o.getInitialIndexLevel()))
				.put("fallbackBondApplicable", ExistenceChecker.isSet(o.getFallbackBondApplicable()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InflationRateSpecification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InflationRateSpecification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<InflationRateSpecification> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		InflationRateSpecification.InflationRateSpecificationBuilder o = (InflationRateSpecification.InflationRateSpecificationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("inflationLag", ExistenceChecker.isSet(o.getInflationLag()))
				.put("indexSource", ExistenceChecker.isSet(o.getIndexSource()))
				.put("mainPublication", ExistenceChecker.isSet(o.getMainPublication()))
				.put("interpolationMethod", ExistenceChecker.isSet(o.getInterpolationMethod()))
				.put("initialIndexLevel", ExistenceChecker.isSet(o.getInitialIndexLevel()))
				.put("fallbackBondApplicable", ExistenceChecker.isSet(o.getFallbackBondApplicable()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InflationRateSpecification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InflationRateSpecification", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
