package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.InitialMargin;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class InitialMarginOnlyExistsValidator implements ValidatorWithArg<InitialMargin, String> {

	@Override
	public ValidationResult<InitialMargin> validate(RosettaPath path, InitialMargin o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("marginType", ExistenceChecker.isSet(o.getMarginType()))
				.put("margin", ExistenceChecker.isSet(o.getMargin()))
				.put("marginThreshold", ExistenceChecker.isSet(o.getMarginThreshold()))
				.put("minimumTransferAmount", ExistenceChecker.isSet(o.getMinimumTransferAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InitialMargin", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InitialMargin", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<InitialMargin> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		InitialMargin.InitialMarginBuilder o = (InitialMargin.InitialMarginBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("marginType", ExistenceChecker.isSet(o.getMarginType()))
				.put("margin", ExistenceChecker.isSet(o.getMargin()))
				.put("marginThreshold", ExistenceChecker.isSet(o.getMarginThreshold()))
				.put("minimumTransferAmount", ExistenceChecker.isSet(o.getMinimumTransferAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InitialMargin", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InitialMargin", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
