package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.InterestRatePayout;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class InterestRatePayoutOnlyExistsValidator implements ValidatorWithArg<InterestRatePayout, String> {

	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, InterestRatePayout o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("rateSpecification", ExistenceChecker.isSet(o.getRateSpecification()))
				.put("dayCountFraction", ExistenceChecker.isSet(o.getDayCountFraction()))
				.put("calculationPeriodDates", ExistenceChecker.isSet(o.getCalculationPeriodDates()))
				.put("paymentDates", ExistenceChecker.isSet(o.getPaymentDates()))
				.put("paymentDate", ExistenceChecker.isSet(o.getPaymentDate()))
				.put("paymentDelay", ExistenceChecker.isSet(o.getPaymentDelay()))
				.put("resetDates", ExistenceChecker.isSet(o.getResetDates()))
				.put("discountingMethod", ExistenceChecker.isSet(o.getDiscountingMethod()))
				.put("compoundingMethod", ExistenceChecker.isSet(o.getCompoundingMethod()))
				.put("cashflowRepresentation", ExistenceChecker.isSet(o.getCashflowRepresentation()))
				.put("crossCurrencyTerms", ExistenceChecker.isSet(o.getCrossCurrencyTerms()))
				.put("stubPeriod", ExistenceChecker.isSet(o.getStubPeriod()))
				.put("bondReference", ExistenceChecker.isSet(o.getBondReference()))
				.put("fixedAmount", ExistenceChecker.isSet(o.getFixedAmount()))
				.put("floatingAmount", ExistenceChecker.isSet(o.getFloatingAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InterestRatePayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InterestRatePayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<InterestRatePayout> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		InterestRatePayout.InterestRatePayoutBuilder o = (InterestRatePayout.InterestRatePayoutBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("payerReceiver", ExistenceChecker.isSet(o.getPayerReceiver()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("rateSpecification", ExistenceChecker.isSet(o.getRateSpecification()))
				.put("dayCountFraction", ExistenceChecker.isSet(o.getDayCountFraction()))
				.put("calculationPeriodDates", ExistenceChecker.isSet(o.getCalculationPeriodDates()))
				.put("paymentDates", ExistenceChecker.isSet(o.getPaymentDates()))
				.put("paymentDate", ExistenceChecker.isSet(o.getPaymentDate()))
				.put("paymentDelay", ExistenceChecker.isSet(o.getPaymentDelay()))
				.put("resetDates", ExistenceChecker.isSet(o.getResetDates()))
				.put("discountingMethod", ExistenceChecker.isSet(o.getDiscountingMethod()))
				.put("compoundingMethod", ExistenceChecker.isSet(o.getCompoundingMethod()))
				.put("cashflowRepresentation", ExistenceChecker.isSet(o.getCashflowRepresentation()))
				.put("crossCurrencyTerms", ExistenceChecker.isSet(o.getCrossCurrencyTerms()))
				.put("stubPeriod", ExistenceChecker.isSet(o.getStubPeriod()))
				.put("bondReference", ExistenceChecker.isSet(o.getBondReference()))
				.put("fixedAmount", ExistenceChecker.isSet(o.getFixedAmount()))
				.put("floatingAmount", ExistenceChecker.isSet(o.getFloatingAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("InterestRatePayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("InterestRatePayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
