package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.LegalAgreementBase;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class LegalAgreementBaseOnlyExistsValidator implements ValidatorWithArg<LegalAgreementBase, String> {

	@Override
	public ValidationResult<LegalAgreementBase> validate(RosettaPath path, LegalAgreementBase o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("agreementDate", ExistenceChecker.isSet(o.getAgreementDate()))
				.put("effectiveDate", ExistenceChecker.isSet(o.getEffectiveDate()))
				.put("identifier", ExistenceChecker.isSet(o.getIdentifier()))
				.put("lineage", ExistenceChecker.isSet(o.getLineage()))
				.put("agreementType", ExistenceChecker.isSet(o.getAgreementType()))
				.put("contractualParty", ExistenceChecker.isSet(o.getContractualParty()))
				.put("otherParty", ExistenceChecker.isSet(o.getOtherParty()))
				.put("umbrellaAgreement", ExistenceChecker.isSet(o.getUmbrellaAgreement()))
				.put("partyInformation", ExistenceChecker.isSet(o.getPartyInformation()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LegalAgreementBase", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LegalAgreementBase", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LegalAgreementBase> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		LegalAgreementBase.LegalAgreementBaseBuilder o = (LegalAgreementBase.LegalAgreementBaseBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("agreementDate", ExistenceChecker.isSet(o.getAgreementDate()))
				.put("effectiveDate", ExistenceChecker.isSet(o.getEffectiveDate()))
				.put("identifier", ExistenceChecker.isSet(o.getIdentifier()))
				.put("lineage", ExistenceChecker.isSet(o.getLineage()))
				.put("agreementType", ExistenceChecker.isSet(o.getAgreementType()))
				.put("contractualParty", ExistenceChecker.isSet(o.getContractualParty()))
				.put("otherParty", ExistenceChecker.isSet(o.getOtherParty()))
				.put("umbrellaAgreement", ExistenceChecker.isSet(o.getUmbrellaAgreement()))
				.put("partyInformation", ExistenceChecker.isSet(o.getPartyInformation()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LegalAgreementBase", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LegalAgreementBase", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
