package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.LegalAgreement;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class LegalAgreementOnlyExistsValidator implements ValidatorWithArg<LegalAgreement, String> {

	@Override
	public ValidationResult<LegalAgreement> validate(RosettaPath path, LegalAgreement o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("csdInitialMargin2016EnglishLaw", ExistenceChecker.isSet(o.getCsdInitialMargin2016EnglishLaw()))
				.put("csaInitialMargin2016JapaneseLaw", ExistenceChecker.isSet(o.getCsaInitialMargin2016JapaneseLaw()))
				.put("csaInitialMargin2016NewYorkLaw", ExistenceChecker.isSet(o.getCsaInitialMargin2016NewYorkLaw()))
				.put("csaVariationMargin2016NewYorkLaw", ExistenceChecker.isSet(o.getCsaVariationMargin2016NewYorkLaw()))
				.put("equitySwapMasterConfirmation2018", ExistenceChecker.isSet(o.getEquitySwapMasterConfirmation2018()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LegalAgreement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LegalAgreement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LegalAgreement> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		LegalAgreement.LegalAgreementBuilder o = (LegalAgreement.LegalAgreementBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("csdInitialMargin2016EnglishLaw", ExistenceChecker.isSet(o.getCsdInitialMargin2016EnglishLaw()))
				.put("csaInitialMargin2016JapaneseLaw", ExistenceChecker.isSet(o.getCsaInitialMargin2016JapaneseLaw()))
				.put("csaInitialMargin2016NewYorkLaw", ExistenceChecker.isSet(o.getCsaInitialMargin2016NewYorkLaw()))
				.put("csaVariationMargin2016NewYorkLaw", ExistenceChecker.isSet(o.getCsaVariationMargin2016NewYorkLaw()))
				.put("equitySwapMasterConfirmation2018", ExistenceChecker.isSet(o.getEquitySwapMasterConfirmation2018()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LegalAgreement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LegalAgreement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
