package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.LimitApplicable;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class LimitApplicableOnlyExistsValidator implements ValidatorWithArg<LimitApplicable, String> {

	@Override
	public ValidationResult<LimitApplicable> validate(RosettaPath path, LimitApplicable o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("limitType", ExistenceChecker.isSet(o.getLimitType()))
				.put("clipSize", ExistenceChecker.isSet(o.getClipSize()))
				.put("amountUtilized", ExistenceChecker.isSet(o.getAmountUtilized()))
				.put("utilization", ExistenceChecker.isSet(o.getUtilization()))
				.put("amountRemaining", ExistenceChecker.isSet(o.getAmountRemaining()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("velocity", ExistenceChecker.isSet(o.getVelocity()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LimitApplicable", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LimitApplicable", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<LimitApplicable> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		LimitApplicable.LimitApplicableBuilder o = (LimitApplicable.LimitApplicableBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("limitType", ExistenceChecker.isSet(o.getLimitType()))
				.put("clipSize", ExistenceChecker.isSet(o.getClipSize()))
				.put("amountUtilized", ExistenceChecker.isSet(o.getAmountUtilized()))
				.put("utilization", ExistenceChecker.isSet(o.getUtilization()))
				.put("amountRemaining", ExistenceChecker.isSet(o.getAmountRemaining()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("velocity", ExistenceChecker.isSet(o.getVelocity()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("LimitApplicable", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("LimitApplicable", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
