package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Lineage;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class LineageOnlyExistsValidator implements ValidatorWithArg<Lineage, String> {

	@Override
	public ValidationResult<Lineage> validate(RosettaPath path, Lineage o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("contractReference", ExistenceChecker.isSet(o.getContractReference()))
				.put("eventReference", ExistenceChecker.isSet(o.getEventReference()))
				.put("executionReference", ExistenceChecker.isSet(o.getExecutionReference()))
				.put("cashflowReference", ExistenceChecker.isSet(o.getCashflowReference()))
				.put("creditDefaultPayoutReference", ExistenceChecker.isSet(o.getCreditDefaultPayoutReference()))
				.put("interestRatePayoutReference", ExistenceChecker.isSet(o.getInterestRatePayoutReference()))
				.put("optionPayoutReference", ExistenceChecker.isSet(o.getOptionPayoutReference()))
				.put("equityPayoutReference", ExistenceChecker.isSet(o.getEquityPayoutReference()))
				.put("transferReference", ExistenceChecker.isSet(o.getTransferReference()))
				.put("legalAgreement", ExistenceChecker.isSet(o.getLegalAgreement()))
				.put("portfolioStateReference", ExistenceChecker.isSet(o.getPortfolioStateReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Lineage", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Lineage", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Lineage> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Lineage.LineageBuilder o = (Lineage.LineageBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("contractReference", ExistenceChecker.isSet(o.getContractReference()))
				.put("eventReference", ExistenceChecker.isSet(o.getEventReference()))
				.put("executionReference", ExistenceChecker.isSet(o.getExecutionReference()))
				.put("cashflowReference", ExistenceChecker.isSet(o.getCashflowReference()))
				.put("creditDefaultPayoutReference", ExistenceChecker.isSet(o.getCreditDefaultPayoutReference()))
				.put("interestRatePayoutReference", ExistenceChecker.isSet(o.getInterestRatePayoutReference()))
				.put("optionPayoutReference", ExistenceChecker.isSet(o.getOptionPayoutReference()))
				.put("equityPayoutReference", ExistenceChecker.isSet(o.getEquityPayoutReference()))
				.put("transferReference", ExistenceChecker.isSet(o.getTransferReference()))
				.put("legalAgreement", ExistenceChecker.isSet(o.getLegalAgreement()))
				.put("portfolioStateReference", ExistenceChecker.isSet(o.getPortfolioStateReference()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Lineage", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Lineage", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
