package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.NaturalPerson;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class NaturalPersonOnlyExistsValidator implements ValidatorWithArg<NaturalPerson, String> {

	@Override
	public ValidationResult<NaturalPerson> validate(RosettaPath path, NaturalPerson o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("honorific", ExistenceChecker.isSet(o.getHonorific()))
				.put("firstName", ExistenceChecker.isSet(o.getFirstName()))
				.put("middleName", ExistenceChecker.isSet(o.getMiddleName()))
				.put("initial", ExistenceChecker.isSet(o.getInitial()))
				.put("surname", ExistenceChecker.isSet(o.getSurname()))
				.put("suffix", ExistenceChecker.isSet(o.getSuffix()))
				.put("dateOfBirth", ExistenceChecker.isSet(o.getDateOfBirth()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("NaturalPerson", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("NaturalPerson", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<NaturalPerson> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		NaturalPerson.NaturalPersonBuilder o = (NaturalPerson.NaturalPersonBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("honorific", ExistenceChecker.isSet(o.getHonorific()))
				.put("firstName", ExistenceChecker.isSet(o.getFirstName()))
				.put("middleName", ExistenceChecker.isSet(o.getMiddleName()))
				.put("initial", ExistenceChecker.isSet(o.getInitial()))
				.put("surname", ExistenceChecker.isSet(o.getSurname()))
				.put("suffix", ExistenceChecker.isSet(o.getSuffix()))
				.put("dateOfBirth", ExistenceChecker.isSet(o.getDateOfBirth()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("NaturalPerson", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("NaturalPerson", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
