package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.New;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class NewOnlyExistsValidator implements ValidatorWithArg<New, String> {

	@Override
	public ValidationResult<New> validate(RosettaPath path, New o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("txId", ExistenceChecker.isSet(o.getTxId()))
				.put("exctgPty", ExistenceChecker.isSet(o.getExctgPty()))
				.put("invstmtPtyInd", ExistenceChecker.isSet(o.getInvstmtPtyInd()))
				.put("submitgPty", ExistenceChecker.isSet(o.getSubmitgPty()))
				.put("buyr", ExistenceChecker.isSet(o.getBuyr()))
				.put("sellr", ExistenceChecker.isSet(o.getSellr()))
				.put("ordrTrnsmssn", ExistenceChecker.isSet(o.getOrdrTrnsmssn()))
				.put("tx", ExistenceChecker.isSet(o.getTx()))
				.put("finInstrm", ExistenceChecker.isSet(o.getFinInstrm()))
				.put("invstmtDcsnPrsn", ExistenceChecker.isSet(o.getInvstmtDcsnPrsn()))
				.put("exctgPrsn", ExistenceChecker.isSet(o.getExctgPrsn()))
				.put("addtlAttrbts", ExistenceChecker.isSet(o.getAddtlAttrbts()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("New", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("New", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<New> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		New.NewBuilder o = (New.NewBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("txId", ExistenceChecker.isSet(o.getTxId()))
				.put("exctgPty", ExistenceChecker.isSet(o.getExctgPty()))
				.put("invstmtPtyInd", ExistenceChecker.isSet(o.getInvstmtPtyInd()))
				.put("submitgPty", ExistenceChecker.isSet(o.getSubmitgPty()))
				.put("buyr", ExistenceChecker.isSet(o.getBuyr()))
				.put("sellr", ExistenceChecker.isSet(o.getSellr()))
				.put("ordrTrnsmssn", ExistenceChecker.isSet(o.getOrdrTrnsmssn()))
				.put("tx", ExistenceChecker.isSet(o.getTx()))
				.put("finInstrm", ExistenceChecker.isSet(o.getFinInstrm()))
				.put("invstmtDcsnPrsn", ExistenceChecker.isSet(o.getInvstmtDcsnPrsn()))
				.put("exctgPrsn", ExistenceChecker.isSet(o.getExctgPrsn()))
				.put("addtlAttrbts", ExistenceChecker.isSet(o.getAddtlAttrbts()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("New", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("New", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
