package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.NotionalStepRule;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class NotionalStepRuleOnlyExistsValidator implements ValidatorWithArg<NotionalStepRule, String> {

	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, NotionalStepRule o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculationPeriodDatesReference", ExistenceChecker.isSet(o.getCalculationPeriodDatesReference()))
				.put("stepFrequency", ExistenceChecker.isSet(o.getStepFrequency()))
				.put("firstNotionalStepDate", ExistenceChecker.isSet(o.getFirstNotionalStepDate()))
				.put("lastNotionalStepDate", ExistenceChecker.isSet(o.getLastNotionalStepDate()))
				.put("notionalStepAmount", ExistenceChecker.isSet(o.getNotionalStepAmount()))
				.put("notionalStepRate", ExistenceChecker.isSet(o.getNotionalStepRate()))
				.put("stepRelativeTo", ExistenceChecker.isSet(o.getStepRelativeTo()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("NotionalStepRule", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("NotionalStepRule", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<NotionalStepRule> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		NotionalStepRule.NotionalStepRuleBuilder o = (NotionalStepRule.NotionalStepRuleBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculationPeriodDatesReference", ExistenceChecker.isSet(o.getCalculationPeriodDatesReference()))
				.put("stepFrequency", ExistenceChecker.isSet(o.getStepFrequency()))
				.put("firstNotionalStepDate", ExistenceChecker.isSet(o.getFirstNotionalStepDate()))
				.put("lastNotionalStepDate", ExistenceChecker.isSet(o.getLastNotionalStepDate()))
				.put("notionalStepAmount", ExistenceChecker.isSet(o.getNotionalStepAmount()))
				.put("notionalStepRate", ExistenceChecker.isSet(o.getNotionalStepRate()))
				.put("stepRelativeTo", ExistenceChecker.isSet(o.getStepRelativeTo()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("NotionalStepRule", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("NotionalStepRule", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
