package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Obligations;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ObligationsOnlyExistsValidator implements ValidatorWithArg<Obligations, String> {

	@Override
	public ValidationResult<Obligations> validate(RosettaPath path, Obligations o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("category", ExistenceChecker.isSet(o.getCategory()))
				.put("notSubordinated", ExistenceChecker.isSet(o.getNotSubordinated()))
				.put("specifiedCurrency", ExistenceChecker.isSet(o.getSpecifiedCurrency()))
				.put("notSovereignLender", ExistenceChecker.isSet(o.getNotSovereignLender()))
				.put("notDomesticCurrency", ExistenceChecker.isSet(o.getNotDomesticCurrency()))
				.put("notDomesticLaw", ExistenceChecker.isSet(o.getNotDomesticLaw()))
				.put("listed", ExistenceChecker.isSet(o.getListed()))
				.put("notDomesticIssuance", ExistenceChecker.isSet(o.getNotDomesticIssuance()))
				.put("fullFaithAndCreditObLiability", ExistenceChecker.isSet(o.getFullFaithAndCreditObLiability()))
				.put("generalFundObligationLiability", ExistenceChecker.isSet(o.getGeneralFundObligationLiability()))
				.put("revenueObligationLiability", ExistenceChecker.isSet(o.getRevenueObligationLiability()))
				.put("notContingent", ExistenceChecker.isSet(o.getNotContingent()))
				.put("excluded", ExistenceChecker.isSet(o.getExcluded()))
				.put("othReferenceEntityObligations", ExistenceChecker.isSet(o.getOthReferenceEntityObligations()))
				.put("designatedPriority", ExistenceChecker.isSet(o.getDesignatedPriority()))
				.put("cashSettlementOnly", ExistenceChecker.isSet(o.getCashSettlementOnly()))
				.put("deliveryOfCommitments", ExistenceChecker.isSet(o.getDeliveryOfCommitments()))
				.put("continuity", ExistenceChecker.isSet(o.getContinuity()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Obligations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Obligations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Obligations> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Obligations.ObligationsBuilder o = (Obligations.ObligationsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("category", ExistenceChecker.isSet(o.getCategory()))
				.put("notSubordinated", ExistenceChecker.isSet(o.getNotSubordinated()))
				.put("specifiedCurrency", ExistenceChecker.isSet(o.getSpecifiedCurrency()))
				.put("notSovereignLender", ExistenceChecker.isSet(o.getNotSovereignLender()))
				.put("notDomesticCurrency", ExistenceChecker.isSet(o.getNotDomesticCurrency()))
				.put("notDomesticLaw", ExistenceChecker.isSet(o.getNotDomesticLaw()))
				.put("listed", ExistenceChecker.isSet(o.getListed()))
				.put("notDomesticIssuance", ExistenceChecker.isSet(o.getNotDomesticIssuance()))
				.put("fullFaithAndCreditObLiability", ExistenceChecker.isSet(o.getFullFaithAndCreditObLiability()))
				.put("generalFundObligationLiability", ExistenceChecker.isSet(o.getGeneralFundObligationLiability()))
				.put("revenueObligationLiability", ExistenceChecker.isSet(o.getRevenueObligationLiability()))
				.put("notContingent", ExistenceChecker.isSet(o.getNotContingent()))
				.put("excluded", ExistenceChecker.isSet(o.getExcluded()))
				.put("othReferenceEntityObligations", ExistenceChecker.isSet(o.getOthReferenceEntityObligations()))
				.put("designatedPriority", ExistenceChecker.isSet(o.getDesignatedPriority()))
				.put("cashSettlementOnly", ExistenceChecker.isSet(o.getCashSettlementOnly()))
				.put("deliveryOfCommitments", ExistenceChecker.isSet(o.getDeliveryOfCommitments()))
				.put("continuity", ExistenceChecker.isSet(o.getContinuity()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Obligations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Obligations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
