package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ObservationPrimitive;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ObservationPrimitiveOnlyExistsValidator implements ValidatorWithArg<ObservationPrimitive, String> {

	@Override
	public ValidationResult<ObservationPrimitive> validate(RosettaPath path, ObservationPrimitive o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("source", ExistenceChecker.isSet(o.getSource()))
				.put("observation", ExistenceChecker.isSet(o.getObservation()))
				.put("date", ExistenceChecker.isSet(o.getDate()))
				.put("time", ExistenceChecker.isSet(o.getTime()))
				.put("side", ExistenceChecker.isSet(o.getSide()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ObservationPrimitive", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ObservationPrimitive", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ObservationPrimitive> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ObservationPrimitive.ObservationPrimitiveBuilder o = (ObservationPrimitive.ObservationPrimitiveBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("source", ExistenceChecker.isSet(o.getSource()))
				.put("observation", ExistenceChecker.isSet(o.getObservation()))
				.put("date", ExistenceChecker.isSet(o.getDate()))
				.put("time", ExistenceChecker.isSet(o.getTime()))
				.put("side", ExistenceChecker.isSet(o.getSide()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ObservationPrimitive", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ObservationPrimitive", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
