package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.OptionCashSettlement;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class OptionCashSettlementOnlyExistsValidator implements ValidatorWithArg<OptionCashSettlement, String> {

	@Override
	public ValidationResult<OptionCashSettlement> validate(RosettaPath path, OptionCashSettlement o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("cashSettlementValuationTime", ExistenceChecker.isSet(o.getCashSettlementValuationTime()))
				.put("cashSettlementValuationDate", ExistenceChecker.isSet(o.getCashSettlementValuationDate()))
				.put("cashSettlementPaymentDate", ExistenceChecker.isSet(o.getCashSettlementPaymentDate()))
				.put("cashPriceMethod", ExistenceChecker.isSet(o.getCashPriceMethod()))
				.put("cashPriceAlternateMethod", ExistenceChecker.isSet(o.getCashPriceAlternateMethod()))
				.put("parYieldCurveAdjustedMethod", ExistenceChecker.isSet(o.getParYieldCurveAdjustedMethod()))
				.put("zeroCouponYieldAdjustedMethod", ExistenceChecker.isSet(o.getZeroCouponYieldAdjustedMethod()))
				.put("parYieldCurveUnadjustedMethod", ExistenceChecker.isSet(o.getParYieldCurveUnadjustedMethod()))
				.put("crossCurrencyMethod", ExistenceChecker.isSet(o.getCrossCurrencyMethod()))
				.put("collateralizedCashPriceMethod", ExistenceChecker.isSet(o.getCollateralizedCashPriceMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionCashSettlement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionCashSettlement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionCashSettlement> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		OptionCashSettlement.OptionCashSettlementBuilder o = (OptionCashSettlement.OptionCashSettlementBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("cashSettlementValuationTime", ExistenceChecker.isSet(o.getCashSettlementValuationTime()))
				.put("cashSettlementValuationDate", ExistenceChecker.isSet(o.getCashSettlementValuationDate()))
				.put("cashSettlementPaymentDate", ExistenceChecker.isSet(o.getCashSettlementPaymentDate()))
				.put("cashPriceMethod", ExistenceChecker.isSet(o.getCashPriceMethod()))
				.put("cashPriceAlternateMethod", ExistenceChecker.isSet(o.getCashPriceAlternateMethod()))
				.put("parYieldCurveAdjustedMethod", ExistenceChecker.isSet(o.getParYieldCurveAdjustedMethod()))
				.put("zeroCouponYieldAdjustedMethod", ExistenceChecker.isSet(o.getZeroCouponYieldAdjustedMethod()))
				.put("parYieldCurveUnadjustedMethod", ExistenceChecker.isSet(o.getParYieldCurveUnadjustedMethod()))
				.put("crossCurrencyMethod", ExistenceChecker.isSet(o.getCrossCurrencyMethod()))
				.put("collateralizedCashPriceMethod", ExistenceChecker.isSet(o.getCollateralizedCashPriceMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionCashSettlement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionCashSettlement", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
