package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.OptionFeature;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class OptionFeatureOnlyExistsValidator implements ValidatorWithArg<OptionFeature, String> {

	@Override
	public ValidationResult<OptionFeature> validate(RosettaPath path, OptionFeature o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("fxFeature", ExistenceChecker.isSet(o.getFxFeature()))
				.put("strategyFeature", ExistenceChecker.isSet(o.getStrategyFeature()))
				.put("asian", ExistenceChecker.isSet(o.getAsian()))
				.put("barrier", ExistenceChecker.isSet(o.getBarrier()))
				.put("knock", ExistenceChecker.isSet(o.getKnock()))
				.put("passThrough", ExistenceChecker.isSet(o.getPassThrough()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionFeature", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionFeature", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionFeature> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		OptionFeature.OptionFeatureBuilder o = (OptionFeature.OptionFeatureBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("fxFeature", ExistenceChecker.isSet(o.getFxFeature()))
				.put("strategyFeature", ExistenceChecker.isSet(o.getStrategyFeature()))
				.put("asian", ExistenceChecker.isSet(o.getAsian()))
				.put("barrier", ExistenceChecker.isSet(o.getBarrier()))
				.put("knock", ExistenceChecker.isSet(o.getKnock()))
				.put("passThrough", ExistenceChecker.isSet(o.getPassThrough()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionFeature", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionFeature", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
