package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.OptionPayout;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class OptionPayoutOnlyExistsValidator implements ValidatorWithArg<OptionPayout, String> {

	@Override
	public ValidationResult<OptionPayout> validate(RosettaPath path, OptionPayout o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("buyerSeller", ExistenceChecker.isSet(o.getBuyerSeller()))
				.put("optionType", ExistenceChecker.isSet(o.getOptionType()))
				.put("feature", ExistenceChecker.isSet(o.getFeature()))
				.put("denomination", ExistenceChecker.isSet(o.getDenomination()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("exerciseTerms", ExistenceChecker.isSet(o.getExerciseTerms()))
				.put("underlier", ExistenceChecker.isSet(o.getUnderlier()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionPayout> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		OptionPayout.OptionPayoutBuilder o = (OptionPayout.OptionPayoutBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("buyerSeller", ExistenceChecker.isSet(o.getBuyerSeller()))
				.put("optionType", ExistenceChecker.isSet(o.getOptionType()))
				.put("feature", ExistenceChecker.isSet(o.getFeature()))
				.put("denomination", ExistenceChecker.isSet(o.getDenomination()))
				.put("quantity", ExistenceChecker.isSet(o.getQuantity()))
				.put("exerciseTerms", ExistenceChecker.isSet(o.getExerciseTerms()))
				.put("underlier", ExistenceChecker.isSet(o.getUnderlier()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionPayout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
