package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.OptionStrike;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class OptionStrikeOnlyExistsValidator implements ValidatorWithArg<OptionStrike, String> {

	@Override
	public ValidationResult<OptionStrike> validate(RosettaPath path, OptionStrike o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("spread", ExistenceChecker.isSet(o.getSpread()))
				.put("price", ExistenceChecker.isSet(o.getPrice()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("percentage", ExistenceChecker.isSet(o.getPercentage()))
				.put("strikeReference", ExistenceChecker.isSet(o.getStrikeReference()))
				.put("referenceSwapCurve", ExistenceChecker.isSet(o.getReferenceSwapCurve()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionStrike", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionStrike", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<OptionStrike> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		OptionStrike.OptionStrikeBuilder o = (OptionStrike.OptionStrikeBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("spread", ExistenceChecker.isSet(o.getSpread()))
				.put("price", ExistenceChecker.isSet(o.getPrice()))
				.put("currency", ExistenceChecker.isSet(o.getCurrency()))
				.put("percentage", ExistenceChecker.isSet(o.getPercentage()))
				.put("strikeReference", ExistenceChecker.isSet(o.getStrikeReference()))
				.put("referenceSwapCurve", ExistenceChecker.isSet(o.getReferenceSwapCurve()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("OptionStrike", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("OptionStrike", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
