package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.PartyContractInformation;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PartyContractInformationOnlyExistsValidator implements ValidatorWithArg<PartyContractInformation, String> {

	@Override
	public ValidationResult<PartyContractInformation> validate(RosettaPath path, PartyContractInformation o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("partyReference", ExistenceChecker.isSet(o.getPartyReference()))
				.put("naturalPersonRole", ExistenceChecker.isSet(o.getNaturalPersonRole()))
				.put("relatedParty", ExistenceChecker.isSet(o.getRelatedParty()))
				.put("accountReference", ExistenceChecker.isSet(o.getAccountReference()))
				.put("category", ExistenceChecker.isSet(o.getCategory()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PartyContractInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PartyContractInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PartyContractInformation> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		PartyContractInformation.PartyContractInformationBuilder o = (PartyContractInformation.PartyContractInformationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("partyReference", ExistenceChecker.isSet(o.getPartyReference()))
				.put("naturalPersonRole", ExistenceChecker.isSet(o.getNaturalPersonRole()))
				.put("relatedParty", ExistenceChecker.isSet(o.getRelatedParty()))
				.put("accountReference", ExistenceChecker.isSet(o.getAccountReference()))
				.put("category", ExistenceChecker.isSet(o.getCategory()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PartyContractInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PartyContractInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
