package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.PaymentCalculationPeriod;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PaymentCalculationPeriodOnlyExistsValidator implements ValidatorWithArg<PaymentCalculationPeriod, String> {

	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, PaymentCalculationPeriod o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedPaymentDate", ExistenceChecker.isSet(o.getUnadjustedPaymentDate()))
				.put("adjustedPaymentDate", ExistenceChecker.isSet(o.getAdjustedPaymentDate()))
				.put("calculationPeriod", ExistenceChecker.isSet(o.getCalculationPeriod()))
				.put("fixedPaymentAmount", ExistenceChecker.isSet(o.getFixedPaymentAmount()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("forecastPaymentAmount", ExistenceChecker.isSet(o.getForecastPaymentAmount()))
				.put("presentValueAmount", ExistenceChecker.isSet(o.getPresentValueAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PaymentCalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PaymentCalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PaymentCalculationPeriod> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		PaymentCalculationPeriod.PaymentCalculationPeriodBuilder o = (PaymentCalculationPeriod.PaymentCalculationPeriodBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedPaymentDate", ExistenceChecker.isSet(o.getUnadjustedPaymentDate()))
				.put("adjustedPaymentDate", ExistenceChecker.isSet(o.getAdjustedPaymentDate()))
				.put("calculationPeriod", ExistenceChecker.isSet(o.getCalculationPeriod()))
				.put("fixedPaymentAmount", ExistenceChecker.isSet(o.getFixedPaymentAmount()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("forecastPaymentAmount", ExistenceChecker.isSet(o.getForecastPaymentAmount()))
				.put("presentValueAmount", ExistenceChecker.isSet(o.getPresentValueAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PaymentCalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PaymentCalculationPeriod", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
