package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.PaymentDates;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PaymentDatesOnlyExistsValidator implements ValidatorWithArg<PaymentDates, String> {

	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, PaymentDates o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dateRelativeTo", ExistenceChecker.isSet(o.getDateRelativeTo()))
				.put("paymentFrequency", ExistenceChecker.isSet(o.getPaymentFrequency()))
				.put("firstPaymentDate", ExistenceChecker.isSet(o.getFirstPaymentDate()))
				.put("lastPaymentDate", ExistenceChecker.isSet(o.getLastPaymentDate()))
				.put("payRelativeTo", ExistenceChecker.isSet(o.getPayRelativeTo()))
				.put("paymentDaysOffset", ExistenceChecker.isSet(o.getPaymentDaysOffset()))
				.put("paymentDatesAdjustments", ExistenceChecker.isSet(o.getPaymentDatesAdjustments()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PaymentDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PaymentDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PaymentDates> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		PaymentDates.PaymentDatesBuilder o = (PaymentDates.PaymentDatesBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("dateRelativeTo", ExistenceChecker.isSet(o.getDateRelativeTo()))
				.put("paymentFrequency", ExistenceChecker.isSet(o.getPaymentFrequency()))
				.put("firstPaymentDate", ExistenceChecker.isSet(o.getFirstPaymentDate()))
				.put("lastPaymentDate", ExistenceChecker.isSet(o.getLastPaymentDate()))
				.put("payRelativeTo", ExistenceChecker.isSet(o.getPayRelativeTo()))
				.put("paymentDaysOffset", ExistenceChecker.isSet(o.getPaymentDaysOffset()))
				.put("paymentDatesAdjustments", ExistenceChecker.isSet(o.getPaymentDatesAdjustments()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PaymentDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PaymentDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
