package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Payout;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PayoutOnlyExistsValidator implements ValidatorWithArg<Payout, String> {

	@Override
	public ValidationResult<Payout> validate(RosettaPath path, Payout o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("interestRatePayout", ExistenceChecker.isSet(o.getInterestRatePayout()))
				.put("creditDefaultPayout", ExistenceChecker.isSet(o.getCreditDefaultPayout()))
				.put("equityPayout", ExistenceChecker.isSet(o.getEquityPayout()))
				.put("optionPayout", ExistenceChecker.isSet(o.getOptionPayout()))
				.put("forwardPayout", ExistenceChecker.isSet(o.getForwardPayout()))
				.put("securityPayout", ExistenceChecker.isSet(o.getSecurityPayout()))
				.put("cashflow", ExistenceChecker.isSet(o.getCashflow()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Payout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Payout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Payout> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Payout.PayoutBuilder o = (Payout.PayoutBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("interestRatePayout", ExistenceChecker.isSet(o.getInterestRatePayout()))
				.put("creditDefaultPayout", ExistenceChecker.isSet(o.getCreditDefaultPayout()))
				.put("equityPayout", ExistenceChecker.isSet(o.getEquityPayout()))
				.put("optionPayout", ExistenceChecker.isSet(o.getOptionPayout()))
				.put("forwardPayout", ExistenceChecker.isSet(o.getForwardPayout()))
				.put("securityPayout", ExistenceChecker.isSet(o.getSecurityPayout()))
				.put("cashflow", ExistenceChecker.isSet(o.getCashflow()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Payout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Payout", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
