package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.PrincipalExchange;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PrincipalExchangeOnlyExistsValidator implements ValidatorWithArg<PrincipalExchange, String> {

	@Override
	public ValidationResult<PrincipalExchange> validate(RosettaPath path, PrincipalExchange o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedPrincipalExchangeDate", ExistenceChecker.isSet(o.getUnadjustedPrincipalExchangeDate()))
				.put("adjustedPrincipalExchangeDate", ExistenceChecker.isSet(o.getAdjustedPrincipalExchangeDate()))
				.put("principalExchangeAmount", ExistenceChecker.isSet(o.getPrincipalExchangeAmount()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("presentValuePrincipalExchangeAmount", ExistenceChecker.isSet(o.getPresentValuePrincipalExchangeAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PrincipalExchange", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PrincipalExchange", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<PrincipalExchange> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		PrincipalExchange.PrincipalExchangeBuilder o = (PrincipalExchange.PrincipalExchangeBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("unadjustedPrincipalExchangeDate", ExistenceChecker.isSet(o.getUnadjustedPrincipalExchangeDate()))
				.put("adjustedPrincipalExchangeDate", ExistenceChecker.isSet(o.getAdjustedPrincipalExchangeDate()))
				.put("principalExchangeAmount", ExistenceChecker.isSet(o.getPrincipalExchangeAmount()))
				.put("discountFactor", ExistenceChecker.isSet(o.getDiscountFactor()))
				.put("presentValuePrincipalExchangeAmount", ExistenceChecker.isSet(o.getPresentValuePrincipalExchangeAmount()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("PrincipalExchange", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("PrincipalExchange", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
