package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.RateObservation;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class RateObservationOnlyExistsValidator implements ValidatorWithArg<RateObservation, String> {

	@Override
	public ValidationResult<RateObservation> validate(RosettaPath path, RateObservation o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("resetDate", ExistenceChecker.isSet(o.getResetDate()))
				.put("adjustedFixingDate", ExistenceChecker.isSet(o.getAdjustedFixingDate()))
				.put("observedRate", ExistenceChecker.isSet(o.getObservedRate()))
				.put("treatedRate", ExistenceChecker.isSet(o.getTreatedRate()))
				.put("observationWeight", ExistenceChecker.isSet(o.getObservationWeight()))
				.put("rateReference", ExistenceChecker.isSet(o.getRateReference()))
				.put("forecastRate", ExistenceChecker.isSet(o.getForecastRate()))
				.put("treatedForecastRate", ExistenceChecker.isSet(o.getTreatedForecastRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("RateObservation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("RateObservation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<RateObservation> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		RateObservation.RateObservationBuilder o = (RateObservation.RateObservationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("resetDate", ExistenceChecker.isSet(o.getResetDate()))
				.put("adjustedFixingDate", ExistenceChecker.isSet(o.getAdjustedFixingDate()))
				.put("observedRate", ExistenceChecker.isSet(o.getObservedRate()))
				.put("treatedRate", ExistenceChecker.isSet(o.getTreatedRate()))
				.put("observationWeight", ExistenceChecker.isSet(o.getObservationWeight()))
				.put("rateReference", ExistenceChecker.isSet(o.getRateReference()))
				.put("forecastRate", ExistenceChecker.isSet(o.getForecastRate()))
				.put("treatedForecastRate", ExistenceChecker.isSet(o.getTreatedForecastRate()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("RateObservation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("RateObservation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
