package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ReferenceInformation;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ReferenceInformationOnlyExistsValidator implements ValidatorWithArg<ReferenceInformation, String> {

	@Override
	public ValidationResult<ReferenceInformation> validate(RosettaPath path, ReferenceInformation o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("referenceEntity", ExistenceChecker.isSet(o.getReferenceEntity()))
				.put("referenceObligation", ExistenceChecker.isSet(o.getReferenceObligation()))
				.put("noReferenceObligation", ExistenceChecker.isSet(o.getNoReferenceObligation()))
				.put("unknownReferenceObligation", ExistenceChecker.isSet(o.getUnknownReferenceObligation()))
				.put("allGuarantees", ExistenceChecker.isSet(o.getAllGuarantees()))
				.put("referencePrice", ExistenceChecker.isSet(o.getReferencePrice()))
				.put("referencePolicy", ExistenceChecker.isSet(o.getReferencePolicy()))
				.put("securedList", ExistenceChecker.isSet(o.getSecuredList()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ReferenceInformation> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ReferenceInformation.ReferenceInformationBuilder o = (ReferenceInformation.ReferenceInformationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("referenceEntity", ExistenceChecker.isSet(o.getReferenceEntity()))
				.put("referenceObligation", ExistenceChecker.isSet(o.getReferenceObligation()))
				.put("noReferenceObligation", ExistenceChecker.isSet(o.getNoReferenceObligation()))
				.put("unknownReferenceObligation", ExistenceChecker.isSet(o.getUnknownReferenceObligation()))
				.put("allGuarantees", ExistenceChecker.isSet(o.getAllGuarantees()))
				.put("referencePrice", ExistenceChecker.isSet(o.getReferencePrice()))
				.put("referencePolicy", ExistenceChecker.isSet(o.getReferencePolicy()))
				.put("securedList", ExistenceChecker.isSet(o.getSecuredList()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ReferenceInformation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
