package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ReferenceObligation;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ReferenceObligationOnlyExistsValidator implements ValidatorWithArg<ReferenceObligation, String> {

	@Override
	public ValidationResult<ReferenceObligation> validate(RosettaPath path, ReferenceObligation o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bond", ExistenceChecker.isSet(o.getBond()))
				.put("convertibleBond", ExistenceChecker.isSet(o.getConvertibleBond()))
				.put("mortgageBackedSecurity", ExistenceChecker.isSet(o.getMortgageBackedSecurity()))
				.put("loan", ExistenceChecker.isSet(o.getLoan()))
				.put("primaryObligor", ExistenceChecker.isSet(o.getPrimaryObligor()))
				.put("primaryObligorReference", ExistenceChecker.isSet(o.getPrimaryObligorReference()))
				.put("guarantor", ExistenceChecker.isSet(o.getGuarantor()))
				.put("guarantorReference", ExistenceChecker.isSet(o.getGuarantorReference()))
				.put("standardReferenceObligation", ExistenceChecker.isSet(o.getStandardReferenceObligation()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ReferenceObligation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ReferenceObligation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ReferenceObligation> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ReferenceObligation.ReferenceObligationBuilder o = (ReferenceObligation.ReferenceObligationBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bond", ExistenceChecker.isSet(o.getBond()))
				.put("convertibleBond", ExistenceChecker.isSet(o.getConvertibleBond()))
				.put("mortgageBackedSecurity", ExistenceChecker.isSet(o.getMortgageBackedSecurity()))
				.put("loan", ExistenceChecker.isSet(o.getLoan()))
				.put("primaryObligor", ExistenceChecker.isSet(o.getPrimaryObligor()))
				.put("primaryObligorReference", ExistenceChecker.isSet(o.getPrimaryObligorReference()))
				.put("guarantor", ExistenceChecker.isSet(o.getGuarantor()))
				.put("guarantorReference", ExistenceChecker.isSet(o.getGuarantorReference()))
				.put("standardReferenceObligation", ExistenceChecker.isSet(o.getStandardReferenceObligation()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ReferenceObligation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ReferenceObligation", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
