package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Representations;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class RepresentationsOnlyExistsValidator implements ValidatorWithArg<Representations, String> {

	@Override
	public ValidationResult<Representations> validate(RosettaPath path, Representations o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("nonReliance", ExistenceChecker.isSet(o.getNonReliance()))
				.put("agreementsRegardingHedging", ExistenceChecker.isSet(o.getAgreementsRegardingHedging()))
				.put("indexDisclaimer", ExistenceChecker.isSet(o.getIndexDisclaimer()))
				.put("additionalAcknowledgements", ExistenceChecker.isSet(o.getAdditionalAcknowledgements()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Representations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Representations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Representations> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Representations.RepresentationsBuilder o = (Representations.RepresentationsBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("nonReliance", ExistenceChecker.isSet(o.getNonReliance()))
				.put("agreementsRegardingHedging", ExistenceChecker.isSet(o.getAgreementsRegardingHedging()))
				.put("indexDisclaimer", ExistenceChecker.isSet(o.getIndexDisclaimer()))
				.put("additionalAcknowledgements", ExistenceChecker.isSet(o.getAdditionalAcknowledgements()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Representations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Representations", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
