package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ResetDates;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ResetDatesOnlyExistsValidator implements ValidatorWithArg<ResetDates, String> {

	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, ResetDates o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculationPeriodDatesReference", ExistenceChecker.isSet(o.getCalculationPeriodDatesReference()))
				.put("resetRelativeTo", ExistenceChecker.isSet(o.getResetRelativeTo()))
				.put("initialFixingDate", ExistenceChecker.isSet(o.getInitialFixingDate()))
				.put("fixingDates", ExistenceChecker.isSet(o.getFixingDates()))
				.put("finalFixingDate", ExistenceChecker.isSet(o.getFinalFixingDate()))
				.put("rateCutOffDaysOffset", ExistenceChecker.isSet(o.getRateCutOffDaysOffset()))
				.put("resetFrequency", ExistenceChecker.isSet(o.getResetFrequency()))
				.put("resetDatesAdjustments", ExistenceChecker.isSet(o.getResetDatesAdjustments()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ResetDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ResetDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ResetDates> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ResetDates.ResetDatesBuilder o = (ResetDates.ResetDatesBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("calculationPeriodDatesReference", ExistenceChecker.isSet(o.getCalculationPeriodDatesReference()))
				.put("resetRelativeTo", ExistenceChecker.isSet(o.getResetRelativeTo()))
				.put("initialFixingDate", ExistenceChecker.isSet(o.getInitialFixingDate()))
				.put("fixingDates", ExistenceChecker.isSet(o.getFixingDates()))
				.put("finalFixingDate", ExistenceChecker.isSet(o.getFinalFixingDate()))
				.put("rateCutOffDaysOffset", ExistenceChecker.isSet(o.getRateCutOffDaysOffset()))
				.put("resetFrequency", ExistenceChecker.isSet(o.getResetFrequency()))
				.put("resetDatesAdjustments", ExistenceChecker.isSet(o.getResetDatesAdjustments()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ResetDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ResetDates", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
