package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.ResolvablePayoutQuantity;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ResolvablePayoutQuantityOnlyExistsValidator implements ValidatorWithArg<ResolvablePayoutQuantity, String> {

	@Override
	public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, ResolvablePayoutQuantity o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("quantitySchedule", ExistenceChecker.isSet(o.getQuantitySchedule()))
				.put("quantityNotationTag", ExistenceChecker.isSet(o.getQuantityNotationTag()))
				.put("quantityReference", ExistenceChecker.isSet(o.getQuantityReference()))
				.put("quantityMultiplier", ExistenceChecker.isSet(o.getQuantityMultiplier()))
				.put("reset", ExistenceChecker.isSet(o.getReset()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ResolvablePayoutQuantity", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ResolvablePayoutQuantity", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<ResolvablePayoutQuantity> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder o = (ResolvablePayoutQuantity.ResolvablePayoutQuantityBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("quantitySchedule", ExistenceChecker.isSet(o.getQuantitySchedule()))
				.put("quantityNotationTag", ExistenceChecker.isSet(o.getQuantityNotationTag()))
				.put("quantityReference", ExistenceChecker.isSet(o.getQuantityReference()))
				.put("quantityMultiplier", ExistenceChecker.isSet(o.getQuantityMultiplier()))
				.put("reset", ExistenceChecker.isSet(o.getReset()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("ResolvablePayoutQuantity", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("ResolvablePayoutQuantity", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
