package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Resource;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class ResourceOnlyExistsValidator implements ValidatorWithArg<Resource, String> {

	@Override
	public ValidationResult<Resource> validate(RosettaPath path, Resource o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("resourceId", ExistenceChecker.isSet(o.getResourceId()))
				.put("resourceType", ExistenceChecker.isSet(o.getResourceType()))
				.put("language", ExistenceChecker.isSet(o.getLanguage()))
				.put("sizeInBytes", ExistenceChecker.isSet(o.getSizeInBytes()))
				.put("length", ExistenceChecker.isSet(o.getLength()))
				.put("mimeType", ExistenceChecker.isSet(o.getMimeType()))
				.put("name", ExistenceChecker.isSet(o.getName()))
				.put("comments", ExistenceChecker.isSet(o.getComments()))
				.put("string", ExistenceChecker.isSet(o.getString()))
				.put("url", ExistenceChecker.isSet(o.getUrl()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Resource", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Resource", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Resource> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Resource.ResourceBuilder o = (Resource.ResourceBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("resourceId", ExistenceChecker.isSet(o.getResourceId()))
				.put("resourceType", ExistenceChecker.isSet(o.getResourceType()))
				.put("language", ExistenceChecker.isSet(o.getLanguage()))
				.put("sizeInBytes", ExistenceChecker.isSet(o.getSizeInBytes()))
				.put("length", ExistenceChecker.isSet(o.getLength()))
				.put("mimeType", ExistenceChecker.isSet(o.getMimeType()))
				.put("name", ExistenceChecker.isSet(o.getName()))
				.put("comments", ExistenceChecker.isSet(o.getComments()))
				.put("string", ExistenceChecker.isSet(o.getString()))
				.put("url", ExistenceChecker.isSet(o.getUrl()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Resource", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Resource", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
