package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.SecurityLeg;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class SecurityLegOnlyExistsValidator implements ValidatorWithArg<SecurityLeg, String> {

	@Override
	public ValidationResult<SecurityLeg> validate(RosettaPath path, SecurityLeg o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("buyerSeller", ExistenceChecker.isSet(o.getBuyerSeller()))
				.put("settlementDate", ExistenceChecker.isSet(o.getSettlementDate()))
				.put("settlementAmount", ExistenceChecker.isSet(o.getSettlementAmount()))
				.put("settlementCurrency", ExistenceChecker.isSet(o.getSettlementCurrency()))
				.put("fxRate", ExistenceChecker.isSet(o.getFxRate()))
				.put("deliveryDate", ExistenceChecker.isSet(o.getDeliveryDate()))
				.put("deliveryMethod", ExistenceChecker.isSet(o.getDeliveryMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("SecurityLeg", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("SecurityLeg", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<SecurityLeg> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		SecurityLeg.SecurityLegBuilder o = (SecurityLeg.SecurityLegBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("buyerSeller", ExistenceChecker.isSet(o.getBuyerSeller()))
				.put("settlementDate", ExistenceChecker.isSet(o.getSettlementDate()))
				.put("settlementAmount", ExistenceChecker.isSet(o.getSettlementAmount()))
				.put("settlementCurrency", ExistenceChecker.isSet(o.getSettlementCurrency()))
				.put("fxRate", ExistenceChecker.isSet(o.getFxRate()))
				.put("deliveryDate", ExistenceChecker.isSet(o.getDeliveryDate()))
				.put("deliveryMethod", ExistenceChecker.isSet(o.getDeliveryMethod()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("SecurityLeg", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("SecurityLeg", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
