package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Security;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class SecurityOnlyExistsValidator implements ValidatorWithArg<Security, String> {

	@Override
	public ValidationResult<Security> validate(RosettaPath path, Security o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bond", ExistenceChecker.isSet(o.getBond()))
				.put("convertibleBond", ExistenceChecker.isSet(o.getConvertibleBond()))
				.put("equity", ExistenceChecker.isSet(o.getEquity()))
				.put("exchangeTradedFund", ExistenceChecker.isSet(o.getExchangeTradedFund()))
				.put("mortgageBackedSecurity", ExistenceChecker.isSet(o.getMortgageBackedSecurity()))
				.put("mutualFund", ExistenceChecker.isSet(o.getMutualFund()))
				.put("warrant", ExistenceChecker.isSet(o.getWarrant()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Security", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Security", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Security> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Security.SecurityBuilder o = (Security.SecurityBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("bond", ExistenceChecker.isSet(o.getBond()))
				.put("convertibleBond", ExistenceChecker.isSet(o.getConvertibleBond()))
				.put("equity", ExistenceChecker.isSet(o.getEquity()))
				.put("exchangeTradedFund", ExistenceChecker.isSet(o.getExchangeTradedFund()))
				.put("mortgageBackedSecurity", ExistenceChecker.isSet(o.getMortgageBackedSecurity()))
				.put("mutualFund", ExistenceChecker.isSet(o.getMutualFund()))
				.put("warrant", ExistenceChecker.isSet(o.getWarrant()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Security", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Security", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
