package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.TransactedPrice;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class TransactedPriceOnlyExistsValidator implements ValidatorWithArg<TransactedPrice, String> {

	@Override
	public ValidationResult<TransactedPrice> validate(RosettaPath path, TransactedPrice o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("marketFixedRate", ExistenceChecker.isSet(o.getMarketFixedRate()))
				.put("initialPoints", ExistenceChecker.isSet(o.getInitialPoints()))
				.put("marketPrice", ExistenceChecker.isSet(o.getMarketPrice()))
				.put("quotationStyle", ExistenceChecker.isSet(o.getQuotationStyle()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("TransactedPrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("TransactedPrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<TransactedPrice> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		TransactedPrice.TransactedPriceBuilder o = (TransactedPrice.TransactedPriceBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("marketFixedRate", ExistenceChecker.isSet(o.getMarketFixedRate()))
				.put("initialPoints", ExistenceChecker.isSet(o.getInitialPoints()))
				.put("marketPrice", ExistenceChecker.isSet(o.getMarketPrice()))
				.put("quotationStyle", ExistenceChecker.isSet(o.getQuotationStyle()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("TransactedPrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("TransactedPrice", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
