package org.isda.cdm.validation.exists;

import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.RosettaModelObjectBuilder;

import org.isda.cdm.Tx;

import com.google.common.collect.ImmutableMap;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class TxOnlyExistsValidator implements ValidatorWithArg<Tx, String> {

	@Override
	public ValidationResult<Tx> validate(RosettaPath path, Tx o, String field) {
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("newTx", ExistenceChecker.isSet(o.getNewTx()))
				.put("tradDt", ExistenceChecker.isSet(o.getTradDt()))
				.put("tradgCpcty", ExistenceChecker.isSet(o.getTradgCpcty()))
				.put("qty", ExistenceChecker.isSet(o.getQty()))
				.put("pric", ExistenceChecker.isSet(o.getPric()))
				.put("tradVn", ExistenceChecker.isSet(o.getTradVn()))
				.put("ctryOfBrnch", ExistenceChecker.isSet(o.getCtryOfBrnch()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Tx", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Tx", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
	
	@Override
	public ValidationResult<Tx> validate(RosettaPath path, RosettaModelObjectBuilder b, String field) {
		Tx.TxBuilder o = (Tx.TxBuilder)b;
		Map<String,Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("newTx", ExistenceChecker.isSet(o.getNewTx()))
				.put("tradDt", ExistenceChecker.isSet(o.getTradDt()))
				.put("tradgCpcty", ExistenceChecker.isSet(o.getTradgCpcty()))
				.put("qty", ExistenceChecker.isSet(o.getQty()))
				.put("pric", ExistenceChecker.isSet(o.getPric()))
				.put("tradVn", ExistenceChecker.isSet(o.getTradVn()))
				.put("ctryOfBrnch", ExistenceChecker.isSet(o.getCtryOfBrnch()))
				.build();
		// Exclude given method name
		Map<String, Boolean> otherFieldsExistenceMap = fieldExistenceMap.entrySet().stream()
				.filter(x -> !x.getKey().equals(field))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
		// Find any other fields that are set
		Set<String> setFields = otherFieldsExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue))
				.keySet();

		if (!setFields.isEmpty()) {
			return failure("Tx", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "",
					String.format("[%s] is not the only field set. Other set fields: %s", field, setFields));
		}
		return success("Tx", ValidationType.ONLY_EXISTS, o.getClass().getSimpleName(), path, "");
	}
}
