package org.isda.cdm;

import com.rosetta.model.lib.annotations.RosettaSynonym;

/**
 * The enumerated values to specify the list of floating rate index.
 * 
 * @version 2.5.4
 */
public enum FloatingRateIndexEnum {
    

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AED-EBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "AED-EBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AED-EBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "AED-EBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "AED-EBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	AED_EBOR_REUTERS("AED-EBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	AUD_AONIA_OIS_COMPOUND("AUD-AONIA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-AONIA-OIS-COMPOUND-SwapMarker", source = "CME_ClearedConfirm_1_17")
	AUD_AONIA_OIS_COMPOUND_SWAP_MARKER("AUD-AONIA-OIS-COMPOUND-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-BBR-AUBBSW", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-BBR-AUBBSW", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-BBR-AUBBSW", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-BBR-AUBBSW", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-BBR-AUBBSW", source = "CME_ClearedConfirm_1_17")
	AUD_BBR_AUBBSW("AUD-BBR-AUBBSW"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-BBR-BBSW", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-BBR-BBSW", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW", source = "CME_ClearedConfirm_1_17")
	AUD_BBR_BBSW("AUD-BBR-BBSW"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-BBR-BBSW-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-BBR-BBSW-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-BBR-BBSW-Bloomberg", source = "CME_ClearedConfirm_1_17")
	AUD_BBR_BBSW_BLOOMBERG("AUD-BBR-BBSW-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-BBR-BBSY (BID)", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-BBR-BBSY (BID)", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-BBR-BBSY (BID)", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-BBR-BBSY (BID)", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-BBR-BBSY (BID)", source = "CME_ClearedConfirm_1_17")
	AUD_BBR_BBSY__BID_("AUD-BBR-BBSY (BID)"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-BBR-ISDC", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-BBR-ISDC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-BBR-ISDC", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-BBR-ISDC", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-BBR-ISDC", source = "CME_ClearedConfirm_1_17")
	AUD_BBR_ISDC("AUD-BBR-ISDC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	AUD_LIBOR_BBA("AUD-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	AUD_LIBOR_BBA_BLOOMBERG("AUD-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	AUD_LIBOR_REFERENCE_BANKS("AUD-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP", source = "CME_ClearedConfirm_1_17")
	AUD_QUARTERLY_SWAP_RATE_ICAP("AUD-Quarterly Swap Rate-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Quarterly Swap Rate-ICAP-Reference Banks", source = "CME_ClearedConfirm_1_17")
	AUD_QUARTERLY_SWAP_RATE_ICAP_REFERENCE_BANKS("AUD-Quarterly Swap Rate-ICAP-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	AUD_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("AUD-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	AUD_SEMI_ANNUAL_SWAP_RATE_BGCANTOR_REFERENCE_BANKS("AUD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "CME_ClearedConfirm_1_17")
	AUD_SEMI_ANNUAL_SWAP_RATE_ICAP_REFERENCE_BANKS("AUD-Semi-Annual Swap Rate-ICAP-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Semi-annual Swap Rate-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Semi-annual Swap Rate-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Semi-annual Swap Rate-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Semi-annual Swap Rate-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Semi-annual Swap Rate-ICAP", source = "CME_ClearedConfirm_1_17")
	AUD_SEMI_ANNUAL_SWAP_RATE_ICAP("AUD-Semi-annual Swap Rate-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "AUD-Swap Rate-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "AUD-Swap Rate-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "AUD-Swap Rate-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "AUD-Swap Rate-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "AUD-Swap Rate-Reuters", source = "CME_ClearedConfirm_1_17")
	AUD_SWAP_RATE_REUTERS("AUD-Swap Rate-Reuters"),
	

	/**
	 * Refers to the Overnight Brazilian Interbank Deposit Rate Annualized known as the average (Media) of the DI-OVER-EXTRA Grupo as published by CETIP (Camara de Custodia e Liquidacao).
	 */
	@RosettaSynonym(value = "BRL-CDI", source = "FpML_5_10")
	@RosettaSynonym(value = "BRL-CDI", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "BRL-CDI", source = "DTCC_11_0")
	@RosettaSynonym(value = "BRL-CDI", source = "DTCC_9_0")
	@RosettaSynonym(value = "BRL-CDI", source = "CME_ClearedConfirm_1_17")
	BRL_CDI("BRL-CDI"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-CDOR", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-CDOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-CDOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-CDOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-CDOR", source = "CME_ClearedConfirm_1_17")
	CAD_BA_CDOR("CAD-BA-CDOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-CDOR-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-CDOR-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-CDOR-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-CDOR-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-CDOR-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CAD_BA_CDOR_BLOOMBERG("CAD-BA-CDOR-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-ISDD", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-ISDD", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-ISDD", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-ISDD", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-ISDD", source = "CME_ClearedConfirm_1_17")
	CAD_BA_ISDD("CAD-BA-ISDD"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CAD_BA_REFERENCE_BANKS("CAD-BA-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-Reuters", source = "CME_ClearedConfirm_1_17")
	CAD_BA_REUTERS("CAD-BA-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-BA-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-BA-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-BA-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-BA-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-BA-Telerate", source = "CME_ClearedConfirm_1_17")
	CAD_BA_TELERATE("CAD-BA-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-CORRA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-CORRA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-CORRA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-CORRA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-CORRA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	CAD_CORRA_OIS_COMPOUND("CAD-CORRA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-ISDA-Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-ISDA-Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-ISDA-Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-ISDA-Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-ISDA-Swap Rate", source = "CME_ClearedConfirm_1_17")
	CAD_ISDA_SWAP_RATE("CAD-ISDA-Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	CAD_LIBOR_BBA("CAD-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CAD_LIBOR_BBA_BLOOMBERG("CAD-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-LIBOR-BBA-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-LIBOR-BBA-SwapMarker", source = "CME_ClearedConfirm_1_17")
	CAD_LIBOR_BBA_SWAP_MARKER("CAD-LIBOR-BBA-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CAD_LIBOR_REFERENCE_BANKS("CAD-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-REPO-CORRA", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-REPO-CORRA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-REPO-CORRA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-REPO-CORRA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-REPO-CORRA", source = "CME_ClearedConfirm_1_17")
	CAD_REPO_CORRA("CAD-REPO-CORRA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-TBILL-ISDD", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-TBILL-ISDD", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-TBILL-ISDD", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-TBILL-ISDD", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-TBILL-ISDD", source = "CME_ClearedConfirm_1_17")
	CAD_TBILL_ISDD("CAD-TBILL-ISDD"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-TBILL-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-TBILL-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-TBILL-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-TBILL-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-TBILL-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CAD_TBILL_REFERENCE_BANKS("CAD-TBILL-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-TBILL-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-TBILL-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-TBILL-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-TBILL-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-TBILL-Reuters", source = "CME_ClearedConfirm_1_17")
	CAD_TBILL_REUTERS("CAD-TBILL-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CAD-TBILL-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "CAD-TBILL-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CAD-TBILL-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CAD-TBILL-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CAD-TBILL-Telerate", source = "CME_ClearedConfirm_1_17")
	CAD_TBILL_TELERATE("CAD-TBILL-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_3_M_LIBOR_SWAP_CME_VS_LCH_ICAP("CHF-3M LIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CHF_3_M_LIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("CHF-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_3_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP("CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CHF_3_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP_BLOOMBERG("CHF-3M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CHF_6_M_LIBORSWAP_CME_VS_LCH_ICAP_BLOOMBERG("CHF-6M LIBORSWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_6_M_LIBOR_SWAP_CME_VS_LCH_ICAP("CHF-6M LIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_6_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP("CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CHF_6_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP_BLOOMBERG("CHF-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-Annual Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-Annual Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate", source = "CME_ClearedConfirm_1_17")
	CHF_ANNUAL_SWAP_RATE("CHF-Annual Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-Annual Swap Rate-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_ANNUAL_SWAP_RATE_11_00_ICAP("CHF-Annual Swap Rate-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CHF_ANNUAL_SWAP_RATE_REFERENCE_BANKS("CHF-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_BASIS_SWAP_3_M_VS_6_M_LIBOR_11_00_ICAP("CHF-Basis Swap-3m vs 6m-LIBOR-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-ISDAFIX-Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-ISDAFIX-Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-ISDAFIX-Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-ISDAFIX-Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-ISDAFIX-Swap Rate", source = "CME_ClearedConfirm_1_17")
	CHF_ISDAFIX_SWAP_RATE("CHF-ISDAFIX-Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	CHF_LIBOR_BBA("CHF-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CHF_LIBOR_BBA_BLOOMBERG("CHF-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-LIBOR-ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-LIBOR-ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-LIBOR-ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-LIBOR-ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-LIBOR-ISDA", source = "CME_ClearedConfirm_1_17")
	CHF_LIBOR_ISDA("CHF-LIBOR-ISDA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CHF_LIBOR_REFERENCE_BANKS("CHF-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-OIS-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-OIS-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-OIS-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-OIS-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-OIS-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_OIS_11_00_ICAP("CHF-OIS-11:00-ICAP"),
	

	/**
	 * PPer 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-SARON-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-SARON-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-SARON-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-SARON-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-SARON-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	CHF_SARON_OIS_COMPOUND("CHF-SARON-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF-TOIS-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF-TOIS-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF-TOIS-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF-TOIS-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF-TOIS-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	CHF_TOIS_OIS_COMPOUND("CHF-TOIS-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CHF USD-Basis Swaps-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "CHF USD-Basis Swaps-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CHF USD-Basis Swaps-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "CHF USD-Basis Swaps-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "CHF USD-Basis Swaps-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	CHF_USD_BASIS_SWAPS_11_00_ICAP("CHF USD-Basis Swaps-11:00-ICAP"),
	

	/**
	 * Refers to the Indice Camara Promedio (&#39;ICP&#39;) rate for Chilean Pesos which, for a Reset Date, is determined and published by the Asociacion de Bancos e Instituciones Financieras de Chile A.G. (&#39;ABIF&#39;) in accordance with the &#39;Reglamento Indice de Camara Promedio&#39; of the ABIF as published in the Diario Oficial de la Republica de Chile (the &#39;ICP Rules&#39;) and which is reported on the ABIF website by not later than 10:00 a.m., Santiago time, on that Reset Date.
	 */
	@RosettaSynonym(value = "CLP-TNA", source = "FpML_5_10")
	@RosettaSynonym(value = "CLP-TNA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CLP-TNA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CLP-TNA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CLP-TNA", source = "CME_ClearedConfirm_1_17")
	CLP_TNA("CLP-TNA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CL-CLICP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "CL-CLICP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CL-CLICP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "CL-CLICP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "CL-CLICP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	CL_CLICP_BLOOMBERG("CL-CLICP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNH-HIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CNH-HIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNH-HIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNH-HIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNH-HIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CNH_HIBOR_REFERENCE_BANKS("CNH-HIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNH-HIBOR-TMA", source = "FpML_5_10")
	@RosettaSynonym(value = "CNH-HIBOR-TMA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNH-HIBOR-TMA", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNH-HIBOR-TMA", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNH-HIBOR-TMA", source = "CME_ClearedConfirm_1_17")
	CNH_HIBOR_TMA("CNH-HIBOR-TMA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY 7-Repo Compounding Date", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY 7-Repo Compounding Date", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY 7-Repo Compounding Date", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY 7-Repo Compounding Date", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY 7-Repo Compounding Date", source = "CME_ClearedConfirm_1_17")
	CNY_7_REPO_COMPOUNDING_DATE("CNY 7-Repo Compounding Date"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-CNREPOFIX=CFXS-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-CNREPOFIX=CFXS-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-CNREPOFIX=CFXS-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-CNREPOFIX=CFXS-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-CNREPOFIX=CFXS-Reuters", source = "CME_ClearedConfirm_1_17")
	CNY_CNREPOFIX_CFXS_REUTERS("CNY-CNREPOFIX=CFXS-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-PBOCB-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-PBOCB-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-PBOCB-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-PBOCB-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-PBOCB-Reuters", source = "CME_ClearedConfirm_1_17")
	CNY_PBOCB_REUTERS("CNY-PBOCB-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION", source = "CME_ClearedConfirm_1_17")
	CNY_QUARTERLY_7_DAY_REPO_NON_DELIVERABLE_SWAP_RATE_TRADITION("CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CNY_QUARTERLY_7_DAY_REPO_NON_DELIVERABLE_SWAP_RATE_TRADITION_REFERENCE_BANKS("CNY-Quarterly 7 day Repo Non Deliverable Swap Rate-TRADITION-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction..
	 */
	@RosettaSynonym(value = "CNY-SHIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-SHIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-SHIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-SHIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-SHIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	CNY_SHIBOR_REUTERS("CNY-SHIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	CNY_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("CNY-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CNY_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("CNY-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CNY-Shibor-OIS-Compounding", source = "FpML_5_10")
	@RosettaSynonym(value = "CNY-Shibor-OIS-Compounding", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CNY-Shibor-OIS-Compounding", source = "DTCC_11_0")
	@RosettaSynonym(value = "CNY-Shibor-OIS-Compounding", source = "DTCC_9_0")
	@RosettaSynonym(value = "CNY-Shibor-OIS-Compounding", source = "CME_ClearedConfirm_1_17")
	CNY_SHIBOR_OIS_COMPOUNDING("CNY-Shibor-OIS-Compounding"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "COP-IBR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "COP-IBR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "COP-IBR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "COP-IBR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "COP-IBR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	COP_IBR_OIS_COMPOUND("COP-IBR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CZK-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	CZK_ANNUAL_SWAP_RATE_11_00_BGCANTOR("CZK-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CZK-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CZK-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CZK_ANNUAL_SWAP_RATE_REFERENCE_BANKS("CZK-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CZK-PRIBOR-PRBO", source = "FpML_5_10")
	@RosettaSynonym(value = "CZK-PRIBOR-PRBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CZK-PRIBOR-PRBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "CZK-PRIBOR-PRBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "CZK-PRIBOR-PRBO", source = "CME_ClearedConfirm_1_17")
	CZK_PRIBOR_PRBO("CZK-PRIBOR-PRBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "CZK-PRIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "CZK-PRIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "CZK-PRIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "CZK-PRIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "CZK-PRIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	CZK_PRIBOR_REFERENCE_BANKS("CZK-PRIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CIBOR2-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CIBOR2-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CIBOR2-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CIBOR2-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CIBOR2-Bloomberg", source = "CME_ClearedConfirm_1_17")
	DKK_CIBOR_2_BLOOMBERG("DKK-CIBOR2-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CIBOR2-DKNA13", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CIBOR2-DKNA13", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CIBOR2-DKNA13", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CIBOR2-DKNA13", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CIBOR2-DKNA13", source = "CME_ClearedConfirm_1_17")
	DKK_CIBOR2_DKNA13("DKK-CIBOR2-DKNA13"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13", source = "CME_ClearedConfirm_1_17")
	DKK_CIBOR_DKNA13("DKK-CIBOR-DKNA13"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CIBOR-DKNA13-Bloomberg", source = "CME_ClearedConfirm_1_17")
	DKK_CIBOR_DKNA_13_BLOOMBERG("DKK-CIBOR-DKNA13-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	DKK_CIBOR_REFERENCE_BANKS("DKK-CIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-CITA-DKNA14-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-CITA-DKNA14-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-CITA-DKNA14-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-CITA-DKNA14-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-CITA-DKNA14-COMPOUND", source = "CME_ClearedConfirm_1_17")
	DKK_CITA_DKNA14_COMPOUND("DKK-CITA-DKNA14-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "DKK-DKKOIS-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "DKK-DKKOIS-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "DKK-DKKOIS-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "DKK-DKKOIS-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "DKK-DKKOIS-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	DKK_DKKOIS_OIS_COMPOUND("DKK-DKKOIS-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_3_M_EURIBOR_SWAP_CME_VS_LCH_ICAP("EUR-3M EURIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_3_M_EURIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("EUR-3M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_3_M_EURIBOR_SWAP_EUREX_VS_LCH_ICAP("EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_3_M_EURIBOR_SWAP_EUREX_VS_LCH_ICAP_BLOOMBERG("EUR-3M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_6_M_EURIBOR_SWAP_CME_VS_LCH_ICAP("EUR-6M EURIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_6_M_EURIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("EUR-6M EURIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_6_M_EURIBOR_SWAP_EUREX_VS_LCH_ICAP("EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_6_M_EURIBOR_SWAP_EUREX_VS_LCH_ICAP_BLOOMBERG("EUR-6M EURIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00("EUR-Annual Swap Rate-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00_BGCANTOR("EUR-Annual Swap Rate-10:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00_BLOOMBERG("EUR-Annual Swap Rate-10:00-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00_ICAP("EUR-Annual Swap Rate-10:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-SwapMarker", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00_SWAP_MARKER("EUR-Annual Swap Rate-10:00-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-10:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_10_00_TRADITION("EUR-Annual Swap Rate-10:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_11_00("EUR-Annual Swap Rate-11:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_11_00_BLOOMBERG("EUR-Annual Swap Rate-11:00-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_11_00_ICAP("EUR-Annual Swap Rate-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-11:00-SwapMarker", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_11_00_SWAP_MARKER("EUR-Annual Swap Rate-11:00-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_3_MONTH("EUR-Annual Swap Rate-3 Month"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-3 Month-SwapMarker", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_3_MONTH_SWAP_MARKER("EUR-Annual Swap Rate-3 Month-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-4:15-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-4:15-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-4:15-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-4:15-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-4:15-TRADITION", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_4_15_TRADITION("EUR-Annual Swap Rate-4:15-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	EUR_ANNUAL_SWAP_RATE_REFERENCE_BANKS("EUR-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_BASIS_SWAP_EONIA_VS_3_M_EUR_IBOR_SWAP_RATES_A_360_10_00_ICAP("EUR Basis Swap-EONIA vs 3m EUR+IBOR Swap Rates-A/360-10:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-AVERAGE", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-AVERAGE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-AVERAGE", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-AVERAGE", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-AVERAGE", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_AVERAGE("EUR-EONIA-AVERAGE"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_10_00_BGCANTOR("EUR-EONIA-OIS-10:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_10_00_ICAP("EUR-EONIA-OIS-10:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-10:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_10_00_TRADITION("EUR-EONIA-OIS-10:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_11_00_ICAP("EUR-EONIA-OIS-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-4:15-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-4:15-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-4:15-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-4:15-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-4:15-TRADITION", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_4_15_TRADITION("EUR-EONIA-OIS-4:15-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_COMPOUND("EUR-EONIA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-OIS-COMPOUND-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_OIS_COMPOUND_BLOOMBERG("EUR-EONIA-OIS-COMPOUND-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EONIA-Swap-Index", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EONIA-Swap-Index", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EONIA-Swap-Index", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EONIA-Swap-Index", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EONIA-Swap-Index", source = "CME_ClearedConfirm_1_17")
	EUR_EONIA_SWAP_INDEX("EUR-EONIA-Swap-Index"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_ACT_365("EUR-EURIBOR-Act/365"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Act/365-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_ACT_365_BLOOMBERG("EUR-EURIBOR-Act/365-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_ANNUAL_BOND_SWAP_VS_1_M_11_00_ICAP("EUR EURIBOR-Annual Bond Swap vs 1m-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_BASIS_SWAP_1_M_VS_3_M_EURIBOR_11_00_ICAP("EUR EURIBOR-Basis Swap-1m vs 3m-Euribor-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_BASIS_SWAP_3_M_VS_6_M_11_00_ICAP("EUR EURIBOR-Basis Swap-3m vs 6m-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_REFERENCE_BANKS("EUR-EURIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_REUTERS("EUR-EURIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURIBOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURIBOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURIBOR-Telerate", source = "CME_ClearedConfirm_1_17")
	EUR_EURIBOR_TELERATE("EUR-EURIBOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-EURONIA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-EURONIA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-EURONIA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-EURONIA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-EURONIA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	EUR_EURONIA_OIS_COMPOUND("EUR-EURONIA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-11:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-11:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-11:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-11:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-11:00", source = "CME_ClearedConfirm_1_17")
	EUR_ISDA_EURIBOR_SWAP_RATE_11_00("EUR-ISDA-EURIBOR Swap Rate-11:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-12:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-12:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-12:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-12:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-ISDA-EURIBOR Swap Rate-12:00", source = "CME_ClearedConfirm_1_17")
	EUR_ISDA_EURIBOR_SWAP_RATE_12_00("EUR-ISDA-EURIBOR Swap Rate-12:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-10:00", source = "CME_ClearedConfirm_1_17")
	EUR_ISDA_LIBOR_SWAP_RATE_10_00("EUR-ISDA-LIBOR Swap Rate-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-11:00", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-11:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-11:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-11:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-ISDA-LIBOR Swap Rate-11:00", source = "CME_ClearedConfirm_1_17")
	EUR_ISDA_LIBOR_SWAP_RATE_11_00("EUR-ISDA-LIBOR Swap Rate-11:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	EUR_LIBOR_BBA("EUR-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	EUR_LIBOR_BBA_BLOOMBERG("EUR-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	EUR_LIBOR_REFERENCE_BANKS("EUR-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TAM-CDC", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TAM-CDC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TAM-CDC", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TAM-CDC", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TAM-CDC", source = "CME_ClearedConfirm_1_17")
	EUR_TAM_CDC("EUR-TAM-CDC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC10-CNO", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC10-CNO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO", source = "CME_ClearedConfirm_1_17")
	EUR_TEC10_CNO("EUR-TEC10-CNO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC10-CNO-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC10-CNO-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC10-CNO-SwapMarker", source = "CME_ClearedConfirm_1_17")
	EUR_TEC_10_CNO_SWAP_MARKER("EUR-TEC10-CNO-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC10-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC10-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC10-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC10-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC10-Reference Banks", source = "CME_ClearedConfirm_1_17")
	EUR_TEC_10_REFERENCE_BANKS("EUR-TEC10-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC5-CNO", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC5-CNO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO", source = "CME_ClearedConfirm_1_17")
	EUR_TEC5_CNO("EUR-TEC5-CNO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC5-CNO-SwapMarker", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC5-CNO-SwapMarker", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO-SwapMarker", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO-SwapMarker", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC5-CNO-SwapMarker", source = "CME_ClearedConfirm_1_17")
	EUR_TEC_5_CNO_SWAP_MARKER("EUR-TEC5-CNO-SwapMarker"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TEC5-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TEC5-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TEC5-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TEC5-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TEC5-Reference Banks", source = "CME_ClearedConfirm_1_17")
	EUR_TEC_5_REFERENCE_BANKS("EUR-TEC5-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR-TMM-CDC-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR-TMM-CDC-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR-TMM-CDC-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR-TMM-CDC-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR-TMM-CDC-COMPOUND", source = "CME_ClearedConfirm_1_17")
	EUR_TMM_CDC_COMPOUND("EUR-TMM-CDC-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "EUR USD-Basis Swaps-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "EUR USD-Basis Swaps-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "EUR USD-Basis Swaps-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "EUR USD-Basis Swaps-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "EUR USD-Basis Swaps-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	EUR_USD_BASIS_SWAPS_11_00_ICAP("EUR USD-Basis Swaps-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	GBP_6_M_LIBOR_SWAP_CME_VS_LCH_ICAP("GBP-6M LIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	GBP_6_M_LIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("GBP-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	GBP_6_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP("GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	GBP_6_M_LIBOR_SWAP_EUREX_VS_LCH_ICAP_BLOOMBERG("GBP-6M LIBOR SWAP-EUREX vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-ISDA-Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-ISDA-Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-ISDA-Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-ISDA-Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-ISDA-Swap Rate", source = "CME_ClearedConfirm_1_17")
	GBP_ISDA_SWAP_RATE("GBP-ISDA-Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	GBP_LIBOR_BBA("GBP-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	GBP_LIBOR_BBA_BLOOMBERG("GBP-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-LIBOR-ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-LIBOR-ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-LIBOR-ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-LIBOR-ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-LIBOR-ISDA", source = "CME_ClearedConfirm_1_17")
	GBP_LIBOR_ISDA("GBP-LIBOR-ISDA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	GBP_LIBOR_REFERENCE_BANKS("GBP-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-SONIA-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-SONIA-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-SONIA-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-SONIA-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-SONIA-COMPOUND", source = "CME_ClearedConfirm_1_17")
	GBP_SONIA_COMPOUND("GBP-SONIA-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	GBP_SONIA_OIS_11_00_ICAP("GBP-SONIA-OIS-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	GBP_SONIA_OIS_11_00_TRADITION("GBP-SONIA-OIS-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-SONIA-OIS-4:15-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-SONIA-OIS-4:15-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-4:15-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-4:15-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-SONIA-OIS-4:15-TRADITION", source = "CME_ClearedConfirm_1_17")
	GBP_SONIA_OIS_4_15_TRADITION("GBP-SONIA-OIS-4:15-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE("GBP-Semi-Annual Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE_11_00_ICAP("GBP-Semi-Annual Swap Rate-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE_11_00_TRADITION("GBP-Semi Annual Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-4:15-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-4:15-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-4:15-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-4:15-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi Annual Swap Rate-4:15-TRADITION", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE_4_15_TRADITION("GBP-Semi Annual Swap Rate-4:15-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("GBP-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-SwapMarker26", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-SwapMarker26", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-SwapMarker26", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-SwapMarker26", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-Semi-Annual Swap Rate-SwapMarker26", source = "CME_ClearedConfirm_1_17")
	GBP_SEMI_ANNUAL_SWAP_RATE_SWAP_MARKER_26("GBP-Semi-Annual Swap Rate-SwapMarker26"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP USD-Basis Swaps-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP USD-Basis Swaps-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP USD-Basis Swaps-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP USD-Basis Swaps-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP USD-Basis Swaps-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	GBP_USD_BASIS_SWAPS_11_00_ICAP("GBP USD-Basis Swaps-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-WMBA-RONIA-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-WMBA-RONIA-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-WMBA-RONIA-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-WMBA-RONIA-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-WMBA-RONIA-COMPOUND", source = "CME_ClearedConfirm_1_17")
	GBP_WMBA_RONIA_COMPOUND("GBP-WMBA-RONIA-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GBP-WMBA-SONIA-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "GBP-WMBA-SONIA-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GBP-WMBA-SONIA-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "GBP-WMBA-SONIA-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "GBP-WMBA-SONIA-COMPOUND", source = "CME_ClearedConfirm_1_17")
	GBP_WMBA_SONIA_COMPOUND("GBP-WMBA-SONIA-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GRD-ATHIBOR-ATHIBOR", source = "FpML_5_10")
	@RosettaSynonym(value = "GRD-ATHIBOR-ATHIBOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-ATHIBOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-ATHIBOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-ATHIBOR", source = "CME_ClearedConfirm_1_17")
	GRD_ATHIBOR_ATHIBOR("GRD-ATHIBOR-ATHIBOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GRD-ATHIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "GRD-ATHIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	GRD_ATHIBOR_REFERENCE_BANKS("GRD-ATHIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GRD-ATHIBOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "GRD-ATHIBOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "GRD-ATHIBOR-Telerate", source = "CME_ClearedConfirm_1_17")
	GRD_ATHIBOR_TELERATE("GRD-ATHIBOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GRD-ATHIMID-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "GRD-ATHIMID-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reference Banks", source = "CME_ClearedConfirm_1_17")
	GRD_ATHIMID_REFERENCE_BANKS("GRD-ATHIMID-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "GRD-ATHIMID-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "GRD-ATHIMID-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "GRD-ATHIMID-Reuters", source = "CME_ClearedConfirm_1_17")
	GRD_ATHIMID_REUTERS("GRD-ATHIMID-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR=", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR=", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR=", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR=", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR=", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_HIBOR_("HKD-HIBOR-HIBOR="),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-HIBOR-Bloomberg", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_HIBOR_BLOOMBERG("HKD-HIBOR-HIBOR-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-HKAB", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_HKAB("HKD-HIBOR-HKAB"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-HKAB-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-HKAB-Bloomberg", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_HKAB_BLOOMBERG("HKD-HIBOR-HKAB-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-ISDC", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-ISDC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-ISDC", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-ISDC", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-ISDC", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_ISDC("HKD-HIBOR-ISDC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	HKD_HIBOR_REFERENCE_BANKS("HKD-HIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-HONIX-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-HONIX-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-HONIX-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-HONIX-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-HONIX-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	HKD_HONIX_OIS_COMPOUND("HKD-HONIX-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-11:00", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-11:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-11:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-11:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-11:00", source = "CME_ClearedConfirm_1_17")
	HKD_ISDA_SWAP_RATE_11_00("HKD-ISDA-Swap Rate-11:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-4:00", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-4:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-4:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-4:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-ISDA-Swap Rate-4:00", source = "CME_ClearedConfirm_1_17")
	HKD_ISDA_SWAP_RATE_4_00("HKD-ISDA-Swap Rate-4:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_ANNUAL_SWAP_RATE_11_00_BGCANTOR("HKD-Quarterly-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_ANNUAL_SWAP_RATE_11_00_TRADITION("HKD-Quarterly-Annual Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_ANNUAL_SWAP_RATE_4_00_BGCANTOR("HKD-Quarterly-Annual Swap Rate-4:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_ANNUAL_SWAP_RATE_REFERENCE_BANKS("HKD-Quarterly-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_QUARTERLY_SWAP_RATE_11_00_ICAP("HKD-Quarterly-Quarterly Swap Rate-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_QUARTERLY_SWAP_RATE_4_00_ICAP("HKD-Quarterly-Quarterly Swap Rate-4:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "HKD-Quarterly-Quarterly Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	HKD_QUARTERLY_QUARTERLY_SWAP_RATE_REFERENCE_BANKS("HKD-Quarterly-Quarterly Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HUF-BUBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "HUF-BUBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	HUF_BUBOR_REFERENCE_BANKS("HUF-BUBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "HUF-BUBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "HUF-BUBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "HUF-BUBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	HUF_BUBOR_REUTERS("HUF-BUBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-IDMA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-IDMA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-IDMA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-IDMA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-IDMA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	IDR_IDMA_BLOOMBERG("IDR-IDMA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-IDRFIX", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-IDRFIX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-IDRFIX", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-IDRFIX", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-IDRFIX", source = "CME_ClearedConfirm_1_17")
	IDR_IDRFIX("IDR-IDRFIX"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-JIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-JIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-JIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-JIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-JIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	IDR_JIBOR_REUTERS("IDR-JIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-SBI-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-SBI-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-SBI-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-SBI-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-SBI-Reuters", source = "CME_ClearedConfirm_1_17")
	IDR_SBI_REUTERS("IDR-SBI-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-SOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-SOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-SOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-SOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-SOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	IDR_SOR_REFERENCE_BANKS("IDR-SOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-SOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-SOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-SOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-SOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-SOR-Reuters", source = "CME_ClearedConfirm_1_17")
	IDR_SOR_REUTERS("IDR-SOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-SOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-SOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-SOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-SOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-SOR-Telerate", source = "CME_ClearedConfirm_1_17")
	IDR_SOR_TELERATE("IDR-SOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	IDR_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("IDR-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "IDR-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	IDR_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("IDR-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ILS-TELBOR01-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "ILS-TELBOR01-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ILS-TELBOR01-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "ILS-TELBOR01-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "ILS-TELBOR01-Reuters", source = "CME_ClearedConfirm_1_17")
	ILS_TELBOR_01_REUTERS("ILS-TELBOR01-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ILS-TELBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ILS-TELBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ILS-TELBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ILS-TELBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ILS-TELBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ILS_TELBOR_REFERENCE_BANKS("ILS-TELBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-BMK", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-BMK", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-BMK", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-BMK", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-BMK", source = "CME_ClearedConfirm_1_17")
	INR_BMK("INR-BMK"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-CMT", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-CMT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-CMT", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-CMT", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-CMT", source = "CME_ClearedConfirm_1_17")
	INR_CMT("INR-CMT"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-FBIL-MIBOR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-FBIL-MIBOR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-FBIL-MIBOR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-FBIL-MIBOR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-FBIL-MIBOR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	INR_FBIL_MIBOR_OIS_COMPOUND("INR-FBIL-MIBOR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-INBMK-REUTERS", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-INBMK-REUTERS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-INBMK-REUTERS", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-INBMK-REUTERS", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-INBMK-REUTERS", source = "CME_ClearedConfirm_1_17")
	INR_INBMK_REUTERS("INR-INBMK-REUTERS"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-MIBOR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-MIBOR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-MIBOR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-MIBOR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-MIBOR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	INR_MIBOR_OIS_COMPOUND("INR-MIBOR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-MIFOR", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-MIFOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-MIFOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-MIFOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-MIFOR", source = "CME_ClearedConfirm_1_17")
	INR_MIFOR("INR-MIFOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-MIOIS", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-MIOIS", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-MIOIS", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-MIOIS", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-MIOIS", source = "CME_ClearedConfirm_1_17")
	INR_MIOIS("INR-MIOIS"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-MITOR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-MITOR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-MITOR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-MITOR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-MITOR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	INR_MITOR_OIS_COMPOUND("INR-MITOR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	INR_REFERENCE_BANKS("INR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-11:30-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-11:30-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-11:30-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-11:30-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-11:30-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	INR_SEMI_ANNUAL_SWAP_RATE_11_30_BGCANTOR("INR-Semi-Annual Swap Rate-11:30-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon", source = "CME_ClearedConfirm_1_17")
	INR_SEMI_ANNUAL_SWAP_RATE_NON_DELIVERABLE_16_00_TULLETT_PREBON("INR-Semi Annual Swap Rate-Non-deliverable-16:00-Tullett Prebon"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "INR-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	INR_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("INR-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ISK-REIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ISK-REIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ISK_REIBOR_REFERENCE_BANKS("ISK-REIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ISK-REIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "ISK-REIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "ISK-REIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	ISK_REIBOR_REUTERS("ISK-REIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-Annual Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	JPY_ANNUAL_SWAP_RATE_11_00_TRADITION("JPY-Annual Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-Annual Swap Rate-3:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-3:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-3:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-3:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-Annual Swap Rate-3:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	JPY_ANNUAL_SWAP_RATE_3_00_TRADITION("JPY-Annual Swap Rate-3:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-10:00", source = "CME_ClearedConfirm_1_17")
	JPY_BBSF_BLOOMBERG_10_00("JPY-BBSF-Bloomberg-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-15:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-15:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-15:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-15:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-BBSF-Bloomberg-15:00", source = "CME_ClearedConfirm_1_17")
	JPY_BBSF_BLOOMBERG_15_00("JPY-BBSF-Bloomberg-15:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-10:00", source = "CME_ClearedConfirm_1_17")
	JPY_ISDA_SWAP_RATE_10_00("JPY-ISDA-Swap Rate-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-15:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-15:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-15:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-15:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-ISDA-Swap Rate-15:00", source = "CME_ClearedConfirm_1_17")
	JPY_ISDA_SWAP_RATE_15_00("JPY-ISDA-Swap Rate-15:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	JPY_LIBOR_BBA("JPY-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	JPY_LIBOR_BBA_BLOOMBERG("JPY-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LIBOR-FRASETT", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LIBOR-FRASETT", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LIBOR-FRASETT", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LIBOR-FRASETT", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LIBOR-FRASETT", source = "CME_ClearedConfirm_1_17")
	JPY_LIBOR_FRASETT("JPY-LIBOR-FRASETT"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LIBOR-ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LIBOR-ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LIBOR-ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LIBOR-ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LIBOR-ISDA", source = "CME_ClearedConfirm_1_17")
	JPY_LIBOR_ISDA("JPY-LIBOR-ISDA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	JPY_LIBOR_REFERENCE_BANKS("JPY-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LTPR-MHCB", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LTPR-MHCB", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LTPR-MHCB", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LTPR-MHCB", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LTPR-MHCB", source = "CME_ClearedConfirm_1_17")
	JPY_LTPR_MHCB("JPY-LTPR-MHCB"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-LTPR-TBC", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-LTPR-TBC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-LTPR-TBC", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-LTPR-TBC", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-LTPR-TBC", source = "CME_ClearedConfirm_1_17")
	JPY_LTPR_TBC("JPY-LTPR-TBC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-MUTANCALL-TONAR", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-MUTANCALL-TONAR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-MUTANCALL-TONAR", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-MUTANCALL-TONAR", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-MUTANCALL-TONAR", source = "CME_ClearedConfirm_1_17")
	JPY_MUTANCALL_TONAR("JPY-MUTANCALL-TONAR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-OIS-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-OIS-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	JPY_OIS_11_00_ICAP("JPY-OIS-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-OIS-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-OIS-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-OIS-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	JPY_OIS_11_00_TRADITION("JPY-OIS-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-OIS-3:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-OIS-3:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-OIS-3:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-OIS-3:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-OIS-3:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	JPY_OIS_3_00_TRADITION("JPY-OIS-3:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-Quoting Banks-LIBOR", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-Quoting Banks-LIBOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-Quoting Banks-LIBOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-Quoting Banks-LIBOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-Quoting Banks-LIBOR", source = "CME_ClearedConfirm_1_17")
	JPY_QUOTING_BANKS_LIBOR("JPY-Quoting Banks-LIBOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-STPR-Quoting Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-STPR-Quoting Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-STPR-Quoting Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-STPR-Quoting Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-STPR-Quoting Banks", source = "CME_ClearedConfirm_1_17")
	JPY_STPR_QUOTING_BANKS("JPY-STPR-Quoting Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-17096", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-17096", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-17096", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-17096", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-17096", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_17096("JPY-TIBOR-17096"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-17097", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-17097", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-17097", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-17097", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-17097", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_17097("JPY-TIBOR-17097"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-DTIBOR01", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-DTIBOR01", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-DTIBOR01", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-DTIBOR01", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-DTIBOR01", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_DTIBOR01("JPY-TIBOR-DTIBOR01"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM("JPY-TIBOR-TIBM"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM-Reference Banks", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM_REFERENCE_BANKS("JPY-TIBOR-TIBM-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (10 Banks)", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (10 Banks)", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (10 Banks)", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (10 Banks)", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (10 Banks)", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM_10_BANKS("JPY-TIBOR-TIBM (10 Banks)"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (5 Banks)", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (5 Banks)", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (5 Banks)", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (5 Banks)", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (5 Banks)", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM_5_BANKS("JPY-TIBOR-TIBM (5 Banks)"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM_ALL_BANKS("JPY-TIBOR-TIBM (All Banks)"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-TIBM (All Banks)-Bloomberg", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_TIBM_ALL_BANKS_BLOOMBERG("JPY-TIBOR-TIBM (All Banks)-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TIBOR-ZTIBOR", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TIBOR-ZTIBOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TIBOR-ZTIBOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TIBOR-ZTIBOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TIBOR-ZTIBOR", source = "CME_ClearedConfirm_1_17")
	JPY_TIBOR_ZTIBOR("JPY-TIBOR-ZTIBOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TONA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TONA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TONA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TONA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TONA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	JPY_TONA_OIS_COMPOUND("JPY-TONA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TSR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TSR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TSR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TSR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TSR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	JPY_TSR_REFERENCE_BANKS("JPY-TSR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TSR-Reuters-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TSR-Reuters-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-10:00", source = "CME_ClearedConfirm_1_17")
	JPY_TSR_REUTERS_10_00("JPY-TSR-Reuters-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TSR-Reuters-15:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TSR-Reuters-15:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-15:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-15:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TSR-Reuters-15:00", source = "CME_ClearedConfirm_1_17")
	JPY_TSR_REUTERS_15_00("JPY-TSR-Reuters-15:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TSR-Telerate-10:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TSR-Telerate-10:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-10:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-10:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-10:00", source = "CME_ClearedConfirm_1_17")
	JPY_TSR_TELERATE_10_00("JPY-TSR-Telerate-10:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY-TSR-Telerate-15:00", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY-TSR-Telerate-15:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-15:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-15:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY-TSR-Telerate-15:00", source = "CME_ClearedConfirm_1_17")
	JPY_TSR_TELERATE_15_00("JPY-TSR-Telerate-15:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "JPY USD-Basis Swaps-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "JPY USD-Basis Swaps-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "JPY USD-Basis Swaps-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "JPY USD-Basis Swaps-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "JPY USD-Basis Swaps-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	JPY_USD_BASIS_SWAPS_11_00_ICAP("JPY USD-Basis Swaps-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "KRW-Bond-3222", source = "FpML_5_10")
	@RosettaSynonym(value = "KRW-Bond-3222", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "KRW-Bond-3222", source = "DTCC_11_0")
	@RosettaSynonym(value = "KRW-Bond-3222", source = "DTCC_9_0")
	@RosettaSynonym(value = "KRW-Bond-3222", source = "CME_ClearedConfirm_1_17")
	KRW_BOND_3222("KRW-Bond-3222"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "KRW-CD-3220", source = "FpML_5_10")
	@RosettaSynonym(value = "KRW-CD-3220", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "KRW-CD-3220", source = "DTCC_11_0")
	@RosettaSynonym(value = "KRW-CD-3220", source = "DTCC_9_0")
	@RosettaSynonym(value = "KRW-CD-3220", source = "CME_ClearedConfirm_1_17")
	KRW_CD_3220("KRW-CD-3220"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "KRW-CD-KSDA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "KRW-CD-KSDA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "KRW-CD-KSDA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "KRW-CD-KSDA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "KRW-CD-KSDA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	KRW_CD_KSDA_BLOOMBERG("KRW-CD-KSDA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "KRW-Quarterly Annual Swap Rate-3:30-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "KRW-Quarterly Annual Swap Rate-3:30-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "KRW-Quarterly Annual Swap Rate-3:30-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "KRW-Quarterly Annual Swap Rate-3:30-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "KRW-Quarterly Annual Swap Rate-3:30-ICAP", source = "CME_ClearedConfirm_1_17")
	KRW_QUARTERLY_ANNUAL_SWAP_RATE_3_30_ICAP("KRW-Quarterly Annual Swap Rate-3:30-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MXN-TIIE-Banxico", source = "FpML_5_10")
	@RosettaSynonym(value = "MXN-TIIE-Banxico", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico", source = "DTCC_11_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico", source = "DTCC_9_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico", source = "CME_ClearedConfirm_1_17")
	MXN_TIIE_BANXICO("MXN-TIIE-Banxico"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Bloomberg", source = "CME_ClearedConfirm_1_17")
	MXN_TIIE_BANXICO_BLOOMBERG("MXN-TIIE-Banxico-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "MXN-TIIE-Banxico-Reference Banks", source = "CME_ClearedConfirm_1_17")
	MXN_TIIE_BANXICO_REFERENCE_BANKS("MXN-TIIE-Banxico-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MYR-KLIBOR-BNM", source = "FpML_5_10")
	@RosettaSynonym(value = "MYR-KLIBOR-BNM", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MYR-KLIBOR-BNM", source = "DTCC_11_0")
	@RosettaSynonym(value = "MYR-KLIBOR-BNM", source = "DTCC_9_0")
	@RosettaSynonym(value = "MYR-KLIBOR-BNM", source = "CME_ClearedConfirm_1_17")
	MYR_KLIBOR_BNM("MYR-KLIBOR-BNM"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MYR-KLIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "MYR-KLIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MYR-KLIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "MYR-KLIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "MYR-KLIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	MYR_KLIBOR_REFERENCE_BANKS("MYR-KLIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	MYR_QUARTERLY_SWAP_RATE_11_00_TRADITION("MYR-Quarterly Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "MYR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	MYR_QUARTERLY_SWAP_RATE_TRADITION_REFERENCE_BANKS("MYR-Quarterly Swap Rate-TRADITION-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NOK-NIBOR-NIBR", source = "FpML_5_10")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR", source = "DTCC_11_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR", source = "DTCC_9_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR", source = "CME_ClearedConfirm_1_17")
	NOK_NIBOR_NIBR("NOK-NIBOR-NIBR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NOK-NIBOR-NIBR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "NOK-NIBOR-NIBR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	NOK_NIBOR_NIBR_REFERENCE_BANKS("NOK-NIBOR-NIBR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NOK-NIBOR-OIBOR", source = "FpML_5_10")
	@RosettaSynonym(value = "NOK-NIBOR-OIBOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NOK-NIBOR-OIBOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "NOK-NIBOR-OIBOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "NOK-NIBOR-OIBOR", source = "CME_ClearedConfirm_1_17")
	NOK_NIBOR_OIBOR("NOK-NIBOR-OIBOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NOK-NIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "NOK-NIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NOK-NIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "NOK-NIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "NOK-NIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	NOK_NIBOR_REFERENCE_BANKS("NOK-NIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-BBR-BID", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-BBR-BID", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-BBR-BID", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-BBR-BID", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-BBR-BID", source = "CME_ClearedConfirm_1_17")
	NZD_BBR_BID("NZD-BBR-BID"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-BBR-FRA", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-BBR-FRA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-BBR-FRA", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-BBR-FRA", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-BBR-FRA", source = "CME_ClearedConfirm_1_17")
	NZD_BBR_FRA("NZD-BBR-FRA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-BBR-ISDC", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-BBR-ISDC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-BBR-ISDC", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-BBR-ISDC", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-BBR-ISDC", source = "CME_ClearedConfirm_1_17")
	NZD_BBR_ISDC("NZD-BBR-ISDC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-BBR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-BBR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-BBR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-BBR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-BBR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	NZD_BBR_REFERENCE_BANKS("NZD-BBR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-BBR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-BBR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-BBR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-BBR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-BBR-Telerate", source = "CME_ClearedConfirm_1_17")
	NZD_BBR_TELERATE("NZD-BBR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-NZIONA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-NZIONA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-NZIONA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-NZIONA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-NZIONA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	NZD_NZIONA_OIS_COMPOUND("NZD-NZIONA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	NZD_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("NZD-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	NZD_SEMI_ANNUAL_SWAP_RATE_BGCANTOR_REFERENCE_BANKS("NZD-Semi-Annual Swap Rate-BGCANTOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP", source = "CME_ClearedConfirm_1_17")
	NZD_SWAP_RATE_ICAP("NZD-Swap Rate-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "NZD-Swap Rate-ICAP-Reference Banks", source = "CME_ClearedConfirm_1_17")
	NZD_SWAP_RATE_ICAP_REFERENCE_BANKS("NZD-Swap Rate-ICAP-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PHP-PHIREF-BAP", source = "FpML_5_10")
	@RosettaSynonym(value = "PHP-PHIREF-BAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PHP-PHIREF-BAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "PHP-PHIREF-BAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "PHP-PHIREF-BAP", source = "CME_ClearedConfirm_1_17")
	PHP_PHIREF_BAP("PHP-PHIREF-BAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PHP-PHIREF-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "PHP-PHIREF-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PHP-PHIREF-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "PHP-PHIREF-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "PHP-PHIREF-Bloomberg", source = "CME_ClearedConfirm_1_17")
	PHP_PHIREF_BLOOMBERG("PHP-PHIREF-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PHP-PHIREF-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "PHP-PHIREF-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PHP-PHIREF-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "PHP-PHIREF-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "PHP-PHIREF-Reference Banks", source = "CME_ClearedConfirm_1_17")
	PHP_PHIREF_REFERENCE_BANKS("PHP-PHIREF-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	PHP_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("PHP-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "PHP-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	PHP_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("PHP-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PLN-POLONIA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "PLN-POLONIA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLN-POLONIA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLN-POLONIA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLN-POLONIA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	PLN_POLONIA_OIS_COMPOUND("PLN-POLONIA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PLN-WIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "PLN-WIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLN-WIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLN-WIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLN-WIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	PLN_WIBOR_REFERENCE_BANKS("PLN-WIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PLN-WIBOR-WIBO", source = "FpML_5_10")
	@RosettaSynonym(value = "PLN-WIBOR-WIBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLN-WIBOR-WIBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLN-WIBOR-WIBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLN-WIBOR-WIBO", source = "CME_ClearedConfirm_1_17")
	PLN_WIBOR_WIBO("PLN-WIBOR-WIBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PLZ-WIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "PLZ-WIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLZ-WIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLZ-WIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLZ-WIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	PLZ_WIBOR_REFERENCE_BANKS("PLZ-WIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "PLZ-WIBOR-WIBO", source = "FpML_5_10")
	@RosettaSynonym(value = "PLZ-WIBOR-WIBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "PLZ-WIBOR-WIBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "PLZ-WIBOR-WIBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "PLZ-WIBOR-WIBO", source = "CME_ClearedConfirm_1_17")
	PLZ_WIBOR_WIBO("PLZ-WIBOR-WIBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "REPOFUNDS RATE-FRANCE-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "REPOFUNDS RATE-FRANCE-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-FRANCE-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-FRANCE-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-FRANCE-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	REPOFUNDS_RATE_FRANCE_OIS_COMPOUND("REPOFUNDS RATE-FRANCE-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "REPOFUNDS RATE-GERMANY-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "REPOFUNDS RATE-GERMANY-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-GERMANY-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-GERMANY-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-GERMANY-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	REPOFUNDS_RATE_GERMANY_OIS_COMPOUND("REPOFUNDS RATE-GERMANY-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "REPOFUNDS RATE-ITALY-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "REPOFUNDS RATE-ITALY-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-ITALY-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-ITALY-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "REPOFUNDS RATE-ITALY-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	REPOFUNDS_RATE_ITALY_OIS_COMPOUND("REPOFUNDS RATE-ITALY-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RON-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "RON-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	RON_ANNUAL_SWAP_RATE_11_00_BGCANTOR("RON-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RON-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "RON-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "RON-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	RON_ANNUAL_SWAP_RATE_REFERENCE_BANKS("RON-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RON-RBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "RON-RBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RON-RBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "RON-RBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "RON-RBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	RON_RBOR_REUTERS("RON-RBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	RUB_ANNUAL_SWAP_RATE_11_00_BGCANTOR("RUB-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-Annual Swap Rate-12:45-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-12:45-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-12:45-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-12:45-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-12:45-TRADITION", source = "CME_ClearedConfirm_1_17")
	RUB_ANNUAL_SWAP_RATE_12_45_TRADITION("RUB-Annual Swap Rate-12:45-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-Annual Swap Rate-4:15-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-4:15-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-4:15-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-4:15-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-4:15-TRADITION", source = "CME_ClearedConfirm_1_17")
	RUB_ANNUAL_SWAP_RATE_4_15_TRADITION("RUB-Annual Swap Rate-4:15-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	RUB_ANNUAL_SWAP_RATE_REFERENCE_BANKS("RUB-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-Annual Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	RUB_ANNUAL_SWAP_RATE_TRADITION_REFERENCE_BANKS("RUB-Annual Swap Rate-TRADITION-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-MOSPRIME-NFEA", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-MOSPRIME-NFEA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-NFEA", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-NFEA", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-NFEA", source = "CME_ClearedConfirm_1_17")
	RUB_MOSPRIME_NFEA("RUB-MOSPRIME-NFEA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-MOSPRIME-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-MOSPRIME-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-MOSPRIME-Reference Banks", source = "CME_ClearedConfirm_1_17")
	RUB_MOSPRIME_REFERENCE_BANKS("RUB-MOSPRIME-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "RUB-RUONIA-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "RUB-RUONIA-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "RUB-RUONIA-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "RUB-RUONIA-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "RUB-RUONIA-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	RUB_RUONIA_OIS_COMPOUND("RUB-RUONIA-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SAR-SRIOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SAR-SRIOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SAR-SRIOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SAR-SRIOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SAR-SRIOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SAR_SRIOR_REFERENCE_BANKS("SAR-SRIOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SAR-SRIOR-SUAA", source = "FpML_5_10")
	@RosettaSynonym(value = "SAR-SRIOR-SUAA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SAR-SRIOR-SUAA", source = "DTCC_11_0")
	@RosettaSynonym(value = "SAR-SRIOR-SUAA", source = "DTCC_9_0")
	@RosettaSynonym(value = "SAR-SRIOR-SUAA", source = "CME_ClearedConfirm_1_17")
	SAR_SRIOR_SUAA("SAR-SRIOR-SUAA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-Annual Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-Annual Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate", source = "CME_ClearedConfirm_1_17")
	SEK_ANNUAL_SWAP_RATE("SEK-Annual Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-Annual Swap Rate-SESWFI", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-Annual Swap Rate-SESWFI", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate-SESWFI", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate-SESWFI", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-Annual Swap Rate-SESWFI", source = "CME_ClearedConfirm_1_17")
	SEK_ANNUAL_SWAP_RATE_SESWFI("SEK-Annual Swap Rate-SESWFI"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-SIOR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-SIOR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-SIOR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-SIOR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-SIOR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	SEK_SIOR_OIS_COMPOUND("SEK-SIOR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-STIBOR-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-STIBOR-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-STIBOR-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-STIBOR-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-STIBOR-Bloomberg", source = "CME_ClearedConfirm_1_17")
	SEK_STIBOR_BLOOMBERG("SEK-STIBOR-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-STIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-STIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-STIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-STIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-STIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SEK_STIBOR_REFERENCE_BANKS("SEK-STIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SEK-STIBOR-SIDE", source = "FpML_5_10")
	@RosettaSynonym(value = "SEK-STIBOR-SIDE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SEK-STIBOR-SIDE", source = "DTCC_11_0")
	@RosettaSynonym(value = "SEK-STIBOR-SIDE", source = "DTCC_9_0")
	@RosettaSynonym(value = "SEK-STIBOR-SIDE", source = "CME_ClearedConfirm_1_17")
	SEK_STIBOR_SIDE("SEK-STIBOR-SIDE"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SIBOR_REFERENCE_BANKS("SGD-SIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	SGD_SIBOR_REUTERS("SGD-SIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SIBOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SIBOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SIBOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SIBOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SIBOR-Telerate", source = "CME_ClearedConfirm_1_17")
	SGD_SIBOR_TELERATE("SGD-SIBOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SONAR-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SONAR-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	SGD_SONAR_OIS_COMPOUND("SGD-SONAR-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SONAR-OIS-VWAP-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SONAR-OIS-VWAP-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-VWAP-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-VWAP-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SONAR-OIS-VWAP-COMPOUND", source = "CME_ClearedConfirm_1_17")
	SGD_SONAR_OIS_VWAP_COMPOUND("SGD-SONAR-OIS-VWAP-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SOR_REFERENCE_BANKS("SGD-SOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SOR-Reuters", source = "CME_ClearedConfirm_1_17")
	SGD_SOR_REUTERS("SGD-SOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SOR-Telerate", source = "CME_ClearedConfirm_1_17")
	SGD_SOR_TELERATE("SGD-SOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SOR-VWAP", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SOR-VWAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP", source = "CME_ClearedConfirm_1_17")
	SGD_SOR_VWAP("SGD-SOR-VWAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-SOR-VWAP-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-SOR-VWAP-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-SOR-VWAP-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SOR_VWAP_REFERENCE_BANKS("SGD-SOR-VWAP-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_CURRENCY_BASIS_SWAP_RATE_11_00_TULLETT_PREBON("SGD-Semi-Annual Currency Basis Swap Rate-11:00-Tullett Prebon"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_CURRENCY_BASIS_SWAP_RATE_16_00_TULLETT_PREBON("SGD-Semi-Annual Currency Basis Swap Rate-16:00-Tullett Prebon"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("SGD-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11.00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11.00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11.00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11.00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11.00-TRADITION", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_11_00_TRADITION("SGD-Semi-Annual Swap Rate-11.00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_11_00_TULLETT_PREBON("SGD-Semi-Annual Swap Rate-11:00-Tullett Prebon"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_16_00_TULLETT_PREBON("SGD-Semi-Annual Swap Rate-16:00-Tullett Prebon"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_ICAP("SGD-Semi-Annual Swap Rate-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-ICAP-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_ICAP_REFERENCE_BANKS("SGD-Semi-Annual Swap Rate-ICAP-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("SGD-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SGD_SEMI_ANNUAL_SWAP_RATE_TRADITION_REFERENCE_BANKS("SGD-Semi-Annual Swap Rate-TRADITION-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SKK-BRIBOR-BRBO", source = "FpML_5_10")
	@RosettaSynonym(value = "SKK-BRIBOR-BRBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SKK-BRIBOR-BRBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "SKK-BRIBOR-BRBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "SKK-BRIBOR-BRBO", source = "CME_ClearedConfirm_1_17")
	SKK_BRIBOR_BRBO("SKK-BRIBOR-BRBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SKK-BRIBOR-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "SKK-BRIBOR-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Bloomberg", source = "CME_ClearedConfirm_1_17")
	SKK_BRIBOR_BLOOMBERG("SKK-BRIBOR-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SKK-BRIBOR-NBSK07", source = "FpML_5_10")
	@RosettaSynonym(value = "SKK-BRIBOR-NBSK07", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SKK-BRIBOR-NBSK07", source = "DTCC_11_0")
	@RosettaSynonym(value = "SKK-BRIBOR-NBSK07", source = "DTCC_9_0")
	@RosettaSynonym(value = "SKK-BRIBOR-NBSK07", source = "CME_ClearedConfirm_1_17")
	SKK_BRIBOR_NBSK07("SKK-BRIBOR-NBSK07"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "SKK-BRIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "SKK-BRIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "SKK-BRIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	SKK_BRIBOR_REFERENCE_BANKS("SKK-BRIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-SOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-SOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-SOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-SOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-SOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	THB_SOR_REFERENCE_BANKS("THB-SOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-SOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-SOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-SOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-SOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-SOR-Reuters", source = "CME_ClearedConfirm_1_17")
	THB_SOR_REUTERS("THB-SOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-SOR-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-SOR-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-SOR-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-SOR-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-SOR-Telerate", source = "CME_ClearedConfirm_1_17")
	THB_SOR_TELERATE("THB-SOR-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	THB_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("THB-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	THB_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("THB-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-THBFIX-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-THBFIX-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-THBFIX-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-THBFIX-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-THBFIX-Reference Banks", source = "CME_ClearedConfirm_1_17")
	THB_THBFIX_REFERENCE_BANKS("THB-THBFIX-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "THB-THBFIX-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "THB-THBFIX-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "THB-THBFIX-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "THB-THBFIX-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "THB-THBFIX-Reuters", source = "CME_ClearedConfirm_1_17")
	THB_THBFIX_REUTERS("THB-THBFIX-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY Annual Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY Annual Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY Annual Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY Annual Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY Annual Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	TRY_ANNUAL_SWAP_RATE_11_00_TRADITION("TRY Annual Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY-Annual Swap Rate-11:15-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-11:15-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-11:15-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-11:15-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-11:15-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	TRY_ANNUAL_SWAP_RATE_11_15_BGCANTOR("TRY-Annual Swap Rate-11:15-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	TRY_ANNUAL_SWAP_RATE_REFERENCE_BANKS("TRY-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	TRY_SEMI_ANNUAL_SWAP_RATE_TRADITION_REFERENCE_BANKS("TRY-Semi-Annual Swap Rate-TRADITION-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY-TRYIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	TRY_TRYIBOR_REFERENCE_BANKS("TRY-TRYIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TRY-TRYIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "TRY-TRYIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	TRY_TRYIBOR_REUTERS("TRY-TRYIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	TWD_QUARTERLY_ANNUAL_SWAP_RATE_11_00_BGCANTOR("TWD-Quarterly-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-Quarterly-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	TWD_QUARTERLY_ANNUAL_SWAP_RATE_REFERENCE_BANKS("TWD-Quarterly-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-Reference Dealers", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-Reference Dealers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-Reference Dealers", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-Reference Dealers", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-Reference Dealers", source = "CME_ClearedConfirm_1_17")
	TWD_REFERENCE_DEALERS("TWD-Reference Dealers"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-Reuters-6165", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-Reuters-6165", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-Reuters-6165", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-Reuters-6165", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-Reuters-6165", source = "CME_ClearedConfirm_1_17")
	TWD_REUTERS_6165("TWD-Reuters-6165"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-TAIBIR01", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-TAIBIR01", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-TAIBIR01", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-TAIBIR01", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-TAIBIR01", source = "CME_ClearedConfirm_1_17")
	TWD_TAIBIR01("TWD-TAIBIR01"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-TAIBIR02", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-TAIBIR02", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-TAIBIR02", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-TAIBIR02", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-TAIBIR02", source = "CME_ClearedConfirm_1_17")
	TWD_TAIBIR02("TWD-TAIBIR02"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-TAIBOR-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-TAIBOR-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Bloomberg", source = "CME_ClearedConfirm_1_17")
	TWD_TAIBOR_BLOOMBERG("TWD-TAIBOR-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-TAIBOR-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-TAIBOR-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-TAIBOR-Reuters", source = "CME_ClearedConfirm_1_17")
	TWD_TAIBOR_REUTERS("TWD-TAIBOR-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-TWCPBA", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-TWCPBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-TWCPBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-TWCPBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-TWCPBA", source = "CME_ClearedConfirm_1_17")
	TWD_TWCPBA("TWD-TWCPBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "TWD-Telerate-6165", source = "FpML_5_10")
	@RosettaSynonym(value = "TWD-Telerate-6165", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "TWD-Telerate-6165", source = "DTCC_11_0")
	@RosettaSynonym(value = "TWD-Telerate-6165", source = "DTCC_9_0")
	@RosettaSynonym(value = "TWD-Telerate-6165", source = "CME_ClearedConfirm_1_17")
	TWD_TELERATE_6165("TWD-Telerate-6165"),
	

	/**
	 * United Kingdom: GBP - Non-revised Retail Price Index Excluding Mortgage Interest Payments (UKRPIX)
	 */
	@RosettaSynonym(value = "UK-RPIX", source = "FpML_5_10")
	@RosettaSynonym(value = "UK-RPIX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "UK-RPIX", source = "DTCC_11_0")
	@RosettaSynonym(value = "UK-RPIX", source = "DTCC_9_0")
	@RosettaSynonym(value = "UK-RPIX", source = "CME_ClearedConfirm_1_17")
	UK_RPIX("UK-RPIX"),
	

	/**
	 * United States: USA - Non-revised Consumer Price Index - Urban (CPI-U)
	 */
	@RosettaSynonym(value = "USA-CPI-U", source = "FpML_5_10")
	@RosettaSynonym(value = "USA-CPI-U", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USA-CPI-U", source = "DTCC_11_0")
	@RosettaSynonym(value = "USA-CPI-U", source = "DTCC_9_0")
	@RosettaSynonym(value = "USA-CPI-U", source = "CME_ClearedConfirm_1_17")
	USA_CPI_U("USA-CPI-U"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_3_M_LIBOR_SWAP_CME_VS_LCH_ICAP("USD-3M LIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	USD_3_M_LIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("USD-3M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_6_M_LIBOR_SWAP_CME_VS_LCH_ICAP("USD-6M LIBOR SWAP-CME vs LCH-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg", source = "CME_ClearedConfirm_1_17")
	USD_6_M_LIBOR_SWAP_CME_VS_LCH_ICAP_BLOOMBERG("USD-6M LIBOR SWAP-CME vs LCH-ICAP-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	USD_ANNUAL_SWAP_RATE_11_00_BGCANTOR("USD-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	USD_ANNUAL_SWAP_RATE_11_00_TRADITION("USD-Annual Swap Rate-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Annual Swap Rate-4:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Annual Swap Rate-4:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-4:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-4:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Annual Swap Rate-4:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	USD_ANNUAL_SWAP_RATE_4_00_TRADITION("USD-Annual Swap Rate-4:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-BA-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-BA-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-BA-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-BA-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-BA-H.15", source = "CME_ClearedConfirm_1_17")
	USD_BA_H_15("USD-BA-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-BA-Reference Dealers", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-BA-Reference Dealers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-BA-Reference Dealers", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-BA-Reference Dealers", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-BA-Reference Dealers", source = "CME_ClearedConfirm_1_17")
	USD_BA_REFERENCE_DEALERS("USD-BA-Reference Dealers"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-BMA Municipal Swap Index", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-BMA Municipal Swap Index", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-BMA Municipal Swap Index", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-BMA Municipal Swap Index", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-BMA Municipal Swap Index", source = "CME_ClearedConfirm_1_17")
	USD_BMA_MUNICIPAL_SWAP_INDEX("USD-BMA Municipal Swap Index"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CD-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CD-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CD-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CD-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CD-H.15", source = "CME_ClearedConfirm_1_17")
	USD_CD_H_15("USD-CD-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CD-Reference Dealers", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CD-Reference Dealers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CD-Reference Dealers", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CD-Reference Dealers", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CD-Reference Dealers", source = "CME_ClearedConfirm_1_17")
	USD_CD_REFERENCE_DEALERS("USD-CD-Reference Dealers"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMS-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMS-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks", source = "CME_ClearedConfirm_1_17")
	USD_CMS_REFERENCE_BANKS("USD-CMS-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMS-Reference Banks-ICAP SwapPX", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMS-Reference Banks-ICAP SwapPX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks-ICAP SwapPX", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks-ICAP SwapPX", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMS-Reference Banks-ICAP SwapPX", source = "CME_ClearedConfirm_1_17")
	USD_CMS_REFERENCE_BANKS_ICAP_SWAP_PX("USD-CMS-Reference Banks-ICAP SwapPX"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMS-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMS-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMS-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMS-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMS-Reuters", source = "CME_ClearedConfirm_1_17")
	USD_CMS_REUTERS("USD-CMS-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMS-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMS-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMS-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMS-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMS-Telerate", source = "CME_ClearedConfirm_1_17")
	USD_CMS_TELERATE("USD-CMS-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMT-T7051", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMT-T7051", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMT-T7051", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMT-T7051", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMT-T7051", source = "CME_ClearedConfirm_1_17")
	USD_CMT_T7051("USD-CMT-T7051"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CMT-T7052", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CMT-T7052", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CMT-T7052", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CMT-T7052", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CMT-T7052", source = "CME_ClearedConfirm_1_17")
	USD_CMT_T7052("USD-CMT-T7052"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-COF11-FHLBSF", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-COF11-FHLBSF", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-COF11-FHLBSF", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-COF11-FHLBSF", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-COF11-FHLBSF", source = "CME_ClearedConfirm_1_17")
	USD_COF11_FHLBSF("USD-COF11-FHLBSF"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-COF11-Reuters", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-COF11-Reuters", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-COF11-Reuters", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-COF11-Reuters", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-COF11-Reuters", source = "CME_ClearedConfirm_1_17")
	USD_COF_11_REUTERS("USD-COF11-Reuters"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-COF11-Telerate", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-COF11-Telerate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-COF11-Telerate", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-COF11-Telerate", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-COF11-Telerate", source = "CME_ClearedConfirm_1_17")
	USD_COF_11_TELERATE("USD-COF11-Telerate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CP-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CP-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CP-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CP-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CP-H.15", source = "CME_ClearedConfirm_1_17")
	USD_CP_H_15("USD-CP-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-CP-Reference Dealers", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-CP-Reference Dealers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-CP-Reference Dealers", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-CP-Reference Dealers", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-CP-Reference Dealers", source = "CME_ClearedConfirm_1_17")
	USD_CP_REFERENCE_DEALERS("USD-CP-Reference Dealers"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-FFCB-DISCO", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-FFCB-DISCO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-FFCB-DISCO", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-FFCB-DISCO", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-FFCB-DISCO", source = "CME_ClearedConfirm_1_17")
	USD_FFCB_DISCO("USD-FFCB-DISCO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Federal Funds-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Federal Funds-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15", source = "CME_ClearedConfirm_1_17")
	USD_FEDERAL_FUNDS_H_15("USD-Federal Funds-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Federal Funds-H.15-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-Bloomberg", source = "CME_ClearedConfirm_1_17")
	USD_FEDERAL_FUNDS_H_15_BLOOMBERG("USD-Federal Funds-H.15-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Federal Funds-H.15-OIS-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-OIS-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-OIS-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-OIS-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Federal Funds-H.15-OIS-COMPOUND", source = "CME_ClearedConfirm_1_17")
	USD_FEDERAL_FUNDS_H_15_OIS_COMPOUND("USD-Federal Funds-H.15-OIS-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Federal Funds-Reference Dealers", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Federal Funds-Reference Dealers", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Federal Funds-Reference Dealers", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Federal Funds-Reference Dealers", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Federal Funds-Reference Dealers", source = "CME_ClearedConfirm_1_17")
	USD_FEDERAL_FUNDS_REFERENCE_DEALERS("USD-Federal Funds-Reference Dealers"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate", source = "CME_ClearedConfirm_1_17")
	USD_ISDAFIX_3_SWAP_RATE("USD-ISDAFIX3-Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate-3:00", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate-3:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate-3:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate-3:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-ISDAFIX3-Swap Rate-3:00", source = "CME_ClearedConfirm_1_17")
	USD_ISDAFIX_3_SWAP_RATE_3_00("USD-ISDAFIX3-Swap Rate-3:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-ISDA-Swap Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate", source = "CME_ClearedConfirm_1_17")
	USD_ISDA_SWAP_RATE("USD-ISDA-Swap Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-ISDA-Swap Rate-3:00", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate-3:00", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate-3:00", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate-3:00", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-ISDA-Swap Rate-3:00", source = "CME_ClearedConfirm_1_17")
	USD_ISDA_SWAP_RATE_3_00("USD-ISDA-Swap Rate-3:00"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-LIBOR-BBA", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-LIBOR-BBA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA", source = "CME_ClearedConfirm_1_17")
	USD_LIBOR_BBA("USD-LIBOR-BBA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-LIBOR-BBA-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-LIBOR-BBA-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-LIBOR-BBA-Bloomberg", source = "CME_ClearedConfirm_1_17")
	USD_LIBOR_BBA_BLOOMBERG("USD-LIBOR-BBA-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-LIBOR-ISDA", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-LIBOR-ISDA", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-LIBOR-ISDA", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-LIBOR-ISDA", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-LIBOR-ISDA", source = "CME_ClearedConfirm_1_17")
	USD_LIBOR_ISDA("USD-LIBOR-ISDA"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-LIBOR-LIBO", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-LIBOR-LIBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-LIBOR-LIBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-LIBOR-LIBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-LIBOR-LIBO", source = "CME_ClearedConfirm_1_17")
	USD_LIBOR_LIBO("USD-LIBOR-LIBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-LIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-LIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-LIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-LIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-LIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	USD_LIBOR_REFERENCE_BANKS("USD-LIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Municipal Swap Libor Ratio-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Municipal Swap Libor Ratio-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Municipal Swap Libor Ratio-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Municipal Swap Libor Ratio-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Municipal Swap Libor Ratio-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_MUNICIPAL_SWAP_LIBOR_RATIO_11_00_ICAP("USD-Municipal Swap Libor Ratio-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Municipal Swap Rate-11:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Municipal Swap Rate-11:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Municipal Swap Rate-11:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Municipal Swap Rate-11:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Municipal Swap Rate-11:00-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_MUNICIPAL_SWAP_RATE_11_00_ICAP("USD-Municipal Swap Rate-11:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	USD_OIS_11_00_BGCANTOR("USD-OIS-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-11:00-LON-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-11:00-LON-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-11:00-LON-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-11:00-LON-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-11:00-LON-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_OIS_11_00_LON_ICAP("USD-OIS-11:00-LON-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-11:00-NY-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-11:00-NY-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-11:00-NY-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-11:00-NY-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-11:00-NY-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_OIS_11_00_NY_ICAP("USD-OIS-11:00-NY-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-11:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-11:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-11:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-11:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-11:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	USD_OIS_11_00_TRADITION("USD-OIS-11:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-3:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-3:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-3:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-3:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-3:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	USD_OIS_3_00_BGCANTOR("USD-OIS-3:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-3:00-NY-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-3:00-NY-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-3:00-NY-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-3:00-NY-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-3:00-NY-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_OIS_3_00_NY_ICAP("USD-OIS-3:00-NY-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-OIS-4:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-OIS-4:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-OIS-4:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-OIS-4:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-OIS-4:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	USD_OIS_4_00_TRADITION("USD-OIS-4:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Overnight Bank Funding Rate", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Overnight Bank Funding Rate", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Overnight Bank Funding Rate", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Overnight Bank Funding Rate", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Overnight Bank Funding Rate", source = "CME_ClearedConfirm_1_17")
	USD_OVERNIGHT_BANK_FUNDING_RATE("USD-Overnight Bank Funding Rate"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Prime-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Prime-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Prime-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Prime-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Prime-H.15", source = "CME_ClearedConfirm_1_17")
	USD_PRIME_H_15("USD-Prime-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Prime-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Prime-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Prime-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Prime-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Prime-Reference Banks", source = "CME_ClearedConfirm_1_17")
	USD_PRIME_REFERENCE_BANKS("USD-Prime-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-SIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-SIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-SIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-SIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-SIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	USD_SIBOR_REFERENCE_BANKS("USD-SIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-SIBOR-SIBO", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-SIBOR-SIBO", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-SIBOR-SIBO", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-SIBOR-SIBO", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-SIBOR-SIBO", source = "CME_ClearedConfirm_1_17")
	USD_SIBOR_SIBO("USD-SIBOR-SIBO"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-SIFMA Municipal Swap Index", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-SIFMA Municipal Swap Index", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-SIFMA Municipal Swap Index", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-SIFMA Municipal Swap Index", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-SIFMA Municipal Swap Index", source = "CME_ClearedConfirm_1_17")
	USD_SIFMA_MUNICIPAL_SWAP_INDEX("USD-SIFMA Municipal Swap Index"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-SOFR-COMPOUND", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-SOFR-COMPOUND", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-SOFR-COMPOUND", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-SOFR-COMPOUND", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-SOFR-COMPOUND", source = "CME_ClearedConfirm_1_17")
	USD_SOFR_COMPOUND("USD-SOFR-COMPOUND"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-S&P Index-High Grade", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-S&P Index-High Grade", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-S&P Index-High Grade", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-S&P Index-High Grade", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-S&P Index-High Grade", source = "CME_ClearedConfirm_1_17")
	USD_S_P_INDEX_HIGH_GRADE("USD-S&P Index-High Grade"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-TBILL-H.15", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-TBILL-H.15", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-TBILL-H.15", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-TBILL-H.15", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-TBILL-H.15", source = "CME_ClearedConfirm_1_17")
	USD_TBILL_H_15("USD-TBILL-H.15"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-TBILL-H.15-Bloomberg", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-TBILL-H.15-Bloomberg", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-TBILL-H.15-Bloomberg", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-TBILL-H.15-Bloomberg", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-TBILL-H.15-Bloomberg", source = "CME_ClearedConfirm_1_17")
	USD_TBILL_H_15_BLOOMBERG("USD-TBILL-H.15-Bloomberg"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-TBILL-Secondary Market", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-TBILL-Secondary Market", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-TBILL-Secondary Market", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-TBILL-Secondary Market", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-TBILL-Secondary Market", source = "CME_ClearedConfirm_1_17")
	USD_TBILL_SECONDARY_MARKET("USD-TBILL-Secondary Market"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-TIBOR-ISDC", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-TIBOR-ISDC", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-TIBOR-ISDC", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-TIBOR-ISDC", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-TIBOR-ISDC", source = "CME_ClearedConfirm_1_17")
	USD_TIBOR_ISDC("USD-TIBOR-ISDC"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-TIBOR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-TIBOR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-TIBOR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-TIBOR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-TIBOR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	USD_TIBOR_REFERENCE_BANKS("USD-TIBOR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury-19901-3:00-ICAP", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury-19901-3:00-ICAP", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury-19901-3:00-ICAP", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury-19901-3:00-ICAP", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury-19901-3:00-ICAP", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_19901_3_00_ICAP("USD-Treasury-19901-3:00-ICAP"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury Rate-ICAP BrokerTec", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury Rate-ICAP BrokerTec", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury Rate-ICAP BrokerTec", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury Rate-ICAP BrokerTec", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury Rate-ICAP BrokerTec", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_RATE_ICAP_BROKER_TEC("USD-Treasury Rate-ICAP BrokerTec"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker100", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker100", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker100", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker100", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker100", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_RATE_SWAP_MARKER_100("USD-Treasury Rate-SwapMarker100"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker99", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker99", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker99", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker99", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury Rate-SwapMarker99", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_RATE_SWAP_MARKER_99("USD-Treasury Rate-SwapMarker99"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury Rate-T19901", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury Rate-T19901", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T19901", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T19901", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T19901", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_RATE_T_19901("USD-Treasury Rate-T19901"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "USD-Treasury Rate-T500", source = "FpML_5_10")
	@RosettaSynonym(value = "USD-Treasury Rate-T500", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T500", source = "DTCC_11_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T500", source = "DTCC_9_0")
	@RosettaSynonym(value = "USD-Treasury Rate-T500", source = "CME_ClearedConfirm_1_17")
	USD_TREASURY_RATE_T_500("USD-Treasury Rate-T500"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "FpML_5_10")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_11_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "DTCC_9_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-11:00-BGCANTOR", source = "CME_ClearedConfirm_1_17")
	VND_SEMI_ANNUAL_SWAP_RATE_11_00_BGCANTOR("VND-Semi-Annual Swap Rate-11:00-BGCANTOR"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "VND-Semi-Annual Swap Rate-Reference Banks", source = "CME_ClearedConfirm_1_17")
	VND_SEMI_ANNUAL_SWAP_RATE_REFERENCE_BANKS("VND-Semi-Annual Swap Rate-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-DEPOSIT-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-DEPOSIT-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ZAR_DEPOSIT_REFERENCE_BANKS("ZAR-DEPOSIT-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-DEPOSIT-SAFEX", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-DEPOSIT-SAFEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-SAFEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-SAFEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-DEPOSIT-SAFEX", source = "CME_ClearedConfirm_1_17")
	ZAR_DEPOSIT_SAFEX("ZAR-DEPOSIT-SAFEX"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-JIBAR-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-JIBAR-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-JIBAR-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-JIBAR-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-JIBAR-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ZAR_JIBAR_REFERENCE_BANKS("ZAR-JIBAR-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-JIBAR-SAFEX", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-JIBAR-SAFEX", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-JIBAR-SAFEX", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-JIBAR-SAFEX", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-JIBAR-SAFEX", source = "CME_ClearedConfirm_1_17")
	ZAR_JIBAR_SAFEX("ZAR-JIBAR-SAFEX"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE", source = "CME_ClearedConfirm_1_17")
	ZAR_PRIME_AVERAGE("ZAR-PRIME-AVERAGE"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-PRIME-AVERAGE-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ZAR_PRIME_AVERAGE_REFERENCE_BANKS("ZAR-PRIME-AVERAGE-Reference Banks"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-1:00-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-1:00-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-1:00-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-1:00-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-1:00-TRADITION", source = "CME_ClearedConfirm_1_17")
	ZAR_QUARTERLY_SWAP_RATE_1_00_TRADITION("ZAR-Quarterly Swap Rate-1:00-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-5:30-TRADITION", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-5:30-TRADITION", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-5:30-TRADITION", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-5:30-TRADITION", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-5:30-TRADITION", source = "CME_ClearedConfirm_1_17")
	ZAR_QUARTERLY_SWAP_RATE_5_30_TRADITION("ZAR-Quarterly Swap Rate-5:30-TRADITION"),
	

	/**
	 * Per 2006 ISDA Definitions or Annex to the 2000 ISDA Definitions, Section 7.1 Rate Options, as amended and supplemented through the date on which parties enter into the relevant transaction.
	 */
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "FpML_5_10")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "CME_SubmissionIRS_1_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "DTCC_11_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "DTCC_9_0")
	@RosettaSynonym(value = "ZAR-Quarterly Swap Rate-TRADITION-Reference Banks", source = "CME_ClearedConfirm_1_17")
	ZAR_QUARTERLY_SWAP_RATE_TRADITION_REFERENCE_BANKS("ZAR-Quarterly Swap Rate-TRADITION-Reference Banks")
	;

	private final String displayName;
	
	FloatingRateIndexEnum() {
		this.displayName = null;
	}

	FloatingRateIndexEnum(String displayName) {
		this.displayName = displayName;
	}

	@Override
	public String toString() {
		return displayName != null ?  displayName : name();
	}
	
}
