package org.isda.cdm.blueprint;
			
import com.regnosys.rosetta.blueprints.Blueprint;
import com.regnosys.rosetta.blueprints.BlueprintBuilder;
import com.regnosys.rosetta.blueprints.BlueprintInstance;
import com.regnosys.rosetta.blueprints.runner.actions.rosetta.RosettaActionFactory;
import com.regnosys.rosetta.blueprints.runner.data.StringIdentifier;
import com.regnosys.rosetta.blueprints.runner.nodes.SinkNode;
import com.regnosys.rosetta.blueprints.runner.nodes.SourceNode;
import com.rosetta.model.lib.functions.MapperS;
import com.rosetta.model.lib.functions.MapperTree;
import com.rosetta.model.lib.meta.FieldWithMeta;
import java.math.BigDecimal;
import org.isda.cdm.Contract;
import org.isda.cdm.ContractualProduct;
import org.isda.cdm.ContractualQuantity;
import org.isda.cdm.CreditDefaultPayout;
import org.isda.cdm.EconomicTerms;
import org.isda.cdm.Event;
import org.isda.cdm.ExecutionQuantity;
import org.isda.cdm.FxLinkedNotionalSchedule;
import org.isda.cdm.InterestRatePayout;
import org.isda.cdm.Money;
import org.isda.cdm.NonNegativeAmountSchedule;
import org.isda.cdm.NonNegativeQuantity;
import org.isda.cdm.NonNegativeSchedule;
import org.isda.cdm.NonNegativeStep;
import org.isda.cdm.NotionalSchedule;
import org.isda.cdm.NotionalStepRule;
import org.isda.cdm.OptionPayout;
import org.isda.cdm.Payout;
import org.isda.cdm.PrimitiveEvent;
import org.isda.cdm.ProtectionTerms;
import org.isda.cdm.Quantity;
import org.isda.cdm.QuantityChangePrimitive;
import org.isda.cdm.QuantityNotation;
import org.isda.cdm.Trade;
import static com.regnosys.rosetta.blueprints.BlueprintBuilder.*;
import static com.rosetta.model.lib.validation.MapperTreeValidatorHelper.*;
import static com.rosetta.model.lib.validation.ValidatorHelper.*;

/**
 * @version 2.5.4
 */
public class DerivativeNotionalIncreaseDecreaseRule<INKEY extends Comparable<INKEY>> implements Blueprint<Event, String, INKEY, INKEY> {

	private final RosettaActionFactory actionFactory;

	public DerivativeNotionalIncreaseDecreaseRule(RosettaActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}
	
	@Override
	public String getName() {
		return "DerivativeNotionalIncreaseDecrease"; 
	}
	
	@Override
	public String getURI() {
		return "model-reg-reporting.rosetta#DerivativeNotionalIncreaseDecrease";
	}
	
	
	@Override
	public BlueprintInstance<Event, String, INKEY, INKEY> blueprint() { 
		return 
			startsWith(actionFactory, actionFactory.<Event, String, INKEY>newRosettaSingleMapper("model-reg-reporting.rosetta#//@elements.48/@nodes/@node", "choice", new StringIdentifier("choice"), event -> doIf(greaterThanEquals(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount))))))))))),MapperS.of("INCR"),doIf(lessThan(MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getAfter", QuantityChangePrimitive::getAfter).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount)))))))))), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<Quantity>map("getQuantity", ContractualQuantity::getQuantity).<BigDecimal>map("getAmount", Quantity::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", NonNegativeSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).<String>groupBy(g->new MapperS<>(g).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NonNegativeStep>mapC("getStep", NonNegativeSchedule::getStep).<BigDecimal>map("getStepValue", NonNegativeStep::getStepValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<NotionalSchedule>map("getNotionalSchedule", ContractualQuantity::getNotionalSchedule).<String>groupBy(g->new MapperS<>(g).<NonNegativeAmountSchedule>map("getNotionalStepSchedule", NotionalSchedule::getNotionalStepSchedule).map("getCurrency", NonNegativeAmountSchedule::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<NotionalStepRule>map("getNotionalStepParameters", NotionalSchedule::getNotionalStepParameters).<BigDecimal>map("getNotionalStepAmount", NotionalStepRule::getNotionalStepAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<InterestRatePayout>mapC("getInterestRatePayout", Payout::getInterestRatePayout).<ContractualQuantity>map("getQuantity", InterestRatePayout::getQuantity).<FxLinkedNotionalSchedule>map("getFxLinkedNotional", ContractualQuantity::getFxLinkedNotional).<String>groupBy(g->new MapperS<>(g).map("getVaryingNotionalCurrency", FxLinkedNotionalSchedule::getVaryingNotionalCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getInitialValue", FxLinkedNotionalSchedule::getInitialValue)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<CreditDefaultPayout>map("getCreditDefaultPayout", Payout::getCreditDefaultPayout).<ProtectionTerms>mapC("getProtectionTerms", CreditDefaultPayout::getProtectionTerms).<Money>map("getNotionalAmount", ProtectionTerms::getNotionalAmount).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.and(MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ContractualProduct>map("getContractualProduct", Contract::getContractualProduct).<EconomicTerms>map("getEconomicTerms", ContractualProduct::getEconomicTerms).<Payout>map("getPayout", EconomicTerms::getPayout).<OptionPayout>mapC("getOptionPayout", Payout::getOptionPayout).<ContractualQuantity>map("getQuantity", OptionPayout::getQuantity).map("getNotionalAmount", ContractualQuantity::getNotionalAmount).<Money>map("getValue", FieldWithMeta::getValue).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Money::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Money::getAmount)), MapperTree.of(MapperS.of(event).<PrimitiveEvent>map("getPrimitive", Event::getPrimitive).<QuantityChangePrimitive>mapC("getQuantityChange", PrimitiveEvent::getQuantityChange).<Trade>map("getBefore", QuantityChangePrimitive::getBefore).<Contract>map("getContract", Trade::getContract).<ExecutionQuantity>map("getContractualQuantity", Contract::getContractualQuantity).<QuantityNotation>mapC("getQuantityNotation", ExecutionQuantity::getQuantityNotation).<NonNegativeQuantity>map("getQuantity", QuantityNotation::getQuantity).<String>groupBy(g->new MapperS<>(g).map("getCurrency", Quantity::getCurrency).<String>map("getValue", FieldWithMeta::getValue)).<BigDecimal>map("getAmount", Quantity::getAmount))))))))))),MapperS.of("DECR"),MapperS.of("")))))
			.toBlueprint(getURI(), getName());
	}
}
